/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.algorithms;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.Template;
import org.apache.log4j.Logger;
import java.util.ArrayList;
import java.util.List;


/**
 * Contains several implementation of Metrics
 * <p/>
 * inner classes must be declared static
 *
 * @note from ma: don't forget that these distance funcs return greater distances
 * for longer vectors.  Need to have normalized equivalents that take
 * the number of components into account when comparing statistics
 * across i.e. different size feature vectors.
 */

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class CMetrics {

    /**
     * For logging support
     */
    protected static final transient Logger klog = XLogger.getLogger(CMetrics.class);

    private static final int MIN_NUM_FOR_VAR = 3;

    // @maint add a metric and this array might need updating
    public static CMetric[] createAllMetrics() {
        return new CMetric[]{
                /*new Pearson(),
                new Cosine(),
                new Euclidean(),
                new Manhatten(),
                new FeatureVar(),
                new FeatureVarD(),
                new tTest(),
                new Signal2Noise(),
                new None(),
                new RegressionSlope(),
                new Bhattacharyya(),
                new PearsonD(),
                new ClassMeansDiff(), new ClassMediansDiff(),
                new ClassOfInterestMean(), new ClassOfInterestMedian(),
                new ClassMeansRatio(),
                new ClassMediansRatio(),
                new ClassMeansLog2Ratio(),
                new ClassMediansLog2Ratio(),
                new ClassRatio(),
                new ClassDiff(),
                new ClassLog2Ratio(),
                new FeatureVar(),
                new FeatureVarD(),*/
                new ChiSquare_Allele(),
                new ChiSquare_Geno(),
                new tTest_Allele(),
                new tTest_Geno(),              
                new Diff_of_Major_Alleles()
                
        };
    }

    public static CMetric[] createAllContinuousMetrics() {

        final CMetric[] metrics = createAllMetrics();
        final List list = new ArrayList();

        for (int i = 0; i < metrics.length; i++) {
            if (metrics[i].isContinuous()) {
                list.add(metrics[i]);
            }
        }

        return (CMetric[]) list.toArray(new CMetric[list.size()]);
    }

    public static CMetric[] createAllCategoricalMetrics() {
        final CMetric[] metrics = createAllMetrics();
        final List list = new ArrayList();

        for (int i = 0; i < metrics.length; i++) {
            if (metrics[i].isCategorical()) {
                list.add(metrics[i]);
            }
        }

        return (CMetric[]) list.toArray(new CMetric[list.size()]);

        /*
        return new Metric[]{
                new Cosine(), new Euclidean(), new Manhatten(), new Pearson()
        };
        */
    }

    /*public static CorrMetric[] createAllCorrMetrics() {
        return new CorrMetric[]{
                new Pearson(), new Cosine()
        };
    }

    // @maint add a metric and this array might need updating
    public static RankMetric[] createAllRankMetrics() {
        return new RankMetric[]{
                new Spearman(), new KendallsTau()
        };
    }

    public static RankMetric lookupRankMetric(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup RankMetric for null object");
        }

        if (obj instanceof RankMetric) {
            return (RankMetric) obj;
        }

        RankMetric[] all = createAllRankMetrics();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookup RankMetrics by: " + obj);
    }*/

    //---------------------------------------------------------------------------//
    //---------------------------------------------------------------------------//
    //---------------------------------------------------------------------------//

    /**
     * Private class constructor to prevent construction outside.
     */
    private CMetrics() {
    }

    public static CMetric lookupMetric(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof CMetric) {
            return (CMetric) obj;
        }

        CMetric[] all = createAllMetrics();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookupMetric for: " + obj);
    }

    /*public static CorrMetric lookupCorrMetric(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof CorrMetric) {
            return (CorrMetric) obj;
        }

        CorrMetric[] all = createAllCorrMetrics();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookupCorrMetric for: " + obj);
    }*/


    /**
     * Class constructor
     */
    public static abstract class AbstractCMetric implements CMetric {

        protected VectorSplitter fSplitter;

        private Type fType;

        public AbstractCMetric(final Type type) {
            this.fType = type;
            this.fSplitter = new VectorSplitter();
        }

        public boolean isCategorical() {
            return (fType.equals(CATEGORICAL) || fType.equals(CAT_AND_CONT));
        }

        public boolean isContinuous() {
            return (fType.equals(CONTINUOUS) || fType.equals(CAT_AND_CONT));
        }

        public Type getType() {
            return fType;
        }

        public int hashCode() {
            return getName().hashCode();
        }

        public boolean equals(Object obj) {

            if (obj instanceof CMetric) {
                return getName().equalsIgnoreCase(((CMetric) obj).getName());
            }

            return false;
        }

        public String toString() {
            return getName();
        }
    }

    public static class ChiSquare_Geno extends AbstractCMetric {

        public static final String NAME = "ChiSquare_Geno";

        public ChiSquare_Geno() {
            super(CATEGORICAL);
        }

        public double getScore(ArrayList genotype, Template template, int code) {
            ArrayList[] vs = fSplitter.splitBiphasic(genotype, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return GsaaAlgorithms.chiSquareGeno(vs[0], vs[1], code);
            } else {
                return GsaaAlgorithms.chiSquareGeno(vs[1], vs[0], code);
            }
        }

        public double getScore(ArrayList x, ArrayList y, int code) {
            ArrayList[] vs=new ArrayList[2];
            vs[0]=x;
            vs[1]=y;
            return GsaaAlgorithms.chiSquareGeno(x, y, code);
        }

        public String getName() {
            return NAME;
        }

        public CMetric cloneCMetric() {
            return new ChiSquare_Geno();
        }

    }

    public static class ChiSquare_Allele extends AbstractCMetric {

        public static final String NAME = "ChiSquare_Allele";

        public ChiSquare_Allele() {
            super(CATEGORICAL);
        }

        public double getScore(ArrayList genotype, Template template, int code) {
            ArrayList[] vs = fSplitter.splitBiphasic(genotype, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return GsaaAlgorithms.chiSquareAllele(vs[0], vs[1], code);
            } else {
                return GsaaAlgorithms.chiSquareAllele(vs[1], vs[0], code);
            }
        }

        public double getScore(ArrayList x, ArrayList y, int code) {
            ArrayList[] vs=new ArrayList[2];
            vs[0]=x;
            vs[1]=y;
            return GsaaAlgorithms.chiSquareAllele(x, y, code);
        }

        public String getName() {
            return NAME;
        }

        public CMetric cloneCMetric() {
            return new ChiSquare_Allele();
        }

    }

    public static class tTest_Geno extends AbstractCMetric {

        public static final String NAME = "tTest_Geno";

        public tTest_Geno() {
            super(CATEGORICAL);
        }

        public double getScore(ArrayList genotype, Template template, int code) {
            ArrayList[] vs = fSplitter.splitBiphasic(genotype, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return GsaaAlgorithms.tTestGeno(vs[0], vs[1], code);
            } else {
                return GsaaAlgorithms.tTestGeno(vs[1], vs[0], code);
            }
        }

        public double getScore(ArrayList x, ArrayList y, int code) {
            ArrayList[] vs=new ArrayList[2];
            vs[0]=x;
            vs[1]=y;
            return GsaaAlgorithms.tTestGeno(x, y, code);
        }

        public String getName() {
            return NAME;
        }

        public CMetric cloneCMetric() {
            return new tTest_Geno();
        }

    }

    public static class tTest_Allele extends AbstractCMetric {

        public static final String NAME = "tTest_Allele";

        public tTest_Allele() {
            super(CATEGORICAL);
        }

        public double getScore(ArrayList genotype, Template template, int code) {
            ArrayList[] vs = fSplitter.splitBiphasic(genotype, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return GsaaAlgorithms.tTestAllele(vs[0], vs[1], code);
            } else {
                return GsaaAlgorithms.tTestAllele(vs[1], vs[0], code);
            }
        }

        public double getScore(ArrayList x, ArrayList y, int code) {
            ArrayList[] vs=new ArrayList[2];
            vs[0]=x;
            vs[1]=y;
            return GsaaAlgorithms.tTestAllele(x, y, code);
        }

        public String getName() {
            return NAME;
        }

        public CMetric cloneCMetric() {
            return new tTest_Allele();
        }

    }

    public static class Diff_of_Major_Alleles extends AbstractCMetric {


        public static final String NAME = "Diff_of_Major_Alleles";

        public Diff_of_Major_Alleles() {
            super(CATEGORICAL);
        }

        public double getScore(ArrayList genotype, Template template, int code) {
            ArrayList[] vs = fSplitter.splitBiphasic(genotype, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return GsaaAlgorithms.freqsDiff(vs[0], vs[1], code);
            } else {
                return GsaaAlgorithms.freqsDiff(vs[1], vs[0], code);
            }
        }

        public double getScore(ArrayList x, ArrayList y, int code) {
            ArrayList[] vs=new ArrayList[2];
            vs[0]=x;
            vs[1]=y;
            return GsaaAlgorithms.freqsDiff(x, y, code);
        }

        public String getName() {
            return NAME;
        }

        public CMetric cloneCMetric() {
            return new Diff_of_Major_Alleles();
        }

    }


    /*public static class ChiSquare extends AbstractMetric {


        public static final String NAME = "ChiSquare";

        public ChiSquare() {
            super(CATEGORICAL);
        }

        public double getScore(Vector profile, Template template, Map params) {
            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.chiSquare(vs[0], vs[1]);
            } else {
                return XMath.chiSquare(vs[1], vs[0]);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.chiSquare(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ChiSquare();
        }

    }*/

}    // End CMetrics

