/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.algorithms;

import java.util.ArrayList;
import java.text.DecimalFormat;
import java.math.BigDecimal;
import org.apache.commons.math.MathRuntimeException;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class GsaaAlgorithms {

    private static DecimalFormat df = new DecimalFormat("0.000000");

    public GsaaAlgorithms() {
    }

    public static double chiSquareGeno(final ArrayList x, final ArrayList y, final int code) {
        double value = 0;
        double ratioNN = 0;

        if (code == 3) {
            throw new IllegalArgumentException("Wrong coding format. Got: A-C-G-T; Require: 0-1-2 or AA-AB-BB");
        }

        if (code == 1) {
            int[][] counts = digitize012(x, y);

            int sum_x = x.size() - counts[0][3];
            int sum_y = y.size() - counts[1][3];
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            long[] observed1 = new long[counts[0].length - 1];
            long[] observed2 = new long[counts[0].length - 1];
            for (int i = 0; i < counts[0].length - 1; i++) {
                observed1[i] = counts[0][i];
                observed2[i] = counts[1][i];

                //genotype based chi-square may run into problems without this in some cases.
                if (observed1[i] == 0 && observed2[i] == 0) {
                    observed1[i] = 1;
                    observed2[i] = 1;
                }
            }
            value = TestUtils.chiSquareDataSetsComparison(observed1, observed2);
        }

        if (code == 2) {
            int[][] counts = digitizeAb(x, y);

            int sum_x = x.size() - counts[0][3];
            int sum_y = y.size() - counts[1][3];
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            long[] observed1 = new long[counts[0].length - 1];
            long[] observed2 = new long[counts[0].length - 1];
            for (int i = 0; i < counts[0].length - 1; i++) {
                observed1[i] = counts[0][i];
                observed2[i] = counts[1][i];

                //genotype based chi-square may run into problems without this in some cases.
                if (observed1[i] == 0 && observed2[i] == 0) {
                    observed1[i] = 1;
                    observed2[i] = 1;
                }
            }
            value = TestUtils.chiSquareDataSetsComparison(observed1, observed2);
        }

        return value * (1 - ratioNN);
    }

    public static double chiSquareAllele(final ArrayList x, final ArrayList y, final int code) {
        double value = 0;
        double ratioNN = 0;

        if (code == 3) {
            throw new IllegalArgumentException("Wrong coding format. Got: A-C-G-T; Require: 0-1-2 or AA-AB-BB");
        }

        if (code == 1) {
            int[][] counts = digitize012(x, y);

            int sum_x = (x.size() - counts[0][3]) * 2;
            int sum_y = (y.size() - counts[1][3]) * 2;
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            long[] observed1 = new long[2];
            long[] observed2 = new long[2];
            observed1[0] = counts[0][0] * 2 + counts[0][1];
            observed1[1] = counts[0][2] * 2 + counts[0][1];
            observed2[0] = counts[1][0] * 2 + counts[1][1];
            observed2[1] = counts[1][2] * 2 + counts[1][1];

            for (int i = 0; i < 2; i++) {
                //allele based chi-square may run into problems without this in some cases.
                if (observed1[i] == 0 && observed2[i] == 0) {
                    observed1[i] = 1;
                    observed2[i] = 1;
                }
            }

            value = TestUtils.chiSquareDataSetsComparison(observed1, observed2);
        }

        if (code == 2) {
            int[][] counts = digitizeAb(x, y);

            int sum_x = (x.size() - counts[0][3]) * 2;
            int sum_y = (y.size() - counts[1][3]) * 2;
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            long[] observed1 = new long[2];
            long[] observed2 = new long[2];
            observed1[0] = counts[0][0] * 2 + counts[0][1];
            observed1[1] = counts[0][2] * 2 + counts[0][1];
            observed2[0] = counts[1][0] * 2 + counts[1][1];
            observed2[1] = counts[1][2] * 2 + counts[1][1];

            for (int i = 0; i < 2; i++) {
                //allele based chi-square may run into problems without this in some cases.
                if (observed1[i] == 0 && observed2[i] == 0) {
                    observed1[i] = 1;
                    observed2[i] = 1;
                }
            }

            value = TestUtils.chiSquareDataSetsComparison(observed1, observed2);
        }

        return value * (1 - ratioNN);
    }

    public static double tTestGeno(final ArrayList x, final ArrayList y, final int code) {
        double value = 0;
        double ratioNN = 0;

        if (code == 3) {
            throw new IllegalArgumentException("Wrong coding format. Got: A-C-G-T; Require: 0-1-2 or AA-AB-BB");
        }

        if (code == 1) {
            int[][] counts = digitize012(x, y);

            int sum_x = x.size() - counts[0][3];
            int sum_y = y.size() - counts[1][3];
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            long[] observed1 = new long[counts[0].length - 1];
            long[] observed2 = new long[counts[0].length - 1];
            for (int i = 0; i < counts[0].length - 1; i++) {
                observed1[i] = counts[0][i];
                observed2[i] = counts[1][i];

                if (observed1[i] == 0 && observed2[i] == 0) {
                    observed1[i] = 1;
                    observed2[i] = 1;
                }
            }
            value = tTestDataSetsComparison(observed1, observed2);
        }

        if (code == 2) {
            int[][] counts = digitizeAb(x, y);

            int sum_x = x.size() - counts[0][3];
            int sum_y = y.size() - counts[1][3];
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            long[] observed1 = new long[counts[0].length - 1];
            long[] observed2 = new long[counts[0].length - 1];
            for (int i = 0; i < counts[0].length - 1; i++) {
                observed1[i] = counts[0][i];
                observed2[i] = counts[1][i];

                if (observed1[i] == 0 && observed2[i] == 0) {
                    observed1[i] = 1;
                    observed2[i] = 1;
                }
            }
            value = tTestDataSetsComparison(observed1, observed2);
        }

        return value * (1 - ratioNN);
    }

    public static double tTestAllele(final ArrayList x, final ArrayList y, final int code) {
        double value = 0;
        double ratioNN = 0;

        if (code == 3) {
            throw new IllegalArgumentException("Wrong coding format. Got: A-C-G-T; Require: 0-1-2 or AA-AB-BB");
        }

        if (code == 1) {
            int[][] counts = digitize012(x, y);

            int sum_x = (x.size() - counts[0][3]) * 2;
            int sum_y = (y.size() - counts[1][3]) * 2;
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            long[] observed1 = new long[2];
            long[] observed2 = new long[2];
            observed1[0] = counts[0][0] * 2 + counts[0][1];
            observed1[1] = counts[0][2] * 2 + counts[0][1];
            observed2[0] = counts[1][0] * 2 + counts[1][1];
            observed2[1] = counts[1][2] * 2 + counts[1][1];

            for (int i = 0; i < 2; i++) {
                if (observed1[i] == 0 && observed2[i] == 0) {
                    observed1[i] = 1;
                    observed2[i] = 1;
                }
            }

            value = tTestDataSetsComparison(observed1, observed2);
        }

        if (code == 2) {
            int[][] counts = digitizeAb(x, y);

            int sum_x = (x.size() - counts[0][3]) * 2;
            int sum_y = (y.size() - counts[1][3]) * 2;
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            long[] observed1 = new long[2];
            long[] observed2 = new long[2];
            observed1[0] = counts[0][0] * 2 + counts[0][1];
            observed1[1] = counts[0][2] * 2 + counts[0][1];
            observed2[0] = counts[1][0] * 2 + counts[1][1];
            observed2[1] = counts[1][2] * 2 + counts[1][1];

            for (int i = 0; i < 2; i++) {
                if (observed1[i] == 0 && observed2[i] == 0) {
                    observed1[i] = 1;
                    observed2[i] = 1;
                }
            }

            value = tTestDataSetsComparison(observed1, observed2);
        }

        return value * (1 - ratioNN);
    }

    public static double freqsDiff(final ArrayList x, final ArrayList y, final int code) {
        double value = 0;
        double ratioNN = 0;

        if (code == 1) {
            int[][] counts = digitize012(x, y);

            int sum_x = (x.size() - counts[0][3]) * 2;
            int sum_y = (y.size() - counts[1][3]) * 2;
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            if (counts[1][0] * 2 + counts[1][1] >= counts[1][1] + counts[1][2] * 2) {
                value = div(counts[0][0] * 2 + counts[0][1], sum_x) - div(counts[1][0] * 2 + counts[1][1], sum_y);
            } else {
                value = div(counts[0][1] + counts[0][2] * 2, sum_x) - div(counts[1][1] + counts[1][2] * 2, sum_y);
            }
        }

        if (code == 2) {
            int[][] counts = digitizeAb(x, y);
            int sum_x = (x.size() - counts[0][3]) * 2;
            int sum_y = (y.size() - counts[1][3]) * 2;
            ratioNN = div(counts[0][3] + counts[1][3], x.size() + y.size());

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            if (counts[1][0] * 2 + counts[1][1] >= counts[1][1] + counts[1][2] * 2) {
                value = div(counts[0][0] * 2 + counts[0][1], sum_x) - div(counts[1][0] * 2 + counts[1][1], sum_y);
            } else {
                value = div(counts[0][1] + counts[0][2] * 2, sum_x) - div(counts[1][1] + counts[1][2] * 2, sum_y);
            }
        }

        if (code == 3) {
            int[][] counts = digitizeAcgt(x, y);
            int sum_x = x.size() * 2 - counts[0][4];
            int sum_y = y.size() * 2 - counts[1][4];
            ratioNN = div(counts[0][4] + counts[1][4], (x.size() + y.size()) * 2);

            if (sum_x == 0 || sum_y == 0) {
                return 0;
            }

            int index = indexOfMax(new int[]{counts[1][0], counts[1][1], counts[1][2], counts[1][3]});
            value = div(counts[0][index], sum_x) - div(counts[1][index], sum_y);
        }

        return Math.abs(value * (1 - ratioNN));
    }

    public static int[][] digitizeAcgt(final ArrayList x, final ArrayList y) {
        //A-C-G-T-N
        int[][] freqs = new int[2][5];
        for (int i = 0; i < x.size(); i++) {
            String genotype = (String) x.get(i);
            for (int j = 0; j < genotype.length(); j++) {
                switch (genotype.charAt(j)) {
                    case 'A':
                        freqs[0][0]++;
                        break;
                    case 'C':
                        freqs[0][1]++;
                        break;
                    case 'G':
                        freqs[0][2]++;
                        break;
                    case 'T':
                        freqs[0][3]++;
                        break;
                    case 'N':
                        freqs[0][4]++;
                        break;
                    default:
                        throw new IllegalArgumentException("The code of SNP dataset is not correct");
                    //break;
                }
            }
        }

        for (int i = 0; i < y.size(); i++) {
            String genotype = (String) y.get(i);
            for (int j = 0; j < genotype.length(); j++) {
                switch (genotype.charAt(j)) {
                    case 'A':
                        freqs[1][0]++;
                        break;
                    case 'C':
                        freqs[1][1]++;
                        break;
                    case 'G':
                        freqs[1][2]++;
                        break;
                    case 'T':
                        freqs[1][3]++;
                        break;
                    case 'N':
                        freqs[1][4]++;
                        break;
                    default:
                        throw new IllegalArgumentException("The code of SNP dataset is not correct");
                    //break;
                }
            }
        }
        return freqs;
    }

    public static int[][] digitizeAb(final ArrayList x, final ArrayList y) {
        //AA-AB-BB-NoCall
        int[][] freqs = new int[2][4];
        for (int i = 0; i < x.size(); i++) {
            String genotype = (String) x.get(i);
            if (genotype.equalsIgnoreCase("AA")) {
                freqs[0][0]++;
            } else if (genotype.equalsIgnoreCase("AB")) {
                freqs[0][1]++;
            } else if (genotype.equalsIgnoreCase("BB")) {
                freqs[0][2]++;
            } else {
                freqs[0][3]++;
            }
        }

        for (int i = 0; i < y.size(); i++) {
            String genotype = (String) y.get(i);
            if (genotype.equalsIgnoreCase("AA")) {
                freqs[1][0]++;
            } else if (genotype.equalsIgnoreCase("AB")) {
                freqs[1][1]++;
            } else if (genotype.equalsIgnoreCase("BB")) {
                freqs[1][2]++;
            } else {
                freqs[1][3]++;
            }
        }

        return freqs;
    }

    public static int[][] digitize012(final ArrayList x, final ArrayList y) {
        //0-1-2-3
        int[][] freqs = new int[2][4];
        for (int i = 0; i < x.size(); i++) {
            String genotype = (String) x.get(i);
            if (genotype.equalsIgnoreCase("0")) {
                freqs[0][0]++;
            } else if (genotype.equalsIgnoreCase("1")) {
                freqs[0][1]++;
            } else if (genotype.equalsIgnoreCase("2")) {
                freqs[0][2]++;
            } else {
                freqs[0][3]++;
            }
        }

        for (int i = 0; i < y.size(); i++) {
            String genotype = (String) y.get(i);
            if (genotype.equalsIgnoreCase("0")) {
                freqs[1][0]++;
            } else if (genotype.equalsIgnoreCase("1")) {
                freqs[1][1]++;
            } else if (genotype.equalsIgnoreCase("2")) {
                freqs[1][2]++;
            } else {
                freqs[1][3]++;
            }
        }

        return freqs;
    }

    public static double div(double v1, double v2, int scale) {
        if (scale < 0) {
            throw new IllegalArgumentException(
                    "The scale must be a positive integer or zero");
        }
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.divide(b2, scale, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    public static double div(int v1, int v2, int scale) {
        if (scale < 0) {
            throw new IllegalArgumentException(
                    "The scale must be a positive integer or zero");
        }
        BigDecimal b1 = new BigDecimal(Integer.toString(v1));
        BigDecimal b2 = new BigDecimal(Integer.toString(v2));
        return b1.divide(b2, scale, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    public static double div(int v1, int v2) {
        return (double) v1 / (double) v2;
    }

    public static int indexOfMax(final int[] freqs) {
        int index = 0;
        int max = 0;

        for (int i = 0; i < freqs.length; i++) {
            if (max < freqs[i]) {
                max = freqs[i];
                index = i;
            }
        }
        return index;
    }

    public static double tTestDataSetsComparison(long[] observed1, long[] observed2)
            throws IllegalArgumentException {

        // Make sure lengths are same
        if (observed1.length < 2) {
            throw MathRuntimeException.createIllegalArgumentException(
                    "observed array length = {0}, must be at least 2",
                    observed1.length);
        }
        if (observed1.length != observed2.length) {
            throw MathRuntimeException.createIllegalArgumentException(
                    "dimension mismatch {0} != {1}",
                    observed1.length, observed2.length);
        }

        // Ensure non-negative counts
        checkNonNegative(observed1);
        checkNonNegative(observed2);

        // Compute and compare count sums
        long countSum1 = 0;
        long countSum2 = 0;
        boolean unequalCounts = false;
        double weight = 0.0;
        for (int i = 0; i < observed1.length; i++) {
            countSum1 += observed1[i];
            countSum2 += observed2[i];
        }
        // Ensure neither sample is uniformly 0
        if (countSum1 == 0) {
            throw MathRuntimeException.createIllegalArgumentException(
                    "observed counts are all 0 in first observed array");
        }
        if (countSum2 == 0) {
            throw MathRuntimeException.createIllegalArgumentException(
                    "observed counts are all 0 in second observed array");
        }
        // Compare and compute weight only if different
        unequalCounts = (countSum1 != countSum2);
        if (unequalCounts) {
            weight = Math.sqrt((double) countSum1 / (double) countSum2);
        }
        // Compute ChiSquare statistic
        double sumSq = 0.0d;
        double dev = 0.0d;
        double obs1 = 0.0d;
        double obs2 = 0.0d;
        for (int i = 0; i < observed1.length; i++) {
            if (observed1[i] == 0 && observed2[i] == 0) {
                throw MathRuntimeException.createIllegalArgumentException(
                        "observed counts are both zero for entry {0}", i);
            } else {
                obs1 = observed1[i];
                obs2 = observed2[i];
                if (unequalCounts) { // apply weights
                    dev = obs1 / weight - obs2 * weight;
                } else {
                    dev = obs1 - obs2;
                }
                sumSq += Math.abs(dev) / (obs1 + obs2);
            }
        }
        return sumSq;
    }

    public static void checkNonNegative(long[] in) throws IllegalArgumentException {
        for (int i = 0; i < in.length; i++) {
            if (in[i] < 0) {
                throw MathRuntimeException.createIllegalArgumentException(
                        "element {0} is negative: {1}",
                        i, in[i]);
            }
        }
    }
}
