/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.objects;

import edu.mit.broad.genome.math.IMatrix;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.utils.ImmutedException;
import org.apache.log4j.Logger;


/**
 *
 *
 */
public class CharMatrix implements IMatrix {

    /**
     * (javax.vecmath note)
     * Implementation note:
     * The size of the CharMatrix does NOT automaticly grow.
     * It does only when setSize is called. I believe this is the spec
     * and makes less confusion, less user bugs.
     */

    /**
     * @maint IMP IMP IMP IMP IMP
     * If any fields are added to this class, make sure to update
     * the constructors and the set(CharMatrix m) method
     * Also, make sure to do an immutability check if needed.
     * <p/>
     * The data of the CharMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     * <p/>
     * <p/>
     * The data of the CharMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     */

    /**
     * The data of the CharMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     */
    protected char elementData[];

    /**
     * The number of rows in this CharMatrix.
     */
    private int fRowCnt;

    /**
     * The number of columns in this CharMatrix.
     */
    private int fColCnt;

    /**
     * Flag for whether this CharMatrix can have its values altered or not
     */
    private boolean fImmuted;

    /**
     * For logging support
     */
    private static final transient Logger klog = XLogger.getLogger(CharMatrix.class);

    /**
     * Class Constructor.
     * Must call set() after this form of teh constructor.
     */
    public CharMatrix() {
    }

    /**
     * Class Constructor.
     * Constructs an fRowCnt by fColCnt all zero CharMatrix. (as change)
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible CharMatrix index values.
     *
     * @param nrows number of rows in this CharMatrix.
     * @param ncols number of columns in this CharMatrix.
     */
    public CharMatrix(int nrows, int ncols) {
        init(nrows, ncols);
    }

    private void init(int nrows, int ncols) {
        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = new char[fRowCnt * fColCnt];
    }

    /**
     * Class Constructor.
     * Data is copied over from specified char array.
     * Constructs an fRowCnt by fColCnt CharMatrix initialized to the values
     * in the CharMatrix array.  The array values are copied in one row at
     * a time in row major fashion.  The array should be at least
     * fRowCnt*fColCnt in length.
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible CharMatrix index values.
     *
     * @param nrows      number of rows in this CharMatrix.
     * @param ncols      number of columns in this CharMatrix.
     * @param CharMatrix a 1D array that specifies a CharMatrix in row major fashion
     */
    public CharMatrix(int nrows, int ncols, char CharMatrix[]) {

        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = new char[fRowCnt * fColCnt];

        set(CharMatrix);
    }

    /**
     * Class Constructor.
     * Data is NOT shared
     *
     * @param CharMatrixA
     * @param CharMatrixB
     */
    public CharMatrix(CharMatrix CharMatrixA, CharMatrix CharMatrixB) {
        this(new CharMatrix[]{CharMatrixA, CharMatrixB});
    }

    /**
     * Class Constructor.
     * Data is NOT shared
     * matrices must be of same number of columns
     * (nrows can be arb - thats the point!)
     */
    public CharMatrix(CharMatrix[] matrices) {

        int c = matrices[0].getNumCol();

        for (int i = 0; i < matrices.length; i++) {
            if (matrices[i].getNumCol() != c) {
                throw new RuntimeException("Unequal ncols: " + c + " and "
                        + matrices[i].getNumCol());
            }
        }

        int nrows = 0;

        for (int i = 0; i < matrices.length; i++) {
            nrows += matrices[i].getNumRow();
        }

        this.fRowCnt = nrows;
        this.fColCnt = c;
        this.elementData = new char[fRowCnt * fColCnt];

        int pos = 0;

        for (int i = 0; i < matrices.length; i++) {
            System.arraycopy(matrices[i].elementData, 0, elementData, pos,
                    matrices[i].elementData.length);

            pos += matrices[i].elementData.length;
        }
    }

    /**
     * Class Constructor.
     * Constructs a new CharMatrix and copies the initial values
     * from the parameter CharMatrix.
     *
     * @param matrix the source of the initial values of the new CharMatrix
     *               <p/>
     *               as added: if shared values, then this CharMatrix shares the elements with the specified amtric
     *               else the array is systemarrycopied
     */
    public CharMatrix(CharMatrix matrix, boolean shareValues) {

        this.fRowCnt = matrix.fRowCnt;
        this.fColCnt = matrix.fColCnt;

        // DONT thats the whole point! (often)
        if (shareValues) {
            this.fImmuted = matrix.fImmuted;
        } else {
            this.fImmuted = false;
        }

        if (shareValues) {
            this.elementData = matrix.elementData;
        } else {
            int newSize = fRowCnt * fColCnt;

            this.elementData = new char[newSize];

            System.arraycopy(matrix.elementData, 0, elementData, 0, newSize);
        }
    }

    /**
     * The cloned CharMatrix is NOT immutable even if the CharMatrix it was cloned from was.
     */
    public CharMatrix cloneDeep() {

        klog.debug("Making cloneDeep of CharMatrix");
        //TraceUtils.showTrace();

        return new CharMatrix(this, false);
    }

    /**
     * Fill the CharMatrix with specified value
     *
     * @param val
     */
    public void fill(char val) {
        checkImmutable();
        for (int i = 0; i < elementData.length; i++) {
            elementData[i] = val;
        }
    }

    /**
     * Copies a sub-CharMatrix derived from this CharMatrix into the target CharMatrix.
     * The upper left of the sub-CharMatrix is located at (rowSource, colSource);
     * the lower right of the sub-CharMatrix is located at
     * (lastRowSource,lastColSource).  The sub-CharMatrix is copied into the
     * the target CharMatrix starting at (rowDest, colDest).
     *
     * @param rowSource the top-most row of the sub-CharMatrix
     * @param colSource the left-most column of the sub-CharMatrix
     * @param numRow    the number of rows in the sub-CharMatrix
     * @param numCol    the number of columns in the sub-CharMatrix
     * @param rowDest   the top-most row of the position of the copied sub-CharMatrix
     *                  within the target CharMatrix
     * @param colDest   the left-most column of the position of the copied sub-CharMatrix
     *                  within the target CharMatrix
     * @param target    the CharMatrix into which the sub-CharMatrix will be copied
     */
    public void copySubCharMatrix(int rowSource, int colSource, int numRow, int numCol,
                                  int rowDest, int colDest, CharMatrix target) {

        if ((rowSource < 0) || (colSource < 0) || (rowDest < 0) || (colDest < 0)) {
            throw new ArrayIndexOutOfBoundsException("rowSource,colSource,rowDest,colDest < 0.");
        } else if ((fRowCnt < numRow + rowSource) || (fColCnt < numCol + colSource)) {
            throw new ArrayIndexOutOfBoundsException("Source CharMatrix too small.");
        } else if ((target.fRowCnt < numRow + rowDest) || (target.fColCnt < numCol + colDest)) {
            throw new ArrayIndexOutOfBoundsException("Target CharMatrix too small.");
        }

        for (int i = 0; i < numRow; i++) {
            for (int j = 0; j < numCol; j++) {
                target.elementData[(i + rowDest) * fColCnt + (j + colDest)] =
                        elementData[(i + rowSource) * fColCnt + (j + colSource)];
            }
        }
    }

    /**
     * Changes the size of this CharMatrix dynamically.  If the size is increased
     * no data values will be lost.  If the size is decreased, only those data
     * values whose CharMatrix positions were eliminated will be lost.
     *
     * @param nrows number of desired rows in this CharMatrix
     * @param ncols number of desired columns in this CharMatrix
     */
    public void setSize(int nrows, int ncols) {

        checkImmutable();

        if ((fRowCnt < 0) || (fColCnt < 0)) {
            throw new NegativeArraySizeException("nrows or ncols < 0");
        }

        int oldfRowCnt = this.fRowCnt;
        int oldfColCnt = this.fColCnt;
        int oldSize = this.fRowCnt * this.fColCnt;

        this.fRowCnt = nrows;
        this.fColCnt = ncols;

        int newSize = fRowCnt * fColCnt;
        char[] oldData = elementData;

        if (oldfColCnt == fColCnt) {

            // no need to reload elements.
            if (fRowCnt <= oldfRowCnt) {
                return;
            }

            // have to allocate memory.
            elementData = new char[newSize];

            // copy the old data.
            System.arraycopy(oldData, 0, elementData, 0, oldSize);

            // no need to pad. 0.0 is automaticly padded.
            // for (int i = oldSize; i < newSize; i++)
            //     elementData[i] = 0.0;
        } else {
            elementData = new char[newSize];
            for (int i = 0; i < oldfRowCnt; i++) {
                System.arraycopy(oldData, i * oldfColCnt, elementData, i * fColCnt, oldfColCnt);
            }
        }
    }

    /**
     * Sets the value of this CharMatrix to the values found in the array parameter.
     * The values are copied in one row at a time, in row major
     * fashion.  The array should be at least equal in length to
     * the number of CharMatrix rows times the number of CharMatrix columns
     * in this CharMatrix.
     *
     * @param CharMatrix the row major source array
     */
    public void set(char CharMatrix[]) {

        checkImmutable();

        int size = fRowCnt * fColCnt;

        System.arraycopy(CharMatrix, 0, elementData, 0, size);
    }

    /**
     * Sets the value of this CharMatrix to the values found in CharMatrix m1.
     *
     * @param m1 the source CharMatrix
     */
    public void set(CharMatrix m1, boolean shareValues) {

        checkImmutable();

        // This implementation is in 'no automatic size grow' policy.
        // When size mismatch, exception will be thrown from the below.
        if ((m1.fRowCnt < fRowCnt) || (m1.fColCnt < fColCnt)) {
            throw new ArrayIndexOutOfBoundsException("m1 smaller than this CharMatrix");
        }

        if (shareValues) {
            this.elementData = m1.elementData;
            this.fRowCnt = m1.fRowCnt;
            this.fColCnt = m1.fColCnt;
            this.fImmuted = m1.fImmuted;
        } else {
            System.arraycopy(m1.elementData, 0, elementData, 0, fRowCnt * fColCnt);
        }
    }

    /**
     * Returns the number of rows in this CharMatrix.
     *
     * @return number of rows in this CharMatrix
     */
    public int getNumRow() {
        return fRowCnt;
    }

    /**
     * Returns the number of colmuns in this CharMatrix.
     *
     * @return number of columns in this CharMatrix
     */
    public int getNumCol() {
        return fColCnt;
    }

    public int getDim() {
        return getNumRow() * getNumCol();
    }

    /**
     * Retrieves the value at the specified row and column of this CharMatrix.
     *
     * @param row    the row number to be retrieved (zero indexed)
     * @param column the column number to be retrieved (zero indexed)
     * @return the value at the indexed element
     */
    public char getElement(int row, int column) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > CharMatrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > CharMatrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        return elementData[row * fColCnt + column];
    }

    /**
     * When you dont care about rows and columns
     * A bit faster than doing the other way
     *
     * @param absoluteIndex
     * @return
     */
    public char getElement(int absoluteIndex) {
        return elementData[absoluteIndex];
    }

    public char[] getAllElements() {
        char[] dest = new char[elementData.length];
        System.arraycopy(elementData, 0, dest, 0, elementData.length);
        return dest;
    }


    public int getAbsoluteSize() {
        return elementData.length;
    }

    /**
     * Modifies the value at the specified row and column of this CharMatrix.
     *
     * @param row    the row number to be modified (zero indexed)
     * @param column the column number to be modified (zero indexed)
     * @param value  the new CharMatrix element value
     */
    public void setElement(int row, int column, char value) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > CharMatrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > CharMatrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        elementData[row * fColCnt + column] = value;
    }

    public void setElementByIndexPos(int indexPos, char value) {

        checkImmutable();

        if (elementData.length <= indexPos) {
            throw new ArrayIndexOutOfBoundsException("row:" + indexPos + " > CharMatrix's length:"
                    + elementData.length);
        }

        if (indexPos < 0) {
            throw new ArrayIndexOutOfBoundsException("indexPos:" + indexPos + " < 0");
        }

        elementData[indexPos] = value;
    }

    /**
     * A safe copy is returned.
     */
    public char[] getRow(int row) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > CharMatrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        char[] ret = new char[fColCnt];

        System.arraycopy(elementData, row * fColCnt, ret, 0, fColCnt);

        return ret;
    }

    /**
     * Places the values of the specified column into the array parameter.
     *
     * @param col   the target column number
     * @param array the array into which the column values will be placed
     */
    private void _getColumnInPlace(int col, char array[]) {

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > CharMatrix's fColCnt:"
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array.length:" + array.length
                    + " < CharMatrix's fRowCnt=" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            array[i] = elementData[i * fColCnt + col];
        }
    }

    /**
     * A safe copy is returned.
     */
    public char[] getColumn(int col) {

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        char[] ret = new char[fRowCnt];

        for (int i = 0; i < fRowCnt; i++) {
            ret[i] = elementData[i * fColCnt + col];
        }

        return ret;
    }

    /**
     * Copy the values from the array into the specified row of this
     * CharMatrix.
     *
     * @param row   the row of this CharMatrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setRow(int row, char array[]) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > CharMatrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (array.length < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length
                    + " < CharMatrix's fColCnt=" + fColCnt);
        }

        System.arraycopy(array, 0, elementData, row * fColCnt, fColCnt);
    }

    /**
     * Copy the values from the array into the specified column of this
     * CharMatrix.
     *
     * @param col   the column of this CharMatrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setColumn(int col, char array[]) {

        checkImmutable();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > CharMatrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length
                    + " < CharMatrix's fRowCnt:" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            elementData[i * fColCnt + col] = array[i];
        }
    }

    /**
     * Transposes this CharMatrix in place.
     * <p/>
     * Note: Not a mem eff way of doing this (see note below)
     */
    public void transpose() {

        checkImmutable();

        //Print.out(elementData);
        char[] array = new char[elementData.length];
        int pos = 0;

        for (int c = 0; c < this.getNumCol(); c++) {
            char[] col = new char[this.getNumRow()];

            _getColumnInPlace(c, col);
            System.arraycopy(col, 0, array, pos, col.length);

            pos += col.length;
        }

        int oldrowcnt = this.fRowCnt;
        int oldcolcnt = this.fColCnt;

        this.elementData = array;
        this.fColCnt = oldrowcnt;
        this.fRowCnt = oldcolcnt;
    }

    /**
     * Places the CharMatrix values of the transpose of CharMatrix m1 into this CharMatrix. <p>
     *
     * @param m1 the CharMatrix to be transposed (but not modified)
     */
    public void transpose(CharMatrix m1) {

        checkImmutable();
        set(m1, false);
        transpose();
    }

    /**
     * Returns a string that contains the values of this CharMatrix.
     *
     * @return the String representation
     */
    public String toString() {

        String nl = System.getProperty("line.separator");
        StringBuffer out = new StringBuffer("[");

        out.append(nl);

        for (int i = 0; i < fRowCnt; i++) {
            out.append("  [");

            for (int j = 0; j < fColCnt; j++) {
                if (0 < j) {
                    out.append("\t");
                }

                out.append(elementData[i * fColCnt + j]);
            }

            if (i + 1 < fRowCnt) {
                out.append("]");
                out.append(nl);
            } else {
                out.append("] ]");
            }
        }

        return out.toString();
    }

    /**
     * Returns a hash number based on the data values in this
     * object.  Two different CharMatrix objects with identical data values
     * (ie, returns true for equals(CharMatrix) ) will return the same hash
     * number.  Two objects with different data members may return the
     * same hash value, although this is not likely.
     *
     * @return the integer hash value
     */
    public int hashCode() {

        int hash = 0;

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            long bits = Double.doubleToLongBits(elementData[i]);

            hash ^= (int) (bits ^ (bits >> 32));
        }

        return hash;
    }

    /**
     * Returns true if all of the data members of CharMatrix4d m1 are
     * equal to the corresponding data members in this CharMatrix4d.
     *
     * @param m1 The CharMatrix with which the comparison is made.
     * @return true or false
     */
    public boolean equals(CharMatrix m1) {

        if (m1 == null) {
            return false;
        }

        if (m1.fRowCnt != fRowCnt) {
            return false;
        }

        if (m1.fColCnt != fColCnt) {
            return false;
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (elementData[i * fColCnt + j] != m1.elementData[i * fColCnt + j]) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Returns true if the Object o1 is of type CharMatrix and all of the data
     * members of t1 are equal to the corresponding data members in this
     * CharMatrix.
     *
     * @param o1 the object with which the comparison is made.
     */
    public boolean equals(Object o1) {
        return (o1 != null) && (o1 instanceof CharMatrix) && equals((CharMatrix) o1);
    }

    public void setImmutable() {
        this.fImmuted = true;
    }

    public boolean isImmutable() {
        return fImmuted;
    }

    private void checkImmutable() {

        if (fImmuted) {
            throw new ImmutedException();
        }
    }

}        // End CharMatrix

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
