/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.objects;


import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.LabelledVector;
import edu.mit.broad.genome.objects.Annot;
import edu.mit.broad.genome.objects.AbstractDataset;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.Template;
import gnu.trove.TObjectIntHashMap;

import java.io.File;
import java.util.*;

/**
 * Basic implementation of a Dataset.
 * Several forms of constructors to create Datasets.
 * <p/>
 * Data is always duplicated and never shared. Dataset is immutable after creation.
 * This is the chief class that provides access to Dataset objects. Its used everywhere!
 */

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class SnpDataset extends AbstractDataset {

    private StringMatrix fMatrix;

    // these are Lists and not Sets as we want to be able to maintain the order
    // we however check for duplicates in the constructors
    private List fRowNames;
    private List fColNames;
    private List fAnn;
    private File dsFile;
    private List fGenes;
    private String fQuickInfo = "";

    /**
     * subclasses MUST call init after using this form of the constructor.
     */
    protected SnpDataset() {
    }

    // @rls must all be od f the same size
    public SnpDataset(final String name, final RankedList[] rls) {

        final List rowNames = new ArrayList(rls[0].getRankedNames());
        final List colNames = new ArrayList();
        int numRows = rls[0].getSize();
        StringMatrix m = new StringMatrix(numRows, rls.length);
        for (int c = 0; c < rls.length; c++) {
            colNames.add(rls[c].getName());
            for (int r = 0; r < numRows; r++) {
                String score = Double.toString(rls[c].getScore(rowNames.get(r).toString()));
                m.setElement(r, c, score);
            }
        }

        init(name, m, rowNames, colNames, null);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     */
    public SnpDataset(final String name, final StringMatrix matrix, final List ann) {

        List rowNames = new ArrayList(matrix.getNumRow());
        List colNames = new ArrayList(matrix.getNumCol());

        for (int r = 0; r < matrix.getNumRow(); r++) {
            rowNames.add("row_" + r);
        }

        for (int c = 0; c < matrix.getNumCol(); c++) {
            colNames.add("col_" + c);
        }

        init(name, matrix, rowNames, colNames, ann);
    }

    public SnpDataset(final String name, final StringMatrix matrix, List rowNames, List colNames, final List ann) {
        init(name, matrix, rowNames, colNames, ann);
    }

    protected void init(final String dsName,
            final StringMatrix matrix,
            final List rowNames,
            final List colNames,
            final List ann) {
        init_rows_and_cols(dsName, rowNames, colNames, ann);
        initMatrix(matrix, rowNames, colNames);
    }

    // This is one half of the CORE init (see below)
    private void initMatrix(final StringMatrix matrix, final List rowNames, final List colNames) {

        if (matrix == null) {
            throw new IllegalArgumentException("Matrix cannot be null");
        }


        if (matrix.getNumRow() != rowNames.size()) {
            throw new IllegalArgumentException("Matrix nrows: " + matrix.getNumRow() + " and rownames: " + rowNames.size() + " do not match in size");
        }

        if (matrix.getNumCol() != colNames.size()) {
            throw new IllegalArgumentException("Matrix ncols: " + matrix.getNumCol() + " and colnames: " + colNames.size() + " do not match in size");
        }

        this.fMatrix = matrix;
        this.fMatrix.setImmutable();    // IMP notice.
    }

    // This is one half of the CORE init (see above)
    protected void init_rows_and_cols(final String dsName,
            final List rowNames,
            final List colNames,
            final List ann) {

        super.initialize(dsName);

        if (rowNames == null) {
            throw new IllegalArgumentException("rowNames cannot be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("colNames cannot be null");
        }

        this.fRowNames = rowNames;
        this.fRowNames = Collections.unmodifiableList(fRowNames);
        this.fColNames = colNames;
        this.fColNames = Collections.unmodifiableList(fColNames);

        // annot sanity checks
        if (ann != null) {
            this.fAnn = ann;

            // data integrity checks: no column names must be duplicated
            // ditto for rows but we avoid doing that as its sloooow
            ensureAllUniqueValues(fColNames);

        }
    }

    protected static void ensureAllUniqueValues(List list) {
        Set set = new HashSet();
        for (int i = 0; i < list.size(); i++) {
            Object obj = list.get(i);
            if (set.contains(obj)) {
                throw new IllegalArgumentException("Duplicate COL names are NOT allowed in Datasets. The offending entry was: " + obj + " at pos: " + i);
            }
            set.add(obj);
        }

        set.clear();
    }

    public void setGenes(List genes){
        if (genes == null) {
            throw new IllegalArgumentException("genes cannot be null: " + genes);
        }
        fGenes=genes;
    }

    public List getGenes( ){
        return fGenes;
    }

    public ArrayList getAnn() {
        ArrayList ann = new ArrayList();
        if (fAnn != null) {
            //log.debug("Annotation is null -- will use a default ann " + getName());
            for (int i = 0; i < fAnn.size(); i++) {
                ArrayList temp = new ArrayList();
                for (int j = 0; j < ((ArrayList) fAnn.get(i)).size(); j++) {
                    String s = (String) ((ArrayList) fAnn.get(i)).get(j);
                    temp.add(s);
                }
                ann.add(temp);
            }
        }
        return ann;
    }

    public String getRowName(int rown) {
        return (String) fRowNames.get(rown);
    }

    public List getRowNames() {
        return Collections.unmodifiableList(fRowNames);
    }

    // @todo check impact: trove added Jan 2006
    private TObjectIntHashMap fRowIndexNameHashMap;

    public int getRowIndex(String rowName) {

        if (rowName == null) {
            throw new IllegalArgumentException("rowName cannot be null: " + rowName);
        }

        if (fRowIndexNameHashMap == null) {
            fRowIndexNameHashMap = new TObjectIntHashMap();
            for (int r = 0; r < fRowNames.size(); r++) {
                fRowIndexNameHashMap.put(fRowNames.get(r), r);
            }
        }

        int index = fRowIndexNameHashMap.get(rowName);

        // Theres som confusion over whether missing returns 0 or -1 from Trove
        if (index == 0) {
            if (fRowIndexNameHashMap.containsKey(rowName) == false) {
                return index = -1;
            }
        }

        return index;

    //return fRowNames.indexOf(rowName);
    }

    public String[] getRowNamesArray() {
        return (String[]) fRowNames.toArray(new String[fRowNames.size()]);
    }

    public List getColumnNames() {
        return Collections.unmodifiableList(fColNames);
    }

    public List getColumnNames(final Template prefixTemplateClasses) {
        List colNames = new ArrayList(fColNames);
        for (int c = 0; c < colNames.size(); c++) {
            colNames.set(c, prefixTemplateClasses.getClassByProfilePos(c).getName() + "_" + colNames.get(c));
        }
        return colNames;
    }

    public int getColumnIndex(String colName) {
        return fColNames.indexOf(colName);
    }

    public String[] getColumnNamesArray() {
        return (String[]) fColNames.toArray(new String[fColNames.size()]);
    }

    public String getColumnName(int coln) {
        return (String) fColNames.get(coln);
    }

    public int getNumRow() {
        return fRowNames.size();
    }

    public int getNumCol() {
        return fColNames.size();
    }

    public StringMatrix getStringMatrix(){
        if (fMatrix == null) {
            throw new IllegalArgumentException("Matrix cannot be null");
        }
        return fMatrix.cloneDeep();
    }

    public String getQuickInfo() {
        if (fQuickInfo == null) {
            _setQuickInfo();
        }
        return fQuickInfo;
    }

    private void _setQuickInfo() {
        StringBuffer buf = new StringBuffer().append(getNumRow()).append('x').append(getNumCol());
        fQuickInfo = buf.toString();

    }

    public static List toList(final String[] ss) {
        final List list = new ArrayList(ss.length);
        for (int i = 0; i < ss.length; i++) {
            list.add(ss[i]);
        }

        return list;
    }

    public Vector getRow(String rowName) {
        return getRow(getRowIndex(rowName));
    }

    public String[] getColumnStr(int coln){
        String[] col = new String [getNumRow()];
        for(int i=0; i<fMatrix.getNumRow(); i++){
            col[i]=fMatrix.getElement(i, coln);
        }
        return col;
    }

    public Vector getRow(int rown) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Vector[] getRows(GeneSet gset) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Vector getColumn(int coln) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Vector getColumn(int coln, GeneSet forTheseGenes) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Vector getColumn(String colName, GeneSet forTheseGenes) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public LabelledVector getColumnLV(int coln) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public LabelledVector getColumnLV(String colName) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Vector getColumn(String colName) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Vector[] getColumns(String[] colNames) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public RankedList getColumnRL(String colName, SortMode sort, Order order) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public RankedList getColumnRL(int coln, SortMode sort, Order order) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public float getElement(int row, int column) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public float getElement(String rowName, int column) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public float getElement(String rowName, String colName) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public float getElement(int row, String colName) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public GeneSet getRowNamesGeneSet() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public GeneSet getColumnNamesGeneSet() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getDim() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    /*public edu.mit.broad.genome.math.Matrix getMatrix() {
    throw new UnsupportedOperationException("Not supported yet.");
    }*/
    public Annot getAnnot() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Matrix getMatrix() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}    // End class SNPDataset

