/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.params;


import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.swing.choosers.GFileFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.URL;
import java.util.*;


/**
 * Object to capture commandline params
 */
public class FileParam extends AbstractParam {

    private GFileFieldPlusChooser fChooser;

    /**
     * Class constructor
     *
     * @param name
     * @param type
     * @param desc
     */
    public FileParam(final String name, final String nameEnglish, final String desc, final File hint, final boolean reqd) {
        this(name, nameEnglish, desc, new File[]{hint}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hints
     * @param reqd
     */
    public FileParam(final String name, final String nameEnglish, final String desc, final File[] hints, final boolean reqd) {
        super(name, nameEnglish, File.class, desc, hints, reqd);
    }

    /**
     * no hint
     *
     * @param name
     * @param desc
     * @param reqd
     */
    public FileParam(final boolean reqd) {
        this(FILE, FILE_ENGLISH, FILE_DESC, new File[]{}, reqd);
    }

    public FileParam(final String name, final boolean reqd) {
        this(name, FILE_ENGLISH, FILE_DESC, new File[]{}, reqd);
    }

    public FileParam(final String name, final String nameEnglish, final boolean reqd) {
        this(name, nameEnglish, FILE_DESC, new File[]{}, reqd);
    }

    public FileParam(final String name, final String nameEnglish, final String desc, final boolean reqd) {
        this(name, nameEnglish, desc, new File[]{}, reqd);
    }

    public void setValue(final Object value) {

        if (value == null) {
            super.setValue(value);
        } else if (value instanceof File) {
            super.setValue(value);
        } else if (value instanceof String) {
            this.setValue(value.toString());
        } else if (value instanceof URL) {
            this.setValue(value.toString());
        } else {
            throw new IllegalArgumentException("Invalid type, only File and String accepted. Specified: " + value + " class: "
                    + value.getClass());
        }
    }

    public String getFileName_wo_ext() {
        return NamingConventions.removeExtension(getFile().getName());
    }

    public String getContent() throws IOException {
        return ParseUtils.slurp(getFile(), true);
    }

    public String[] getStrings() throws IOException {
        return ParseUtils.slurpIntoStrings(getFile(), true);
    }

    public String[] getStringsUnique() throws IOException {
        String[] ss = getStrings();
        List list = new ArrayList();
        for (int i = 0; i < ss.length; i++) {
            if (!list.contains(ss[i])) {
                list.add(ss[i]);
            }
        }

        return (String[]) list.toArray(new String[list.size()]);
    }

    public Set getStrings_set() throws IOException {
        String[] ss = getStrings();
        Set set = new HashSet();
        for (int i = 0; i < ss.length; i++) {
            set.add(ss[i]);
        }

        return set;
    }

    public Map getMap_2_field(final boolean barfOnDuplicateKeyAssocs) throws Exception {

        final String[] lines = getStringsUnique();
        final Map map = new HashMap();
        final List duplicates = new ArrayList();

        for (int i = 0; i < lines.length; i++) {
            String[] ss = ParseUtils.string2strings(lines[i], "\t", false);
            if (ss.length != 2) {
                throw new IllegalArgumentException("Bad format: " + lines[i]);
            }

            Object curr = map.get(ss[0]);
            if (curr != null) {
                String curr_s = curr.toString();
                if (curr_s.equalsIgnoreCase(ss[1])) {
                    // do nothing
                } else {
                    duplicates.add(lines[i]);
                }
            } else {
                map.put(ss[0], ss[1]);
            }
        }

        if (duplicates.isEmpty() == false) {
            klog.warn("Some NMs map to > 1 gene symbol: " + duplicates.size() + "\n");
            Printf.outl(duplicates);

            if (barfOnDuplicateKeyAssocs) {
                throw new IllegalArgumentException("Bad format");
            }
        }

        return map;
    }


    public void setValue(final File file) {
        super.setValue(file);
    }

    public void setValue(final String path) {
        super.setValue(path); // changed for the url thing Feb 2006
        //super.setValue(new File(filepath));
    }

    public void setValue_url(final String filepath) {
        super.setValue(filepath);
    }

    public boolean isFileBased() {
        return true;
    }

    public BufferedReader getReader() throws IOException {
        return new BufferedReader(new FileReader(getFile()));
    }

    public File getFile() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        if (val instanceof File) {
            return (File) val;
        } else {
            return new File(val.toString());
        }

    }

    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        } else {

            if (val instanceof URL) {
                return val.toString();
            } else if (val instanceof File) {
                final File file = (File) getValue();
                if (full) {
                    return file.getAbsolutePath();
                } else {
                    return file.getName();
                }
            } else if (NamingConventions.isURL(val.toString())) {
                return val.toString();
            } else {
                return val.toString();
            }
        }
    }

    public GFieldPlusChooser getSelectionComponent() {

        if (fChooser == null) {
            fChooser = new GFileFieldPlusChooser();
            if (getValue() != null) {
                fChooser.setValue(getValue());
            } else {
                fChooser.setValue(getDefault());
            }

            ParamHelper.addDocumentListener(fChooser.getTextField(), this);

        }

        return fChooser;
    }


}    // End class FileParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
