/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.parsers;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.AbstractParser;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserException;

import edu.duke.igsp.app.objects.SnpDataset;
import edu.duke.igsp.app.objects.StringMatrix;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */

/**
 * # some comments
 * 10095	24
 * rs#	       Chr	Pos	    073-127_Call	001-101_Call	023-117_Call	023-117re_Call	090-323_Call	109-319_Call	095--325_Call	121-329_Call	127-137_Call	143-339_Call	142-343_Call	148-341_Call	146-139_Call	072-128_Call	113-332_Call	040-302_Call	046-123_Call	097-320_Call	039-303_Call	050-308_Call	058-309_Call	061-311_Call	155-348_Call	014-109_Call
 * rs1393064   1	3088998	CC	GG	CC	CC	CC	CC	CC	CC	NN	CC	CG	CC	CC	CC	CC	CG	NN	CG	CC	CC	CG	CG	CG	CC
 * rs966321	   1	4215064	AC	AA	CC	CC	AC	AA	CC	AC	AA	CC	AA	AC	AC	AC	AC	CC	NN	AA	AA	AA	AA	AC	AC	AC
 * rs1599169   1	5034491	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG
 * rs580309	   1	5211912	CT	CT	NN	CC	CC	NN	NN	NN	NN	CT	NN	NN	NN	CC	CT	CT	NN	CT	TT	CC	CC	CC	CC	NN
 * rs1414379   1	5697261	CT	TT	TT	TT	TT	CT	CC	TT	TT	CT	TT	CT	CT	TT	TT	CT	CT	CT	CT	TT	CT	TT	TT	NN
 * rs1890191   1	5809727	AA	GG	AG	AG	AA	AG	AA	AG	AG	AA	AA	AA	AA	AG	AA	AA	AG	AA	AG	GG	AG	AG	AG	GG
 */
public class GenotypeParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public GenotypeParser() {
        super(Dataset.class);
    }

    /**
     * Export a Dataset to file in snp format
     * Only works with Datasets
     *
     * @see "Above for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {
        _export(pob, startExport(pob, file));
    }

    public void export(final PersistentObject pob, final OutputStream os) throws Exception {
        _export(pob, startExport(pob, os, null));
    }

    private void _export(final PersistentObject pob, final PrintWriter pw) throws Exception {

        final Dataset ds = (Dataset) pob;
        FeatureAnnot ann = null;
        if (ds.getAnnot() != null) {
            ann = ds.getAnnot().getFeatureAnnot();
        }

        //log.debug("Annotation is: " + ann);

        pw.println(" ");
        pw.println(ds.getNumRow() + "\t" + ds.getNumCol());
        pw.print("Probeset_id" + "\t" + "Chr" + "\t" +"Pos" + "\t");

        for (int i = 0; i < ds.getNumCol(); i++) {
            pw.print(ds.getColumnName(i));
            if (i != ds.getNumCol() - 1) {
                pw.print('\t');
            }
        }

        pw.println();

        // Give preference to Native desc if it exists
        // If not, use the symbol desc
        for (int r = 0; r < ds.getNumRow(); r++) {
            StringBuffer buf = new StringBuffer();
            String rowName = ds.getRowName(r);
            buf.append(rowName).append('\t');
            String desc = Constants.NA;
            if (ann != null) {
                if (ann.hasNativeDescriptions()) {
                    desc = ann.getNativeDesc(rowName);
                } else {
                    String symbol = ann.getGeneSymbol(rowName);
                    if (symbol != null) {
                        desc = symbol + ":" + ann.getGeneTitle(rowName);
                    }
                }
            }

            if (desc == null) {
                desc = Constants.NA;
            }

            buf.append(desc).append('\t');
            buf.append(ds.getRow(r).toString('\t'));
            pw.println(buf.toString());
        }

        pw.close();

        doneExport();
    }    // End export

    /**
     * @param name
     * @param s
     * @return
     * @throws Exception
     */
    public List parse(String objname, String s) throws Exception {

        StringReader sr = new StringReader(s);
        BufferedReader bin = new BufferedReader(sr);
        return _parse(objname, bin);
    }

    /**
     * Parse a Dataset from memory
     *
     * @param s
     * @return
     * @throws Exception
     */
    public List parse(String s) throws Exception {
        String name = NamingConventions.generateName(Constants.SNP);
        return parse(name, s);
    }

    /**
     * @returns 1 Dataset object
     * Always DatasetAnnotation object produced , but if underlying df has none then na is used
     * @see above for format
     */
    public List parse(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        return _parse(sourcepath, bin);

    }

    /// does the real parsing
    // expects the bin to be untouched
    private List _parse(String objName, BufferedReader bin) throws Exception {
        objName = NamingConventions.removeExtension(objName);
        String currLine = nextLine(bin);

        // 1st  non-empty, non-comment line is numrows and numcols
        int[] nstuff = ParseUtils.string2ints(currLine, " \t");
        if (nstuff.length != 2) {
            throw new ParserException("Snp file with bad row/col info on line: " + currLine);
        }

        int nrows = nstuff[0];
        int ncols = nstuff[1];

        // First 2 fields name and desc are to be ignored
        currLine = nextLine(bin);
        List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

        colnames.remove(0);            // remove rs#
        colnames.remove(0);            // remove chr
        colnames.remove(0);            // remove pos

        if (colnames.size() != ncols) {
            throw new ParserException("Bad snp format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
        }

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List lines = new ArrayList();

        currLine = nextLineTrimless(bin);

        // save all rows so that we can determine how many rows exist
        while (currLine != null) {
            lines.add(currLine);
            currLine = nextLineTrimless(bin); /// imp for mv datasets -> last col(s) can be a tab
        }

        if (lines.size() != nrows) {
            throw new ParserException("Bad snp format -- exepcted nrows from specification on header line: " + nrows + " but found in data: " + lines.size());
        }

        bin.close();

        return _parseHasDesc(objName, lines, colnames);
    }

    public List[] parseRowAndColNamesOnly(final File dsFile) throws Exception {

        final BufferedReader bin = new BufferedReader(new FileReader(dsFile));

        String currLine = nextLine(bin);

        // 1st  non-empty, non-comment line is numrows and numcols
        int[] nstuff = ParseUtils.string2ints(currLine, " \t");
        if (nstuff.length != 2) {
            throw new ParserException("Snp file with bad row/col info on line: " + currLine);
        }

        int nrows = nstuff[0];
        int ncols = nstuff[1];

        // First 2 fields name and desc are to be ignored
        currLine = nextLine(bin);
        final List colNames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

        colNames.remove(0);
        colNames.remove(0);
        colNames.remove(0);

        if (colNames.size() != ncols) {
            throw new ParserException("Bad snp format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colNames.size());
        }

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List rowNames = new ArrayList();

        currLine = nextLineTrimless(bin);

        // save all rows so that we can determine how many rows exist
        // Makes this ULTRA fasts - thats the point
        while (currLine != null) {
            StringTokenizer tok = new StringTokenizer(currLine, "\t");
            rowNames.add(tok.nextToken());
            currLine = nextLineTrimless(bin); /// imp for mv datasets -> last col(s) can be a tab
        }

        if (rowNames.size() != nrows) {
            throw new ParserException("Bad snp format -- exepcted nrows from specification on header line: " + nrows + " but found in data: " + rowNames.size());
        }

        bin.close();

        return new List[]{rowNames, colNames};
    }

    private List _parseHasDesc(String objName, List lines, List colNames) throws Exception {
        objName = NamingConventions.removeExtension(objName);
        StringMatrix matrix = new StringMatrix(lines.size(), colNames.size());
        List rowNames = new ArrayList();
        ArrayList rowDescs = new ArrayList();

        for (int i = 0; i < lines.size(); i++) {
            String currLine = (String) lines.get(i);
            List fields = string2stringsV2(currLine, colNames.size() + 3); // spaces allowed in name & desc field so DONT tokenize them

            if (fields.size() != colNames.size() + 3) {
                //System.out.println(">> " + fields);
                throw new ParserException("Bad format - expect ncols: " + (colNames.size() + 3)
                        + " but found: " + fields.size() + " on line >"
                        + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
            }

            String rowname = fields.get(0).toString().trim();
            if (rowname.length() == 0) {
                throw new ParserException("Bad rowname - cant be empty at: " + i + " >" + currLine);
            }

            String chr = fields.get(1).toString().trim();
            String pos = fields.get(2).toString().trim();
            if (chr.length() == 0) {
                throw new ParserException("Bad chromosome - cant be empty at: " + i + " >" + currLine);
            }
            if (pos.length() == 0) {
                throw new ParserException("Bad position - cant be empty at: " + i + " >" + currLine);
            }

            ArrayList desc=new ArrayList();
            //desc.add(rowname);
            desc.add(chr);
            desc.add(pos);
            rowDescs.add(desc);
            rowNames.add(rowname);

            for (int f = 3; f < fields.size(); f++) {
                String s = fields.get(f).toString().trim();
                if (s.length() == 0) {
                    s = "NN";
                }
                matrix.setElement(i, f - 3, s);
            }
        }

        final SnpDataset ds = new SnpDataset(objName, matrix, rowNames, colNames, rowDescs);
        doneImport();
        return unmodlist(new PersistentObject[]{ds});
    }

}    // End of class GenotypeParser


