/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.parsers;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.AbstractParser;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserException;

import edu.duke.igsp.app.objects.NumDataset;
import edu.mit.broad.genome.math.Matrix;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: July, 2009
 *
 */

/**
*	#	some	comments
*	10095	24
*	rs#	Chr	Pos	Sample1	Sample2	Sample3	Sample4	Sample5	Sample6	Sample7
*	rs1393064	1	3088998	46.05	82.17	16.87	98.6	141.02	114.32	134.34
*	rs966321	1	4215064	108.34	59.04	25.61	33.11	42.53	9.12	9.36
*	rs1599169	1	5034491	7.43	13.94	8.55	21.13	15.09	19.05	16.47
*	rs580309	1	5211912	11.4	3	3.16	2.34	4.43	1.56	6.04
*	rs1414379	1	5697261	148.09	317.17	316.66	147.23	125.78	261.39	268.41
*	rs1890191	1	5809727	8.62	9.12	1572.53	5.91	5.31	11.98	128.77
*/
public class NumDatasetParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public NumDatasetParser() {
        super(Dataset.class);
    }

    /**
     * Export a Dataset to file
     * Only works with Datasets
     *
     * @see "Above for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {
        _export(pob, startExport(pob, file));
    }

    public void export(final PersistentObject pob, final OutputStream os) throws Exception {
        _export(pob, startExport(pob, os, null));
    }

    private void _export(final PersistentObject pob, final PrintWriter pw) throws Exception {

        final Dataset ds = (Dataset) pob;
        FeatureAnnot ann = null;
        if (ds.getAnnot() != null) {
            ann = ds.getAnnot().getFeatureAnnot();
        }

        //log.debug("Annotation is: " + ann);

        pw.println(" ");
        pw.println(ds.getNumRow() + "\t" + ds.getNumCol());
        pw.print("Probeset_id" + "\t" + "Chr" + "\t" +"Pos" + "\t");

        for (int i = 0; i < ds.getNumCol(); i++) {
            pw.print(ds.getColumnName(i));
            if (i != ds.getNumCol() - 1) {
                pw.print('\t');
            }
        }

        pw.println();

        // Give preference to Native desc if it exists
        // If not, use the symbol desc
        for (int r = 0; r < ds.getNumRow(); r++) {
            StringBuffer buf = new StringBuffer();
            String rowName = ds.getRowName(r);
            buf.append(rowName).append('\t');
            String desc = Constants.NA;
            if (ann != null) {
                if (ann.hasNativeDescriptions()) {
                    desc = ann.getNativeDesc(rowName);
                } else {
                    String symbol = ann.getGeneSymbol(rowName);
                    if (symbol != null) {
                        desc = symbol + ":" + ann.getGeneTitle(rowName);
                    }
                }
            }

            if (desc == null) {
                desc = Constants.NA;
            }

            buf.append(desc).append('\t');
            buf.append(ds.getRow(r).toString('\t'));
            pw.println(buf.toString());
        }

        pw.close();

        doneExport();
    }    // End export

    /**
     * @param name
     * @param s
     * @return
     * @throws Exception
     */
    public List parse(String objname, String s) throws Exception {

        StringReader sr = new StringReader(s);
        BufferedReader bin = new BufferedReader(sr);
        return _parse(objname, bin);
    }

    /**
     * Parse a Dataset from memory
     *
     * @param s
     * @return
     * @throws Exception
     */
    public List parse(String s) throws Exception {
        String name = NamingConventions.generateName(Constants.NUM);
        return parse(name, s);
    }

    /**
     * @returns 1 Dataset object
     * Always DatasetAnnotation object produced , but if underlying df has none then na is used
     * @see above for format
     */
    public List parse(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        return _parse(sourcepath, bin);

    }

    /// does the real parsing
    // expects the bin to be untouched
    private List _parse(String objName, BufferedReader bin) throws Exception {
        objName = NamingConventions.removeExtension(objName);
        String currLine = nextLine(bin);

        // 1st  non-empty, non-comment line is numrows and numcols
        int[] nstuff = ParseUtils.string2ints(currLine, " \t");
        if (nstuff.length != 2) {
            throw new ParserException("file with bad row/col info on line: " + currLine);
        }

        int nrows = nstuff[0];
        int ncols = nstuff[1];

        // First 2 fields name and desc are to be ignored
        currLine = nextLine(bin);
        List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

        colnames.remove(0);            // remove rs#
        colnames.remove(0);            // remove chr
        colnames.remove(0);            // remove pos

        if (colnames.size() != ncols) {
            throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
        }

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List lines = new ArrayList();

        currLine = nextLineTrimless(bin);

        // save all rows so that we can determine how many rows exist
        while (currLine != null) {
            lines.add(currLine);
            currLine = nextLineTrimless(bin); /// imp for mv datasets -> last col(s) can be a tab
        }

        if (lines.size() != nrows) {
            throw new ParserException("Bad format -- exepcted nrows from specification on header line: " + nrows + " but found in data: " + lines.size());
        }

        bin.close();

        return _parseHasDesc(objName, lines, colnames);
    }

    public List[] parseRowAndColNamesOnly(final File dsFile) throws Exception {

        final BufferedReader bin = new BufferedReader(new FileReader(dsFile));

        String currLine = nextLine(bin);

        // 1st  non-empty, non-comment line is numrows and numcols
        int[] nstuff = ParseUtils.string2ints(currLine, " \t");
        if (nstuff.length != 2) {
            throw new ParserException("file with bad row/col info on line: " + currLine);
        }

        int nrows = nstuff[0];
        int ncols = nstuff[1];

        // First 2 fields name and desc are to be ignored
        currLine = nextLine(bin);
        final List colNames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

        colNames.remove(0);
        colNames.remove(0);
        colNames.remove(0);

        if (colNames.size() != ncols) {
            throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colNames.size());
        }

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List rowNames = new ArrayList();

        currLine = nextLineTrimless(bin);

        // save all rows so that we can determine how many rows exist
        // Makes this ULTRA fasts - thats the point
        while (currLine != null) {
            StringTokenizer tok = new StringTokenizer(currLine, "\t");
            rowNames.add(tok.nextToken());
            currLine = nextLineTrimless(bin); /// imp for mv datasets -> last col(s) can be a tab
        }

        if (rowNames.size() != nrows) {
            throw new ParserException("Bad format -- exepcted nrows from specification on header line: " + nrows + " but found in data: " + rowNames.size());
        }

        bin.close();

        return new List[]{rowNames, colNames};
    }

    private List _parseHasDesc(String objName, List lines, List colNames) throws Exception {
        objName = NamingConventions.removeExtension(objName);
        Matrix matrix = new Matrix(lines.size(), colNames.size());
        List rowNames = new ArrayList();
        ArrayList rowDescs = new ArrayList();

        for (int i = 0; i < lines.size(); i++) {
            String currLine = (String) lines.get(i);
            List fields = string2stringsV2(currLine, colNames.size() + 3); // spaces allowed in name & desc field so DONT tokenize them

            if (fields.size() != colNames.size() + 3) {
                //System.out.println(">> " + fields);
                throw new ParserException("Bad format - expect ncols: " + (colNames.size() + 3)
                        + " but found: " + fields.size() + " on line >"
                        + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
            }

            String rowname = fields.get(0).toString().trim();
            if (rowname.length() == 0) {
                throw new ParserException("Bad rowname - cant be empty at: " + i + " >" + currLine);
            }

            String chr = fields.get(1).toString().trim();
            String pos = fields.get(2).toString().trim();
            if (chr.length() == 0) {
                throw new ParserException("Bad chromosome - cant be empty at: " + i + " >" + currLine);
            }
            if (pos.length() == 0) {
                throw new ParserException("Bad position - cant be empty at: " + i + " >" + currLine);
            }

            ArrayList desc=new ArrayList();
            //desc.add(rowname);
            desc.add(chr);
            desc.add(pos);
            rowDescs.add(desc);
            rowNames.add(rowname);

            for (int f = 3; f < fields.size(); f++) {
                String s = fields.get(f).toString().trim();
                float val;
                if (s.length() == 0) {
                    val = Float.NaN;
                } else {
                    try {
                        val = Float.parseFloat(s);
                    } catch (Exception e) {
                        System.out.println(">" + s + "<");
                        //val = Float.NaN;
                        throw e;
                    }
                }
                matrix.setElement(i, f - 3, val);
            }
        }

        final NumDataset ds = new NumDataset(objName, matrix, rowNames, colNames, rowDescs);
        doneImport();
        return unmodlist(new PersistentObject[]{ds});
    }

}    // End of class NumDatasetParser



