/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.parsers;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserException;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import edu.duke.igsp.app.objects.GeneBean;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.HashMap;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: July, 2009
 *
 */
/**
 * # some comments
 * 10095	24
 * rs#	       Chr	Pos	    073-127_Call	001-101_Call	023-117_Call	023-117re_Call	090-323_Call	109-319_Call	095--325_Call	121-329_Call	127-137_Call	143-339_Call	142-343_Call	148-341_Call	146-139_Call	072-128_Call	113-332_Call	040-302_Call	046-123_Call	097-320_Call	039-303_Call	050-308_Call	058-309_Call	061-311_Call	155-348_Call	014-109_Call
 * rs1393064   1	3088998	CC	GG	CC	CC	CC	CC	CC	CC	NN	CC	CG	CC	CC	CC	CC	CG	NN	CG	CC	CC	CG	CG	CG	CC
 * rs966321	   1	4215064	AC	AA	CC	CC	AC	AA	CC	AC	AA	CC	AA	AC	AC	AC	AC	CC	NN	AA	AA	AA	AA	AC	AC	AC
 * rs1599169   1	5034491	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG
 * rs580309	   1	5211912	CT	CT	NN	CC	CC	NN	NN	NN	NN	CT	NN	NN	NN	CC	CT	CT	NN	CT	TT	CC	CC	CC	CC	NN
 * rs1414379   1	5697261	CT	TT	TT	TT	TT	CT	CC	TT	TT	CT	TT	CT	CT	TT	TT	CT	CT	CT	CT	TT	CT	TT	TT	NN
 * rs1890191   1	5809727	AA	GG	AG	AG	AA	AG	AA	AG	AG	AA	AA	AA	AA	AG	AA	AA	AG	AA	AG	GG	AG	AG	AG	GG
 */
public class SnpGeneStrucParser {

    private static final Logger xlog = XLogger.getLogger(NumGeneStrucParser.class);

    public HashMap parse(ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        HashMap dist = new HashMap();

        int total = 0;
        int valid = 0;
        ArrayList cache = new ArrayList();
        try {
            FileReader input = new FileReader(path);
            BufferedReader br = new BufferedReader(input);
            String currLine = nextLine(br);

            // 1st  non-empty, non-comment line is numrows and numcols
            int[] nstuff = ParseUtils.string2ints(currLine, "\t");
            if (nstuff.length != 2) {
                throw new ParserException("file with bad row/col info on line: " + currLine);
            }

            int nrows = nstuff[0];
            int ncols = nstuff[1];

            currLine = nextLine(br);
            List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

            colnames.remove(0);            // remove rs#
            colnames.remove(0);            // remove chr
            colnames.remove(0);            // remove pos

            if (colnames.size() != ncols) {
                throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
            }

            if (colnames.size() != itemsOfTemplate) {
                throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
            }

            currLine = nextLineTrimless(br);

            for (int i = 0; i < genes.size(); i++) {
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                int start = ((GeneBean) genes.get(i)).getStart() - before;
                if (start < 0) {
                    start = 0;
                }
                int end = ((GeneBean) genes.get(i)).getEnd();
                if (end == 0) {
                    continue;
                } else {
                    end = end + after;
                }

                ArrayList probes = new ArrayList(cache);
                if (cache != null && cache.size() != 0) {
                    cache.clear();
                }
                if (probes != null && probes.size() != 0) {
                    for (int j = 0; j < probes.size(); j++) {
                        int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
                        if (start <= pos_c) {
                            ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
                            cache.add(probe_c);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                        } else {
                            probes.remove(j);
                            j--;
                        }

                    }
                }

                while ((currLine != null) && (currLine.length() != 0)) {
                    total++;
                    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

                    if (fields.size() != ncols + 3) {
                        //System.out.println(">> " + fields);
                        throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
                    }
                    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
                    int probePos = Integer.parseInt(fields.get(2).toString().trim());
                    if (geneChr != probeChr) {
                        if (geneChr < probeChr) {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            total--;
                            break;
                        } else {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            if (probes != null && probes.size() != 0) {
                                probes.clear();
                            }
                            currLine = nextLineTrimless(br);
                            continue;
                        }
                    }

                    if (start <= probePos && probePos <= end) {
                        cache.add(fields.clone());

                        fields.remove(0);
                        fields.remove(0);
                        fields.remove(0);
                        probes.add(fields);
                        valid++;
                    }

                    if (end < probePos) {
                        total--;
                        break;
                    }

                    currLine = nextLineTrimless(br);
                }

                removeHomo(probes);
                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    dist.put(geneSymbol, probes);

                }
            }

            xlog.info("There are " + total + " probes in the original dataset");
            xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dist;
    }

    protected String nextLine(final BufferedReader bin) throws IOException {
        String currLine = bin.readLine();
        if (currLine == null) {
            return null;
        }

        currLine = currLine.trim();

        while ((currLine != null) && ((currLine.length() == 0) || (currLine.startsWith(Constants.COMMENT_CHAR)))) {
            // System.out.println(">>> " + currLine);
            currLine = bin.readLine();
            if (currLine != null) {
                currLine = currLine.trim();
            }
        }
        return currLine;
    }

    protected String nextLineTrimless(BufferedReader bin) throws IOException {
        String currLine = bin.readLine();
        if (currLine == null) {
            return null;
        }

        //currLine = currLine.trim();

        while ((currLine != null) && ((currLine.length() == 0) || (currLine.startsWith(Constants.COMMENT_CHAR)))) {
            currLine = bin.readLine();
        }

        return currLine;
    }

    public static void removeHomo(ArrayList profile) {
        if (profile != null && profile.size() != 0) {
            for (int i = 0; i < profile.size(); i++) {
                ArrayList bases = new ArrayList();
                for (int j = 0; j < ((ArrayList) profile.get(i)).size(); j++) {
                    String s = (String) ((ArrayList) profile.get(i)).get(j);
                    for (int k = 0; k < s.length(); k++) {
                        bases.add(new Character(s.charAt(k)));
                    }
                }

                //A, C, G, T, N
                int[] freq = new int[5];
                for (int j = 0; j < bases.size(); j++) {
                    char c = ((Character) (bases.get(j))).charValue();
                    switch (c) {
                        case 'A':
                            freq[0]++;
                            break;
                        case 'C':
                            freq[1]++;
                            break;
                        case 'G':
                            freq[2]++;
                            break;
                        case 'T':
                            freq[3]++;
                            break;
                        case 'N':
                            freq[4]++;
                            break;
                        default:
                            break;
                    }
                }

                int flag = 0;
                for (int j = 0; j < freq.length - 1; j++) {
                    if (freq[j] > 0) {
                        flag++;
                    }
                }
                if (flag == 1) {
                    profile.remove(i);
                    i--;
                }
            }
        }
    }

    private static void enforceEqualSize(final ArrayList al) {
        if (al.size() == 0) {
            return;
        }

        int size = ((ArrayList) al.get(0)).size();
        for (int i = 0; i < al.size(); i++) {
            if (((ArrayList) al.get(i)).size() != size) {
                throw new IllegalArgumentException("Vector lengths not equal first=" + size + " and y=" + ((ArrayList) al.get(i)).size() + " at index " + i);
            }
        }
    }

    protected ArrayList string2stringsV2(String s, int expectedLen) throws IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        String delim = "\t"; // @note, always
        StringTokenizer tok = new StringTokenizer(s, delim, true); // note including the delim in rets
        ArrayList ret = new ArrayList();
        String prev = null;
        //System.out.println("PARSING>" + s + "<"  + tok.countTokens());

        int cnt = 0;
        while (tok.hasMoreTokens()) {
            final String curr = tok.nextToken(); // dont trim as curr might be a tab!

            if (cnt == 0) { // the first field
                ret.add(curr.trim()); // always add it, empty or not
            } else {
                if (curr.equals(delim)) {
                    if (prev.equals(delim)) { // 2 consecutive tabs
                        ret.add(""); //empty field
                    } else { // omit because its _the delim_
                    }
                } else {
                    ret.add(curr.trim()); // a real word, ok to trim. Then add
                }
            }

            prev = curr;
            cnt++;
        }

        if (ret.size() == expectedLen) {
            return ret;
        } else if (ret.size() < expectedLen) {
            /// fill out whatever's left with empty
            for (int i = ret.size(); i < expectedLen; i++) {
                ret.add("");
            }
            return ret;

        } else {
            // @note added Nov 28, 2005
            // delete any extra tabs (ret.size() > expectedLen
            ArrayList real_ret = new ArrayList();
            for (int i = 0; i < ret.size(); i++) {
                if (i < expectedLen) {
                    real_ret.add(ret.get(i));
                } else {
                    // dont add empty
                    Object obj = ret.get(i);
                    if (obj == null || obj.toString().trim().length() == 0) {
                        // dont add
                    } else {
                        real_ret.add(obj); // cant do anything  might be a genuine format error
                    }
                }
            }

            return real_ret;
        }
    }
}

