/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb;

import edu.mit.broad.genome.NotImplementedException;

import java.util.HashSet;
import java.util.Set;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Organism {


    public static final Organism GENERIC = new Organism("Generic", -999, null);
    public static final Organism XSPECIES = new Organism("Human,Mouse,Rat,Dog", -999, null);

    public static final Organism CHIMPANZEE = new Organism("Chimpanzee", -999, null);
    public static final Organism PIG = new Organism("Pig", -999, null);
    public static final Organism RHESUS = new Organism("Rhesus", -999, null);

    public static final Organism HUMAN = new Organism("Homo sapiens", 9606, "Hs");
    public static final Organism ZEBRA_FISH = new Organism("Zebra Fish", 7955, "Dr");
    public static final Organism HUMAN_en = new Organism("Human", 9606, "Hs");
    public static final Organism MOUSE = new Organism("Mus musculus", 10090, "Mm");
    public static final Organism MOUSE_en = new Organism("Mouse", 10090, "Mm");
    public static final Organism DROSOPHILLA = new Organism("Drosophilla", 7227, "Dm");
    public static final Organism RAT = new Organism("Rattus norvegicus", 10116, "Rn");
    public static final Organism RAT_EN = new Organism("Rat", 10116, "Rn");
    public static final Organism YEAST = new Organism("Saccharomyces cerevisiae", 4932, "Sc");
    public static final Organism ARABIDOPSIS = new Organism("Arabidopsis thaliana", 3702, "At");
    public static final Organism ECOLI = new Organism("Escherichia coli", 562, "Ec");
    public static final Organism CELEGANS = new Organism("Caenorhabditis elegans", 6239, "Ce");
    public static final Organism MALARIA = new Organism("Plasmodium falciparum", 5833, "Pf");
    public static final Organism DANIO = new Organism("Danio rerio", 5811, "Dr");
    public static final Organism MAC = new Organism("Macaca mulatta", 5891, "Mi");

    public static final Organism[] ALL_TAGS = new Organism[]{HUMAN, MOUSE, RAT, YEAST, DROSOPHILLA, XSPECIES};
    public static final Organism[] ALL_TAGS_en = new Organism[]{GENERIC, ZEBRA_FISH, PIG, CHIMPANZEE, HUMAN, HUMAN_en,
            MOUSE, MOUSE_en, RAT, RAT_EN, YEAST, DROSOPHILLA, XSPECIES, RHESUS, DANIO, MAC};

    private String fValue;

    private int fTaxonID;

    private String fUnigeneCode;

    public static Set getTaxIds(final Organism[] orgs) {
        Set all = new HashSet();
        for (int i = 0; i < orgs.length; i++) {
            all.add("" + orgs[i].getTaxonID());
        }

        return all;
    }

    public static Set getUnigeneCodes(final Organism[] orgs) {
        Set all = new HashSet();
        for (int i = 0; i < orgs.length; i++) {
            all.add("" + orgs[i].getUnigeneCode());
        }

        return all;
    }

    public static String[] getNames(final Organism[] orgs) {
        String[] names = new String[orgs.length];
        for (int i = 0; i < orgs.length; i++) {
            names[i] = orgs[i].getName();
        }

        return names;
    }

    /**
     * privatized constructor.
     * Use one of the declared objects.
     */
    private Organism(final String name, final int taxonId, final String unigene2LetterCode) {
        this.fValue = name;
        this.fTaxonID = taxonId;
        this.fUnigeneCode = unigene2LetterCode;
    }


    /**
     * Cloing mechanism
     *
     * @param org
     */
    public Organism(Organism org) {
        this.fValue = org.fValue;
    }

    public Object getValue() {
        return fValue;
    }

    public String toString() {
        return fValue;
    }

    public String toFormattedString() {
        return fValue;
    }

    public String getName() {
        return fValue;
    }

    public String getUnigeneCode() {
        if (fUnigeneCode == null) {
            throw new IllegalStateException("No known unigene code for organism: " + fUnigeneCode);
        }
        return fUnigeneCode;
    }

    public int getTaxonID() {
        if (fTaxonID == -999) {
            throw new NotImplementedException(getName());
        } else {
            return fTaxonID;
        }
    }

    public boolean isHuman() {
        return fTaxonID == 9606;
    }

    public boolean isGeneric() {
        return getName().equalsIgnoreCase("Generic");
    }

    public boolean isMouse() {
        return fTaxonID == 10090;
    }

    public boolean equals(String org) {
        if (org == null) {
            return false;
        } else {
            return fValue.equalsIgnoreCase(org);
        }
    }

    public boolean equals(Organism org) {
        if (org == null) {
            return false;
        } else {
            return equals(org.fValue);
        }
    }


    public static Organism parseOrganism(String fromValue) {

        if (fromValue == null) {
            throw new NullPointerException("Param fromvalue cannot be null");
        }

        fromValue = fromValue.trim();

        for (int i = 0; i < ALL_TAGS_en.length; i++) {
            if (fromValue.equalsIgnoreCase(ALL_TAGS_en[i].toFormattedString())) {
                return ALL_TAGS_en[i];
            }
        }

        throw new IllegalArgumentException("Cannot parse Organism from: " + fromValue);
    }
}    // Organism

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
