/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb;

import edu.mit.broad.genome.*;
import edu.mit.broad.genome.alg.ComparatorFactory;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.*;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.vdb.chip.*;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.Chip2ChipMapperSlimJit;
import edu.mit.broad.vdb.map.Chip2ChipMapperSymbol;
import edu.mit.broad.vdb.meg.*;
import edu.mit.broad.xbench.core.api.Application;
import org.apache.log4j.Logger;
import org.genepattern.uiutil.FTPList;
import xapps.gsea.GseaWebResources;

import java.io.File;
import java.io.FilenameFilter;
import java.io.Serializable;
import java.net.URL;
import java.util.*;

/**
 * Vdb related resources that are available at runtime
 * This class is very-gsea desktop application related. You likely do not want to use it if you are packaging
 * gsea in some other mode.
 */

public class VdbRuntimeResources {

    private static final Logger klog = XLogger.getLogger(VdbRuntimeResources.class);

    static Set affy_prefixes_hack = new HashSet();

    static {
        affy_prefixes_hack.add("HG");
        affy_prefixes_hack.add("HC");
        affy_prefixes_hack.add("HU");
        affy_prefixes_hack.add("Hu");
        affy_prefixes_hack.add("MG");
        affy_prefixes_hack.add("MOE");
        affy_prefixes_hack.add("Mu");
        affy_prefixes_hack.add("Mouse");
    }

    public static String entrezGeneId2GeneSymbol(final String ezid, final boolean onlyHuman) throws Exception {
        return getEntrezGeneDb(onlyHuman).getSymbol(ezid);
    }

    public static Set entrezGeneId2GeneSymbol(final Set ezids, final boolean onlyHuman) throws Exception {
        return getEntrezGeneDb(onlyHuman).getSymbols(ezids);
    }

    public static String entrezGeneId2GeneSymbol_one_or_null(final Set ezids, final boolean onlyHuman) throws Exception {
        Set set = entrezGeneId2GeneSymbol(ezids, onlyHuman);
        if (set == null || set.isEmpty()) {
            return null;
        } else {
            return ((String[]) set.toArray(new String[set.size()]))[0];
        }
    }

    private static EntrezGeneDb kEntrezGeneDb;

    public static EntrezGeneDb getEntrezGeneDb(final boolean onlyHuman) throws Exception {
        if (kEntrezGeneDb == null) {
            EntrezGeneDbParser parser = new EntrezGeneDbParser();
            File file = new File(Application.getVdbManager().getDatabasesDir(), "gene_info");
            kEntrezGeneDb = parser.parseEntrezDb(file, onlyHuman);
        }

        return kEntrezGeneDb;
    }

    public static class XChip {

        public static int indexOf(final String letter, final String number, final List sampleNames) {

            for (int i = 0; i < sampleNames.size(); i++) {
                String sn = sampleNames.get(i).toString();
                int index = sn.indexOf(letter + number);
                if (index != -1) {
                    return i;
                }
            }

            return -1;
        }

        public static String[] getHTALetters() {
            return new String[]{"A", "B", "C", "D", "E", "F", "G", "H"};
        }

        public static String[] getHTANumbers() {
            return new String[]{"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"};
        }

        public static String[] getCelFileNames(final String barcode) {

            final String[] letters = getHTALetters();
            final String[] numbers = getHTANumbers();
            final String[] names = new String[96];

            int cnt = 0;
            for (int r = 0; r < letters.length; r++) {
                for (int c = 0; c < numbers.length; c++) {
                    names[cnt++] = barcode + "." + letters[r] + numbers[c];
                }
            }

            //Printf.outl(names);
            return names;
        }

        // some cel files might be missing
        public static File[] getCelFiles(final String barcode, final boolean omitMissingOnes) throws Exception {

            final String[] letters = getHTALetters();
            final String[] numbers = getHTANumbers();
            final List files = new ArrayList();

            for (int r = 0; r < letters.length; r++) {
                for (int c = 0; c < numbers.length; c++) {
                    File file = getCelFile(barcode + "." + letters[r] + numbers[c], false);
                    if (omitMissingOnes) {
                        if (file != null && file.exists()) {
                            files.add(file);
                        }
                    } else {
                        files.add(file); // always add
                    }
                }
            }

            return (File[]) files.toArray(new File[files.size()]);
        }

        public static File[] getTxtFiles(final String barcode, final boolean omitMissingOnes) throws Exception {

            final String[] letters = getHTALetters();
            final String[] numbers = getHTANumbers();
            final List files = new ArrayList();

            for (int r = 0; r < letters.length; r++) {
                for (int c = 0; c < numbers.length; c++) {
                    File file = getTxtFile(barcode + "." + letters[r] + numbers[c], false);
                    if (omitMissingOnes) {
                        if (file != null && file.exists()) {
                            files.add(file);
                        }
                    } else {
                        files.add(file); // always add
                    }
                }
            }

            return (File[]) files.toArray(new File[files.size()]);
        }

        public static APMMatrix getAPMMatrix(final String[] scanNames) throws Exception {
            File[] txtFiles = getTxtFiles(scanNames, true);
            return ((DefaultDataset) MiscParsers.getDatasetFromTxtFiles("foo", txtFiles, true)).getAPMMatrix();
        }

        public static Dataset addAPMMatrix(final Dataset ds) throws Exception {
            final File[] txtFiles = getTxtFiles(ds.getColumnNamesArray(), true);
            final APMMatrix apm = ((DefaultDataset) MiscParsers.getDatasetFromTxtFiles("foo", txtFiles, true)).getAPMMatrix();
            ((DefaultDataset) ds).cloneShallow(apm);
            return ds;
        }

        static boolean doneOnce = false;

        public static File getTxtFile(final String txtFileOrSampleName_or_path,
                                      final boolean barfOnMissing) {
            throw new NotImplementedException();
        }

        public static File[] getTxtFiles(final String[] scanNames, final boolean barfOnMissingOnes) throws Exception {

            final List files = new ArrayList();

            for (int r = 0; r < scanNames.length; r++) {
                File file = getTxtFile(scanNames[r], barfOnMissingOnes);
                files.add(file);
            }

            return (File[]) files.toArray(new File[files.size()]);
        }

    } // End class XChip


    public static String[] getPossibleTxtFileNames(String txtFileName) {

        // catch a common error
        if (txtFileName.endsWith(".CEL") || txtFileName.endsWith(".cel")) {
            txtFileName = txtFileName.substring(0, txtFileName.length() - 4);
        }

        // case INsensitive
        return new String[]{
                txtFileName, // directly -- no extension
                txtFileName + ".TXT",
                txtFileName + ".TXT.gz",
                txtFileName + ".TXT.bz2",
                txtFileName + ".txt",
                txtFileName + ".txt.gz",
                txtFileName + ".txt.bz2",
        };
    }


    public static String[] getPossibleCelFileNames(final String celFileName) {
        // case INsensitive
        return new String[]{
                celFileName, // directly -- no extension
                celFileName + ".CEL",
                celFileName + ".CEL.gz",
                celFileName + ".CEL.bz2",
                celFileName + ".cel",
                celFileName + ".cel.gz",
                celFileName + ".cel.bz2",
        };
    }

    public static File[] getCelFiles(final String[] celFileOrSampleName_or_path) throws Exception {
        final Errors errors = new Errors();
        final File[] files = new File[celFileOrSampleName_or_path.length];

        for (int i = 0; i < celFileOrSampleName_or_path.length; i++) {
            try {
                files[i] = getCelFile(celFileOrSampleName_or_path[i], true);
            } catch (Exception e) {
                errors.add(e);
            }
        }

        errors.barfIfNotEmpty();

        return files;
    }

    public static File getCelFile(final String celFileOrSampleName_or_path, final boolean barfOnMissing) throws Exception {

        final String[] names = VdbRuntimeResources.getPossibleCelFileNames(celFileOrSampleName_or_path);
        final File[] rootDirs = new File[]{VdbRuntimeResources.getXChipCelFileDir()};
        return getFile(names, rootDirs, celFileOrSampleName_or_path, barfOnMissing);
    }


    public static File getFile(final String[] possible_names,
                               final File[] rootDirs,
                               final String fileName,
                               final boolean barfOnMissing) {

        if (fileName == null) {
            throw new IllegalArgumentException("Param celFileOrSampleName_or_path cannot be null");
        }

        // as is (i.e pwd);
        for (int i = 0; i < possible_names.length; i++) {
            File file = new File(possible_names[i]);
            if (file.exists()) {
                return file;
            }
        }

        for (int d = 0; d < rootDirs.length; d++) {
            for (int i = 0; i < possible_names.length; i++) {
                File file = new File(rootDirs[d], possible_names[i]);
                if (file.exists()) {
                    return file;
                }
            }
        }

        if (barfOnMissing) {
            throw new RuntimeException("Cannot find file for: " + fileName);
        } else {
            return null;
        }
    }


    public static File getXChipCelFileDir() {
        return new File("/xchip/data01/Affy/Genechip/RawCelFiles");
    }

    // @note michale said there a new partition data03 with all the txt files (Jan 2006)
    public static File getXChipTxtFileDir() {
        return new File("/xchip/data03/Affy/Genechip/TxtFiles");
    }

    // -------------------------------------------------------------------------------------------- //
    // ------------------------FLAT FILE DATABASES - USUALLY ONLY ON UNIX ------------------------- //
    // -------------------------------------------------------------------------------------------- //

    private static Unigene kUnigene;

    // There are ~50000 unigene records
    public static Unigene getUnigene(boolean keepInMemory) throws Exception {
        Unigene unigene;
        if (kUnigene == null) {
            unigene = new UnigeneParser().parseUnigene(new File(Application.getVdbManager().getDatabasesDir(), "Hs.data.txt"), true);
            if (keepInMemory) {
                kUnigene = unigene;
            }
        } else {
            unigene = kUnigene;
        }

        return unigene;
    }

    private static GeneDb kGeneDb;

    public static GeneDb getHugoDb(boolean keepInMemory) throws Exception {

        GeneDb hugodb;
        if (kGeneDb == null) {
            hugodb = new HugoDbParser().parseHugoDb(new File(Application.getVdbManager().getDatabasesDir(), "Hs.Hugo.txt"));
            if (keepInMemory) {
                kGeneDb = hugodb;
            }
        } else {
            hugodb = kGeneDb;
        }

        return hugodb;
    }


    private static AliasDb kAliasDb;

    public static AliasDb getAliasDb() throws Exception {

        if (kAliasDb == null) {
            //kAliasDb = new AliasDbImpl(new File(kDatabasesDir, "Hs.Aliases.txt"), null);
            kAliasDb = new AliasDbImpl(null, getChip_Gene_Symbol()); // @note change Oct 2006
        }

        return kAliasDb;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * @maint if a chip is added that you dont have annotations data for, add to the list here
     */
    private static WeakHashMap kChipNameChipFileObject = new WeakHashMap();

    static FilenameFilter createArrayAnnotationFnf() {
        return new FilenameFilter() {
            public boolean accept(File file, String s) {
                return s.endsWith(Constants.CSV) || s.endsWith(Constants.CHIP) || s.endsWith(Constants.MAP);
            }
        };
    }


    /**
     * IMP IMP: note only of the chip has been vdb'ed (i.e data exists / we care about it)
     *
     * @param chipName
     * @return
     */
    public static boolean isValidChipPlatform(String chipName) throws Exception {

        if (chipName == null) {
            throw new IllegalArgumentException("Param chipName cannot be null");
        }

        List validChipNames = Arrays.asList(getChipNames_All());
        boolean hasIt = validChipNames.contains(chipName);
        if (!hasIt) {
            // case INsensntive check
            for (Iterator iterator = validChipNames.iterator(); iterator.hasNext();) {
                String key = iterator.next().toString();
                if (key.equalsIgnoreCase(chipName)) {
                    hasIt = true;
                    break;
                }
            }
        }

        //System.out.println(">> " + kValidChipPlatformNames);

        return hasIt;
    }


    public static FeatureAnnot getFeatureAnnot_Gene_Symbol() throws Exception {
        return new FeatureAnnotChipImpl(getChip_Gene_Symbol(), getDefaultGenesOfInterest());
    }

    public static GenesOfInterest getDefaultGenesOfInterest() {
        return Application.getVdbManager().getDefaultGenesOfInterest();
    }

    // seq accessions is a special chip: accesions + symbols + aliases
    // see ChipParser
    public static Chip getChip_Seq_Accession() {
        return getChip(Constants.SEQ_ACCESSION);
    }

    public static Chip getChip_Stanford_Source() {
        return getChip(Constants.STANFORD_SOURCE_ACCESSIONS);
    }

    private static EntrezWebChip kEntrezWebChip;

    public static EntrezWebChip getChip_EntrezWeb() throws Exception {
        if (kEntrezWebChip == null) {
            Chip combo = ChipHelper.createComboChip(new Chip[]{getChip_Seq_Accession(), getChip_Stanford_Source()});
            kEntrezWebChip = new EntrezWebChip(combo);
        }
        return kEntrezWebChip;
    }

    public static Chip getChip_Gene_Symbol() {
        return getChip(Constants.GENE_SYMBOL);
    }

    public static boolean isChipSeqAccession(final Chip chip) {
        return chip != null && isChipSeqAccession(chip.getName());
    }

    public static boolean isChipGeneSymbol(final Chip chip) {
        return chip != null && isChipGeneSymbol(chip.getName());
    }

    public static boolean isChipGeneSymbol(String chipName) {
        chipName = _nameOrPath2Name(chipName).toUpperCase();
        return chipName.startsWith(Constants.GENE_SYMBOL);
    }

    public static boolean isChipSeqAccession(String chipName) {
        chipName = _nameOrPath2Name(chipName).toUpperCase();
        return chipName.startsWith(Constants.SEQ_ACCESSION);
    }

    public static Chip2ChipMapperSymbol getGeneSymbolMapper(final Chip chip, final boolean itsTheSource) throws Exception {
        if (itsTheSource) {
            return new Chip2ChipMapperSymbol(chip, getChip_Gene_Symbol());
        } else {
            return new Chip2ChipMapperSymbol(getChip_Gene_Symbol(), chip);
        }
    }

    public static boolean isChipAffy_hacky(String name) {
        name = _nameOrPath2Name(name);
        name = name.toUpperCase();
        for (Iterator iterator = affy_prefixes_hack.iterator(); iterator.hasNext();) {
            String key = iterator.next().toString().toUpperCase();
            if (name.startsWith(key)) {
                return true;
            }
        }

        return false;
    }

    private static Chip kAffy_combo_chip;

    public static Chip getChip_Affy() {
        try {
            if (kAffy_combo_chip == null) {
                kAffy_combo_chip = ChipHelper.createComboChip(getChips_Affy());
                kAffy_combo_chip.cloneShallow(Constants.AFFYMETRIX);
            }

            return kAffy_combo_chip;
        } catch (Throwable t) {
            throw new RuntimeException(t);
        }
    }

    public static Chip[] getChips_Affy() {

        try {
            final String[] affy_names = ParseUtils.slurpIntoArray(JarResources.toURL("AffyChipNames.txt"), true);
            return getChips(affy_names);
        } catch (Throwable t) {
            throw new RuntimeException(t);
        }
    }

    public static Chip[] getChips_Affy(final Organism org) throws Exception {
        final Chip[] chips = getChips_Affy();
        String s = org.getValue().toString().substring(0, 1).toUpperCase();
        List list = new ArrayList();
        for (int i = 0; i < chips.length; i++) {
            if (chips[i].getName().toUpperCase().startsWith(s)) {
                list.add(chips[i]);
            }
        }

        return (Chip[]) list.toArray(new Chip[list.size()]);
    }

    public static Chip getChip_All_Affy_combo(Organism org) throws Exception {
        return ChipHelper.createComboChip(getChips_Affy(org));
    }

    public static Chip getChip_combo(final String[] names) throws Exception {
        final Chip[] chips = new Chip[names.length];
        for (int i = 0; i < names.length; i++) {
            chips[i] = getChip(names[i]);
        }

        return ChipHelper.createComboChip(chips);
    }

    public static Chip[] getChips(final String[] namesOrPaths) throws Exception {
        final Chip[] chips = new Chip[namesOrPaths.length];
        for (int i = 0; i < namesOrPaths.length; i++) {
            chips[i] = getChip(namesOrPaths[i]);
        }

        return chips;
    }


    private static Map kGeoNameAffyNameMap;

    public static String geo2affy_chip_convert(String name) {

        if (name != null && name.startsWith("GPL")) {
            try {
                if (kGeoNameAffyNameMap == null) {
                    URL url = JarResources.toURL("GEO2AffyChipNames_mapping.txt");
                    kGeoNameAffyNameMap = ParseUtils.readKeyVal(url, false, false, false);
                }

                if (kGeoNameAffyNameMap != null) {
                    Object obj = kGeoNameAffyNameMap.get(name);
                    if (obj != null) {
                        return obj.toString();
                    }
                }

            } catch (Throwable t) {

            }

        }

        return name; // no change

    }

    public static Chip getChip(String chipNameOrPath) {

        if (chipNameOrPath == null) {
            throw new IllegalArgumentException("Param chipNameOrPath cannot be null");
        }


        chipNameOrPath = chipNameOrPath.trim();

        // check if a file path provided
        final File tmpf = new File(chipNameOrPath);

        if (tmpf.exists() && tmpf.isFile() && tmpf.getName().endsWith(".chip")) {
            return new FileInMemoryChip(tmpf.getName(), tmpf.getPath());
        }

        if (chipNameOrPath.toUpperCase().startsWith("AFFY")) {
            return getChip_Affy();
        }

        //System.out.println(">>>>" + chipName + "<");

        // Ok, its from a file path or ftp location
        //String chipName = _nameOrPath2Name(chipNameOrPath);

        Object obj = kChipNameChipFileObject.get(chipNameOrPath);
        if (obj == null) {
            String chipFile_source = getChipFile_source(chipNameOrPath);
            obj = new FileInMemoryChip(chipNameOrPath, chipFile_source);
            kChipNameChipFileObject.put(chipNameOrPath, obj);
        }
        return (Chip) obj;
    }

    private static String _nameOrPath2Name(final String nop) {
        if (nop.startsWith("ftp.")) {
            return nop.substring("gseaftp.broadinstitute.org://pub/gsea/annotations/".length()); // @note no ftp: prefix
        } else if (nop.startsWith("ftp:")) {
            return nop.substring(FTP_ANNOTATION_BASE.length()); // @note no ftp: prefix
        }

        if (nop.startsWith("gseaftp.")) {
            return nop.substring("gseaftp.broadinstitute.org://pub/gsea/annotations/".length()); // @note no ftp: prefix
        } else if (nop.startsWith("ftp:")) {
            return nop.substring(FTP_ANNOTATION_BASE.length()); // @note no ftp: prefix
        }

        return new File(nop).getName();
    }


    private static String FTP_ANNOTATION_BASE = "ftp://gseaftp.broadinstitute.org/pub/gsea/annotations/";

    private static String fChipLocBase = FTP_ANNOTATION_BASE; // default us the Broad FTP site

    public static void setChipPathBase(String base) {
        fChipLocBase = base;
    }

    public static String getChipFile_source(String chipNameOrPath) {
        if (chipNameOrPath == null) {
            throw new IllegalArgumentException("Param chipNameOrPath cannot be null");
        }

        if (chipNameOrPath.startsWith("ftp")) {

            // common error is ftp.broad... while it should be ftp://ftp.broad...
            if (chipNameOrPath.startsWith("ftp.")) {
                chipNameOrPath = "ftp://" + chipNameOrPath;
            }

            return chipNameOrPath;
        }

        if (chipNameOrPath.startsWith("gseaftp")) {

            // common error is gseaftp.broad... while it should be ftp://gseaftp.broad...
            if (chipNameOrPath.startsWith("gseaftp.")) {
                chipNameOrPath = "ftp://" + chipNameOrPath;
            }

            return chipNameOrPath;
        }

        chipNameOrPath = chipNameOrPath.replace('-', '_');

        if (chipNameOrPath.equalsIgnoreCase("GeneSymbol")) {
            chipNameOrPath = Constants.GENE_SYMBOL;
        }

        if (chipNameOrPath.equalsIgnoreCase("SeqAccession")) {
            chipNameOrPath = Constants.SEQ_ACCESSION;
        }

        String path = fChipLocBase + chipNameOrPath;
        if (!path.endsWith(".chip")) {
            path += ".chip";
        }

        return path;

        /*
        // want a better case INsensitive lookup
        final File[] files = kAnnDir.listFiles(FileUtils.createExtsFileNameFilter(new String[]{Constants.CSV, Constants.CHIP}));

        // @note order
        final String[] acceptableNames = new String[]{
                chipName + "_annot." + Constants.CSV,
                chipName + "_annot." + Constants.CHIP,
                chipName + "." + Constants.CSV,
                chipName + "." + Constants.MAP,
                chipName + "." + Constants.CHIP // last as most mem INeff
        };

        for (int f = 0; f < files.length; f++) {
            for (int i = 0; i < acceptableNames.length; i++) {
                if (files[f].getName().equalsIgnoreCase(acceptableNames[i])) { // @note caseInsensitive lookup
                    return files[f];
                }
            }
        }

        throw new RuntimeException("No .csv nor .chip file found for chip name: >" + chipName + "<\nThe annotations dir is at: " + kAnnDir.getPath());
        */


    }

    // init all chips -- both deep and lean but return only
    private static Set kChipNameErrors = new HashSet();

    private static List kValidChipPlatformNames_tt = null;

    // All chips from the FTP site
    public static String[] getChipNames_All() throws Exception {
        if (kValidChipPlatformNames_tt == null) {
            FTPList ftpList = new FTPList(GseaWebResources.getGseaFTPServer(),
                                          GseaWebResources.getGseaFTPServerUserName(),
                                          GseaWebResources.getGseaFTPServerPassword(),
                                          GseaWebResources.getGseaFTPServerChipDir());
            ftpList.quit();

            kValidChipPlatformNames_tt = Arrays.asList(ftpList.getFileNames());
        }

        return (String[]) kValidChipPlatformNames_tt.toArray(new String[kValidChipPlatformNames_tt.size()]);
    }

    public static Chip[] getChips_All_safe() {
        try {
            return getChips_All();
        } catch (Throwable t) {
            klog.error(t);
            return new Chip[]{};
        }
    }

    public static Chip[] getChips_All() throws Exception {
        final String[] names = getChipNames_All();

        List chips = new ArrayList();
        for (int i = 0; i < names.length; i++) {
            Object obj = names[i];
            if (kChipNameErrors.contains(obj)) {
                // do nothing as these already errored out before
            } else {
                try {
                    Chip chip = getChip(names[i]);
                    chips.add(chip);
                } catch (Throwable t) {
                    // @todo see if this is still needed
                    kChipNameErrors.add(obj);
                    //klog.warn(t);
                    t.printStackTrace();
                }
            }
        }

        // lets now do a hack so tha the gene symbol and affy ones come up first
        Collections.sort(chips, new ComparatorFactory.ChipNameComparator());

        return (Chip[]) chips.toArray(new Chip[chips.size()]);
    }

    //----------------------------------------------------------------------------------//
    // -------------------------------------------------------------------------------------------- //
    //----------------------------------------------------------------------------------//

    /**
     * AVOID STATIC INITS - that loads data at startup and increases launch time
     */

    // Duplicated code as dont want to refer to utils pkg
    static String removeExtension(final String filename) {

        if (filename.indexOf('#') != -1) { // aux names are of the form: foo.cls#classA so removing the ext is deleterious
            return filename;
        }

        if (filename.endsWith(".")) { // simpley strip off the period
            return filename.substring(0, filename.length() - 1);
        }

        // we also dont want to do the following:
        // foo_genes.hgu95av2.grp_mapped_symbols_from_HuGeneFL -> foo_genes.hgu95av2
        // (this would happen if we simply remove the extension)
        final int index = filename.lastIndexOf('.');

        if (-1 == index) { // no periods
            return filename;
        } else {
            String just_name = filename.substring(0, index);
            String ext = filename.substring(index + 1, filename.length());
            if (ext.length() <= 4) {
                return just_name; // common heuristic
            }

            StringTokenizer tok = new StringTokenizer(ext, "_");
            tok.nextToken(); // get rid of the "ext"
            StringBuffer extra = new StringBuffer("_");
            while (tok.hasMoreElements()) {
                extra.append(tok.nextToken());
                if (tok.hasMoreElements()) {
                    extra.append("_");
                }
            }

            if (extra.length() != 1) {
                return just_name + extra;
            } else {
                return just_name;
            }

            //return filename.substring(0, index);
        }
    }

    // ------------------------------------------------------------------------------------- //
    // -------------------------------- CHIP2CHIP MAPPING DB STUFF ------------------------- //
    // ------------------------------------------------------------------------------------- //

    static FilenameFilter createChip2ChipMapFnf() {
        return new FilenameFilter() {
            public boolean accept(File file, String s) {
                return s.endsWith("map");
            }
        };
    }


    /**
     * Internal class for a lightweight wrapper
     */
    public static class MappedChipWrapper implements Serializable { // pob so that paramchoosers work

        private Object fMapFileOrC2C;

        /**
         * Class constructor
         *
         * @param mapFileOrC2C
         */
        private MappedChipWrapper(final Object mapFileOrC2C) {

            //klog.debug("MAKING A MappedChipWrapper for: " + mapFileOrC2C);
            //TraceUtils.showTrace();

            if (mapFileOrC2C == null) {
                throw new IllegalArgumentException("Param mapFileOrC2C cannot be null");
            }

            // Heres a hack to allow a gene symbol to gene symbol mapping from the cmd line


            if (mapFileOrC2C instanceof File || mapFileOrC2C instanceof Chip2ChipMapper) {
                this.fMapFileOrC2C = mapFileOrC2C;
            } else {
                throw new IllegalArgumentException("Unexpected object type: " + mapFileOrC2C);
            }

        }

        private void writeObject(java.io.ObjectOutputStream out) {
            throw new NotImplementedException("out: " + out);
        }

        private void readObject(java.io.ObjectInputStream in) {
            throw new NotImplementedException("in: " + in);
        }

        // @note imp do NOT cache!!!
        // also imp to to not make public this method
        // The object can be either:
        //  1) a chip2chip mapper already
        // 2) a valid map file (in chip2chip usually or somewhere)
        // 3) a symbol.foo.symbol virtual mapping
        private Chip2ChipMapper createMappedChip() throws Exception {

            //klog.debug("CLASS CLASS >>>>> " + fMapFileOrC2C + " " + fMapFileOrC2C.getClass());
            if (fMapFileOrC2C instanceof Chip2ChipMapper) {
                return (Chip2ChipMapper) fMapFileOrC2C;
            } else {
                File file = (File) fMapFileOrC2C;
                if (file.exists() == false) { //// try harder: Gene_Symbol.foochip.Gene_Symbol
                    klog.info("Chipping in symbol 2 symbol mode: " + file);
                    final String[] fields = ParseUtils.string2strings(file.getName(), ".", false);
                    // sanity checks
                    if (fields.length != 3) {
                        throw new IllegalStateException("Expected 3 fields, got: " + fields.length + " so unknown mapping db: " + file);
                    }

                    if (fields[2].equalsIgnoreCase(Constants.GENE_SYMBOL) == false) {
                        throw new IllegalStateException("Expected: Gene_Symbol");
                    }

                    Chip chipSource = getChip(fields[0]);
                    Chip chipTarget = getChip(fields[1]);
                    return new Chip2ChipMapperSymbol(chipSource, chipTarget);
                } else {
                    return new Chip2ChipMapperSlimJit(file);
                }
            }
        }

        public String toString() {
            if (fMapFileOrC2C instanceof Chip2ChipMapper) {
                return ((Chip2ChipMapper) fMapFileOrC2C).getName();
            } else {
                return FileUtils.removeExtension((File) fMapFileOrC2C);
            }
        }

        public String getPath() {
            if (fMapFileOrC2C instanceof Chip2ChipMapper) {
                return ((Chip2ChipMapper) fMapFileOrC2C).getName();
            } else {
                return ((File) fMapFileOrC2C).getPath();
            }
        }

        public int hashCode() {
            return fMapFileOrC2C.hashCode();
        }

        public boolean equals(Object obj) {

            if (fMapFileOrC2C instanceof Chip2ChipMapper) {
                Chip2ChipMapper cm = (Chip2ChipMapper) fMapFileOrC2C;
                return cm.equals(obj);
            } else {
                File file = (File) fMapFileOrC2C;
                return file.equals(obj);
            }

        }

    }

} // End VdbRuntimeResources
