/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.chip;

import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.Hyperlink;
import edu.mit.broad.genome.reports.DefaultHyperlink;
import edu.mit.broad.genome.reports.web.WebResources;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.Chip2ChipMapperSymbol;
import edu.mit.broad.vdb.meg.Gene;
import gnu.trove.TObjectIntHashMap;
import gnu.trove.TObjectIntIterator;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Jan 22, 2005
 *         Time: 9:04:14 PM
 */
public abstract class AbstractChip extends AbstractObject implements Chip {

    protected String fSourcePath;

    public Chip cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    /**
     * Class constructor
     */
    protected AbstractChip() {
    }

    /**
     * Class constructor
     *
     * @param chipName
     * @param sourcePath
     */
    public AbstractChip(final String chipName,
                        final String sourcePath) {

        // @note dont do common init routine yet -> we're in skeletonmode
        super.initialize(chipName);

        if (sourcePath == null) {
            throw new IllegalArgumentException("Parameter sourcePath cannot be null");
        }

        this.fSourcePath = sourcePath;
    }

    public Hyperlink getHyperlink(final String probeName) {
        String base = _getProbeBaseLinkURL(probeName);
        // @note assumes something like: http://foo.bar?probeName
        if (base != null) {
            return new DefaultHyperlink(probeName, base + probeName);
        } else {
            return null;
        }
    }

    private static final String[] AFFY_CLUES = new String[]{"AFFX", "_at", "_st", "_s",};

    private String fBaseLinkURL;
    private boolean fDoneLookup;

    private String _getProbeBaseLinkURL(String probeName) {

        if (fDoneLookup) {
            return fBaseLinkURL;
        }

        if (VdbRuntimeResources.isChipGeneSymbol(this)) {
            this.fBaseLinkURL = WebResources.STANFORD_SOURCE_GENE.getUrlPrefix();
            this.fDoneLookup = true;
            return this.fBaseLinkURL;
        }

        // @note hack for now, simply assume its affymetrix
        boolean isAffy = false;

        for (int i = 0; i < AFFY_CLUES.length; i++) {
            if (probeName.toUpperCase().startsWith(AFFY_CLUES[i].toUpperCase())) {
                isAffy = true;
                break;
            }

            if (probeName.toUpperCase().endsWith(AFFY_CLUES[i].toUpperCase())) {
                isAffy = true;
                break;
            }
        }

        if (isAffy) {
            this.fBaseLinkURL = WebResources.NETAFFX.getUrlPrefix();
        } else {
            this.fBaseLinkURL = null;
        }

        this.fDoneLookup = true;
        return fBaseLinkURL;
    }

    public String getSourcePath() {
        return fSourcePath;
    }

    protected File getFile() {
        return new File(fSourcePath);
    }

    public boolean equals(Object obj) {
        if (obj instanceof Chip) {
            if (((Chip) obj).getName().equalsIgnoreCase(getName())) {
                return true;
            }
        }

        return false;
    }

    public int hashCode() {
        return getName().hashCode();
    }

    public String toString() {
        return getName();
    }

    public String[] probize(final String[] forTheseSymbols) {
        return probize_gs(new FSet("symbols_" + forTheseSymbols.length, forTheseSymbols)).getMembersArray();
    }

    public GeneSet probize_gs(final String[] forTheseSymbols) {
        return probize_gs(new FSet("symbols_" + forTheseSymbols.length, forTheseSymbols));
    }

    public String[] probize(final GeneSet forTheseSymbols) {
        return probize_gs(forTheseSymbols).getMembersArray();
    }

    // This is the core converter
    public GeneSet probize_gs(final GeneSet forTheseSymbols) {
        try {
            final Chip2ChipMapper mapper = new Chip2ChipMapperSymbol(VdbRuntimeResources.getChip_Gene_Symbol(), this);
            return mapper.map(forTheseSymbols, false).getMappedGeneSet(true);
        } catch (Throwable t) {
            throw new RuntimeException(t);
        }
    }

    public String[] getProbeNamesArr() throws Exception {
        Set set = getProbeNames();
        return (String[]) set.toArray(new String[set.size()]);
    }

    public GeneSet getProbeNamesAsGeneSet() throws Exception {
        return new FSet(getName(), getProbeNames());
    }

    // will error out if no such probe
    public Gene getHugo(final String probeName) throws Exception {
        return getProbe(probeName).getGene();
    }

    public Set getHugos(final GeneSet probeNames) throws Exception {
        Set hugos = new HashSet();

        for (int i = 0; i < probeNames.getNumMembers(); i++) {
            hugos.add(getHugo(probeNames.getMember(i)));
        }

        return hugos;
    }

    public Set getSymbols(final GeneSet probeNames, final NullSymbolMode nmode) throws Exception {

        final Set set = new HashSet();

        for (int i = 0; i < probeNames.getNumMembers(); i++) {
            String probeId = probeNames.getMember(i);

            String symbol = getSymbol(probeId, nmode);
            if (symbol != null) {
                set.add(symbol);
            }
            /*
            Hugo hugo = getHugo(probeId);
            nmode.processSymbol(probeId, hugo, set);
            */

        }

        return set;
    }

    public GeneSet getSymbols_gs(final GeneSet probeNames, final NullSymbolMode nmode) throws Exception {

        final Set set = new HashSet();

        for (int i = 0; i < probeNames.getNumMembers(); i++) {
            String probeId = probeNames.getMember(i);
            String symbol = getSymbol(probeId, nmode);
            if (symbol != null) {
                set.add(symbol);
            }
        }

        return new FSet(probeNames.getName(true) + "_symbolized", set);
    }

    public Set getSymbols(final NullSymbolMode nmode) throws Exception {
        return getSymbols(getProbeNamesAsGeneSet(), nmode);
    }

    public GeneSetPair getSymbols(final GeneSetPair probes, final NullSymbolMode nmode) throws Exception {
        return new GeneSetPairImpl(probes.getName() + "_symbols",
                new FSet(probes.getUp().getName(true), getSymbols(probes.getUp(), nmode)),
                new FSet(probes.getDown().getName(true), getSymbols(probes.getDown(), nmode))
        );
    }

    public GeneSetMatrixPair getSymbols(final GeneSetMatrixPair probes, final NullSymbolMode nmode) throws Exception {
        final GeneSetPair[] pairs = new GeneSetPair[probes.getSize()];
        for (int i = 0; i < probes.getSize(); i++) {
            pairs[i] = getSymbols(probes.get(i), nmode);
        }

        return new GeneSetMatrixPairImpl(probes.getName() + "_symbols", pairs);
    }

    public Set getSymbols(final List probeNames, final NullSymbolMode nmode) throws Exception {

        Set set = new HashSet();

        for (int i = 0; i < probeNames.size(); i++) {
            String probeId = probeNames.get(i).toString();
            Gene gene = null;
            if (isProbe(probeId)) {
                gene = getHugo(probeId);
            }
            nmode.processSymbol(probeId, gene, set);
        }

        return set;
    }

    public List getSymbols_synched(final List probeNames, final NullSymbolMode nmode) throws Exception {

        List list = new ArrayList();

        for (int i = 0; i < probeNames.size(); i++) {
            String probeId = probeNames.get(i).toString();
            Gene gene = null;
            if (isProbe(probeId)) {
                gene = getHugo(probeId);
            }
            nmode.processSymbol(probeId, gene, list);
        }

        return list;
    }

    // catch the exception
    public String getSymbol(final String probeName, final NullSymbolMode nmode) {
        try {
            if (isProbe(probeName)) {
                return nmode.getSymbol(probeName, getHugo(probeName));
            } else {
                return nmode.getSymbol(probeName, null);
            }
        } catch (Throwable t) {
            log.error(t);
            return "";
            //return Constants.NA;
        }

    }

    // catch the exception
    public String getTitle(final String probeName, final NullSymbolMode nmode) {
        try {
            if (isProbe(probeName)) {
                return nmode.getTitle(probeName, getHugo(probeName));
            } else {
                return nmode.getTitle(probeName, null);
            }
        } catch (Throwable t) {
            log.error(t);
            return "";
            //return Constants.NA;
        }

    }

    public StringDataframe annotate(final IDataframe sdf) {

        final StringMatrix sm = new StringMatrix(sdf.getNumRow(), sdf.getNumCol() + 2);
        final String[] colNames = new String[sdf.getNumCol() + 2];
        colNames[0] = Headers.GENE_SYMBOL;
        colNames[1] = Headers.GENE_TITLE;

        for (int c = 0; c < sdf.getNumCol(); c++) {
            colNames[c + 2] = sdf.getColumnName(c);
        }

        // first set the symbol and title column
        for (int r = 0; r < sdf.getNumRow(); r++) {
            String name = sdf.getRowName(r);
            sm.setElement(r, 0, getSymbol(name, OMIT_NULLS));
            sm.setElement(r, 1, getTitle(name, OMIT_NULLS));
        }

        // then set the rest as is
        for (int c = 0; c < sdf.getNumCol(); c++) {
            sm.setColumn(c + 2, sdf.getColumnAsStrings(c));
        }

        return new StringDataframe(sdf.getName(), sm, sdf.getRowNames(), colNames, true, true);
    }

    public StringDataframe annotate(final RankedList rl) {
        final StringMatrix sm = new StringMatrix(rl.getSize(), 3);
        final String[] colNames = new String[]{"SCORE", Headers.GENE_SYMBOL, Headers.GENE_TITLE};

        // first set the symbol and title column
        for (int r = 0; r < rl.getSize(); r++) {
            String name = rl.getRankName(r);
            sm.setElement(r, 0, rl.getScore(r));
            sm.setElement(r, 1, getSymbol(name, OMIT_NULLS));
            sm.setElement(r, 2, getTitle(name, OMIT_NULLS));
        }

        return new StringDataframe(rl.getName(), sm, rl.getRankedNames(), colNames, true, true);
    }

    public RankedList symbolize(final RankedList rl) {
        String[] names = new String[rl.getSize()];
        for (int r = 0; r < rl.getSize(); r++) {
            names[r] = getSymbol(rl.getRankName(r), REPLACE_WITH_PROBEID);
        }

        return new DefaultRankedList(rl.getName(), names, rl.getScoresV(true), true);
    }

    public GeneSetMatrix symbolize(final GeneSetMatrix gm) {

        GeneSet[] gsets = new GeneSet[gm.getNumGeneSets()];
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            gsets[i] = symbolize(gm.getGeneSet(i));
        }

        return new DefaultGeneSetMatrix(gm.getName() + "_symbolized", gsets);
    }

    public GeneSet symbolize(final GeneSet gset) {
        final List names = new ArrayList();
        for (int r = 0; r < gset.getNumMembers(); r++) {
            final String n = getSymbol(gset.getMember(r), OMIT_NULLS);
            if (n != null && !names.contains(n)) {
                names.add(n);
            }
        }

        return new FSet(gset.getName(), names, false); // already checked
    }

    public StringDataframe symbolize(String name, final TObjectIntHashMap probeNameCntMap) {
        String[] colNames = new String[]{"SYMBOL", "TITLE", "CNT"};

        StringMatrix sm = new StringMatrix(probeNameCntMap.size(), colNames.length);
        String[] rowNames = new String[probeNameCntMap.size()];

        TObjectIntIterator iterator = probeNameCntMap.iterator();

        for (int i = probeNameCntMap.size(); i-- > 0;) {
            iterator.advance();
            rowNames[i] = iterator.key().toString();
            sm.setElement(i, 0, getSymbol(rowNames[i], REPLACE_WITH_PROBEID));
            sm.setElement(i, 1, getTitle(rowNames[i], REPLACE_WITH_PROBEID));
            sm.setElement(i, 2, probeNameCntMap.get(rowNames[i]));
        }

        return new StringDataframe(name, sm, rowNames, colNames, true);
    }

    public Probe[] getProbes(final GeneSet probeNames) throws Exception {
        Probe[] probes = new Probe[probeNames.getNumMembers()];
        for (int p = 0; p < probeNames.getNumMembers(); p++) {
            probes[p] = getProbe(probeNames.getMember(p));
        }

        return probes;
    }

    public Probe[] getProbes(final Dataset ds) throws Exception {
        if (ds == null) {
            throw new IllegalArgumentException("Parameter ds cannot be null");
        }
        Set probes = new HashSet();
        for (int r = 0; r < ds.getNumRow(); r++) {
            String rowName = ds.getRowName(r);
            if (isProbe(rowName)) {
                probes.add(getProbe(rowName));
            }
        }

        return (Probe[]) probes.toArray(new Probe[probes.size()]);
    }

    public GeneSet getProbeNamesAsGeneSet(final Dataset ds) throws Exception {
        Set probeNames = _toNamesS(getProbes(ds));
        return new FSet(ds.getName() + "_" + getName(), probeNames);
    }

    private static String[] _toNames(final Probe[] probes) {
        String[] names = new String[probes.length];
        for (int p = 0; p < probes.length; p++) {
            names[p] = probes[p].getName();
        }

        return names;
    }

    private static Set _toNamesS(final Probe[] probes) {
        Set names = new HashSet();
        for (int p = 0; p < probes.length; p++) {
            names.add(probes[p].getName());
        }

        return names;
    }

} // End class AbstractChip
