package edu.mit.broad.vdb.chip;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.parsers.EntrezGeneWebPageParser;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.meg.Gene;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * Entrez web backed
 * <p/>
 * PLUS a special file for accessions got manually from SOURCE
 * <p/>
 * <p/>
 *
 * @author Aravind Subramanian
 *         Date: Oct 23, 2006
 *         Time: 12:59:32 PM
 */
public class EntrezWebChip {

    private static final Logger klog = XLogger.getLogger(EntrezWebChip.class);

    private Chip fChip_local;

    /**
     * Class constructor
     *
     * @param chipName
     * @param sourcePath
     * @param probes
     */
    public EntrezWebChip(final Chip local) {
        this.fChip_local = local;
    }

    public String getSymbol(final String probeName, final boolean lookupAccessionsOnlineIfNeeded, final boolean checkAffyFirst) throws Exception {
        Probe probe = getProbe(probeName, lookupAccessionsOnlineIfNeeded, checkAffyFirst);
        if (probe != null && !Gene.Helper.isNull(probe.getGene())) {
            return probe.getGene().getSymbol();
        }

        return null;
    }

    private Map fAccHugoMap = new HashMap();

    private final EntrezGeneWebPageParser parser = new EntrezGeneWebPageParser();

    static class ProbeWithSource extends SimpleProbe {

        String source;

        ProbeWithSource(String probeId, String symbol, String title, String source) {
            super(probeId, symbol, title);
            this.source = source;
        }

        ProbeWithSource(Probe probe, String source) {
            super(probe.getName(), probe.getGene().getSymbol(), probe.getGene().getTitle());
            this.source = source;
        }

    }


    public Probe getProbe(String accName, final boolean lookupAccessionsOnlineIfNeeded, final boolean checkAffyFirst) throws Exception {

        accName = accName.toUpperCase(); // @note
        accName = _removeExtension(accName); // @note

        Probe probe = null;
        String source = null;

        if (fChip_local.isProbe(accName)) {
            probe = fChip_local.getProbe(accName);
            source = "local";
        }

        if (probe == null && checkAffyFirst && VdbRuntimeResources.getChip_Affy().isProbe(accName)) {
            probe = VdbRuntimeResources.getChip_Affy().getProbe(accName);
            source = "affy";
        }

        if (probe == null && lookupAccessionsOnlineIfNeeded) {
            Object curr = fAccHugoMap.get(accName);
            if (curr == null) {
                String symbol = parser.parseSymbol(accName);
                //System.out.println("looking up ezweb: " + accName + " got: " + curr);
                if (symbol == null) {
                    curr = new SimpleProbe(accName, Gene.NULL_GENE.getSymbol(), Gene.NULL_GENE.getTitle());
                } else {
                    curr = new SimpleProbe(accName, symbol, VdbRuntimeResources.getChip_Gene_Symbol().getTitle(symbol, Chip.OMIT_NULLS));
                    source = "ezweb";
                }
                fAccHugoMap.put(accName, curr);
            }
            probe = (Probe) curr;
        }

        return new ProbeWithSource(probe, source);
    }

    private String _removeExtension(String probeName) {
        if (probeName == null) {
            return null;
        }

        // remove the version # if present
        String ext = NamingConventions.getExtension(probeName);
        if (ext != null && NamingConventions.isInteger(ext)) {
            probeName = NamingConventions.removeExtension(probeName);
        }

        return probeName;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    public GeneSet symbolize(final GeneSet gset, final boolean lookupAccessionsOnlineIfNeeded, final boolean checkAffyFirst) throws Exception {
        final List names = new ArrayList();
        for (int r = 0; r < gset.getNumMembers(); r++) {
            final String n = getSymbol(gset.getMember(r), lookupAccessionsOnlineIfNeeded, checkAffyFirst);
            if (n != null && !names.contains(n)) {
                names.add(n);
            }
        }

        return new FSet(gset.getName(), names, false); // already checked
    }

    public AccessionLookup symbolize_with_details(final GeneSet accs,
                                                  final boolean checkAffy) throws Exception {

        final Set symbols = new HashSet();

        final AccessionLookup ezd = new AccessionLookup();

        final String[] colNames = new String[]{"SYMBOL", "SOURCE", "TITLE"};

        final StringMatrix sm = new StringMatrix(accs.getNumMembers(), colNames.length);

        int known_valid_symbols = 0;

        final Set unknown = new HashSet();
        final Set known = new HashSet();

        for (int i = 0; i < accs.getNumMembers(); i++) {
            final String accName = accs.getMember(i);
            final Probe probe = getProbe(accName, true, checkAffy);

            if (probe != null && probe.getGene().getSymbol() != null) {
                //System.out.println(accs.getMember(i) + "=" + probe.getHugo().getSymbol() + probe.getHugo().getTitle());
                sm.setElement(i, 0, probe.getGene().getSymbol());
                sm.setElement(i, 1, ((ProbeWithSource) probe).source);
                sm.setElement(i, 2, probe.getGene().getTitle());

                if (!NamingConventions.isNull(probe.getGene().getTitle())) {
                    known_valid_symbols++;
                }

                symbols.add(probe.getGene().getSymbol());
                known.add(accs.getMember(i));
            } else {
                unknown.add(accs.getMember(i));
                //System.out.println("Could not find for: " + accs.getMember(i));
            }

            if (i % 100 == 0) {
                System.out.println("Done lookup: " + (i + 1) + " " + accs.getNumMembers());
            }
        }

        ezd.unknownAccessions = new FSet(accs.getName(true) + "_unknown", unknown);
        ezd.knownAccessions = new FSet(accs.getName(true) + "_known", known);
        ezd.deconReport = new StringDataframe("lookup", sm, accs.getMembersArray(), colNames, true);
        ezd.symbols = new FSet(accs.getName(true) + "_symbolized", symbols);

        klog.info("# accessions in: " + accs.getNumMembers() + " known: " + known.size() + " UNknown: " + unknown.size() + " known_valid_symbols: " + known_valid_symbols);

        return ezd;
    }

    public static class AccessionLookup {

        public StringDataframe deconReport;

        public GeneSet symbols;

        public GeneSet unknownAccessions;

        public GeneSet knownAccessions;

        public GeneSet getUnknownAccessions() {
            return unknownAccessions;
        }

        public GeneSet getKnownAccessions() {
            return knownAccessions;
        }

    } // End class EntrezGeneDownload

} // End class EntrezWebChip
