/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.chip;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.vdb.meg.Gene;
import edu.mit.broad.vdb.meg.SimpleGene;

import java.util.List;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Jan 22, 2005
 *         Time: 8:17:51 PM
 */
public class NullSymbolModes {

    public static class OmitNullSymbolMode implements NullSymbolMode {

        public OmitNullSymbolMode() {
        }

        public void processSymbol(final String probeId, final Gene gene, final Set set) {
            if (_isNull(gene)) {
                // omit
            } else {
                set.add(gene.getSymbol());
            }
        }

        public void processHugo(final String probeId, final Gene gene, final Set set) {
            if (_isNull(gene)) {
                // omit
            } else {
                set.add(gene);
            }
        }

        public void processSymbol(final String probeId, final Gene gene, final List list) {
            if (_isNull(gene)) {
                list.add(null);
            } else {
                list.add(gene.getSymbol());
            }
        }

        public void processHugo(final String probeId, final Gene gene, final List list) {
            if (_isNull(gene)) {
                list.add(null);
            } else {
                list.add(gene);
            }
        }

        public String getTitle(String probeId, Gene gene) {
            if (_isNull(gene)) {
                return null;
            } else {
                return gene.getTitle();
            }
        }

        public String getSymbol(String probeId, Gene gene) {
            if (_isNull(gene)) {
                return null;
            } else {
                return gene.getSymbol();
            }
        }

    }

    public static class ReplaceWithProbeIdMode implements NullSymbolMode {
        ReplaceWithProbeIdMode() {
        }

        public void processSymbol(String probeId, Gene gene, Set set) {
            if (_isNull(gene)) {
                if (probeId == null) {
                    throw new IllegalArgumentException("Parameter probeId cannot be null");
                }
                set.add(probeId);
            } else {
                set.add(gene.getSymbol());
            }
        }

        public void processHugo(String probeId, Gene gene, Set set) {
            if (_isNull(gene)) {
                if (probeId == null) {
                    throw new IllegalArgumentException("Parameter probeId cannot be null");
                }
                set.add(new SimpleGene(probeId, "gene-less probe"));
            } else {
                set.add(gene);
            }
        }

        public String getSymbol(String probeId, Gene gene) {
            if (_isNull(gene)) {
                return probeId;
            } else {
                return gene.getSymbol();
            }
        }

        public String getTitle(String probeId, Gene gene) {
            if (_isNull(gene)) {
                return probeId;
            } else {
                return gene.getTitle();
            }
        }

        public void processSymbol(final String probeId, final Gene gene, final List list) {
            if (_isNull(gene)) {
                list.add(probeId);
            } else {
                list.add(gene.getSymbol());
            }
        }

        public void processHugo(final String probeId, final Gene gene, final List list) {
            if (_isNull(gene)) {
                list.add(probeId);
            } else {
                list.add(gene);
            }
        }
    }


    private static boolean _isNull(Gene gene) {
        if (gene == null || NamingConventions.isNull(gene.getSymbol())) {
            return true;
        } else {
            return false;
        }
    }

} // End class NullSymbolModes

