/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.map;

import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.utils.ParseException;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import gnu.trove.THashMap;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Set;

/**
 * Jit -> just in time loading to keep it light
 *
 * @author Aravind Subramanian
 *         Date: Feb 17, 2005
 *         Time: 9:54:33 PM
 */
public class Chip2ChipMapperSlimJit extends Chip2ChipMapperAbstract {

    private static final long serialVersionUID = 3328220991976115699L;

    private File fMapFile;

    private MapUtils.Struc fStruc;

    private boolean fInited;

    /**
     * Class constructor
     *
     * @param serFile
     */
    public Chip2ChipMapperSlimJit(final File serFile) {
        if (serFile == null || serFile.exists() == false) {
            throw new IllegalArgumentException("Param serFile cannot be null or non existant: " + serFile);
        }

        this.fMapFile = serFile;
        this.fStruc = new MapUtils.Struc(fMapFile);

        //TraceUtils.showTrace("got: " + serFile.getPath() + " " + fStruc + " " + fStruc.sourceChipName + " " + fStruc.targetChipName);

    }

    public String getChipsId() {
        // @note imp to not init for this call
        return fStruc.createChipId();
    }

    public String getName() {
        // @note imp to not init for this call
        return fStruc.createId();
    }

    protected void checkAndInit() {
        if (fInited) {
            return;
        }

        try {
            THashMap map = _parse();
            super.init(fStruc.sourceChipName, fStruc.targetChipName, fStruc.getMappingDbType(), map);
            fInited = true;
        } catch (Throwable t) {
            t.printStackTrace();
            throw new RuntimeException(t);
        }

    }

    // @note parsing logic is here rather than in regular place
    private THashMap _parse() throws Exception {
        BufferedReader buf = new BufferedReader(new FileReader(fMapFile));
        String currLine = buf.readLine();
        int cnt = 1;
        THashMap map = new THashMap();
        while (currLine != null) {
            currLine = currLine.trim();
            String[] fields = ParseUtils.string2strings(currLine, "\t", false);
            if (fields.length != 2) {
                throw new ParseException("Bad format for map file: " + fMapFile + " on line: " + cnt + " >" + currLine + "<");
            }

            java.util.Set target_probes = ParseUtils.string2stringsSet(fields[1], "", false);
            Object curr = map.get(fields[0]);
            if (curr == null) {
                map.put(fields[0], target_probes);
            } else {
                ((java.util.Set) curr).addAll(target_probes);
                map.put(fields[0], curr);
            }
            currLine = buf.readLine();
            cnt++;
        }

        buf.close();

        return map;
    }

    public MappingDbType getMappingDbType() {
        if (fInited) {
            return super.getMappingDbType();
        } else {
            return MappingDbTypes.lookupMappingType(fStruc.mappingDbType);
        }
    }

    public Chip getSourceChip() {
        if (fInited) {
            return super.getSourceChip();
        } else {
            return VdbRuntimeResources.getChip(fStruc.sourceChipName);
        }
    }

    public Chip getTargetChip() {
        if (fInited) {
            return super.getTargetChip();
        } else {
            return VdbRuntimeResources.getChip(fStruc.targetChipName);
        }
    }

    public boolean equals(Chip sourceChip, Chip targetChip, MappingDbType db) {
        if (fInited) {
            return super.equals(sourceChip, targetChip, db);
        } else {
            return fStruc.equals(sourceChip, targetChip, db);
        }
    }

    public boolean equals(Chip sourceChip, Chip targetChip) {
        if (fInited) {
            return super.equals(sourceChip, targetChip);
        } else {
            return fStruc.equals(sourceChip, targetChip);
        }
    }

    // ------------------------------------------------------------------- //
    // --------- THE REST ALL NEED DEEP CALLS E.G INITIALIZATION --------- //
    // ------------------------------------------------------------------- //

    public int getNumSourceProbes() {
        checkAndInit();
        return super.getNumSourceProbes();
    }

    public Set map(String sourceProbeName) {
        checkAndInit();
        return super.map(sourceProbeName);
    }

    public String[] getSourceProbes() {
        checkAndInit();
        return super.getSourceProbes();
    }

}
