package edu.mit.broad.vdb.meg;

/**
 * @author Aravind Subramanian
 * Date: Oct 16, 2006
 * Time: 9:04:20 PM
 */

import edu.mit.broad.genome.*;
import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.StringDataframeParser;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.chip.Probe;
import edu.mit.broad.vdb.chip.SimpleProbe2;

import java.io.File;
import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Oct 31, 2005
 *         Time: 12:19:48 PM
 */
public class AliasDbImpl extends AbstractObject implements AliasDb {

    // Either file or chip must be specified
    private File fFile_opt;

    private Chip fChip_opt;

    /**
     * Class constructor
     * <p/>
     * Either file or chip must be specified
     *
     * @param file
     */
    public AliasDbImpl(final File file_opt, final Chip chip_opt) {
        super.initialize(_name(file_opt, chip_opt));
        this.fFile_opt = file_opt;
        this.fChip_opt = chip_opt;
    }

    public String getQuickInfo() {
        return null;
    }

    public boolean isAlias(final String alias) throws Exception {
        String symbol = getSymbol(alias);
        if (symbol == null) {
            return false;
        } else {
            return true;
        }
    }

    // may return null
    public String getSymbol(final String alias) throws Exception {
        if (alias == null) {
            throw new IllegalArgumentException("Param alias cannot be null");
        }

        _initAliasSymbolMap();

        Object obj = fAliasSymbolMap.get(alias);
        if (obj != null) {
            return obj.toString();
        }

        obj = fAliasSymbolMap.get(alias.toUpperCase());
        if (obj != null) {
            return obj.toString();
        }

        return null;
    }

    public Gene getHugo(final String alias) throws Exception {
        if (fChip_opt == null) {
            throw new NotImplementedException("Only works for in mem gene symbol chip");
        }

        String symbol = getSymbol(alias);
        return fChip_opt.getHugo(symbol);
    }

    public Probe[] getAliasesAsProbes() throws Exception {
        _initAliasSymbolMap();
        final Probe[] probes = new Probe[fAliasSymbolMap.size()];
        int cnt = 0;
        for (Iterator iterator = fAliasSymbolMap.keySet().iterator(); iterator.hasNext();) {
            String alias = iterator.next().toString();
            String symbol = fAliasSymbolMap.get(alias).toString();
            probes[cnt++] = new SimpleProbe2(alias, symbol, VdbRuntimeResources.getChip_Gene_Symbol());
        }

        return probes;
    }

    // may return null
    public Map getSymbolAliasesMap() throws Exception {
        _initSymbolAliasesMap();
        return Collections.unmodifiableMap(fSymbolAliasesMap);
    }

    public Set getAliases(final String symbol_in) throws Exception {

        if (symbol_in == null) {
            throw new IllegalArgumentException("Param symbol_in cannot be null");
        }

        _initSymbolAliasesMap();

        Object obj = fSymbolAliasesMap.get(symbol_in);
        if (obj != null) {
            return (Set) obj;
        }

        obj = fSymbolAliasesMap.get(symbol_in.toUpperCase());
        if (obj != null) {
            return (Set) obj;
        }

        return null;
    }

    public String[] getAliases_array(final String symbol_in) throws Exception {
        Set set = getAliases(symbol_in);
        if (set != null) {
            return (String[]) set.toArray(new String[set.size()]);
        }

        return null;
    }

    public int getNumRecords() {
        throw new NotImplementedException();
    }

    public String getSourcePath() {
        if (fFile_opt != null) {
            return fFile_opt.getPath();
        } else {
            return fChip_opt.getSourcePath();
        }
    }

    public Version getVersion() {
        return new Version(1);
    }

    // -------------------------------------------------------------------------------------------- //

    private static String _name(final File file_opt, final Chip chip_opt) {
        if (chip_opt != null && file_opt != null) {
            throw new IllegalArgumentException("Both file: " + file_opt + " and chip cannot be specified: " + chip_opt);
        }
        if (file_opt != null) {
            return file_opt.getName();
        } else if (chip_opt != null) {
            return chip_opt.getName();
        } else {
            throw new IllegalArgumentException("Both file and chip cannot be null");
        }
    }

    public static void save(final Map symbolAliasMap, final File file) throws Exception {
        StringBuffer buf = new StringBuffer();
        for (Iterator iterator = symbolAliasMap.keySet().iterator(); iterator.hasNext();) {
            String symbol = iterator.next().toString();
            symbol = NamingConventions.symbolize(symbol);
            Set aliases = (Set) symbolAliasMap.get(symbol);
            buf.append(symbol).append('\t').append(Printf.Full.toString(aliases)).append('\n');
        }

        FileUtils.write(buf.toString(), file);
    }

    // -------------------------------------------------------------------------------------------- //

    private Map fSymbolAliasesMap;

    private void _initSymbolAliasesMap() throws Exception {

        if (fSymbolAliasesMap != null) {
            return;
        }

        final Map map = new HashMap();

        if (fFile_opt != null) {

            final StringDataframe sdf = new StringDataframeParser().parseSdf(fFile_opt);
            // only 2 columns
            // NAME ALIASES

            for (int r = 0; r < sdf.getNumRow(); r++) {
                String symbol = sdf.getRowName(r);
                symbol = NamingConventions.symbolize(symbol);
                Set accessions = ParseUtils.string2stringsSet(sdf.getElement(r, 0), Constants.INTRA_FIELD_DELIM_S, false);
                Set accessions_uc = new HashSet();
                for (Iterator iterator = accessions.iterator(); iterator.hasNext();) {
                    Object key = iterator.next();
                    accessions_uc.add(key.toString().toUpperCase());
                }
                map.put(symbol.toUpperCase(), accessions_uc); // @note case
            }
        } else {
            final Probe[] probes = fChip_opt.getProbes();
            for (int i = 0; i < probes.length; i++) {
                Gene gene = probes[i].getGene();
                if (gene != null && gene != Gene.NULL_GENE && gene.getAliases() != null && !gene.getAliases().isEmpty()) {
                    final String symbol = gene.getSymbol();
                    map.put(symbol, gene.getAliases());
                }
            }
        }

        log.info("# of symbols with aliases: " + map.size());

        this.fSymbolAliasesMap = map;
    }

    private Map fAliasSymbolMap;

    private void _initAliasSymbolMap() throws Exception {
        if (fAliasSymbolMap != null) {
            return;
        }

        final Map map = new HashMap();
        final Set duplicates = new HashSet();

        if (fFile_opt != null) {
            final StringDataframe sdf = new StringDataframeParser().parseSdf(fFile_opt);
            // only 2 columns
            // NAME ALIASES
            for (int r = 0; r < sdf.getNumRow(); r++) {
                final String symbol = NamingConventions.symbolize(sdf.getRowName(r));
                final String[] accessions = ParseUtils.string2strings(sdf.getElement(r, 0), Constants.INTRA_FIELD_DELIM_S, false);
                final Set accessions_suc = new HashSet();
                for (int s = 0; s < accessions.length; s++) {
                    accessions_suc.add(accessions[s].toUpperCase()); // @note aliases always stored in UC
                }

                final String[] accessions_uc = (String[]) accessions_suc.toArray(new String[accessions_suc.size()]);
                for (int s = 0; s < accessions_uc.length; s++) {
                    if (map.containsKey(accessions_uc[s])) {
                        duplicates.add(accessions_uc[s]);
                    } else { // ignore duplicates -- dont add at all
                        map.put(accessions_uc[s], symbol);
                    }
                }
            }

        } else {

            final Probe[] probes = fChip_opt.getProbes();
            for (int i = 0; i < probes.length; i++) {
                Gene gene = probes[i].getGene();
                if (gene != null && gene != Gene.NULL_GENE && gene.getAliases() != null && !gene.getAliases().isEmpty()) {
                    final String symbol = gene.getSymbol();
                    final String[] aliases = gene.getAliasesArray();
                    for (int s = 0; s < aliases.length; s++) {
                        String alias = aliases[s].toUpperCase(); // @note aliases always stored in UC
                        if (map.containsKey(alias)) {
                            duplicates.add(alias);
                        } else { // ignore duplicates -- dont add at all
                            map.put(alias, symbol);
                        }
                    }
                }
            }
        }

        if (!duplicates.isEmpty()) {
            log.warn("There are duplicate entry for accessions: " + duplicates.size() + " ignoring them ..." + " total #: " + map.size());
            //throw new IllegalStateException("Duplicate entry for accessions: " + duplicates.size() + "\n" + duplicates);
        }

        log.info("# of aliases: " + map.size());

        this.fAliasSymbolMap = map;
    }

} // End class AliasDbImpl

