package edu.mit.broad.vdb.meg;

/**
 * Author: Aravind Subramanian
 * Date: Oct 16, 2006
 * Time: 6:49:58 PM
 */

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

import edu.mit.broad.genome.Version;
import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.vdb.Organism;
import edu.mit.broad.vdb.Vdb;
import edu.mit.broad.vdb.chip.GeneProbe;
import edu.mit.broad.vdb.chip.Probe;

import java.io.File;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: aravind subramanian
 * Date: Jan 1, 2004
 * Time: 11:50:38 PM
 * To change this template use Options | File Templates.
 */
public class EntrezGeneDb extends AbstractObject implements Vdb {

    private String fSourcePath;
    private Version fVersion;
    private EntrezGene[] fEntrezGenes;


    public EntrezGeneDb cloneShallow(final Map geneIdSetMap) {

        for (int i = 0; i < getNumRecords(); i++) {
            EntrezGene gene = getEntrezGene(i);
            Object obj = geneIdSetMap.get(gene.getEntrezID() + "");
            if (obj != null) {
                Set accs = (Set) obj;
                gene.cloneShallow(accs);
            }
        }

        return this;
    }

    /**
     * Class constructor
     *
     * @param source
     * @param version
     * @param entrezs
     */
    public EntrezGeneDb(final String source, final Version version, final EntrezGene[] entrezs) {
        init(source, version, entrezs);
    }

    /**
     * Class constructor
     *
     * @param source
     * @param version
     * @param entrezs
     */
    public EntrezGeneDb(final File source, final Version version, final EntrezGene[] entrezs) {
        init(source.getPath(), version, entrezs);
    }

    public String getQuickInfo() {
        return null;
    }

    public EntrezGeneDb extract(final Organism org) {

        List org_entrezs = new ArrayList();

        for (int i = 0; i < getNumRecords(); i++) {
            if (getEntrezGene(i).isOrg(org)) {
                org_entrezs.add(getEntrezGene(i));
            }
        }

        return new EntrezGeneDb(getSourcePath() + "_" + org.getName(), getVersion(), (EntrezGene[]) org_entrezs.toArray(new EntrezGene[org_entrezs.size()]));
    }

    public Probe[] createProbes_by_symbol(final Organism org) {

        List org_entrezs = new ArrayList();

        for (int i = 0; i < getNumRecords(); i++) {
            if (getEntrezGene(i).isOrg(org)) {
                org_entrezs.add(getEntrezGene(i));
            }
        }

        return (Probe[]) org_entrezs.toArray(new Probe[org_entrezs.size()]);
    }

    public Probe[] createProbes_by_symbol() {

        List org_entrezs = new ArrayList();

        for (int i = 0; i < getNumRecords(); i++) {
            org_entrezs.add(getEntrezGene(i));
        }

        return (Probe[]) org_entrezs.toArray(new Probe[org_entrezs.size()]);
    }

    public Probe[] createProbes_by_accession() {

        List all = new ArrayList();

        for (int i = 0; i < getNumRecords(); i++) {
            all.addAll(createProbes_by_accession(getEntrezGene(i)));
        }

        return (Probe[]) all.toArray(new Probe[all.size()]);
    }

    public Probe[] createProbes_by_accession(final Organism forThisOrg) {

        List all = new ArrayList();

        for (int i = 0; i < getNumRecords(); i++) {
            if (getEntrezGene(i).isOrg(forThisOrg)) {
                all.addAll(createProbes_by_accession(getEntrezGene(i)));
            }
        }

        return (Probe[]) all.toArray(new Probe[all.size()]);
    }

    public List createProbes_by_accession(final EntrezGene gene) {

        Set accessions = gene.getSeqAccessions();
        if (accessions == null || accessions.isEmpty()) {
            return Collections.EMPTY_LIST;
        }

        List probes = new ArrayList();

        for (Iterator iterator = accessions.iterator(); iterator.hasNext();) {
            Object key = iterator.next();
            probes.add(new GeneProbe(key.toString(), gene));
        }

        return probes;
    }

    // common init routine
    private void init(final String source, final Version version, final EntrezGene[] entrezs) {

        if (source == null) {
            throw new IllegalArgumentException("Parameter source cannot be null");
        }

        if (version == null) {
            throw new IllegalArgumentException("Parameter version cannot be null");
        }

        if (entrezs == null) {
            throw new IllegalArgumentException("Parameter entrezs cannot be null");
        }

        super.initialize(source);

        // sanity checks

        Set uniq_ids = new HashSet();
        Set dups = new HashSet();
        List list = new ArrayList();
        for (int i = 0; i < entrezs.length; i++) {
            if (uniq_ids.contains(entrezs[i].getEntrezID())) {
                dups.add(entrezs[i].getEntrezID());
                //dups.add(entrezs[i].getEntrezGeneID());
            } else {
                list.add(entrezs[i]);
                uniq_ids.add(entrezs[i].getSymbol());
            }
        }

        if (dups.isEmpty() == false) {
            // throw new IllegalArgumentException("There were Duplicate records: " + dups.size() + "\n" + dups);
            log.warn("There were Duplicate Entrez records (which were discarded): " + dups.size());
        }

        this.fVersion = version;
        this.fSourcePath = source;
        this.fEntrezGenes = (EntrezGene[]) list.toArray(new EntrezGene[list.size()]);
    }

    public void printfInfo() {
        StringBuffer buf = new StringBuffer();
        buf.append("# of unique entrez records: ").append(fEntrezGenes.length);
        System.out.println(buf.toString());
    }

    public String getSourcePath() {
        return fSourcePath;
    }

    public EntrezGene[] getEntrezGenes() {
        return fEntrezGenes;
    }

    public EntrezGene getEntrezGene(int i) {
        return fEntrezGenes[i];
    }

    public EntrezGene getEntrezGene(long ezid) {
        initGeneIdMap();
        Object obj = fGeneIdEntrezGeneMap.get(ezid);
        if (obj != null) {
            return (EntrezGene) obj;
        }

        throw new IllegalArgumentException("No such Entrez ID: >" + ezid + "<");
    }

    public int getNumRecords() {
        return fEntrezGenes.length;
    }

    public Version getVersion() {
        return fVersion;
    }

    public Map createGeneSymbolEntrezGeneMap() {
        initGeneSymbolMap();
        return new HashMap(fGeneSymbolEntrezGeneMap);
    }

    private Map fGeneSymbolEntrezGeneMap;

    private void initGeneSymbolMap() {
        if (fGeneSymbolEntrezGeneMap == null) {
            fGeneSymbolEntrezGeneMap = new HashMap();
            for (int i = 0; i < fEntrezGenes.length; i++) {
                fGeneSymbolEntrezGeneMap.put(fEntrezGenes[i].getSymbol(), fEntrezGenes[i]);
            }
        }
    }

    private Map fGeneIdEntrezGeneMap;

    private void initGeneIdMap() {
        if (fGeneIdEntrezGeneMap == null) {
            fGeneIdEntrezGeneMap = new HashMap();
            for (int i = 0; i < fEntrezGenes.length; i++) {
                fGeneIdEntrezGeneMap.put(fEntrezGenes[i].getEntrezID(), fEntrezGenes[i]);
            }
        }
    }

    public boolean isSymbol(final String symbol) {
        initGeneSymbolMap();
        return fGeneSymbolEntrezGeneMap.containsKey(symbol);
    }

    public EntrezGene getEntrezGene(final String symbol) {
        initGeneSymbolMap();
        Object obj = fGeneSymbolEntrezGeneMap.get(symbol);
        if (obj != null) {
            return (EntrezGene) obj;
        } else {
            return null;
        }
    }


    public Map createGeneSymbolAccessionMap() {

        final Map symbolAccessionMap = new HashMap();
        for (int i = 0; i < getNumRecords(); i++) {
            final EntrezGene eg = getEntrezGene(i);
            if (eg.getSymbol() != null) {
                final Set accessions = eg.getSeqAccessions();
                if (accessions != null && !accessions.isEmpty()) {
                    Object curr = symbolAccessionMap.get(eg.getSymbol());
                    if (curr == null) {
                        curr = new HashSet();
                    }
                    ((Set) curr).addAll(accessions);
                    symbolAccessionMap.put(eg.getSymbol(), curr);
                }
            }
        }

        return symbolAccessionMap;
    }

    private Map fEntrezIdGeneSymbolMap;

    public Map createEntrezIdGeneSymbolMap() {
        if (fEntrezIdGeneSymbolMap == null) {
            final Map map = new HashMap();
            for (int i = 0; i < getNumRecords(); i++) {
                final EntrezGene eg = getEntrezGene(i);
                if (!Gene.Helper.isNull(eg)) {
                    map.put(eg.getEntrezID(), eg.getSymbol());
                }
            }

            this.fEntrezIdGeneSymbolMap = map;
        }

        return fEntrezIdGeneSymbolMap;
    }

    public String getSymbol(final String ezid) {
        createEntrezIdGeneSymbolMap();
        Object obj = fEntrezIdGeneSymbolMap.get(ezid);
        if (obj != null) {
            return obj.toString();
        } else {
            return null;
        }
    }

    public Set getSymbols(final Set ezids) {
        Set matches = new HashSet();
        for (Iterator iterator = ezids.iterator(); iterator.hasNext();) {
            Object key = iterator.next();
            String symbol = getSymbol(key.toString());
            if (symbol != null) {
                matches.add(symbol);
            }
        }

        return matches;
    }

    public Set getSymbols() {
        Set set = new HashSet();
        for (int i = 0; i < getNumRecords(); i++) {
            EntrezGene eg = getEntrezGene(i);
            if (!Gene.Helper.isNull(eg)) {
                set.add(getEntrezGene(i).getSymbol());
            }
        }

        return set;
    }

} // End class EntrezGeneDb
