/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.meg;

import edu.mit.broad.genome.Version;
import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.vdb.Vdb;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Jan 1, 2004
 *         Time: 11:50:38 PM
 */
public class GeneDb extends AbstractObject implements Vdb {

    private String fSourcePath;
    private Version fVersion;
    private Gene[] fGenes;

    /**
     * Class constructor
     *
     * @param source
     * @param version
     * @param genes
     */
    public GeneDb(final String source, final Version version, final Gene[] genes) {
        init(source, version, genes);
    }

    /**
     * Class constructor
     *
     * @param source
     * @param version
     * @param genes
     */
    public GeneDb(final File source, final Version version, final Gene[] genes) {
        init(source.getPath(), version, genes);
    }

    public String getQuickInfo() {
        return null;
    }

    // common init routine
    private void init(final String source, final Version version, final Gene[] genes) {

        if (source == null) {
            throw new IllegalArgumentException("Parameter source cannot be null");
        }

        if (version == null) {
            throw new IllegalArgumentException("Parameter version cannot be null");
        }

        if (genes == null) {
            throw new IllegalArgumentException("Parameter hugos cannot be null");
        }

        super.initialize(source);

        // sanity checks

        Set ids = new HashSet();
        Set dups = new HashSet();
        for (int i = 0; i < genes.length; i++) {
            if (ids.contains(genes[i].getSymbol())) {
                dups.add(genes[i].getSymbol());
            }

            ids.add(genes[i].getSymbol());
        }

        if (dups.isEmpty() == false) {
            throw new IllegalArgumentException("There were Duplicate HUGO records: " + dups.size() + "\n" + dups);
        }


        this.fVersion = version;
        this.fSourcePath = source;
        this.fGenes = genes;
    }

    public void printfInfo() {
        StringBuffer buf = new StringBuffer();
        buf.append("# of unique hugo records: ").append(fGenes.length);
        System.out.println(buf.toString());
    }

    public String getSourcePath() {
        return fSourcePath;
    }

    public Gene[] getHugos() {
        return fGenes;
    }

    public Gene getHugo(int i) {
        return fGenes[i];
    }

    public int getNumRecords() {
        return fGenes.length;
    }

    public Version getVersion() {
        return fVersion;
    }

    public Map createGeneSymbolHugoMap() {
        initGeneSymbolMap();
        return new HashMap(fGeneSymbolHugoMap);
    }

    private Map fGeneSymbolHugoMap;

    private void initGeneSymbolMap() {
        if (fGeneSymbolHugoMap == null) {
            fGeneSymbolHugoMap = new HashMap();
            for (int i = 0; i < fGenes.length; i++) {
                fGeneSymbolHugoMap.put(fGenes[i].getSymbol(), fGenes[i]);
            }
        }
    }

    public boolean isSymbol(final String symbol) {
        initGeneSymbolMap();
        return fGeneSymbolHugoMap.containsKey(symbol);
    }

    public Gene getHugo(final String symbol) {
        initGeneSymbolMap();
        Object obj = fGeneSymbolHugoMap.get(symbol);
        if (obj != null) {
            return (Gene) obj;
        } else {
            return null;
        }
    }


    public Map createGeneSymbolAccessionMap() {

        final Map symbolAccessionMap = new HashMap();
        for (int i = 0; i < getNumRecords(); i++) {
            final Gene gene = getHugo(i);
            if (gene.getSymbol() != null) {
                final Set accessions = gene.getSeqAccessions();
                if (accessions != null && !accessions.isEmpty()) {
                    Object curr = symbolAccessionMap.get(gene.getSymbol());
                    if (curr == null) {
                        curr = new HashSet();
                    }
                    ((Set) curr).addAll(accessions);
                    symbolAccessionMap.put(gene.getSymbol(), curr);
                }
            }
        }

        return symbolAccessionMap;
    }

    public Set getSymbols() {
        Set set = new HashSet();
        for (int i = 0; i < getNumRecords(); i++) {
            Gene gene = getHugo(i);
            if (!Gene.Helper.isNull(gene)) {
                set.add(getHugo(i).getSymbol());
            }
        }

        return set;
    }

} // End class HugoDb
