/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.meg;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.Version;
import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.vdb.Vdb;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.chip.FileInMemoryChip;
import edu.mit.broad.vdb.chip.Probe;

import java.io.File;
import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Jan 1, 2004
 *         Time: 8:40:33 PM
 */
public class Unigene extends AbstractObject implements Vdb {

    private String fSourcePath;
    private Version fVersion;
    private Unigene.Record[] fUnigenes;

    /**
     * Class constructor
     *
     * @param source
     * @param version
     * @param unigenes
     */
    public Unigene(final String source, final Version version, final Unigene.Record[] unigenes) {
        init(source, version, unigenes);
    }

    public Unigene(final File source, final Version version, final Unigene.Record[] unigenes) {
        init(source.getPath(), version, unigenes);
    }

    private void init(final String source, final Version version, final Unigene.Record[] unigenes) {
        if (source == null) {
            throw new IllegalArgumentException("Parameter source cannot be null");
        }

        if (version == null) {
            throw new IllegalArgumentException("Parameter version cannot be null");
        }

        if (unigenes == null) {
            throw new IllegalArgumentException("Parameter unigenes cannot be null");
        }

        super.initialize(source);

        // sanity checks

        Set ids = new HashSet();
        for (int i = 0; i < unigenes.length; i++) {
            if (ids.contains(unigenes[i].getUnigeneId())) {
                throw new IllegalArgumentException("Duplicate Unigene record: " + unigenes[i].getUnigeneId());
            }

            ids.add(unigenes[i].getUnigeneId());
        }

        this.fVersion = version;
        this.fSourcePath = source;
        this.fUnigenes = unigenes;
    }

    public String getQuickInfo() {
        return fVersion + " " + getNumRecords();
    }


    private Chip fChip;

    public Chip createChip() {
        if (fChip == null) {
            this.fChip = new FileInMemoryChip("Unigene", null, getRecords());
        }

        return fChip;
    }

    public int getNumRecords() {
        return fUnigenes.length;
    }

    public Unigene.Record getRecord(int i) {
        return fUnigenes[i];
    }

    public Unigene.Record[] getRecords() {
        return fUnigenes;
    }

    public Map createGeneSymbolAccessionMap() {

        final Map symbolAccessionMap = new HashMap();
        for (int i = 0; i < getNumRecords(); i++) {
            Unigene.Record recd = getRecord(i);
            if (recd.getGeneSymbol() != null) {
                Set accessions = new HashSet();
                accessions.add(recd.getUnigeneId()); // always add this
                Set accessionsw = recd.getSeqAccessions();
                if (accessionsw != null) {
                    accessions.addAll(accessionsw);
                }

                Object curr = symbolAccessionMap.get(recd.getGeneSymbol());
                if (curr == null) {
                    curr = new HashSet();
                }
                ((Set) curr).addAll(accessions);
                symbolAccessionMap.put(recd.getGeneSymbol(), curr);
            }
        }

        return symbolAccessionMap;
    }

    private Map fSymbolRecordMap;

    private void createGeneSymbolRecordMap() {

        if (this.fSymbolRecordMap == null) {
            this.fSymbolRecordMap = new HashMap();
            for (int i = 0; i < getNumRecords(); i++) {
                Unigene.Record recd = getRecord(i);
                if (recd.getGeneSymbol() != null) {
                    fSymbolRecordMap.put(recd.getGeneSymbol(), recd); // over write any pre-existing gunk
                }
            }
        }

    }

    public Record getRecord(String geneSymbol) {
        createGeneSymbolRecordMap();
        Object obj = this.fSymbolRecordMap.get(geneSymbol);
        if (obj != null) {
            return (Record) obj;
        }

        return null;
    }

    public void printfInfo() {
        StringBuffer buf = new StringBuffer();
        buf.append("# of unique unigene records by symbol: ").append(fUnigenes.length);

        final Map hugoUnigeneSetMap = new HashMap();

        for (int i = 0; i < fUnigenes.length; i++) {
            String symbol = fUnigenes[i].getGeneSymbol();
            if (symbol != null) {
                Object curr = hugoUnigeneSetMap.get(symbol);
                if (curr == null) {
                    curr = new HashSet();
                }
                ((Set) curr).add(fUnigenes[i]);
                hugoUnigeneSetMap.put(symbol, curr);
            }
        }

        buf.append("# of hugos with unigenes: ").append(hugoUnigeneSetMap.size());

        System.out.println(buf.toString());
    }

    public String getSourcePath() {
        return fSourcePath;
    }

    public Unigene.Record[] getUnigenes() {
        return fUnigenes;
    }

    public Version getVersion() {
        return fVersion;
    }

    /**
     * Inner class that represents
     * a single unigene record
     */
    public static class Record implements Probe {

        private String fUnigeneId;

        private String fGeneTitle;

        private String fGeneSymbol;

        private String fCytoBand;

        private String fLocusLinkId;

        private Set fSeqAccessions;

        /**
         * Class constructor
         *
         * @param unigeneid
         * @param title
         * @param geneSymbol
         * @param cytoband
         * @param llid
         * @param seqAccessions
         */
        public Record(final String unigeneid,
                      final String geneSymbol,
                      final String title,
                      final String cytoband,
                      final String llid,
                      final Set seqAccessions) {

            if (unigeneid == null) {
                throw new IllegalArgumentException("Parameter id cannot be null");
            }

            NamingConventions.enforceUnigeneIdFormat(unigeneid);

            this.fUnigeneId = unigeneid;
            this.fGeneTitle = title;
            this.fGeneSymbol = geneSymbol;
            this.fCytoBand = cytoband;
            this.fLocusLinkId = llid;
            this.fSeqAccessions = Collections.unmodifiableSet(seqAccessions);

        }

        public Probe cloneDeepProbe() {
            throw new NotImplementedException();
        }

        public String getName() {
            return getUnigeneId();
        }

        private Gene fGene;

        public Gene getGene() {
            if (fGene == null) {
                if (getGeneSymbol() == null) {
                    this.fGene = Gene.NULL_GENE;
                } else {
                    this.fGene = new DefaultGene(getGeneSymbol(), getGeneTitle(), null, null, getSeqAccessions());
                }
            }

            return fGene;
        }

        public int hashCode() {
            return fUnigeneId.hashCode();
        }

        public String toString() {
            return fUnigeneId;
        }

        public boolean equals(final Object obj) {
            if (obj instanceof Record) {
                return ((Record) obj).getUnigeneId().equalsIgnoreCase(this.fUnigeneId);
            } else {
                return false;
            }
        }

        public String getUnigeneId() {
            return fUnigeneId;
        }

        public String getGeneTitle() {
            return fGeneTitle;
        }

        public String getGeneSymbol() {
            return fGeneSymbol;
        }

        public Set getSeqAccessions() {
            return fSeqAccessions;
        }

        public String getCytoBand() {
            return fCytoBand;
        }

        public String getLocusLinkId() {
            return fLocusLinkId;
        }

    } // End inner class Record

} // End Unigene

