package edu.mit.broad.vdb.msigdb;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.chip.FileInMemoryChip;
import edu.mit.broad.vdb.chip.Probe;
import edu.mit.broad.vdb.chip.SimpleProbe;
import edu.mit.broad.vdb.meg.Gene;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Nov 1, 2005
 *         Time: 9:27:26 AM
 */
public class DeconAccessions {

    private static final Logger klog = XLogger.getLogger(DeconAccessions.class);

    /**
     * Class constructor
     *
     * @param identifiers
     * @throws Exception
     */
    private DeconAccessions() throws Exception {
    }

    public static GeneSetMatrix symbolize(final GeneSetMatrix gm_orig,
                                          final boolean omitInvalids,
                                          final boolean suffix_gene_set_name) throws Exception {
        final GeneSet[] deconed = new GeneSet[gm_orig.getNumGeneSets()];
        for (int g = 0; g < gm_orig.getNumGeneSets(); g++) {
            deconed[g] = symbolize(gm_orig.getGeneSet(g), omitInvalids, suffix_gene_set_name);
        }

        return new DefaultGeneSetMatrix(gm_orig.getName() + "_symbolized", deconed);
    }

    public static GeneSet symbolize(final GeneSet orig,
                                    final boolean omitInvalids,
                                    final boolean suffix_gene_set_name) throws Exception {

        final Set set = new HashSet();
        for (int i = 0; i < orig.getNumMembers(); i++) {
            final String orig_id = orig.getMember(i);
            final Lookup lookup = creatLookup(orig_id);

            if (lookup.hasSymbolMatch() == false) {
                if (omitInvalids == false) {
                    set.add(orig_id);
                } // else ignore
            } else {
                set.add(lookup.getMatch().getSymbol()); // i.e the symbol or the alias AFTER converting to a symbol (if it was an alias)
            }
        }

        String newName;

        if (suffix_gene_set_name) {
            newName = orig.getName(true) + "_symbolized";
        } else {
            newName = orig.getName(true);
        }

        return new FSet(newName, set);
    }

    public static GeneSet getOnlyExactMatches(final GeneSet orig) throws Exception {
        final Set set = new HashSet();
        for (int i = 0; i < orig.getNumMembers(); i++) {
            final String orig_id = orig.getMember(i);
            final Lookup lookup = creatLookup(orig_id);
            if (lookup.isDitto()) {
                set.add(lookup.getMatch().getSymbol());
            }
        }

        return new FSet(orig.getName() + "_only_exact_matches", set);
    }

    public static GeneSet getOnlyAliases(final GeneSet orig) throws Exception {
        final Set set = new HashSet();
        for (int i = 0; i < orig.getNumMembers(); i++) {
            final String orig_id = orig.getMember(i);
            final Lookup lookup = creatLookup(orig_id);
            if (lookup.isAlias()) {
                set.add(lookup.getMatch().getSymbol());
            }
        }

        return new FSet(orig.getName() + "_only_aliases", set);
    }

    public static GeneSet getOnlyInvalids(final GeneSet orig) throws Exception {
        final Set set = new HashSet();
        for (int i = 0; i < orig.getNumMembers(); i++) {
            final String orig_id = orig.getMember(i);
            final Lookup lookup = creatLookup(orig_id);
            if (!lookup.hasSymbolMatch()) {
                set.add(orig_id);
            }
        }

        return new FSet(orig.getName() + "_only_invalids", set);
    }

    public static StringDataframe createReport(final String name, final Set orignalIds) throws Exception {
        return createReport(new FSet(name, orignalIds));
    }

    public static StringDataframe createReport(final Chip chip) throws Exception {
        return createReport(new FSet(chip.getName(), chip.getSymbols(Chip.OMIT_NULLS)));
    }

    public static StringDataframe createReport(final GeneSet orignalIds) throws Exception {
        final String[] colNames = new String[]{"STATUS", "LOOKED_UP_SYMBOL", "TITLE"};
        final String[] rowNames = new String[orignalIds.getNumMembers()];
        int numInvalid = 0;
        int numAliased = 0;
        int numAccessions = 0;
        StringMatrix sm = new StringMatrix(rowNames.length, colNames.length);
        for (int r = 0; r < orignalIds.getNumMembers(); r++) {
            String member = orignalIds.getMember(r);
            rowNames[r] = member;
            Lookup lookup = creatLookup(member);
            if (lookup.isDitto()) {
                sm.setElement(r, 0, "valid_symbol");
                sm.setElement(r, 1, lookup.getMatch().getSymbol());
                sm.setElement(r, 2, lookup.getMatch().getTitle());
            } else if (lookup.hasSymbolMatch() == false) {
                numInvalid++;
                sm.setElement(r, 0, "no_match");
            } else if (lookup.isAlias()) {
                numAliased++;
                sm.setElement(r, 0, "alias");
                sm.setElement(r, 1, lookup.getMatch().getSymbol());
                sm.setElement(r, 2, lookup.getMatch().getTitle());
            } else {
                numAccessions++;
                sm.setElement(r, 0, "accession");
                sm.setElement(r, 1, lookup.getMatch().getSymbol());
                sm.setElement(r, 2, lookup.getMatch().getTitle());
            }
        }

        klog.info("# unique symbols: " + orignalIds.getNumMembers() + " #aliased: " + numAliased + " #accessions " + numAccessions + " #invalid: " + numInvalid);
        return new StringDataframe(orignalIds.getName(), sm, rowNames, colNames, true);
    }

    // Changes any aliases or accessions in the
    public static Chip createDeconChip(final Chip origChip) throws Exception {

        final List probes = new ArrayList();

        int numChanged = 0;

        for (int i = 0; i < origChip.getNumProbes(); i++) {
            final Probe probe = origChip.getProbe(i);
            final String name = probe.getName();
            final Gene gene = probe.getGene();
            String symbol = null;
            String title = null;

            if (!NamingConventions.isNull(gene)) {
                symbol = gene.getSymbol();
                title = gene.getTitle();

                // @note this is the magix -> turn the alias into a true symbol
                final Lookup lookup = creatLookup(symbol);
                if (lookup.isAlias()) {
                    Gene deconH = lookup.getMatch();
                    symbol = deconH.getSymbol();
                    title = deconH.getTitle();
                    numChanged++;
                }

            }

            Probe deconProbe = new SimpleProbe(name, symbol, title);
            probes.add(deconProbe);
        }

        klog.info("For chip: " + origChip.getName() + " # changed: " + numChanged + " / " + origChip.getNumProbes() + " probes");

        return new FileInMemoryChip(origChip.getName(), Chip.COMBO_DUMMY_SOURCE,
                (Probe[]) probes.toArray(new Probe[probes.size()]));
    }

    public static Lookup[] createLookups(final GeneSet originalIds) throws Exception {
        final Lookup[] lookups = new Lookup[originalIds.getNumMembers()];
        for (int i = 0; i < originalIds.getNumMembers(); i++) {
            lookups[i] = creatLookup(originalIds.getMember(i));
        }

        return lookups;
    }


    public static Lookup creatLookup(final String originalId) throws Exception {
        Gene gene = null;
        boolean isAlias = false;

        if (VdbRuntimeResources.getChip_Gene_Symbol().isProbe(originalId)) {
            gene = VdbRuntimeResources.getChip_Gene_Symbol().getProbe(originalId).getGene();
            isAlias = false;
        } else {

            if (VdbRuntimeResources.getAliasDb().isAlias(originalId)) {
                gene = VdbRuntimeResources.getAliasDb().getHugo(originalId);
                isAlias = true;
            } else {
                // seq accessions is a special chip: accessions + symbols + aliases
                if (VdbRuntimeResources.getChip_Seq_Accession().isProbe(originalId)) {
                    gene = VdbRuntimeResources.getChip_Seq_Accession().getHugo(originalId);
                    isAlias = false;
                }
            }
        }

        /* OLD WAY
        // seq accessions is a special chip: accessions + symbols + aliases
        // so this WILL pick up aliases
        if (VdbRuntimeResources.getChip_Seq_Accession().isProbe(originalId)) {
            hugo = VdbRuntimeResources.getChip_Seq_Accession().getHugo(originalId);
        }
        */

        return new Lookup(originalId, gene, isAlias);
    }

    /**
     * Inner class representing one lookup
     */
    public static class Lookup {

        private String fOriginalId;

        private Gene fMatch;

        private boolean isAlias;

        Lookup(final String original, final Gene match, final boolean isAnAlias) {
            this.fOriginalId = original;
            this.fMatch = match;
            this.isAlias = isAnAlias;
        }

        public String getOriginal() {
            return fOriginalId;
        }

        public String getSymbol() {
            if (fMatch != null) {
                return fMatch.getSymbol();
            } else {
                return fOriginalId;
            }
        }

        public Gene getMatch() {
            return fMatch;
        }

        public boolean isDitto() {
            return fMatch != null && fMatch.getSymbol().equalsIgnoreCase(fOriginalId);
        }

        public boolean hasSymbolMatch() {
            return fMatch != null;
        }

        public boolean isAlias() {
            return isAlias;
            //return !isDitto() && !isNull();
        }

    } // End class Lookup

} // End class DeconAccessions
