/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.msigdb;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import gnu.trove.THashMap;

import java.util.*;


/**
 * @author Aravind Subramanian
 *         Date: Jun 1, 2004
 *         Time: 2:33:37 AM
 */
public class MSigDBImpl extends AbstractObject implements MSigDB {

    private GeneSetAnnotation[] fGeneSetAnns;

    private String fVersion;

    private String fBuildDate;

    private Map fGeneSetNameGeneSetAnnotationMap;

    private FeatureAnnot fFeatureAnnot;

    private SampleAnnot fSampleAnnot;

    /**
     * Class constructor
     *
     * @param name
     * @param gsanns
     */
    public MSigDBImpl(final String name, final String version, final String build_date, final GeneSetAnnotation[] gsanns) {
        initHere(name, version, build_date, gsanns);
    }

    public MSigDBImpl(final String name, final String version, final String build_date, final MSigDB[] dbs) {
        List list = new ArrayList();
        for (int i = 0; i < dbs.length; i++) {
            list.addAll(dbs[i].getGeneSetAnnotations_set());
        }

        this.initHere(name, version, build_date, (GeneSetAnnotation[]) list.toArray(new GeneSetAnnotation[list.size()]));
    }

    private void initHere(final String name, final String version, final String build_date, final GeneSetAnnotation[] gsanns) {
        super.initialize(name);
        // chekc to make sure names are unique

        Set duplicates = new HashSet();
        Set all = new HashSet();
        this.fGeneSetNameGeneSetAnnotationMap = new THashMap();
        for (int i = 0; i < gsanns.length; i++) {
            if (all.contains(gsanns[i].getStandardName())) {
                duplicates.add(gsanns[i].getStandardName());
            } else {
                all.add(gsanns[i].getStandardName());
                fGeneSetNameGeneSetAnnotationMap.put(gsanns[i].getStandardName(), gsanns[i]);
            }
        }

        if (!duplicates.isEmpty()) {
            throw new IllegalArgumentException("Duplicate gene set annotations: " + duplicates.size() + "\n" + duplicates);
        }

        this.fGeneSetAnns = gsanns;
        this.fVersion = version;
        this.fBuildDate = build_date;
    }

    public String getQuickInfo() {
        return null;
    }

    public String[] getStandardNames() {
        final String[] stdnames = new String[getNumGeneSets()];
        for (int i = 0; i < getNumGeneSets(); i++) {
            stdnames[i] = getGeneSetAnnotation(i).getStandardName();
        }

        return stdnames;
    }

    public FeatureAnnot getAsFeatureAnnot() {
        if (fFeatureAnnot == null) {
            this.fFeatureAnnot = new FeatureAnnotMSigDbImpl(this);
        }

        return fFeatureAnnot;
    }

    public SampleAnnot getAsSampleAnnot() {
        if (fSampleAnnot == null) {
            String[] stdnames = getStandardNames();
            this.fSampleAnnot = new SampleAnnotImpl(getName(), stdnames, new ColumnsImpl(stdnames));
        }

        return fSampleAnnot;
    }

    public String getVersion() {
        return fVersion;
    }

    public String getBuildDate() {
        return fBuildDate;
    }

    public GeneSetMatrix createGeneSetMatrix(final String[] gsetNames, final boolean originalIds) {

        final GeneSet[] gsets = new GeneSet[gsetNames.length];
        for (int i = 0; i < gsetNames.length; i++) {
            gsets[i] = getGeneSetAnnotation(gsetNames[i]).getGeneSet(originalIds);
        }

        String name = "gsets_" + gsetNames.length + "_" + _toName(originalIds);
        return new DefaultGeneSetMatrix(name, gsets);
    }

    public GeneSetMatrix createGeneSetMatrix(final boolean originalIds) {
        final GeneSet[] gsets = new GeneSet[getNumGeneSets()];
        for (int i = 0; i < getNumGeneSets(); i++) {
            gsets[i] = getGeneSetAnnotation(i).getGeneSet(originalIds);
        }

        return new DefaultGeneSetMatrix(_toName(originalIds), gsets);
    }

    private String _toName(final boolean originalIds) {
        String name = getName();
        if (originalIds) {
            name = name + "." + getVersion() + ".native_ids";
        } else {
            name = name + "." + getVersion() + ".symbols";
        }

        return name;
    }

    public GeneSetAnnotation[] getGeneSetAnnotations() {
        return fGeneSetAnns;
    }

    public Set getGeneSetAnnotations_set() {
        Set set = new HashSet();
        for (int i = 0; i < fGeneSetAnns.length; i++) {
            set.add(fGeneSetAnns[i]);
        }

        return set;
    }

    public MSigDB[] split(boolean byCategory, boolean bySubCategory) {
        final List all = new ArrayList();

        if (byCategory || bySubCategory) {
            // first split by category
            final MSigDB[] db_cats = this.splitByCategory();
            if (byCategory) {
                for (int c = 0; c < db_cats.length; c++) {
                    all.add(db_cats[c]);
                }
            }

            if (bySubCategory) {
                for (int i = 0; i < db_cats.length; i++) {
                    MSigDB[] subc = db_cats[i].splitByTags();
                    for (int c = 0; c < subc.length; c++) {
                        all.add(subc[c]);
                    }
                }
            }
        }

        return (MSigDB[]) all.toArray(new MSigDB[all.size()]);
    }

    public MSigDB[] splitByTags() {
        final Map map = new HashMap();

        for (int i = 0; i < fGeneSetAnns.length; i++) {
            String[] tags = fGeneSetAnns[i].getTagsArray();

            for (int t = 0; t < tags.length; t++) {

                Object curr = map.get(tags[t]);
                if (curr == null) {
                    curr = new ArrayList();
                }
                ((List) curr).add(fGeneSetAnns[i]);
                map.put(tags[t], curr);
            }
        }

        // @note assumes only 1 category
        String categName = "";
        if (fGeneSetAnns.length > 0) {
            categName = fGeneSetAnns[0].getCategory().getCode();
        }

        final MSigDB[] dbs = new MSigDB[map.size()];
        int cnt = 0;
        for (Iterator iterator = map.keySet().iterator(); iterator.hasNext();) {
            String sc_name = iterator.next().toString();
            List anns = (List) map.get(sc_name);
            dbs[cnt++] = new MSigDBImpl(categName + "." + sc_name,
                    fVersion, fBuildDate,
                    (GeneSetAnnotation[]) anns.toArray(new GeneSetAnnotation[anns.size()]));
        }

        return dbs;
    }

    public MSigDB[] splitByCategory() {
        final Map map = new HashMap();

        for (int i = 0; i < fGeneSetAnns.length; i++) {
            Object curr = map.get(fGeneSetAnns[i].getCategory().getCode());
            if (curr == null) {
                curr = new ArrayList();
            }
            ((List) curr).add(fGeneSetAnns[i]);
            map.put(fGeneSetAnns[i].getCategory().getCode(), curr);
        }

        final MSigDB[] dbs = new MSigDB[map.size()];
        int cnt = 0;
        for (Iterator iterator = map.keySet().iterator(); iterator.hasNext();) {
            Object key = iterator.next();
            List anns = (List) map.get(key);
            dbs[cnt++] = new MSigDBImpl(key.toString(), fVersion, fBuildDate, (GeneSetAnnotation[]) anns.toArray(new GeneSetAnnotation[anns.size()]));
        }

        return dbs;
    }

    public GeneSetAnnotation getGeneSetAnnotation(int g) {
        return fGeneSetAnns[g];
    }

    public GeneSetAnnotation getGeneSetAnnotation(final String gsStdName) {

        if (gsStdName == null) {
            throw new IllegalArgumentException("Param gsStdName cannot be null");
        }

        Object obj = fGeneSetNameGeneSetAnnotationMap.get(gsStdName);
        if (obj == null) {
            // try harder
            obj = fGeneSetNameGeneSetAnnotationMap.get(gsStdName.toUpperCase());
            if (obj == null) {
                throw new IllegalArgumentException("No gset ann with std name: " + gsStdName);
            }
        }

        return (GeneSetAnnotation) obj;
    }

    public int getNumGeneSets() {
        return fGeneSetAnns.length;
    }

    public int getNumGenes() {
        throw new NotImplementedException();
    }

    public GeneSet getAllGenes(final boolean originalIds) {
        Set all = new HashSet();
        for (int i = 0; i < getNumGeneSets(); i++) {
            GeneSetAnnotation gann = getGeneSetAnnotation(i);
            GeneSet gset = gann.getGeneSet(originalIds);
            all.addAll(gset.getMembersS());
        }

        return new FSet(getName(), all);
    }

} // End class MSigDbImpl

