package edu.mit.broad.vdb.sampledb;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.DatasetMetrics;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Sep 6, 2005
 *         Time: 4:46:51 PM
 */// A category (i.e column) such as Lineage, Source etc
public class Category {

    private Logger log = XLogger.getLogger(Category.class);

    private static final Logger klog = XLogger.getLogger(Category.class);

    private String fName;

    private State[] fStates;

    private SampleAnnot fAnnot;

    /**
     * IMP: Keep this protected
     * Class constructor
     *
     * @param name
     * @param states
     */
    public Category(final String name, final State[] states, final SampleAnnot annot) {

        if (name == null) {
            throw new IllegalArgumentException("Param name cannot be null");
        }

        if (states == null) {
            throw new IllegalArgumentException("Param states cannot be null");
        }

        if (annot == null) {
            throw new IllegalArgumentException("Param annot cannot be null");
        }

        for (int i = 0; i < states.length; i++) {
            if (states[i] == null) {
                throw new IllegalArgumentException("States cannot be null at: " + i);
            }
        }

        this.fName = name;
        this.fStates = states;
        this.fAnnot = annot;
    }

    public String getName() {
        return fName;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Dataset collapseSamplesIntoStates(final Dataset ds, final boolean useMean) {

        final Matrix m = new Matrix(ds.getNumRow(), getNumStates());

        for (int s = 0; s < getNumStates(); s++) {
            State state = getState(s);
            String[] colNames = state.getSampleNames();
            //Printf.out(colNames);
            // now combine by taking the average value for a gene across all samples
            final edu.mit.broad.genome.math.Vector values = new edu.mit.broad.genome.math.Vector(ds.getNumRow());
            for (int r = 0; r < ds.getNumRow(); r++) {
                edu.mit.broad.genome.math.Vector v = new edu.mit.broad.genome.math.Vector(colNames.length);
                for (int c = 0; c < colNames.length; c++) {
                    v.setElement(c, ds.getElement(r, colNames[c]));
                }

                values.setElement(r, v.meanOrMedian(useMean));
            }

            m.setColumn(s, values);
        }

        Annot annot = null;
        if (ds.getAnnot() != null && ds.getAnnot().getFeatureAnnot() != null) {
            SampleAnnot sampleAnnot = new SampleAnnotImpl(getName(), getStateNames(), null);
            annot = new AnnotImpl(ds.getAnnot().getFeatureAnnot(), sampleAnnot);
        }

        return new DefaultDataset(ds.getName() + "_state_collapsed_" + getNumStates(), m, ds.getRowNamesArray(), getStateNames(), true, annot);
    }

    // -------------------------------------------------------------------------------------------- //

    public GeneSetMatrixPair createGeneSets_ova(final String refState, final Dataset ds, final int numMarkers) {
        final Template[] tss = createTemplates(refState, ds);
        return _makeSets(refState, ds, tss, numMarkers);
    }

    public GeneSetMatrixPair createGeneSets_ova(final Dataset ds, final int numMarkers) {
        final TemplateFromAnnot dt = createTemplate();
        final Template[] tss = TemplateFactory.extractAllOvaTemplates(dt, true);
        return _makeSets("ova", ds, tss, numMarkers);
    }

    public GeneSetMatrixPair createGeneSets_ap(final Dataset ds, final int numMarkers) {
        final TemplateFromAnnot dt = createTemplate();
        final Template[] tss = TemplateFactory.extractAllPairsTemplates(dt);
        log.debug("# of templates =" + tss.length + " from states: " + getNumStates());
        return _makeSets("ap", ds, tss, numMarkers);
    }

    public GeneSetPair[] createGeneSets_ap2(final Dataset ds, final int numMarkers) {
        final TemplateFromAnnot dt = createTemplate();
        final Template[] tss = TemplateFactory.extractAllPairsTemplates(dt);
        return _makeSets_gsp(ds, tss, numMarkers);
    }

    private static GeneSetMatrixPair _makeSets(final String prefix,
                                               final Dataset fullDs,
                                               final Template[] tss,
                                               final int numMarkers) {


        final DatasetMetrics dm = new DatasetMetrics();

        final List gsets_up = new ArrayList();
        final List gsets_dn = new ArrayList();

        final RankedList[] rls = new RankedList[tss.length];

        for (int t = 0; t < tss.length; t++) {
            //for (int t = 0; t < 1; t++) {
            final String tname = tss[t].getName();

            System.out.println("Doing: " + tname + " " + (t + 1) + " / " + tss.length);

            final DatasetTemplate subdt = new DatasetGenerators().extract(fullDs, tss[t]);
            final RankedList rl = dm.scoreDataset(subdt).cloneShallowRL(tname);

            rls[t] = rl;

            final GeneSet up = rl.getNamesOfUpOrDnXRanks_gs(numMarkers, true);
            gsets_up.add(up);
            final GeneSet dn = rl.getNamesOfUpOrDnXRanks_gs(numMarkers, false);
            gsets_dn.add(dn);
        }

        // Save the gene sets
        GeneSetMatrix gm_up = new DefaultGeneSetMatrix(prefix + "_up", gsets_up);
        GeneSetMatrix gm_dn = new DefaultGeneSetMatrix(prefix + "_dn", gsets_dn);

        return new GeneSetMatrixPairImpl(gm_up, gm_dn);
    }

    private static GeneSetPair[] _makeSets_gsp(final Dataset fullDs,
                                               final Template[] tss,
                                               final int numMarkers) {


        final DatasetMetrics dm = new DatasetMetrics();

        GeneSetPair[] pairs = new GeneSetPair[tss.length];

        final RankedList[] rls = new RankedList[tss.length];

        for (int t = 0; t < tss.length; t++) {
            final String tname = tss[t].getName();
            System.out.println("Doing: " + tname + " " + (t + 1) + " / " + tss.length);
            final DatasetTemplate subdt = new DatasetGenerators().extract(fullDs, tss[t]);
            final RankedList rl = dm.scoreDataset(subdt).cloneShallowRL(tname);

            rls[t] = rl;

            final GeneSet up = rl.getNamesOfUpOrDnXRanks_gs(numMarkers, true);
            final GeneSet dn = rl.getNamesOfUpOrDnXRanks_gs(numMarkers, false);
            pairs[t] = new GeneSetPairImpl(up, dn);
        }

        return pairs;
    }

    // -------------------------------------------------------------------------------------------- //
    // ----------------------------------------- STATE METHODS ------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public int getNumStates() {
        return fStates.length;
    }

    public int getNumSamples(final String stateName) {
        return getState(stateName).getNumSamples();
    }

    private Map stateNameStateMap;

    public State getState(final String stateName) {
        if (stateNameStateMap == null) {
            this.stateNameStateMap = new HashMap();
            for (int i = 0; i < fStates.length; i++) {
                stateNameStateMap.put(fStates[i].getName(), fStates[i]);
            }
        }

        Object obj = stateNameStateMap.get(stateName);
        if (obj == null) {
            throw new IllegalArgumentException("Np such state in this categ: " + stateName);
        }

        return (State) obj;
    }

    // ALL states
    public State[] getStates() {
        return fStates;
    }

    public State getState(final int s) {
        return fStates[s];
    }

    public StateExpression[] getStateExpressions(final Dataset ds) {

        StateExpression[] sxs = new StateExpression[fStates.length];
        for (int i = 0; i < fStates.length; i++) {
            sxs[i] = new StateExpressionImpl(fStates[i], new DatasetGenerators().extractColumns(fStates[i].getName(), ds, fStates[i].getSampleNames()));
        }

        return sxs;
    }

    public String[] getStateNames() {
        String[] names = new String[fStates.length];
        for (int i = 0; i < fStates.length; i++) {
            names[i] = fStates[i].getName();
        }

        return names;
    }

    public List getStateNames_list() {
        return Arrays.asList(getStateNames());
    }

    public State[] getStates(final Set stateNames) {

        final Iterator iterator = stateNames.iterator();
        final Set ret = new HashSet();

        while (iterator.hasNext()) {
            final String stateName = iterator.next().toString();
            for (int v = 0; v < fStates.length; v++) {
                if (fStates[v].getName().equals(stateName)) {
                    ret.add(fStates[v]);
                }
            }
        }

        return (State[]) ret.toArray(new State[ret.size()]);
    }

    public State[] getStates(final String[] stateNames) {
        // dont do this as we want to keep the order
        /*
        Set set = new HashSet();
        for (int i = 0; i < stateNames.length; i++) {
            set.add(stateNames[i]);
        }

        return getStates(set);
        */

        final List ret = new ArrayList();
        for (int i = 0; i < stateNames.length; i++) {
            final String stateName = stateNames[i];
            for (int v = 0; v < fStates.length; v++) {
                if (fStates[v].getName().equalsIgnoreCase(stateName)) {
                    ret.add(fStates[v]);
                }
            }
        }

        if (ret.size() == 0) {
            throw new IllegalArgumentException("No hits for states: " + Printf.outs(stateNames));
        }

        return (State[]) ret.toArray(new State[ret.size()]);
    }

    // -------------------------------------------------------------------------------------------- //
    // --------------------------------------- SAMPLE METHODS ----------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Sample[] getSamples() {
        List samples = new ArrayList();
        for (int i = 0; i < fStates.length; i++) {
            samples.addAll(fStates[i].getSamples_list());
        }

        return (Sample[]) samples.toArray(new Sample[samples.size()]);
    }

    public String[] getSampleNames() {
        Sample[] samples = getSamples();
        String[] names = new String[samples.length];
        for (int i = 0; i < samples.length; i++) {
            names[i] = samples[i].getName();
        }

        return names;
    }

    public Sample[] getSamples(final String[] stateNames) {
        State[] states = getStates(stateNames);
        List samples = new ArrayList();
        for (int i = 0; i < states.length; i++) {
            samples.addAll(states[i].getSamples_list());
        }

        return (Sample[]) samples.toArray(new Sample[samples.size()]);
    }

    public String[] getSampleNames(final String[] stateNames) {
        Sample[] samples = getSamples(stateNames);

        String[] names = new String[samples.length];
        for (int i = 0; i < samples.length; i++) {
            names[i] = samples[i].getName();
        }

        return names;
    }

    public String[] getSampleNames(final String stateName) {
        return getState(stateName).getSampleNames();
    }

    // -------------------------------------------------------------------------------------------- //
    // --------------------------------------- TEMPLATE METHODS ----------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    // Each state becomes a different template class
    // Uses ALL states

    public TemplateFromAnnot createTemplate() {
        final State[] states = this.getStates();
        return createTemplate(states);
    }

    public TemplateFromAnnot createTemplate(final State[] onlyUseTheseStates) {
        return TemplateFactory.createTemplate_from_annot(getName(), onlyUseTheseStates, fAnnot);
    }

    public TemplateFromAnnot[] createTemplates(final String refStateName, final Dataset ds) {
        throw new NotImplementedException();

    }

    public TemplateFromAnnot[] createTemplates(final String[] refStateNames, final Dataset ds) {
        throw new NotImplementedException();
    }

    public TemplateFromAnnot createTemplate(final String[] onlyUseTheseStates) {
        return TemplateFactory.createTemplate_from_annot(getName(), getStates(onlyUseTheseStates), fAnnot);
    }

    // The resultant template will have its positions synched with the provided dataset
    public TemplateFromAnnot createTemplate(final String[] statesForClassA, final String[] statesForClassB, final Dataset ds) {

        final String classAName = _toName(statesForClassA);
        final String classBName = _toName(statesForClassB);

        final String[] sampleNamesClassA = getSampleNames(statesForClassA);
        final String[] sampleNamesClassB = getSampleNames(statesForClassB);

        String[] ord = new String[sampleNamesClassA.length + sampleNamesClassB.length];
        int cnt = 0;
        for (int i = 0; i < sampleNamesClassA.length; i++) {
            ord[cnt++] = sampleNamesClassA[i];
        }

        for (int i = 0; i < sampleNamesClassB.length; i++) {
            ord[cnt++] = sampleNamesClassB[i];
        }

        Template t = TemplateFactory.createCategoricalTemplate(classAName + "_vs_" + classBName, ds, classAName, sampleNamesClassA, classBName, sampleNamesClassB);

        return new TemplateFromAnnot(t, fAnnot.getSamples(ord), sampleNamesClassA, sampleNamesClassB);
    }

    public TemplateFromAnnot createTemplate(final String stateForClassA, final String[] statesForClassB, final Dataset ds) {
        return createTemplate(new String[]{stateForClassA}, statesForClassB, ds);
    }

    public TemplateFromAnnot createTemplate(final String stateForClassA, final String stateForClassB, final Dataset ds) {
        return createTemplate(new String[]{stateForClassA}, new String[]{stateForClassB}, ds);
    }

    public TemplateFromAnnot createTemplate_ova(final String the_one_state) {
        final TemplateFromAnnot all = createTemplate();
        final Template t = TemplateFactory.extractOvaTemplate(all, all.getClass(the_one_state));
        List sample_names_classA = new ArrayList(Arrays.asList(getSampleNames(the_one_state)));

        final List sample_names_rest = new ArrayList();
        final String[] allSampleNames = getSampleNames();
        for (int i = 0; i < allSampleNames.length; i++) {
            if (sample_names_classA.contains(allSampleNames[i]) == false) {
                sample_names_rest.add(allSampleNames[i]);
            }
        }

        List all_sample_names = new ArrayList();
        all_sample_names.addAll(sample_names_classA);
        all_sample_names.addAll(sample_names_rest);

        final Sample[] samples = fAnnot.getSamples((String[]) all_sample_names.toArray(new String[all_sample_names.size()]));

        return new TemplateFromAnnot(t, samples,
                (String[]) sample_names_classA.toArray(new String[sample_names_classA.size()]),
                (String[]) sample_names_rest.toArray(new String[sample_names_rest.size()]));
    }

    public TemplateFromAnnot[] createTemplate_ap(final String the_one_state, final Dataset ds) {

        final String[] stateNames = getStateNames();
        final List templates = new ArrayList();

        for (int i = 0; i < stateNames.length; i++) {
            if (stateNames[i].equals(the_one_state) == false) {
                templates.add(createTemplate(the_one_state, stateNames[i], ds));
            }
        }

        return (TemplateFromAnnot[]) templates.toArray(new TemplateFromAnnot[templates.size()]);
    }

    public TemplateFromAnnot[] createTemplate_ap_all(final Dataset ds) {

        List all = new ArrayList();
        for (int i = 0; i < getNumStates(); i++) {
            TemplateFromAnnot[] tann = createTemplate_ap(getState(i).getName(), ds);
            all.addAll(Arrays.asList(tann));
        }

        return (TemplateFromAnnot[]) all.toArray(new TemplateFromAnnot[all.size()]);
    }

    public TemplateFromAnnot[] createTemplate_ova_all() {

        TemplateFromAnnot[] tss = new TemplateFromAnnot[getNumStates()];
        for (int s = 0; s < getNumStates(); s++) {
            tss[s] = createTemplate_ova(getState(s).getName());
        }

        return tss;
    }

    /*
    public TemplateFromAnnot[] createTemplate_ap() {
        final TemplateFromAnnot dt = createTemplate();
        return TemplateFactory.extractAllPairsTemplates(dt);
    }
    */

    // -------------------------------------------------------------------------------------------- //

    private static String _toName(final String[] statesForClassA) {
        StringBuffer classAName = new StringBuffer();
        for (int i = 0; i < statesForClassA.length; i++) {
            classAName.append(statesForClassA[i]);
            if (i != statesForClassA.length - 1) {
                classAName.append('_');
            }
        }

        return classAName.toString();
    }

} // End class Category
