/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.GHyperLinkLabel;
import edu.mit.broad.genome.swing.GPopupChecker;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.utils.containers.FileBookmark;
import edu.mit.broad.xbench.core.api.Application;
import org.apache.log4j.Logger;
import org.genepattern.uiutil.FTPFile;

import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

/**
 * Collection of renderers for components bearing (all or some) objects
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class RendererFactory2 {

    /**
     * For logging support
     */
    private static final Logger klog = XLogger.getLogger(RendererFactory2.class);

    public static final Icon FTP_FILE_ICON = JarResources.getIcon("FTPFile.gif");

    /**
     * GramTableCellRenderer for table headers.
     * Headers in bold font -- looks like excel headers.
     */
    public static class BoldHeaderRenderer extends DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int col) {


            if (value == null) {
                value = Constants.NA;
            }

            this.setFont(GuiHelper.FONT_DEFAULT_BOLD);

            this.setText(value.toString());
            this.setToolTipText(value.toString());
            this.setHorizontalAlignment(SwingConstants.CENTER);
            this.setVerticalAlignment(SwingConstants.CENTER);
            this.setBorder(BorderFactory.createRaisedBevelBorder());
            this.setBackground(Color.lightGray);
            this.setForeground(Color.black);

            return this;
        }
    }    // End BoldHeaderRenderer

    /**
     * GramTableCellRenderer for table cell
     * Cells in specified color -- looks like excel headers.
     */
    public static class ColoredBgCellRenderer extends DefaultTableCellRenderer {

        final Color fColor;

        public ColoredBgCellRenderer(Color c) {
            this.fColor = c;
        }

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int col) {

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);

            if (!isSelected) { // else cell selections cant be seen
                this.setBackground(fColor);
            }
            return this;
        }
    }    // End ColoredBgCellRenderer

    /**
     * Adapts any renderer and makes it work on only specified column(s)
     */
    public static class RendererColumnsAdapter extends DefaultTableCellRenderer {

        private final int[] cols;
        private final TableCellRenderer renderer;

        public RendererColumnsAdapter(TableCellRenderer rend, int[] cols2appyrendererto) {
            this.renderer = rend;
            this.cols = cols2appyrendererto;
        }

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int col) {

            for (int c = 0; c < cols.length; c++) {
                if (cols[c] == col) {
                    return renderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
                }
            }

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
            return this;
        }
    }    // End RendererColumnsAdapter

    /**
     * Adapts any renderer and makes it work on only specified column
     */
    public static class RendererColumnAdapter extends DefaultTableCellRenderer {

        private final int col;
        private final TableCellRenderer renderer;

        public RendererColumnAdapter(TableCellRenderer rend, int col2appyrendererto) {
            this.renderer = rend;
            this.col = col2appyrendererto;
        }

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int col) {

            if (this.col == col) {
                return renderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
            }

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
            return this;
        }
    }    // End RendererColumnAdapter

    /**
     * For rendering the lists/combo boxes that have nodes properly
     * <p/>
     * In adddition to working on presentable objects also works on
     * persistenobjects by looking up their node delegate - this mechanism uses the
     * sco system along with additional caching so fairly perf efficient.
     */
    public static class CommonLookAndDoubleClickListRenderer extends CommonLookListRenderer {

        /**
         * Class Constructor.
         * This form of the constructor enables a popup menu for the jlist
         * apart from the usual rendering stuff.
         */
        public CommonLookAndDoubleClickListRenderer(final JList renderedlist) {

            // notice adding listener to the list and not to the renderer
            renderedlist.addMouseListener(new MyPopupMouseListener());

            // IMP to NOT place this piece of code in the popupmenu checker - that causes
            // the widget to launch twice
            renderedlist.addMouseListener(new MouseAdapter() {

                public void mouseClicked(MouseEvent e) {

                    if (e.getClickCount() == 2) {

                        //log.debug("Doing double click");
                        Object obj = renderedlist.getSelectedValue();

                        if (obj == null) {
                            return;
                        }

                        Application.getWindowManager().runDefaultAction(obj);
                    }
                }
            });
        }

    }


    /**
     * For rendering the lists/combo boxes containing known Files and
     * objects.
     * <p/>
     * No events
     *
     * @see CommonLookAndDoubleClickListRenderer
     */
    public static class CommonLookListRenderer extends DefaultListCellRenderer {
        private boolean ifFileOnlyShowName;

        public CommonLookListRenderer(final boolean ifFileOnlyShowName) {
            this.ifFileOnlyShowName = ifFileOnlyShowName;
        }

        public CommonLookListRenderer() {
            this(false); // default is to show the full path
        }

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            /*
            klog.debug(">>>>>>>>>>>>>>>>>value " + value);
            if (value != null) {
                klog.debug(" class: " + value.getClass());
            }
            */

            // order is important
            if (value instanceof PersistentObject) {
                PersistentObject pob = (PersistentObject) value;

                if (pob.getQuickInfo() != null) {
                    StringBuffer buf = new StringBuffer("<html><body>").append(pob.getName());
                    buf.append("<font color=#666666> [").append(pob.getQuickInfo()).append(']').append("</font></html></body>");
                    this.setText(buf.toString());
                } else {
                    this.setText(pob.getName());
                }

                File f = null;

                if (ParserFactory.getCache().isCached(pob)) {
                    f = ParserFactory.getCache().getSourceFile(pob);
                }

                if (f != null) {
                    this.setToolTipText(f.getAbsolutePath());
                } else {
                    this.setToolTipText("Unknown origins of file: " + f);
                }
            } else if (value instanceof File) {
                if (ifFileOnlyShowName) {
                    this.setText(((File) value).getName());
                } else {
                    this.setText(((File) value).getAbsolutePath());
                }
                this.setIcon(DataFormat.getIcon(value));
                this.setToolTipText(((File) value).getAbsolutePath());
            } else if (value instanceof XChart) {
                this.setText(((XChart) value).getName());
                this.setIcon(XChart.ICON);
            } else if (value instanceof FTPFile) {

                String s = ((FTPFile) value).getPath();
                String slc = s.toLowerCase();
                if (slc.indexOf("c1.") != -1) {
                    s = s + " [Positional]";
                } else if (slc.indexOf("c2.") != -1) {
                    s = s + " [Curated]";
                } else if (slc.indexOf("c3.") != -1) {
                    s = s + " [Motif]";
                } else if (slc.indexOf("c4.") != -1) {
                    s = s + " [Computational]";
                }

                this.setText(s);
                this.setIcon(FTP_FILE_ICON);
            }

            return this;
        }
    }    // End CommonLookListRenderer

    /**
     * dont understand it but this thorws a class cast exception when used
     * something to do with DataFormat @see DataExportTable for an alt impl.
     */
    public static class DataFormatListRenderer extends DefaultListCellRenderer {

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            if (value != null) {
                klog.debug(value + " class: " + value.getClass());
                //if (value.getClass().equals(DataFormat.class.getName())) {
                if (value instanceof edu.mit.broad.genome.parsers.DataFormat) {
                    DataFormat df = (DataFormat) value;
                    this.setText(df.getName());
                    this.setIcon(df.getIcon());
                    this.setToolTipText(df.getDesc());
                }
            } else {
                klog.debug("Unexpected Null value: " + value);
            }

            return this;

        }
    }    // End DataFormatListRenderer

    /**
     * renderer to display FileBookmarks fo a jlist / combobox
     * The name is in bold and the file is in blue normal
     */
    public static class FileBookmarkListRenderer extends DefaultListCellRenderer {

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // this takes care of the background / border painting for focus
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            if (value instanceof FileBookmark) {
                FileBookmark fb = (FileBookmark) value;
                this.setText("<html><b>" + fb.getName() + "</b>&nbsp(<font color=blue>" + fb.getLocation().getPath() + "</font>)</html>");
            } else if (value instanceof JSeparator) {
                return (JSeparator) value;
            }

            this.setToolTipText(this.getText());
            return this;
        }
    }    // End FileBookmarkListRenderer

    /**
     * Popup displayer
     */
    static class MyPopupMouseListener extends GPopupChecker {

        /**
         * Let the node do the actual handling of popup
         * events.
         * Here, just fetch the menu from the DataObject
         * and display in the appropriate location
         */
        protected void maybeShowPopup(MouseEvent e) {

            if (e.isPopupTrigger()) {
                Object ot = e.getSource();

                //log.debug("Source is: " + ot);

                if (ot instanceof JTree) {
                    Object obj = ((JTree) ot).getSelectionPath().getLastPathComponent();

                    //log.debug("Trying to show popup for: " + obj);
                    JPopupMenu popup = Application.getWindowManager().createPopupMenu(obj);

                    popup.show(e.getComponent(), e.getX(), e.getY());
                } else if (ot instanceof JList) {
                    Object obj = ((JList) ot).getSelectedValue();

                    //log.debug("Trying to show popup for: " + obj);

                    if (obj == null) {
                        return;
                    }

                    JPopupMenu popup = Application.getWindowManager().createPopupMenu(obj);

                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        }
    }    // End of inner class MyPopupMouseListener


    /**
     * Does nothing other than display a tool tip for the item
     * Useful when the item name is long but the combobox/lis is small
     * and ehence the item is only partially visible.
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class ToolTipRenderer extends DefaultListCellRenderer {

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            this.setToolTipText(this.getText());
            return this;
        }
    }    // End ListCellRenderer


    public static class HyperLinkRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int col) {

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
            this.setText(GHyperLinkLabel.formatForHyperLink(value.toString()));
            return this;
        }
    }    // End HyperLinkRenderer

}        // End RendererFactory2

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
