/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;


import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.core.Window;
import edu.mit.broad.xbench.core.api.Application;
import foxtrot.Job;
import foxtrot.Worker;

import javax.swing.*;
import javax.swing.event.InternalFrameListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;


/**
 * Base action class for actions that result in opening up a widget in a new window.
 * <p/>
 * Implementing classes should prefer to extends ObjectAction or FileAction or FilesAction
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public abstract class WidgetAction extends XAction implements MouseListener {

    /**
     * Indicates whether or not to respond only to double clicks. Default false
     */
    protected boolean fOnlyDc;

    /**
     * null by default
     */
    protected Dimension fExplicitSize;
    protected edu.mit.broad.xbench.core.Window fWindow;

    /**
     * Implementing subclasses must specify
     *
     * @return
     */
    public abstract Widget getWidget() throws Exception;

    /**
     * IMP dont call in action constructor
     * call just before you create the widget!!
     *
     * @param w
     * @param h
     */
    protected void setSize(int w, int h, boolean usedefaultifdefaultisbigger) {
        // IMP dont place this in the class init are -- cause the pp to recursive loop

        if (usedefaultifdefaultisbigger) {
            Dimension d = Application.getWindowManager().getExpectedWindowSize();
            if (d != null) {
                if (d.width > w) {
                    w = d.width;
                }

                if (d.height > h) {
                    h = d.height;
                }
            }
        }

        this.fExplicitSize = new Dimension(w, h);

    }

    /**
     * Use with care.
     * For use when widgets are instantiated only once
     *
     * @see PrefAction for an example
     *      Default is false i.e widgets get their own window every time.
     */
    protected Window getWindow() {
        return fWindow;
    }

    /**
     * as modified amber to run under foxtrot
     *
     * @param evt
     */

    //sometimes FOXTROT detects null pointer exceptions when this is used??
    // but seem to have got over it mostly -- so use
    public void actionPerformed(final ActionEvent evt) {

        if (fOnlyDc) {
            return;
        }

        //log.debug("actionPerformed: " + evt);
        Worker.post(new Job() {
            public Object run() {
                Runnable r = createTask(evt);
                Thread t = new Thread(r);
                // @note else the action makes the vdbgui hang
                t.setPriority(Thread.MIN_PRIORITY);
                t.start();
                //log.debug("started thread");
                return null;
            }
        });

    }

    /*
     * WITHOUT FOXTROT - orig amber style impl.
     *  DONT DELETE YET
     * public void actionPerformed(ActionEvent evt) {
     *   //log.debug("actionPerformed: " + evt);
     *
     *   Runnable r = createTask(evt);
     *   Thread t = new Thread(r);
     *   t.start();
     *   //log.debug("started thread");
     * }
     */
    public Runnable createTask(final ActionEvent evt) {

        return new Runnable() {

            public void run() {

                try {
                    waitCursor(getDescription());

                    //log.debug("Waiting for action");
                    Widget widget = getWidget();

                    if (widget != null) {
                        if (fExplicitSize != null) {
                            fWindow = Application.getWindowManager().openWindow(widget, fExplicitSize);
                        } else {
                            fWindow = Application.getWindowManager().openWindow(widget);
                        }

                        //window.setIcon(true); // causes the window to launch and then instantly become an icon

                        if (fWindow instanceof JInternalFrame) {
                            ((JInternalFrame) fWindow).setJMenuBar(widget.getJMenuBar());
                            InternalFrameListener ifl = getInternalFrameListener();
                            if (ifl != null) {
                                ((JInternalFrame) fWindow).addInternalFrameListener(ifl);
                            }
                        }

                    } else {
                        log.debug("null widget - no window opended");
                    }

                } catch (Throwable t) {
                    Application.getWindowManager().showError("Trouble making widget", t);
                } finally {
                    defaultCursor();
                }
            }
        };
    }

    /**
     * Subclasses can override to install a real listener
     * Null here
     *
     * @return
     */
    protected InternalFrameListener getInternalFrameListener() {
        return null;
    }

    /**
     * Make this action only respond to double cliks
     * Obvioulsy not valid if used in a popup / menu but useful
     * for buttons / jlists
     *
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     */
    public void setDoubleClickOnly(boolean onlydc) {
        this.fOnlyDc = onlydc;
    }

    public void mouseClicked(MouseEvent e) {

        if (fOnlyDc == false) {
            e.consume();

            return;
        }

        if (e.getClickCount() == 2) {

            //log.debug("Doing double click");
            fOnlyDc = false;

            actionPerformed(new ActionEvent(e.getSource(), e.getID(), this.getId(),
                    e.getModifiers()));

            fOnlyDc = true;
        }
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }
}    // End WidgetAction

/*--- Formatted in Sun Java Convention Style on Fri, Nov 15, '02 ---*/
