/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.core;


import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.Metrics;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.GLabelledItemPanel;
import edu.mit.broad.genome.swing.choosers.GFileFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GColorFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GDirFieldPlusChooser;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.xbench.RendererFactory2;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.xbench.xchoosers.TemplateComboBoxModel;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.io.File;


/**
 * Factory of binders and xchoosers for objects.
 * <p/>
 * Binder -> something that attaches objects to a control such as a JComboBox
 * Chooser -> a UI that allows interactive choice of some object(s)
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ObjectBindery {

    private static final Logger klog = XLogger.getLogger(ObjectBindery.class);

    /**
     * Privatized class constructor
     * Only static methods.
     */
    private ObjectBindery() {
    }

    public static JComboBox createMetricsBox() {
        return new JComboBox(Metrics.createAllMetrics());
    }

    public static JComboBox createSortModeBox() {
        return new JComboBox(SortMode.ALL);
    }

    public static JComboBox createOrderBox() {
        return new JComboBox(Order.ALL);
    }

    public static void bind(JComboBox cb, Class[] classes, boolean addNos) {

        if (cb == null) {
            throw new IllegalArgumentException("param cb cannot be null");
        }

        ComboBoxModel model = ParserFactory.getCache().createBoxModel(classes, addNos);

        cb.setModel(model);
        cb.setRenderer(new RendererFactory2.CommonLookListRenderer());

    }

    /**
     * Bind objects in the object cache to the specified JComboBox.
     *
     * @param cb
     * @param cl
     */
    public static void bind(final JComboBox cb, final Class cl) {

        if (cb == null) {
            throw new IllegalArgumentException("param cb cannot be null");
        }

        if (cl == null) {
            throw new IllegalArgumentException("Parameter cl cannot be null");
        }

        //log.debug("Looking to bind: " + cl);

        ComboBoxModel model;

        if (cl == edu.mit.broad.vdb.chip.Chip.class) {
            model = new DefaultComboBoxModel(VdbRuntimeResources.getChips_All_safe());
        } else {
            model = ParserFactory.getCache().createBoxModel(cl);
        }

        cb.setModel(model);
        cb.setRenderer(new RendererFactory2.CommonLookListRenderer());
    }

    public static void bindTemplate(final JComboBox cb, final TemplateMode mode) {

        if (cb == null) {
            throw new IllegalArgumentException("param cb cannot be null");
        }

        ComboBoxModel model = ParserFactory.getCache().createBoxModel(Template.class);
        TemplateComboBoxModel tmodel = new TemplateComboBoxModel(model, mode);
        cb.setModel(tmodel);
        cb.setRenderer(new RendererFactory2.CommonLookListRenderer());
    }

    /**
     * Bind objects in the object cache to the specified JList.
     *
     * @param jl
     * @param cl
     */
    public static void bind(JList jl, Class cl) {

        if (jl == null) {
            throw new IllegalArgumentException("param jl cannot be null");
        }

        ComboBoxModel model;

        if (cl == edu.mit.broad.vdb.chip.Chip.class) {
            model = new DefaultComboBoxModel(VdbRuntimeResources.getChips_All_safe());
        } else {
            model = ParserFactory.getCache().createBoxModel(cl);
        }

        jl.setModel(model);

        if (jl.getModel().getSize() > 0) {
            jl.setSelectedIndex(0);
        }

        jl.setCellRenderer(new RendererFactory2.CommonLookAndDoubleClickListRenderer(jl));
    }

    public static void bind(JList jl, Class[] classes) {

        if (jl == null) {
            throw new IllegalArgumentException("param jl cannot be null");
        }

        ComboBoxModel model;

        model = ParserFactory.getCache().createBoxModel(classes, false);

        jl.setModel(model);

        if (jl.getModel().getSize() > 0) {
            jl.setSelectedIndex(0);
        }

        jl.setCellRenderer(new RendererFactory2.CommonLookAndDoubleClickListRenderer(jl));
    }

    public static ComboBoxModel getModel(Class[] classes) {
        return ParserFactory.getCache().createBoxModel(classes, false);
    }

    public static ComboBoxModel getModel(Class c) {
        return ParserFactory.getCache().createBoxModel(c);
    }

    public static ComboBoxModel getHackAuxGeneSetsBoxModel() {
        return ParserFactory.getCache().hackCreateAuxGeneSetsBoxModel();
    }

    /**
     * @param allowmultiple If true allows > 1 Dataset to be selected
     * @return
     */
    public static Dataset[] showDatasetChooser(boolean allowmultiple) {

        Object[] sels = _showChooser(allowmultiple, Dataset.class, "Select Dataset");

        if (sels != null) {
            Dataset[] dss = new Dataset[sels.length];

            for (int i = 0; i < sels.length; i++) {
                dss[i] = (Dataset) sels[i];
            }

            return dss;
        } else {
            return null;
        }
    }

    /**
     * Convenience methods for allowing selection of 1 Dataset
     *
     * @return
     * @see showDatasetChooser(boolean allowmultiple)
     */
    public static Dataset showDatasetChooser() {

        Dataset[] dss = showDatasetChooser(false);

        if ((dss != null) && (dss.length > 0)) {
            return dss[0];
        } else {
            return null;
        }
    }

    public static HClustDatasetStanford[] showHClustDatasetChooser(boolean allowmultiple) {

        Object[] sels = _showChooser(allowmultiple, HClustDatasetStanford.class, "Select HClust Dataset (cdt)");

        if (sels != null) {
            HClustDatasetStanford[] cdses = new HClustDatasetStanford[sels.length];

            for (int i = 0; i < sels.length; i++) {
                cdses[i] = (HClustDatasetStanford) sels[i];
            }

            return cdses;
        } else {
            return null;
        }
    }

    public static HClustDatasetStanford showHClustDatasetChooser() {

        HClustDatasetStanford[] cdses = showHClustDatasetChooser(false);

        if ((cdses != null) && (cdses.length > 0)) {
            return cdses[0];
        } else {
            return null;
        }
    }

    public static EnrichmentDb[] showEdbChooser(boolean allowmultiple) {

        Object[] sels = _showChooser(allowmultiple, EnrichmentDb.class, "Select Enrichment Database");

        if (sels != null) {
            EnrichmentDb[] edbs = new EnrichmentDb[sels.length];

            for (int i = 0; i < sels.length; i++) {
                edbs[i] = (EnrichmentDb) sels[i];
            }

            return edbs;
        } else {
            return null;
        }
    }


    /**
     * Convenience methods for allowing selection of 1 Dataset
     *
     * @return
     * @see showDatasetChooser(boolean allowmultiple)
     */

    public static EnrichmentDb showEdbChooser() {

        EnrichmentDb[] edbs = showEdbChooser(false);

        if ((edbs != null) && (edbs.length > 0)) {
            return edbs[0];
        } else {
            return null;
        }
    }

    /**
     * @param allowmultiple If true allows > 1 Template to be selected
     * @return
     */
    public static Template[] showTemplateChooser(boolean allowmultiple) {

        Object[] sels = _showChooser(allowmultiple, Template.class, "Select Template");

        if (sels != null) {
            Template[] tss = new Template[sels.length];

            for (int i = 0; i < sels.length; i++) {
                tss[i] = (Template) sels[i];
            }

            return tss;
        } else {
            return null;
        }
    }

    /**
     * Convenience methods for allowing selection of 1 Template
     *
     * @return
     * @see showTemplateChooser(boolean allowmultiple)
     */
    public static Template showTemplateChooser() {

        Template[] tss = showTemplateChooser(false);

        if ((tss != null) && (tss.length > 0)) {
            return tss[0];
        } else {
            return null;
        }
    }

    /**
     * @param allowmultiple If true allows > 1 DatasetAnnotation to be selected
     * @return
     */
    public static FeatureAnnot[] showAnnotationChooser(boolean allowmultiple) {

        Object[] sels = _showChooser(allowmultiple, FeatureAnnot.class, "Select Annotation");

        if (sels != null) {
            FeatureAnnot[] anns = new FeatureAnnot[sels.length];

            for (int i = 0; i < sels.length; i++) {
                anns[i] = (FeatureAnnot) sels[i];
            }

            return anns;
        } else {
            return null;
        }
    }

    /**
     * Convenience methods for allowing selection of 1 DatasetAnnotation
     *
     * @return
     * @see showDatasetAnnotationChooser(boolean allowmultiple)
     */
    public static FeatureAnnot showAnnotationChooser() {

        FeatureAnnot[] ass = showAnnotationChooser(false);

        if ((ass != null) && (ass.length > 0)) {
            return ass[0];
        } else {
            return null;
        }
    }

    /**
     * @param allowmultiple If true allows > 1 Dataset to be selected
     * @return
     */
    public static GeneSetMatrix[] showGeneSetMatrixChooser(boolean allowmultiple) {

        Object[] sels = _showChooser(allowmultiple, GeneSetMatrix.class, "Select GeneSetMatrix");

        if (sels != null) {
            GeneSetMatrix[] gms = new GeneSetMatrix[sels.length];

            for (int i = 0; i < sels.length; i++) {
                gms[i] = (GeneSetMatrix) sels[i];
            }

            return gms;
        } else {
            return null;
        }
    }

    /**
     * Convenience methods for allowing selection of 1 GeneSetMatrix
     *
     * @return
     */
    public static GeneSetMatrix showGeneSetMatrixChooser() {

        GeneSetMatrix[] gmxs = showGeneSetMatrixChooser(false);

        if ((gmxs != null) && (gmxs.length > 0)) {
            return gmxs[0];
        } else {
            return null;
        }
    }

    // common code to build pob shoosers
    private static Object[] _showChooser(boolean allowmultiple, final Class pobclass, String ddtitle) {

        final JList jl = new JList();

        // dont use object binder -> that associates its own double clicks etc
        //ObjectBinder.bind(jl, Dataset.class);
        jl.setModel(ParserFactory.getCache().createBoxModel(pobclass));
        jl.setSelectedIndex(0);
        jl.setCellRenderer(new RendererFactory2.CommonLookListRenderer());

        if (!allowmultiple) {
            jl.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        } else {
            jl.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        }

        ApplicationDialog desc = new ApplicationDialog(ddtitle, new JScrollPane(jl));
        desc.enableDoubleClickableJList(jl);

        int res = desc.show();

        if (res == ApplicationDialog.CANCEL_OPTION) {
            return null;
        } else {
            return jl.getSelectedValues();
        }
    }

    /**
     * Allows building (and then selecting) an adhoc dataset.
     * Typically something small or an interesting subset of
     * a real Dataset.
     *
     * @param ds
     * @param ds
     * @param ds
     * @return Presents a UI that allows for selecting a location to save
     *         the specified Dataset.
     *         <p/>
     *         Presents a UI that allows for selecting a location to save
     *         the specified Dataset.
     *         <p/>
     *         Presents a UI that allows for selecting a location to save
     *         the specified Dataset.
     */

    /**
     * Presents a UI that allows for selecting a location to save
     * the specified Dataset.
     *
     * @param ds
     */
    public static void showDatasetSaver(Dataset ds) {
        showObjectSaver(ds);
    }

    /**
     * Presents a UI that allows for selecting a location to save
     * the specified object. UI is rooted off specified parent comp.
     *
     * @param pob
     * @param comp
     */
    public static void showObjectSaver(PersistentObject pob) {
        // @todo
        if (1 == 1) {
            throw new RuntimeException("to do");
        }

        GFileFieldPlusChooser chooser = new GFileFieldPlusChooser();
        ApplicationDialog dd = new ApplicationDialog("Save Object As", chooser);
        int res = dd.show();

        if (res == ApplicationDialog.OK_OPTION) {
            try {
                File file = chooser.getFile();

                // fix the extension if needed
                // sometimes want to save a pob thats not loaded yet
                // cant determine its class. Hence the pob null check
                // But most of the time this check does occur as pobs to-save
                // are generally already loaded!

                /*
                 * if ( (file != null) && (pob != null) ) {
                 *   file = ParserFactory.ensureCorrectExt(file, pob);
                 * }
                 */

                // determine file validity
                if (file == null) {
                    // do nothing
                } else if (file.exists()) {
                    boolean resov = Application.getWindowManager().showConfirm("File exists -- overwrite?");
                    if (resov) {
                        ParserFactory.save(pob, file);
                    }
                } else {
                    ParserFactory.save(pob, file);
                }
            } catch (Throwable t) {
                Application.getWindowManager().showError("Could not save Dataset", t);
            }
        }
    }


    public static void showFSetSaver(final FSet fset, boolean isNameEditable) {

        GLabelledItemPanel panel = new GLabelledItemPanel();

        final JTextField tf = new JTextField(fset.getName());
        tf.setEditable(isNameEditable);
        panel.addItem("GeneSet Name", tf);

        final GDirFieldPlusChooser dir = new GDirFieldPlusChooser();
        panel.addItem("In Folder", dir);

        GColorFieldPlusChooser color = new GColorFieldPlusChooser(Color.BLACK);
        panel.addItem("Color(optional)", color);
        ApplicationDialog ad = new ApplicationDialog("Save GeneSet ...", panel);

        FSet tsFSet;
        int res = ad.show();
        if (res == ApplicationDialog.OK_OPTION) {
            tsFSet = fset;
            File file = new File(dir.getFile(), fset.getName());
            try {
                ParserFactory.save(tsFSet, file);
            } catch (Throwable t) {
                Application.getWindowManager().showError("Could not save GeneSet", t);
            }
        }
    }

    /**
     * Auto saves specified pob into a file in the runtime result dir
     * Does not barf on errors -- just prints to stdout.
     * Runs the auto save in a thread, so client code should not make any assumptions on whether
     * data has been saved yet or not.
     * Never throws an exception, just prints a log message on failure.
     */
    public static void autoSaveObject(final PersistentObject pob) {

        try {
            Runnable r = _autoSave(pob);
            Thread thread = new Thread(r);

            thread.start();
        } catch (Throwable t) {
            klog.fatal("Trouble autosaving pob", t);
        }
    }

    /**
     * Auto saves pob into a file in the user's result dir
     *
     * @param ds
     * @see see ap handler for where the result dir is.
     *      Useful when you dont want to bother the user with having to say "please save this for me".
     */

    // does the saving in a thread as might be time consuming
    private static Runnable _autoSave(final PersistentObject pob) {

        return new Runnable() {

            public void run() {

                try {
                    String name = NamingConventions.generateName(pob);
                    File file = new File(Application.getVdbManager().getTmpDir(), name);

                    ParserFactory.save(pob, file);
                } catch (Throwable t) {
                    klog.error("Trouble saving pob whilst in thread", t);
                } finally {
                    //
                }
            }
        };
    }


}    // End ObjectBindery


