/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.dataio;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.xbench.RendererFactory2;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 *
 *
 */
// intentionally package visibility

class DataImportTable extends JTable {

    /**
     * For logging support
     */
    private static final Logger klog = XLogger.getLogger(Model.class);
    private Model fModel;

    //@maint order of columns tied to ImportDataSource
    public static final String[] COL_HEADERS = {"Include", "Object Name", "Import Status",
            "Location"};

    /**
     * Contains ExportDataStructure objects
     */
    private List fImports;

    /**
     * Class constructor
     */
    protected DataImportTable() {
        super();
        init(new File[]{});
    }

    /**
     * Class Constructor.
     */
    public DataImportTable(File[] files) {

        super();

        init(files);
    }

    private void init(File[] files) {

        fImports = new ArrayList();
        for (int i = 0; i < files.length; i++) {
            fImports.add(new ImportDataStructure(files[i], ImportStage.WAITING));
        }

        this.fModel = new Model();

        this.setModel(fModel);

        Renderer rend = new Renderer();

        this.setDefaultRenderer(ImportDataStructure.class, rend);
        this.setDefaultRenderer(String.class, rend);
        this.setDefaultRenderer(ImportStage.class, rend);

        this.getTableHeader().setReorderingAllowed(false);
        this.getTableHeader().setDefaultRenderer(new RendererFactory2.BoldHeaderRenderer());

        //this.setDefaultRenderer(Boolean.class, rend); // the default jtable does this better
        this.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        GuiHelper.Table.sizeCells2FitByScanning(this);
        this.revalidate();

    }

    public void setFiles(File[] files) {
        init(files);
    }

    public void addFiles(File[] files) {
        //log.debug("Added file: " + files);

        for (int i = 0; i < files.length; i++) {
            fImports.add(new ImportDataStructure(files[i], ImportStage.WAITING));
        }

        fModel.fireTableDataChanged();
    }

    public File[] getFiles() {
        File[] files = new File[fImports.size()];
        for (int i = 0; i < fImports.size(); i++) {
            files[i] = ((ImportDataStructure) fImports.get(i)).file;
        }
        return files;
    }

    /**
     * Sets the specified status for all rows
     *
     * @param stage
     */
    public void setAllStatus(ImportStage stage) {

        for (int i = 0; i < fImports.size(); i++) {
            ImportDataStructure imp = (ImportDataStructure) fImports.get(i);

            imp.set(2, stage);
        }

        this.repaint();
    }

    /**
     * Set the status of the ExportDataStructure at specified index (0 to n-1)
     */
    public void setStatus(ImportStage stage, int rowindex) {

        ImportDataStructure imp = (ImportDataStructure) fImports.get(rowindex);

        imp.set(2, stage);

        //this.revalidate();
        this.repaint();
    }

    /**
     * @return Whether or not the importdatafile at row is de/selected (by user)
     *         or not.
     */
    public boolean isSelectedForImport(int row) {

        ImportDataStructure imp = (ImportDataStructure) fImports.get(row);

        return imp.isSelected();
    }

    /**
     * Inner class: an implementation of AbstractTableModel for Templates. <br>
     * <p/>
     * Requirements:
     * <p/>
     * 4 columns
     * <p/>
     * checkbox import or not (all are checked by default)
     * file name (not editable)
     * object name(editable, default parsed from file name is specified)
     * parse status (red if failed, green if successful)
     * thats actually doen in rendering.
     */
    class Model extends AbstractTableModel {

        /**
         * AbstractTableModel impl.
         *
         * @return The number of rows (in this context - the number of Features)
         */
        public int getRowCount() {
            return fImports.size();
        }

        /**
         * AbstractTableModel impl.
         */
        public int getColumnCount() {
            return COL_HEADERS.length;
        }

        /**
         * AbstractTableModel impl.
         */
        public Object getValueAt(int row, int col) {

            ImportDataStructure imp = (ImportDataStructure) fImports.get(row);

            return imp.get(col);
        }

        /**
         * @return Column header for column c (0 to n-1)
         */
        public String getColumnName(int c) {
            return COL_HEADERS[c];
        }

        public void setValueAt(Object value, int row, int col) {

            if (isCellEditable(row, col)) {
                ImportDataStructure imp = (ImportDataStructure) fImports.get(row);

                imp.set(col, value);
                fireTableCellUpdated(row, col);
            } else {
                klog.warn("Cell not editable, row: " + row + " col: " + col);
            }
        }

        public Class getColumnClass(int col) {
            return getValueAt(0, col).getClass();
        }

        public boolean isCellEditable(int row, int col) {

            ImportDataStructure imp = (ImportDataStructure) fImports.get(row);

            return imp.isEditable(col);
        }
    }    // End Model

    /**
     * Data structure to encapsulate a single importable data source
     * and auxillary info for display in the import table.
     * One such object for every data source to-be-imported.
     * <p/>
     * package visibilty intentional.
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     * @tdone consider moving this class into DataImportTable
     */

    //package visibilty intentional.
    class ImportDataStructure {

        private Boolean doimport;
        private final File file;
        private String objectname;
        private ImportStage stage;

        // @maint must be manually kept in synch with the fColHeader size and order
        protected ImportDataStructure(File afile, ImportStage is) {

            doimport = Boolean.TRUE;
            this.file = afile;

            //objectname = FileUtils.removeExtension(source.getPath());
            objectname = file.getName();
            stage = is;
        }

        protected Object get(int coln) {

            if (coln == 0) {
                return doimport;
            } else if (coln == 1) {
                return objectname;
            } else if (coln == 2) {
                return stage;
            } else if (coln == 3) {
                return file.getPath();
            } else {
                throw new IllegalArgumentException("Unknown coln: " + coln);
            }
        }

        protected void set(int col, Object obj) {

            if (col == 0) {
                doimport = (Boolean) obj;
            } else if (col == 1) {
                objectname = obj.toString();
            } else if (col == 2) {
                stage = (ImportStage) obj;
            } else {
                throw new IllegalArgumentException("Illegal set coln: " + col);
            }
        }

        protected boolean isEditable(int col) {

            if ((col == 0) || (col == 1) || (col == 2)) {
                return true;
            } else {
                return false;
            }
        }

        protected boolean isSelected() {
            return doimport.booleanValue();
        }

        protected JComponent getComponent(int col) {

            // again, this is very closely tied to the order of columns.
            if (col == 0) {
                return new JCheckBox("", doimport.booleanValue());
            } else if (col == 1) {
                return new JLabel(get(1).toString());
            } else if (col == 2) {
                JLabel label = new JLabel(stage.name);
                label.setForeground(stage.color);
                return label;
            } else if (col == 3) {
                return new JLabel(get(col).toString());
            } else {

                //log.error("Unexpected column index col=" + col);
                return new JLabel("program error: col=" + col);
            }
        }
    }    // End ExportDataStructure

    /**
     * Class Renderer
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private class Renderer extends DefaultTableCellRenderer {

        /**
         * Rendering goals:
         * <ul>
         * <li> import/parse status -> if error mark in red, else in green, if waiting, in black.
         * <li> display the first column (to include in import or not) as a checkbox
         * <li> object name is ediatbel
         * <li>
         * </ul>
         */
        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int col) {

            //super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
            ImportDataStructure imp = (ImportDataStructure) fImports.get(row);


            Component comp = imp.getComponent(col);

            /* dosnt seem to work
            if (isSelected) { // dont mess with th bg
                System.out.println("###### " + isSelected);
                comp.setBackground(table.getSelectionBackground());
            } else {
                comp.setBackground(table.getBackground());
            }

            comp.setBackground(Color.BLUE);
            comp.setFont(table.getFont());
            */

            return comp;
        }
    }    // End Renderer
}        // End DataImportTable

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
