/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.dataio;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.ProgressMonitorInputStream2;
import edu.mit.broad.genome.swing.dnd.DndTarget;
import edu.mit.broad.genome.swing.dnd.DropTargetDecorator;
import edu.mit.broad.xbench.core.Widget;
import foxtrot.Job;
import foxtrot.Worker;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;


/**
 * User interface for representing files that are files in.
 * Dnd functionality for from explorer into parser widget.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @veto consider if when parsing a single file (which might actually
 * happen most of the time) a simple GUI should be presented?
 * <br>veto->Nope uniform ui is better.
 */
public class ParserWidget extends JPanel implements Widget, DndTarget {

    private static final Logger klog = XLogger.getLogger(ParserWidget.class);
    public static final Icon ICON = JarResources.getIcon("Parser16.gif");
    public static final String TITLE = "Import Data";
    private final DataImportTable fTable = new DataImportTable();
    private final JScrollPane tableScrollPane = new JScrollPane(fTable);
    private final JButton runButton = GuiHelper.Button.createStartButton("Start Import");
    private ParserWidget fInstance = this;

    private JProgressBar fSharedBar;

    /**
     * Class constructor
     */
    public ParserWidget() {
        this(new File[]{});
    }

    /**
     * Class Constructor.
     */
    public ParserWidget(File[] files) {
        new DropTargetDecorator(this);
        //log.debug("Opening on files: " + files + " number: " + files.length);
        init(files);
    }

    private void init(File[] files) {

        fTable.setFiles(files);
        jbInit();

        runButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Worker.post(new Job() {
                    public Object run() {
                        Runnable r = createImportTask();
                        Thread t = new Thread(r);
                        // @note else the action makes the vdbgui hang
                        t.setPriority(Thread.MIN_PRIORITY);
                        t.start();
                        return null;
                    }
                });
            }
        });
    }

    private Runnable createImportTask() {

        return new Runnable() {

            public void run() {

                //log.debug("doing files: " + files);
                // imp to have the try/catch withing the loop so that
                // one error doesnt stop all
                File[] files = fTable.getFiles();
                klog.info("# files to parse: " + files.length);

                for (int i = 0; i < files.length; i++) {
                    //TraceUtils.showTrace();
                    if (fTable.isSelectedForImport(i) == false) {
                        fTable.setStatus(ImportStage.WAITING, i);
                        continue;
                    }
                    try {
                        fTable.setStatus(ImportStage.PARSING, i);
                        FileInputStream in = new FileInputStream(files[i]);
                        ProgressMonitorInputStream2 pis = new ProgressMonitorInputStream2(in, fSharedBar);
                        ParserFactory.read(files[i].getPath(), pis, true);    // just gets placed in memory, no other thing
                        fTable.setStatus(ImportStage.SUCCESS, i);
                    } catch (Throwable t) {
                        klog.error("Parsing trouble", t);
                        fTable.setStatus(ImportStage.ERROR, i);
                    }
                }
            }
        };
    }


    // builds the UI for the multi data source import
    private void jbInit() {

        TitledBorder titledBorder2 = new TitledBorder(BorderFactory.createEtchedBorder(),
                "Import Data (drag files from the file explorer and drop them into this window): ");

        tableScrollPane.setBorder(titledBorder2);

        // IMP to get the table size properly
        tableScrollPane.setPreferredSize(new Dimension(GuiHelper.DIMENSION_STANDARD_WINDOW.width,
                200));
        tableScrollPane.revalidate();
        fTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
        this.setLayout(new BorderLayout());
        this.add(tableScrollPane, BorderLayout.CENTER);

        fSharedBar = new JProgressBar();
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(fSharedBar, BorderLayout.CENTER);
        panel.add(runButton, BorderLayout.EAST);
        this.add(panel, BorderLayout.SOUTH);

    }

    public String getTitle() {
        return TITLE;
    }

    public JMenuBar getJMenuBar() {
        return EMPTY_MENU_BAR;
    }

    public boolean allowClose() {
        return true;
    }

    public JComponent getWindowUI() {
        return this;
    }

    public Icon getIcon() {
        return ICON;
    }

    /**
     * DndTarget impl
     *
     * @return
     */
    public Component getDroppableIntoComponent() {
        return this;
    }

    /**
     * DndTarget impl.
     *
     * @param obj
     */
    public void setDropData(Object obj) {

        //log.debug("Drop data: " + obj + " class: " + obj.getClass());
        if (obj == null) {
            return;
        } else {
            List list = (List) obj;
            List files2add = new ArrayList();

            for (int i = 0; i < list.size(); i++) {
                if (list.get(i) instanceof File) {
                    files2add.add(list.get(i));
                }
            }

            fTable.addFiles((File[]) files2add.toArray(new File[files2add.size()]));
            fTable.revalidate();
            fTable.repaint();
            this.revalidate();
        }
    }

    /**
     * DndTarget impl.
     *
     * @return
     */

    // accepts one or more files dnd into it
    public DataFlavor[] getDroppableFlavors() {
        return new DataFlavor[]{DataFlavor.javaFileListFlavor};
    }
}    // End ParserWidget

/*--- Formatted in Sun Java Convention Style on Sat, Oct 19, '02 ---*/
