/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.explorer.filemgr;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.io.FileTransferable;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.GPopupChecker;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.SimpleInternalPanel;
import edu.mit.broad.genome.swing.dnd.DndSource;
import edu.mit.broad.genome.swing.dnd.DragSourceDecorator;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.xbench.actions.misc_actions.CopyFileNamesAction;
import edu.mit.broad.xbench.actions.misc_actions.CopyFilesAction;
import edu.mit.broad.xbench.actions.misc_actions.FilesSelectable;
import edu.mit.broad.xbench.core.api.Application;
import foxtrot.Job;
import foxtrot.Worker;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

/**
 * Class JRecentFilesList
 * With a list of files opened in the "recent" past.
 * <p/>
 * DND
 * This is a drag source -> Files can be dragged out into appropriate receivers
 * This is NOT a drag receiver
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class JRecentFilesList extends JList implements DndSource, FilesSelectable {

    private final Logger log = XLogger.getLogger(JRecentFilesList.class);
    private JRecentFilesList fInstance;
    public static final String DEFAULT_TITLE = "Recent files";
    public static final Icon ICON = JarResources.getIcon("History24.gif");
    private static final Font kFont = new Font("Arial", Font.PLAIN, 12);
    private CopyFilePathAction fCopyAction;

    private JButton bPurge;

    /**
     * @param title
     * @return
     */
    public static JComponent createComponent(final String title) {
        JRecentFilesList recentFiles = new JRecentFilesList();
        JScrollPane sp = new JScrollPane(recentFiles);
        SimpleInternalPanel sif = new SimpleInternalPanel(title, recentFiles.bPurge);
        sif.add(sp);
        return sif;
    }

    /**
     * Class constructor
     * Creates a explorer like jlist with popups et al
     */
    public JRecentFilesList() {

        fInstance = this;

        this.setModel(Application.getFileManager().getRecentFilesStore());

        this.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        this.bPurge = new JButton("Purge");
        bPurge.addActionListener(new ActionListener() {
            public void actionPerformed(final ActionEvent evt) {
                boolean proceed = Application.getWindowManager().showConfirm("Delete file history?", "<html><body><b>" +
                        "This will remove recently used files from this list (but NOT delete the files themselves)</b>" +
                        "</body></html>"
                );

                if (proceed) {
                    Application.getFileManager().getRecentFilesStore().clearAll();
                    fInstance.refresh();
                }
            }
        });

        this.fCopyAction = new CopyFilePathAction();
        this.setCellRenderer(new ListCellRenderer());
        this.setBackground(GuiHelper.COLOR_VERY_LIGHT_GRAY);

        // enable Dnd
        new DragSourceDecorator(this);

        CopyFilesAction cfa = new CopyFilesAction(this);
        this.addKeyListener(cfa.createCtrlCKeyListener());

        CopyFileNamesAction cfna = new CopyFileNamesAction(this);
        this.addKeyListener(cfna.createCtrlZKeyListener());

    }

    /**
     * FilesSelectable impl
     *
     * @return
     */
    public FileTransferable getSelectedFiles() {
        return (FileTransferable) getTransferable();

    }

    public String getSelectedFileNames() {
        return ((FileTransferable) getTransferable()).getFileNames();
    }

    /**
     * FilesSelectable impl
     *
     * @return
     */
    public void refresh() {
    }

    /**
     * FilesSelectable impl
     *
     * @return
     */
    public Component getDisplayComponent() {
        return null;
    }

    public Transferable getTransferable() {

        Object[] objs = fInstance.getSelectedValues();

        if ((objs == null) || (objs.length == 0)) {
            return new FileTransferable(new File[]{});
        }

        File[] files = new File[objs.length];

        for (int i = 0; i < objs.length; i++) {
            files[i] = new File(objs[i].toString());
        }

        return new FileTransferable(files);
    }

    public Component getDraggableComponent() {
        return this;
    }

    /**
     * Class ListCellRenderer
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    class ListCellRenderer extends DefaultListCellRenderer {

        public ListCellRenderer() {

            //log.debug(">>>> " + fInstance);

            // IMP to NOT place this piece of code in the popupmenu checker - that causes
            // the widget to launch twice
            fInstance.addMouseListener(new MouseAdapter() {

                public void mouseClicked(MouseEvent e) {
                    if (e.getClickCount() == 2) {

                        if (fInstance.getSelectedValue() == null) {
                            return;
                        }

                        File file = new File(fInstance.getSelectedValue().toString());
                        log.info("Loading ... " + file.getPath());
                        Application.getWindowManager().runDefaultAction(file);

                        /*
                        if (file.exists()) {
                            boolean success = Application.getWindowManager().runDefaultAction(file);
                            if (success) {
                                Application.getWindowManager().showMessage("Successfully loaded: " + file.getName());
                            }
                        }
                        */
                    }
                }
            });

            // notice adding listener to the list and not to the renderer
            fInstance.addMouseListener(new MyPopupMouseListener());
        }

        /**
         * showing the full path makes it kinda clutterred
         * instead go ../parent_dir/file_name
         * having curr dir displayed makes uniqueness visible.
         */
        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            //log.debug(value + " class: " + value.getClass());
            if (value instanceof String) {

                File file = new File(value.toString());
                this.setText(FileUtils.shortenedPathRepresentation(file));
                this.setIcon(DataFormat.getIcon(file));
                this.setToolTipText(value.toString());
                this.setFont(kFont);

                if (!file.exists()) {
                    this.setForeground(Color.red);
                }
            }

            return this;
        }
    }    // End ListCellRenderer

    /**
     * Popup displayer
     */
    class MyPopupMouseListener extends GPopupChecker {

        protected void maybeShowPopup(MouseEvent e) {

            if (e.isPopupTrigger()) {

                //log.debug("Launching popup trigger");
                Object[] sel = fInstance.getSelectedValues();

                if (sel == null) {
                    return;
                }

                JPopupMenu popup = null;

                log.debug(">> sel = " + sel.length);
                if (sel.length > 1) {
                    popup = new JPopupMenu();
                    File[] files = new File[sel.length];
                    for (int i = 0; i < sel.length; i++) {
                        //log.debug(sel[i].getClass());
                        files[i] = new File(sel[i].toString());
                    }

                    // old style that brings up a widget window
                    //popup.add(new ParserAction(files));
                    // new way simply loads
                    popup.add(new ImportFilesAction(files));

                } else if (sel.length == 1) {
                    popup = Application.getWindowManager().createPopupMenu(new File(sel[0].toString()));
                }

                if (popup != null) {
                    popup.add(new JSeparator());
                    popup.add(fCopyAction);
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        }
    }    // End of inner class MyPopupMouseListener

    /**
     * Class CopyFilePathAction
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    class CopyFilePathAction extends AbstractAction {

        CopyFilePathAction() {

            this.putValue(Action.NAME, "Copy File(s)");
            this.putValue(Action.SMALL_ICON, JarResources.getIcon("Copy16.gif"));
            this.putValue(Action.SHORT_DESCRIPTION, "Copy File(s)");
        }

        public void actionPerformed(ActionEvent evt) {
            Transferable t = getTransferable();
            Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
            clip.setContents(t, (ClipboardOwner) t);    // we know that FileTransferable is whats returned and that it is also a cpowner
        }
    }

    static class ImportFilesAction extends AbstractAction {

        File[] files;

        ImportFilesAction(File[] files) {
            this.files = files;
            this.putValue(Action.NAME, "Import data from selected " + files.length + " files");
            this.putValue(Action.SHORT_DESCRIPTION, "Import data from selected files");
        }

        public void actionPerformed(ActionEvent evt) {
            Worker.post(new Job() {
                public Object run() {
                    Runnable r = ParserFactory.createImportTask(files);
                    Thread t = new Thread(r);
                    t.setPriority(Thread.MIN_PRIORITY);
                    t.start();
                    return null;
                }
            });
        }
    }
}    // End JRecentFilesList

/*--- Formatted in Sun Java Convention Style on Mon, Oct 21, '02 ---*/

/**
 * Class constructor
 * For use with file chooser dialogs
 *
 * @param fcd

public JRecentFilesList(final XFileChooser fcd) {

if (fcd == null) {
throw new IllegalArgumentException("Null FileChooserDialog param not allowed");
}

fInstance = this;

init();

}
 */
/*  com out jan 2006 refactoring Application
fcd.addActionListener(new ActionListener() {

    public void actionPerformed(ActionEvent e) {

        //log.debug("Action!!!!!!!!");
        Worker.post(new Job() {

            public Object run() {

                File[] files = fcd.getSelectedFiles();

                if (files == null) {
                    return null;
                }

                for (int i = 0; i < files.length; i++) {
                    // hash files arent real
                    if (!AuxUtils.isAuxFile(files[i])) {
                        //kRecent.add(files[i].getAbsolutePath()); // @todo
                    }
                }

                //kRecent.save(); // @todo

                return null;
            }
        });
    }
});

this.addListSelectionListener(new ListSelectionListener() {

    public void valueChanged(ListSelectionEvent e) {

        //log.debug(e + " " + fInstance.getSelectedValue());
        Object obj = fInstance.getSelectedValue();

        if (obj == null) {
            return;
        }

        File file = new File(obj.toString());

        //log.debug(file + " exists: " + file.exists());
        //fcd.reset();
        if (file.isDirectory()) {
            fcd.setCurrentLocation(file.getAbsolutePath());
        } else {
            fcd.setCurrentLocation(file.getParent());
        }

        fcd.setCurrentLocation(file.getAbsolutePath());

        // setting doesnt work
        //fcd.setSelectedFile(file);
        //log.debug(fcd.getSelectedFile());
    }
});
*/