/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.prefs;


import edu.mit.broad.genome.Conf;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooserTable;
import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.core.api.Application;
import foxtrot.Task;
import foxtrot.Worker;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Map;
import java.util.WeakHashMap;

/**
 * Widget that builds the local file system view along with a few
 * easy access buttons
 * <p/>
 * Keep it simple:
 * <p/>
 * Two tabs -> 1 for FILE/DIR params
 * -> rest in second tab
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class PreferencesWidget extends JPanel implements Widget {

    public static final String TITLE = "Preferences";
    public static final Icon ICON = JarResources.getIcon("Preferences16.gif");
    private final Logger log = XLogger.getLogger(PreferencesWidget.class);
    private PreferencesWidget fInstance;
    private Map fCategoryPrefTableMap;

    /**
     * Class Constructor.
     */
    public PreferencesWidget() {

        super();

        fCategoryPrefTableMap = new WeakHashMap();
        init();
    }

    private void init() {

        fInstance = this;

        jbInit();
    }

    public JButton createResetButton() {

        JButton bReset = new JButton("Reset", GuiHelper.ICON_RESTORE16);
        bReset.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    Worker.post(new Task() {
                        public Object run() throws Exception {
                            resetAll();
                            return null;
                        }
                    });
                } catch (Throwable t) {
                    Application.getWindowManager().showError("Unable to reset preferences", t);
                }
            }
        });

        return bReset;
    }

    // make the table lazily so that initial display is faster

    private JButton[] fCatButtons;

    private JLabel labelRestart;

    private void jbInit() {

        this.setLayout(new BorderLayout());
        this.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));

        JToolBar butPanel = new JToolBar(JToolBar.VERTICAL);
        //JPanel butPanel = new JPanel(new VerticalFlowLayout());
        butPanel.setFloatable(false);
        //butPanel.setBackground(Color.LIGHT_GRAY); // makes the selected tab's font invisible (as font def is gray!)
        final PreferenceCategory[] cats = XPreferencesFactory.ALL_CATEGORIES;
        boolean debug = Conf.isDebugMode();

        int cnt = 0;
        for (int i = 0; i < cats.length; i++) {
            if (!debug && cats[i].isDebugCategory()) {
                continue;
            } else {
                cnt++;
            }
        }

        fCatButtons = new JButton[cnt];
        cnt = 0;
        for (int i = 0; i < cats.length; i++) {
            if (!debug && cats[i].isDebugCategory()) {
                continue;
            }

            fCatButtons[cnt] = createCategoryButton(cats[i]);
            butPanel.add(fCatButtons[cnt++]);
        }

        //butPanel.setPreferredSize(new Dimension( (int)fCatButtons[0].getPreferredSize().getWidth(), 1000));
        JScrollPane sp = new JScrollPane(butPanel);
        //sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        this.add(sp, BorderLayout.WEST);

        labelRestart = new JLabel();
        this.add(labelRestart, BorderLayout.SOUTH);

        // borders make the UI loook better
        this.add(Box.createVerticalStrut(5), BorderLayout.NORTH);
        this.add(Box.createHorizontalStrut(5), BorderLayout.EAST);

    }

    private JButton createCategoryButton(final PreferenceCategory cat) {

        final JButton but = new JButton(cat.getName(), cat.getIcon());
        but.setToolTipText(cat.getDesc());

        but.setVerticalTextPosition(JButton.BOTTOM);
        but.setVerticalAlignment(JButton.BOTTOM);
        but.setHorizontalTextPosition(SwingConstants.CENTER);
        but.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                changePref(cat);
                for (int i = 0; i < fCatButtons.length; i++) {
                    if (fCatButtons[i] != but) {
                        fCatButtons[i].setEnabled(true);
                        //fCatButtons[i].setBackground(Color.LIGHT_GRAY); // doesnt work
                    }
                }
                but.setEnabled(false);
            }
        });

        // duplicated code from GuiHelper so that we can NOT rollover if but is disabled

        final Border bevel = new BevelBorder(BevelBorder.RAISED, GuiHelper.COLOR_DARK_BLUE, GuiHelper.COLOR_DARK_BLUE);
        but.setBorder(bevel);
        but.setRolloverEnabled(false); // needed for alloy lnf to work properly
        but.setBorderPainted(false);
        but.addMouseListener(new MouseAdapter() {
            public void mouseEntered(MouseEvent e) {
                if (but.isEnabled()) {
                    but.setBorderPainted(true);
                }

                if (but.isSelected()) {
                    but.setBackground(Color.GREEN);
                }

            }

            public void mouseExited(MouseEvent e) {
                but.setBorderPainted(false);
            }
        });

        return but;
    }


    Component curr = null;

    private void changePref(PreferenceCategory cat) {

        if (curr != null) {
            this.remove(curr);
        }

        PrefUI pt = createPrefTable(cat);
        curr = pt.table;
        this.add(curr, BorderLayout.CENTER);
        this.revalidate();
        this.repaint();
    }

    private PrefUI createPrefTable(PreferenceCategory cat) {

        Object obj = fCategoryPrefTableMap.get(cat);

        if (obj == null) {

            final PrefUI pt = new PrefUI(cat.getName());
            Preference[] aprefs = cat.getPreferences();
            for (int p = 0; p < aprefs.length; p++) {
                Preference pref = aprefs[p];
                if ((Conf.isDebugMode() == false) && (pref.isDebugPreference())) {
                    continue;
                }
                if (pref.needsRestart()) {
                    pt.labels.add("<html><body><b>" + pref.getName() + "</b><font color=#FF6600>[Effective on restart]</font></body></html>");
                } else {
                    pt.labels.add(pref.getName());
                }
                pt.jfcs.add(pref.getSelectionComponent());
            }

            pt.table = new GFieldPlusChooserTable((String[]) pt.labels.toArray(new String[pt.labels.size()]),
                    (GFieldPlusChooser[]) pt.jfcs.toArray(new GFieldPlusChooser[pt.jfcs.size()]));

            obj = pt;
            fCategoryPrefTableMap.put(cat, obj);
        }

        return (PrefUI) obj;

    }

    private void resetAll() {

        PreferenceCategory[] cats = XPreferencesFactory.ALL_CATEGORIES;
        for (int i = 0; i < cats.length; i++) {
            Preference[] prefs = cats[i].getPreferences();
            for (int p = 0; p < prefs.length; p++) {
                Object val = prefs[p].getSelectionComponent().getValue();
                if (val != null) {
                    prefs[p].reset();
                }
            }
        }
        fInstance.revalidate();
    }

    public void saveAll() throws Exception {
        PreferenceCategory[] cats = XPreferencesFactory.ALL_CATEGORIES;
        for (int i = 0; i < cats.length; i++) {
            Preference[] prefs = cats[i].getPreferences();
            for (int p = 0; p < prefs.length; p++) {
                Object val = prefs[p].getSelectionComponent().getValue();
                if (val != null) {
                    prefs[p].setValue(val);
                }
            }
        }
        Preference.kPrefs.flush();
    }

    public JComponent getWindowUI() {
        return this;
    }

    public String getTitle() {
        return TITLE;
    }

    public boolean allowClose() {
        return true;
    }

    public JMenuBar getJMenuBar() {
        return Widget.EMPTY_MENU_BAR;
    }

    public Icon getIcon() {
        return ICON;
    }

    class PrefUI {
        String name;
        java.util.List labels;
        java.util.List jfcs;
        GFieldPlusChooserTable table;

        PrefUI(String name) {
            this.name = name;
            this.labels = new ArrayList();
            this.jfcs = new ArrayList();
        }
    }

}    // End PreferenceWidget
