/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.tui;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.reports.api.Report;
import edu.mit.broad.genome.swing.GLabelledItemPanel;
import edu.mit.broad.genome.swing.GStartPauseButton;
import edu.mit.broad.genome.swing.GThreadPriorityBox;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.xbench.actions.XDCAction;
import edu.mit.broad.xbench.core.ApplicationDialog;
import edu.mit.broad.xbench.core.JObjectsList;
import edu.mit.broad.xbench.core.XSounds;
import edu.mit.broad.xbench.core.api.Application;
import gnu.trove.TIntObjectHashMap;
import org.apache.log4j.Logger;
import xtools.api.Tool;
import xtools.api.param.ParamSet;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Constructor;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * Singleton
 * Runs tools on diff threads and keep their status & records their params / ouytput
 * Available as a table to use in a widget
 * <p/>
 * Placed the createTable method in here as easier to handle updates etc.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class TaskManager {

    private static final Logger klog = XLogger.getLogger(TaskManager.class);

    /**
     * Column headers for table model
     */
    private static final String[] COL_HEADERS = new String[]{" ", "Name", "Status"};

    /**
     * @maint keep in synch with col names above
     */
    private static final int COL_NUM = 0;
    private static final int COL_NAME = 1;
    private static final int COL_STATUS = 2;
    private final Model fModel;

    private JTable fTaskTable;

    private boolean fOnClickShowResultsInBrowserOnly; // false by default

    /**
     * The singleton instance
     */
    private static TaskManager kInstance;

    /**
     * Holds ToolRunnable objects
     */
    private final List fToolRunnables;

    /**
     * @return Get a ref to the singleton
     */
    public static TaskManager getInstance() {

        if (kInstance == null) {
            synchronized (TaskManager.class) {
                if (kInstance == null) {
                    kInstance = new TaskManager();
                }
            }
        }

        return kInstance;
    }

    /**
     * Privatized Class Constructor.
     * Use getInstance to get a ref to the singleton
     */
    private TaskManager() {

        fToolRunnables = new ArrayList();

        // must be made now - cant be done lazily
        fModel = new Model();
    }

    public void setOnClickShowResultsInBrowserOnly(boolean value) {
        this.fOnClickShowResultsInBrowserOnly = value;
    }

    /**
     * Runs tool on a thread.
     * specified Tool is just a "template" -> reflection invoked another tool
     * and that is filled with specified paramset
     * Will throw exception if the Tool barfs.
     * Adds tool to the ones it manages
     *
     * @param tool
     * @throws Exception
     */
    public Tool run(Tool tool, ParamSet pset, int priority) throws Exception {

        if (tool == null) {
            throw new IllegalArgumentException("Param tool cannot be null");
        }

        if (pset == null) {
            throw new IllegalArgumentException("Param pset cannot be null");
        }

        // errors here are propogated right away
        // -- tool NOT added to table
        // no task created
        Tool clonedTool;

        try {
            clonedTool = createTool(tool, pset);
        } catch (Exception t) {
            ToolRunnable pstate = ToolRunnable.createParamErrorToolState(tool, pset, t);    // @note adding tool skeleton directly

            fToolRunnables.add(pstate);
            kInstance.updateTable();

            throw t;
        }

        ToolRunnable trunnable = new ToolRunnable(clonedTool);

        fToolRunnables.add(trunnable);
        kInstance.updateTable();

        Thread t = new Thread(trunnable);
        t.setPriority(priority);
        trunnable.owner_thread = t;

        t.start();
        klog.debug("Started executing Tool: " + clonedTool.getName() + " priority: " + priority);
        return clonedTool;
    }

    // unbelievably wierd problems with Tool  - class is NULL  -throws npe
    // format: test(xtools.gsea.Gsea)
    // maybe coz class discoverer is used??
    // Creates a NEW Tool -> spec one used as skeleton only
    // New Tool made is NOT run
    // Exceptions are all reflection related + if any params are not set

    public static Tool createTool(final Tool tool, final ParamSet pset) throws Exception {

        String toolName = tool.getName();
        Class toolClass = Class.forName(toolName);
        klog.debug("toolClass: " + toolClass + " pset: " + pset);
        Class[] initArgsClass = new Class[]{Properties.class};    // reqd to have a ParamSet constructor
        Constructor initArgsConstructor = toolClass.getConstructor(initArgsClass);
        klog.debug(initArgsConstructor);
        Properties prp = pset.toProperties();
        prp.remove("help"); // @note imp else produces usage!!
        Properties[] initArgs = new Properties[]{prp};
        System.out.println(">> " + prp);
        return (Tool) initArgsConstructor.newInstance((Object[])initArgs);
    }

    public static Tool createTool(final String toolName) throws Exception {
        Class toolClass = Class.forName(toolName);
        //klog.debug("ToolName: " + toolName + " class: " + toolClass);
        Class[] initArgsClass = new Class[]{};
        Constructor initArgsConstructor = toolClass.getConstructor(initArgsClass);
        return (Tool) initArgsConstructor.newInstance(new Object[]{});
    }

    private void updateTable() {
        // it does a a jig, but thats ok as visual indicator of a change in state
        fModel.fireTableStructureChanged(); // needed for consistent updates
        fTaskTable.repaint();
        fTaskTable.revalidate();
        setColNumWidth(fTaskTable);
    }

    /**
     * Inner class
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private class Model extends AbstractTableModel {

        private TIntObjectHashMap fRowToolButtonMap;
        private TIntObjectHashMap fRowReportButtonMap;

        private Model() {
            fRowToolButtonMap = new TIntObjectHashMap();
            fRowReportButtonMap = new TIntObjectHashMap();
        }

        public int getRowCount() {
            return fToolRunnables.size();
        }

        public int getColumnCount() {
            return COL_HEADERS.length;
        }

        /*
        * JTable uses this method to determine the default renderer/
        * editor for each cell.  If we didn't implement this method,
        * then the components dont show up.
        */
        public Class getColumnClass(int col) {

            // NO!
            //return getValueAt(0, col).getClass();
            // synched with the Editor
            return Object.class;
        }

        public String getColumnName(int col) {
            return COL_HEADERS[col];
        }

        /**
         * Everything is editable ->> but not the data -- needed for the
         * buttons etc to work
         *
         * @return always true
         */
        public boolean isCellEditable(int row, int col) {
            return col != COL_NUM;
        }

        /**
         * dont bother with setValueAt() ->> let the renderer here take care of things
         * <p/>
         * NO! Have to do it here as the Editor gets data from getValueAt
         * Let the renderer handle ->> always return the pstate
         *
         * @return object value at specified row and column.
         * <p/>
         * NO! Have to do it here as the Editor gets data from getValueAt
         * Let the renderer handle ->> always return the pstate
         * @return object value at specified row and column.
         */

        /**
         * NO! Have to do it here as the Editor gets data from getValueAt
         * Let the renderer handle ->> always return the pstate
         *
         * @return object value at specified row and column.
         */

        private final Icon LHS_TOOL_ICON = JarResources.getIcon("dirty_ov.gif");

        public Object getValueAt(int row, int col) {

            ToolRunnable trunnable = (ToolRunnable) fToolRunnables.get(row);

            if (col == COL_NUM) {
                return new Integer(row + 1);
            } else if (col == COL_NAME) {

                JButton but;
                if (fRowToolButtonMap.get(row) == null) {
                    //klog.debug("Making new button for row: " + row);
                    but = new JButton();
                    but.setBorderPainted(false);
                    but.setFocusPainted(false);
                    fRowToolButtonMap.put(row, but);
                } else {
                    but = (JButton) fRowToolButtonMap.get(row);
                }

                // @todo clone tool
                Tool tool = trunnable.tool;
                SingleToolLauncherAction action = new SingleToolLauncherAction(tool, trunnable.pset, null);
                but.setAction(action);
                but.setIcon(LHS_TOOL_ICON);
                // decided against double clicking -- seemed to be more consistent
                //action.setDoubleClickOnly(true); // for consistency
                //but.addMouseListener(action); // needed for the doubleclicking

                return but;

            } else if (col == COL_STATUS) {

                JButton but;
                if (fRowReportButtonMap.get(row) == null) {
                    //klog.debug("Making new button for row: " + row);
                    but = new JButton();
                    but.setBorderPainted(false);
                    but.setFocusPainted(false);
                    fRowReportButtonMap.put(row, but);
                } else {
                    but = (JButton) fRowReportButtonMap.get(row);
                }

                final ToolRunnableStateAction action = new ToolRunnableStateAction(trunnable, fOnClickShowResultsInBrowserOnly);
                but.setAction(action);

                // decided against dclicking
                //action.setDoubleClickOnly(true); // for consistency
                // but.addMouseListener(action); // needed for the doubleclicking

                if (trunnable.state == ExecState.SUCCESS) {
                    Report report = trunnable.tool.getReport();
                    if (report == null) {
                        but.setText("No Report Produced");
                    } else {
                        int len = report.getNumPagesMade();
                        if (len == 0) {
                            but.setText("0 Result Objects");
                        } else {
                            //but.setIcon(JarResources.getIcon("ToolRunnerResults.png"));
                            but.setForeground(Color.GREEN);
                            //but.setText("Success");
                            but.setText("<html><body><font color=green>Success " + len + "</font></body></html>");
                            but.setIcon(GuiHelper.ICON_ELLIPSIS);
                            but.setVerticalTextPosition(JButton.TOP);
                            but.setToolTipText(but.getText());
                            //but.setFont(GuiHelper.FONT_DEFAULT);
                        }
                    }
                } else {
                    but.setText(trunnable.state.name);
                }

                but.setHorizontalAlignment(SwingConstants.CENTER);
                but.setForeground(trunnable.state.color);
                //but.setBackground(Color.white);
                but.setBorderPainted(false);

                return but;

            } else {
                return "Bad col: " + col;
            }
        }
    }    // End Model

    /**
     * A custom renderer for table cells that works in conjunction with the table
     * and the Model to display the components corerectly
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private class Renderer extends DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int column) {

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
            //klog.debug("rendering value: " + value);
            if (value instanceof Component) {
                return (Component) value;
            } else {
                return this;
            }
        }
    }    // End inner class GramTableCellRenderer

    /**
     * Object that represents a Tool and its ExecState
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    static class ToolRunnable implements Runnable {

        private Tool tool;

        // may NOT be same as Tool pset!! esp when creation failed
        // see createParamErrorToolState
        private ParamSet pset;
        private ExecState state;
        private Throwable throwable;

        private Thread owner_thread;

        private ThreadController thread_controller;

        private boolean wasKilled;

        /**
         * creates a new ToolState
         */
        private ToolRunnable(Tool p) {

            if (p == null) {
                throw new IllegalArgumentException("Param p cannot be null");
            }

            this.tool = p;
            this.pset = p.getParamSet();    // same one -- instantiated correctly
            this.state = ExecState.WAITING;
            this.throwable = null;
        }

        private static ToolRunnable createParamErrorToolState(Tool p, ParamSet pset, Throwable throwable) {

            if (pset == null) {
                throw new IllegalArgumentException("Param pset cannto be null");
            }

            ToolRunnable ps = new ToolRunnable(p);

            ps.state = ExecState.PARAM_ERROR;
            ps.throwable = throwable;
            ps.pset = pset;    // change it to specified one

            return ps;
        }

        public void run() {

            try {
                this.state = ExecState.RUNNING;

                kInstance.updateTable();
                this.tool.execute();

                if (wasKilled) {
                    this.state = ExecState.KILLED;
                    XSounds.playToolKilled();
                } else {
                    this.state = ExecState.SUCCESS;
                    XSounds.playToolSuccess();
                }

            } catch (Throwable t) {
                this.state = ExecState.EXEC_ERROR;
                this.throwable = t;
                klog.error("Tool exec error", t);
                if (tool != null && tool.getReport() != null) {
                    tool.getReport().setErroredOut();
                }

            } finally {
                kInstance.updateTable();
            }
        }
    }

    /**
     * IMP: the taskmanager doesnt handle updates to its datamodel well --> "repaint"
     * doesnt get fired and found it easier to just place the table making stuff in here
     * so that we have access and can handle the repaints.
     *
     * @return A JTable holding data represented by the task managers model
     */
    public JTable createTable() {
        this.fTaskTable = new JTable(fModel);

        // table visual properties
        fTaskTable.setRowSelectionAllowed(false);
        fTaskTable.setColumnSelectionAllowed(false);
        fTaskTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        //table.setRowHeight(30);
        fTaskTable.setShowVerticalLines(true);
        fTaskTable.setGridColor(Color.black);
        fTaskTable.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        fTaskTable.getTableHeader().setReorderingAllowed(false);
        //table.getTableHeader().setDefaultRenderer(new RendererFactory2. .BoldHeaderRenderer()); // dont want it too prominent
        fTaskTable.setDefaultRenderer(Object.class, new Renderer());

        //fTaskTable.setRowHeight(40);

        // the components stay un-responsive unless this is set
        //table.setDefaultEditor(Component.class, new Editor());
        TableCellEditor ed = new Editor();

        fTaskTable.setDefaultEditor(Object.class, ed);

        // we know what the columns are so set their size explicitly
        setColNumWidth(fTaskTable);

        return fTaskTable;
    }

    private static final Icon RESULTS_ICON = JarResources.getIcon("Results.gif");

    // needs to be called after every fire structure changed
    // else its ok at first, and then after first fire, it changes back to default width
    private void setColNumWidth(JTable table) {
        TableColumn column = table.getColumnModel().getColumn(COL_NUM);

        column.setMinWidth(0);
        column.setMaxWidth(20);
        column.setPreferredWidth(20);
    }

    /**
     * Display a window depending on what status the tools state is
     * Error, Param error -> display of the stack trace
     */
    private static class ToolRunnableStateAction extends XDCAction {

        private final ToolRunnable trunnable;
        private final ToolRunnableStateAction fTrsaInstance = this;
        private boolean fDoBrowser;

        private ToolRunnableStateAction(final ToolRunnable trunnable, final boolean doBrowser) {

            this.trunnable = trunnable;
            this.fDoBrowser = doBrowser;

            this.putValue(Action.NAME, getName());

            // dont add icon
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }


        public void actionPerformed(final ActionEvent evt) {
            if (fOnlyDoubleClick) { // respond only to double clicks
                return;
            }
            klog.debug("running ToolRunnableStateAction");

            // doesnt work with foxtrot -window doesnt show up -- not sure why???
            //try {
            // Worker.post(new Task() {
            //public Object run() throws Exception {

            if (trunnable.state == ExecState.WAITING) {
                Application.getWindowManager().showMessage("Waiting for: " + trunnable.tool.getName());
            } else if (trunnable.state == ExecState.PARAM_ERROR) {
                kInstance.updateTable();
                Application.getWindowManager().showError("One or more parameter(s) were not specified",
                        trunnable.throwable);

            } else if (trunnable.state == ExecState.EXEC_ERROR) {
                kInstance.updateTable();
                Application.getWindowManager().showError("Tool execution error", trunnable.throwable);
            } else if ((trunnable.state == ExecState.RUNNING) || (trunnable.state == ExecState.PAUSED)) {
                kInstance.updateTable();
                if (trunnable.thread_controller == null) {
                    trunnable.thread_controller = new ThreadController(trunnable);
                }
                trunnable.thread_controller.showControl(trunnable);

            } else if ((trunnable.state == ExecState.SUCCESS) || (trunnable.state == ExecState.KILLED)) {
                kInstance.updateTable();
                Report report = trunnable.tool.getReport();
                if (report == null) {
                    fTrsaInstance.setEnabled(false);// not clickable
                    Application.getWindowManager().showMessage("No report data produced");
                } else {
                    if (fDoBrowser) {
                        try {
                            URL url = report.getReportIndex().toURL();
                            if (url != null) {
                                Desktop.getDesktop().browse(url.toURI());
                            } else {
                                Application.getWindowManager().showMessage("No report produced");
                            }
                        } catch (Throwable t) {
                            Application.getWindowManager().showError(t);
                        }
                    } else {
                        JObjectsList jol = new JObjectsList(report.getFilesProduced());
                        JObjectsList.displayInWindow("Results for: " + trunnable.tool.getName(), RESULTS_ICON, jol);
                    }

                    //ToolResultsAction action = new ToolResultsAction(trunnable);
                    //action.setDoubleClickOnly(true); dont !! this action already does that check
                    //action.actionPerformed(evt);
                }
            } else {
                kInstance.updateTable();
                Application.getWindowManager().showMessage("No actions defined for this state " + trunnable.state + " " + trunnable.tool.getName());
            }

            //kInstance.updateTable();

            /*
        return null;
         }
          });
         } catch(Exception x) {
           log.error("Foxtrot error", x);
        }
        */
        }

        public String getId() {
            return "ToolStateAction";
        }

        public String getName() {
            return "Tool State";
        }

        public Icon getIcon() {
            return ToolLauncher.ICON;
        }

        public String getDescription() {
            return "Details on the Tools State";
        }
    }    // End ToolStateAction

    /**
     * Opened in a modal dialog
     */
    private static class ThreadController {
        private static final String RESUME = "Resume";
        private static final String PAUSE = "Pause";
        private ToolRunnable trunnable;
        private GStartPauseButton bPauseResume;

        ThreadController(ToolRunnable tr) {
            this.trunnable = tr;
        }

        private boolean isProcessComplete() {
            return (!trunnable.owner_thread.isAlive());
        }

        private void showAlreadyCompleteMessage() {
            Application.getWindowManager().showMessage("Process already complete");
        }

        private JPanel createPanel() {

            final GLabelledItemPanel panel = new GLabelledItemPanel();

            final GThreadPriorityBox cbPriority = new GThreadPriorityBox(true, true, false);

            if (trunnable.owner_thread.isAlive()) {
                cbPriority.setSelectedPriority(trunnable.owner_thread.getPriority());
            }

            cbPriority.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    // dont use foxtrot for faster response.
                    // This is typcally used when a tool is runing and we click with the intent of doing something right-away
                    // so prefer as little delay as possible.
                    //Worker.post(new Job() {

                    //public Object run() {
                    if (isProcessComplete()) {
                        showAlreadyCompleteMessage();
                    } else {
                        trunnable.owner_thread.setPriority(cbPriority.getPriority());
                    }
                    // return null;
                    //}

                    //});

                }
            });

            panel.addItem("Change CPU Usage", cbPriority);

            if (trunnable.state == ExecState.PAUSED) {
                bPauseResume = new GStartPauseButton(GStartPauseButton.RESUME_MODE, false);
                panel.addItem(RESUME, bPauseResume);
            } else {
                bPauseResume = new GStartPauseButton(GStartPauseButton.PAUSE_MODE, false);
                panel.addItem(PAUSE, bPauseResume);
            }

            // dont use foxtrot - we want this to happen immeadiatly after click and not
            // wait for other things.
            // ok this method of controlling threads aint great. Threads should nto be paused/killed in this manner.
            // but:
            // 1) Tools dont really make any locks
            // 2) it works ok and is simple to use
            // so, go with this for now, and maybe change later
            // @todo improve thread control mechanism
            bPauseResume.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (isProcessComplete()) {
                        showAlreadyCompleteMessage();
                    } else {
                        if (bPauseResume.isInPauseMode()) {
                            try {
                                bPauseResume.setMode(GStartPauseButton.RESUME_MODE);
                                panel.setLabelText(1, RESUME);
                                trunnable.owner_thread.suspend();
                                trunnable.state = ExecState.PAUSED;
                            } catch (Throwable t) {
                                t.printStackTrace();
                            }
                        } else {
                            bPauseResume.setMode(GStartPauseButton.PAUSE_MODE);
                            panel.setLabelText(1, PAUSE);
                            trunnable.state = ExecState.RUNNING;
                            trunnable.owner_thread.resume();
                        }

                        //klog.debug("updating table");
                        kInstance.updateTable();

                    }
                }
            });

            /*@note not implemented in the tools usually
            JButton bKill = new JButton(JarResources.getIcon("Stop.gif"));
            bKill.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (isProcessComplete()) {
                        showAlreadyCompleteMessage();
                    } else {
                        boolean proceed = Application.getWindowManager().showConfirm("Confirm tool murder", "<html><body><b>This will kill the Tool! " +
                                "Proceed?</b><br><font color=gray>Note: the process might run for several secs more before ending<br>" +
                                "Reports generated upto this point, if any, will be displayed</font></body></html>"
                        );
                        if (proceed) {
                            klog.debug("killing thread");
                            trunnable.owner_thread.resume(); // in case we were paused
                            trunnable.owner_thread.interrupt();
                            trunnable.tool.requestKill();
                            trunnable.wasKilled = true;
                            kInstance.updateTable();
                        }
                    }
                }
            });

            panel.addItem("Kill", bKill);
            */

            return panel;
        }

        private void showControl(final ToolRunnable trunnable) {

            ApplicationDialog dd = new ApplicationDialog("Process Control for: " + trunnable.tool.getName(),
                    createPanel());

            //dd.setPre
            dd.setOnlyShowCloseOption();
            dd.show();
        }

    } // End internal class ThreadController

    /**
     *
     */
    private class Editor extends AbstractCellEditor implements TableCellEditor {

        private Object currVal;

        /**
         * TableCellEditor impl.
         * The core method to implement to acheive desired effect.
         */
        public Component getTableCellEditorComponent(JTable table, Object value,
                                                     boolean isSelected, int row, int column) {

            //klog.debug("Getting editor: " + value);
            currVal = value;

            if (value instanceof Component) {
                return (Component) value;
            } else {

                currVal = new JLabel(value.toString());
                return (JLabel) currVal;
            }
        }

        public Object getCellEditorValue() {
            //klog.debug("Curr value: " + currVal);
            return currVal;
        }
    }    // End inner class Editor
}        // End TaskManager
