/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.clustering.hierarchical;

import javax.swing.*;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Lisa Simirenko
 * @author jgould
 */
public class ArrayTreePanel extends JPanel implements Scrollable, Tree {

    protected String[] arrayIds;

    protected int bottomGutter = 4;

    protected int elementWidth;

    protected EventListenerList listenerList = new EventListenerList();

    protected int maxIndex;

    protected Map nodes;

    protected String rootId;

    protected Color selectedColor = Color.magenta;

    protected Node selectedNode = null;

    /**
     * for Aravind to get tree to line up
     */
    protected boolean subtractElementWidth;

    protected int topGutter = 4;

    protected double ymax;

    protected double ymin;

    protected double yPixPerUnit;

    // transform from world to pixel coodinates
    protected double[] pixelMatrix = new double[6];

    /**
     * Creates a new instance of ArrayTreePanel
     *
     * @param parser The parser
     */
    public ArrayTreePanel(AtrGtrReader parser) {
        super();
        init(parser.getNodes(), parser.getRootNodeId(), parser.getIds());
        ymin = 1;
        ymax = parser.getMinArrayCorr();
    }

    /**
     * Creates a new instance of ArrayTreePanel
     *
     * @param nodes  id to Node map
     * @param rootId the id of the root node
     * @param ids    ordered ids of the leaf nodes
     */
    public ArrayTreePanel(Map nodes, String rootId, String[] ids) {
        super();
        init(nodes, rootId, ids);
    }

    public void addListSelectionListener(ListSelectionListener l) {
        listenerList.add(ListSelectionListener.class, l);
    }

    public void deselectSelectedNode() {
        if (selectedNode != null) {
            selectedNode.setSelected(false);
        }
        repaint();
    }

    /**
     * this method draws nodes recursively by calling drawSingleNode(), and
     * draw() for both left and right if not leaves
     */
    public void draw(Graphics2D g2, Node node) {
        Node left = node.getLeft();
        Node right = node.getRight();

        drawSingleNode(g2, node);

        if (left.isLeaf() == false) {
            draw(g2, left);
        }
        if (right.isLeaf() == false) {
            draw(g2, right);
        }
    }

    public void drawSingleNode(Graphics2D g2, Node node) {
        // set color for tree here
        Color color = node.getColor();
        if (node.isSelected()) {
            color = this.selectedColor;
        }

        Node left = node.getLeft();
        Node right = node.getRight();

        // set up points for poly line
        int rx = 0;
        int lx = 0;
        int ry = 0;
        int ly = 0;

        if (right != null) {
            rx = (int) ((right.getIndex() * elementWidth) + elementWidth / 2.0);
            ry = yToPix(right.correlation);
        }
        if (left != null) {
            lx = (int) ((left.getIndex() * elementWidth) + elementWidth / 2.0);
            ly = yToPix(left.correlation);
        }
        if (subtractElementWidth) { // for aravind
            lx -= elementWidth;
            rx -= elementWidth;
        }
        int ty = yToPix(node.correlation);
        g2.setColor(color);
        g2.drawPolyline(new int[]{rx, rx, lx, lx}, new int[]{ry, ty, ty,
                ly}, 4);

    }

    public void flip() {
        Node temp = selectedNode.getLeft();
        Node right = selectedNode.getRight();
        selectedNode.setRight(temp);
        selectedNode.setLeft(right);
        int min = selectedNode.getMinIndex();
        int max = selectedNode.getMaxIndex();
        int index = max;
        String[] tempIds = (String[]) arrayIds.clone();
        for (int i = min; i <= max; i++) {
            Node n = (Node) nodes.get(arrayIds[i]);
            n.setIndex(index);
            n.setMaxIndex(index);
            n.setMinIndex(index);
            tempIds[i] = arrayIds[index];
            index--;
        }
        arrayIds = tempIds;
        selectedNode.recomputeIndices();
    }

    /**
     * helper method to find rectangle bounding each node *
     */
    public Point getNodePosition(Node node) {
        int x = (int) ((node.getIndex() * elementWidth) + elementWidth / 2.0);
        int y = yToPix(node.getCorrelation());
        return new Point(x, y);
    }

    // scrollable interface methods
    public Dimension getPreferredScrollableViewportSize() {
        return this.getPreferredSize();
    }

    public Dimension getPreferredSize() {
        Dimension d = super.getPreferredSize();
        d.width = maxIndex * elementWidth;
        return d;
    }

    public int getScrollableBlockIncrement(java.awt.Rectangle visableRectangle,
                                           int orientation, int direction) {
        int block = 10;
        if (orientation == SwingConstants.VERTICAL) {
            block = 10;
        } else if (orientation == SwingConstants.HORIZONTAL) {
            block = elementWidth * 5;
        }
        return block;
    }

    public boolean getScrollableTracksViewportHeight() {
        return true;
    }

    public boolean getScrollableTracksViewportWidth() {
        return false;
    }

    public int getScrollableUnitIncrement(java.awt.Rectangle visableRectangle,
                                          int orientation, int direction) {

        int block = 10;
        if (orientation == SwingConstants.VERTICAL) {
            block = 10;
        } else if (orientation == SwingConstants.HORIZONTAL) {
            block = elementWidth;
        }
        return block;
    }

    public Node getSelectedNode() {
        return selectedNode;
    }

    public String getToolTipText(MouseEvent e) {
        Node node = getNode(e);
        if (node != null) {
            return node.getId() + " " + node.getIndex();
        }
        return null;
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        // pixelMatrix[0] = xPixPerUnit;
        yPixPerUnit = Math.max(getHeight() - bottomGutter - topGutter - 1, 1)
                / (ymax - ymin);
        // pixelMatrix[0] = xPixPerUnit;
        pixelMatrix[1] = 0;
        pixelMatrix[2] = 0;
        pixelMatrix[3] = -yPixPerUnit;
        // pixelMatrix[4] = -xmin*xPixPerUnit+leftGutter;
        pixelMatrix[5] = ymax * yPixPerUnit + topGutter;
        // get root node
        Node rootNode = (Node) nodes.get(rootId);
        // this should draw each node recursively
        draw(g2, rootNode);
    }

    public void removeListSelectionListener(ListSelectionListener l) {
        listenerList.remove(ListSelectionListener.class, l);
    }

    public void setBottomGutter(int gutter) {
        bottomGutter = gutter;
    }

    public void setElementWidth(int elementWidth) {
        this.elementWidth = elementWidth;
    }

    /**
     * Set the HashMap of Node objects to draw image of values *
     */
    public void setNodes(HashMap map) {
        nodes = map;
    }

    public void setSelectedNodeColor(Color color) {
        Node node = selectedNode;
        if (node == null) {
            node = (Node) nodes.get(rootId);
        }
        node.setRootColor(color);
    }

    public void setSubtractElementWidth(boolean b) {
        this.subtractElementWidth = b;
    }

    public void setTopGutter(int gutter) {
        topGutter = gutter;
    }

    public void setYMax(double d) {
        ymax = d;
    }

    public void setYMin(double d) {
        ymin = d;
    }

    public BufferedImage snapshot() {
        BufferedImage bi = new BufferedImage(this.getPreferredSize().width,
                this.getPreferredSize().height, BufferedImage.TYPE_3BYTE_BGR);
        Graphics g = bi.getGraphics();
        g.setColor(Color.white);
        g.fillRect(0, 0, bi.getWidth(), bi.getHeight());
        g.setColor(Color.black);
        this.paintComponent(g);
        g.dispose();
        return bi;
    }

    protected Node getNode(MouseEvent e) {
        double x = e.getX();
        double y = e.getY();
        int leafIndex = (int) (x / elementWidth);
        // leaf must be a sample
        String leafid;
        if (leafIndex >= 0 && leafIndex <= arrayIds.length) {
            leafid = arrayIds[leafIndex];
        } else {
            return null;
        }
        Node n = (Node) nodes.get(leafid);

        if (n != null) {
            while (!n.isRoot()) {
                Node parent = n.getParent();
                Point p = getNodePosition(parent);
                if (Math.abs(p.x - x) < 4 && Math.abs(p.y - y) < 4) {
                    return parent;
                }
                n = parent;
            }
        }
        return null;
    }

    protected void notifyListeners() {
        int min = selectedNode != null ? selectedNode.getMinIndex() : -1;
        int max = selectedNode != null ? selectedNode.getMaxIndex() : -1;
        ListSelectionEvent e = new ListSelectionEvent(this, min, max, false);
        Object[] listeners = listenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ListSelectionListener.class) {
                ((ListSelectionListener) listeners[i + 1]).valueChanged(e);
            }
        }
    }

    /**
     * Gets the number of pixels per world unit in the y direction. Y pixels per
     * unit is positive if y increases from bottom to top.
     *
     * @return pixels per unit
     */
    public double getYPixPerUnit() {
        return -pixelMatrix[3];
    }

    int yToPix(double y) {
        double pix = pixelMatrix[3] * y + pixelMatrix[5];
        return (int) Math.floor(pix);
    }

    private void init(Map nodes, String rootId, String[] ids) {
        this.nodes = nodes;
        maxIndex = nodes.size();
        this.rootId = rootId;
        arrayIds = ids;
        this.addMouseListener(new MyMouseListener());
    }

    class MyMouseListener extends MouseAdapter {
        public void mouseClicked(MouseEvent mouseEvent) {
            // clear last node selected
            if (selectedNode != null) {
                selectedNode.setSelected(false);
            }
            selectedNode = getNode(mouseEvent);

            if (selectedNode != null) {
                selectedNode.setSelected(true);
            }
            repaint();
            notifyListeners();

        }

    }
}
