/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

/*
 * ArrayTreePanel.java
 *
 * Created on June 20, 2003, 3:41 PM
 */

package org.genepattern.clustering.hierarchical;

import javax.swing.*;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Lisa Simirenko
 * @author jgould
 */
public class GeneTreePanel extends JPanel implements Scrollable, Tree {

    protected String rootId;

    protected Map nodes;

    protected int maxIndex;

    protected int elementHeight;

    protected Color selectedColor = Color.magenta;

    protected Node selectedNode = null;

    protected String[] geneIds;

    protected int leftGutter = 4;

    protected int rightGutter = 4;

    protected double xPixPerUnit;

    protected double xmax;

    protected double xmin;

    protected EventListenerList listenerList = new EventListenerList();

    private double[] pixelMatrix = new double[6];

    /**
     * Creates a new instance of GeneTreePanel
     *
     * @param parser The parser
     */
    public GeneTreePanel(AtrGtrReader parser) {
        super();
        init(parser.getNodes(), parser.getRootNodeId(), parser.getIds());
        xmin = parser.getMinArrayCorr();
        xmax = 1;
    }

    /**
     * Creates a new instance of GeneTreePanel
     *
     * @param nodes  id to Node map
     * @param rootId the id of the root node
     * @param ids    ordered ids of the leaf nodes
     */
    public GeneTreePanel(Map nodes, String rootId, String[] ids) {
        super();
        init(nodes, rootId, ids);
    }

    public void addListSelectionListener(ListSelectionListener l) {
        listenerList.add(ListSelectionListener.class, l);
    }

    public void removeListSelectionListener(ListSelectionListener l) {
        listenerList.remove(ListSelectionListener.class, l);
    }

    protected void notifyListeners() {
        int min = selectedNode != null ? selectedNode.getMinIndex() : -1;
        int max = selectedNode != null ? selectedNode.getMaxIndex() : -1;
        ListSelectionEvent e = new ListSelectionEvent(this, min, max, false);
        Object[] listeners = listenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ListSelectionListener.class) {
                ((ListSelectionListener) listeners[i + 1]).valueChanged(e);
            }
        }
    }

    public void setElementHeight(int elementHeight) {
        this.elementHeight = elementHeight;
    }

    public void setSelectedNode(Node n) {
        selectedNode = n;
    }

    public Node getSelectedNode() {
        return selectedNode;
    }

    /**
     * Set the HashMap of Node objects to draw image of values *
     */
    public void setNodes(HashMap map) {
        nodes = map;
    }

    public BufferedImage snapshot() {
        BufferedImage bi = new BufferedImage(this.getPreferredSize().width,
                this.getPreferredSize().height, BufferedImage.TYPE_3BYTE_BGR);
        Graphics g = bi.getGraphics();
        g.setColor(Color.white);
        g.fillRect(0, 0, bi.getWidth(), bi.getHeight());
        g.setColor(Color.black);
        this.paintComponent(g);
        g.dispose();
        return bi;

    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;

        xPixPerUnit = Math.max(getWidth() - leftGutter - rightGutter - 1, 1)
                / (xmax - xmin);
        pixelMatrix[0] = xPixPerUnit;
        pixelMatrix[1] = 0;
        pixelMatrix[2] = 0;
        // pixelMatrix[3] = -yPixPerUnit;
        pixelMatrix[4] = -xmin * xPixPerUnit + leftGutter;
        // pixelMatrix[5] = ymax * yPixPerUnit + topGutter;

        // get root node
        Node rootNode = (Node) nodes.get(rootId);
        // this should draw each node recursively
        draw(g2, rootNode);
    }

    /**
     * this method draws nodes recursively by calling drawSingleNode(), and
     * draw() for both left and right if not leaves
     */
    public void draw(Graphics2D g2, Node node) {
        Node left = node.getLeft();
        Node right = node.getRight();

        drawSingleNode(g2, node);

        if (left.isLeaf() == false) {
            draw(g2, left);
        }
        if (right.isLeaf() == false) {
            draw(g2, right);
        }
    }

    int xToPix(double x) {
        double pix = pixelMatrix[0] * x + pixelMatrix[4];
        return (int) Math.floor(pix);
    }

    public void drawSingleNode(Graphics2D g2, Node node) {
        // set color for tree here
        Color color = node.getColor();
        if (node.isSelected()) {
            color = this.selectedColor;
        }

        Node left = node.getLeft();
        Node right = node.getRight();

        // set up points for poly line
        int ry = 0;
        int ly = 0;
        int rx = 0;
        int lx = 0;

        if (right != null) {
            ry = (int) ((right.getIndex() * elementHeight) + elementHeight / 2.0);
            rx = xToPix(right.correlation);
        }
        if (left != null) {
            ly = (int) ((left.getIndex() * elementHeight) + elementHeight / 2.0);
            lx = xToPix(left.correlation);

        }
        int tx = xToPix(node.correlation);

        g2.setColor(color);
        g2.drawPolyline(new int[]{rx, tx, tx, lx}, new int[]{ry, ry, ly,
                ly}, 4);

    }

    public Dimension getPreferredSize() {
        Dimension d = super.getPreferredSize();
        d.height = maxIndex * elementHeight;
        return d;
    }

    private void init(Map nodes, String rootId, String[] ids) {
        this.nodes = nodes;
        maxIndex = nodes.size();
        this.rootId = rootId;
        geneIds = ids;
        this.addMouseListener(new MyMouseListener());
    }

    class MyMouseListener extends MouseAdapter {
        public void mouseClicked(MouseEvent mouseEvent) {
            // clear last node selected
            if (selectedNode != null) {
                selectedNode.setSelected(false);
            }
            selectedNode = null;

            double x = mouseEvent.getX();
            double y = mouseEvent.getY();

            int leafIndex = (int) (y / elementHeight);
            // since a leaf must be a gene - get gene from geneMap
            String leafid;

            if (leafIndex >= 0 && leafIndex <= geneIds.length) {
                leafid = geneIds[leafIndex];
            } else {
                return;
            }

            Node n = (Node) nodes.get(leafid);
            if (n != null) {
                while (!n.isRoot()) {
                    Node parent = n.getParent();
                    Point p = getNodePosition(parent);
                    if (Math.abs(p.x - x) < 4 && Math.abs(p.y - y) < 4) {
                        selectedNode = parent;
                        break;
                    }
                    n = parent;
                }
            }

            if (selectedNode != null) {
                selectedNode.setSelected(true);
            }
            repaint();
            notifyListeners();
        }

    }

    /**
     * helper method to find rectangle bounding each node *
     */
    public Point getNodePosition(Node node) {
        int x = xToPix(node.correlation);
        int y = (int) ((node.getIndex() * elementHeight) + elementHeight / 2.0);
        return new Point(x, y);
    }

    // scrollable interface methods
    public Dimension getPreferredScrollableViewportSize() {
        return this.getPreferredSize();
    }

    public int getScrollableBlockIncrement(java.awt.Rectangle visableRectangle,
                                           int orientation, int direction) {
        int block = 10;
        if (orientation == SwingConstants.VERTICAL) {
            block = 10;
        } else if (orientation == SwingConstants.HORIZONTAL) {
            block = elementHeight * 5;
        }
        return block;
    }

    public boolean getScrollableTracksViewportHeight() {
        return true;
    }

    public boolean getScrollableTracksViewportWidth() {
        return false;
    }

    public int getScrollableUnitIncrement(java.awt.Rectangle visableRectangle,
                                          int orientation, int direction) {

        int block = 10;
        if (orientation == SwingConstants.VERTICAL) {
            block = 10;
        } else if (orientation == SwingConstants.HORIZONTAL) {
            block = elementHeight;
        }
        return block;
    }

    public void deselectSelectedNode() {
        if (selectedNode != null) {
            selectedNode.setSelected(false);
        }
        repaint();
    }

    public void setSelectedNodeColor(Color color) {
        Node node = selectedNode;
        if (node == null) {
            node = (Node) nodes.get(rootId);
        }
        node.setRootColor(color);
    }

    public void flip() {
        Node temp = selectedNode.getLeft();
        Node right = selectedNode.getRight();
        selectedNode.setRight(temp);
        selectedNode.setLeft(right);
        int min = selectedNode.getMinIndex();
        int max = selectedNode.getMaxIndex();
        int index = max;
        String[] tempIds = (String[]) geneIds.clone();
        for (int i = min; i <= max; i++) {
            Node n = (Node) nodes.get(geneIds[i]);
            n.setIndex(index);
            n.setMaxIndex(index);
            n.setMinIndex(index);
            tempIds[i] = geneIds[index];
            index--;
        }
        geneIds = tempIds;
        selectedNode.recomputeIndices();
    }

    public void setLeftGutter(int gutter) {
        leftGutter = gutter;
    }

    public void setRightGutter(int gutter) {
        rightGutter = gutter;
    }

    public void setXMax(double x) {
        xmax = x;
    }

    public void setXMin(double x) {
        xmin = x;
    }

}
