/*
  The Broad Institute
  SOFTWARE COPYRIGHT NOTICE AGREEMENT
  This software and its documentation are copyright (2003-2006) by the
  Broad Institute/Massachusetts Institute of Technology. All rights are
  reserved.

  This software is supplied without any warranty or guaranteed support
  whatsoever. Neither the Broad Institute nor MIT can be responsible for its
  use, misuse, or functionality.
*/


package org.genepattern.clustering.hierarchical.image;

import org.genepattern.clustering.hierarchical.ArrayTreePanel;
import org.genepattern.clustering.hierarchical.AtrGtrReader;
import org.genepattern.clustering.hierarchical.GeneTreePanel;
import org.genepattern.data.expr.ExpressionData;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.heatmap.image.DisplaySettings;
import org.genepattern.heatmap.image.FeatureAnnotator;
import org.genepattern.heatmap.image.HeatMap;
import org.genepattern.heatmap.image.SampleAnnotator;
import org.genepattern.io.ImageUtil;
import org.genepattern.io.ParseException;
import org.genepattern.io.expr.stanford.CdtParser;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.FileInputStream;
import java.io.IOException;

public class HCLImage {

    /**
     * @param cdtParser
     * @param gtrReader the AtrGtrReader for the .gtr file or <tt>null</tt> if genes
     *                  were not clustered
     * @param atrReader the AtrGtrReader for the .atr file or <tt>null</tt> if
     *                  samples were not clustered
     * @param ds
     * @param sa
     * @param fa
     * @return the image
     */
    public static BufferedImage createImage(CdtParser cdtParser,
                                            AtrGtrReader gtrReader, AtrGtrReader atrReader, DisplaySettings ds,
                                            SampleAnnotator sa, FeatureAnnotator fa) {
        ExpressionData data = cdtParser.getExpressionData();
        GeneTreePanel geneTree = null;
        int geneTreeWidth = 150;
        int arrayTreeHeight = 150;
        if (gtrReader != null) {
            geneTree = new GeneTreePanel(gtrReader);
            geneTree.setElementHeight(ds.rowSize);
            Dimension size = new Dimension(geneTreeWidth, geneTree
                    .getPreferredSize().height);
            geneTree.setPreferredSize(size);
            geneTree.setSize(size);
        }
        ArrayTreePanel arrayTree = null;
        if (atrReader != null) {
            arrayTree = new ArrayTreePanel(atrReader);
            arrayTree.setElementWidth(ds.columnSize);
            Dimension size = new Dimension(arrayTree.getPreferredSize().width,
                    arrayTreeHeight);
            arrayTree.setPreferredSize(size);
            arrayTree.setSize(size);
        }

        return createImage(data, geneTree != null ? geneTree.snapshot() : null,
                arrayTree != null ? arrayTree.snapshot() : null, ds, sa, fa);
    }

    public static BufferedImage createImage(IExpressionData data,
                                            BufferedImage geneTree, BufferedImage arrayTree,
                                            DisplaySettings ds, SampleAnnotator sa, FeatureAnnotator fa) {

        HeatMap.HeatMapImage result = HeatMap.createImage2(data, ds, sa, fa);
        int heatMapStart = result.headerHeight;
        BufferedImage bi = result.image;

        int width = bi.getWidth();
        int height = bi.getHeight();
        int arrayTreeX = result.leftBorder;
        int heatMapY = 0;
        int heatMapX = 0;
        int geneTreeY = heatMapStart;
        if (geneTree != null) {
            int geneTreeWidth = geneTree.getWidth();
            width += geneTreeWidth;
            arrayTreeX += geneTreeWidth;
            heatMapX = geneTreeWidth;
        }
        if (arrayTree != null) {
            int arrayTreeHeight = arrayTree.getHeight();
            height += arrayTreeHeight;
            heatMapY = arrayTreeHeight;
            geneTreeY += arrayTreeHeight;
        }

        BufferedImage composite = new BufferedImage(width, height,
                BufferedImage.TYPE_3BYTE_BGR);

        Graphics2D g = composite.createGraphics();
        g.setColor(Color.white);
        g.fillRect(0, 0, composite.getWidth(), composite.getHeight());
        g.setColor(Color.black);
        if (arrayTree != null) {
            g.drawImage(arrayTree, arrayTreeX, 0, null);
        }
        if (geneTree != null) {
            g.drawImage(geneTree, 0, geneTreeY, null);
        }
        g.drawImage(bi, heatMapX, heatMapY, null);
        g.dispose();
        return composite;
    }


    public static void saveImage(String cdtFile, String gtrFile,
                                 String atrFile, DisplaySettings ds, SampleAnnotator sa,
                                 FeatureAnnotator fa, String outputFileName, String outputFileFormat)
            throws ParseException {
        CdtParser parser = new CdtParser();

        FileInputStream fis = null;
        try {
            fis = new FileInputStream(cdtFile);
            parser.parse(fis);
        } catch (IOException e) {
            throw new ParseException(
                    "An error occurred while reading the file " + cdtFile);
        } finally {
            try {
                if (fis != null) {
                    fis.close();
                }
            } catch (IOException e) {
            }
        }
        AtrGtrReader gtrReader = null;
        if (gtrFile != null) {
            try {
                gtrReader = new AtrGtrReader((String[]) parser.getGeneIds()
                        .toArray(new String[0]), gtrFile);
            } catch (IOException e) {
                throw new ParseException(
                        "An error occurred while reading the file " + gtrFile);
            }
        }
        AtrGtrReader atrReader = null;
        if (atrFile != null) {
            try {
                atrReader = new AtrGtrReader(parser.getArrayIds(), atrFile);
            } catch (IOException e) {
                throw new ParseException(
                        "An error occurred while reading the file " + atrFile);
            }
        }

        ImageUtil.saveImage(createImage(parser, gtrReader, atrReader, ds, sa,
                fa), outputFileName, outputFileFormat);
    }
}
