/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.data.matrix;

import gnu.trove.TIntArrayList;
import gnu.trove.TObjectIntHashMap;
import org.genepattern.data.expr.ExpressionData;
import org.genepattern.data.expr.IExpressionData;

import java.text.NumberFormat;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Joshua Gould
 */
public class IExpressionDataUtil {
    private IExpressionDataUtil() {
    }

    public static Set intersection(Object[] obj1, Object[] obj2) {
        Set s1 = new HashSet(Arrays.asList(obj1));
        Set s2 = new HashSet(Arrays.asList(obj2));
        s1.retainAll(s2);
        return s1;
    }

    public static String[] getDataNames(IExpressionData data) {
        String[] s = new String[data.getDataCount()];
        for (int i = 0, size = data.getDataCount(); i < size; i++) {
            s[i] = data.getDataName(i);
        }
        return s;
    }

    public static String[] getRowMetadataNames(IExpressionData data) {
        String[] s = new String[data.getRowMetadataCount()];
        for (int i = 0, size = data.getRowMetadataCount(); i < size; i++) {
            s[i] = data.getRowMetadataName(i);
        }
        return s;
    }

    public static String[] getColumnMetadataNames(IExpressionData data) {
        String[] s = new String[data.getColumnMetadataCount()];
        for (int i = 0, size = data.getColumnMetadataCount(); i < size; i++) {
            s[i] = data.getColumnMetadataName(i);
        }
        return s;
    }

    public static String[] getColumnNames(IExpressionData data) {
        String[] s = new String[data.getColumnCount()];
        for (int i = 0, size = data.getColumnCount(); i < size; i++) {
            s[i] = data.getColumnName(i);
        }
        return s;
    }

    public String[] getRowNames(IExpressionData data) {
        String[] s = new String[data.getRowCount()];
        for (int i = 0, size = data.getRowCount(); i < size; i++) {
            s[i] = data.getRowName(i);
        }
        return s;
    }

    public static IExpressionData mergeColumnsView(
            final IExpressionData expressionDataOne,
            final IExpressionData expressionDataTwo) {
        final TIntArrayList oneRowIndices = new TIntArrayList();
        final TIntArrayList twoRowIndices = new TIntArrayList();
        final int columnsOne = expressionDataOne.getColumnCount();
        final int columnsTwo = expressionDataTwo.getColumnCount();

        for (int i = 0, rows = expressionDataOne.getRowCount(); i < rows; i++) {
            String rowName = expressionDataOne.getRowName(i);

            int index = expressionDataTwo.getRowIndex(rowName);

            if (index != -1) {
                oneRowIndices.add(i);
                twoRowIndices.add(index);
            }
        }

        final int columns = columnsOne + columnsTwo;
        final String[] columnNames = new String[columns];

        System.arraycopy(getColumnNames(expressionDataOne), 0, columnNames, 0,
                columnsOne);
        System.arraycopy(getColumnNames(expressionDataTwo), 0, columnNames,
                columnsOne, columnsTwo);

        org.genepattern.io.Util.makeUnique(columnNames);
        final TObjectIntHashMap nameToIndex = new TObjectIntHashMap(
                columnNames.length);
        for (int i = 0; i < columnNames.length; i++) {
            nameToIndex.put(columnNames[i], i);
        }

        final String[] commonData = (String[]) new HashSet(intersection(
                getDataNames(expressionDataOne),
                getDataNames(expressionDataTwo))).toArray(new String[0]);

        final String[] commonRowMetadata = (String[]) new HashSet(intersection(
                getRowMetadataNames(expressionDataOne),
                getRowMetadataNames(expressionDataTwo))).toArray(new String[0]);
        final String[] commonColumnMetadata = (String[]) new HashSet(
                intersection(getColumnMetadataNames(expressionDataOne),
                        getColumnMetadataNames(expressionDataTwo)))
                .toArray(new String[0]);

        return new IExpressionData() {

            private IExpressionData getData(int column) {
                IExpressionData data = column < columnsOne ? expressionDataOne
                        : expressionDataTwo;
                return data;
            }

            private int getColumn(int column) {
                if (column < columnsOne) {
                    return column;
                }
                return column - columnsOne;
            }

            private int getRow(int row, int column) {
                if (column < columnsOne) {
                    return oneRowIndices.get(row);
                }
                return twoRowIndices.get(row);
            }

            public String getValueAsString(int row, int column) {
                return getData(column).getValueAsString(getRow(row, column),
                        getColumn(column));
            }

            public double getValue(int row, int column) {
                return getData(column).getValue(getRow(row, column),
                        getColumn(column));
            }

            public String getRowName(int row) {
                return expressionDataOne.getRowName(oneRowIndices.get(row));
            }

            public int getRowCount() {
                return oneRowIndices.size();
            }

            public int getColumnCount() {
                return columns;
            }

            public String getColumnName(int column) {
                return columnNames[column];
            }

            public int getRowIndex(String rowName) {
                int index = oneRowIndices.binarySearch(expressionDataOne
                        .getRowIndex(rowName));
                if (index < 0) {
                    index = -1;
                }
                return index;
            }

            public int getColumnIndex(String columnName) {
                return nameToIndex.containsKey(columnName) ? nameToIndex
                        .get(columnName) : -1;
            }

            public Object getData(int row, int column, String name) {
                return getData(column).getData(getRow(row, column),
                        getColumn(column), name);

            }

            public String getRowMetadata(int row, String name) {
                return expressionDataOne.getRowMetadata(oneRowIndices.get(row),
                        name);
            }

            public String getColumnMetadata(int column, String name) {
                return getData(column).getColumnMetadata(getColumn(column),
                        name);
            }

            public String getDataName(int index) {
                return commonData[index];
            }

            public int getDataCount() {
                return commonData.length;
            }

            public String getRowMetadataName(int index) {
                return commonRowMetadata[index];
            }

            public int getRowMetadataCount() {
                return commonRowMetadata.length;
            }

            public String getColumnMetadataName(int index) {
                return commonColumnMetadata[index];
            }

            public int getColumnMetadataCount() {
                return commonColumnMetadata.length;
            }

        };

    }

    public static IExpressionData transposeView(final IExpressionData dataset) {

        return new IExpressionData() {

            public String getValueAsString(int row, int column) {
                return dataset.getValueAsString(column, row);
            }

            public double getValue(int row, int column) {
                return dataset.getValue(column, row);
            }

            public String getRowName(int row) {
                return dataset.getColumnName(row);
            }

            public int getRowCount() {
                return dataset.getColumnCount();
            }

            public int getColumnCount() {
                return dataset.getRowCount();
            }

            public String getColumnName(int column) {
                return dataset.getRowName(column);
            }

            public String getRowMetadata(int row, String name) {
                return dataset.getColumnMetadata(row, name);
            }

            public String getColumnMetadata(int column, String name) {
                return dataset.getRowMetadata(column, name);
            }

            public int getRowIndex(String rowName) {
                return dataset.getColumnIndex(rowName);
            }

            public int getColumnIndex(String columnName) {
                return dataset.getRowIndex(columnName);
            }

            public Object getData(int row, int column, String name) {
                return dataset.getData(column, row, name);
            }

            public String getDataName(int index) {
                return dataset.getDataName(index);
            }

            public int getDataCount() {
                return dataset.getDataCount();
            }

            public String getRowMetadataName(int index) {
                return dataset.getColumnMetadataName(index);
            }

            public int getRowMetadataCount() {
                return dataset.getColumnMetadataCount();
            }

            public String getColumnMetadataName(int index) {
                return dataset.getRowMetadataName(index);
            }

            public int getColumnMetadataCount() {
                return dataset.getRowMetadataCount();
            }

        };

    }

    /**
     * Gets a sliced view of the given data
     *
     * @param dataset     The data
     * @param rowNames    The row names
     * @param columnNames The column names
     * @return Sliced data
     */

    public static IExpressionData sliceView(final IExpressionData dataset,
                                            String[] rowNames, String[] columnNames) {
        int[] rowIndices = rowNames != null ? new int[rowNames.length] : null;
        int[] columnIndices = columnNames != null ? new int[columnNames.length]
                : null;
        if (rowIndices != null) {
            for (int i = 0, rows = rowIndices.length; i < rows; i++) {
                int index = dataset.getRowIndex(rowNames[i]);
                if (index == -1) {
                    throw new IllegalArgumentException("Row name "
                            + rowNames[i] + " not found.");
                }
                rowIndices[i] = index;
            }
        }

        if (columnIndices != null) {
            for (int i = 0, cols = columnIndices.length; i < cols; i++) {
                int index = dataset.getColumnIndex(columnNames[i]);
                if (index == -1) {
                    throw new IllegalArgumentException("Column name "
                            + columnNames[i] + " not found.");
                }
                columnIndices[i] = index;
            }
        }

        return sliceView(dataset, rowIndices, columnIndices);

    }

    /**
     * Gets a sliced view of the given data
     *
     * @param dataset        The data
     * @param _rowIndices    The row indices
     * @param _columnIndices The column indices
     * @return Sliced data
     */
    public static IExpressionData sliceView(final IExpressionData dataset,
                                            int[] _rowIndices, int[] _columnIndices) {
        if (dataset instanceof ExpressionData) {
            return ((ExpressionData) dataset)
                    .slice(_rowIndices, _columnIndices);
        }
        if (_rowIndices == null) {
            _rowIndices = new int[dataset.getRowCount()];
            for (int i = dataset.getRowCount(); --i >= 0;) {
                _rowIndices[i] = i;
            }
        }
        if (_columnIndices == null) {
            _columnIndices = new int[dataset.getColumnCount()];
            for (int i = dataset.getColumnCount(); --i >= 0;) {
                _columnIndices[i] = i;
            }
        }
        final int[] rowIndices = _rowIndices;
        final int[] columnIndices = _columnIndices;

        return new IExpressionData() {

            public String getValueAsString(int row, int column) {
                return dataset.getValueAsString(rowIndices[row],
                        columnIndices[column]);
            }

            public double getValue(int row, int column) {
                return dataset.getValue(rowIndices[row], columnIndices[column]);
            }

            public String getRowName(int row) {
                return dataset.getRowName(rowIndices[row]);
            }

            public int getRowCount() {
                return rowIndices.length;
            }

            public int getColumnCount() {
                return columnIndices.length;
            }

            public String getColumnName(int column) {
                return dataset.getColumnName(columnIndices[column]);
            }

            public String getRowMetadata(int row, String name) {
                return dataset.getColumnMetadata(rowIndices[row], name);
            }

            public String getColumnMetadata(int column, String name) {
                return dataset.getColumnMetadata(columnIndices[column], name);
            }

            public int getRowIndex(String rowName) {
                throw new UnsupportedOperationException();
            }

            public int getColumnIndex(String columnName) {
                throw new UnsupportedOperationException();
            }

            public Object getData(int row, int column, String name) {
                return dataset.getData(rowIndices[row], columnIndices[column],
                        name);
            }

            public String getDataName(int index) {
                return dataset.getDataName(index);
            }

            public int getDataCount() {
                return dataset.getDataCount();
            }

            public String getRowMetadataName(int index) {
                return dataset.getRowMetadataName(index);
            }

            public int getRowMetadataCount() {
                return dataset.getRowMetadataCount();
            }

            public String getColumnMetadataName(int index) {
                return dataset.getColumnMetadataName(index);
            }

            public int getColumnMetadataCount() {
                return dataset.getColumnMetadataCount();
            }

        };
    }

    public static IExpressionData createRandomData(int rows, int columns) {
        double[][] data = new double[rows][columns];
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                data[i][j] = Math.random() * 300 + 1;
            }
        }
        DoubleMatrix2D matrix = new DoubleMatrix2D(data);
        ExpressionData d = new ExpressionData(matrix, null, null);
        return d;
    }

    public static String toString(IExpressionData d) {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(4);
        StringBuffer sb = new StringBuffer();
        for (int i = 0, rows = d.getRowCount(); i < rows; i++) {
            for (int j = 0, columns = d.getColumnCount(); j < columns; j++) {
                sb.append(nf.format(d.getValue(i, j)));
                sb.append(" ");
            }
            sb.append("\n");
        }
        return sb.toString();
    }

    public static IClassVector slice(IClassVector classVector, int[] order) {
        String[] x = new String[order.length];
        for (int i = 0, length = order.length; i < length; i++) {
            x[i] = classVector
                    .getClassName(classVector.getAssignment(order[i]));
        }

        String[] classes = new String[classVector.getClassCount()];
        for (int i = 0, num = classVector.getClassCount(); i < num; i++) {
            classes[i] = classVector.getClassName(i);
        }
        return new ClassVector(x, classes);
    }

    public static DoubleMatrix2D toMatrix(IExpressionData trainData) {
        DoubleMatrix2D matrix = new DoubleMatrix2D(trainData.getRowCount(),
                trainData.getColumnCount());
        for (int i = 0, rows = trainData.getRowCount(); i < rows; i++) {
            matrix.setRowName(i, trainData.getRowName(i));
            for (int j = 0, columns = trainData.getColumnCount(); j < columns; j++) {
                matrix.set(i, j, trainData.getValue(i, j));
            }
        }
        for (int j = 0, columns = trainData.getColumnCount(); j < columns; j++) {
            matrix.setColumnName(j, trainData.getColumnName(j));
        }
        return matrix;
    }
}
