package org.genepattern.gsea;

import com.jidesoft.grid.SortableTable;
import com.jidesoft.swing.JideTabbedPane;
import edu.mit.broad.genome.objects.DefaultGeneSetMatrix;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.parsers.ParserFactory;
import foxtrot.Task;
import foxtrot.Worker;
import org.genepattern.menu.PlotAction;
import org.genepattern.uiutil.OS;
import org.genepattern.uiutil.UIUtil;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

/**
 * @author jgould
 */
public class GeneSetOverlapWidget {

    JideTabbedPane tabbedPane;

    static int runs = 0;

    private ViewAndSearchComponent viewAndSearchComponent;

    private GeneSetMatrix geneMatrix;

    static {
        System.setProperty(PlotAction.SHOW_ACCELERATORS_PROPERTY, "false");
    }

    public GeneSetOverlapWidget() {
        tabbedPane = new JideTabbedPane();
        tabbedPane.setHideOneTab(true);
        tabbedPane.setTabEditingAllowed(true);
        tabbedPane.setShowCloseButtonOnTab(true);
        init();
        tabbedPane.addTab("Gene Set Overlap", viewAndSearchComponent);
        tabbedPane.setTabClosableAt(0, false);
    }

    public GeneSetOverlapWidget(JideTabbedPane tabbedPane) {
        this.tabbedPane = tabbedPane;
        UIUtil.setMessageHandler(new XToolsMessageHandler());
        init();
        tabbedPane.addTab("Gene Set Overlap", viewAndSearchComponent); // FIXME
        // add
        // input
        // panel
    }

    private void init() {
        ActionListener runListener = new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                runAnalysis();
            }
        };

        viewAndSearchComponent = new ViewAndSearchComponent(
                "Run gene set overlap analysis", new int[]{0}, runListener);
    }

    public void setData(GeneSetMatrix geneMatrix) {
        this.geneMatrix = geneMatrix;
        viewAndSearchComponent.setTableModel(createTableModel());
    }

    protected TableModel createTableModel() {
        TableModel tableModel = new AbstractTableModel() {

            public int getRowCount() {
                return geneMatrix.getNumGeneSets();
            }

            public int getColumnCount() {
                return 2;
            }

            public Object getValueAt(int rowIndex, int columnIndex) {
                switch (columnIndex) {
                    case 0:
                        return geneMatrix.getGeneSet(rowIndex).getName(true);
                    case 1:
                        return new Integer(geneMatrix.getGeneSet(rowIndex)
                                .getNumMembers());
                }
                return null;

            }

            public String getColumnName(int j) {
                switch (j) {
                    case 0:
                        return "Gene Set";
                    case 1:
                        return "Size";
                }
                return null;
            }

            public Class getColumnClass(int j) {
                switch (j) {
                    case 0:
                        return String.class;
                    case 1:
                        return Integer.class;
                }
                return null;
            }

        };
        return tableModel;
    }

    private void runAnalysis() {
        tabbedPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        try {
            Worker.post(new Task() {
                public Object run() throws Exception {
                    _runAnalysis();
                    return null;
                }
            });
        } catch (Throwable t) {
            t.printStackTrace();
            tabbedPane.setCursor(Cursor
                    .getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            UIUtil
                    .showErrorDialog(tabbedPane.getTopLevelAncestor(),
                            "An error occurred while running gene set overlap analysis");
        }
        tabbedPane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    private void _runAnalysis() {
        SortableTable table = viewAndSearchComponent.getTable();
        int[] rows = table.getSelectedRows();

        final GeneSet[] gsets = new GeneSet[rows.length];
        for (int r = 0; r < rows.length; r++) {
            gsets[r] = geneMatrix.getGeneSet(table.getActualRowAt(r));
        }
        final GeneSetMatrix lev_gmx = new DefaultGeneSetMatrix("", gsets);
        LeadingEdgeAnalysis analysis = new LeadingEdgeAnalysis(lev_gmx, null,
                (Frame) tabbedPane.getTopLevelAncestor());
        runs++;
        tabbedPane.addTab("Gene Set Overlap Analysis-" + runs, analysis
                .getComponent());
        tabbedPane.setSelectedIndex(tabbedPane.getTabCount() - 1);
    }

    public static void main(String[] args) { // for running using GenePattern
        com.jidesoft.utils.Lm.verifyLicense(
                "Broad Institute of MIT and Harvard",
                "Gene set  enrichment analysis java desktop application",
                "YSjBO6OJfF9WbavzI73Jt1HgDI4x9L21");
        OS.setLookAndFeel();
        String geneMatrixFile = args[0];

        String title = "Gene Set Overlap Viewer - "
                + new File(geneMatrixFile).getName();

        GeneSetMatrix gmt = null;
        try {
            gmt = ParserFactory.readGeneSetMatrix(new File(geneMatrixFile), false);
        } catch (Exception e1) {
            e1.printStackTrace();
            UIUtil.showMessageDialog(null,
                    "An error occurred while reading the input file");
            System.exit(0);
        }
        JFrame frame = new JFrame();
        frame.setTitle(title);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        GeneSetOverlapWidget widget = new GeneSetOverlapWidget();
        widget.setData(gmt);
        frame.getContentPane().add(widget.tabbedPane);
        UIUtil.sizeToScreen(frame);
        frame.setVisible(true);

    }

}
