/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.gsea;

import org.genepattern.data.expr.IExpressionData;
import org.tigr.microarray.mev.cluster.algorithm.AlgorithmData;
import org.tigr.microarray.mev.cluster.algorithm.AlgorithmException;
import org.tigr.microarray.mev.cluster.algorithm.impl.HCL;
import org.tigr.util.FloatMatrix;

import java.util.Arrays;

/**
 * @author Joshua Gould
 */

public class HCLAlgorithm {

    public final static int AVERAGE_LINKAGE = 0;

    public final static int COMPLETE_LINKAGE = 1;

    public final static int SINGLE_LINKAGE = -1;

    public static final int COSINE = 2;

    public static final int COVARIANCE = 3;

    public static final int DOTPRODUCT = 5;

    public static final int EUCLIDEAN = 4;

    public static final int KENDALLSTAU = 10;

    public static final int MANHATTAN = 8;

    public static final int MUTUALINFORMATION = 11;

    public static final int PEARSON = 1;

    public static final int PEARSONSQARED = 7;

    public static final int PEARSONUNCENTERED = 6;

    public static final int SPEARMANRANK = 9;

    public final static int ROWS = 0;

    public final static int BOTH = 2;

    public final static int COLUMNS = 1;

    private int clusterBy;

    private int distanceMeasure;

    private int[] genes_child_1_array;

    private int[] genes_child_2_array;

    private FloatMatrix genes_height;

    private int[] genes_node_order;

    private int[] genes_order;

    private int mergeType;

    private int[] samples_child_1_array;

    private int[] samples_child_2_array;

    private FloatMatrix samples_height;

    private int[] samples_node_order;

    private int[] samples_order;

    public HCLAlgorithm(IExpressionData expressionData, int mergeType,
                        int distanceMeasure, int clusterBy) throws AlgorithmException {
        this.mergeType = mergeType;
        this.distanceMeasure = distanceMeasure;
        this.clusterBy = clusterBy;
        AlgorithmData samplesResult = null;
        AlgorithmData genesResult = null;
        FloatMatrix inputMatrix = new FloatMatrix(expressionData.getRowCount(),
                expressionData.getColumnCount());

        for (int i = 0, rows = expressionData.getRowCount(); i < rows; i++) {
            for (int j = 0, cols = expressionData.getColumnCount(); j < cols; j++) {
                inputMatrix.set(i, j, (float) expressionData.getValue(i, j));
            }
        }
        if (clusterBy == ROWS || clusterBy == BOTH) {
            HCL hcl = new HCL();
            AlgorithmData data = new AlgorithmData();
            data.addMatrix("experiment", inputMatrix);
            data.addParam("distance-function", "" + distanceMeasure);
            data.addParam("distance-absolute", "false");
            data.addParam("calculate-genes", "true");
            data.addParam("method-linkage", "" + mergeType);
            genesResult = hcl.execute(data);
            genes_child_1_array = genesResult.getIntArray("child-1-array");
            genes_child_2_array = genesResult.getIntArray("child-2-array");
            genes_node_order = genesResult.getIntArray("node-order");
            genes_order = getLeafOrder(genes_node_order, genes_child_1_array,
                    genes_child_2_array, null);
            genes_height = genesResult.getMatrix("height");
        }

        if (clusterBy == COLUMNS || clusterBy == BOTH) {
            HCL hcl = new HCL();
            AlgorithmData data = new AlgorithmData();
            data.addMatrix("experiment", inputMatrix);
            data.addParam("distance-function", "" + distanceMeasure);
            data.addParam("distance-absolute", "false");
            data.addParam("calculate-genes", "false");
            data.addParam("method-linkage", "0");
            samplesResult = hcl.execute(data);
            samples_child_1_array = samplesResult.getIntArray("child-1-array");
            samples_child_2_array = samplesResult.getIntArray("child-2-array");
            samples_node_order = samplesResult.getIntArray("node-order");
            samples_order = getLeafOrder(samples_node_order,
                    samples_child_1_array, samples_child_2_array, null);
            samples_height = samplesResult.getMatrix("height");
        }
    }

    public int[] getGenes_child_1_array() {
        return genes_child_1_array;
    }

    public int[] getGenes_child_2_array() {
        return genes_child_2_array;
    }

    public FloatMatrix getGenes_height() {
        return genes_height;
    }

    public int[] getGenes_order() {
        return genes_order;
    }

    public int[] getSamples_child_1_array() {
        return samples_child_1_array;
    }

    public int[] getSamples_child_2_array() {
        return samples_child_2_array;
    }

    public FloatMatrix getSamples_height() {
        return samples_height;
    }

    public int[] getSamples_node_order() {
        return samples_node_order;
    }

    public int[] getGenes_node_order() {
        return genes_node_order;
    }

    public int[] getSamples_order() {
        return samples_order;
    }

    private int fillLeafOrder(int[] leafOrder, int[] child1, int[] child2,
                              int pos, int index, int[] indices) {
        if (child1[index] != -1) {
            pos = fillLeafOrder(leafOrder, child1, child2, pos, child1[index],
                    indices);
        }
        if (child2[index] != -1) {
            pos = fillLeafOrder(leafOrder, child1, child2, pos, child2[index],
                    indices);
        } else {
            leafOrder[pos] = indices == null ? index : indices[index];
            pos++;
        }
        return pos;
    }

    private int[] getLeafOrder(int[] nodeOrder, int[] child1, int[] child2,
                               int[] indices) {
        int[] leafOrder = new int[nodeOrder.length];
        Arrays.fill(leafOrder, -1);
        fillLeafOrder(leafOrder, child1, child2, 0, child1.length - 2, indices);
        return leafOrder;
    }

    public int getClusterBy() {
        return clusterBy;
    }

    public int getDistanceMeasure() {
        return distanceMeasure;
    }

    public int getMergeType() {
        return mergeType;
    }
}