package org.genepattern.gsea;

import com.jidesoft.docking.DefaultDockingManager;
import com.jidesoft.docking.DockContext;
import com.jidesoft.docking.DockableFrame;
import com.jidesoft.icons.JideIconsFactory;
import com.jidesoft.swing.JideSplitPane;
import edu.mit.broad.genome.alg.GeneSetStats;
import edu.mit.broad.genome.alg.cluster.ClusteringAlgs;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import gnu.trove.TFloatIntHashMap;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.heatmap.ColorScheme;
import org.genepattern.menu.jfree.JFreeMenuBar;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * @author jgould
 */
public class LeadingEdgeAnalysis {

    private JPanel mainComponent;

    private LeadingEdgePanel leadingEdgePanel;

    private final static String LEADING_EDGE_MATRIX_KEY = "Leading Edge Matrix";

    private final static String GENE_SET_SIMILARITY_MATRIX_KEY = "Gene Set Similarity Matrix";

    private final static String GENE_HISTOGRAM_KEY = "Gene Histogram";

    private final static String JACQUARD_HISTOGRAM_KEY = "Jacquard Histogram of Gene Sets";

    private Dataset _morph(Dataset ds, RankedList rl) {
        Matrix m = new Matrix(ds.getNumRow(), ds.getNumCol());
        for (int r = 0; r < ds.getNumRow(); r++) {
            for (int c = 0; c < ds.getNumCol(); c++) {
                float score = rl.getScore(ds.getColumnName(c));
                float value = ds.getElement(r, c);
                if (value == 1) {
                    m.setElement(r, c, score);
                } else {
                    m.setElement(r, c, value); // ought to be zero
                }
            }
        }

        return new DefaultDataset(ds.getName(), m, ds.getRowNames(), ds
                .getColumnNames(), true, ds.getAnnot());
    }

    public void setResultDirectory(File file) {
        leadingEdgePanel.setResultDirectory(file);
    }

    public LeadingEdgeAnalysis(final GeneSetMatrix lev_gmx,
                               RankedList rankedList, final Frame parent) {
        GeneSet[] gsets = lev_gmx.getGeneSets();
        final BitSetDataset lev_bsd = new BitSetDataset(lev_gmx);
        final Dataset lev_ds = lev_bsd.toDataset(true, false);

        Dataset lev_ds_clustered = null;

        try {
            lev_ds_clustered = ClusteringAlgs.cluster(lev_ds)
                    .getDatasetOrdered();

        } catch (Throwable t) {
            lev_ds_clustered = lev_ds;
            t.printStackTrace();
        }

        final Dataset lev_ds_clustered_m = rankedList != null ? _morph(
                lev_ds_clustered, rankedList) : lev_ds_clustered;

        leadingEdgePanel = new LeadingEdgePanel(parent, null);
        ColorScheme cs = rankedList != null ? GPWrappers
                .createColorScheme_for_lev_with_score(lev_ds_clustered_m)
                : new ColorScheme() {

            public Color getColor(int row, int column) {
                return lev_ds_clustered_m.getElement(row, column) == 0 ? Color.white
                        : Color.yellow;
            }

            public void setDataset(IExpressionData d) {
            }

            public Component getLegend() {
                return null;
            }

        };

        leadingEdgePanel.setData(GPWrappers
                .createIExpressionData(lev_ds_clustered_m), false, cs);

        GeneSetStats stats = new GeneSetStats();
        GeneSetStats.RedStruc rs = stats.calcRedundancy(gsets, false);

        GeneSetSimilarityPanel geneSetSimilarityPanel = new GeneSetSimilarityPanel(
                parent);
        // reorder genesets so that they are in the same order as the clustered
        // gene sets
        GeneSet[] reorderedGeneSets = new GeneSet[gsets.length];
        Map geneSetName2Index = new HashMap();
        for (int i = 0; i < gsets.length; i++) {
            geneSetName2Index.put(gsets[i].getName(true), new Integer(i));
        }

        for (int i = 0; i < gsets.length; i++) {
            String geneSetName = lev_ds_clustered_m.getRowName(i);
            reorderedGeneSets[i] = gsets[((Integer) geneSetName2Index
                    .get(geneSetName)).intValue()];
        }

        geneSetSimilarityPanel.setGeneSets(reorderedGeneSets);

        JacquardHistogram jacquardHistogram = new JacquardHistogram();
        TFloatIntHashMap jacquardToOccurrencesMap = rs.jacquardDistrib;
        jacquardHistogram.setJacquardToOccurrencesMap(jacquardToOccurrencesMap);

        final GeneHistogram geneHistogram = new GeneHistogram();
        RankedList featureFrequency = rs.featureFreq;
        geneHistogram.setFeatureFrequency(featureFrequency,
                rankedList != null ? rankedList : null);

        ChartMouseListener listener = new ChartMouseListener() {

            public void chartMouseClicked(ChartMouseEvent event) {
                int mouseX = event.getTrigger().getX();
                int mouseY = event.getTrigger().getY();
                int index = geneHistogram.getXIndex(mouseX, mouseY);
                if (index != -1) {
                    String geneName = geneHistogram.getGeneName(index);
                    int columnIndex = lev_ds_clustered_m
                            .getColumnIndex(geneName);
                    leadingEdgePanel.getHeatMapComponent().getSampleTable()
                            .setColumnSelectionInterval(columnIndex,
                                    columnIndex);
                }
            }

            public void chartMouseMoved(ChartMouseEvent event) {
            }

        };

        geneHistogram.getChartPanel().addChartMouseListener(listener);

        JScrollPane leadingEdgeSP = new JScrollPane(leadingEdgePanel
                .getHeatMapComponent());
        leadingEdgeSP.setBorder(null);

        JScrollPane geneSetSP = new JScrollPane(geneSetSimilarityPanel
                .getHeatMapComponent());
        geneSetSP.setBorder(null);

        if (false) {
            JideSplitPane splitPane1 = createSplitPane(JideSplitPane.HORIZONTAL_SPLIT);
            splitPane1.addPane(leadingEdgeSP);
            splitPane1.addPane(geneSetSP);

            JideSplitPane splitPane2 = createSplitPane(JideSplitPane.HORIZONTAL_SPLIT);
            splitPane2.addPane(new JScrollPane(geneHistogram));
            splitPane2.addPane(new JScrollPane(jacquardHistogram));

            JideSplitPane splitPane3 = createSplitPane(JideSplitPane.VERTICAL_SPLIT);
            splitPane3.addPane(splitPane1);
            splitPane3.addPane(splitPane2);
        } else {
            mainComponent = new JPanel();
            UIManager.getDefaults().put("JideSplitPane.dividerSize",
                    new Integer(9));

            DefaultDockingManager manager = new DefaultDockingManager(null,
                    mainComponent) {
                public void activateFrame(String key) {
                    super.activateFrame(key);
                }
            };

            manager
                    .setInitSplitPriority(DefaultDockingManager.SPLIT_EAST_WEST_SOUTH_NORTH);
            manager.setShowWorkspace(false);

            manager.setRearrangable(false);
            manager.setAutohidable(false);
            manager.setFloatable(false);
            manager.setShowTitleBar(false);
            manager.getWorkspace().setAcceptDockableFrame(false);
            manager.setProportionalSplits(true);

            manager.beginLoadLayoutData();
            int width = parent.getWidth();
            int height = parent.getHeight();

            DockableFrame f = new DockableFrame(
                    LEADING_EDGE_MATRIX_KEY,
                    JideIconsFactory
                            .getImageIcon(JideIconsFactory.DockableFrame.FRAME1));
            f.getContentPane().add(leadingEdgeSP);
            f.setInitIndex(0);
            f.setInitSide(DockContext.DOCK_SIDE_NORTH);
            f.setPreferredSize(new Dimension(width / 2, height / 2));
            // f.setBorder(createBorder(LEADING_EDGE_MATRIX_KEY));

            f.setJMenuBar(leadingEdgePanel.getMenuBar());

            manager.addFrame(f);

            f = new DockableFrame(
                    GENE_SET_SIMILARITY_MATRIX_KEY,
                    JideIconsFactory
                            .getImageIcon(JideIconsFactory.DockableFrame.FRAME1));
            f.getContentPane().add(geneSetSP);
            f.setInitIndex(1);
            f.setInitSide(DockContext.DOCK_SIDE_NORTH);
            f.setPreferredSize(new Dimension(width / 2, height / 2));
            // f.setBorder(createBorder(GENE_SET_SIMILARITY_MATRIX_KEY));
            f.setJMenuBar(geneSetSimilarityPanel.getMenuBar());

            manager.addFrame(f);

            f = new DockableFrame(GENE_HISTOGRAM_KEY, JideIconsFactory
                    .getImageIcon(JideIconsFactory.DockableFrame.FRAME1));
            geneHistogram.setPreferredSize(new Dimension(100, 100));
            f.getContentPane().add(new JScrollPane(geneHistogram));
            f.setInitIndex(0);
            f.setInitSide(DockContext.DOCK_SIDE_SOUTH);
            f.setPreferredSize(new Dimension(width / 2, height / 2));
            // f.setBorder(createBorder(GENE_HISTOGRAM));
            JFreeMenuBar menuBar = new JFreeMenuBar(geneHistogram
                    .getChartPanel(), parent);
            f.setJMenuBar(menuBar);
            manager.addFrame(f);

            f = new DockableFrame(JACQUARD_HISTOGRAM_KEY, JideIconsFactory
                    .getImageIcon(JideIconsFactory.DockableFrame.FRAME1));
            jacquardHistogram.setPreferredSize(new Dimension(100, 100));
            f.getContentPane().add(new JScrollPane(jacquardHistogram));
            f.setInitIndex(1);
            f.setInitSide(DockContext.DOCK_SIDE_SOUTH);
            f.setPreferredSize(new Dimension(width / 2, height / 2));
            // f.setBorder(createBorder(JACQUARD_HISTOGRAM));
            menuBar = new JFreeMenuBar(jacquardHistogram.getChartPanel(),
                    parent);
            f.setJMenuBar(menuBar);
            manager.addFrame(f);

            manager.loadLayoutData();
        }

    }

    private Border createBorder(String key) {
        return BorderFactory.createCompoundBorder(BorderFactory
                .createLineBorder(Color.BLACK), BorderFactory
                .createTitledBorder(BorderFactory.createEmptyBorder(), key));
    }

    public Component getComponent() {
        return mainComponent;
    }

    private JideSplitPane createSplitPane(int split) {
        JideSplitPane splitPane1 = new JideSplitPane(split);
        splitPane1.setBorder(null);
        splitPane1.setProportionalLayout(true);
        return splitPane1;
    }

    public void saveImage() {

    }

    public void reset() {

    }

    public void displayOptions() {

    }
}
