package org.genepattern.gsea;

import com.jidesoft.swing.JideTabbedPane;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.alg.ComparatorFactory;
import edu.mit.broad.genome.alg.gsea.PValueCalculator;
import edu.mit.broad.genome.alg.gsea.PValueCalculatorImpls;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.DefaultGeneSetMatrix;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentScore;
import edu.mit.broad.genome.parsers.EdbFolderParser;
import edu.mit.broad.genome.reports.EnrichmentReports;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.tui.TaskManager;
import foxtrot.Task;
import foxtrot.Worker;
import org.genepattern.menu.PlotAction;
import org.genepattern.uiutil.OS;
import org.genepattern.uiutil.UIUtil;
import xtools.api.param.ParamSet;
import xtools.gsea.LeadingEdgeTool;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

/**
 * @author Joshua Gould
 * @author Aravind Subramanian
 */
public class LeadingEdgeWidget implements Widget {

    private static final Icon ICON = JarResources.getIcon("Lev16_b.gif");

    private EnrichmentDb edb;

    JideTabbedPane tabbedPane;

    /**
     * counts how many times analysis was run
     */
    static int runs = 0;

    private ViewAndSearchComponent viewAndSearchComponent;

    private JLabel positiveLabel;

    private JLabel negativeLabel;

    private JPanel phenotypePanel;

    /**
     * whether viewer is running in GenePattern
     */
    private boolean runningInGenePattern;

    /**
     * Used in GenePattern to use output directory for HTML report
     */
    private JFileChooser htmlReportDirChooser;

    /**
     * index in table of gene set
     */
    final static int GENE_SET_INDEX = 0;

    /**
     * index in table of score
     */
    final static int SCORE_INDEX = 2;

    static {
        // don't show accelerators in menus
        System.setProperty(PlotAction.SHOW_ACCELERATORS_PROPERTY, "false");
    }

    public LeadingEdgeWidget(EnrichmentDb edb) {

        if (edb == null) {
            throw new IllegalArgumentException("Param edb cannot be null");
        }

        runningInGenePattern = true;
        tabbedPane = new JideTabbedPane();
        tabbedPane.setHideOneTab(true);
        tabbedPane.setTabEditingAllowed(true);
        tabbedPane.setShowCloseButtonOnTab(true);
        this.edb = edb;
        init(edb);
        tabbedPane.addTab("GSEA Results", viewAndSearchComponent);
        tabbedPane.setTabClosableAt(0, false);

    }

    public LeadingEdgeWidget(JideTabbedPane tabbedPane, EnrichmentDb edb) {

        if (edb == null) {
            throw new IllegalArgumentException("Param edb cannot be null");
        }

        runningInGenePattern = false;
        this.tabbedPane = tabbedPane;
        UIUtil.setMessageHandler(new XToolsMessageHandler());
        this.edb = edb;
        init(edb);
        tabbedPane.addTab("GSEA Results", viewAndSearchComponent);
    }

    public JMenuBar getJMenuBar() {
        return EMPTY_MENU_BAR;
    }

    public JComponent getWindowUI() {
        return tabbedPane;
    }

    public JComponent getViewAndSearchComponent() {
        return viewAndSearchComponent;
    }

    public String getTitle() {
        return null;
    }

    public boolean allowClose() {
        return true;
    }

    public Icon getIcon() {
        return ICON;
    }

    private void init(EnrichmentDb edb) {

        positiveLabel = new JLabel(" ");
        positiveLabel.setForeground(Color.RED);

        String tcp = "na pos";
        String tcn = "na neg";
        if (edb.getTemplate() != null) {
            tcn = edb.getTemplate().getClassName(0);
            tcn = edb.getTemplate().getClassName(1);
        }

        positiveLabel.setText("positive phenotype: " + tcp + "   ");

        negativeLabel = new JLabel(" ");
        negativeLabel.setForeground(Color.BLUE);
        negativeLabel.setText("negative phenotype: " + tcn);

        phenotypePanel = new JPanel();
        phenotypePanel.add(positiveLabel);
        phenotypePanel.add(negativeLabel);


        final ActionListener runListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                runAnalysis();
            }
        };

        viewAndSearchComponent = new ViewAndSearchComponent(
                "Run leading edge analysis", new int[]{GENE_SET_INDEX},
                runListener, true, phenotypePanel);
        JButton buildhtmlReportBtn = new JButton("Build HTML Report",
                GuiHelper.ICON_START16);
        buildhtmlReportBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                buildHtmlReport();
            }
        });
        viewAndSearchComponent.getButtonPanel().add(buildhtmlReportBtn);
        viewAndSearchComponent.getTableSelectionListener().addComponent(
                buildhtmlReportBtn);

        viewAndSearchComponent.setTableModel(createTableModel());
        viewAndSearchComponent.getTable().setDefaultRenderer(String.class,
                new ESRenderer(GENE_SET_INDEX, SCORE_INDEX));

    }

    private void buildHtmlReport() {
        File outputDir = edb.getEdbDir();
        if (runningInGenePattern) {
            if (htmlReportDirChooser == null) {
                htmlReportDirChooser = new JFileChooser();
                htmlReportDirChooser.setDialogTitle("Choose output directory");
                htmlReportDirChooser
                        .setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            }
            if (htmlReportDirChooser.showOpenDialog(tabbedPane
                    .getTopLevelAncestor()) != JFileChooser.APPROVE_OPTION) {
                return;
            }
            outputDir = htmlReportDirChooser.getSelectedFile();
        }
        final File _outputDir = outputDir;
        tabbedPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        try {
            Worker.post(new Task() {
                public Object run() throws Exception {
                    LeadingEdgeTool tool = new LeadingEdgeTool();
                    ParamSet set = tool.getParamSet();
                    if (runningInGenePattern) {
                        set.getAnalysisDirParam().setValue(_outputDir);
                    }
                    tool.fGseaResultDirParam.setValue(edb.getEdbDir());
                    tool.fGeneSetNamesParam.setValue(viewAndSearchComponent
                            .getSelectedColumnArray(GENE_SET_INDEX));
                    if (!runningInGenePattern) {
                        TaskManager.getInstance().run(tool, tool.getParamSet(),
                                Thread.MIN_PRIORITY);
                    } else {
                        tool.execute();
                    }
                    return null;
                }
            });
        } catch (Throwable t) {
            t.printStackTrace();
            tabbedPane.setCursor(Cursor
                    .getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            UIUtil.showErrorDialog(tabbedPane.getTopLevelAncestor(),
                    "An error occurred while building the HTML report");
        }
        tabbedPane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    protected TableModel createTableModel() {
        return new MyTableModel(edb);
    }

    static class MyTableModel extends AbstractTableModel {
        String[] columnNames = {"Gene Set", "Size", "ES", "NES", "NOM p-val",
                "FDR q-val", "FWER p-val", "Rank at Max", "Leading Edge"};

        Class[] columnClasses = {String.class, Integer.class, Float.class,
                Float.class, Float.class, Float.class, Float.class,
                Integer.class, String.class};

        EnrichmentResult[] ers;

        public MyTableModel(EnrichmentDb edb_original) {
            String normModeName = "meandiv"; // XXX hard coded
            final PValueCalculator pvc = new PValueCalculatorImpls.GseaImpl(
                    normModeName);
            final EnrichmentResult[] results = pvc
                    .calcNPValuesAndFDR(edb_original.getResults());
            final EnrichmentDb edb = edb_original.cloneDeep(results);
            ers = edb
                    .getResults(new ComparatorFactory.EnrichmentResultByNESComparator(
                            Order.DESCENDING));
        }

        public int getRowCount() {
            return ers.length;
        }

        public int getColumnCount() {
            return columnNames.length;
        }

        public Class getColumnClass(int j) {
            return columnClasses[j];
        }

        public String getColumnName(int j) {
            return columnNames[j];
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            GeneSet gset = ers[rowIndex].getGeneSet();
            EnrichmentScore score = ers[rowIndex].getScore();
            switch (columnIndex) {
                case 0:
                    return gset.getName(true);
                case 1:
                    return new Integer(gset.getNumMembers());
                case 2:
                    return new Float(score.getES());
                case 3:
                    return new Float(score.getNES());
                case 4:
                    return new Float(score.getNP());
                case 5:
                    return new Float(score.getFDR());
                case 6:
                    return new Float(score.getFWER());
                case 7:
                    return new Integer(ers[rowIndex].getSignal().getRankAtMax());
                case 8:
                    return EnrichmentReports.getLeadingEdge(ers[rowIndex]);
            }
            return null;

        }
    }

    /**
     * GramTableCellRenderer for table headers. Headers in bold font -- looks
     * like excel headers.
     */
    class ESRenderer extends DefaultTableCellRenderer {

        private int nameColumnIndex;

        private int scoreColumnIndex;

        ESRenderer(int nameColumnIndex, int scoreColumnIndex) {
            this.nameColumnIndex = nameColumnIndex;
            this.scoreColumnIndex = scoreColumnIndex;
        }

        public Component getTableCellRendererComponent(JTable table,
                                                       Object value, boolean isSelected, boolean hasFocus, int row,
                                                       int col) {

            row = viewAndSearchComponent.getTable().getActualRowAt(row);

            super.getTableCellRendererComponent(table, value, isSelected,
                    hasFocus, row, col);

            if (col == nameColumnIndex) {
                float es = ((Float) table.getValueAt(row, scoreColumnIndex))
                        .floatValue();
                if (XMath.isPositive(es)) {
                    this.setForeground(Color.RED);
                } else {
                    this.setForeground(Color.BLUE);
                }
                this.setText(value.toString());
            } else {
                this.setForeground(Color.BLACK);
            }

            return this;
        }
    }

    private void runAnalysis() {
        tabbedPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        try {
            LeadingEdgeAnalysis analysis = (LeadingEdgeAnalysis) Worker
                    .post(new Task() {
                        public Object run() throws Exception {
                            return runAnalysis(viewAndSearchComponent
                                    .getSelectedColumnArray(GENE_SET_INDEX));
                        }
                    });
            tabbedPane.addTab("Leading Edge Analysis-" + runs, analysis
                    .getComponent());
            tabbedPane.setSelectedComponent(analysis.getComponent());
            // tabbedPane.setTabClosableAt(tabbedPane.getTabCount() - 1, true);
            tabbedPane.setCursor(Cursor
                    .getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        } catch (Throwable t) {
            t.printStackTrace();
            tabbedPane.setCursor(Cursor
                    .getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            UIUtil.showErrorDialog(tabbedPane.getTopLevelAncestor(),
                    "An error occurred while running leading edge analysis");
        }
    }

    private LeadingEdgeAnalysis runAnalysis(String[] gsetNames) {
        final EnrichmentResult[] results = new EnrichmentResult[gsetNames.length];
        final GeneSet[] gsets = new GeneSet[results.length];
        for (int r = 0; r < gsetNames.length; r++) {
            results[r] = edb.getResultForGeneSet(gsetNames[r]);
            gsets[r] = results[r].getSignal().getAsGeneSet();
        }
        final GeneSetMatrix lev_gmx = new DefaultGeneSetMatrix(
                "leading_edge_matrix_for_" + edb.getName(), gsets);
        LeadingEdgeAnalysis analysis = new LeadingEdgeAnalysis(lev_gmx, edb
                .getRankedList(), (Frame) tabbedPane.getTopLevelAncestor());
        analysis.setResultDirectory(edb.getEdbDir());
        runs++;
        return analysis;
    }

    public static void main(String[] args) { // for running using GenePattern
        com.jidesoft.utils.Lm.verifyLicense(
                "Broad Institute of MIT and Harvard",
                "Gene set  enrichment analysis java desktop application",
                "YSjBO6OJfF9WbavzI73Jt1HgDI4x9L21");
        OS.setLookAndFeel();

        String zipFile = args[0];
        File dir = null;
        try {
            dir = File.createTempFile("gsea", null);
        } catch (IOException e) {
            e.printStackTrace();
            UIUtil.showMessageDialog(null,
                    "An error occurred while reading the input file");
            System.exit(0);
        }
        dir.delete();
        dir.mkdir();
        final File f = dir;
        Runtime.getRuntime().addShutdownHook(new Thread() {
            public void run() {
                //org.genepattern.io.Util.deleteRecursive(f, null);

            }
        });
        /*
        if (args.length == 1) {
            ArchiveUtil.unzipRecursive(new File(zipFile), dir);
        }
        */
        String title = "Leading Edge Viewer - " + new File(zipFile).getName();

        EnrichmentDb edb = null;
        try {
            EdbFolderParser folderParser = new EdbFolderParser();
            folderParser.setSilentMode(true);
            edb = (EnrichmentDb) folderParser.parse(dir.getCanonicalPath(),
                    (InputStream) null).get(0);

        } catch (Exception e1) {
            e1.printStackTrace();
            UIUtil.showMessageDialog(null,
                    "An error occurred while reading the input file");
            System.exit(0);
        }
        JFrame frame = new JFrame();
        frame.setTitle(title);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        LeadingEdgeWidget widget = new LeadingEdgeWidget(edb);
        frame.getContentPane().add(widget.tabbedPane);
        UIUtil.sizeToScreen(frame);
        frame.setVisible(true);

    }

}
