/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.heatmap.image;

import com.sun.media.jai.codec.ImageEncodeParam;
import com.sun.media.jai.codec.JPEGEncodeParam;
import org.genepattern.data.expr.ExpressionConstants;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.heatmap.RowColorScheme;

import javax.media.jai.JAI;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * This class is used to draw a heat map.
 *
 * @author Joshua Gould
 */
public class HeatMap {

    public static int LOG_SCALE = 0;

    public static int LINEAR_SCALE = 1;

    public static int COLOR_RESPONSE_ROW = 0;

    public static int COLOR_RESPONSE_GLOBAL = 1;

    String[] rowDescriptions;

    IExpressionData data;

    /**
     * width and height of one 'cell' in the heatmap
     */
    Dimension elementSize = new Dimension(10, 10);

    /**
     * width of 'cells', gene names, left inset, gene class label area
     */
    int contentWidth = 0;

    /**
     * height of this heatmap
     */
    int height = 0;

    boolean antiAliasing = true;

    String fontFamilyName = "monospaced";

    int fontStyle = Font.PLAIN;

    private HeatMapHeader header;

    private int geneNameWidth;

    private int leftBorder = 0;

    private Font font;

    /**
     * The max width of the gene annotations
     */
    int maxGeneAnnotationsWidth = 0;

    /**
     * The number of pixels after the gene name and before the gene annotation
     */
    int spaceAfterGeneNames = 10;

    /** if not null, draw a filled square to the left of the row name */

    /**
     * width of filled square
     */
    private int annotationWidth = 6;

    private int numFeatureClasses;

    int numSampleClasses;

    private FeatureAnnotator featureAnnotator;

    SampleAnnotator sampleAnnotator;

    private int[] annotationWidths;

    DisplaySettings ds = new DisplaySettings();

    /**
     * Constructs an <code>HeatMap</code> with specified data
     *
     * @param data Description of the Parameter
     */
    public HeatMap(IExpressionData data, Color[] colorMap) {
        this.data = data;
        this.rowDescriptions = new String[data.getRowCount()];
        for (int i = 0, rows = data.getRowCount(); i < rows; i++) {
            rowDescriptions[i] = data.getRowMetadata(i,
                    ExpressionConstants.DESC);
            if (rowDescriptions[i] == null) {
                rowDescriptions[i] = "";
            }
        }

        this.header = new HeatMapHeader(this);
        ds.colorConverter = RowColorScheme.getRowInstance(colorMap);
    }

    public static Color createColor(String triplet) {
        String[] rgb = triplet.split(":");
        if (rgb.length != 3) {
            throw new IllegalArgumentException("Invalid rgb triplet " + triplet);
        }
        int r = 0, g = 0, b = 0;
        try {
            r = Integer.parseInt(rgb[0]);
        } catch (NumberFormatException nfe) {
            throw new IllegalArgumentException(
                    "Red component is not an integer " + triplet);
        }
        try {
            g = Integer.parseInt(rgb[1]);
        } catch (NumberFormatException nfe) {
            throw new IllegalArgumentException(
                    "Green component is not an integer " + triplet);
        }
        try {
            b = Integer.parseInt(rgb[2]);
        } catch (NumberFormatException nfe) {
            throw new IllegalArgumentException(
                    "Blue component is not an integer " + triplet);
        }
        return new Color(r, g, b);
    }

    public static Color[] parseColorMap(String fileName) {
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(fileName));
            String s = null;
            List colors = new ArrayList();
            while ((s = br.readLine()) != null) {
                if (s.trim().equals("")) {
                    continue;
                }
                Color c = createColor(s);
                colors.add(c);
            }
            return (Color[]) colors.toArray(new Color[0]);
        } catch (IOException ioe) {
            return null;
        } finally {
            if (br != null) {
                try {
                    br.close();
                } catch (IOException e) {
                }
            }
        }
    }

    public static BufferedImage createImage(IExpressionData data,
                                            DisplaySettings ds, SampleAnnotator sa, FeatureAnnotator fa) {
        HeatMap heatMap = createHeatMap(data, ds, fa, sa);
        updateHeatMapSize(heatMap, "png");

        BufferedImage bi = heatMap.snapshot();
        return bi;
    }

    /**
     * @param rows    rows in dataset
     * @param columns columns in dataset
     * @param image   the heatmap image
     * @param ds      display settings
     * @return image map
     */
    public static String createHtmlImageMap(int rows, int columns,
                                            HeatMapImage image, DisplaySettings ds, LinkGenerator lg) {
        int leftBorder = image.leftBorder;
        int headerHeight = image.headerHeight;
        int rowSize = ds.rowSize;
        int columnSize = ds.columnSize;
        StringBuffer sb = new StringBuffer();
        sb.append("<map name=\"" + lg.getMapName() + "\">");
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                int x = j * columnSize + leftBorder;
                int y = i * rowSize + headerHeight;
                int x2 = x + columnSize;
                int y2 = y + rowSize;
                String url = lg.getUrl(i, j);
                sb.append("<area shape=\"rect\" coords=\"" + x + "," + y + ","
                        + x2 + "," + y2 + "\" href=\"" + url + "\">");
            }
        }

        sb.append("</map>");
        return sb.toString();
    }

    public static HeatMapImage createImage2(IExpressionData data,
                                            DisplaySettings ds, SampleAnnotator sa, FeatureAnnotator fa) {
        HeatMap heatMap = createHeatMap(data, ds, fa, sa);
        updateHeatMapSize(heatMap, "png");

        BufferedImage bi = heatMap.snapshot();
        HeatMapImage hmi = new HeatMapImage();
        hmi.image = bi;
        hmi.headerHeight = heatMap.header.height;
        hmi.leftBorder = heatMap.leftBorder;
        return hmi;
    }

    public static class HeatMapImage {
        public BufferedImage image;

        public int headerHeight;

        public int leftBorder;
    }

    private static HeatMap createHeatMap(IExpressionData data,
                                         DisplaySettings ds, FeatureAnnotator fa, SampleAnnotator sa) {
        HeatMap heatMap = new HeatMap(data, RowColorScheme.getDefaultColorMap());
        heatMap.setDisplaySettings(ds);
        heatMap.setSampleAnnotator(sa);
        heatMap.setFeatureAnnotator(fa);
        return heatMap;
    }

    private static void updateHeatMapSize(HeatMap heatMap,
                                          String outputFileFormat) {
        Graphics2D g = null;

        BufferedImage bi = new BufferedImage(100, 100,
                BufferedImage.TYPE_3BYTE_BGR);
        g = bi.createGraphics();

        heatMap.updateSize(g);
        heatMap.header.updateSize(heatMap.contentWidth,
                heatMap.elementSize.width, g);

        g.dispose();
    }

    public static void saveImage(IExpressionData data, DisplaySettings ds,
                                 Map featureName2Colors, Map sampleName2Colors,
                                 String outputFileName, String outputFileFormat) throws IOException {

        saveImage(data, ds, mapToFeatureAnnotator(featureName2Colors),
                mapToSampleAnnotator(sampleName2Colors), outputFileName,
                outputFileFormat);
    }

    public static void saveImage(IExpressionData data, DisplaySettings ds,
                                 FeatureAnnotator fa, SampleAnnotator sa, String outputFileName,
                                 String outputFileFormat) throws IOException {

        HeatMap heatMap = createHeatMap(data, ds, fa, sa);
        updateHeatMapSize(heatMap, outputFileFormat);
        writeImageToFile(heatMap, outputFileName, outputFileFormat);
    }

    private static SampleAnnotator mapToSampleAnnotator(
            final Map sampleName2Colors) {
        return new SampleAnnotator() {

            public Color getPhenotypeColor(String sampleName) {
                return null;
            }

            public boolean hasPhenotypeColors() {
                return false;
            }

            public String getLabel(int i) {
                return null;
            }

            public List getColors(String sampleName) {
                return (List) sampleName2Colors.get(sampleName);
            }

        };
    }

    private static FeatureAnnotator mapToFeatureAnnotator(
            final Map featureName2Colors) {
        return new FeatureAnnotator() {

            public String getAnnotation(String feature, int j) {
                return null;
            }

            public int getColumnCount() {
                return 0;
            }

            public List getColors(String featureName) {
                return (List) featureName2Colors.get(featureName);
            }

        };
    }

    private void setDisplaySettings(DisplaySettings ds) {
        this.ds = ds;
        setElementSize(ds.rowSize, ds.columnSize);
        header.drawSampleNames = ds.drawColumnNames;
        ds.colorConverter.setDataset(data);
    }

    private void setSampleAnnotator(SampleAnnotator annotator) {
        this.sampleAnnotator = annotator;
        if (sampleAnnotator != null) {
            for (int j = 0; j < data.getColumnCount(); j++) {
                List colors = sampleAnnotator.getColors(data.getColumnName(j));
                if (colors != null) {
                    numSampleClasses = Math
                            .max(numSampleClasses, colors.size());
                }
            }
        }
    }

    private void setFeatureAnnotator(FeatureAnnotator annotator) {
        this.featureAnnotator = annotator;
        if (featureAnnotator != null) {
            for (int j = 0; j < data.getRowCount(); j++) {
                List colors = featureAnnotator.getColors(data.getRowName(j));
                if (colors != null) {
                    numFeatureClasses = Math.max(numFeatureClasses, colors
                            .size());
                }
            }
        }
    }

    private static void writeImageToFile(HeatMap heatMap,
                                         String outputFileName, String outputFileFormat) throws IOException {

        if (outputFileFormat.equals("jpeg")) {
            if (!outputFileName.toLowerCase().endsWith(".jpg")
                    && !outputFileName.toLowerCase().endsWith(".jpeg")) {
                outputFileName += ".jpg";
            }
        } else if (outputFileFormat.equals("png")) {
            if (!outputFileName.toLowerCase().endsWith(".png")) {
                outputFileName += ".png";
            }
        } else if (outputFileFormat.equals("tiff")) {
            if (!outputFileName.toLowerCase().endsWith(".tiff")) {
                outputFileName += ".tiff";
            }
        } else if (outputFileFormat.equals("bmp")) {
            if (!outputFileName.toLowerCase().endsWith(".bmp")) {
                outputFileName += ".bmp";
            }
        } else if (outputFileFormat.equals("eps")) {
            if (!outputFileName.toLowerCase().endsWith(".eps")
                    && !outputFileName.toLowerCase().endsWith(".ps")) {
                outputFileName += ".eps";
            }
        }

        /*
        Graphics2D epsGraphics = null;
        if (outputFileFormat.equals("eps")) {
            epsGraphics = new EpsGraphics2D();
        }

        if (!outputFileFormat.equals("eps")) {

            ImageEncodeParam fParam = null;
            if (outputFileFormat.equals("jpeg")) {
                JPEGEncodeParam jpegParam = new JPEGEncodeParam();
                jpegParam.setQuality(1.0f);
                fParam = jpegParam;
            } else if (outputFileFormat.equals("png")) {
                fParam = new com.sun.media.jai.codec.PNGEncodeParam.RGB();
            } else if (outputFileFormat.equals("tiff")) {
                com.sun.media.jai.codec.TIFFEncodeParam param = new com.sun.media.jai.codec.TIFFEncodeParam();
                param
                        .setCompression(com.sun.media.jai.codec.TIFFEncodeParam.COMPRESSION_NONE);
                fParam = param;
            } else if (outputFileFormat.equals("bmp")) {
                fParam = new com.sun.media.jai.codec.BMPEncodeParam();
            } else {
                throw new IllegalArgumentException("Unknown output file format");
            }
            // ImageIO.write(heatMap.snapshot(), "jpeg", new
            // File(outputFileName));
            JAI.create("filestore", heatMap.snapshot(), outputFileName,
                    outputFileFormat, fParam);

        } else {
            epsGraphics.setFont(heatMap.header.font);
            heatMap.header.draw(epsGraphics);
            epsGraphics.translate(0, heatMap.header.height);
            epsGraphics.setFont(heatMap.font);
            heatMap.draw(epsGraphics);
            // ((EpsGraphics2D) epsGraphics).flush();
            // ((EpsGraphics2D) epsGraphics).close();

            String output = epsGraphics.toString();
            try {
                java.io.PrintWriter pw = new java.io.PrintWriter(
                        new java.io.FileWriter(outputFileName));
                pw.print(output);
                pw.close();
            } catch (IOException ioe) {
                throw new IOException(
                        "An error occurred while saving the postscript file.\nCause: "
                                + ioe.getMessage());
            }
            epsGraphics.dispose();

        }
        */

        ImageEncodeParam fParam = null;
        if (outputFileFormat.equals("jpeg")) {
            JPEGEncodeParam jpegParam = new JPEGEncodeParam();
            jpegParam.setQuality(1.0f);
            fParam = jpegParam;
        } else if (outputFileFormat.equals("png")) {
            fParam = new com.sun.media.jai.codec.PNGEncodeParam.RGB();
        } else if (outputFileFormat.equals("tiff")) {
            com.sun.media.jai.codec.TIFFEncodeParam param = new com.sun.media.jai.codec.TIFFEncodeParam();
            param
                    .setCompression(com.sun.media.jai.codec.TIFFEncodeParam.COMPRESSION_NONE);
            fParam = param;
        } else if (outputFileFormat.equals("bmp")) {
            fParam = new com.sun.media.jai.codec.BMPEncodeParam();
        } else {
            throw new IllegalArgumentException("Unknown output file format");
        }
        // ImageIO.write(heatMap.snapshot(), "jpeg", new
        // File(outputFileName));
        JAI.create("filestore", heatMap.snapshot(), outputFileName,
                outputFileFormat, fParam);
    }

    void draw(Graphics2D g2) {
        final int samples = data.getColumnCount();
        int left = 0;
        int right = samples;
        int top = 0;
        int bottom = data.getRowCount();

        // draw rectangles
        for (int row = top; row < bottom; row++) {
            for (int column = left; column < right; column++) {
                int x = column * elementSize.width + leftBorder;
                int y = row * elementSize.height;
                g2.setColor(ds.colorConverter.getColor(row, column));
                g2.fillRect(x, y, elementSize.width, elementSize.height);
            }
        }
        int expWidth = samples * this.elementSize.width + 5;

        if (featureAnnotator != null) { // draw color bars
            for (int row = 0; row < data.getRowCount(); row++) {
                List colors = featureAnnotator.getColors(data.getRowName(row));
                if (colors != null) {
                    for (int j = 0; j < colors.size(); j++) {
                        g2.setColor((Color) colors.get(j));
                        g2.fillRect(
                                annotationWidth * j + expWidth + leftBorder,
                                row * elementSize.height, annotationWidth,
                                elementSize.height);
                    }
                }

            }

        }

        if (this.antiAliasing) {
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_OFF);
            g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                    RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        }

        // draw gene ids
        FontMetrics fm = g2.getFontMetrics();
        int uniqX = elementSize.width * samples + 10;

        if (featureAnnotator != null) {
            uniqX += annotationWidth * numFeatureClasses;
        }

        if (this.ds.drawRowNames || this.ds.drawRowDescriptions) {
            if (right >= samples) {
                g2.setColor(Color.black);

                for (int row = top; row < bottom; row++) {
                    // int annY = (row + 1) * elementSize.height -
                    // fm.getDescent();
                    int annY = row * elementSize.height + fm.getAscent();
                    if (this.ds.drawRowNames) {
                        String label = data.getRowName(row);
                        g2.drawString(label, uniqX + leftBorder, annY);
                    }
                    if (ds.drawRowDescriptions) {
                        int geneAnnotationX = uniqX + leftBorder
                                + geneNameWidth;
                        if (ds.drawRowNames) {
                            geneAnnotationX += spaceAfterGeneNames;
                        }
                        String annot = rowDescriptions[row];
                        if (annot != null) {
                            g2.drawString(annot, geneAnnotationX, annY);
                        }

                    }
                }
            }
        }

        if (featureAnnotator != null) {
            int annotationStartX = uniqX + leftBorder;
            g2.setColor(Color.BLACK);
            if (ds.drawRowNames) {
                annotationStartX += geneNameWidth + spaceAfterGeneNames;
            }
            if (ds.drawRowDescriptions) {
                annotationStartX += maxGeneAnnotationsWidth
                        + spaceAfterGeneNames;
            }
            for (int i = 0, rows = data.getRowCount(); i < rows; i++) {
                String rowName = data.getRowName(i);
                int annY = i * elementSize.height + fm.getAscent();
                for (int j = 0, cols = featureAnnotator.getColumnCount(); j < cols; j++) {
                    String s = featureAnnotator.getAnnotation(rowName, j);
                    if (s != null) {
                        int x = annotationStartX + 10 * j;
                        if (j > 0) {
                            x += annotationWidths[j - 1];
                        }
                        g2.drawString(s, x, annY);
                    }
                }

            }
        }

        if (ds.drawGrid || ds.showFeatureGridLines || ds.showSampleGridLines) {
            g2.setColor(ds.gridLinesColor);
            int leftx = left * elementSize.width + leftBorder;

            int rightx = right * elementSize.width + leftBorder; // increase
            // if
            // drawing
            // border to
            // row name
            if (!ds.drawGrid && ds.showFeatureGridLines) {
                leftx = rightx;
            }
            if (ds.showFeatureGridLines) {
                rightx = contentWidth;
            }
            if (ds.drawGrid || ds.showFeatureGridLines) {
                for (int row = top; row <= bottom; row++) { // draw horizontal
                    // lines
                    int y = row * elementSize.height;
                    if (ds.upperTriangular) {
                        int leftDiag = row * elementSize.width + leftBorder;
                        g2.drawLine(leftDiag, y, rightx, y);
                    } else {
                        g2.drawLine(leftx, y, rightx, y);
                    }
                }
            }

            int topY = 0;
            int bottomy = bottom * elementSize.height;

            if (ds.showSampleGridLines && ds.drawGrid) {
                g2.translate(0, -header.height);
                bottomy = height + header.height;
            } else if (ds.showSampleGridLines && !ds.drawGrid) {
                g2.translate(0, -header.height);
                bottomy = header.height;
            }

            if (ds.drawGrid || ds.showSampleGridLines) {
                for (int column = left; column <= right; column++) {
                    int x = column * elementSize.width + leftBorder;
                    if (ds.upperTriangular) {
                        int bottomDiag = elementSize.height * column;
                        g2.drawLine(x, topY, x, bottomDiag);
                    } else {
                        g2.drawLine(x, topY, x, bottomy);
                    }
                }
            }

        }
    }

    public BufferedImage snapshot() {
        int headerHeight = header.height;
        BufferedImage bi = new BufferedImage(contentWidth, height
                + headerHeight, BufferedImage.TYPE_3BYTE_BGR);

        Graphics2D g2 = bi.createGraphics();
        g2.setColor(Color.white);
        g2.fillRect(0, 0, bi.getWidth(), bi.getHeight());
        g2.setColor(Color.black);
        g2.setFont(header.font);
        header.draw(g2);
        g2.translate(0, headerHeight);
        g2.setFont(this.font);
        this.draw(g2);
        g2.dispose();
        return bi;
    }

    /**
     * Updates the size of this heatmap
     *
     * @param g Description of the Parameter
     */
    void updateSize(Graphics2D g) {

        int fontHeight = Math.min(14, elementSize.height);
        font = new Font(fontFamilyName, fontStyle, fontHeight);
        g.setFont(font);
        // setFont(font);
        int width = elementSize.width * data.getColumnCount() + 1 + leftBorder;
        if (ds.drawRowNames) {
            this.geneNameWidth = getMaxGeneNamesWidth(g);
            width += 20 + this.geneNameWidth;
        }

        if (ds.drawRowDescriptions) {
            this.maxGeneAnnotationsWidth = getMaxGeneDescriptionsWidth(g);
            if (ds.drawRowNames) {
                width += spaceAfterGeneNames;
            }
            width += this.maxGeneAnnotationsWidth;
        }

        if (featureAnnotator != null) {
            width += annotationWidth * numFeatureClasses;
            annotationWidths = getAnnotationsWidth(g);
            for (int i = 0; i < annotationWidths.length; i++) {
                width += annotationWidths[i] + 10;
            }
        }

        this.contentWidth = width;
        this.height = elementSize.height * data.getRowCount() + 1;
        if (!ds.drawHeatMapElements) {
            height = 0;
        }
    }

    private int[] getAnnotationsWidth(Graphics2D g) {
        FontMetrics fm = g.getFontMetrics();
        int[] widths = new int[featureAnnotator.getColumnCount()];
        for (int i = 0, rows = data.getRowCount(); i < rows; i++) {
            String rowName = data.getRowName(i);
            for (int j = 0, cols = featureAnnotator.getColumnCount(); j < cols; j++) {
                String annot = featureAnnotator.getAnnotation(rowName, j);
                if (annot != null) {
                    widths[j] = Math.max(widths[j], fm.stringWidth(annot));
                }
            }
        }
        return widths;

    }

    public void setShowSampleNames(boolean b) {
        header.setShowSampleNames(b);
    }

    public void setShowGeneNames(boolean b) {
        ds.drawRowNames = b;
    }

    public void setElementSize(int width, int height) {
        elementSize.width = width;
        elementSize.height = height;
    }

    public void setShowGridLines(boolean b) {
        ds.showFeatureGridLines = b;
    }

    public void setGridLinesColor(Color c) {
        if (c == null) {
            throw new IllegalArgumentException(
                    "Grid lines color can not be null.");
        }
        ds.gridLinesColor = c;
    }

    int getMaxGeneNamesWidth(Graphics2D g) {
        return getMaxWidth(g, true);
    }

    int getMaxGeneDescriptionsWidth(Graphics2D g) {
        return getMaxWidth(g, false);
    }

    /**
     * Returns max width of annotation strings.
     *
     * @param g Description of the Parameter
     * @return The maxWidth value
     */
    int getMaxWidth(Graphics2D g, boolean geneNames) {
        if (g == null) {
            return 0;
        }
        if (antiAliasing) {
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_OFF);
            g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                    RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        }
        FontMetrics fm = g.getFontMetrics();
        int max = 0;
        String str;
        for (int i = 0; i < data.getRowCount(); i++) {
            if (geneNames) {
                str = data.getRowName(i);
            } else {
                str = rowDescriptions[i];
            }
            if (str != null) {
                max = Math.max(max, fm.stringWidth(str));
            }
        }
        return max;
    }

    int getTopIndex(int top) {
        if (top < 0) {
            return 0;
        }
        return top / elementSize.height;
    }

    int getLeftIndex(int left) {
        if (left < leftBorder) {
            return 0;
        }
        return (left - leftBorder) / elementSize.width;
    }

    int getRightIndex(int right, int limit) {
        if (right < 0) {
            return 0;
        }
        int result = right / elementSize.width + 1;
        return result > limit ? limit : result;
    }

    int getBottomIndex(int bottom, int limit) {
        if (bottom < 0) {
            return 0;
        }
        int result = bottom / elementSize.height + 1;
        return result > limit ? limit : result;
    }

    public int getLeftBorder() {
        return leftBorder;
    }

    public void setLeftBorder(int leftBorder) {
        this.leftBorder = leftBorder;
    }

}
