/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.io;

import org.genepattern.data.expr.ExpressionData;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.data.matrix.ClassVector;
import org.genepattern.data.matrix.IClassVector;
import org.genepattern.io.expr.*;
import org.genepattern.io.expr.cls.ClsReader;
import org.genepattern.io.expr.cls.ClsWriter;
import org.genepattern.io.expr.gct.GctParser;
import org.genepattern.io.expr.gct.GctWriter;
import org.genepattern.io.expr.odf.OdfDatasetWriter;
import org.genepattern.io.expr.odf.OdfParserAdapter;
import org.genepattern.io.expr.res.ResParser;
import org.genepattern.io.expr.res.ResWriter;
import org.genepattern.io.expr.snp.SnpParser;
import org.genepattern.io.expr.snp.SnpWriter;

import java.io.*;
import java.util.*;

/**
 * A class containing static convenience methods for reading and writing data
 *
 * @author Joshua Gould
 */
public class IOUtil {
    static Map suffix2ExpressionReaders;

    static ClsReader clsReader = new ClsReader();

    static Map formatNameToExpressionWriterMap;

    private static boolean debug = false;

    private static FeatureListReader featureListReader = new FeatureListReader();

    private IOUtil() {
    }

    /**
     * Reads the cls document at the given pathname
     *
     * @param pathname The pathname string
     * @return The class vector
     * @throws IOException    If an error occurs while reading from the file
     * @throws ParseException If there is a problem with the data
     */
    public static ClassVector readCls(String pathname) throws IOException,
            ParseException {
        return clsReader.read(pathname);
    }

    /**
     * Gets a list of features at the given file pathname string
     *
     * @param pathname The pathname string
     * @return the feature list
     * @throws IOException If an error occurs while reading from the file
     */
    public static List readFeatureList(String pathname) throws IOException {
        return featureListReader.read(pathname);
    }

    /**
     * Writes the given feature list to a file
     *
     * @param features The features
     * @param pathname A pathname string
     * @throws IOException If an error occurs while saving the data
     */
    public static void writeFeatureList(String[] features, String pathname)
            throws IOException {
        PrintWriter pw = null;
        try {
            pw = new PrintWriter(new FileWriter(pathname));
            for (int i = 0; i < features.length; i++) {
                pw.println(features[i]);
            }

        } finally {
            if (pw != null) {
                pw.close();
            }
        }

    }

    /**
     * Reads the expression data at the given pathname. The type of the returned
     * object is determined by the expressionDataCreator argument.
     *
     * @param pathname              The file pathname
     * @param expressionDataCreator The expression data creator
     * @return An object containing the expression data
     * @throws IOException    If an error occurs while reading from the file
     * @throws ParseException If there is a problem with the data
     */
    public static Object readExpressionData(String pathname,
                                            IExpressionDataCreator expressionDataCreator) throws IOException,
            ParseException {
        IExpressionDataParser parser = getExpressionParser(pathname);
        return ReaderUtil.read(parser, pathname, expressionDataCreator);
    }

    /**
     * Reads the expression data at the given pathname and returns a new
     * <tt>ExpressionData</tt> instance.
     *
     * @param pathname The file pathname
     * @return An <tt>ExpressionData</tt> instance
     * @throws IOException    If an error occurs while reading from the file
     * @throws ParseException If there is a problem with the data
     */
    public static ExpressionData readExpressionData(String pathname)
            throws IOException, ParseException {
        return (ExpressionData) readExpressionData(pathname,
                new ExpressionDataCreator());
    }

    /**
     * Reads the expression data at the given input stream and returns a new
     * <tt>ExpressionData</tt> instance.
     *
     * @param filename The filename. The filename extension is used to deduce the
     *                 type of document at the given input stream
     * @param is       The input stream. The stream should be closed by the caller.
     * @return An <tt>ExpressionData</tt> instance
     * @throws IOException    If an error occurs while reading from the file
     * @throws ParseException If there is a problem with the data
     */
    public static ExpressionData readExpressionData(String filename,
                                                    InputStream is) throws IOException, ParseException {
        IExpressionDataParser parser = getExpressionReader(filename, false);
        if (parser == null) {
            return null;
        }
        return (ExpressionData) ReaderUtil.read(parser, is,
                new ExpressionDataCreator());

    }

    /**
     * Writes the given class vector to a file
     *
     * @param cv                 The class vector
     * @param pathname           A pathname string
     * @param checkFileExtension Whether the correct file extension will be added to the
     *                           pathname if it is not present.
     * @return The pathname that the data was saved to
     * @throws IOException If an error occurs while saving the data
     */
    public static String writeCls(IClassVector cv, String pathname,
                                  boolean checkFileExtension) throws IOException {
        FileOutputStream fos = null;
        try {
            ClsWriter writer = new ClsWriter();
            if (checkFileExtension) {
                pathname = writer.checkFileExtension(pathname);
            }
            fos = new FileOutputStream(pathname);
            writer.write(cv, fos);
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
            } catch (IOException ioe) {
            }
        }
        return pathname;
    }

    /**
     * Writes expression data to a file in the given format. The correct file
     * extension will be added to the pathname if it is not present. If there is
     * already a file present at the given pathname, its contents are discarded.
     *
     * @param data               the expression data.
     * @param formatName         a String containing the informal name of a format (e.g., "res"
     *                           or "gct".)
     * @param pathname           a pathname string
     * @param checkFileExtension Whether the correct file extension will be added to the
     *                           pathname if it is not present.
     * @return The pathname that the data was saved to
     * @throws IOException If an error occurs while saving the data
     */
    public static String write(IExpressionData data, String formatName,
                               String pathname, boolean checkFileExtension) throws IOException {
        IExpressionDataWriter writer = (IExpressionDataWriter) formatNameToExpressionWriterMap
                .get(formatName);
        if (writer == null) {
            throw new IOException("No writer to save the data in " + formatName
                    + " format.");
        }
        if (checkFileExtension) {
            pathname = writer.checkFileExtension(pathname);
        }
        OutputStream os = null;
        try {
            os = new BufferedOutputStream(new FileOutputStream(pathname));
            writer.write(data, os);
        } finally {
            try {
                if (os != null) {
                    os.flush();
                    os.close();
                }
            } catch (IOException ioe) {
            }
        }
        return pathname;
    }

    /**
     * Parses the odf document at the given pathname using the specified handler
     *
     * @param pathname A pathname string
     * @param handler  The odf handler
     * @return <code>true</code> if the odf document was read successfully
     * @throws IOException    If an error occurs while reading from the file
     * @throws ParseException If there is a problem with the data
     */

    public static void readOdf(String pathname, IOdfHandler handler)
            throws ParseException, IOException {
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(pathname);
            OdfParser parser = new OdfParser();
            parser.setHandler(handler);
            parser.parse(fis);
            fis.close();
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException x) {
                }
            }
        }
    }

    private static IExpressionDataParser tryAllReaders(Iterator it,
                                                       String pathname) throws IOException {
        while (it.hasNext()) {
            IExpressionDataParser r = (IExpressionDataParser) it.next();
            FileInputStream fis = null;
            try {
                fis = new FileInputStream(pathname);
                try {
                    if (r.canDecode(fis)) {
                        return r;
                    }
                } catch (Exception e) {
                    if (debug) {
                        e.printStackTrace();
                    }
                }
            } finally {
                if (fis != null) {
                    fis.close();
                }
            }
        }
        return null;
    }

    private static Iterator lookupProviders(Class c, Iterator defaultIt) {
        try {
            Class klass = Class.forName("javax.imageio.spi.ServiceRegistry");
            java.lang.reflect.Method m = klass.getMethod("lookupProviders",
                    new Class[]{Class.class});
            return (Iterator) m.invoke(null, new Object[]{c});
            // return javax.imageio.spi.ServiceRegistry.lookupProviders(c);
        } catch (Exception e) {
            return defaultIt;
        }
    }

    /*
     * public static boolean isFeatureList(String pathname) { String suffix =
     * pathname.substring(pathname.lastIndexOf(".") + 1,
     * pathname.length()).toLowerCase();
     * featureListReader.getFileSuffixes().contains(suffix); return
     * pathname.endsWith(".grp"); }
     */
    /**
     * Gets an expression data parser that can read the document at the given
     * pathname or <code>null</code> if no reader is found.
     *
     * @param pathname A pathname string
     * @return The expression reader
     */
    public static IExpressionDataParser getExpressionParser(String pathname) {
        return getExpressionReader(pathname, true);
    }

    /**
     * Gets an expression data reader that can read the document at the given
     * pathname or <code>null</code> if no reader is found.
     *
     * @param pathname A pathname string
     * @param whether  to try all readers if the type of the document can not be
     *                 deduced from the file extension
     * @return The expression reader
     */
    public static IExpressionDataParser getExpressionReader(String pathname,
                                                            boolean tryAll) {
        IExpressionDataParser reader = null;
        int dotIndex = pathname.lastIndexOf(".");
        if (dotIndex != -1) {// see if file has an extension
            String suffix = pathname.substring(dotIndex + 1, pathname.length());
            reader = (IExpressionDataParser) suffix2ExpressionReaders
                    .get(suffix);
        }
        if (reader == null && tryAll) {
            try {
                reader = tryAllReaders(suffix2ExpressionReaders.values()
                        .iterator(), pathname);
            } catch (IOException ioe) {
                return null;
            }
        }
        return reader;
    }

    /**
     * Returns a IExpressionDataWriter that can encode the named format
     *
     * @param formatName a String containing the informal name of a format (e.g., "res"
     *                   or "gct".
     * @return a IExpressionDataWriter that can encode in the specified format
     *         or <codee>null</code> if no such IExpressionDataWriter exists.
     */
    public static IExpressionDataWriter getExpressionWriterByFormatName(
            String formatName) {
        return (IExpressionDataWriter) formatNameToExpressionWriterMap
                .get(formatName);
    }

    static {
        List defaultWriters = Arrays.asList(new IExpressionDataWriter[]{
                new ResWriter(), new GctWriter(), new OdfDatasetWriter(),
                new SnpWriter()});

        Iterator it = defaultWriters.iterator();// lookupProviders(edu.mit.broad.io.expr.IExpressionDataWriter.class,
        // defaultWriters);
        formatNameToExpressionWriterMap = new HashMap();
        while (it.hasNext()) {
            IExpressionDataWriter writer = (IExpressionDataWriter) it.next();
            formatNameToExpressionWriterMap.put(writer.getFormatName(), writer);
        }
    }

    static {
        Iterator defaultParsers = Arrays.asList(
                new IExpressionDataParser[]{new GctParser(), new ResParser(),
                        new OdfParserAdapter(), new SnpParser()}).iterator();// defaults
        // if
        // using java 1.3
        Iterator it = defaultParsers;// lookupProviders(edu.mit.broad.io.expr.IExpressionDataParser.class,
        // defaultReaders);
        suffix2ExpressionReaders = new HashMap();
        while (it.hasNext()) {
            IExpressionDataParser parser = (IExpressionDataParser) it.next();
            List suffixes = parser.getFileSuffixes();
            for (int j = 0, cnt = suffixes.size(); j < cnt; j++) {
                String suffix = (String) suffixes.get(j);
                // XXX if > 1 reader can read a file with the same suffix, one
                // of the readers will be 'lost'
                suffix2ExpressionReaders.put(suffix, parser);
            }
        }
        String temp = System.getProperty("edu.mit.broad.gp.debug");
        if ("true".equals(temp)) {
            debug = true;
        }
    }
}