/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.module;

import org.genepattern.data.expr.ExpressionData;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.data.matrix.ClassVector;
import org.genepattern.data.matrix.DoubleMatrix2D;
import org.genepattern.data.matrix.IClassVector;
import org.genepattern.io.IOUtil;
import org.genepattern.io.IOdfHandler;
import org.genepattern.io.ParseException;
import org.genepattern.io.expr.*;
import org.genepattern.io.expr.odf.OdfDatasetWriter;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * A class containing static convenience methods for tasks that are frequently
 * encountered when writing GenePattern modules
 *
 * @author Joshua Gould
 */
public class AnalysisUtil {

    private AnalysisUtil() {
    }

    /**
     * Prints the given message to the standard error stream and terminates the
     * currently running Java Virtual Machine. The message of the given
     * throwable is appended to the message if it is not <code>null</code>
     *
     * @param message The message
     * @param e       The throwable
     */
    public static void exit(String message, Throwable e) {
        String exceptionMessage = e.getMessage();
        if (exceptionMessage != null) {
            message += "\nCause: " + exceptionMessage;
        }
        exit(message);
    }

    /**
     * Prints the given message to the standard error stream and terminates the
     * currently running Java Virtual Machine
     *
     * @param message The message
     */
    public static void exit(String message) {
        System.err.println(message);
        System.exit(1);
    }

    /**
     * Reads the cls document at the given pathname. Prints a message to the
     * standard error stream and terminates the currently running Java Virtual
     * Machine if an error occurs.
     *
     * @param pathname The pathname string
     * @return The class vector
     */
    public static ClassVector readClassVector(String pathname) {
        try {
            return IOUtil.readCls(pathname);
        } catch (IOException e) {
            fileReadError(null, pathname);
        } catch (ParseException e) {
            fileReadError(e, pathname);
        }
        return null;
    }

    /**
     * Gets a list of features at the given file pathname string. Prints a
     * message to the standard error stream and terminates the currently running
     * Java Virtual Machine if an error occurs.
     *
     * @param pathname The pathname string
     * @return the feature list
     */
    public static List readFeatureList(String pathname) {
        try {
            return IOUtil.readFeatureList(pathname);
        } catch (IOException e) {
            fileReadError(null, pathname);
            return null;
        }
    }

    /**
     * Writes the given feature list to a file. Prints a message to the standard
     * error stream and terminates the currently running Java Virtual Machine if
     * an error occurs.
     *
     * @param features The features
     * @param pathname A pathname string
     */
    public static void writeFeatureList(String[] features, String pathname) {
        try {
            IOUtil.writeFeatureList(features, pathname);
        } catch (IOException e) {
            fileReadError(e, pathname);
        }
    }

    /**
     * Reads the expression data at the given pathname. The type of the returned
     * object is determined by the expressionDataCreator argument. Prints a
     * message to the standard error stream and terminates the currently running
     * Java Virtual Machine if an error occurs.
     *
     * @param pathname The file pathname
     * @param parser   The expression data parser
     * @param creator  The expression data creator
     * @return An object containing the expression data
     */
    public static Object readExpressionData(IExpressionDataParser parser,
                                            String pathname, IExpressionDataCreator creator) {
        try {
            return ReaderUtil.read(parser, pathname, creator);
        } catch (ParseException e) {
            fileReadError(e, pathname);
        } catch (IOException e) {
            fileReadError(null, pathname);
        }
        return null;

    }

    /**
     * Reads the expression data at the given pathname. Prints a message to the
     * standard error stream and terminates the currently running Java Virtual
     * Machine if an error occurs.
     *
     * @param pathname The file pathname
     * @param parser   The expression data parser
     * @return An <tt>ExpressionData</tt> instance
     */
    public static ExpressionData readExpressionData(
            IExpressionDataParser parser, String pathname) {
        return (ExpressionData) readExpressionData(parser, pathname,
                new ExpressionDataCreator());
    }

    /**
     * Reads the expression data at the given pathname. Prints a message to the
     * standard error stream and terminates the currently running Java Virtual
     * Machine if an error occurs.
     *
     * @param pathname The file pathname
     * @return An <tt>ExpressionData</tt> instance
     */
    public static ExpressionData readExpressionData(String pathname) {
        return (ExpressionData) readExpressionData(
                getExpressionReader(pathname), pathname,
                new ExpressionDataCreator());
    }

    /**
     * Checks to see that the number of columns in the given matrix are equal to
     * the size of the given class vector. Prints a message to the standard
     * error stream and terminates the currently running Java Virtual Machine if
     * the dimensions do not agree.
     *
     * @param data The 2-dimensional matrix
     * @param cv   The class vector
     */
    public static void checkDimensions(DoubleMatrix2D data, IClassVector cv) {
        if (cv.size() != data.getColumnCount()) {
            exit("Number of columns in dataset(" + data.getColumnCount()
                    + ") does not match the number of class assignments("
                    + cv.size() + ").");
        }
    }

    /**
     * Checks to see that the number of columns in the given expression data are
     * equal to the size of the given class vector. Prints a message to the
     * standard error stream and terminates the currently running Java Virtual
     * Machine if the dimensions do not agree.
     *
     * @param data The expression data
     * @param cv   The class vector
     */
    public static void checkDimensions(IExpressionData data, IClassVector cv) {
        if (cv.size() != data.getColumnCount()) {
            exit("Number of columns in dataset(" + data.getColumnCount()
                    + ") does not match the number of class assignments("
                    + cv.size() + ").");
        }
    }

    /**
     * Parses the odf document at the given pathname using the specified handler
     *
     * @param pathname A pathname string
     * @param handler  The odf handler
     */

    public static void readOdf(String pathname, IOdfHandler handler) {
        try {
            IOUtil.readOdf(pathname, handler);
        } catch (ParseException e) {
            fileReadError(e, pathname);
        } catch (IOException e) {
            fileReadError(null, pathname);
        }
    }

    /**
     * Writes the given class vector to a file. Prints a message to the standard
     * error stream and terminates the currently running Java Virtual Machine if
     * an error occurs.
     *
     * @param cv                 The class vector
     * @param pathname           A pathname string
     * @param checkFileExtension Whether the correct file extension will be added to the
     *                           pathname if it is not present.
     * @return The pathname that the data was saved to
     */
    public static String writeCls(IClassVector cv, String pathname,
                                  boolean checkFileExtension) {
        try {
            return IOUtil.writeCls(cv, pathname, checkFileExtension);
        } catch (IOException e) {
            fileSaveError(e, pathname);
            return null;
        }
    }

    /**
     * Writes expression data to a file in the given format. The correct file
     * extension will be added to the pathname if it is not present. If there is
     * already a file present at the given pathname, its contents are discarded.
     * Prints a message to the standard error stream and terminates the
     * currently running Java Virtual Machine if an error occurs.
     *
     * @param data               the expression data.
     * @param formatName         a String containing the informal name of a format (e.g., "res"
     *                           or "gct".)
     * @param pathname           a pathname string
     * @param checkFileExtension Whether the correct file extension will be added to the
     *                           pathname if it is not present.
     * @return The pathname that the data was saved to
     */
    public static String write(IExpressionData data, String formatName,
                               String pathname, boolean checkFileExtension) {
        try {
            return IOUtil.write(data, formatName, pathname, checkFileExtension);
        } catch (IOException ioe) {
            fileSaveError(ioe, pathname);
            return null;
        }
    }

    private static void fileReadError(Exception e, String pathname) {
        String message = "An error occured while reading the file "
                + getFileName(pathname) + ".";
        String exceptionMessage = e != null ? e.getMessage() : null;
        if (exceptionMessage != null) {
            message += "\nCause: " + exceptionMessage;
        }
        exit(message);
    }

    private static void fileSaveError(Exception e, String pathname) {
        String msg = "An error occured while attempting to save the file "
                + getFileName(pathname) + ".";
        String excMsg = e.getMessage();
        if (excMsg != null) {
            msg += "\nCause: " + excMsg;
        }
        exit(msg);
    }

    /**
     * Gets an expression data parser that can read the document at the given
     * pathname or <code>null</code> if no parser is found. Prints a message
     * to the standard error stream and terminates the currently running Java
     * Virtual Machine if no parser is found or an error occurs.
     *
     * @param pathname A pathname string
     * @return The expression parser
     * @deprecated use #getExpressionDataParser instead
     */
    public static IExpressionDataParser getExpressionReader(String pathname) {
        return getExpressionDataParser(pathname);
    }

    /**
     * Gets an expression data parser that can read the document at the given
     * pathname or <code>null</code> if no parser is found. Prints a message
     * to the standard error stream and terminates the currently running Java
     * Virtual Machine if no parser is found or an error occurs.
     *
     * @param pathname A pathname string
     * @return The expression parser
     */
    public static IExpressionDataParser getExpressionDataParser(String pathname) {
        IExpressionDataParser parser = IOUtil.getExpressionParser(pathname);
        if (parser == null) {
            exit("Invalid input type: " + getFileName(pathname)
                    + " is not a res, gct, or odf dataset file.");
        }
        return parser;
    }

    /**
     * Gets the name of the file at the given pathname. Removes the axis prefix
     * if necessary.
     *
     * @param pathname The pathname string
     * @return The file name
     */
    public static String getFileName(String pathname) {
        String name = new File(pathname).getName();
        return name.replaceFirst("Axis[0-9]*axis_", "");
    }

    static {
        IExpressionDataWriter writer = IOUtil
                .getExpressionWriterByFormatName("odf");
        if (writer instanceof OdfDatasetWriter) {
            ((OdfDatasetWriter) writer).setPrependExecutionLog(true);
        }
    }
}
