/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.module;

import org.genepattern.data.expr.ExpressionData;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.data.matrix.ClassVector;
import org.genepattern.io.IOUtil;
import org.genepattern.io.IOdfHandler;
import org.genepattern.io.ParseException;
import org.genepattern.io.expr.ExpressionDataCreator;
import org.genepattern.io.expr.IExpressionDataCreator;
import org.genepattern.io.expr.IExpressionDataParser;
import org.genepattern.io.expr.ReaderUtil;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * A class containing static convenience methods for tasks that are frequently
 * encountered when writing GenePattern visualizers
 *
 * @author Joshua Gould
 */
public class VisualizerUtil {

    private VisualizerUtil() {
    }

    public static void centerOnScreen(Component c) {
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit()
                .getScreenSize();
        c.setLocation((screenSize.width - c.getWidth()) / 2,
                (screenSize.height - c.getHeight()) / 2);
    }

    public static void showFrame(Window c) {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        c.setSize(screenSize);
        c.setLocation((screenSize.width - c.getWidth()) / 2,
                (screenSize.height - c.getHeight()) / 2);
        c.setVisible(true);
    }

    /**
     * Brings up an error message dialog displaying the given message . The
     * message of the given throwable is appended to the message if it is not
     * <code>null</code>
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code> has no <code>Frame</code>, a
     *                        default <code>Frame</code> is used
     * @param message         The message
     * @param e               The throwable
     */
    public static void error(Component parentComponent, String message,
                             Throwable e) {
        String exceptionMessage = e.getMessage();
        if (exceptionMessage != null) {
            message += "\nCause: " + exceptionMessage;
        }
        error(parentComponent, message);
    }

    /**
     * Brings up an error message dialog
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code> has no <code>Frame</code>, a
     *                        default <code>Frame</code> is used
     * @param message         The message
     */
    public static void error(Component parentComponent, String message) {
        JOptionPane.showMessageDialog(parentComponent, message, "Error",
                JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Reads the cls document at the given pathname. Brings up an error message
     * dialog if an error occurs.
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code>
     * @param pathname        The pathname string
     * @return The class vector
     */
    public static ClassVector readCls(Component parentComponent, String pathname) {

        try {
            return IOUtil.readCls(pathname);
        } catch (IOException e) {
            error(parentComponent, "An error occured while reading the file "
                    + new File(pathname).getName());
            return null;
        } catch (ParseException e) {
            fileReadError(e, parentComponent, pathname);
            return null;
        }

    }

    /**
     * Gets a list of features at the given file pathname string. Brings up an
     * error message dialog if an error occurs.
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code>
     * @param pathname        The pathname string
     * @return the feature list
     */
    public static List readFeatureList(Component parentComponent,
                                       String pathname) {
        try {
            return IOUtil.readFeatureList(pathname);
        } catch (IOException e) {
            error(parentComponent, "An error occured while reading the file "
                    + new File(pathname).getName());
            return null;
        }
    }

    /**
     * Reads the expression data at the given pathname. The type of the returned
     * object is determined by the expressionDataCreator argument. Brings up an
     * error message dialog if an error occurs.
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code>
     * @param parser          The expression data parser
     * @param pathname        The file pathname
     * @param creator         The expression data creator
     * @return An object containing the expression data
     */
    public static Object readExpressionData(Component parentComponent,
                                            IExpressionDataParser parser, String pathname,
                                            IExpressionDataCreator creator) {
        try {
            return ReaderUtil.read(parser, pathname, creator);
        } catch (IOException e) {
            error(parentComponent, "An error occured while reading the file "
                    + new File(pathname).getName());
            return null;
        } catch (ParseException e) {
            fileReadError(e, parentComponent, pathname);
            return null;
        }
    }

    /**
     * Reads the expression data at the given pathname. Brings up an error
     * message dialog if an error occurs.
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code>
     * @param parser          The expression data parser
     * @param pathname        The file pathname
     * @return An <tt>ExpressionData</tt> instance
     */
    public static ExpressionData readExpressionData(Component parentComponent,
                                                    IExpressionDataParser parser, String pathname) {
        try {
            return (ExpressionData) ReaderUtil.read(parser, pathname,
                    new ExpressionDataCreator());

        } catch (IOException e) {
            error(parentComponent, "An error occured while reading the file "
                    + new File(pathname).getName());
            return null;
        } catch (ParseException e) {
            fileReadError(e, parentComponent, pathname);
            return null;
        }
    }

    /**
     * Reads the expression data at the given pathname and returns a new
     * <tt>ExpressionData</tt> instance. *
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed
     * @param file            The file
     * @return An <tt>ExpressionData</tt> instance
     */
    public static ExpressionData readExpressionData(Component parentComponent,
                                                    File file) {
        try {
            return IOUtil.readExpressionData(file.getCanonicalPath());
        } catch (IOException e) {
            error(parentComponent, "An error occured while reading the file "
                    + file.getName());
            return null;
        } catch (ParseException e) {
            fileReadError(e, parentComponent, file.getName());
            return null;
        }

    }

    /**
     * Reads the expression data at the given pathname. Brings up an error
     * message dialog if an error occurs.
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed
     * @param parser          The expression data parser
     * @param pathname        The file pathname
     * @return An <tt>ExpressionData</tt> instance
     */
    public static ExpressionData getExpressionData(Component parentComponent,
                                                   IExpressionDataParser parser, String pathname) {
        try {
            return (ExpressionData) ReaderUtil.read(parser, pathname,
                    new ExpressionDataCreator());

        } catch (IOException e) {
            error(parentComponent, "An error occured while reading the file "
                    + new File(pathname).getName());
            return null;
        } catch (ParseException e) {
            fileReadError(e, parentComponent, pathname);
            return null;
        }
    }

    /**
     * Parses the odf document at the given pathname using the specified handler
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code>
     * @param pathname        A pathname string
     * @param handler         The odf handler
     * @return <code>true</code> if the odf document was read successfully
     */

    public static boolean readOdf(Component parentComponent, String pathname,
                                  IOdfHandler handler) {
        try {
            IOUtil.readOdf(pathname, handler);
            return true;
        } catch (IOException e) {
            error(parentComponent, "An error occured while reading the file "
                    + new File(pathname).getName());
            return false;
        } catch (ParseException e) {
            fileReadError(e, parentComponent, pathname);
            return false;
        }
    }

    /**
     * Writes the given class vector to a file. Brings up an error message
     * dialog if an error occurs.
     *
     * @param parentComponent    determines the <code>Frame</code> in which the dialog is
     *                           displayed; if <code>null</code>, or if the
     *                           <code>parentComponent</code>
     * @param cv                 The class vector
     * @param pathname           A pathname string
     * @param checkFileExtension Whether the correct file extension will be added to the
     *                           pathname if it is not present.
     * @return The pathname that the data was saved to
     */
    public static String writeCls(Component parentComponent, ClassVector cv,
                                  String pathname, boolean checkFileExtension) {
        try {
            return IOUtil.writeCls(cv, pathname, checkFileExtension);
        } catch (Exception e) {
            fileSaveError(e, parentComponent, pathname);
            return null;
        }
    }

    /**
     * Writes expression data to a file in the given format. The correct file
     * extension will be added to the pathname if it is not present. If there is
     * already a file present at the given pathname, its contents are discarded.
     * Brings up an error message dialog if an error occurs.
     *
     * @param parentComponent    determines the <code>Frame</code> in which the dialog is
     *                           displayed; if <code>null</code>, or if the
     *                           <code>parentComponent</code>
     * @param data               the expression data.
     * @param formatName         a String containing the informal name of a format (e.g., "res"
     *                           or "gct".)
     * @param pathname           a pathname string
     * @param checkFileExtension Whether the correct file extension will be added to the
     *                           pathname if it is not present.
     * @return The pathname that the data was saved to
     */
    public static String write(Component parentComponent, IExpressionData data,
                               String formatName, String pathname, boolean checkFileExtension) {
        try {
            return IOUtil.write(data, formatName, pathname, checkFileExtension);
        } catch (IOException ioe) {
            fileSaveError(ioe, parentComponent, pathname);
            return null;
        }
    }

    private static void fileReadError(Exception e, Component parentComponent,
                                      String pathname) {
        String message = "An error occured while reading the file "
                + AnalysisUtil.getFileName(pathname) + ".";
        String exceptionMessage = e.getMessage();
        if (exceptionMessage != null) {
            message += "\nCause: " + exceptionMessage;
        }
        error(parentComponent, message);
    }

    private static void fileSaveError(Exception e, Component parentComponent,
                                      String pathname) {
        String msg = "An error occured while attempting to save the file "
                + AnalysisUtil.getFileName(pathname) + ".";
        String excMsg = e.getMessage();
        if (excMsg != null) {
            msg += "\nCause: " + excMsg;
        }
        error(parentComponent, msg);
    }

    /**
     * Gets an expression data parser that can read the document at the given
     * pathname or <code>null</code> if no parser is found. Brings up an error
     * message dialog if an error occurs.
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code>
     * @param pathname        A pathname string
     * @return The expression parser
     */
    public static IExpressionDataParser getExpressionDataParser(
            Component parentComponent, String pathname) {
        try {
            IExpressionDataParser parser = IOUtil.getExpressionParser(pathname);
            if (parser == null) {
                error(parentComponent, "Invalid input type: "
                        + AnalysisUtil.getFileName(pathname)
                        + " is not a res, gct, or odf dataset file.");
            }
            return parser;
        } catch (Exception e) {
            fileReadError(e, parentComponent, pathname);
            return null;
        }
    }

    /**
     * Gets an expression data parser that can read the document at the given
     * pathname or <code>null</code> if no parser is found. Brings up an error
     * message dialog if an error occurs.
     *
     * @param parentComponent determines the <code>Frame</code> in which the dialog is
     *                        displayed; if <code>null</code>, or if the
     *                        <code>parentComponent</code>
     * @param pathname        A pathname string
     * @return The expression parser
     * @deprecated use #getExpressionDataParser instead
     */
    public static IExpressionDataParser getExpressionReader(
            Component parentComponent, String pathname) {
        return getExpressionDataParser(parentComponent, pathname);
    }

}
