package xapps.api.vtools;

import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.HClustDataset;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDbMulti;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import edu.mit.broad.genome.viewers.Viewer;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.xbench.core.api.VTool;
import org.apache.log4j.Logger;
import xapps.api.vtools.stackables.StackablesManager;
import xapps.api.vtools.stackables.VStackables;
import xapps.api.vtools.viewers.EnrichmentViewer;
import xapps.api.vtools.viewers.HeatMapViewer;
import xtools.api.param.BooleanParam;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

/**
 * These are like tools except they work in the user interface
 *
 * @author Aravind Subramanian
 *         Date: Jan 14, 2006
 *         Time: 8:41:15 PM
 */

public class Hooks {

    protected static final Logger klog = XLogger.getLogger(StackablesManager.class);

    public static final Icon ICON_SAVE = JarResources.getIcon("Save16.gif");
    public static final Icon ICON_SAVE_GCT = JarResources.getIcon("Gct16.gif");

    public static BooleanParam createOnlySelectedFieldsParam() {
        return new BooleanParam("selected", "Only selected fields", "Only selected fields", true, false);
    }

    public static interface BaseHook {

        // @note need to change this to save name
        public String getSaveNameHint();

        public String getSaveName();

        public String getComponentMessage();

        public JTextField getTextField();

    }

    public static interface Hook extends BaseHook {

        public Viewer createHeatMapViewer(final Dataset ds, final Template t_opt, final HClustDataset clust_opt);

        public Viewer createHeatMapViewer(final Dataset ds);

    }

    public static abstract class AbstractHook implements Hook {

        private JTextField tfName;

        public JTextField getTextField() {

            if (tfName == null) {
                tfName = new JTextField(60);

                tfName.setBorder(BorderFactory.createTitledBorder("Enter a short name for the result"));
            }

            String sn = getSaveNameHint();
            if (sn != null && sn.length() > 0) {
                tfName.setText(sn);
            }

            return tfName;
        }

        public String getSaveName() { // dont make safe - give back whatever user asks for
            if (tfName != null) {
                return tfName.getText();
            } else {
                return "na";
            }
        }

    } // End class AbstractHook

    // class rather than interface because we want things to be able to access the fields easilly (w/o getters)
    public static abstract class Matrix extends AbstractHook {

        public JTable fColTable;
        public JTable fRowTable;
        public Dataset fDataset;
        public SampleAnnot fSampleAnnot_synched;
        public VStackables.SampleAndTemplateLabels stl_opt;
        public Template fTemplate;

        public abstract Dataset getDataset(final BooleanParam onlySelectedFieldsParam, final VTool tool);

        public int[] getSelectedColumns() {
            return fColTable.getSelectedColumns();
        }

        public int[] getSelectedRows() {
            return fRowTable.getSelectedRows();
        }

        public int getSelectedColumnCount() {
            return fColTable.getSelectedColumnCount();
        }

        public int getSelectedRowCount() {
            return fRowTable.getSelectedRowCount();
        }

    }

    public static class Gex extends Matrix {

        public Gex(final Dataset ds,
                   final SampleAnnot sa_synched,
                   final Template t,
                   final JTable rowTable,
                   final JTable colTable,
                   final VStackables.SampleAndTemplateLabels stl) {
            this.fDataset = ds;
            this.fSampleAnnot_synched = sa_synched;
            this.fTemplate = t;
            this.fRowTable = rowTable;
            this.fColTable = colTable;
            this.stl_opt = stl;
        }

        public String getSaveNameHint() {
            if (fRowTable != null && fColTable != null) {
                int[] selRows = fRowTable.getSelectedRows();
                int[] selCols = fColTable.getSelectedColumns();
                return fDataset.getName() + "_rows" + selRows.length + "_cols" + selCols.length;
            }

            return null;
        }

        public String getComponentMessage() {
            if (fRowTable != null && fColTable != null) {
                final int[] selRows = fRowTable.getSelectedRows();
                final int[] selCols = fColTable.getSelectedColumns();
                return "# selected rows: " + selRows.length + "\n# selected columns: " + selCols.length;
            } else {
                return null;
            }
        }

        public Dataset getDataset(final BooleanParam onlySelectedFieldsParam, final VTool tool) {
            if (onlySelectedFieldsParam.isTrue()) {
                int[] selRows = fRowTable.getSelectedRows();

                int[] selCols = fColTable.getSelectedColumns();

                if (selRows != null && selRows.length > 0) {
                    return new DatasetGenerators().extract(tool.getSaveName(), fDataset, selRows, selCols);
                } else { // use ALL rows for hist'ing
                    return new DatasetGenerators().extractColumns(tool.getSaveName(), fDataset, selCols, true);
                }
            } else {
                return fDataset;
            }

        }

        public Viewer createHeatMapViewer(final Dataset ds, final Template t_opt, final HClustDataset clust_opt) {
            return new HeatMapViewer(ds, t_opt, clust_opt);
        }

        public Viewer createHeatMapViewer(final Dataset ds) {
            return new HeatMapViewer(ds);
        }

    } // End class Hook

    public static class Ess extends Gex {

        public EnrichmentDbMulti edb_multi_synched;

        public Ess(final EnrichmentDbMulti edb_multi_synched,
                   final Dataset ds,
                   final SampleAnnot sa_synched,
                   final Template t,
                   final JTable rowTable,
                   final JTable colTable,
                   final VStackables.SampleAndTemplateLabels stl) {

            super(ds, sa_synched, t, rowTable, colTable, stl);

            if (ds.getNumCol() != edb_multi_synched.getNumEdbs()) {
                throw new MismatchedSizeException("edb_multi", edb_multi_synched.getNumEdbs(), "ds", ds.getNumCol());
            }

            this.edb_multi_synched = edb_multi_synched;
        }

        // These are the main overrides that help make a ev rather than a vanilla hmv
        public Viewer createHeatMapViewer(final Dataset ds, final Template t_opt, final HClustDataset clust_opt) {
            return new EnrichmentViewer(ds, t_opt, clust_opt, edb_multi_synched);
        }

        // These are the main overrides that help make a ev rather than a vanilla hmv
        public Viewer createHeatMapViewer(final Dataset ds) {
            return new EnrichmentViewer(ds, null, null, edb_multi_synched);
        }


        public String[] getSelectedCompNames() {
            final int[] cols = getSelectedColumns();
            final String[] names = new String[cols.length];
            for (int c = 0; c < names.length; c++) {
                names[c] = fDataset.getColumnName(cols[c]);
            }
            return names;
        }

        public String[] getSelectedGeneSetNames() {
            final int[] rows = getSelectedRows();
            final String[] names = new String[rows.length];
            for (int r = 0; r < names.length; r++) {
                names[r] = fDataset.getRowName(rows[r]);
            }
            return names;
        }

        // edb -> row and col
        // for now do over ALL gene sets
        public EnrichmentDbMulti getSelectedEdbs(final boolean qualify_by_sel_rows, final boolean qualify_to_sel_cols) {

            final String[] sel_gset_names = getSelectedGeneSetNames();
            final String[] sel_comp_names = getSelectedCompNames();

            EnrichmentDbMulti edb_multi;

            // more efficient to call edb_multi
            if (qualify_by_sel_rows && qualify_to_sel_cols) {
                edb_multi = edb_multi_synched.cloneDeep(getSaveNameHint(), sel_gset_names, sel_comp_names);
            } else if (qualify_by_sel_rows) {
                edb_multi = edb_multi_synched.cloneDeep_gs(getSaveNameHint(), sel_gset_names);
            } else if (qualify_to_sel_cols) {
                edb_multi = edb_multi_synched.cloneDeep(getSaveNameHint(), sel_comp_names);
            } else {
                throw new IllegalArgumentException("Both cant be false: " + qualify_by_sel_rows + " " + qualify_to_sel_cols);
            }

            if (edb_multi.getNumEdbs() == 0) {
                throw new IllegalArgumentException("Zero edb edb_multi ... no cols selected?");
            }

            if (edb_multi.getNumRow() == 0) {
                throw new IllegalArgumentException("Zero row edb_multi ... no rows selected?");
            }

            klog.debug("Dim of selected edb_multi rows: " + edb_multi.getNumRow() + " cols: " + edb_multi.getNumEdbs());

            return edb_multi;

            /*
                       final int[] cols = getSelectedColumns();
            final EnrichmentDb[] edbs = new EnrichmentDb[cols.length]; // one edb for each col
            for (int c = 0; c < cols.length; c++) {
                edbs[c] = edb_multi_synched.getEdb(fDataset.getColumnName(cols[c]));

                if (qualifyBySelectedRows) {
                   edbs[c] = edbs[c].get

                }
            }
            return edbs;
            */

        }

        public EnrichmentResult[] getResults() {

            // @todo multiple rows

            String gsetName = getSelectedGeneSetNames()[0];

            return edb_multi_synched.getResults(gsetName); // @todo all comp vs sel comps
        }

        public EnrichmentResult[] getResults_row_and_col_qual() {

            int[] cols = getSelectedColumns();

            List res = new ArrayList();
            String gsetName = getSelectedGeneSetNames()[0];

            EnrichmentResult[] ress = edb_multi_synched.getResults(gsetName);
            for (int i = 0; i < cols.length; i++) {
                res.add(ress[cols[i]]);
            }

            return (EnrichmentResult[]) res.toArray(new EnrichmentResult[res.size()]);
        }

    } // End class Hook


} // End class Hooks

