package xapps.api.vtools.stackables;

import edu.mit.broad.genome.math.ColorScheme;
import edu.mit.broad.genome.math.ColorSchemes;
import edu.mit.broad.genome.math.ScaleMode;
import edu.mit.broad.genome.models.NumberedProxyModel;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GPWrappers;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.viewers.DatasetFeatureAnnViewer;
import org.genepattern.clustering.hierarchical.AtrGtrReader;
import org.genepattern.clustering.hierarchical.GeneTreePanel;
import org.genepattern.heatmap.HeatMapPanel;
import org.genepattern.heatmap.image.DisplaySettings;
import org.genepattern.heatmap.image.HeatMap;

import javax.swing.*;
import javax.swing.table.JTableHeader;
import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * @author Aravind Subramanian
 *         Date: Jan 1, 2006
 *         Time: 9:22:00 PM
 */
public class HStackables {

    // wraps Josh's heat map into a stackable
    public static class HeatMapComponent implements Stackable.Horizontal {

        private HeatMapPanel fHmcPanel;

        private Dataset ds;

        private int cellHeight;

        private ScaleMode fScaleMode = ScaleMode.REL_MEAN;
        private ColorScheme fColorScheme = new ColorSchemes.BroadCancer();

        public HeatMapComponent(final Dataset ds) {
            this.ds = ds;
            this.fHmcPanel = new HeatMapPanel(GPWrappers.createIExpressionData(ds));
            this.fHmcPanel.setShowToolTipText(true);
        }

        public JComponent getComponent() {
            return fHmcPanel;
        }

        public HeatMapPanel getPanel() {
            return fHmcPanel;
        }

        public BufferedImage createImage() {

            final DisplaySettings disp = new DisplaySettings();
            disp.drawGrid = true;
            disp.rowSize = cellHeight;
            disp.columnSize = cellHeight;

            disp.sampleAnnonationsHeight = 25;
            disp.showSampleGridLines = false; // i prefer this for gsea
            disp.showFeatureGridLines = true; // i prefer this for gsea

            disp.drawRowDescriptions = false;
            disp.drawRowNames = true;

            return HeatMap.createImage(fHmcPanel.getData(), disp, null, null);
        }

        public void setColorScheme(final ColorScheme cs) {
            setScaleModeAndColorScheme(fScaleMode, cs);
        }

        public void setScaleMode(final ScaleMode sm) {
            setScaleModeAndColorScheme(sm, fColorScheme);
        }

        public Component getLegend() {

            System.out.println(">>>>>>>>>>>> " + fHmcPanel.getColorConverter());
            System.out.println(">>>>>>>>>>>> " + fHmcPanel.getColorConverter().getLegend());

            return fHmcPanel.getColorConverter().getLegend();
        }

        public void setScaleModeAndColorScheme(final ScaleMode sm, final ColorScheme cs) {
            this.fScaleMode = sm;
            this.fColorScheme = cs;
            fHmcPanel.setColorConverter(GPWrappers.createColorScheme(ds, fScaleMode, fColorScheme));
            fHmcPanel.invalidate();
            fHmcPanel.validate();
            fHmcPanel.repaint();
        }

        public void setCellHeight(int height) {
            this.cellHeight = height;
            fHmcPanel.setColumnSize(height);
            fHmcPanel.setRowSize(height);
            fHmcPanel.invalidate();
            fHmcPanel.validate();
            fHmcPanel.repaint(); // @todo are these reval calls needed??
        }

    } // End class HeatMapComponent

    /**
     * Class that makes a feature (row) annotation component
     */
    public static class FeatureAnnotComponentWithMetrics extends FeatureAnnotComponentSimple {

        /**
         * Class constructor
         *
         * @param ds
         */
        public FeatureAnnotComponentWithMetrics(final Dataset ds) {
            this.fDataset = ds;

            DatasetFeatureAnnViewer.ModelWithMetrics modelWithMetrics = new DatasetFeatureAnnViewer.ModelWithMetrics(ds);

            featureTable = new JTable();
            featureTable.setModel(new NumberedProxyModel(modelWithMetrics)); // @note changed for jide
            this.featureTable.setDefaultRenderer(Object.class, new DatasetFeatureAnnViewer.RendererSimple(fDataset));

            Helper.setColWidth(featureTable, 0, 50); // # setting col width makes the col UNresizable

            featureTable.setShowGrid(true);
            featureTable.setGridColor(Color.LIGHT_GRAY);
            featureTable.setCellSelectionEnabled(true);
            featureTable.setColumnSelectionAllowed(true);

            //featureTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

            // @note that tool tip is on by default in the Jtable
            // this might be intenseive (see the chka.de note)

            /* @todo nice popup annot message
            featureTable.addMouseListener(new MouseAdapter() {
                pub
                public void mouseClicked(MouseEvent e) {
                    if (e.getClickCount() == 2) {
                        //int index = list.locationToIndex(e.getPoint());
                    }
                }
            });
            */

            GuiHelper.Table.makeColumnSelectable(featureTable);
        }

    } // End class FeatureAnnotComponentWithMetrics


    public static class FeatureAnnotComponentSimple implements Stackable.Horizontal {

        protected JTable featureTable;

        protected Dataset fDataset;

        protected FeatureAnnotComponentSimple() {
        }

        /**
         * Class constructor
         *
         * @param ds
         */
        public FeatureAnnotComponentSimple(final Dataset ds) {
            this.fDataset = ds;

            DatasetFeatureAnnViewer.Model model = new DatasetFeatureAnnViewer.Model(ds);

            featureTable = new JTable();
            featureTable.setModel(new NumberedProxyModel(model)); // @note changed for jide
            this.featureTable.setDefaultRenderer(Object.class, new DatasetFeatureAnnViewer.RendererSimple(fDataset));

            Helper.setColWidth(featureTable, 0, 50); // # setting col width makes the col UNresizable

            featureTable.setShowGrid(true);
            featureTable.setGridColor(Color.LIGHT_GRAY);
            featureTable.setCellSelectionEnabled(true);
            featureTable.setColumnSelectionAllowed(true);

            GuiHelper.Table.makeColumnSelectable(featureTable);
        }

        public BufferedImage createImage() {
            return ManagerHelper.createBufferedImage(featureTable);
        }

        public JComponent getComponent() {
            return featureTable;
        }

        public void setCellHeight(int height) {
            featureTable.setRowHeight(height);
            this.featureTable.setFont(new Font(featureTable.getFont().getName(), featureTable.getFont().getStyle(), height));
        }

        public Stackable.Horizontal getTableHeaderAsStackable() {
            return new TableHeader(featureTable);
        }

    } // End class FeatureAnnotComponentSimple

    static class TableHeader implements Stackable.Horizontal {

        private JPanel panel;

        public TableHeader(final JTable table) {
            this.panel = new JPanel(new BorderLayout());
            JTableHeader th = table.getTableHeader();
            JPanel empty = new JPanel();
            empty.setPreferredSize(th.getPreferredSize());
            panel.add(empty, BorderLayout.CENTER);
            panel.add(th, BorderLayout.SOUTH);
        }

        public JComponent getComponent() {
            return panel;
        }

        public void setCellHeight(int height) {
            // Nothing to do
        }

        public BufferedImage createImage() {
            return ManagerHelper.createBufferedImage(getComponent());
        }

    } // End class FeatureAnnotComponent


    /**
     * Class that makes a feature (row) annotation component
     */
    public static class GtrTree implements Stackable.Horizontal {

        private GeneTreePanel geneTree = null;
        private int geneTreeWidth = 150;

        /**
         * Class constructor
         *
         * @param ds
         */
        public GtrTree(final AtrGtrReader reader) {
            this(new GeneTreePanel(reader));
        }

        public GtrTree(final GeneTreePanel gtp) {

            int cell = 8;
            this.geneTree = gtp;
            geneTree.setLeftGutter(0);
            geneTree.setRightGutter(0);
            geneTree.setElementHeight(cell);
            Dimension size = new Dimension(geneTreeWidth, geneTree
                    .getPreferredSize().height);
            geneTree.setPreferredSize(size);
            geneTree.setSize(size);

        }

        public BufferedImage createImage() {
            return geneTree.snapshot();
        }

        public JComponent getComponent() {
            return geneTree;
        }

        public void setCellHeight(int height) {

            geneTree.setElementHeight(height);
            Dimension size = new Dimension(geneTreeWidth, geneTree
                    .getPreferredSize().height);
            geneTree.setPreferredSize(size);
            geneTree.setSize(size);
        }

    } // End class GtrTree

} // End class HStackables
