package xapps.gsea;

import com.jidesoft.grid.SortableTable;
import com.jidesoft.swing.JideTabbedPane;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.viewers.AbstractViewer;
import edu.mit.broad.vdb.msigdb.GeneSetAnnotation;
import edu.mit.broad.vdb.msigdb.MSigDB;
import xapps.browser.MGUIUtils;
import xapps.browser.VToolsForMSigDBBrowser;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import java.awt.*;

/**
 * @author Aravind Subramanian
 *         Date: Feb 27, 2006
 *         Time: 12:04:57 PM
 */
public class GeneSetAnnotationWithOverlaps {

    private GeneSet query;

    private One[] fOnes;

    private String[] fAllGeneSetNames;

    private GeneSetAnnotationWithOverlaps fInstance = this;

    /**
     * Class constructor
     *
     * @param query
     * @param ones
     */
    public GeneSetAnnotationWithOverlaps(final GeneSet query, final One[] ones) {
        this.query = query;
        this.fOnes = ones;
    }

    public TableModel createModel() {
        return new Model();
    }

    public Viewer createViewer(MSigDB msigdb) {
        return new Viewer(query.getName(true), msigdb);
    }


    public String[] getAllDisplayedGeneSetNames() {

        if (fAllGeneSetNames == null) {
            fAllGeneSetNames = new String[fOnes.length];
            for (int i = 0; i < fOnes.length; i++) {
                fAllGeneSetNames[i] = fOnes[i].getAnn().getGeneSet(true).getName(true);
            }
        }

        return fAllGeneSetNames;
    }


    public static class One {

        private GeneSetAnnotation gsa;

        private int inter;

        private double hyper;

        private double jacq;

        public One(final GeneSetAnnotation gsa, final int inter, final int querySize) {
            this.gsa = gsa;
            this.inter = inter;
            int N = querySize + gsa.getGeneSet(false).getNumMembers();
            int K = gsa.getGeneSet(false).getNumMembers();
            int m = querySize;

            this.hyper = 1 - XMath.hypergeometric(inter, N, K, m);

            this.jacq = (float) inter / (float) (N - inter);
        }

        public int getOverlap() {
            return inter;
        }

        public double getHyper() {
            return hyper;
        }

        public double getJacquard() {
            return jacq;
        }

        public GeneSetAnnotation getAnn() {
            return gsa;
        }

    } // End inner class One


    private static String[] COL_NAMES = new String[]{
            "NAME",  // 0
            "OVERLAP", // 1
            "JACQUARD", // 2
            "# GENES",          // 3
            "DESCRIPTION",      // 4
            "COLLECTION"        // 5
    };


    /**
     * Inner class hat models as a table
     */
    class Model extends AbstractTableModel {


        /**
         * Class Constructor.
         * Cretaes a default row name based annotation for use.
         */
        public Model() {
        }

        public int getColumnCount() {
            return COL_NAMES.length;
        }

        public int getRowCount() {
            return fOnes.length;
        }

        public String getColumnName(int col) {
            return COL_NAMES[col];
        }

        public Object getValueAt(int row, int col) {

            GeneSetAnnotation ann = fOnes[row].getAnn();

            if (col == 0) {
                return ann.getStandardName();
            } else if (col == 1) {
                return fOnes[row].getOverlap() + "";
            } else if (col == 2) {
                return fOnes[row].getJacquard() + "";
            } else if (col == 3) {
                return new Integer(ann.getGeneSet(false).getNumMembers());
            } else if (col == 4) {
                return ann.getDescription().getBrief();
            } else if (col == 5) {
                return ann.getCategory().getName();
            } else {
                return "huh";
            }

        }

        public Class getColumnClass(int col) {
            return String.class;
        }

        public boolean isEditable() {
            return false;
        }

        public boolean isCellEditable(int row, int col) {
            return false;
        }
    }    // End GeneSetOverlapModel


    class Viewer extends AbstractViewer {

        /**
         * Class constructor
         *
         * @param ds
         */
        public Viewer(final String queryName, final MSigDB msigdb) {
            super("Overlaps", null, "Overlaps query: " + queryName);
            jbInit(msigdb);
        }

        private void jbInit(final MSigDB msigdb) {
            this.setLayout(new BorderLayout());
            TableModel dmodel = createModel();
            SortableTable table = createTable(dmodel, true, true);
            table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
            this.setLayout(new BorderLayout());
            JScrollPane sp = new JScrollPane(table);
            this.add(sp, BorderLayout.CENTER);

            this.add(MGUIUtils.createExportSetsControlPanel(new MyViewerHook(true, msigdb, table),
                    new MyViewerHook(false, msigdb, table)), BorderLayout.SOUTH);

            this.revalidate();
        }

    }    // End Viewer


    class MyViewerHook implements VToolsForMSigDBBrowser.ViewerHook {
        private boolean onlySelected;

        private MSigDB msigdb;

        private SortableTable table;

        MyViewerHook(boolean onlySelected, MSigDB msigdb, final SortableTable table) {
            this.onlySelected = onlySelected;
            this.msigdb = msigdb;
            this.table = table;
        }


        public String[] getSelectedGeneSetNames() {
            final int[] rows = table.getSelectedRows();
            final String[] names = new String[rows.length];
            for (int i = 0; i < rows.length; i++) {
                names[i] = table.getModel().getValueAt(rows[i], 1).toString();
            }

            return names;
        }

        public int getNumSelectedRows() {
            return table.getSelectedRowCount();
        }

        public MSigDB getMSigDB() {
            return msigdb;
        }

        public boolean isProcessOnlySelected() {
            return onlySelected;
        }

        public JideTabbedPane getViewerWindow_opt() {
            return null;
        }

        public String[] getAllDisplayedGeneSetNames() {
            return fInstance.getAllDisplayedGeneSetNames();
        }

    } // End class MyViewerHook


} // End class GeneSetAnnotationWithOverlaps
