package xexamples;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserException;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.chip.FileInMemoryChip;
import edu.mit.broad.vdb.chip.Probe;
import edu.mit.broad.vdb.chip.SimpleProbe;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.Chip2ChipMapperSymbol;
import edu.mit.broad.vdb.map.MGeneSet;
import edu.mit.broad.vdb.map.MGeneSetMatrix;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author: Aravind Subramanian
 * Date: Mar 28, 2007
 * Time: 9:53:18 PM
 */
public class ParseChipFilesForDBExample {


    public static void main(String[] args) {

        try {

            // (1) READ IN THE SOURCE CHIP
            Chip source_chip = ParserFactory.readChip(new File("MG_U74Av2.chip"), false);

            // (2) Read in a GeneSet to-be-mapped (this MUST be in the SOURCE CHIP format)
            GeneSetMatrix gm = ParserFactory.readGeneSetMatrix(new File("my_mouse_gene_set.grp"), false);
            System.out.println("genesets: " + gm.getQuickInfo());

            // (3) Read in the TARGET chip
            Chip target_chip = ParserFactory.readChip(new File("HG_U133A.chip"), false);

            // (4) Construct a mapper
            Chip2ChipMapper mapper = new Chip2ChipMapperSymbol(source_chip, target_chip);

            // Map it
            MGeneSetMatrix mgms = mapper.map(gm, true);

            for (int i = 0; i < mgms.getNumMappedSets(); i++) {
                MGeneSet mapped_gene_set = mgms.getMappedGeneSet(i);
                System.out.println("Before mapping: " + gm.getGeneSet(i).getNumMembers() + " after mapping: " + mapped_gene_set.getMappedGeneSet(true).getNumMembers());
            }

            // Save the mapped genesetmatrix

            ParserFactory.saveGmt(mgms.getMappedGeneSetMatrix("someprefix"), new File("my_mapped_gene_sets.gmt"));

        } catch (Throwable t) {
            t.printStackTrace();
        }

    }

    private static final String PROBE_SET_ID = "Probe Set ID";
    private static final String GENE_TITLE = "Gene Title";
    private static final String GENE_SYMBOL = "Gene Symbol";
    private static final String ALIASES = "Aliases";
    private static final String TAXON = "Taxon ID";

    // Added a quick API for the DB of chip files
    // 2 types of files (1) symbol/accessions (2) array files
    // Probe Set ID	Gene Symbol	Gene Title	Taxon ID	ORIGINAL_GENE_SYMBOL	ORIGINAL_GENE_TITLE	CHANGED
    public Chip parse_from_dot_chip_forDB(final File chipFile) throws Exception {

        BufferedReader bin = new BufferedReader(new InputStreamReader(new FileInputStream(chipFile)));

        String currLine = nextLine(bin);

        List colHeaders = ParseUtils.string2stringsList(currLine, "\t");
        int ps_index = indexOf(PROBE_SET_ID, colHeaders, true);
        int symbol_index = indexOf(GENE_SYMBOL, colHeaders, true);
        int title_index = indexOf(GENE_TITLE, colHeaders, true);
        int taxon_index = indexOf(TAXON, colHeaders, true);
        int symbol_orig_index = indexOf("ORIGINAL_GENE_SYMBOL", colHeaders, true);
        int title_orig_index = indexOf("ORIGINAL_GENE_TITLE", colHeaders, true);

        // save all rows so that we can determine how many rows exist
        List probesList = new ArrayList();
        currLine = nextLine(bin);
        Set names = new HashSet();

        while (currLine != null) {
            final String[] fields = ParseUtils.string2strings(currLine, "\t", true);

            String probeName = fields[ps_index];

            if (probeName != null && !names.contains(probeName)) {
                String symbol = fields[symbol_index];
                String title = fields[title_index];
                long taxon_id = Long.parseLong(fields[taxon_index]);
                String symbol_orig = fields[symbol_orig_index];
                String title_orig = fields[title_orig_index];

                Probe probe = new SimpleProbe(probeName, symbol, title);
                probesList.add(probe);
            }

            if (probeName != null) {
                names.add(probeName);
            }

            currLine = nextLine(bin);
        }

        final Probe[] probes = (Probe[]) probesList.toArray(new Probe[probesList.size()]);
        final Chip chip = new FileInMemoryChip(FileUtils.removePath(chipFile.getPath()), chipFile.getPath(), probes);

        bin.close();

        return chip;
    }

    protected String nextLine(final BufferedReader bin) throws IOException {

        String currLine = bin.readLine();
        if (currLine == null) {
            return null;
        }

        currLine = currLine.trim();

        while ((currLine != null) && ((currLine.length() == 0) || (currLine.startsWith(Constants.COMMENT_CHAR)))) {
            currLine = bin.readLine();
            if (currLine != null) {
                currLine = currLine.trim();
            }
        }

        return currLine;
    }

    protected static int indexOf(final String s, final List list, final boolean barfIfMising) throws ParserException {
        int index = list.indexOf(s);
        if (index == -1 && barfIfMising) {
            throw new ParserException("Column not found: " + s);
        }
        return index;
    }
}


