/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtest.api;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ConfImpl implements TestConf {

    /**
     * By default, test properties are loaded from this file
     */
    public static TestConf DEFAULT = null;

    public static final String RESOURCES_JAR_PATH = "edu/mit/broad/genome/resources/"; // DONT use leading slash!

    public static final String DEFAULT_TEST_PROPS_NAME = "default_test.prp";

    public static final String DEFAULT_TEST_PROPS_JAR_PATH = RESOURCES_JAR_PATH + DEFAULT_TEST_PROPS_NAME;

    static {
        try {

            DEFAULT = new ConfImpl(DEFAULT_TEST_PROPS_JAR_PATH);

        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    private static final String BASE_URL = "BASE_URL";
    private final String fLoc;
    private URL fConfUrl;

    // test conf is read into this structire
    private final Properties fProp;

    /**
     * For logging support
     */
    protected final static Logger log = XLogger.getLogger(ConfImpl.class);

    /**
     * Class Constructor.
     * Initializes to specified properties file.
     * DONT USE LOGGING in the constructor!
     */
    public ConfImpl(String loc) {

        if (loc == null) {
            throw new IllegalArgumentException("Parameter loc cannot be null");
        }

        //System.out.println("Initing ConfImp to location: " + loc); // dont use log

        this.fLoc = loc;

        //load Test's own properties
        fProp = new Properties();

        try {
            fConfUrl = JarResources.toURLFromPkgPath(fLoc);
            //System.out.println("### loading from: " + loc + " url: " + fConfUrl + " " + log);
            if (fConfUrl == null) {
                System.out.println("Could not get url for location: " + fLoc);
            }

            InputStream is = fConfUrl.openStream();
            fProp.load(is);
            //System.out.println(">>>>" + fProp);
        } catch (Throwable e) {
            System.out.println("FATAL: Could not read in test properties");
            e.printStackTrace();
            // suppress here as a favor to DEFAULT
        }
    }

    public String getProperty(String key) {

        //log.debug("key=" + key + " and val=" + fProp.getProperty(key));
        if (fProp.getProperty(key) == null) {
            throw new RuntimeException("No such key specified: " + key);
        }

        return fProp.getProperty(key);
    }


    public URL getConfUrl() {
        return fConfUrl;
    }


    /**
     * Returns URL for the root of the test data dir
     */
    public URL getDataRoot() throws MalformedURLException {

        String val = getProperty(BASE_URL);

        if (val == null) {
            throw new TestException("No value for the key " + BASE_URL
                    + " in the test properties file: " + fLoc);
        }


        URL url = JarResources.toURLFromPkgPath(val);

        if (url == null) {
            throw new TestException("Could not load system resource: " + val);
        }

        return url;
    }

    /**
     * Returns a URL object for the specified test resource name
     * Example: getResource(TestConf.RES_URL);
     */
    public URL getUrl(String key) throws MalformedURLException {

        String val = getProperty(key);

        if (val == null) {
            log.info("Could not find resource: " + key);
        }

        URL url = new URL(getDataRoot() + "/" + val);

        return url;
    }

    public File getFile(String key) throws IOException {

        try {
            URL url = getUrl(key);

            if (url == null) {
                throw new IOException("Could not find resource for: " + key);
            }

            File file = new File(url.getFile());

            if (!file.exists()) {
                throw new IOException("File does not exist: " + file.getAbsolutePath());
            }

            return file;
        } catch (MalformedURLException e) {
            throw new IOException(e.getMessage());
        }
    }

    public String toString() {
        return this.getClass().getName() + " with loc " + fLoc;
    }
}    // End ConfImpl

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
