/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtest.api;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.utils.ChainedException;
import junit.framework.TestResult;
import junit.framework.TestSuite;
import org.apache.log4j.Logger;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;


/**
 * Base class for running all tests in a package or some the location
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

// IMP: I am NOT a TestCase or TestSuite!
public class Harness {

    /**
     * For logging support
     */
    protected Logger log;
    protected xtest.api.Test fTest;
    private int fMax = -1;
    private boolean fInited;

    /**
     * Class Constructor.
     */
    public Harness(TestType type, File classPathRoot, Package pkg) throws IOException {

        this();

        init(type, classPathRoot, pkg);
    }

    private Harness() {
    }

    /**
     * Sets the max number of test cases to run when in
     * batch text runner mode.
     * (Not applicable for interactiveRunner and guiRunner)
     * If not set, all appropriate test cases found are run.
     */
    public void setMaxNumberToRun(int max) {
        fMax = max;
    }

    /**
     * Does the real class initialization.
     */
    private void init(TestType type, File classPathRoot, Package pkg) throws IOException {

        fTest = new Test();
        log = XLogger.getLogger(Harness.class);

        System.setProperty(TestAll.TEST_CLASS_ROOT, classPathRoot.getAbsolutePath());

        if (pkg != null) {
            System.setProperty(TestAll.TEST_PACKAGE_ROOT, pkg.getName());
        }

        System.setProperty(TestAll.TEST_TYPE, type.toString());

        fInited = true;
    }

    private void checkInit() {

        if (!fInited) {
            throw new IllegalStateException("Harness not initialized properly");
        }
    }

    public void guiRunner() {

        checkInit();

        junit.swingui.TestRunner runner = new junit.swingui.TestRunner();
        String[] tests = new String[1];

        tests[0] = xtest.api.TestAll.class.getName();

        runner.start(tests);
        log.info("Done launching runner GUI");
    }

    public void textRunner() throws Exception {

        checkInit();

        try {

            // this doesnt give enough summary info at the end of the tests - its hard
            // to make out if there were several tests run.
            //junit.textui.TestRunner.run(edu.mit.broad.genome.edu.mit.broad.xtest.api.TestAll.suite());
            // Thus the stuff below is basically there to provide a nice status reports at the end
            // with: test name ... status
            TestSuite suite = (TestSuite) xtest.api.TestAll.suite();
            Enumeration en = suite.tests();
            junit.textui.TestRunner runner = new junit.textui.TestRunner();
            ArrayList testList = new ArrayList();
            int cnt = 0;

            while (en.hasMoreElements()) {
                cnt++;

                if ((fMax != -1) && (cnt > fMax)) {
                    log.warn("Cutting short number of tests run as max # to run is set to = "
                            + fMax);

                    break;
                }

                TestSuite ts = (TestSuite) en.nextElement();
                String name = getName(ts);

                // i'm not sure if this is synchronized enough??
                // what if the test completes before the listener is added??
                // thats why i changed from a listener to checking the reseukts directly
                TestResult tr = runner.doRun(ts, false);
                MyTestResult mtr = new MyTestResult(name, tr);

                testList.add(mtr);
            }

            printTextRunnerSummary(testList);
        } catch (Throwable t) {
            throw new ChainedException(t);
        }
    }

    // works in concert with textRunner
    // summarize test runs - first the successes and then the failures
    // in alphabetical order
    private void printTextRunnerSummary(ArrayList testList) {

        ArrayList successList = new ArrayList();
        ArrayList failureList = new ArrayList();

        for (int i = 0; i < testList.size(); i++) {
            MyTestResult mtr = (MyTestResult) testList.get(i);

            if (mtr.wasSuccessful()) {
                successList.add(mtr);
            } else {
                failureList.add(mtr);
            }
        }

        Comparator comp = new MyTestResultComparator();

        Collections.sort(successList, comp);
        Collections.sort(failureList, comp);
        System.out.println("-------------------------------------------------");

        for (int i = 0; i < successList.size(); i++) {
            MyTestResult mtr = (MyTestResult) successList.get(i);

            System.out.println(mtr.getResultSummary());
        }

        System.out.println("-------------------------------------------------");

        for (int i = 0; i < failureList.size(); i++) {
            MyTestResult mtr = (MyTestResult) failureList.get(i);

            System.out.println(mtr.getResultSummary());
        }

        if (failureList.size() == 0) {
            System.out.println("No failures");
        }

        System.out.println("-------------------------------------------------");
        System.out.println("# of tests run = " + testList.size());
        System.out.println("# of successes = " + successList.size());
        System.out.println("# of failures  = " + failureList.size());
    }

    private String getName(TestSuite ts) {

        int indx = ts.getName().lastIndexOf(".");

        return ts.getName().substring(indx + 1);
    }

    /**
     * Runs every TestCase file individually and prompts after each one.
     */
    public void interactiveRunner() throws Exception {

        checkInit();

        try {
            TestSuite suite = (TestSuite) xtest.api.TestAll.suite();
            Enumeration en = suite.tests();
            junit.textui.TestRunner runner = new junit.textui.TestRunner();

            while (en.hasMoreElements()) {
                TestSuite ts = (TestSuite) en.nextElement();
                TestResult tr = runner.doRun(ts, true);
                String name = getName(ts);

                System.out.println("---------------------------------------------------------");

                if (tr.wasSuccessful()) {
                    System.out.println("Test: " + name + " ran all " + tr.runCount()
                            + " tests successfully");
                } else {
                    System.out.println("Test: " + name + " had ERRORS = " + tr.errorCount()
                            + " had FAILURES = " + tr.failureCount());
                }

                System.out.println("---------------------------------------------------------");
            }
        } catch (Throwable t) {
            throw new ChainedException(t);
        }
    }

    /**
     * Class MyTestResult
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private class MyTestResult {

        private final String fName;
        private final TestResult fResult;

        public MyTestResult(String testName, TestResult tr) {
            fName = testName;
            fResult = tr;
        }

        public boolean wasSuccessful() {
            return fResult.wasSuccessful();
        }

        // summary must easily fir on one line
        public String getResultSummary() {

            StringBuffer buf = new StringBuffer(fName);

            if (fResult.wasSuccessful()) {
                buf.append(" ... success");
            } else {
                buf.append(" ... FAILED errors=" + fResult.errorCount() + " failures="
                        + fResult.failureCount());
            }

            return buf.toString();
        }

        // to help out MyTestResultComparator
        public String toString() {
            return fName;
        }
    }    // End inner class MyTestResult

    /**
     * Class MyTestResultComparator
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public class MyTestResultComparator implements java.util.Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {
            return pn1.toString().compareTo(pn2.toString());
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End MyTestResultComparator
}        // End Harness

/*
     tr.addListener(this); // need to use listenr as we dont wait (as in interactive)
implements junit.framework.TestListener
public void addError(junit.framework.Test test, java.lang.Throwable t) {

}

public void addFailure(junit.framework.Test test, AssertionFailedError t) {

}

public void endTest(junit.framework.Test test) {
    test.countTestCases();

}

public void startTest(junit.framework.Test test) {
}

*/

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
