/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtest.api;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetMetrics;
import edu.mit.broad.genome.alg.Metrics;
import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.charts.XChartUtils;
import edu.mit.broad.genome.charts.XComboChart;
import edu.mit.broad.genome.math.LabelledVectorProcessor;
import edu.mit.broad.genome.math.LabelledVectorProcessors;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.pages.Page;
import edu.mit.broad.genome.swing.ImageViewerFrame;
import edu.mit.broad.genome.swing.SCLayout;
import edu.mit.broad.genome.viewers.XChartViewer;
import edu.mit.broad.xbench.core.WindowComponent;
import edu.mit.broad.xbench.core.api.Application;
import org.apache.log4j.Logger;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.JFreeChart;
import xapps.api.BareBonesApplication;
import xapps.api.MostMain;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.net.MalformedURLException;
import java.util.HashMap;

/**
 * This is NOT a junit "testCase" class.
 * Rather its a utility class for use when testing/debugging.
 * For instance by real junit-style "testCase" classes.
 * <p/>
 * So this class:
 * <ul>
 * <li> is a collection of methods useful when writing real test classes
 * </ul>
 * <p/>
 * Use the public methods of this class to get references to TEST
 * objects, files etc without having to hardcode paths to the underlying files.
 * The standard test objects are stored under test/data.
 * <p/>
 * DONT use other than for testing!
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @done clearly differentiate between static and non static public methods here
 * non-static are those that need some initialization and hence need Test.init() to have
 * been called.
 * Static methods dont need any initialization - no need for reading the test properties
 * file etc - and hence can be .. static.
 * changed -> not sure why this is needed (plus easier on intellij to have all as non-static)
 */

// dont make me final -- would like to extend

public class Test {

    // Tell Jide we are valid
    static {
        // tells jide and sets lnf
        new MostMain().setLnF();
    }

    /**
     * For logging support
     */
    protected final Logger log = XLogger.getLogger(Test.class);

    /**
     * The configuration that is used to get locations etc of test resources
     */
    private final TestConf fConf;

    private BareBonesApplication bare;

    /**
     * Class Constructor.
     * Initializes to the default test properties source.
     * ConfImpl.DEFAULT
     */
    public Test() {
        this(ConfImpl.DEFAULT);
    }

    /**
     * Class Constructor.
     * Initializes to the specified test configuration.
     * <p/>
     * also does a fake installation procedure useful for testing outside of netbeans
     */
    public Test(TestConf conf) {
        this.fConf = conf;
    }


    public void registerApp() {
        if (bare == null) {
            bare = new BareBonesApplication();
            Application.registerHandler(bare);
        }

    }

    static class BaseDataset {
        File baseDir;

        private String datasetFileName;
        private String datasetCollapsedFileName;
        private String templateFileName;
        //private String gmxFileName;
        //private String gsetFileName;
        private String sampleAnnotFileName;

        BaseDataset(String datasetFileName,
                    String datasetCollapsedFileName,
                    String templateFileName,
                    //String gsetFileName,
                    //String gmxFileName,
                    String sampleAnnotFileName) {

            this.baseDir = Application.getVdbManager().getTestDataDir();
            this.datasetFileName = datasetFileName;
            this.datasetCollapsedFileName = datasetCollapsedFileName;
            this.templateFileName = templateFileName;
            //this.gsetFileName = gsetFileName;
            //this.gmxFileName = gmxFileName;
            this.sampleAnnotFileName = sampleAnnotFileName;
        }

        /*
        public Dataset getDataset(String name) throws Exception {
            return ParserFactory.readDataset(new File(baseDir, name), true);
        }

        public Template getTemplate(String name) throws Exception {
            return ParserFactory.readTemplate(new File(baseDir, name), true);
        }

        public GeneSetMatrix getGeneSetMatrix(String name) throws Exception {
            return ParserFactory.readGenevMatrix(new File(baseDir, name), true);
        }

        public GeneSet getGeneSet(String name) throws Exception {
            return ParserFactory.readGeneSet(new File(baseDir, name), true);
        }
        */

        public Dataset getDataset() throws Exception {
            return ParserFactory.readDataset(new File(baseDir, datasetFileName), true);
        }

        public Dataset getDatasetCollapsed() throws Exception {
            return ParserFactory.readDataset(new File(baseDir, datasetCollapsedFileName), true);
        }

        public Template getTemplate() throws Exception {
            return ParserFactory.readTemplate(new File(baseDir, templateFileName), true);
        }

        /*
        public GeneSetMatrix getGeneSetMatrix() throws Exception {
            return ParserFactory.readGeneSetMatrix(new File(baseDir, gmxFileName), true);
        }

        public GeneSet getGeneSet() throws Exception {
            return ParserFactory.readGeneSet(new File(baseDir, gsetFileName), true);
        }
        */

        public GeneSet getGeneSet_affy() throws Exception {
            return ParserFactory.readGeneSet(new File(baseDir, "test.affy.grp"), true);
        }

        public GeneSet getGeneSet_symbols() throws Exception {
            return ParserFactory.readGeneSet(new File(baseDir, "test.symbols.grp"), true);
        }

        public GeneSetMatrix getGeneSetMatrix_affy() throws Exception {
            return ParserFactory.readGeneSetMatrix(new File(baseDir, "test.affy.gmt"), true);
        }

        public GeneSetMatrix getGeneSetMatrix_symbols() throws Exception {
            return ParserFactory.readGeneSetMatrix(new File(baseDir, "test.symbols.gmt"), true);
        }

        public GeneSet[] getGeneSets_symbols() throws Exception {
            return ParserFactory.readGeneSetMatrix(new File(baseDir, "test.symbols.gmt"), true).getGeneSets();
        }

        public GeneSet[] getGeneSets_symbols_small() throws Exception {
            return ParserFactory.readGeneSetMatrix(new File(baseDir, "test.symbols_small.gmt"), true).getGeneSets();
        }

    } // End class BaseDataset


    public static class AllAml extends BaseDataset {

        public AllAml() {
            super("allaml.gct",
                    "allaml_collapsed.gct",
                    "allaml.cls",
                    ""
            );
        }

    }


    /**
     * Reads in from resource file and returns a test Dataset.
     */
    public Dataset getDataset() throws Exception {
        return ParserFactory.readDataset(fConf.getFile(TestConf.RES_URL), true);
    }

    public ScoredDataset getDatasetScored() throws Exception {

        Dataset ds = getDataset();
        Template t = getTemplate2Class();

        final LabelledVectorProcessor rp = new LabelledVectorProcessors.None();

        DatasetMetrics dm = new DatasetMetrics();
        return dm.scoreDataset(new Metrics.Signal2Noise(), SortMode.REAL, Order.DESCENDING, new HashMap(), rp, ds, t);
    }

    public Dataset getDatasetSmall() throws Exception {
        return ParserFactory.readDataset(fConf.getFile(TestConf.SMALL_RES_URL), true);
    }

    public Dataset getDatasetGct() throws Exception {
        return ParserFactory.readDatasetGct(fConf.getFile(TestConf.GCT_URL), true);
    }

    public HClustDatasetStanford getHClustDatasetSmall() throws Exception {
        return ParserFactory.readHClustDataset(fConf.getFile(TestConf.SMALL_CDT_URL), true);
    }

    public HClustTree getHClustTreeGenesSmall() throws Exception {
        return ParserFactory.readHClustTree(fConf.getFile(TestConf.SMALL_GTR_URL), true);
    }

    public HClustTree getHClustTreeSamplesSmall() throws Exception {
        return ParserFactory.readHClustTree(fConf.getFile(TestConf.SMALL_ATR_URL), true);
    }

    public HClustDatasetStanford getHClustDataset() throws Exception {
        return ParserFactory.readHClustDataset(fConf.getFile(TestConf.CDT_URL), true);
    }

    public HClustTree getHClustTreeGenes() throws Exception {
        return ParserFactory.readHClustTree(fConf.getFile(TestConf.GTR_URL), true);
    }

    public HClustTree getHClustTreeSamples() throws Exception {
        return ParserFactory.readHClustTree(fConf.getFile(TestConf.ATR_URL), true);
    }

    public Dataframe getDataframe() throws Exception {
        return ParserFactory.readDataframe(fConf.getFile(TestConf.DATAFRAME_URL), true);
    }

    public StringDataframe getStringDataframe() throws Exception {
        return ParserFactory.readStringDataframe(fConf.getFile(TestConf.STRINGDATAFRAME_URL), true);
    }

    public FeatureAnnot getDatasetAnnotation() throws Exception {
        Dataset ds = getDataset();
        return ds.getAnnot().getFeatureAnnot();
    }

    /**
     * Reads in from resource file amd returns a test Template
     */
    public Template getTemplate() throws Exception {
        return ParserFactory.readTemplate(fConf.getFile(TestConf.CLS_URL));
    }

    /**
     * Reads in from resource file amd returns a test 2-class Template
     */
    public Template getTemplate2Class() throws Exception {
        return ParserFactory.readTemplate(fConf.getFile(TestConf.CLS2_URL));
    }

    public Template getTemplateNumeric() throws Exception {
        return ParserFactory.readTemplate(fConf.getFile(TestConf.CLS_NUMERIC_URL));
    }

    /**
     * Reads in from resource file amd returns a test FSet
     */
    public GeneSet getGeneSet() throws Exception {
        return ParserFactory.readGeneSet(fConf.getFile(TestConf.GRP_URL), true);
    }

    public GeneSetMatrix getGeneSetMatrix() throws Exception {
        return ParserFactory.readGeneSetMatrix(fConf.getFile(TestConf.GMX_URL), true);
    }

    public GeneSetMatrix getGeneSetMatrixT() throws Exception {
        return (GeneSetMatrix) ParserFactory.read(fConf.getFile(TestConf.GMT_URL));
    }

    public java.awt.Image getJpegImage() throws MalformedURLException {

        //URL url = fConf.getUrl(TestConf.JPG_IMAGE_URL);

        //return Utilities.loadImage(url.getPath());
        return null;
    }


    public void displayInApp(WindowComponent wc) {
        //wc.getWindowUI().revalidate();
        bare.getWindowManager().openWindow(wc);
    }

    /**
     * Utility method to display a component.
     * Creates a new JFrame, adds specified component and displays the JFrame.
     * Size is set to comps preferred size.
     *
     * @see display(int width, int height, Component comp)
     */

    // staic as needs no test init
    public void display(Component comp) {

        Dimension dim = comp.getPreferredSize();

        // sometimes the comp size hasnt been set yet.
        // arbitarily decide that that the case if height < 200
        // and use a standard 500 by 500
        if (dim.height < 200) {
            display(500, 500, comp);
        } else {
            display(dim.width, dim.height + 30, comp);    // +30 makes it look better
        }
    }

    public void display(Component comp1, Component comp2) {
        display(new Component[]{comp1, comp2});
    }

    public void display(Component comp1, Component comp2, Component comp3) {
        display(new Component[]{comp1, comp2, comp3});
    }

    public void display(Component[] comps) {

        JPanel panel = new JPanel(new SCLayout(comps.length));
        for (int i = 0; i < comps.length; i++) {
            panel.add(comps[i]);
        }

        display(panel);
    }

    /**
     * Utility method to display a component.
     * Creates a new JFrame of specified size, adds specified component
     * and displays.
     */

    // staic as needs no test init
    public void display(int width, int height, Component comp) {

        JFrame frame = new JFrame();

        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(comp, BorderLayout.CENTER);
        frame.setSize(width, height);
        frame.setTitle("Test Display");
        frame.setVisible(true);
        // frame.pack();
        //frame.show();
    }

    public void display(final JFrame frame) {
        //frame.setTitle("Test Display");
        frame.setSize(500, 500);
        frame.setVisible(true);
    }

    public void display(final BufferedImage bi) {
        display(new ImageViewerFrame(bi, false, "foo"));
    }

    public void display(JFreeChart chart) {
        XChartUtils.display(chart);
    }

    public void display(XChart chart) {
        XChartUtils.display(chart);
    }

    public void display(XComboChart chart) {
        XChartUtils.display(chart.getCombinedChart());
    }

    public void display(XChart[] charts) {
        XChartViewer v = new XChartViewer("charts _" + charts.length, charts);
        display(v.getWindowUI());
    }

    public void display(String title, JFreeChart chart) {

        ChartFrame frame = new ChartFrame(title, chart);

        frame.pack();
        frame.setVisible(true);
    }

    /**
     * Place the tree in a scrollpane and then display.
     */
    public void display(JTree tree) {
        displayInScrollPane(tree);
    }

    public void display(JTable table) {
        displayInScrollPane(table);
    }

    public void display(Action action) {

        JButton but = new JButton(action);

        display(but);
    }

    public void displayHtml(String html) {
        JEditorPane editorPane = new JEditorPane();
        editorPane.setEditable(false);
        editorPane.setContentType("text/html");
        editorPane.setText(html);
        displayInScrollPane(editorPane);

        System.out.println(html);
    }

    public void displayInScrollPane(Component comp) {
        display(500, 500, new JScrollPane(comp));
    }

    public void save(Page page) throws Exception {
        File file = File.createTempFile(page.getName(), "." + page.getExt());
        log.info("Writting Page to: " + file);
        FileOutputStream os = new FileOutputStream(file);
        page.write(os, file.getParentFile());
        os.close();
    }

}    // End Test

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
