/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtest.api;


import edu.mit.broad.genome.XLogger;
import junit.framework.TestCase;
import org.apache.log4j.Logger;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Iterator;


/**
 * Class TestCaseLoader
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class TestCaseLoader {

    final private ArrayList fClasses = new ArrayList();
    final private TestType fRequiredType;

    /**
     * The name of the fields in a TestCase class that tells this loader what test type it is
     */
    public static final String FIELD_NAME = "TEST_ALL_TEST_TYPE";

    /**
     * For logging support
     */
    private static final Logger klog = XLogger.getLogger(TestCaseLoader.class);

    /**
     * Class Constructor.
     * Load all the test cases possible that derive
     * from <code>baseClass</code> and cannot be ignored.
     *
     * @param classNamesIterator An iterator over a collection of fully qualified class names
     */
    public TestCaseLoader(final TestType reqdTestType) {

        if (reqdTestType == null) {
            throw new NullPointerException("Parameter reqdTestType is null");
        }

        this.fRequiredType = reqdTestType;
    }

    /**
     * Load the classes that represent test cases we are interested in.
     *
     * @param classNamesIterator An iterator over a collection of fully qualified class names
     */
    public void loadTestCases(final Iterator classNamesIterator) {

        while (classNamesIterator.hasNext()) {
            String className = (String) classNamesIterator.next();

            try {
                Class candidateClass = Class.forName(className);

                addClassIfTestCase(candidateClass);
            } catch (ClassNotFoundException e) {    // these are java.reflec errors
                klog.error("Cannot load class: " + className + " " + e.getMessage());
            } catch (NoClassDefFoundError e) {
                klog.error("Cannot load class that " + className + " is dependant on");
            }
        }
    }

    /**
     * Obtain an iterator over the collection of test case classes loaded by <code>loadTestCases</code>
     */
    public Iterator getClasses() {
        return fClasses.iterator();
    }

    /**
     * Adds <code>testCaseClass</code> to the list of classdes
     * if the class is a test case we wish to load. Calls
     * <code>shouldLoadTestCase ()</code> to determine that.
     */
    private void addClassIfTestCase(final Class testCaseClass) {

        if (shouldAddTestCase(testCaseClass)) {
            fClasses.add(testCaseClass);
        }
    }

    /**
     * Determine if we should load this test case.
     */
    private boolean shouldAddTestCase(final Class testCaseClass) {

        boolean isOfTheCorrectType = false;

        if (TestCase.class.isAssignableFrom(testCaseClass)) {
            try {

                // as: using getField works better than getDecalaredField as it checks superclasses too.
                Field testAllIgnoreThisField = testCaseClass.getField(FIELD_NAME);

                //JField testAllIgnoreThisField = testCaseClass.getDeclaredField(FIELD_NAME);
                final int EXPECTED_MODIFIERS = Modifier.STATIC | Modifier.PUBLIC | Modifier.FINAL;

                if (((testAllIgnoreThisField.getModifiers() & EXPECTED_MODIFIERS) != EXPECTED_MODIFIERS)
                        || (testAllIgnoreThisField.getType() != TestType.class)) {
                    throw new IllegalAccessException(FIELD_NAME
                            + " should be static public final String");
                }

                TestType testType = (TestType) testAllIgnoreThisField.get(testCaseClass);

                isOfTheCorrectType = fRequiredType.equals(testType);

                // we dont want to barf on fields exceptions - just give a diagnosis and proceed with the rest
            } catch (NoSuchFieldException e) {
                klog.info("Ignoring " + testCaseClass.getName()
                        + " as it does not define the magic fields " + FIELD_NAME);
            } catch (IllegalAccessException e) {
                klog.warn("Invalid accessors for the fields: " + testCaseClass.getName() + "."
                        + FIELD_NAME + " ... ignoring for tests", e);
            } catch (Throwable t) {
                klog.warn("Unexpected error for the fields: " + testCaseClass.getName() + "."
                        + FIELD_NAME + " ... ignoring for tests", t);
            }
        }

        return isOfTheCorrectType;
    }
}    // End TestCaseLoader

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
