/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api;


import edu.mit.broad.genome.Conf;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.GeneSetGenerators;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.reports.api.Report;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.genome.reports.api.ToolReport;
import edu.mit.broad.genome.utils.ClassUtils;
import edu.mit.broad.genome.utils.CmdLineArgs;
import edu.mit.broad.genome.utils.SystemUtils;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.xbench.core.api.Application;
import org.apache.ecs.html.Div;
import org.apache.ecs.html.H4;
import org.apache.log4j.Logger;
import xtools.api.param.*;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.lang.reflect.Field;
import java.util.*;


/**
 * Interface for a runnable (cmd line or through junit) Tool.
 * <p/>
 * TestCase so that a Tool can be run as a junit test
 * (similar to UnitTest)
 * <p/>
 * Tool impls can be run from junit runner or from cmd line
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public abstract class AbstractTool extends UnitTestTool implements Tool {

    // @imp note: needed here as otherwise jar resources tries to load icons
    static {
        if (!SystemUtils.isPropertyDefined("java.awt.headless")) {
            System.setProperty("java.awt.headless", "true");
        }
    }

    public static final String REPORT_INDEX = "xtools.report_index";

    private PrintStream fOut;

    /**
     * For logging support
     */
    protected transient Logger log;

    protected static final transient Logger klog = XLogger.getLogger(AbstractTool.class);

    protected ToolParamSet fParamSet;

    private edu.mit.broad.genome.utils.Timer fTimer;

    private boolean fHelpMode;

    protected static final Object[] EMPTY_OBJECTS = new Object[]{};

    private boolean fKillRequested;

    /**
     * ----------- PARAMETERS COMMON TO ALL TOOLS -------------
     */
    protected final BooleanParam fHelpParam = new BooleanParam("help",
            "Usage information on the tool", false);

    private ReportDirParam fReportDirParam;
    private GuiParam fGuiParam;
    private ReportLabelParam fRptLabelParam;
    protected ToolReport fReport;

    /**
     * @maint If the standard params above are updated, this var needs attention
     */
    private static int SHARED_PARAM_CNT = 4;

    /**
     * Class constructor
     */
    protected AbstractTool() {
        this("dummy");
    }

    public String getHelpURL() {
        return JarResources.getHelpURL(getName());
    }

    public void setOutputStream(final PrintStream sout) {
        if (sout == null) {
            throw new IllegalArgumentException("Param sout cannot be null");
        }

        this.fOut = sout;
    }

    public PrintStream getOutputStream() {
        if (fOut == null) {
            fOut = System.out; // default
        }

        return fOut;
    }

    /**
     * Only for junit usage
     *
     * @param toolName
     */

    // Dont call declareParams()  - class vars arent yet inited
    // constructed using instantiation and npe is thrown.
    protected AbstractTool(final String toolName) {

        super(toolName);
        this.fTimer = new edu.mit.broad.genome.utils.Timer();
        //this.fHelpMode = Boolean.getBoolean(System.getProperty("help")); // just doesnt work!!
        //this.fHelpMode = SystemUtils.isPropertyTrue("help");
        this.log = XLogger.getLogger(this.getClass());

        if (Application.isHandlerSet() == false) {
            Application.registerHandler(new XToolsApplication());
        }

        this.fParamSet = new ToolParamSet();

        this.fGuiParam = new GuiParam();
        this.fRptLabelParam = new ReportLabelParam(false);
        this.fReportDirParam = new ReportDirParam(false);

        // @note always add the common ones
        fParamSet.addParam(fGuiParam);
        fParamSet.addParam(fReportDirParam);
        fParamSet.addParam(fRptLabelParam);
    }

    /**
     * subclasses must call in their class constructor after a call to super()
     */
    protected void init(final Properties prp) {

        fParamSet.addParam(fHelpParam);

        declareParams();

        if (prp != null) {
            // by default, if help spec do usage
            if (prp.containsKey("help") || prp.containsKey("HELP")) {
                this.fHelpMode = true;
                /*
                String val = prp.getProperty("help");
                if (val != null && val.equalsIgnoreCase("false")) {
                    this.fHelpMode = false;
                } else {
                    this.fHelpMode = true;
                }
                */
            }
        }

        //System.out.println(">>>> prp: " + prp + " " + fHelpMode + " val>>" + prp.getProperty("help"));

        if (isHelpMode()) {
            // no filling nor checking
            fParamSet.printfUsage();
            Conf.exitSystem(false);
        } else {
            fParamSet.fill(prp);
            //ParamSet.printf();
            //fParamSet.check();
            //ensureAllDeclaredWereAdded();
            checkHeadlessState();
        }

    }

    protected void init(final String[] args) {

        try {

            Properties prp = CmdLineArgs.parse(args);

            String argline = CmdLineArgs.toString(args);
            int index = argline.indexOf("help");
            if (index != -1) {
                prp.setProperty("help", Boolean.TRUE.toString());
            }

            String param_file_path = prp.getProperty(ParamSet.PARAM_FILE);
            if (param_file_path != null && param_file_path.length() > 0) {
                enhanceParams(param_file_path, prp);
            }

            init(prp);

        } catch (Throwable t) {
            t.printStackTrace();
            // if the rpt dir was made try to rename it so that easily identifiable
            if (fReport != null) {
                fReport.setErroredOut();
            } else {
                log.info("No report dir was made yet (analysis errored out)");
            }

            Conf.exitSystem(true);
        }
    }

    private void enhanceParams(final String filepathORurl, final Properties prp) throws Exception {
        Properties addPrp = ParseUtils.readKeyVal(filepathORurl, false, true, false);
        for (Iterator it = addPrp.keySet().iterator(); it.hasNext();) {
            String param_name = it.next().toString();
            String param_val = addPrp.getProperty(param_name);
            if (prp.containsKey(param_name)) {
                String extant_param_val = prp.getProperty(param_name);
                if (!param_val.equals(extant_param_val)) {
                    log.warn("Ignoring param_file key: " + param_name + " value: " + param_val + " in favor of cmd line value: " + extant_param_val);
                }
            } else {
                prp.setProperty(param_name, param_val);
            }
        }
    }

    protected void startExec() throws IOException {
        startExec(getHeader());
    }

    protected String getHeader() {
        StringBuffer buf = new StringBuffer();
        buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                "<h3 style=\"text-align: left\"><font color=\"#808080\">Report for: ").append(getName()).append("</font></h3>\n").append("</div>");
        return buf.toString();
    }

    protected void startExec(final String optHeader) throws IOException {
        this.startExec(makeReportIndexPage(), optHeader); // leave report sub dir making ON
    }

    // @note this is the core start report method
    protected void startExec(final ReportIndexState indexState) throws IOException {
        fTimer.start();
        fReport = new ToolReport(this, true, indexState);
        //log.info("Running " + getName() + " with reports: " + fRptLabelParam.getReportLabel() + " folder: " + fReport.getReportDir() + " indexState: " + indexState.toString());
    }

    protected void startExec(boolean makeReportIndexPage, boolean keepTrackOfPagesInHtmlIndex, boolean keepTrackOfPages) throws IOException {
        startExec(new ReportIndexState(makeReportIndexPage, keepTrackOfPagesInHtmlIndex, keepTrackOfPages, getHeader()));
    }

    protected void startExec(boolean makeReportIndexPage) throws IOException {
        startExec(makeReportIndexPage, getHeader());
    }

    protected void startExec(boolean makeReportIndexPage, boolean keepTrackOfPagesInHtmlIndex) throws IOException {
        startExec(new ReportIndexState(makeReportIndexPage, keepTrackOfPagesInHtmlIndex, false, getHeader()));
    }

    protected void startExec(final boolean makeReportIndexPage, final String headerOpt) throws IOException {
        this.startExec(new ReportIndexState(makeReportIndexPage, headerOpt));
    }

    private boolean makeReportIndexPage() {
        String p = System.getProperty(REPORT_INDEX);

        if (p == null || p.length() == 0) {
            return true; // @note IMP if not specified then it is ON
        } else {
            return Boolean.valueOf(p).booleanValue();
        }
    }

    public void doneExec() {
        //fOut.close(); // dont let the giver take care of that

        if (fReport.getToolComments().isEmpty() == false) {
            Div div = new Div();
            H4 h4 = new H4("Comments");
            div.addElement(h4);
            div.addElement(fReport.getToolComments().toHTML());
            if (fReport.getIndexPage() != null) {
                fReport.getIndexPage().addBlock(div, false);
            }
        }

        fReport.closeReport(true);

        if (fGuiParam.isTrue()) {
            fReport.display();
        }

        fTimer.stop();
        fTimer.printTimeTakenS();
    }

    public Report getReport() {
        return fReport;
    }

    /**
     * Make sure that if we are headless (aka on unix, lsf) that the GUI param is always off
     */
    private void checkHeadlessState() {

        /* comm out dec 18
        if (GraphicsEnvironment.isHeadless()) {
            GuiParam gui = fParamSet.getGuiParam();
            if (gui == null) {
                gui = new GuiParam();
                fParamSet.addParam(gui);
            }
            gui.setValue(false); // always
        }
        */
    }

    private int countParamFields(Class cl) {
        //log.debug("Counting for class name: " + cl.getName());
        Field[] fields = cl.getDeclaredFields();
        int paramcnt = 0;

        for (int i = 0; i < fields.length; i++) {

            //log.debug(("name = " + fields[i].getName() + " " + fields[i].getDeclaringClass() + " class: " + fields[i].getType()));
            Class fc = fields[i].getType();
            boolean isParam = Param.class.isAssignableFrom(fc);

            if (isParam) {
                paramcnt++;
                //log.debug("Param: " + fields[i].getName());
            }
        }

        return paramcnt;
    }

    public void requestKill() {
        fKillRequested = true;
    }

    protected boolean isKillRequested() {
        return fKillRequested;
    }

    /**
     * problem -> declared variables but forget to add to param set
     * (its in a diff method)
     * cant automagically add all declared variables -> relect not possible
     * So instread use reflect to check (by count) that all declared vars have been added
     * <p/>
     * IMP: However, doesnt work if the tool is another abstract subclass (make the subclass run the cjheck, see above)
     * Fixed, in a hack way: hardcoded name of this class
     * Only supports 1 additional subclass!!
     */
    protected void ensureAllDeclaredWereAdded() {

        int paramCnt = SHARED_PARAM_CNT;

        paramCnt += countParamFields(this.getClass());

        if (!this.getClass().getSuperclass().getName().equals("xtools.api.AbstractTool")) { // @note
            paramCnt += countParamFields(this.getClass().getSuperclass());
        }

        if (!this.getClass().getSuperclass().getSuperclass().getName().equals("xtools.api.AbstractTool")) { // @note
            paramCnt += countParamFields(this.getClass().getSuperclass().getSuperclass());
        }

        if (fParamSet.getNumParams() != paramCnt) {
            StringBuffer buf =
                    new StringBuffer("Have you forgotten to update declareParams()?").append('\n');

            buf.append("In ParamSet # declared: ").append(fParamSet.getNumParams()).append(" is NOT equal to deduced thro reflection # : ").append(paramCnt);

            throw new IllegalStateException(buf.toString());
        }

    }

    public boolean isHelpMode() {
        return fHelpMode;
    }

    public ParamSet getParamSet() {
        fParamSet.sort();
        return fParamSet;
    }

    public String getName() {
        return this.getClass().getName();
    }

    public String getTitle() {
        String sn = ClassUtils.shorten(getName());
        String desc = getDesc();

        if (desc == null || desc.length() == 0) {
            return "<html><body><b>" + sn + "</b>" + "</body></html>";
        } else {
            return "<html><body><b>" + sn + "</b>: " + desc + "</body></html>";
        }

        //return "<html><body><b>" + sn.toUpperCase() + "</b>: Set parameters and run enrichment tests</body></html>";
    }

    // Real tools must override
    public String getDesc() {
        return "";
    }

    public void printfUsage() {
        fParamSet.printfUsage();
    }

    public long getExecutionTime() {
        return fTimer.getTimeTaken();
    }

    public long getStartTime() {
        return fTimer.getStartTime();
    }

    public long getStopTime() {
        return fTimer.getStopTime();
    }

    protected static void addDT(DatasetReqdParam dp, TemplateMultiChooserParam tp, List dssList, List tssList) throws Exception {
        if ((dp.isSpecified() && !tp.isSpecified()) || (!dp.isSpecified() && tp.isSpecified())) {
            throw new IllegalArgumentException("Both ds and template (or neither) must be specified");
        }

        if (!dp.isSpecified()) {
            return;
        }

        final Dataset fullDs = dp.getDataset();
        final Template[] templates = tp.getTemplates(fullDs);
        for (int i = 0; i < templates.length; i++) {
            Template t = templates[i];
            tssList.add(t);
            dssList.add(fullDs);
        }
    }

    protected static void addDT(DatasetOptParam dp, TemplateMultiChooserParam tp, List dssList, List tssList) throws Exception {
        if (!dp.isSpecified()) {
            return;
        }

        if (!tp.isSpecified()) {
            throw new IllegalArgumentException("No tss specified while dataset param is specified is invalid");
        }

        final Dataset fullDs = dp.getDataset();
        final Template[] templates = tp.getTemplates(fullDs);
        //DatasetGenerators gen = new DatasetGenerators();
        for (int i = 0; i < templates.length; i++) {
            Template t = templates[i];
            tssList.add(t);
            //dssList.add(gen.extract(fullDs, t)); // @note na after t changes
            dssList.add(fullDs);
        }
    }

    public Dataset getDataset(final DatasetOptParam predResults, final DataframeOptParam predResults_sdf) throws Exception {

        ensureAtLeastAndOnlyOneSpecified(new Param[]{predResults, predResults_sdf});

        if (predResults != null && predResults.isSpecified()) {
            return predResults.getDataset();
        }

        if (predResults_sdf != null && predResults_sdf.isSpecified()) {
            return predResults_sdf.getDataframe().toDataset(true);
        }

        throw new IllegalArgumentException("Either dataset or d must be specified");
    }

    protected static void enforceDatasetEqualRowLengths(DatasetReqdParam a, DatasetOptParam[] params) throws Exception {

        int numRow = a.getDataset().getNumRow();

        for (int i = 0; i < params.length; i++) {
            if (params[i].isSpecified()) {
                if (numRow != params[i].getDataset().getNumRow()) {
                    throw new IllegalArgumentException("num rows in datasets must be equal a: " + numRow + " but found: " + params[i].getDataset().getNumRow() + " at pos: " + i + " " + params[i].getDataset().getName());
                }
            }
        }
    }

    public static void ensureIfSpecifiedThenBothAreSpecified(Param a, Param b) {

        boolean gotA = false;

        if (a != null && a.isSpecified()) {
            gotA = true;
        }


        boolean gotB = false;

        if (b != null && b.isSpecified()) {
            gotB = true;
        }

        if ((gotA && !gotB) || (!gotA && gotB)) {
            if (a != null && b != null) {
                throw new IllegalArgumentException("If specified then both of these params need to be specified: a: " + a.getName() + " b: " + b.getName());
            } else {
                throw new IllegalArgumentException("If specified then both of these params need to be specified");
            }
        }

    }

    public static void ensureIfSpecifiedThenOnlyOneIsSpecified(Param a, Param b) {

        boolean gotA = false;

        if (a != null && a.isSpecified()) {
            gotA = true;
        }

        boolean gotB = false;

        if (b != null && b.isSpecified()) {
            gotB = true;
        }

        if (gotA && gotB) {
            throw new IllegalArgumentException("If specified then only one of these params need to be specified: a: " + a.getName() + " b: " + b.getName());
        }

    }

    protected void ensureAtLeastAndOnlyOneSpecified(Param[] params) {

        if (params == null) {
            throw new IllegalArgumentException("Parameter params cannot be null");
        }

        int numTrue = 0;
        for (int i = 0; i < params.length; i++) {
            if (params[i].isSpecified()) {
                numTrue++;
            }
        }

        if (numTrue == 1) {
            return;
        }

        StringBuffer buf = new StringBuffer("Total Specified: " + numTrue).append('\n');
        for (int i = 0; i < params.length; i++) {
            buf.append(params[i].getName()).append(' ');
        }

        throw new IllegalArgumentException("At LEAST & only ONE of the following parameters must be specified: " + buf.toString());

    }

    protected void ensureAtLeastOneSpecified(final Param[] params) {

        if (params == null) {
            throw new IllegalArgumentException("Parameter params cannot be null");
        }

        int numTrue = 0;
        for (int i = 0; i < params.length; i++) {
            if (params[i].isSpecified()) {
                numTrue++;
            }
        }

        if (numTrue > 0) {
            return;
        }

        StringBuffer buf = new StringBuffer("Total Specified: " + numTrue).append('\n');
        for (int i = 0; i < params.length; i++) {
            buf.append('>').append(params[i].getName()).append("< ").append(params[i].getDesc()).append('\n');
        }

        throw new IllegalArgumentException("At LEAST of the following parameters must be specified: " + buf.toString());
    }

    protected static void tool_main(final AbstractTool tool) {

        if (tool == null) {
            throw new IllegalArgumentException("Param tool cannot be null");
        }

        boolean was_error = false;
        try {

            tool.execute();

        } catch (Throwable t) {
            // if the rpt dir was made try to rename it so that easily identifiable
            was_error = true;
            t.printStackTrace();
        }

        if (was_error && tool.getReport() != null) {
            tool.getReport().setErroredOut();
        }

        if (tool.getParamSet().getGuiParam().isFalse()) {
            Conf.exitSystem(was_error);
        } else {
            // dont exit!!
        }
    }

    protected static String removeExtension(PersistentObject pob) {
        return NamingConventions.removeExtension(pob);
    }

    /**
     * A number of Helper methods for dealing with params, common extractions etc
     */
    public static class Helper {

        public static String[] getStringsArray(final StringMultiInputParam namesParam,
                                               final FileParam namesFofnParam) throws IOException {
            Set patternsSet = getStrings(namesParam, namesFofnParam);
            return (String[]) patternsSet.toArray(new String[patternsSet.size()]);
        }

        public static Set getStrings(final StringMultiInputParam namesParam,
                                     final FileParam namesFofnParam) throws IOException {
            Set patternsSet;

            // If patterns is specified in a file then the ES details page is NEVER made
            // Otherwise, it is made if the dataset etc params are all specified

            if (namesParam.isSpecified()) {
                patternsSet = namesParam.getStringsS();
            } else if (namesFofnParam.isSpecified()) {
                File f = namesFofnParam.getFile();
                patternsSet = ParseUtils.slurpIntoSet(f, true);
            } else {
                throw new IllegalArgumentException("Either 1 of pattern or file of patterns must be specified");
            }

            return patternsSet;
        }


        public static GeneSet[] getGeneSets(final Dataset ds,
                                            final GeneSetMatrixMultiChooserParam gmxParam,
                                            final IntegerParam geneSetMinSizeParam,
                                            final IntegerParam geneSetMaxSizeParam) throws Exception {

            final GeneSet[] gsets = gmxParam.getGeneSetMatrixCombo(false).getGeneSets();
            return getGeneSets(ds, gsets, geneSetMinSizeParam, geneSetMaxSizeParam);
        }


        public static GeneSet[] getGeneSets(final RankedList rl,
                                            final GeneSetMatrixMultiChooserParam gmxParam,
                                            final IntegerParam geneSetMinSizeParam,
                                            final IntegerParam geneSetMaxSizeParam) throws Exception {

            final GeneSet[] gsets = gmxParam.getGeneSetMatrixCombo(false).getGeneSets();
            return getGeneSets(rl, gsets, geneSetMinSizeParam, geneSetMaxSizeParam);
        }

        // The magic here is:
        // the ds and the gene sets have to match
        // The ds prior to this call was either collapsed or not collapsed
        // If not collapsed, do nothing at all. If it was collapsed
        // If they dont match (i.e all gsets are 0) using the chip to map the gsets
        // in two ways:
        // 1) from their source format to gene symbols
        public static GeneSet[] getGeneSets(final Object ds_or_rl,
                                            GeneSet[] gsets,
                                            final IntegerParam geneSetMinSizeParam,
                                            final IntegerParam geneSetMaxSizeParam) throws Exception {

            if (geneSetMaxSizeParam.getIValue() < geneSetMinSizeParam.getIValue()) {
                throw new IllegalArgumentException("Max size cannot be less than min size");
            }

            klog.info("Got gsets: " + gsets.length + " now preprocessing them ... min: " + geneSetMinSizeParam.getIValue() + " max: " + geneSetMaxSizeParam.getIValue());

            if (geneSetMinSizeParam.getIValue() != geneSetMaxSizeParam.getIValue()) {
                //int bef = gsets.length;
                boolean do_cloning;
                if (geneSetMinSizeParam.isSpecified()) {
                    gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, geneSetMinSizeParam.getIValue(), ds_or_rl, true);
                    do_cloning = false;
                } else {
                    do_cloning = true;
                }

                klog.info("Done preproc for smaller than: " + geneSetMinSizeParam.getIValue());

                if (geneSetMaxSizeParam.isSpecified()) {
                    gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, geneSetMaxSizeParam.getIValue(), ds_or_rl, do_cloning);
                }


            } else { // @note hack
                klog.info("Skipped gene set size filtering");
            }

            klog.debug("Done geneset preproc starting analysis ...");

            // Finally remove all 0 size gene sets (if min is 0 these will still be in there)
            return removeAllZeroMemberSets(gsets);
        }

        private static GeneSet[] removeAllZeroMemberSets(final GeneSet[] gsets) {

            // Finally remove all 0 size gene sets (if min is 0 these will still be in there)
            List list = new ArrayList();
            for (int i = 0; i < gsets.length; i++) {
                if (gsets[i].getNumMembers() > 0) {
                    list.add(gsets[i]);
                }
            }

            return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
        }

        public static GeneSet[] getGeneSets(final GeneSetMatrixMultiChooserParam gmxParam,
                                            final IntegerParam geneSetMinSizeParam,
                                            final IntegerParam geneSetMaxSizeParam) throws Exception {

            if (geneSetMaxSizeParam.getIValue() < geneSetMinSizeParam.getIValue()) {
                throw new IllegalArgumentException("Max size cannot be less than min size");
            }

            // @note IMP the geneset qual done on the original (unextracted) dataset
            GeneSet[] gsets = gmxParam.getGeneSetMatrixCombo(false).getGeneSets();

            klog.info("Got gsets: " + gsets.length + " now preprocessing them ...");

            if (geneSetMinSizeParam.getIValue() != geneSetMaxSizeParam.getIValue()) {
                if (geneSetMinSizeParam.isSpecified()) {
                    gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, geneSetMinSizeParam.getIValue());
                }

                klog.info("Done preproc for smaller than: " + geneSetMinSizeParam.getIValue());

                if (geneSetMaxSizeParam.isSpecified()) {
                    gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, geneSetMaxSizeParam.getIValue());
                }

            } else { // @note hack
                klog.info("Skipped gene set size filtering");
            }

            klog.debug("Done geneset preproc starting analysis ...");

            // Finally remove all 0 size gene sets (if min is 0 these will still be in there)
            return removeAllZeroMemberSets(gsets);
        }
    }

    public static GeneSet[] getGeneSets(final GeneSetOptParam[] gsetParams) throws Exception {

        List sets = new ArrayList();
        for (int i = 0; i < gsetParams.length; i++) {
            if (gsetParams[i].isSpecified()) {
                sets.add(gsetParams[i].getGeneSet());
            }
        }

        return (GeneSet[]) sets.toArray(new GeneSet[sets.size()]);
    }

    protected static SampleAnnot getSampleAnnot(final SampleAnnotParam ap) throws Exception {
        if (ap != null && ap.isSpecified()) {
            return ap.getAnnot();
        }

        // Try the default finally
        SampleAnnot def = Application.getVdbManager().getDefaultSampleAnnot();
        if (def != null) {
            return def;
        }

        // give up
        return null;

    }

    public static void setSampleAnnot(final Dataset ds, final SampleAnnotParam sannotParam) throws Exception {
        if (sannotParam != null && sannotParam.isSpecified()) {
            ds.getAnnot().setSampleAnnot(sannotParam.getAnnot());
        }
    }

    public static void setSampleAnnot(final Dataset ds, final SampleAnnot sannot) {
        ds.getAnnot().setSampleAnnot(sannot);
    }

    public static void setChip(final Dataset ds, final ChipOptParam chipOptParam, final GenesOfInterestParam ginParam) throws Exception {

        if (chipOptParam != null && chipOptParam.isSpecified()) {
            setChip(ds, chipOptParam.getChip(), ginParam);
        }

    }


    public static void setChip(final Dataset ds, final ChipOptParam chipOptParam) throws Exception {
        setChip(ds, chipOptParam, null);
    }

    public static void setChip(final Dataset ds, final ChipChooserMultiParam chipOptParam, final GenesOfInterestParam ginParam) throws Exception {

        if (chipOptParam != null && chipOptParam.isSpecified()) {
            setChip(ds, chipOptParam.getChipCombo(), ginParam);
        }

    }

    public static void setChip(final Dataset ds, final ChipChooserMultiParam chipOptParam) throws Exception {

        if (chipOptParam != null && chipOptParam.isSpecified()) {
            setChip(ds, chipOptParam.getChipCombo(), (GenesOfInterestParam) null);
        }

    }

    public static void setChip(final Dataset ds, final ChipReqdParam chipParam, final GenesOfInterestParam ginParam) throws Exception {

        if (chipParam != null) {
            setChip(ds, chipParam.getChip(), ginParam);
        }

    }

    public static void setChip(final Dataset ds, final Chip chip) throws Exception {
        setChip(ds, chip, (GenesOfInterest) null);
    }

    public static void setChip(final Dataset ds, final Chip chip, final GenesOfInterestParam ginParam_opt) throws Exception {

        GenesOfInterest gin = null;

        if (ginParam_opt != null && ginParam_opt.isSpecified()) {
            gin = ginParam_opt.getGenesOfInterest();
        }
        setChip(ds, chip, gin);
    }

    public static void setChip(final Dataset ds, final Chip chip, final GenesOfInterest gin_opt) throws Exception {

        if (chip != null) {
            ColorMap.Rows cmr = null;
            if (gin_opt != null) {
                cmr = gin_opt.getAsColorMap();
            } else {
                if (VdbRuntimeResources.getDefaultGenesOfInterest() != null) {
                    cmr = VdbRuntimeResources.getDefaultGenesOfInterest().getAsColorMap();
                }
            }
            ds.getAnnot().setChip(chip, cmr);
        }

    }

    protected static File getFile(String ext, File dir) {
        File[] files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            if (files[i].getName().endsWith(ext)) {
                return files[i];
            }
        }

        throw new IllegalArgumentException("No file with ext: " + ext + " found in: " + dir);
    }


    protected static boolean isCreateSubDir(final Object[] objs) {
        if (objs == null) {
            return false;
        }

        return objs.length > 1;
    }

    protected static Template[] getTemplates(Dataset fullDs,
                                             TemplateMultiChooserParam fTemplatesParam,
                                             ProbesParam fProbesNamesParam) throws Exception {

        if (fTemplatesParam.isSpecified()) {
            return fTemplatesParam.getTemplates(fullDs);
        } else if (fProbesNamesParam.isSpecified()) {
            String[] names = fProbesNamesParam.getProbeNames();
            return TemplateFactory.createContinuousTemplates(names, fullDs);
        } else {
            return new Template[]{};
        }

    }

}    // End AbstractTool

/* trouble is cohort does its own extraction thing so double extraction!???
protected static Ffn.Cohort[] getCohorts(DatasetParam dsParam, TemplateMultiParam tParam) throws Exception{
    List cohorts = new ArrayList();
    Dataset fullDs = new Dataset
    Template[] templates = tParam.getTemplates();
    for (int i=0; i < templates.length; i++) {
        cohorts.add(new Ffn.Cohort(ds, templates[i]);
    }

    return (Ffn.Cohort[])cohorts.toArray(new Ffn.Cohort[cohorts.size()]);

}

        */

