/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.TraceUtils;
import edu.mit.broad.genome.alg.GeneSetGenerators;
import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.io.FTPUtils;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.MGeneSet;
import edu.mit.broad.vdb.map.MGeneSetMatrix;
import edu.mit.broad.xbench.RendererFactory2;
import edu.mit.broad.xbench.core.ObjectBindery;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;
import org.apache.maven.artifact.versioning.DefaultArtifactVersion;
import org.genepattern.uiutil.FTPFile;
import org.genepattern.uiutil.FTPList;
import xapps.gsea.GseaWebResources;
import xtools.api.ui.GeneSetMatrixChooserUI;
import xtools.api.ui.NamedModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;
import java.util.List;
import xtools.gsea.GsaaSeqSP;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
abstract class GeneSetMatrixChooserAbstractParam extends AbstractParam {

    private boolean fMultipleAllowed = true;

    private GeneSetMatrixChooserAbstractParam.MyPobActionListener fAl;

    protected GeneSetMatrixChooserUI fChooser;

    private boolean fIncludeGmx = true;

    /**
     * Class constructor
     *
     * @param name
     * @param nameEnglish
     * @param desc
     * @param reqd
     * @param multipleAlllowed
     * @param includeGeneSetMatrix
     */
    GeneSetMatrixChooserAbstractParam(final String name,
                                      final String nameEnglish,
                                      final String desc,
                                      final boolean reqd,
                                      final boolean multipleAlllowed,
                                      final boolean includeGeneSetMatrix) {
        super(name, nameEnglish, GeneSetMatrix[].class, desc, new GeneSetMatrix[]{}, new GeneSetMatrix[]{}, reqd);
        this.fMultipleAllowed = multipleAlllowed;
        this.fIncludeGmx = includeGeneSetMatrix;
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param reqd
     * @param multipleAlllowed
     * @param includeGmx
     */
    GeneSetMatrixChooserAbstractParam(final String name,
                                      final String desc,
                                      final boolean reqd,
                                      final boolean multipleAlllowed,
                                      final boolean includeGmx) {
        super(name, GeneSetMatrix[].class, desc, new GeneSetMatrix[]{}, new GeneSetMatrix[]{}, reqd);
        this.fMultipleAllowed = multipleAlllowed;
        this.fIncludeGmx = includeGmx;
    }

    public GeneSet[] getGeneSets(final List gsetNames) throws Exception {
        final GeneSet[] gsets = getGeneSetMatrixCombo(true).getGeneSets();
        final List<GeneSet> use = new ArrayList<GeneSet>();
        for (int i = 0; i < gsets.length; i++) {
            if (gsetNames.contains(gsets[i].getName())) {
                use.add(gsets[i]);
            }
        }

        return (GeneSet[]) use.toArray(new GeneSet[use.size()]);
    }

    public GeneSet[] getGeneSets(final IntegerParam minParam,
                                 final IntegerParam maxParam,
                                 final Set restrictToThese,
                                 final boolean removeNativeGmNames) throws Exception {
        final GeneSet[] gsets = getGeneSetMatrixCombo(removeNativeGmNames).getGeneSets();
        final int min = minParam.getIValue();
        final int max = maxParam.getIValue();
        return GeneSetGenerators.removeGeneSetsBySize(gsets, min, max, restrictToThese);
    }

    public GeneSet[] getGeneSets(final IntegerParam minParam,
                                 final IntegerParam maxParam,
                                 final Dataset ds,
                                 final boolean removeNativeGmNames) throws Exception {
        final int min = minParam.getIValue();
        final int max = maxParam.getIValue();
        return getGeneSets(min, max, ds, removeNativeGmNames);
    }

    public GeneSet[] getGeneSets(final int min,
                                 final int max,
                                 final Dataset ds,
                                 final boolean removeNativeGmNames) throws Exception {

        GeneSet[] gsets = getGeneSetMatrixCombo(removeNativeGmNames).getGeneSets();
        return GeneSetGenerators.removeGeneSetsBySize(gsets, min, max, ds);
    }

    public GeneSet[] getGeneSets(final IntegerParam minParam,
                                 final IntegerParam maxParam,
                                 final DatasetOptParam dsParam) throws Exception {

        GeneSet[] gsets = _getGeneSets().gsets;

        boolean do_cloning = true;

        if (dsParam.isSpecified()) {
            if (minParam.isSpecified()) {
                gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, minParam.getIValue(), dsParam.getDataset(), do_cloning);
                do_cloning = false;
            }

            if (maxParam.isSpecified()) {
                gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, maxParam.getIValue(), dsParam.getDataset(), do_cloning);
            }
        } else {
            if (minParam.isSpecified()) {
                gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, minParam.getIValue());
            }

            if (maxParam.isSpecified()) {
                gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, maxParam.getIValue());
            }
        }

        return gsets;
    }

    public GeneSet[] getGeneSets(final DatasetOptParam dsParam,
                                 final boolean removeNativeNames) throws Exception {

        final GeneSet[] gsets = _getGeneSets().gsets;

        if (dsParam.isSpecified()) {
            return GeneSetGenerators.qualifyGeneSets(gsets, dsParam.getDataset(), removeNativeNames);
        } else {
            return gsets;
        }
    }

    public GeneSet[] getGeneSets(final Dataset ds,
                                 final boolean removeNativeNames) throws Exception {
        final GeneSet[] gsets = _getGeneSets().gsets;
        return GeneSetGenerators.qualifyGeneSets(gsets, ds, removeNativeNames);
    }

    public GeneSet[] getGeneSets(final boolean removeNativeGmNames) throws Exception {
        return getGeneSetMatrixCombo(removeNativeGmNames).getGeneSets();
    }

    public GeneSet[] getGeneSets(final IntegerParam minParam,
                                 final IntegerParam maxParam,
                                 final boolean removeNativeGmNames) throws Exception {

        GeneSet[] gsets = _getGeneSets().gsets;

        if (minParam.isSpecified()) {
            gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, minParam.getIValue());
        }

        if (maxParam.isSpecified()) {
            gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, maxParam.getIValue());
        }

        return gsets;
    }

    public GeneSet getGeneSetCombo() throws Exception {
        GeneSet[] gsets = _getGeneSets().gsets;
        if (gsets.length == 0) {
            throw new IllegalArgumentException("Not specified!!");
        } else if (gsets.length == 1) {
            return gsets[0];
        } else {
            return new FSet("combo_geneset", gsets);
        }
    }

    public GeneSetMatrix getGeneSetMatrixCombo(final boolean removeNativeGmNames) throws Exception {
        return _getGeneSets().toGm(removeNativeGmNames);
    }

    // @todo rethink
    private boolean _isSymbols(final GeneSet[] gsets) throws Exception {

        final Chip chip_gs = VdbRuntimeResources.getChip_Gene_Symbol();

        int numSymbols = 0;
        int numNotSymbols = 0;

        for (int i = 0; i < gsets.length; i++) {
            for (int j = 0; j < gsets[i].getNumMembers(); j++) {
                if (chip_gs.isProbe(gsets[i].getMember(j))) {
                    numSymbols++;
                } else {
                    numNotSymbols++;
                }

                if (numSymbols > 10) {
                    return true;
                }

                if (numNotSymbols > 100) {
                    return false;
                }
            }
        }

        return false;
    }

    private GeneSet[] mapToSymbols(final GeneSet[] orig_gsets, final Chip sourceChip) throws Exception {
        final Chip2ChipMapper mapper = VdbRuntimeResources.getGeneSymbolMapper(sourceChip, true);
        final MGeneSetMatrix mgm = mapper.map(new DefaultGeneSetMatrix("foo", orig_gsets), false);
        final MGeneSet[] mgsets = mgm.getMappedGeneSets();
        final GeneSet[] gsets = new GeneSet[mgsets.length];

        for (int m = 0; m < mgsets.length; m++) {
            gsets[m] = mgsets[m].getMappedGeneSet(true);
        }

        return gsets;
    }

    private GeneSet[] mapFromSymbols(final GeneSet[] orig_gsets, final Chip sourceChip) throws Exception {
        final Chip2ChipMapper mapper = VdbRuntimeResources.getGeneSymbolMapper(sourceChip, false);
        final MGeneSetMatrix mgm = mapper.map(new DefaultGeneSetMatrix("foo", orig_gsets), false);
        final MGeneSet[] mgsets = mgm.getMappedGeneSets();
        final GeneSet[] gsets = new GeneSet[mgsets.length];

        for (int m = 0; m < mgsets.length; m++) {
            gsets[m] = mgsets[m].getMappedGeneSet(true);
        }

        return gsets;
    }

    public GeneSet[] getGeneSetsCombo(final boolean removeNativeGmNames,
                                      final FeatureSpaceReqdParam fFeatureSpaceParam,
                                      final ChipOptParam chipParam) throws Exception {

        GeneSet[] gsets = _getGeneSets().toGm(removeNativeGmNames).getGeneSets();

        boolean gsetsAreLikelySymbols = _isSymbols(gsets);

        if (fFeatureSpaceParam.isSymbols() && gsetsAreLikelySymbols) {
            klog.debug("No gene set mapping automagix was done");
        } else if (fFeatureSpaceParam.isSymbols() && !gsetsAreLikelySymbols) {
            klog.debug("Internally mapping gene sets to Gene Symbols");
            if (chipParam.isSpecified() == false) {
                throw new IllegalArgumentException("Chip param must be specified. This is needed for mapping gene sets from native to gene symbols");
            }
            gsets = mapToSymbols(gsets, chipParam.getChip());
        } else if (fFeatureSpaceParam.isNative() && gsetsAreLikelySymbols) {
            klog.debug("Internally mapping gene sets from Gene Symbols to native");
            if (chipParam.isSpecified() == false) {
                throw new IllegalArgumentException("Chip param must be specified. This is needed for mapping gene sets from symbols to native(i.e datasets) format");
            }
            gsets = mapFromSymbols(gsets, chipParam.getChip());
        } else if (fFeatureSpaceParam.isNative() && !gsetsAreLikelySymbols) {
            klog.debug("No gene set mapping automagix was done");
        }

        return gsets;
    }

    public GeneSetMatrix getGeneSetMatrixCombo(final IntegerParam minParam, final IntegerParam maxParam, final boolean removeNativeGmNames) throws Exception {
        return _getGeneSets().toGm(minParam, maxParam, removeNativeGmNames);
    }

    public GeneSetMatrix getGeneSetMatrixCombo(final IntegerParam minParam,
                                               final IntegerParam maxParam,
                                               final Dataset ds,
                                               final boolean removeNativeGmNames) throws Exception {
        return _getGeneSets().toGm(minParam, maxParam, ds, removeNativeGmNames);
    }

    public GeneSetMatrix getGeneSetMatrixCombo(final IntegerParam minParam,
                                               final Dataset ds,
                                               final boolean removeNativeGmNames) throws Exception {
        return _getGeneSets().toGm(minParam.getIValue(), Integer.MAX_VALUE, ds, removeNativeGmNames);
    }

    public GeneSetMatrix getGeneSetMatrixCombo(final DatasetMultiChooserParam param,
                                               final boolean removeNativeGmNames) throws Exception {
        if (param.isSpecified()) {
            Dataset[] dss = param.getDatasets();
            return new DefaultGeneSetMatrix("combo_qualified", getGeneSets(removeNativeGmNames), dss);
        } else {
            return getGeneSetMatrixCombo(removeNativeGmNames);
        }
    }

    public GeneSetMatrix getGeneSetMatrixCombo(final DatasetOptParam param,
                                               final boolean removeNativeGmNames) throws Exception {
        if (param.isSpecified()) {
            Dataset[] dss = new Dataset[]{param.getDataset()};
            String name = _getName() + "_qualified";
            return new DefaultGeneSetMatrix(name, getGeneSets(removeNativeGmNames), dss);
        } else {
            return getGeneSetMatrixCombo(removeNativeGmNames);
        }
    }

    public GeneSetMatrix getGeneSetMatrixCombo(final DatasetReqdParam param,
                                               final boolean removeNativeGmNames) throws Exception {
        String name = _getName() + "_qualified";
        return new DefaultGeneSetMatrix(name, getGeneSets(removeNativeGmNames), new Dataset[]{param.getDataset()});
    }

    public GeneSetMatrix getGeneSetMatrixCombo(final Dataset ds,
                                               final boolean removeNativeGmNames) throws Exception {
        String name = _getName() + "_qualified";
        return new DefaultGeneSetMatrix(name, getGeneSets(removeNativeGmNames), new Dataset[]{ds});
    }

    //-----------------------------------------------------------------------//
    //------------------------- CORE METHODS --------------------------------//
    //-----------------------------------------------------------------------//
    protected Object[] _getObjects() throws Exception {

        Object val = getValue();

        Object[] objs;

        // cant et a ftp file object because it has to be a string n the obect chooser text area
        // log.debug("value = " + val + " class: " + val.getClass());

        if (val instanceof String) {
            String[] paths = _parse(val.toString());
            objs = new Object[paths.length];
            for (int p = 0; p < paths.length; p++) {
                if (paths[p].startsWith("Genesets:")) {
                    String baseName = paths[p].substring(9);
                    int index = baseName.lastIndexOf('.');
                    String name = baseName.substring(0, index).replace('.', '_') + ".gmt";
                    String path = "web/"+name;
                    InputStream is = (new GsaaSeqSP()).getClass().getResourceAsStream(path);
                    GeneSetMatrix gm = ParserFactory.readGeneSetMatrix(paths[p], is, false);
                    if (AuxUtils.isAux(paths[p])) {
                        objs[p] = gm.getGeneSet(AuxUtils.getAuxNameOnlyIncludingHash(paths[p]));
                    } else {
                        objs[p] = gm;
                    }                
                } else if (AuxUtils.isAux(paths[p])) {
                    objs[p] = ParserFactory.readGeneSet(new File(paths[p]), true);
                } else {
                    objs[p] = ParserFactory.read(new File(paths[p]), true);
                }
            }
        } else if (val instanceof Object[]) {
            objs = (Object[]) val;
        } else {
            objs = new Object[]{val};
        }

        return objs;
    }

    /*
    protected GeneSet[] _getGeneSets() throws Exception {

    }
    */

    /**
     * Internal; sttic to preserve the name of the input gene matrix
     * in cases where there is only 1 specified (95% of the time)
     */
    class GeneSetsStruc {
        GeneSet[] gsets;
        String name;

        GeneSetsStruc(final String name, final GeneSet[] gsets) {
            this.gsets = gsets;
            this.name = name;
        }

        GeneSetMatrix toGm(boolean removeNativeGmNames) {
            return new DefaultGeneSetMatrix(name, gsets, removeNativeGmNames);
        }

        GeneSetMatrix toGm(final IntegerParam minParam,
                           final IntegerParam maxParam, boolean removeNativeGmNames) throws Exception {
            return new DefaultGeneSetMatrix(name, getGeneSets(minParam, maxParam, removeNativeGmNames));
        }

        GeneSetMatrix toGm(final IntegerParam minParam,
                           final IntegerParam maxParam, Dataset ds,
                           boolean removeNativeGmNames) throws Exception {
            return new DefaultGeneSetMatrix(name, getGeneSets(minParam, maxParam, ds, removeNativeGmNames));
        }

        GeneSetMatrix toGm(final int min,
                           final int max, Dataset ds,
                           boolean removeNativeGmNames) throws Exception {
            return new DefaultGeneSetMatrix(name, getGeneSets(min, max, ds, removeNativeGmNames));
        }
    }

    private String _getName(final Object[] objs) {

        String name = "combo";

        if (objs.length == 1 && objs[0] instanceof GeneSetMatrix) {
            name = ((GeneSetMatrix) objs[0]).getName();
        }

        return name;

    }

    private String _getName() throws Exception {
        return _getName(_getObjects());
    }

    protected GeneSetMatrixChooserAbstractParam.GeneSetsStruc _getGeneSets() throws Exception {

        Object[] objs = _getObjects();

        //log.debug("num of selections: " + objs.length);

        if (isReqd() && objs.length == 0) {
            throw new IllegalArgumentException("Must specify GeneSetMatrix parameter: " + getNameEnglish() + " (" + getDesc() + ")");
        }

        List gsets = new ArrayList();

        String name = _getName(objs);
        for (int i = 0; i < objs.length; i++) {
            if (objs[i] instanceof GeneSetMatrix) {
                gsets.addAll(((GeneSetMatrix) objs[i]).getGeneSetsL());
            } else if (objs[i] instanceof GeneSet) {
                gsets.add(objs[i]);
            } else if (objs[i] instanceof Dataset) {
                gsets.add(((Dataset) objs[i]).getRowNamesGeneSet());
            } else if (objs[i] instanceof Chip) {
                gsets.add(((Chip) objs[i]).getProbeNamesAsGeneSet());
            } else {
                throw new IllegalArgumentException("Unknown object: " + objs[i]);
            }
        }

        return new GeneSetMatrixChooserAbstractParam.GeneSetsStruc(name, (GeneSet[]) gsets.toArray(new GeneSet[gsets.size()]));
    }

    protected GeneSetMatrix[] _getGeneSetMatrices() throws Exception {
        Object[] objs = _getObjects();

        //System.out.println(">>> " + objs + " " + objs.length);

        GeneSetMatrix[] gms = new GeneSetMatrix[objs.length];

        for (int i = 0; i < objs.length; i++) {
            if (objs[i] instanceof GeneSetMatrix) {
                gms[i] = (GeneSetMatrix) objs[i];
            } else if (objs[i] instanceof GeneSet) {
                GeneSet gset = (GeneSet) objs[i];
                gms[i] = new DefaultGeneSetMatrix(gset.getName(), new GeneSet[]{gset});
            } else if (objs[i] instanceof Dataset) {
                GeneSet gset = ((Dataset) objs[i]).getRowNamesGeneSet();
                gms[i] = new DefaultGeneSetMatrix(gset.getName(), new GeneSet[]{gset});
            } else if (objs[i] instanceof Chip) {
                GeneSet gset = ((Chip) objs[i]).getProbeNamesAsGeneSet();
                gms[i] = new DefaultGeneSetMatrix(gset.getName(), new GeneSet[]{gset});
            } else {
                throw new IllegalArgumentException("Unknown object: " + objs[i]);
            }
        }

        if (isReqd() && gms.length == 0) {
            throw new IllegalArgumentException("Must specify GeneSetMatrix parameters, got: " + gms.length);
        }

        return gms;
    }


    /**
     * @param paths
     * @note special overriding of base class
     * can be a combo of genesets and genematrices
     */
    public void setValue(final String[] paths) {
        try {

            // @todo this is kludgy and SLOW (because of the object cache node update)
            System.out.println(">> number of paths: " + paths.length + " >" + paths);
            TraceUtils.showTrace();
            Object[] objs = new Object[paths.length];
            for (int i = 0; i < paths.length; i++) {
                System.out.println(">>> reading path: " + paths[i]);
                File file = new File(paths[i]);
                objs[i] = ParserFactory.read(file, true);
            }

            // @note special handling as a hack
            //this.setValue(objs);
            super.setValue(objs);

        } catch (Throwable t) {
            throw new RuntimeException(t);
        }
    }

    private static String[] _parse(final String s) {

        if (s == null) {
            throw new IllegalArgumentException("Parameter s cannot be null");
        }

        Set vals = ParseUtils.string2stringsSet(s, ",", false); // only commas!!

        System.out.println("to parse>" + s + "< got: " + vals);

        Set use = new HashSet();
        for (Iterator it = vals.iterator(); it.hasNext();) {
            String key = it.next().toString();
            if (key.length() > 0) {
                use.add(key);
            }
        }

        return (String[]) use.toArray(new String[use.size()]);
    }

    public void setValue(final String commadelimvalues) {
        setValue(_parse(commadelimvalues));
    }

    // override base class method to do for both pobs and strings
    private String format(final Object[] vals) {
        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {

            if (vals[i] == null) {
                continue;
            }

            log.debug(vals[i].getClass());

            if (vals[i] instanceof PersistentObject) {
                String p = ParserFactory.getCache().getSourcePath(vals[i]);
                buf.append(p);
            } else {
                buf.append(vals[i].toString().trim());
            }

            if (i != vals.length - 1) {
                buf.append(',');
            }
        }

        return buf.toString();
    }

    // override base impl
    protected ActionListener getActionListener() {
        if (fAl == null) {
            this.fAl = new GeneSetMatrixChooserAbstractParam.MyPobActionListener(fMultipleAllowed, fIncludeGmx);
            fAl.setChooser(fChooser);
        }

        return fAl;
    }

    public boolean isFileBased() {
        return true;
    }

    // redo from the abstract super class here as we dont want to swap out the model
    // (model is the datasets etc)
    // REIMPLEMENT HERE as we want to (possibly) use both genesets and genesetmatrixes
    private static class MyPobActionListener implements ActionListener {

        private boolean fMultipleAllowed;
        private boolean fIncludeGmx;
        private GeneSetMatrixChooserUI fChooser;

        public MyPobActionListener(boolean multipleAllowed, boolean includeGeneSetMatrix) {
            this.fMultipleAllowed = multipleAllowed;
            this.fIncludeGmx = includeGeneSetMatrix;
        }

        // cant have this in the class constructor as the action list needs to
        // be instantiated before the chooser object is made
        public void setChooser(GeneSetMatrixChooserUI chooser) {
            this.fChooser = chooser;
        }

        final DefaultListCellRenderer rend = new GeneSetMatrixChooserAbstractParam.GeneSetsFromFTPSiteRenderer();

        private ListModel createFTPModel() {

            if (XPreferencesFactory.kOnlineMode.getBoolean() == false) {
                DefaultListModel model = new DefaultListModel();
                model.addElement("Offline mode");
                model.addElement("Change this in Menu=>Preferences");
                model.addElement("Choose gene sets from other tabs");
                return model;
            } else {
                try { 
                    InputStream is = (new GsaaSeqSP()).getClass().getResourceAsStream("web/genesets.txt");
                    InputStreamReader isr = new InputStreamReader(is);
                    ArrayList al = readData(isr);
                    DefaultListModel model = new DefaultListModel();
                    for (int i = 0; i < al.size(); i++) {
                        String s = (String) al.get(i);
                        model.addElement(s);
                    }
                    return model;
                } catch (Exception e) {
                    klog.error(e);
                    DefaultListModel model = new DefaultListModel();
                    model.addElement("Service temporarily unavailable");
                    model.addElement(e.getMessage());
                    model.addElement("Choose gene sets from other tabs");
                    return model;                    
                }
            }
        }
        
        public static ArrayList readData(InputStreamReader isr) {
            ArrayList result = new ArrayList();
            try {
                BufferedReader br = new BufferedReader(isr);
                String s = br.readLine();
                while (s != null) {
                    result.add(s.trim());
                    s = br.readLine();
                }

                br.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
            return result;
        }        


        public void actionPerformed(ActionEvent e) {

            if (fChooser == null) {
                //klog.debug("Chooser not yet inited: " + fChooser);
                return;
            }

            int selmode;
            if (fMultipleAllowed == false) {
                selmode = ListSelectionModel.SINGLE_SELECTION;
            } else {
                selmode = ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;
            }

            NamedModel[] models;
            final NamedModel gsetsModel = new NamedModel("Gene sets (grp)", ObjectBindery.getModel(GeneSet.class));
            final NamedModel subsetsModel = new NamedModel("Subsets", ObjectBindery.getHackAuxGeneSetsBoxModel());
            final NamedModel gmFromFTPModel = new NamedModel("Gene matrix (from website)", createFTPModel());


            if (fIncludeGmx) {
                models = new NamedModel[]
                        {
                                gmFromFTPModel,
                                gsetsModel,
                                new NamedModel("Gene matrix (local gmx/gmt)", ObjectBindery.getModel(GeneSetMatrix.class)),
                                subsetsModel
                        };
            } else {
                models = new NamedModel[]
                        {
                                gsetsModel,
                                subsetsModel
                        };
            }

            final Object[] sels = fChooser.getJListWindow().showDirectlyWithModels(models, selmode, rend);

            if ((sels == null) || (sels.length == 0)) { // <-- @note
            } else {
                String str = ChooserHelper.formatPob(sels);
                fChooser.setText(str);
            }
        }
    }

    // have to make the strs into paths
    public String getValueStringRepresentation(final boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        // log.debug("value: " + val.getClass() + " " + val);

        if (val instanceof String) {
            return (String) val;
        } else if (val instanceof Object[]) {
            Object[] objs = (Object[]) val;
            return format(objs);
        } else {
            return format(new Object[]{val});
        }

    }


    public GFieldPlusChooser getSelectionComponent() {

        if (fChooser == null) {
            //fChooser = new GOptionsFieldPlusChooser(getActionListener(), Application.getWindowManager().getRootFrame());
            // do in 2 stages, as the al needs a valid (non-null) chooser at its construction
            fChooser = new GeneSetMatrixChooserUI(false);
            fChooser.setCustomActionListener(getActionListener());
            String text = this.getValueStringRepresentation(false);
            if (text == null) {
                text = format((Object[]) getDefault());
            }

            if (isFileBased()) { // as otherwise lots of exceptions thrown if user edits a bad file
                // @todo but probelm is that no way to cancel and "null out" a choice once made
                //fChooser.getTextField().setEditable(false);
            }

            fChooser.setText(text);
            ParamHelper.addDocumentListener(fChooser.getTextField(), this);
        }

        return fChooser;
    }


    public static class GeneSetsFromFTPSiteComparator implements Comparator {

        /* private static member used to track highest seen version */
        private static String highestVersionId;
        private static DefaultArtifactVersion highestVersion;

        public static String getHighestVersionId() {
            return highestVersionId;
        }

        /**
         * Compare method that yields the following ordering of gmt file names:
         *       versions will be listed in descending order
         *       within a given version, collection types (c1, c2, c3, c4, c5)
         *         will be listed in increasing order
         *       within a given version and collection type, subset types
         *         (e.g., all vs. cp) will be listed in lexicographic order
         */
        public int compare(Object pn1, Object pn2) {

            GeneSetMatrixChooserAbstractParam.MSigDBFilenameParser p1 = new GeneSetMatrixChooserAbstractParam.MSigDBFilenameParser(pn1.toString());
            GeneSetMatrixChooserAbstractParam.MSigDBFilenameParser p2 = new GeneSetMatrixChooserAbstractParam.MSigDBFilenameParser(pn2.toString());

            DefaultArtifactVersion version1 = new DefaultArtifactVersion(p1.getCanonicalVersionId());
            DefaultArtifactVersion version2 = new DefaultArtifactVersion(p2.getCanonicalVersionId());

            /* want to keep track of highest version seen */
            if (highestVersion == null) {
                if (version1.compareTo(version2) < 0) {
                    highestVersion = version2;
                    highestVersionId = p2.getVersionId();
                }
                else {
                    highestVersion = version1;
                    highestVersionId = p2.getVersionId();
                }
            }
            else {
                if (highestVersion.compareTo(version2) < 0) {
                    highestVersion = version2;
                    highestVersionId = p2.getVersionId();
                }
            }

            if (!version1.equals(version2)) {
                return version2.compareTo(version1);
            }
            else {
                int collectionNum1 = p1.getCollectionIdNum();
                int collectionNum2 = p2.getCollectionIdNum();
                if (collectionNum1 != collectionNum2) {
                    return collectionNum1 - collectionNum2;
                }
                else {
                    String subsetId1 = p1.getSubsetId();
                    String subsetId2 = p2.getSubsetId();
                    return subsetId1.compareTo(subsetId2);
                }
            }
        }

        public boolean equals(Object o2) {
            return false;
        }
    }    // End GeneSetsFromFTPSiteComparator

    /**
     * static nested class for parsing names of gmt files retrievable from
     * MSigDB
     */
    public static class MSigDBFilenameParser {

        static String MSIGDB_VERSION_V1 = "v1";
        static String MSIGDB_VERSION_V2 = "v2";

        private String filename = null;
        private String versionId = null;
        private String collectionId = null;
        private String subsetId = null;
        private int collectionIdNum = 0;

        public MSigDBFilenameParser(String filename) {
            if (filename == null)
                throw new IllegalArgumentException("MSigDB Filename cannot be null");
            this.filename = filename;

            collectionId = filename.substring(0,2);
            collectionIdNum = Integer.parseInt(collectionId.substring(1));
            versionId = filename.substring(filename.lastIndexOf("v"), filename.lastIndexOf(".symbols.gmt"));

            if (!versionId.equals(MSIGDB_VERSION_V1) && !versionId.equals(MSIGDB_VERSION_V2)) {
                subsetId = filename.substring(3, filename.lastIndexOf(versionId)-1);
            }
        }

        public String getFilename() {
            return filename;
        }

        public String getVersionId() {
            return versionId;
        }

        public String getCanonicalVersionId() {
            if (versionId.equals(MSIGDB_VERSION_V1) || versionId.equals(MSIGDB_VERSION_V2)) {
                return versionId + ".0";
            }
            else
                return versionId;
        }

        public String getCollectionId() {
            return collectionId;
        }

        public int getCollectionIdNum() {
            return collectionIdNum;
        }

        public String getSubsetId() {
            return subsetId;
        }

    }

    private static boolean isVersion(String versionSuffix, String name) {
        int index = name.toLowerCase().indexOf(versionSuffix);
        if (index == -1) {
            return false;
        } else {
            return true;
        }
    }

    public static class GeneSetsFromFTPSiteRenderer extends DefaultListCellRenderer {
        private boolean ifFileOnlyShowName;

        public GeneSetsFromFTPSiteRenderer(final boolean ifFileOnlyShowName) {
            this.ifFileOnlyShowName = ifFileOnlyShowName;
        }

        public GeneSetsFromFTPSiteRenderer() {
            this(false); // default is to show the full path
        }

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            // order is important
            if (value instanceof PersistentObject) {
                PersistentObject pob = (PersistentObject) value;

                if (pob.getQuickInfo() != null) {
                    StringBuffer buf = new StringBuffer("<html><body>").append(pob.getName());
                    buf.append("<font color=#666666> [").append(pob.getQuickInfo()).append(']').append("</font></html></body>");
                    this.setText(buf.toString());
                } else {
                    this.setText(pob.getName());
                }

                File f = null;

                if (ParserFactory.getCache().isCached(pob)) {
                    f = ParserFactory.getCache().getSourceFile(pob);
                }

                if (f != null) {
                    this.setToolTipText(f.getAbsolutePath());
                } else {
                    this.setToolTipText("Unknown origins of file: " + f);
                }
            } else if (value instanceof File) {
                if (ifFileOnlyShowName) {
                    this.setText(((File) value).getName());
                } else {
                    this.setText(((File) value).getAbsolutePath());
                }
                this.setIcon(DataFormat.getIcon(value));
                this.setToolTipText(((File) value).getAbsolutePath());
            } else if (value instanceof XChart) {
                this.setText(((XChart) value).getName());
                this.setIcon(XChart.ICON);
            } else if (value instanceof FTPFile) {

                String s = ((FTPFile) value).getName();
                String slc = s.toLowerCase();
                if (slc.indexOf("c1.") != -1) {
                    s = s + " [Positional]";
                } else if (slc.indexOf("c2.") != -1) {
                    s = s + " [Curated]";
                } else if (slc.indexOf("c3.") != -1) {
                    s = s + " [Motif]";
                } else if (slc.indexOf("c4.") != -1) {
                    s = s + " [Computational]";
                } else if (slc.indexOf("c5.") != -1) {
                    s = s + " [Gene ontology]";
                } else if (slc.indexOf("c6.") != -1) {
                    s = s + " [Oncogenic signatures]";
                }

                if (isVersion( GeneSetMatrixChooserAbstractParam.GeneSetsFromFTPSiteComparator.getHighestVersionId(), slc)) {
                    Font font = this.getFont();
                    String fontName = font.getFontName();
                    int fontSize = font.getSize();
                    this.setFont(new Font(fontName, Font.BOLD, fontSize));
                }

                this.setText(s);
                this.setIcon(RendererFactory2.FTP_FILE_ICON);
            }

            return this;
        }
    }    // End CommonLookListRenderer

} // End class AbstractGeneSetMatrixChooserParam
