/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetMetrics;
import edu.mit.broad.genome.alg.markers.PermutationTestImpl;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;
import org.apache.log4j.Logger;

import java.io.File;
import java.util.Collections;
import java.util.Map;


/**
 * Class ParamFactory
 * collection of commonly used Param objects
 * Factory for commonly used parameters
 * Instantiation through this class helps keep the name and descs etc consistent
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ParamFactory {

    private static final Logger klog = XLogger.getLogger(ParamFactory.class);

    /**
     * Class constructor
     */
    private ParamFactory() {
    }


    public static BooleanParam createForceRefreshParam() {
        return new BooleanParam("force_refresh", "overwrite even if already present in bged_cache", false, false);
    }

    public static BooleanParam createCollapseParam() {
        return new BooleanParam("collapse", "Collapse probe sets => 1 gene", "Combine all probe set ids for a given gene into a single value", true, false);
    }

    // -------------------------------------------------------------------------------------------- //

    public static IntegerParam createHeatMapCellSizeParam() {
        return createHeatMapCellSizeParam(8);
    }

    public static IntegerParam createHeatMapCellSizeParam(final int size) {
        return new IntegerParam("cell_size", "cell size", size, false);
    }

    public static IntegerParam createNumTrollsParam() {
        return new IntegerParam("num", "num codes", 5, false);
    }

    public static FloatParam createThreshParam() {
        return new FloatParam("thresh", "threshold for prediction", 0.5f, true);
    }

    public static FloatParam createPosFoldChangeParam() {
        return new FloatParam("pos", "pos fold change", 1.585f, false);
    }

    public static FloatParam createNegFoldChangeParam() {
        return new FloatParam("neg", "neg fold change", 1.585f, false);
    }

    public static IntegerParam createNumNotMidParam() {
        return new IntegerParam("not_mid_thresh", "num not mid threshold", 1, false);
    }

    // -------------------------------------------------------------------------------------------- //


    public static BooleanParam createMakeResParam(final boolean def, final boolean reqd) {
        return new BooleanParam("make_res", "make res files",
                "make res files with a and p calls", def, reqd);
    }

    public static FileParam createCelFileNamesParam(final boolean reqd) {
        return new FileParam("cel",
                "Create a dataset using these CEL files",
                "Create a dataset using these CEL files", new File[]{}, reqd);
    }

    public static StringInputParam createPlateNameParam(final boolean reqd) {
        return new StringInputParam("plate", "HTA plate bar code", reqd);
    }

    public static StringMultiInputParam createPlateNamesParam(final boolean reqd) {
        return new StringMultiInputParam("plate", "HTA plate bar code", reqd);
    }

    public static BooleanParam createZipReportParam(final boolean reqd) {
        return new BooleanParam("zip_report", "Make a zipped file with all reports",
                "Create a zipped file with all files made by the report. This can be emailed to share results",
                false, reqd, Param.ADVANCED);
    }

    public static BooleanParam createLogitParam(final boolean def, final boolean reqd) {
        return new BooleanParam("logit", "logit", "take log", def, reqd);
    }

    public static IntegerParam createNumBinsParam(int def, boolean reqd) {
        return new IntegerParam("nbins", "number of bins", def, reqd);
    }

    public static FloatParam createMaxParam(float def, boolean reqd) {
        return new FloatParam("max", "Maximum value", def, reqd);
    }

    public static FloatParam createMinParam(float def, boolean reqd) {
        return new FloatParam("min", "Minimum value", def, reqd);
    }

    public static FloatParam createMaxParam(float[] defs, boolean reqd) {
        return new FloatParam("max", "Maximum value", defs, reqd);
    }

    public static FloatParam createMinParam(float[] defs, boolean reqd) {
        return new FloatParam("min", "Minimum value", defs, reqd);
    }

    public static FloatParam createCutoffLowerPValueBoundParam(boolean reqd) {
        return new FloatParam("plower", "Lower P-value bound", 0, reqd);
    }

    public static FloatParam createCutoffUpperPValueBoundParam(boolean reqd) {
        return new FloatParam("plower", "Lower P-value bound", 0, reqd);
    }

    public static StringReqdParam createRPathParam() {
        //double \\ are IMP
        return new StringReqdParam("rpath", "path to R", new String[]{
                "C:\\Program Files\\R\\rw1090\\bin\\R", "/usr/bin/R"});
    }

    public static BooleanParam createGeneSetUnionParam(boolean def, boolean reqd) {
        return new BooleanParam("union",
                "union of all input gene sets", def, reqd);
    }

    public static BooleanParam createGeneSetIntersectionParam(boolean def, boolean reqd) {
        return new BooleanParam("common", "intersection of all input gene sets", def, reqd);

    }

    public static BooleanParam createIgnoreUnenrichedParam(boolean def, boolean reqd) {
        return new BooleanParam("ignore_unenriched", "ignore unenriched", def, reqd);

    }

    public static BooleanParam createIgnoreUnenrichedParam() {
        return createIgnoreUnenrichedParam(true, false);

    }

    // @todo remove me later
    public static BooleanParam createScoreByPhenotypeCorrParam(boolean def, boolean reqd) {
        return new BooleanParam("pheno_corr", "score by pheno corr", def, reqd);

    }

    public static BooleanParam createShowMegDetailsParam(boolean def, boolean reqd) {
        return new BooleanParam("show_details", "Show detailed info on genes/genesets (might be meory intensive)", def, reqd);

    }

    public static BooleanParam createMakeReportSubDirParam(boolean def, boolean reqd) {
        return new BooleanParam("make_rpt_dir", "make report specific sub directories", def, reqd);

    }

    public static BooleanParam createBpogParam(boolean reqd) {
        return createBpogParam(true, reqd);

    }

    public static BooleanParam createBpogParam(boolean def, boolean reqd) {
        return new BooleanParam("bpog", "Create a blue-pinkogram image (dont for large number of markers", def, reqd);
    }

    public static BooleanParam createRandomParam(boolean def, boolean reqd) {
        return new BooleanParam("rnd", "Do random mode param", def, reqd);
    }

    public static IntegersChooserParam createModesParam(int def) {
        return new IntegersChooserParam("mode", "mode", new Integer(def), false);
    }

    public static DirParam createEdbDirParam(boolean reqd) {
        return new DirParam("edb_folder", "folder where the edb files to be collected are", reqd);

    }

    public static DirParam createEdbAltDirParam(boolean reqd) {
        return new DirParam("alt_datadir", "folder with the datasets, gsets and templates that go into the edb(used in pref of whats in the edb)", reqd);

    }

    public static StringReqdParam createExecModeParam() {
        return new StringReqdParam("exec", "execution mode", new String[]{"local", "lsf", "lsf_batch"});
    }

    //@note added by Qing Xiong - qing.xiong@duke.edu
    public static StringParam createSnpSetMetric(boolean reqd) {
        return new StringParam("cmetric_snpset", "Metric for SNP set association anlaysis", "Gene markers are ranked using this metric to produce the gene list in SNP set association analysis", new String[]{"Maximum"}, reqd);
    }
    public static StringParam createDataIntegrationMetric(boolean reqd) {
        return new StringParam("imetric", "Metric for gene association anlaysis", "Gene markers are ranked using this metric to produce the gene list in gene association analysis", new String[]{"ZScore_Sum", "Fisher_Method", "Rank_Sum"}, reqd);
    }
    /*public static StringParam createGeneSetMetric(boolean reqd) {
        return new StringParam("smetric", "Metric for gene set association anlaysis", "Gene sets/pathways are ranked using this metric to produce the list of gene sets in gene set association analysis", new String[]{"Weighted_KS", "SAM_GS", "Fisher_Method"}, reqd);
    }*/
    public static StringParam createGeneSetMetric(boolean reqd) {
        return new StringParam("smetric", "Metric for gene set association anlaysis", "Gene sets/pathways are ranked using this metric to produce the list of gene sets in gene set association analysis", new String[]{"Weighted_KS"}, reqd);
    }
    public static StringParam createCpnMappingModeParam(boolean reqd) {
        return new StringParam("cmetric_cpn", "Metric for ranking copy number sets", "Metric for ranking copy number sets: average/sum", new String[]{"average", "sum"}, reqd);
    }
    public static StringParam createRnaMappingModeParam(boolean reqd) {
        return new StringParam("cmetric_rna", "Metric for ranking microRNA sets", "Metric for ranking microRNA sets: average/sum", new String[]{"average", "sum"}, reqd);
    }
    public static StringParam createMetMappingModeParam(boolean reqd) {
        return new StringParam("cmetric_met", "Metric for ranking methylation sets", "Metric for ranking methylation sets: average/sum", new String[]{"average", "sum"}, reqd);
    }
    public static StringParam createNumMappingModeParam(boolean reqd) {
        return new StringParam("cmetric_num", "Mapping mode", "Mapping mode: sum/average", new String[]{"mean", "sum"}, reqd);
    }

    public static StringReqdParam createSpeciesParam() {
        return new StringReqdParam("species", "Species", "Species", new String[]{"Human_GRCh37", "Human_NCBI36", "Mouse", "Rat", "Zebrafish", "Arabidopsis_Thaliana", "Cat", "C.elegans", "Chicken", "Chimpanzee", "C.intestinalis", "Cow", "Dog", "Fruitfly", "Fugu", "Gorilla", "Horse", "Macaque", "Marmoset", "Opossum", "Orangutan", "Pig", "Rabbit", "Saccharomyces_Cerevisiae", "Stickleback", "Turkey","Simulation"});
    }

    public static StringReqdParam createMetricParam() {
        return new StringReqdParam("metric", "Metric for differential expression anlaysis", "Gene markers are ranked using this metric to produce the gene list in differential expression analysis", new String[]{"edgeR", "DESeq", "NOISeq", "DEGseq", "baySeq"});
    }

    public static StringReqdParam createGSAMetricParamSeqSP() {
        return new StringReqdParam("gsametric", "Metric for gene set anlaysis", "Gene sets are ranked using this metric", new String[]{"Weighted_KS", "L2Norm", "GeometricMean", "Mean", "FisherMethod", "WeightedSigRatio", "SigRatio"});
        //return new StringReqdParam("gsametric", "Metric for gene set anlaysis", "Gene sets are ranked using this metric", new String[]{"Weighted_KS", "L2Norm", "GeometricMean", "Mean", "FisherMethod", "RankSum", "WeightedSigRatio", "SigRatio", "TruncatedProduct", "MinP"});    
    }

    public static StringReqdParam createDEMetricParamSeqSP() {
        return new StringReqdParam("demetric", "Metric for differential expression anlaysis", "Gene markers are ranked using this metric to produce the gene list in differential expression analysis", new String[]{"Signal2Noise", "log2Ratio","Signal2Noise_log2Ratio"});
    }

    public static StringReqdParam createGSAMetricParamSeqGP() {
        return new StringReqdParam("gsametric", "Metric for gene set anlaysis", "Gene sets are ranked using this metric", new String[]{"KS"});
    }

    public static StringReqdParam createDEMetricParamSeqGP() {
        return new StringReqdParam("demetric", "Metric for differential expression anlaysis", "Gene markers are ranked using this metric to produce the gene list in differential expression analysis", new String[]{"edgeR", "DESeq", "NOISeq", "DEGseq", "baySeq"});
    }

    public static StringReqdParam createPThresholdParamSeq() {
        return new StringReqdParam("pthreshold", "P value threshold", "Genes with P value more than this threshold are EXCLUDED from WeightedSigRatio, SigRatio, and TruncatedProduct analysis", new String[]{"0.01", "0.05", "0.06", "0.07", "0.08", "1"});
    }

    public static StringInputParam createPThresholdParam() {
        return new StringInputParam("pthreshold", "P value threshold", "Genes with P value more than this threshold are EXCLUDED from WeightedSigRatio, SigRatio, and TruncatedProduct analysis", "0.05", false);
    }
    //----------------------

    public static BooleanParam createIncludeEmptyProfiles() {
        return new BooleanParam("include_empty", "Include empty (fully missing) probes as an empty (fully missing) profile in the dataset", false, false);
    }

    public static BooleanParam createUseGmtFormatParam(boolean def, boolean reqd) {
        return new BooleanParam("gmt_format", "Use the GeneSetMatrix - Transposed format", def, reqd);
    }

    public static BooleanParam createPreviewRunParam(String desc) {
        return new BooleanParam("preview", desc, true, false);
    }

    public static StringInputParam createFilterModParam() {
        return new StringInputParam("filt_params", "Filter parameters", false);
    }

    public static StringMultiInputParam createFilterModsParam() {
        return new StringMultiInputParam("filt_params", "Filter parameters", false);
    }

    public static BooleanParam createUseMetaGeneDbParam(boolean def, boolean reqd) {
        return new BooleanParam("use_thesarus", "Use the metagenedb", def, reqd);
    }

    public static BooleanParam createUseGenomeThesarusParam(boolean def, boolean reqd) {
        return new BooleanParam("use_thesarus", "Enhance annotations by using the GenomeThesarus", def, reqd);
    }

    public static BooleanParam createUseGenomeThesarusParam(boolean reqd) {
        return createUseGenomeThesarusParam(false, reqd);
    }

    public static BooleanParam createShowEtiologyParam(boolean reqd) {
        return createShowEtiologyParam(false, reqd);
    }

    public static BooleanParam createShowEtiologyParam(boolean def, boolean reqd) {
        return new BooleanParam("show_etiology", "Output verbose mapping details", "Show the etiology for the features", def, reqd);
    }

    public static GeneSetMatrixMultiChooserParam createGeneSetMatrixDbParam(boolean reqd) {
        return new GeneSetMatrixMultiChooserParam("gm_db", "GeneSetMatrix db", "GeneSetMatrix db", reqd);
    }

    public static GeneSetMatrixMultiChooserParam createGeneSetMatrixClusterParam(boolean reqd) {
        return new GeneSetMatrixMultiChooserParam("gm_cluster", "gm cluster", "GeneSetMatrix Clusters", reqd);
    }

    public static StringMultiInputParam createGeneSetNamesParam(boolean reqd) {
        return new StringMultiInputParam("gset_names", "Names of GeneSets, one per row", reqd);
    }

    public static IntegerParam createStartAtRankParam(int def, boolean reqd) {
        return new IntegerParam("start_rank", "Start the analysis at this rank in the sorted marker list(inclusive (first 0))", def, reqd);
    }

    public static IntegerParam createStopAtRankParam(int def, boolean reqd) {
        return new IntegerParam("stop_rank", "Stop the analysis at this rank in the sorted marker list(inclusive, (last n))", def, reqd);
    }

    public static IntegerParam createWindowSizeParam(int def, boolean reqd) {
        return new IntegerParam("win", "window size", def, reqd);
    }

    public static IntegerParam createStepSizeParam(int def, boolean reqd) {
        return new IntegerParam("step", "step size - how much to move between windows", def, reqd);
    }

    public static BooleanParam createNormForSizeParam(boolean def, boolean reqd) {
        return new BooleanParam("norm", "Normalize scores/reports for geneset size (area normalization)", def, reqd);
    }

    public static BooleanParam createBalanceParam(boolean def, boolean reqd) {
        return new BooleanParam("balance", "do balanced template randomizations", def, reqd);
    }

    public static BooleanParam createReversePhenotypeParam(boolean def, boolean reqd) {
        return new BooleanParam("reverse_pheno", "also reverse the phenotype for the analysis", def, reqd);
    }

    public static FloatsChooserParam createSigParam(float[] def_and_hints, boolean reqd) {
        Float[] dh = new Float[def_and_hints.length];
        for (int i = 0; i < def_and_hints.length; i++) {
            dh[i] = new Float(def_and_hints[i]);

        }

        return new FloatsChooserParam("sig", "Significance levels", dh, dh, reqd); // all sigs are the defaults
    }

    public static FloatsChooserParam createSigParam(boolean reqd) {
        Float[] dh = new Float[PermutationTestImpl.DEFAULT_SIG_LEVELS.length];
        for (int i = 0; i < PermutationTestImpl.DEFAULT_SIG_LEVELS.length; i++) {
            dh[i] = new Float(PermutationTestImpl.DEFAULT_SIG_LEVELS[i]);

        }

        return new FloatsChooserParam("sig", "Significance levels", dh, dh, reqd); // all sigs are the defaults
    }

    public static BooleanParam createDeepParam(String desc, boolean def, boolean reqd) {
        return new BooleanParam("deep", desc, def, reqd);
    }

    public static BooleanParam createDeepParam(boolean def, boolean reqd) {
        return createDeepParam("Generate verbose output for the tool(warning: might be memory intensive)", reqd, def);
    }

    public static BooleanParam createDumpParam(String desc, boolean def, boolean reqd) {
        return new BooleanParam("dump", desc, def, reqd);
    }

    public static BooleanParam createDumpParam(boolean def, boolean reqd) {
        return createDumpParam("Generate verbose output for the tool(warning: might be memory intensive)", reqd, def);
    }

    public static BooleanParam createTransposeDatasetParam(boolean def, boolean reqd) {
        return new BooleanParam("transpose", "transpose the dataset (make row cols and cols rows)", def, reqd);
    }

    public static BooleanParam createHelpParam(boolean reqd) {
        return new BooleanParam("help", "Help on required and optional Tool flags", reqd);
    }

    public static IntegerParam createNumRndGeneSetsParam(boolean reqd) {
        return new IntegerParam("num_rnd_genesets", "num_rnd_genesets", "The number of random genesets", new int[]{0, 1, 10, 100, 1000}, reqd);
    }

    public static IntegerParam createNumRndTemplatesParam(boolean reqd) {
        return new IntegerParam("num_rnd_templates", "num_rnd_templates", "The number of random templates", new int[]{0, 1, 10, 50, 100, 1000}, reqd);
    }

    public static IntegerParam createNumPermParam(boolean reqd) {
        return createNumPermParam(1000, reqd);
    }

    public static IntegerParam createNumPermParam(final int nperm, boolean reqd) {
        return new IntegerParam("nperm", "Number of permutations", "The number of permutations", nperm, new int[]{0, 1, 10, 100, 1000}, reqd);
    }

    public static IntegerParam createNumSplitsParam(boolean reqd) {
        return new IntegerParam("nsplits", "Number of splits", "The number random splits of the dataset", new int[]{0, 1, 10, 100, 1000}, reqd);
    }

    public static IntegerParam createNumLocParam(boolean reqd) {
        return new IntegerParam("nloc", "The number of leave one out cross validations to do", -1, reqd);
    }

    public static BooleanParam createShuffleGeneSetParam(boolean def, boolean reqd) {
        return new BooleanParam("group", "Shuffle the gene group", def, reqd);
    }

    public static BooleanParam createBiasedVarParam(final boolean reqd) {
        return new BooleanParam("biased", "Biased variance for class metrics", "Use biased var(N) instead of UNbiased(N-1)", XPreferencesFactory.kBiasedVar.getBoolean(), reqd);
    }

    public static BooleanParam createMedianParam(final boolean reqd) {
        return new BooleanParam("median", "Median for class  metrics", "Use the median of each class instead of the mean for the class seperation metrics", XPreferencesFactory.kMedian.getBoolean(), reqd);
    }

    public static BooleanParam createMedianParam(final boolean reqd, final String desc) {
        return new BooleanParam("median", "Median for class metrics", desc, XPreferencesFactory.kMedian.getBoolean(), reqd);
    }

    public static BooleanParam createTwoSidedParam(final boolean reqd) {
        return new BooleanParam("two_sided", "Take markers form the top and bottom", reqd);
    }

    public static BooleanParam createTwoSidedParam(final boolean def, final boolean reqd) {
        return new BooleanParam("two_sided", "Take markers form the top and bottom", def, reqd);
    }

    public static BooleanParam createFixLowVarParam(boolean reqd) {
        return new BooleanParam("fixlowvar", "Fix for low variance", XPreferencesFactory.kFixLowVar.getBoolean(), reqd);
    }

    public static IntegerParam createNumMarkersParam(int def, boolean reqd) {
        return new IntegerParam("num", "Number of markers", "Number of markers", def, reqd);
    }

    public static IntegerParam createNumMarkersParam(String name, int def, boolean reqd) {
        return new IntegerParam(name, "Number of markers", "Number of markers", def, reqd);
    }

    public static IntegersChooserParam createNumFeaturesParam(boolean reqd) {
        return new IntegersChooserParam("nfeats", "Number of features", "Number of features", reqd);
    }

    public static BooleanParam createOvaParam(boolean def, boolean reqd) {
        return new BooleanParam("ova", "Split up specified Template into ONE_versus_ALL pairs and use in the analysis", def, reqd);
    }

    public static BooleanParam createAllPairsParam(boolean def, boolean reqd) {
        return new BooleanParam("allpairs", "Split up specified Template into APP_PAIRS and use in the analysis", def, reqd);
    }

    public static IntegerParam createGeneSetMinSizeParam(int def, boolean reqd) {
        return new IntegerParam("set_min", "Min size: exclude smaller sets", "Gene sets smaller than this number are EXCLUDED from the analysis", def, reqd);
    }

    public static IntegerParam createGeneSetMinSizeParam(boolean reqd) {
        return createGeneSetMinSizeParam(10, reqd);
    }

    public static IntegerParam createGeneSetMaxSizeParam(int def, boolean reqd) {
        return new IntegerParam("set_max", "Max size: exclude larger sets", "Gene sets larger than this number are EXCLUDED from the analysis", def, reqd);
    }

    public static IntegerParam createGeneSetMaxSizeParam(boolean reqd) {
        return createGeneSetMaxSizeParam(500, reqd);
    }

    //Qing Xiong
    public static IntegerParam createBpBeforeGeneParam(int def, boolean reqd) {
        return new IntegerParam("set_a_upstream", "Base pairs upstream gene", "Number of base pairs upstream the gene included in the SNP mapping region", def, reqd);
    }

    public static IntegerParam createBpBeforeGeneParam(boolean reqd) {
        return createBpBeforeGeneParam(1000, reqd);
    }

    public static IntegerParam createBpAfterGeneParam(int def, boolean reqd) {
        return new IntegerParam("set_b_downstream", "Base pairs downstream gene", "Number of base pairs downstream the gene included in the SNP mapping region", def, reqd);
    }

    public static IntegerParam createBpAfterGeneParam(boolean reqd) {
        return createBpAfterGeneParam(0, reqd);
    }

    public static IntegerParam createNumOfSnpParam(int def, boolean reqd) {
        return new IntegerParam("num_snp", "Number of SNPs", "Number of SNPs mapped to a gene", def, reqd);
    }

    public static IntegerParam createNumOfSnpParam(boolean reqd) {
        return createNumOfSnpParam(5, reqd);
    }
    //--------------------

    public static BooleanParam createAdvancedParam(boolean reqd) {
        return new BooleanParam("adv", "Advanced mode", reqd);
    }

    public static BooleanParam createFixedRndGeneSetSizeParam(boolean reqd) {
        return new BooleanParam("fix_gset_size", "use the fixed gset size", reqd);
    }

    // @todo change me might have to modify the integerschooser param to have an init
    // and ditto for its super also
    public static IntegersChooserParam createIntegers(String name, String desc, int[] defs, boolean reqd) {
        Integer[] bdefs = new Integer[defs.length];
        for (int i = 0; i < defs.length; i++) {
            bdefs[i] = new Integer(defs[i]);

        }

        return new IntegersChooserParam(name, desc, bdefs, reqd);
    }


    public static IntegerParam createCutoffNumPermParam(boolean reqd) {
        return new IntegerParam("nperm", "number of perms", "The number of permutations", new int[]{1, 10, 100, 1000}, reqd);
    }

    public static FloatParam createMinFoldParam(boolean reqd) {
        return createMinFoldParam(3.0f, reqd);
    }

    public static FloatParam createMinFoldParam(float minFold, boolean reqd) {
        return new FloatParam("min_fold", "Only include those rows with max/min >= specified value", minFold, reqd);
    }

    public static FloatParam createMinDeltaParam(boolean reqd) {
        return createMinDeltaParam(50f, reqd);
    }

    public static FloatParam createMinDeltaParam(float minDelta, boolean reqd) {
        return new FloatParam("min_delta", "Only include those rows with max - min >= specified value", minDelta, reqd);
    }

    public static IntegerParam createMinNumInstancesParam(boolean reqd) {
        return new IntegerParam("min_inst", "Only include drugs with mor einsances than these", 3, reqd);
    }

    public static FloatParam createMinDiffParam(boolean reqd) {
        return new FloatParam("min_diff", "Only include those rows with max-min >= specified value", 3f, reqd);
    }


    public static FloatParam createMinThresholdParam(float min, boolean reqd) {
        return new FloatParam("min_gex", "Change all values < specified to the specified value", min, reqd);
    }

    public static FloatParam createMinThresholdParam(boolean reqd) {
        return createMinThresholdParam(20, reqd);
    }

    public static FloatParam createMaxThresholdParam(float max, boolean reqd) {
        return new FloatParam("max_gex", "Change all values > specified to the specified value", max, reqd);
    }

    public static FloatParam createMaxThresholdParam(boolean reqd) {
        return createMaxThresholdParam(16000f, reqd);
    }

    public static BooleanParam createNormalizeRowsParam(boolean def, boolean reqd) {
        return new BooleanParam("normalize_rows", "Normalioze rows to mean 0 and stddev 1", def, reqd);
    }

    public static FloatsChooserParam createPercentsParam(boolean reqd) {
        return new FloatsChooserParam("percents", "sig levels for the disc mode", new Float(0.05f), reqd);
    }

    private static final float[] pvf = new float[]{0.05f, 0.1f, 0.15f, 0.2f, 0.5f, 1.0f, 0.01f, 0.025f, 0.001f, 0.005f};

    public static FloatParam createNominalPValueCutoffParam(boolean reqd) {
        return new FloatParam("np", "nominal p-value cutoff", pvf, reqd);
    }

    public static FloatParam createDiscoveryValueCutoffParam(boolean reqd) {
        return new FloatParam("dp", "discovery pvalue cutoff", pvf, reqd);
    }

    public static FloatParam createValidationPValueCutoffParam(boolean reqd) {
        return new FloatParam("vp", "validation pvalue cutoff", pvf, reqd);
    }

    public static FloatParam createPValueCutoffParam(boolean reqd) {
        return new FloatParam("pvalue_cutoff", "pvalue cutoff", pvf, reqd);
    }

    public static ChipReqdParam createChipSourceParam() {
        return new ChipReqdParam("chip_source", "chip_source", "The chip that orthology/homology mappings are taken from");
    }

    public static ChipReqdParam createChipTargetParam() {
        return new ChipReqdParam("chip_target", "chip_target", "The chip that orthology/homology mappings are converted to");
    }

    /*
    public static ChipMultiChooserParam createChipsFromParam(boolean reqd) {
        return new ChipMultiChooserParam("chip_from", Param.CHIP_ENGLISH, "The chip that orthology/homology mappings are taken from", reqd, false);
    }

    public static ChipMultiChooserParam createChipsToParam(boolean reqd) {
        return new ChipMultiChooserParam("chip_to", Param.CHIP_ENGLISH, "The chip that orthology/homology mappings are converted to", reqd, false);
    }
    */

    public static ChipChooserMultiParam createChipsSourceParam(boolean reqd) {
        return new ChipChooserMultiParam("chip_source", "Source chip(s)", "The starting chip - from which the orthology/homology mappings are taken", reqd);
    }

    public static ChipChooserMultiParam createChipsTargetParam(boolean reqd) {
        return new ChipChooserMultiParam("chip_target", "Target chip(s)", "The destination chip - to which orthology/homology mappings are converted to", reqd);
    }

    public static IntegersChooserParam createMarkerSpansParam(int[] defs, int[] hints, boolean reqd) {
        Integer[] ihints = new Integer[hints.length];
        for (int i = 0; i < hints.length; i++) {
            ihints[i] = new Integer(hints[i]);
        }

        Integer[] idefs = new Integer[hints.length];
        for (int i = 0; i < defs.length; i++) {
            idefs[i] = new Integer(defs[i]);
        }

        return new IntegersChooserParam("top", "lengths of markers lists", idefs, ihints, reqd);
    }

    public static Map getMetricParams(BooleanParam medianParam) {
        Map map = DatasetMetrics.getDefaultMetricParams();
        map.put(Headers.USE_MEDIAN, medianParam.getValue());
        return Collections.unmodifiableMap(map);
    }

    public static Map getMetricParams(boolean useMedian) {
        Map map = DatasetMetrics.getDefaultMetricParams();
        map.put(Headers.USE_MEDIAN, Boolean.valueOf(useMedian));
        return Collections.unmodifiableMap(map);
    }

    // barfs if size is zero or content is zero for all sets
    public static void checkAndBarfIfZeroSets(final GeneSet[] qual_gsets) {

        boolean wasError = false;
        if (qual_gsets.length == 0) {
            wasError = true;
        } else {
            boolean at_least_one_non_empty_set = false;
            for (int i = 0; i < qual_gsets.length; i++) {
                if (qual_gsets[i].getNumMembers() > 0) {
                    at_least_one_non_empty_set = true;
                    break;
                }
            }
            if (!at_least_one_non_empty_set) {
                wasError = true;
            }
        }

        if (wasError) {
            throw new BadParamException("None of the gene sets passed the size thresholds", 1001);
        }
    }

    public static void checkIfCollapsedIsEmpty(final CollapsedDetails cd) {

        if (!cd.wasCollapsed) {
            return;
        }

        if (cd.getNumRow_orig() == 0) { // huh
            return;
        }

        if (cd.getNumRow_collapsed() != 0) {
            return;
        }

        throw new BadParamException("The collapsed dataset was empty when used with chip:" + cd.getChipName(), 1005);
    }


}    // End ParamFactory

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
