/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GStringsInputFieldPlusChooser;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * entry 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @imp enters in a text are not a list chooser
 */
public class StringMultiInputParam extends AbstractParam {

    private GStringsInputFieldPlusChooser fChooser;
    private static final String PARSE_DELIMS = ",\t\n";// dont parse on spaces

    /**
     * Class constructor
     *
     * @param name
     * @param type
     * @param desc
     */
    public StringMultiInputParam(String name, String desc, String def_andonly_hint, boolean reqd) {
        this(name, desc, new String[]{def_andonly_hint}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param reqd
     */
    public StringMultiInputParam(String name, String desc, boolean reqd) {
        this(name, desc, new String[]{}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hint
     * @param reqd
     */
    public StringMultiInputParam(String name, String nameEnglish, String desc, String[] def_and_hints, boolean reqd) {
        super(name, nameEnglish, String.class, desc, def_and_hints, reqd);
    }

    public StringMultiInputParam(String name, String desc, String[] def_and_hints, boolean reqd) {
        super(name, String.class, desc, def_and_hints, reqd);
    }

    /**
     * @param value String[] or a String(comma delimited parsed, space insensitive)
     */
    public void setValue(Object value) {

        // TraceUtils.showTrace();

        if (value == null) {
            super.setValue(value);
        } else if (value instanceof String[]) {
            super.setValue(value);
        } else if (value instanceof String) {
            super.setValue(_parse((String) value));
        } else {
            throw new IllegalArgumentException("Invalid type, only String[] and comma-delim parsable String accepted. Specified: "
                    + value + " class: " + value.getClass());
        }
    }

    public boolean isSpecified() {
        boolean is = super.isSpecified();

        if (!is) {
            return is;
        }

        Object val = super.getValue();
        if (val == null) {
            return false;
        }

        String[] ss = getStrings();

        //Printf.out(ss);
        if (ss.length == 0) {
            return false;
        }

        return true;
    }

    private String[] _parse(String s) {
        //log.debug("setting: " + s);
        return ParseUtils.string2strings(s, PARSE_DELIMS, false);
    }

    public void setValue(String commadelimvalues) {
        super.setValue(_parse(commadelimvalues));
    }

    public void setValue(String[] values) {
        super.setValue(values);
    }

    public GeneSet getAsGeneSet(String name) {
        return new FSet(name, getStrings());
    }

    public GeneSet getAsGeneSet() {
        return getAsGeneSet(getName());
    }

    public String[] getStrings() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        if (val instanceof String) {
            String s = ((String) val).trim();
            if (s.length() != 0) {
                return new String[]{s};
            } else {
                return null;
            }
        }

        return (String[]) val;
    }

    public float[] getFloats() {
        String[] ss = getStrings();

        float[] ret = new float[ss.length];
        for (int i = 0; i < ss.length; i++) {
            ret[i] = Float.parseFloat(ss[i]);
        }

        return ret;
    }

    public Vector getAsVector() {
        return new Vector(getFloats());
    }

    public int[] getInts() {
        String[] ss = getStrings();

        int[] ret = new int[ss.length];
        for (int i = 0; i < ss.length; i++) {
            ret[i] = Integer.parseInt(ss[i]);
        }

        return ret;
    }

    public Set getStringsS() {
        String[] ss = getStrings();
        Set set = new HashSet();
        for (int i = 0; i < ss.length; i++) {
            set.add(ss[i]);
        }

        return set;
    }

    public String[] getStringsUnique() {
        String[] ss = getStrings();
        List list = new ArrayList(); // use list so that the order is preserved
        for (int i = 0; i < ss.length; i++) {
            if (list.indexOf(ss[i]) == -1) {
                list.add(ss[i]);
            }
        }

        return (String[]) list.toArray(new String[list.size()]);
    }

    public String getString(int pos) {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return ((String[]) val)[pos];
    }

    // param full ignored
    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        if (val instanceof String) {
            return (String) val;
        }

        String[] ss = (String[]) val;

        return format(ss);
    }

    protected static String format(Object[] vals) {

        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {
            if (vals[i] == null) {
                // do nothing
            } else {
                buf.append(vals[i].toString().trim());
                if (i != vals.length - 1) {
                    buf.append(',');
                }
            }
        }

        return buf.toString();
    }

    public int size() {
        return ((String[]) getValue()).length;
    }

    /**
     * Ensures that the string members are all unique and barfs if they are not
     * Not always applicable - let client code decide when to call to check.
     */
    public void ensureUnique() {

        Set set = new HashSet(this.size());

        for (int i = 0; i < this.size(); i++) {
            if (set.contains(this.getString(i))) {
                throw new IllegalArgumentException("Duplicated String param: " + this.getString(i));
            } else {
                set.add(this.getString(i));
            }
        }
    }

    public GFieldPlusChooser getSelectionComponent() {
        if (fChooser == null) {
            String text = this.getValueStringRepresentation(false);
            if (text == null) {
                text = format((String[]) getDefault());
            }

            if (text == null) {
                text = "";
            }

            fChooser = new GStringsInputFieldPlusChooser(text, PARSE_DELIMS);
            //fChooser.setListSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            ParamHelper.addDocumentListener(fChooser.getTextField(), this);
        }

        return fChooser;

    }

    public boolean isFileBased() {
        return false;
    }

}    // End class StringMultiInputParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
