/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.TemplateFactory;
import edu.mit.broad.genome.objects.TemplateMode;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.genome.utils.SystemUtils;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.xbench.xchoosers.TemplateChooserUI;
import edu.mit.broad.xbench.xchoosers.TemplateSelection;
import org.apache.log4j.Logger;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * Helper (common base class) for the TemplateXXXParams
 * <p/>
 * <p/>
 * If single template then:
 * if mode == cont, cb -> only continuous ones and in the jlist only the one continuous one
 * if mode == 2class, cb -> ALL categorical ones and in the jlist only the 2 class categorical ones
 * if mode == multi, cb -> all categorical but NO ova and allpairs
 * <p/>
 * if multi templatge then
 * 1) same
 * 2) same but add ALLPAIRS AND OVA as options
 * 3) all including OVA and ALL_PAIRS
 */
abstract class TemplateChooserAbstractParam extends StringMultiChooserParam implements ActionListener {

    protected TemplateMode fMode;

    private TemplateChooserUI fTemplateChooser;

    private boolean fIsMultiAllowed;

    private TemplateSelection fCurrBag;

    private Logger log = XLogger.getLogger(TemplateChooserAbstractParam.class);

    /**
     * Class constructor
     *
     * @param reqd
     */
    protected TemplateChooserAbstractParam(boolean reqd, TemplateMode mode, boolean isMultiAllowed) {
        this(CLS, CLS_ENGLISH, CLS_DESC, reqd, mode, isMultiAllowed);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param nameEnglish
     * @param desc
     * @param reqd
     * @param mode
     * @param isMultiAllowed
     */
    protected TemplateChooserAbstractParam(final String name,
                                           final String nameEnglish,
                                           final String desc,
                                           final boolean reqd,
                                           final TemplateMode mode,
                                           final boolean isMultiAllowed) {
        super(name, nameEnglish, desc, new String[]{}, new String[]{}, reqd);
        this.fMode = mode;
        this.fIsMultiAllowed = isMultiAllowed;
    }

    public String getLongDesc() {

        if (fLongDesc == null) {
            fLongDesc = new StringBuffer("<html><body>");
            fLongDesc.append(CLS_DESC);
            fLongDesc.append("<br>");

            if (!fIsMultiAllowed) {
                fLongDesc.append(" Only 1 Template allowed<br>");
            } else {
                fLongDesc.append(" One or more Templates allowed<br>");
            }

            fLongDesc.append(fMode.getDesc());
        }

        return fLongDesc.toString();
    }

    public boolean isFileBased() {
        return true;
    }

    protected GFieldPlusChooser _getSelectionComponent() {

        if (fTemplateChooser == null) {
            //log.debug("creating TemplateChooser component");
            fTemplateChooser = new TemplateChooserUI(fIsMultiAllowed, fMode);
            fChooser = new TemplateChooserUI.Field(this);
            fChooser.setValue(getValueStringRepresentation(true));
            ParamHelper.addDocumentListener(fChooser.getTextField(), this);
        }

        return fChooser;
    }

    public void actionPerformed(ActionEvent e) {
        try {
            TemplateSelection bag = fTemplateChooser.showChooser(fCurrBag);
            if (bag != null) {
                fChooser.setText(bag.formatForUI());
            }
            //this.fCurrBag = bag;
        } catch (Throwable t) {
            Application.getWindowManager().showError("Could not query Phenotypes", t);
        }
    }

    /*
    protected Template _getTemplate() throws Exception {

        String[] ss = getStrings();

        if (ss.length == 0) {
            throw new IllegalArgumentException("No phenotypes were specified -- 0 length string array");
        }

        if ((fIsMultiAllowed == false) && (ss.length > 1)) {
            throw new IllegalArgumentException("Only 1 Phenotype argument is accepted specified: " + ss.length + " " + getValueStringRepresentation(true));
        }

        String path = ss[0];
        // need to weed out those that have same name as the template
        // i.e path/to/test.cls#test.cls -> /path/to/test.cls

        if (AuxUtils.isAux(path)) {
            Template mainTemplate = _getMainTemplate(path);
            File baseFile = ParserFactory.getCache().getSourceFile(mainTemplate);
            String baseName = AuxUtils.getBaseStringFromAux(mainTemplate.getName());
            String auxname = AuxUtils.getAuxNameOnlyNoHash(path);
            File file;
            if ((auxname.equals(mainTemplate.getName())) && (auxname.endsWith(Constants.CLS))) {
                file = baseFile; // eg /path/to/test.cls#test.cls
            } else {
                file = new File(baseFile.getParentFile(), baseName + "#" + auxname);
            }

            log.debug("Getting (possibly aux) template from: " + file + " >path: " + path);
            return ParserFactory.readTemplate(file, true);
        } else {
            return ParserFactory.readTemplate(new File(path), true);
        }

    }
    */

    private Template[] _getGeneTemplates(final String[] ss, final Dataset ds) {

        if (ss == null || ss.length == 0) {
            return null;
        }

        try {
            ds.getRowIndex(ss[0]);

            Template[] tss = new Template[ss.length];
            for (int i = 0; i < ss.length; i++) {
                tss[i] = TemplateFactory.createContinuousTemplate(ss[i], ds);
            }

            return tss;

        } catch (Throwable t) {
            return null; // not a gene template
        }
    }

    // the first field has the fullpath, rest dont
    // dont use any UI stuff here -- has to work from the cmd line too
    // might be a mix of:
    // path/to/foo,#bar,/some/path/bar,#zog
    // so do sequentially, using the previous path always
    // @imp disabled magic words auxes (see old method below for it)
    protected Template[] _getTemplates(Dataset dsOptX) throws Exception {

        String[] ss = getStrings();
        Printf.out(ss);

        final Template[] nn_ts = _getGeneTemplates(ss, dsOptX);
        if (nn_ts != null) {
            return nn_ts;
        }

        if (ss.length == 0) {
            throw new IllegalArgumentException("No templates specified -- 0 length str array");
        }

        List templates = new ArrayList();
        Template currMainTemplate = null;

        for (int i = 0; i < ss.length; i++) {
            if (!_isPath(ss[i])) { // either aux or an auto-splitter word, with just the auxname or with a fullpath
                currMainTemplate = _getSourceTemplate(ss[i], currMainTemplate);
                String auxname = AuxUtils.getAuxNameOnlyNoHash(ss[i]);
                log.debug("parsing: " + ss[i] + "< and i got auxname>" + auxname + "<" + " currMaintemplate: " + currMainTemplate.getName());
                if (auxname.equals(Constants.ONE_VERSUS_ALL)) {
                    if (!fIsMultiAllowed) {
                        throw new IllegalArgumentException("Invalid Template option: " + auxname + " multiple templates not allowed in this usage");
                    } else {
                        Template[] ova = TemplateFactory.extractAllOvaTemplates(currMainTemplate, false);
                        for (int t = 0; t < ova.length; t++) {
                            templates.add(ova[t]);
                        }
                    }
                } else if (auxname.equals(Constants.ONE_VERSUS_ALL_ONLY_FORWARD)) {
                    if (!fIsMultiAllowed) {
                        throw new IllegalArgumentException("Invalid Template option: " + auxname + " multiple templates not allowed in this usage");
                    } else {
                        Template[] ova = TemplateFactory.extractAllOvaTemplates(currMainTemplate, true);
                        for (int t = 0; t < ova.length; t++) {
                            templates.add(ova[t]);
                        }
                    }
                } else if (auxname.equals(Constants.ALL_PAIRS)) {
                    if (!fIsMultiAllowed) {
                        throw new IllegalArgumentException("Invalid Template option: " + auxname + " multiple templates not allowed in this usage");
                    } else {
                        Template[] ap = TemplateFactory.extractAllPairsTemplates(currMainTemplate);
                        for (int t = 0; t < ap.length; t++) {
                            templates.add(ap[t]);
                        }
                    }
                } else {
                    File file = ParserFactory.getCache().getSourceFile(currMainTemplate);
                    file = AuxUtils.getBaseFileFromAuxFile(file);
                    Template t = ParserFactory.readTemplate(new File(file.getPath() + "#" + auxname), true);
                    templates.add(t);
                }

            } else {//has to be a path
                //Template t = ParserFactory.readTemplate(new File(ss[i]));
                Template t = ParserFactory.readTemplate(new File(ss[i]), true, false, true); // @note imp to use cache for the templates from sample names
                templates.add(t);
                currMainTemplate = t;
                //Template[] tss = ParserFactory.readTemplates(new File(ss[i]), true, false, false);
                //for (int t = 0; t < tss.length; t++) {
                //  templates.add(tss[t]);
                // }
            }
        }

        return (Template[]) templates.toArray(new Template[templates.size()]);

        /*
        for (int i = 0; i < tss.length; i++) {
            if (tss[i] instanceof PreTemplate) {
                tss[i] = ((PreTemplate) tss[i]).createTemplate(dsOpt);
            }
        }

        return tss;
        */
    }

    private boolean _isPath(String pathOrName) {
        if (pathOrName.indexOf(File.separator) == -1) {
            return false;
        } else {
            return true;
        }
    }

    private Template _getSourceTemplate(String pathOrName, Template currTemplate) throws Exception {
        //System.out.println("asking for: " + pathOrName);
        if (pathOrName.indexOf(File.separatorChar) == -1) { // eg, not a path
            if (currTemplate == null) {
                // may still be in cwd (dir of execution so try) josh gould fix
                return ParserFactory.readTemplate(new File(SystemUtils.getPwd(), pathOrName));
                //throw new RuntimeException("No path specified nor can it be inferred from currTemplate as it is null");
            } else {
                return currTemplate;
            }
        } else {
            return ParserFactory.readTemplate(new File(pathOrName));
        }
    }

    private Template _getMainTemplate(String auxorfullpath) throws Exception {

        File basefile = AuxUtils.getBaseFileFromFullPath(auxorfullpath);
        return ParserFactory.readTemplate(basefile);

    }

} // End class TemplateParamHelper
