/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.chip2chip;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.reports.RichDataframe;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.genome.reports.api.ToolReport;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import edu.mit.broad.genome.reports.web.LinkedFactory;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.MGeneSetMatrix;
import edu.mit.broad.vdb.map.MapUtils;
import edu.mit.broad.vdb.map.MappingEtiology;
import gnu.trove.TIntObjectHashMap;
import xtools.api.ReportBlocks;
import xtools.api.param.*;

import java.io.File;
import java.util.Properties;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Chip2Chip_orig_restore_later extends AbstractMappingTool {

    private final GeneSetMatrixMultiChooserParam fGeneSetMatrixParam = new GeneSetMatrixMultiChooserParam(true);

    private final ChipComboModeReqdParam fCombineChipsModeReqd = new ChipComboModeReqdParam();

    private final BooleanParam fShowEtiology = ParamFactory.createShowEtiologyParam(true, false);

    private final GeneSetMatrixFormatParam fGmFormatParam = new GeneSetMatrixFormatParam(false);

    /**
     * Class constructor
     *
     * @param properties
     */
    public Chip2Chip_orig_restore_later(Properties properties) {
        super.init(properties);
    }

    public Chip2Chip_orig_restore_later(String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public Chip2Chip_orig_restore_later() {
        declareParams();
    }


    public static String createHeader() {
        StringBuffer buf = new StringBuffer();
        buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                "<h3 style=\"text-align: left\"><font color=\"#808080\">Chip2Chip Report").append("</font></h3>\n").append("</div>");

        return buf.toString();
    }

    /**
     * @throws java.lang.Exception
     */
    public void execute() throws Exception {
        // to preserve memory & for custom indexing
        final ReportIndexState state = new ReportIndexState(true, false, true, createHeader());
        startExec(state);

        final GeneSetMatrix gm = fGeneSetMatrixParam.getGeneSetMatrixCombo(true);
        final Chip2ChipMapper[] mappers = fChip2ChipMapperParam.getMappers();

        doMapping(mappers, gm, fShowEtiology.isTrue(), fGmFormatParam, fReport);

        doneExec();
    }

    protected static void doMapping(final Chip2ChipMapper[] mappers,
                                    final GeneSetMatrix gm_to_map,
                                    final boolean showEtiology,
                                    final GeneSetMatrixFormatParam fGmFormatParam,
                                    final ToolReport report) throws Exception {

        final MGeneSetMatrix[] mgms = new MGeneSetMatrix[mappers.length];

        ReportBlocks.SimpleBlockListing list_etiology = null;
        RichDataframe[] summary_tables = new RichDataframe[mappers.length];
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < mappers.length; i++) {
            mgms[i] = mappers[i].map(gm_to_map, showEtiology);
            buf.append(mappers[i].getTargetChip().getName());

            // @note one per mapper
            if (showEtiology) {
                if (list_etiology == null) {
                    list_etiology = new ReportBlocks.SimpleBlockListing("Mapping details", report);
                }

                final MappingEtiology[] mes = mgms[i].getEtiologies();
                File[] files = new File[mes.length];
                for (int m = 0; m < mes.length; m++) {
                    String desc = "Mapping details for " + gm_to_map.getGeneSetName(m) + " by " + mappers[i].getName();
                    files[m] = report.savePageTxt(gm_to_map.getGeneSetName(m) + "_" + mappers[i].getName() + "_etiology",
                            desc,
                            mes[m].getStory(), true, false);
                    //list_etiology.add("Mapping details for", gm.getGeneSetName(m), " by " + mappers[i].getName(), files[m]);
                }

                // Make a summary etiology always
                final String[] colNames = new String[]{"#SIZE BEFORE MAPPING", "#SIZE AFTER MAPPING", "#SOURCE PROBES THAT HAD ONE OR MORE MATCHES", "DETAILS"};
                final String[] rowNames = new String[gm_to_map.getNumGeneSets()];
                final StringMatrix sm = new StringMatrix(rowNames.length, colNames.length);
                final TIntObjectHashMap cell_id_linkMap = new TIntObjectHashMap();
                for (int r = 0; r < gm_to_map.getNumGeneSets(); r++) {
                    rowNames[r] = gm_to_map.getGeneSetName(r);
                    sm.setElement(r, 0, gm_to_map.getGeneSet(r).getNumMembers());
                    sm.setElement(r, 1, mgms[i].getMappedGeneSet(r).getMappedGeneSet(true).getNumMembers());
                    sm.setElement(r, 2, mgms[i].getMappedGeneSet(r).getEtiology().getTotalNumOfSourceProbesActuallyMapped());
                    sm.setElement(r, 3, "Details >>");
                    cell_id_linkMap.put(sm.getElementPos(r, 3), new LinkedFactory.SimpleLinkedFile("Details>>", files[r]));
                }

                summary_tables[i] = new RichDataframe(new StringDataframe(mappers[i].getName(), sm, rowNames, colNames, true),
                        null, null, cell_id_linkMap);
            } // End etiology
        }

        final GeneSetMatrix mapped_gm = MapUtils.createCombinedGeneSetMatrix(NamingConventions.removeExtension(gm_to_map.getName()) + "_mapped_to_" + buf.toString(), mgms);
        final File gm_file = report.savePage(mapped_gm, fGmFormatParam);

        final ReportBlocks.SimpleBlockListing list = new ReportBlocks.SimpleBlockListing("The following " + gm_to_map.getNumGeneSets() + " gene sets were mapped. <br>The results are in this gmt file", report);
        list.add(gm_file.getName(), gm_file);
        list.close();

        // do this after the rest
        if (list_etiology != null) {
            list_etiology.close();
        }

        HtmlReportIndexPage htmlPageHtml = report.getIndexPage();
        if (showEtiology) {
            for (int r = 0; r < summary_tables.length; r++) {
                final File xlsFile = report.savePageXls(summary_tables[r]);
                htmlPageHtml.addTable(summary_tables[r], xlsFile.getName(), true, true);
            }
        }

        report.getIndexPage().setAddBrowseFooter(false);

    }

    public String getDesc() {
        return "Map probe sets between chip platforms";
    }

    public Param[] getAdditionalParams() {
        return new Param[]{
                fGeneSetMatrixParam,
                fCombineChipsModeReqd,
                fShowEtiology,
                fGmFormatParam
        };
    }

    public static void main(String[] args) {
        Chip2Chip_orig_restore_later tool = new Chip2Chip_orig_restore_later(args);
        tool_main(tool);
    }

}    // End Chip2Chip