/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.genesettools;


import edu.mit.broad.genome.alg.AlgUtils;
import edu.mit.broad.genome.objects.GeneSet;
import xtools.api.AbstractTool;
import xtools.api.ReportBlocks;
import xtools.api.ToolCategory;
import xtools.api.param.GeneSetMatrixMultiChooserParam;
import xtools.api.param.IntegerParam;
import xtools.api.param.Param;
import xtools.api.param.ParamFactory;

import java.io.File;
import java.util.Properties;

/**
 * Commandline Tool to run the StackedComparator Tests
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class FindManyOverlaps extends AbstractTool {

    private final GeneSetMatrixMultiChooserParam fDbGeneSetsParam = new GeneSetMatrixMultiChooserParam("db",
            "Gene Set (find overlaps to this set)", Param.GRP_DESC, true, true, false);
    private final GeneSetMatrixMultiChooserParam fQueryGeneSetsParam = new GeneSetMatrixMultiChooserParam("query",
            "query gene sets", "query gene sets", true);

    private final IntegerParam fGeneSetMinSizeParam = ParamFactory.createGeneSetMinSizeParam(false);
    private final IntegerParam fGeneSetMaxSizeParam = ParamFactory.createGeneSetMaxSizeParam(false);

    private final IntegerParam fMinOverlapParam = new IntegerParam("min_overlap", "dont show smaller overlaps", 5, false);

    /**
     * Class constructor
     *
     * @param properties
     */
    public FindManyOverlaps(Properties properties) {
        super.init(properties);
    }

    /**
     * @param args
     */
    public FindManyOverlaps(String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public FindManyOverlaps() {
        declareParams();
    }

    /**
     * @throws Exception
     */
    public void execute() throws Exception {

        startExec(true);

        final GeneSet[] gsets_db = fQueryGeneSetsParam.getGeneSets(fGeneSetMinSizeParam, fGeneSetMaxSizeParam, true);
        final GeneSet[] gsets_query = fDbGeneSetsParam.getGeneSets(fGeneSetMinSizeParam, fGeneSetMaxSizeParam, true);
        final StringBuffer buf = new StringBuffer();

        buf.append("QUERY_NAME\tQUERY_SIZE\tDB_NAME\tDB_SIZE\tOVERLAP\tFRACTION\n");

        for (int c = 0; c < gsets_query.length; c++) { // each query gets 3 columns
            String query_name = gsets_query[c].getName(true);
            for (int r = 0; r < gsets_db.length; r++) {
                final String db_name = gsets_db[r].getName(true);
                final GeneSet ol = AlgUtils.intersect(db_name, gsets_db[r], gsets_query[c]);
                if (ol.getNumMembers() > fMinOverlapParam.getIValue()) {
                    float frac = (float) ol.getNumMembers() / (float) gsets_query[c].getNumMembers();
                    buf.append(query_name).append('\t').append(gsets_query[c].getNumMembers()).append('\t');
                    buf.append(db_name).append('\t').append(gsets_db[r].getNumMembers()).append('\t');
                    buf.append(ol.getNumMembers()).append('\t').append(frac).append('\n');
                }
            }
        }

        final File file = fReport.savePageTxt("overlaps", "overlaps", buf.toString());

        final ReportBlocks.SimpleBlockListing list = new ReportBlocks.SimpleBlockListing("Overlaps are listed below", fReport);

        list.add("Overlaps", file);
        list.close();

        doneExec();
    }

    public ToolCategory getCategory() {
        return ToolCategory.GENESET_TOOLS;
    }

    public void declareParams() {
        fParamSet.addParam(fMinOverlapParam);
        fParamSet.addParam(fDbGeneSetsParam);
        fParamSet.addParam(fQueryGeneSetsParam);
        fParamSet.addParam(fGeneSetMinSizeParam);
        fParamSet.addParam(fGeneSetMaxSizeParam);
    }

    public String getDesc() {
        return "Find pairwise overlaps between a gene set and a target database";
    }

    public static void main(String[] args) {
        FindManyOverlaps tool = new FindManyOverlaps(args);
        tool_main(tool);
    }

}    // End FindManyOverlaps


