/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.genesettools;


import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.TemplateMode;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import edu.mit.broad.xbench.heatmap.GramImager;
import org.apache.ecs.html.Div;
import org.apache.ecs.html.H4;
import org.apache.ecs.html.UL;
import xapps.api.vtools.viewers.VizFactory;
import xtools.api.AbstractTool;
import xtools.api.ToolCategory;
import xtools.api.param.*;

import java.awt.image.BufferedImage;
import java.util.Properties;

/**
 * Tool to extract a dataset by column names
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GeneSetHeatMap extends AbstractTool {

    private final DatasetReqdParam fDatasetParam = new DatasetReqdParam();

    private final StringMultiInputParam fNamesParam = new StringMultiInputParam("names", "Extract these features", "Row or column names from dataset, whitespace or comma delimited", new String[]{}, false);

    private final GeneSetMatrixMultiChooserParam fGeneSetParam = new GeneSetMatrixMultiChooserParam(Param.GRP,
            Param.GRP_ENGLISH, Param.GRP_DESC, false, true, false);

    private final ChipOptParam fChipParam = new ChipOptParam();

    private final TemplateSingleChooserParam fTemplateParam = new TemplateSingleChooserParam(false, TemplateMode.ALL);

    private final GenesOfInterestParam fGinParam = new GenesOfInterestParam();

    /**
     * Class constructor
     *
     * @param properties
     */
    public GeneSetHeatMap(Properties properties) {
        super.init(properties);
    }

    public GeneSetHeatMap(String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public GeneSetHeatMap() {
        declareParams();
    }

    /**
     * @throws java.lang.Exception
     */
    public void execute() throws Exception {

        startExec();

        final Dataset fullDs = fDatasetParam.getDataset();

        GeneSet gset;
        if (fNamesParam.isSpecified()) {
            gset = fNamesParam.getAsGeneSet();
        } else if (fGeneSetParam.isSpecified()) {
            gset = fGeneSetParam.getGeneSets(true)[0];
        } else {
            throw new IllegalArgumentException("At least one of " + fNamesParam.getName() + " or " + fGeneSetParam.getName() + " must be specified");
        }

        final Dataset exDs = new DatasetGenerators().extractRows(fullDs, gset);
        if (exDs.getNumRow() == 0) {
            throw new IllegalArgumentException("No members of the specified gene set are in the dataset!");
        }

        final HtmlReportIndexPage htmlReportIndexPage = fReport.getIndexPage();
        Div div = new Div();
        H4 h4 = new H4("Heat map of dataset " + fullDs.getName() + " in the space of gene set: " + gset.getName(true));
        div.addElement(h4);
        UL ul = new UL();

        fReport.savePageInvisibly2Cache(exDs.getName(), "Expression values in the space of the specified gene set", exDs);

        div.addElement(ul);
        htmlReportIndexPage.addBlock(div, false);

        GramImager imager = VizFactory.createGramImager();
        setChip(fullDs, fChipParam, fGinParam);

        BufferedImage bi;

        if (fTemplateParam.isSpecified()) {
            bi = imager.createBpogImage(exDs, fTemplateParam.getTemplate());
        } else {
            bi = imager.createBpogImage(exDs);
        }


        htmlReportIndexPage.addImage("HeatMap", "Heat map in the space of the specified gene set", bi);

        doneExec();
    }

    public ToolCategory getCategory() {
        return ToolCategory.GENESET_TOOLS;
    }

    public String getDesc() {
        return "Draw a heat map for a gene set";
    }

    public void declareParams() {
        fParamSet.addParam(fDatasetParam);
        fParamSet.addParamPseudoReqd(fGeneSetParam);
        fParamSet.addParamPseudoReqd(fNamesParam);
        fParamSet.addParam(fChipParam);
        fParamSet.addParam(fTemplateParam);
        fParamSet.addParam(fGinParam);
    }

    public static void main(String[] args) {
        GeneSetHeatMap tool = new GeneSetHeatMap(args);
        tool_main(tool);
    }

}    // End GeneSetHeatMap


