/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.genesettools;


import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.DatasetMetrics;
import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.alg.gsea.DefaultGeneSetCohort;
import edu.mit.broad.genome.alg.gsea.GeneSetCohort;
import edu.mit.broad.genome.alg.gsea.GeneSetScoringTables;
import edu.mit.broad.genome.alg.gsea.KSCore;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import xtools.api.AbstractTool;
import xtools.api.ToolCategory;
import xtools.api.param.*;

import java.util.Map;
import java.util.Properties;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GeneSetStats extends AbstractTool {

    private final DatasetReqdParam fDatasetParam = new DatasetReqdParam();
    private final TemplateMultiChooserParam fTemplateParam = new TemplateMultiChooserParam(true, TemplateMode.CATEGORICAL_2_CLASS_ONLY);

    private final GeneSetMatrixMultiChooserParam fGeneSetMatrixParam = new GeneSetMatrixMultiChooserParam(true);

    private final OrderParam fOrderParam = new OrderParam(false);
    private final SortParam fSortParam = new SortParam(false);
    private final MetricParam fMetricParam = new MetricParam(false);
    private final BooleanParam fMedianParam = ParamFactory.createMedianParam(false);
    private final RankedListProcessorParam fRankedListProcParam = new RankedListProcessorParam(false);


    /**
     * Class constructor
     *
     * @param properties
     */
    public GeneSetStats(Properties properties) {
        super.init(properties);
    }

    public GeneSetStats(String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public GeneSetStats() {
        declareParams();
    }

    /**
     * @throws java.lang.Exception
     */
    public void execute() throws Exception {
        startExec();

        final Dataset fullDs = fDatasetParam.getDataset();
        final Template[] tss = fTemplateParam.getTemplates(fullDs);
        final SortMode sort = fSortParam.getMode();
        final Order order = fOrderParam.getOrder();
        final Metric metric = fMetricParam.getMetric();
        final LabelledVectorProcessor rp = fRankedListProcParam.getProcessor();

        final Map mps = fMetricParam.getMetricParams(fMedianParam);

        final GeneSet[] gsets = fGeneSetMatrixParam.getGeneSets(fullDs, true);

        // colnames
        final String[] colNames = new String[]{"ES", "S2N_MEAN", "S2N_MEDIAN", "MEAN_CLASS_A", "MEAN_CLASS_B", "MEDIAN_CLASS_A", "MEDIAN_CLASS_B", "#GENES"};

        final String[] rowNames = new String[gsets.length];

        final KSCore core = new KSCore();
        final DatasetMetrics dm = new DatasetMetrics();
        final DatasetGenerators gen = new DatasetGenerators();

        final Dataframe[] dfs = new Dataframe[tss.length];
        for (int t = 0; t < tss.length; t++) {
            final ScoredDataset sds = dm.scoreDataset(metric, sort, order, mps, rp, fullDs, tss[t]);
            final Dataset dsA = gen.extract(fullDs, tss[t].getClass(0));
            final Dataset dsB = gen.extract(fullDs, tss[t].getClass(1));
            final Matrix m = new Matrix(rowNames.length, colNames.length);
            final GeneSetCohort gcoh = new DefaultGeneSetCohort(new GeneSetScoringTables.Classic(), sds, gsets, true);
            final float[] ess = core.calculateKSScores(gcoh);

            for (int r = 0; r < gsets.length; r++) {
                int coln = 0;
                m.setElement(r, coln++, ess[r]);

                float[] s2n_scores = sds.getScores(gsets[r]);
                m.setElement(r, coln++, XMath.mean(s2n_scores));
                m.setElement(r, coln++, XMath.median(s2n_scores));

                Vector[] vas = dsA.getRows(gsets[r]);
                Vector[] vbs = dsB.getRows(gsets[r]);

                m.setElement(r, coln++, XMath.mean_of_means(vas));
                m.setElement(r, coln++, XMath.mean_of_means(vbs));

                m.setElement(r, coln++, XMath.median_of_medians(vas));
                m.setElement(r, coln++, XMath.median_of_medians(vbs));

                m.setElement(r, coln, gsets[r].getNumMembers());

                if (t == 0) {
                    rowNames[r] = gsets[r].getName();
                }
            }

            dfs[t] = new Dataframe("stats_for_" + tss[t].getName(), m, rowNames, colNames, true);
            fReport.savePageDF("Statistics for dataset: " + fullDs.getName() + " and phenotype: " + tss[t].getName(), dfs[t]);
        }

        // now save report by gene set
        for (int i = 0; i < gsets.length; i++) {
            Matrix m = new Matrix(tss.length, colNames.length);
            String[] trowNames = new String[tss.length];
            for (int r = 0; r < tss.length; r++) {
                m.setRow(r, dfs[r].getRow(gsets[i].getName()));
                trowNames[r] = tss[r].getName();
            }

            fReport.savePageDF("Statistics for gene set: " + gsets[i].getName(), new Dataframe("stats_for_" + gsets[i].getName(), m, trowNames, colNames, true));
        }

        doneExec();

    }

    public ToolCategory getCategory() {
        return ToolCategory.GENESET_TOOLS;
    }

    public String getDesc() {
        return "Calculate some useful statistics about the input gene sets";
    }

    public void declareParams() {
        fParamSet.addParam(fDatasetParam);
        fParamSet.addParam(fTemplateParam);
        fParamSet.addParam(fSortParam);
        fParamSet.addParam(fOrderParam);
        fParamSet.addParam(fGeneSetMatrixParam);
        fParamSet.addParam(fMetricParam);
        fParamSet.addParam(fMedianParam);
        fParamSet.addParam(fRankedListProcParam);
    }

    public static void main(String[] args) {
        GeneSetStats tool = new GeneSetStats(args);
        tool_main(tool);
    }

}    // End GeneSetStatistics

