package xtools.gsea;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.alg.gsea.GeneSetCohortGenerator;
import edu.mit.broad.genome.alg.gsea.KSTests;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.genome.parsers.EdbFolderParser;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.GsaaEnrichmentReports;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import xtools.api.param.*;
import edu.duke.igsp.app.params.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public abstract class AbstractGsaa2Tool extends AbstractGsaaTool {
    protected final MetricParam fMetricParam = new MetricParam(createMetricsForExp(), false);
    protected final ExpDatasetParam fExpDatasetParam = new ExpDatasetParam(true);
    protected final SnpDatasetParam fSnpDatasetParam = new SnpDatasetParam(true);
    protected final ExpTemplateParam fExpTemplateParam = new ExpTemplateParam(true);
    protected final SnpTemplateParam fSnpTemplateParam = new SnpTemplateParam(true);
    protected final CMetricParam fCMetricParam = new CMetricParam(createMetricsForSnp(), false);
    protected final StringReqdParam fSpeciesParam = ParamFactory.createSpeciesParam();
    protected final OrderParam fOrderParam = new OrderParam(false);
    protected final SortParam fSortParam = new SortParam(false);
    protected final PermuteTypeChooserParam fPermuteTypeParamType = PermuteTypeChooserParam.createTemplate(true);
    protected final BooleanParam fMedianParam = ParamFactory.createMedianParam(false);
    protected final IntegerParam fNumMarkersParam = ParamFactory.createNumMarkersParam(100, false);
    protected final BooleanParam fSaveRndRankedListsParam = new BooleanParam("save_rnd_lists", "Save random ranked lists", "Save random ranked lists (might be very large)", false, false);
    protected final TemplateRandomizerTypeParam fRndTypeParam = new TemplateRandomizerTypeParam(
            TemplateRandomizerType.NO_BALANCE,
            new TemplateRandomizerType[]{TemplateRandomizerType.NO_BALANCE,
                TemplateRandomizerType.EQUALIZE_AND_BALANCE}, true);

    /**
     * Class constructor
     *
     * @param properties
     */
    protected AbstractGsaa2Tool() {
    }

    protected void doAdditionalParams() {
        fParamSet.addParam(fExpDatasetParam);
        fParamSet.addParam(fSnpDatasetParam);
        fParamSet.addParamPseudoReqd(fExpTemplateParam);
        fParamSet.addParamPseudoReqd(fSnpTemplateParam);
        fParamSet.addParam(fSpeciesParam);
        fParamSet.addParamBasic(fMetricParam);
        fParamSet.addParamBasic(fCMetricParam);
        fParamSet.addParamPseudoReqd(fPermuteTypeParamType);
        fParamSet.addParamAdv(fMedianParam);
        fParamSet.addParamAdv(fNumMarkersParam);
        fParamSet.addParamAdv(fSaveRndRankedListsParam);
        fParamSet.addParamAdv(fRndTypeParam);
    }

    protected EnrichmentDb execute_one(final CollapsedDetails.Data fullCd,
            final Template template,
            final GeneSet[] gsets,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final String imetric,
            final String smetric) throws Exception {

        final int nperms = fNumPermParam.getIValue();
        final Metric metric = fMetricParam.getMetric();
        final SortMode sort = fSortParam.getMode();
        final Order order = fOrderParam.getOrder();

        final LabelledVectorProcessor lvp = new LabelledVectorProcessors.None();

        final RandomSeedGenerator rst = fRndSeedTypeParam.createSeed();
        final Map mps = fMetricParam.getMetricParams(fMedianParam);
        final GeneSetCohortGenerator gcohgen = fGcohGenReqdParam.createGeneSetCohortGenerator(false);

        final DatasetTemplate dt = new DatasetGenerators().extract(fullCd.getDataset(), template);

        log.debug(">>>>> Using samples: " + dt.getDataset().getColumnNames());

        final KSTests tests = new KSTests(getOutputStream());

        List store_rnd_ranked_lists_here_opt = null;
        if (fSaveRndRankedListsParam.isTrue()) {
            store_rnd_ranked_lists_here_opt = new ArrayList();
        }

        return tests.executeGsaaM(
                dt,
                gsets,
                nperms,
                metric,
                imetric,
                smetric,
                sort,
                order,
                lvp,
                rst,
                fRndTypeParam.getRandomizerType(),
                mps,
                gcohgen,
                fPermuteTypeParamType.permuteTemplate(),
                fNumMarkersParam.getIValue(),
                store_rnd_ranked_lists_here_opt,
                datasets,
                templates,
                params);

    }

    /**
     * @throws Exception
     */
    protected void execute_one_with_reporting(final CollapsedDetails.Data fullCd,
            final Template template,
            final GeneSet[] gsets,
            final HtmlReportIndexPage reportIndexPage,
            final boolean makeSubDir,
            final GeneSet[] origGeneSets,
            final int showDetailsForTopXSets,
            final boolean makeZippedReport,
            final boolean makeGeneSetReports,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final String imetric,
            final String smetric) throws Exception {

        final EnrichmentDb edb = execute_one(fullCd, template, gsets, datasets, templates, params, imetric, smetric);

        // -------------------------------------------------------------------------------------------- //
        // rest are for the reporting

        final Metric metric = fMetricParam.getMetric();
        final int minSize = fGeneSetMinSizeParam.getIValue();
        final int maxSize = fGeneSetMaxSizeParam.getIValue();

        final DatasetTemplate dt = new DatasetGenerators().extract(fullCd.getDataset(), template);

        List store_rnd_ranked_lists_here_opt = null;
        if (fSaveRndRankedListsParam.isTrue()) {
            store_rnd_ranked_lists_here_opt = new ArrayList();
        }

        // Make the report
        GsaaEnrichmentReports.Ret ret = GsaaEnrichmentReports.createGseaLikeReport(
                edb,
                getOutputStream(),
                fullCd,
                reportIndexPage,
                makeSubDir,
                fReport,
                showDetailsForTopXSets,
                minSize, maxSize,
                makeGeneSetReports,
                makeZippedReport,
                origGeneSets,
                metric.getName(),
                fNormModeParam.getNormModeName());

        // Save the rnd ranked lists
        if (store_rnd_ranked_lists_here_opt != null && store_rnd_ranked_lists_here_opt.isEmpty() == false) {
            File dir = fReport.createSubDir("random_ranked_lists");
            for (int r = 0; r < store_rnd_ranked_lists_here_opt.size(); r++) {
                RankedList rl = (RankedList) store_rnd_ranked_lists_here_opt.get(r);
                File file = new File(dir, rl.getName());
                ParserFactory.save(rl, file, false);
            }
        }

        // Make an edb folder thing
        new EdbFolderParser().export(ret.edb, ret.savedInDir);
    }
}    // End AbstractGsaa2Tool

