package xtools.gsea;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.gsea.GeneSetCohortGenerator;
import edu.mit.broad.genome.alg.gsea.KSTests;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.genome.parsers.EdbFolderParserLessKS;
import edu.mit.broad.genome.parsers.EdbFolderParserLessMinp;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.Seq1EnrichmentReports;
import edu.mit.broad.genome.reports.Seq2EnrichmentReports;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import xtools.api.param.*;
import edu.duke.igsp.app.params.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Qing Xiong <qingx@swu.edu.cn> & <qxiong@email.unc.edu>
 * Date: January 22, 2013
 *
 */
public abstract class AbstractGsaaSeqSP2Tool extends AbstractGsaaSeqSPTool {

    private static final Logger xlog = XLogger.getLogger(AbstractGsaaSeqSP2Tool.class);
    protected final ExpDatasetParam fDatasetParam = new ExpDatasetParam(true);
    protected final ExpTemplateParam fTemplateParam = new ExpTemplateParam(true);
    protected final StringInputParam fPThresholdParam = ParamFactory.createPThresholdParam();
    protected final StringReqdParam fDEMetricParam = ParamFactory.createDEMetricParamSeqSP();
    protected final StringReqdParam fGSAMetricParam = ParamFactory.createGSAMetricParamSeqSP();
    protected final OrderParam fOrderParam = new OrderParam(false);
    protected final SortParam fSortParam = new SortParam(false);
    protected final PermuteTypeChooserParam fPermuteTypeParamType = PermuteTypeChooserParam.createTemplateOrGeneSet(true);
    protected final BooleanParam fSaveRndRankedListsParam = new BooleanParam("save_rnd_lists", "Save random ranked lists", "Save random ranked lists (might be very large)", false, false);
    protected final TemplateRandomizerTypeParam fRndTypeParam = new TemplateRandomizerTypeParam(
            TemplateRandomizerType.NO_BALANCE,
            new TemplateRandomizerType[]{TemplateRandomizerType.NO_BALANCE,
                TemplateRandomizerType.EQUALIZE_AND_BALANCE}, true);

    /**
     * Class constructor
     *
     * @param properties
     */
    protected AbstractGsaaSeqSP2Tool() {
    }

    protected void doAdditionalParams() {
        fParamSet.addParam(fDatasetParam);
        fParamSet.addParamPseudoReqd(fTemplateParam);
        fParamSet.addParamBasic(fPThresholdParam);
        fParamSet.addParamBasic(fGSAMetricParam);
        fParamSet.addParamBasic(fDEMetricParam);
        fParamSet.addParamPseudoReqd(fPermuteTypeParamType);
        fParamSet.addParamAdv(fSaveRndRankedListsParam);
        fParamSet.addParamAdv(fRndTypeParam);
    }

    protected EnrichmentDb execute_one(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final CollapsedDetails.Data fullCd,
            final Template template,
            final GeneSet[] gsets,
            final List store_rnd_ranked_lists_here_opt) throws Exception {

        final int nperms = fNumPermParam.getIValue();
        final SortMode sort = fSortParam.getMode();
        final Order order = fOrderParam.getOrder();
        final LabelledVectorProcessor lvp = new LabelledVectorProcessors.None();
        final RandomSeedGenerator rst = fRndSeedTypeParam.createSeed();
        final GeneSetCohortGenerator gcohgen = fGcohGenReqdParam.createGeneSetCohortGenerator(false);
        final DatasetTemplate dt = new DatasetGenerators().extract(fullCd.getDataset(), template);

        log.debug(">>>>> Using samples: " + dt.getDataset().getColumnNames());

        final KSTests tests = new KSTests(getOutputStream());

        return tests.executeGsaaSeqsp(
                demetric,
                gsametric,
                pthreshold,
                dt,
                gsets,
                nperms,
                sort,
                order,
                lvp,
                rst,
                fRndTypeParam.getRandomizerType(),
                gcohgen,
                fPermuteTypeParamType.permuteTemplate(),
                store_rnd_ranked_lists_here_opt);

    }

    /**
     * @throws Exception
     */
    protected void execute_one_with_reporting(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final CollapsedDetails.Data fullCd,
            final Template template,
            final GeneSet[] gsets,
            final HtmlReportIndexPage reportIndexPage,
            final boolean makeSubDir,
            final GeneSet[] origGeneSets,
            final int showDetailsForTopXSets,
            final boolean makeZippedReport,
            final boolean makeGeneSetReports) throws Exception {

        List store_rnd_ranked_lists_here_opt = null;
        if (fSaveRndRankedListsParam.isTrue()) {
            store_rnd_ranked_lists_here_opt = new ArrayList();
        }

        final EnrichmentDb edb = execute_one(demetric, gsametric, pthreshold, fullCd, template, gsets, store_rnd_ranked_lists_here_opt);

        // -------------------------------------------------------------------------------------------- //
        // rest are for the reporting

        //final Metric metric = fMetricParam.getMetric();
        final int minSize = fGeneSetMinSizeParam.getIValue();
        final int maxSize = fGeneSetMaxSizeParam.getIValue();

        final DatasetTemplate dt = new DatasetGenerators().extract(fullCd.getDataset(), template);

        // Make the report
        if (gsametric.equalsIgnoreCase("Weighted_KS")) {
            Seq1EnrichmentReports.Ret ret = Seq1EnrichmentReports.createGseaLikeReport(
                    edb,
                    getOutputStream(),
                    fullCd,
                    reportIndexPage,
                    makeSubDir,
                    fReport,
                    showDetailsForTopXSets,
                    minSize, maxSize,
                    makeGeneSetReports,
                    makeZippedReport,
                    origGeneSets,
                    fNormModeParam.getNormModeName());
            // Save the rnd ranked lists
            if (store_rnd_ranked_lists_here_opt != null && store_rnd_ranked_lists_here_opt.isEmpty() == false) {
                File dir = fReport.createSubDir("random_ranked_lists");
                for (int r = 0; r < store_rnd_ranked_lists_here_opt.size(); r++) {
                    RankedList rl = (RankedList) store_rnd_ranked_lists_here_opt.get(r);
                    File file = new File(dir, rl.getName());
                    ParserFactory.save(rl, file, false);
                }
            }

            if (fPermuteTypeParamType.permuteTemplate() && dt.getTemplate().isCategorical()) {
                if (dt.getTemplate().getClass(0).getSize() < 7) {
                    fReport.addComment("Warning: Phenotype permutation was performed but the number of samples in class A is < 7, phenotype: " + dt.getTemplateName());
                }

                if (dt.getTemplate().getClass(1).getSize() < 7) {
                    fReport.addComment("Warning: Phenotype permutation was performed but the number of samples in class B is < 7, phenotype: " + dt.getTemplateName());
                }

                if (dt.getTemplate().getNumItems() < 14) {
                    fReport.addComment("With small datasets, there might not be enough random permutations of sample labels to generate a sufficient null distribution. " +
                            "In such cases, gene_set randomization might be a better choice.");
                }
            }

            // Make an edb folder thing
            new EdbFolderParserLessKS().export(ret.edb, ret.savedInDir);
        } else {
            Seq2EnrichmentReports.Ret ret = Seq2EnrichmentReports.createGseaLikeReportSeq_p(
                    edb,
                    getOutputStream(),
                    fullCd,
                    reportIndexPage,
                    makeSubDir,
                    fReport,
                    showDetailsForTopXSets,
                    minSize, maxSize,
                    makeGeneSetReports,
                    makeZippedReport,
                    origGeneSets,
                    fNormModeParam.getNormModeName());          
            // Save the rnd ranked lists
            if (store_rnd_ranked_lists_here_opt != null && store_rnd_ranked_lists_here_opt.isEmpty() == false) {
                File dir = fReport.createSubDir("random_ranked_lists");
                for (int r = 0; r < store_rnd_ranked_lists_here_opt.size(); r++) {
                    RankedList rl = (RankedList) store_rnd_ranked_lists_here_opt.get(r);
                    File file = new File(dir, rl.getName());
                    ParserFactory.save(rl, file, false);
                }
            }

            if (fPermuteTypeParamType.permuteTemplate() && dt.getTemplate().isCategorical()) {
                if (dt.getTemplate().getClass(0).getSize() < 7) {
                    fReport.addComment("Warning: Phenotype permutation was performed but the number of samples in class A is < 7, phenotype: " + dt.getTemplateName());
                }

                if (dt.getTemplate().getClass(1).getSize() < 7) {
                    fReport.addComment("Warning: Phenotype permutation was performed but the number of samples in class B is < 7, phenotype: " + dt.getTemplateName());
                }

                if (dt.getTemplate().getNumItems() < 14) {
                    fReport.addComment("With small datasets, there might not be enough random permutations of sample labels to generate a sufficient null distribution. " +
                            "In such cases, gene_set randomization might be a better choice.");
                }
            }

            // Make an edb folder thing
            new EdbFolderParserLessMinp().export(ret.edb, ret.savedInDir);
        }
    }
}
