package xtools.gsea;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import xtools.api.AbstractTool;
import xtools.api.ToolCategory;
import xtools.api.param.*;
import java.util.HashSet;
import java.util.Set;


/**
 * @author Qing Xiong <qingx@swu.edu.cn> & <qxiong@email.unc.edu>
 * Date: January 22, 2013
 *
 */
public abstract class AbstractGsaaSeqSPTool extends AbstractTool {
   private static final Logger xlog = XLogger.getLogger(AbstractGsaaSeqSPTool.class);
    protected GeneSetMatrixMultiChooserParam fGeneSetMatrixParam;
    protected final IntegerParam fGeneSetMinSizeParam = ParamFactory.createGeneSetMinSizeParam(15, false);
    protected final IntegerParam fGeneSetMaxSizeParam = ParamFactory.createGeneSetMaxSizeParam(500, false);
    protected final IntegerParam fNumPermParam = ParamFactory.createNumPermParam(true);
    protected final RandomSeedTypeParam fRndSeedTypeParam = new RandomSeedTypeParam(false);
    protected final NormModeReqdParam fNormModeParam = new NormModeReqdParam("GSAA_Seq");
    protected final GeneSetScoringTableReqdParam fGcohGenReqdParam = new GeneSetScoringTableReqdParam();

    /**
     * Class constructor
     *
     * @param properties
     */
    protected AbstractGsaaSeqSPTool() {
    }

    public ToolCategory getCategory() {
        return ToolCategory.GSEA;
    }

    protected CollapsedDetails.Data getDataset_no_collapse(final Dataset origDs) throws Exception {
        CollapsedDetails.Data cd = new CollapsedDetails.Data();
        cd.orig = origDs;
        cd.wasCollapsed = false;
        cd.collapsed = origDs;
        log.info("No dataset collapsing was done .. using original as is");
        return cd;
    }

    protected CollapsedDetails.Ranked getRankedList_no_collapse(final RankedList origRL) throws Exception {
        CollapsedDetails.Ranked cd = new CollapsedDetails.Ranked();
        cd.orig = origRL;
        cd.wasCollapsed = false;
        cd.collapsed = origRL;
        log.info("No ranked list collapsing was done .. using original as is");
        return cd;
    }

    protected abstract Param[] getAdditionalParams();

    protected void doAdditionalParams() {
    }

    public void declareParams() {
        this.fGeneSetMatrixParam = new GeneSetMatrixMultiChooserParam(true);

        // reqd
        fParamSet.addParam(fGeneSetMatrixParam);
        fParamSet.addParam(fNumPermParam);
        doAdditionalParams();

        // basic
        fParamSet.addParamBasic(fGcohGenReqdParam);
        fParamSet.addParamBasic(fGeneSetMinSizeParam);
        fParamSet.addParamBasic(fGeneSetMaxSizeParam);
        
        // advanced
        fParamSet.addParamAdv(fRndSeedTypeParam);
        fParamSet.addParamAdv(fNormModeParam);

        Param[] add = getAdditionalParams();
        for (int i = 0; i < add.length; i++) {
            fParamSet.addParam(add[i]);
        }
    }

    protected Dataset uniquize(final Dataset ds) {

        final GeneSet gset = ds.getRowNamesGeneSet();
        if (gset.getNumMembers() == ds.getNumRow()) {
            return ds;
        } else {
            StringBuffer buf = new StringBuffer();
            buf.append("There were duplicate row identifiers in the specified dataset. One id was arbitarilly choosen. Details are below");
            buf.append("\nGenerally, this is OK, but if you want to avoid this automagic, edit your dataset so that all row ids are unique\n");
            buf.append('\n');
            buf.append("# of row ids in original dataset: ").append(ds.getNumRow()).append('\n');
            buf.append("# of row UNIQUE ids in original dataset: ").append(gset.getNumMembers()).append('\n');
            buf.append("# The duplicates were\n");

            Set all = new HashSet();
            for (int i = 0; i < ds.getNumRow(); i++) {
                String member = ds.getRowName(i);
                if (all.contains(member)) {
                    buf.append(member).append('\n');
                }

                all.add(member);
            }

            fReport.addComment(buf.toString());
            return new DatasetGenerators().extractRows(ds, gset);
        }

    }


    protected RankedList uniquize(final RankedList rl) {

        final GeneSet gset = new FSet(rl.getName(), rl.getName(), rl.getRankedNames(), true);
        if (gset.getNumMembers() == rl.getSize()) {
            return rl;
        } else {
            StringBuffer buf = new StringBuffer();
            buf.append("There were duplicate row identifiers in the specified ranked list. One id was arbitarilly choosen. Details are below. ");
            buf.append("\nGenerally, this is OK, but if you want to avoid this automagic, edit your ranked list so that all row ids are unique\n");
            buf.append('\n');
            buf.append("# of row ids in original dataset: ").append(rl.getSize()).append('\n');
            buf.append("# of row UNIQUE ids in original dataset: ").append(gset.getNumMembers()).append('\n');
            buf.append("# The duplicates were\n<br><pre>");

            Set all = new HashSet();
            for (int i = 0; i < rl.getSize(); i++) {
                String member = rl.getRankName(i);
                if (all.contains(member)) {
                    buf.append(member).append('\n');
                }

                all.add(member);
            }

            buf.append("</pre>");
            fReport.addComment(buf.toString());
            return rl.extractRanked(gset);
        }
    }

}
