package xtools.gsea;

import edu.mit.broad.genome.alg.gsea.GeneSetCohortGenerator;
import edu.mit.broad.genome.alg.gsea.KSTests;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.esmatrix.db.SnpEnrichmentDb;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.genome.parsers.SnpEdbFolderParser;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.GsaaSnpEnrichmentReports;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import xtools.api.param.*;
import edu.duke.igsp.app.algorithms.*;
import edu.duke.igsp.app.params.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public abstract class AbstractGsaaSnp2Tool extends AbstractGsaaSnpTool {
    protected final SnpDatasetParam fSnpDatasetParam = new SnpDatasetParam(true);
    protected final SnpTemplateParam fSnpTemplateParam = new SnpTemplateParam(true);
    protected final GeneListParam fGeneListParam = new GeneListParam(true);
    protected final CMetricParam fCMetricParam = new CMetricParam(createMetricsForSnp(), false);
    protected final StringReqdParam fSpeciesParam = ParamFactory.createSpeciesParam();
    protected final OrderParam fOrderParam = new OrderParam(false);
    protected final SortParam fSortParam = new SortParam(false);
    protected final PermuteTypeChooserParam fPermuteTypeParamType = PermuteTypeChooserParam.createTemplateOrGeneSet(true);
    protected final BooleanParam fSaveRndRankedListsParam = new BooleanParam("save_rnd_lists", "Save random ranked lists", "Save random ranked lists (might be very large)", false, false);
    protected final TemplateRandomizerTypeParam fRndTypeParam = new TemplateRandomizerTypeParam(
            TemplateRandomizerType.NO_BALANCE,
            new TemplateRandomizerType[]{TemplateRandomizerType.NO_BALANCE,
                    TemplateRandomizerType.EQUALIZE_AND_BALANCE}, true);

    /**
     * Class constructor
     *
     * @param properties
     */
    protected AbstractGsaaSnp2Tool() {
    }

    protected void doAdditionalParams() {
        fParamSet.addParam(fSnpDatasetParam);
        fParamSet.addParam(fGeneListParam);
        fParamSet.addParam(fSnpTemplateParam);
        fParamSet.addParam(fSpeciesParam);
        fParamSet.addParamBasic(fCMetricParam);
        fParamSet.addParamPseudoReqd(fPermuteTypeParamType);
        fParamSet.addParamAdv(fSaveRndRankedListsParam);
        fParamSet.addParamAdv(fRndTypeParam);
    }

    protected SnpEnrichmentDb execute_one(final Dataset ds,
                                       final Template template,
                                       final GeneSet[] gsets,
                                       final Template[] rndTemplates
    ) throws Exception {

        final int nperms = fNumPermParam.getIValue();
        final CMetric cmetric = fCMetricParam.getCMetric();
        final SortMode sort = fSortParam.getMode();
        final Order order = fOrderParam.getOrder();
        final LabelledVectorProcessor lvp = new LabelledVectorProcessors.None(); // @note
        final RandomSeedGenerator rst = fRndSeedTypeParam.createSeed();
        final GeneSetCohortGenerator gcohgen = fGcohGenReqdParam.createGeneSetCohortGenerator(false);
        final DatasetTemplate dt =new DatasetTemplate(ds, template);
        final KSTests tests = new KSTests(getOutputStream());
        List store_rnd_ranked_lists_here_opt = null;
        if (fSaveRndRankedListsParam.isTrue()) {
            store_rnd_ranked_lists_here_opt = new ArrayList();
        }

        return tests.executeGsaaSnpKS(
                dt,
                gsets,
                nperms,
                sort,
                order,
                lvp,
                rst,
                fRndTypeParam.getRandomizerType(),
                gcohgen,
                fPermuteTypeParamType.permuteTemplate(),
                store_rnd_ranked_lists_here_opt,
                cmetric,
                rndTemplates
        );

    }

    /**
     * @throws Exception
     */
    protected void execute_one_with_reporting(final Dataset fullDs,
                                              final Template template,
                                              final GeneSet[] gsets,
                                              final HtmlReportIndexPage reportIndexPage,
                                              final boolean makeSubDir,
                                              final GeneSet[] origGeneSets,
                                              final int showDetailsForTopXSets,
                                              final boolean makeZippedReport,
                                              final boolean makeGeneSetReports,
                                              final Template[] rndTemplates

    ) throws Exception {

        final SnpEnrichmentDb edb = execute_one(fullDs, template, gsets, rndTemplates);

        // -------------------------------------------------------------------------------------------- //
        // rest are for the reporting

        final CMetric cmetric = fCMetricParam.getCMetric();
        final int minSize = fGeneSetMinSizeParam.getIValue();
        final int maxSize = fGeneSetMaxSizeParam.getIValue();

        final DatasetTemplate dt = new DatasetTemplate(fullDs, template);

        List store_rnd_ranked_lists_here_opt = null;
        if (fSaveRndRankedListsParam.isTrue()) {
            store_rnd_ranked_lists_here_opt = new ArrayList();
        }

        // Make the report
        GsaaSnpEnrichmentReports.Ret ret = GsaaSnpEnrichmentReports.createGseaLikeReport(
                edb,
                getOutputStream(),
                fullDs,
                reportIndexPage,
                makeSubDir,
                fReport,
                showDetailsForTopXSets,
                minSize, maxSize,
                makeGeneSetReports,
                makeZippedReport,
                origGeneSets,
                cmetric.getName(),
                fNormModeParam.getNormModeName());

        // Save the rnd ranked lists
        if (store_rnd_ranked_lists_here_opt != null && store_rnd_ranked_lists_here_opt.isEmpty() == false) {
            File dir = fReport.createSubDir("random_ranked_lists");
            for (int r = 0; r < store_rnd_ranked_lists_here_opt.size(); r++) {
                RankedList rl = (RankedList) store_rnd_ranked_lists_here_opt.get(r);
                File file = new File(dir, rl.getName());
                ParserFactory.save(rl, file, false);
            }
        }

        if (fPermuteTypeParamType.permuteTemplate() && dt.getTemplate().isCategorical()) {
            if (dt.getTemplate().getClass(0).getSize() < 7) {
                fReport.addComment("Warning: Phenotype permutation was performed but the number of samples in class A is < 7, phenotype: " + dt.getTemplateName());
            }

            if (dt.getTemplate().getClass(1).getSize() < 7) {
                fReport.addComment("Warning: Phenotype permutation was performed but the number of samples in class B is < 7, phenotype: " + dt.getTemplateName());
            }

            if (dt.getTemplate().getNumItems() < 14) {
                fReport.addComment("With small datasets, there might not be enough random permutations of sample labels to generate a sufficient null distribution. " +
                        "In such cases, gene_set randomization might be a better choice.");
            }
        }

        // Make an edb folder thing
        new SnpEdbFolderParser().export(ret.edb, ret.savedInDir);
    }

}    // End AbstractGsaaSnp2Tool
