package xtools.gsea;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.alg.Metrics;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.vdb.chip.Chip;
import xtools.api.AbstractTool;
import xtools.api.ToolCategory;
import xtools.api.param.*;
import edu.duke.igsp.app.algorithms.*;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public abstract class AbstractGsaaTool extends AbstractTool {

    private static final Logger xlog = XLogger.getLogger(AbstractGsaaTool.class);
    protected GeneSetMatrixMultiChooserParam fGeneSetMatrixParam;
    protected ChipOptParam fChipParam;
    protected final IntegerParam fGeneSetMinSizeParam = ParamFactory.createGeneSetMinSizeParam(15, false);
    protected final IntegerParam fGeneSetMaxSizeParam = ParamFactory.createGeneSetMaxSizeParam(500, false);
    protected final IntegerParam fBpBeforeGeneParam = ParamFactory.createBpBeforeGeneParam(1000, false);
    protected final IntegerParam fBpAfterGeneParam = ParamFactory.createBpAfterGeneParam(0, false);
    protected final StringParam fSnpSetMetric = ParamFactory.createSnpSetMetric(false);
    protected final StringParam fIMetricParam = ParamFactory.createDataIntegrationMetric(false);;
    protected final StringParam fSMetricParam = ParamFactory.createGeneSetMetric(false);;    
    protected final IntegerParam fNumPermParam = ParamFactory.createNumPermParam(true);
    protected final RandomSeedTypeParam fRndSeedTypeParam = new RandomSeedTypeParam(false);
    protected final FeatureSpaceReqdParam fFeatureSpaceParam = new FeatureSpaceReqdParam();
    protected final ModeReqdParam fCollapseModeParam = new ModeReqdParam("mode", "Collapsing mode for probe sets => 1 gene", "Collapsing mode for probe sets => 1 gene", new String[]{"Max_probe", "Median_of_probes"});
    protected final BooleanParam fIncludeOnlySymbols = new BooleanParam("include_only_symbols", "Omit features with no symbol match", "If there is no known gene symbol match for a probe set omit if from the collapsed dataset", true, false);
    protected final NormModeReqdParam fNormModeParam = new NormModeReqdParam();
    protected final GeneSetScoringTableReqdParam fGcohGenReqdParam = new GeneSetScoringTableReqdParam();

    /**
     * Class constructor
     *
     * @param properties
     */
    protected AbstractGsaaTool() {
    }

    public ToolCategory getCategory() {
        return ToolCategory.GSEA;
    }

    protected CollapsedDetails.Data getDataset(final Dataset origDs) throws Exception {
        CollapsedDetails.Data cd = new CollapsedDetails.Data();
        cd.orig = origDs;

        if (fFeatureSpaceParam.isSymbols()) {
            if (!fChipParam.isSpecified()) {
                // dont as the chip param isnt really reqd (and hence isnt caught in the usual way)
                //throw new MissingReqdParamException(_getMissingChipMessage());
                throw new BadParamException("Chip parameter must be specified as you asked to analyze" +
                        " in the space of gene symbols. Chip is used to collapse probe ids into symbols", 1002);
            }

            final Chip chip = fChipParam.getChipCombo();
            Dataset collapsed = new DatasetGenerators().collapse(origDs, chip,
                    fIncludeOnlySymbols.isTrue(), fCollapseModeParam.getStringIndexChoosen());
            log.info("Collapsing dataset was done. Original: " + origDs.getQuickInfo() + " collapsed: " + collapsed.getQuickInfo());

            cd.chip = chip;
            cd.wasCollapsed = true;
            cd.collapsed = collapsed;
            ParamFactory.checkIfCollapsedIsEmpty(cd);

        } else {
            cd.wasCollapsed = false;
            cd.collapsed = origDs;
            log.info("No dataset collapsing was done .. using original as is");
        }

        return cd;
    }

    protected CollapsedDetails.Data getDataset_no_collapse(final Dataset origDs) throws Exception {
        CollapsedDetails.Data cd = new CollapsedDetails.Data();
        cd.orig = origDs;
        cd.wasCollapsed = false;
        cd.collapsed = origDs;
        log.info("No dataset collapsing was done .. using original as is");
        return cd;
    }

    protected CollapsedDetails.Ranked getRankedList(final RankedList origRL) throws Exception {
        CollapsedDetails.Ranked cd = new CollapsedDetails.Ranked();
        cd.orig = origRL;

        if (fFeatureSpaceParam.isSymbols()) {
            if (!fChipParam.isSpecified()) {
                //throw new MissingReqdParamException(_getMissingChipMessage());
                throw new BadParamException("Chip parameter must be specified as you asked to analyze" +
                        " in the space of gene symbols. Chip is used to collapse probe ids into symbols.", 1002);
            }
            final Chip chip = fChipParam.getChipCombo();
            RankedList collapsed = new DatasetGenerators().collapse(origRL, chip,
                    fIncludeOnlySymbols.isTrue(), fCollapseModeParam.getStringIndexChoosen());
            log.info("Collapsing dataset was done. Original: " + origRL.getQuickInfo() + " collapsed: " + collapsed.getQuickInfo());

            cd.chip = chip;
            cd.wasCollapsed = true;
            cd.collapsed = collapsed;
            ParamFactory.checkIfCollapsedIsEmpty(cd);

        } else {
            cd.wasCollapsed = false;
            cd.collapsed = origRL;
            log.info("No ranked list collapsing was done .. using original as is");
        }

        return cd;
    }

    // @maint add a metric and this array might need updating
    public static Metric[] createMetricsForExp() {
        return new Metric[]{
                    new Metrics.Signal2Noise(),
                    new Metrics.tTest(),
                    new Metrics.Log2Ratio()
                };
    }

    // @maint add a metric and this array might need updating
    public static CMetric[] createMetricsForSnp() {
        return new CMetric[]{
                    new CMetrics.ChiSquare_Allele(),
                    new CMetrics.ChiSquare_Geno(),
                    new CMetrics.tTest_Allele(),
                    new CMetrics.tTest_Geno(),
                    new CMetrics.Diff_of_Major_Alleles()};
    }

    protected abstract Param[] getAdditionalParams();

    protected void doAdditionalParams() {
    }

    public void declareParams() {
        this.fChipParam = new ChipOptParam(false);
        this.fGeneSetMatrixParam = new GeneSetMatrixMultiChooserParam(true);
        fParamSet.addParam(fGeneSetMatrixParam);
        doAdditionalParams();
        fParamSet.addParam(fNumPermParam);
        fParamSet.addParamBasic(fGcohGenReqdParam);
        fParamSet.addParamBasic(fGeneSetMinSizeParam);
        fParamSet.addParamBasic(fGeneSetMaxSizeParam);
        fParamSet.addParamPseudoReqd(fChipParam);
        fParamSet.addParamPseudoReqd(fFeatureSpaceParam);
        fParamSet.addParamBasic(fBpBeforeGeneParam);
        fParamSet.addParamBasic(fBpAfterGeneParam);
        fParamSet.addParamBasic(fSnpSetMetric);
        fParamSet.addParamBasic(fIMetricParam);
        fParamSet.addParamBasic(fSMetricParam);        
        fParamSet.addParamAdv(fRndSeedTypeParam);
        fParamSet.addParamAdv(fNormModeParam);
        fParamSet.addParamAdv(fIncludeOnlySymbols);
        fParamSet.addParamAdv(fCollapseModeParam);

        Param[] add = getAdditionalParams();
        for (int i = 0; i < add.length; i++) {
            fParamSet.addParam(add[i]);
        }
    }

    protected Dataset uniquize(final Dataset ds) {

        final GeneSet gset = ds.getRowNamesGeneSet();
        if (gset.getNumMembers() == ds.getNumRow()) {
            return ds;
        } else {
            StringBuffer buf = new StringBuffer();
            buf.append("There were duplicate row identifiers in the specified dataset. One id was arbitarilly choosen. Details are below");
            buf.append("\nGenerally, this is OK, but if you want to avoid this automagic, edit your dataset so that all row ids are unique\n");
            buf.append('\n');
            buf.append("# of row ids in original dataset: ").append(ds.getNumRow()).append('\n');
            buf.append("# of row UNIQUE ids in original dataset: ").append(gset.getNumMembers()).append('\n');
            buf.append("# The duplicates were\n");

            Set all = new HashSet();
            for (int i = 0; i < ds.getNumRow(); i++) {
                String member = ds.getRowName(i);
                if (all.contains(member)) {
                    buf.append(member).append('\n');
                }

                all.add(member);
            }

            fReport.addComment(buf.toString());
            return new DatasetGenerators().extractRows(ds, gset);
        }

    }

    protected RankedList uniquize(final RankedList rl) {

        final GeneSet gset = new FSet(rl.getName(), rl.getName(), rl.getRankedNames(), true);
        if (gset.getNumMembers() == rl.getSize()) {
            return rl;
        } else {
            StringBuffer buf = new StringBuffer();
            buf.append("There were duplicate row identifiers in the specified ranked list. One id was arbitarilly choosen. Details are below. ");
            buf.append("\nGenerally, this is OK, but if you want to avoid this automagic, edit your ranked list so that all row ids are unique\n");
            buf.append('\n');
            buf.append("# of row ids in original dataset: ").append(rl.getSize()).append('\n');
            buf.append("# of row UNIQUE ids in original dataset: ").append(gset.getNumMembers()).append('\n');
            buf.append("# The duplicates were\n<br><pre>");

            Set all = new HashSet();
            for (int i = 0; i < rl.getSize(); i++) {
                String member = rl.getRankName(i);
                if (all.contains(member)) {
                    buf.append(member).append('\n');
                }

                all.add(member);
            }

            buf.append("</pre>");
            fReport.addComment(buf.toString());
            return rl.extractRanked(gset);
        }
    }

}    // End AbstractGsaaTool

