package xtools.gsea;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.alg.gsea.GeneSetCohortGenerator;
import edu.mit.broad.genome.alg.gsea.KSTests;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.TemplateMode;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.genome.parsers.EdbFolderParser;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.EnrichmentReports;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import xtools.api.param.*;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public abstract class AbstractGsea2Tool extends AbstractGseaTool {

    protected final DatasetReqdParam fDatasetParam = new DatasetReqdParam();

    protected final TemplateSingleChooserParam fTemplateParam =
            new TemplateSingleChooserParam(Param.CLS, "Phenotype labels", TemplateMode.CATEGORICAL_2_CLASS_AND_NUMERIC, true);

    protected final MetricParam fMetricParam = new MetricParam(createMetricsForGsea(), false);
    protected final OrderParam fOrderParam = new OrderParam(false);
    protected final SortParam fSortParam = new SortParam(false);
    protected final PermuteTypeChooserParam fPermuteTypeParamType = PermuteTypeChooserParam.createTemplateOrGeneSet(true);
    protected final BooleanParam fMedianParam = ParamFactory.createMedianParam(false);
    protected final IntegerParam fNumMarkersParam = ParamFactory.createNumMarkersParam(100, false);

    protected final BooleanParam fSaveRndRankedListsParam = new BooleanParam("save_rnd_lists", "Save random ranked lists", "Save random ranked lists (might be very large)", false, false);

    protected final TemplateRandomizerTypeParam fRndTypeParam = new TemplateRandomizerTypeParam(
            TemplateRandomizerType.NO_BALANCE,
            new TemplateRandomizerType[]{TemplateRandomizerType.NO_BALANCE,
                    TemplateRandomizerType.EQUALIZE_AND_BALANCE}, true);

    /**
     * Class constructor
     *
     * @param properties
     */
    protected AbstractGsea2Tool() {
    }

    protected void doAdditionalParams() {
        fParamSet.addParam(fDatasetParam);
        fParamSet.addParamPseudoReqd(fTemplateParam);
        fParamSet.addParamBasic(fMetricParam);
        fParamSet.addParamBasic(fOrderParam);
        fParamSet.addParamBasic(fSortParam);
        fParamSet.addParamPseudoReqd(fPermuteTypeParamType);
        fParamSet.addParamAdv(fMedianParam);
        fParamSet.addParamAdv(fNumMarkersParam);
        fParamSet.addParamAdv(fSaveRndRankedListsParam);
        fParamSet.addParamAdv(fRndTypeParam);
    }

    protected EnrichmentDb execute_one(final CollapsedDetails.Data fullCd,
                                       final Template template,
                                       final GeneSet[] gsets

    ) throws Exception {

        final int nperms = fNumPermParam.getIValue();
        final Metric metric = fMetricParam.getMetric();
        final SortMode sort = fSortParam.getMode();
        final Order order = fOrderParam.getOrder();

        final LabelledVectorProcessor lvp = new LabelledVectorProcessors.None(); // @note

        final RandomSeedGenerator rst = fRndSeedTypeParam.createSeed();
        final Map mps = fMetricParam.getMetricParams(fMedianParam);
        final GeneSetCohortGenerator gcohgen = fGcohGenReqdParam.createGeneSetCohortGenerator(false);

        final DatasetTemplate dt = new DatasetGenerators().extract(fullCd.getDataset(), template);

        log.debug(">>>>> Using samples: " + dt.getDataset().getColumnNames());

        final KSTests tests = new KSTests(getOutputStream());

        List store_rnd_ranked_lists_here_opt = null;
        if (fSaveRndRankedListsParam.isTrue()) {
            store_rnd_ranked_lists_here_opt = new ArrayList();
        }

        return tests.executeGsea(
                dt,
                gsets,
                nperms,
                metric,
                sort,
                order,
                lvp,
                rst,
                fRndTypeParam.getRandomizerType(),
                mps,
                gcohgen,
                fPermuteTypeParamType.permuteTemplate(),
                fNumMarkersParam.getIValue(),
                store_rnd_ranked_lists_here_opt
        );

    }

    /**
     * @throws Exception
     */
    protected void execute_one_with_reporting(final CollapsedDetails.Data fullCd,
                                              final Template template,
                                              final GeneSet[] gsets,
                                              final HtmlReportIndexPage reportIndexPage,
                                              final boolean makeSubDir,
                                              final GeneSet[] origGeneSets,
                                              final int showDetailsForTopXSets,
                                              final boolean makeZippedReport,
                                              final boolean makeGeneSetReports

    ) throws Exception {

        final EnrichmentDb edb = execute_one(fullCd, template, gsets);

        // -------------------------------------------------------------------------------------------- //
        // rest are for the reporting

        final Metric metric = fMetricParam.getMetric();
        final int minSize = fGeneSetMinSizeParam.getIValue();
        final int maxSize = fGeneSetMaxSizeParam.getIValue();

        final DatasetTemplate dt = new DatasetGenerators().extract(fullCd.getDataset(), template);

        List store_rnd_ranked_lists_here_opt = null;
        if (fSaveRndRankedListsParam.isTrue()) {
            store_rnd_ranked_lists_here_opt = new ArrayList();
        }

        // Make the report
        EnrichmentReports.Ret ret = EnrichmentReports.createGseaLikeReport(
                edb,
                getOutputStream(),
                fullCd,
                reportIndexPage,
                makeSubDir,
                fReport,
                showDetailsForTopXSets,
                minSize, maxSize,
                makeGeneSetReports,
                makeZippedReport,
                origGeneSets,
                metric.getName(),
                fNormModeParam.getNormModeName());

        // Save the rnd ranked lists
        if (store_rnd_ranked_lists_here_opt != null && store_rnd_ranked_lists_here_opt.isEmpty() == false) {
            File dir = fReport.createSubDir("random_ranked_lists");
            for (int r = 0; r < store_rnd_ranked_lists_here_opt.size(); r++) {
                RankedList rl = (RankedList) store_rnd_ranked_lists_here_opt.get(r);
                File file = new File(dir, rl.getName());
                ParserFactory.save(rl, file, false);
            }
        }

        if (fPermuteTypeParamType.permuteTemplate() && dt.getTemplate().isCategorical()) {
            if (dt.getTemplate().getClass(0).getSize() < 7) {
                fReport.addComment("Warning: Phenotype permutation was performed but the number of samples in class A is < 7, phenotype: " + dt.getTemplateName());
            }

            if (dt.getTemplate().getClass(1).getSize() < 7) {
                fReport.addComment("Warning: Phenotype permutation was performed but the number of samples in class B is < 7, phenotype: " + dt.getTemplateName());
            }

            if (dt.getTemplate().getNumItems() < 14) {
                fReport.addComment("With small datasets, there might not be enough random permutations of sample labels to generate a sufficient null distribution. " +
                        "In such cases, gene_set randomization might be a better choice.");
            }
        }

        // Make an edb folder thing
        new EdbFolderParser().export(ret.edb, ret.savedInDir);
    }

}    // End AbstractGsea2Tool
