/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.gsea;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.alg.Metrics;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.vdb.chip.Chip;
import xtools.api.AbstractTool;
import xtools.api.ToolCategory;
import xtools.api.param.*;

import java.util.HashSet;
import java.util.Set;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public abstract class AbstractGseaTool extends AbstractTool {

    protected GeneSetMatrixMultiChooserParam fGeneSetMatrixParam;
    protected ChipOptParam fChipParam;

    // These settings for gene set size correspond to that used in the paper
    protected final IntegerParam fGeneSetMinSizeParam = ParamFactory.createGeneSetMinSizeParam(15, false);
    protected final IntegerParam fGeneSetMaxSizeParam = ParamFactory.createGeneSetMaxSizeParam(500, false);

    protected final IntegerParam fNumPermParam = ParamFactory.createNumPermParam(true);
    protected final RandomSeedTypeParam fRndSeedTypeParam = new RandomSeedTypeParam(false);

    protected final FeatureSpaceReqdParam fFeatureSpaceParam = new FeatureSpaceReqdParam();
    protected final ModeReqdParam fCollapseModeParam = new ModeReqdParam("mode", "Collapsing mode for probe sets => 1 gene", "Collapsing mode for probe sets => 1 gene", new String[]{"Max_probe", "Median_of_probes"});
    protected final BooleanParam fIncludeOnlySymbols = new BooleanParam("include_only_symbols", "Omit features with no symbol match", "If there is no known gene symbol match for a probe set omit if from the collapsed dataset", true, false);

    // restrict to just the regular norm mode??
    protected final NormModeReqdParam fNormModeParam = new NormModeReqdParam();
    protected final GeneSetScoringTableReqdParam fGcohGenReqdParam = new GeneSetScoringTableReqdParam();

    /**
     * Class constructor
     *
     * @param properties
     */
    protected AbstractGseaTool() {
    }

    public ToolCategory getCategory() {
        return ToolCategory.GSEA;
    }

    protected CollapsedDetails.Data getDataset(final Dataset origDs) throws Exception {
        CollapsedDetails.Data cd = new CollapsedDetails.Data();
        cd.orig = origDs;

        if (fFeatureSpaceParam.isSymbols()) {
            if (!fChipParam.isSpecified()) {
                // dont as the chip param isnt really reqd (and hence isnt caught in the usual way)
                //throw new MissingReqdParamException(_getMissingChipMessage());
                throw new BadParamException("Chip parameter must be specified as you asked to analyze" +
                        " in the space of gene symbols. Chip is used to collapse probe ids into symbols", 1002);
            }

            final Chip chip = fChipParam.getChipCombo();
            Dataset collapsed = new DatasetGenerators().collapse(origDs, chip,
                    fIncludeOnlySymbols.isTrue(), fCollapseModeParam.getStringIndexChoosen());
            log.info("Collapsing dataset was done. Original: " + origDs.getQuickInfo() + " collapsed: " + collapsed.getQuickInfo());

            cd.chip = chip;
            cd.wasCollapsed = true;
            cd.collapsed = collapsed;
            ParamFactory.checkIfCollapsedIsEmpty(cd);

        } else {
            cd.wasCollapsed = false;
            cd.collapsed = origDs;
            log.info("No dataset collapsing was done .. using original as is");
        }

        return cd;
    }

    protected CollapsedDetails.Ranked getRankedList(final RankedList origRL) throws Exception {
        CollapsedDetails.Ranked cd = new CollapsedDetails.Ranked();
        cd.orig = origRL;

        if (fFeatureSpaceParam.isSymbols()) {
            if (!fChipParam.isSpecified()) {
                //throw new MissingReqdParamException(_getMissingChipMessage());
                throw new BadParamException("Chip parameter must be specified as you asked to analyze" +
                        " in the space of gene symbols. Chip is used to collapse probe ids into symbols.", 1002);
            }
            final Chip chip = fChipParam.getChipCombo();
            RankedList collapsed = new DatasetGenerators().collapse(origRL, chip,
                    fIncludeOnlySymbols.isTrue(), fCollapseModeParam.getStringIndexChoosen());
            log.info("Collapsing dataset was done. Original: " + origRL.getQuickInfo() + " collapsed: " + collapsed.getQuickInfo());

            cd.chip = chip;
            cd.wasCollapsed = true;
            cd.collapsed = collapsed;
            ParamFactory.checkIfCollapsedIsEmpty(cd);

        } else {
            cd.wasCollapsed = false;
            cd.collapsed = origRL;
            log.info("No ranked list collapsing was done .. using original as is");
        }

        return cd;
    }

    // @maint add a metric and this array might need updating
    public static Metric[] createMetricsForGsea() {
        return new Metric[]{
                new Metrics.Signal2Noise(),
                new Metrics.tTest(),
                new Metrics.Cosine(),
                new Metrics.Euclidean(),
                new Metrics.Manhatten(),
                new Metrics.Pearson(),
                // new Metrics.None(),
                new Metrics.ClassRatio(),
                new Metrics.ClassDiff(),
                new Metrics.ClassLog2Ratio()
        };
    }

    protected abstract Param[] getAdditionalParams();

    protected void doAdditionalParams() {
    }

    public void declareParams() {

        //this.fChipParam = new ChipChooserMultiParam(false);
        this.fChipParam = new ChipOptParam(false);
        this.fGeneSetMatrixParam = new GeneSetMatrixMultiChooserParam(true);

        // reqd

        fParamSet.addParam(fGeneSetMatrixParam);
        fParamSet.addParam(fNumPermParam);

        // basic
        fParamSet.addParamBasic(fGcohGenReqdParam);
        fParamSet.addParamBasic(fGeneSetMinSizeParam);
        fParamSet.addParamBasic(fGeneSetMaxSizeParam);
        fParamSet.addParamPseudoReqd(fChipParam);
        fParamSet.addParamPseudoReqd(fFeatureSpaceParam);

        // advanced
        fParamSet.addParamAdv(fRndSeedTypeParam);
        fParamSet.addParamAdv(fNormModeParam);
        fParamSet.addParamAdv(fIncludeOnlySymbols);
        fParamSet.addParamAdv(fCollapseModeParam);

        doAdditionalParams();

        Param[] add = getAdditionalParams();
        for (int i = 0; i < add.length; i++) {
            fParamSet.addParam(add[i]);
        }
    }

    protected Dataset uniquize(final Dataset ds) {

        final GeneSet gset = ds.getRowNamesGeneSet();
        if (gset.getNumMembers() == ds.getNumRow()) {
            return ds;
        } else {
            StringBuffer buf = new StringBuffer();
            buf.append("There were duplicate row identifiers in the specified dataset. One id was arbitarilly choosen. Details are below");
            buf.append("\nGenerally, this is OK, but if you want to avoid this automagic, edit your dataset so that all row ids are unique\n");
            buf.append('\n');
            buf.append("# of row ids in original dataset: ").append(ds.getNumRow()).append('\n');
            buf.append("# of row UNIQUE ids in original dataset: ").append(gset.getNumMembers()).append('\n');
            buf.append("# The duplicates were\n");

            Set all = new HashSet();
            for (int i = 0; i < ds.getNumRow(); i++) {
                String member = ds.getRowName(i);
                if (all.contains(member)) {
                    buf.append(member).append('\n');
                }

                all.add(member);
            }

            fReport.addComment(buf.toString());
            return new DatasetGenerators().extractRows(ds, gset);
        }

    }


    protected RankedList uniquize(final RankedList rl) {

        final GeneSet gset = new FSet(rl.getName(), rl.getName(), rl.getRankedNames(), true);
        if (gset.getNumMembers() == rl.getSize()) {
            return rl;
        } else {
            StringBuffer buf = new StringBuffer();
            buf.append("There were duplicate row identifiers in the specified ranked list. One id was arbitarilly choosen. Details are below. ");
            buf.append("\nGenerally, this is OK, but if you want to avoid this automagic, edit your ranked list so that all row ids are unique\n");
            buf.append('\n');
            buf.append("# of row ids in original dataset: ").append(rl.getSize()).append('\n');
            buf.append("# of row UNIQUE ids in original dataset: ").append(gset.getNumMembers()).append('\n');
            buf.append("# The duplicates were\n<br><pre>");

            Set all = new HashSet();
            for (int i = 0; i < rl.getSize(); i++) {
                String member = rl.getRankName(i);
                if (all.contains(member)) {
                    buf.append(member).append('\n');
                }

                all.add(member);
            }

            buf.append("</pre>");
            fReport.addComment(buf.toString());
            return rl.extractRanked(gset);
        }
    }

}    // End AbstractGseaTool

/*--- Formatted in Sun Java Convention Style on Sun, Oct 27, '02 ---*/
