/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.gsea;

import edu.mit.broad.genome.StandardException;
import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.genome.objects.TemplateFactoryRandomizer;
import edu.mit.broad.genome.math.RandomSeedGenerator;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.FeatureAnnot;
import edu.mit.broad.genome.objects.FeatureAnnotImpl;
import edu.mit.broad.genome.objects.SampleAnnotImpl;
import edu.mit.broad.genome.objects.AnnotImpl;
import edu.mit.broad.genome.objects.DefaultDataset;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.genome.math.Matrix;
import xtools.api.param.*;
import edu.duke.igsp.app.services.MappingService;
import edu.duke.igsp.app.algorithms.CMetric;
import edu.duke.igsp.app.struts.SnpStruc;
import edu.duke.igsp.app.tools.GsaaTool;
import java.util.Properties;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class Gsaa extends AbstractGsaa2Tool {

    private final IntegerParam fShowDetailsForTopXSetsParam = new IntegerParam("plot_top_x", "Plot graphs for the top sets of each phenotype", "Plot GSEA mountain and related plots for the top sets of each phenotype", 20, false, Param.ADVANCED);
    private final BooleanParam fMakeZippedReportParam = ParamFactory.createZipReportParam(false);
    private final BooleanParam fMakeGeneSetReportsParam = new BooleanParam("make_sets", "Make detailed gene set report", "Create detailed gene set reports (heat-map, mountain plot etc) for every associated gene set", true, false, Param.ADVANCED);

    /**
     * Class constructor
     *
     * @param properties
     */
    public Gsaa(final Properties properties) {
        super.init(properties);
    }

    public Gsaa(final String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public Gsaa() {
        declareParams();
    }

    @Override
    public void execute() throws Exception {
        // to preserve memory & for custom indexing
        final ReportIndexState state = new ReportIndexState(true, false, false, createHeader());
        startExec(state);

        int countDataset = 0;

        final Metric metric = fMetricParam.getMetric();
        if (!fGeneSetMatrixParam.isSpecified()) {
            Application.getWindowManager().showError("Gene set dataset must be specified");
            throw new IllegalArgumentException("Gene set dataset must be specified");
        }
        //expression module
        if (!fExpDatasetParam.isSpecified()) {
            Application.getWindowManager().showError("Gene expression dataset must be specified");
            throw new IllegalArgumentException("Gene expression dataset must be specified");
        }
        if (!fSnpDatasetParam.isSpecified()) {
            Application.getWindowManager().showError("SNP dataset must be specified");
            throw new IllegalArgumentException("SNP dataset must be specified");
        }
        if (!fExpTemplateParam.isSpecified()) {
            Application.getWindowManager().showError("Phenotype labels for gene expression dataset must be specified");
            throw new IllegalArgumentException("Phenotype labels for gene expression dataset must be specified");
        }
        if (!fSnpTemplateParam.isSpecified()) {
            Application.getWindowManager().showError("Phenotype labels for SNP dataset must be specified");
            throw new IllegalArgumentException("Phenotype labels for SNP dataset must be specified");
        }
        String imetric = (String) fIMetricParam.getValue();
        String smetric = (String) fSMetricParam.getValue();
        String normMode = (String)fNormModeParam.getValue();
        if ((smetric.equalsIgnoreCase("Fisher_Method")||smetric.equalsIgnoreCase("SAM_GS"))&&(normMode.equalsIgnoreCase("None (only valid for Weighted_KS)"))) {
            throw new IllegalArgumentException("Normalization mode is not compatible with the metric for gene set association analysis. Normalization mode:None is only valid for metric:Weighted_KS, choose other normalization modes.");
        }

        final Dataset ds_exp = fExpDatasetParam.getDataset(fChipParam, null);

        final Dataset fullDs_exp = uniquize(ds_exp);
        final CollapsedDetails.Data cd_exp = super.getDataset(fullDs_exp);
        final String[] rowNames = cd_exp.getDataset().getRowNamesArray();

        countDataset++;

        final Template template_exp = fExpTemplateParam.getTemplate();
        if (template_exp.isCategorical()) {
            if (!metric.isCategorical()) {
                throw new StandardException("As the phenotype choosen was categorical, only categorical metrics are allowed. Got: " + metric.getName(), 1010);
            }
        } else {
            throw new IllegalArgumentException("Only categorical expression templates allowed. Got: " + template_exp.getName());
        }

        HashMap datasets = new HashMap();
        HashMap templates = new HashMap();
        HashMap params = new HashMap();

        String species = fSpeciesParam.getString();

        InputStreamReader isr = null;
        if (species.equalsIgnoreCase("Human_GRCh37")) {
            //species = "homo_sapiens";
            InputStream is = getClass().getResourceAsStream("genepos/homo_sapiens.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Human_NCBI36")) {
            //species = "homo_sapiens";
            InputStream is = getClass().getResourceAsStream("genepos/homo_sapiens_NCBI36.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Mouse")) {
            //species = "mus_musculus";
            InputStream is = getClass().getResourceAsStream("genepos/mus_musculus.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Rat")) {
            //species = "rattus_norvegicus";
            InputStream is = getClass().getResourceAsStream("genepos/rattus_norvegicus.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Zebrafish")) {
            //species = "danio_rerio";
            InputStream is = getClass().getResourceAsStream("genepos/danio_rerio.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Arabidopsis_Thaliana")) {
            //species = "arabidopsis_thaliana";
            InputStream is = getClass().getResourceAsStream("genepos/arabidopsis_thaliana.txt");
            isr = new InputStreamReader(is);  
        } else if (species.equalsIgnoreCase("Cat")) {
            //species = "felis_catus";
            InputStream is = getClass().getResourceAsStream("genepos/felis_catus.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("C.elegans")) {
            //species = "caenorhabditis_elegans";
            InputStream is = getClass().getResourceAsStream("genepos/caenorhabditis_elegans.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Chicken")) {
            //species = "gallus_gallus";
            InputStream is = getClass().getResourceAsStream("genepos/gallus_gallus.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Chimpanzee")) {
            //species = "pan_troglodytes";
            InputStream is = getClass().getResourceAsStream("genepos/pan_troglodytes.txt");
            isr = new InputStreamReader(is);                       
        } else if (species.equalsIgnoreCase("C.intestinalis")) {
            //species = "ciona_intestinalis";
            InputStream is = getClass().getResourceAsStream("genepos/ciona_intestinalis.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Cow")) {
            //species = "bos_taurus";
            InputStream is = getClass().getResourceAsStream("genepos/bos_taurus.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Dog")) {
            //species = "canis_familiaris";
            InputStream is = getClass().getResourceAsStream("genepos/canis_familiaris.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Fruitfly")) {
            //species = "drosophila_melanogaster";
            InputStream is = getClass().getResourceAsStream("genepos/drosophila_melanogaster.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Fugu")) {
            //species = "tetraodon_nigroviridis";
            InputStream is = getClass().getResourceAsStream("genepos/tetraodon_nigroviridis.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Gorilla")) {
            //species = "gorilla_gorilla";
            InputStream is = getClass().getResourceAsStream("genepos/gorilla_gorilla.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Horse")) {
            //species = "equus_caballus";
            InputStream is = getClass().getResourceAsStream("genepos/equus_caballus.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Macaque")) {
            //species = "macaca_mulatta";
            InputStream is = getClass().getResourceAsStream("genepos/macaca_mulatta.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Marmoset")) {
            //species = "callithrix_jacchus";
            InputStream is = getClass().getResourceAsStream("genepos/callithrix_jacchus.txt");
            isr = new InputStreamReader(is);           
        } else if (species.equalsIgnoreCase("Opossum")) {
            //species = "monodelphis_domestica";
            InputStream is = getClass().getResourceAsStream("genepos/monodelphis_domestica.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Orangutan")) {
            //species = "pongo_abelii";
            InputStream is = getClass().getResourceAsStream("genepos/pongo_abelii.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Pig")) {
            //species = "sus_scrofa";
            InputStream is = getClass().getResourceAsStream("genepos/sus_scrofa.txt");
            isr = new InputStreamReader(is);             
        } else if (species.equalsIgnoreCase("Rabbit")) {
            //species = "oryctolagus_cuniculus";
            InputStream is = getClass().getResourceAsStream("genepos/oryctolagus_cuniculus.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Saccharomyces_Cerevisiae")) {
            //species = "saccharomyces_cerevisiae";
            InputStream is = getClass().getResourceAsStream("genepos/saccharomyces_cerevisiae.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Stickleback")) {
            //species = "gasterosteus_aculeatus";
            InputStream is = getClass().getResourceAsStream("genepos/gasterosteus_aculeatus.txt");
            isr = new InputStreamReader(is);  
        } else if (species.equalsIgnoreCase("Turkey")) {
            //species = "meleagris_gallopavo";
            InputStream is = getClass().getResourceAsStream("genepos/meleagris_gallopavo.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Simulation")) {
            //species = "simulation";
            InputStream is = getClass().getResourceAsStream("genepos/sim.txt");
            isr = new InputStreamReader(is); 
        }else{
            
        }

        final ArrayList genes = MappingService.lookUp(rowNames, isr);
        final int before = ((Integer) fBpBeforeGeneParam.getValue()).intValue();
        final int after = ((Integer) fBpAfterGeneParam.getValue()).intValue();

        //snp module
        List description = new ArrayList();
        if (fSnpDatasetParam.getValue() != null) {
            final Template template_snp_ori = fSnpTemplateParam.getTemplate();
            final CMetric cmetric = fCMetricParam.getCMetric();
            if (template_snp_ori.isCategorical()) {
                if (!cmetric.isCategorical()) {
                    throw new StandardException("As the phenotype choosen was categorical, only categorical metrics are allowed. Got: " + cmetric.getName(), 1010);
                }
            } else {
                throw new IllegalArgumentException("Only categorical templates allowed. Got: " + template_snp_ori.getName());
            }

            int numOfItems = template_snp_ori.getNumItems();

            boolean permuteTemplate = fPermuteTypeParamType.permuteTemplate();
            final int nperms = fNumPermParam.getIValue();
            final RandomSeedGenerator rst = fRndSeedTypeParam.createSeed();
            final TemplateRandomizerType rt = fRndTypeParam.getRandomizerType();
            String snpSetMetric = (String) fSnpSetMetric.getValue();
            SnpStruc snpStruc = null;
            Template[] rndTemplates = null;
            if (permuteTemplate) {
                rndTemplates = TemplateFactoryRandomizer.createRandomTemplates(nperms, template_snp_ori, rt, rst);
                if (snpSetMetric.equalsIgnoreCase("Maximum")) {
                    snpStruc = (SnpStruc) fSnpDatasetParam.getSnpStruc_phenotype_maximum(cmetric, template_snp_ori, rndTemplates, genes, before, after, numOfItems);
                }
            } else {
                if (snpSetMetric.equalsIgnoreCase("Maximum")) {
                    snpStruc = (SnpStruc) fSnpDatasetParam.getSnpStruc_geneset_maximum(cmetric, template_snp_ori, genes, before, after, numOfItems);
                }
            }

            final List scoredDataset = snpStruc.getScoredDataset();
            final GeneSet[] origGeneSets_snp = snpStruc.getSnpSet();
            ArrayList scores = new ArrayList();
            if (snpSetMetric.equalsIgnoreCase("Maximum")) {
                scores = (ArrayList) scoredDataset;
            }

            //compute p-values for genes
            List pValues = new ArrayList();
            if (permuteTemplate) {
                for (int i = 0; i < scores.size(); i++) {
                    ArrayList geneScores = (ArrayList) scores.get(i);
                    float real = ((Float) geneScores.get(0)).floatValue();
                    double p =GsaaTool.getPValue(real, geneScores);
                    p=XMath.roundOff(p, 4);
                    pValues.add((new Double(p)).toString());
                }
            } else {
                for (int i = 0; i < scores.size(); i++) {
                    pValues.add("na");
                }
            }

            final HashMap snpGeneStruc = new HashMap();

            List genesetNames = new ArrayList();
            List maxSnpNames = new ArrayList();
            for (int i = 0; i < origGeneSets_snp.length; i++) {

                ArrayList es = new ArrayList();
                for (int j = 0; j < ((ArrayList) scores.get(i)).size(); j++) {
                    es.add(((ArrayList) scores.get(i)).get(j));
                }

                snpGeneStruc.put(origGeneSets_snp[i].getName(), es);
                genesetNames.add(origGeneSets_snp[i].getName());
                maxSnpNames.add(origGeneSets_snp[i].getMembersArray()[0]);
            }

            List geneNames = cd_exp.getDataset().getRowNames();
            for (int i = 0; i < geneNames.size(); i++) {
                String geneName = (String) geneNames.get(i);
                int flag = 0;
                for (int j = 0; j < genesetNames.size(); j++) {
                    String genesetName = (String) genesetNames.get(j);
                    if (geneName.equalsIgnoreCase(genesetName)) {
                        flag++;
                        description.add(maxSnpNames.get(j)+": "+pValues.get(j));
                        genesetNames.remove(j);
                        pValues.remove(j);
                        maxSnpNames.remove(j);
                        break;
                    }
                }

                if (flag == 0) {
                    description.add("na");
                }
            }

            HashMap param_snp = new HashMap();
            param_snp.put("METRIC", cmetric);
            param_snp.put("RND_TEMPLATES", rndTemplates);
            params.put("SNP", param_snp);

            datasets.put("SNP", snpGeneStruc);
            templates.put("SNP", template_snp_ori);
            countDataset++;
        }

        forcePhenoIdentical(template_exp, templates);

        final GeneSet[] origGeneSets = fGeneSetMatrixParam.getGeneSetMatrixCombo(true).getGeneSets();

        final GeneSet[] gsets = Helper.getGeneSets(cd_exp.getDataset(), origGeneSets, fGeneSetMinSizeParam, fGeneSetMaxSizeParam);

        ParamFactory.checkAndBarfIfZeroSets(gsets);

        final HtmlReportIndexPage htmlReportIndexPage = fReport.getIndexPage();
        final boolean makeSubDir = false;

        final String objName = cd_exp.getDataset().getName();
        final List geneNames = cd_exp.getDataset().getRowNames();
        final List colNames = cd_exp.getDataset().getColumnNames();
        final Matrix matrix = cd_exp.getDataset().getMatrix();
        final FeatureAnnot ann = new FeatureAnnotImpl(objName, geneNames, description);
        final SampleAnnot sann = new SampleAnnotImpl(objName, colNames, null);
        final Dataset ds = new DefaultDataset(objName, matrix, geneNames, colNames, true, new AnnotImpl(ann, sann));
        final CollapsedDetails.Data collapsed_ds = super.getDataset_no_collapse(ds);

        execute_one_with_reporting(collapsed_ds, template_exp, gsets,
                htmlReportIndexPage, makeSubDir, origGeneSets, fShowDetailsForTopXSetsParam.getIValue(), fMakeZippedReportParam.isTrue(), fMakeGeneSetReportsParam.isTrue(), datasets, templates, params, imetric, smetric);

        if (fMakeZippedReportParam.isTrue()) {
            fReport.closeReport(true);
            fReport.zipReport();
        }

        doneExec();
    }

    @Override
    protected Param[] getAdditionalParams() {
        return new Param[]{fShowDetailsForTopXSetsParam, fMakeZippedReportParam, fMakeGeneSetReportsParam};
    }

    @Override
    public String getDesc() {
        return "Set parameters and run GSAA";
    }

    public static void main(String[] args) {
        Gsaa tool = new Gsaa(args);
        tool_main(tool);
    }

    public static String createHeader(final DatasetReqdParam dsr) {
        try {
            if (dsr.isSpecified()) {
                Dataset ds = dsr.getDataset();
                StringBuffer buf = new StringBuffer();
                buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                        "<h3 style=\"text-align: left\"><font color=\"#808080\">GSAA Report for ").append(
                        "Dataset ").append(ds.getName()).append("</font></h3>\n").append("</div>");

                return buf.toString();
            }
        } catch (Throwable t) {
            t.printStackTrace();
        }

        return null;
    }

    public static String createHeader() {
        try {
            StringBuffer buf = new StringBuffer();
            buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                    "<h3 style=\"text-align: left\"><font color=\"#808080\">GSAA Report").append("</font></h3>\n").append("</div>");

            return buf.toString();
        } catch (Throwable t) {
            t.printStackTrace();
        }

        return null;
    }

    public static void forcePhenoIdentical(Template template_exp, HashMap templates) {
        int numOfClassExp = template_exp.getNumClasses();
        if (numOfClassExp != 2) {
            throw new IllegalArgumentException("The number of phenotype labels must remain 2. Got: " + numOfClassExp);
        }
        Iterator itr = templates.keySet().iterator();
        while (itr.hasNext()) {
            String t_name = (String) itr.next();
            Template template = (Template) templates.get(t_name);
            int numOfClass = template.getNumClasses();
            if (numOfClass != 2) {
                throw new IllegalArgumentException("The number of phenotype labels must remain 2. Got: " + numOfClass);
            }
            for (int i = 0; i < numOfClassExp; i++) {
                String nameOfClassExp = template_exp.getClass(i).getName();
                String nameOfClass = template.getClass(i).getName();
                if (!nameOfClassExp.equalsIgnoreCase(nameOfClass)) {
                    throw new IllegalArgumentException("Phenotype labels must remain the same. Got: " + nameOfClassExp + " and " + nameOfClass);
                }
            }
        }
    }
    
    public static void forceTwoClasses(Template template) {
        int numOfClass = template.getNumClasses();
        if (numOfClass != 2) {
            throw new IllegalArgumentException("The number of phenotype labels must remain 2. Got: " + numOfClass);
        }
    }    
}    // End Gsaa

