/**
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2013 by the
 * Department of Computer Science and Technology/Southwest University and Department of Genetics/University of North Carolina at Chapel Hill.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither Southwest University nor University of North Carolina at Chapel Hill can be responsible for
 * its use, misuse, or functionality.
 */

package xtools.gsea;

import org.tigr.microarray.mev.cluster.algorithm.AbortException;
import org.tigr.microarray.mev.cluster.algorithm.AlgorithmException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Comparator;
import java.math.BigDecimal;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import edu.duke.igsp.app.tools.FileTool;

/**
 * @author Qing Xiong <qingx@swu.edu.cn> & <qxiong@email.unc.edu>
 * Date: January 22, 2013
 *
 */
public class GsaaDegseq {

    private static final Logger xlog = XLogger.getLogger(GsaaEdgeR.class);
    private int[][] countMatrix;
    private int[] groupAssignments;
    private int numGenes, numExps;
    private boolean debug = false;
    private String[] geneNames;
    private String[] sampleNames;
    private String out = null;
    // results
    ArrayList resultMatrix = new ArrayList();

    /**
     * This method execute calculation and return result
     */
    public void execute(String[] geneNames, String[] sampleNames, int[] groupAssignments, int[][] countMatrix, int numGenes, int numExps, String out) {
        this.geneNames = geneNames;
        this.sampleNames = sampleNames;
        this.groupAssignments = groupAssignments;
        this.countMatrix = countMatrix;
        this.numGenes = numGenes;
        this.numExps = numExps;
        this.out = out;

        //Run r cmds
        runRAlg();
    }

    /**
     * Function to create R session in memory and execute DEGSEQ
     * @throws AbortException
     */
    public void runRAlg() {
        try {
            if (RCaller.startRSession() == null) {
                xlog.debug("Could not get REngine");
                throw new AbortException();
            }
        } catch (AbortException ae) {
            ae.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            String rCmd = "library(DEGseq)";
            RCaller.evalR(rCmd);

            String fileLoc = out + "counts.txt";
            fileLoc = fileLoc.replace("\\", "/");
            writeMatrixToFile(fileLoc, countMatrix, geneNames, sampleNames);

            String grp1ColIndStr = "";
            String grp2ColIndStr = "";
            for (int i = 0; i < groupAssignments.length; i++) {
                if (groupAssignments[i] == 1) {
                    grp1ColIndStr = grp1ColIndStr + (i + 2) + ",";
                }
                if (groupAssignments[i] == 2) {
                    grp2ColIndStr = grp2ColIndStr + (i + 2) + ",";
                }
            }
            grp1ColIndStr = grp1ColIndStr.substring(0, grp1ColIndStr.length() - 1);
            grp2ColIndStr = grp2ColIndStr.substring(0, grp2ColIndStr.length() - 1);

            rCmd = "geneExpMatrix1 <- readGeneExp(file='" + fileLoc + "', geneCol=1, valCol=c(" + grp1ColIndStr + "))";
            RCaller.evalR(rCmd);

            rCmd = "geneExpMatrix2 <- readGeneExp(file='" + fileLoc + "', geneCol=1, valCol=c(" + grp2ColIndStr + "))";
            RCaller.evalR(rCmd);

            int grp1 = 1;
            int grp2 = 1;
            for (int j = 0; j < groupAssignments.length; j++) {
                if (groupAssignments[j] == 1) {
                    grp1++;
                }
                if (groupAssignments[j] == 2) {
                    grp2++;
                }
            }

            fileLoc = out;
            fileLoc = fileLoc.replace("\\", "/");
            rCmd = "DEGexp(geneExpMatrix1=geneExpMatrix1, geneCol1=1, expCol1=c(seq(from=2,to=" + grp1 + ")), groupLabel1='Grp1', geneExpMatrix2=geneExpMatrix2, geneCol2=1, expCol2=c(seq(from=2,to=" + grp2 + ")), groupLabel2='Grp2', method='MARS', outputDir='" + fileLoc + "')";           
            RCaller.evalR(rCmd);
            RCaller.endRSession();
            RCaller.endR();

            ArrayList results = FileTool.readData(fileLoc+"output_score.txt");
            results.remove(0);
            
            fileLoc = out + "pvalues.txt";
            fileLoc = fileLoc.replace("\\", "/");
            writePvalueToFile(results, fileLoc);

            fileLoc = out + "scores.rnk";
            fileLoc = fileLoc.replace("\\", "/");
            writeSortedPvalueToFile(results, fileLoc);

        } catch (AbortException ae) {
            ae.printStackTrace();
        } catch (AlgorithmException alge) {
            alge.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private float[] doubletoFloat(double[] ds) {
        float[] tmp = new float[ds.length];
        for (int i = 0; i < ds.length; i++) {
            tmp[i] = (float) ds[i];
        }
        return tmp;
    }

    //applicable to BigDecimal objects and rank transformation
    public List sort(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(6);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }

            BigDecimal bd = new BigDecimal(value_s);
            temp.add(rowname);
            temp.add(bd);
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(1);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(1);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return 1; // 1 for ascending order
                } else if (i == -1) {
                    return -1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });

        for (int i = 0; i < ls.size(); i++) {
            ((ArrayList) ls.get(i)).set(1, new Double(ls.size() - i));
        }
        return ls;
    }

    //applicable to BigDecimal objects
    public List sort1(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(6);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }

            BigDecimal bd1 = new BigDecimal(value_s);
            BigDecimal bd2 = BigFunctions.ln(bd1, 6);
            BigDecimal bd3 = new BigDecimal("-1");
            BigDecimal bd4 = bd2.multiply(bd3);
            BigDecimal bd = bd4.abs();

            /*BigDecimal bd1 = new BigDecimal("1");
            BigDecimal bd2 = new BigDecimal(value_s);
            BigDecimal bd =bd1.subtract(bd2);*/

            temp.add(rowname);
            temp.add(bd);
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(1);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(1);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return -1; // 1 for ascending order
                } else if (i == -1) {
                    return 1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });

        BigDecimal sum = new BigDecimal("0");
        for (int i = 0; i < ls.size(); i++) {
            BigDecimal bd = (BigDecimal) ((ArrayList) ls.get(i)).get(1);
            sum.add(bd);
        }

        for (int i = 0; i < ls.size(); i++) {
            BigDecimal bd = (BigDecimal) ((ArrayList) ls.get(i)).get(1);
            bd = bd.divide(sum);
            ((ArrayList) ls.get(i)).set(1, bd);
        }
        return ls;
    }

    //applicable to BigDecimal objects
    public List sort2(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(6);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }

            BigDecimal bd1 = new BigDecimal("1");
            BigDecimal bd2 = new BigDecimal(value_s);
            BigDecimal bd = bd1.subtract(bd2);

            temp.add(rowname);
            temp.add(bd);
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(1);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(1);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return -1; // 1 for ascending order
                } else if (i == -1) {
                    return 1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });

        return ls;
    }

    //applicable to Double objects
    public List sort3(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(6);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }
            double value = Double.parseDouble(value_s);
            //value = (-1) * Math.log(value);
            //value=Math.abs(value);
            value = 1 - value;

            temp.add(rowname);
            temp.add(new Double(value));
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                final double d1 = ((Double) ((ArrayList) o1).get(1)).doubleValue();
                final double d2 = ((Double) ((ArrayList) o2).get(1)).doubleValue();
                if (d1 > d2) {
                    return -1; // 1 for ascending order
                } else if (d1 < d2) {
                    return 1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });
        return ls;
    }

    public void writeSortedPvalueToFile(ArrayList results, String fileLoc) {
        List l = sort(results);
        try {
            BufferedWriter output = new BufferedWriter(new FileWriter(fileLoc));
            for (int i = 0; i < l.size(); i++) {
                ArrayList al = (ArrayList) l.get(i);
                //output.write((String) al.get(0) + "\t" + ((BigDecimal) al.get(1)).toString());
                output.write((String) al.get(0) + "\t" + ((Double) al.get(1)).toString());
                output.newLine();
            }
            output.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void writePvalueToFile(ArrayList results, String fileLoc) {
        try {
            BufferedWriter output = new BufferedWriter(new FileWriter(fileLoc));
            for (int i = 0; i < results.size(); i++) {
                String rowname = (String) ((ArrayList) results.get(i)).get(0);
                String value_s = (String) ((ArrayList) results.get(i)).get(6);
                output.write(rowname + "\t" + value_s);
                output.newLine();
            }
            output.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String writeMatrixToFile(String fileLoc, int[][] fm, String[] rowNames, String[] samples) {
        try {
            BufferedWriter out = new BufferedWriter(new FileWriter(fileLoc));

            int row = this.numGenes;
            int col = this.numExps;
            String srtVector = "Symbol" + "\t";

            if (debug) {
                System.out.println("row, col, rowName " + row);
                System.out.println("row, col, rowName " + col);
                System.out.println("row, col, rowName " + rowNames[1]);
            }

            //write sample names
            int iRow;
            for (iRow = 0; iRow < samples.length - 1; iRow++) {
                srtVector += samples[iRow] + "\t";
            }
            out.write(srtVector + samples[samples.length - 1] + "\n");

            srtVector = "";
            for (iRow = 0; iRow < row; iRow++) {
                srtVector = rowNames[iRow] + "\t";
                for (int jCol = 0; jCol < col; jCol++) {
                    if (jCol == col - 1) {
                        srtVector += fm[iRow][jCol] + "\n";
                    } else {
                        srtVector += fm[iRow][jCol] + "\t";
                    }
                }
                out.write(srtVector);
                srtVector = "";
            }
            out.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return fileLoc;
    }

    private void removeFile(String fileName) {
        File f = new File(fileName);
        f.delete();
    }
}
