/**
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2013 by the
 * Department of Computer Science and Technology/Southwest University and Department of Genetics/University of North Carolina at Chapel Hill.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither Southwest University nor University of North Carolina at Chapel Hill can be responsible for
 * its use, misuse, or functionality.
 */

package xtools.gsea;

import org.tigr.microarray.mev.cluster.algorithm.AbortException;
import org.tigr.microarray.mev.cluster.algorithm.AlgorithmException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Comparator;
import java.math.BigDecimal;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import edu.duke.igsp.app.tools.FileTool;

/**
 * @author Qing Xiong <qingx@swu.edu.cn> & <qxiong@email.unc.edu>
 * Date: January 22, 2013
 *
 */
public class GsaaEdgeR {

    private static final Logger xlog = XLogger.getLogger(GsaaEdgeR.class);
    private int[][] countMatrix;
    private int[] groupAssignments;
    private int numGenes, numExps;
    private boolean debug = false;
    private String[] geneNames;
    private String[] sampleNames;
    private String out = null;
    // results
    ArrayList resultMatrix = new ArrayList();

    /**
     * This method execute calculation and return result,
     */
    public void execute(String[] geneLabels, String[] sampleLabels, int[] groupAssignments, int[][] countMatrix, int numGenes, int numExps, String out) {
        this.geneNames = geneLabels;
        this.sampleNames = sampleLabels;
        this.groupAssignments = groupAssignments;
        this.countMatrix = countMatrix;
        this.numGenes = numGenes;
        this.numExps = numExps;
        this.out = out;

        // Run R cmds
        runRAlg();
    }

    /**
     * Function to create R session in memory and execute edgeR
     */
    public void runRAlg() {
        try {
            if (RCaller.startRSession() == null) {
                xlog.debug("Could not get REngine");
                throw new AbortException();
            }
        } catch (AbortException ae) {
            ae.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            String rCmd = "library(edgeR)";
            RCaller.evalR(rCmd);

            String fileLoc = out + "counts.txt";
            fileLoc = fileLoc.replace("\\", "/");
            writeMatrixToFile(fileLoc, countMatrix, geneNames, sampleNames);
            rCmd = "x <- read.delim('" + fileLoc + "', row.names='Symbol')";
            RCaller.evalR(rCmd);

            String grp = "";
            for (int i = 0; i < this.groupAssignments.length; i++) {
                grp = grp + groupAssignments[i] + ",";
            }
            grp = grp.substring(0, grp.length()-1);

            rCmd = "group <- c("+grp+")";
            RCaller.evalR(rCmd);

            rCmd = "y <- DGEList(counts = x, group = group)";
            RCaller.evalR(rCmd);
            
            rCmd = "y <- calcNormFactors(y)";
            RCaller.evalR(rCmd);            

            rCmd = "y <- estimateCommonDisp(y)";
            RCaller.evalR(rCmd);

            rCmd = "y <- estimateTagwiseDisp(y)";
            RCaller.evalR(rCmd);

            rCmd = "de <- exactTest(y)";
            RCaller.evalR(rCmd);

            rCmd = "de <- topTags(de, n=length(de), adjust.method='BH', sort.by='p.value')";
            RCaller.evalR(rCmd);

            //removeFile(fileLoc);
            fileLoc = out + "edgeR.txt";
            fileLoc = fileLoc.replace("\\", "/");
            rCmd = "write.table(de$table, file='" + fileLoc + "', quote=FALSE)";
            RCaller.evalR(rCmd);
            RCaller.endRSession();
            RCaller.endR();

            ArrayList results = FileTool.readData(fileLoc);
            results.remove(0);

            fileLoc = out + "pvalues.txt";
            fileLoc = fileLoc.replace("\\", "/");
            writePvalueToFile(results, fileLoc);

            fileLoc = out + "scores.rnk";
            fileLoc = fileLoc.replace("\\", "/");
            writeSortedPvalueToFile(results, fileLoc);

        } catch (AbortException ae) {
            ae.printStackTrace();
        } catch (AlgorithmException alge) {
            alge.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private float[] doubletoFloat(double[] _tmp) {
        float[] tmp = new float[_tmp.length];
        for (int i = 0; i < _tmp.length; i++) {
            tmp[i] = (float) _tmp[i];
        }
        return tmp;
    }

    public void writePvalueToFile(ArrayList results, String fileLoc) {
        try {
            BufferedWriter output = new BufferedWriter(new FileWriter(fileLoc));
            for (int i = 0; i < results.size(); i++) {
                String rowname = (String) ((ArrayList) results.get(i)).get(0);
                String value_s = (String) ((ArrayList) results.get(i)).get(3);
                output.write(rowname + "\t" + value_s);
                output.newLine();
            }
            output.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void writeSortedPvalueToFile(ArrayList results, String fileLoc) {
        List l = sort(results);
        try {
            BufferedWriter output = new BufferedWriter(new FileWriter(fileLoc));
            for (int i = 0; i < l.size(); i++) {
                ArrayList al = (ArrayList) l.get(i);
                //output.write((String) al.get(0) + "\t" + ((BigDecimal) al.get(1)).toString());
                output.write((String) al.get(0) + "\t" + ((Double) al.get(1)).toString());
                output.newLine();
            }
            output.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //applicable to BigDecimal objects and rank transformation
    public List sort(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(3);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }

            BigDecimal bd = new BigDecimal(value_s);
            temp.add(rowname);
            temp.add(bd);
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(1);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(1);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return 1; // 1 for ascending order
                } else if (i == -1) {
                    return -1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });

        for (int i = 0; i < ls.size(); i++) {
            ((ArrayList) ls.get(i)).set(1, new Double(ls.size() - i));
        }
        return ls;
    }

    //applicable to BigDecimal objects
    public List sort1(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(3);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }

            BigDecimal bd = new BigDecimal(value_s);
            temp.add(rowname);
            temp.add(bd);
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(1);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(1);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return 1; // 1 for ascending order
                } else if (i == -1) {
                    return -1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });

        BigDecimal firstNonZero = new BigDecimal("0");
        for (int i = 0; i < ls.size(); i++) {
            BigDecimal value = (BigDecimal) ((ArrayList) ls.get(i)).get(1);
            if (!(value.toString()).equalsIgnoreCase("0")) {
                firstNonZero = value;
                break;
            }
        }

        for (int i = 0; i < ls.size(); i++) {
            BigDecimal bd1 = (BigDecimal) ((ArrayList) ls.get(i)).get(1);
            if ((bd1.toString()).equalsIgnoreCase("0")) {
                bd1 = firstNonZero.divide(new BigDecimal(10));
            }

            BigDecimal bd2 = BigFunctions.ln(bd1, 6);
            BigDecimal bd3 = new BigDecimal("-1");
            BigDecimal bd4 = bd2.multiply(bd3);
            BigDecimal bd = bd4.abs();
            ((ArrayList) ls.get(i)).set(1, bd);
        }

        BigDecimal sum = new BigDecimal("0");
        for (int i = 0; i < ls.size(); i++) {
            BigDecimal bd = (BigDecimal) ((ArrayList) ls.get(i)).get(1);
            sum.add(bd);
        }

        for (int i = 0; i < ls.size(); i++) {
            BigDecimal bd = (BigDecimal) ((ArrayList) ls.get(i)).get(1);
            bd = bd.divide(sum);
            ((ArrayList) ls.get(i)).set(1, bd);
        }
        return ls;
    }

    //applicable to BigDecimal objects
    public List sort2(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(3);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }

            BigDecimal bd1 = new BigDecimal("1");
            BigDecimal bd2 = new BigDecimal(value_s);
            BigDecimal bd = bd1.subtract(bd2);

            temp.add(rowname);
            temp.add(bd);
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(1);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(1);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return -1; // 1 for ascending order
                } else if (i == -1) {
                    return 1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });
        return ls;
    }

    //applicable to Double objects
    public List sort3(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(3);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }
            double value = Double.parseDouble(value_s);
            //value = (-1) * Math.log(value);
            //value=Math.abs(value);
            value = 1 - value;
            temp.add(rowname);
            temp.add(new Double(value));
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                final double d1 = ((Double) ((ArrayList) o1).get(1)).doubleValue();
                final double d2 = ((Double) ((ArrayList) o2).get(1)).doubleValue();
                if (d1 > d2) {
                    return -1; // 1 for ascending order
                } else if (d1 < d2) {
                    return 1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });
        return ls;
    }

    private void writeMatrixToFile(String fileLoc, int[][] fm, String[] rowNames, String[] samples) {
        try {
            BufferedWriter output = new BufferedWriter(new FileWriter(fileLoc));

            int row = this.numGenes;
            int col = this.numExps;
            String srtVector = "Symbol" + "\t";

            if (debug) {
                System.out.println("row, col, rowName " + row);
                System.out.println("row, col, rowName " + col);
                System.out.println("row, col, rowName " + rowNames[1]);
            }

            //write sample names
            int iRow;
            for (iRow = 0; iRow < samples.length - 1; iRow++) {
                srtVector += samples[iRow] + "\t";
            }
            output.write(srtVector + samples[samples.length - 1] + "\n");

            srtVector = "";
            for (iRow = 0; iRow < row; iRow++) {
                srtVector = rowNames[iRow] + "\t";
                for (int jCol = 0; jCol < col; jCol++) {
                    if (jCol == col - 1) {
                        srtVector += fm[iRow][jCol] + "\n";
                    } else {
                        srtVector += fm[iRow][jCol] + "\t";
                    }
                }
                output.write(srtVector);
                srtVector = "";
            }
            output.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void removeFile(String fileName) {
        File f = new File(fileName);
        f.delete();
    }
}
