/**
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2013 by the
 * Department of Computer Science and Technology/Southwest University and Department of Genetics/University of North Carolina at Chapel Hill.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither Southwest University nor University of North Carolina at Chapel Hill can be responsible for
 * its use, misuse, or functionality.
 */

package xtools.gsea;

import org.tigr.microarray.mev.cluster.algorithm.AbortException;
import org.tigr.microarray.mev.cluster.algorithm.AlgorithmException;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Comparator;
import java.math.BigDecimal;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import edu.duke.igsp.app.tools.FileTool;

/**
 * @author Qing Xiong <qingx@swu.edu.cn> & <qxiong@email.unc.edu>
 * Date: January 22, 2013
 *
 */
public class GsaaNoiseq {

    private static final Logger xlog = XLogger.getLogger(GsaaEdgeR.class);
    private int[][] countMatrix;
    private int[] groupAssignments;
    private int numGenes, numExps;
    private boolean debug = false;
    private String[] geneNames;
    private String[] sampleNames;
    private String out = null;
    // results
    ArrayList resultMatrix = new ArrayList();

    /**
     * This method execute calculation and return result
     */
    public void execute(String[] geneNames, String[] sampleNames, int[] groupAssignments, int[][] countMatrix, int numGenes, int numExps, String out) {
        this.geneNames = geneNames;
        this.sampleNames = sampleNames;
        this.groupAssignments = groupAssignments;
        this.countMatrix = countMatrix;
        this.numGenes = numGenes;
        this.numExps = numExps;
        this.out = out;

        //Run r cmds
        runRAlg();
    }

    /**
     * Function to create R session in memory and execute DESEQ
     * @throws AbortException
     */
    public void runRAlg() {
        try {
            if (RCaller.startRSession() == null) {
                xlog.debug("Could not get REngine");
                throw new AbortException();
            }
        } catch (AbortException ae) {
            ae.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            String rCmd = "library(NOISeq)";
            RCaller.evalR(rCmd);

            String fileLoc = out + "counts.txt";
            fileLoc = fileLoc.replace("\\", "/");
            writeMatrixToFile(fileLoc, countMatrix, geneNames, sampleNames);
            rCmd = "mycounts <- read.delim('" + fileLoc + "', row.names='Symbol')";
            RCaller.evalR(rCmd);

            String grp = "";
            for (int i = 0; i < this.groupAssignments.length; i++) {
                grp = grp + groupAssignments[i] + ",";
            }
            grp = grp.substring(0, grp.length()-1);

            rCmd = "myfactors = data.frame(F=c("+grp+"))";
            RCaller.evalR(rCmd);            

            rCmd = "mydata <- readData(data=mycounts, factors=myfactors, length = NULL, biotype = NULL, chromosome = NULL, gc = NULL)";
            RCaller.evalR(rCmd);

            rCmd = "mynoiseqbio <- noiseqbio(mydata, k=0.5, norm = 'tmm', factor = 'F', conditions = c('1', '2'), lc = 1, r = 1000, adj = 1.5, plot = FALSE, a0per = 0.9, random.seed = 12345, filter = 2)";
            RCaller.evalR(rCmd);

            fileLoc = out + "NOISeq.txt";
            fileLoc = fileLoc.replace("\\", "/");
            rCmd = "write.table(mynoiseqbio@results, file='" + fileLoc + "', quote=FALSE)";
            RCaller.evalR(rCmd);
            RCaller.endRSession();
            RCaller.endR();

            ArrayList results = FileTool.readData(fileLoc);
            results.remove(0);

            fileLoc = out + "prob.txt";
            fileLoc = fileLoc.replace("\\", "/");
            writePvalueToFile(results, fileLoc);

            fileLoc = out + "scores.rnk";
            fileLoc = fileLoc.replace("\\", "/");
            writeSortedPvalueToFile(results, fileLoc);
        } catch (AbortException ae) {
            ae.printStackTrace();
        } catch (AlgorithmException alge) {
            alge.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private float[] doubletoFloat(double[] ds) {
        float[] tmp = new float[ds.length];
        for (int i = 0; i < ds.length; i++) {
            tmp[i] = (float) ds[i];
        }
        return tmp;
    }

    //applicable to BigDecimal objects and rank transformation
    public List sort(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(4);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "0";
            }

            BigDecimal bd = new BigDecimal(value_s);
            temp.add(rowname);
            temp.add(bd);
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(1);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(1);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return -1; // 1 for ascending order
                } else if (i == -1) {
                    return 1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });

        for (int i = 0; i < ls.size(); i++) {
            ((ArrayList) ls.get(i)).set(1, new Double(ls.size() - i));
        }
        return ls;
    }

    //applicable to Double objects
    public List sort1(ArrayList al) {
        List ls = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            String rowname = (String) ((ArrayList) al.get(i)).get(0);
            String value_s = (String) ((ArrayList) al.get(i)).get(1);
            if (value_s.equalsIgnoreCase("NA")) {
                value_s = "1";
            }
            double value = Double.parseDouble(value_s);

            temp.add(rowname);
            temp.add(new Double(value));
            ls.add(temp);
        }

        Collections.sort(ls, new Comparator<List>() {

            public int compare(List o1, List o2) {
                final double d1 = ((Double) ((ArrayList) o1).get(1)).doubleValue();
                final double d2 = ((Double) ((ArrayList) o2).get(1)).doubleValue();
                if (d1 > d2) {
                    return -1; // 1 for ascending order
                } else if (d1 < d2) {
                    return 1; // -1 for ascending order
                } else {
                    return 0;
                }
            }
        });
        return ls;
    }

    public void writeSortedPvalueToFile(ArrayList results, String fileLoc) {
        List l = sort(results);
        try {
            BufferedWriter output = new BufferedWriter(new FileWriter(fileLoc));
            for (int i = 0; i < l.size(); i++) {
                ArrayList al = (ArrayList) l.get(i);
                output.write((String) al.get(0) + "\t" + ((Double) al.get(1)).toString());
                output.newLine();
            }
            output.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void writePvalueToFile(ArrayList results, String fileLoc) {
        try {
            BufferedWriter output = new BufferedWriter(new FileWriter(fileLoc));
            for (int i = 0; i < results.size(); i++) {
                String rowname = (String) ((ArrayList) results.get(i)).get(0);
                String value_s = (String) ((ArrayList) results.get(i)).get(4);
                output.write(rowname + "\t" + value_s);
                output.newLine();
            }
            output.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String writeMatrixToFile(String fileLoc, int[][] fm, String[] rowNames, String[] samples) {
        try {
            BufferedWriter out = new BufferedWriter(new FileWriter(fileLoc));

            int row = this.numGenes;
            int col = this.numExps;
            String srtVector = "Symbol" + "\t";

            if (debug) {
                System.out.println("row, col, rowName " + row);
                System.out.println("row, col, rowName " + col);
                System.out.println("row, col, rowName " + rowNames[1]);
            }

            //write sample names
            int iRow;
            for (iRow = 0; iRow < samples.length - 1; iRow++) {
                srtVector += samples[iRow] + "\t";
            }
            out.write(srtVector + samples[samples.length - 1] + "\n");

            srtVector = "";
            for (iRow = 0; iRow < row; iRow++) {
                srtVector = rowNames[iRow] + "\t";
                for (int jCol = 0; jCol < col; jCol++) {
                    if (jCol == col - 1) {
                        srtVector += fm[iRow][jCol] + "\n";
                    } else {
                        srtVector += fm[iRow][jCol] + "\t";
                    }
                }
                out.write(srtVector);
                srtVector = "";
            }
            out.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return fileLoc;
    }

    private void removeFile(String fileName) {
        File f = new File(fileName);
        f.delete();
    }

    void copy(File in, File out){
        try {
            FileInputStream fis = new FileInputStream(in);
            FileOutputStream fos = new FileOutputStream(out);
            byte[] buf = new byte[1024];
            int i = 0;
            while ((i = fis.read(buf)) != -1) {
                fos.write(buf, 0, i);
            }
            fos.close();
            fis.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void copyData(InputStreamReader isr, String savePath) {
        try {
            BufferedReader br = new BufferedReader(isr);
            FileWriter fw = new FileWriter(savePath);
            BufferedWriter bw = new BufferedWriter(fw);
            String s = br.readLine();
            while (s != null) {
                bw.write(s);
                bw.newLine();
                s = br.readLine();
            }
            br.close();
            bw.close();
            fw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}

