/**
 * Department of Genetics
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2012 by the
 * Department of Genetics/University of North Carolina at Chapel Hill.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither Department of Genetics nor University of North Carolina at Chapel Hill can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.gsea;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.gsea.GeneSetCohortGenerator;
import edu.mit.broad.genome.alg.gsea.KSTests;
import edu.mit.broad.genome.math.RandomSeedGenerator;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.genome.parsers.EdbFolderParser;
import edu.mit.broad.genome.reports.Seq1EnrichmentReports;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import edu.mit.broad.vdb.chip.Chip;
import xtools.api.param.*;
import org.apache.log4j.Logger;
import edu.duke.igsp.app.tools.ArrayListTool;
import java.util.*;
import java.io.*;
import java.text.SimpleDateFormat;

import java.util.Properties;

/**
 * @author Qing Xiong <qxiong@email.unc.edu>
 * Date: September 22, 2012
 *
 */
public class GsaaSeqGP extends AbstractGsaaSeqGP2Tool{
    private static final Logger klog = XLogger.getLogger(GsaaSeqGP.class);

    private final IntegerParam fShowDetailsForTopXSetsParam = new IntegerParam("plot_top_x", "Plot graphs for the top sets of each phenotype", "Plot GSAASeq mountain and related plots for the top sets of each phenotype", 20, false, Param.ADVANCED);
    private final BooleanParam fMakeZippedReportParam = ParamFactory.createZipReportParam(false);
    private final BooleanParam fMakeGeneSetReportsParam = new BooleanParam("make_sets", "Make detailed gene set report", "Create detailed gene set reports (heat-map, mountain plot etc) for every associated gene set", true, false, Param.ADVANCED);
    private GeneSet[] fOrigGeneSets;

    public GsaaSeqGP(final Properties properties) {
        super.init(properties);
    }

    public GsaaSeqGP(final String[] args) {
        super.init(args);
    }

    public GsaaSeqGP() {
        declareParams();
    }

    public void execute() throws Exception {        
        final ReportIndexState state = new ReportIndexState(true, false, false, createHeader());
        startExec(state);

        if (!fGeneSetMatrixParam.isSpecified()) {
            Application.getWindowManager().showError("Gene set dataset must be specified");
            throw new IllegalArgumentException("Gene set dataset must be specified");
        }

        if (!fDatasetParam.isSpecified()) {
            Application.getWindowManager().showError("Gene expression dataset must be specified");
            throw new IllegalArgumentException("Gene expression dataset must be specified");
        }

        if (!fTemplateParam.isSpecified()) {
            Application.getWindowManager().showError("Phenotype labels for gene expression dataset must be specified");
            throw new IllegalArgumentException("Phenotype labels for gene expression dataset must be specified");
        }

        Dataset ds = fDatasetParam.getDataset();
        final Dataset fullDs = uniquize(ds);
        final CollapsedDetails.Data cd = super.getDataset_no_collapse(fullDs);

        final Template template = fTemplateParam.getTemplate();

        this.fOrigGeneSets = fGeneSetMatrixParam.getGeneSetMatrixCombo(true).getGeneSets();
        final GeneSet[] gsets = Helper.getGeneSets(cd.getDataset(), fOrigGeneSets, fGeneSetMinSizeParam, fGeneSetMaxSizeParam);
        ParamFactory.checkAndBarfIfZeroSets(gsets);        

        final HtmlReportIndexPage htmlReportIndexPage = fReport.getIndexPage();

        final String[] geneNames = cd.getDataset().getRowNamesArray();
        final int numGenes = geneNames.length;

        final String[] sampleNames = cd.getDataset().getColumnNamesArray();
        final int numExps = sampleNames.length;

        final Matrix matrix = cd.getDataset().getMatrix();
        final int[][] countMatrix = new int[matrix.getNumRow()][matrix.getNumCol()];
        for (int i = 0; i < countMatrix.length; i++) {
            for (int j = 0; j < countMatrix[0].length; j++) {
                countMatrix[i][j] = (int) matrix.getElement(i, j);
            }
        }

        List phenos = new ArrayList();
        int numOfSamples = template.getNumItems();
        for (int i = 0; i < numOfSamples; i++) {
            String classname = template.getClassByProfilePos(i).getName();
            int index = template.getClassIndex(classname) + 1;
            String s = (new Integer(index)).toString();
            phenos.add(s);
        }
        final int[] groupAssignments = ArrayListTool.list2int(phenos);

        String demetric = fDEMetricParam.getString();

        String out = fReport.getReportDir().getAbsolutePath();
        String fileSep = System.getProperty("file.separator");
        (new File(out + fileSep + "gsaaseq")).mkdir();
        out = out + fileSep + "gsaaseq" + fileSep;         

        if (demetric.equalsIgnoreCase("edgeR")) {
            GsaaEdgeR edger = new GsaaEdgeR();
            edger.execute(geneNames, sampleNames, groupAssignments, countMatrix, numGenes, numExps, out);
        } else if (demetric.equalsIgnoreCase("DESeq")) {
            GsaaDeseq deseq = new GsaaDeseq();
            deseq.execute(geneNames, sampleNames, groupAssignments, countMatrix, numGenes, numExps, out);
        } else if (demetric.equalsIgnoreCase("DEGSeq")) {
            GsaaDegseq degseq = new GsaaDegseq();
            degseq.execute(geneNames, sampleNames, groupAssignments, countMatrix, numGenes, numExps, out);
        } else if (demetric.equalsIgnoreCase("baySeq")) {
            GsaaBayseq bayseq = new GsaaBayseq();
            bayseq.execute(geneNames, sampleNames, groupAssignments, countMatrix, numGenes, numExps, out);
        } else if (demetric.equalsIgnoreCase("NOISeq")) {
            GsaaNoiseq noiseq = new GsaaNoiseq();
            noiseq.execute(geneNames, sampleNames, groupAssignments, countMatrix, numGenes, numExps, out);
        } else{
        }        
        
        String fileLoc = out + "scores.rnk";
        InputStream is = new FileInputStream(fileLoc);
        RankedList fullR1 = uniquize(ParserFactory.readRankedList(fileLoc, is, true));

        final CollapsedDetails.Ranked cdr = getRankedList_no_collapse(fullR1);
        execute_one(cdr, fullDs, template, gsets, htmlReportIndexPage);

        if (fMakeZippedReportParam.isTrue()) {
            // custom close before zipping
            fReport.closeReport(true);
            fReport.zipReport();
        }

        doneExec();  
                  
    }

    private void execute_one(final edu.mit.broad.genome.objects.strucs.CollapsedDetails fullRL,
            final Dataset ds_shared_opt,
            final Template template_shared_opt,
            final GeneSet[] gsets,
            final HtmlReportIndexPage reportIndexPage) throws Exception {

        final int nperms = fNumPermParam.getIValue();
        final int topXSets = fShowDetailsForTopXSetsParam.getIValue();
        final RandomSeedGenerator rst = fRndSeedTypeParam.createSeed();
        final GeneSetCohortGenerator gcohgen = fGcohGenReqdParam.createGeneSetCohortGenerator(false);
        final int minSize = fGeneSetMinSizeParam.getIValue();
        final int maxSize = fGeneSetMaxSizeParam.getIValue();
        Chip chip = null;
        FeatureAnnot fann;
        RankedList rl = ((CollapsedDetails.Ranked) fullRL).getRankedList();
        fann = new FeatureAnnotImpl(rl.getName(), rl.getRankedNames(), null);

        final KSTests tests = new KSTests(getOutputStream());

        EnrichmentDb edb = tests.executeGsaaseqgp(
                rl,
                ds_shared_opt,
                template_shared_opt,
                gsets,
                nperms,
                rst,
                chip,
                gcohgen);

        // Make the report
        Seq1EnrichmentReports.Ret ret = Seq1EnrichmentReports.createGseaLikeReport_gp(edb, getOutputStream(),
                fullRL, reportIndexPage, false,
                fReport, topXSets, minSize, maxSize,
                fMakeGeneSetReportsParam.isTrue(), fMakeZippedReportParam.isTrue(),
                fOrigGeneSets, fNormModeParam.getNormModeName(), fann);

        // Make an edb folder thing
        new EdbFolderParser().export(ret.edb, ret.savedInDir);

    }

    protected Param[] getAdditionalParams() {
        return new Param[]{fShowDetailsForTopXSetsParam, fMakeZippedReportParam, fMakeGeneSetReportsParam};
    }

    public String getDesc() {
        return "Set parameters and run GSAASeqGP";
    }

    public static void main(String[] args) {
        GsaaSeqGP tool = new GsaaSeqGP(args);
        tool_main(tool);
    }

    public static String createHeader() {
        try {
            StringBuffer buf = new StringBuffer();
            buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                    "<h3 style=\"text-align: left\"><font color=\"#808080\">GSAASeqGP Report").append("</font></h3>\n").append("</div>");

            return buf.toString();
        } catch (Throwable t) {
            t.printStackTrace();
        }

        return null;
    }

} // End GsaaSeqGP
