/**
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2013 by the
 * Department of Computer Science and Technology/Southwest University and Department of Genetics/University of North Carolina at Chapel Hill.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither Southwest University nor University of North Carolina at Chapel Hill can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.gsea;

import edu.duke.igsp.app.tools.GsaaSeqTool;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.strucs.CollapsedDetails;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import edu.mit.broad.xbench.core.api.Application;
import xtools.api.param.*;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.Annot;
import edu.mit.broad.genome.objects.DefaultDataset;
import org.apache.log4j.Logger;
import java.util.*;
import java.io.*;
import java.text.SimpleDateFormat;


/**
 * @author Qing Xiong <qingx@swu.edu.cn> & <qxiong@email.unc.edu>
 * Date: January 22, 2013
 *
 */
public class GsaaSeqSP extends AbstractGsaaSeqSP2Tool{  
    private static final Logger klog = XLogger.getLogger(GsaaSeqSP.class);

    private final IntegerParam fShowDetailsForTopXSetsParam = new IntegerParam("plot_top_x", "Plot graphs for the top sets of each phenotype", "Plot GSAASeq mountain and related plots for the top sets of each phenotype", 20, false, Param.ADVANCED);
    private final BooleanParam fMakeZippedReportParam = ParamFactory.createZipReportParam(false);
    private final BooleanParam fMakeGeneSetReportsParam = new BooleanParam("make_sets", "Make detailed gene set report", "Create detailed gene set reports (heat-map, mountain plot etc) for every associated gene set", true, false, Param.ADVANCED);

    public GsaaSeqSP(final Properties properties) {
        super.init(properties);
    }

    public GsaaSeqSP(final String[] args) {
        super.init(args);
    }

    public GsaaSeqSP() {
        declareParams();
    }

    public void execute() throws Exception {        
        final ReportIndexState state = new ReportIndexState(true, false, false, createHeader());
        startExec(state);

        if (!fGeneSetMatrixParam.isSpecified()) {
            Application.getWindowManager().showError("Gene set dataset must be specified");
            throw new IllegalArgumentException("Gene set dataset must be specified");
        }

        if (!fDatasetParam.isSpecified()) {
            Application.getWindowManager().showError("Gene expression dataset must be specified");
            throw new IllegalArgumentException("Gene expression dataset must be specified");
        }

        if (!fTemplateParam.isSpecified()) {
            Application.getWindowManager().showError("Phenotype labels for gene expression dataset must be specified");
            throw new IllegalArgumentException("Phenotype labels for gene expression dataset must be specified");
        }

        Dataset ds = fDatasetParam.getDataset();
        final Dataset fullDs = uniquize(ds);
        final CollapsedDetails.Data cd = super.getDataset_no_collapse(fullDs);
        
        final Template template = fTemplateParam.getTemplate();
        forceTwoClasses(template);
        
        //add 1 to all counts to avoid zero counts
        int numRow = cd.getDataset().getMatrix().getNumRow();
        int numCol = cd.getDataset().getMatrix().getNumCol();
        Matrix m = new Matrix(numRow, numCol);
        for(int i=0; i<numRow; i++){
            for(int j=0; j<numCol; j++){
                double d = (double)cd.getDataset().getMatrix().getElement(i, j);
                m.setElement(i, j, (float)(d+1));
            }
        }
        
        m=GsaaSeqTool.normalizeDESeq(m);
        //Matrix m=cd.getDataset().getMatrix();
        String objName = cd.getDataset().getName();
        String[] rowNames = cd.getDataset().getRowNamesArray();
        String[] colNames = cd.getDataset().getColumnNamesArray();
        Annot ann = cd.getDataset().getAnnot();
        final Dataset dataset = new DefaultDataset(objName, m, rowNames, colNames, true, ann); 
        final CollapsedDetails.Data cdataset = super.getDataset_no_collapse(dataset);     
 
        final GeneSet[] origGeneSets = fGeneSetMatrixParam.getGeneSetMatrixCombo(true).getGeneSets();
        final GeneSet[] gsets = Helper.getGeneSets(cdataset.getDataset(), origGeneSets, fGeneSetMinSizeParam, fGeneSetMaxSizeParam);
        ParamFactory.checkAndBarfIfZeroSets(gsets);

        String gsametric = fGSAMetricParam.getString();
        String demetric = fDEMetricParam.getString();        
        String pthreshold = fPThresholdParam.getString();

        final HtmlReportIndexPage htmlReportIndexPage = fReport.getIndexPage();
        final boolean makeSubDir = false;

        execute_one_with_reporting(demetric, gsametric, pthreshold, cdataset, template, gsets,
                htmlReportIndexPage, makeSubDir, origGeneSets, fShowDetailsForTopXSetsParam.getIValue(), fMakeZippedReportParam.isTrue(), fMakeGeneSetReportsParam.isTrue());

        if (fMakeZippedReportParam.isTrue()) {
            // custom close before zipping
            fReport.closeReport(true);
            fReport.zipReport();
        }

        doneExec();   
          
    }

    protected Param[] getAdditionalParams() {
        return new Param[]{fShowDetailsForTopXSetsParam, fMakeZippedReportParam, fMakeGeneSetReportsParam};
    }

    public String getDesc() {
        return "Set parameters and run GSAASeqSP";
    }

    public static void main(String[] args) {
        GsaaSeqSP tool = new GsaaSeqSP(args);
        tool_main(tool);
    }

    public static String createHeader() {
        try {
            StringBuffer buf = new StringBuffer();
            buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                    "<h3 style=\"text-align: left\"><font color=\"#808080\">GSAASeqSP Report").append("</font></h3>\n").append("</div>");

            return buf.toString();
        } catch (Throwable t) {
            t.printStackTrace();
        }

        return null;
    }
    
    public static void forceTwoClasses(Template template) {
        int numOfClass = template.getNumClasses();
        if (numOfClass != 2) {
            throw new IllegalArgumentException("The number of phenotype labels must remain 2. Got: " + numOfClass);
        }
    }   

} // End GsaaSeqSP
