/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.gsea;

import edu.mit.broad.genome.StandardException;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.DefaultDataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.TemplateFactoryRandomizer;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.math.RandomSeedGenerator;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.genome.objects.FeatureAnnot;
import edu.mit.broad.genome.objects.FeatureAnnotImpl;
import edu.mit.broad.genome.objects.SampleAnnotImpl;
import edu.mit.broad.genome.objects.AnnotImpl;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.genome.alg.gsea.GeneSetCohortGenerator;
import edu.mit.broad.genome.alg.DatasetMetrics;
import xtools.api.param.*;
import edu.duke.igsp.app.tools.FileTool;
import edu.duke.igsp.app.services.MappingService;
import edu.duke.igsp.app.algorithms.CMetric;
import edu.duke.igsp.app.struts.SnpStruc;
import edu.duke.igsp.app.tools.GsaaTool;
import java.util.Properties;
import java.util.ArrayList;
import java.util.List;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class GsaaSnp extends AbstractGsaaSnp2Tool {

    private final IntegerParam fShowDetailsForTopXSetsParam = new IntegerParam("plot_top_x", "Plot graphs for the top sets of each phenotype", "Plot GSEA mountain and related plots for the top sets of each phenotype", 20, false, Param.ADVANCED);
    private final BooleanParam fMakeZippedReportParam = ParamFactory.createZipReportParam(false);
    private final BooleanParam fMakeGeneSetReportsParam = new BooleanParam("make_sets", "Make detailed gene set report", "Create detailed gene set reports (heat-map, mountain plot etc) for every associated gene set", true, false, Param.ADVANCED);

    /**
     * Class constructor
     *
     * @param properties
     */
    public GsaaSnp(final Properties properties) {
        super.init(properties);
    }

    public GsaaSnp(final String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public GsaaSnp() {
        declareParams();
    }

    public void execute() throws Exception {

        // to preserve memory & for custom indexing
        final ReportIndexState state = new ReportIndexState(true, false, false, createHeader());
        startExec(state);

        if (!fGeneSetMatrixParam.isSpecified()) {
            Application.getWindowManager().showError("Gene set dataset must be specified");
            throw new IllegalArgumentException("Gene set dataset must be specified");
        }
        if (!fSnpDatasetParam.isSpecified()) {
            Application.getWindowManager().showError("SNP dataset must be specified");
            throw new IllegalArgumentException("SNP dataset must be specified");
        }
        if (!fSnpTemplateParam.isSpecified()) {
            Application.getWindowManager().showError("Phenotype labels for SNP dataset must be specified");
            throw new IllegalArgumentException("Phenotype labels for SNP dataset must be specified");
        }

        final Template template_snp_ori = fSnpTemplateParam.getTemplate();
        forceTwoClasses(template_snp_ori);
        final CMetric cmetric = fCMetricParam.getCMetric();
        if (template_snp_ori.isCategorical()) {
            if (!cmetric.isCategorical()) {
                throw new StandardException("As the phenotype choosen was categorical, only categorical metrics are allowed. Got: " + cmetric.getName(), 1010);
            }
        } else {
            throw new IllegalArgumentException("Only categorical templates allowed. Got: " + template_snp_ori.getName());
        }

        int numOfClass = template_snp_ori.getNumClasses();
        if (numOfClass != 2) {
            throw new IllegalArgumentException("The number of phenotype labels must remain 2. Got: " + numOfClass);
        }

        if (!fSnpDatasetParam.isSpecified()) {
            Application.getWindowManager().showError("SNP dataset must be specified");
        }

        String species = fSpeciesParam.getString();
        InputStreamReader isr = null;
        if (species.equalsIgnoreCase("Human_GRCh37")) {
            //species = "homo_sapiens";
            InputStream is = getClass().getResourceAsStream("genepos/homo_sapiens.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Human_NCBI36")) {
            //species = "homo_sapiens";
            InputStream is = getClass().getResourceAsStream("genepos/homo_sapiens_NCBI36.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Mouse")) {
            //species = "mus_musculus";
            InputStream is = getClass().getResourceAsStream("genepos/mus_musculus.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Rat")) {
            //species = "rattus_norvegicus";
            InputStream is = getClass().getResourceAsStream("genepos/rattus_norvegicus.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Zebrafish")) {
            //species = "danio_rerio";
            InputStream is = getClass().getResourceAsStream("genepos/danio_rerio.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Arabidopsis_Thaliana")) {
            //species = "arabidopsis_thaliana";
            InputStream is = getClass().getResourceAsStream("genepos/arabidopsis_thaliana.txt");
            isr = new InputStreamReader(is);  
        } else if (species.equalsIgnoreCase("Cat")) {
            //species = "felis_catus";
            InputStream is = getClass().getResourceAsStream("genepos/felis_catus.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("C.elegans")) {
            //species = "caenorhabditis_elegans";
            InputStream is = getClass().getResourceAsStream("genepos/caenorhabditis_elegans.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Chicken")) {
            //species = "gallus_gallus";
            InputStream is = getClass().getResourceAsStream("genepos/gallus_gallus.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Chimpanzee")) {
            //species = "pan_troglodytes";
            InputStream is = getClass().getResourceAsStream("genepos/pan_troglodytes.txt");
            isr = new InputStreamReader(is);                       
        } else if (species.equalsIgnoreCase("C.intestinalis")) {
            //species = "ciona_intestinalis";
            InputStream is = getClass().getResourceAsStream("genepos/ciona_intestinalis.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Cow")) {
            //species = "bos_taurus";
            InputStream is = getClass().getResourceAsStream("genepos/bos_taurus.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Dog")) {
            //species = "canis_familiaris";
            InputStream is = getClass().getResourceAsStream("genepos/canis_familiaris.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Fruitfly")) {
            //species = "drosophila_melanogaster";
            InputStream is = getClass().getResourceAsStream("genepos/drosophila_melanogaster.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Fugu")) {
            //species = "tetraodon_nigroviridis";
            InputStream is = getClass().getResourceAsStream("genepos/tetraodon_nigroviridis.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Gorilla")) {
            //species = "gorilla_gorilla";
            InputStream is = getClass().getResourceAsStream("genepos/gorilla_gorilla.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Horse")) {
            //species = "equus_caballus";
            InputStream is = getClass().getResourceAsStream("genepos/equus_caballus.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Macaque")) {
            //species = "macaca_mulatta";
            InputStream is = getClass().getResourceAsStream("genepos/macaca_mulatta.txt");
            isr = new InputStreamReader(is);            
        } else if (species.equalsIgnoreCase("Marmoset")) {
            //species = "callithrix_jacchus";
            InputStream is = getClass().getResourceAsStream("genepos/callithrix_jacchus.txt");
            isr = new InputStreamReader(is);           
        } else if (species.equalsIgnoreCase("Opossum")) {
            //species = "monodelphis_domestica";
            InputStream is = getClass().getResourceAsStream("genepos/monodelphis_domestica.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Orangutan")) {
            //species = "pongo_abelii";
            InputStream is = getClass().getResourceAsStream("genepos/pongo_abelii.txt");
            isr = new InputStreamReader(is); 
        } else if (species.equalsIgnoreCase("Pig")) {
            //species = "sus_scrofa";
            InputStream is = getClass().getResourceAsStream("genepos/sus_scrofa.txt");
            isr = new InputStreamReader(is);             
        } else if (species.equalsIgnoreCase("Rabbit")) {
            //species = "oryctolagus_cuniculus";
            InputStream is = getClass().getResourceAsStream("genepos/oryctolagus_cuniculus.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Saccharomyces_Cerevisiae")) {
            //species = "saccharomyces_cerevisiae";
            InputStream is = getClass().getResourceAsStream("genepos/saccharomyces_cerevisiae.txt");
            isr = new InputStreamReader(is);
        } else if (species.equalsIgnoreCase("Stickleback")) {
            //species = "gasterosteus_aculeatus";
            InputStream is = getClass().getResourceAsStream("genepos/gasterosteus_aculeatus.txt");
            isr = new InputStreamReader(is);  
        } else if (species.equalsIgnoreCase("Turkey")) {
            //species = "meleagris_gallopavo";
            InputStream is = getClass().getResourceAsStream("genepos/meleagris_gallopavo.txt");
            isr = new InputStreamReader(is);              
        } else if (species.equalsIgnoreCase("Simulation")) {
            //species = "simulation";
            InputStream is = getClass().getResourceAsStream("genepos/sim.txt");
            isr = new InputStreamReader(is); 
        }else{
            
        }

        ArrayList genes = new ArrayList();
        if (!fGeneListParam.isSpecified()) {
            genes = FileTool.toBeans(isr);
        } else {
            final String[] geneNames = fGeneListParam.getGeneList();
            genes = MappingService.lookUp(geneNames, isr);
        }
        int oriSize = genes.size();
        log.info("At the beginning there are : " + oriSize + " genes");

        int before = ((Integer) fBpBeforeGeneParam.getValue()).intValue();
        int after = ((Integer) fBpAfterGeneParam.getValue()).intValue();
        int numOfItems = template_snp_ori.getNumItems();

        boolean permuteTemplate = fPermuteTypeParamType.permuteTemplate();
        final int nperms = fNumPermParam.getIValue();
        final RandomSeedGenerator rst = fRndSeedTypeParam.createSeed();
        final TemplateRandomizerType rt = fRndTypeParam.getRandomizerType();
        String snpSetMetric = (String) fSnpSetMetric.getValue();
        SnpStruc snpStruc = null;
        Template[] rndTemplates = null;
        if (permuteTemplate) {
            rndTemplates = TemplateFactoryRandomizer.createRandomTemplates(nperms, template_snp_ori, rt, rst);
            if (snpSetMetric.equalsIgnoreCase("Maximum")) {
                snpStruc = (SnpStruc) fSnpDatasetParam.getSnpStruc_phenotype_maximum(cmetric, template_snp_ori, rndTemplates, genes, before, after, numOfItems);
            }
        } else {
            if (snpSetMetric.equalsIgnoreCase("Maximum")) {
                snpStruc = (SnpStruc) fSnpDatasetParam.getSnpStruc_geneset_maximum(cmetric, template_snp_ori, genes, before, after, numOfItems);
            }
        }

        //construct dataset
        final List scoredDataset = snpStruc.getScoredDataset();
        final GeneSet[] origGeneSets_snp = snpStruc.getSnpSet();
        String objName = fSnpDatasetParam.getDatasetName();
        final GeneSetCohortGenerator gcohgen = fGcohGenReqdParam.createGeneSetCohortGenerator(false);
        final DatasetMetrics dm = new DatasetMetrics();

        ArrayList scores = new ArrayList();
        if (snpSetMetric.equalsIgnoreCase("Maximum")) {
            scores = (ArrayList) scoredDataset;
        }

        //compute p-value for genes
        List pValues = new ArrayList();
        if (permuteTemplate) {
            for (int i = 0; i < scores.size(); i++) {
                double p = 0;
                ArrayList geneScores = (ArrayList) scores.get(i);
                float real = ((Float) geneScores.get(0)).floatValue();
                p = GsaaTool.getPValue(real, geneScores);
                p = XMath.roundOff(p, 4);
                pValues.add((new Double(p)).toString());
            }
        } else {
            for (int i = 0; i < scores.size(); i++) {
                pValues.add("na");
            }
        }

        ArrayList rowNames_al = new ArrayList();
        List description = new ArrayList();         
        for (int i = 0; i < origGeneSets_snp.length; i++) {
            rowNames_al.add(origGeneSets_snp[i].getName());
            String maxSnpNames=origGeneSets_snp[i].getMembersArray()[0];  
            description.add(maxSnpNames+": "+pValues.get(i));            
        }

        //construct dataset
        String[] rowNames = new String[rowNames_al.size()];
        for (int i = 0; i < rowNames_al.size(); i++) {
            rowNames[i] = (String) rowNames_al.get(i);
        }

        String[] colNames = new String[((ArrayList) scores.get(0)).size()];
        for (int i = 0; i < colNames.length; i++) {
            if (i == 0) {
                colNames[i] = "RealScore";
            } else {
                colNames[i] = "Perm" + i;
            }
        }

        Matrix matrix = new Matrix(rowNames.length, colNames.length);
        for (int i = 0; i < rowNames.length; i++) {
            for (int j = 0; j < colNames.length; j++) {
                float score = ((Float) ((ArrayList) scores.get(i)).get(j)).floatValue();
                matrix.setElement(i, j, score);
            }
        }

        log.info("Mapping resulted in filtering out " + (oriSize - rowNames.length) + "/" + oriSize + " genes, the remainning " + rowNames.length + " genes were used in the analysis");

        final FeatureAnnot ann = new FeatureAnnotImpl(objName, rowNames_al, description);
        final SampleAnnot sann = new SampleAnnotImpl(objName, colNames, null);
        final Dataset ds = new DefaultDataset(objName, matrix, rowNames, colNames, true, new AnnotImpl(ann, sann));
        final Dataset fullDs = uniquize(ds);

        final GeneSet[] origGeneSets = fGeneSetMatrixParam.getGeneSetMatrixCombo(true).getGeneSets();

        final GeneSet[] gsets = Helper.getGeneSets(fullDs, origGeneSets, fGeneSetMinSizeParam, fGeneSetMaxSizeParam);
        ParamFactory.checkAndBarfIfZeroSets(gsets);

        final HtmlReportIndexPage htmlReportIndexPage = fReport.getIndexPage();
        final boolean makeSubDir = false;

        execute_one_with_reporting(fullDs, template_snp_ori, gsets, htmlReportIndexPage, makeSubDir, origGeneSets, fShowDetailsForTopXSetsParam.getIValue(), fMakeZippedReportParam.isTrue(), fMakeGeneSetReportsParam.isTrue(), rndTemplates);
        if (fMakeZippedReportParam.isTrue()) {
            fReport.closeReport(true);
            fReport.zipReport();
        }

        doneExec();
    }

    protected Param[] getAdditionalParams() {
        return new Param[]{fShowDetailsForTopXSetsParam, fMakeZippedReportParam, fMakeGeneSetReportsParam};
    }

    public String getDesc() {
        return "Set parameters and run GSAA_SNP";
    }

    public static void main(String[] args) {
        GsaaSnp tool = new GsaaSnp(args);
        tool_main(tool);
    }

    public static String createHeader(final DatasetReqdParam dsr) {
        try {
            if (dsr.isSpecified()) {
                Dataset ds = dsr.getDataset();
                StringBuffer buf = new StringBuffer();
                buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                        "<h3 style=\"text-align: left\"><font color=\"#808080\">GSAA Report for ").append(
                        "Dataset ").append(ds.getName()).append("</font></h3>\n").append("</div>");

                return buf.toString();
            }
        } catch (Throwable t) {
            t.printStackTrace();
        }

        return null;
    }

    public static String createHeader() {
        try {
            StringBuffer buf = new StringBuffer();
            buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                    "<h3 style=\"text-align: left\"><font color=\"#808080\">GSAA_SNP Report").append("</font></h3>\n").append("</div>");

            return buf.toString();
        } catch (Throwable t) {
            t.printStackTrace();
        }

        return null;
    }
    
    public static void forceTwoClasses(Template template) {
        int numOfClass = template.getNumClasses();
        if (numOfClass != 2) {
            throw new IllegalArgumentException("The number of phenotype labels must remain 2. Got: " + numOfClass);
        }
    }    
}    // End GsaaSnp

