package xtools.gsea;


import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.alg.cluster.ClusteringAlgs;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.genome.reports.pages.HtmlFormat;
import edu.mit.broad.genome.reports.pages.HtmlReportIndexPage;
import edu.mit.broad.genome.utils.ImageUtils;
import org.apache.ecs.StringElement;
import org.apache.ecs.html.*;
import xapps.api.vtools.viewers.VizFactory;
import xtools.api.AbstractTool;
import xtools.api.ToolCategory;
import xtools.api.param.DirParam;
import xtools.api.param.StringMultiInputParam;

import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Properties;

/**
 * Commandline Tool to run the Tests
 * <p/>
 * Runs it on a ffn of dataset+template+geneSetmatrix
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class LeadingEdgeTool extends AbstractTool {

    public DirParam fGseaResultDirParam = new DirParam(true);

    public StringMultiInputParam fGeneSetNamesParam = new StringMultiInputParam("gsets", "gene sets", true);

    /**
     * Class constructor
     *
     * @param properties
     */
    public LeadingEdgeTool(final Properties properties) {
        super.init(properties);
    }

    public LeadingEdgeTool(final String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public LeadingEdgeTool() {
        declareParams();
    }

    private Dataset _morph(Dataset ds, RankedList rl) {
        Matrix m = new Matrix(ds.getNumRow(), ds.getNumCol());
        for (int r = 0; r < ds.getNumRow(); r++) {
            for (int c = 0; c < ds.getNumCol(); c++) {
                float score = rl.getScore(ds.getColumnName(c));
                float value = ds.getElement(r, c);
                if (value == 1) {
                    m.setElement(r, c, score);
                } else {
                    m.setElement(r, c, value); // ought to be zero
                }
            }
        }


        return new DefaultDataset(ds.getName(), m, ds.getRowNames(), ds.getColumnNames(), true, ds.getAnnot());
    }

    /**
     * @throws java.lang.Exception
     */
    public void execute() throws Exception {

        // to preserve memory & for custom indexing
        final ReportIndexState state = new ReportIndexState(true, false, false, getHeader(fGseaResultDirParam));
        startExec(state);

        final File gseaResultDir = fGseaResultDirParam.getFile();
        final EnrichmentDb edb = ParserFactory.readEdb(gseaResultDir, true);
        final String[] gsetNames = fGeneSetNamesParam.getStrings();

        final EnrichmentResult[] results = new EnrichmentResult[gsetNames.length];
        final GeneSet[] gsets = new GeneSet[results.length];
        for (int r = 0; r < gsetNames.length; r++) {
            log.debug("gset: " + gsetNames[r]);
            results[r] = edb.getResultForGeneSet(gsetNames[r]);
            gsets[r] = results[r].getSignal().getAsGeneSet();
        }

        final GeneSetMatrix lev_gmx = new DefaultGeneSetMatrix("leading_edge_matrix_for_" + edb.getName(), gsets);
        final BitSetDataset lev_bsd = new BitSetDataset(lev_gmx);
        final Dataset lev_ds = lev_bsd.toDataset(true, false);
        final File lev_ds_file = fReport.savePage(lev_ds, false);

        Dataset lev_ds_clustered = null;
        //Dataset lev_ds_clustered_matlab = null;
        try {
            log.info("Clustering signal matrix: " + lev_ds.getQuickInfo() + " " + lev_ds_file.getPath());
            lev_ds_clustered = ClusteringAlgs.cluster(lev_ds).getDatasetOrdered();

        } catch (Throwable t) {
            fReport.addError("Trouble clustering", t);
        }

        // -------------------------------------------------------------------------------------------- //
        // Make the Reports
        final HtmlReportIndexPage reportIndexPage = fReport.getIndexPage();

        Div div = new Div();
        H2 h2 = new H2("Leading edge results for enrichment database: <b>" + edb.getName() + "</b>");
        div.addElement(h2);

        // -------------------------------------------------------------------------------------------- //
        div = new Div();
        div.addElement(new H4("Leading edge analysis: Clustered results"));
        UL ul = new UL();
        ul.addElement(new LI("There were " + results.length + " gene sets used in the leading edge analysis (see below for details)"));

        Dataset lev_ds_clustered_m = _morph(lev_ds_clustered, edb.getRankedList());

        BufferedImage bi = VizFactory.createGramImager(GPWrappers.createColorScheme_for_lev_with_score(lev_ds_clustered_m)).createBpogImage(lev_ds_clustered_m);
        File lev_clust_hm_file = fReport.createFile("leading_edge_heat_map_clustered.png", "foo");
        ImageUtils.savePng(bi, lev_clust_hm_file);
        StringElement line3 = HtmlFormat.Links.hyper("Heat map of clustered leading edge subsets", lev_clust_hm_file, ". Rows are gene sets and columns are genes. This matrix is clustered", fReport.getReportDir());
        ul.addElement(new LI(line3));
        div.addElement(ul);

        reportIndexPage.addBlock(div, true);

        // -------------------------------------------------------------------------------------------- //

        // Then add details on the gene sets used in the lea
        Table table = new Table();
        TH th = new TH();
        //th.addElement(new TD("Gene set"));
        th.addElement(new TD("# members"));
        th.addElement(new TD("# members<br>in signal"));
        th.addElement(new TD("Tag %"));
        th.addElement(new TD("List %"));
        th.addElement(new TD("Signal strength"));
        table.addElement(th);
        for (int r = 0; r < results.length; r++) {
            TR tr = new TR();

            String gsetName = results[r].getGeneSetName();
            File gsetGseaFile = new File(gseaResultDir, gsetName + ".html");
            A link = new A(gsetGseaFile.toURI().toString(), gsetName);
            // link.setHref(gsetGseaFile.toURI().toString());
            //HtmlFormat.Links.setHref(link, new File(gsetName + ".html"), gseaResultDir);

            tr.addElement(new TD(link));
            tr.addElement(new TD("" + results[r].getGeneSet().getNumMembers()));
            GeneSetSignal signal = results[r].getSignal();
            tr.addElement(new TD("" + signal.getSize()));
            tr.addElement(new TD(Printf.format(signal.getTagFraction() * 100, 0) + "%"));
            tr.addElement(new TD(Printf.format(signal.getListFraction() * 100, 0) + "%"));
            tr.addElement(new TD(Printf.format(signal.getSignalStrength() * 100, 0) + "%"));
            table.addElement(tr);
        }

        div = new Div();
        div.addElement(new H4("Details of gene sets and signals used in this analysis"));
        div.addElement(table);
        reportIndexPage.addBlock(div, true);

        // -------------------------------------------------------------------------------------------- //

        div = new Div();
        div.addElement(new H4("Leading edge analysis: other files made"));
        ul = new UL();
        final File lev_ds_clustered_file = fReport.savePage(lev_ds_clustered, false);
        StringElement line2 = HtmlFormat.Links.hyper("Clustered dataset (gct)", lev_ds_clustered_file, "for the specified gene sets. 1's denote membership of a gene in the leading edge subset", fReport.getReportDir());
        ul.addElement(new LI(line2));

        File lev_gmx_file = fReport.savePageGmx(lev_gmx);
        StringElement line1 = HtmlFormat.Links.hyper("GeneSetMatrix (gmx)", lev_gmx_file, "of leading edge subsets", fReport.getReportDir());
        ul.addElement(new LI(line1));

        line2 = HtmlFormat.Links.hyper("Dataset (gct)", lev_ds_file, "for the specified gene sets. 1's denote membership of a gene in the leading edge subset", fReport.getReportDir());
        ul.addElement(new LI(line2));

        BufferedImage lev_image = VizFactory.createGramImager(GPWrappers.createColorScheme_for_lev_with_score(lev_ds)).createBpogImage(_morph(lev_ds, edb.getRankedList()));
        File lev_hm_file = fReport.createFile("leading_edge_heat_map_unclustered.png", "foo");
        ImageUtils.savePng(lev_image, lev_hm_file);
        line3 = HtmlFormat.Links.hyper("Heat map of leading edge subsets", lev_hm_file, ". Rows are gene sets and columns are genes. This matrix is NOT clustered", fReport.getReportDir());
        ul.addElement(new LI(line3));
        div.addElement(ul);

        reportIndexPage.addBlock(div, true);

        // -------------------------------------------------------------------------------------------- //
        // Other
        div = new Div();
        ul = new UL();
        div.addElement(new H4("Other"));
        ul.addElement(new LI(HtmlFormat.Links.hyper("Parameters", fReport.getParamsFile(), "used for this analysis", fReport.getReportDir())));
        div.addElement(ul);
        reportIndexPage.addBlock(div, false);

        reportIndexPage.setAddBrowseFooter(false); // turn off the little browse footer

        doneExec();
    }

    public ToolCategory getCategory() {
        return ToolCategory.GSEA;
    }

    public void declareParams() {

        fParamSet.addParam(fGseaResultDirParam);
        fParamSet.addParam(fGeneSetNamesParam);

    }

    public static void main(String[] args) {

        LeadingEdgeTool tool = new LeadingEdgeTool(args);
        tool_main(tool);

    }

    static String getHeader(final DirParam dirp) {
        try {
            if (dirp.isSpecified()) {
                String dpn = dirp.getDir().getName();
                StringBuffer buf = new StringBuffer();
                buf.append("<div id=\"footer\" style=\"width: 905; height: 35\">\n").append(
                        "<h3 style=\"text-align: left\"><font color=\"#808080\">Leading edge report for ").append(
                        "GSEA result folder: ").append(dpn).append("</font></h3>\n").append("</div>");

                return buf.toString();
            }
        } catch (Throwable t) {
            t.printStackTrace();
        }

        return null;
    }

}    // End LeadingEdgeTool

/*
try {
    log.info("Clustering signal matrix (for MatLab): " + lev_ds.getQuickInfo() + " " + lev_ds_file.getPath());
    final HClustDatasetJosh hclust = MatlabGaddy.hclustBoth(fReport.getName(),
            lev_ds_file,
            new Normalize2Param(),
            new LinkageParam(),
            new ClusterDistanceParam(),
            new ClusterAxisParam());

    lev_ds_clustered_matlab = hclust.getDatasetOrdered();

} catch (Throwable t) {
    error = t;
    fReport.addError("Trouble clustering", t);
}
*/

// -------------------------------------------------------------------------------------------- //
// @todo tmp
//final File lev_ds_clustered__matlab_file = fReport.savePage(lev_ds_clustered_matlab);
//line2 = HtmlFormat.Links.hyper("Dataset (gct)", lev_ds_clustered__matlab_file, "for the specified gene sets. 1's denote membership of a gene in the leading edge subset", fReport.getReportDir());
//ul.addElement(new LI(line2));

// bi = VizFactory.createGramImager().createBpogImage(lev_ds_clustered_matlab);
//File lev_clust_hm_file_matlab = fReport.createFile("leading_edge_heat_map_clustered_MATLAB.png", "foo");
//ImageUtils.savePng(bi, lev_clust_hm_file_matlab);
//line3 = HtmlFormat.Links.hyper("Heat map of leading edge subsets (MATLAB)", lev_clust_hm_file_matlab, ". Rows are gene sets and columns are genes. This matrix is clustered", fReport.getReportDir());
//ul.addElement(new LI(line3));
//div.addElement(ul);