package xtools.gsea;

import org.rosuda.JRI.Rengine;
import org.rosuda.JRI.REXP;
import org.tigr.util.FloatMatrix;
import java.util.*;

public class RCaller {

    static Rengine re;

    private RCaller(String[] args) {
        // just making sure we have the right version of everything
        if (!Rengine.versionCheck()) {
            System.err.println("** Version mismatch - Java files don't match library version.");
            return;
        }
        if (re != null) {
            //System.out.println("Rengine already exists");
            return;
        }
        System.out.println("Creating Rengine (with arguments)");
        // 1) we pass the arguments from the command line
        // 2) we won't use the main loop at first, we'll start it later
        //    (that's the "false" as second argument)
        // 3) the callbacks are implemented by the TextConsole class above
        re = new Rengine(args, false, new TextConsole());
        System.out.println("Rengine created, waiting for R");
        // the engine creates R is a new thread, so we should wait until it's ready
        if (!re.waitForR()) {
            System.out.println("Cannot load R");
            return;
        }
    }

    public static Rengine startRSession() throws Exception {
        try {
            if (!Rengine.versionCheck()) {
                System.err.println("** Version mismatch - Java files don't match library version.");
                throw new Exception("Java class version mismatch");
            }

            if ((re = Rengine.getMainEngine()) != null) {
                //System.out.println("Rengine already exists");
                cleanUp();
                return re;
            }
            System.out.println("Creating Rengine (with arguments)");
            String[] args = {"--no-save"};
            re = new Rengine(args, false, new TextConsole());
        } catch (Exception e) {
            System.out.println("Error loading R");
            e.printStackTrace();
        }

        System.out.println("Rengine created, waiting for R");
        // the engine creates R is a new thread, so we should wait until it's ready
        if (!re.waitForR()) {
            System.out.println("Cannot load R");
            throw new Exception("waitForR() error: Cannot load R");
        }
        return re;
    }

    public static void endR() {
        re.end();
        System.out.println("ended R session");

    }

    public static REXP evalR(String cmd) throws Exception {
        REXP x;
        try {
            x = re.eval(cmd);
        } catch (Exception e) {
            throw e;
        }
        return x;
    }

    private static void cleanUp() throws Exception {
        evalR("rm(list = ls())");
    }

    public static void endRSession() throws Exception {
        cleanUp();
    }

    public static REXP createRDataMatrix(String name, FloatMatrix fm) {
        REXP x = null;
        int row = fm.getRowDimension();
        int col = fm.getColumnDimension();
        String srtVector = "c(";
        int jCol = 0;
        for (int iRow = 0; iRow < row; iRow++) {
            for (jCol = 0; jCol < col - 1; jCol++) {
                srtVector += fm.get(iRow, jCol) + ",";
            }
            srtVector += fm.get(iRow, jCol);
        }
        srtVector += ")";
        String cmdR = name + "<- matrix(" + srtVector + ",nrow=" + row + ",ncol=" + col + ",byrow=TRUE)";
        System.out.println("createRDataMatrix " + cmdR);
        re.eval(cmdR);
        x = re.eval(name);
        return x;
    }

    /**
     * R MAtrix notation
     * mdat <- matrix(c(1,2,3, 11,12,13), nrow = 2, ncol=3, byrow=TRUE,dimnames = list(c("row1", "row2"),c("C.1", "C.2", "C.3")))
     * @param name
     * @param fm
     * @param rowNames
     * @param colNames
     * @return
     */
    public static REXP createRDataMatrix(String name, FloatMatrix fm, ArrayList rowNames, ArrayList colNames) {
        REXP x = null;
        int row = fm.getRowDimension();
        int col = fm.getColumnDimension();
        String srtVector = "c(";
        int jCol = 0;
        for (int iRow = 0; iRow < row; iRow++) {
            for (jCol = 0; jCol < col - 1; jCol++) {
                srtVector += fm.get(iRow, jCol) + ",";
            }
            srtVector += fm.get(iRow, jCol);
        }
        srtVector += ")";
        String dimnames = "dimnames = list(";
        String rNames = "c(";
        int iRow = 0;
        for (; iRow < row - 1; iRow++) {
            rNames += "'" + (String) rowNames.get(iRow) + "',";
        }
        rNames += "'" + (String) rowNames.get(iRow) + "')";

        String cNames = "c(";
        for (iRow = 0; iRow < row - 1; iRow++) {
            cNames += "'" + (String) colNames.get(iRow) + "',";
        }
        cNames += "'" + (String) colNames.get(iRow) + "')";

        dimnames += rNames + "," + cNames + ")";

        String cmdR = name + "<- matrix(" + srtVector + ",nrow=" + row + ",ncol=" + col + ",byrow=TRUE," + dimnames + ")";
        System.out.println("createRDataMatrix " + cmdR);
        re.eval(cmdR);
        x = re.eval(name);
        return x;
    }

    public static REXP createRDataMatrix(String name, FloatMatrix fm, String[] rowNames, String[] colNames) {
        REXP x = null;
        int row = fm.getRowDimension();
        int col = fm.getColumnDimension();
        String srtVector = "c(";

        for (int iRow = 0; iRow < row; iRow++) {
            for (int jCol = 0; jCol < col; jCol++) {
                if (iRow == row - 1 & jCol == col - 1) {
                    srtVector += fm.get(iRow, jCol);
                } else {
                    srtVector += fm.get(iRow, jCol) + ",";
                }
            }
        }
        //srtVector = srtVector.substring(0, srtVector.length()-2);
        srtVector += ")";
        String dimnames = "dimnames = list(";
        String rNames = "c(";
        int iRow = 0;
        for (; iRow < row - 1; iRow++) {
            rNames += "'" + (String) rowNames[iRow] + "',";
        }
        rNames += "'" + (String) rowNames[iRow] + "')";

        String cNames = "c(";
        for (iRow = 0; iRow < col - 1; iRow++) {
            cNames += "'" + (String) colNames[iRow] + "',";
        }
        cNames += "'" + (String) colNames[iRow] + "')";

        dimnames += rNames + "," + cNames + ")";

        String cmdR = name + "<- matrix(" + srtVector + ",nrow=" + row + ",ncol=" + col + ",byrow=TRUE," + dimnames + ")";
        System.out.println("createRDataMatrix " + cmdR);
        re.eval(cmdR);
        x = re.eval(name);
        return x;
    }

    public static void createRDataMatrixFromFile(String rObj, String filePath, boolean rowNames, String[] colNames) throws Exception {
        //Create col names vector
        String cNames = "cols <- c(0,";
        int iRow = 0;
        for (; iRow < colNames.length - 1; iRow++) {
            cNames += "'" + (String) colNames[iRow] + "',";
        }
        cNames += "'" + (String) colNames[iRow] + "')";

        //re.eval(cNames);
        evalR(cNames);
        //Create a data frame
        String cmdR = rObj + " <- read.delim('" + filePath + "', header=FALSE, sep='\t', row.names=1, col.names=cols)";
        //re.eval(cmdR);
        evalR(cmdR);
        //Convert to matrix
        //re.eval(rObj + " <- as.matrix(" + rObj + ")");
        evalR(rObj + " <- as.matrix(" + rObj + ")");
    }

    public static void createRDataFrameFromFile(String rObj, String filePath, boolean rowNames, String[] colNames) throws Exception {
        //Create col names vector
        String cNames = "cols <- c(0,";
        int iRow = 0;
        for (; iRow < colNames.length - 1; iRow++) {
            cNames += "'" + (String) colNames[iRow] + "',";
        }
        cNames += "'" + (String) colNames[iRow] + "')";

        //re.eval(cNames);
        evalR(cNames);
        //Create a data frame
        String cmdR = rObj + " <- read.delim('" + filePath + "', header=FALSE, sep='\t', row.names=1, col.names=cols)";
        //re.eval(cmdR);
        evalR(cmdR);
    }
}
