/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.munge;


import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.GeneSetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import xtools.api.AbstractTool;
import xtools.api.ReportBlocks;
import xtools.api.ToolCategory;
import xtools.api.param.*;

import java.io.File;
import java.util.Properties;

/**
 * Tool to extract a dataset by column names
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class XtractDataset extends AbstractTool {

    private final DatasetReqdParam fDatasetParam = new DatasetReqdParam();

    private final BooleanParam fIsColumnNames = new BooleanParam("column", "Extract columns instead of rows", "Specified names are column names (and not the default of row names)", false, false);

    private final StringMultiInputParam fNamesParam = new StringMultiInputParam("names", "Extract these features", "Row or column names from dataset, whitespace or comma delimited", new String[]{}, false);

    private final GeneSetMatrixMultiChooserParam fGeneSetsParam = new GeneSetMatrixMultiChooserParam(false, true, false);

    private final DatasetFormatParam fDatasetOutputFormatParam = new DatasetFormatParam(false);

    private final ModeReqdParam fIncludeModeParam = new ModeReqdParam("mode", "Include or Exclude specified features", "Include or Exclude specified features", new String[]{"include", "exclude"});

    /**
     * Class constructor
     *
     * @param properties
     */
    public XtractDataset(Properties properties) {
        super.init(properties);
    }

    public XtractDataset(String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public XtractDataset() {
        declareParams();
    }

    /**
     * @throws java.lang.Exception
     */
    public void execute() throws Exception {

        startExec(false); // lets make our custom index

        final Dataset fullDs = fDatasetParam.getDataset();

        GeneSet[] gsets;
        if (fNamesParam.isSpecified()) {
            gsets = new GeneSet[]{fNamesParam.getAsGeneSet()};
        } else if (fGeneSetsParam.isSpecified()) {
            gsets = fGeneSetsParam.getGeneSets(true);
        } else {
            throw new IllegalArgumentException("At least one of " + fNamesParam.getName() + " or " + fGeneSetsParam.getName() + " must be specified");
        }

        if (fIncludeModeParam.getStringIndexChoosen() == 1) {
            log.info("EXcluding specified names if applicable");
            GeneSet all;
            if (fIsColumnNames.isTrue()) {
                all = fullDs.getColumnNamesGeneSet();
            } else {
                all = fullDs.getRowNamesGeneSet();
            }

            for (int i = 0; i < gsets.length; i++) {
                int cnt = all.getNumMembers();
                gsets[i] = GeneSetGenerators.subtractGeneSet(all, gsets[i]);
                log.info("Before: " + cnt + " after subtracting: " + gsets[i].getNumMembers());
            }
        } else {
            log.info("INcluding specified names if applicable");
        }

        final Dataset[] dss = new DatasetGenerators().extractDatasets(fullDs, gsets, fIsColumnNames.isTrue());
        final ReportBlocks.SimpleBlockListing list = new ReportBlocks.SimpleBlockListing("The following datasets were extracted", fReport);

        for (int d = 0; d < dss.length; d++) {
            if (dss[d] == null || dss[d].getNumRow() == 0) {
                StringBuffer buf = new StringBuffer("Error: there were no matches to : " + fullDs.getName());
                buf.append(" from gene set ").append(gsets[d].getName());
                buf.append("\n<br>Check to make sure the features spaces are the same for the gene set and dataset");
                list.add(buf.toString());
            } else {
                File file = fReport.savePage(dss[d], fDatasetOutputFormatParam);
                list.add("Extracted data to file", dss[d].getName(), file);
            }
        }

        list.close();
        doneExec();
    }

    public ToolCategory getCategory() {
        return ToolCategory.DATASET;
    }

    public String getDesc() {
        return "Extract expression values for a gene set from a dataset";
    }

    public void declareParams() {
        //fParamSet.addParam(fTemplateParam);
        fParamSet.addParam(fDatasetParam);
        fParamSet.addParamPseudoReqd(fGeneSetsParam);

        fParamSet.addParamBasic(fNamesParam);
        fParamSet.addParamBasic(fDatasetOutputFormatParam);
        fParamSet.addParamBasic(fIsColumnNames);
        fParamSet.addParamBasic(fIncludeModeParam);
    }

    public static void main(String[] args) {
        XtractDataset tool = new XtractDataset(args);
        tool_main(tool);

    }

}    // End XTractDataset


