/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome;

import edu.mit.broad.genome.reports.NullSafeStringBuffer;
import edu.mit.broad.xbench.core.api.Application;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Jan 8, 2004
 *         Time: 12:03:30 PM
 */
public class Errors {

    private static final Logger klog = XLogger.getLogger(Errors.class);

    private List fErrors_as_strings;
    private List fErrors_as_throwables;
    private String fErrorName;

    /**
     * Class constructor
     */
    public Errors() {
        this("ERROR(S)");
    }

    /**
     * Class constructor
     *
     * @param errorName
     */
    public Errors(final String errorName) {
        this.fErrorName = errorName;
        this.fErrors_as_throwables = new ArrayList();
        this.fErrors_as_strings = new ArrayList();
    }

    public String getName() {
        return fErrorName;
    }

    public void add(final String s, final Set fields) {
        NullSafeStringBuffer sbuf = new NullSafeStringBuffer(true);
        sbuf.append(s).append('\t').append(fields, '\t');
        fErrors_as_strings.add(sbuf.toString());
    }

    public void add(final String s) {
        if (fErrors_as_strings.contains(s) == false) {
            fErrors_as_strings.add(s);
        }
    }

    public void add(final Throwable t) {
        fErrors_as_strings.add(getAsString(t));
        if (t != null && !fErrors_as_throwables.contains(t) == false) {
            fErrors_as_throwables.add(t);
        }
    }

    public void add(final String msg, final Throwable t) {
        fErrors_as_strings.add(msg + "\n" + getAsString(t));
        if (t != null && fErrors_as_throwables.contains(t) == false) {
            fErrors_as_throwables.add(t);
        }
    }

    public boolean isEmpty() {
        boolean isEmpty = fErrors_as_strings.isEmpty();

        if (!isEmpty) {
            return isEmpty;
        }

        return fErrors_as_throwables.isEmpty();
    }

    public int getSize() {
        return fErrors_as_strings.size();
    }

    public Throwable[] getErrors() {
        return (Throwable[]) fErrors_as_throwables.toArray(new Throwable[fErrors_as_throwables.size()]);
    }

    public String getErrors(final boolean html) {

        StringBuffer buf = new StringBuffer("There were errors: ").append(fErrorName).append(" #:").append(fErrors_as_strings.size());
        if (html) {
            buf.append("<br>");
        } else {
            buf.append("\n");
        }

        for (int i = 0; i < fErrors_as_strings.size(); i++) {
            buf.append(fErrors_as_strings.get(i).toString());
            if (html) {
                buf.append("<br>");
            } else {
                buf.append("\n");
            }
        }

        return buf.toString();
    }

    public void showMessageIfNotEmpty() {
        if (isEmpty() == false) {
            Application.getWindowManager().showError(this);
        }
    }

    public void warnIfNotEmpty() {
        if (fErrors_as_strings.isEmpty() == false) {
            StringBuffer buf = new StringBuffer("There were errors: ").append(fErrorName).append(" #:").append(fErrors_as_strings.size()).append("\n");
            for (int i = 0; i < fErrors_as_strings.size(); i++) {
                buf.append(fErrors_as_strings.get(i).toString()).append('\n');
            }
            klog.warn(buf);
        }
    }

    public void barfIfNotEmpty() throws Exception {
        barfIfNotEmpty(null);
    }

    public void barfIfNotEmpty(final String errorName) throws Exception {
        String err_msg = _barfIfNotEmpty_message(errorName);
        if (err_msg != null) {
            throw new Exception(err_msg);
        }
    }

    private String _barfIfNotEmpty_message(final String errorName) {
        if (errorName != null) {
            fErrorName = errorName;
        }

        if (fErrors_as_strings.isEmpty() == false) {
            StringBuffer buf = new StringBuffer("There were errors: ").append(fErrorName).append(" #:").append(fErrors_as_strings.size()).append("\n");
            for (int i = 0; i < fErrors_as_strings.size(); i++) {
                buf.append(fErrors_as_strings.get(i).toString()).append('\n');
            }
            return buf.toString();
        } else {
            klog.info("NO error(s) for: " + fErrorName);
            return null;
        }
    }

    public void barfIfNotEmpty_IO() throws IOException {
        barfIfNotEmpty_IO(null);
    }

    public void barfIfNotEmpty_IO(String errorName) throws IOException {
        String err_msg = _barfIfNotEmpty_message(errorName);
        if (err_msg != null) {
            throw new IOException(err_msg);
        }
    }

    public void barfIfNotEmptyRuntime() throws RuntimeException {
        barfIfNotEmptyRuntime(null);
    }

    public void barfIfNotEmptyRuntime(String msg) throws RuntimeException {
        if (fErrors_as_strings.isEmpty() == false) {
            StringBuffer buf = new StringBuffer();
            if (msg != null && msg.length() > 0) {
                buf.append(msg).append('\n');
            }

            buf.append("There were errors: ").append(fErrorName).append(" #:").append(fErrors_as_strings.size()).append("\n");
            for (int i = 0; i < fErrors_as_strings.size(); i++) {
                buf.append(fErrors_as_strings.get(i).toString()).append('\n');
            }
            throw new RuntimeException(buf.toString());
        } else {
            //klog.info("NO error(s) for: " + fErrorName);
        }
    }

    // duplicated code from TraceUtils due to package restraints
    private static String getAsString(Throwable e) {

        if (e == null) {
            return "null exception";
        }

        StringWriter buf = new StringWriter();

        e.printStackTrace(new PrintWriter(buf));

        return buf.toString();
    }

} // End Errors
