/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome;

import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.utils.Strings;
import edu.mit.broad.vdb.meg.Gene;

import java.io.File;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * Class NamingConventions
 * <p/>
 * Rules for object / default etc naming conventions are all captured here.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class NamingConventions {

    private static int kObjectCounter = 1;
    private static int kImageCounter = 1;

    private static final int MAX_FILE_LEN_ALLOWED = 100;


    public static boolean isHTA(String celFileName) {

        celFileName = celFileName.toUpperCase();

        if (celFileName.startsWith("E") || celFileName.startsWith("C")) {
            return false;
        }

        if (celFileName.startsWith("5")) {
            return true;
        }

        throw new IllegalArgumentException("Unlnown cel file name: >" + celFileName + "<");
    }

    public static String getBgedSeriesName(final File seriesDirOrSoftFile) {
        return getBgedSeriesName(seriesDirOrSoftFile.getName());
    }

    public static String getBgedSeriesName(final String seriesDirOrSoftFile) {
        String name = NamingConventions.removeExtension(seriesDirOrSoftFile);
        String seriesId = ParseUtils.string2strings(name, "_", false)[0].toUpperCase();

        if (seriesId.startsWith("GSE") || seriesId.startsWith("BSE") || seriesId.startsWith("E-MEXP")) {
            return seriesId;
        } else {
            throw new IllegalArgumentException("Not a valid series id: " + seriesDirOrSoftFile + " >" + seriesId);
        }

    }

    /**
     * NO decimals
     * If no p or q, then null
     * <p/>
     * 10p11.23 => 10p11
     * 10p11.2-p12 => 10p11
     * 20q11.1-q13.1 => 20q11
     * 20q11.2 => 20q11
     *
     * @param cyto
     * @return
     */
    public static String parseCyto(String cyto) {

        if (isNull(cyto)) {
            return null;
        }

        if ((cyto.indexOf('p') == -1) && (cyto.indexOf('q') == -1)) {
            return null;
        }

        if ((cyto.indexOf("pter") != -1) || (cyto.indexOf("ptel") != -1)) {
            return null;
        }

        if ((cyto.indexOf("qter") != -1) || (cyto.indexOf("qtel") != -1)) {
            return null;
        }

        if (cyto.indexOf("cen") != -1) {
            return null;
        }

        cyto = cyto.toLowerCase();

        if (cyto.startsWith("x")) {
            cyto = "X" + cyto.substring(1, cyto.length());
        }

        if (cyto.startsWith("y")) {
            cyto = "Y" + cyto.substring(1, cyto.length());
        }

        cyto = ParseUtils.string2strings(cyto, "-.; |", false)[0];

        if (cyto.startsWith("chr") == false) {
            return "chr" + cyto;
        } else {
            return cyto;
        }
    }

    public static boolean isInteger(final String s) {
        try {
            Integer.parseInt(s);
            return true;
        } catch (NumberFormatException e) {
            return false;
        }
    }

    public static String getPlateName(String wellName) {
        // 5202764005791175120104-R1.B10 is stil w/o the R
        StringTokenizer tok = new StringTokenizer(wellName, ".-");
        return tok.nextToken();
    }


    public static String splitLongHashName(final String possibleLongNameWithHashes, final String prefixStr) {
        if (possibleLongNameWithHashes == null ||
                possibleLongNameWithHashes.indexOf("#") == -1 ||
                possibleLongNameWithHashes.length() < 80) {
            return possibleLongNameWithHashes;
        }

        StringBuffer buf = new StringBuffer();

        for (int c = 0; c < possibleLongNameWithHashes.length(); c++) {
            char cr = possibleLongNameWithHashes.charAt(c);
            if (cr == '#') {
                buf.append(prefixStr);
            }
            buf.append(cr);
        }

        return buf.toString();
    }

    public static String getPathTruncated(final File dir) {

        File parent = dir.getParentFile();

        if (parent == null) {
            return dir.getPath();
        } else {
            return "..." + File.separatorChar + parent.getName() + File.separator + dir.getName();
        }

    }


    public static boolean isURL(final String path) {
        if (path.toLowerCase().startsWith("http")
                || path.toLowerCase().startsWith("www")
                || path.toLowerCase().startsWith("ftp")
                ) {
            return true;
        } else {
            return false;
        }
    }

    /*
    private static Set kMonths;

    static {
        kMonths = new HashSet();
        kMonths.add("JAN");
        kMonths.add("FEB");
        kMonths.add("MAR");
        kMonths.add("APR");
        kMonths.add("MAY");
        kMonths.add("JUN");
        kMonths.add("JUL");
        kMonths.add("AUG");
        kMonths.add("SEP");
        kMonths.add("OCT");
        kMonths.add("NOV");
        kMonths.add("DEC");
    }
    */

    public static String titleize(String title) {

        if (isNull(title)) {
            return Constants.NULL;
        } else {

            if (title.startsWith("\"")) {
                title = title.substring(1, title.length());
            }

            if (title.endsWith("\"")) {
                title = title.substring(0, title.length() - 1);
            }

            return title;
        }

    }

    public static String symbolize(String symbol) {

        if (symbol == null) {
            return null;
        }

        symbol = symbol.trim();

        if (symbol.equals("---") ||
                // symbol.equalsIgnoreCase(Constants.NA) || // this is a problem as there is a gene symbol called NA !!
                symbol.equalsIgnoreCase(Constants.NULL) ||
                symbol.length() == 0) {

            return null;
        }

        symbol = symbol.toUpperCase(); // @note IMP IMp all sumbols are UC only

        /*
        // Then want to cath errors such as
        // 2-APR and convert them into the correct APR2
        // For the excel bug
        StringTokenizer tok = new StringTokenizer(symbol, "-");
        if (tok.countTokens() == 2) {
            String one = tok.nextToken();
            String two = tok.nextToken();
            try {
                int num = Integer.parseInt(one);
                if (kMonths.contains(two)) {
                    //System.out.println("### morphed: " + symbol + " into: " + two + num);
                    return two + num;
                }

                // do below
            } catch (Throwable t) {
                // keep going below
                //return symbol; // not a
            }
        }
        */

        // Finally if the symbol is toolung, truncate it
        // This happens when affy truncates many eaually likely genes into 1
        // e.g KLRA15 /// KLRA20 ...

        if (symbol.length() > 20) {
            return symbol.substring(0, 20).trim();
        } else {
            return symbol;
        }

    }

    public static String createNiceEnglishDate_for_dirs() {
        //Format formatter = new SimpleDateFormat("MMM_dd_yy");
        //Format formatter = new SimpleDateFormat("MMM_dd");
        // Get today's date
        Date date = new Date();
        Format formatter = new SimpleDateFormat("MMM");
        String s = (formatter.format(date));
        formatter = new SimpleDateFormat("dd");
        s = (s + (formatter.format(date))).toLowerCase();
        //s = s.replace('_', '\s');
        return s;
    }

    public static String nullize(final String s) {
        if (isNull(s)) {
            return null;
        } else {
            return s;
        }
    }

    public static boolean isNull(Gene gene) {

        return gene == null || isNull(gene.getSymbol());

    }

    public static boolean isNull(String s) {
        if (s == null) {
            return true;
        }

        s = s.trim();

        if (s.length() == 0) {
            return true;
        }

        if (s.equalsIgnoreCase(Constants.NA)) {
            return true;
        }

        if (s.equalsIgnoreCase(Constants.NULL)) {
            return true;
        }


        return false;
    }


    public static String parseUniqueFromMetaSetName(final String metasetName) {
        StringTokenizer tok = new StringTokenizer(metasetName, "_u");
        String n = null;
        while (tok.hasMoreTokens()) {
            n = tok.nextToken();
        }

        return "MS" + n;
    }

    public static boolean isGoId(final String s) {
        if (s == null) {
            return false;
        }

        return s.startsWith("GO:");
    }

    public static void enforceMegIdFormat(final String megid) {

        if (megid.startsWith(Headers.MEG) == false && megid.startsWith(Headers.DMEG) == false) {
            throw new IllegalArgumentException("Bad format for megid: " + megid);
        }

        for (int c = 0; c < BAD_PROBE_NAME_CHARS.length; c++) {
            if (megid.indexOf(BAD_PROBE_NAME_CHARS[c]) != -1) {
                throw new IllegalArgumentException("Bad char in megid: " + megid);
            }
        }
    }

    public static void enforceMegIdFormat(final String[] megids) {

        for (int i = 0; i < megids.length; i++) {
            enforceMegIdFormat(megids[i]);
        }

    }

    // keep the list small as we dont want to enforce too many rules
    // mostly to ensure that stringtokenizers dont barf
    private static final char[] BAD_PROBE_NAME_CHARS = new char[]{'?', ',', '\t', '\n', Constants.INTRA_FIELD_DELIM};

    // simple check to make sure there arent wierd characters in probe names
    // which could have strange downstream consequences
    public static boolean isValidProbeName(final String probeName) {
        if (probeName == null) {
            throw new IllegalArgumentException("Parameter probeName cannot be null");
        }

        for (int c = 0; c < BAD_PROBE_NAME_CHARS.length; c++) {
            if (probeName.indexOf(BAD_PROBE_NAME_CHARS[c]) != -1) {
                return false;
            }
        }

        return true;
    }

    private static final String DMEG_ = Headers.DMEG + "_";

    public static boolean isDummyMeg(final String megid) {
        return megid.startsWith(DMEG_);
    }

    public static Set getSetFromDummyMegId(String dmegid) {
        dmegid = dmegid.substring(DMEG_.length());

        StringTokenizer tok = new StringTokenizer(dmegid, Constants.INTRA_FIELD_DELIM_S);
        Set set = new HashSet();
        while (tok.hasMoreElements()) {
            set.add(tok.nextToken());
        }
        return set;
    }

    public static void enforceUnigeneIdFormat(final String unigeneid) {
        boolean valid = isUnigeneIdFormat(unigeneid);
        if (!valid) {
            throw new IllegalArgumentException("Bad format for unigeneid: " + unigeneid);
        }
    }

    public static boolean isUnigeneIdFormat(final String unigeneid) {
        return (unigeneid.startsWith("Hs.") || unigeneid.startsWith("Ms."));
    }

    public static String removeExtension(final String fileName) {

        if (fileName == null) {
            throw new IllegalArgumentException("Param fileName cannot be null");
        }

        if (fileName.indexOf(".") == -1) {
            return fileName;
        }

        if (fileName.indexOf('#') != -1) { // aux names are of the form: foo.cls#classA so removing the ext is deleterious
            return fileName;
        }

        if (fileName.endsWith(".")) { // simply strip off the period
            return fileName.substring(0, fileName.length() - 1);
        }

        // we also dont want to do the following:
        // foo_genes.hgu95av2.grp_mapped_symbols_from_HuGeneFL -> foo_genes.hgu95av2
        // (this would happen if we simply remove the extension)
        final int index = fileName.lastIndexOf('.');

        if (-1 == index) { // no periods
            return fileName;
        } else {
            String just_name = fileName.substring(0, index);
            String ext = fileName.substring(index + 1, fileName.length());
            if (ext.length() <= 4) {
                return just_name; // common heuristic
            }

            StringTokenizer tok = new StringTokenizer(ext, "_");
            tok.nextToken(); // get rid of the "ext"
            StringBuffer extra = new StringBuffer("_");
            while (tok.hasMoreElements()) {
                extra.append(tok.nextToken());
                if (tok.hasMoreElements()) {
                    extra.append("_");
                }
            }

            if (extra.length() != 1) {
                return just_name + extra;
            } else {
                return just_name;
            }

            //return filename.substring(0, index);
        }
    }

    public static String removeExtension(final PersistentObject pob) {
        if (pob == null) {
            throw new IllegalArgumentException("Parameter pob cannot be null");
        }
        return removeExtension(pob.getName());
    }

    public static String generateName(final XChart xchart) {
        return createSafeFileName(xchart.getName()) + "_" + kImageCounter++;
    }

    public static String generateNameForImage(final String title) {
        return createSafeFileName(title) + "_" + kImageCounter++;
    }

    // replaces chars that make file paths barf such as @ and #
    public static File createSafeFile(final File inDir, final String name) {
        return new File(inDir, createSafeFileName(name));
    }

    public static String createSafeFileName(String name) {
        name = name.trim();
        String safename = name.replace('@', '_'); // this does all occurrences
        safename = safename.replace('#', '_');
        safename = safename.replace(' ', '_'); // get rid of whitespace
        safename = safename.replace('%', '_');
        safename = safename.replace('$', '_');
        safename = safename.replace(':', '_'); // IE often doesnt like colons in linked to files
        safename = safename.replace('*', '_');
        safename = safename.replace('\\', '_');
        safename = safename.replace('/', '_');
        safename = safename.replace(Constants.INTRA_FIELD_DELIM, '_');

        if (safename.length() >= MAX_FILE_LEN_ALLOWED) {
            String ext = getExtension(safename);
            safename = safename.substring(0, MAX_FILE_LEN_ALLOWED) + "." + ext; // too many chars make excel (for instance) not recognize the file
        }

        return safename;
    }

    public static boolean isBadCharacter(String name, boolean allowSpaces, boolean allowForwardSlash, boolean allowHyphens) {

        if (name == null) {
            throw new IllegalArgumentException("Param name cannot be null");
        }

        name = name.trim();

        if (!allowSpaces && name.indexOf(" ") != -1) {
            return true;
        }

        if (name.indexOf('@') != -1) {
            return true;
        }
        if (name.indexOf('#') != -1) {
            return true;
        }

        if (!allowHyphens && name.indexOf('-') != -1) {
            return true;
        }

        if (name.indexOf('%') != -1) {
            return true;
        }
        if (name.indexOf('$') != -1) {
            return true;
        }
        if (name.indexOf(':') != -1) {
            return true;
        }
        if (name.indexOf('*') != -1) {
            return true;
        }
        if (name.indexOf('\\') != -1) {
            return true;
        }

        if (!allowForwardSlash && name.indexOf('/') != -1) {
            return true;
        }

        // Ok, passed all
        return false;
    }

    public static Object[] parseReportTimestampFromName(final String rptname) {

        StringTokenizer tok = new StringTokenizer(rptname, ".");
        if (tok.countTokens() != 4) {
            throw new IllegalArgumentException("Invalid rpt name format: " + rptname + " got tokens #: " + tok.countTokens());
        }
        StringBuffer buf = new StringBuffer();

        buf.append(tok.nextToken()).append('.');
        buf.append(tok.nextToken());

        return new Object[]{buf.toString(), new Long(Long.parseLong(tok.nextToken()))};
    }


    /**
     * Convention:
     * <p/>
     * orig_name.short_desc_tag.ts.ext
     * <p/>
     * short_desc_tag can be null
     *
     * @param short_desc_tag
     * @param pob
     * @return
     */
    public static String generateName(final PersistentObject pob, final String short_desc_tag, final boolean ultraUniquenessSafe) {

        if (pob == null) {
            return _generateName(null, short_desc_tag, null);
        } else {
            String ext = (DataFormat.getExtension(pob));
            return _generateName(pob.getName(), short_desc_tag, ext, ultraUniquenessSafe);
        }
    }

    public static String generateName(final Dataset ds, final Template t, final boolean stripExtensions) {

        StringBuffer buf = new StringBuffer();

        if (stripExtensions) {
            buf.append(removeExtension(ds.getName())).append(".");
            buf.append(removeExtension(t.getName()));
        } else {
            buf.append(ds.getName()).append('.');
            buf.append(t.getName());
        }

        return buf.toString();
    }

    public static String generateName(final Template t, final GeneSet gset, boolean stripExtensions) {

        StringBuffer buf = new StringBuffer();

        if (stripExtensions) {
            buf.append(removeExtension(t.getName())).append(".");
            buf.append(removeExtension(gset.getName()));
        } else {
            buf.append(t.getName()).append('.');
            buf.append(gset.getName());
        }

        return buf.toString();
    }

    public static String generateName(final Dataset ds, final GeneSet gset, final boolean stripExtensions) {

        StringBuffer buf = new StringBuffer();

        if (stripExtensions) {
            buf.append(removeExtension(ds.getName())).append(".");
            buf.append(removeExtension(gset.getName()));
        } else {
            buf.append(ds.getName()).append('.');
            buf.append(gset.getName());
        }

        return buf.toString();
    }

    public static String generateName(final Dataset ds, final Template t, final GeneSet gset) {
        StringBuffer buf = new StringBuffer();
        buf.append(removeExtension(ds.getName())).append('.');
        buf.append(removeExtension(t.getName())).append('.');
        buf.append(removeExtension(gset.getName()));
        return buf.toString();
    }

    public static String generateName(final RankedList rl, final Template t, final GeneSet gset) {
        StringBuffer buf = new StringBuffer();
        buf.append(removeExtension(rl.getName())).append('.');
        if (t != null) {
            buf.append(removeExtension(t.getName())).append('.');
        }
        buf.append(removeExtension(gset.getName()));
        return buf.toString();
    }

    public static String generateName(final Dataset ds, final Template t, final boolean stripExtensions, final DataFormat df) {

        StringBuffer buf = new StringBuffer();

        if (stripExtensions) {
            buf.append(removeExtension(ds.getName())).append(".");
            buf.append(removeExtension(t.getName()));
        } else {
            buf.append(ds.getName()).append('.');
            buf.append(t.getName());
        }

        buf.append('.').append(df.getExtension());

        return buf.toString();
    }

    public static String generateName(final Dataset ds, final Template.Class tc, final boolean stripExtensions) {

        StringBuffer buf = new StringBuffer();

        if (stripExtensions) {
            buf.append(removeExtension(ds.getName())).append(".");
            buf.append(removeExtension(tc.getName()));
        } else {
            buf.append(ds.getName()).append('.');
            buf.append(tc.getName());
        }

        return buf.toString();
    }

    public static String generateName(final PersistentObject pob, final String short_desc_tag) {
        return generateName(pob, short_desc_tag, false);
    }

    public static String generateName(final PersistentObject pob) {
        return generateName(pob, null, false);
    }

    public static String generateName(final PersistentObject pob, final boolean ultraUniquenessSafe) {
        return generateName(pob, null, ultraUniquenessSafe);
    }

    //-------------------------------------------------------------------//
    public static String generateLocName(final int loCol, final Dataset parentDs) {
        return new StringBuffer(removeExtension(parentDs.getName())).append('.').append("tst").append(loCol).toString();
    }

    public static String generateNonLocName(final int loCol, final Dataset parentDs) {
        return new StringBuffer(removeExtension(parentDs.getName())).append('.').append("trn").append(loCol).toString();
    }
    //-------------------------------------------------------------------//

    //-------------------------------------------------------------------//

    public static String generateLocName(final int loCol, final Template t) {
        return new StringBuffer(removeExtension(t.getName())).append("#tst").append(loCol).toString();
    }

    public static String generateNonLocName(final int loCol, final Template t) {
        return new StringBuffer(removeExtension(t.getName())).append("#trn").append(loCol).toString();
    }
    //-------------------------------------------------------------------//

    //-------------------------------------------------------------------//

    public static String generateLocName(final int[] locs, final Template t) {
        String s = Printf.format(locs, '_');
        StringBuffer buf = new StringBuffer(removeExtension(t.getName())).append("#tst").append(s);
        return buf.toString();
    }

    public static String generateNonLocName(final int[] locs, final Template t) {
        String s = Printf.format(locs, '_');
        StringBuffer buf = new StringBuffer(removeExtension(t.getName())).append("#trn").append(s);
        return buf.toString();
    }
    //-------------------------------------------------------------------//

    //-------------------------------------------------------------------//

    public static String generateLocName(final int loCol, final Dataset parentDs, final Template t) {
        StringBuffer n = new StringBuffer(generateName(parentDs, t, true));
        return n.append('.').append("tst").append(loCol).toString();
    }

    public static String generateNonLocName(final int loCol, final Dataset parentDs, final Template t) {
        StringBuffer n = new StringBuffer(generateName(parentDs, t, true));
        return n.append('.').append("trn").append(loCol).toString();
    }
    //-------------------------------------------------------------------//

    public static String generateName(final String ext) {
        return generateName(ext, false);
    }

    public static String generateName(final String ext, final boolean ultraUniquenessSafe) {

        StringBuffer buf;

        if (ultraUniquenessSafe) {
            buf = new StringBuffer(generateUniqueNameTag());
        } else {
            buf = new StringBuffer(generateCounterNameTag());
        }

        buf.append('.').append(ext);

        return buf.toString();
    }

    public static String generateName(final boolean ultraUniquenessSafe) {
        if (ultraUniquenessSafe) {
            return generateUniqueNameTag();
        } else {
            return generateCounterNameTag();
        }
    }

    /**
     * format:
     * Pobclass.ext
     * <p/>
     * uuid for uniqueness
     *
     * @param pobClass
     * @return
     */
    public static String generateName(final Class pobClass, final boolean ultraUniquenessSafe) {

        String name = DataFormat.getName(pobClass);

        if (name == null) {
            name = "unknown." + pobClass;
        }

        String ext = DataFormat.getExtension(pobClass);

        return _generateName(name, null, ext, ultraUniquenessSafe);
    }

    public static String generateName(Class pobClass) {
        return generateName(pobClass, false);
    }

    public static String generateName(final Class pobClass, final String short_desc_tag, final boolean ultraUniquenessSafe) {

        String name = DataFormat.getName(pobClass);

        if (name == null) {
            name = "unknown." + pobClass;
        }

        String ext = DataFormat.getExtension(pobClass);

        return _generateName(name, short_desc_tag, ext, ultraUniquenessSafe);
    }

    public static String generateName(Class pobClass, String short_desc_tag) {
        return generateName(pobClass, short_desc_tag, false);
    }

    private static String _generateName(final String origname, final String short_desc_tag, final String ext) {
        return _generateName(origname, short_desc_tag, ext, false);
    }

    // orig_name.short_desc_tag.ts_or_counter.ext
    private static String _generateName(final String origname, final String short_desc_tag, final String ext, final boolean ultraUniquenessSafe) {

        StringBuffer buf = new StringBuffer();

        if (origname != null) {
            buf.append(removeExtension(origname));
            buf.append('.');
        }

        if (short_desc_tag != null) {
            Strings.replace(short_desc_tag, " ", "");
            Strings.replace(short_desc_tag, "\t", "");
            buf.append(short_desc_tag);
            buf.append('.');
        }

        if (ultraUniquenessSafe) {
            buf.append(generateUniqueNameTag());
        } else {
            buf.append(generateCounterNameTag());
        }

        if (ext != null) {
            buf.append('.');
            buf.append(ext);
        }

        return buf.toString();
    }

    private static synchronized String generateUniqueNameTag() {
        return new StringBuffer().append(System.currentTimeMillis()).toString();
    }

    // use me if you decide decided against System.currentTimeMillis() as it
    // makes the names very long and uniintuitive
    // as to who came first
    public static synchronized String generateCounterNameTag() {
        return new StringBuffer().append(kObjectCounter++).toString();
    }

    /**
     * Gets the extension of a specified file name. The extension is
     * everything after the last dot.
     * <p/>
     * <pre>
     * foo.txt    --> "txt"
     * a\b\c.jpg  --> "jpg"
     * foo        --> ""
     * </pre>
     *
     * @param f String representing the file name
     * @return extension of the file (or <code>""</code> if it had none)
     */
    public static String getExtension(final String f) {

        String ext;
        int pos;

        pos = f.lastIndexOf(".");

        if (pos == -1) {
            //log.warn("No extension found - returning \"\" file=" + f);
            ext = "";
        } else {
            ext = f.substring(pos + 1);
        }

        //og.debug("got: " + f + " foind: " + ext);

        if (f.indexOf(".meta.") != -1) {
            return "meta." + ext;
        } else {
            return ext;
        }
    }

    /**
     * Gets the extension of a specified file. The extension is
     * everything after the last dot.
     * <p/>
     * <pre>
     * foo.txt    --> "txt"
     * a\b\c.jpg  --> "jpg"
     * foo        --> ""
     * </pre>
     *
     * @param f the File
     * @return extension of the file (or <code>""</code> if it had none)
     * @see getExtension(String f)
     */
    public static String getExtension(File f) {
        return getExtension(f.getName());
    }

}    // End NamingConventions
