/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome;


import com.braju.format.Format;
import com.braju.format.Parameters;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.utils.ColorUtils;
import gnu.trove.*;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;


/**
 * Console or log printing utilities for debugging.
 * <p/>
 * Not very oopy, but experience has proven that its better to collect these
 * into a single class then embed printf() functionality into objects.
 * <p/>
 * Also:
 * <p/>
 * /**
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @Version: 0.1 of August 11, 1997
 * @Author: Elliotte Rusty Harold (elharo@sunsite.unc.edu)
 * @see http://www.ibiblio.org/javafaq/formatter/
 *      <p/>
 *      Formatter
 *      <p/>
 *      This class provides C-like formatting functions that allow
 *      programmers to convert an integer or floating point number
 *      into a string with a specified, width, precision and format.
 *      For instance this might be used to format monetary data to
 *      two decimal places.<P>
 *      <p/>
 *      Because Java does not have variable length argument lists like C,
 *      a different strategy must be employed. Each number is passed
 *      to a format() method along with formatting instructions. The format()
 *      method returns a formatted string which may then be passed to
 *      System.out.println() or other methods. In short this is more
 *      similar to C's sprintf() than to printf(). <P>
 *      <p/>
 *      There are a number of possible things one can do when a number
 *      will not fit inside the specified format. You can throw an exception,
 *      truncate the number, return an error string, or expand the width.
 *      Here I've chosen to expand the width. <P>
 *      <p/>
 *      The rounding of these precisions still needs work. Currently
 *      excess digits are merely truncated.<P>
 */

public class Printf {

    private static final Logger klog = XLogger.getLogger(Printf.class);

    private static final int DEFAULT_PRECISION = 4;
    private static final String DEFAULT_PRECISION_STR = "%.4f";
    private static final char DEFAULT_DELIM = ',';

    /**
     * Privatized class constructor to prevent instantiation.
     */
    private Printf() {
    }


    /**
     * Methods that format the full thing -- not just a snippet
     */
    public static class Full {

        public static String toString(final Set set) {
            final StringBuffer buf = new StringBuffer();
            for (Iterator it = set.iterator(); it.hasNext();) {
                Object key = it.next();
                if (key != null) {
                    buf.append(key);
                }

                if (it.hasNext()) {
                    buf.append(Constants.INTRA_FIELD_DELIM);
                }

            }

            return buf.toString();
        }

        public static String format4(float coph) {
            return _format_float(Float.toString(coph), 4);
        }

        public static String format6(float coph) {
            return _format_float(Float.toString(coph), 6);
        }

        public static String format5(float coph) {
            return _format_float(Float.toString(coph), 5);
        }

        public static String format5(double coph) {
            return _format_float(Double.toString(coph), 5);
        }

        public static String format(float fl, int maxNumChars) {
            return _format_float(Float.toString(fl), maxNumChars);
        }

        private static String _format_float(String flt, int maxChars) {

            int index = flt.indexOf("E");
            if (index != -1) {
                return flt; // dangerous to truncate has the E thing
            }

            if (flt.length() > maxChars) {
                flt = flt.substring(0, maxChars);
            }

            return flt;
        }

        public static String formatPValue(float pvalue) {

            if (pvalue == 0) {
                return "< 0.001"; // @todo hardcoded limit
            }

            return format(pvalue, 5);
        }

        public static String format(LabelledVector lv, Struc struc, boolean replace_with_space) {

            String[] ss = new String[lv.getSize()];
            for (int r = 0; r < lv.getSize(); r++) {
                String name = AuxUtils.getAuxNameOnlyNoHash(lv.getLabel(r));
                name = succintize(name, struc);
                if (replace_with_space) {
                    name = name.replace('_', ' ');
                }
                ss[r] = name + " " + format(lv.getScore(r), 8);
            }

            return formatSuccint(ss, struc, false); // as already succinitized
        }

        public static String format(LabelledObject lo, Struc struc, boolean replace_with_space) {

            String[] ss = new String[lo.getSize()];
            for (int r = 0; r < lo.getSize(); r++) {
                String name = AuxUtils.getAuxNameOnlyNoHash(lo.getLabel(r));
                name = succintize(name, struc);
                if (replace_with_space) {
                    name = name.replace('_', ' ');
                }

                HyperGeometricStruc hyp = (HyperGeometricStruc) lo.getElement(r);

                ss[r] = name + " " + format((float) hyp.getPValue(), 8);
            }

            return formatSuccint(ss, struc, false); // as already succinitized
        }

        // maxnumwords -> max num words per row
        public static String formatSuccint(String[] ss, Struc struc) {
            return formatSuccint(ss, struc, true);
        }

        private static String formatSuccint(String[] ss, Struc struc, boolean succinitize) {
            int mr = struc.maxNumRows;
            if (struc.maxNumRows > ss.length) {
                mr = ss.length;
            }

            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < mr; i++) {
                if (NamingConventions.isNull(ss[i])) {
                    // dont add anything
                    //buf.append("-");
                } else {
                    if (succinitize) {
                        String s = succintize(ss[i], struc);
                        buf.append(s);
                    } else {
                        buf.append(ss[i]);
                    }
                }
                // if (i != ss.length - 1) {
                if (struc.html) {
                    buf.append("<br>");
                } else {
                    buf.append(Constants.INTRA_FIELD_DELIM);
                }
                //}
            }

            return buf.toString();
        }

        // the core method
        public static String format(String[] ss, Struc struc) {
            int maxNumRows = struc.maxNumRows;
            if (maxNumRows > ss.length) {
                maxNumRows = ss.length;
            }

            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < maxNumRows; i++) {
                if (NamingConventions.isNull(ss[i])) {
                    // dont add anything
                    buf.append("-");
                } else {
                    buf.append(succintize(ss[i], struc));
                }
                // if (i != ss.length - 1) {
                if (struc.html) {
                    buf.append("<br>");
                } else {
                    buf.append(Constants.INTRA_FIELD_DELIM);
                }
                //}
            }

            return buf.toString();
        }

        public static String succintize(String s, Struc struc) {
            StringTokenizer tok = new StringTokenizer(s, " ,()-");
            if (tok.countTokens() < struc.maxWordsPerLine || s.length() < struc.showFullIfLessCharsThan) {
                return s; // no need to succinitize, show the full thing
            }
            StringBuffer buf = new StringBuffer();

            int cnt = 0;
            while (tok.hasMoreTokens()) {
                buf.append(tok.nextToken());
                if (cnt > struc.maxWordsPerLine) {
                    break;
                } else {
                    cnt++;
                    buf.append(" ");
                }
            }

            if (buf.length() > struc.maxCharsAllowedPerLine) {
                return buf.substring(0, struc.maxCharsAllowedPerLine);
            } else {
                return buf.toString();
            }
        }

    } // End class Full


    public static class Struc {

        int maxCharsAllowedPerLine;
        int maxWordsPerLine;
        int maxNumRows;
        int showFullIfLessCharsThan;
        boolean html;

        public Struc(int maxWordsPerLine, int showFullIfLessCharsThan, int maxCharsAllowedPerLine, int maxNumRows, boolean html) {
            this.maxWordsPerLine = maxWordsPerLine;
            this.showFullIfLessCharsThan = showFullIfLessCharsThan;
            this.maxCharsAllowedPerLine = maxCharsAllowedPerLine;
            this.maxNumRows = maxNumRows;
            this.html = html;
        }

    }

    //-----------------------------------------------------------------------//

    /*
    public static void outl(final TFloatArrayList list) {
        log.info(outs(list));
    }

    public static String outs(final TFloatArrayList list) {

        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < list.size(); i++) {
            buf.append(list.get(i))
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }

        return buf.toString();
    }
    */

    // ---------------------------------------------------------- //

    public static void outl(final TObjectFloatHashMap map) {
        klog.info(outs(map));
    }

    public static String outs(final TObjectFloatHashMap map) {

        TObjectFloatIterator iterator = map.iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }

        return buf.toString();
    }

    public static void outl(final TObjectIntHashMap map) {
        klog.info(outs(map));
    }


    public static String outs(final TIntObjectHashMap map) {

        if (map == null) {
            return "null TIntObjectHashMap";
        } else if (map.isEmpty()) {
            return "empty TIntObjectHashMap";
        }

        TIntObjectIterator iterator = map.iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }

        return buf.toString();
    }

    public static void outl(final THashMap map) {
        klog.info(outs(map));
    }

    public static String outs(final THashMap map) {

        if (map == null) {
            return "null TIntObjectHashMap";
        } else if (map.isEmpty()) {
            return "empty TIntObjectHashMap";
        }

        Iterator iterator = map.keySet().iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.next();
            Object obj = iterator.next();
            buf.append(obj).append('\t').append(map.get(obj)).append('\n');
        }

        return buf.toString();
    }

    //-----------------------------------------------------------------------//

    // ---------------------------------------------------------- //

    public static void outl(final TFloatHashSet set) {
        klog.info(outs(set));
    }

    public static String outs(final TFloatHashSet set) {

        if (set == null) {
            return "null TFloatHashSet";
        } else if (set.isEmpty()) {
            return "empty TFloatHashSet";
        }

        TFloatIterator iterator = set.iterator();

        StringBuffer buf = new StringBuffer();
        buf.append("size: ").append(set.size()).append('\n');

        int cnt = 0;
        for (int i = set.size(); i-- > 0;) {
            float obj = iterator.next();
            buf.append(obj).append(',');
            cnt++;

            if (cnt > 10) {
                buf.append("\nand so on ...");
            }
        }

        return buf.toString();
    }

    //-----------------------------------------------------------------------//
    public static void outl(final Dataset ds) {
        klog.debug(outs(ds));
    }

    public static void out(final Dataset ds) {
        System.out.println(outs(ds).toString());
    }

    // nrows, ncols, colnames, max value, min value, chip(tentative), comment assoc with res file
    public static String info(final Dataset ds) {

        StringBuffer buf = new StringBuffer("Name: ").append(ds.getName()).append('\n');
        buf.append("Num Rows: ").append(ds.getNumRow()).append('\n');
        buf.append("Num Cols: ").append(ds.getNumCol()).append('\n');

        // dont expensive for many datasets
        //buf.append("Max Value: ").append(ds.getM)).append('\n');
        String comm = ds.getComment();
        if ((comm != null) && (comm.length() > 0)) {
            buf.append("\nComments\n");
            buf.append(ds.getComment());
        }


        return buf.toString();

    }

    public static StringBuffer outs(final Dataset ds) {

        if (ds == null) {
            return new StringBuffer("Null Dataset");
        }

        StringBuffer buf = new StringBuffer("Name: " + ds.getName() + " Id: "
                + ds.getId()).append('\n');

        buf.append("Num rows = ").append(ds.getNumRow()).append('\n');
        buf.append("Num cols = ").append(ds.getNumCol()).append('\n');

        int max = 25;

        if (max > ds.getNumRow()) {
            max = ds.getNumRow();
        }

        for (int i = 0; i < max; i++) {
            buf.append(ds.getRowName(i)).append('\t').append(ds.getRow(i)).append('\n');
        }

        buf.append("and so on ...\n");

        return buf;
    }

    public static void outl(final Dataset[] dss) {
        klog.debug(outs(dss).toString());
    }

    public static void out(final Dataset[] dss) {
        System.out.println(outs(dss).toString());
    }

    public static StringBuffer outs(final Dataset[] dss) {

        StringBuffer buf = new StringBuffer();

        if (dss == null) {
            return new StringBuffer("null datasets");
        }

        buf.append("Number of datasets = ").append(dss.length).append('\n');

        for (int i = 0; i < dss.length; i++) {
            buf.append(outs(dss[i])).append('\n');
        }

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void outl(final FeatureAnnot ann) {
        klog.debug(outs(ann));
    }

    public static void out(final FeatureAnnot ann) {
        System.out.println(outs(ann).toString());
    }

    public static StringBuffer outs(final FeatureAnnot ann) {

        if (ann == null) {
            return new StringBuffer("Null Ann");
        }

        StringBuffer buf = new StringBuffer("Name: " + ann.getName() + " Id: "
                + ann.getId()).append('\n');

        buf.append("Num rows = ").append(ann.getNumFeatures()).append('\n');

        buf.append('\n');

        /*
        int max = 25;

        if (max > ann.getNumFeatures()) {
            max = ann.getNumFeatures();
        }
        */

        buf.append("and so on ...\n");

        return buf;
    }

    public static void outl(final FeatureAnnot[] anns) {
        klog.debug(outs(anns).toString());
    }

    public static void out(final FeatureAnnot[] anns) {
        System.out.println(outs(anns).toString());
    }

    public static StringBuffer outs(final FeatureAnnot[] anns) {

        StringBuffer buf = new StringBuffer();

        if (anns == null) {
            return new StringBuffer("null datasets annotations");
        }

        buf.append("Number of datasets = ").append(anns.length).append('\n');

        for (int i = 0; i < anns.length; i++) {
            buf.append(outs(anns[i])).append('\n');
        }

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void outl(final Range[] ranges) {
        klog.info(outs(ranges));
    }

    public static void out(final Range[] ranges) {
        System.out.println(outs(ranges));
    }

    public static StringBuffer outs(final Range[] ranges) {

        if (ranges == null) {
            return new StringBuffer("Null ranges array");
        }

        StringBuffer buf = new StringBuffer("Ranges: " + ranges.length).append('\n');

        for (int i = 0; i < ranges.length; i++) {
            buf.append(ranges[i].getName()).append('\n');
        }

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final JTable table) {
        System.out.println(((DefaultTableModel) table.getModel()).getDataVector());
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final Vector v) {

        if (v == null) {
            System.out.println("Null vector");
        } else {
            System.out.println(v.toString());
        }
    }

    public static void outl(final Vector v) {

        if (v == null) {
            klog.info("Null vector");
        } else {
            klog.info(v.toString());
        }
    }

    public static String format(final Vector v) {
        return format(v, DEFAULT_PRECISION, DEFAULT_DELIM);
    }

    public static String format(final Vector v, final int precision) {
        return format(v, precision, DEFAULT_DELIM);
    }

    public static String format(final Vector v, final char delim) {
        return format(v, DEFAULT_PRECISION, delim);
    }

    public static String format(final Vector v, final int precision, final char delim) {

        if (v == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < v.getSize(); i++) {
            buf.append(format(v.getElement(i), precision));
            if (i != v.getSize() - 1) {
                buf.append(delim);
            }
        }

        return buf.toString();
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final Vector[] vs) {
        System.out.println(outs(vs));
    }

    public static void outl(final Vector[] vs) {
        klog.info(outs(vs));
    }

    public static StringBuffer outs(final Vector[] vs) {

        if (null == vs) {
            return new StringBuffer("null array");
        }

        StringBuffer buf = new StringBuffer("Number of vectors = " + vs.length).append('\n');

        for (int i = 0; i < vs.length; i++) {
            buf.append(vs[i].toString()).append('\n');
        }

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final AddressedVector iv) {
        System.out.println(outs(iv));
    }

    public static void outl(final AddressedVector iv) {
        klog.info(outs(iv));
    }

    public static String outs(final AddressedVector iv) {
        if (iv == null) {
            return "null AddressedVector";
        } else {
            StringBuffer buf = new StringBuffer();
            buf.append("size: ").append(iv.getSize()).append('\n');
            int max = 100;
            if (max > iv.getSize()) {
                max = iv.getSize();
            }
            for (int i = 0; i < max; i++) {
                buf.append(iv.getAddress(i)).append(',').append(iv.getScore(i)).append('\t');
            }

            return buf.toString();
        }

    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final float[] arr) {
        System.out.println(outs(arr));
    }

    public static void outl(final float[] arr) {
        klog.info(outs(arr));
    }

    public static String outs(final float[] arr) {

        if (arr == null) {
            return "Null array";
        }

        StringBuffer buf = new StringBuffer("array length: " + arr.length).append('\n');
        int max = 20;

        if (max > arr.length) {
            max = arr.length;
        }

        for (int i = 0; i < max; i++) {
            buf.append(arr[i]).append(' ');
        }

        buf.append("\nand so on ...\n");

        return buf.toString();
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final double[] arr) {
        System.out.println(outs(arr));
    }

    public static void outl(final double[] arr) {
        klog.info(outs(arr));
    }

    public static String outs(final double[] arr) {

        if (arr == null) {
            return "Null array";
        }

        StringBuffer buf = new StringBuffer("array length: " + arr.length).append('\n');
        int max = 20;

        if (max > arr.length) {
            max = arr.length;
        }

        for (int i = 0; i < max; i++) {
            buf.append(arr[i]).append(' ');
        }

        buf.append("\nand so on ...\n");

        return buf.toString();
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final int[] arr) {
        System.out.println(outs(arr));
    }

    public static void outl(final int[] arr) {
        klog.info(outs(arr));
    }

    public static String outs(final int[] arr) {

        if (arr == null) {
            return "Null array";
        }

        StringBuffer buf = new StringBuffer("array length: " + arr.length).append('\n');
        int max = 200;

        if (arr.length < max) {
            max = arr.length;
        }

        for (int i = 0; i < max; i++) {
            buf.append(arr[i]).append(' ');
        }

        buf.append("\nand so on ...\n");

        return buf.toString();
    }

    public static String format(int[] arr) {
        return format(arr, DEFAULT_DELIM);
    }

    public static String format(int[] arr, char delim) {

        if (arr == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < arr.length; i++) {
            buf.append(arr[i]);
            if (i != arr.length - 1) {
                buf.append(delim);
            }
        }

        return buf.toString();
    }

    public static String format(final float[] f, final int precision, final char delim) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < f.length; i++) {
            buf.append(format(f[i], precision));
            if (i != f.length - 1) {
                buf.append(delim);
            }
        }
        return buf.toString();
    }

    public static String format(final float[] f, final int precision) {
        return format(f, precision, DEFAULT_DELIM);
    }

    public static String format(final float[] f) {
        return format(f, DEFAULT_PRECISION, DEFAULT_DELIM);
    }

    public static String format(final float f, final int precision) {
        String s = Format.sprintf(new StringBuffer("%.").append(precision).append('f').toString(), new Parameters(f));
        if (s.endsWith(".")) {
            return s.substring(0, s.length() - 1);
        } else {
            return s;
        }
    }

    public static String format(final double f, final int precision) {
        String s = Format.sprintf(new StringBuffer("%.").append(precision).append('f').toString(), new Parameters(f));
        if (s.endsWith(".")) {
            return s.substring(0, s.length() - 1);
        } else {
            return s;
        }
    }

    public static String format(final float f) {
        return Format.sprintf(DEFAULT_PRECISION_STR, new Parameters(f));
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final Object[] arr) {
        System.out.println(outs(arr));
    }

    // truncates if too long
    public static void outl(final Object[] arr) {

        if (null == arr) {
            klog.debug("null array");

            return;
        }

        if (arr.length == 0) {
            klog.debug("empty array");
            return;
        }

        StringBuffer buf = new StringBuffer();
        buf.append("# of elements = ").append(arr.length).append('\n');

        for (int i = 0; i < arr.length; i++) {
            buf.append(arr[i]).append(' ');

            if (i > 10) {
                buf.append("\nand so on ...");

                break;
            }
        }

        klog.debug(buf.toString());
    }

    // no truncation
    public static String outs(final Object[] arr) {

        if (null == arr) {
            return "null array";
        }

        if (arr.length == 0) {
            return ("empty array");
        }

        StringBuffer buf = new StringBuffer();
        buf.append("# of elements = ").append(arr.length).append('\n');

        for (int i = 0; i < arr.length; i++) {
            buf.append(arr[i]).append(' ');
        }

        return buf.toString();
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(final ListModel model) {
        System.out.println(outs(model));
    }

    public static void outl(final ListModel model) {
        klog.debug(outs(model));
    }

    public static String outs(final ListModel model) {

        if (null == model) {
            return "null array";
        }

        if (model.getSize() == 0) {
            return ("empty model");
        }

        StringBuffer buf = new StringBuffer();
        buf.append("# of elements = ").append(model.getSize()).append('\n');

        for (int i = 0; i < model.getSize(); i++) {
            buf.append(model.getElementAt(i)).append(" class: ").append(model.getElementAt(i).getClass()).append('\n');
        }

        return buf.toString();
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void outl(final DataFlavor[] flavors) {
        klog.debug(outs(flavors).toString());
    }

    public static void out(final DataFlavor[] flavors) {
        System.out.println(outs(flavors).toString());
    }

    public static StringBuffer outs(final DataFlavor[] flavors) {

        if (flavors == null) {
            return new StringBuffer("null flavor array");
        }

        StringBuffer buf = new StringBuffer("Flavors are\n");

        for (int i = 0, n = flavors.length; (i < n) && (i < 5); ++i) {
            if (flavors[i] == null) {
                buf.append("null flavor at: ").append(i).append("\n");
            } else {
                buf.append("[").append(i).append("] ").append(flavors[i]).append('\n');
            }
        }

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//

    /**
     * @maint Must be updated if a Print method is added for a persistent object
     */
    public static void outl(final PersistentObject pob) {
        klog.debug(outs(pob).toString());
    }

    public static void out(PersistentObject pob) {
        System.out.println(outs(pob).toString());
    }

    public static StringBuffer outs(PersistentObject pob) {

        StringBuffer buf;

        if (pob == null) {
            return new StringBuffer("Null Pob.Class");
        }

        buf = new StringBuffer("-------------------------------\n");

        buf.append("Id=").append(pob.getId()).append("\n");
        buf.append("Name=").append(pob.getName()).append("\n");
        buf.append("--------------------------------");

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//

    public static void outl(final RankedList rl) {
        klog.debug(outs(rl).toString());
    }

    public static void out(final RankedList rl) {
        System.out.println(outs(rl).toString());
    }

    public static StringBuffer outs(final RankedList lv) {

        StringBuffer buf;

        if (lv == null) {
            return new StringBuffer("Null RankedList");
        }

        buf = new StringBuffer("-------------------------------\n");

        buf.append("Name=").append(lv.getName()).append("\n");
        buf.append("Size=").append(lv.getSize()).append("\n");
        for (int r = 0; r < lv.getSize(); r++) {
            buf.append(lv.getRankName(r)).append('\t').append(lv.getScore(r)).append('\n');
            if (r > 10) {
                buf.append('\n').append(" and so on ...").append('\n');
                break;
            }
        }
        buf.append("--------------------------------");

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//

    public static void outl(final LabelledVector lv) {
        klog.debug(outs(lv).toString());
    }

    public static void out(final LabelledVector lv) {
        System.out.println(outs(lv).toString());
    }

    public static StringBuffer outs(final LabelledVector lv) {

        StringBuffer buf;

        if (lv == null) {
            return new StringBuffer("Null LabelledVector");
        }

        buf = new StringBuffer("-------------------------------\n");

        buf.append("Label=").append(lv.getName()).append("\n");
        buf.append("Size=").append(lv.getSize()).append("\n");
        for (int r = 0; r < lv.getSize(); r++) {
            buf.append(lv.getLabel(r)).append('\t').append(lv.getScore(r)).append('\n');
            if (r > 10) {
                buf.append('\n').append(" and so on ...").append('\n');
                break;
            }
        }
        buf.append("--------------------------------");

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//

    public static void outl(final GeneSet gset) {
        klog.debug(outs(gset).toString());
    }

    public static void out(final GeneSet gset) {
        System.out.println(outs(gset).toString());
    }

    public static StringBuffer outs(GeneSet gset) {

        StringBuffer buf = new StringBuffer();

        if (gset == null) {
            return new StringBuffer("Null GeneSet.Class");
        }

        buf.append("Name=").append(gset.getName()).append("\t# members: ").append(gset.getNumMembers()).append("\n");

        int max = 100;
        if (gset.getNumMembers() < max) {
            max = gset.getNumMembers();
        }

        for (int i = 0; i < max; i++) {
            buf.append(gset.getMember(i)).append(' ');
        }

        buf.append('\n');

        return buf;
    }

    public static void outl(final GeneSet[] gsets) {
        klog.debug(outs(gsets).toString());
    }

    public static void out(final GeneSet[] gsets) {
        System.out.println(outs(gsets).toString());
    }

    public static StringBuffer outs(final GeneSet[] gsets) {
        StringBuffer buf = new StringBuffer();
        for (int f = 0; f < gsets.length; f++) {
            buf.append(outs(gsets[f]));
        }

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void outl(final Template template) {
        klog.debug(outs(template).toString());
    }

    public static void out(final Template template) {
        System.out.println(outs(template).toString());
    }

    public static StringBuffer outs(final Template template) {

        StringBuffer buf = new StringBuffer();

        if (template == null) {
            return new StringBuffer("null template");
        }

        buf.append("Template id: ").append(template.getId()).append(" name: ").append(template.getName()).append('\n');
        buf.append("Number of classes: ").append(template.getNumClasses()).append('\n');
        buf.append("Classes:\n");

        for (int i = 0; i < template.getNumClasses(); i++) {
            buf.append(template.getClass(i).getMembershipInfo()).append('\n').append('\n');
        }

        buf.append('\n');


        buf.append("Template is continuous: ").append(template.isContinuous()).append('\n');
        buf.append("Template is aux: ").append(template.isAux()).append('\n');
        buf.append("Class of Interest (COI) is: ").append(template.getClassOfInterestName()).append(" Index: ").append(template.getClassOfInterestIndex());
        buf.append('\n');

        buf.append("Total # of items: ").append(template.getNumItems()).append('\n').append('\n');
        buf.append("Output as string follows:\n");
        buf.append(template.getAsString(false)).append('\n');

        return buf;
    }

    public static void outlv(final Template template) {
        klog.debug(outsv(template).toString());
    }

    public static void outv(final Template template) {
        System.out.println(outsv(template).toString());
    }

    public static StringBuffer outsv(final Template template) {

        StringBuffer buf = new StringBuffer();

        if (template == null) {
            return new StringBuffer("null template");
        }

        buf.append("Template id=").append(template.getId()).append(" name=").append(template.getName()).append('\n');
        buf.append("Number of classes=").append(template.getNumClasses()).append('\n');
        buf.append("Number of items=").append(template.getNumItems()).append('\n');

        StringBuffer s = new StringBuffer();

        s.append(template.getNumItems());
        s.append(" ");
        s.append(template.getNumClasses());
        s.append(" ");
        s.append("1");    // dont know what this is?? but never seem anything other than 1
        s.append("\n");
        s.append("# ");

        for (int i = 0; i < template.getNumClasses(); i++) {
            s.append(template.getClassName(i));

            if (i != template.getNumClasses() - 1) {
                s.append(" ");
            }
        }

        s.append("\n");

        int[] profile_pos = template.getProfilePositionsOrdered();
        for (int i = 0; i < profile_pos.length; i++) {
            int pp = profile_pos[i];
            s.append(template.getItemByProfilePos(pp).getId()).append('[').append(pp);

            s.append(']');

            if (i != template.getNumItems() - 1) {
                s.append(" ");
            }
        }

        s.append("\n");
        buf.append(s).append('\n');

        return buf;
    }

    public static void outl(final Template[] templates) {
        klog.debug(outs(templates).toString());
    }

    public static void out(final Template[] templates) {
        System.out.println(outs(templates).toString());
    }

    public static StringBuffer outs(final Template[] tss) {

        StringBuffer buf = new StringBuffer();

        if (tss == null) {
            return new StringBuffer("null templates");
        }

        buf.append("Number of templates = ").append(tss.length).append('\n');

        for (int i = 0; i < tss.length; i++) {
            buf.append(outsv(tss[i])).append('\n');
        }

        return buf;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void outl(Template.Class tc) {
        klog.debug(outs(tc).toString());
    }

    public static void out(Template.Class tc) {
        System.out.println(outs(tc).toString());
    }

    public static StringBuffer outs(Template.Class tc) {

        StringBuffer s = new StringBuffer();

        if (tc == null) {
            return new StringBuffer("null Template.Class");
        }

        s.append("[Template.Class] name=").append(tc.getName()).append(" numitems=").append(tc.getSize()).append('\n');

        Template.Item[] items = tc.getItemsOrderedByProfilePos();

        for (int i = 0; i < items.length; i++) {
            s.append(items[i].getId()).append(" (").append(items[i].getProfilePosition()).append(")");
        }

        s.append('\n');

        return s;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void outl(Template.Item ti) {
        klog.debug(outs(ti).toString());
    }

    public static void out(Template.Item ti) {
        System.out.println(outs(ti).toString());
    }

    public static StringBuffer outs(Template.Item ti) {

        StringBuffer s = new StringBuffer();

        if (ti == null) {
            return new StringBuffer("null Template.Item");
        }

        s.append("Item id: ").append(ti.getId()).append(" pos: ").append(ti.getProfilePosition()).append('\n');

        return s;
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void out(Matrix matrix) {
        System.out.println(outs(matrix));
    }

    public static void outl(Matrix matrix) {
        klog.debug(outs(matrix));
    }

    public static String outs(Matrix matrix) {

        if (matrix == null) {
            return "Null matrix";
        }

        StringBuffer buf = new StringBuffer("Matrix: numrows=" + matrix.getNumRow() + " numcols="
                + matrix.getNumCol() + " immutable: "
                + matrix.isImmutable() + "\n");
        int nr = 10;

        if (matrix.getNumRow() < nr) {
            nr = matrix.getNumRow();
        }

        for (int i = 0; i < nr; i++) {
            for (int c = 0; c < matrix.getNumCol(); c++) {
                buf.append(matrix.getElement(i, c)).append(' ');
            }

            buf.append('\n');
        }

        return buf.toString();
    }

    //-----------------------------------------------------------------------//
    //-----------------------------------------------------------------------//
    public static void outl(Set set) {
        klog.debug(outs(set).toString());
    }

    public static void out(Set set) {
        System.out.println(outs(set).toString());
    }

    public static StringBuffer outs(Set set) {

        if (set == null) {
            return new StringBuffer("null Set");
        }

        return outs(set.iterator());
    }

    public static StringBuffer outs(Set set, String delim) {

        if (set == null) {
            return new StringBuffer(Constants.NULL);
        }

        StringBuffer s = new StringBuffer();

        Iterator it = set.iterator();
        int size = set.size();

        int cnt = 0;
        while (it.hasNext()) {
            Object key = it.next();

            if (key == null) {
                s.append(Constants.NULL);
            } else {
                s.append(key);
            }

            if (cnt != size - 1) {
                s.append(delim);
            }

            cnt++;
        }

        return s;
    }

    public static void outl(List list) {
        klog.debug(outs(list).toString());
    }

    public static void out(List list) {
        System.out.println(outs(list).toString());
    }

    public static StringBuffer outs(List list) {

        if (list == null) {
            return new StringBuffer("null List");
        }

        return outs(list.iterator());
    }

    public static void outl(final Iterator it) {
        klog.debug(outs(it).toString());
    }

    public static void out(final Iterator it) {
        System.out.println(outs(it).toString());
    }

    public static StringBuffer outs(final Iterator it) {

        StringBuffer s = new StringBuffer();

        if (it == null) {
            return new StringBuffer("null Iterator");
        }

        int cnt = 0;

        while (it.hasNext()) {
            Object key = it.next();

            if (key == null) {
                s.append("null");
            } else {
                s.append(key);
            }

            s.append('\t');

            if (cnt % 20 == 0) {
                s.append('\n');
            }

            if (cnt >= 100) {
                s.append("\nand so on ...");
                break;
            }

            cnt++;
        }

        while (it.hasNext()) {
            it.next();

            cnt++;
        }

        s.append('\n');
        s.append("Number of elements: ").append(cnt);
        s.append('\n');
        return s;
    }

    public String toStringTabDelimited(GeneSet gset) {
        StringBuffer buf = new StringBuffer();
        Iterator bi = gset.getMembers().iterator();
        while (bi.hasNext()) {
            buf.append(bi.next()).append('\t');
        }
        buf.append('\n');

        return buf.toString();
    }

    //-----------------------------------------------------------------------//

    public static String outs(String[] ss, String delim) {
        if (ss == null) {
            return Constants.NULL_ARRAY;
        } else if (ss.length == 0) {
            return Constants.EMPTY_ARRAY;
        } else {
            StringBuffer buf = new StringBuffer();
            for (int s = 0; s < ss.length; s++) {
                buf.append(ss[s]);
                if (s != ss.length - 1) {
                    buf.append(delim);
                }
            }

            return buf.toString();
        }
    }

    public static String outs(TObjectIntHashMap map) {
        StringBuffer buf = new StringBuffer();

        // faster iteration by avoiding hasNext():
        TObjectIntIterator it = map.iterator();
        for (int i = map.size(); i-- > 0;) {
            it.advance();
            buf.append(it.key()).append('\t').append(it.value()).append('\n');
        }

        return buf.toString();
    }

    public static String outs(TIntIntHashMap map) {
        StringBuffer buf = new StringBuffer();

        // faster iteration by avoiding hasNext():
        TIntIntIterator it = map.iterator();
        for (int i = map.size(); i-- > 0;) {
            it.advance();
            buf.append(it.key()).append('\t').append(it.value()).append('\n');
        }

        return buf.toString();
    }


    public static class Convert {


        /**
         * Excel related formatting methods
         */
        public static class Excel {

            /**
             * Simple tab delimited text with first row as col headers
             */
            public static void toExcel(final JTable table, final PrintWriter pw) {

                TableModel model = table.getModel();
                TableColumnModel tcm = table.getColumnModel();

                for (int c = 0; c < tcm.getColumnCount(); c++) {
                    if (c != 0) {
                        pw.print('\t');
                    }

                    pw.print(tcm.getColumn(c).getHeaderValue().toString());
                }

                pw.print("\n");

                for (int r = 0; r < model.getRowCount(); r++) {
                    for (int c = 0; c < model.getColumnCount(); c++) {
                        if (c != 0) {
                            pw.print('\t');
                        }

                        pw.print(model.getValueAt(r, c).toString());
                    }

                    pw.print('\n');
                }
            }

            /**
             * Simple tab delimited text with first row as col headers
             */
            public static void toExcel(final JTable table, final File file) throws IOException {

                klog.debug("Saving toExcel to: " + file.getAbsolutePath());

                PrintWriter pw = new PrintWriter(new FileOutputStream(file));

                toExcel(table, pw);
                pw.close();
            }

            /**
             * Simple tab delimited text with first row as ds column names
             */
            public static void toExcel(final Dataset ds, final PrintWriter pw) {

                for (int c = 0; c < ds.getNumCol(); c++) {
                    if (c != 0) {
                        pw.print('\t');
                    }

                    pw.print(ds.getColumnName(c));
                }

                pw.print('\n');

                for (int r = 0; r < ds.getNumRow(); r++) {
                    for (int c = 0; c < ds.getNumCol(); c++) {
                        if (c != 0) {
                            pw.print('\t');
                        }

                        pw.print(ds.getElement(r, c));
                    }

                    pw.print('\n');
                }
            }

            /**
             * Simple tab delimited text with first row as ds column names
             */
            public static void toExcel(final Dataset ds, final File file) throws IOException {

                klog.debug("Saving toExcel to: " + file.getAbsolutePath());

                PrintWriter pw = new PrintWriter(new FileOutputStream(file));

                toExcel(ds, pw);
                pw.close();
            }

        }

        /**
         * HTML formatting methods.
         */
        public static class Html {

            /**
             * header of a html page, incl the start body tag
             */
            public static StringBuffer getHeader() {

                StringBuffer buf =
                        new StringBuffer(
                                "<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">\n");

                buf.append("<HTML>\n<HEAD>\n");
                buf.append("<META NAME=\"Author\" CONTENT=\"xomics\">\n");
                buf.append("</HEAD>\n");
                buf.append("<BODY>\n");

                return buf;
            }

            /**
             * foooter for a html page including the end body tag
             */
            public static StringBuffer getFooter() {
                return new StringBuffer("\n</BODY>\n</HTML>\n");
            }

            public static void toHtml(JTable table, String align, int border, File file)
                    throws IOException {

                klog.debug("Saving toHtml to: " + file.getAbsolutePath());

                PrintWriter pw = new PrintWriter(new FileOutputStream(file));

                pw.print(getHeader());
                toHtml(table, align, border, pw);
                pw.print(getFooter());
                pw.close();
            }

            /**
             * Table formated as
             * <p/>
             * <pre>
             * <p/>
             * <TABLE BORDER COLS=4>
             * <TR> <TH>Heading 1   <TH> Heading 2  <TH> Heading 3    <TH> Heading 4 </TH> </TR>
             * <TR>
             *      <TD>foo</TD>
             *      <TD>bar</TD>
             *      <TD>baz</TD>
             *      <TD>zor</TD>
             * </TR>
             * <TR>
             *      <TD></TD>
             * ...and so on
             * </TR>
             * </TABLE>
             * <p/>
             * ALIGN=LEFT|RIGHT|CENTER
             */
            public static void toHtml(JTable table, String align, int border, PrintWriter pw) {

                TableModel model = table.getModel();
                StringBuffer buf =
                        new StringBuffer("<TABLE BORDER=\"").append(border).append("\" ALIGN= \"");

                buf.append(align).append("\" COLS=").append(model.getColumnCount()).append(">\n");
                pw.print(buf.toString());

                TableColumnModel tcm = table.getColumnModel();

                pw.print("<TR>\n");

                for (int c = 0; c < tcm.getColumnCount(); c++) {
                    buf = new StringBuffer("\t<TH>").append(
                            tcm.getColumn(c).getHeaderValue().toString()).append("</TH>\n");
                }

                pw.print("</TR>\n");

                for (int r = 0; r < model.getRowCount(); r++) {
                    pw.print("<TR>\n");

                    for (int c = 0; c < model.getColumnCount(); c++) {
                        buf = new StringBuffer("\t<TD>").append(model.getValueAt(r,
                                c).toString()).append("</TD>\n");

                        pw.print(buf.toString());
                    }

                    pw.print("</TR>\n");
                }

                pw.print("</TABLE>\n");
            }

            public static void toHtml(Dataset ds, String align, int border, File file)
                    throws IOException {

                klog.debug("Saving toHtml to: " + file.getAbsolutePath());

                PrintWriter pw = new PrintWriter(new FileOutputStream(file));

                pw.print(getHeader());
                toHtml(ds, align, border, pw);
                pw.print(getFooter());
                pw.close();
            }

            /**
             * Column header: <H2>some name</H2><BR><HR>
             * <p/>
             * If color ds,  the table cells bg color is set as the color rathern than
             * the using the float
             * <td BGCOLOR="#FF6666"></td>
             * <p/>
             * BORDER=0 CELLSPACING=0 CELLPADDING=0
             *
             * @todo Trouble with cds is that the cells are sized according to the size of the
             * column header
             */
            public static void toHtml(Dataset ds, String align, int border, PrintWriter pw) {

                if (ds == null) {
                    throw new NullPointerException("dataset to convert cant be null");
                }

                StringBuffer buf = new StringBuffer("<H2>").append(ds.getName()).append("</H2><BR>\n");

                buf.append("<TABLE BORDER=\"").append(border).append("\" ALIGN= \"");
                buf.append(align).append("\" COLS=").append(ds.getNumCol()).append(">\n");
                buf.append("<TR>\n");
                buf.append("\t<TH>Feature Name</TH>\n");

                for (int c = 0; c < ds.getNumCol(); c++) {
                    buf.append("\t<TH>").append(ds.getColumnName(c)).append("</TH>\n");
                }

                buf.append("</TR>\n");
                pw.print(buf.toString());

                // buf = new StringBuffer();

                for (int r = 0; r < ds.getNumRow(); r++) {
                    pw.print("<TR>\n");
                    pw.print("\t<TD>");
                    pw.print(ds.getRowName(r));
                    pw.print("</TD>");

                    for (int c = 0; c < ds.getNumCol(); c++) {
                        if (ds instanceof ColorDataset) {
                            String cstr = ColorUtils.toHex(((ColorDataset) ds).getColor(r, c));

                            //System.out.println(">> " + cstr);
                            pw.print("\t<TD BGCOLOR=\"");
                            pw.print(cstr);

                            // netscape and ie seem to want at least 1 char in the cell before they render it
                            pw.print("\">&nbsp</TD>");
                        } else {
                            pw.print("\t<TD>");
                            pw.print(ds.getElement(r, c));
                            pw.print("</TD>");
                        }
                    }

                    pw.print("\n</TR>\n");
                }

                pw.print("</TABLE>\n");
            }

            /**
             * The pob summary in a html page dumped in a pre section.
             */
            public static void toHtml(PersistentObject pob, PrintWriter pw) {

                StringBuffer buf = new StringBuffer("<H2>").append(pob.getName()).append("</H2><BR>\n");

                buf.append("<PRE>\n");

                //buf.append(pob.getSummary()).append("\n</PRE><BR>\n");
                pw.print(buf.toString());
            }

            public static void toHtml(PersistentObject pob, File file) throws IOException {

                klog.debug("Saving toHtml to: " + file.getAbsolutePath());

                PrintWriter pw = new PrintWriter(new FileOutputStream(file));

                pw.print(getHeader());
                toHtml(pob, pw);
                pw.print(getFooter());
                pw.close();
            }
        }    // End Convert

    }


}    // End Printf

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

/*
public static void toHtml(Annotation ann, String align, int border, File file) throws IOException {
    log.debug("Saving toHtml to: " + file.getAbsolutePath());
    PrintWriter pw = new PrintWriter(new FileOutputStream(file));
    pw.print(getHeader());
    toHtml(ann, align, border, pw);
    pw.print(getFooter());
    pw.close();
}
*/

/**
 * Column header: <H2>some name</H2><BR><HR>
 * <p/>
 * The pob summary in a html page dumped in a pre section.
 */

/*
 public static void toHtml(Annotation ann, String align, int border, PrintWriter pw) {
    if (ann == null) throw new NullPointerException("Annotation to convert cant be null");
    StringBuffer buf = new StringBuffer("<H2>").append(ann.getName()).append("</H2><BR>\n");
    buf.append("<TABLE BORDER=\"").append(border).append("\" ALIGN= \"");
    buf.append(align).append("\" COLS=").append(ann.getNumEntryTypes()).append(">\n");

    buf.append("<TR>\n");
    for (int c=0; c < ann.getNumEntryTypes(); c++) {
        buf.append("\t<TH>").append(ann.getEntryType(c)).append("</TH>\n");
    }
    buf.append("</TR>\n");
    pw.print(buf.toString());

    buf = new StringBuffer();
    for (int r=0; r < ann.getNumFeatures(); r++) {
        pw.print("<TR>\n");
        for (int c=0; c < ann.getNumEntryTypes(); c++) {
            pw.print("\t<TD>");
            pw.print(ann.getFeatureEntry(r, c));
            pw.print("</TD>");
        }
        pw.print("\n</TR>\n");
    }
    pw.print("</TABLE>\n");
}
*/