/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome;


import java.util.StringTokenizer;
import java.util.Vector;


/**
 * A version number composed of an arbitrary number of integers plus
 * an optional trailing string.
 *
 * @author <a href="mailto:kdowney@amberarcher.com">Kyle F. Downey</a>
 * @note as: originally from org.amberarcher.util;
 */
public class Version {

    public static final Version ALL = new Version(new int[0]);
    public static final Version NOT_IMPLEMENTED = new Version("todo");

    /**
     * Zero-args constructor for serialization.
     */
    public Version() {
        this(new int[0]);
    }

    /**
     * Helper method that parses the Version from a string using
     * {@link #fromString(java.lang.String)}.
     */
    public Version(String versionString) {
        fromString(versionString);
    }

    /**
     * Helper method for a common case of a one-part version number.
     */
    public Version(int majorVersion) {
        this(new int[]{majorVersion});
    }

    /**
     * Helper method for a common case of a two-part version number.
     */
    public Version(int majorVersion, int minorVersion) {
        this(new int[]{majorVersion, minorVersion});
    }

    /**
     * Helper method for a common case of a three-part version number.
     */
    public Version(int majorVersion, int minorVersion, int microVersion) {
        this(new int[]{majorVersion, minorVersion, microVersion});
    }

    /**
     * Creates a version number with the given components.
     */
    public Version(int[] versionComponents) {
        this(versionComponents, EMPTY_QUALIFIER);
    }

    /**
     * The arbitrary case: any number of version number components,
     * plus a trailing string.
     */
    public Version(int[] versionComponents, String qualifier) {
        this.versionComponents = versionComponents;
        this.qualifier = qualifier;
    }

    /**
     * gets a numbered version component: always returns properly,
     * with zero returned for all out-of-range requests
     */
    public int getVersionComponent(int componentNum) {

        if (componentNum >= versionComponents.length) {
            return 0;
        } else {
            return versionComponents[componentNum];
        }
    }

    /**
     * Gets the trailing string for the version number; may be empty ("").
     */
    public String getQualifier() {
        return qualifier;
    }

    /**
     * checks whether the given version is earlier than this one
     */
    public boolean precedes(Version ver) {

        for (int i = 0; i < versionComponents.length; i++) {
            if (i == ver.versionComponents.length) {
                return false;
            } else {
                if (versionComponents[i] > ver.versionComponents[i]) {
                    return false;
                }
            }
        }

        // if we get all the way through, they must be equal to this
        // point, but the given version number is longer; this must
        // mean it's more recent
        if (versionComponents.length < ver.versionComponents.length) {
            return true;
        } else if (versionComponents.length > ver.versionComponents.length) {
            return false;
        }

        // last case: equal numbers; time to compare a qualifying string,
        // if any
        else {
            if (hasQualifier(this) && hasQualifier(ver)) {
                return qualifier.compareTo(ver.qualifier) == -1;
            } else if (!hasQualifier(this) && hasQualifier(ver)) {
                return true;
            } else if (hasQualifier(this) && !hasQualifier(ver)) {
                return false;
            } else {
                return false;
            }
        }
    }

    /**
     * checks whether the given version is more recent than this one
     */
    public boolean succeeds(Version ver) {
        return (!precedes(ver) && !equals(ver));
    }

    public int compareTo(Object o) {

        Version other = (Version) o;

        if (equals(other)) {
            return 0;
        } else if (precedes(other)) {
            return -1;
        } else {
            return 1;
        }
    }

    /**
     * compares two version numbers to see if they are
     * logically equal rather than just lexicographically equal;
     * for example, the code is intelligent enough to know
     * that 1.2 and 1.2.0.0 are equal
     */
    public boolean equals(Object obj) {

        if (this == obj) {
            return true;
        }

        if ((null == obj) || (obj.getClass() != getClass())) {
            return false;
        }

        Version other = (Version) obj;

        for (int ii = 0; ii < versionComponents.length; ii++) {

            // we use getVersionComponents() here because it gives
            // us back a 0 if out of range; we want to discount
            // differences in trailing zeroes...
            if (versionComponents[ii] != other.getVersionComponent(ii)) {
                return false;
            }
        }

        // they may have trailing numbers; make sure they're zeroes only!
        if (versionComponents.length > other.versionComponents.length) {
            int lenDiff = versionComponents.length - other.versionComponents.length;

            for (int i = 0; i < lenDiff; i++) {
                if (versionComponents[versionComponents.length - lenDiff + i] != 0) {
                    return false;
                }
            }
        } else if (versionComponents.length < other.versionComponents.length) {
            int lenDiff = other.versionComponents.length - versionComponents.length;

            for (int i = 0; i < lenDiff; i++) {
                if (other.versionComponents[other.versionComponents.length - lenDiff + i] != 0) {
                    return false;
                }
            }
        }

        // last thing to check: qualifying string at the end: if both
        // present, have to be equal, otherwise both can't be present
        if (hasQualifier(this) && hasQualifier(other)) {
            return (qualifier.equals(other.qualifier));
        } else if (!hasQualifier(this) && !hasQualifier(other)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * produces a version string of the form
     * w[.x.y. ...][text] from this Version
     */
    public String toString() {

        // applies to all versions
        if (versionComponents.length == 0) {
            return "ALL";
        }

        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < versionComponents.length; i++) {
            sb.append(String.valueOf(versionComponents[i]));
            sb.append(".");
        }

        // trim trailing "."
        sb.setLength(sb.length() - 1);
        sb.append(qualifier);

        return sb.toString();
    }

    /**
     * parses a version string of the form w[.x.y. ... .n][string] to
     * produce a Version. TODO: possibly replace this with a SableCC-based
     * parser. This hand-lexing and -parsing is nasty and error-prone.
     */
    public void fromString(String fullVersionString) {

        // start with the default qualifier...
        qualifier = EMPTY_QUALIFIER;

        // handle global case first
        if (fullVersionString.equals("ALL")) {
            versionComponents = new int[0];
            qualifier = EMPTY_QUALIFIER;

            return;
        }

        int len = fullVersionString.length();

        // loop through, finding integer components
        Vector compList = new Vector();
        StringTokenizer stoker = new StringTokenizer(fullVersionString, ".");

        while (stoker.hasMoreTokens()) {
            compList.addElement(stoker.nextToken());
        }

        int[] tmpVersionComponents = new int[compList.size()];
        int ii = 0;

        try {
            for (ii = 0; ii < compList.size(); ii++) {
                tmpVersionComponents[ii] = Integer.parseInt((String) compList.elementAt(ii));
            }
        } catch (NumberFormatException e) {

            // if we're on the last element, it's possible we have
            // something like "1.5.22-GA", where "-GA" is the qualifier
            // and the 22 needs to be separated from it
            if (ii == (compList.size() - 1)) {
                String lastElement = (String) compList.elementAt(ii);
                StringBuffer numStr = new StringBuffer();
                int jj = 0;

                for (jj = 0; jj < lastElement.length(); jj++) {
                    if (!Character.isDigit(lastElement.charAt(jj))) {
                        break;
                    } else {
                        numStr.append(lastElement.charAt(jj));
                    }
                }

                if (numStr.length() > 0) {
                    try {
                        tmpVersionComponents[ii] = Integer.parseInt(numStr.toString());
                    } catch (NumberFormatException ex) {

                        // shouldn't happen
                        throw new RuntimeException("parse exception: " + e.getMessage());
                    }

                    // qualifier is the remainder
                    qualifier = lastElement.substring(jj);
                } else {
                    qualifier = lastElement;
                }
            } else {

                // we don't accept non-integers until the very last component
                throw new RuntimeException("parse exception: " + e.getMessage());
            }
        }

        versionComponents = tmpVersionComponents;
    }

    public Object clone() {

        Version ver = null;

        try {
            ver = (Version) super.clone();
        } catch (CloneNotSupportedException e) {
            throw new InternalError(e.toString());
        }

        ver.versionComponents = versionComponents;
        ver.qualifier = qualifier;

        return ver;
    }

    private static final String EMPTY_QUALIFIER = "";

    /**
     * Checks whether the Version qualifier is empty.
     */
    private static boolean hasQualifier(Version ver) {
        return !ver.qualifier.equals(EMPTY_QUALIFIER);
    }

    // build number components
    private int[] versionComponents;
    private String qualifier;
}

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
