/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.FrequencyMap;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.utils.ArrayUtils;
import gnu.trove.TObjectIntHashMap;
import gnu.trove.TObjectIntIterator;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * Several alg related utility methods. Dumping ground for ones that fit
 * well in other places.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class AlgUtils {

    private static final Logger klog = XLogger.getLogger(AlgUtils.class);

    /**
     * Privatized class constructor to prevent instantiation.
     */
    private AlgUtils() {
    }

    public static Set getAllMembers(final GeneSet[] gsets) {
        Set all = new HashSet();
        for (int i = 0; i < gsets.length; i++) {
            all.addAll(gsets[i].getMembersS());
        }

        return all;
    }

    public static int getMaxSize(final FrequencyMap[] fmaps) {

        int maxSize = 0;

        for (int i = 0; i < fmaps.length; i++) {
            int size = fmaps[i].getSize();
            if (maxSize < size) {
                maxSize = size;
            }
        }

        return maxSize;
    }

    public static LabelledVector getGeneSetSizes(final GeneSetMatrix gm) {

        Vector v = new Vector(gm.getNumGeneSets());
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            v.setElement(i, gm.getGeneSet(i).getNumMembers());
        }

        return new LabelledVector(gm.getGeneSetNamesL(true), v, true);
    }

    public static int countTotal(final TObjectIntHashMap map) {

        int tot = 0;

        // faster iteration by avoiding hasNext():
        TObjectIntIterator iterator = map.iterator();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            tot += iterator.value();
        }

        return tot;
    }

    /**
     * @param a
     * @param b
     * @return
     */
    public static String[] findRowNamesInAAndNotInB(final Dataset a, final Dataset b) {

        List anotb = new ArrayList();

        for (int i = 0; i < a.getNumRow(); i++) {

            String name = a.getRowName(i);
            if (b.getRowIndex(name) == -1) {
                anotb.add(name);
            }

        }

        return (String[]) anotb.toArray(new String[anotb.size()]);

    }

    /**
     * converts ds names to indices
     * if barfonmissing == true then brafing on missing names
     *
     * @param names
     * @param ds
     * @param barfonmissing
     * @return
     */
    public static int[] rowNames2Indices(final String[] names, final Dataset ds, final boolean barfonmissing) {

        if (ds == null) {
            throw new NullPointerException("Dataset arg cant be null");
        }

        if (names == null) {
            throw new NullPointerException("Names arr arg cant be null");
        }

        List hits = new ArrayList();

        // Hmm, maybe theres a more efficent way to lookup than to use indexof??
        for (int i = 0; i < names.length; i++) {
            int indx = ds.getRowIndex(names[i]);

            if ((indx == -1) && (barfonmissing)) {
                throw new RuntimeException("ds doesnt contain: " + names[i]);
            }

            if (indx != -1) {
                hits.add(new Integer(indx));
            }
        }

        klog.debug("Number of hits = " + hits.size());

        int hi[] = new int[hits.size()];

        for (int i = 0; i < hits.size(); i++) {
            hi[i] = ((Integer) hits.get(i)).intValue();
        }

        return hi;
    }


    /**
     * @param ds
     * @param startFromOne
     * @return
     */
    public static Integer[] createRowNumberArray(final Dataset ds, final boolean startFromOne) {

        int cnt = 0;

        if (startFromOne) {
            cnt = 1;
        }

        Integer[] arr = new Integer[ds.getNumRow()];

        for (int i = 0; i < ds.getNumRow(); i++) {
            arr[i] = new Integer(cnt++);
        }

        return arr;
    }

    /**
     * @param ds
     * @param startFromOne
     * @return
     */
    public static java.util.Vector createRowNumberVector(final Dataset ds, final boolean startFromOne) {

        java.util.Vector v = new java.util.Vector(ds.getNumRow());
        int cnt = 0;

        if (startFromOne) {
            cnt = 1;
        }

        for (int i = 0; i < ds.getNumRow(); i++) {
            v.addElement(new Integer(cnt++));
        }

        return v;
    }

    public static Set getRowNames(final int startIndex, final int stopIndex, final Dataset ds) {

        Set names = new HashSet();
        for (int i = startIndex; i <= stopIndex; i++) {
            if ((i < 0) || (i >= ds.getNumRow())) {
                // keep going
            } else {
                names.add(ds.getRowName(i));
            }
        }

        return names;
    }

    public static int getMaxNumRows(final Dataset[] dss) {

        int[] values = new int[dss.length];
        for (int i = 0; i < dss.length; i++) {
            values[i] = dss[i].getNumRow();
        }

        return XMath.max(values);
    }

    public static int getMaxSize(final RankedList[] rls) {

        int[] values = new int[rls.length];
        for (int i = 0; i < rls.length; i++) {
            values[i] = rls[i].getSize();
        }

        return XMath.max(values);
    }

    public static int getMaxNumCols(final Dataset[] dss) {

        int[] values = new int[dss.length];
        for (int i = 0; i < dss.length; i++) {
            values[i] = dss[i].getNumCol();
        }

        return XMath.max(values);
    }

    public static int getTotalNumCols(final Dataset[] dss) {
        int total = 0;

        for (int i = 0; i < dss.length; i++) {
            total += dss[i].getNumCol();
        }

        return total;
    }

    public static float getMaxValue(final Dataset[] dss) {
        float globalMax = Float.NEGATIVE_INFINITY;

        for (int i = 0; i < dss.length; i++) {
            Matrix matrix = dss[i].getMatrix();
            float max = matrix.max().value;
            if (max > globalMax) {
                globalMax = max;
            }
        }

        return globalMax;
    }

    public static float getMinValue(final Dataset[] dss) {
        float globalMin = Float.POSITIVE_INFINITY;

        for (int i = 0; i < dss.length; i++) {
            Matrix matrix = dss[i].getMatrix();
            float min = matrix.min().value;
            if (min < globalMin) {
                globalMin = min;
            }
        }

        return globalMin;
    }

    /**
     * Unique names across all gms AND fsets
     *
     * @param gms
     * @return
     */
    public static String[] getUniqueMemberNames(final GeneSetMatrix[] gms) {
        Set names = new HashSet();
        for (int i = 0; i < gms.length; i++) {
            names.addAll(gms[i].getAllMemberNamesOnlyOnceS());
        }

        return (String[]) names.toArray(new String[names.size()]);
    }

    public static String[] getUniqueMemberNames(final GeneSet[] gsets) {
        Set names = new HashSet();
        for (int i = 0; i < gsets.length; i++) {
            names.addAll(gsets[i].getMembersS());
        }

        return (String[]) names.toArray(new String[names.size()]);
    }

    /**
     * Names of fsets across GeneMatrices, unique - duplicates obv possible, but
     * hashed for return here
     *
     * @param gmxs
     * @return
     */
    public static String[] getUniqueGeneSetNames(final GeneSetMatrix[] gmxs) {
        Set names = new HashSet();
        for (int i = 0; i < gmxs.length; i++) {
            String[] gn = gmxs[i].getGeneSetNames(false);
            for (int g = 0; g < gn.length; g++) {
                names.add(gn[g]);
            }
        }

        return (String[]) names.toArray(new String[names.size()]);
    }


    public static String[] getAllPobNames(final PersistentObject[] pobs, boolean stripBase) {
        List names = getAllPobNamesL(pobs, stripBase);
        return (String[]) names.toArray(new String[names.size()]);
    }

    public static List getAllPobNamesL(final PersistentObject[] pobs, boolean stripBase) {
        List names = new ArrayList();
        for (int i = 0; i < pobs.length; i++) {
            if (stripBase) {
                names.add(AuxUtils.getAuxNameOnlyNoHash(pobs[i].getName()));
            } else {
                names.add(pobs[i].getName());
            }
        }

        return names;
    }

    public static Set getAllPobNamesS(final PersistentObject[] pobs) {
        Set names = new HashSet();
        for (int i = 0; i < pobs.length; i++) {
            names.add(pobs[i].getName());
        }

        return names;
    }

    // strips the base off
    public static Set getAllPobNamesFromAuxNamesS(final Set pobNames) {
        Set names = new HashSet();

        for (Iterator it = pobNames.iterator(); it.hasNext();) {
            String key = it.next().toString();
            names.add(AuxUtils.getAuxNameOnlyNoHash(key));
        }

        return names;
    }

    public static GeneSet[] getAllGeneSets_ensureuniquename(final GeneSetMatrix[] gms) {

        Set set = new HashSet();

        for (int i = 0; i < gms.length; i++) {
            GeneSet[] gsets = gms[i].getGeneSets();
            for (int f = 0; f < gsets.length; f++) {

                if (set.contains(gsets[f].getName())) {
                    throw new RuntimeException("Non-unique fset: " + gsets[f].getName() + " in gmx: " + gms[i].getName());
                }

                set.add(gsets[f]);
            }
        }

        return (FSet[]) set.toArray(new FSet[set.size()]);
    }

    public static int getMaxMemberCount(List genesets) {
        int max = 0;
        for (int i = 0; i < genesets.size(); i++) {
            GeneSet gset = (GeneSet) genesets.get(i);
            if (max < gset.getNumMembers()) {
                max = gset.getNumMembers();
            }
        }

        return max;
    }

    public static int getMaxMemberCount(GeneSet[] gsets) {
        int max = 0;
        for (int i = 0; i < gsets.length; i++) {
            if (max < gsets[i].getNumMembers()) {
                max = gsets[i].getNumMembers();
            }
        }

        return max;
    }

    public static Set unionRowNames(Dataset[] dss) {
        Set names = new HashSet();
        for (int i = 0; i < dss.length; i++) {
            names.addAll(dss[i].getRowNames());
        }

        return names;
    }

    public static Set intersectRowNames(Dataset[] dss) {

        FrequencyMap fmap = new FrequencyMap("some");

        for (int i = 0; i < dss.length; i++) {
            fmap.addAll(dss[i].getRowNames());
        }

        return fmap.getElements(dss.length);
    }

    public static void ensureUniqueColNames(Dataset[] dss) {
        Set names = new HashSet();
        for (int i = 0; i < dss.length; i++) {
            for (int c = 0; c < dss[i].getNumCol(); c++) {
                if (names.contains(dss[i].getColumnName(c))) {
                    throw new IllegalArgumentException("Duplicate column names (acorss datasets): " + dss[i].getColumnName(c) + " in dataset: " + dss[i].getName());
                }
                names.add(dss[i].getColumnName(c));
            }
        }
    }

    /**
     * Fsets are NOT checked for unqueness
     *
     * @param gms
     * @return
     */
    public static int getTotalNumOfGeneSets(final GeneSetMatrix[] gms) {
        int tot = 0;
        for (int i = 0; i < gms.length; i++) {
            tot += gms[i].getNumGeneSets();
        }

        return tot;
    }

    public static int getTotalNumOfGeneSets_ignore_duplicate_gmxs_by_name(final GeneSetMatrix[] gms) {
        Set set = new HashSet();
        int tot = 0;
        for (int i = 0; i < gms.length; i++) {
            if (set.contains(gms[i].getName())) {
                //
            } else {
                tot += gms[i].getNumGeneSets();

                set.add(gms[i].getName());
            }
        }

        return tot;
    }


    public static GeneSet unionAll(final String name, final GeneSet[] gsets) {

        Set all = new HashSet();
        for (int f = 0; f < gsets.length; f++) {
            for (int i = 0; i < gsets[f].getNumMembers(); i++) {
                all.add(gsets[f].getMember(i));
            }
        }

        return new FSet(name, all);
    }

    public static int unionAllCount(final GeneSet[] gsets) {

        Set all = new HashSet();
        for (int f = 0; f < gsets.length; f++) {
            for (int i = 0; i < gsets[f].getNumMembers(); i++) {
                all.add(gsets[f].getMember(i));
            }
        }

        return all.size();
    }

    public static GeneSet unionAll(final String name, final Collection[] colls) {

        Set all = new HashSet();
        for (int f = 0; f < colls.length; f++) {
            all.addAll(colls[f]);
        }

        return new FSet(name, all);
    }

    public static GeneSet subtract(final GeneSet theOne, final GeneSet dontUseAnyThatAreInMe) {
        return subtract("subtracted_" + theOne.getName(), theOne, dontUseAnyThatAreInMe);
    }

    public static GeneSet subtract(final String name, final GeneSet theOne, final GeneSet dontUseAnyThatAreInMe) {
        List list = new ArrayList();
        for (int i = 0; i < theOne.getNumMembers(); i++) {
            String member = theOne.getMember(i);
            if (!dontUseAnyThatAreInMe.isMember(member)) {
                list.add(member);
            }
        }

        return new FSet(name, list, false);
    }

    public static Set subtract(final Collection theOne, final Collection dontUseAnyThatAreInMe) {
        Set set = new HashSet();

        for (Iterator it = theOne.iterator(); it.hasNext();) {
            String member = it.next().toString();
            if (!dontUseAnyThatAreInMe.contains(member)) {
                set.add(member);
            }
        }

        return set;
    }

    public static Set subtract(final String[] theOne, final String[] dontUseAnyThatAreInMe) {
        return subtract(ArrayUtils.toSet(theOne), ArrayUtils.toSet(dontUseAnyThatAreInMe));
    }

    /**
     * @param gsets
     * @return
     */
    public static TObjectIntHashMap frequencyMap(final GeneSet[] gsets) {
        // IMP needed as returns 0 and not -1 on not hits!!
        TObjectIntHashMap map = new TObjectIntHashMap();

        for (int f = 0; f < gsets.length; f++) {
            for (int i = 0; i < gsets[f].getNumMembers(); i++) {

                if (map.containsKey(gsets[f].getMember(i))) {
                    int curr = map.get(gsets[f].getMember(i));
                    curr++;
                    map.put(gsets[f].getMember(i), curr);
                } else {
                    map.put(gsets[f].getMember(i), 1);
                }
            }
        }

        return map;

    }

    public static int getNumOfMembers(final Dataset ds, final GeneSet gs) {

        int ntrue = 0;

        for (int i = 0; i < ds.getNumRow(); i++) {
            if (gs.isMember(ds.getRowName(i))) {
                ntrue++;
            }
        }

        return ntrue;
    }

    public static int getNumOfMembers(final RankedList rl, final GeneSet gs) {

        int ntrue = 0;

        for (int r = 0; r < rl.getSize(); r++) {
            if (gs.isMember(rl.getRankName(r))) {
                ntrue++;
            }
        }

        return ntrue;
    }

    public static int getNumOfMembers(final Set ds, final GeneSet gs) {

        int ntrue = 0;

        Iterator it = ds.iterator();
        while (it.hasNext()) {
            String s = it.next().toString();
            if (gs.isMember(s)) {
                ntrue++;
            }
        }
        return ntrue;
    }

    public static int getNumGenes(boolean uniqueOnly, GeneSet[] gsets) {
        Collection all;

        if (uniqueOnly) {
            all = new HashSet();
        } else {
            all = new ArrayList();
        }
        for (int g = 0; g < gsets.length; g++) {
            all.addAll(gsets[g].getMembers());
        }

        return all.size();
    }

    public static GeneSet intersectAll(final String name, final GeneSet[] gsets) {
        return GeneSetGenerators.createIntersectionGeneSet(name, gsets);
    }

    public static GeneSet intersectAll(final String name, final Collection[] colls) {

        GeneSet all = unionAll(name + "tmpunion", colls);
        Set common = new HashSet();

        for (int i = 0; i < all.getNumMembers(); i++) {
            String item = all.getMember(i);
            boolean add = true;
            for (int f = 0; f < colls.length; f++) {
                if (!colls[f].contains(item)) {
                    add = false;
                    break;
                }
            }

            if (add) {
                common.add(item);
            }
        }

        return new FSet(name, common);
    }

    public static GeneSet intersect(final String name, final GeneSet a, final GeneSet b) {

        Set common = new HashSet();

        for (int i = 0; i < a.getNumMembers(); i++) {
            String member = a.getMember(i);
            if (b.isMember(member)) {
                common.add(member);
            }
        }

        return new FSet(name, common);
    }

    public static int intersectSize(final GeneSet a, final GeneSet b) {

        Set common = new HashSet();

        for (int i = 0; i < a.getNumMembers(); i++) {
            String member = a.getMember(i);
            if (b.isMember(member)) {
                common.add(member);
            }
        }

        return common.size();
    }

    public static int intersectSize(final RankedList a, final RankedList b) {

        Set common = new HashSet();

        for (int r = 0; r < a.getSize(); r++) {
            String member = a.getRankName(r);
            if (b.isMember(member)) {
                common.add(member);
            }
        }

        return common.size();
    }

    public static int intersectSize(final GeneSet a, final GeneSet[] bss) {

        Set common = new HashSet();

        GeneSet b = new FSet("b", bss);

        for (int i = 0; i < a.getNumMembers(); i++) {
            String member = a.getMember(i);
            if (b.isMember(member)) {
                common.add(member);
            }
        }

        return common.size();
    }

    public static String getComboName(EnrichmentDb[] edbs) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < edbs.length; i++) {
            buf.append(edbs[i]);
            if (i != edbs.length - 1) {
                buf.append('_');
            }
        }

        return buf.toString();
    }

    public static Set getGeneSetNamesS(final EnrichmentResult[] dtgs) {
        Set names = new HashSet();
        for (int i = 0; i < dtgs.length; i++) {
            names.add(dtgs[i].getGeneSet().getName());
        }

        return names;
    }

    public static GeneSet[] getGeneSets(final EnrichmentResult[] dtgs) {
        List all = new ArrayList();
        for (int i = 0; i < dtgs.length; i++) {
            all.add(dtgs[i].getGeneSet());
        }

        return (GeneSet[]) all.toArray(new GeneSet[all.size()]);
    }

    public static Set getTemplateNamesS(final Template[] tss) {
        Set names = new HashSet();
        for (int i = 0; i < tss.length; i++) {
            names.add(tss[i].getName());
        }

        return names;
    }

    public static Set getTemplateNamesS(final EnrichmentResult[] dtgs) {
        Set names = new HashSet();
        for (int i = 0; i < dtgs.length; i++) {
            names.add(dtgs[i].getTemplate().getName());
        }

        return names;
    }

    public static void enforceNoDuplicates(EnrichmentResult[] dtgs) {

        Set names = new HashSet();

        for (int i = 0; i < dtgs.length; i++) {
            String name = dtgs[i].getTemplate().getName() + "_" + dtgs[i].getGeneSet().getName();
            if (names.contains(name)) {
                throw new IllegalStateException("Duplicated dtg: " + name);
            }

            names.add(name);
        }

    }

}    // End AlgUtils

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
