/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.gsea.MhtMode;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.RankedFeature;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.ScoredDataset;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentScore;
import edu.mit.broad.genome.objects.strucs.NamedColor;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.vdb.VdbRuntimeResources;
import gnu.trove.TObjectIntHashMap;
import org.apache.log4j.Logger;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Comparator;
import java.util.Map;
import java.util.TreeMap;


/**
 * Collection of usefule comparators
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ComparatorFactory {

    private static final Logger klog = XLogger.getLogger(ComparatorFactory.class);

    /**
     * Privatized Class constructor
     */
    private ComparatorFactory() {
    }

    public static class NamedColorComparator implements Comparator {

        public int compare(final Object pn1, final Object pn2) {

            final NamedColor nc1 = (NamedColor) pn1;
            final NamedColor nc2 = (NamedColor) pn2;

            return nc1.name.compareTo(nc2.name);
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End class NamedColorComparator

    public static class EnrichmentResultByNESComparator implements Comparator {

        Order fOrder;

        public EnrichmentResultByNESComparator(final Order order) {
            this.fOrder = order;
        }

        public int compare(final Object pn1, final Object pn2) {

            final EnrichmentResult result1 = (EnrichmentResult) pn1;
            final EnrichmentResult result2 = (EnrichmentResult) pn2;

            final float nes1 = result1.getScore().getNES();
            final float nes2 = result2.getScore().getNES();

            if (fOrder.isAscending()) {
                if (nes1 < nes2) {
                    return -1;
                } else if (nes1 > nes2) {
                    return +1;
                } else {
                    return 0;
                }

            } else {
                if (nes1 < nes2) {
                    return +1;
                } else if (nes1 > nes2) {
                    return -1;
                } else {
                    return 0;
                }
            }

        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End class EnrichmentResultComparator


    public static class EnrichmentResultByFDRComparator implements Comparator {

        Order fOrder;

        public EnrichmentResultByFDRComparator(final Order order) {
            this.fOrder = order;
        }

        public int compare(final Object pn1, final Object pn2) {

            final EnrichmentResult result1 = (EnrichmentResult) pn1;
            final EnrichmentResult result2 = (EnrichmentResult) pn2;

            final float fdr1 = result1.getScore().getFDR();
            final float fdr2 = result2.getScore().getFDR();

            if (fOrder.isAscending()) {
                if (fdr1 < fdr2) {
                    return +1;
                } else if (fdr1 > fdr2) {
                    return -1;
                } else {
                    return 0;
                }

            } else {
                if (fdr1 < fdr2) {
                    return -1;
                } else if (fdr1 > fdr2) {
                    return +1;
                } else {
                    return 0;
                }
            }

        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End class EnrichmentResultComparator
    
    public static class EnrichmentResultByFDRPComparator implements Comparator {

        Order fOrder;

        public EnrichmentResultByFDRPComparator(final Order order) {
            this.fOrder = order;
        }

        public int compare(final Object pn1, final Object pn2) {

            final EnrichmentResult result1 = (EnrichmentResult) pn1;
            final EnrichmentResult result2 = (EnrichmentResult) pn2;

            final BigDecimal fdr1 = result1.getScore().getFDRB();
            final BigDecimal fdr2 = result2.getScore().getFDRB();

            if (fOrder.isAscending()) {
                if (fdr1.compareTo(fdr2)==-1) {
                    return +1;
                } else if (fdr1.compareTo(fdr2)==1) {
                    return -1;
                } else {
                    return 0;
                }

            } else {
                if (fdr1.compareTo(fdr2)==-1) {
                    return -1;
                } else if (fdr1.compareTo(fdr2)==1) {
                    return +1;
                } else {
                    return 0;
                }
            }

        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End class EnrichmentResultComparator    


    public static class EnrichmentResultByNPComparator implements Comparator {

        Order fOrder;

        public EnrichmentResultByNPComparator(final Order order) {
            this.fOrder = order;
        }

        public int compare(final Object pn1, final Object pn2) {

            final EnrichmentResult result1 = (EnrichmentResult) pn1;
            final EnrichmentResult result2 = (EnrichmentResult) pn2;

            final float np1 = result1.getScore().getNP();
            final float np2 = result2.getScore().getNP();

            if (fOrder.isAscending()) {
                if (np1 < np2) {
                    return +1;
                } else if (np1 > np2) {
                    return -1;
                } else {
                    return 0;
                }

            } else {
                if (np1 < np2) {
                    return -1;
                } else if (np1 > np2) {
                    return +1;
                } else {
                    return 0;
                }
            }

        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End class EnrichmentResultComparator


    public static class PobComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            PersistentObject pob1 = (PersistentObject) pn1;
            PersistentObject pob2 = (PersistentObject) pn2;
            return pob1.getName().compareTo(pob2.getName());
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End FileExtComparator

    public static class FileExtComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            String ext1 = FileUtils.getExtension(pn1.toString());
            String ext2 = FileUtils.getExtension(pn2.toString());

            return ext1.compareTo(ext2);
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End FileExtComparator

    /**
     * /**
     * DatasetFeaturePositionComparator
     * List of Dtg objects
     */
    public static class DtgByNESComparator implements Comparator {

        private MhtMode fMhtMode;
        private TObjectIntHashMap fDtgObjectIndexMap;
        private SortMode fSort;
        private Order fOrder;

        public DtgByNESComparator(SortMode sort, Order order, EnrichmentResult[] dtgs, MhtMode mmode) {
            this.fMhtMode = mmode;
            mmode.setData(dtgs); // @note imp -- this synchs things up
            this.fDtgObjectIndexMap = new TObjectIntHashMap();
            for (int i = 0; i < dtgs.length; i++) {
                if (fDtgObjectIndexMap.containsKey(dtgs[i])) {
                    throw new IllegalArgumentException("Repeated Dtg: " + dtgs[i] + " .. cant sort of not unique");
                }
                fDtgObjectIndexMap.put(dtgs[i], i);
            }

            this.fSort = sort;
            this.fOrder = order;
        }

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            EnrichmentResult dtg1 = (EnrichmentResult) pn1;
            EnrichmentResult dtg2 = (EnrichmentResult) pn2;

            int index1 = fDtgObjectIndexMap.get(dtg1);
            int index2 = fDtgObjectIndexMap.get(dtg2);

            float nes1 = fMhtMode.getRealNES(index1);
            float nes2 = fMhtMode.getRealNES(index2);

            if (fSort.isAbsolute()) {
                nes1 = Math.abs(nes1);
                nes2 = Math.abs(nes2);
            }

            if (fOrder.isAscending()) {
                if (nes1 < nes2) {
                    return -1;
                } else if (nes1 > nes2) {
                    return +1;
                } else {
                    return 0;
                }

            } else {
                if (nes1 < nes2) {
                    return +1;
                } else if (nes1 > nes2) {
                    return -1;
                } else {
                    return 0;
                }
            }

        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End DatasetFeaturePositionComparator /**

    public static class KSScoreStrucComparator implements Comparator {

        public KSScoreStrucComparator() {
        }

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            EnrichmentScore sd1 = (EnrichmentScore) pn1;
            EnrichmentScore sd2 = (EnrichmentScore) pn2;

            double score1 = sd1.getES();
            double score2 = sd2.getES();

            int size1 = sd1.getNumHits();
            int size2 = sd2.getNumHits();

            if (size1 != size2) {
                throw new IllegalArgumentException("Cannot compare gsets of diff sizes: " +
                        sd1.getNumHits() + " " + sd2.getNumHits());
            }

            if (score1 < score2) {
                return -1;
            } else if (score1 > score2) {
                return +1;
            } else {
                return 0;
            }

        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End DatasetFeaturePositionComparator

    public static class StringComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            if ((pn1 == null) || (pn2 == null)) {
                return 0;
            }

            return pn1.toString().compareTo(pn2.toString());
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End class StringComparator

    /**
     * Sorted in ascending order
     * comparator for usage to sort maps that have float values.
     *
     * @see http://groups.google.com/groups?hl=en&threadm=7ruch5%248qq%244%40f40-3.zfn.uni-bremen.de&rnum=1&prev=/groups%3Fq%3Dsorting%2Ba%2Bmap%2Bgroup:comp.lang.java.*%2Bgroup:comp.lang.java.*%26hl%3Den%26selm%3D7ruch5%25248qq%25244%2540f40-3.zfn.uni-bremen.de%26rnum%3D1
     */
    public static class FloatMapValueComparator implements Comparator {

        final boolean asc;

        public FloatMapValueComparator(boolean asc) {
            this.asc = asc;
        }

        public int compare(Object a, Object b) {

            Map.Entry m1 = (Map.Entry) a;
            Map.Entry m2 = (Map.Entry) b;
            float f1 = ((Float) m1.getValue()).floatValue();
            float f2 = ((Float) m2.getValue()).floatValue();

            if (asc) {
                if (f1 < f2) {
                    return -1;
                }

                if (f1 > f2) {
                    return +1;
                }

                return 0;
            } else {
                if (f1 < f2) {
                    return +1;
                }

                if (f1 > f2) {
                    return -1;
                }

                return 0;
            }
        }
    } // End class FloatMapValueComparator

    /**
     * DatasetFeaturePositionComparator
     */
    public static class DatasetFeaturePositionComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            int pos1 = ((RankedFeature) pn1).getRank();
            int pos2 = ((RankedFeature) pn2).getRank();

            if (pos1 < pos2) {
                return -1;
            } else if (pos1 == pos2) {
                return 0;
            } else {
                return +1;
            }
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End DatasetFeaturePositionComparator


    /**
     *
     */
    public static class ScoredDatasetScoreComparator implements Comparator {
        final ScoredDataset fSds;

        public ScoredDatasetScoreComparator(ScoredDataset sds) {
            this.fSds = sds;
        }

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            String name1 = (String) pn1;
            String name2 = (String) pn2;

            int rank1 = fSds.getRank(name1);
            int rank2 = fSds.getRank(name2);

            //System.out.println(">>> " + rank1 +  " " + rank2);

            if (rank1 == -1 || rank2 == -1) {
                throw new IllegalArgumentException("Specified label not in the sds: " + name1 + " " + name2);
            }

            if (rank1 < rank2) {
                return -1;
            } else if (rank1 == rank2) {
                return 0;
            } else {
                return +1;
            }
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End ScoredDatasetScoreComparator


    public static class RankedListComparator implements Comparator {
        final RankedList rl;

        public RankedListComparator(RankedList rl) {
            this.rl = rl;
        }

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            String name1 = (String) pn1;
            String name2 = (String) pn2;

            int rank1 = rl.getRank(name1);
            int rank2 = rl.getRank(name2);

            //System.out.println(">>> " + rank1 +  " " + rank2);

            if (rank1 == -1) {
                System.out.println("warn not in sds: " + name1);
                //throw new IllegalArgumentException("Specified label not in the sds: " + name1 + " " + name2);
            }
            if (rank2 == -1) {
                System.out.println("warn not in sds: " + name2);
            }

            if (rank1 < rank2) {
                return -1;
            } else if (rank1 == rank2) {
                return 0;
            } else {
                return +1;
            }
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End ScoredDatasetScoreComparator


    /**
     * Returns a Comparator that uses a Comparable object's natural ordering,
     * except null values are always considered low order. This Comparator
     * allows naturally ordered TreeMaps to support null values.
     */
    public static Comparator createNullLowOrderComparator() {
        return new NullLowOrder();
    }

    /**
     * Returns a Comparator that wraps the given Comparator except null values
     * are always considered low order. This fixes Comparators that don't
     * support comparisons against null and allows them to be used in TreeMaps.
     */
    public static Comparator createNullLowOrderComparator(Comparator c) {
        return new NullLowOrderC(c);
    }

    /**
     * Returns a Comparator that uses a Comparable object's natural ordering,
     * except null values are always considered high order. This Comparator
     * allows naturally ordered TreeMaps to support null values.
     */
    public static Comparator createNullHighOrderComparator() {
        return new NullHighOrder();
    }

    /**
     * Returns a Comparator that wraps the given Comparator except null values
     * are always considered high order. This fixes Comparators that don't
     * support comparisons against null and allows them to be used in TreeMaps.
     */
    public static Comparator createNullHighOrderComparator(Comparator c) {
        return new NullHighOrderC(c);
    }

    /**
     * Returns a Comparator that uses a Comparable object's natural ordering,
     * except null values are always considered equal order. This Comparator
     * should not be used in a TreeMap, but can be used in a sorter.
     */
    public static Comparator createNullEqualOrderComparator() {
        return new NullEqualOrder();
    }

    /**
     * Returns a Comparator that wraps the given Comparator except null values
     * are always considered equal order. This Comparator should not be used in
     * a TreeMap, but can be used in a sorter.
     */
    public static Comparator createNullEqualOrderComparator(Comparator c) {
        return new NullEqualOrderC(c);
    }

    /**
     * Returns a Comparator that wraps the given Comparator, but orders in
     * reverse.
     */
    public static Comparator createReversedComparator(Comparator c) {
        return new ReverseOrderC(c);
    }

    /**
     * Class NullLowOrder
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class NullLowOrder implements Comparator, Serializable {

        public int compare(Object obj1, Object obj2) {

            if (obj1 != null) {
                return (obj2 != null)
                        ? ((Comparable) obj1).compareTo(obj2)
                        : 1;
            } else {
                return (obj2 != null)
                        ? -1
                        : 0;
            }
        }

        // Serializable singleton classes should always do this.
        private Object readResolve() {
            return new NullLowOrder();
        }

    }

    /**
     * Class NullLowOrderC
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class NullLowOrderC implements Comparator, Serializable {

        private final Comparator c;

        public NullLowOrderC(Comparator c) {
            this.c = c;
        }

        public int compare(Object obj1, Object obj2) {

            if (obj1 != null) {
                return (obj2 != null)
                        ? c.compare(obj1, obj2)
                        : 1;
            } else {
                return (obj2 != null)
                        ? -1
                        : 0;
            }
        }
    }

    /**
     * Class NullHighOrder
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class NullHighOrder implements Comparator, Serializable {

        public int compare(Object obj1, Object obj2) {

            if (obj1 != null) {
                return (obj2 != null)
                        ? ((Comparable) obj1).compareTo(obj2)
                        : -1;
            } else {
                return (obj2 != null)
                        ? 1
                        : 0;
            }
        }

    }

    /**
     * Class NullHighOrderC
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class NullHighOrderC implements Comparator, Serializable {

        private final Comparator c;

        public NullHighOrderC(Comparator c) {
            this.c = c;
        }

        public int compare(Object obj1, Object obj2) {

            if (obj1 != null) {
                return (obj2 != null)
                        ? c.compare(obj1, obj2)
                        : -1;
            } else {
                return (obj2 != null)
                        ? 1
                        : 0;
            }
        }
    }

    /**
     * Class NullEqualOrder
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class NullEqualOrder implements Comparator, Serializable {

        public int compare(Object obj1, Object obj2) {

            return ((obj1 != null) && (obj2 != null))
                    ? ((Comparable) obj1).compareTo(obj2)
                    : 0;
        }

    }

    /**
     * Class NullEqualOrderC
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class NullEqualOrderC implements Comparator, Serializable {

        private final Comparator c;

        public NullEqualOrderC(Comparator c) {
            this.c = c;
        }

        public int compare(Object obj1, Object obj2) {

            return ((obj1 != null) && (obj2 != null))
                    ? c.compare(obj1, obj2)
                    : 0;
        }
    }

    /**
     * Class ReverseOrderC
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class ReverseOrderC implements Comparator, Serializable {

        private final Comparator c;

        public ReverseOrderC(Comparator c) {
            this.c = c;
        }

        public int compare(Object obj1, Object obj2) {
            return c.compare(obj2, obj1);
        }
    }


    public static class FloatValueComparator implements Comparator {

        TreeMap map;

        /*
        public TreeMap createFloatValueComparatoredTreeMap() {
            this.map = new TreeMap(this);
            return map;
        }
        */

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            Float f1 = (Float) map.get(pn1);
            Float f2 = (Float) map.get(pn2);

            if (f1.floatValue() < f2.floatValue()) {
                return -1;
            } else if (f1.floatValue() > f2.floatValue()) {
                return +1;
            } else {
                return 0;
            }
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End FloatValueComparator


    public static class ChipNameComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            String s1 = pn1.toString();
            String s2 = pn2.toString();

            if (VdbRuntimeResources.isChipGeneSymbol(s1)) {
                return -1;
            } else if (VdbRuntimeResources.isChipGeneSymbol(s2)) {
                return 1;
            }

            if (VdbRuntimeResources.isChipSeqAccession(s1)) {
                return -1;
            } else if (VdbRuntimeResources.isChipSeqAccession(s2)) {
                return 1;
            }

            if (s1.startsWith("HG") && s2.startsWith("HG")) {

            } else if (s1.startsWith("Hu") && s2.startsWith("Hu")) {
            } else if (s1.startsWith("Gene_Symbol") && s2.startsWith("Gene_Symbol")) {
            } else {

                if (s1.startsWith("HG") || s1.startsWith("Hu") || s1.startsWith("Gene_Symbol")) {
                    return -1;
                } else if (s2.startsWith("HG") || s2.startsWith("Hu") || s2.startsWith("Gene_Symbol")) {
                    return +1;
                }

            }

            return s1.compareTo(s2);
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End ChipNameComparator

}

/*--- Formatted in Sun Java Convention Style on Thu, Nov 7, '02 ---*/
