/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;


import edu.mit.broad.genome.Errors;
import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.distrib.RangeFactory;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.MissingValueDatasetParser;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.utils.ArrayUtils;
import edu.mit.broad.genome.utils.Strings;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.chip.NullSymbolMode;
import edu.mit.broad.vdb.map.ProbeMergeMode;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.vdb.sampledb.State;
import gnu.trove.TIntArrayList;
import gnu.trove.TIntHashSet;
import gnu.trove.TIntIntHashMap;
import org.apache.log4j.Logger;
import xtools.api.param.BadParamException;
import edu.duke.igsp.app.objects.SnpDataset;
import edu.duke.igsp.app.objects.NumDataset;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.util.*;

/**
 * Methods to generate datasets in various ways including:
 * 1) from artificial fdr
 * 2) by splitting in pre-existing datasets
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DatasetGenerators {

    private final Logger log = XLogger.getLogger(DatasetGenerators.class);

    /**
     * Class constructor
     */
    public DatasetGenerators() {
    }


    public static class RowByScoreComparator implements Comparator {

        Dataset ds;

        public RowByScoreComparator(final Dataset ds) {
            this.ds = ds;
        }

        public int compare(final Object pn1, final Object pn2) {

            String name1 = pn1.toString();
            String name2 = pn2.toString();

            RankedList row1 = ds.getRowRL(name1, SortMode.REAL, Order.DESCENDING);
            RankedList row2 = ds.getRowRL(name2, SortMode.REAL, Order.DESCENDING);


            for (int i = 0; i < ds.getNumCol(); i++) {

                int colMax_1 = ds.getColumnIndex(row1.getRankName(i));
                int colMax_2 = ds.getColumnIndex(row2.getRankName(i));

                if (colMax_1 > colMax_2) {
                    return +1;
                } else if (colMax_1 < colMax_2) {
                    return -1;
                }

            }

            return 0;
        }

        public boolean equals(Object o2) {
            return false;
        }
    }    // End class RowByScoreComparator

    public Dataset orderRows_by_ova_score(final Dataset ds) {
        List rowNames = new ArrayList(ds.getRowNames());
        Collections.sort(rowNames, new DatasetGenerators.RowByScoreComparator(ds));
        return new DatasetGenerators().orderDatasetRows(ds.getName() + "_ord", rowNames, ds);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    public String[] getColumnNamesByPrefix(final String prefix, final Dataset ds) {
        List matches = new ArrayList();
        for (int c = 0; c < ds.getNumCol(); c++) {
            if (ds.getColumnName(c).toUpperCase().startsWith(prefix.toUpperCase())) {
                matches.add(ds.getColumnName(c));
            }
        }

        log.info("# of matches by prefix " + prefix + " : " + matches.size());

        return (String[]) matches.toArray(new String[matches.size()]);
    }

    public Dataset createPairwiseDataset(final Dataset ds,
                                         final CorrMetric cm,
                                         final boolean doByRow,
                                         final boolean removeFeaturesWithZeroCorrToAllOthers) {


        Matrix pMatrix;

        String[] rowNames;

        if (doByRow) {
            rowNames = ds.getRowNamesArray();
            pMatrix = new Matrix(ds.getNumRow(), ds.getNumRow());
            for (int ra = 0; ra < ds.getNumRow(); ra++) {
                Vector rowa = ds.getRow(ra);
                for (int rb = 0; rb < ds.getNumRow(); rb++) {
                    Vector rowb = ds.getRow(rb);
                    float score = (float) cm.getCorr(rowa, rowb);
                    pMatrix.setElement(ra, rb, score);
                }
            }
        } else {
            rowNames = ds.getColumnNamesArray();
            pMatrix = new Matrix(ds.getNumCol(), ds.getNumCol());
            for (int ca = 0; ca < ds.getNumCol(); ca++) {
                Vector colA = ds.getColumn(ca);
                for (int cb = 0; cb < ds.getNumCol(); cb++) {
                    Vector colB = ds.getColumn(cb);
                    float score = (float) cm.getCorr(colA, colB);
                    pMatrix.setElement(ca, cb, score);
                }

            }
        }

        Dataset pwd = new DefaultDataset(ds.getName() + "_pwd", pMatrix, rowNames, rowNames, true);

        if (removeFeaturesWithZeroCorrToAllOthers) {
            List includeTheseRows = new ArrayList();
            for (int r = 0; r < pwd.getNumRow(); r++) {
                for (int c = 0; c < pwd.getNumCol(); c++) {
                    if (r != c && pwd.getElement(r, c) != 0) { // off diagonal non zero thing
                        includeTheseRows.add(pwd.getRowName(r));
                        break;
                    }
                }
            }

            pwd = new DatasetGenerators().extractRows(pwd, includeTheseRows);
        }

        return pwd;
    }


    public Dataset createComboDataset_append_columns(final String name, final Dataset a, final Dataset b) {

        List colNames = new ArrayList(a.getColumnNames());
        colNames.addAll(b.getColumnNames());

        final String[] rowNames = a.getRowNamesArray();
        final Matrix combo = new Matrix(rowNames.length, colNames.size());

        for (int r = 0; r < a.getNumRow(); r++) {
            int coln = 0;
            for (int c = 0; c < a.getNumCol(); c++) {
                combo.setElement(r, coln++, a.getElement(r, c));
            }

            for (int c = 0; c < b.getNumCol(); c++) {
                combo.setElement(r, coln++, b.getElement(rowNames[r], c)); // @note manual synch
            }
        }


        return new DefaultDataset(name, combo, rowNames, (String[]) colNames.toArray(new String[colNames.size()]), true, null);
    }


    // @note There might be duplicates!!
    public List getComboMarkers(final Dataset fullDs, final Template[] tss, final int xGenes, boolean up) {

        List genes = new ArrayList();
        for (int i = 0; i < tss.length; i++) {
            DatasetTemplate dti = new DatasetGenerators().extract(fullDs, tss[i]);
            ScoredDataset sdsi = new DatasetMetrics().scoreDataset(dti);
            genes.addAll(sdsi.getNamesOfUpOrDnXRanks(xGenes, up));  // @note ordered (thats kinda the point)
        }

        return genes;
    }

    public Dataset createDatasetOrderedByMarkerGenes(final String newName,
                                                     final Dataset fullDs,
                                                     final Template[] tss,
                                                     final int xGenes,
                                                     final boolean top) {

        final List ordered_rows = getComboMarkers(fullDs, tss, xGenes, top);

        /*
        for (int t = 0; t < tss.length; t++) {
            Template.Class cl = tss[t].getClass(0);
            List cc = cl.extractColumnNames(fullDs);
            System.out.println(">> " + cl.getName() + " # samples: " + cc.size() + " cl: " + cl.getSize());
            for (int f = 0; f < cc.size(); f++) {
                String cn = cc.get(f).toString();
                System.out.println(">>>> cn: " + cn);
            }
        }
        */


        final List ordered_cols = getOrderedColNames(fullDs, tss, true);
        //System.out.println("### ordered cols: " + ordered_cols.size());
        //Printf.outl(ordered_cols);
        return extractSortedRowsAndColumns(fullDs, ordered_rows, ordered_cols);
    }

    public List getOrderedColNames(final Dataset ds, final Template[] tss, final boolean classA) {

        List colNames = new ArrayList();
        Errors errors = new Errors();
        for (int t = 0; t < tss.length; t++) {
            Template.Class cl;
            if (classA) {
                cl = tss[t].getClass(0);
            } else {
                cl = tss[t].getClass(1);
            }
            List cc = cl.extractColumnNames(ds);
            for (int f = 0; f < cc.size(); f++) {
                String cn = cc.get(f).toString();
                //System.out.println(">>>> cn: " + cn);
                if (colNames.contains(cn) == false) {
                    colNames.add(cn);
                } else {
                    errors.add(cn);
                }
            }
        }

        errors.barfIfNotEmptyRuntime();

        return colNames;
    }

    public ColorDataset createColorDataset(final int numRanges, final RankedList rl, final ColorScheme cs) {

        final Range[] ranges = RangeFactory.createRanges(numRanges, 0, rl.getSize());

        final Vector v_full = rl.getScoresV(false);

        // Generate a color dataset
        Matrix m = new Matrix(1, ranges.length);
        for (int c = 0; c < ranges.length; c++) {
            m.setElement(0, c, (float) v_full.mean((int) ranges[c].getMin(), (int) ranges[c].getMax()));
        }


        return new ColorDatasetImpl(new DefaultDataset("foo", m, null), cs);
    }

    public Dataset createDiscDatasetFromAPCalls(final Dataset resDataset) {
        APMMatrix apm = ((DefaultDataset) resDataset).getAPMMatrix();
        return new DefaultDataset(resDataset.getName() + "_ap_discz", apm, resDataset.getRowNames(), resDataset.getColumnNames(), true, resDataset.getAnnot());
    }

    public Dataset log2(final Dataset ds) {
        final Matrix m = ds.getMatrix();
        final Matrix d = new Matrix(m, false);
        d.log2();
        return new DefaultDataset(ds.getName() + "_log2", d, ds.getRowNames(), ds.getColumnNames(),
                true, false, false, ds.getAnnot(), _getAPM(ds));
    }

    public Dataset exp2(final Dataset ds) {
        final Matrix m = ds.getMatrix();
        final Matrix d = new Matrix(m, false);
        d.exp(2);
        return new DefaultDataset(ds.getName() + "_nat",
                d, ds.getRowNames(),
                ds.getColumnNames(),
                true, false, false, ds.getAnnot(), _getAPM(ds));
    }

    private static APMMatrix _getAPM(Dataset ds) {
        APMMatrix apm = null;
        if (ds instanceof DefaultDataset) {
            apm = ((DefaultDataset) ds).getAPMMatrix();
        }

        return apm;
    }

    public LabelledVector collapseIntoOneMetaSample(final Dataset ds) {

        // now combine by taking the average value for a gene across all samples
        final String[] rowNames = new String[ds.getNumRow()];
        final Vector values = new Vector(ds.getNumRow());
        for (int r = 0; r < ds.getNumRow(); r++) {
            Vector v = ds.getRow(r);
            values.setElement(r, v.mean());
            rowNames[r] = ds.getRowName(r);
        }

        return new LabelledVector(ds.getName() + "_meta", rowNames, values);
    }


    public Dataset collapse(final Dataset ds,
                            final Chip chip) {
        return collapse(ds, chip, true, 0); // @note defaults
    }

    public Dataset collapse(final Dataset ds) {
        return collapse(ds, ds.getAnnot().getChip());
    }

    public Dataset collapse(final Dataset origDs,
                            final Chip chip,
                            final boolean includeOnlySymbols,
                            final int collapse_gex_mode) {

        return collapse_core(origDs, chip, includeOnlySymbols, collapse_gex_mode).symbolized;
    }

    // collapse_gex_mode -> 0 max vector
    // 1 -> median
    // @note IMP: because the chip files are already processed to remove aliases etc we dont have to do a alias lookup
    // simply do it against the gene symbol chip
    public CollapsedDataset collapse_core(final Dataset origDs,
                                          final Chip chip,
                                          final boolean includeOnlySymbols,
                                          final int collapse_gex_mode) {
        if (origDs == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        if (chip == null) {
            throw new IllegalArgumentException("Param chip cannot be null");
        }

        final Map symbolStrucMap = new HashMap();

        NullSymbolMode nm;
        if (includeOnlySymbols) {
            nm = Chip.OMIT_NULLS;
        } else {
            nm = Chip.REPLACE_WITH_PROBEID;
        }

        //for (int i = 0; i < gset.getNumMembers(); i++) {
        for (int r = 0; r < origDs.getNumRow(); r++) {
            String ps = origDs.getRowName(r);
            //String ps = gset.getMember(i);
            String symbol = chip.getSymbol(ps, nm);

            if (symbol != null && includeOnlySymbols && ps.equals(symbol)) {
                // @todo hack as the symbol lookup isnt alwawys removed
            } else {

                if (symbol != null) {
                    Object obj = symbolStrucMap.get(symbol);
                    if (obj == null) {
                        obj = new CollapseStruc(symbol);
                    }
                    ((CollapseStruc) obj).add(ps);
                    symbolStrucMap.put(symbol, obj);
                }
            }

        }

        //System.out.println("DOING : " + Printf.outs(origDs.getColumnNames()));

        //List rowValues = new ArrayList();
        Matrix m = new Matrix(symbolStrucMap.size(), origDs.getNumCol());
        List rowNames = new ArrayList();
        Iterator it = symbolStrucMap.keySet().iterator();
        int row = 0;
        while (it.hasNext()) {
            Object o = it.next();
            CollapseStruc collapseStruc = (CollapseStruc) symbolStrucMap.get(o);
            rowNames.add(collapseStruc.symbol);

            // System.out.println("doing: " + collapseStruc.symbol);

            final String[] pss = collapseStruc.getProbes();
            if (pss.length == 1) {
                String ps = pss[0];
                //System.out.println("checking for: " + ps);
                m.setRow(row, origDs.getRow(ps));
            } else {
                Vector[] vss = origDs.getRows(new FSet("foo", "foo", pss));
                //m.setRow(row, XMath.meanVector(vss));
                //m.setRow(row, XMath.medianVector(vss));
                if (collapse_gex_mode == 0) {
                    //m.setRow(row, XMath.maxVector_TTTT(vss, collapseStruc.symbol));
                    m.setRow(row, XMath.maxVector(vss));
                } else {
                    m.setRow(row, XMath.medianVector(vss));
                }
            }
            row++;
        }

        String name = origDs.getName() + "_collapsed_to_symbols";
        // @note chip is now always gene symbol
        // cant do this as the features are now diff
        //Annot annot = new AnnotImpl(new FeatureAnnotChipImpl(origDs.getAnnot().getFeatureAnnot(), chip), origDs.getAnnot().getSampleAnnot());
        // @note IMP to not use old row descs
        Annot annot = new AnnotImpl(new FeatureAnnotImpl(name, rowNames, null,
                VdbRuntimeResources.getChip_Gene_Symbol()), origDs.getAnnot().getSampleAnnot_global());

        CollapsedDataset cds = new CollapsedDataset();
        cds.orig = origDs;
        cds.symbolized = new DefaultDataset(name, m, rowNames, origDs.getColumnNames(), true, annot);
        cds.symbolCollapseStrucMap = symbolStrucMap;
        return cds;
    }

    // collapse_gex_mode -> 0 max vector
    // 1 -> median
    public RankedList collapse(final RankedList origRL,
                               final Chip chip,
                               final boolean includeOnlySymbols,
                               final int collapse_gex_mode) {
        if (origRL == null) {
            throw new IllegalArgumentException("Param origRL cannot be null");
        }

        if (chip == null) {
            throw new IllegalArgumentException("Param chip cannot be null");
        }

        final Map symbolStrucMap = new HashMap();

        NullSymbolMode nm;
        if (includeOnlySymbols) {
            nm = Chip.OMIT_NULLS;
        } else {
            nm = Chip.REPLACE_WITH_PROBEID;
        }

        for (int r = 0; r < origRL.getSize(); r++) {
            final String ps = origRL.getRankName(r);
            final String symbol = chip.getSymbol(ps, nm);

            if (symbol != null && includeOnlySymbols && ps.equals(symbol)) {
                // @todo hack as the symbol lookup isnt always removed
            } else {

                if (symbol != null) {
                    Object obj = symbolStrucMap.get(symbol);
                    if (obj == null) {
                        obj = new CollapseStruc(symbol);
                    }
                    ((CollapseStruc) obj).add(ps);
                    symbolStrucMap.put(symbol, obj);
                }
            }
        }

        final Vector cl_scores = new Vector(symbolStrucMap.size());
        final List cl_rowNames = new ArrayList();
        final Iterator it = symbolStrucMap.keySet().iterator();

        int row = 0;
        while (it.hasNext()) {
            final Object o = it.next();
            final CollapseStruc collapseStruc = (CollapseStruc) symbolStrucMap.get(o);
            cl_rowNames.add(collapseStruc.symbol);
            final String[] pss = collapseStruc.getProbes();
            if (pss.length == 1) {
                String ps = pss[0];
                //System.out.println("checking for: " + ps);
                cl_scores.setElement(row, origRL.getScore(ps));
            } else {
                float[] fss = origRL.getScores(new FSet("foo", "foo", pss));
                //m.setRow(row, XMath.meanVector(vss));
                //m.setRow(row, XMath.medianVector(vss));
                if (collapse_gex_mode == 0) {
                    cl_scores.setElement(row, XMath.max(fss));
                } else {
                    cl_scores.setElement(row, XMath.median(fss));
                }
            }
            row++;
        }

        String newName = origRL.getName() + "_collapsed";

        if (cl_scores.getSize() == 0) {
            throw new BadParamException("The collapsed dataset was empty when used with chip:" + chip.getName(), 1005);
        }

        return RankedListGenerators.sortByVectorAndGetRankedList(cl_scores, SortMode.REAL, Order.DESCENDING, cl_rowNames).cloneShallowRL(newName);
        // cant do this ad the scores might have got re-arranged
        //return new DefaultRankedList(name, cl_rowNames, cl_scores, true, true);
    }

    public static class CollapsedDataset {
        public Dataset symbolized;
        public Dataset orig;
        public Map symbolCollapseStrucMap;
    }

    public static class CollapseStruc {

        String symbol;
        Set probes;

        CollapseStruc(String symbol) {
            this.symbol = symbol;
            this.probes = new HashSet();
        }

        private void add(String ps) {
            this.probes.add(ps);
        }

        public String[] getProbes() {
            return (String[]) probes.toArray(new String[probes.size()]);
        }

        public String toString() {
            return symbol;
        }

        public int hashCode() {
            return symbol.hashCode();
        }

        public boolean equals(Object obj) {
            return symbol.equals(obj);
        }

    } // End class CollapseStruc

    // to the base 2
    public Dataset naturalize(final Dataset ds) {
        Matrix m = ds.getMatrix();
        Matrix m_nat = new Matrix(m.getNumRow(), m.getNumCol());
        for (int r = 0; r < m.getNumRow(); r++) {
            for (int c = 0; c < m.getNumCol(); c++) {
                m_nat.setElement(r, c, (float) Math.pow(2, m.getElement(r, c)));
            }
        }

        return new DefaultDataset(ds.getName() + "_nat", m_nat, ds.getRowNames(), ds.getColumnNames(), true, false, false, ds.getAnnot());
    }

    public Dataset normalize(final Dataset ds) {
        Matrix m = ds.getMatrix().cloneDeep();
        m.normalize(true);
        return new DefaultDataset(ds.getName() + "_norm", m, ds.getRowNames(), ds.getColumnNames(), true, false, false, ds.getAnnot());
    }

    /*
    public DatasetTemplate createExtractedContiguousDataset(final Dataset ds, final Template t, final GeneSet gset) {
        Dataset exds = extractRows(ds, gset);
        return createExtractedContiguousDataset(exds, t);
    }

    public DatasetTemplate createExtractedContiguousDataset(final Dataset ds, final Template t) {
        //log.debug("## IN: " + ds.getNumCol() + " " + t.getNumItems());

        DatasetTemplate ex = extract(ds, t);
        //log.debug("## after extraction: " + ex.getDataset().getNumCol() + " t: " + ex.getTemplate().getNumItems());

        if (t.isClassesContiguous()) {
            log.debug("->Template is already contiguous: " + t.getName() + " ... not extracting");
            return ex; // no change other than extraction (if that was indeed a change)
        } else {
            log.debug("->Making a new contiguous template... for " + t.getName());
            Object[] res = TemplateFactory.createTemplateContiguousClasses(ex.getTemplate());
            Template newt = (Template) res[0];
            TIntIntHashMap oldPosNewPosMap = (TIntIntHashMap) res[1];
            Dataset eds = ex.getDataset(false);
            Dataset newds = extractColumns(eds.getName(), eds, oldPosNewPosMap);
            return new DatasetTemplate(newds, newt);
        }

    }
    */

    /**
     * @param comboName
     * @param dss
     * @return
     */
    public Dataset appendColumnwise(final String comboName, final Dataset[] dss) {

        int ncols = 0;
        for (int i = 0; i < dss.length; i++) {
            ncols += dss[i].getNumCol();
        }

        Matrix combo = new Matrix(dss[0].getNumRow(), ncols);
        List colNames = new ArrayList();
        int coln = 0;
        for (int i = 0; i < dss.length; i++) {
            for (int c = 0; c < dss[i].getNumCol(); c++) {
                //colNames.add("col_" + coln);
                colNames.add(dss[i].getColumnName(c));
                combo.setColumn(coln++, dss[i].getColumn(c));
            }
        }

        FeatureAnnot fann = dss[0].getAnnot().getFeatureAnnot();
        SampleAnnot sann = new SampleAnnotImpl(comboName, colNames, null);

        return new DefaultDataset(comboName, combo, dss[0].getRowNames(), colNames, true, new AnnotImpl(fann, sann));
    }


    /**
     * @param ds
     * @return
     */
    public Dataset transpose(final Dataset ds) {
        Matrix mt = new Matrix(ds.getMatrix(), false);
        mt.transpose();

        Annot annot = ds.getAnnot();

        /* @todo
        Annot annot_t = null;

        if (annot != null) {
            annot_t = new AnnotImpl(annot.getSampleAnnot_global(), annot.getFeatureAnnot());
        }
        */

        return new DefaultDataset(ds.getName() + "_transposed", mt, ds.getColumnNames(), ds.getRowNames(), true, null);
    }


    // @note the dss rows must be synched - this IS checked
    // row names can (OBVIOUSLY) be duplicate
    // but column names cannot be
    public Dataset createComboDataset(final String name, final Dataset[] dss, final boolean union) {

        Set rowNamesSet;

        if (union) {
            rowNamesSet = AlgUtils.unionRowNames(dss);
        } else {
            rowNamesSet = AlgUtils.intersectRowNames(dss);
        }

        String[] rowNames = (String[]) rowNamesSet.toArray(new String[rowNamesSet.size()]);
        int totNumCols = AlgUtils.getTotalNumCols(dss);
        AlgUtils.ensureUniqueColNames(dss);
        log.debug("ComboDataset rowNames: " + rowNames.length + " colSize: " + totNumCols);

        Matrix combo = new Matrix(rowNames.length, totNumCols);
        List colNames = new ArrayList(totNumCols);

        int currCol = 0;

        List rowNamesList = new ArrayList(rowNames.length);
        for (int d = 0; d < dss.length; d++) {
            log.info("Combining Dataset: " + dss[d].getName() + " " + (d + 1) + " / " + dss.length);
            for (int r = 0; r < rowNames.length; r++) {// this loop is quite slooow
                Vector profile;
                int index = dss[d].getRowIndex(rowNames[r]);
                if (index != -1) {
                    profile = dss[d].getRow(index);
                } else {
                    profile = new Vector(dss[d].getNumCol(), Float.NaN);
                }

                for (int c = 0; c < profile.getSize(); c++) {
                    combo.setElement(r, currCol + c, profile.getElement(c));
                }

                if (d == 0) {
                    rowNamesList.add(rowNames[r]);
                }

                if (r % 4000 == 0) {
                    System.out.println("Combined row: " + r + " / " + rowNames.length);
                }
            }

            colNames.addAll(dss[d].getColumnNames());
            currCol += dss[d].getNumCol();
        }

        // sanity check
        if (currCol != totNumCols) {
            throw new IllegalStateException("Prog error: currCol: " + currCol + " totNumCols: " + totNumCols);
        }

        return new DefaultDataset(name, combo, rowNamesList, colNames, true, null); // @todo consider making a combo ann
    }

    public Dataset createComboDataset(final String name,
                                      final Dataset[] dss,
                                      final GeneSet joinOnTheseRowNames,
                                      final boolean barfOnMissing) {

        if (dss == null) {
            throw new IllegalArgumentException("Param dss cant be null");
        }

        int numCols = 0;
        List colNames_list = new ArrayList();
        for (int i = 0; i < dss.length; i++) {
            numCols += dss[i].getNumCol();
            colNames_list.addAll(dss[i].getColumnNames());
        }

        final Matrix combo = new Matrix(joinOnTheseRowNames.getNumMembers(), numCols);

        for (int r = 0; r < joinOnTheseRowNames.getNumMembers(); r++) {
            final String rowName = joinOnTheseRowNames.getMember(r);
            int coln = 0;
            for (int d = 0; d < dss.length; d++) {
                for (int c = 0; c < dss[d].getNumCol(); c++) {
                    int rowIndex = dss[d].getRowIndex(rowName);
                    if (rowIndex != -1) {
                        combo.setElement(r, coln++, dss[d].getElement(rowIndex, c));
                    } else {
                        combo.setElement(r, coln++, Float.NaN);
                    }
                }
            }
        }

        return new DefaultDataset(name, combo, joinOnTheseRowNames.getMembersArray(), (String[]) colNames_list.toArray(new String[colNames_list.size()]), true);
    }

    /**
     * @param ds
     * @return
     */
    public static boolean isAuxDataset(final Dataset ds) {
        if (ds == null) {
            throw new IllegalArgumentException("Parameter ds cannot be null");
        }

        return Strings.contains(ds.getName(), "@");
    }

    /**
     * Create a dataset by combining matrices in left to right or top to bottom mode
     * <p/>
     * Mode: left to right
     * <p/>
     * Dataset
     * matrix_1 matrix_2 matrix_3 ...
     * <p/>
     * 1) Matrices are added to the dataset left to right 0 to n-1
     * 2) Matrices must have equal number of rows
     * 3) Matrices can have different numbers of columns
     * <p/>
     * Mode: top to bottom
     * Dataset
     * matrix_1
     * matrix_2
     * matrix_3
     * ...
     * <p/>
     * 1) Matrices are added to the dataset top to bottom 0 to n-1
     * 2) Matrices must have equal number of columns
     * 3) Matrices can have different numbers of rows
     * <p/>
     * Row names are autogenerated as gene_row#
     * Col names are autogenerated as class_0, class_1, ... class_n-1
     */
    public Dataset matrix2dataset(final String name, final Matrix[] matrices, final boolean addleft2right) {

        List colnames, rownames;

        if (addleft2right) {
            int nrows = matrices[0].getNumRow();

            // check for equal row sizes
            for (int i = 0; i < matrices.length; i++) {
                if (matrices[i].getNumRow() != nrows) {
                    throw new IllegalArgumentException("Unequal number of rows: " + nrows
                            + " in matrix: " + i);
                }
            }

            rownames = new ArrayList(nrows);

            for (int i = 0; i < nrows; i++) {
                rownames.add("gene." + i);
            }

            colnames = new ArrayList();

            for (int i = 0; i < matrices.length; i++) {
                for (int j = 0; j < matrices[i].getNumCol(); j++) {
                    colnames.add("sample." + i);
                }
            }
        } else {
            int ncols = matrices[0].getNumCol();

            // check for equal col sizes
            for (int i = 0; i < matrices.length; i++) {
                if (matrices[i].getNumCol() != ncols) {
                    throw new IllegalArgumentException("Unequal number of cols: " + ncols
                            + " in matrix: " + i);
                }
            }

            colnames = new ArrayList(ncols);

            for (int i = 0; i < ncols; i++) {
                colnames.add("sample." + i);
            }

            rownames = new ArrayList();

            int rown = 0;

            for (int i = 0; i < matrices.length; i++) {
                for (int j = 0; j < matrices[i].getNumRow(); j++) {
                    rownames.add("gene." + rown++);
                }
            }
        }

        Matrix matrix = new Matrix(matrices, true);
        // OK to allow the DefaultDataset constructor to reuse the structures as all matrix, rownames and colnames
        // were made anew in this method
        return new DefaultDataset(name, matrix, rownames, colnames, true, null);
    }

    /**
     * Convenience method
     */
    public Dataset matrix2dataset(final String name, final Matrix matrix) {
        return matrix2dataset(name, new Matrix[]{matrix}, true);    // true or false makes no difference
    }

    public Dataset vector2dataset(String name, edu.mit.broad.genome.math.Vector[] vectors) {

        Matrix matrix = new Matrix(vectors, true, false);

        return matrix2dataset(name, matrix);
    }

    /**
     * Reads a tab delim file of only matrix values and makes a dataset
     * file should have only numbers of the matrix - no headers/labels
     */
    public Dataset matrix2dataset(final String name, final File file) throws Exception {

        int nrows = ParseUtils.countLines(file, true);
        int ncols = ParseUtils.countColumns(file, 1, true);

        log.info("nrows = " + nrows + " ncols = " + ncols);

        Matrix matrix = new Matrix(nrows, ncols);
        BufferedReader in = new BufferedReader(new FileReader(file));
        String line;
        int row = 0;

        while ((line = in.readLine()) != null) {
            StringTokenizer tok = new StringTokenizer(line, " \t");
            int col = 0;

            while (tok.hasMoreElements()) {
                float fl = Float.parseFloat(tok.nextToken());

                matrix.setElement(row, col, fl);

                col++;
            }

            row++;
        }

        in.close();

        //Print.out(matrix);
        return matrix2dataset(name, matrix);
    }

    /**
     * only specified cols are used, all rows are used
     *
     * @param colOrder
     * @param ds
     * @return
     */
    public Dataset orderDatasetColumns(final String name, final int[] colOrder, final Dataset ds) {

        Matrix matrix = new Matrix(ds.getNumRow(), colOrder.length);
        List colnames = new ArrayList(colOrder.length);
        for (int c = 0; c < colOrder.length; c++) {
            int colIndex = colOrder[c];
            matrix.setColumn(c, ds.getColumn(colIndex));
            colnames.add(ds.getColumnName(colIndex));
        }

        // fine for the sample annot as it doesnt care about col orders
        return new DefaultDataset(name, matrix, ds.getRowNames(), colnames, true, false, true, ds.getAnnot(), _getAPM(ds));
    }

    // @note only works if all profile positions specified in the template are also in the da
    // (i.e they need to have the same length)
    /* is this dangerous ('cause the ds and t will not be in synch)
    public Dataset orderDatasetColumns(final String name, final Template t, final Dataset ds) {
        int[] profile_positions_ordered = t.getProfilePositionsOrdered();
        log.debug("profile_positions_ordered: " + Printf.outs(profile_positions_ordered));
        return orderDatasetColumns(name, profile_positions_ordered, ds);
    }
    */

    public Dataset orderDatasetColumns(final String name, final List order, final Dataset ds, final boolean caseSensitive) {
        String[] ss = (String[]) order.toArray(new String[order.size()]);
        return orderDatasetColumns(name, ss, ds, caseSensitive);
    }

    public Dataset orderDatasetColumns(final String name, final String[] order, final Dataset ds, final boolean caseSensitive) {

        /*
        if (order.length != ds.getNumCol()) {
            throw new MismatchedSizeException("order", order.length, "dataset cols", ds.getNumCol());
        }
        */

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        if (order == null || order.length == 0) {
            throw new IllegalArgumentException("Order names cannot be null or empty: " + order);
        }

        if (order.length > ds.getNumCol()) {
            throw new IllegalArgumentException("# of orders: " + order.length + " cannot be more than # of cols: " + ds.getNumCol());
        }

        boolean alreadyOrdered = true;
        // To optimize a quick check to see if already ordered
        for (int c = 0; c < order.length; c++) {
            if (ds.getColumnIndex(order[c]) != c) {
                alreadyOrdered = false;
                break;
            }
        }

        if (alreadyOrdered) {
            log.debug("Already ordered dataset!");
            return ds;
        }

        Matrix matrix = new Matrix(ds.getNumRow(), order.length);
        List colNames = new ArrayList(order.length);
        for (int c = 0; c < order.length; c++) {
            int colIndex = ds.getColumnIndex(order[c]);

            // try harder
            if (colIndex == -1 && order[c].endsWith(".cel")) {
                String n = order[c].substring(0, order[c].length() - 4);
                colIndex = ds.getColumnIndex(n);
            }

            if (colIndex == -1 && order[c].endsWith(".CEL")) {
                String n = order[c].substring(0, order[c].length() - 4);
                colIndex = ds.getColumnIndex(n);
            }

            // even harder if case IN
            if (caseSensitive == false) {
                // @todo
            }


            if (colIndex == -1) {
                throw new IllegalArgumentException("Invalid order name specified >" + order[c] + "< Not found in dataset. Dataset cols are\n" + ds.getColumnNames());
            }
            matrix.setColumn(c, ds.getColumn(colIndex));
            colNames.add(ds.getColumnName(colIndex));
        }

        return new DefaultDataset(name, matrix, ds.getRowNames(), colNames, true, false, true, ds.getAnnot());
    }

    public Dataset orderDatasetRows(final String name, final List order, final Dataset ds) {
        String[] ss = (String[]) order.toArray(new String[order.size()]);
        return orderDatasetRows(name, ss, ds);
    }

    public Dataset orderDatasetRows(final String name, final String[] order, final Dataset ds) {

        if (order.length != ds.getNumRow()) {
            throw new MismatchedSizeException("order", order.length, "dataset rows", ds.getNumRow());
        }

        /*
        final List rlist = new ArrayList();
        for (int i = 0; i < ds.getNumRow(); i++) {
            rlist.add(ds.getRowName(i));
        }


        TObjectIntHashMap rlist = new TObjectIntHashMap();
         for (int i = 0; i < ds.getNumRow(); i++) {
            rlist.put(ds.getRowName(i), i);
        }
  */

        final Matrix matrix = new Matrix(order.length, ds.getNumCol());
        final List rowNames = new ArrayList(order.length);
        for (int r = 0; r < order.length; r++) {

            int rowIndex = ds.getRowIndex(order[r]);
            //int rowIndex = rlist.indexOf(order[r].trim());
            if (rowIndex == -1) {
                /*
                for (int i = 0; i < 5; i++) {
                    System.out.println(">> " + rlist.get(0).getClass());
                }*/
                throw new IllegalArgumentException("Invalid order name specified >" + order[r] + "< Not found in dataset. Rows are: " + ds.getRowNames());
            }
            matrix.setRow(r, ds.getRow(rowIndex));
            rowNames.add(ds.getRowName(rowIndex));
        }

        return new DefaultDataset(name, matrix, rowNames, ds.getColumnNames(), true, true, false, ds.getAnnot(), _getAPM(ds));
    }

    public Dataset orderDatasetRowsAndColumns(final String name, final String[] order, final Dataset ds, final boolean caseSensitive) {
        Dataset ods = orderDatasetRows(name, order, ds);
        return orderDatasetColumns(name, order, ods, caseSensitive);
    }

    public Dataset orderDatasetRowsAndColumns(final String name, final List order, final Dataset ds, final boolean caseSensitive) {
        Dataset ods = orderDatasetRows(name, order, ds);
        return orderDatasetColumns(name, order, ods, caseSensitive);
    }

    public Dataset orderDatasetRowsAndColumns(final String name, final String[] rowOrder, final String[] colOrder,
                                              final Dataset ds, final boolean caseSensitive) {
        Dataset ods = orderDatasetRows(name, rowOrder, ds);
        return orderDatasetColumns(name, colOrder, ods, caseSensitive);
    }


    public Dataset orderDatasetRowsAndColumns(final String name,
                                              final int[] rowIndexOrder,
                                              final int[] colIndexOrder,
                                              final Dataset ds) {

        if (rowIndexOrder == null) {
            throw new IllegalArgumentException("Param rowIndexOrder cannot be null");
        }

        if (colIndexOrder == null) {
            throw new IllegalArgumentException("Param colIndexOrder cannot be null");
        }

        final String[] row_order = new String[rowIndexOrder.length];
        final String[] col_order = new String[colIndexOrder.length];

        for (int r = 0; r < rowIndexOrder.length; r++) {
            row_order[r] = ds.getRowName(rowIndexOrder[r]);
        }

        for (int c = 0; c < colIndexOrder.length; c++) {
            col_order[c] = ds.getColumnName(colIndexOrder[c]);
        }

        return orderDatasetRowsAndColumns(name, row_order, col_order, ds, true);
    }

    public Dataset orderDatasetRowsAndColumns(final String name, final List rowOrder, final List colOrder,
                                              final Dataset ds, final boolean caseSensitive) {
        Dataset ods = orderDatasetRows(name, rowOrder, ds);
        return orderDatasetColumns(name, colOrder, ods, caseSensitive);
    }

    /**
     * Split the full dataset into subdatasets based on specified set of Templates
     * Datasets returned are fully independent of each other and of the fullDs
     * (memory-wise)
     * Data might be shared (depends on how the templates relate to one another)
     */
    public Dataset[] split(final Dataset fullDs, final Template[] tss) {

        Dataset[] subds = new Dataset[tss.length];
        Matrix[] mats = new Matrix[tss.length];
        List rownames = new ArrayList(fullDs.getRowNames());

        Collections.copy(rownames, fullDs.getRowNames());

        for (int t = 0; t < tss.length; t++) {
            List colnames = new ArrayList(tss[t].getNumItems());

            mats[t] = new Matrix(fullDs.getNumRow(), tss[t].getNumItems());

            int[] profilePos = tss[t].getProfilePositionsOrdered();
            for (int c = 0; c < profilePos.length; c++) {
                //System.out.println(">> " + templates[i].getItem(c).getPosition() + " " + templates[i].getItem(c).getProperty(Template.Item.OLD_POS));
                int origpos = profilePos[c];
                mats[t].setColumn(c, fullDs.getColumn(origpos));
                colnames.add(fullDs.getColumnName(origpos));
            }

            System.out.println("---------");
            // @note convention
            String name = fullDs.getName().concat("@").concat(tss[t].getName());

            // ok to reuse data in DefaultDataset as nothing is shared with parameter fullDs
            subds[t] = new DefaultDataset(name, mats[t], rownames, colnames, true, fullDs.getAnnot());
        }

        return subds;
    }

    /**
     * Splits specified dataset by all classes in the Template
     *
     * @param fullDs
     * @param template
     * @return
     */
    public Dataset[] extractByTemplateClasses(final Dataset fullDs, final Template template) {
        Dataset[] splits = new Dataset[template.getNumClasses()];

        for (int i = 0; i < splits.length; i++) {
            Template.Class cl = template.getClass(i);
            splits[i] = extract(fullDs, cl);
        }

        return splits;
    }

    public DatasetTemplate extract(final Dataset fullDs, final Template template) {
        return extract(fullDs, template, true);
    }

    //added by Qing Xiong - qing.xiong@duke.edu
    public DatasetTemplate extractSnp(final SnpDataset snpDs, final Template template) {
        return extractSnp(snpDs, template, true);
    }

    public DatasetTemplate extractNumDataset(final NumDataset numDs, final Template template) {
        return extractNumDataset(numDs, template, true);
    }
    //-------------

    public Dataset[] extract_ds(final Dataset fullDs, final Template[] tss, boolean add2cache) {
        DatasetTemplate[] dts = extract(fullDs, tss);
        return DatasetTemplate.toDatasets(dts, add2cache);
    }

    public DatasetTemplate[] extract(final Dataset fullDs, final Template[] tss) {
        DatasetTemplate[] dts = new DatasetTemplate[tss.length];

        for (int t = 0; t < tss.length; t++) {
            Template template = tss[t];
            dts[t] = extract(fullDs, template);
        }

        return dts;
    }

    public Dataset extract(final Dataset fullDs, final Template.Class tc) {

        if (fullDs == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        if (tc == null) {
            throw new IllegalArgumentException("Parameter tc cannot be null");
        }


        Matrix matrix = new Matrix(fullDs.getNumRow(), tc.getSize());
        List colnames = new ArrayList(tc.getSize());

        int[] profile_positions = tc.getOrderedProfilePositions();

        for (int c = 0; c < profile_positions.length; c++) {
            int col2use = profile_positions[c];
            matrix.setColumn(c, fullDs.getColumn(col2use));
            colnames.add(fullDs.getColumnName(col2use));
        }

        String name = NamingConventions.generateName(fullDs, tc, true);
        return new DefaultDataset(name, matrix, fullDs.getRowNames(), colnames, true, false, true, fullDs.getAnnot());
    }

    public static synchronized DatasetTemplate extract(final Dataset fullDs,
                                                       final Template origT,
                                                       final boolean verbose) {
        return TemplateFactory.extract(fullDs, origT, verbose);
    }

    //added by Qing Xiong - qing.xiong@duke.edu
    public static synchronized DatasetTemplate extractSnp(final SnpDataset snpDs,
                                                       final Template origT,
                                                       final boolean verbose) {
        return TemplateFactory.extractSnp(snpDs, origT, verbose);
    }

    public static synchronized DatasetTemplate extractNumDataset(final NumDataset numDs,
                                                       final Template origT,
                                                       final boolean verbose) {
        return TemplateFactory.extractNumDataset(numDs, origT, verbose);
    }
    //----------------

    /**
     * Create a new dataset with profile data only for specified probes
     */
    public Dataset extractRows(final String newName, final Dataset ds, final List rowNames) {

        if (newName == null) {
            throw new IllegalArgumentException("Parameter newName cannot be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Parameter probenames cannot be null");
        }

        if (ds == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        DatasetBuilder builder = new DatasetBuilder(newName, ds.getColumnNames());
        int misscnt = 0;
        List hitNames = new ArrayList();

        for (int i = 0; i < rowNames.size(); i++) {
            String probeName = (String) rowNames.get(i);

            //System.out.println("### looking up probe name>" + (String)probenames.get(i) + "<");
            int index = ds.getRowIndex(probeName);
            //System.out.println("GOT =" + index);

            if (index == -1) {
                //log.debug("No match in dataset for probe: " + probenames.get(i));
                //System.out.println("Missing: " + probenames.get(i));
                misscnt++;
            } else {
                hitNames.add(probeName);
                builder.addRow(index, ds);
            }
        }

        if (misscnt != 0) {
            log.warn("Not all probes had matches. Total probes:" + rowNames.size()
                    + " missing number:" + misscnt + " hits:" + hitNames.size());
        }

        // need tro

        return builder.generate(ds.getAnnot());
    }

    public Dataset extractRows(final String newName, final Dataset fullDs, final Set rowNames) {
        return extractRows(newName, fullDs, new ArrayList(rowNames));
    }

    public Dataset extractRows(final String newName, final Dataset fullDs, final int[] rowIndices) {
        String[] rowNames = new String[rowIndices.length];
        for (int r = 0; r < rowIndices.length; r++) {
            rowNames[r] = fullDs.getRowName(rowIndices[r]);
        }
        return extractRows(newName, fullDs, rowNames);
    }

    public Dataset extractRows(final String newName, final Dataset fullDs, final int[] rowIndices, final boolean INclude) {

        if (INclude) {
            return extractRows(newName, fullDs, rowIndices);
        } else {
            TIntArrayList useRows = new TIntArrayList();
            TIntArrayList excludeRows = new TIntArrayList(rowIndices);
            for (int r = 0; r < fullDs.getNumRow(); r++) {
                if (excludeRows.contains(r) == false) {
                    useRows.add(r);
                }
            }

            return extractRows(newName, fullDs, useRows.toNativeArray());
        }

    }

    public Dataset extractRows_exclude_row(final String newName, final Dataset fullDs, final int rowIndex) {
        List rowNames = new ArrayList(fullDs.getRowNames()); // @note cloning
        rowNames.remove(fullDs.getRowName(rowIndex));
        return extractRows(newName, fullDs, rowNames);
    }

    public Dataset extractRows(final String name, final Dataset fullDs, final String[] rowNames) {

        if (rowNames == null) {
            throw new IllegalArgumentException("Parameter rowNames cannot be null");
        }

        List list = new ArrayList(rowNames.length);
        for (int i = 0; i < rowNames.length; i++) {
            list.add(rowNames[i]);
        }

        return extractRows(name, fullDs, list);
    }

    public Dataset[] extractDatasets(final Dataset fullDs, final GeneSet[] gsets, final boolean isColumnNames) {

        if (fullDs == null) {
            throw new IllegalArgumentException("Param fullDs cannot be null");
        }

        if (gsets == null) {
            throw new IllegalArgumentException("Param gsets cannot be null");
        }

        Dataset[] dss = new Dataset[gsets.length];
        for (int g = 0; g < gsets.length; g++) {
            if (gsets[g] == null) {
                throw new IllegalArgumentException("Param gsets cannot be null at index: " + g);
            }

            GeneSet gset = gsets[g];
            String name = AuxUtils.getAuxNameOnlyNoHash(gset.getName());
            if (isColumnNames) {
                List foundNames = new ArrayList();
                List missingNames = new ArrayList();

                for (int i = 0; i < gset.getNumMembers(); i++) {
                    if (fullDs.getColumnIndex(gset.getMember(i)) != -1) {
                        foundNames.add(gset.getMember(i));
                    } else {
                        missingNames.add(gset.getMember(i));
                    }
                }

                dss[g] = new DatasetGenerators().extractColumns(fullDs.getName() + "xtracted_ncols_" + foundNames.size() + "_" + name, fullDs, foundNames);

                if (missingNames.size() != 0) {
                    log.warn("Some specified samples were not found in the dataset: " + missingNames.size());
                    System.out.println(missingNames);
                }

            } else {
                dss[g] = new DatasetGenerators().extractRows(fullDs.getName() + "xtracted_rows_" + name, fullDs, gset);
            }

        }

        return dss;
    }

    public Dataset extractRows(final Dataset fullDs, final GeneSet gset) {
        if (gset == null) {
            throw new IllegalArgumentException("Parameter gset cannot be null");
        }

        if (fullDs == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        String name = NamingConventions.generateName(fullDs, gset, true);
        return extractRows(name, fullDs, gset.getMembers());
    }

    public Dataset extractRows_with_apm(final Dataset fullDs, final GeneSet gset) {
        if (gset == null) {
            throw new IllegalArgumentException("Parameter gset cannot be null");
        }

        if (fullDs == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        APMMatrix orig_amp = ((DefaultDataset) fullDs).getAPMMatrix();
        Dataset newDs = extractRows(fullDs, gset.getMembers());

        APMMatrix apm = new APMMatrix(newDs.getNumRow(), newDs.getNumCol());
        for (int r = 0; r < newDs.getNumRow(); r++) {
            int orig_row_index = fullDs.getRowIndex(newDs.getRowName(r));
            for (int c = 0; c < newDs.getNumCol(); c++) {
                apm.setElement(r, c, orig_amp.getElement(orig_row_index, c));
            }
        }

        return ((DefaultDataset) newDs).cloneShallow(apm);
    }

    public Dataset extractRows(final Dataset ds, final List rowNames) {
        if (rowNames == null) {
            throw new IllegalArgumentException("Parameter names cannot be null");
        }

        if (ds == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        String name = ds.getName() + "_nrows" + rowNames.size();
        return extractRows(name, ds, rowNames);
    }

    public Dataset extractSortedRowsAndColumns(final Dataset fullDs, final List rowNames, final List colNames) {
        Dataset ds = extractColumns("foo", fullDs, colNames);
        return extractRows(fullDs.getName() + "_rows_cols_sorted", ds, rowNames);
    }

    public Dataset extractRowsSorted(final ScoredDataset fullSds, final GeneSet gset) {

        if (gset == null) {
            throw new IllegalArgumentException("Parameter fset cannot be null");
        }

        if (fullSds == null) {
            throw new IllegalArgumentException("Parameter fullSds cannot be null");
        }

        // the trick is to order the gene set by the scores in the sds
        // then the usual extract method picks up rows in the same order as the fset
        GeneSet ofset = new FSet(gset, fullSds);

        //System.out.println(">>>> " + ofset.getMembers());

        String name = NamingConventions.generateName(fullSds, gset, true);
        return extractRows(name, fullSds, ofset.getMembers());
    }

    public ScoredDataset extractRowsSds(final String newName, final ScoredDataset fullSds, final int[] rowIndices) {
        if (fullSds == null) {
            throw new IllegalArgumentException("Parameter fullSds cannot be null");
        }

        if (rowIndices == null) {
            throw new IllegalArgumentException("Parameter rowIndices cannot be null");
        }

        List list = new ArrayList();
        for (int i = 0; i < rowIndices.length; i++) {
            list.add(fullSds.getRow(rowIndices[i]));
        }

        return ((ScoredDatasetImpl) fullSds).extractSds(newName, new FSet(newName, list, false));
    }

    public ScoredDataset extractRowsSds(final ScoredDataset fullSds, final int[] rowIndices) {
        return extractRowsSds(fullSds.getName() + "_" + rowIndices.length, fullSds, rowIndices);
    }

    public Dataset extractRowsSorted(final ScoredDataset fullSds, final Template t, final GeneSet gset) {

        if (gset == null) {
            throw new IllegalArgumentException("Parameter gset cannot be null");
        }

        if (fullSds == null) {
            throw new IllegalArgumentException("Parameter fullSds cannot be null");
        }

        if (t == null) {
            throw new IllegalArgumentException("Parameter t cannot be null");
        }

        // the trick is to order the fset by the scores in the sds
        // then the usual extract method picks up rows in the same order as the fset
        GeneSet ogset = new FSet(gset, fullSds);

        //System.out.println(">>>> " + ofset.getMembers());

        Dataset ds = new DatasetGenerators().extract(fullSds, t).getDataset(false);
        String name = NamingConventions.generateName(fullSds, gset, true);
        return extractRows(name, ds, ogset.getMembers());
    }

    public Dataset extractRows(final String name, final Dataset fullDs, final GeneSet gset) {

        if (gset == null) {
            throw new IllegalArgumentException("Parameter gset cannot be null");
        }

        return extractRows(name, fullDs, gset.getMembers());
    }

    public Dataset extractRows(final String name, final Dataset fullDs, final GeneSet gset, final boolean INclude) {

        if (INclude) {
            return extractRows(name, fullDs, gset);
        } else {
            return extractRows(name, fullDs, GeneSetGenerators.subtractGeneSet(fullDs.getRowNamesGeneSet(), gset));
        }
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    public Dataset extractRowsAndColumns_with_apm(final String name,
                                                  final Dataset fullDs,
                                                  final GeneSet gset,
                                                  final String[] useTheseColumnNames,
                                                  final APMMatrix apm) {

        Dataset ds = extractRows(fullDs, gset);
        ds = extractColumns(name, ds, useTheseColumnNames);
        ((DefaultDataset) ds).cloneShallow(apm);
        return ds;
    }

    // -------------------------------------------------------------------------------------------- //


    /**
     * Create a truncated dataset with just the first nrows
     */
    public Dataset truncateRows(final int nrows, final Dataset ds) {

        int[] rowindices = ArrayUtils.fill(0, nrows, 1);

        return new DefaultDataset(NamingConventions.generateName(ds, "truncated." + nrows), rowindices, ds);
    }


    /**
     * Only those genes that are on datasetA are used
     *
     * @param aNameBStringArrayMap
     * @param dsA
     * @param dsB
     * @return
     */
    public Dataset[] createMappedDatasets(final ProbeMergeMode mmode, final Map aNameBStringArrayMap, final Dataset dsA, final Dataset dsB) {
        String[] aanames = (String[]) aNameBStringArrayMap.keySet().toArray(new String[aNameBStringArrayMap.size()]);
        Set set = new HashSet();
        for (int i = 0; i < aanames.length; i++) {
            if (dsA.getRowIndex(aanames[i]) != -1) {
                set.add(aanames[i]);
            }
        }

        String[] usedANames = (String[]) set.toArray(new String[set.size()]);

        log.debug("For mapping: dsA: " + dsA.getNumRow() + " of which # with mappings: " + usedANames.length);


        Dataset mappedB = createMappedDataset(mmode, usedANames, aNameBStringArrayMap, dsB);

        // @note that not all usedANames might make it into B -> some might get excluded due to mapping mode
        // so must make sure to extract not by usedAnames but b whats actually in B
        //Dataset mappedA = extract("mapped_" + dsA.getName(), usedANames, dsA); // wrong!!
        Dataset mappedA = extractRows("mapped_" + dsA.getName(), dsA, mappedB.getRowNames());

        return new Dataset[]{mappedA, mappedB};
    }

    public Dataset[] createMappedDatasetsS(final ProbeMergeMode mmode, final Map aNameBSetMap, final Dataset dsA, final Dataset dsB) {
        String[] aanames = (String[]) aNameBSetMap.keySet().toArray(new String[aNameBSetMap.size()]);
        Set set = new HashSet();
        for (int i = 0; i < aanames.length; i++) {
            if (dsA.getRowIndex(aanames[i]) != -1) {
                set.add(aanames[i]);
            }
        }

        String[] usedANames = (String[]) set.toArray(new String[set.size()]);

        log.debug("For mapping: dsA: " + dsA.getNumRow() + " of which # with mappings: " + usedANames.length);

        Dataset mappedB = createMappedDatasetS(mmode, usedANames, aNameBSetMap, dsB);

        // @note that not all usedANames might make it into B -> some might get excluded due to mapping mode
        // so must make sure to extract not by usedAnames but b whats actually in B
        //Dataset mappedA = extract("mapped_" + dsA.getName(), usedANames, dsA); // wrong!!
        Dataset mappedA = extractRows("mapped_" + dsA.getName(), dsA, mappedB.getRowNames());

        return new Dataset[]{mappedA, mappedB};
    }

    /**
     * ALL genes in the mapping are used
     *
     * @param aNameBStringArrayMap
     * @param dsB
     * @return
     */
    public Dataset createMappedDataset(final ProbeMergeMode mmode, final Map aNameBStringArrayMap, final Dataset dsB) {
        String[] anames = (String[]) aNameBStringArrayMap.keySet().toArray(new String[aNameBStringArrayMap.size()]);
        return createMappedDataset(mmode, anames, aNameBStringArrayMap, dsB);
    }

    public Dataset createMappedDatasetS(final ProbeMergeMode mmode, final Map aNameBSetMap, Dataset dsB) {
        String[] anames = (String[]) aNameBSetMap.keySet().toArray(new String[aNameBSetMap.size()]);
        return createMappedDatasetS(mmode, anames, aNameBSetMap, dsB);
    }

    /**
     * Only those that are in the anames are used
     *
     * @param name
     * @param stringnameStringArrayMap
     * @param fullDs
     * @return
     */
    public Dataset createMappedDataset(ProbeMergeMode mmode, String[] anames, Map aNameBStringArrayMap, Dataset dsB) {

        // now we need to average out for the mapped dataset
        // also need to change the gene names to be theg same as the first
        List rownames = new ArrayList();
        List profiles = new ArrayList();
        for (int r = 0; r < anames.length; r++) {
            Object obj = aNameBStringArrayMap.get(anames[r]);
            if (obj == null) {
                throw new IllegalArgumentException("Bad mapping -- null value in map structure for a name: " + anames[r]);
            }

            String[] bnames = (String[]) obj;
            Vector profile = null;

            boolean wasUsed = false;

            if (bnames.length > 1) {

                if (mmode == ProbeMergeMode.AVERAGE_MULTI_PROBES) {

                    int cntUsed = 0;
                    profile = new Vector(dsB.getNumCol());
                    for (int b = 0; b < bnames.length; b++) {
                        int bindex = dsB.getRowIndex(bnames[b]);
                        if (bindex == -1) {
                            log.warn("Missing probe -- no row in datasetB for the mapped B probe: " + bnames[b]);
                        } else {
                            Vector v = dsB.getRow(bindex);
                            profile.sum(v);
                            cntUsed++;
                        }
                    }

                    if (cntUsed == 0) {
                        log.warn("For probe A: " + anames[r] + " none of the mapped B probes " + bnames.length + " were found in the dataset B being mapped: " + dsB.getName());
                    } else {
                        profile.divide(cntUsed);
                        wasUsed = true;
                    }
                } else if (mmode == ProbeMergeMode.OMIT_MULTI_PROBES) {
                    // ignore
                } else {
                    throw new RuntimeException("Unknown mappingmode: " + mmode);
                }

            } else if (bnames.length == 1) {
                int bindex = dsB.getRowIndex(bnames[0]);

                if (bindex == -1) {
                    log.warn("B probe with a valid mapping to DatasetA was not found in DatasetB: >" + bnames[0] + "<");
                } else {
                    profile = dsB.getRow(bindex);
                    wasUsed = true;
                }

            } else {
                throw new IllegalStateException("Bad mapping structure - no mappings at all for: " + anames[r]);
            }

            if (wasUsed) {
                rownames.add(anames[r]);
                profiles.add(profile);
            }
        }

        Matrix matrix = new Matrix(profiles.size(), dsB.getNumCol());
        for (int r = 0; r < profiles.size(); r++) {
            matrix.setRow(r, (Vector) profiles.get(r));
        }

        return new DefaultDataset("mapped_" + dsB.getName(), matrix, rownames, dsB.getColumnNames(), true, true, false, null);
    }


    public Dataset createMappedDatasetS(final ProbeMergeMode mmode,
                                        final String[] anames,
                                        final Map aNameBSetMap,
                                        final Dataset dsB) {
        // now we need to average out for the mapped dataset
        // also need to change the gene names to be theg same as the first
        List rownames = new ArrayList();
        List profiles = new ArrayList();
        for (int r = 0; r < anames.length; r++) {
            Object obj = aNameBSetMap.get(anames[r]);
            if (obj == null) {
                throw new IllegalArgumentException("Bad mapping -- null value in map structure for A name: " + anames[r]);
            }

            Set set = (Set) obj;
            String[] bnames = (String[]) set.toArray(new String[set.size()]);
            Vector profile = null;

            boolean wasUsed = false;

            if (bnames.length > 1) {

                if (mmode == ProbeMergeMode.AVERAGE_MULTI_PROBES) {

                    int cntUsed = 0;
                    profile = new Vector(dsB.getNumCol());
                    for (int b = 0; b < bnames.length; b++) {
                        int bindex = dsB.getRowIndex(bnames[b]);
                        if (bindex == -1) {
                            log.warn("Missing probe -- no row in datasetB for the mapped B probe: " + bnames[b]);
                        } else {
                            Vector v = dsB.getRow(bindex);
                            profile.sum(v);
                            cntUsed++;
                        }
                    }

                    if (cntUsed == 0) {
                        log.warn("For probe A: " + anames[r] + " none of the mapped B probes " + bnames.length + " were found in the dataset B being mapped: " + dsB.getName());
                    } else {
                        profile.divide(cntUsed);
                        wasUsed = true;
                    }
                } else if (mmode == ProbeMergeMode.OMIT_MULTI_PROBES) {
                    // ignore
                } else {
                    throw new RuntimeException("Unknown mappingmode: " + mmode);
                }

            } else if (bnames.length == 1) {
                int bindex = dsB.getRowIndex(bnames[0]);

                if (bindex == -1) {
                    log.warn("B probe with a valid mapping to DatasetA was not found in DatasetB: >" + bnames[0] + "<");
                } else {
                    profile = dsB.getRow(bindex);
                    wasUsed = true;
                }

            } else {
                // dont exception out -> its valid for probes to not have mappings
                //throw new IllegalStateException("Bad mapping structure - no mappings at all for: " + anames[r]);
            }

            if (wasUsed) {
                rownames.add(anames[r]);
                profiles.add(profile);
            }
        }

        Matrix matrix = new Matrix(profiles.size(), dsB.getNumCol());
        for (int r = 0; r < profiles.size(); r++) {
            matrix.setRow(r, (Vector) profiles.get(r));
        }

        return new DefaultDataset("mapped_" + dsB.getName(), matrix, rownames, dsB.getColumnNames(), true, true, false, null);
    }

    public Dataset extract(final Dataset fullDs,
                           final State state) {
        return extractColumns(state.getName(), fullDs, state.getSampleNames());
    }

    /**
     * all rows (genes) and only specified samples (columns)
     *
     * @param fullDs
     * @param sampleNames
     * @return
     */
    public Dataset extractColumns(final String name,
                                  final Dataset fullDs,
                                  final List sampleNames) {
        if (name == null) {
            throw new IllegalArgumentException("Parameter name cannot be null");
        }

        if (fullDs == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        if (sampleNames == null) {
            throw new IllegalArgumentException("Parameter sampleNames cannot be null");
        }

        final Matrix matrix = new Matrix(fullDs.getNumRow(), sampleNames.size());

        for (int i = 0; i < sampleNames.size(); i++) {
            Object sname = sampleNames.get(i);
            if (sname == null) {
                throw new IllegalArgumentException("Null sample name at: " + i);
            }

            int coln = fullDs.getColumnIndex(sname.toString());
            if (coln == -1) {
                throw new RuntimeException("No such sample/column: >" + sname + "<" + " in dataset: >" + fullDs.getName() + "<" + "\nColumn names are:\n" + fullDs.getColumnNames());
            }

            matrix.setColumn(i, fullDs.getColumn(coln));
        }

        /*
        @note that i dont subset the sa or fa
        Perf is the only reason
        Annot annot = fullDs.getAnnot();
        if (annot != null && annot)
            */

        return new DefaultDataset(name, matrix, fullDs.getRowNames(), sampleNames, true, false, false, fullDs.getAnnot());
    }

    public Dataset extractColumns(final String name, final Dataset fullDs, final TIntIntHashMap oldPosNewPosMap) {
        if (name == null) {
            throw new IllegalArgumentException("Parameter name cannot be null");
        }

        if (fullDs == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        if (oldPosNewPosMap == null) {
            throw new IllegalArgumentException("Parameter oldPosNewPosMap cannot be null");
        }

        Matrix matrix = new Matrix(fullDs.getNumRow(), fullDs.getNumCol());
        List colNames = new ArrayList();
        TIntHashSet colsSeen = new TIntHashSet();
        for (int c = 0; c < fullDs.getNumCol(); c++) {
            int newPos = oldPosNewPosMap.get(c);

            if (colsSeen.contains(newPos)) {
                throw new IllegalArgumentException("Duplicted new col position: " + newPos);
            }

            String colName = fullDs.getColumnName(newPos);
            colNames.add(colName);
            matrix.setColumn(c, fullDs.getColumn(newPos));
        }

        return new DefaultDataset(name, matrix, fullDs.getRowNames(), colNames, true, false, true, fullDs.getAnnot());
    }

    public Dataset extractColumns(final String name, final Dataset fullDs, final String[] sampleNames) {

        if (sampleNames == null) {
            throw new IllegalArgumentException("Parameter sampleNames cannot be null");
        }

        List list = new ArrayList(sampleNames.length);
        for (int i = 0; i < sampleNames.length; i++) {
            list.add(sampleNames[i]);
        }

        return extractColumns(name, fullDs, list);
    }

    public Dataset extractColumns(final String name, final Dataset fullDs, final String[] sampleNames, final boolean includeSpecifiedCols) {

        if (includeSpecifiedCols) {
            return extractColumns(name, fullDs, sampleNames);
        } else {
            if (sampleNames == null) {
                throw new IllegalArgumentException("Parameter sampleNames cannot be null");
            }

            List omit = Arrays.asList(sampleNames);
            List use = new ArrayList();
            for (int c = 0; c < fullDs.getNumCol(); c++) {
                String colName = fullDs.getColumnName(c);
                if (!omit.contains(colName)) {
                    use.add(colName);
                }
            }

            return extractColumns(name, fullDs, use);
        }

    }

    public Dataset extractColumns(final String name,
                                  final Dataset fullDs,
                                  final int[] cols,
                                  final boolean includeSpecifiedCols) {

        int[] allIndices = XMath.toIndices(fullDs.getNumCol(), false);
        allIndices = XMath.toIndices(allIndices, cols, includeSpecifiedCols);

        List sampleNames = new ArrayList();
        for (int i = 0; i < allIndices.length; i++) {
            sampleNames.add(fullDs.getColumnName(allIndices[i]));
        }

        return extractColumns(name, fullDs, sampleNames);
    }

    public Dataset extract(final String newName,
                           final Dataset fullDs,
                           final int[] rowIndices,
                           final int[] colIndices) {
        Dataset ds = extractRows(newName, fullDs, rowIndices);
        return extractColumns(newName, ds, colIndices, true);
    }


    // file format must be standard
    // inspired by ken ross's code
    public Dataset imputeDataset(final Imputer imputer, final File datasetFile) throws Exception {

        FileInputStream is = new FileInputStream(datasetFile);
        Dataset ds = new MissingValueDatasetParser().parseDataset(datasetFile.getName(), is);

        DatasetBuilder builder = new DatasetBuilder(datasetFile.getName() + "_imputed_" + imputer.getName(), ds.getColumnNames());

        for (int r = 0; r < ds.getNumRow(); r++) {
            Vector v = ds.getRow(r);
            builder.addRow(ds.getRowName(r), imputer.impute(v));
        }

        return builder.generate(ds.getAnnot());
    }

}    // End DatasetGenerators

/*--- Formatted in Sun Java Convention Style on Mon, Sep 30, '02 ---*/
