/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.markers.PermutationTest;
import edu.mit.broad.genome.alg.markers.PermutationTestImpl;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;
import org.apache.log4j.Logger;
import edu.mit.broad.genome.objects.Template;
import edu.duke.igsp.app.algorithms.*;
import edu.duke.igsp.app.tools.GsaaSeqTool;
import java.util.*;

/**
 * @author Aravind Subramanian
 * @revised by Qing Xiong <qxiong@email.unc.edu>
 * @version %I%, %G%
 */
public class DatasetMetrics {

    private final Logger log = XLogger.getLogger(this.getClass());

    /**
     * Class Constructor.
     */
    public DatasetMetrics() {
    }

    public static Map getDefaultMetricParams() {
        Map params = new HashMap();
        params.put(Headers.USE_MEDIAN, XPreferencesFactory.kMedian.getBooleanO());
        params.put(Headers.FIX_LOW, XPreferencesFactory.kFixLowVar.getBooleanO());
        params.put(Headers.USE_BIASED, XPreferencesFactory.kBiasedVar.getBooleanO());
        return params;
    }

    /**
     * Uses common paramters to run a permutation test and save output in the specified folder
     *
     * @param ds
     * @param t
     */
    public PermutationTest runCategoricalPermutationTest(final int numMarkers,
            final int numPerms,
            final Dataset ds,
            final Template t) throws Exception {

        return new PermutationTestImpl(numMarkers,
                numPerms, new LabelledVectorProcessors.None(), new Metrics.Signal2Noise(), SortMode.REAL, Order.DESCENDING,
                getDefaultMetricParams(), ds, t, null,
                new RandomSeedGenerators.Timestamp(),
                TemplateRandomizerType.NO_BALANCE, false, true);
    }

    // IMP metrics are NOT lined up, i.e each is independently sorted
    public Dataframe scoreDatasets(final String name,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset[] dss,
            final Template[] tss) {

        int nrows = SanityKit.getAndEnforceEqualNumRows(dss);
        SanityKit.enforceEqualNum(dss, tss);

        Matrix matrix = new Matrix(nrows, dss.length);

        int coln = 0;
        List colNames = new ArrayList();
        for (int c = 0; c < dss.length; c++) {
            ScoredDataset sds = scoreDataset(metric, sort, order, metricParams, lvp, dss[c], tss[c]);
            matrix.setColumn(coln++, sds.getScoresV(true));
            colNames.add(SanityKit.getName(dss[c], tss[c]));
        }

        return new Dataframe(name, matrix, colNames, true);
    }

    public StringDataframe scoreDatasetsS(final String name,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset[] dss,
            final Template[] tss) {

        int nrows = SanityKit.getAndEnforceEqualNumRows(dss);
        SanityKit.enforceEqualNum(dss, tss);

        StringMatrix smatrix = new StringMatrix(nrows, dss.length * 2);

        int coln = 0;
        List colNames = new ArrayList();
        for (int c = 0; c < dss.length; c++) {
            ScoredDataset sds = scoreDataset(metric, sort, order, metricParams, lvp, dss[c], tss[c]);
            smatrix.setColumn(coln++, sds.getRankedNamesArray());
            smatrix.setColumn(coln++, sds.getScoresV(true));
            colNames.add("Name" + SanityKit.getName(dss[c], tss[c]));
            colNames.add("Score" + SanityKit.getName(dss[c], tss[c]));
        }

        return new StringDataframe(name, smatrix, colNames, true);
    }

    public StringDataframe scoreDatasetsS(final String name,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset[] dss,
            final Template[] tss,
            final GeneSet reportOnlyForTheseGenes) {


        SanityKit.getAndEnforceEqualNumRows(dss);
        SanityKit.enforceEqualNum(dss, tss);
        StringMatrix sm = new StringMatrix(reportOnlyForTheseGenes.getNumMembers(), dss.length * 2);

        DatasetGenerators gen = new DatasetGenerators();

        int currCol = 0;
        List colNames = new ArrayList();
        for (int i = 0; i < dss.length; i++) {
            Dataset ds = gen.extractRows(dss[i], reportOnlyForTheseGenes);
            LabelledVector lv = scoreDatasetLv(metric, sort, order, metricParams, lvp, ds, tss[i]);
            sm.setColumn(currCol++, lv.getLabelsArray());
            sm.setColumn(currCol++, lv.getScoresV(false).toArray());
            colNames.add("Name_" + dss[i].getName());
            colNames.add("Score_" + dss[i].getName());
        }

        return new StringDataframe(name, sm, colNames, true);
    }

    // we want to restrict the values reported to those in the fset
    // assumption here that all datasets have all of these row names
    // but remember that the fset names are in each alt column and not the row labels of the sdf!
    public StringDataframe restrict(final StringDataframe sdfFull, final GeneSet gset) {

        if (sdfFull.getNumCol() % 2 != 0) {
            throw new IllegalArgumentException("sdfFull has to have even number of columns!: " + sdfFull.getNumCol());
        }

        StringMatrix smatrix = new StringMatrix(gset.getNumMembers(), sdfFull.getNumCol());

        for (int i = 0; i < sdfFull.getNumCol(); i = i + 2) {
            int cola = i;
            int colb = i + 1;

            List names = sdfFull.getColumnAsList(cola);
            //System.out.println(">> " + names);
            String[] values = sdfFull.getColumn(colb);

            String[] newnames = new String[gset.getNumMembers()];
            String[] newvalues = new String[gset.getNumMembers()];
            for (int f = 0; f < gset.getNumMembers(); f++) {
                newnames[f] = gset.getMember(f);
                //System.out.println("Looking for: " + newnames[f]);
                int index = names.indexOf(newnames[f]);
                newvalues[f] = values[index];
            }
            smatrix.setColumn(cola, newnames);
            smatrix.setColumn(colb, newvalues);
        }

        return new StringDataframe(sdfFull.getName(), smatrix, sdfFull.getColumnNames(), true);
    }

    // @note uses hardcoded default metrics etc
    public ScoredDataset scoreDataset(final Dataset ds, final Template t) {
        return scoreDataset(new Metrics.Signal2Noise(), SortMode.REAL, Order.DESCENDING,
                new HashMap(), new LabelledVectorProcessors.None(), ds, t);
    }

    public ScoredDataset scoreDataset_fold(final Dataset ds, final Template t, final boolean useMean) {
        Metric metric;

        if (useMean) {
            metric = new Metrics.ClassMeansRatio();
        } else {
            metric = new Metrics.ClassMediansRatio();
        }

        return scoreDataset(metric, SortMode.REAL, Order.DESCENDING,
                new HashMap(), new LabelledVectorProcessors.None(), ds, t);
    }

    public ScoredDataset scoreDataset(final DT dt) {
        return scoreDataset(dt.ds, dt.t);

    }

    public ScoredDataset scoreDataset(final DatasetTemplate dt) {
        return scoreDataset(dt.getDataset(), dt.getTemplate());
    }

    /**
     * Score AND sort/order a Dataset according to specified parameters
     * <p/>
     * Also see the Unsorted version below for better perf in some
     * cases.
     *
     * @param metric
     * @param sort
     * @param order
     * @param ds
     * @param template
     * @return
     */
    public ScoredDataset scoreDataset(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        AddressedVector av = calcSortedMetric(metric, sort, order, metricParams, lvp, ds, template);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredDataset scoreDataset_gsaaseq_ks(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        AddressedVector av = calcSortedMetric_gsaaseq_ks(demetric, sort, order, lvp, ds, template);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredDataset scoreDataset_gsaaseqsp_ks(
            final List ess,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final int colNum) {

        AddressedVector av = calcSortedMetric_gsaaseqsp_ks(ess, sort, order, lvp, colNum);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredDataset scoreDataset_gsaaseq_p(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        AddressedVector av = calcSortedMetric_gsaaseq_p(demetric, sort, order, lvp, ds, template);

        return new ScoredDatasetImpl(av, ds);
    }
    
    public ScoredDataset scoreDataset_gsaaseq_p_direc(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        AddressedVector av = calcSortedMetric_gsaaseq_p_direc(demetric, sort, order, lvp, ds, template);

        return new ScoredDatasetImpl(av, ds);
    }
    
    public ScoredDataset scoreDataset_gsaaseq_p_p(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        AddressedVector av = calcSortedMetric_gsaaseq_p_p(demetric, sort, order, lvp, ds, template);

        return new ScoredDatasetImpl(av, ds);
    }    

    public ScoredDataset scoreDataset_gsaaseqsp_p(
            final List ess,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final int colNum) {

        AddressedVector av = calcSortedMetric_gsaaseqsp_p(ess, sort, order, lvp, colNum);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredDataset scoreDataset_m(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        AddressedVector av = calcSortedMetric_m(metric, sort, order, metricParams, lvp, ds, template, datasets, templates, params, colNum);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredDataset scoreDataset_integrated(
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final List dsExp,
            final String[] rowNames,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        AddressedVector av = calcSortedMetric_integrated(sort, order, lvp, dsExp, rowNames, datasets, templates, params, colNum);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredDataset scoreDataset_shuffleTemplate_gsaasnp_ks(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final int colScore) {

        AddressedVector av = calcSortedMetric_shuffleTemplate_gsaasnp_ks(sort, order, lvp, ds, template, colScore);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredDataset scoreDataset_ssea(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final CMetric cmetric) {

        AddressedVector av = calcSortedMetric_ssea(sort, order, lvp, ds, template, cmetric);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredDataset scoreDataset_gsaasnp_ks(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final int colScore) {

        AddressedVector av = calcSortedMetric_gsaasnp_ks(sort, order, lvp, ds, template, colScore);

        return new ScoredDatasetImpl(av, ds);
    }

    public ScoredStruc scoreDatasetStruc(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        ScoredStruc ss = calcSortedMetricStruc(metric, sort, order, metricParams, lvp, ds, template);
        ss.sds = new ScoredDatasetImpl(ss.av, ds);
        return ss;
    }

    public ScoredStruc scoreDatasetStruc_m(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        ScoredStruc ss = calcSortedMetricStruc_m(metric, sort, order, metricParams, lvp, ds, template, datasets, templates, params, colNum);
        ss.sds = new ScoredDatasetImpl(ss.av, ds);
        return ss;
    }

    public ScoredStruc scoreDatasetStruc_pvalue(
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final List dsExp,
            final String[] rowNames,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        ScoredStruc ss = calcSortedMetricStruc_pvalue(sort, order, lvp, dsExp, rowNames, datasets, templates, params, colNum);
        ss.sds = new ScoredDatasetImpl(ss.av, ds);
        return ss;
    }

    public ScoredStruc scoreDatasetStruc_integrated(
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final List integratedExp,
            final String[] rowNames,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        ScoredStruc ss = calcSortedMetricStruc_integrated(sort, order, lvp, integratedExp, rowNames, datasets, templates, params, colNum);
        ss.sds = new ScoredDatasetImpl(ss.av, ds);
        return ss;
    }

    public ScoredStruc scoreDatasetStruc_snp(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final CMetric cmetric,
            final HashMap snpGeneStruc,
            final String mappingMode,
            final int numOfSnps) {

        ScoredStruc ss = calcSortedMetricStruc_snp(sort, order, lvp, ds, template, cmetric, snpGeneStruc, mappingMode, numOfSnps);
        //ScoredStruc ss = calcSortedMetricStruc(metric, sort, order, metricParams, lvp, ds, template);
        ss.sds = new ScoredDatasetImpl(ss.av, ds);
        return ss;
    }

    public ScoredStruc scoreDatasetStruc_shuffleTemplate_ssea(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        ScoredStruc ss = calcSortedMetricStruc_shuffleTemplate_ssea(sort, order, lvp, ds, template, 0);
        //ScoredStruc ss = calcSortedMetricStruc(metric, sort, order, metricParams, lvp, ds, template);
        ss.sds = new ScoredDatasetImpl(ss.av, ds);
        return ss;
    }

    public ScoredStruc scoreDatasetStruc_shuffleTemplate_gsaasnp(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        ScoredStruc ss = calcSortedMetricStruc_shuffleTemplate_gsaasnp_ks(sort, order, lvp, ds, template, 0);
        //ScoredStruc ss = calcSortedMetricStruc(metric, sort, order, metricParams, lvp, ds, template);
        ss.sds = new ScoredDatasetImpl(ss.av, ds);
        return ss;
    }

    public ScoredDataset scoreDataset(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final DT dt) {

        AddressedVector iv = calcSortedMetric(metric, sort, order, metricParams, lvp, dt.ds, dt.t);

        return new ScoredDatasetImpl(iv, dt.ds);
    }

    public ScoredDataset scoreDataset(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final DatasetTemplate dt) {

        AddressedVector iv = calcSortedMetric(metric, sort, order, metricParams, lvp, dt.getDataset(), dt.getTemplate());
        return new ScoredDatasetImpl(iv, dt.getDataset());
    }

    public ScoredDataset[] scoreDataset(final Metric metric,
            final SortMode[] sorts,
            final Order[] orders,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final DT dt) {

        List dels = _calcUNSOMetric(metric, metricParams, lvp, dt.ds, dt.t);

        ScoredDataset[] sds = new ScoredDataset[orders.length];
        for (int i = 0; i < orders.length; i++) {
            DoubleElement.sort(sorts[i], orders[i], dels);
            sds[i] = new ScoredDatasetImpl(new AddressedVector(dels), dt.ds);
        }

        return sds;
    }

    public ScoredDataset[] scoreDataset(final Metric metric,
            final SortMode[] sorts,
            final Order[] orders,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        List dels = _calcUNSOMetric(metric, metricParams, lvp, ds, template);

        ScoredDataset[] sds = new ScoredDataset[orders.length];
        for (int i = 0; i < orders.length; i++) {
            DoubleElement.sort(sorts[i], orders[i], dels);
            sds[i] = new ScoredDatasetImpl(new AddressedVector(dels), ds);
        }

        return sds;
    }

    public ScoredDataset scoreDataset(final String name,
            final int numfromtop,
            final int numfrombot,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        AddressedVector iv = calcSortedMetric(numfromtop, numfrombot, metric, sort, order, metricParams, lvp, ds, template);

        return new ScoredDatasetImpl(name, iv, ds);
    }

    public ScoredDataset scoreDataset(final int numfromtop,
            final int numfrombot,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        AddressedVector iv = calcSortedMetric(numfromtop, numfrombot, metric, sort, order, metricParams, lvp, ds, template);

        return new ScoredDatasetImpl(iv, ds);
    }

    public LabelledVector scoreDatasetLv(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        AddressedVector iv = calcSortedMetric(metric, sort, order, metricParams, lvp, ds, template);
        return new LabelledVector(iv, ds);
    }

    /**
     * Score AND sort/order a Dataset according to specified parameters
     * Results are sorted/ordered.
     * <p/>
     * Also see the Unsorted version below for better perf in some
     * cases.
     *
     * @param metric
     * @param sort
     * @param order
     * @param ds
     * @param template
     * @param metricParams
     * @return
     */
    public AddressedVector calcSortedMetric(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        return calcSortedMetricStruc(metric, sort, order, metricParams, lvp, ds, template).av;
    }

    public AddressedVector calcSortedMetric_gsaaseq_ks(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        return calcSortedMetricStruc_gsaaseq_ks(demetric, sort, order, lvp, ds, template).av;
    }

    public AddressedVector calcSortedMetric_gsaaseqsp_ks(
            final List ess,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final int colNum) {

        return calcSortedMetricStruc_gsaaseqsp_ks(ess, sort, order, lvp, colNum).av;
    }

    public AddressedVector calcSortedMetric_gsaaseqsp_p(
            final List ess,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final int colNum) {

        return calcSortedMetricStruc_gsaaseqsp_p(ess, sort, order, lvp, colNum).av;
    }

    public AddressedVector calcSortedMetric_gsaaseq_p(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        return calcSortedMetricStruc_gsaaseq_p(demetric, sort, order, lvp, ds, template).av;
    }
    
    public AddressedVector calcSortedMetric_gsaaseq_p_direc(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        return calcSortedMetricStruc_gsaaseq_p_direc(demetric, sort, order, lvp, ds, template).av;
    }
    
    public AddressedVector calcSortedMetric_gsaaseq_p_p(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        return calcSortedMetricStruc_gsaaseq_p_p(demetric, sort, order, lvp, ds, template).av;
    }    

    public AddressedVector calcSortedMetric_m(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        return calcSortedMetricStruc_m(metric, sort, order, metricParams, lvp, ds, template, datasets, templates, params, colNum).av;
    }

    public AddressedVector calcSortedMetric_integrated(
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final List dsExp,
            final String[] rowNames,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        return calcSortedMetricStruc_integrated(sort, order, lvp, dsExp, rowNames, datasets, templates, params, colNum).av;
    }

    public AddressedVector calcSortedMetric_shuffleTemplate_ssea(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final int colScore) {

        return calcSortedMetricStruc_shuffleTemplate_ssea(sort, order, lvp, ds, template, colScore).av;
    }

    public AddressedVector calcSortedMetric_shuffleTemplate_gsaasnp_ks(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final int colScore) {

        return calcSortedMetricStruc_shuffleTemplate_gsaasnp_ks(sort, order, lvp, ds, template, colScore).av;
    }

    public AddressedVector calcSortedMetric_ssea(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final CMetric cmetric) {

        return calcSortedMetricStruc_ssea(sort, order, lvp, ds, template, cmetric).av;
    }

    public AddressedVector calcSortedMetric_gsaasnp_ks(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final int colScore) {

        return calcSortedMetricStruc_gsaasnp_ks(sort, order, lvp, ds, template, colScore).av;
    }

    public ScoredStruc calcSortedMetricStruc(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        /* huh why??
        if (ds.hasProperty("extracted")) {
        log.warn("calcSortedmetric on an extracted ds: " + ds.getName());
        }
         */

        //log.info("Running: " + metric.getName() + " on: " + ds.getName());
        List dels = new ArrayList(ds.getNumRow());
        DoubleElement[] datasetSynchedDels = new DoubleElement[ds.getNumRow()];

        for (int i = 0; i < ds.getNumRow(); i++) {

            double dist = metric.getScore(ds.getRow(i), template, metricParams);

            DoubleElement del = new DoubleElement(i, dist);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);

        lvp.process(dels); // @note

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_gsaaseq_ks(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(ds.getNumRow());
        DoubleElement[] datasetSynchedDels = new DoubleElement[ds.getNumRow()];
        
        double[] descores = new double[ds.getNumRow()];
        try {
            if (demetric.equalsIgnoreCase("Signal2Noise_log2Ratio")) {
                double[] diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template);                
                double[] ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template);
                GsaaSeqTool.normalizeByMeanSdCol(diffs, ratios);
                descores = GsaaSeqTool.combineByMeanOfScores(diffs, ratios);
                double[] descores_direct = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template);
                for(int i=0; i<descores.length; i++){
                    if(descores_direct[i]<0){
                        descores[i]=descores[i]*(-1);
                    }
                }
            } else if (demetric.equalsIgnoreCase("Signal2Noise")) {
                descores = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template);
            } else if (demetric.equalsIgnoreCase("log2Ratio")) {
                descores = GsaaSeqTool.calcScoresLog2RatioSeq(ds, template);
            } else {
                throw (new Exception("Metric for defferential expression analysis is not applicable"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }     

        for (int i = 0; i < ds.getNumRow(); i++) {
            DoubleElement del = new DoubleElement(i, descores[i]);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);

        lvp.process(dels);

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_gsaaseqsp_ks(
            final List ess,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final int colNum) {

        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(ess.size());
        DoubleElement[] datasetSynchedDels = new DoubleElement[ess.size()];

        for (int i = 0; i < ess.size(); i++) {
            double dist_exp = ((Double)((ArrayList)ess.get(i)).get(colNum)).doubleValue();
            DoubleElement del = new DoubleElement(i, dist_exp);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);
        lvp.process(dels);

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_gsaaseqsp_p(
            final List ess,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final int colNum) {

        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(ess.size());
        DoubleElement[] datasetSynchedDels = new DoubleElement[ess.size()];

        for (int i = 0; i < ess.size(); i++) {
            double dist_exp = ((Double)((ArrayList)ess.get(i)).get(colNum)).doubleValue();
            DoubleElement del = new DoubleElement(i, dist_exp);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }
        DoubleElement.sort(sort, order, dels);
        lvp.process(dels);

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }
    
    public ScoredStruc calcSortedMetricStruc_gsaaseq_p_direc(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(ds.getNumRow());
        DoubleElement[] datasetSynchedDels = new DoubleElement[ds.getNumRow()];
        
        double[] descores = new double[ds.getNumRow()];
        try {
            if (demetric.equalsIgnoreCase("Signal2Noise_log2Ratio")) {
                double[] diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template);                
                double[] ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template);
                GsaaSeqTool.normalizeByMeanSdCol(diffs, ratios);
                descores = GsaaSeqTool.combineByMeanOfScores(diffs, ratios);
                double[] descores_direct = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template);
                for(int i=0; i<descores.length; i++){
                    if(descores_direct[i]<0){
                        descores[i]=descores[i]*(-1);
                    }
                }
            } else if (demetric.equalsIgnoreCase("Signal2Noise")) {
                descores = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template);
            } else if (demetric.equalsIgnoreCase("log2Ratio")) {
                descores = GsaaSeqTool.calcScoresLog2RatioSeq(ds, template);
            } else {
                throw (new Exception("Metric for defferential expression analysis is not applicable"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }            

        for (int i = 0; i < ds.getNumRow(); i++) {
            DoubleElement del = new DoubleElement(i, descores[i]);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);
        lvp.process(dels);

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_gsaaseq_p(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(ds.getNumRow());
        DoubleElement[] datasetSynchedDels = new DoubleElement[ds.getNumRow()];
        
        double[] descores = new double[ds.getNumRow()];
        try {
            if (demetric.equalsIgnoreCase("Signal2Noise_log2Ratio")) {
                double[] diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template);                
                double[] ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template);
                GsaaSeqTool.normalizeByMeanSdCol(diffs, ratios);
                descores = GsaaSeqTool.combineByMeanOfScores(diffs, ratios);
            } else if (demetric.equalsIgnoreCase("Signal2Noise")) {
                descores = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template);
            } else if (demetric.equalsIgnoreCase("log2Ratio")) {
                descores = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template);
            } else {
                throw (new Exception("Metric for defferential expression analysis is not applicable"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }            

        for (int i = 0; i < ds.getNumRow(); i++) {
            DoubleElement del = new DoubleElement(i, descores[i]);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);
        lvp.process(dels);

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }
    
    public ScoredStruc calcSortedMetricStruc_gsaaseq_p_p(
            final String demetric,            
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(ds.getNumRow());
        DoubleElement[] datasetSynchedDels = new DoubleElement[ds.getNumRow()];
        
        double[] descoresP = new double[ds.getNumRow()];        
        try {
            if (demetric.equalsIgnoreCase("Signal2Noise_log2Ratio")) {
                double[] diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template);                
                double[] ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template);
                descoresP = GsaaSeqTool.calcPMoreThan(diffs, ratios);
            } else if (demetric.equalsIgnoreCase("Signal2Noise")) {
                double[] diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template);
                descoresP = GsaaSeqTool.calcPMoreThan(diffs);
            } else if (demetric.equalsIgnoreCase("log2Ratio")) {
                double[] ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template);
                descoresP = GsaaSeqTool.calcPMoreThan(ratios);
            } else {
                throw (new Exception("Metric for defferential expression analysis is not applicable"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }          

        for (int i = 0; i < ds.getNumRow(); i++) {
            DoubleElement del = new DoubleElement(i, descoresP[i]);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);
        lvp.process(dels);

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }    

    //normalize datasets by mean_sd
    public ScoredStruc calcSortedMetricStruc_m(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(ds.getNumRow());
        DoubleElement[] datasetSynchedDels = new DoubleElement[ds.getNumRow()];

        List delsExp = new ArrayList(ds.getNumRow());
        List signs = new ArrayList();

        List delsCpn = null;
        HashMap cpnGeneStruc = null;
        Template templateCpn = null;
        if (datasets.containsKey("CPN")) {
            delsCpn = new ArrayList(ds.getNumRow());
            cpnGeneStruc = (HashMap) (datasets.get("CPN"));
            templateCpn = (Template) (templates.get("CPN"));
        }

        List delsMet = null;
        HashMap metGeneStruc = null;
        Template templateMet = null;
        if (datasets.containsKey("MET")) {
            delsMet = new ArrayList(ds.getNumRow());
            metGeneStruc = (HashMap) (datasets.get("MET"));
            templateMet = (Template) (templates.get("MET"));
        }

        List delsRna = null;
        HashMap rnaGeneStruc = null;
        Template templateRna = null;
        if (datasets.containsKey("RNA")) {
            delsRna = new ArrayList(ds.getNumRow());
            rnaGeneStruc = (HashMap) (datasets.get("RNA"));
            templateRna = (Template) (templates.get("RNA"));
        }

        List delsSnp = null;
        HashMap snpGeneStruc = null;
        //Template templateSnp = null;
        //HashMap paramsSnp = null;
        //String mappingModeSnp = null;
        //String numOfSnps = null;
        //CMetric cmetric = null;

        if (datasets.containsKey("SNP")) {
            delsSnp = new ArrayList(ds.getNumRow());
            snpGeneStruc = (HashMap) (datasets.get("SNP"));
            //templateSnp = (Template) (templates.get("SNP"));
            //paramsSnp = (HashMap) (params.get("SNP"));
            //mappingModeSnp = (String) (paramsSnp.get("MAPPING_MODE"));
            //numOfSnps = (String) (paramsSnp.get("NUM_OF_SNPS"));
            //cmetric = (CMetric) (paramsSnp.get("METRIC"));
        }

        String[] rowNames = ds.getRowNamesArray();
        int code = 0;
        for (int i = 0; i < ds.getNumRow(); i++) {
            //expression
            double dist_exp = metric.getScore(ds.getRow(i), template, metricParams);
            DoubleElement del_exp = new DoubleElement(i, dist_exp);
            delsExp.add(del_exp);

            String geneName = rowNames[i];
            if (dist_exp >= 0) {
                signs.add("p");
            } else {
                signs.add("n");
            }

            //copy number
            if (cpnGeneStruc != null && cpnGeneStruc.size() != 0) {
                ArrayList cpns = (ArrayList) cpnGeneStruc.get(geneName);
                double dist_cpn = 0;
                if (cpns != null && cpns.size() != 0) {
                    float[] fs = new float[cpns.size()];
                    for (int j = 0; j < cpns.size(); j++) {
                        float f = ((Float) cpns.get(j)).floatValue();
                        fs[j] = f;
                    }
                    Vector cv = new Vector(fs);
                    dist_cpn = Math.abs(metric.getScore(cv, templateCpn, metricParams));
                }
                if (dist_exp < 0) {
                    dist_cpn = 0 - dist_cpn;
                }
                DoubleElement del_cpn = new DoubleElement(i, dist_cpn);
                delsCpn.add(del_cpn);
            }

            //methylation
            if (metGeneStruc != null && metGeneStruc.size() != 0) {
                ArrayList mets = (ArrayList) metGeneStruc.get(geneName);
                double dist_met = 0;
                if (mets != null && mets.size() != 0) {
                    float[] fs = new float[mets.size()];
                    for (int j = 0; j < mets.size(); j++) {
                        float f = ((Float) mets.get(j)).floatValue();
                        fs[j] = f;
                    }
                    Vector mv = new Vector(fs);
                    dist_met = Math.abs(metric.getScore(mv, templateMet, metricParams));
                }
                if (dist_exp < 0) {
                    dist_met = 0 - dist_met;
                }
                DoubleElement del_met = new DoubleElement(i, dist_met);
                delsMet.add(del_met);
            }

            //microRNA
            if (rnaGeneStruc != null && rnaGeneStruc.size() != 0) {
                ArrayList rnas = (ArrayList) rnaGeneStruc.get(geneName);
                double dist_rna = 0;
                if (rnas != null && rnas.size() != 0) {
                    float[] fs = new float[rnas.size()];
                    for (int j = 0; j < rnas.size(); j++) {
                        float f = ((Float) rnas.get(j)).floatValue();
                        fs[j] = f;
                    }
                    Vector rv = new Vector(fs);
                    dist_rna = Math.abs(metric.getScore(rv, templateRna, metricParams));
                }
                if (dist_exp < 0) {
                    dist_rna = 0 - dist_rna;
                }
                DoubleElement del_rna = new DoubleElement(i, dist_rna);
                delsRna.add(del_rna);
            }

            //SNP
            //StringTokenizer st = new StringTokenizer(geneName, "///");
            //geneName = st.nextToken().trim();
            if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                double dist_snp = 0;
                ArrayList al = (ArrayList) snpGeneStruc.get(geneName);
                if (al != null) {
                    dist_snp = (double) ((Float) al.get(colNum)).floatValue();
                }

                if (dist_exp < 0) {
                    dist_snp = 0 - dist_snp;
                }
                DoubleElement del_snp = new DoubleElement(i, dist_snp);
                delsSnp.add(del_snp);
            }
        }

        lvp.process(delsExp);
        if (delsCpn != null && delsCpn.size() != 0) {
            lvp.process(delsCpn);
        }
        if (delsMet != null && delsMet.size() != 0) {
            lvp.process(delsMet);
        }
        if (delsRna != null && delsRna.size() != 0) {
            lvp.process(delsRna);
        }
        if (delsSnp != null && delsSnp.size() != 0) {
            lvp.process(delsSnp);
        }

        for (int i = 0;
                i < datasetSynchedDels.length;
                i++) {
            double dist = 0;
            double dist_exp = ((DoubleElement) delsExp.get(i)).fValue;
            dist += dist_exp;
            if (delsCpn != null && delsCpn.size() != 0) {
                double dist_cpn = ((DoubleElement) delsCpn.get(i)).fValue;
                dist += dist_cpn;
            }
            if (delsMet != null && delsMet.size() != 0) {
                double dist_met = ((DoubleElement) delsMet.get(i)).fValue;
                dist += dist_met;
            }
            if (delsRna != null && delsRna.size() != 0) {
                double dist_rna = ((DoubleElement) delsRna.get(i)).fValue;
                dist += dist_rna;
            }
            if (delsSnp != null && delsSnp.size() != 0) {
                double dist_snp = ((DoubleElement) delsSnp.get(i)).fValue;
                dist += dist_snp;
            }
            DoubleElement del = new DoubleElement(i, dist);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        double min = 0;
        for (int i = 0; i < datasetSynchedDels.length; i++) {
            double value = datasetSynchedDels[i].fValue;
            if (value < min) {
                min = value;
            }
        }
        for (int i = 0; i < datasetSynchedDels.length; i++) {
            double value = ((DoubleElement) dels.get(i)).fValue - min;
            if (((String) signs.get(i)).equalsIgnoreCase("n")) {
                value = 0 - value;
            }
            DoubleElement del = new DoubleElement(i, value);
            dels.set(i, del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);
        return new ScoredStruc(datasetSynchedDels,
                new AddressedVector(dels));
    }

    //fisher's method
    public ScoredStruc calcSortedMetricStruc_pvalue(
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final List dsExp,
            final String[] rowNames,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        if (dsExp == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(rowNames.length);
        DoubleElement[] datasetSynchedDels = new DoubleElement[rowNames.length];

        List delsExp = new ArrayList(rowNames.length);

        List delsCpn = null;
        HashMap cpnGeneStruc = null;
        Template templateCpn = null;
        if (datasets.containsKey("CPN")) {
            delsCpn = new ArrayList(rowNames.length);
            cpnGeneStruc = (HashMap) (datasets.get("CPN"));
            templateCpn = (Template) (templates.get("CPN"));
        }

        List delsMet = null;
        HashMap metGeneStruc = null;
        Template templateMet = null;
        if (datasets.containsKey("MET")) {
            delsMet = new ArrayList(rowNames.length);
            metGeneStruc = (HashMap) (datasets.get("MET"));
            templateMet = (Template) (templates.get("MET"));
        }

        List delsRna = null;
        HashMap rnaGeneStruc = null;
        Template templateRna = null;
        if (datasets.containsKey("RNA")) {
            delsRna = new ArrayList(rowNames.length);
            rnaGeneStruc = (HashMap) (datasets.get("RNA"));
            templateRna = (Template) (templates.get("RNA"));
        }

        List delsSnp = null;
        HashMap snpGeneStruc = null;
        //Template templateSnp = null;
        //HashMap paramsSnp = null;
        //String mappingModeSnp = null;
        //String numOfSnps = null;
        //CMetric cmetric = null;

        if (datasets.containsKey("SNP")) {
            delsSnp = new ArrayList(rowNames.length);
            snpGeneStruc = (HashMap) (datasets.get("SNP"));
            //templateSnp = (Template) (templates.get("SNP"));
            //paramsSnp = (HashMap) (params.get("SNP"));
            //mappingModeSnp = (String) (paramsSnp.get("MAPPING_MODE"));
            //numOfSnps = (String) (paramsSnp.get("NUM_OF_SNPS"));
            //cmetric = (CMetric) (paramsSnp.get("METRIC"));
        }

        for (int i = 0; i < rowNames.length; i++) {
            //expression
            ArrayList pvalues = new ArrayList();

            double dist_exp = 0;
            ArrayList al_exp = (ArrayList) dsExp.get(i);
            dist_exp = (double) ((Float) al_exp.get(colNum)).floatValue();
            //DoubleElement del_exp = new DoubleElement(i, dist_exp);
            //delsExp.add(del_exp);
            delsExp.add(new Double(dist_exp));

            //SNP
            //StringTokenizer st = new StringTokenizer(geneName, "///");
            //geneName = st.nextToken().trim();
            if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                double dist_snp = 0;
                ArrayList al_snp = (ArrayList) snpGeneStruc.get(rowNames[i]);
                if (al_snp != null) {
                    dist_snp = (double) ((Float) al_snp.get(colNum)).floatValue();
                }
                //DoubleElement del_snp = new DoubleElement(i, dist_snp);
                //delsSnp.add(del_snp);
                delsSnp.add(new Double(dist_snp));
            }
        }

        /*lvp.process(delsExp);
        if (delsCpn != null && delsCpn.size() != 0) {
        lvp.process(delsCpn);
        }
        if (delsMet != null && delsMet.size() != 0) {
        lvp.process(delsMet);
        }
        if (delsRna != null && delsRna.size() != 0) {
        lvp.process(delsRna);
        }
        if (delsSnp != null && delsSnp.size() != 0) {
        lvp.process(delsSnp);
        }*/

        for (int i = 0; i < datasetSynchedDels.length; i++) {
            double dist = 0;
            ArrayList pvalues = new ArrayList();
            double dist_exp_ori = ((Double) delsExp.get(i)).doubleValue();
            double dist_exp = Math.abs(dist_exp_ori);
            pvalues.add(new Double(dist_exp));
            if (delsSnp != null && delsSnp.size() != 0) {
                double dist_snp = ((Double) delsSnp.get(i)).doubleValue();
                pvalues.add(new Double(dist_snp));
            } else {
                double dist_snp = 1;
                pvalues.add(new Double(dist_snp));
            }
            double x2 = fisherTest(pvalues);
            if (dist_exp_ori < 0) {
                x2 = x2 * (-1);
            }
            DoubleElement del = new DoubleElement(i, x2);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        lvp.process(dels);

        DoubleElement.sort(sort, order, dels);
        return new ScoredStruc(datasetSynchedDels,
                new AddressedVector(dels));
    }

    //integrated method
    public ScoredStruc calcSortedMetricStruc_integrated(
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final List dsintegrated,
            final String[] rowNames,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final int colNum) {

        if (dsintegrated == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        List dels = new ArrayList(rowNames.length);
        DoubleElement[] datasetSynchedDels = new DoubleElement[rowNames.length];

        for (int i = 0; i < datasetSynchedDels.length; i++) {
            double score = ((Double) ((ArrayList) dsintegrated.get(i)).get(colNum)).doubleValue();
            DoubleElement del = new DoubleElement(i, score);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        lvp.process(dels);

        DoubleElement.sort(sort, order, dels);
        return new ScoredStruc(datasetSynchedDels,
                new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_snp(//final Metric metric,

            final SortMode sort,
            final Order order,
            //final Map metricParams,

            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final CMetric cmetric,
            final HashMap snpGeneStruc,
            final String mappingMode,
            final int numOfSnps) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

// DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        /* huh why??
        if (ds.hasProperty("extracted")) {
        log.warn("calcSortedmetric on an extracted ds: " + ds.getName());
        }
         */

//log.info("Running: " + metric.getName() + " on: " + ds.getName());
        List dels = new ArrayList(snpGeneStruc.size());
        DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];

        String[] rowNames = ds.getRowNamesArray();
        int code = 0;
        for (int i = 0; i <
                ds.getNumRow(); i++) {
            //Qing Xiong
            String geneName = rowNames[i];

            //StringTokenizer st = new StringTokenizer(geneName, "///");
            //geneName = st.nextToken().trim();
            ArrayList snps = (ArrayList) snpGeneStruc.get(geneName);

            double dist_snp = 0;

            if (snps != null && snps.size() != 0) {

                ArrayList snp = (ArrayList) snps.get(0);

                if (code == 0) {
                    code = getCode(snp);
                }

                ArrayList dist_snp_all = new ArrayList();
                for (int j = 0; j <
                        snps.size(); j++) {
                    dist_snp_all.add(cmetric.getScore((ArrayList) snps.get(j), template, code));
                }

                if (dist_snp_all != null && dist_snp_all.size() != 0) {
                    Collections.sort(dist_snp_all);
                    Collections.reverse(dist_snp_all);
                    if (mappingMode.equalsIgnoreCase("single")) {
                        dist_snp = ((Double) dist_snp_all.get(0)).doubleValue();
                    } else if (mappingMode.equalsIgnoreCase("all")) {
                        for (int k = 0; k <
                                dist_snp_all.size(); k++) {
                            dist_snp += ((Double) dist_snp_all.get(k)).doubleValue();
                        }

                    } else if (mappingMode.equalsIgnoreCase("average")) {
                        for (int k = 0; k <
                                dist_snp_all.size(); k++) {
                            dist_snp += ((Double) dist_snp_all.get(k)).doubleValue();
                        }

                        dist_snp = dist_snp / dist_snp_all.size();
                    } else {
                        for (int k = 0; k <
                                numOfSnps; k++) {
                            dist_snp += ((Double) dist_snp_all.get(k)).doubleValue();
                            if (k == dist_snp_all.size() - 1) {
                                break;
                            }

                        }
                    }
                }
            }

            DoubleElement del = new DoubleElement(i, dist_snp);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);

        lvp.process(dels); // @note

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_shuffleTemplate_ssea(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final int colScore) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        String[] rowNames = ds.getRowNamesArray();
        List dels = new ArrayList(rowNames.length);
        DoubleElement[] datasetSynchedDels = new DoubleElement[rowNames.length];

        Matrix m = ds.getMatrix();
        int numCol = m.getNumCol();
        for (int i = 0; i < ds.getNumRow(); i++) {
            //real score is in first column
            double dist_snp = m.getElement(i * numCol + colScore);
            DoubleElement del = new DoubleElement(i, dist_snp);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);
        lvp.process(dels); // @note

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_shuffleTemplate_gsaasnp_ks(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final int colScore) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        String[] rowNames = ds.getRowNamesArray();
        List dels = new ArrayList(rowNames.length);
        DoubleElement[] datasetSynchedDels = new DoubleElement[rowNames.length];

        Matrix m = ds.getMatrix();
        int numCol = m.getNumCol();
        for (int i = 0; i < ds.getNumRow(); i++) {
            //real score is in first column
            double dist_snp = m.getElement(i * numCol + colScore);
            DoubleElement del = new DoubleElement(i, dist_snp);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);
        lvp.process(dels); // @note

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_ssea(//final Metric metric,

            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final CMetric cmetric) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // DONT check for Template -- it can be null for some metrics
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        /* huh why??
        if (ds.hasProperty("extracted")) {
        log.warn("calcSortedmetric on an extracted ds: " + ds.getName());
        }
         */

        //log.info("Running: " + metric.getName() + " on: " + ds.getName());
        String[] rowNames = ds.getRowNamesArray();
        List dels = new ArrayList(rowNames.length);
        DoubleElement[] datasetSynchedDels = new DoubleElement[rowNames.length];

        Matrix m = ds.getMatrix();
        for (int i = 0; i < ds.getNumRow(); i++) {
            double dist_snp = m.getElement(i);
            DoubleElement del = new DoubleElement(i, dist_snp);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);

        lvp.process(dels); // @note

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    public ScoredStruc calcSortedMetricStruc_gsaasnp_ks(//final Metric metric,
            final SortMode sort,
            final Order order,
            //final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final int colScore) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        String[] rowNames = ds.getRowNamesArray();
        List dels = new ArrayList(rowNames.length);
        DoubleElement[] datasetSynchedDels = new DoubleElement[rowNames.length];

        Matrix m = ds.getMatrix();
        int numCol = m.getNumCol();
        for (int i = 0; i < ds.getNumRow(); i++) {
            //real score is in first column
            double dist_snp = m.getElement(i * numCol + colScore);
            DoubleElement del = new DoubleElement(i, dist_snp);
            dels.add(del);
            datasetSynchedDels[i] = del;
        }

        DoubleElement.sort(sort, order, dels);
        lvp.process(dels); // @note

        return new ScoredStruc(datasetSynchedDels, new AddressedVector(dels));
    }

    /**
     * Inner class representing ???
     */
    public static class ScoredStruc {

        public AddressedVector av;
        public DoubleElement[] datasetSynchedDels; // synched with the dataset that was scored, but not score sorted
        public ScoredDataset sds;

        ScoredStruc(DoubleElement[] dels, AddressedVector av) {
            this.datasetSynchedDels = dels;
            this.av = av;
        }

        public float[] getDatasetSynchedScores() {
            float[] fls = new float[datasetSynchedDels.length];
            for (int f = 0; f < datasetSynchedDels.length; f++) {
                fls[f] = (float) datasetSynchedDels[f].fValue;
            }

            return fls;
        }
    }

    public List _calcUNSOMetric(
            final Metric metric,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        if (template == null) {
            throw new IllegalArgumentException("Param template cannot be null");
        }

        if (metric == null) {
            throw new IllegalArgumentException("Param metric cannot be null");
        }

// DONT check for Template -- it can be null for some metrics
//log.info("Running: " + metric.getName() + " on: " + ds.getName());
        List dels = new ArrayList(ds.getNumRow());

        for (int i = 0; i <
                ds.getNumRow(); i++) {
            double dist = metric.getScore(ds.getRow(i), template, metricParams);
            dels.add(new DoubleElement(i, dist));
        }

        lvp.process(dels); // @note

        return dels;
    }

    /**
     * Similar to calcSortedMetric, except that only specified number of markers are used.
     *
     * @param numfromtop
     * @param numfrombot
     * @param metric
     * @param sort
     * @param order
     * @param ds
     * @param template
     * @param metricParams
     * @return
     */
    public AddressedVector calcSortedMetric(
            final int numfromtop,
            final int numfrombot,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        if (numfromtop > ds.getNumRow()) {
            throw new IllegalArgumentException("Too many top markers requested: " + numfromtop + " Dataset size is: " + ds.getNumRow());
        }

        if (numfrombot > ds.getNumRow()) {
            throw new IllegalArgumentException("Too many bottom markers requested: " + numfromtop + " Dataset size is: " + ds.getNumRow());
        }

        AddressedVector iv = calcSortedMetric(metric, sort, order, metricParams, lvp, ds, template);

        int[] indices = new int[numfromtop + numfrombot];
        Vector scores = new Vector(numfromtop + numfrombot);
        int i;
        for (i = 0; i <
                numfromtop; i++) {
            indices[i] = iv.getAddress(i);
            scores.setElement(i, iv.getScore(i));
        }

        for (int j = iv.getSize() - numfrombot; j <
                iv.getSize(); j++) {
            indices[i] = iv.getAddress(j);
            scores.setElement(i, iv.getScore(j));
            i++;

        }



        return new AddressedVector(scores, indices, true);
    }

    /**
     * An UNsorted version of calcMetric()
     * More efficient to use if not interested in sorting a the vector
     * after creation.
     *
     * @param metric
     * @param ds
     * @param template
     * @param params
     * @return
     */
    public Vector calcUNSOMetric(
            final Metric metric,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {
        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        if (metric == null) {
            throw new IllegalArgumentException("Param metric cannot be null");
        }

//log.info("Running: " + metric.getName() + " on: " + ds.getName());
        Vector scores = new Vector(ds.getNumRow());
        for (int i = 0; i <
                ds.getNumRow(); i++) {
            scores.setElement(i, (float) metric.getScore(ds.getRow(i), template, metricParams));
        }

        return lvp.process(scores); // @note
    }

    public Dataframe calcUNSOMetrics(
            final Metric[] metrics,
            final Map[] metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        if (metrics.length != metricParams.length) {
            throw new MismatchedSizeException("metrics", metrics, "metricParams", metricParams);
        }

        Matrix matrix = new Matrix(ds.getNumRow(), metrics.length);
        List colNames = new ArrayList();

        for (int m = 0; m <
                metrics.length; m++) {
            Vector scores = calcUNSOMetric(metrics[m], metricParams[m], lvp, ds, template);
            matrix.setColumn(m, scores);
            colNames.add(metrics[m].getName());
        }

        return new Dataframe("all_metrics_for_" + template.getName(), matrix, ds.getRowNames(), colNames, true);
    }

    public Dataframe calcUNSOMetrics(
            final Metric[] metrics,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template) {

        Map[] mps = new Map[metrics.length];
        for (int m = 0; m <
                metrics.length; m++) {
            mps[m] = metricParams;
        }

        return calcUNSOMetrics(metrics, mps, lvp, ds, template);
    }

    /**
     * Permutation test style scoring a dataset with none of the frills of the PermutationTest class
     *
     * @param nperms
     * @param metric
     * @param sort
     * @param order
     * @param topX
     * @param ds
     * @param t
     * @return
     */
    public Matrix scoreAndPermuteDataset(
            final int nperms,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final int topX,
            final Dataset ds,
            final Template t,
            final RandomSeedGenerator rst) {
        // first cat analysis
        // find the topX markers
        Matrix matrix = new Matrix(topX, 1 + nperms); // each col represents a perm and each row the top X ranked marker
        ScoredDataset sds = scoreDataset(metric, sort, order, metricParams, lvp, ds, t);
        matrix.setColumn(0, sds.extractRanked(topX, true).getScoresV(false));

        Template[] rndCatTemplates = TemplateFactoryRandomizer.createRandomTemplates(nperms, t, rst);
        for (int c = 0; c <
                rndCatTemplates.length; c++) {
            ScoredDataset rndSds = scoreDataset(metric, sort, order, metricParams, lvp, ds, rndCatTemplates[c]);
            matrix.setColumn(c + 1, rndSds.extractRanked(topX, true).getScoresV(false));
        }

        return matrix;
    }

    public RankedFeature[] getTopXMarkers(final int topX,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template t) {


        if (topX > ds.getNumRow()) {
            throw new IllegalArgumentException("topX: " + topX + " cannot be > than num of rows in dataset: " + ds.getNumRow());
        }

        ScoredDataset sds = scoreDataset(metric, sort, order, metricParams, lvp, ds, t);
        DefaultRankedFeature[] topXf = new DefaultRankedFeature[topX];

        for (int i = 0; i <
                topX; i++) {
            topXf[i] = new DefaultRankedFeature(sds.getRowName(i), i, sds.getScore(i));
        }

        return topXf;
    }

    public GeneSet getTopXMarkersAsGeneSet(
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final int topX,
            final Dataset ds,
            final Template t) {


        if (topX > ds.getNumRow()) {
            throw new IllegalArgumentException("topX: " + topX + " cannot be > than num of rows in dataset: " + ds.getNumRow());
        }

        ScoredDataset sds = scoreDataset(metric, sort, order, metricParams, lvp, ds, t);
        Set topXf = new HashSet();

        for (int i = 0; i <
                topX; i++) {
            String name = sds.getRowName(i);
            if (topXf.contains(name)) {
                throw new IllegalStateException("Duplicate row: " + name);
            }

            topXf.add(name);
        }

        return new FSet(ds.getName() + "_" + t.getName() + "_" + topX + "markers", topXf);
    }

    public GeneSet[] getTopXMarkers(final int[] topXss,
            final Metric metric,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template t) {

        Order order = Order.DESCENDING;

        ScoredDataset sds = scoreDataset(metric, SortMode.REAL, order, metricParams, lvp, ds, t);

        GeneSet[] gsets = new GeneSet[topXss.length];
        for (int i = 0; i <
                topXss.length; i++) {
            gsets[i] = sds.getNamesOfUpOrDnXRanks_gs(topXss[i], true);
        }

        return gsets;
    }

    public GeneSet getTopXMarker(
            final int topX,
            final Metric metric,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template t) {

        Order order = Order.DESCENDING;

        ScoredDataset sds = scoreDataset(metric, SortMode.REAL, order, metricParams, lvp, ds, t);
        return sds.getNamesOfUpOrDnXRanks_gs(topX, true);
    }

//Qing Xiong
    public int getCode(ArrayList al) {
        ArrayList genoLabel = new ArrayList();
        for (int j = 0; j < al.size(); j++) {
            for (int k = 0; k <
                    ((String) al.get(j)).length(); k++) {
                String s = ((String) al.get(j)).substring(k, k + 1);
                int flag = 0;
                for (int l = 0; l <
                        genoLabel.size(); l++) {
                    if (s.equalsIgnoreCase((String) genoLabel.get(l))) {
                        flag++;
                    }

                }
                if (flag == 0) {
                    genoLabel.add(s);
                }

            }
        }

        int code = 0;
        int flag = 0;

        for (int j = 0; j <
                genoLabel.size(); j++) {
            String s = (String) genoLabel.get(j);
            if (s.equalsIgnoreCase("0") || s.equalsIgnoreCase("1") || s.equalsIgnoreCase("2")) {
                flag++;
                code =
                        1; //0-1-2-3
                break;

            }


            if (s.equalsIgnoreCase("B")) {
                flag++;
                code =
                        2; //AA-AB-BB
                break;

            }


        }
        if (flag == 0) {
            code = 3;
        }

        return code;
    }

    public double fisherTest(ArrayList pvalues) {
        double x2 = 0;
        for (int i = 0; i < pvalues.size(); i++) {
            x2 += Math.log(((Double) pvalues.get(i)).doubleValue());
        }
        x2 = (-2) * x2;
        return x2;
    }
    /*public ArrayList[] splitByTemplateClass(final ArrayList profile, final Template template) {
    ArrayListTool.printArrayListS(profile);
    final ArrayList[] lists = new ArrayList[template.getNumClasses()];

    // @note IMP: within a class the items need not be ordered by asc profile pos
    for (int i = 0; i < template.getNumClasses(); i++) {
    Template.Class cl = template.getClass(i); // @note IMP simply get the class NOT get teh class by first profile pos
    ArrayList list = new ArrayList();

    for (int p = 0; p < cl.getSize(); p++) {
    int pos = cl.getItem(p).getProfilePosition();
    list.add(profile.get(pos));
    }

    lists[i] = list;
    }

    return lists;
    }*/
}    // End DatasetMetrics

class ArraySort implements Comparator<ArrayList> {

    private int columnNumber;
    private String order;

    public ArraySort(int columnNumber, String order) {
        this.columnNumber = columnNumber;
        this.order = order;
    }

    public int compare(ArrayList a, ArrayList b) {
        if ("desc".equals(order)) {
            double diff = ((DoubleElement) b.get(columnNumber)).fValue - ((DoubleElement) a.get(columnNumber)).fValue;
            if (diff > 0) {
                return 1;
            } else if (diff == 0) {
                return 0;
            } else {
                return -1;
            }
        } else {
            double diff = ((DoubleElement) a.get(columnNumber)).fValue - ((DoubleElement) b.get(columnNumber)).fValue;
            if (diff > 0) {
                return 1;
            } else if (diff == 0) {
                return 0;
            } else {
                return -1;
            }
        }
    }
}
