package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import org.apache.log4j.Logger;

import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: May 2, 2006
 *         Time: 8:03:45 PM
 */
public class DatasetNormalizers {

    private final Logger log = XLogger.getLogger(DatasetGenerators.class);

    public static String[] NORM_MODES = new String[]
            {
                    "none", // 0
                    "col", // 1
                    "row", // 2
                    "col_rank",  // 3
                    "log", // 4
                    "zscore_mean", // 5
                    "zscore_median", // 6
                    "zscore_mad_mean",  // 7
                    "zscore_mad_median",  // 8
                    "cmap", // 9
                    "cmap2", // 10
                    "gsea" // 11
            };

    /**
     * Class constructor
     */
    public DatasetNormalizers() {
    }

    public static Dataset normalizeByDiagonal(final Dataset orig) {

        Matrix m = orig.getMatrix().cloneDeep();
        m = m.normalize(true);

        return new DefaultDataset(orig.getName() + "_norm", m, orig.getRowNamesArray(), orig.getColumnNamesArray(), true);
    }

    // -------------------------------------------------------------------------------------------- //

    /**
     * Inner class with nirm details
     */
    public static class NormDetail {

        private Dataset fNormDataset;

        private float[] scale_factors;

        private int refIndex;

        public int getRefIndex() {
            return refIndex;
        }

        public float[] getScaleFactors() {
            return scale_factors;
        }

        public LabelledVector getScaleFactors_lv() {
            return new LabelledVector("scale_factors", fNormDataset.getColumnNamesArray(), getScaleFactors());
        }

        public Dataset getDatasetNormalized() {
            return fNormDataset;
        }

    } // End class NormDetail

    /**
     * @param orig_ds
     * @param scaleFactorFromTheseGenesOnly_opt
     *
     * @return
     */
    public NormDetail normalizeDatasetByRegressionRefIsMedian(final Dataset orig_ds, final GeneSet scaleFactorFromTheseGenesOnly_opt) {
        final NormDetail nd = new NormDetail();

        Dataset use_ds_for_stats;
        if (scaleFactorFromTheseGenesOnly_opt == null) {
            use_ds_for_stats = orig_ds;
        } else {
            use_ds_for_stats = new DatasetGenerators().extractRows(orig_ds, scaleFactorFromTheseGenesOnly_opt);
        }

        final int numCols = orig_ds.getNumCol();
        final Vector medians = new Vector(numCols);
        final Vector[] vss_for_stats = new Vector[numCols]; // cloning here so that we can use below again w/o remaking
        Vector[] full_size_vectors = new Vector[numCols];

        for (int c = 0; c < numCols; c++) {
            vss_for_stats[c] = use_ds_for_stats.getColumn(c);
            medians.setElement(c, vss_for_stats[c].median());
        }

        nd.refIndex = medians.medianIndex();
        nd.scale_factors = new float[numCols];

        log.debug("Median scan is: " + use_ds_for_stats.getColumnName(nd.refIndex));

        if (scaleFactorFromTheseGenesOnly_opt == null) {
            full_size_vectors = vss_for_stats;
        } else {
            for (int c = 0; c < numCols; c++) {
                full_size_vectors[c] = orig_ds.getColumn(c);
            }
        }

        for (int c = 0; c < numCols; c++) {
            if (c != nd.refIndex) {
                final double slope = XMath.regressionSlope(vss_for_stats[c], vss_for_stats[nd.refIndex]);
                float scale_factor = 1.0f / (float) slope; // @note
                full_size_vectors[c].divide(scale_factor);
                nd.scale_factors[c] = (float) slope;
            }
        }

        final Matrix m = new Matrix(full_size_vectors, false, false);

        nd.fNormDataset = new DefaultDataset(use_ds_for_stats.getName() + "_norm", m, use_ds_for_stats.getRowNames(), use_ds_for_stats.getColumnNames(), true, false, false, use_ds_for_stats.getAnnot());

        return nd;
    }

    public NormDetail normalizeDatasetByRegression(final Dataset orig_ds, final LabelledVector normByThisx, final GeneSet scaleFactorFromTheseGenesOnly_opt) {

        final NormDetail nd = new NormDetail();

        Dataset use_ds_for_stats;
        if (scaleFactorFromTheseGenesOnly_opt == null) {
            use_ds_for_stats = orig_ds;
        } else {
            use_ds_for_stats = new DatasetGenerators().extractRows(orig_ds, scaleFactorFromTheseGenesOnly_opt);
        }

        Vector ref_vector = new Vector(normByThisx.getSize());
        for (int r = 0; r < use_ds_for_stats.getNumRow(); r++) {
            ref_vector.setElement(r, normByThisx.getScore(use_ds_for_stats.getRowName(r)));
        }

        final int numCols = orig_ds.getNumCol();
        final Vector[] vss_for_scaling_stats = new Vector[numCols]; // cloning here so that we can use below again w/o remaking
        Vector[] full_size_vectors = new Vector[numCols];

        for (int c = 0; c < numCols; c++) {
            vss_for_scaling_stats[c] = use_ds_for_stats.getColumn(c);
        }

        if (scaleFactorFromTheseGenesOnly_opt == null) {
            full_size_vectors = vss_for_scaling_stats;
        } else {
            for (int c = 0; c < numCols; c++) {
                full_size_vectors[c] = orig_ds.getColumn(c);
            }
        }

        nd.scale_factors = new float[numCols];
        for (int c = 0; c < numCols; c++) {
            final double slope = XMath.regressionSlope(vss_for_scaling_stats[c], ref_vector);
            float scale_factor = 1.0f / (float) slope; // @note
            //full_size_vectors[c].divide(scale_factor);
            full_size_vectors[c].multiply(scale_factor);
            nd.scale_factors[c] = (float) slope;
        }

        final Matrix m = new Matrix(full_size_vectors, false, false);

        nd.fNormDataset = new DefaultDataset(use_ds_for_stats.getName() + "_norm", m, use_ds_for_stats.getRowNames(), use_ds_for_stats.getColumnNames(), true, false, false, use_ds_for_stats.getAnnot());

        return nd;
    }

    public NormDetail normalizeDatasetByRegressionRefIsMedian(final Dataset ds) {
        return normalizeDatasetByRegressionRefIsMedian(ds, null);
    }

    // -------------------------------------------------------------------------------------------- //

    public Dataset normalize(final String newName,
                             final Dataset origDs,
                             final int index) {
        return normalize(newName, origDs, SortMode.REAL, Order.DESCENDING, index);
    }

    public Dataset normalize(final Dataset origDs,
                             final int index) {
        final String suffix = DatasetNormalizers.NORM_MODES[index];
        final String newName = origDs.getName() + "_" + suffix;
        return normalize(newName, origDs, SortMode.REAL, Order.DESCENDING, index);
    }

    public Dataset normalize(final String newName,
                             final Dataset origDs,
                             final SortMode sort,
                             final Order order,
                             final int index) {

        Matrix matrix = new Matrix(origDs.getNumRow(), origDs.getNumCol());

        if (index == 0) {
            log.info("Doing nothing");
            return origDs;
        } else if (index == 1) {
            log.info("Doing column value norm");
            for (int c = 0; c < origDs.getNumCol(); c++) {
                Vector v = origDs.getColumn(c);
                v.normalizeToMeanStddev(true, false);
                matrix.setColumn(c, v);
            }
        } else if (index == 2) {
            log.info("Doing row value norm");
            for (int r = 0; r < origDs.getNumRow(); r++) {
                Vector v = origDs.getRow(r);
                v.normalizeToMeanStddev(true, false);
                matrix.setRow(r, v);
            }
        } else if (index == 3) {
            log.info("Doing column rank norm");
            for (int c = 0; c < origDs.getNumCol(); c++) {
                AddressedVector av = new AddressedVector(origDs.getColumn(c), false);
                List names = origDs.getRowNames();
                RankedList rl = RankedListGenerators.sortByVectorAndGetRankedList(av, sort, order, names);
                for (int r = 0; r < rl.getSize(); r++) {
                    matrix.setElement(r, c, rl.getRank(origDs.getRowName(r)));
                }
            }
        } else if (index == 4) { // @todo improve
            log.info("Doing log norm");
            // threshold all neg values to 0
            matrix = origDs.getMatrix().cloneDeep();
            //matrix.threshold(100, 16000);
            matrix.log();
        } else if (index == 5) {
            log.info("Doing zcore_mean");
            Matrix m = origDs.getMatrix();
            matrix = m.normalizeByZScore(true, true);
        } else if (index == 6) {
            log.info("Doing zscore_median");
            Matrix m = origDs.getMatrix();
            matrix = m.normalizeByZScore(true, false);
        } else if (index == 7) {
            log.info("Doing zscore_mad_mean");
            Matrix m = origDs.getMatrix();
            matrix = m.normalizeByZScore_mad(true, true);
        } else if (index == 8) {
            log.info("Doing zscore_mad_median");
            Matrix m = origDs.getMatrix();
            matrix = m.normalizeByZScore_mad(true, false);
        } else if (index == 9) {
            log.info("Doing cmap norm");
            Matrix m = origDs.getMatrix();
            matrix = m.normalizeToMinNegOneMaxOne(true);
        } else if (index == 10) {
            log.info("Doing cmap norm 2");
            Matrix m = origDs.getMatrix();
            matrix = m.normalizeToMinNegOneMaxOne(true);
            for (int r = 0; r < matrix.getNumRow(); r++) {
                for (int c = 0; c < matrix.getNumCol(); c++) {
                    if (Math.abs(matrix.getElement(r, c)) < 0.75) {
                        matrix.setElement(r, c, 0);
                    }
                }
            }
        } else if (index == 11) {
            log.info("Doing gsea_norm");
            Matrix m = origDs.getMatrix();
            matrix = m.normalizeByGsea(true);
        } else {
            throw new IllegalArgumentException("Unknown mode: " + index);
        }

        return new DefaultDataset(newName,
                matrix, origDs.getRowNames(), origDs.getColumnNames(), true, false, false, origDs.getAnnot());

    }


} // End class DatasetNormalizers
