/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;


import edu.mit.broad.genome.Errors;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.gsea.GeneSetCohort;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import gnu.trove.TFloatArrayList;
import gnu.trove.THashMap;
import gnu.trove.TObjectFloatHashMap;
import gnu.trove.TObjectIntHashMap;
import org.apache.log4j.Logger;
import xtools.api.param.BadParamException;

import java.util.*;

/**
 * Object that captures statistics for dataset rows
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DatasetStatsCore {

    private Logger log = XLogger.getLogger(DatasetStatsCore.class);

    /**
     * Inner class APM
     */
    public static class APM {

        public static TObjectIntHashMap hashPCallsByFeature(final Dataset ds) {
            return hashPCallsByFeature(ds, ds.getColumnNames());
        }

        /*
        public static TObjectIntHashMap hashPCallsByFeature(final Dataset ds, final Template.Class cl) {
            return hashPCallsByFeature(ds, cl.extractColumnNames(ds));
        }
        */

        public static TObjectIntHashMap hashPCallsByFeature(final Dataset ds, final List colNames_opt) {

            if (ds == null) {
                throw new IllegalArgumentException("Param ds cannot be null");
            }

            final APMMatrix apm = ((DefaultDataset) ds).getAPMMatrix();
            final TObjectIntHashMap probeNamePCallCounterMap = new TObjectIntHashMap();

            if (apm == null) {
                throw new IllegalArgumentException("AP calls na ... only RES files allowed. Got: " + ds.getName());
            }

            if (colNames_opt != null && colNames_opt.size() == 0) {
                throw new IllegalArgumentException("colNames was not null but zero size: " + colNames_opt);
            }

            if (colNames_opt != null) {
                Errors errors = new Errors("Missing in the apm data");
                for (int c = 0; c < colNames_opt.size(); c++) {
                    if (ds.getColumnIndex(colNames_opt.get(c).toString()) == -1) {
                        errors.add(colNames_opt.get(c).toString());
                    }
                }

                errors.barfIfNotEmptyRuntime();
                ;
            }

            boolean atLeastOneCol = false;
            for (int r = 0; r < apm.getNumRow(); r++) {
                int cnt = 0;
                for (int c = 0; c < apm.getNumCol(); c++) {
                    final String colName = ds.getColumnName(c);
                    if (colNames_opt != null && !colNames_opt.contains(colName)) {
                        // do nothing
                    } else {
                        atLeastOneCol = true;
                        if (apm.isPresent(r, c)) {
                            cnt++;
                        }
                    }
                }

                probeNamePCallCounterMap.put(ds.getRowName(r), cnt);
            }

            if (!atLeastOneCol) {
                throw new IllegalArgumentException("No hit in apm dataset with cols: " + ds.getColumnNames() + "\nto specified colNames: " + colNames_opt);
            }

            return probeNamePCallCounterMap;
        }

    } // End class APM


    /**
     * Class constructor
     * Stateless
     */
    public DatasetStatsCore() {
    }

    // key ->probe name, value -> rarray of ranges
    public THashMap hashProbeRanges(final Range[] rangesIn, final Dataset ds, final boolean stringIzeValues) {

        final THashMap map = new THashMap();

        for (int r = 0; r < ds.getNumRow(); r++) {
            final Range[] clones = SimpleRange.cloneShallow(rangesIn);
            final Vector v = ds.getRow(r);
            v.bin(clones, true);
            if (stringIzeValues) {
                map.put(ds.getRowName(r), SimpleRange.toString_counts_only(clones, '\t'));
            } else {
                map.put(ds.getRowName(r), clones);
            }
        }

        return map;
    }

    public TObjectFloatHashMap hashProbeVariation(final Dataset ds) {

        final TObjectFloatHashMap map = new TObjectFloatHashMap();

        for (int r = 0; r < ds.getNumRow(); r++) {
            final Vector v = ds.getRow(r);
            map.put(ds.getRowName(r), (float) v.stddev(false, false));
        }

        return map;
    }

    /**
     * Captures the scores associated with a Marker
     * Only for categorical 2 way distinctions
     */
    public static class TwoClassMarkerStats {

        private double score;
        private double me0;
        private double stdev0;
        private double me1;
        private double stdev1;

        private double me_all;
        private double stdev_all;

        public double getScore() {
            return score;
        }

        public double getMe0() {
            return me0;
        }

        public double getMe_all() {
            return me_all;
        }

        public double getStdev0() {
            return stdev0;
        }

        public double getMe1() {
            return me1;
        }

        public double getStdev1() {
            return stdev1;
        }

        public double getStdev_all() {
            return stdev_all;
        }

    }

    public Map calc2MetricMarkerScores(final Dataset ds,
                                       final Template template) {
        return calc2ClassCategoricalMetricMarkerScores(ds, template, new Metrics.Signal2Noise(), DatasetMetrics.getDefaultMetricParams());
    }

    public Map calc2MetricMarkerScores_class_means(final Dataset ds,
                                                   final Template template) {
        return calc2ClassCategoricalMetricMarkerScores(ds, template, new Metrics.ClassMeansRatio(), DatasetMetrics.getDefaultMetricParams());
    }


    // key -> feature name, value -> TwoClassMarkerStats
    public Map calc2ClassCategoricalMetricMarkerScores(final Dataset ds,
                                                       final Template template,
                                                       final Metric metric,
                                                       final Map params) {

        if (!metric.isCategorical()) {
            throw new IllegalArgumentException("Not a 2 class categorical metric: " + metric);
        }

        Map all = new HashMap();

        boolean usebiased = AlgMap.isBiased(params);
        boolean fixlow = AlgMap.isFixLowVar(params);
        boolean useMedian = AlgMap.isMedian(params);

        VectorSplitter splitter = new VectorSplitter(metric.getMinNumSamplesNeededPerClassForCalculation());

        for (int r = 0; r < ds.getNumRow(); r++) {

            String rowName = ds.getRowName(r);
            //System.out.println(">>" + rowName);

            Vector profile = ds.getRow(r);
            Vector[] vs = splitter.splitBiphasic_nansafe(profile, template);

            // Pause for a helpful error message for gsea GUI
            // need to watch out for throwing out baby with bathwater: sometimes, esp with cdna data there might not be enough or any values in the split vectors
            // DOC this rather than catching

            if (r == 0) {
                if (metric.getName().equalsIgnoreCase(Metrics.Signal2Noise.NAME) || metric.getName().equalsIgnoreCase(Metrics.tTest.NAME)) {
                    if (ds.getNumCol() < 6) {
                        throw new BadParamException("Too few samples in the dataset to use this metric", 1006);
                    }

                    if (vs[0].getSize() < 3) {
                        throw new BadParamException("Too few samples in class A of the dataset to use this metric", 1006);
                    }

                    if (vs[1].getSize() < 3) {
                        throw new BadParamException("Too few samples in class B of the dataset to use this metric", 1006);
                    }
                }
            }

            TwoClassMarkerStats stats = new TwoClassMarkerStats();

            // sometimes, esp with cdna data there might not be enough or any values in the split vectors
            if (vs == null || vs[0] == null || vs[1] == null) {
                stats.score = Float.NaN;
                stats.me0 = Float.NaN;
                stats.me1 = Float.NaN;
                stats.me_all = Float.NaN;
                stats.stdev0 = Float.NaN;
                stats.stdev1 = Float.NaN;
                stats.stdev_all = Float.NaN;
                log.warn("Omitting as too few good data points: " + rowName);
                //throw new RuntimeException("Omitting as too few good data points: " + rowName + " " + template.getName() + "\n" + template.getAsString(false));
            } else {
                stats.score = (float) metric.getScore(profile, template, params);
                // @note not all of the cols in the ds might be used in the template
                // so this test is not correct
                /*
                final Vector full_row_as_is = template.extractProfile(r, ds);
                if (vs[0].getSize() + vs[1].getSize() != full_row.getSize()) {
                    throw new IllegalStateException();
                }
                */
                Vector full = new Vector(vs);

                if (useMedian) {
                    stats.me0 = vs[0].median();
                    stats.me1 = vs[1].median();
                    stats.me_all = full.median();
                } else {
                    stats.me0 = vs[0].mean();
                    stats.me1 = vs[1].mean();
                    stats.me_all = full.mean();
                }

                stats.stdev0 = (float) vs[0].stddev(usebiased, fixlow);
                stats.stdev1 = (float) vs[1].stddev(usebiased, fixlow);
                stats.stdev_all = (float) full.stddev(usebiased, fixlow);
            }

            if (all.containsKey(rowName)) {
                throw new RuntimeException("Duplicate row names in dataset: " + ds.getName() + " rowname: " + rowName + " row: " + r);
            }
            all.put(rowName, stats);

        }

        return Collections.unmodifiableMap(all);
    }

    // @note defaults
    public TwoClassMarkerStats calc2ClassCategoricalMetricMarkerScores(final Dataset ds,
                                                                       final Template template,
                                                                       final String rowName) {
        return calc2ClassCategoricalMetricMarkerScores(ds, template, new Metrics.Signal2Noise(), new HashMap(), rowName);
    }

    public TwoClassMarkerStats calc2ClassCategoricalMetricMarkerScores(final Dataset ds,
                                                                       final Template template,
                                                                       final Metric metric,
                                                                       final Map params,
                                                                       final String rowName) {

        if (!metric.isCategorical()) {
            throw new IllegalArgumentException("Not a 2 class categorical metric: " + metric);
        }

        boolean usebiased = AlgMap.isBiased(params);
        boolean fixlow = AlgMap.isFixLowVar(params);
        boolean usemedian = AlgMap.isMedian(params);

        final VectorSplitter splitter = new VectorSplitter(1);
        final Vector profile = ds.getRow(rowName);
        final Vector[] vs = splitter.splitBiphasic_nansafe(profile, template);
        final TwoClassMarkerStats stats = new TwoClassMarkerStats();

        stats.score = (float) metric.getScore(profile, template, params);

        // if (vs != null && vs[0] != null && vs[1] != null) {

        if (usemedian) {
            stats.me0 = vs[0].median();
            stats.me1 = vs[1].median();
        } else {
            stats.me0 = vs[0].mean();
            stats.me1 = vs[1].mean();
        }

        stats.stdev0 = (float) vs[0].stddev(usebiased, fixlow);
        stats.stdev1 = (float) vs[1].stddev(usebiased, fixlow);

        return stats;
    }

    public Dataframe calcRowStatsDf(VectorFunctor.Statistic[] stats, Dataset ds) {

        Vector[] vss = calcRowStats(stats, ds);
        Matrix matrix = new Matrix(ds.getNumRow(), stats.length);
        List colNames = new ArrayList(stats.length);
        for (int s = 0; s < stats.length; s++) {
            matrix.setColumn(s, vss[s]);
            colNames.add(stats[s].getName());
        }

        return new Dataframe("all_stats_for_" + ds.getName(), matrix, ds.getRowNames(), colNames, true);
    }

    public Vector calcRowStat(int excludeColByPosInDs, VectorFunctor.Statistic stat, Dataset ds, Template.Class tc) {
        return calcRowStats(excludeColByPosInDs, new VectorFunctor.Statistic[]{stat}, ds, tc)[0];
    }

    public Vector[] calcRowStats(VectorFunctor.Statistic[] stats, Dataset ds) {

        if (ds == null) {
            throw new IllegalArgumentException("Parameter ds cannot be null");
        }

        Vector[] vss = new Vector[stats.length];
        for (int s = 0; s < stats.length; s++) {
            vss[s] = new Vector(ds.getNumRow());
        }

        for (int r = 0; r < ds.getNumRow(); r++) {
            Vector row = ds.getRow(r);
            for (int s = 0; s < stats.length; s++) {
                vss[s].setElement(r, (float) stats[s].getValue(row));
            }
        }

        return vss;
    }

    // IMP ds must already be extracted etc as needed
    public Vector[] calcRowStats(VectorFunctor.Statistic[] stats, Dataset ds, Template.Class tc) {

        Vector[] vss = new Vector[stats.length];
        for (int s = 0; s < stats.length; s++) {
            vss[s] = new Vector(ds.getNumRow());
        }

        for (int r = 0; r < ds.getNumRow(); r++) {
            Vector profile = tc.extractByTemplateClass(ds.getRow(r));
            for (int s = 0; s < stats.length; s++) {
                vss[s].setElement(r, (float) stats[s].getValue(profile));
            }
        }

        return vss;
    }

    public Vector calcRowStat(VectorFunctor.Statistic stat, Dataset ds, Template.Class tc) {

        Vector v = new Vector(ds.getNumRow());

        for (int r = 0; r < ds.getNumRow(); r++) {
            Vector profile = tc.extractByTemplateClass(ds.getRow(r));
            v.setElement(r, (float) stat.getValue(profile));
        }

        return v;
    }

    public Vector[] calcRowStats(int excludeColByPosInDs, VectorFunctor.Statistic[] stats, Dataset ds, Template.Class tc) {

        Vector[] vss = new Vector[stats.length];
        for (int s = 0; s < stats.length; s++) {
            vss[s] = new Vector(ds.getNumRow());
        }

        for (int r = 0; r < ds.getNumRow(); r++) {
            Vector profile = tc.extractByTemplateClass(ds.getRow(r), excludeColByPosInDs);
            for (int s = 0; s < stats.length; s++) {
                vss[s].setElement(r, (float) stats[s].getValue(profile));
            }
        }

        return vss;
    }

    // IMP ds must already be extracted etc as needed
    public Vector calcRowStat(VectorFunctor.Statistic stat, Dataset ds) {
        return calcRowStats(new VectorFunctor.Statistic[]{stat}, ds)[0];
    }

    // IMP includes dist to toCol from toCol (i.e 0)
    public Vector getColumnDistances(int toCol, Dataset ds, Metric metric, Map mps) {

        Vector v = new Vector(ds.getNumCol() - 1);
        Vector it = ds.getColumn(toCol);
        boolean found = false;
        int cnt = 0;
        for (int c = 0; c < ds.getNumCol(); c++) {
            if (c == toCol) {
                found = true;
            }
            double score = metric.getScore(it, ds.getColumn(c), mps);
            v.setElement(cnt++, (float) score);

        }

        if (!found) {
            throw new IllegalArgumentException("Specified toCol: " + toCol + " was not encountered in the dataset");
        }

        return v;
    }

    public Vector extractGeneSetValuesFromDataset(int colInDs, Dataset ds, GeneSet gset) {

        int num = gset.getNumMembers(ds);
        Vector v = new Vector(num);

        int cnt = 0;
        for (int r = 0; r < ds.getNumRow(); r++) {
            String name = ds.getRowName(r);

            if (gset.isMember(name)) {
                v.setElement(cnt++, ds.getElement(r, colInDs));
            }
        }

        if (cnt != num) {
            throw new IllegalStateException("Failed sanity check cnt: " + cnt + " num: " + num);
        }

        return v;
    }

    public Vector[] extractGeneSetValuesFromDataset(int colInDs, Dataset ds, GeneSetCohort gcoh) {

        final TFloatArrayList[] tss = new TFloatArrayList[gcoh.getNumGeneSets()];

        for (int r = 0; r < ds.getNumRow(); r++) {
            String name = ds.getRowName(r);
            float elem = ds.getElement(r, colInDs);

            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {

                if (r == 0) { // init
                    tss[g] = new TFloatArrayList();
                }

                if (gcoh.isMember(g, name)) {
                    tss[g].add(elem);
                }
            }
        }

        final Vector[] vss = new Vector[tss.length];
        for (int i = 0; i < tss.length; i++) {
            vss[i] = new Vector(tss[i]);
        }

        return vss;
    }

    public ColGeneSetsVector[] extractGeneSetValuesFromDataset(Dataset ds, GeneSetCohort gcoh) {
        if (ds == null) {
            throw new IllegalArgumentException("Parameter ds cannot be null");
        }

        if (gcoh == null) {
            throw new IllegalArgumentException("Parameter gcoh cannot be null");
        }

        ColGeneSetsVector[] colgss = new ColGeneSetsVector[ds.getNumCol()];

        for (int c = 0; c < colgss.length; c++) {
            colgss[c] = new ColGeneSetsVector(c, gcoh.getNumGeneSets());
        }

        for (int r = 0; r < ds.getNumRow(); r++) {
            String name = ds.getRowName(r);
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                if (gcoh.isMember(g, name)) {
                    for (int c = 0; c < ds.getNumCol(); c++) {
                        colgss[c].values[g].add(ds.getElement(r, c));
                    }
                }
            }
        }

        return colgss;
    }

    class ColGeneSetsVector {
        TFloatArrayList[] values;
        int colInDs;

        ColGeneSetsVector(int colInDs, int numGeneSets) {
            this.colInDs = colInDs;
            this.values = new TFloatArrayList[numGeneSets];
            for (int g = 0; g < numGeneSets; g++) {
                values[g] = new TFloatArrayList();
            }
        }

        Vector[] toVector() {
            Vector[] vss = new Vector[values.length];
            for (int i = 0; i < values.length; i++) {
                vss[i] = new Vector(values[i]);
            }

            return vss;
        }
    }

    // rows are geneset names
    // cols are sample names (no change)
    public Dataset[] dataset2EsDatasets(final Dataset ds, final GeneSetCohort gcoh, final VectorFunctor.Statistic[] stats) {

        Matrix[] esMatrixs = new Matrix[stats.length];

        for (int m = 0; m < stats.length; m++) {
            esMatrixs[m] = new Matrix(ds.getNumCol(), gcoh.getNumGeneSets());
        }

        for (int c = 0; c < ds.getNumCol(); c++) {
            Vector[] vss = extractGeneSetValuesFromDataset(c, ds, gcoh);

            for (int r = 0; r < vss.length; r++) {
                for (int m = 0; m < stats.length; m++) {
                    esMatrixs[m].setElement(r, c, (float) stats[m].getValue(vss[r]));
                }
            }

        }

        List rowNames = gcoh.getGeneSetNames();
        Dataset[] dss = new Dataset[stats.length];
        for (int m = 0; m < stats.length; m++) {
            dss[m] = new DefaultDataset(ds.getName() + "_" + stats[m].getName(), esMatrixs[m], rowNames, ds.getColumnNames(), true, true, false, ds.getAnnot());
        }

        return dss;
    }

    // more optimzed version
    public Dataset[] dataset2EsDatasets2(final Dataset ds, final GeneSetCohort gcoh, final VectorFunctor.Statistic[] stats) {

        Matrix[] esMatrixs = new Matrix[stats.length];

        for (int m = 0; m < stats.length; m++) {
            esMatrixs[m] = new Matrix(gcoh.getNumGeneSets(), ds.getNumCol());
        }

        ColGeneSetsVector[] colgss = extractGeneSetValuesFromDataset(ds, gcoh);
        for (int c = 0; c < colgss.length; c++) {
            Vector[] vss = colgss[c].toVector();
            for (int r = 0; r < vss.length; r++) {
                for (int m = 0; m < stats.length; m++) {
                    esMatrixs[m].setElement(r, c, (float) stats[m].getValue(vss[r]));
                }
            }
        }

        List rowNames = gcoh.getGeneSetNames();
        Dataset[] dss = new Dataset[stats.length];
        for (int m = 0; m < stats.length; m++) {
            dss[m] = new DefaultDataset(ds.getName() + "_" + stats[m].getName(), esMatrixs[m], rowNames, ds.getColumnNames(), true, true, false, ds.getAnnot());
        }

        return dss;
    }

}    // End DatasetStats

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
