/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import gnu.trove.TIntIntHashMap;
import gnu.trove.TObjectIntHashMap;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Apr 14, 2003
 *         Time: 12:41:32 PM
 */
public class FrequencyAnalysis {


    public static class Kmer {

        List fMers;

        public Kmer(final String[] mers) {

            this.fMers = new ArrayList();
            for (int i = 0; i < mers.length; i++) {
                if (fMers.contains(mers[i])) {
                    throw new IllegalArgumentException("Duplicate mer: " + mers[i]);
                }
                fMers.add(mers[i]);
            }

            if (fMers.size() != mers.length) {
                throw new IllegalArgumentException("Duplicate mers: " + fMers);
            }

            Collections.sort(fMers, new ComparatorFactory.StringComparator());
        }

        public int getK() {
            return fMers.size();
        }

        public String toString() {
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < getK(); i++) {
                buf.append(getMer(i));
                if (i != getK() - 1) {
                    buf.append(",");
                }
            }

            return buf.toString();
        }

        public int hashCode() {
            return toString().hashCode();
        }

        public String getMer(int i) {
            return fMers.get(i).toString();
        }

        /*
        public List getAsSet() {
            Set set = new HashSet();
            for (int i = 0; i < fMers.length; i++) {
                set.add(fMers[i]);
            }

            return set;
        }
        */

        public boolean contains(final String mer) {
            return fMers.contains(mer);
        }

        public boolean equals(Object obj) {
            if (!(obj instanceof Kmer)) {
                return false;
            }

            Kmer in = (Kmer) obj;

            if (in.getK() != getK()) {
                return false;
            }


            int numFound = 0;
            for (int i = 0; i < in.getK(); i++) {
                if (contains(in.getMer(i))) {
                    numFound++;
                }
            }

            if (numFound != in.getK()) {
                return false;
            }


            return true;
        }

    } // End class Kmer


    public static Kmer[] to2Mers(final String[] words) {

        List list = new ArrayList();

        for (int i = 0; i < words.length; i++) {
            for (int j = 0; j < words.length; j++) {
                if (i == j || i > j) {
                    continue;
                }
                list.add(new Kmer(new String[]{words[i], words[j]}));
            }
        }

        return (Kmer[]) list.toArray(new Kmer[list.size()]);
    }

    public static Kmer[] to3Mers(final String[] words) {

        List list = new ArrayList();

        for (int i = 0; i < words.length; i++) {
            for (int j = 0; j < words.length; j++) {
                for (int k = 0; k < words.length; k++) {
                    Set set = new HashSet();
                    set.add(words[i]);
                    set.add(words[j]);
                    set.add(words[k]);
                    if (set.size() == 3) {
                        Kmer kmer = new Kmer(new String[]{words[i], words[j], words[k]});
                        if (list.contains(kmer) == false) {
                            list.add(kmer);
                        }
                    }
                }
            }
        }

        return (Kmer[]) list.toArray(new Kmer[list.size()]);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public static TObjectIntHashMap featureFreq(String[] feats) {

        // IMP needed as returns 0 and not -1 on no hits!!
        TObjectIntHashMap map = new TObjectIntHashMap();

        for (int i = 0; i < feats.length; i++) {
            int curr = 0;
            if (map.containsKey(feats[i])) {
                curr = map.get(feats[i]);
            }
            curr++;
            map.put(feats[i], curr);
            //System.out.println("setting " + words[i] + " " + curr);
            // int curr = map.get(words[i]);
            //map.put(words[i], curr++);
        }

        return map;
    }

    public static TIntIntHashMap valueArrayLengthFreq(Map map) {

        // key -> length of values in map, value -> number of times this length is seen */
        TIntIntHashMap imap = new TIntIntHashMap();
        for (Iterator it = map.keySet().iterator(); it.hasNext();) {
            Object key = it.next();
            int valuelen = ((String[]) map.get(key)).length;
            int curr = imap.get(valuelen);
            curr++;
            imap.put(valuelen, curr);
        }

        return imap;
    }

    public static TIntIntHashMap valueSetLengthFreq(Map map) {

        // key -> length of values in map, value -> number of times this length is seen */
        TIntIntHashMap imap = new TIntIntHashMap();
        for (Iterator it = map.keySet().iterator(); it.hasNext();) {
            Object key = it.next();
            int valuelen = ((Set) map.get(key)).size();
            int curr = 0;
            if (imap.containsKey(valuelen)) {
                curr = imap.get(valuelen);
            }
            curr++;
            imap.put(valuelen, curr);

            // imap.put(valuelen, curr++); trove doesnt like this!!!
        }

        return imap;
    }

    public static int totSize(Set[] sets, boolean unique) {

        if (unique) {
            Set un = new HashSet();
            for (int i = 0; i < sets.length; i++) {
                un.addAll(sets[i]);
            }

            return un.size();
        } else {
            int tot = 0;
            for (int i = 0; i < sets.length; i++) {
                tot += sets[i].size();
            }

            return tot;
        }

    }

} // End FrequencyAnalysis
