/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.distrib.RangeFactory;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DatasetWrapper;
import edu.mit.broad.genome.objects.strucs.GeneSetRandomizerType;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.chip.Probe;
import edu.mit.broad.vdb.meg.Gene;
import gnu.trove.TObjectIntHashMap;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Aug 15, 2003
 *         Time: 10:00:54 PM
 */
public class GeneSetGenerators {

    private static Logger klog = XLogger.getLogger(GeneSetGenerators.class);

    /**
     * Privatized class constructor
     * Only static methods
     */
    private GeneSetGenerators() {
    }

    // @note uses hardoced common paramters for marker selection
    public static GeneSetPair createGeneSetFromMarkers(int topAndBotX, Dataset ds, Template t) {
        return new DatasetMetrics().scoreDataset(ds, t).getNamesOfUpAndDnXRanks(topAndBotX);
    }

    // leave out the sets and the replace it with the intersection of all others
    public static GeneSet[] createLeaveOneOutIntersectionSets(final GeneSet[] gsets) {
        final GeneSet[] lovs = new GeneSet[gsets.length];
        for (int i = 0; i < gsets.length; i++) {
            final GeneSet[] sets = new GeneSet[gsets.length - 1];
            int cnt = 0;
            for (int j = 0; j < gsets.length; j++) {
                if (i != j) {
                    sets[cnt++] = gsets[j];
                }
            }
            lovs[i] = createIntersectionGeneSet(gsets[i].getName(true) + "_lov", sets);
        }

        return lovs;
    }

    public static Dataset createIntersection_dataset(final GeneSet[] gsets) {
        Matrix m = new Matrix(gsets.length, gsets.length);

        String[] rowNames = new String[gsets.length];
        for (int r = 0; r < gsets.length; r++) {
            rowNames[r] = gsets[r].getName(true);
            for (int c = 0; c < gsets.length; c++) {
                m.setElement(r, c, gsets[r].getNumMembers(gsets[c]));
            }
        }

        return new DefaultDataset("inter", m, rowNames, rowNames, true);
    }

    public static GeneSet_RL_IntersectionStruc[] createLeaveOneOutIntersectionSets(final GeneSet[] gsets, final int pickTopXCommonProbes) {
        final GeneSet_RL_IntersectionStruc[] lovs = new GeneSet_RL_IntersectionStruc[gsets.length];
        for (int i = 0; i < gsets.length; i++) {
            final GeneSet[] sets = new GeneSet[gsets.length - 1];
            int cnt = 0;
            for (int j = 0; j < gsets.length; j++) {
                if (i != j) {
                    sets[cnt++] = gsets[j];
                }
            }
            lovs[i] = createIntersectionGeneSet(gsets[i].getName(true) + "_lov", sets, pickTopXCommonProbes);
        }

        return lovs;
    }

    public static class GeneSet_RL_IntersectionStruc {

        public RankedList featureFreq;

        public GeneSet geneSet;

        GeneSet_RL_IntersectionStruc(String name, int pickTopXCommonProbes, RankedList rl_feature_freqs) {
            this.featureFreq = rl_feature_freqs;
            this.geneSet = rl_feature_freqs.getNamesOfUpOrDnXRanks_gs(pickTopXCommonProbes, true).cloneShallow(name);
        }

        public static GeneSet[] toGeneSets(GeneSet_RL_IntersectionStruc[] strucs) {
            final GeneSet[] gsets = new GeneSet[strucs.length];
            for (int i = 0; i < strucs.length; i++) {
                gsets[i] = strucs[i].geneSet;
            }

            return gsets;
        }

    }

    public static GeneSet createIntersectionGeneSet(final GeneSet a, final GeneSet b) {
        return createIntersectionGeneSet(a.getName(true) + "_intersect_" + b.getName(true), new GeneSet[]{a, b});
    }

    public static GeneSet createIntersectionGeneSet(final String name, final GeneSet[] gsets) {

        GeneSet all = AlgUtils.unionAll(name + "tmpunion", gsets);
        Set common = new HashSet();

        for (int i = 0; i < all.getNumMembers(); i++) {
            String item = all.getMember(i);
            boolean add = true;
            for (int f = 0; f < gsets.length; f++) {
                if (!gsets[f].isMember(item)) {
                    add = false;
                    break;
                }
            }

            if (add) {
                common.add(item);
            }
        }

        return new FSet(name, common);
    }


    public static GeneSet createGeneSetInAButNotInB(final GeneSet gsetA, final GeneSet gsetB) {

        Set set = new HashSet();

        for (int i = 0; i < gsetA.getNumMembers(); i++) {
            String member = gsetA.getMember(i);
            if (!gsetB.isMember(member)) {
                set.add(member);
            }
        }

        return new FSet("uniq_to_" + gsetA.getName(), set);
    }

    public static GeneSet_RL_IntersectionStruc createIntersectionGeneSet(final String name,
                                                                         final GeneSet[] gsets,
                                                                         final int pickTopXCommonProbes) {
        RankedList rl_features = new GeneSetStats().getFeatureFrequency_rl(gsets, Order.DESCENDING);
        return new GeneSet_RL_IntersectionStruc(name, pickTopXCommonProbes, rl_features);
    }

    public static GeneSetMatrixPair createGeneSets(final Dataset ds, final int topX, final int botX, final Order order) {

        if (XMath.isNegative(topX)) {
            throw new IllegalArgumentException("Cant do for negative topX: " + topX);
        }

        if (XMath.isNegative(botX)) {
            throw new IllegalArgumentException("Cant do for negative botX: " + topX);
        }

        final GeneSet[] gsets_up = new GeneSet[ds.getNumCol()];
        final GeneSet[] gsets_dn = new GeneSet[ds.getNumCol()];

        for (int c = 0; c < ds.getNumCol(); c++) {
            LabelledVector lv = ds.getColumnLV(c);
            RankedList rl = lv.sort(SortMode.REAL, order);
            gsets_up[c] = rl.getNamesOfUpOrDnXRanks_gs(topX, true);
            gsets_dn[c] = rl.getNamesOfUpOrDnXRanks_gs(botX, false);
            if (c % 25 == 0) {
                klog.info("Done column " + (c + 1) + " / " + ds.getNumCol());
            }
        }

        return new GeneSetMatrixPairImpl(new DefaultGeneSetMatrix("up_" + ds.getName(), gsets_up), new DefaultGeneSetMatrix("dn_" + ds.getName(), gsets_dn));
    }


    public static GeneSet[] createGeneSets(final RankedList[] rls, final int num, final boolean up) {
        final GeneSet[] gsets = new GeneSet[rls.length];
        for (int i = 0; i < rls.length; i++) {
            gsets[i] = rls[i].getNamesOfUpOrDnXRanks_gs(num, up);
        }
        return gsets;
    }

    public static GeneSetMatrixPair createGeneSets(final Dataset ds, final float valueCutoffUp, final float valueCutoffDn) {

        klog.debug("createGeneSets valueCutoffUp: " + valueCutoffUp + " valueCutoffDn: " + valueCutoffDn);

        final GeneSet[] gsets_up = new GeneSet[ds.getNumCol()];
        final GeneSet[] gsets_dn = new GeneSet[ds.getNumCol()];

        for (int c = 0; c < ds.getNumCol(); c++) {
            final LabelledVector lv = ds.getColumnLV(c);
            //final RankedList rl = lv.sort(SortMode.REAL, order); // ordering doesnt matter as looking for more than
            final List up = new ArrayList();
            final List dn = new ArrayList();

            for (int r = 0; r < lv.getSize(); r++) {
                float value = lv.getScore(r);
                if (value >= valueCutoffUp) {
                    up.add(lv.getLabel(r));
                } else if (value <= valueCutoffDn) {
                    dn.add(lv.getLabel(r));
                }
            }

            //System.out.println(">> " + up.size());
            //System.out.println(">> " + dn.size());
            gsets_up[c] = new FSet(ds.getColumnName(c) + " _up_" + up.size(), up, false);
            gsets_dn[c] = new FSet(ds.getColumnName(c) + " _dn_" + dn.size(), dn, false);
            if (c % 25 == 0) {
                klog.info("Done column " + (c + 1) + " / " + ds.getNumCol());
            }
        }

        klog.debug("createGeneSets upSets: " + gsets_up.length + " dnSets: " + gsets_dn.length);
        return new GeneSetMatrixPairImpl(
                new DefaultGeneSetMatrix("up_" + ds.getName(), gsets_up),
                new DefaultGeneSetMatrix("dn_" + ds.getName(), gsets_dn));
    }

    public static GeneSet createIntersectionSpace(final String name, final Dataset[] dss) {

        GeneSet[] sets = new GeneSet[dss.length];
        for (int i = 0; i < dss.length; i++) {
            sets[i] = dss[i].getRowNamesGeneSet();
        }

        return createIntersectionGeneSet(name, sets);
    }

    public static GeneSet createIntersectionSpace(final String name, final DatasetWrapper[] dws, final boolean add2cache) {
        GeneSet commonSpace = null;
        for (int i = 0; i < dws.length; i++) {
            System.out.println("\n\n----------> PROCESSING FILES: " + dws[i].getFile());
            final Dataset ds = dws[i].getDataset(add2cache);
            GeneSet gset = ds.getRowNamesGeneSet();

            if (commonSpace == null) {
                commonSpace = gset;
            } else {
                commonSpace = AlgUtils.intersect("foo", commonSpace, gset);
            }
        }

        if (commonSpace != null) {
            return commonSpace.cloneShallow(name);
        } else {
            throw new IllegalArgumentException("Empty DatasetWrapper: " + dws.length);
        }

    }

    public static GeneSet removeIntersection(GeneSet removeFromThis, GeneSet anyThatAreInThis) {

        List use = new ArrayList();
        for (int i = 0; i < removeFromThis.getNumMembers(); i++) {
            if (!anyThatAreInThis.isMember(removeFromThis.getMember(i))) {
                use.add(removeFromThis.getMember(i));
            }
        }

        return new FSet(removeFromThis.getName(true) + "_minus_" + anyThatAreInThis.getName(true), use, false);
    }

    public static GeneSet[] removeAffxProbes(GeneSet[] gsets) {

        GeneSet[] rgsets = new GeneSet[gsets.length];
        for (int i = 0; i < gsets.length; i++) {
            rgsets[i] = removeAffxProbes(gsets[i]);
        }

        return rgsets;
    }

    public static GeneSet removeAffxProbes(GeneSet gset) {
        Set probes2use = new HashSet();
        for (int i = 0; i < gset.getNumMembers(); i++) {
            String name = gset.getMember(i);
            if (!name.startsWith(Headers.AFFX_CONTROL_PREFIX)) {
                probes2use.add(name);
            }
        }

        return new FSet(gset.getName(), probes2use);
    }

    public static GeneSet[] createByRestrictingProbesPerGene(final GeneSet[] gsets,
                                                             final Chip chip,
                                                             final int maxProbesPerGene) throws Exception {
        GeneSet[] restric = new GeneSet[gsets.length];
        for (int i = 0; i < gsets.length; i++) {
            //restric[i] = gsets[i].cloneShallow(chip, false);
            restric[i] = gsets[i].cloneShallow();
            restric[i] = createByRestrictingProbesPerGene(restric[i], chip, maxProbesPerGene);
        }

        return restric;
    }

    // note that if NO symbol, the probe is ALWAYS used
    public static GeneSet createByRestrictingProbesPerGene(final GeneSet gset,
                                                           final Chip chip,
                                                           final int maxProbesPerGene) throws Exception {

        if (gset == null) {
            throw new IllegalArgumentException("Param gset cannot be null");
        }

        if (chip == null) {
            throw new IllegalArgumentException("Param chip cannot be null");
        }

        final Set probes2use = new HashSet();
        final TObjectIntHashMap symbolCntMap = new TObjectIntHashMap();
        for (int m = 0; m < gset.getNumMembers(); m++) {
            String probeName = gset.getMember(m);
            Gene gene = chip.getHugo(probeName);
            if (gene == null) {
                probes2use.add(gset.getMember(m));
            } else {
                String symbol = gene.getSymbol();
                if (symbol != null) {
                    int cnt = symbolCntMap.get(symbol);
                    cnt = cnt + 1;
                    if (cnt > maxProbesPerGene) {
                        continue;
                    } else {
                        probes2use.add(gset.getMember(m));
                    }
                    symbolCntMap.put(symbol, cnt);
                } else {
                    probes2use.add(gset.getMember(m));
                }
            }
        }

        return new FSet(gset.getName(), probes2use);
    }

    public static GeneSet subtractGeneSet(GeneSet a, GeneSet removeThese) {
        Set use = new HashSet(a.getMembersS());

        for (int i = 0; i < removeThese.getNumMembers(); i++) {
            use.remove(removeThese.getMember(i));
        }

        return new FSet(a.getName() + "_minus" + removeThese.getName(), use);
    }

    public static GeneSet subtractGeneSet(GeneSet a, Set removeThese) {
        Set use = new HashSet(a.getMembersS());

        for (Iterator iterator = removeThese.iterator(); iterator.hasNext();) {
            String s = iterator.next().toString();
            use.remove(s);

        }

        return new FSet(a.getName() + "_minus" + removeThese.size(), use);
    }

    public static GeneSet[] createLovGeneSet(int lovIndex, GeneSet[] gsets) {
        GeneSet[] ret = new GeneSet[gsets.length - 1];
        int cnt = 0;
        for (int i = 0; i < gsets.length; i++) {
            if (i != lovIndex) {
                ret[cnt++] = gsets[i];
            }
        }

        return ret;
    }

    public static GeneSetMatrix restrictMatrix(GeneSet toTheseNamesGeneNames, GeneSetMatrix gm) {
        GeneSet[] redsets = new GeneSet[gm.getNumGeneSets()];
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            GeneSet curr = gm.getGeneSet(i);
            Set genes = new HashSet();
            for (int r = 0; r < curr.getNumMembers(); r++) {
                if (toTheseNamesGeneNames.isMember(curr.getMember(r))) {
                    genes.add(curr.getMember(r));
                }

            }
            redsets[i] = new FSet(curr.getName(), genes);
        }

        return new DefaultGeneSetMatrix(gm.getName() + "_red_" + toTheseNamesGeneNames.getName(), redsets);
    }

    // matrix only on those gnes that at least occur in x gene sets
    public static GeneSetMatrix restrictMatrix(int minProbeFreq, GeneSetMatrix gm) {
        GeneSetStats stats = new GeneSetStats();
        LabelledVector lvFreq = stats.getFeatureFrequency_lv(gm);
        GeneSet[] redsets = new GeneSet[gm.getNumGeneSets()];
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            GeneSet curr = gm.getGeneSet(i);
            Set genes = new HashSet();
            for (int r = 0; r < curr.getNumMembers(); r++) {
                String geneName = curr.getMember(r);
                int freq = (int) lvFreq.getScore(geneName);
                if (freq >= minProbeFreq) {
                    genes.add(geneName);
                }
            }
            redsets[i] = new FSet(curr.getName(), genes);
        }

        return new DefaultGeneSetMatrix(gm.getName() + "_red_" + minProbeFreq, redsets);
    }

    public static GeneSet[] restrictGeneSets2MaxSize(int size, GeneSet[] gsets, Dataset ds, boolean barfIfNonePass) {
        return restrictGeneSets2MaxSize(size, gsets, ds.getRowNames(), barfIfNonePass);
    }

    public static GeneSet[] restrictGeneSets2MaxSize(int size, GeneSet[] gsets, List allNames, boolean barfIfNonePass) {

        // make all the same size
        List list = new ArrayList();
        klog.info("restricting genesets to size: " + size);
        for (int i = 0; i < gsets.length; i++) {
            if (gsets[i].getNumMembers(allNames) >= size) {
                FSet set = new FSet(gsets[i].getName() + "_" + size, gsets[i].getNameEnglish(), gsets[i].getMembers().subList(0, size), false);
                list.add(set);
            }

            if (i % 100 == 0) {
                System.out.println("restricted: " + i + " / " + gsets.length);
            }
        }

        klog.info("input #: " + gsets.length + " after restriction: " + list.size());

        if (barfIfNonePass && gsets.length == 0) {
            throw new IllegalArgumentException("After size restriction, 0 gene sets passed! Size: " + size);
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
    }

    public static GeneSet[] restrictGeneSets2MaxSize(int size, GeneSet[] gsets, boolean barfIfNonePass) {

        // make all the same size
        List list = new ArrayList();
        klog.info("restricting genesets to size: " + size);
        for (int i = 0; i < gsets.length; i++) {
            if (gsets[i].getNumMembers() >= size) {
                FSet set = new FSet(gsets[i].getName() + "_" + size, gsets[i].getNameEnglish(), gsets[i].getMembers().subList(0, size), false);
                list.add(set);
            }

            if (i % 100 == 0) {
                System.out.println("restricted: " + i + " / " + gsets.length);
            }
        }

        klog.info("input #: " + gsets.length + " after restriction: " + list.size());

        if (barfIfNonePass && gsets.length == 0) {
            throw new IllegalArgumentException("After size restriction, 0 gene sets passed! Size: " + size);
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
    }

    public static GeneSet createProbeNameGeneSet(String name, Probe[] probes) throws Exception {
        Set names = new HashSet();
        for (int p = 0; p < probes.length; p++) {
            names.add(probes[p].getName());
        }

        return new FSet(name, names);
    }


    //  fromIndex - low endpoint (inclusive) of the subList.
    // toIndex - high endpoint (exclusive) of the subList.
    public static GeneSet createGeneSet(int startIndex, int stopIndex, boolean checkForDuplicates, final GeneSet gset) {
        return new FSet(gset.getName() + "_" + startIndex + "_to_" + stopIndex, gset.getNameEnglish(), gset.getMembers().subList(startIndex, stopIndex), checkForDuplicates);
    }

    public static GeneSet createGeneSet(GeneSetMatrix gm) {
        Set names = new HashSet();
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            names.addAll(gm.getGeneSet(i).getMembersS());
        }

        return new FSet(gm.getName() + "_setized", names);
    }

    public static GeneSet[] createGeneSets(String[] tab_delim_strings_gset_name_member) {

        Map gsetNameMemberSetMap = new HashMap();
        for (int i = 0; i < tab_delim_strings_gset_name_member.length; i++) {
            String[] ss = ParseUtils.string2strings(tab_delim_strings_gset_name_member[i], "\t", true);
            if (ss != null && ss.length == 2) {
                Object curr = gsetNameMemberSetMap.get(ss[0]);
                if (curr == null) {
                    curr = new HashSet();
                }
                ((Set) curr).add(ss[1]);
                gsetNameMemberSetMap.put(ss[0], curr);
            } else {
                klog.warn("Ignoring row: " + i + ">" + tab_delim_strings_gset_name_member[i]);
            }
        }

        return createGeneSets(gsetNameMemberSetMap);
    }

    /**
     * @param gsetNameMembersSet key -> GeneSet name, value -> a Set of member names
     * @return
     */
    public static GeneSet[] createGeneSets(final Map gsetNameMembersSet) {
        final GeneSet[] gsets = new GeneSet[gsetNameMembersSet.size()];
        final Iterator it = gsetNameMembersSet.keySet().iterator();
        int cnt = 0;
        while (it.hasNext()) {
            final Object gsetName = it.next();
            final Set members = (Set) gsetNameMembersSet.get(gsetName);
            gsets[cnt++] = new FSet(gsetName.toString(), members);
        }

        return gsets;
    }

    public static GeneSet[] createGeneSets_by_hashing_values(final Map memberIsKeySetNameIsValue) {

        final Map gsetNameMembersSetMap = new HashMap();

        final Iterator it = memberIsKeySetNameIsValue.keySet().iterator();
        while (it.hasNext()) {
            final String memberName = it.next().toString();
            final String setName = memberIsKeySetNameIsValue.get(memberName).toString();
            Object curr = gsetNameMembersSetMap.get(setName);
            if (curr == null) {
                curr = new HashSet();
            }
            ((Set) curr).add(memberName);
            gsetNameMembersSetMap.put(setName, curr);
        }

        return createGeneSets(gsetNameMembersSetMap);
    }

    public static GeneSet[] createRandomGeneSetsFixedSize(final int numRndGeneSets,
                                                          final RankedList rl,
                                                          final GeneSet gset,
                                                          final RandomSeedGenerator rst) {

        String prefix = NamingConventions.removeExtension(gset);

        // Qualify as all members may not be in the dataset
        int nmembers = gset.getNumMembers(rl);

        GeneSet[] rndgsets = new GeneSet[numRndGeneSets];

        for (int g = 0; g < numRndGeneSets; g++) {

            // IMP random from 0 to nrows not nmembers. duh!.
            int[] randomrowindices = XMath.randomlySampleWithoutReplacement(nmembers, rl.getSize(), rst);

            //Print.out(randomrowindices);

            if (randomrowindices.length != nmembers) {
                throw new IllegalStateException("random indices generated: " + randomrowindices.length + " not equal to # members: " + nmembers);
            }

            Set members = new HashSet();

            for (int i = 0; i < nmembers; i++) {
                //System.out.println(">> " + randomrowindices[i] + " " + ds.getRowName(randomrowindices[i]));
                members.add(rl.getRankName(randomrowindices[i]));
            }

            if (members.size() != nmembers) {
                klog.warn("Bad randomization -- repeated rnd members were made members: " + members.size() + " but wanted: " + nmembers);
            }

            rndgsets[g] = new FSet(prefix + "_" + g, members);

        }

        return rndgsets;
    }

    public static GeneSet[] createRandomGeneSets(int numRndGeneSets, Dataset ds, GeneSet gset, RandomSeedGenerator rst) {

        // As all members may not be in the dataset
        int nmembers = gset.getNumMembers(ds);

        return createRandomGeneSets(numRndGeneSets, ds, nmembers, rst);
    }

    public static GeneSet[] createRandomGeneSets(int numRndGeneSets, Dataset ds, int nmembers, RandomSeedGenerator rst) {

        GeneSet[] rndgsets = new GeneSet[numRndGeneSets];

        for (int g = 0; g < numRndGeneSets; g++) {

            // IMP random from 0 to nrows not nmembers. duh!.
            int[] randomrowindices = XMath.randomlySampleWithoutReplacement(nmembers, ds.getNumRow(), rst);

            //Print.out(randomrowindices);

            if (randomrowindices.length != nmembers) {
                throw new IllegalStateException("random indices generated: " + randomrowindices.length + " not equal to # members: " + nmembers);
            }

            Set members = new HashSet();

            for (int i = 0; i < nmembers; i++) {
                //System.out.println(">> " + randomrowindices[i] + " " + ds.getRowName(randomrowindices[i]));
                members.add(ds.getRowName(randomrowindices[i]));
            }

            if (members.size() != nmembers) {
                klog.warn("Bad randomization -- repeated rnd members were made members: " + members.size() + " but wanted: " + nmembers);
            }

            rndgsets[g] = new FSet(ds.getName() + "_" + g, members);
        }

        return rndgsets;
    }

    // @maint has to be manually kept in synch with GeneSetRandomizerType
    public static GeneSet[] createRandomGeneSets(final int num,
                                                 final Dataset ds,
                                                 final GeneSet gset,
                                                 final int minSize,
                                                 final RandomSeedGenerator rst,
                                                 final GeneSetRandomizerType grt) {

        if (grt == GeneSetRandomizerType.RANDOM_TAGS_FIXED_SIZE) {
            return createRandomGeneSets(num, ds, gset, rst);
        } else if (grt == GeneSetRandomizerType.RANDOM_TAGS_RANDOM_SIZE) {
            return createRandomGeneSetsWithRandomSizesUptoSize(num, ds, minSize, gset, rst);
        } else {
            throw new IllegalArgumentException("Unknown RandomizerType: " + grt);
        }

    }

    // all rnd subsets of size: rndSubsetSize
    public static GeneSet[] createRandomGeneSets(int rndSubsetSize, int numRndSubSets, GeneSet gset, RandomSeedGenerator rst) {

        String prefix = NamingConventions.removeExtension(gset);

        int nmembers = gset.getNumMembers();

        GeneSet[] rndgsets = new GeneSet[numRndSubSets];

        for (int g = 0; g < numRndSubSets; g++) {
            int[] randomrowindices = XMath.randomlySampleWithoutReplacement(rndSubsetSize, gset.getNumMembers(), rst);
            //Print.out(randomrowindices);

            Set members = new HashSet();

            for (int i = 0; i < randomrowindices.length; i++) {
                //System.out.println(">> " + randomrowindices[i] + " " + ds.getRowName(randomrowindices[i]));
                members.add(gset.getMember(randomrowindices[i]));
            }

            if (members.size() != randomrowindices.length) {
                klog.warn("Bad randomization -- repeated rnd members were made members: " + members.size() + " but wanted: " + nmembers);
            }

            rndgsets[g] = new FSet(prefix + "_" + g, members);

        }

        return rndgsets;
    }

    // rnd gset sizes are not fixed at the size of input gset
    // they can range from 0 to size of input gset
    public static GeneSet[] createRandomGeneSetsWithRandomSizesUptoSize(final int numRndGeneSets,
                                                                        final Dataset ds,
                                                                        final int minSize,
                                                                        final GeneSet gset,
                                                                        final RandomSeedGenerator rst) {

        String prefix = NamingConventions.removeExtension(gset);

        // As all members may not be in the dataset
        int nmembers = gset.getNumMembers(ds);

        return createRandomGeneSetsWithRandomSizesUptoSize(prefix, numRndGeneSets, ds, minSize, nmembers, rst);
    }


    public static GeneSet[] createRandomGeneSetsWithRandomSizesUptoSize(final String gsetNamePrefix,
                                                                        final int numRndGeneSets,
                                                                        final Dataset ds,
                                                                        final int fromSize,
                                                                        final int uptoSize,
                                                                        final RandomSeedGenerator rst) {
        return createRandomGeneSetsWithRandomSizesUptoSize(gsetNamePrefix, numRndGeneSets, ds.getRowNames(), fromSize, uptoSize, rst);
    }

    public static GeneSet[] createRandomGeneSetsWithRandomSizesUptoSize(final String gsetNamePrefix,
                                                                        final int numRndGeneSets,
                                                                        final GeneSet gset,
                                                                        final int minSize,
                                                                        final int uptoSize,
                                                                        final RandomSeedGenerator rst) {
        return createRandomGeneSetsWithRandomSizesUptoSize(gsetNamePrefix, numRndGeneSets, gset.getMembers(), minSize, uptoSize, rst);
    }

    public static GeneSet[] createRandomGeneSetsWithRandomSizesUptoSize(final String gsetNamePrefix,
                                                                        final int numRndGeneSets,
                                                                        final List names,
                                                                        final int minSize,
                                                                        final int uptoSize,
                                                                        final RandomSeedGenerator rst) {

        // As all members may not be in the dataset
        GeneSet[] rndgsets = new GeneSet[numRndGeneSets];

        int cnt = 0;
        while (true) {
            int num2use = rst.getRandom().nextInt(uptoSize); // upto a max of nmembers in the gset

            // @note bad hack
            if (num2use == 0 || num2use < minSize) {
                continue;
                //num2use = rst.getRandom().nextInt(uptoSize); // dont know how to disable getting 0 ??
            }

            // IMP random from 0 to nrows not nmembers. duh!.
            int[] randomrowindices = XMath.randomlySampleWithoutReplacement(num2use, names.size(), rst);

            if (randomrowindices.length != num2use) {
                throw new IllegalStateException("random indices generated: " + randomrowindices.length + " not equal to # members: " + uptoSize);
            }

            Set members = new HashSet(num2use);

            for (int i = 0; i < num2use; i++) {
                members.add(names.get(randomrowindices[i]));
            }

            rndgsets[cnt] = new FSet(gsetNamePrefix + "_" + cnt + "_" + num2use, members);
            cnt++;

            if (cnt >= numRndGeneSets) {
                break;
            }
        }

        return rndgsets;
    }

    public static GeneSet createRandomGeneSet(String name, int nmembers, Dataset ds, RandomSeedGenerator rst) {

        // IMP random from 0 to nrows not nmembers. duh!.
        int[] randomrowindices = XMath.randomlySampleWithoutReplacement(nmembers, ds.getNumRow(), rst);

        Set members = new HashSet(nmembers);

        for (int i = 0; i < nmembers; i++) {
            //System.out.println(">> " + randomrowindices[i] + " " + ds.getRowName(randomrowindices[i]));
            members.add(ds.getRowName(randomrowindices[i]));
        }

        return new FSet(name, members);
    }

    public static GeneSet createRandomGeneSet(String name, int nmembers, Dataset ds, Random rnd) {

        // IMP random from 0 to nrows not nmembers. duh!.
        int[] randomrowindices = XMath.randomlySampleWithoutReplacement(nmembers, ds.getNumRow(), rnd);

        Set members = new HashSet(nmembers);

        for (int i = 0; i < nmembers; i++) {
            //System.out.println(">> " + randomrowindices[i] + " " + ds.getRowName(randomrowindices[i]));
            members.add(ds.getRowName(randomrowindices[i]));
        }

        return new FSet(name, members);
    }

    public static GeneSet createRandomGeneSet(String name, int nmembers, Dataset ds) {

        // IMP random from 0 to nrows not nmembers. duh!.
        int[] randomrowindices = XMath.randomlySampleWithoutReplacement(nmembers, ds.getNumRow(), new Random());

        Set members = new HashSet(nmembers);

        for (int i = 0; i < nmembers; i++) {
            //System.out.println(">> " + randomrowindices[i] + " " + ds.getRowName(randomrowindices[i]));
            members.add(ds.getRowName(randomrowindices[i]));
        }

        return new FSet(name, members);
    }

    public static GeneSet createRandomGeneSet(String name, int nmembers, GeneSet gset, RandomSeedGenerator rst) {

        // IMP random from 0 to nrows not nmembers. duh!.
        int[] randomrowindices = XMath.randomlySampleWithoutReplacement(nmembers, gset.getNumMembers(), rst);

        Set members = new HashSet(nmembers);

        for (int i = 0; i < nmembers; i++) {
            //System.out.println(">> " + randomrowindices[i] + " " + ds.getRowName(randomrowindices[i]));
            members.add(gset.getMember(randomrowindices[i]));
        }

        return new FSet(name, members);
    }

    public static GeneSet[] createRandomGeneSets(String name, int nmembers, Dataset ds, int num, RandomSeedGenerator rst) {

        GeneSet[] gsets = new GeneSet[num];
        for (int i = 0; i < num; i++) {
            gsets[i] = createRandomGeneSet(name, nmembers, ds, rst);
        }

        return gsets;
    }

    // imp stop is like length so n not n-1
    public static GeneSet[] createPans(int fromIndex, int toIndex, int stepSize, GeneSet bigGeneSet) {

        if (toIndex > bigGeneSet.getNumMembers()) {
            throw new IllegalArgumentException("stop rank: " + toIndex + " cannot be > that gset size: " + bigGeneSet.getNumMembers());
        }

        SimpleRange[] ranges = RangeFactory.createPans(fromIndex, toIndex, stepSize, true);

        return _createGeneSets(ranges, bigGeneSet);

    }

    public static GeneSet[] createPans(int stepSize, GeneSet bigGeneSet) {
        return createPans(0, bigGeneSet.getNumMembers(), stepSize, bigGeneSet);
    }

    public static GeneSet[] createPans(int fromIndex, int toIndex, int stepSize, ScoredDataset sds) {
        GeneSet gset = new FSet(sds.getName(), null, sds.getRankedNames(), false);
        return createPans(fromIndex, toIndex, stepSize, gset);
    }

    public static GeneSet[] createPans(int stepSize, ScoredDataset sds) {
        GeneSet gset = new FSet(sds.getName(), null, sds.getRankedNames(), false);
        return createPans(0, sds.getNumRow(), stepSize, gset);
    }

    // imp stop is like length so n not n-1
    public static GeneSet[] createWindows(int fromIndex, int toIndex, int winSize, int stepSize, GeneSet bigGeneSet) {
        SimpleRange[] ranges = RangeFactory.createWindows(fromIndex, toIndex, winSize, stepSize);
        return _createGeneSets(ranges, bigGeneSet);
    }

    private static GeneSet[] _createGeneSets(Range[] ranges, GeneSet bigGeneSet) {

        List gsets = new ArrayList(ranges.length);

        List bigList = bigGeneSet.getMembers();
        for (int i = 0; i < ranges.length; i++) {
            List subList = bigList.subList((int) ranges[i].getMin(), (int) ranges[i].getMax());
            gsets.add(new FSet(bigGeneSet.getName() + "_" + ranges[i].getMin() + "_" + ranges[i].getMax(), bigGeneSet.getNameEnglish(), subList, false));
        }

        return (FSet[]) gsets.toArray(new FSet[gsets.size()]);
    }

    public static GeneSet[] createWindows(int winSize, int stepSize, GeneSet bigGeneSet) {
        return createWindows(0, bigGeneSet.getNumMembers(), winSize, stepSize, bigGeneSet);
    }

    public static GeneSet[] createWindows(int fromIndex, int toIndex, int winSize, int stepSize, ScoredDataset sds) {
        GeneSet gset = new FSet(sds.getName(), null, sds.getRankedNames(), false);
        return createWindows(fromIndex, toIndex, winSize, stepSize, gset);
    }

    public static List createWindowsL(int fromIndex, int toIndex, int winSize, int stepSize, ScoredDataset sds) {
        GeneSet[] gsets = createWindows(fromIndex, toIndex, winSize, stepSize, sds);
        List gsetsList = new ArrayList(gsets.length);
        for (int i = 0; i < gsets.length; i++) {
            gsetsList.add(gsets[i]);
        }

        return gsetsList;
    }

    public static GeneSet createGeneSet(String name, RankedFeature[] dfs) {
        Set set = new HashSet();
        for (int i = 0; i < dfs.length; i++) {
            set.add(dfs[i].getName());
        }

        return new FSet(name, set);
    }

    public static GeneSet[] qualifyGeneSets(final GeneSet[] gsets, Dataset ds, boolean removeNativeNames) {

        GeneSet[] ret = new GeneSet[gsets.length];

        for (int i = 0; i < gsets.length; i++) {
            ret[i] = gsets[i].cloneDeep(ds);
            if (removeNativeNames) {
                ret[i] = ret[i].cloneShallow(AuxUtils.getAuxNameOnlyNoHash(ret[i].getName()));
            }

            if (i != 0 && i % 100 == 0) {
                System.out.println("qualfied gset: " + gsets[i].getName() + " " + (i + 1) + " / " + gsets.length);
            }
        }

        return ret;
    }

    public static GeneSet[] qualifyGeneSets2(final GeneSet[] gsets, Dataset ds, boolean removeNativeNames) {

        GeneSet[] ret = new GeneSet[gsets.length];

        for (int i = 0; i < gsets.length; i++) {
            ret[i] = gsets[i].cloneDeep(ds);
            if (removeNativeNames) {
                ret[i] = ret[i].cloneShallow(AuxUtils.getAuxNameOnlyNoHash(ret[i].getName()) + "_" + ret[i].getNumMembers());
            }

            if (i != 0 && i % 100 == 0) {
                System.out.println("qualfied gset: " + gsets[i].getName() + " " + (i + 1) + " / " + gsets.length);
            }
        }

        return ret;
    }

    public static GeneSet[] removeGeneSetsSmallerThan(final GeneSet[] gsets, final int cutoff) {

        List list = new ArrayList();

        for (int i = 0; i < gsets.length; i++) {
            if (gsets[i].getNumMembers() < cutoff) {
                // ignore
            } else {
                list.add(gsets[i]);
            }
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);

    }

    public static GeneSet[] removeGeneSetsSmallerThan(final GeneSet[] gsets, final int cutoff,
                                                      final Object ds_or_rl, final boolean firstQualifyByCloning) {
        if (ds_or_rl instanceof Dataset) {
            return removeGeneSetsSmallerThan(gsets, cutoff, (Dataset) ds_or_rl, firstQualifyByCloning);
        } else {
            return removeGeneSetsSmallerThan(gsets, cutoff, (RankedList) ds_or_rl, firstQualifyByCloning);
        }
    }

    public static GeneSet[] removeGeneSetsSmallerThan(final GeneSet[] gsets, final int cutoff, final RankedList rl, final boolean firstQualifyByCloning) {
        // as an opt first apply the ds less filter
        GeneSet[] ogsets = removeGeneSetsSmallerThan(gsets, cutoff);

        List list = new ArrayList();
        for (int i = 0; i < ogsets.length; i++) {
            GeneSet gset;

            if (firstQualifyByCloning) {
                gset = ogsets[i].cloneDeep(rl);
            } else {
                gset = ogsets[i];
            }

            int num = gset.getNumMembers();
            if (num < cutoff) {
                // ignore
            } else {
                list.add(gset);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsSmallerThan: " + cutoff + " for: " + (i + 1) + " / " + ogsets.length);
            }
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
    }

    public static GeneSet[] removeGeneSetsSmallerThan(final GeneSet[] gsets, final int cutoff, final Dataset ds, final boolean firstQualifyByCloning) {
        // as an opt first apply the ds less filter
        GeneSet[] ogsets = removeGeneSetsSmallerThan(gsets, cutoff);

        /* orig impl changed for ds qual jan 05
        // then do the ds checking
        for (int i = 0; i < ogsets.length; i++) {
            int num = AlgUtils.getNumOfMembers(ds, ogsets[i]);
            if (num < cutoff) {
                ; // ignore
            } else {
                list.add(ogsets[i]);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsSmallerThan " + (i + 1) + " / " + ogsets.length);
            }
        }

        */

        List list = new ArrayList();
        for (int i = 0; i < ogsets.length; i++) {
            GeneSet gset;

            if (firstQualifyByCloning) {
                gset = ogsets[i].cloneDeep(ds);
            } else {
                gset = ogsets[i];
            }

            int num = gset.getNumMembers();
            if (num < cutoff) {
                // ignore
            } else {
                list.add(gset);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsSmallerThan: " + cutoff + " for: " + (i + 1) + " / " + ogsets.length);
            }
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);

    }

    public static GeneSet[] removeGeneSetsLargerThan(final GeneSet[] gsets,
                                                     final int cutoff,
                                                     final Object ds_or_rl,
                                                     final boolean firstQualifyByCloning) {
        if (ds_or_rl instanceof Dataset) {
            return removeGeneSetsLargerThan(gsets, cutoff, (Dataset) ds_or_rl, firstQualifyByCloning);
        } else {
            return removeGeneSetsLargerThan(gsets, cutoff, (RankedList) ds_or_rl, firstQualifyByCloning);
        }

    }

    public static GeneSet[] removeGeneSetsLargerThan(final GeneSet[] ogsets,
                                                     final int cutoff,
                                                     final RankedList rl,
                                                     final boolean firstQualifyByCloning) {

        // as an opt first apply the ds less filter
        //GeneSet[] ogsets = removeGeneSetsLargerThan(gsets, cutoff); // dont do this!!! wrong!!

        List list = new ArrayList();
        for (int i = 0; i < ogsets.length; i++) {

            GeneSet gset;

            if (firstQualifyByCloning) {
                gset = ogsets[i].cloneDeep(rl);
            } else {
                gset = ogsets[i];
            }


            int num = AlgUtils.getNumOfMembers(rl, gset);
            if (num > cutoff) {
                // ignore
            } else {
                list.add(gset);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsLargerThan " + (i + 1) + " / " + ogsets.length);
            }
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
    }

    public static GeneSet[] removeGeneSetsLargerThan(final GeneSet[] ogsets,
                                                     final int cutoff,
                                                     final Dataset ds,
                                                     final boolean firstQualifyByCloning) {

        // as an opt first apply the ds less filter
        //GeneSet[] ogsets = removeGeneSetsLargerThan(gsets, cutoff); // dont do this!!! wrong!!

        List list = new ArrayList();
        for (int i = 0; i < ogsets.length; i++) {

            GeneSet gset;

            if (firstQualifyByCloning) {
                gset = ogsets[i].cloneDeep(ds);
            } else {
                gset = ogsets[i];
            }


            int num = AlgUtils.getNumOfMembers(ds, gset);
            if (num > cutoff) {
                // ignore
            } else {
                list.add(gset);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsLargerThan " + (i + 1) + " / " + ogsets.length);
            }
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
    }

    public static GeneSet[] removeGeneSetsBySize(final GeneSetMatrix gmx, final int min, final int max,
                                                 final Dataset ds) {

        return removeGeneSetsBySize(gmx.getGeneSets(), min, max, ds);
    }

    public static GeneSet[] removeGeneSetsBySize(final GeneSet[] gsets, final int min, final int max,
                                                 final Dataset ds) {
        // Do the ds checking
        List list = new ArrayList();

        /* orig impl before ds qualify clone
        for (int i = 0; i < gsets.length; i++) {
            int num = AlgUtils.getNumOfMembers(ds, gsets[i]);
            if (num < min || num > max) {
                ; // ignore
            } else {
                list.add(gsets[i]);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsBySize " + (i + 1) + " / " + gsets.length);
            }
        }
        */

        for (int i = 0; i < gsets.length; i++) {
            GeneSet gset = gsets[i].cloneDeep(ds);
            int num = gset.getNumMembers();
            if (num < min || num > max) {
                // ignore
            } else {
                list.add(gset);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsBySize " + (i + 1) + " / " + gsets.length);
            }
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
    }

    public static GeneSet[] removeGeneSetsBySize(final GeneSet[] gsets,
                                                 final int min,
                                                 final int max) {
        // Do the ds checking
        List list = new ArrayList();

        for (int i = 0; i < gsets.length; i++) {
            int num = gsets[i].getNumMembers();
            if (num < min || num > max) {
                // ignore
            } else {
                list.add(gsets[i]);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsBySize " + (i + 1) + " / " + gsets.length);
            }
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
    }

    public static GeneSetMatrix removeGeneSetsBySize(final GeneSetMatrix gmx,
                                                     final int min,
                                                     final int max) {
        return new DefaultGeneSetMatrix(gmx.getName() + "_" + min + "_" + max,
                removeGeneSetsBySize(gmx.getGeneSets(), min, max));
    }

    public static GeneSet[] removeGeneSetsBySize(final GeneSet[] gsets,
                                                 final int min,
                                                 final int max,
                                                 final Set restrict2TheseIds) {
        // then do the ds checking
        List list = new ArrayList();
        for (int i = 0; i < gsets.length; i++) {
            int num = AlgUtils.getNumOfMembers(restrict2TheseIds, gsets[i]);
            if (num < min || num > max) {
                // ignore
            } else {
                list.add(gsets[i]);
            }

            if (i != 0 && i % 500 == 0) {
                System.out.println("Done removeGeneSetsBySize " + (i + 1) + " / " + gsets.length);
            }
        }

        return (GeneSet[]) list.toArray(new GeneSet[list.size()]);
    }

    public static GeneSet[] removeGeneSetsLargerThan(final GeneSet[] gsets, final int cutoff) {

        List list = new ArrayList();

        for (int i = 0; i < gsets.length; i++) {
            if (gsets[i].getNumMembers() > cutoff) {
                // ignore
            } else {
                list.add(gsets[i]);
            }
        }

        return (FSet[]) list.toArray(new FSet[list.size()]);

    }

}