/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.meg.Gene;
import gnu.trove.TFloatIntHashMap;
import gnu.trove.TObjectIntHashMap;
import gnu.trove.TObjectIntIterator;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Jul 13, 2003
 *         Time: 2:04:45 AM
 */
public class GeneSetStats {

    private final Logger log = XLogger.getLogger(GeneSetStats.class);

    /**
     * Class Constructor.
     */
    public GeneSetStats() {
    }

    public RedStruc calcRedundancy(final GeneSetMatrix gm,
                                   final boolean do2mersalso) {
        return calcRedundancy(gm.getGeneSets(), do2mersalso);
    }

    public float calcJacquardMean(final GeneSetMatrix gm) {
        return calcRedundancy(gm.getGeneSets(), false).jacquardMean;
    }

    // rec = number of unique 2mers / total number of possible 2 mers
    // NOT same as co-occurrence
    public RedStruc calcRedundancy(final GeneSet[] gsets, final boolean do2mersalso) {

        RedStruc rs = new RedStruc();
        if (do2mersalso) {
            rs.allFoundTwoMers = new HashSet();
            for (int i = 0; i < gsets.length; i++) {
                int n = gsets[i].getNumMembers();
                int num = n * (n - 1) / 2;
                Set s = hashTwoMers(gsets[i]);
                if (s.size() != num) {
                    log.warn("Bad: " + s.size() + " " + num);
                    //throw new IllegalStateException("Bad: " + s.size() + " " + num);
                }
                rs.allFoundTwoMers.addAll(s);
                rs.totNumPairsPossible += num;
            }
        }

        //      * Sets as rows, genes as columns. Aligned
        BitSetDataset bsd = new BitSetDataset(new DefaultGeneSetMatrix("foo", gsets));
        Dataset ds = bsd.and_by_or();

        LabelledVector lv = getFeatureFrequency_lv(gsets);
        rs.featureFreq = lv.sort(SortMode.REAL, Order.DESCENDING);

        float sum = 0;
        // @note IMP intentionally using hash set so that we dont have to worry about nums
        rs.jacquardDistrib = new TFloatIntHashMap();
        for (int r = 0; r < ds.getNumRow(); r++) {
            for (int c = 0; c < ds.getNumCol(); c++) {
                sum += ds.getElement(r, c);
                if (r < c) {
                    int curr = rs.jacquardDistrib.get(ds.getElement(r, c));
                    curr++;
                    rs.jacquardDistrib.put(ds.getElement(r, c), curr);
                }
            }
        }

        rs.jacquardMean = sum / (ds.getNumRow() * ds.getNumRow());

        //log.info("Total possible: " + rs.totNumPairsPossible + " from sets: " + gsets.length + " total unique seen: " + rs.allFoundTwoMers.size());

        return rs;
    }

    // @todo optimize
    public Set hashTwoMers(final GeneSet gset) {
        Set set = new HashSet();
        for (int i = 0; i < gset.getNumMembers(); i++) {
            String a = gset.getMember(i);
            for (int j = 0; j < gset.getNumMembers(); j++) {
                if (j != i) {
                    set.add(new TwoMer(a, gset.getMember(j)));
                }
            }
        }

        return set;
    }

    public static class RedStruc {

        public int totNumPairsPossible;
        public Set allFoundTwoMers;

        public float jacquardMean;
        public TFloatIntHashMap jacquardDistrib;

        public RankedList featureFreq;

        // disabled for now
        public float getTwoMerFraction() {
            if (allFoundTwoMers == null) {
                return Float.NaN;
            } else {
                return ((float) allFoundTwoMers.size()) / (float) totNumPairsPossible;
            }
        }
    }

    // more efficient to make the call with all gsets as we do the gmann qualification

    public LabelledObject[] calcHyperGeometric_genesets(final GeneSet[] gsets,
                                                        final GeneSetMatrix gmdb,
                                                        final double maxPValueAllowed,
                                                        final int maxGeneSetSizeAllowed,
                                                        final Collection symbolSet) {

        if (gsets == null) {
            throw new IllegalArgumentException("Parameter gsets cannot be null");
        }

        if (gmdb == null) {
            throw new IllegalArgumentException("Parameter gmdb cannot be null");
        }

        /**
         * @param k The number of black balls drawn
         * @param N The total number of balls
         * @param K The number of black balls
         * @param n The number of balls drawn
         * @return The hypergeometric value
         */

        int K[] = new int[gsets.length];
        List[] names = new ArrayList[gsets.length];
        List[] hypers = new ArrayList[gsets.length];

        // init stuff
        for (int i = 0; i < gsets.length; i++) {
            K[i] = gsets[i].getNumMembers(symbolSet);
            names[i] = new ArrayList();
            hypers[i] = new ArrayList();

            // sanity check
            if (K[i] != gsets[i].getNumMembers()) {
                throw new IllegalArgumentException("gsets not qualified!!: " + K[i] + " " + gsets[i].getNumMembers() + " for: " + gsets[i].getName() + " " + gsets[i].getMembersS()); // @todo remove me later waste of compute time
            }

        }

        int N = symbolSet.size();
        log.info("symbol size for hypergeo: " + N);

        for (int i = 0; i < gmdb.getNumGeneSets(); i++) {
            GeneSet db = gmdb.getGeneSet(i).cloneDeep(symbolSet); // @note qualification to the symbol set
            int n = db.getNumMembers();
            if (n == 0) {
                log.warn("No matches to the symbol set");
            } else if (n < maxGeneSetSizeAllowed) { // do only for smaller sets

                for (int g = 0; g < gsets.length; g++) {
                    GeneSet kgs = AlgUtils.intersect("foo", gsets[g], db);
                    int k = kgs.getNumMembers();
                    double pv = XMath.hypergeometric(k, N, K[g], n);
                    if (pv <= maxPValueAllowed) {
                        if (k == 0) {
                            // @todo maybe a bug check later
                            //log.debug("k: " + k + " N: " + N + " K: " + K[g] + " n: " + n + " p: " + pv);
                        } else {
                            hypers[g].add(new HyperGeometricStruc(pv, k, gsets[g].getNumMembers(), n, kgs));
                            names[g].add(db.getName());
                        }
                    }
                }
            }
        }

        LabelledObject[] lvs = new LabelledObject[gsets.length];
        for (int g = 0; g < gsets.length; g++) {
            lvs[g] = new LabelledObject(AuxUtils.getAuxNameOnlyNoHash(gsets[g].getName()), names[g], hypers[g]);
            lvs[g] = sort(lvs[g]);
        }

        return lvs;
    }

    /**
     * Class constructor
     *
     * @param dstName
     * @param gmSigns
     * @param gmDb
     * @param chip
     * @return
     * @throws Exception
     */
    public StringDataframe createHyperStatsReportInProbeSpace(final String dstName,
                                                              final GeneSetMatrix gmSigns,
                                                              final GeneSetMatrix gmDb,
                                                              final Chip chip) throws Exception {

        LabelledVector freq = getFeatureFrequency_lv(gmSigns);
        TObjectIntHashMap dbfreq = getFeatureFrequency(gmDb);
        LabelledVector hyper = getHyperGeometric(gmSigns, gmDb);

        List colNames = new ArrayList();

        colNames.add("DEFAULT_TITLE"); // always do desc
        colNames.add("FREQ_IN_SIGNALS");
        colNames.add("FREQ_IN_GENSETSDB");
        colNames.add("HYPER_PVALUE");
        StringMatrix matrix = new StringMatrix(freq.getSize(), colNames.size());
        for (int r = 0; r < freq.getSize(); r++) {
            int coln = 0;
            String rowName = freq.getLabel(r);
            matrix.setElement(r, coln++, chip.getHugo(rowName).getTitle());
            matrix.setElement(r, coln++, freq.getScore(r));
            matrix.setElement(r, coln++, dbfreq.get(rowName));
            matrix.setElement(r, coln, hyper.getScore(r));

            if (!rowName.equals(hyper.getLabel(r))) {
                throw new IllegalStateException("Mismatch: " + rowName);
            }

        }

        return new StringDataframe("hyper_stats_" + dstName, matrix, freq.getLabels(), colNames, true);
    }

    // cant do a generic convertion from probes to gene names
    // the reason is this. In curated pathways, for instance, we started of (the curation) with a list of genes
    // and then converted those into probe names. If a gene had 2 probes, then in the prob'ed geneset it contains
    // 2x as many as in the original geneset. Now after finding enrichement and getting the signal, if we generically
    // convert back into genes, then the num of genes in the db will seem to be x, while really we should
    // be counting it as 2x
    public StringDataframe createHyperStatsReportInGeneSymbolSpace(final String dstName,
                                                                   final GeneSetMatrix gmSigns,
                                                                   final GeneSetMatrix gmDb,
                                                                   final Chip chip) throws Exception {
        if (chip == null) {
            throw new IllegalArgumentException("Parameter chip cannot be null");
        }

        TObjectIntHashMap gmSignsMap = getGeneSymbolFrequency(gmSigns, chip, true);
        TObjectIntHashMap gmDbMap = getGeneSymbolFrequency(gmDb, chip, true);

        List colNames = new ArrayList();

        colNames.add("DEFAULT_TITLE"); // always do desc
        colNames.add("FREQ_IN_SIGNALS");
        colNames.add("FREQ_IN_GENSETSDB");
        colNames.add("HYPER_PVALUE");
        StringMatrix matrix = new StringMatrix(gmSignsMap.size(), colNames.size());

        TObjectIntIterator it = gmSignsMap.iterator();
        List rowNames = new ArrayList();

        int n = AlgUtils.countTotal(gmDbMap);
        int m = AlgUtils.countTotal(gmSignsMap);

        // faster iteration by avoiding hasNext():
        int rowCnt = 0;
        for (int i = gmSignsMap.size(); i-- > 0;) {
            it.advance();
            String signGeneSymbol = it.key().toString();
            rowNames.add(signGeneSymbol);
            int k = it.value();
            int r = gmDbMap.get(signGeneSymbol);
            if (r <= 0) {
                throw new IllegalArgumentException("The gmdb is invalid missing entry for: " + signGeneSymbol);
            }

            double pv = XMath.hypergeometric(k, n, r, m);

            int coln = 0;
            Gene gene = chip.getHugo(signGeneSymbol);
            matrix.setElement(rowCnt, coln++, gene.getTitle());
            matrix.setElement(rowCnt, coln++, k);
            matrix.setElement(rowCnt, coln++, r);
            matrix.setElement(rowCnt, coln, pv);
            rowCnt++;
        }

        return new StringDataframe("hyper_stats_genes_" + dstName, matrix, rowNames, colNames, true);
    }

    public TObjectIntHashMap getFeatureFrequency(final GeneSetMatrix gm) {
        final String[] feats = gm.getAllMemberNames();
        return FrequencyAnalysis.featureFreq(feats);
    }

    public TObjectIntHashMap getFeatureFrequency(final GeneSet[] gsets) {
        return getFeatureFrequency(new DefaultGeneSetMatrix("tmp", gsets));
    }

    public LabelledVector getFeatureFrequency_lv(final GeneSet[] gsets) {
        return getFeatureFrequency_lv(new DefaultGeneSetMatrix("tmp", gsets));
    }

    public LabelledVector getFeatureFrequency_lv(final GeneSetMatrix gm) {
        TObjectIntHashMap map = getFeatureFrequency(gm);

        String[] uniqFeats = gm.getAllMemberNamesOnlyOnce();
        Vector v = new Vector(uniqFeats.length);
        for (int i = 0; i < uniqFeats.length; i++) {
            int cnt = map.get(uniqFeats[i]);
            v.setElement(i, cnt);
        }

        return new LabelledVector(uniqFeats, v);
    }

    public RankedList getFeatureFrequency_rl(final GeneSetMatrix gm, Order order) {
        return getFeatureFrequency_lv(gm).sort(SortMode.REAL, order); // Most frequent first
    }

    public RankedList getFeatureFrequency_rl(final GeneSet[] gsets, Order order) {
        return getFeatureFrequency_lv(gsets).sort(SortMode.REAL, order); // Most frequent first
    }

    public TObjectIntHashMap getGeneSymbolFrequency(final GeneSetMatrix gm,
                                                    final Chip chip,
                                                    final boolean countDuplicateSymbolsWithinASet) throws Exception {

        if (chip == null) {
            throw new IllegalArgumentException("Parameter chip cannot be null");
        }

        if (gm == null) {
            throw new IllegalArgumentException("Parameter gm cannot be null");
        }

        List gmDbGenesList = new ArrayList(); // will have duplicates, and we want that
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            Set symbols = chip.getSymbols(gm.getGeneSet(i), Chip.REPLACE_WITH_PROBEID);

            if (countDuplicateSymbolsWithinASet) {
                gmDbGenesList.addAll(symbols);
            } else {
                gmDbGenesList.addAll(new HashSet(symbols)); // get rid off duplicates within a set
            }
        }

        String[] gmDbGenes = (String[]) gmDbGenesList.toArray(new String[gmDbGenesList.size()]);
        return FrequencyAnalysis.featureFreq(gmDbGenes);
    }

    public LabelledVector getHyperGeometric(final GeneSetMatrix gm, final GeneSetMatrix gmdb) {

        int n = gmdb.getAllMemberNamesOnlyOnceS().size();

        String[] featsInDb = gmdb.getAllMemberNames();
        TObjectIntHashMap mapDb = FrequencyAnalysis.featureFreq(featsInDb);

        String[] feats = gm.getAllMemberNames();
        TObjectIntHashMap map = FrequencyAnalysis.featureFreq(feats);
        /**
         *  @param k The number of black balls drawn
         *  @param n The total number of balls
         *  @param r The number of black balls
         *  @param m The number of balls drawn
         *  @return The hypergeometric value
         */
        String[] uniqFeats = gm.getAllMemberNamesOnlyOnce();
        int m = uniqFeats.length;

        Vector v = new Vector(uniqFeats.length);
        for (int i = 0; i < uniqFeats.length; i++) {
            int k = map.get(uniqFeats[i]); // num of enriched black balls
            int r = mapDb.get(uniqFeats[i]); // num of black balls in the db
            double score = XMath.hypergeometric(k, n, r, m);
            v.setElement(i, (float) score);
        }

        return new LabelledVector(uniqFeats, v);
    }

    public BitSetDataset getCoOccs(final GeneSetMatrix gm,
                                   final int minFreqCutoffInclusive) {

        LabelledVector lv = getFeatureFrequency_lv(gm);
        String[] names = lv.getLabelsForValuesArray(minFreqCutoffInclusive, Integer.MAX_VALUE);

        String name = gm.getName() + "_cutoff_" + minFreqCutoffInclusive;
        return gm.getGeneSetMembershipBitSetDataset(name, names);
    }


    private LabelledObject sort(final LabelledObject lo) {

        int size = lo.getSize();

        List oiList = new ArrayList(size);
        for (int i = 0; i < size; i++) {
            oiList.add(new ObjectIndex(lo.getElement(i), i));
        }

        Collections.sort(oiList, new HyperPValueComparator());

        List labels = new ArrayList(oiList.size());
        List objects = new ArrayList(oiList.size());
        for (int i = 0; i < oiList.size(); i++) {
            ObjectIndex oi = (ObjectIndex) oiList.get(i);
            labels.add(lo.getLabel(oi.getIndex()));
            objects.add(lo.getElement(oi.getIndex()));
        }

        return new LabelledObject(lo.getName(), labels, objects);
    }


    static class HyperPValueComparator implements Comparator {

        public int compare(Object pn1, Object pn2) {
            //System.out.println(">>> " + pn1);

            ObjectIndex oi1 = (ObjectIndex) pn1;
            ObjectIndex oi2 = (ObjectIndex) pn2;

            HyperGeometricStruc h1 = (HyperGeometricStruc) oi1.getObject();
            HyperGeometricStruc h2 = (HyperGeometricStruc) oi2.getObject();

            if (h1.getPValue() < h2.getPValue()) {
                return -1;
            } else if (h1.getPValue() == h2.getPValue()) {
                return 0;
            } else {
                return +1;
            }
        }


        public boolean equals(Object o2) {
            return false;
        }
    }    // End HyperPValueComparator


} // End class GeneSetMatrixStats

