/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.DefaultGeneSetMatrix;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.parsers.AuxUtils;

/**
 *
 */


public class InfoCrystalAlg {

    private InfoCrystalAlg() {

    }

    /**
     * @param gm
     * @return
     */
    public static XWay createXWay(GeneSetMatrix gm) {

        if (gm.getNumGeneSets() == 4) {
            return new InfoCrystalAlg.FourWay(gm.getGeneSet(0), gm.getGeneSet(1), gm.getGeneSet(2), gm.getGeneSet(3));
        } else if (gm.getNumGeneSets() == 3) {
            return new InfoCrystalAlg.ThreeWay(gm.getGeneSet(0), gm.getGeneSet(1), gm.getGeneSet(2));
        } else if (gm.getNumGeneSets() == 2) {
            return new InfoCrystalAlg.TwoWay(gm.getGeneSet(0), gm.getGeneSet(1));
        } else if (gm.getNumGeneSets() == 5) {
            return new InfoCrystalAlg.FiveWay(gm.getGeneSet(0), gm.getGeneSet(1), gm.getGeneSet(2), gm.getGeneSet(3), gm.getGeneSet(4));
        } else {
            throw new IllegalArgumentException("InfoCrystalAlg Only implemented for 2, 3, 4 & 5 GeneSets in the GeneSetMatrix. Found #: " + gm.getNumGeneSets() + "\nTry using FindManyOverlaps for intersecting many many gene sets");
        }
    }

    public interface XWay {

        public String getAsString();

        public GeneSetMatrix getGeneSetMatrix();

        public int[] getInts();

    }


    private static void add(GeneSet fset, StringBuffer buf) {
        buf.append(fset.getName()).append('\t').append(fset.getNumMembers()).append('\n');
    }

    private static void add(GeneSet fset, String alias, StringBuffer buf) {
        buf.append(fset.getName()).append('\t').append(alias).append('\t').append(fset.getNumMembers()).append('\n');
    }

    /**
     *
     */
    public static class TwoWay implements XWay {

        private GeneSet aFull;
        private GeneSet bFull;
        private GeneSet ab;
        private GeneSet aOnly;
        private GeneSet bOnly;
        private String name;

        public TwoWay(final GeneSet a, final GeneSet b) {

            final String aName = AuxUtils.getAuxNameOnlyNoHash(a);
            final String bName = AuxUtils.getAuxNameOnlyNoHash(b);

            this.aFull = a.cloneDeep(aName + "_FULL", a.getNameEnglish());
            this.bFull = b.cloneDeep(bName + "_FULL", b.getNameEnglish());
            this.ab = AlgUtils.intersect("ab", a, b);
            this.aOnly = AlgUtils.subtract(aName + "_ONLY", aFull, bFull);
            this.bOnly = AlgUtils.subtract(bName + "_ONLY", bFull, aFull);
            this.name = a.getName() + "_" + b.getName() + "_twoway";
        }

        public String getAsString() {

            StringBuffer buf = new StringBuffer(name).append('\n');
            add(aFull, buf);
            add(bFull, buf);
            add(ab, buf);
            add(aOnly, buf);
            add(bOnly, buf);

            return buf.toString();
        }

        public GeneSetMatrix getGeneSetMatrix() {
            return new DefaultGeneSetMatrix(name, new GeneSet[]{aFull, bFull, ab, aOnly, bOnly});
        }

        public int[] getInts() {
            return new int[]{ab.getNumMembers()};
        }

    }

    public static class ThreeWay implements XWay {

        private GeneSet aFull;
        private GeneSet bFull;
        private GeneSet cFull;
        private GeneSet ab;
        private GeneSet abOnly;
        private GeneSet ac;
        private GeneSet acOnly;
        private GeneSet bc;
        private GeneSet bcOnly;
        private GeneSet aOnly;
        private GeneSet bOnly;
        private GeneSet cOnly;
        private GeneSet abc;
        private String name;

        public ThreeWay(final GeneSet a, final GeneSet b, final GeneSet c) {

            final String aName = AuxUtils.getAuxNameOnlyNoHash(a);
            final String bName = AuxUtils.getAuxNameOnlyNoHash(b);
            final String cName = AuxUtils.getAuxNameOnlyNoHash(c);

            aFull = a.cloneDeep(aName + "_FULL", a.getNameEnglish());
            bFull = b.cloneDeep(bName + "_FULL", b.getNameEnglish());
            cFull = c.cloneDeep(cName + "_FULL", c.getNameEnglish());

            ab = AlgUtils.intersect(aName + "_INTERSECT_" + bName, a, b);
            ac = AlgUtils.intersect(aName + "_INTERSECT_" + cName, a, c);
            bc = AlgUtils.intersect(bName + "_INTERSECT_" + cName, b, c);
            abc = AlgUtils.intersectAll("abc", new GeneSet[]{a, b, c});

            aOnly = AlgUtils.subtract(aName + "_ONLY", aFull, AlgUtils.unionAll("foo", new GeneSet[]{ab, ac}));
            bOnly = AlgUtils.subtract(bName + "_ONLY", bFull, AlgUtils.unionAll("foo", new GeneSet[]{ab, bc}));
            cOnly = AlgUtils.subtract(cName + "_ONLY", cFull, AlgUtils.unionAll("foo", new GeneSet[]{ac, bc}));
            this.name = a.getName() + "_" + b.getName() + "_" + c.getName() + "_threeway";

            abOnly = AlgUtils.subtract(aName + "_" + bName + "_ONLY", ab, abc);
            acOnly = AlgUtils.subtract(aName + "_" + cName + "_ONLY", ac, abc);
            bcOnly = AlgUtils.subtract(bName + "_" + cName + "_ONLY", bc, abc);

        }

        public String getAsString() {

            /*
           Left circle	aOnly
           Top Overlap	ab
           Right Circle	bOnly
           Right Overlap	bc
           Bottom Circle	cOnly
           Left Overlap	ac
           Center	abc
           */

            StringBuffer buf = new StringBuffer();
            add(aFull, "", buf);
            add(bFull, "", buf);
            add(cFull, "", buf);
            add(ab, "", buf);
            add(ac, "", buf);
            add(bc, "", buf);
            add(aOnly, "Left Circle", buf);
            add(bOnly, "Right Circle", buf);
            add(cOnly, "Bottom Circle", buf);

            add(abOnly, "Top Overlap", buf);
            add(acOnly, "Left Overlap", buf);
            add(bcOnly, "Right Overlap", buf);

            add(abc, "Center", buf);

            return buf.toString();
        }

        public GeneSetMatrix getGeneSetMatrix() {
            return new DefaultGeneSetMatrix(name, new GeneSet[]{aFull, bFull, cFull, ab, ac, bc, aOnly, bOnly, cOnly, abc});
        }

        public int[] getInts() {
            return new int[]{ab.getNumMembers(), ac.getNumMembers(), bc.getNumMembers()};
        }

    }

    /**
     *
     */
    public static class FourWay implements XWay {

        // the fsets being compared
        private GeneSet aFull;
        private GeneSet bFull;
        private GeneSet cFull;
        private GeneSet dFull;

        // pairwises
        private GeneSet ab;
        private GeneSet ac;
        private GeneSet ad;

        private GeneSet bc;
        private GeneSet bd;
        private GeneSet cd;

        // hmm these are not represented?
        private GeneSet abc;
        private GeneSet acd;
        private GeneSet abd;
        private GeneSet bcd;

        // 3-ways with Not fourth
        private GeneSet abcNotd;
        private GeneSet acdNotb;
        private GeneSet abdNotc;
        private GeneSet bcdNota;

        // all
        private GeneSet abcd;

        // ones by themselves
        private GeneSet aUsed;
        private GeneSet bUsed;
        private GeneSet cUsed;
        private GeneSet dUsed;

        private String name;

        public FourWay(final GeneSet a, final GeneSet b, final GeneSet c, final GeneSet d) {

            final String aName = AuxUtils.getAuxNameOnlyNoHash(a);
            final String bName = AuxUtils.getAuxNameOnlyNoHash(b);
            final String cName = AuxUtils.getAuxNameOnlyNoHash(c);
            final String dName = AuxUtils.getAuxNameOnlyNoHash(d);

            this.aFull = a.cloneDeep(aName + "_FULL", a.getNameEnglish());
            this.bFull = b.cloneDeep(bName + "_FULL", b.getNameEnglish());
            this.cFull = c.cloneDeep(cName + "_FULL", c.getNameEnglish());
            this.dFull = d.cloneDeep(dName + "_FULL", d.getNameEnglish());

            // all pairwises
            this.ab = AlgUtils.intersect(aName + "_INTERSECT_" + bName, a, b);
            this.ac = AlgUtils.intersect(aName + "_INTERSECT_" + cName, a, c);
            this.ad = AlgUtils.intersect(aName + "_INTERSECT_" + dName, a, d);
            this.bc = AlgUtils.intersect(bName + "_INTERSECT_" + cName, b, c);
            this.bd = AlgUtils.intersect(bName + "_INTERSECT_" + dName, b, d);
            this.cd = AlgUtils.intersect(cName + "_INTERSECT_" + dName, c, d);

            // all three ways
            // 3-ways
            this.abc = AlgUtils.intersectAll("abc", new GeneSet[]{a, b, c});
            this.acd = AlgUtils.intersectAll("acd", new GeneSet[]{a, c, d});
            this.abd = AlgUtils.intersectAll("abd", new GeneSet[]{a, b, d});
            this.bcd = AlgUtils.intersectAll("bcd", new GeneSet[]{b, c, d});

            this.abcNotd = AlgUtils.subtract("abcNotd", this.abc, this.dFull);
            this.acdNotb = AlgUtils.subtract("acdNotb", this.acd, this.bFull);
            this.abdNotc = AlgUtils.subtract("abdNotc", this.abd, this.cFull);
            this.bcdNota = AlgUtils.subtract("bcdNota", this.bcd, this.aFull);

            // all
            this.abcd = AlgUtils.intersectAll("abcd", new GeneSet[]{a, b, c, d});

            // ones by themselves
            this.aUsed = AlgUtils.unionAll("aUsed", new GeneSet[]{this.ab, this.ac, this.ad});
            this.bUsed = AlgUtils.unionAll("bUsed", new GeneSet[]{this.ab, this.bc, this.bd});
            this.cUsed = AlgUtils.unionAll("cUsed", new GeneSet[]{this.ac, this.bc, this.cd});
            this.dUsed = AlgUtils.unionAll("dUsed", new GeneSet[]{this.ad, this.bd, this.cd});

            this.name = a.getName() + "_" + b.getName() + "_" + c.getName() + "_" + c.getName() + "_fourway";

        }

        public String getAsString() {

            StringBuffer buf = new StringBuffer();
            add(aFull, buf);
            add(bFull, buf);
            add(cFull, buf);
            add(dFull, buf);

            add(ab, buf);
            add(ac, buf);
            add(ad, buf);
            add(bc, buf);
            add(bd, buf);
            add(cd, buf);
            add(abc, buf);
            add(acd, buf);
            add(abd, buf);
            add(bcd, buf);

            add(abcNotd, buf);
            add(acdNotb, buf);
            add(abdNotc, buf);
            add(bcdNota, buf);

            // all
            add(abcd, buf);

            // ones by themselves
            add(aUsed, buf);
            add(bUsed, buf);
            add(cUsed, buf);
            add(dUsed, buf);

            return buf.toString();
        }

        public int[] getInts() {
            return new int[]{abc.getNumMembers(), acd.getNumMembers(), bcd.getNumMembers(), abd.getNumMembers()};
        }

        public GeneSetMatrix getGeneSetMatrix() {
            return new DefaultGeneSetMatrix(name, new GeneSet[]{aFull, bFull, cFull, dFull, ab, ac, ad, bc, bd, cd, abc, acd, abd, bcd, abcNotd, acdNotb,
                    abdNotc, bcdNota, abcd, aUsed, bUsed, cUsed, dUsed});
        }
    }

    /**
     *
     */
    public static class FiveWay implements XWay {

        // the fsets being compared
        private GeneSet aFull;
        private GeneSet bFull;
        private GeneSet cFull;
        private GeneSet dFull;
        private GeneSet eFull;

        // pairwise
        private GeneSet ab;
        private GeneSet ac;
        private GeneSet ad;
        private GeneSet ae;

        private GeneSet bc;
        private GeneSet bd;
        private GeneSet be;

        private GeneSet cd;
        private GeneSet ce;

        private GeneSet de;

        // all
        private GeneSet abcde;

        // ones by themselves
        private GeneSet aUsed;
        private GeneSet bUsed;
        private GeneSet cUsed;
        private GeneSet dUsed;
        private GeneSet eUsed;

        private String name;

        public FiveWay(final GeneSet a, final GeneSet b, final GeneSet c, final GeneSet d, final GeneSet e) {

            final String aName = AuxUtils.getAuxNameOnlyNoHash(a);
            final String bName = AuxUtils.getAuxNameOnlyNoHash(b);
            final String cName = AuxUtils.getAuxNameOnlyNoHash(c);
            final String dName = AuxUtils.getAuxNameOnlyNoHash(d);
            final String eName = AuxUtils.getAuxNameOnlyNoHash(e);

            this.aFull = a.cloneDeep(aName + "_FULL", a.getNameEnglish());
            this.bFull = b.cloneDeep(bName + "_FULL", b.getNameEnglish());
            this.cFull = c.cloneDeep(cName + "_FULL", c.getNameEnglish());
            this.dFull = d.cloneDeep(dName + "_FULL", d.getNameEnglish());
            this.dFull = e.cloneDeep(eName + "_FULL", e.getNameEnglish());

            // all pairwises
            this.ab = AlgUtils.intersect(aName + "_INTERSECT_" + bName, a, b);
            this.ac = AlgUtils.intersect(aName + "_INTERSECT_" + cName, a, c);
            this.ad = AlgUtils.intersect(aName + "_INTERSECT_" + dName, a, d);
            this.ae = AlgUtils.intersect(aName + "_INTERSECT_" + eName, a, e);

            this.bc = AlgUtils.intersect(bName + "_INTERSECT_" + cName, b, c);
            this.bd = AlgUtils.intersect(bName + "_INTERSECT_" + dName, b, d);
            this.be = AlgUtils.intersect(bName + "_INTERSECT_" + dName, b, e);

            this.cd = AlgUtils.intersect(cName + "_INTERSECT_" + dName, c, d);
            this.ce = AlgUtils.intersect(cName + "_INTERSECT_" + eName, c, e);

            this.ce = AlgUtils.intersect(dName + "_INTERSECT_" + eName, d, e);

            // all
            this.abcde = AlgUtils.intersectAll("abcd", new GeneSet[]{a, b, c, d, e});

            // ones by themselves
            this.aUsed = AlgUtils.unionAll("aUsed", new GeneSet[]{this.ab, this.ac, this.ad, this.ae});
            this.bUsed = AlgUtils.unionAll("bUsed", new GeneSet[]{this.ab, this.bc, this.bd, this.be});
            this.cUsed = AlgUtils.unionAll("cUsed", new GeneSet[]{this.ac, this.bc, this.cd, this.ce});
            this.dUsed = AlgUtils.unionAll("dUsed", new GeneSet[]{this.ad, this.bd, this.cd, this.de});
            this.eUsed = AlgUtils.unionAll("eUsed", new GeneSet[]{this.ae, this.be, this.ce, this.de});

            this.name = a.getName() + "_" + b.getName() + "_" + c.getName() + "_" + c.getName() + "_fourway";

        }

        public String getAsString() {

            StringBuffer buf = new StringBuffer();
            add(aFull, buf);
            add(bFull, buf);
            add(cFull, buf);
            add(dFull, buf);
            add(eFull, buf);

            add(ab, buf);
            add(ac, buf);
            add(ad, buf);
            add(ae, buf);
            add(bc, buf);
            add(bd, buf);
            add(be, buf);
            add(cd, buf);
            add(ce, buf);

            // all
            add(abcde, buf);

            // ones by themselves
            add(aUsed, buf);
            add(bUsed, buf);
            add(cUsed, buf);
            add(dUsed, buf);
            add(eUsed, buf);

            return buf.toString();
        }

        public int[] getInts() {
            //eturn new int[]{abc.getNumMembers(), acd.getNumMembers(), bcd.getNumMembers(), abd.getNumMembers()};
            throw new NotImplementedException(); // @todo
        }

        public GeneSetMatrix getGeneSetMatrix() {
            return new DefaultGeneSetMatrix(name, new GeneSet[]{aFull, bFull, cFull, dFull, eFull,
                    ab, ac, ad, ae,
                    bc, bd, be,
                    cd, ce,
                    abcde,
                    aUsed, bUsed, cUsed, dUsed, eUsed
            });
        }
    }

} // End InfoCrystalAlg

