/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.alg;


import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import static edu.mit.broad.genome.alg.Metric.CATEGORICAL;
import edu.mit.broad.genome.math.AddressedVector;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.Template;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * Contains several implementation of Metrics
 * <p/>
 * inner classes must be declared static
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @note from ma: don't forget that these distance funcs return greater distances
 * for longer vectors.  Need to have normalized equivalents that take
 * the number of components into account when comparing statistics
 * across i.e. different size feature vectors.
 */
public class Metrics {

    /**
     * For logging support
     */
    protected static final transient Logger klog = XLogger.getLogger(Metrics.class);

    private static final int MIN_NUM_FOR_VAR = 3;

    // @maint add a metric and this array might need updating
    public static Metric[] createAllMetrics() {
        return new Metric[]{
                new Pearson(),
                new Cosine(),
                new Euclidean(),
                new Manhatten(),
                new FeatureVar(),
                new FeatureVarD(),
                new tTest(),
                new Signal2Noise(),
                new None(),
                new RegressionSlope(),
                new Bhattacharyya(),
                new PearsonD(),
                new ClassMeansDiff(), new ClassMediansDiff(),
                new ClassOfInterestMean(), new ClassOfInterestMedian(),
                new ClassMeansRatio(),
                new ClassMediansRatio(),
                new ClassMeansLog2Ratio(),
                new ClassMediansLog2Ratio(),
                new ClassRatio(),
                new ClassDiff(),
                new ClassLog2Ratio(),
                new ClassLog2RatioOneSide(), 
                new Log2Ratio(),                                
                new FeatureVar(),
                new FeatureVarD(),
                new ChiSquare()
        };
    }

    public static Metric[] createAllContinuousMetrics() {

        final Metric[] metrics = createAllMetrics();
        final List list = new ArrayList();

        for (int i = 0; i < metrics.length; i++) {
            if (metrics[i].isContinuous()) {
                list.add(metrics[i]);
            }
        }

        return (Metric[]) list.toArray(new Metric[list.size()]);
    }

    public static Metric[] createAllCategoricalMetrics() {
        final Metric[] metrics = createAllMetrics();
        final List list = new ArrayList();

        for (int i = 0; i < metrics.length; i++) {
            if (metrics[i].isCategorical()) {
                list.add(metrics[i]);
            }
        }

        return (Metric[]) list.toArray(new Metric[list.size()]);

        /*
        return new Metric[]{
                new Cosine(), new Euclidean(), new Manhatten(), new Pearson()
        };
        */
    }

    public static CorrMetric[] createAllCorrMetrics() {
        return new CorrMetric[]{
                new Pearson(), new Cosine()
        };
    }

    // @maint add a metric and this array might need updating
    public static RankMetric[] createAllRankMetrics() {
        return new RankMetric[]{
                new Spearman(), new KendallsTau()
        };
    }

    public static Metric[] createClassMeansLog2RatioMetrics() {
        return new Metric[]{
                new ClassMeansLog2Ratio(),
        };
    }

    public static Metric[] createClassMeansDiffMetrics() {
        return new Metric[]{
                new ClassMeansDiff(),
        };
    }
    
    public static Metric[] createSignal2NoiseSeqMetrics() {
        return new Metric[]{
                new Signal2NoiseSeq(),
        };
    }    

    public static RankMetric lookupRankMetric(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup RankMetric for null object");
        }

        if (obj instanceof RankMetric) {
            return (RankMetric) obj;
        }

        RankMetric[] all = createAllRankMetrics();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookup RankMetrics by: " + obj);
    }

    //---------------------------------------------------------------------------//
    //---------------------------------------------------------------------------//
    //---------------------------------------------------------------------------//

    /**
     * Private class constructor to prevent construction outside.
     */
    private Metrics() {
    }

    public static Metric lookupMetric(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof Metric) {
            return (Metric) obj;
        }

        Metric[] all = createAllMetrics();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookupMetric for: " + obj);
    }

    public static CorrMetric lookupCorrMetric(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof CorrMetric) {
            return (CorrMetric) obj;
        }

        CorrMetric[] all = createAllCorrMetrics();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookupCorrMetric for: " + obj);
    }


    /**
     * Class constructor
     */
    public static abstract class AbstractMetric implements Metric {

        protected VectorSplitter fSplitter;

        private Type fType;

        public AbstractMetric(final Type type) {
            this.fType = type;
            this.fSplitter = new VectorSplitter(1); // @note default
        }

        public boolean isCategorical() {
            return (fType.equals(CATEGORICAL) || fType.equals(CAT_AND_CONT));
        }

        public boolean isContinuous() {
            return (fType.equals(CONTINUOUS) || fType.equals(CAT_AND_CONT));
        }

        public Type getType() {
            return fType;
        }

        public int hashCode() {
            return getName().hashCode();
        }

        public boolean equals(Object obj) {

            if (obj instanceof Metric) {
                return getName().equalsIgnoreCase(((Metric) obj).getName());
            }

            return false;
        }

        public String getMissingInfo() {
            return fSplitter.getMissingInfo();
        }

        public String toString() {
            return getName();
        }

        public int getMinNumSamplesNeededPerClassForCalculation() {
            return 1;
        }
    }

    /**
     * Euclidean.
     */
    public static class Euclidean extends AbstractMetric {

        public static final String NAME = "Euclidean";

        /**
         * Class Constructor.
         */
        public Euclidean() {
            super(CONTINUOUS);
        }

        /**
         * No parameters.
         * Template is required.
         * Remember, this is only sensible when the template is continuous.
         * Template needs to be equal in length to profile.
         * Always determining distance of profile to template.
         */
        public double getScore(Vector profile, Template template, Map params) {
            return XMath.euclidean(template.synchProfile(profile), template.toVector());
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.euclidean(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Euclidean();
        }

    }


    /**
     * Manhatten
     */
    public static class Manhatten extends AbstractMetric {

        public static final String NAME = "Manhatten";

        /**
         * Class Constructor.
         */
        public Manhatten() {
            super(CONTINUOUS);
        }

        /**
         * No parameters.
         * Template is required.
         * Remember, this is only sensible when the template is continuous.
         * Template needs to be equal in length to profile.
         * Always determining distance of profile to template.
         */
        public double getScore(Vector profile, Template template, Map params) {
            return XMath.manhatten(template.synchProfile(profile), template.toVector());
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.manhatten(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Manhatten();
        }

    }

    /**
     * Pearson.
     */
    public static class Pearson extends AbstractMetric implements CorrMetric {

        public static final String NAME = "Pearson";

        /**
         * Class Constructor.
         */
        public Pearson() {
            super(CONTINUOUS); // to make it consistent with the others
        }

        /**
         * No parameters
         * Template is required.
         * Remember, this is only sensible when the template is continuous.
         * actualy not necc to to use with categorical vector too
         * Template needs to be equal in length to profile.
         * Always determining distance of profile to template.
         */
        public double getScore(Vector profile, Template template, Map params) {
            final Vector v = template.synchProfile(profile);
            return XMath.pearson(v, template.toVector());
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.pearson(x, y);
        }

        public double getCorr(Vector x, Vector y) {
            return XMath.pearson(x, y);
        }

        public double getCorr(float[] x, float[] y) {
            return XMath.pearson(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Pearson();
        }

    }

    public static class Hamming_only_1s extends AbstractMetric implements CorrMetric {

        public static final String NAME = "Hamming_only_1s";

        /**
         * Class Constructor.
         */
        public Hamming_only_1s() {
            super(CAT_AND_CONT);
        }

        /**
         * No parameters
         * Template is required.
         * Remember, this is only sensible when the template is continuous.
         * actualy not necc to to use with categorical vector too
         * Template needs to be equal in length to profile.
         * Always determining distance of profile to template.
         */
        public double getScore(Vector profile, Template template, Map params) {
            throw new NotImplementedException();
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.hamming_exclude_0_only_if_both(x, y);
        }

        public double getCorr(Vector x, Vector y) {
            return XMath.hamming_exclude_0_only_if_both(x, y);
        }

        public double getCorr(float[] x, float[] y) {
            throw new NotImplementedException();
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Hamming_only_1s();
        }

    }


    /**
     * Cosine dist
     */
    public static class Cosine extends AbstractMetric implements CorrMetric {

        public static final String NAME = "Cosine";

        public Cosine() {
            super(CONTINUOUS);
        }

        /**
         * No parameters
         * Template is required
         * Template needs to be equal in length to profile.
         * Always determining distance of profile to template.
         *
         * @IMP IMP is low good or bad? i think its different in that higher is better
         */
        public double getScore(Vector profile, Template template, Map params) {
            return XMath.cosine(template.synchProfile(profile), template.toVector());
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.cosine(x, y);
        }

        public double getCorr(float[] x, float[] y) {
            return XMath.cosine(x, y);
        }

        public double getCorr(Vector x, Vector y) {
            return XMath.cosine(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Cosine();
        }

    }    // End Cosine


    /**
     * FeatureVar
     */
    public static class FeatureVar extends AbstractMetric {

        public static final String NAME = "FeatureVariation";

        public FeatureVar() {
            super(CONTINUOUS);
        }

        /**
         * params
         * USE_BIASED -> true or false (Boolean objects). Default is false.
         * FIX_LOW    -> true or false (Boolean objects). Default is true
         * Template is NOT required.
         * Not a *distance* metric -> just measures variation within the profile.
         */
        public double getScore(Vector profile, Template template, Map params) {

            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);

            return profile.var(usebiased, fixlow);
        }

        public double getScore(Vector x, Vector y, Map params) {
            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);
            return x.var(usebiased, fixlow);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new FeatureVar();
        }

        public int getMinNumSamplesNeededPerClassForCalculation() {
            return MIN_NUM_FOR_VAR;
        }

    }

    /**
     * Dchip style var
     */
    public static class FeatureVarD extends AbstractMetric {

        public static final String NAME = "Feature_stddev_by_mean";

        public FeatureVarD() {
            super(CONTINUOUS);
        }

        /**
         * params
         * USE_BIASED -> true or false (Boolean objects). Default is false.
         * FIX_LOW    -> true or false (Boolean objects). Default is true
         * Template is NOT required.
         * Not a *distance* metric -> just measures variation within the profile.
         */
        public double getScore(Vector profile, Template template, Map params) {

            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);

            return profile.vard(usebiased, fixlow);
        }

        public double getScore(Vector x, Vector y, Map params) {
            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);
            return x.vard(usebiased, fixlow);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new FeatureVarD();
        }

        public int getMinNumSamplesNeededPerClassForCalculation() {
            return MIN_NUM_FOR_VAR;
        }

    }

    /**
     * Signal2Noise
     */
    public static class Signal2Noise extends AbstractMetric {

        public static final String NAME = "Signal2Noise";

        public Signal2Noise() {
            super(CATEGORICAL);
        }

        /**
         * params:
         * USE_BIASED -> true or false (Boolean objects). Default is FALSE.
         * USE_MEDIAN -> true or false (Boolean objects). Default is TRUE.
         * FIX_LOW    -> true or false (Boolean objects). Default is TRUE
         * Template is required.
         */
        public double getScore(Vector profile, Template template, Map params) {

            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);
            boolean usemedian = AlgMap.isMedian(params);

            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);

            if (vs == null) {
                return 0;
            } else {

                int coiIndex = template.getClassOfInterestIndex();

                if (coiIndex == 0) {
                    return XMath.s2n(vs[0], vs[1], usebiased, usemedian, fixlow);
                } else {
                    return XMath.s2n(vs[1], vs[0], usebiased, usemedian, fixlow);
                }
            }

        }

        public double getScore(Vector x, Vector y, Map params) {
            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);
            boolean usemedian = AlgMap.isMedian(params);
            return XMath.s2n(x, y, usebiased, usemedian, fixlow);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Signal2Noise();
        }

        public int getMinNumSamplesNeededPerClassForCalculation() {
            return MIN_NUM_FOR_VAR;
        }

    }
    
    public static class Signal2NoiseSeq extends AbstractMetric {

        public static final String NAME = "Signal2NoiseSeq";

        public Signal2NoiseSeq() {
            super(CATEGORICAL);
        }

        /**
         * params:
         * USE_BIASED -> true or false (Boolean objects). Default is FALSE.
         * USE_MEDIAN -> true or false (Boolean objects). Default is TRUE.
         * FIX_LOW    -> true or false (Boolean objects). Default is TRUE
         * Template is required.
         */
        public double getScore(Vector profile, Template template, Map params) {

            boolean usebiased = false;
            boolean fixlow = true;
            boolean usemedian = false;

            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);

            if (vs == null) {
                return 0;
            } else {

                int coiIndex = template.getClassOfInterestIndex();

                if (coiIndex == 0) {
                    return XMath.s2n(vs[0], vs[1], usebiased, usemedian, fixlow);
                } else {
                    return XMath.s2n(vs[1], vs[0], usebiased, usemedian, fixlow);
                }
            }

        }

        public double getScore(Vector x, Vector y, Map params) {
            boolean usebiased = false;
            boolean fixlow = true;
            boolean usemedian = false;
            return XMath.s2n(x, y, usebiased, usemedian, fixlow);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Signal2Noise();
        }

        public int getMinNumSamplesNeededPerClassForCalculation() {
            return MIN_NUM_FOR_VAR;
        }

    }    


    public static class None extends AbstractMetric {

        public static final String NAME = "None";

        public None() {
            super(CAT_AND_CONT);
        }

        /**
         * params:
         * USE_BIASED -> true or false (Boolean objects). Default is FALSE.
         * USE_MEDIAN -> true or false (Boolean objects). Default is TRUE.
         * FIX_LOW    -> true or false (Boolean objects). Default is TRUE
         * Template is required.
         */
        public double getScore(Vector profile, Template template, Map params) {
            return profile.getElement(0);
        }

        public double getScore(Vector x, Vector y, Map params) {
            throw new NotImplementedException();
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Signal2Noise();
        }

    }

    /**
     * t-Test
     */
    public static class tTest extends AbstractMetric {

        public static final String NAME = "tTest";

        public tTest() {
            super(CATEGORICAL);
        }

        /**
         * USE_MEDIAN -> true or false (Boolean objects). Default is TRUE.
         * USE_BIASED -> true or false (Boolean objects). Default is FALSE.
         * FIX_LOW    -> true or false (Boolean objects). Default is TRUE
         * Template is required.
         */
        public double getScore(Vector profile, Template template, Map params) {

            boolean usemedian = AlgMap.isMedian(params);
            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);
            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.tTest(vs[0], vs[1], usebiased, usemedian, fixlow);
            } else {
                return XMath.tTest(vs[1], vs[0], usebiased, usemedian, fixlow);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);
            boolean usemedian = AlgMap.isMedian(params);
            return XMath.tTest(x, y, usebiased, usemedian, fixlow);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new tTest();
        }

        public int getMinNumSamplesNeededPerClassForCalculation() {
            return MIN_NUM_FOR_VAR;
        }

    }

    /**
     * Bhattacharyya
     */
    public static class Bhattacharyya extends AbstractMetric {

        public static final String NAME = "Bhattacharyya";

        public Bhattacharyya() {
            super(CATEGORICAL);
        }

        /**
         * USE_BIASED -> true or false (Boolean objects). Default is FALSE.
         * FIX_LOW    -> true or false (Boolean objects). Default is TRUE
         * Template is required.
         */
        public double getScore(Vector profile, Template template, Map params) {

            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);
            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.bhat(vs[0], vs[1], usebiased, fixlow);
            } else {
                return XMath.bhat(vs[1], vs[0], usebiased, fixlow);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);
            return XMath.bhat(x, y, usebiased, fixlow);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Bhattacharyya();
        }

    }    // End Bhattacharyya


    /**
     * @todo RegressionSlope
     */
    public static class RegressionSlope extends AbstractMetric {

        public static final String NAME = "RegressionSlope";

        public RegressionSlope() {
            super(CONTINUOUS);
        }

        /**
         * Template is required.
         */
        public double getScore(Vector profile, Template template, Map params) {
            boolean usebiased = AlgMap.isBiased(params);
            boolean fixlow = AlgMap.isFixLowVar(params);

            Vector[] splits = template.splitByTemplateClass(profile);

            return XMath.regressionSlope(profile, template.toVector(), splits, usebiased, fixlow);
        }

        public double getScore(Vector x, Vector y, Map params) {
            throw new NotImplementedException();
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new RegressionSlope();
        }

    }

    /**
     * Pearson.
     */
    public static class PearsonD extends Pearson {

        public static final String NAME = "Norm.Pearson";

        /**
         * None
         * <p/>
         * Template is required.
         *
         * @todo review with pt
         * Why does it make sense to use Pearson in this fashion - i.e normalized and as a discrete metric?
         * Why does it not make sense to use other cont metrics this way?
         * Pearson gives values from -1 to +1 which is the same range that pnorm on a vector does.
         * <p/>
         * So, needs 2 vectors both in the -1 to +1 range.
         * The ref vector (from template) needs to be biphasic and values at -1 or +1
         * The comp vector needs to be pnormalized.
         * Actually none of those tests are done here.
         * Here its a straigtforward pc calc - ditto to Pearson continuous.
         * <p/>
         * <p/>
         * usebiased and usemedian are meaningless??
         */
        public double getScore(Vector profile, Template template, Map params) {


            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.pearsonD(vs[0], vs[1]);
            } else {
                return XMath.pearsonD(vs[1], vs[0]);
            }

        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.pearsonD(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new PearsonD();
        }

    }

    /**
     * Diff of means.
     */
    public static class ClassMeansDiff extends AbstractMetric {

        public static final String NAME = "Diff_of_Means";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassMeansDiff() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.meansdiff(vs[0], vs[1]);
            } else {
                return XMath.meansdiff(vs[1], vs[0]);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.meansdiff(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassMeansDiff();
        }

    }

    public static class ClassMeansRatio extends AbstractMetric {

        public static final String NAME = "Ratio_of_Means";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassMeansRatio() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.meansratio(vs[0], vs[1]);
            } else {
                return XMath.meansratio(vs[1], vs[0]);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.meansratio(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassMeansRatio();
        }

    }

    public static class ClassRatio extends AbstractMetric {

        public static final String NAME = "Ratio_of_Classes";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassRatio() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            final boolean useMean = AlgMap.isMean(params);

            if (coiIndex == 0) {
                return XMath.meanOrMedianRatio(vs[0], vs[1], useMean);
            } else {
                return XMath.meanOrMedianRatio(vs[1], vs[0], useMean);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.meanOrMedianRatio(x, y, AlgMap.isMean(params));
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassRatio();
        }

    } // End class ClassRatio


    public static class ClassLog2Ratio extends AbstractMetric {

        public static final String NAME = "log2_Ratio_of_Classes";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassLog2Ratio() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            final boolean useMean = AlgMap.isMean(params);

            if (coiIndex == 0) {
                return XMath.log2(XMath.meanOrMedianRatio(vs[0], vs[1], useMean));
            } else {
                return XMath.log2(XMath.meanOrMedianRatio(vs[1], vs[0], useMean));
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.log2(XMath.meanOrMedianRatio(x, y, AlgMap.isMean(params)));
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassLog2Ratio();
        }

    } // End class ClassLog2Ratio
    
    public static class ClassLog2RatioOneSide extends AbstractMetric {

        public static final String NAME = "log2_Ratio_of_Classes_1";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassLog2RatioOneSide() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            final boolean useMean = AlgMap.isMean(params);

            if (coiIndex == 0) {
                double log2ratio = XMath.log2(XMath.meanOrMedianRatio(vs[0], vs[1], useMean));
                if(log2ratio<0){
                    log2ratio=0;
                }
                return log2ratio;
            } else {
                double log2ratio = XMath.log2(XMath.meanOrMedianRatio(vs[1], vs[0], useMean));
                if(log2ratio<0){
                    log2ratio=0;
                }
                return log2ratio;
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.log2(XMath.meanOrMedianRatio(x, y, AlgMap.isMean(params)));
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassLog2Ratio();
        }

    } // End class ClassLog2RatioOneSide    
    
    public static class Log2Ratio extends AbstractMetric {

        public static final String NAME = "log2Ratio";

        /**
         * Class Constructor.
         * Template is required.
         */
        public Log2Ratio() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            final boolean useMean = AlgMap.isMean(params);

            if (coiIndex == 0) {
                return XMath.log2(XMath.meanOrMedianRatio(vs[0], vs[1], useMean));
            } else {
                return XMath.log2(XMath.meanOrMedianRatio(vs[1], vs[0], useMean));
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.log2(XMath.meanOrMedianRatio(x, y, AlgMap.isMean(params)));
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassLog2Ratio();
        }

    } // End class Log2Ratio    

    public static class ClassDiff extends AbstractMetric {

        public static final String NAME = "Diff_of_Classes";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassDiff() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            final boolean useMean = AlgMap.isMean(params);

            if (coiIndex == 0) {
                return XMath.meanOrMedianDiff(vs[0], vs[1], useMean);
            } else {
                return XMath.meanOrMedianDiff(vs[1], vs[0], useMean);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.meanOrMedianDiff(x, y, AlgMap.isMean(params));
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassDiff();
        }

    }


    public static class ClassMediansRatio extends AbstractMetric {

        public static final String NAME = "Ratio_of_Medians";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassMediansRatio() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.mediansratio(vs[0], vs[1]);
            } else {
                return XMath.mediansratio(vs[1], vs[0]);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.mediansratio(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassMediansRatio();
        }

    }

    public static class ClassMeansLog2Ratio extends AbstractMetric {

        public static final String NAME = "log2_ratio_of_means";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassMeansLog2Ratio() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.log2(XMath.meansratio(vs[0], vs[1]));
            } else {
                return XMath.log2(XMath.meansratio(vs[1], vs[0]));
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.log2(XMath.meansratio(x, y));
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassMeansLog2Ratio();
        }


    }

    public static class ClassMediansLog2Ratio extends AbstractMetric {


        public static final String NAME = "log2_ratio_of_medians";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassMediansLog2Ratio() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd.
         * Diff in means between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.log2(XMath.mediansratio(vs[0], vs[1]));
            } else {
                return XMath.log2(XMath.mediansratio(vs[1], vs[0]));
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.log2(XMath.mediansratio(x, y));
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassMediansLog2Ratio();
        }

    }

    /**
     * mean of only the class of interest
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class ClassOfInterestMean extends AbstractMetric {

        public static final String NAME = "ClassOfInterestMean";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassOfInterestMean() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            return vs[coiIndex].mean();
        }

        public double getScore(Vector x, Vector y, Map params) {
            return x.mean();
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassOfInterestMean();
        }

    }

    /**
     * median of only the class of interest
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class ClassOfInterestMedian extends AbstractMetric {

        public static final String NAME = "ClassOfInterestMedian";

        /**
         * Class Constructor.
         * Template is required.
         */
        public ClassOfInterestMedian() {
            super(CATEGORICAL);
        }

        /**
         * No parameters.
         * Template is reqd
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            return vs[coiIndex].median();
        }

        public double getScore(Vector x, Vector y, Map params) {
            return x.median();
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassOfInterestMedian();
        }

    }

    /**
     * Diff of medians
     */
    public static class ClassMediansDiff extends AbstractMetric {

        public static final String NAME = "Diff_of_Medians";

        /**
         * Class Constructor.
         */
        public ClassMediansDiff() {
            super(CATEGORICAL);
        }

        /**
         * No params.
         * <p/>
         * Template is reqd.
         * Diff in medians between classes as split by template
         * (NOT b/w profile and template)
         */
        public double getScore(Vector profile, Template template, Map params) {

            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.mediansdiff(vs[0], vs[1]);
            } else {
                return XMath.mediansdiff(vs[1], vs[0]);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.mediansdiff(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ClassMediansDiff();
        }

    }

    static abstract class AbstractPointMetric implements PointMetric {

        public int hashCode() {
            return getName().hashCode();
        }

        public boolean equals(Object obj) {

            if (obj instanceof Metric) {
                return getName().equalsIgnoreCase(((Metric) obj).getName());
            }

            return false;
        }

    }

    /**
     * Diff of medians
     */
    public static class EuclideanAbsDistance extends AbstractPointMetric {

        public static final String NAME = "Euclidean";

        /**
         * Class Constructor.
         */
        public EuclideanAbsDistance() {
        }

        /**
         * No params.
         * <p/>
         * Template is reqd.
         * Diff in medians between classes as split by template
         * (NOT b/w profile and template)
         */
        public float getScore(float a, float b) {
            return Math.abs(a - b);
        }

        public String getName() {
            return NAME;
        }
    }

    //---------------------------------------------------------------------------//
    //------------------------------ RANK ORDER METRICS -------------------------//
    //---------------------------------------------------------------------------//

    /**
     * http://mathworld.wolfram.com/SpearmanRankCorrelationCoefficient.html
     * <p/>
     * rs = 1 - 6*sum[d**2/(n(n2-1)]
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class Spearman extends AbstractMetric implements RankMetric {

        public static final String NAME = "Spearman";

        public Spearman() {
            super(CONTINUOUS);
        }

        /**
         * USE_MEDIAN -> true or false (Boolean objects). Default is TRUE.
         * USE_BIASED -> true or false (Boolean objects). Default is FALSE.
         * FIX_LOW    -> true or false (Boolean objects). Default is TRUE
         * Template is required.
         */
        public double getScore(Vector profile, Template template, Map params) {
            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            AddressedVector avx = new AddressedVector(vs[0], true);
            AddressedVector avy = new AddressedVector(vs[1], true);
            return getScore(avx, avy);
        }

        public double getScore(Vector x, Vector y, Map params) {
            AddressedVector avx = new AddressedVector(x, true);
            AddressedVector avy = new AddressedVector(y, true);
            return getScore(avx, avy);
        }

        public double getScore(AddressedVector x, AddressedVector y) {

            throw new NotImplementedException();

            /*
            int[] ranksx = RankedListAlgs.sortByVectorAndGetRanks(x, SortMode.REAL, Order.DESCENDING);
            int[] ranksy = RankedListAlgs.sortByVectorAndGetRanks(y, SortMode.REAL, Order.DESCENDING);
            */
            /*
            int[] addrs = x.getAddresses();

            for (int i=0; i < addrs.length; i++) {
                ranksx[i] = x.g
            }
            */

            // only care about relative positions, i.e ranks of corresponding objects
            // imp to understand that we are comparing rank of foox with rank of fooy
            //Print.outl(ranksx);
            //Print.outl(ranksy);

            //return XMath.spearman(ranksx, ranksy);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new Spearman();
        }

    } // End Spearman

    /**
     * @author Aravind Subramanian
     * @version %I%, %G%
     * @see p885 david sheskin parametric and nonparamteric handbook
     */

    public static class KendallsTau implements RankMetric {

        public static final String NAME = "KendallsTau";

        public String getName() {
            return NAME;
        }

        /**
         * @param x
         * @param y
         * @return
         * @todo
         */
        public double getScore(AddressedVector x, AddressedVector y) {

            throw new NotImplementedException();

            /*
            for (int i=0; i < x.getSize(); i++) {

                int xindex = x.getAddress(i);

                int yrank = y.get

            }

            int[] ranksx = x.sortByVectorAndGetAddresses(SortMode.REAL, Order.DESCENDING);
            int[] ranksy = y.sortByVectorAndGetAddresses(SortMode.REAL, Order.DESCENDING);
            */
            // return 0.0;
        }

        public String toString() {
            return NAME;
        }

    } // End KendallsTau

    public static class ChiSquare extends AbstractMetric {


        public static final String NAME = "ChiSquare";

        public ChiSquare() {
            super(CATEGORICAL);
        }

        public double getScore(Vector genotype, Template template, Map params) {
            Vector[] vs = fSplitter.splitBiphasic_nansafe(genotype, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.chiSquare(vs[0], vs[1]);
            } else {
                return XMath.chiSquare(vs[1], vs[0]);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.chiSquare(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ChiSquare();
        }

    }


    /*public static class ChiSquare extends AbstractMetric {


        public static final String NAME = "ChiSquare";

        public ChiSquare() {
            super(CATEGORICAL);
        }

        public double getScore(Vector profile, Template template, Map params) {
            Vector[] vs = fSplitter.splitBiphasic_nansafe(profile, template);
            int coiIndex = template.getClassOfInterestIndex();

            if (coiIndex == 0) {
                return XMath.chiSquare(vs[0], vs[1]);
            } else {
                return XMath.chiSquare(vs[1], vs[0]);
            }
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.chiSquare(x, y);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new ChiSquare();
        }

    }*/


}    // End Metrics

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

/*
    public static class KenFeatureVar125 extends AbstractMetric implements CorrMetric {

        public static final String NAME = "KenFeatureVar125";
        private final float FOLD = 1.25f;

        public double getScore(Vector profile, Template template, Map params) {
            throw new NotImplementedException();
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.kenvar(x, y, FOLD);
        }

        public double getCorr(Vector x, Vector y) {
            return XMath.kenvar(x, y, FOLD);
        }

        public double getCorr(float[] x, float[] y) {
            return XMath.kenvar(x, y, FOLD);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new KenFeatureVar125();
        }

    }

    public static class KenFeatureVar2 extends AbstractMetric implements CorrMetric {

        public static final String NAME = "KenFeatureVar";
        private final float FOLD = 2.0f;


        public double getScore(Vector profile, Template template, Map params) {
            throw new NotImplementedException();
        }

        public double getScore(Vector x, Vector y, Map params) {
            return XMath.kenvar(x, y, FOLD);
        }

        public double getCorr(Vector x, Vector y) {
            return XMath.kenvar(x, y, FOLD);
        }

        public double getCorr(float[] x, float[] y) {
            return XMath.kenvar(x, y, FOLD);
        }

        public String getName() {
            return NAME;
        }

        public Metric cloneMetric() {
            return new KenFeatureVar2();
        }

    }
*/
