/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import org.apache.log4j.Logger;

import java.util.HashMap;

/**
 * @author Aravind Subramanian
 *         Date: Nov 1, 2003
 *         Time: 5:08:20 PM
 */
public class NeighMetrics {

    private final Logger log = XLogger.getLogger(this.getClass());

    /**
     * Neighborhoods of every probe in the specified geneset (pearson dist
     * to all probes in the dataset)
     *
     * @param qnds
     * @param seeds
     * @return
     */
    public Vector[] calcNeighScores(final QuickNeighDataset qnds, final GeneSet seeds) {
        Vector[] scores = new Vector[seeds.getNumMembers()];

        for (int i = 0; i < seeds.getNumMembers(); i++) {
            String seedName = seeds.getMember(i);
            MetricStats seedStats = qnds.getMetricStats(seedName);
            scores[i] = new Vector(qnds.getNumRow());
            for (int r = 0; r < qnds.getNumRow(); r++) {
                scores[i].setElement(r, (float) seedStats.getScore(qnds.getMetricStats(r)));
            }
        }

        return scores;
    }

    public int[] calcNeighSizes(final QuickNeighDataset qnds, final GeneSet seeds, final float cutOff) {
        int[] moreCnts = new int[seeds.getNumMembers()];

        for (int i = 0; i < seeds.getNumMembers(); i++) {
            moreCnts[i] = calcNeighSize(qnds, seeds.getMember(i), cutOff);
        }

        return moreCnts;
    }

    public int calcNeighSize(final QuickNeighDataset qnds, final String seedName, final float cutOff) {

        int moreCnt = 0;

        MetricStats seedStats = qnds.getMetricStats(seedName);
        for (int r = 0; r < qnds.getNumRow(); r++) {
            float score = (float) seedStats.getScore(qnds.getMetricStats(r));
            if (score >= cutOff) {
                moreCnt++;
            }
        }

        return moreCnt;
    }

    // @todo change me to corr metric
    public int calcNeighSize(final Dataset ds, final String seedName, final Metric metric, final float cutOff) {

        int moreCnt = 0;

        final Vector seedProfile = ds.getRow(seedName);

        for (int r = 0; r < ds.getNumRow(); r++) {
            float score = (float) metric.getScore(seedProfile, ds.getRow(r), new HashMap());
            if (score >= cutOff) {
                moreCnt++;
            }
        }

        return moreCnt;
    }

    public int calcNeighSize(final QuickNeighDataset qnds, final Vector profile, final float cutOff) {

        int moreCnt = 0;

        MetricStats seedStats = qnds.createMetricStats(profile);
        for (int r = 0; r < qnds.getNumRow(); r++) {
            float score = (float) seedStats.getScore(qnds.getMetricStats(r));
            if (score >= cutOff) {
                moreCnt++;
            }
        }

        return moreCnt;
    }


    public Vector[] getRealNeighScores(final QuickNeighDataset qnd, final GeneSet seeds) {
        Vector[] scores = new Vector[seeds.getNumMembers()];

        for (int i = 0; i < seeds.getNumMembers(); i++) {
            String seedName = seeds.getMember(i);
            MetricStats seedStats = qnd.getMetricStats(seedName);
            scores[i] = new Vector(qnd.getNumRow());
            for (int r = 0; r < qnd.getNumRow(); r++) {
                scores[i].setElement(r, (float) seedStats.getScore(qnd.getMetricStats(r)));
            }
        }

        return scores;
    }

    public Vector getRealNeighScore(final QuickNeighDataset qnd, final String seedName) {
        Vector scores = new Vector(qnd.getNumRow());
        MetricStats seedStats = qnd.getMetricStats(seedName);
        for (int r = 0; r < qnd.getNumRow(); r++) {
            scores.setElement(r, (float) seedStats.getScore(qnd.getMetricStats(r)));
        }

        return scores;
    }

    public Neighbor[] getRealNeighs(final QuickNeighDataset qnd, final String seedName) {
        Neighbor[] neighs = new Neighbor[qnd.getNumRow()];
        MetricStats seedStats = qnd.getMetricStats(seedName);
        for (int r = 0; r < qnd.getNumRow(); r++) {
            neighs[r] = new Neighbor(qnd.getRowName(r), (float) seedStats.getScore(qnd.getMetricStats(r)));
        }

        return neighs;
    }

    public RankedFeature[] getRealNeighScores(final QuickNeighDataset qnd,
                                              final String seedName,
                                              final RankedFeature[] dfsIn) {
        RankedFeature[] dfs = new RankedFeature[dfsIn.length];

        Vector seedProfile = qnd.getRow(seedName);

        for (int d = 0; d < dfsIn.length; d++) {
            Vector v = qnd.getRow(dfsIn[d].getName());
            dfs[d] = new DefaultRankedFeature(dfsIn[d].getName(), d, (float) XMath.pearson(seedProfile, v));
        }

        return dfs;
    }

    public RankedFeature[] getRndNeighScores(final QuickNeighDataset qnd,
                                             final String seedName,
                                             final RankedFeature[] dfsIn,
                                             final RandomSeedGenerator rst) {

        RankedFeature[] dfs = new RankedFeature[dfsIn.length];
        Template contTemplate = TemplateFactory.createContinuousTemplate(seedName, qnd);
        Vector rndv = TemplateFactoryRandomizer.createRandomTemplateContinuousV(contTemplate, rst);

        for (int d = 0; d < dfsIn.length; d++) {
            Vector v = qnd.getRow(dfsIn[d].getName());
            dfs[d] = new DefaultRankedFeature(dfsIn[d].getName(), d, (float) XMath.pearson(rndv, v));
        }

        return dfs;
    }

    public RankedFeature[] getRealNeighScoresSorted(final QuickNeighDataset qnd,
                                                    final String seedName,
                                                    final int topX) {

        MetricStats seedStats = qnd.getMetricStats(seedName);

        DoubleElement[] dels = new DoubleElement[qnd.getNumRow()];
        for (int r = 0; r < qnd.getNumRow(); r++) {
            dels[r] = new DoubleElement(r, seedStats.getScore(qnd.getMetricStats(r)));
        }

        dels = DoubleElement.sort(SortMode.REAL, Order.DESCENDING, dels);
        AddressedVector av = new AddressedVector(dels);

        RankedFeature[] dfs = new RankedFeature[topX];
        for (int i = 0; i < topX; i++) {
            dfs[i] = new DefaultRankedFeature(qnd.getRowName(av.getAddress(i)), i, av.getScore(i));
        }

        return dfs;
    }


    /**
     * NO SORTING DONE! that is imp
     * the pearson stats for the real gene can be
     * NO real corrs
     * <p/>
     * each col is ONE RND
     * each row is dist of gene r in dataset to the seedName
     *
     * @param ds
     * @param tss
     */
    public Matrix calcRndScoresUNSO(final QuickNeighDataset nds,
                                    final String seedName,
                                    final int nperms,
                                    final RandomSeedGenerator rst) {

        Template realContTemplate = TemplateFactory.createContinuousTemplate(seedName, nds);

        // in matrix, each col is the corrs to one rnd template, and each row a gene
        Matrix matrix = new Matrix(nds.getNumRow(), nperms);

        for (int c = 0; c < nperms; c++) {
            Template rndt = TemplateFactoryRandomizer.createRandomTemplate(realContTemplate, rst);
            MetricStats mstats = nds.createMetricStats(rndt.toVector()); // new stats for the shuffled template
            for (int r = 0; r < nds.getNumRow(); r++) {
                matrix.setElement(r, c, (float) mstats.getScore(nds.getMetricStats(r))); // then distances to this shuffled one
            }

            if (c % 100 == 0) {
                System.out.println("_calcRndScoresUNSO: " + c + " / " + nperms + " for probe: " + seedName);
            }

        }

        return matrix;
    }

    private int[] _calcMoreCntsWithCutoff(final QuickNeighDataset qnd,
                                          final String seedName,
                                          final int nperms,
                                          final int cutoff,
                                          final float[] realScores,
                                          final RandomSeedGenerator rst) {


        final Template contTemplate = TemplateFactory.createContinuousTemplate(seedName, qnd);
        int[] moreCnts = new int[qnd.getNumRow()];

        for (int c = 0; c < nperms; c++) {
            Vector rndv = TemplateFactoryRandomizer.createRandomTemplateContinuousV(contTemplate, rst);
            MetricStats mstats = qnd.createMetricStats(rndv); // new stats for the shuffled template

            for (int r = 0; r < qnd.getNumRow(); r++) {
                if (moreCnts[r] > cutoff) {
                    // do nothing
                } else {
                    float rndScore = (float) mstats.getScore(qnd.getMetricStats(r)); // then distances to this shuffled one
                    if (rndScore > realScores[r]) {
                        moreCnts[r]++;
                    }
                }
            }
        }

        return moreCnts;
    }

} // End NeighMetrics
