/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.NamedColor;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Feb 16, 2004
 *         Time: 12:20:06 AM
 */
public class RankedListAlgs {

    private static final Logger klog = XLogger.getLogger(RankedListAlgs.class);

    public static class TmpStruc {
        public Dataset ds;
        public GeneSet[] gsets;
        public NamedColor[] colors;
    }

    public static TmpStruc combine2(final String name,
                                    final RankedListDb.Extracted[] rexs, // already extracted to "n" genes
                                    final Dataset fullDs,
                                    final ScoreMode smode) {

        final TmpStruc ts = new TmpStruc();
        final RankedList[] rls = new RankedList[rexs.length];
        final List rowNames = new ArrayList();
        final List useColNames = new ArrayList();

        ts.gsets = new GeneSet[rexs.length];
        ts.colors = new NamedColor[rexs.length];
        for (int r = 0; r < rexs.length; r++) {
            rls[r] = rexs[r].getRankedList().extractRanked(smode);
            ts.gsets[r] = rls[r].toGeneSet();
            rowNames.addAll(ts.gsets[r].getMembers());

            if (smode.isPostiveOnly()) {
                ts.colors[r] = new NamedColor(rexs[r].getComp().getClassAShortName(), rexs[r].getComp().getClassAColor());
            } else {
                ts.colors[r] = new NamedColor(rexs[r].getComp().getClassBShortName(), rexs[r].getComp().getClassBColor());
            }

            // @todo improve performance

            DatasetTemplate dt = rexs[r].getDatasetTemplate();

            Template.Class cl = dt.getTemplate().getClass(ts.colors[r].name);

            klog.debug("for rex: " + rexs[r].getName() + " # cols: " + cl.getSize() + " cl: >" + cl.getName() + "< comp: <" + ts.colors[r].name + "<");
            // Only use those columns that are in the class of interest
            List cnn = dt.getDataset(false).getColumnNames();
            int[] pp = cl.getOrderedProfilePositions();
            for (int c = 0; c < pp.length; c++) {
                String cn = cnn.get(pp[c]).toString();
                if (useColNames.contains(cn) == false) {
                    useColNames.add(cn);
                }
            }
        }

        ts.ds = new DatasetGenerators().extractRows(name, fullDs, rowNames);
        ts.ds = new DatasetGenerators().extractColumns(name, ts.ds, useColNames);
        return ts;
    }

    /**
     * A brand new lv is made - input one is NOT touched
     *
     * @param lv
     * @param gset
     * @param shareLabels
     * @return
     */
    public static LabelledVector orderByGeneSet(final LabelledVector lv, final GeneSet gset,
                                                final boolean shareLabels) {
        Vector v = orderByGeneSet(lv, gset);
        return new LabelledVector(gset, v, shareLabels);
    }

    /**
     * Brand new lv is made -- input lv is not touched
     *
     * @param lv
     * @param gset
     * @return
     */
    public static Vector orderByGeneSet(final LabelledVector lv, final GeneSet gset) {

        if (lv == null) {
            throw new IllegalArgumentException("Parameter lv cannot be null");
        }

        if (gset == null) {
            throw new IllegalArgumentException("Parameter fset cannot be null");
        }

        if (lv.getSize() != gset.getNumMembers()) {
            throw new IllegalArgumentException("Mismatched sizes lv: " + lv.getSize() + " and fset: " + gset.getNumMembers());
        }

        Vector v = new Vector(lv.getSize());
        for (int i = 0; i < gset.getNumMembers(); i++) {
            v.setElement(i, lv.getScore(gset.getMember(i)));
        }

        return v;
    }

    public static RankedList getDistancesTo(final LabelledVector lv,
                                            final String toThisLabel,
                                            final PointMetric metric,
                                            final SortMode sort,
                                            final Order order) {

        float to = lv.getScore(toThisLabel);

        final Vector dists = new Vector(lv.getSize());
        DoubleElement[] dels = new DoubleElement[lv.getSize()];

        for (int i = 0; i < lv.getSize(); i++) {
            //float dist = metric.getScore(to, lv.getElement(i)); // @note todo
            float dist = Math.abs(to - lv.getScore(i));
            dists.setElement(i, dist);
            dels[i] = new DoubleElement(i, dist);
        }

        dels = DoubleElement.sort(sort, order, dels);

        return _rl(dels, lv);
    }

    private static RankedList _rl(final DoubleElement[] delsSorted, final LabelledVector lv) {

        List rankedNames = new ArrayList();
        Vector rankedScores = new Vector(delsSorted.length);

        for (int i = 0; i < delsSorted.length; i++) {
            rankedScores.setElement(i, delsSorted[i].fValue);
            rankedNames.add(lv.getLabel(delsSorted[i].fIndex));
        }

        return new DefaultRankedList(rankedNames, rankedScores, true, true);
    }


} // End class RankedListAlgs

