package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.meg.Gene;
import gnu.trove.TFloatArrayList;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Jan 29, 2006
 *         Time: 8:57:41 PM
 */
public class RankedListGenerators {

    private static final Logger klog = XLogger.getLogger(RankedListGenerators.class);

    /**
     * Class constructor
     */
    private RankedListGenerators() {
    }


    public static RankedListDb createRdb(final Dataset fullDs, final SortMode sort, final Order order) {
        final RankedList[] rls = new RankedList[fullDs.getNumCol()];

        for (int c = 0; c < fullDs.getNumCol(); c++) {
            rls[c] = fullDs.getColumnRL(c, sort, order);
        }

        return new RankedListDbSimple(fullDs.getName(), null, null, rls);
    }

    public static RankedListDb createRdb(final String name, final Dataset fullDs, final TemplateCohort tch) {

        /// Create ranked lists for all the comps
        final RankedList[] rls = new RankedList[tch.getNumComps()];
        //RankedList[] rls = new RankedList[4];
        final DatasetMetrics dm = new DatasetMetrics();

        for (int c = 0; c < tch.getNumComps(); c++) {
            // for (int c = 0; c < 4; c++) {

            final TemplateCohort.Comp comp = tch.getComp(c);
            final DatasetTemplate subdt = comp.createDatasetTemplate(fullDs);

            // -------------------------------------------------------------------------------------------- //
            // check

            Template t = subdt.getTemplate();
            if (t.getClass(0).getName().equals(comp.getClassAShortName()) == false) {
                throw new IllegalArgumentException("Mismatched classes: t: <" + t.getClass(0).getName() + "< >" + t.getClass(1).getName() +
                        "> comp: >" + comp.getClassAShortName() + "< >" + comp.getClassBShortName() + "<");
            } else {
                klog.debug("OK classes: t: <" + t.getClass(0).getName() + "< >" + t.getClass(1).getName() +
                        "> comp: >" + comp.getClassAShortName() + "< >" + comp.getClassBShortName() + "<");
            }

            rls[c] = dm.scoreDataset(subdt).cloneShallowRL(comp.getName()); // @note IMP that the rls and the tch have the same name
            if (c % 5 == 0) {
                System.out.println("Done rl: " + (c + 1) + " / " + tch.getNumComps());
            }

            klog.debug("@@@ >" + comp.getClassAShortName() + "< >" + subdt.getTemplate().getClass(0).getName() + "<");

            System.out.println("Done RL: " + rls[c].getName() + " " + (c + 1) + " / " + rls.length);
        }

        final Annot annot_for_rl = new AnnotImpl(fullDs.getAnnot().getFeatureAnnot(), tch.getSampleAnnot());
        return new RankedListDbImpl(name, rls, fullDs.getAnnot().getChip(), annot_for_rl, fullDs, tch);
    }

    public static RankedListDb createRdb_random(final String name,
                                                final Dataset fullDs,
                                                final TemplateCohort.Comp comp,
                                                final TemplateRandomizerType rt,
                                                final RandomSeedGenerator rst,
                                                final int num) {

        final RankedList[] rls_rnd = new RankedList[num];
        final DatasetTemplate real_dt = comp.createDatasetTemplate(fullDs);

        final Template[] rndTemplates = TemplateFactoryRandomizer.createRandomTemplates(num, real_dt.getTemplate(), rt, rst);

        final DatasetMetrics dm = new DatasetMetrics();
        for (int r = 0; r < num; r++) {
            rls_rnd[r] = dm.scoreDataset(real_dt.getDataset(), rndTemplates[r]).cloneShallowRL(comp.getName() + "_rnd_" + r);
            if (r % 10 == 0) {
                System.out.println("Done rl: " + (r + 1) + " / " + num);
            }
        }

        // return new RankedListDbSimple(name, null, fullDs, rndTemplates, rls_rnd);
        return new RankedListDbSimple(name, null, fullDs, rls_rnd);
    }

    public static RankedList[] createSortedLists(final Dataframe df, final RankingMode mode) {

        klog.info("Using sort mode: " + mode.getName());

        RankedList[] rls = new RankedList[df.getNumCol()];
        for (int c = 0; c < df.getNumCol(); c++) {
            TFloatArrayList floats_pos = new TFloatArrayList();
            TFloatArrayList floats_neg = new TFloatArrayList();
            List labels_pos = new ArrayList();
            List labels_neg = new ArrayList();
            Vector v = df.getColumn(c);
            for (int i = 0; i < v.getSize(); i++) {
                if (v.getElement(i) >= 0) {
                    floats_pos.add(v.getElement(i));
                    labels_pos.add(df.getRowName(i));
                } else {
                    floats_neg.add(v.getElement(i));
                    labels_neg.add(df.getRowName(i));
                }
            }

            if (mode == RankingMode.POSITIVE_ONLY) {
                LabelledVector lv = new LabelledVector(df.getColumnName(c), labels_pos, floats_pos);
                rls[c] = lv.sort(SortMode.REAL, Order.DESCENDING); // @note IMP sort
            } else if (mode == RankingMode.NEGATIVE_ONLY) {
                LabelledVector lv = new LabelledVector(df.getColumnName(c), labels_neg, floats_neg);
                rls[c] = lv.sort(SortMode.REAL, Order.ASCENDING); // @note IMP sort
            } else if (mode == RankingMode.ABSOLUTE_VALUE) {
                LabelledVector lv = new LabelledVector(df.getColumnName(c), df.getRowNamesArray(), v);
                rls[c] = lv.sort(SortMode.ABSOLUTE, Order.DESCENDING); // @note IMP sort
            } else if (mode == RankingMode.POS_MAX2_MIN_THEN_NEG_MIN_TO_MAX) {
                throw new NotImplementedException(); // @todo if needed
                /*
                LabelledVector lvpos = new LabelledVector(df.getColumnName(c), labels_pos, floats_pos);
                lvpos = lvpos.sort(SortMode.REAL, Order.DESCENDING);

                LabelledVector lvneg = new LabelledVector(df.getColumnName(c), labels_neg, floats_neg);
                lvneg = lvneg.sort(SortMode.REAL, Order.ASCENDING);
                List names_both = new ArrayList();
                names_both.addAll(lvpos.getRowNames());
                names_both.addAll(lvneg.getRowNames());

                TFloatArrayList floats_both = new TFloatArrayList();
                floats_both.add(lvpos.getScores());
                floats_both.add(lvneg.getScores());

                lvs[c] = new LabelledVector(df.getColumnName(c), names_both, floats_both);
                */
            } else if (mode == RankingMode.REAL_SORT_DESC) {
                LabelledVector lv = new LabelledVector(df.getColumnName(c), df.getRowNamesArray(), v);
                rls[c] = lv.sort(SortMode.REAL, Order.DESCENDING); // @note IMP sort
            } else {
                throw new IllegalArgumentException("Unknown mode: " + mode);


            }
        }

        return rls;
    }

    public static RankedList createRankedList(final GeneSet gset) {

        String[] names = gset.getMembersArray();
        float[] ranked_scores = new float[names.length];
        for (int i = 0; i < names.length; i++) {
            ranked_scores[i] = names.length - i;
        }

        return new DefaultRankedList(gset.getName(true), names, ranked_scores);
    }

    // 2 column pairs
    public static RankedList[] createRankedLists(StringDataframe sdf, SortMode sort, Order order) {

        // sanity checks
        if (sdf.getNumCol() % 2 != 0) {
            throw new IllegalArgumentException("Bad sdf format: expecting 2 column pairs of name\tscore but got ncols: " + sdf.getNumCol());
        }

        klog.info("Number of name-score pairs found: " + sdf.getNumCol() / 2);

        RankedList[] rls = new RankedList[sdf.getNumCol() / 2];
        int cnt = 0;
        for (int c = 0; c < sdf.getNumCol(); c = c + 2) {
            rls[cnt++] = new LabelledVector(sdf.getColumnName(c), sdf.getColumn(c), sdf.getColumnAsVector(c + 1, false)).sort(sort, order);
        }

        return rls;
    }

    public static RankedList[] createPositiveNegativeRankedLists_ignorezeroes(RankedList rl) {

        List labels_pos = new ArrayList();
        TFloatArrayList floats_pos = new TFloatArrayList();
        List labels_neg = new ArrayList();
        TFloatArrayList floats_neg = new TFloatArrayList();

        for (int r = 0; r < rl.getSize(); r++) {
            float score = rl.getScore(r);
            String label = rl.getRankName(r);
            if (score > 0) {
                labels_pos.add(label);
                floats_pos.add(score);
            } else if (score < 0) {
                labels_neg.add(label);
                floats_neg.add(score);
            } // ignore zeroes
        }

        // these arent sorted yet, so must sort (aswe return a ranked list0

        // klog.info("pos rl size: " + labels_pos.size() + " " + labels_neg.size());

        return new RankedList[]{new LabelledVector(rl.getName() + "_pos_only", labels_pos, floats_pos).sort(SortMode.REAL, Order.DESCENDING),
                new LabelledVector(rl.getName() + "_neg_only", labels_neg, floats_neg).sort(SortMode.REAL, Order.ASCENDING)};

    }

    public static RankedList createMappedRankedList(final RankedList rlProbes, Chip chip) throws Exception {

        List names = new ArrayList();
        TFloatArrayList floats = new TFloatArrayList();
        for (int r = 0; r < rlProbes.getSize(); r++) {
            String probeName = rlProbes.getRankName(r);
            Gene gene = chip.getHugo(probeName);
            if (gene != null) {
                String symbol = gene.getSymbol();
                if (!names.contains(symbol)) {
                    names.add(symbol);
                    floats.add(rlProbes.getScore(r));
                }
            }
        }

        return new DefaultRankedList(rlProbes.getName() + "_symbolized", names, floats);
    }


    /**
     * @param rl
     * @param sort
     * @param order
     * @return
     */
    public static RankedList createBySorting(final RankedList rl, final SortMode sort, final Order order) {
        LabelledVector lv = new LabelledVector(rl, true);
        return lv.sort(sort, order); // creates a completely new LV
    }

    public static RankedList createBySorting(final String name, final String[] names, final float[] scores, final SortMode sort, final Order order) {
        LabelledVector lv = new LabelledVector(name, names, scores);
        return lv.sort(sort, order); // creates a completely new LV
    }

    public static RankedList createBySorting(final String name, final List names, final TFloatArrayList scores, final SortMode sort, final Order order) {
        LabelledVector lv = new LabelledVector(name, names, scores);
        return lv.sort(sort, order); // creates a completely new LV
    }

    public static RankedList sort(final LabelledVector lv, final SortMode sort, final Order order) {
        return lv.sort(sort, order);
    }

    /**
     * @param iv
     * @param sort
     * @param order
     * @param names
     * @return
     */
    public static RankedList sortByVectorAndGetRankedList(final String name, final Vector iv, final SortMode sort,
                                                          final Order order, final List names) {

        final DoubleElement[] dels = _sort(new AddressedVector(iv, true), sort, order);
        final Vector v = new Vector(dels.length);
        final List clonedLabels = new ArrayList();
        for (int i = 0; i < dels.length; i++) {
            v.setElement(i, (float) dels[i].fValue);
            clonedLabels.add(names.get(dels[i].fIndex));
        }

        if (name == null) {
            return new DefaultRankedList(clonedLabels, v, true, true);
        } else {
            return new DefaultRankedList(name, clonedLabels, v, true, true);
        }
    }

    public static RankedList sortByVectorAndGetRankedList(final Vector iv, final SortMode sort,
                                                          final Order order, final List names) {
        return RankedListGenerators.sortByVectorAndGetRankedList(null, iv, sort, order, names);
    }

    public static RankedList sortByVectorAndGetRankedList(final AddressedVector av, final SortMode sort,
                                                          final Order order, final List names) {
        DoubleElement[] dels = _sort(av, sort, order);

        Vector v = new Vector(dels.length);
        List labels = new ArrayList();
        for (int i = 0; i < dels.length; i++) {
            v.setElement(i, (float) dels[i].fValue);
            labels.add(names.get(dels[i].fIndex));
        }

        return new DefaultRankedList(labels, v, true, true);
    }

    /**
     * Class constructor
     *
     * @param v
     * @param labels
     * @param sort
     * @param order
     */
    public static LabelledVector createLV(final Vector v, final String[] labels,
                                          final SortMode sort, final Order order) {

        if (v.getSize() != labels.length) {
            throw new MismatchedSizeException("Vector", v.getSize(), "Labels", +labels.length);
        }

        AddressedVector av = new AddressedVector(v, false);
        int[] indexOrder = sortByVectorAndGetAddresses(av, sort, order);
        Vector sorted = new Vector(indexOrder.length);
        String[] slabels = new String[indexOrder.length];
        for (int i = 0; i < indexOrder.length; i++) {
            sorted.setElement(i, av.getScore(indexOrder[i]));
            slabels[i] = labels[indexOrder[i]];
        }

        return new LabelledVector(slabels, sorted);
    }

    public static LabelledVector createLV(final GeneSet gset, RankedList corrs) {

        if (corrs == null) {
            throw new IllegalArgumentException("Parameter corrs cannot be null");
        }

        float[] floats = new float[gset.getNumMembers()];
        String[] labels = new String[gset.getNumMembers()];
        for (int i = 0; i < gset.getNumMembers(); i++) {
            labels[i] = gset.getMember(i);
            floats[i] = corrs.getScore(labels[i]);
        }

        return new LabelledVector(gset.getName() + "_" + corrs.getName(), labels, floats);
    }

    protected static DoubleElement[] _sort(final AddressedVector av, final SortMode sort, final Order order) {
        DoubleElement[] dels = new DoubleElement[av.getSize()];
        for (int i = 0; i < av.getSize(); i++) {
            dels[i] = new DoubleElement(av.getAddress(i), av.getScore(i));
        }

        return DoubleElement.sort(sort, order, dels);
    }

    /**
     * addresses are NOT necc same as ranks. see below for a "rank" method.
     * This AddressedVector is NOT touched.
     *
     * @param sort
     * @param order
     * @return
     */
    public static int[] sortByVectorAndGetAddresses(final AddressedVector av, final SortMode sort,
                                                    final Order order) {
        DoubleElement[] dels = _sort(av, sort, order);
        return DoubleElement.indexArray(dels);
    }

    public static double[] sortByVectorAndGetScores(final AddressedVector av, final SortMode sort, final Order order) {
        DoubleElement[] dels = _sort(av, sort, order);
        return DoubleElement.valueArray(dels);
    }

    /**
     * ranks are always 0 to n-1 irrespective of what the address was
     * shared addresses are arbitarily given adjacent ranks (ranks are NOT shared)
     * This AddressedVector is NOT touched.
     *
     * @param sort
     * @param order
     * @return
     */
    public static int[] sortByVectorAndGetRanks(final AddressedVector av, final SortMode sort, final Order order, boolean startFromOne) {

        int prefix = 0;
        if (startFromOne) {
            prefix = 1;
        }

        DoubleElement[] dels = _sort(av, sort, order);
        int[] ranks = new int[av.getSize()];
        for (int i = 0; i < dels.length; i++) {
            ranks[i] = dels[i].fIndex + prefix;
        }

        Printf.out(ranks);

        return ranks;

        /*

        Vector v = new Vector(dels.length);
        List labels = new ArrayList();
        for (int i = 0; i < dels.length; i++) {
            v.setElement(i, (float) dels[i].fValue);
            labels.add(names.get(dels[i].fIndex));
        }

        return new LabelledVector(labels, v, true);
        */

    }


} // End class RankedListGenerators