/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.Dataframe;
import edu.mit.broad.genome.objects.RankedList;
import gnu.trove.TFloatArrayList;
import gnu.trove.TIntArrayList;
import gnu.trove.TIntFloatHashMap;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Mar 10, 2004
 *         Time: 11:06:57 PM
 */
public class RankedListsComparisonStruc {

    private TIntArrayList ranksA;
    private TIntArrayList ranksB;
    private TFloatArrayList scoresA;
    private TFloatArrayList scoresB;
    private List rowNames;
    private String name;

    private RankedList rA;
    private RankedList rB;

    public RankedListsComparisonStruc(String name, final RankedList rA, final RankedList rB, final boolean allowPartialMatches) {
        this.name = name;
        this.rowNames = new ArrayList();
        this.ranksA = new TIntArrayList();
        this.ranksB = new TIntArrayList();

        this.scoresA = new TFloatArrayList();
        this.scoresB = new TFloatArrayList();

        this.rA = rA;
        this.rB = rB;

        for (int rankA = 0; rankA < rA.getSize(); rankA++) {
            String aName = rA.getRankName(rankA);
            int rankB = rB.getRank(aName);

            if (rankB == -1 && !allowPartialMatches) {
                throw new IllegalArgumentException("RowName in A had no match in B:" + aName);
            }

            if (rankB != -1) {
                add(rankA, rankB, rA.getScore(rankA), rB.getScore(rankB), aName);
            }
        }
    }

    public RankedList getRankedListA() {
        return rA;
    }

    public RankedList getRankedListB() {
        return rB;
    }

    private void add(int rankA, int rankB, float scoreA, float scoreB, String rowName) {
        if (rowNames.contains(rowName)) {
            throw new IllegalArgumentException("Already comtains rank comp for: " + rowName);
        }

        //System.out.println("adding: " + rowName + " " + (rankA+1) + " " + (rankB+1));

        this.ranksA.add(rankA + 1); // +1 as spearman needs ranks from 1 to n and not 0 to n-1
        this.ranksB.add(rankB + 1);
        this.rowNames.add(rowName);
        this.scoresA.add(scoreA);
        this.scoresB.add(scoreB);
    }

    public String getBriefReport() {
        return spearmanRanks() + "\t" + pearsonRanks() + "\t" + pearsonScores() + "\t" + getSize();
    }

    public TIntFloatHashMap getPearsonRanksInWindows(int offset) {

        TIntFloatHashMap floats = new TIntFloatHashMap();
        int span = offset;

        while (true) {
            int end = span;
            if (end > getSize()) {
                end = getSize();
            }

            floats.put(end, (float) pearsonRanks(end));

            span += offset;
            if (span > getSize()) {
                break;
            }
        }

        return floats;
    }

    public int getSize() {
        return ranksA.size();
    }

    public double spearmanRanks() {
        return XMath.spearman(ranksA.toNativeArray(), ranksB.toNativeArray());
    }

    public double pearsonRanks() {
        return XMath.pearson(ranksA.toNativeArray(), ranksB.toNativeArray());
    }

    public double pearsonScores() {
        return XMath.pearson(scoresA.toNativeArray(), scoresB.toNativeArray());
    }

    public double spearmanRanks(int span) {
        return XMath.spearman(ranksA.toNativeArray(0, span), ranksB.toNativeArray(0, span));
    }

    public double pearsonRanks(int span) {
        return XMath.pearson(ranksA.toNativeArray(0, span), ranksB.toNativeArray(0, span));
    }

    public double pearsonScores(int span) {
        return XMath.pearson(scoresA.toNativeArray(0, span), scoresB.toNativeArray(0, span));
    }

    public Dataframe getRankReport() {
        Matrix m = new Matrix(rowNames.size(), 4);
        for (int r = 0; r < rowNames.size(); r++) {
            int coln = 0;
            m.setElement(r, coln++, ranksA.get(r));
            m.setElement(r, coln++, ranksB.get(r));
            m.setElement(r, coln++, scoresA.get(r));
            m.setElement(r, coln, scoresB.get(r));
        }

        List colNames = new ArrayList(4);
        colNames.add("RANK_A");
        colNames.add("RANK_B");
        colNames.add("SCORE_A");
        colNames.add("SCORE_B");

        return new Dataframe(name + "_rank_report", m, rowNames, colNames, true);
    }

}
