/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DefaultMetricWeightStruc;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import gnu.trove.TIntIntHashMap;
import gnu.trove.TObjectIntHashMap;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Feb 12, 2004
 *         Time: 2:58:53 PM
 */
public class TemplateStats {

    private static final Logger klog = XLogger.getLogger(TemplateStats.class);

    private TemplateStats() {
    }

    public static String[] getClassNamesInOrder(final Template t) {
        List names = new ArrayList();

        for (int i = 0; i < t.getNumItems(); i++) {

            Template.Class item = t.getClassByProfilePos(i);
            String iname;
            if (item == null) {
                iname = "N";
            } else {
                iname = item.getName();
            }

            if (names.contains(iname) == false) {
                names.add(iname);
            }
        }

        return (String[]) names.toArray(new String[names.size()]);
    }

    public static edu.mit.broad.genome.math.Vector calcAverageHammingDists(Template real, Template[] rndTemplates) {

        edu.mit.broad.genome.math.Vector v = new edu.mit.broad.genome.math.Vector(rndTemplates.length);
        for (int i = 0; i < rndTemplates.length; i++) {
            v.setElement(i, TemplateStats.calcAverageHammingDist(real, rndTemplates[i]));
        }

        return v;
    }

    public static int calcAverageHammingDist(final Template a, final Template b) {

        int dist = 0;

        for (int i = 0; i < a.getNumItems(); i++) {
            Template.Item itema = a.getItemByProfilePos(i);
            Template.Item itemb = b.getItemByProfilePos(i);
            if (itema.getId().equals(itemb.getId())) {
                ;
            } else {
                dist++;
            }
        }

        return dist;
    }

    public static Composition getComposition(final Template[] tss) {
        return new Composition(tss);
    }

    public static class Composition {

        private Template[] tss;

        private TIntIntHashMap currPosCntMap;

        private StringBuffer fClassBreakup;

        Composition(Template[] tss) {
            this.tss = tss;

            this.currPosCntMap = new TIntIntHashMap();
            this.fClassBreakup = new StringBuffer();

            for (int r = 0; r < tss.length; r++) {
                cntIt(tss[r], currPosCntMap);

                // assumes 2 class template
                this.fClassBreakup.append(tss[r].getNumItems()).append('\t').append(tss[r].getClass(0).getSize()).append('\t').append(tss[r].getClass(1)).append('\n');
            }

        }

        public TObjectIntHashMap getSampleUsage(Dataset ds) {

            TObjectIntHashMap map = new TObjectIntHashMap();

            for (int t = 0; t < tss.length; t++) {

                // every template may not use all samples
                List colNames = tss[t].extractColumnNames(ds);
                for (int i = 0; i < colNames.size(); i++) {
                    String colname = colNames.get(i).toString();
                    int curr = map.get(colname);
                    curr++;
                    map.put(colname, curr);
                }
            }

            return map;
        }

        public String getClassBreakup() {
            return fClassBreakup.toString();
        }

        private void cntIt(Template t, TIntIntHashMap currPosCntMap) {
            int[] profilePositions = t.getProfilePositionsOrdered();
            for (int i = 0; i < profilePositions.length; i++) {
                int currn = currPosCntMap.get(profilePositions[i]);
                currn = currn + 1;
                currPosCntMap.put(profilePositions[i], currn);
            }

            //Printf.outlv(t);
        }

        public TIntIntHashMap getCurrPosCnts() {
            return currPosCntMap;
        }


        public Portion compare(final Dataset ds, final Template real) {
            DatasetGenerators gen = new DatasetGenerators();
            Set class0SampleNames = new HashSet(gen.extract(ds, real.getClass(0)).getColumnNames());
            Set class1SampleNames = new HashSet(gen.extract(ds, real.getClass(1)).getColumnNames());
            List colNames = ds.getColumnNames();

            Portion portion = new Portion(colNames);
            for (int t = 0; t < tss.length; t++) {
                Set[] sets = tss[t].splitByTemplateClass(colNames);
                add2Portion(class0SampleNames, class1SampleNames, sets[0], sets[1], portion);
            }

            return portion;
        }


        private void add2Portion(Set class0SampleNames, Set class1SampleNames, Set split0, Set split1, Portion portion) {

            Usage usage = new Usage();

            for (Iterator it = split0.iterator(); it.hasNext();) {
                Object key = it.next();
                if (class0SampleNames.contains(key)) {
                    usage.real0in0++;
                    portion.addUsage(key.toString(), true);
                } else if (class1SampleNames.contains(key)) {
                    usage.real1in0++;
                    portion.addUsage(key.toString(), false);
                } else {
                    throw new IllegalArgumentException("unknown sample name: " + key);
                }
            }

            for (Iterator it = split1.iterator(); it.hasNext();) {
                Object key = it.next();
                if (class0SampleNames.contains(key)) {
                    usage.real0in1++;
                    portion.addUsage(key.toString(), false);
                } else if (class1SampleNames.contains(key)) {
                    usage.real1in1++;
                    portion.addUsage(key.toString(), true);
                } else {
                    throw new IllegalArgumentException("unknown sample name: " + key);
                }
            }

            portion.addUsage(usage);

        }

    } // End class Composition


    public static class Portion {

        TObjectIntHashMap sampleNameRealUsageMap;
        TObjectIntHashMap sampleNameWrongUsageMap;
        List sampleNames;

        List usages;

        Portion(List sampleNames) {
            this.sampleNames = sampleNames;
            this.sampleNameRealUsageMap = new TObjectIntHashMap();
            this.sampleNameWrongUsageMap = new TObjectIntHashMap();

            for (int i = 0; i < sampleNames.size(); i++) {
                this.sampleNameRealUsageMap.put(sampleNames.get(i), 0);
                this.sampleNameWrongUsageMap.put(sampleNames.get(i), 0);
            }

            this.usages = new ArrayList();
        }

        void addUsage(String sampleName, boolean correct) {
            if (correct) {
                sampleNameRealUsageMap.increment(sampleName);
            } else {
                sampleNameWrongUsageMap.increment(sampleName);
            }
        }

        void addUsage(Usage usage) {
            usages.add(usage);
        }

        public Dataframe toUsageCounts() {
            String[] colnames = new String[]{"real0in0", "real0in1", "real1in0", "real1in1", "tot_CORRECT", "tot_INCORR"};
            Matrix m = new Matrix(usages.size(), colnames.length);
            for (int r = 0; r < usages.size(); r++) {
                int coln = 0;
                Usage usage = (Usage) usages.get(r);
                m.setElement(r, coln++, usage.real0in0);
                m.setElement(r, coln++, usage.real0in1);
                m.setElement(r, coln++, usage.real1in0);
                m.setElement(r, coln++, usage.real1in1);
                m.setElement(r, coln++, usage.getTotalCorrect());
                m.setElement(r, coln, usage.getTotalInCorrect());
            }

            return new Dataframe("usage_counts", m, colnames, true);
        }

        public Dataframe toUsageSamples() {
            String[] colnames = new String[]{"correct_usage", "INcorrect_usage", "total_usage"};
            Matrix m = new Matrix(sampleNames.size(), colnames.length);
            for (int r = 0; r < sampleNames.size(); r++) {
                String sampleName = sampleNames.get(r).toString();
                m.setElement(r, 0, sampleNameRealUsageMap.get(sampleName));
                m.setElement(r, 1, sampleNameWrongUsageMap.get(sampleName));
                m.setElement(r, 2, sampleNameRealUsageMap.get(sampleName) + sampleNameWrongUsageMap.get(sampleName));
            }

            return new Dataframe("sample_usage_stats", m, sampleNames, colnames, true, false);
        }

    }

    static class Usage {
        int real0in0;
        int real0in1;
        int real1in0;
        int real1in1;

        private int getTotalCorrect() {
            return real0in0 + real1in1;
        }

        private int getTotalInCorrect() {
            return real0in1 + real1in0;
        }

    }


    // @todo consider whether to place here or not
    public static TemplateHitStruc createRandomTemplates(final float fudgeLength,
                                                         final float fudgeWeight,
                                                         final int nperms,
                                                         final int maxAttemptsAllowed,
                                                         final SortMode sort,
                                                         Order order,
                                                         final Metric metric,
                                                         final Map mps,
                                                         final LabelledVectorProcessor rp,
                                                         final Dataset ds,
                                                         final Template template,
                                                         final TemplateRandomizerType rt,
                                                         final RandomSeedGenerator rst,
                                                         final RandomRankedListMode rmode) {


        final DatasetMetrics dm = new DatasetMetrics();
        final ScoredDataset sdsReal = dm.scoreDataset(metric, sort, order, mps, rp, ds, template);
        final DefaultMetricWeightStruc real = new DefaultMetricWeightStruc(metric.getName(), sdsReal);
        float real_length_frac = real.getTotalPosLength_frac();
        float real_weight_frac = real.getTotalPosWeight_frac();

        int numNeededLength = 0;
        int numNeededWeight = 0;
        int numNeededBothLengthAndWeight = 0;

        if (rmode.isRandom()) {
            // do nothing
        } else if (rmode.isSameLength()) {
            numNeededLength = nperms;
        } else if (rmode.isSameWeight()) {
            numNeededWeight = nperms;
        } else if (rmode.isSameLengthAndWeight()) {
            numNeededBothLengthAndWeight = nperms;
        } else {
            throw new IllegalArgumentException("Invalid rmode: " + rmode);
        }


        klog.info("For real template real_length_frac: " + real_length_frac + " real_weight_frac: " + real_weight_frac);

        final List hitsLength = new ArrayList();
        final List hitsWeight = new ArrayList();
        final List hitsLengthAndWeight = new ArrayList();
        final List hitsRandom = new ArrayList();

        int numAttempts = 0;

        boolean stopIt = false;
        boolean forcedBreak = false;
        while (true) {
            final Template rndt = TemplateFactoryRandomizer.createRandomTemplates(1, template, rt, rst, true)[0];

            if (rmode.isRandom()) {
                hitsRandom.add(new TemplateHitStruc.Hit(rndt, null, null));

                if (hitsRandom.size() >= nperms) {
                    forcedBreak = false;
                    stopIt = true;
                }

            } else {

                final ScoredDataset sdsRnd = dm.scoreDataset(metric, sort, order, mps, rp, ds, rndt);
                final DefaultMetricWeightStruc rndws = new DefaultMetricWeightStruc(metric.getName(), sdsRnd);
                final float rnd_length_frac = rndws.getTotalPosLength_frac();
                final float rnd_weight_frac = rndws.getTotalPosWeight_frac();

                if (Math.abs(real_length_frac - rnd_length_frac) <= fudgeLength) {
                    hitsLength.add(new TemplateHitStruc.Hit(rndt, rndws, sdsRnd));
                }

                if (Math.abs(real_weight_frac - rnd_weight_frac) <= fudgeWeight) {
                    hitsWeight.add(new TemplateHitStruc.Hit(rndt, rndws, sdsRnd));
                }

                if ((Math.abs(real_length_frac - rnd_length_frac) <= fudgeLength) && (
                        (Math.abs(real_weight_frac - rnd_weight_frac) <= fudgeWeight))) {
                    hitsLengthAndWeight.add(new TemplateHitStruc.Hit(rndt, rndws, sdsRnd));
                }

                if (hitsLength.size() < numNeededLength) {
                    stopIt = false;
                }

                if (hitsWeight.size() < numNeededWeight) {
                    stopIt = false;
                }

                if (hitsLengthAndWeight.size() < numNeededBothLengthAndWeight) {
                    stopIt = false;
                }

                if (hitsLength.size() >= numNeededLength && hitsWeight.size() >= numNeededWeight && hitsLengthAndWeight.size() >= numNeededBothLengthAndWeight) {
                    stopIt = true;
                }

                if (numAttempts % 50 == 0) {
                    System.out.println("numAttempts: " + numAttempts + " / " + maxAttemptsAllowed +
                            " hitsLength: " + hitsLength.size() + " / " + numNeededLength +
                            " hitsWeight: " + hitsWeight.size() + " / " + numNeededWeight +
                            " hitsLengthAndWeight: " + hitsLengthAndWeight.size() + " / " + numNeededBothLengthAndWeight);
                }

                // This is the ultimate decider
                if (numAttempts >= maxAttemptsAllowed) {
                    stopIt = true;
                    forcedBreak = true;
                }

            }

            numAttempts++;

            if (stopIt) {
                break;
            }

        }

        klog.info("Templates length: " + numNeededLength + " / " + hitsLength.size()
                + " weight: " + numNeededWeight + " / " + hitsWeight.size() +
                " BothLengthAndWeight: " + numNeededBothLengthAndWeight + " / " + hitsLengthAndWeight.size());

        if (forcedBreak) {
            klog.warn("Could NOT make as many templates as asked for!");
        }

        TemplateHitStruc hs = new TemplateHitStruc(sdsReal, real, fudgeLength, fudgeWeight);
        hs.rndHits_length = (TemplateHitStruc.Hit[]) hitsLength.toArray(new TemplateHitStruc.Hit[hitsLength.size()]);
        hs.rndHits_weight = (TemplateHitStruc.Hit[]) hitsWeight.toArray(new TemplateHitStruc.Hit[hitsWeight.size()]);
        hs.rndHits_length_and_weight = (TemplateHitStruc.Hit[]) hitsLengthAndWeight.toArray(new TemplateHitStruc.Hit[hitsLengthAndWeight.size()]);
        hs.rndHits_random = (TemplateHitStruc.Hit[]) hitsRandom.toArray(new TemplateHitStruc.Hit[hitsRandom.size()]);

        return hs;
    }

    public static class TemplateHitStruc {
        private MetricWeightStruc real;
        private RankedList reaLRankedList;
        private Hit[] rndHits_random;
        private Hit[] rndHits_length;
        private Hit[] rndHits_weight;
        private Hit[] rndHits_length_and_weight;

        private float fudgeLength;
        private float fudgeWeight;

        TemplateHitStruc(final RankedList reaLRankedList,
                         final MetricWeightStruc real,
                         final float fudgeLength,
                         float fudgeWeight) {

            this.reaLRankedList = reaLRankedList;
            this.real = real;
            this.fudgeLength = fudgeLength;
            this.fudgeWeight = fudgeWeight;
        }

        public RankedList getRealRankedList() {
            return reaLRankedList;
        }

        public Hit[] getRandomHits(final RandomRankedListMode rmode) {

            if (rmode.isRandom()) {
                return rndHits_random;
            } else if (rmode.isSameLength()) {
                return rndHits_length;
            } else if (rmode.isSameWeight()) {
                return rndHits_weight;
            } else if (rmode.isSameLengthAndWeight()) {
                return rndHits_length_and_weight;
            } else if (rmode.isSameES()) {
                throw new RuntimeException("not impl");
            } else {
                throw new IllegalArgumentException("Unknown RankedListMode: " + rmode);
            }
        }

        public float getFudge(final RandomRankedListMode rmode) {
            if (rmode.isRandom()) {
                return Float.NaN;
            } else if (rmode.isSameLength()) {
                return fudgeLength;
            } else if (rmode.isSameWeight()) {
                return fudgeWeight;
            } else if (rmode.isSameLengthAndWeight()) {
                throw new IllegalArgumentException("na");
            } else if (rmode.isSameES()) {
                throw new RuntimeException("not impl");
            } else {
                throw new IllegalArgumentException("Unknown RankedListMode: " + rmode);
            }
        }

        public Template[] getRandomTemplates(final RandomRankedListMode rmode) {
            Hit[] hits = getRandomHits(rmode);
            Template[] rnd = new Template[hits.length];
            for (int i = 0; i < hits.length; i++) {
                rnd[i] = hits[i].getTemplate();
            }

            return rnd;
        }

        public RankedList[] getRandomRankedLists(final RandomRankedListMode rmode) {
            Hit[] hits = getRandomHits(rmode);
            RankedList[] rnd = new RankedList[hits.length];
            for (int i = 0; i < hits.length; i++) {
                rnd[i] = hits[i].getRankedList();
            }

            return rnd;
        }

        public MetricWeightStruc getRealMetricWeightStruc() {
            return real;
        }

        public static class Hit {

            private Template t;
            private MetricWeightStruc mws;
            private RankedList rl;

            Hit(final Template t, final MetricWeightStruc mws, final RankedList rl) {
                this.t = t;
                this.mws = mws;
                this.rl = rl;
            }

            public Template getTemplate() {
                return t;
            }

            public RankedList getRankedList() {
                return rl;
            }

            public MetricWeightStruc getMetricWeightStruc() {
                return mws;
            }
        } // End class Hit
    } // End class TemplateHitStruc


    public static class RandomRankedListMode {

        public static final RandomRankedListMode RANDOM = new RandomRankedListMode("random");
        public static final RandomRankedListMode SAME_LENGTH_AS_REAL = new RandomRankedListMode("same_length");
        public static final RandomRankedListMode SAME_WEIGHT_DELTA_AS_REAL = new RandomRankedListMode("same_weight");
        public static final RandomRankedListMode SAME_LENGTH_AND_WEIGHT_AS_REAL = new RandomRankedListMode("same_length_and_weight");
        public static final RandomRankedListMode SAME_ES_AS_REAL = new RandomRankedListMode("same_es");

        public static RandomRankedListMode[] createAllModes() {
            return new RandomRankedListMode[]{RANDOM, SAME_LENGTH_AS_REAL, SAME_WEIGHT_DELTA_AS_REAL, SAME_LENGTH_AND_WEIGHT_AS_REAL, SAME_ES_AS_REAL};
        }

        public static RandomRankedListMode lookupRandomizerType(Object obj) {
            if (obj == null) {
                throw new IllegalArgumentException("param type cannot be null");
            }


            if (obj instanceof RandomRankedListMode) {
                return (RandomRankedListMode) obj;
            }

            RandomRankedListMode[] modes = createAllModes();
            for (int i = 0; i < modes.length; i++) {
                if (modes[i].getName().equals(obj.toString())) {
                    return modes[i];
                }
            }

            throw new IllegalArgumentException("Unknown RandomRankedListMode type: " + obj);
        }

        private String fName;

        /**
         * Class constructor
         *
         * @param name
         * @param desc
         */
        RandomRankedListMode(String name) {
            this.fName = name;
        }

        public String getName() {
            return fName;
        }

        public String toString() {
            return getName();
        }

        public boolean isRandom() {
            return this.getName().equalsIgnoreCase(RANDOM.getName());
        }

        public boolean isSameLength() {
            return this.getName().equalsIgnoreCase(SAME_LENGTH_AS_REAL.getName());
        }

        public boolean isSameWeight() {
            return this.getName().equalsIgnoreCase(SAME_WEIGHT_DELTA_AS_REAL.getName());
        }

        public boolean isSameLengthAndWeight() {
            return this.getName().equalsIgnoreCase(SAME_LENGTH_AND_WEIGHT_AS_REAL.getName());
        }

        public boolean isSameES() {
            return this.getName().equalsIgnoreCase(SAME_ES_AS_REAL.getName());
        }

    } // End class RandomRankedList


} // End class TemplateStats


