package edu.mit.broad.genome.alg.cluster;

import edu.mit.broad.genome.objects.*;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.gsea.HCLAlgorithm;
import org.genepattern.gsea.HCLViewer;
import xtools.api.param.ClusterAxisParam;
import xtools.api.param.ClusterDistanceParam;
import xtools.api.param.LinkageParam;

/**
 * Wrapper class to access Hierarchical clustering algorithms
 *
 * @author Aravind Subramanian
 *         Date: Feb 27, 2006
 *         Time: 5:11:33 PM
 */
public class ClusteringAlgs {


    private ClusteringAlgs() {
    }

    public static HClustDataset cluster(final Dataset ds) {

        // @note the defaults
        final IExpressionData expressionData = GPWrappers.createIExpressionData(ds);
        final int mergeType = HCLAlgorithm.SINGLE_LINKAGE;
        final int distanceMeasure = HCLAlgorithm.PEARSON;
        final int clusterBy = HCLAlgorithm.BOTH;

        return toHCD(expressionData, ds.getAnnot(), mergeType, distanceMeasure, clusterBy);
    }

    public static HClustDataset cluster_cols(final Dataset ds) {

        // @note the defaults
        final IExpressionData expressionData = GPWrappers.createIExpressionData(ds);
        final int mergeType = HCLAlgorithm.SINGLE_LINKAGE;
        final int distanceMeasure = HCLAlgorithm.PEARSON;
        final int clusterBy = HCLAlgorithm.COLUMNS;

        return toHCD(expressionData, ds.getAnnot(), mergeType, distanceMeasure, clusterBy);
    }

    public static HClustDataset cluster(final Dataset ds,
                                        final LinkageParam linkageParam,
                                        final ClusterDistanceParam clusterDistParam,
                                        final ClusterAxisParam clusterAxisParam
    ) {

        final int mergeType = linkageParam.getModeIndex();
        final int distanceMeasure = clusterDistParam.getModeIndex();
        final int clusterBy = clusterAxisParam.getModeIndex();

        return cluster(ds, mergeType, distanceMeasure, clusterBy);
    }

    // @note does the real thing
    public static HClustDataset cluster(final Dataset ds,
                                        final int linkageMode,
                                        final int clusterDistMode,
                                        final int clusterAxisMode
    ) {

        if (ds == null) {
            throw new IllegalArgumentException("Param dataset cannot be null");
        }

        if (ds.getNumRow() == 0) {
            throw new IllegalArgumentException("Cannot cluster a dataset with 0 rows: " + ds.getQuickInfo());
        }

        if (ds.getNumCol() == 0) {
            throw new IllegalArgumentException("Cannot cluster a dataset with 0 columns: " + ds.getQuickInfo());
        }

        final IExpressionData expressionData = GPWrappers.createIExpressionData(ds);
        return toHCD(expressionData, ds.getAnnot(), linkageMode, clusterDistMode, clusterAxisMode);
    }


    public static HClustDataset cluster(final GeneSetMatrix gm) {
        Dataset ds = new BitSetDataset(gm).toDataset(true, false);
        return cluster(ds);
    }

    // -------------------------------------------------------------------------------------------- //

    private static HClustDataset toHCD(final IExpressionData expressionData,
                                       final Annot annot_opt,
                                       final int linkageMode,
                                       final int clusterDistMode,
                                       final int clusterAxisMode
    ) {

        final HCLViewer hclv = new HCLViewer(expressionData, linkageMode, clusterDistMode, clusterAxisMode);
        final IExpressionData ied = hclv.getReorderedData();
        final Dataset ds_reord = GPWrappers.createDataset(ied, annot_opt);
        return new HClustDataset(ds_reord, hclv.getArrayTreePanel(), hclv.getGeneTreePanel());
    }


} // End class ClusteringAlgs

