/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.distrib;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import gnu.trove.TFloatObjectHashMap;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Nov 29, 2004
 *         Time: 4:15:46 PM
 */
public class DistribAlgs {

    private static final Logger klog = XLogger.getLogger(DistribAlgs.class);


    public static class CenteredRanges {

        Range centerBucket;
        RangePair[] buckets;

        float centeredScore;

        CenteredRanges(float centeredScore) {
            this.centeredScore = centeredScore;
        }

        public RangePair[] getRangePairs() {
            return buckets;
        }

        public Range getCenterBucket() {
            return centerBucket;
        }

    }

    public static class RangePair {
        public Range lessThan;
        public Range moreThan;

        RangePair(Range lessThan, Range moreThan) {
            this.lessThan = lessThan;
            this.moreThan = moreThan;
        }
    }

    // return those ranges that would make the range in which the score falls to be centered
    // (in mean not in counts)
    public static CenteredRanges getCenteredRanges(float score, Range[] allBuckets) {

        CenteredRanges centered = new CenteredRanges(score);

        // pick the bucket in which the real falls
        centered.centerBucket = pickMatchingRange(score, allBuckets);
        allBuckets = removeRange(centered.centerBucket, allBuckets, true);

        // now pick those buckets that are greater than the real bucket
        Range[] moreThanBuckets = getMoreThan_by_min(score, allBuckets);
        moreThanBuckets = sort(SortMode.REAL, Order.ASCENDING, moreThanBuckets);

        Range[] lessThanBuckets = getLessThan_by_max(score, allBuckets);
        lessThanBuckets = sort(SortMode.REAL, Order.DESCENDING, lessThanBuckets);

        int num2use = XMath.min(moreThanBuckets.length, lessThanBuckets.length);

        centered.buckets = new RangePair[num2use];
        for (int i = 0; i < num2use; i++) {
            centered.buckets[i] = new RangePair(lessThanBuckets[i], moreThanBuckets[i]);
        }

        return centered;
    }

    public static final Range[] sort(SortMode sort, Order order, Range[] ranges) {
        TFloatObjectHashMap map = hashByMean(ranges);
        Vector v = new Vector(map.keys());
        v.sort(sort, order);

        List list = new ArrayList();
        for (int i = 0; i < v.getSize(); i++) {
            Range range = (Range) map.get(v.getElement(i));
            list.add(range);
        }

        return (Range[]) list.toArray(new Range[list.size()]);
    }

    public static TFloatObjectHashMap hashByMean(Range[] ranges) {
        TFloatObjectHashMap map = new TFloatObjectHashMap();

        for (int r = 0; r < ranges.length; r++) {
            float key = (float) ranges[r].getMean();
            if (map.containsKey(key)) {
                throw new IllegalArgumentException("Duplicate range: " + key);
            }

            map.put(key, ranges[r]);
        }

        return map;
    }

    public static Range[] getMoreThan_by_min(float value, Range[] ranges) {
        List list = new ArrayList();
        for (int r = 0; r < ranges.length; r++) {
            if (ranges[r].getMin() > value) {
                list.add(ranges[r]);
            }
        }

        return (Range[]) list.toArray(new Range[list.size()]);
    }

    public static Range[] getLessThan_by_max(float value, Range[] ranges) {
        List list = new ArrayList();
        for (int r = 0; r < ranges.length; r++) {
            if (ranges[r].getMax() < value) {
                list.add(ranges[r]);
            }
        }

        return (Range[]) list.toArray(new Range[list.size()]);
    }

    public static Range[] removeRange(Range removeThis, Range[] fromThese, boolean barfIfNotFound) {
        List pass = new ArrayList();
        for (int i = 0; i < fromThese.length; i++) {
            if (fromThese[i] != removeThis) {
                pass.add(fromThese[i]);
            }
        }

        if (pass.size() == fromThese.length) {
            if (barfIfNotFound) {
                throw new IllegalStateException("Nothing removed not found: " + removeThis);
            } else {
                klog.warn("Nothing removed: " + removeThis);
            }
        }
        return (Range[]) pass.toArray(new Range[pass.size()]);
    }

    public static Range pickMatchingRange(final float real, final Range[] ranges) {
        Range range = null;
        for (int r = 0; r < ranges.length; r++) {
            if (ranges[r].isInRange(real)) {
                if (range != null) {
                    throw new IllegalStateException("More than one range matches: " + real);
                }
                range = ranges[r];
            }
        }

        if (range == null) {
            throw new IllegalArgumentException("No match for: " + range + " looked in ranges: " + ranges.length);
        }

        return range;
    }

} // End class DistribTmp

