/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.distrib;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * @author Aravind Subramanian
 *         Date: Mar 27, 2004
 *         Time: 8:25:20 PM
 */
public class RangeFactory {

    private static Logger klog = XLogger.getLogger(RangeFactory.class);


    public static class Bin {

        public static Range[] bin(int[] values, int windowSize) {

            int[] minmax = XMath.minmax(values);

            klog.info("min: " + minmax[0] + " max: " + minmax[0]);

            // first create the ranges

            List ranges = new ArrayList();

            if (minmax[0] == minmax[1]) {
                klog.info("min==max, so only 1 Range");
                ranges.add(new SimpleRange(minmax[0], minmax[1]));
            } else {

                for (int i = minmax[0]; i < minmax[1]; i += windowSize) {
                    ranges.add(new SimpleRange(i, i + windowSize));
                }
            }

            // note that a point on the boundary will arbitarilly be put into the first Range encountered
            int numBinned = 0;
            for (int i = 0; i < values.length; i++) {
                boolean binned = false;
                for (int j = 0; j < ranges.size(); j++) {
                    Range range = (Range) ranges.get(j);
                    if (range.isInRange(values[i])) {
                        range.increment();
                        binned = true;
                        numBinned++;
                        break;
                    }
                }

                if (!binned) {
                    System.out.println("Value was not binned as it didnt fall in any of the Ranges: " + values[i]);
                    //throw new IllegalStateException("Value was not binned as it didnt fall in any of the Ranges: " + values[i]);
                }
            }

            // cant do this as the ranges might already have stuff inthem
            //int tot = RangeUtils.getTotalCount(ranges);
            int tot = numBinned;
            if (tot != values.length) {
                throw new IllegalStateException("Bad binning! Binned: " + tot + " but elements in Vector: " + values.length);
            }

            return (Range[]) ranges.toArray(new Range[ranges.size()]);
        }

    }

    // ---------------------------------------------------------------- //
    // -------------------- FACTORY METHODS --------------------------- //
    // ---------------------------------------------------------------- //

    /**
     * Factory method:
     * For example createRanges(5, 0, 100, 1) does ??
     * Min and max are INclusive
     */
    public static FuzzyRange[] createRanges_fuzzy(double spread, double minInclusive, double maxInclusive, double fuzz) {

        List ranges = new ArrayList();

        for (double i = minInclusive; i <= maxInclusive;) {
            double rmax = i + spread;

            if (rmax > maxInclusive) {
                rmax = maxInclusive;
            }

            ranges.add(new FuzzyRange(i, rmax, fuzz));

            i += spread;
        }

        return (FuzzyRange[]) ranges.toArray(new FuzzyRange[ranges.size()]);
    }

    public static Range[] createRanges(double spread, double minInclusive, double maxInclusive) {

        List ranges = new ArrayList();

        for (double i = minInclusive; i <= maxInclusive;) {
            double rmax = i + spread;

            if (rmax > maxInclusive) {
                rmax = maxInclusive;
            }

            ranges.add(new SimpleRange(i, rmax));

            i += spread;
        }

        return (Range[]) ranges.toArray(new Range[ranges.size()]);
    }

    /**
     * Factory method:
     * For example: between 0 and 1000 makes 20 ranges.
     * Need to calc the range size.
     * Min and max are INclusive
     */
    public static FuzzyRange[] createRanges_fuzzy(int nranges, double minInclusive, double maxInclusive, double fuzz) {

        FuzzyRange[] ranges = new FuzzyRange[nranges];
        double spread = (maxInclusive - minInclusive) / nranges;
        double runningmin = minInclusive;

        for (int i = 0; i < nranges; i++) {
            ranges[i] = new FuzzyRange(runningmin, runningmin + spread, fuzz);
            runningmin += spread;
        }

        return ranges;
    }

    public static Range[] createRanges(int nranges, double minInclusive, double maxInclusive) {

        Range[] ranges = new Range[nranges];
        double spread = (maxInclusive - minInclusive) / nranges;
        double runningmin = minInclusive;

        for (int i = 0; i < nranges; i++) {
            ranges[i] = new SimpleRange(runningmin, runningmin + spread);
            runningmin += spread;
        }

        return ranges;
    }

    /**
     * format:
     * 1-200, 378-678 ...
     *
     * @param text
     * @return
     */
    public static Range[] createRanges(String text) {
        StringTokenizer tok = new StringTokenizer(text, " ,");

        List ranges = new ArrayList();

        while (tok.hasMoreTokens()) {
            String s = tok.nextToken();
            s = s.trim();

            StringTokenizer tok2 = new StringTokenizer(s, " -");
            //log.debug(s + " # tokens = " + tok2.countTokens());

            String one = tok2.nextToken().trim();
            String two = tok2.nextToken().trim();

            float min = Float.parseFloat(one);
            float max = Float.parseFloat(two);
            ranges.add(new SimpleRange(min, max));
        }

        return (Range[]) ranges.toArray(new Range[ranges.size()]);
    }

    /**
     * Factory method:
     * Simply iterates through the values[] array assigning density number of elements to
     * each Range.
     * The values array is not processed in any way.
     * Often it makes sense to process -- see matrix.discz
     * fuzz is not applicable so always 0.
     *
     * @throws IllegalArgumentException If (values.length % density != 0)
     */
    public static Range[] createRanges(int density, double[] values) {

        // dont -- see below
        //if ( (values.length % density) != 0) throw new IllegalArgumentException("Not exactly divisible, remainder: " + (values.length % density));
        Range[] ranges = new Range[(values.length / density)];

        /*
        int cnt = 0;
        for (int i=0; i < ranges.length; i+= density) {
            float[] dest = new float[density];
            System.arraycopy(values, i, dest, 0, density);
            Vector v = new Vector(dest);
            ranges[cnt++] = new Range(v.min(), v.max());
        }
        */
        int startpos = 0;

        for (int i = 0; i < ranges.length; i++) {
            float[] dest = new float[density];

            System.arraycopy(values, startpos, dest, 0, density);

            Vector v = new Vector(dest);

            ranges[i] = new SimpleRange(v.min(), v.max());
            startpos += density;
        }

        // expand the final range to include the last value it not exactly div by
        klog.debug("Expanding final range from: " + ranges[ranges.length - 1].getMax() + " to: "
                + values[values.length - 1]);

        ranges[ranges.length - 1] = new SimpleRange(ranges[ranges.length - 1].getMin(),
                values[values.length - 1]);

        /*
        int cnt = 0;
        for (int i=0; i < values.length / density; i+= density) {
            float[] dest = new float[density];
            System.arraycopy(values, i, dest, 0, density);
            Vector v = new Vector(dest);
            ranges[cnt++] = new Range(v.min(), v.max());
        }
        */

        /*
        Makes no sense to do this -- for equal density, must be exactly divisible
        dont delete
        for (int i=0; i < values.length;) {
            int maxindx = i + density - 1;
            if (maxindx > values.length) maxindx = values.length;
            Vector vector = new Vector(maxindx - i + 1); // not always the dnesity size
            for (int a=0; a < density; a++)
        */
        return ranges;
    }

    public static Range[] createRanges(int density, float[] values) {

        // dont -- see below
        //if ( (values.length % density) != 0) throw new IllegalArgumentException("Not exactly divisible, remainder: " + (values.length % density));
        Range[] ranges = new Range[(values.length / density)];

        int startpos = 0;

        for (int i = 0; i < ranges.length; i++) {
            float[] dest = new float[density];

            System.arraycopy(values, startpos, dest, 0, density);

            Vector v = new Vector(dest);

            ranges[i] = new SimpleRange(v.min(), v.max());
            startpos += density;
        }

        // expand the final range to include the last value it not exactly div by
        klog.debug("Expanding final range from: " + ranges[ranges.length - 1].getMax() + " to: "
                + values[values.length - 1]);

        ranges[ranges.length - 1] = new SimpleRange(ranges[ranges.length - 1].getMin(),
                values[values.length - 1]);

        return ranges;
    }

    // correlations go from neg one to pos one
    private static final float CORR_START = -1.0f;
    private static final float CORR_STOP = 1.0f;
    private static final float CORR_INCREMENT = 0.05f;

    public static Range[] createRangesForCorrelationsAndFill(final Vector values) {
        Range[] ranges = createRanges(CORR_START, CORR_STOP, CORR_INCREMENT);
        values.bin(ranges, true);
        RangeUtils.setImmutable(ranges);
        return ranges;
    }

    public static Range[] createRangesForZeroToOneAndFill(final Vector values, float increment) {
        Range[] ranges = createRanges(0, 1.0f, increment);
        values.bin(ranges, true);
        RangeUtils.setImmutable(ranges);
        return ranges;
    }

    // imp stop is like length so n not n-1
    public static SimpleRange[] createPans(final int fromIndex, final int toIndex,
                                           final int stepSize, final boolean alwaysEnsureToIndexIsIn) {

        int[] ints = RangeUtils.createStopIndices(fromIndex, toIndex, stepSize, alwaysEnsureToIndexIsIn);
        List ranges = new ArrayList(ints.length);
        for (int i = 0; i < ints.length; i++) {
            ranges.add(new SimpleRange(fromIndex, ints[i])); // @notre always starts from smae point (and hence called a pan)
        }

        return (SimpleRange[]) ranges.toArray(new SimpleRange[ranges.size()]);
    }

    public static SimpleRange[] createWindows(final int fromIndex, final int toIndex,
                                              final int windowSize, final int stepSize) {

        RangeUtils.checkRangeIndices(fromIndex, toIndex);

        List ranges = new ArrayList();

        int startIndex = fromIndex;

        boolean doMore = true;

        while (doMore) {
            int stopIndex = startIndex + windowSize;

            if (stopIndex > toIndex) {
                stopIndex = toIndex;
                doMore = false; // force this to be the last one
            }

            ranges.add(new SimpleRange(startIndex, stopIndex));

            startIndex = startIndex + stepSize;
            if (startIndex > toIndex) {
                break;
            }
        }

        return (SimpleRange[]) ranges.toArray(new SimpleRange[ranges.size()]);
    }


}
