/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.distrib;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Range;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.Dataframe;
import gnu.trove.TIntArrayList;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 */
public class RangeUtils {

    /**
     * ints between min and max, both included
     * Will not work well if the FuzzyRange is float based
     *
     * @return
     */
    public static int[] toIntArray(Range range) {
        int size = (int) range.getSpread();

        //log.debug(getSummary(range));
        int[] ints = new int[size];

        int pos = 0;
        for (int i = (int) range.getMin(); i <= (int) range.getMax(); i++) {
            ints[pos++] = i;
        }

        return ints;
    }

    public static void setImmutable(Range[] ranges) {
        if (ranges == null) {
            throw new IllegalArgumentException("Parameter ranges cannot be null");
        }

        for (int r = 0; r < ranges.length; r++) {
            ranges[r].setImmutable();
        }
    }

    /**
     * fromIndex -> low startpoint (INclusive)
     * toIndex - high endpoint (exclusive) -- ditto to list.subList.
     *
     * @param max
     * @param stepSize
     * @return
     */
    public static int[] createStopIndices(int fromIndex, int toIndex, int stepSize, boolean alwaysEnsureToIndexIsIn) {

        checkRangeIndices(fromIndex, toIndex);

        TIntArrayList ints = new TIntArrayList();

        int i = fromIndex;

        i = i + stepSize; // @note, no point starting from that index eg if from index 0, start with index 1

        while (true) {
            ints.add(i);
            i = i + stepSize;
            if (i > toIndex) {
                break;
            }
        }

        if (alwaysEnsureToIndexIsIn) {
            if (ints.contains(toIndex) == false) {
                ints.add(toIndex);
            }
        }

        return ints.toNativeArray();
    }


    private static String[] COL_NAMES = new String[]{"MAX", "MIN", "COUNT", "FRACTION"};

    // each row is a diff range
    public static Dataframe toDataframe(String name, Range[] ranges) {
        int totCnt = getTotalCount(ranges);
        List rowNames = new ArrayList(ranges.length);
        Matrix matrix = new Matrix(ranges.length, COL_NAMES.length);
        for (int r = 0; r < ranges.length; r++) {
            int coln = 0;
            rowNames.add(ranges[r].getName());
            matrix.setElement(r, coln++, (float) ranges[r].getMax());
            matrix.setElement(r, coln++, (float) ranges[r].getMin());
            matrix.setElement(r, coln++, ranges[r].getCount());
            matrix.setElement(r, coln, (float) ranges[r].getCount() / (float) totCnt);
        }

        return new Dataframe(name, matrix, rowNames, COL_NAMES, true, true);
    }

    public static int getTotalCount(Range[] ranges) {
        int tot = 0;
        for (int r = 0; r < ranges.length; r++) {
            tot += ranges[r].getCount();
        }

        return tot;
    }

    protected static void checkRangeIndices(int fromIndex, int toIndex) {

        if (fromIndex >= toIndex) {
            throw new IllegalArgumentException("fromIndex: " + fromIndex + " must be smaller than toIndex: " + toIndex);
        }

    }

    public static Vector toVector(Range[] sbins) {
        Vector vector = new Vector(sbins.length);
        for (int i = 0; i < sbins.length; i++) {
            vector.setElement(i, sbins[i].getCount());
        }

        return vector;
    }


    public static String[] toNames(Range[] sbins) {
        String[] names = new String[sbins.length];
        for (int i = 0; i < names.length; i++) {
            names[i] = sbins[i].getName();
        }

        return names;
    }


} // End RangeUtils
