/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.fdr;


import gnu.trove.TIntHashSet;

import java.util.StringTokenizer;


/**
 * Object represnting p-value cutoffs of various types.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class CutoffByPValue {

    private int fCutoffNperm;
    private float fLowerBound;
    private float fUpperBound;
    private final TIntHashSet fDoneIndices = new TIntHashSet();


    /**
     * Class constructor
     *
     * @param cutoffnperm
     * @param lowerbound
     * @param upperbound
     */
    public CutoffByPValue(float lowerbound, float upperbound, int cutoffnperm) {

        if (lowerbound > upperbound) {
            throw new IllegalArgumentException("Lower bound: " + lowerbound
                    + " cannot be less than upper bound: " + upperbound);
        }

        this.fCutoffNperm = cutoffnperm;
        this.fLowerBound = lowerbound;
        this.fUpperBound = upperbound;
    }

    /**
     * Class constructor
     *
     * @param cutoffnperm
     * @param lowerbound
     * @param upperbound
     */
    public CutoffByPValue(double lowerbound, double upperbound, int cutoffnperm) {
        this((float) lowerbound, (float) upperbound, cutoffnperm);
    }

    public boolean isDone(int currnperm, float currval) {
        return currnperm >= fCutoffNperm && isDone(currval);
    }

    public boolean isDone(float currval) {
        return (currval > fLowerBound) && (currval < fUpperBound);
    }

    public int getNPerm() {
        return fCutoffNperm;
    }

    public float getLowerBound() {
        return fLowerBound;
    }

    public float getUpperBound() {
        return fUpperBound;
    }

    public boolean addIndexIfDone(int index, float currval) {

        boolean add = isDone(currval);

        if (add) {
            fDoneIndices.add(index);
        }

        return add;
    }

    public boolean isDoneIndex(int index) {
        return fDoneIndices.contains(index);
    }

    public CutoffByPValue cloneShallow() {
        return new CutoffByPValue(this.getLowerBound(), this.getUpperBound(), this.getNPerm());
    }

    /**
     * Formatted as:
     * lower_pvalue,upper_pvale, nperm
     * all 3 fields are required
     *
     * @param cs
     * @return
     */
    public static CutoffByPValue createCutoff(String cs) {

        if (cs == null) {
            throw new IllegalArgumentException("Param cs cannot be null");
        }

        StringTokenizer tok = new StringTokenizer(cs, ", ");

        if (tok.countTokens() != 3) {
            throw new IllegalArgumentException("Bad format for cutoff String: " + cs
                    + " expected 3 tokens but got: "
                    + tok.countTokens());
        }

        float lower = Float.parseFloat(tok.nextToken());
        float upper = Float.parseFloat(tok.nextToken());
        int nperm = Integer.parseInt(tok.nextToken());

        return new CutoffByPValue(lower, upper, nperm);
    }
}    // End CutoffByPValue

/*--- Formatted in Sun Java Convention Style on Sat, Nov 9, '02 ---*/
