/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.fdr;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.Range;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.LabelledVector;
import gnu.trove.TObjectIntHashMap;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Aug 3, 2004
 *         Time: 10:13:34 PM
 */
public class Distrib {

    private static final Logger klog = XLogger.getLogger(Distrib.class);

    public static LabelledVector frequencyOfFeatureNames(Distrib.Point[] points) {
        _checkPoints(points);

        TObjectIntHashMap map = new TObjectIntHashMap();

        for (int c = 0; c < points.length; c++) {
            String featureName = points[c].getFeatureName();
            int curr = map.get(featureName);
            curr++;
            map.put(featureName, curr);
        }

        return new LabelledVector(map);
    }

    public static LabelledVector frequencyOfPermNames(Distrib.Point[] points) {

        _checkPoints(points);

        TObjectIntHashMap map = new TObjectIntHashMap();

        for (int c = 0; c < points.length; c++) {
            String colName = points[c].getPermName();
            int curr = map.get(colName);
            curr++;
            map.put(colName, curr);
        }

        return new LabelledVector(map);
    }


    public static Point[] sortDescending(Point[] points) {

        _checkPoints(points);

        List list = toList(points);
        Collections.sort(list, new PointScoreComparator());
        return (Point[]) list.toArray(new Point[list.size()]);
    }


    private static void _checkPoints(Point[] points) {
        if (points == null) {
            throw new IllegalArgumentException("Param points cannot be null");
        }

        if (points.length == 0) {
            klog.warn("Zero length Points: " + points.length);
        }

    }

    public static List toList(Point[] points) {
        _checkPoints(points);

        List list = new ArrayList();
        for (int i = 0; i < points.length; i++) {
            list.add(points[i]);
        }

        return list;
    }

    public static Point[] toArray(List pointsList) {
        return (Point[]) pointsList.toArray(new Point[pointsList.size()]);
    }

    public static Point[] createPoints(Dataset permNameByFeatureSetNameDataset) {

        List points = new ArrayList();
        for (int r = 0; r < permNameByFeatureSetNameDataset.getNumRow(); r++) {
            String permName = permNameByFeatureSetNameDataset.getRowName(r);
            for (int c = 0; c < permNameByFeatureSetNameDataset.getNumCol(); c++) {
                String gsetName = permNameByFeatureSetNameDataset.getColumnName(c);
                Point dp = new Point(permNameByFeatureSetNameDataset.getElement(r, c), permName, gsetName);
                points.add(dp);
            }
        }

        return (Point[]) points.toArray(new Point[points.size()]);
    }

    public static Distrib.Point[] createPoints(Dataset ds, float minScoreAllowedInclusive, float maxScoreAllowedInclusive) {

        List list = new ArrayList();
        for (int r = 0; r < ds.getNumRow(); r++) {
            for (int c = 0; c < ds.getNumCol(); c++) {
                float score = ds.getElement(r, c);
                if (score <= maxScoreAllowedInclusive && score >= minScoreAllowedInclusive) {
                    list.add(new Distrib.Point(score, ds.getRowName(r), ds.getColumnName(c)));
                }
            }
        }

        return (Distrib.Point[]) list.toArray(new Distrib.Point[list.size()]);
    }

    public static Distrib.Point[] createPointsGreaterThan(Dataset ds, float minScoreAllowedInclusive) {

        List list = new ArrayList();
        for (int r = 0; r < ds.getNumRow(); r++) {
            for (int c = 0; c < ds.getNumCol(); c++) {
                float score = ds.getElement(r, c);
                if (score >= minScoreAllowedInclusive) {
                    list.add(new Distrib.Point(score, ds.getRowName(r), ds.getColumnName(c)));
                }
            }
        }

        return (Distrib.Point[]) list.toArray(new Distrib.Point[list.size()]);
    }

    public static Distrib.Point[] createPointsLessThan(Dataset ds, float maxScoreAllowedInclusive) {

        List list = new ArrayList();
        for (int r = 0; r < ds.getNumRow(); r++) {
            for (int c = 0; c < ds.getNumCol(); c++) {
                float score = ds.getElement(r, c);
                if (score <= maxScoreAllowedInclusive) {
                    list.add(new Distrib.Point(score, ds.getRowName(r), ds.getColumnName(c)));
                }
            }
        }

        return (Distrib.Point[]) list.toArray(new Distrib.Point[list.size()]);
    }

    public static Distrib.Point[] createPoints(Dataset ds,
                                               float minScoreAllowedInclusive[],
                                               float maxScoreAllowedInclusive[],
                                               boolean cutoffsAreForEachRow) {

        if (cutoffsAreForEachRow) {
            if (ds.getNumRow() != minScoreAllowedInclusive.length) {
                throw new MismatchedSizeException("Dataset nrows", ds.getNumRow(), "min float array", minScoreAllowedInclusive.length);
            }

            if (ds.getNumRow() != maxScoreAllowedInclusive.length) {
                throw new MismatchedSizeException("Dataset nrows", ds.getNumRow(), "max float array", maxScoreAllowedInclusive.length);
            }
        } else {

            if (ds.getNumCol() != minScoreAllowedInclusive.length) {
                throw new MismatchedSizeException("Dataset ncols", ds.getNumCol(), "min float array", minScoreAllowedInclusive.length);
            }

            if (ds.getNumCol() != maxScoreAllowedInclusive.length) {
                throw new MismatchedSizeException("Dataset ncols", ds.getNumCol(), "max float array", maxScoreAllowedInclusive.length);
            }
        }

        List list = new ArrayList();
        for (int r = 0; r < ds.getNumRow(); r++) {
            for (int c = 0; c < ds.getNumCol(); c++) {
                float score = ds.getElement(r, c);
                float min;
                float max;
                if (cutoffsAreForEachRow) {
                    min = minScoreAllowedInclusive[r];
                    max = maxScoreAllowedInclusive[r];
                } else {
                    min = minScoreAllowedInclusive[c];
                    max = maxScoreAllowedInclusive[c];
                }

                //System.out.println("min: " + min + " max: " + max + " score: " + score);
                if (score <= max && score >= min) {
                    //System.out.println("min: " + min + " max: " + max + " score: " + score);
                    list.add(new Distrib.Point(score, ds.getRowName(r), ds.getColumnName(c)));
                }
            }
        }

        klog.debug("# added: " + list.size());

        return (Distrib.Point[]) list.toArray(new Distrib.Point[list.size()]);
    }

    public static Map binDataPointsByRange(Point[] points, Range[] ranges) {

        _checkPoints(points);


        Map rangePointsListMap = new HashMap();
        for (int i = 0; i < points.length; i++) {
            float value = points[i].getScore();
            for (int r = 0; r < ranges.length; r++) {
                Range range = ranges[r];
                if (range.isInRange(value)) {
                    Object currPoints = rangePointsListMap.get(range);
                    if (currPoints == null) {
                        currPoints = new ArrayList();
                    }
                    ((List) currPoints).add(points[i]);
                    rangePointsListMap.put(range, currPoints);
                }
            }
        }

        return rangePointsListMap;
    }

    public static Point[] getDataPointsMoreThan(float value, List sorted_points) {

        List points = new ArrayList();
        for (int i = 0; i < sorted_points.size(); i++) {
            Point point = (Point) sorted_points.get(i);
            if (value > point.getScore()) {
                break;
            } else {
                points.add(point);
            }
        }

        return (Point[]) points.toArray(new Point[points.size()]);
    }

    public static class PointScoreComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            Point item1 = (Point) pn1;
            Point item2 = (Point) pn2;
            float score1 = item1.getScore();
            float score2 = item2.getScore();

            if (score1 < score2) {
                return +1;
            } else {
                return -1;
            }

        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End PointComparator


    public static class Point {

        private String fPermName;
        private String fFeatureName;
        private float fScore;

        /**
         * Class constructor
         *
         * @param score
         * @param permName
         * @param featureName
         */
        public Point(float score, String permName, String featureName) {
            if (permName == null) {
                throw new IllegalArgumentException("Param permName cannot be null");
            }

            if (featureName == null) {
                throw new IllegalArgumentException("Param featureName cannot be null");
            }

            this.fFeatureName = featureName;
            this.fPermName = permName;
            this.fScore = score;
        }

        public float getScore() {
            return fScore;
        }

        public String getPermName() {
            return fPermName;
        }

        public String getFeatureName() {
            return fFeatureName;
        }

    } // End inner class Point

} // End class Distrib

