/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.fdr;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.RankedListGenerators;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.LabelledVector;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.strucs.DefaultFdrStruc;
import edu.mit.broad.genome.objects.strucs.FdrStruc;
import edu.mit.broad.genome.objects.strucs.SkewCorrectedFdrStruc;
import gnu.trove.THashMap;
import org.apache.log4j.Logger;

/**
 * False discovery rate algorithms.
 *
 * @author Aravind Subramanian
 *         Date: Oct 25, 2004
 *         Time: 11:28:39 PM
 */
public class FdrAlgs {

    private static final Logger klog = XLogger.getLogger(FdrAlgs.class);

    /**
     * @param real_scores
     * @param rnd_scores_by_row
     * @param real_scores_norm
     * @param all_rnd_scores_norm
     * @param sort
     * @param order
     * @return
     */
    public static FdrStruc[] calcFdrs(final LabelledVector real_scores,
            final Dataset rnd_scores_by_row,
            final LabelledVector real_scores_norm,
            final Vector all_rnd_scores_norm,
            final SortMode sort,
            final Order order) {
        // sanity checks
        if (real_scores == null) {
            throw new IllegalArgumentException("Param real_scores cannot be null");
        }

        if (rnd_scores_by_row == null) {
            throw new IllegalArgumentException("Param rnd_scores_by_row cannot be null");
        }

        if (real_scores.getSize() != rnd_scores_by_row.getNumRow()) {
            throw new MismatchedSizeException("real_scores", real_scores.getSize(), "rnd_scores_matrix", rnd_scores_by_row.getNumRow());
        }

        if (real_scores.getSize() != real_scores_norm.getSize()) {
            throw new MismatchedSizeException("real_scores", real_scores.getSize(), "real_scores_norm", real_scores_norm.getSize());
        }

        int dim = rnd_scores_by_row.getNumRow() * rnd_scores_by_row.getNumCol();
        if (all_rnd_scores_norm.getSize() != dim) {
            throw new MismatchedSizeException("all_rnd_scores_norm", all_rnd_scores_norm.getSize(), "matrix size", dim);
        }

        final RankedList real_scores_norm_sorted = real_scores_norm.sort(sort, order);
        final Vector all_rnd_scores_norm_sorted = new Vector(all_rnd_scores_norm, false);
        all_rnd_scores_norm_sorted.sort(sort, order);

        FdrStruc[] fdrs = new FdrStruc[real_scores.getSize()];
        for (int r = 0; r < real_scores.getSize(); r++) {
            String name = real_scores.getLabel(r);
            fdrs[r] = DefaultFdrStruc.create(name, real_scores, real_scores_norm_sorted,
                    rnd_scores_by_row,
                    all_rnd_scores_norm_sorted,
                    order);
        }

        return fdrs;
    }

    public static class FdrMap extends THashMap {

        FdrMap() {
            super();
        }

        public SkewCorrectedFdrStruc getFdr(final String name) {
            Object obj = get(name);
            if (obj == null) {
                throw new IllegalArgumentException("No fdr data for: " + name);
            } else {
                return (SkewCorrectedFdrStruc) obj;
            }
        }

        private void add(final String name, final SkewCorrectedFdrStruc fdr) {
            if (containsKey(name)) {
                throw new IllegalArgumentException("Already have fdr entry for: " + fdr);
            } else {
                put(name, fdr);
            }
        }
    }

    // @note IMP IMP this returns fdr only for those that are pos (or neg) and NOT for all
    public static FdrMap calcFdrs_skewed(final LabelledVector real_scores,
            final Dataset rnd_scores_by_row_FULL,
            final LabelledVector real_scores_norm,
            final DatasetModed all_rnd_scores_norm_moded_pos,
            final DatasetModed all_rnd_scores_norm_moded_neg,
            final SortMode sort) {

        FdrMap map = new FdrMap();

        _calcFdrs_skewed(real_scores, rnd_scores_by_row_FULL, real_scores_norm,
                all_rnd_scores_norm_moded_pos, all_rnd_scores_norm_moded_neg, sort, Order.DESCENDING, true, map);

        _calcFdrs_skewed(real_scores, rnd_scores_by_row_FULL, real_scores_norm,
                all_rnd_scores_norm_moded_pos, all_rnd_scores_norm_moded_neg, sort, Order.ASCENDING, false, map);

        return map;
    }

    // @note IMP IMP this returns fdr only for those that are pos (or neg) and NOT for all
    private static void _calcFdrs_skewed(final LabelledVector real_scores,
            final Dataset rnd_scores_by_row_FULL, // can have excess rows
            final LabelledVector real_scores_norm,
            final DatasetModed all_rnd_scores_norm_moded_pos,
            final DatasetModed all_rnd_scores_norm_moded_neg,
            final SortMode sort,
            final Order order,
            final boolean doPos,
            final FdrMap map) {

        // sanity checks
        if (real_scores == null) {
            throw new IllegalArgumentException("Param real_scores cannot be null");
        }

        if (rnd_scores_by_row_FULL == null) {
            throw new IllegalArgumentException("Param rnd_scores_by_row_FULL cannot be null");
        }

        /* dont check -- it can have excess
        if (real_scores.getSize() != rnd_scores_by_row.getNumRow()) {
        throw new MismatchedSizeException("real_scores", real_scores.getSize(), "rnd_scores_matrix", rnd_scores_by_row.getNumRow());
        }
         */

        if (real_scores.getSize() != real_scores_norm.getSize()) {
            throw new MismatchedSizeException("real_scores", real_scores.getSize(), "real_scores_norm", real_scores_norm.getSize());
        }

        final RankedList real_scores_norm_sorted = real_scores_norm.sort(sort, order);

        for (int r = 0; r < real_scores.getSize(); r++) {
            final String name = real_scores.getLabel(r);
            final float real_score_of_name = real_scores.getScore(r);

            if (doPos && XMath.isPositive(real_score_of_name)) {
                final SkewCorrectedFdrStruc fdr = new SkewCorrectedFdrStruc(name,
                        real_score_of_name,
                        real_scores_norm_sorted,
                        rnd_scores_by_row_FULL.getRow(name), // @note fetching by name, not index
                        all_rnd_scores_norm_moded_pos,
                        all_rnd_scores_norm_moded_neg,
                        doPos);
                map.add(name, fdr);
            } else if (!doPos && XMath.isNegative(real_score_of_name)) {
                if (real_score_of_name != 0) {
                    final SkewCorrectedFdrStruc fdr = new SkewCorrectedFdrStruc(name,
                            real_score_of_name,
                            real_scores_norm_sorted,
                            rnd_scores_by_row_FULL.getRow(name), // // @note fetching by name, not index
                            all_rnd_scores_norm_moded_pos,
                            all_rnd_scores_norm_moded_neg,
                            doPos);
                    map.add(name, fdr);
                }
            }
        }
    }

    /**
     * @param real_scores
     * @param rnd_scores_by_row
     * @param sort
     * @param order
     * @return
     */
    public static FdrStruc[] calcFdrs(final LabelledVector real_scores,
            final Dataset rnd_scores_by_row,
            final SortMode sort,
            final Order order) {

        if (real_scores == null) {
            throw new IllegalArgumentException("Param real_scores cannot be null");
        }

        if (rnd_scores_by_row == null) {
            throw new IllegalArgumentException("Param rnd_scores_by_row cannot be null");
        }

        if (real_scores.getSize() != rnd_scores_by_row.getNumRow()) {
            throw new MismatchedSizeException("real_scores", real_scores.getSize(), "rnd_scores_matrix", rnd_scores_by_row.getNumRow());
        }

        final RankedList real_scores_sorted = RankedListGenerators.sort(real_scores, sort, order);
        final Vector all_rnd_scores_sorted = rnd_scores_by_row.getMatrix().toVector();
        all_rnd_scores_sorted.sort(sort, order);

        final FdrStruc[] fdrs = new FdrStruc[real_scores.getSize()];
        for (int r = 0; r < real_scores.getSize(); r++) {
            final String name = real_scores.getLabel(r);
            final Vector rnd_scores_for_name_only = rnd_scores_by_row.getRow(name);
            fdrs[r] = DefaultFdrStruc.create(name, real_scores_sorted, rnd_scores_for_name_only, all_rnd_scores_sorted, order);
        }

        return fdrs;
    }
} // End class FdrAlgs

