package edu.mit.broad.genome.alg.filters;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.DefaultDataset;
import edu.mit.broad.vdb.sampledb.StateExpression;
import org.apache.log4j.Logger;
import xtools.api.param.FloatParam;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Oct 1, 2005
 *         Time: 4:57:02 PM
 */
public class DatasetFilters {


    private final Logger log = XLogger.getLogger(DatasetFilters.class);

    /**
     * The string that denotes that a probe is an affy control
     */
    public static final String AFFX = "AFFX-";

    /**
     * Class Constructor.
     */
    public DatasetFilters() {
    }

    // -------------------------------------------------------------------------------------------- //
    // ---------------------------------------- STATE FILTERS ------------------------------------ //
    // -------------------------------------------------------------------------------------------- //

    // The regular suite of 3 filters for by state

    public FilteredDataset filter_per_state(final String[] probeNames,
                                            final StateExpression[] sxs,
                                            final DefaultDataset ds_with_apm,
                                            final float minPCallFracPerState,
                                            final int minNumStatesThatMustPass,
                                            final float exprMoreThanThisValue,
                                            final float minGexFracPerState,
                                            final float minFoldChange_between_states,
                                            final float gexThreshold_floor,
                                            final float gexThreshold_ceiling,
                                            final boolean useMeanGex_for_each_class_not_median) {

        FilteredDataset fds_pcall = filterByPCalls_per_state(probeNames, sxs, ds_with_apm, minPCallFracPerState, minNumStatesThatMustPass);

        FilteredDataset fds_gex = filterByExpressionLevel_per_state(fds_pcall.getGeneSet(true).getMembersArray(), sxs, exprMoreThanThisValue, minGexFracPerState, minNumStatesThatMustPass);

        return filterByFoldChange_per_state(fds_gex.getGeneSet(true).getMembersArray(), sxs, minFoldChange_between_states, gexThreshold_floor, gexThreshold_ceiling, useMeanGex_for_each_class_not_median);
    }

    public FilteredDataset filterByPCalls_per_state(final String[] probeNames,
                                                    final StateExpression[] sxs,
                                                    final DefaultDataset ds_with_apm,
                                                    final float minPCallFracPerState,
                                                    final int minNumStatesThatMustPass) {


        List pass = new ArrayList();
        List fail = new ArrayList();

        for (int r = 0; r < probeNames.length; r++) {
            String rowName = probeNames[r];
            int numStatesThatHavePassed = 0;
            for (int s = 0; s < sxs.length; s++) {
                float frac = sxs[s].calcPCallFrac(ds_with_apm, rowName);
                if (frac >= minPCallFracPerState) {
                    numStatesThatHavePassed++;
                }
            }

            if (numStatesThatHavePassed < minNumStatesThatMustPass) {
                fail.add(rowName);
            } else {
                pass.add(rowName);
            }
        }

        return new FilteredDatasetImpl_only_genes("PCALL_FILTER", probeNames, pass, fail, "foo");
    }

    public FilteredDataset filterByExpressionLevel_per_state(
            final String[] probeNames,
            final StateExpression[] sxs,
            final float exprMoreThanThisValue,
            final float minGexFracPerState,
            final int minNumStatesThatMustPass) {

        List pass = new ArrayList();
        List fail = new ArrayList();

        for (int r = 0; r < probeNames.length; r++) {
            String rowName = probeNames[r];
            int numStatesThatHavePassed = 0;
            for (int s = 0; s < sxs.length; s++) {
                float frac = sxs[s].calcExpressionFrac(exprMoreThanThisValue, rowName);
                if (frac >= minGexFracPerState) {
                    numStatesThatHavePassed++;
                }
            }

            if (numStatesThatHavePassed < minNumStatesThatMustPass) {
                fail.add(rowName);
            } else {
                pass.add(rowName);
            }


        }

        return new FilteredDatasetImpl_only_genes("EXPR_FILTER", probeNames, pass, fail, "foo");
    }

    public FilteredDataset filterByFoldChange_per_state(
            final String[] probeNames,
            final StateExpression[] sxs,
            final float minFoldChange_between_states,
            final float gexThreshold_floor,
            final float gexThreshold_ceiling,
            final boolean useMeanGex_for_each_class_not_median) {

        List pass = new ArrayList();
        List fail = new ArrayList();

        for (int r = 0; r < probeNames.length; r++) {
            final String rowName = probeNames[r];
            final Vector v = new Vector(sxs.length);
            for (int s = 0; s < sxs.length; s++) {
                v.setElement(s, sxs[s].calcExpressionMean(rowName, useMeanGex_for_each_class_not_median));
            }

            float min = v.min();
            float max = v.max();

            if (min < gexThreshold_floor) {
                min = gexThreshold_floor;
            }

            if (max > gexThreshold_ceiling) {
                max = gexThreshold_ceiling;
            }

            float foldChange = max / min;

            if (foldChange < minFoldChange_between_states) {
                fail.add(rowName);
            } else {
                pass.add(rowName);
            }

        }

        return new FilteredDatasetImpl_only_genes("FOLD_CHANGE", probeNames, pass, fail, "foo");
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Dataset filterInStandardWayForClustering_instances(final Dataset ds,
                                                              final FloatParam fMinDeltaValueParam
    ) {

        return filterInStandardWayForClustering_instances(ds, fMinDeltaValueParam.getFValue());
    }

    public Dataset filterInStandardWayForClustering_instances(final Dataset ds,
                                                              final float minDeltaValue
    ) {


        List names2Include = new ArrayList();
        for (int r = 0; r < ds.getNumRow(); r++) {
            Vector v = ds.getRow(r);

            float max = v.max();
            float min = v.min();

            float diff;

            if (max == 0 && min == 0) {
                continue;
            } else if (XMath.isPositiveBoth(max, min)) {
                diff = Math.abs(max - min);
            } else if (XMath.isNegativeBoth(max, min)) {
                max = Math.abs(max);
                min = Math.abs(min);
                diff = Math.abs(max - min);
            } else {

                if (XMath.isPositive(max) && XMath.isNegative(min)) {
                    diff = max - min;
                } else {
                    throw new IllegalStateException("Huh: max: " + max + " min: " + min);
                }

            }

            if (diff < 0) {
                throw new IllegalStateException("Huh: max: " + max + " min: " + min);
            }

            if (diff < minDeltaValue) {
                continue;
            }

            /*
            if ((max / min) < minFoldValue) {
                continue;
            }
            */

            names2Include.add(ds.getRowName(r));
        }

        log.info("Before filterInStandardWayForClustering_instances: " + ds.getNumRow() + " after: " + names2Include.size());


        String name = ds.getName() + "_n" + names2Include.size() + "features" + "_d" + minDeltaValue;
        return new DatasetGenerators().extractRows(name, ds, names2Include);
    }

    public Dataset filterInStandardWayForClustering(final Dataset ds,
                                                    final FloatParam fMinGexValueParam,
                                                    final FloatParam fMaxGexValueParam,
                                                    final FloatParam fMinFoldValueParam,
                                                    final FloatParam fMinDeltaValueParam
    ) {

        return filterInStandardWayForClustering(ds,
                fMinGexValueParam.getFValue(),
                fMaxGexValueParam.getFValue(),
                fMinFoldValueParam.getFValue(),
                fMinDeltaValueParam.getFValue());
    }

    /*
    threshold 20, 4) fold change > 3 and delta of 100
    */
    public Dataset filterInStandardWayForClustering(final Dataset ds) {
        return filterInStandardWayForClustering(ds, 20, 16000, 3, 100);
    }

    /**
     * Class constructor
     *
     * @param ds
     * @param minGexValue
     * @param maxGexValue
     * @param minFoldValue
     * @param minDeltaValue
     * @return
     */
    public Dataset filterInStandardWayForClustering(final Dataset ds,
                                                    final float minGexValue,
                                                    final float maxGexValue,
                                                    final float minFoldValue,
                                                    final float minDeltaValue
    ) {


        final Matrix m = ds.getMatrix().cloneDeep();

        m.threshold(minGexValue, maxGexValue);

        final Dataset newDs = new DefaultDataset("foo", m, ds.getRowNamesArray(), ds.getColumnNamesArray(), true, ds.getAnnot());

        List names2Include = new ArrayList();
        for (int r = 0; r < newDs.getNumRow(); r++) {
            Vector v = newDs.getRow(r);
            if (v.max() - v.min() < minDeltaValue) {
                continue;
            }

            if (v.max() / v.min() < minFoldValue) {
                continue;
            }

            names2Include.add(newDs.getRowName(r));
        }

        log.info("Before filterInStandardWayForClustering: " + ds.getNumRow() + " after: " + names2Include.size());


        String name = ds.getName() + "_n" + names2Include.size() + "features_t" + minGexValue + "_" + maxGexValue +
                "_f" + minFoldValue + "_d" + minDeltaValue;
        return new DatasetGenerators().extractRows(name, newDs, names2Include);
    }

    /**
     * if exclude -> then affy probes are removed
     * else ONLY affy probes are used
     */
    public FilteredDataset removeAffyProbes(final Dataset ds) {
        return new FilteredDatasetImpl("affx", ds, removeAffyProbes(ds.getRowNamesArray()), null);
    }

    public String[] removeAffyProbes(final String[] probeSetNames) {

        List passing = new ArrayList();
        for (int r = 0; r < probeSetNames.length; r++) {

            String uc = probeSetNames[r].toUpperCase();

            boolean isAffy = false;
            if (uc.startsWith("AFFX") || uc.startsWith("RAE") || uc.startsWith("HC-") || uc.startsWith("MC_") || uc.startsWith("MC-")) {
                isAffy = true;
            }

            if (!isAffy) {
                passing.add(probeSetNames[r]);
            }
        }

        return (String[]) passing.toArray(new String[passing.size()]);
    }

    public FilteredDataset removeFeaturesIfMaxLessThan(final Dataset ds, final float value) {

        List passing = new ArrayList();
        for (int r = 0; r < ds.getNumRow(); r++) {
            Vector v = ds.getRow(r);
            if (v.max() > value) {
                passing.add(ds.getRowName(r));
            }
        }

        return new FilteredDatasetImpl(ds.getName() + "_value" + value, ds, passing, null);
    }

} // End class DatasetFilters