/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import org.apache.log4j.Logger;

/**
 * @author Aravind Subramanian
 *         Date: Nov 11, 2004
 *         Time: 1:53:11 PM
 */
public class DefaultGeneSetCohort extends AbstractGeneSetCohort {

    private boolean fSilent;

    private GeneSetScoringTable[] fGeneSetScoringTables;

    private GeneSetScoringTable fOrigTable;

    /**
     * Class constructor
     *
     * @param rl
     * @param gsets
     * @param normForSize
     * @param silent
     */
    public DefaultGeneSetCohort(final GeneSetScoringTable scoringTable,
                                final RankedList rl,
                                final GeneSet[] gsets,
                                final boolean silent) {
        this(scoringTable, rl, gsets, GeneToGeneSetMap.generateGeneToGenesetMap(gsets), silent, false);
    }

    public DefaultGeneSetCohort(final GeneSetScoringTable scoringTable,
                                final RankedList rl,
                                final GeneSet[] gsets,
                                final GeneToGeneSetMap g2gsetmap,
                                final boolean silent) {
        this(scoringTable, rl, gsets, g2gsetmap, silent, false);
    }

    /**
     * @param rl
     * @param gsets
     * @param mode
     * @param silent
     * @param qualifySetsAgainstRankedList
     */
    private DefaultGeneSetCohort(final GeneSetScoringTable scoringTable,
                                 final RankedList rl,
                                 final GeneSet[] gsets,
                                 final GeneToGeneSetMap g2gsetmap,
                                 final boolean silent,
                                 final boolean qualifySetsAgainstRankedList) {

        super.init(rl, gsets, g2gsetmap, silent, qualifySetsAgainstRankedList);

        this.fSilent = silent;
        this.fOrigTable = scoringTable;
        this.fGeneSetScoringTables = new GeneSetScoringTable[fGeneSets.length];

        // populate the table
        for (int g = 0; g < fGeneSets.length; g++) { // @note IMP to use the fgsets and it might have gotten clones
            // compute scoring scheme
            fGeneSetScoringTables[g] = scoringTable.createTable(fGeneSets[g], rl, fOrigTable.getRankedList());
            /*if (!silent && g >= 500 && g % 500 == 0) {
                System.out.println("GeneSetCohorted_scored: " + (g + 1) + " / " + fGeneSets.length);
            }*/
        }

    }

    public GeneSetCohort clone(final GeneSet[] gsets, final boolean qualifySetsAgainstRankedList) {
        return new DefaultGeneSetCohort(fOrigTable, getRankedList(), gsets, GeneToGeneSetMap.generateGeneToGenesetMap(gsets), fSilent,
                qualifySetsAgainstRankedList);
    }

    public double getHitPoints(final int gsetNum, final String geneName) {
        return fGeneSetScoringTables[gsetNum].getHitScore(geneName);
    }

    public double getHitPoints_gsaasnp(final int gsetNum, final String geneName) {
        return fGeneSetScoringTables[gsetNum].getHitScore_gsaasnp(geneName);
    }

    public double getHitPoints_ssea(final int gsetNum, final String geneName) {
        return fGeneSetScoringTables[gsetNum].getHitScoreP2(geneName);
    }

    public double getMissPoints(final int gsetNum, final String geneName) {
        return fGeneSetScoringTables[gsetNum].getMissScore(geneName);
    }

    public double getMissPoints_native(final int gsetNum, final String geneName){
        return fGeneSetScoringTables[gsetNum].getMissScore_native(geneName);
    }

    /**
     * Internal class
     */
    public static class Generator implements GeneSetCohortGenerator {

        private Logger log = XLogger.getLogger(Generator.class);

        private boolean fSilent;
        private GeneSetScoringTable fOrigTable;

        public boolean isTwoSidedEs() {
            return fOrigTable.isTwoSidedEs();
        }

        public GeneSetScoringTable getGeneSetScoringTable() {
            return fOrigTable;
        }

        public Generator(final GeneSetScoringTable scoringTable, final boolean silent) {
            this.fOrigTable = scoringTable;
            this.fSilent = silent;
        }

        public GeneSetCohort createGeneSetCohort(final RankedList rl,
                                                 final GeneSet[] gsets,
                                                 final boolean qualifySetsAgainstRankedList,
                                                 final boolean realRl) {

            GeneToGeneSetMap g2gsetmap = GeneToGeneSetMap.generateGeneToGenesetMap(gsets);
            return createGeneSetCohort(rl, gsets, g2gsetmap, qualifySetsAgainstRankedList, realRl);
        }

        public GeneSetCohort createGeneSetCohort(final RankedList rl,
                                                 final GeneSet[] gsets,
                                                 final GeneToGeneSetMap g2gsetMap,
                                                 final boolean qualifySetsAgainstRankedList,
                                                 final boolean realRl) { // @todo consider the real stuff

            if (realRl && fOrigTable instanceof GeneSetScoringTables.WeightedDoubleSidedAs) {
                log.warn("### SETTING REAL RL: " + rl.getName());
                ((GeneSetScoringTables.WeightedDoubleSidedAs) fOrigTable).setReal(rl);
            }

            if (realRl && fOrigTable instanceof GeneSetScoringTables.WeightedDoubleSidedAs1) {
                log.warn("### SETTING REAL RL: " + rl.getName());
                ((GeneSetScoringTables.WeightedDoubleSidedAs1) fOrigTable).setReal(rl);
            }

            return new DefaultGeneSetCohort(fOrigTable,
                    rl,
                    gsets,
                    g2gsetMap,
                    fSilent,
                    qualifySetsAgainstRankedList);
        }

        public GeneSetCohort createGeneSetCohort(final RankedList rl,
                                                 final GeneSet gset,
                                                 final boolean qualifySetAgainstRankedList,
                                                 final boolean realRl) {

            return createGeneSetCohort(rl,
                    new GeneSet[]{gset},
                    null,
                    qualifySetAgainstRankedList,
                    realRl);
        }

    } // End class Generator


} // End class DefaultWeightedGeneSetCohort
