/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.*;
import gnu.trove.THashMap;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Nov 27, 2003
 *         Time: 1:09:14 PM
 */
public class DefaultGseaSignalMatrix extends AbstractObject implements SignalMatrix {

    private List fSignals;
    private THashMap fNameSignalMap;

    private GeneSetMatrix fGeneSetMatrixSignals;

    private GeneSetMatrix fGeneSetMatrixOrig;

    /**
     * Data is NOT duplicated - used as is
     * <p/>
     * Class constructor
     *
     * @param name
     * @param signals
     */
    public DefaultGseaSignalMatrix(final String name,
                                   final GeneSetSignal[] signals,
                                   final GeneSetMatrix origQualGm) {
        initHere(name, signals, origQualGm);
    }

    public DefaultGseaSignalMatrix(final String comboName,
                                   final SignalMatrix[] signMatrices,
                                   final boolean pickLargestSignalForCommonOnes,
                                   final GeneSet[] origQualGeneSets) {
        this(comboName, signMatrices, pickLargestSignalForCommonOnes, new DefaultGeneSetMatrix(comboName, origQualGeneSets));
    }

    /**
     * Class constructor
     *
     * @param comboName
     * @param signMatrices
     * @param pickLargestSignalForCommonOnes
     */
    public DefaultGseaSignalMatrix(final String comboName,
                                   final SignalMatrix[] signMatrices,
                                   final boolean pickLargestSignalForCommonOnes,
                                   final GeneSetMatrix origQualGm) {


        if (pickLargestSignalForCommonOnes == false) {
            throw new NotImplementedException(); // @todo
        }

        // key -> sign name, value -> list of signals by that name
        Map gsetNameSignListMap = new HashMap();

        for (int m = 0; m < signMatrices.length; m++) {
            GeneSetSignal[] signs = signMatrices[m].getSignals();
            for (int s = 0; s < signs.length; s++) {
                Object curr = gsetNameSignListMap.get(signs[s].getName());
                if (curr == null) {
                    curr = new ArrayList();
                }
                ((List) curr).add(signs[s]);
                gsetNameSignListMap.put(signs[s].getName(), curr);
            }
        }

        // now troll through and one (the largest, if there are more than one) for every signal name
        List allSignsList = new ArrayList();
        for (Iterator it = gsetNameSignListMap.keySet().iterator(); it.hasNext();) {
            String signName = it.next().toString();
            List signs = (List) gsetNameSignListMap.get(signName);
            GeneSetSignal maxSign = (GeneSetSignal) signs.get(0);
            for (int i = 0; i < signs.size(); i++) {
                GeneSetSignal curr = (GeneSetSignal) signs.get(i);
                if (curr.getSize() > maxSign.getSize()) {
                    maxSign = curr;
                }
            }

            allSignsList.add(maxSign);
        }
        // sanity check
        if (allSignsList.size() != gsetNameSignListMap.size()) {
            throw new IllegalStateException("Some signals went missing/over-added: " + allSignsList.size() + " " + gsetNameSignListMap.size());
        }

        GeneSetSignal[] allSigns = new GeneSetSignal[allSignsList.size()];
        for (int i = 0; i < allSignsList.size(); i++) {
            allSigns[i] = (GeneSetSignal) allSignsList.get(i);
        }

        initHere(comboName, allSigns, origQualGm);
    }

    public GeneSetMatrix cloneShallow(final String newName) {
        this.setName(newName);
        return this;
    }

    // common init'ing routine here
    // origQualGm must be qualified to just the dataset labels
    private void initHere(final String name, final GeneSetSignal[] signals, final GeneSetMatrix origQualGm) {
        super.initialize(name);

        if (signals.length != origQualGm.getNumGeneSets()) {
            throw new MismatchedSizeException("signals", signals.length, "orig gene sets", origQualGm.getNumGeneSets());
        }

        GeneSet[] gsets = new GeneSet[signals.length];
        this.fSignals = new ArrayList(signals.length);
        this.fNameSignalMap = new THashMap();
        for (int i = 0; i < signals.length; i++) {
            String signalName = signals[i].getName();

            // a check to make sure the orig contains the signal
            GeneSet orig = origQualGm.getGeneSet(signalName);
            if (orig.getNumMembers() < signals[i].getSize()) {
                throw new IllegalStateException("Orig " + orig.getName() + " is smaller than signal! " + signals[i].getSize() + " " + signals[i].getName());
            }

            // @note not doing the more expensive check of making sure members match

            gsets[i] = signals[i].getAsGeneSet();
            fSignals.add(signals[i]);
            fNameSignalMap.put(signals[i].getName(), signals[i]);
        }

        // for the forwarding methods
        this.fGeneSetMatrixSignals = new DefaultGeneSetMatrix(name, gsets);
        this.fGeneSetMatrixOrig = origQualGm;

    }

    public boolean containsSet(final String gsetName) {
        throw new NotImplementedException(); // @todo
    }

    public String getQuickInfo() {
        StringBuffer buf = new StringBuffer().append(getNumSignals()).append(" signs");
        return buf.toString();
    }

    public GeneSetMatrix getOriginalGeneSetMatrix() {
        return fGeneSetMatrixOrig;
    }

    public GeneSet getOriginalGeneSet(final String gsetName) {
        return fGeneSetMatrixOrig.getGeneSet(gsetName);
    }

    public int getNumSignals() {
        return fSignals.size();
    }

    public GeneSetSignal getSignal(final int s) {
        return (GeneSetSignal) fSignals.get(s);
    }


    public GeneSetSignal getSignal(final String signalName) {
        Object obj = fNameSignalMap.get(signalName);
        if (obj == null) {
            throw new IllegalArgumentException("No Signal found for name: " + signalName);
        }

        return (GeneSetSignal) obj;
    }

    /**
     * All genesets
     * directly -- no cloning
     *
     * @return
     */
    public GeneSetSignal[] getSignals() {
        return (GeneSetSignal[]) fSignals.toArray(new GeneSetSignal[fSignals.size()]);
    }

    public List getSignalsL() {
        return Collections.unmodifiableList(fSignals);
    }

    public GeneSetSignal[] getSignals(Set gsetNames) {
        GeneSetSignal[] signs = new GeneSetSignal[gsetNames.size()];
        int cnt = 0;
        for (Iterator it = gsetNames.iterator(); it.hasNext();) {
            String name = it.next().toString();
            signs[cnt++] = getSignal(name);
        }

        return signs;
    }

    // ------------ FORWARDING METHODS ------------------
    public int getNumGeneSets() {
        return getNumSignals();
    }

    public Dataset createTagMatrix(boolean genesAsRows, boolean bufferzeros) {
        return fGeneSetMatrixSignals.createTagMatrix(genesAsRows, bufferzeros);
    }

    public GeneSet getGeneSet(int i) {
        return fGeneSetMatrixSignals.getGeneSet(i);
    }

    public GeneSet getGeneSet(String gsetName) {
        return fGeneSetMatrixSignals.getGeneSet(gsetName);
    }

    public GeneSet[] getGeneSets() {
        return fGeneSetMatrixSignals.getGeneSets();
    }

    public List getGeneSetsL() {
        return fGeneSetMatrixSignals.getGeneSetsL();
    }

    public GeneSet[] getGeneSets(int minSizeInclusive, int maxSizeInclusive) {
        return fGeneSetMatrixSignals.getGeneSets(minSizeInclusive, maxSizeInclusive);
    }

    public List getGeneSetsL(int minSizeInclusive, int maxSizeInclusive) {
        return fGeneSetMatrixSignals.getGeneSetsL(minSizeInclusive, maxSizeInclusive);
    }

    public GeneSet[] getGeneSets(Set gsetNames) {
        return fGeneSetMatrixSignals.getGeneSets(gsetNames);
    }

    public GeneSetMatrix getGeneSetMatrix(final GeneSet gsetNames) {
        return fGeneSetMatrixSignals.getGeneSetMatrix(gsetNames);
    }

    public GeneSet[] getGeneSets(String[] gsetNames) {
        return fGeneSetMatrixSignals.getGeneSets(gsetNames);
    }

    public String[] getGeneSetNames(boolean stripAux) {
        return fGeneSetMatrixSignals.getGeneSetNames(stripAux);
    }

    public List getGeneSetNamesL(boolean stripAux) {
        return fGeneSetMatrixSignals.getGeneSetNamesL(stripAux);
    }

    public Set getGeneSetNamesS(boolean stripAux) {
        return fGeneSetMatrixSignals.getGeneSetNamesS(stripAux);
    }

    public int getMaxGeneSetSize() {
        return fGeneSetMatrixSignals.getMaxGeneSetSize();
    }

    public String[] getAllMemberNames() {
        return fGeneSetMatrixSignals.getAllMemberNames();
    }

    public List getAllMemberNamesL() {
        return fGeneSetMatrixSignals.getAllMemberNamesL();
    }

    public String[] getAllMemberNamesOnlyOnce() {
        return fGeneSetMatrixSignals.getAllMemberNamesOnlyOnce();
    }

    public Set getAllMemberNamesOnlyOnceS() {
        return fGeneSetMatrixSignals.getAllMemberNamesOnlyOnceS();
    }

    public String[] getAllGeneSetNames() {
        return fGeneSetMatrixSignals.getAllMemberNames();
    }

    public GeneSet getAllMemberNames_gset() {
        return new FSet(getName(), getAllMemberNamesOnlyOnceS());
    }

    public BitSet getGeneSetMembershipBitSet(String probeName) {
        return fGeneSetMatrixSignals.getGeneSetMembershipBitSet(probeName);
    }

    public BitSetDataset getGeneSetMembershipBitSetDataset(String name, String[] probeNames) {
        return fGeneSetMatrixSignals.getGeneSetMembershipBitSetDataset(name, probeNames);
    }

    public BitSetDataset getGeneSetMembershipBitSetDataset() {
        return fGeneSetMatrixSignals.getGeneSetMembershipBitSetDataset();
    }

    public String[] getGeneSetMembershipNames(String probeName) {
        return fGeneSetMatrixSignals.getGeneSetMembershipNames(probeName);
    }

    public GeneSet[] getGeneSetMembershipSets(String probeName) {
        return fGeneSetMatrixSignals.getGeneSetMembershipSets(probeName);
    }

    public String getGeneSetName(int i) {
        return fGeneSetMatrixSignals.getGeneSetName(i);
    }

} // End class DefaultGseaSignalMatrix
