/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.Errors;
import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.AlgUtils;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import edu.mit.broad.vdb.sampledb.Sample;
import org.apache.log4j.Logger;

import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Jul 19, 2003
 *         Time: 4:22:59 PM
 */
public class EdbAlgs {

    private static final Logger klog = XLogger.getLogger(EdbAlgs.class);

    /**
     * Class constructor
     * Does nothing
     */
    protected EdbAlgs() {
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------LEADING EDGE RELATED METHODS---------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public static Dataset createGeneSetMembershipDataset(final String name,
                                                         final EnrichmentResult[] results,
                                                         final boolean onlySignal) {

        _nonNull(results);

        final BitSetDataset bsd = new BitSetDataset(toGeneSetMatrix(name, results, onlySignal));
        return bsd.toDataset(true, false);
    }

    public static Dataset createGeneSetMembershipDataset(final String name,
                                                         final EnrichmentResult[] results,
                                                         final RankedList rl,
                                                         final boolean rankNotScores,
                                                         final boolean onlySignal) {
        _nonNull(results);
        final BitSetDataset bsd = new BitSetDataset(toGeneSetMatrix(name, results, onlySignal));
        return bsd.toDataset(rl, rankNotScores, false);
    }

    public static GeneSetMatrix toGeneSetMatrix(final EnrichmentDb edb,
                                                final boolean onlySignal) {
        return toGeneSetMatrix(edb.getName(), edb.getResults(), onlySignal);
    }

    public static GeneSetMatrix toGeneSetMatrix(final String name,
                                                final EnrichmentResult[] results,
                                                final boolean onlySignal) {

        _nonNull(results);

        GeneSet[] gsets = new GeneSet[results.length];
        for (int i = 0; i < results.length; i++) {
            if (onlySignal) {
                gsets[i] = results[i].getSignal().getAsGeneSet();
            } else {
                gsets[i] = results[i].getGeneSet();
            }
        }

        return new DefaultGeneSetMatrix(name, gsets);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public static LabelledVector createRealES(final EnrichmentResult[] results) {
        _nonNull(results);

        Vector v = new Vector(results.length);
        List labels = new ArrayList();
        for (int i = 0; i < results.length; i++) {
            labels.add(results[i].getGeneSetName());
            v.setElement(i, results[i].getScore().getES());
        }

        return new LabelledVector(labels, v, true);
    }

    public static Dataset createRndESDataset(final String name, final EnrichmentResult[] results) {

        _nonNull(results);

        // each gset on a row, cols are perms
        int numPerms = enforceSameNumOfPerms(results);
        Matrix m = new Matrix(results.length, numPerms);

        List rowNames = new ArrayList(results.length);

        for (int r = 0; r < results.length; r++) {
            Vector v = results[r].getRndESS();
            m.setRow(r, v);
            rowNames.add(results[r].getGeneSetName());
        }

        return new DefaultDataset(name + "_rnd_es", m, rowNames, _permColNames(numPerms), true, null);
    }


    public static LabelledVector createNumHits(final EnrichmentDb edb) {
        return createNumHits(edb.getResults());
    }

    public static LabelledVector createNumHits(final EnrichmentResult[] results) {
        _nonNull(results);

        Vector v = new Vector(results.length);
        List labels = new ArrayList();
        for (int i = 0; i < results.length; i++) {
            labels.add(results[i].getGeneSetName());
            v.setElement(i, results[i].getScore().getNumHits());
        }

        return new LabelledVector(labels, v, true);
    }

    public static GeneSet[] getGeneSets(final EnrichmentResult[] results) {
        _nonNull(results);

        final GeneSet[] gsets = new GeneSet[results.length];
        for (int r = 0; r < results.length; r++) {
            gsets[r] = results[r].getGeneSet();
            //gsets[r] = gsets[r].cloneShallow(gsets[r].getName(true));
        }

        return gsets;
    }

    public static GeneSet[] getGeneSetSignals(final EnrichmentResult[] results) {
        _nonNull(results);

        final GeneSet[] gsets = new GeneSet[results.length];
        for (int r = 0; r < results.length; r++) {
            gsets[r] = results[r].getSignal().getAsGeneSet();
        }

        return gsets;
    }

    // -------------------------------------------------------------------------------------------- //
    // -----------------------------------HASHING METHODS ----------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    private static void _nonNull(final EnrichmentResult[] results) {
        if (results == null || results.length == 0) {
            throw new IllegalArgumentException("Param results cannot be null nor zero length: " + results);
        }
    }


    /**
     * key -> gsetName
     * value -> a single er (barfs if there are duplicates)
     *
     * @param results
     * @return
     */
    public static Map hashByGeneSetName(final EnrichmentResult[] results) {

        _nonNull(results);

        final Map map = new HashMap();

        Errors errors = new Errors();
        for (int i = 0; i < results.length; i++) {
            String name = results[i].getGeneSet().getName(true);
            if (map.containsKey(name)) {
                errors.add("Duplicated gene set: " + name);
            } else {
                map.put(name, results[i]);
            }
        }


        errors.barfIfNotEmptyRuntime();

        return map;
    }

    /**
     * @param results
     * @return
     */
    public static Map hashByTemplateName(final EnrichmentResult[] results) {

        _nonNull(results);

        Map map = new HashMap();

        for (int i = 0; i < results.length; i++) {
            String name = results[i].getTemplate().getName();
            Object val = map.get(name);
            if (val == null) {
                val = new ArrayList();
            }
            ((List) val).add(results[i]);
            map.put(name, val);
        }

        return map;
    }

    public static Map hashByTemplate(final EnrichmentResult[] results) {
        _nonNull(results);

        Map map = new HashMap();

        for (int i = 0; i < results.length; i++) {
            Template t = results[i].getTemplate();
            Object val = map.get(t);
            if (val == null) {
                val = new ArrayList();
            }
            ((List) val).add(results[i]);
            map.put(t, val);
        }

        return map;
    }

    // -------------------------------------------------------------------------------------------- //
    // -----------------------------------FILTERING METHODS ----------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    /**
     * @param dtgs
     * @param selectOnlyThisGeneSet
     * @return
     */
    public static EnrichmentResult[] filterByGeneSetName(final EnrichmentResult[] dtgs,
                                                         final GeneSet selectOnlyThisGeneSet) {
        return filterByGeneSetName(dtgs, selectOnlyThisGeneSet.getName());
    }

    /**
     * @param dtgs
     * @param selectOnlyThisGeneSetName
     * @return
     */
    public static EnrichmentResult[] filterByGeneSetName(final EnrichmentResult[] dtgs,
                                                         final String selectOnlyThisGeneSetName) {

        List list = new ArrayList();

        for (int i = 0; i < dtgs.length; i++) {
            if (_equalsG(dtgs[i], selectOnlyThisGeneSetName)) {
                list.add(dtgs[i]);
            }
        }

        return (EnrichmentResult[]) list.toArray(new EnrichmentResult[list.size()]);
    }

    /**
     * @param dtgs
     * @param selectOnlyTheseGeneSets
     * @return
     */
    public static EnrichmentResult[] filterByGeneSetName(final EnrichmentResult[] dtgs,
                                                         final GeneSet[] selectOnlyTheseGeneSets) {

        Set names = AlgUtils.getAllPobNamesS(selectOnlyTheseGeneSets);
        List list = new ArrayList();

        for (int i = 0; i < dtgs.length; i++) {
            if (names.contains(dtgs[i].getGeneSet().getName())) {
                list.add(dtgs[i]);
            }
        }

        return (EnrichmentResult[]) list.toArray(new EnrichmentResult[list.size()]);
    }

    /**
     * @param results
     * @param selectOnlyThisTemplate
     * @return
     */
    public static EnrichmentResult[] filterByTemplateName(final EnrichmentResult[] results,
                                                          final Template selectOnlyThisTemplate) {
        return filterByTemplateName(results, selectOnlyThisTemplate.getName());
    }

    /**
     * @param results
     * @param selectOnlyThisTemplateName
     * @return
     */
    public static EnrichmentResult[] filterByTemplateName(final EnrichmentResult[] results,
                                                          final String selectOnlyThisTemplateName) {

        _nonNull(results);

        List list = new ArrayList();

        for (int i = 0; i < results.length; i++) {
            if (_equalsT(results[i], selectOnlyThisTemplateName)) {
                list.add(results[i]);
            }
        }

        return (EnrichmentResult[]) list.toArray(new EnrichmentResult[list.size()]);
    }

    /**
     * @param results
     * @param selectOnlyTheseTemplates
     * @return
     */
    public static EnrichmentResult[] filterByTemplateNames(final EnrichmentResult[] results,
                                                           final Template[] selectOnlyTheseTemplates) {
        _nonNull(results);

        Set names = AlgUtils.getTemplateNamesS(selectOnlyTheseTemplates);
        List list = new ArrayList();

        for (int i = 0; i < results.length; i++) {
            if (names.contains(results[i].getTemplate().getName())) {
                list.add(results[i]);
            }
        }

        return (EnrichmentResult[]) list.toArray(new EnrichmentResult[list.size()]);
    }

    public static EnrichmentResult[] filterByGeneSetSize(final EnrichmentResult[] results,
                                                         final int minSize,
                                                         final int maxSize,
                                                         final Dataset passthruDs) {

        _nonNull(results);

        List list = new ArrayList();
        for (int i = 0; i < results.length; i++) {
            int size = results[i].getGeneSet().getNumMembers(passthruDs);
            if ((size >= minSize) && (size <= maxSize)) {
                list.add(results[i]);
            }
        }

        return (EnrichmentResult[]) list.toArray(new EnrichmentResult[list.size()]);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public static List getGeneSetNames(final EnrichmentResult[] results) {

        _nonNull(results);

        List list = new ArrayList(results.length);

        for (int i = 0; i < results.length; i++) {
            list.add(results[i].getGeneSet().getName(true)); // note strip aux
        }

        return list;
    }

    public static List getGeneSetNames_unique(final EnrichmentResult[] results) {

        _nonNull(results);

        List list = new ArrayList(results.length);

        for (int i = 0; i < results.length; i++) {
            String name = results[i].getGeneSet().getName(true);  // note strip aux
            if (!list.contains(name)) {
                list.add(name);
            }
        }

        return list;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public static DtgIndex getAndEnforceTheOnlyDtgByGeneSetName(String gsetName, EnrichmentDb edb) {
        return getAndEnforceTheOnlyDtgByGeneSetName(gsetName, edb.getResults());
    }

    public static DtgIndex getAndEnforceTheOnlyDtgByGeneSetName(final String gsetName, final EnrichmentResult[] results) {
        _nonNull(results);

        Set set = new HashSet();
        for (int i = 0; i < results.length; i++) {
            if (results[i].getGeneSet().getName().equals(gsetName)) {
                set.add(new DtgIndex(results[i], i));
            }
        }

        if (set.isEmpty()) {
            throw new IllegalArgumentException("No Dtg by gset name: " + gsetName + " in this dtgs: " + results.length);
        } else if (set.size() > 1) {
            throw new IllegalArgumentException("More than 1 hit for specified gset .. expecting only 1, found" + set.size() + " Dtg by gset name: " + gsetName + " in this dtgs: " + results.length);
        } else {
            return (DtgIndex) set.iterator().next();
        }
    }

    static class DtgIndex {
        int index;
        EnrichmentResult dtg;

        DtgIndex(EnrichmentResult dtg, int index) {
            this.dtg = dtg;
            this.index = index;
        }
    }

    public static class EnrichmentResultMap extends HashMap {

        public EnrichmentResult getResult(String name) {
            Object obj = super.get(name);
            if (obj == null) {
                throw new IllegalArgumentException("No such enrichment result: " + name);
            } else {
                return (EnrichmentResult) obj;
            }
        }

    }

    public static EnrichmentResultMap createResultMap(final EnrichmentResult[] results) {
        _nonNull(results);

        final Errors errors = new Errors();

        EnrichmentResultMap nameEnResultMap = new EnrichmentResultMap();
        for (int i = 0; i < results.length; i++) {
            EnrichmentResult result = results[i];
            if (nameEnResultMap.containsKey(result.getGeneSetName())) {
                errors.add("duplicate name: " + result.getGeneSetName());
            }
            nameEnResultMap.put(result.getGeneSetName(), result);
        }

        return nameEnResultMap;
    }

    public static int enforceSameNumOfPerms(final EnrichmentResult[] results) {
        if (results.length == 0) {
            return 0;
        }

        int num = results[0].getNumPerms();
        for (int i = 0; i < results.length; i++) {

            if (results[i].getNumPerms() != num) {
                throw new MismatchedSizeException("enrichment result: " + results[0].getGeneSetName(), num, "enrichment result: " + results[i].getGeneSetName(), results[i].getNumPerms());
            }
        }

        return num;
    }


    public static float getRndVar(final EnrichmentResult result) {
        Vector v = result.getRndESS();
        return (float) Math.sqrt(v.var(false, false));
    }

    public static float getRndMean(final EnrichmentResult result) {
        return (float) result.getRndESS().mean();
    }

    public static float getNominalPValue(EnrichmentResult dtg) {
        return (float) XMath.getPValueTwoTailed(dtg.getScore().getES(), dtg.getRndESS());
    }


    /**
     * @param dtg
     * @param gsetName
     * @return
     * @note equality based on name and not object reference
     */
    protected static boolean _equalsG(final EnrichmentResult dtg, final String gsetName) {
        return dtg.getGeneSet().getName().equalsIgnoreCase(gsetName);
    }

    /**
     * @param dtg
     * @param gsetName
     * @return
     * @note equality based on name and not object reference
     */
    protected static boolean _equalsT(final EnrichmentResult dtg, final String tName) {
        return dtg.getGeneSet().getName().equalsIgnoreCase(tName);
    }

    protected static List _toList(EnrichmentResult[] dtgs) {
        List list = new ArrayList(dtgs.length);

        for (int i = 0; i < dtgs.length; i++) {
            list.add(dtgs[i]);
        }

        return list;
    }

    public static class BySample {

        private Sample Sample;

        private EnrichmentResult[] dtgs;

        BySample(Sample Sample, EnrichmentResult[] dtgs) {
            this.Sample = Sample;
            this.dtgs = dtgs;
        }

        BySample(Sample Sample, List dtgList) {
            this(Sample, (EnrichmentResult[]) dtgList.toArray(new EnrichmentResult[dtgList.size()]));
        }

        public Sample getSample() {
            return Sample;
        }

        // returns a safe copy thats is shallow  -> array is safe but contents are not
        public EnrichmentResult[] getDtgs() {
            EnrichmentResult[] d = new EnrichmentResult[dtgs.length];
            for (int i = 0; i < d.length; i++) {
                d[i] = dtgs[i];
            }

            return d;
        }

    }

    protected static class SampleDtg {
        Sample Sample;
        EnrichmentResult dtg;

        SampleDtg(Sample Sample, EnrichmentResult dtg) {
            this.Sample = Sample;
            this.dtg = dtg;
        }

        public Sample getSample() {
            return Sample;
        }

        public EnrichmentResult getDtg() {
            return dtg;
        }
    }

    public static class ByGeneSet {

        private GeneSet gset;

        private SampleDtg[] sampledtgs;

        ByGeneSet(GeneSet gset, SampleDtg[] sampledtgs) {
            this.gset = gset;
            this.sampledtgs = sampledtgs;
        }

        ByGeneSet(GeneSet gset, List sampleDtgList) {
            this(gset, (SampleDtg[]) sampleDtgList.toArray(new SampleDtg[sampleDtgList.size()]));
        }

        public GeneSet getGeneSet() {
            return gset;
        }

        // returns a safe copy that is shallow -> array is safe but contents are not
        public SampleDtg[] getSamplesDtgs() {
            SampleDtg[] ss = new SampleDtg[sampledtgs.length];
            for (int i = 0; i < ss.length; i++) {
                ss[i] = sampledtgs[i];
            }

            return ss;
        }

        public Sample[] getSamples() {
            Sample[] ss = new Sample[sampledtgs.length];
            for (int i = 0; i < ss.length; i++) {
                ss[i] = sampledtgs[i].getSample();
            }

            return ss;
        }
    }

    // places each BySample in a col by itself and adds coloring fluff
    public static StringDataframe createStringDataframe(String name, BySample[] bys) {

        int maxNumDtgs = 0;
        for (int i = 0; i < bys.length; i++) {
            if (maxNumDtgs < bys[i].getDtgs().length) {
                maxNumDtgs = bys[i].getDtgs().length;
            }
        }

        StringMatrix sm = new StringMatrix(maxNumDtgs, bys.length);
        List colNames = new ArrayList(bys.length);
        for (int c = 0; c < bys.length; c++) {
            colNames.add(bys[c].getSample().getClass().getName() + "_" + bys[c].getSample().getName());
            EnrichmentResult[] dtgs = bys[c].getDtgs();
            for (int r = 0; r < dtgs.length; r++) {
                sm.setElement(r, c, dtgs[r].getGeneSet().getName());
                if (XMath.isPositive(dtgs[r].getScore().getES())) {
                    sm.setColor(r, c, Color.RED);
                } else {
                    sm.setColor(r, c, Color.BLUE);
                }
            }
        }

        return new StringDataframe(name, sm, colNames, true);
    }

    public static StringDataframe createStringDataframe(String name, ByGeneSet[] byg) {
        int maxNumDtgs = 0;
        for (int i = 0; i < byg.length; i++) {
            if (maxNumDtgs < byg[i].getSamplesDtgs().length) {
                maxNumDtgs = byg[i].getSamplesDtgs().length;
            }
        }

        StringMatrix sm = new StringMatrix(maxNumDtgs, byg.length);
        List colNames = new ArrayList(byg.length);
        for (int c = 0; c < byg.length; c++) {
            colNames.add(byg[c].getGeneSet().getName());
            SampleDtg[] sampledtgs = byg[c].getSamplesDtgs();
            for (int r = 0; r < sampledtgs.length; r++) {
                sm.setElement(r, c, sampledtgs[r].getSample().getClass().getName() + "_" + sampledtgs[r].getSample().getName());
                if (XMath.isPositive(sampledtgs[r].getDtg().getScore().getES())) {
                    sm.setColor(r, c, Color.RED);
                } else {
                    sm.setColor(r, c, Color.BLUE);
                }
            }
        }

        return new StringDataframe(name, sm, colNames, true);
    }

    private static List _permColNames(int nperms) {
        List colNames = new ArrayList();
        for (int c = 0; c < nperms; c++) {
            colNames.add("perm_" + (c + 1));
        }

        return colNames;
    }

} // End EdbAlgs
