/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import edu.mit.broad.genome.math.BooleanVector;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentScore;
import edu.mit.broad.genome.objects.FSet;
import edu.duke.igsp.app.tools.ArrayListTool;
import gnu.trove.TFloatArrayList;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

/**
 * This is the core class that implements the Kolmogorov-Smirnov algorithm
 *
 * @author Aravind Subramanian
 *         Date: Mar 3, 2003
 *         Time: 9:37:58 PM
 * @revised by Qing Xiong <qxiong@email.unc.edu>
 */
public class KSCore {

    private final Logger log = XLogger.getLogger(this.getClass());

    /**
     * Class Constructor.
     * Stateless
     */
    public KSCore() {
    }

    // The common (gsea) way
    public EnrichmentScore[] calculateKSScore(final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes(gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }
    
    public EnrichmentScore[] calculateKSScore_gsaasnp(final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_snp(gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }    

    public EnrichmentScore[] calculateKSScore_seq_ks(final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_seq_ks(gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }

    public EnrichmentScore[] calculateKSScore_snp(final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_snp(gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }

    public EnrichmentScore[] calculateKSScore_parallel(final GeneSetCohort gcoh_exp, final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_parallel(gcoh_exp, gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }

    public EnrichmentScore[] calculateKSScore_fm(final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_fm(gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }

    public EnrichmentScore[] calculateKSScore_fm_parallel(final GeneSetCohort gcoh_exp, final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_fm_parallel(gcoh_exp, gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }

    public EnrichmentScore[] calculateKSScore_samgs(final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_samgs(gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }

    public EnrichmentScore[] calculateKSScore_samgs_parallel(final GeneSetCohort gcoh_exp, final GeneSetCohort gcoh,
            final boolean storeDeep) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_samgs_parallel(gcoh_exp, gcoh, storeDeep, true);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }

    public EnrichmentScore[] calculateKSScore(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes(gcoh, storeDeep, storeRESPointByPoint);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }
    
    public EnrichmentScore[] calculateKSScore_gsaasnp(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_snp(gcoh, storeDeep, storeRESPointByPoint);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }    

    public EnrichmentScore[] calculateKSScore_seq_ks(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        EnrichmentScoreCohort[] cohorts = calculateKSScore_all_modes_seq_ks(gcoh, storeDeep, storeRESPointByPoint);
        EnrichmentScore[] ess = new EnrichmentScore[cohorts.length];
        for (int i = 0; i < ess.length; i++) {
            ess[i] = cohorts[i].es_maxdev_style;
        }

        return ess;
    }

    // Justin Guinneys implementation
    public EnrichmentScoreCohort[] calculateKSScore_all_modes(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            hitCnt[g] = 0;
            genesetJumps[g] = -1;
        }

        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    double missPoints = gcoh.getMissPoints(g, rowName);
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    /*
                    if (i == 0) {
                    System.out.println("old runningScores[g]=" + runningScores[g] + " gap=" + gap);
                    }
                     */

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (isLastRun && !gcoh.isMember(i, rowName)) {
                    runningScores[g] -= gcoh.getMissPoints(g, rowName);
                } else {
                    genesetJumps[g] = r;
                    double sr = gcoh.getHitPoints(g, rowName);
                    //double sr = Math.abs(rl.getScore(r));
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;

                    /*
                    if (i == 0) {
                    System.out.println("old runningScores[g]=" + runningScores[g]);
                    }
                     */

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop

                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }

                }
            }
        }

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    public EnrichmentScoreCohort[] calculateKSScore_all_modes_seq_ks(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            hitCnt[g] = 0;
            genesetJumps[g] = -1;
        }

        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    double missPoints = gcoh.getMissPoints(g, rowName);
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    /*
                    if (i == 0) {
                    System.out.println("old runningScores[g]=" + runningScores[g] + " gap=" + gap);
                    }
                     */

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (isLastRun && !gcoh.isMember(i, rowName)) {
                    runningScores[g] -= gcoh.getMissPoints(g, rowName);
                } else {
                    genesetJumps[g] = r;
                    double sr = gcoh.getHitPoints(g, rowName);
                    //double sr = Math.abs(rl.getScore(r));
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;

                    /*
                    if (i == 0) {
                    System.out.println("old runningScores[g]=" + runningScores[g]);
                    }
                     */

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop

                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }

                }
            }
        }
        
        for (int i = 0; i < ess_maxdev.length; i++) {
            if (ess_maxdev[i] < 0) {
                ess_maxdev[i] = 0.0001f;
            }
        }        

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    public EnrichmentScoreCohort[] calculateKSScore_all_modes_snp(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            hitCnt[g] = 0;
            genesetJumps[g] = -1;
        }

        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    double missPoints = gcoh.getMissPoints(g, rowName);
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    /*
                    if (i == 0) {
                    System.out.println("old runningScores[g]=" + runningScores[g] + " gap=" + gap);
                    }
                     */

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (isLastRun && !gcoh.isMember(i, rowName)) {
                    runningScores[g] -= gcoh.getMissPoints(g, rowName);
                } else {
                    genesetJumps[g] = r;
                    double sr = gcoh.getHitPoints(g, rowName);
                    //double sr = Math.abs(rl.getScore(r));
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;

                    /*
                    if (i == 0) {
                    System.out.println("old runningScores[g]=" + runningScores[g]);
                    }
                     */

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop

                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }              

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }

                }
            }
        }
        
        for (int i = 0; i < ess_maxdev.length; i++) {
            if (ess_maxdev[i] < 0) {
                ess_maxdev[i] = 0.0001f;
            }
        }

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    public EnrichmentScoreCohort[] calculateKSScore_all_modes_parallel(final GeneSetCohort gcoh_exp, final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final float[] ess_maxdev_exp = new float[gcoh.getNumGeneSets()];
        final double[] runningScores_exp = new double[gcoh.getNumGeneSets()];
        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];
        final int[] genesetJumps_exp = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_maxdev_exp[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            runningScores_exp[g] = 0.0;
            hitCnt[g] = 0;
            genesetJumps[g] = -1;
            genesetJumps_exp[g]=-1;
        }

        // pure exp
        final RankedList rl_exp = gcoh_exp.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl_exp.getSize(); r++) {
            isLastRun = r == (rl_exp.getSize() - 1);
            String rowName = rl_exp.getRankName(r);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh_exp.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh_exp.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps_exp[g] - 1;
                if (gap > 0) {
                    double missPoints_exp = gcoh_exp.getMissPoints(g, rowName);
                    runningScores_exp[g] -= gap * missPoints_exp;
                    if (Math.abs(ess_maxdev_exp[g]) < Math.abs(runningScores_exp[g])) { // @note abs here
                        ess_maxdev_exp[g] = (float) runningScores_exp[g]; // @note no abs here!
                    }
                }

                if (isLastRun && !gcoh_exp.isMember(i, rowName)) {
                    runningScores_exp[g] -= gcoh_exp.getMissPoints(g, rowName);
                } else {
                    genesetJumps_exp[g] = r;
                    double sr_exp = gcoh_exp.getHitPoints(g, rowName);
                    if (Double.isNaN(sr_exp) || Double.isInfinite(sr_exp)) { // does this if the total weight is also 0
                        sr_exp = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }
                    runningScores_exp[g] += sr_exp;

                } // End last run loop

                // END JG CHANGES //
                if (Math.abs(ess_maxdev_exp[g]) < Math.abs(runningScores_exp[g])) { // @note abs here
                    ess_maxdev_exp[g] = (float) runningScores_exp[g]; // @note no abs here!
                }
            }
        }

        //integrated dataset
        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    double missPoints = gcoh.getMissPoints(g, rowName);
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    /*
                    if (i == 0) {
                    System.out.println("old runningScores[g]=" + runningScores[g] + " gap=" + gap);
                    }
                     */

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (isLastRun && !gcoh.isMember(i, rowName)) {
                    runningScores[g] -= gcoh.getMissPoints(g, rowName);
                } else {
                    genesetJumps[g] = r;
                    double sr = gcoh.getHitPoints(g, rowName);
                    //double sr = Math.abs(rl.getScore(r));
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }
                    runningScores[g] += sr;

                    /*
                    if (i == 0) {
                    System.out.println("old runningScores[g]=" + runningScores[g]);
                    }
                     */

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop

                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                /*if (isLastRun) {
                    if (ess_maxdev[g] < 0) {
                        ess_maxdev[g] = 0.0001f;
                    }
                    if (ess_maxdev_exp[g] < 0) {
                        ess_maxdev[g] = (-1f) * ess_maxdev[g];
                    }
                } */          

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }

                }
            }
        }
        
        for (int i = 0; i < ess_maxdev.length; i++) {
            if (ess_maxdev[i] < 0) {
                ess_maxdev[i] = 0.0001f;
            }
            if (ess_maxdev_exp[i] < 0) {
                ess_maxdev[i] = (-1f) * ess_maxdev[i];
            }
        }      

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    public EnrichmentScoreCohort[] calculateKSScore_all_modes_fm(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final double[] runningScores_pos = new double[gcoh.getNumGeneSets()];
        final double[] runningScores_neg = new double[gcoh.getNumGeneSets()];
        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            runningScores_pos[g] = 0.0;
            runningScores_neg[g] = 0.0;
            hitCnt[g] = 0;
            genesetJumps[g] = -1;
        }

        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    //double missPoints = gcoh.getMissPoints(g, rowName);
                    double missPoints = 0;
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (!isLastRun || gcoh.isMember(i, rowName)) {
                    genesetJumps[g] = r;
                    //double sr = gcoh.getHitPoints(g, rowName);
                    double sr = Math.log(Math.abs(rl.getScore(r)));
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;
                    if(posList){
                        runningScores_pos[g]+=sr;
                    }else{
                        runningScores_neg[g]+=sr;
                    }

                    if(hitCnt[g]==hitIndices[g].hitsIndices.length-1){
                        runningScores_pos[g] = (-2)*runningScores_pos[g];
                        runningScores_neg[g] = (-2)*runningScores_neg[g];
                        runningScores[g] = (-2)*runningScores[g];
                        if(runningScores_pos[g]<runningScores_neg[g]){
                            runningScores[g]=(-1)*runningScores[g];
                        }
                    }

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop


                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }
                }
            }
        }

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    public EnrichmentScoreCohort[] calculateKSScore_all_modes_fm_parallel(final GeneSetCohort gcoh_exp, final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final HitIndices[] hitIndices_exp = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices_exp[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final float[] ess_maxdev_exp = new float[gcoh.getNumGeneSets()];
        final double[] runningScores_exp = new double[gcoh.getNumGeneSets()];
        final double[] runningScores_pos_exp = new double[gcoh.getNumGeneSets()];
        final double[] runningScores_neg_exp = new double[gcoh.getNumGeneSets()];
        final int[] hitCnt = new int[gcoh.getNumGeneSets()];
        final int[] hitCnt_exp = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];
        final int[] genesetJumps_exp = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_maxdev_exp[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            runningScores_exp[g] = 0.0;
            runningScores_pos_exp[g] = 0.0;
            runningScores_neg_exp[g] = 0.0;
            hitCnt[g] = 0;
            hitCnt_exp[g] = 0;            
            genesetJumps[g] = -1;
            genesetJumps_exp[g] = -1;
        }

        // pure exp
        final RankedList rl_exp = gcoh_exp.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl_exp.getSize(); r++) {
            isLastRun = r == (rl_exp.getSize() - 1);
            String rowName = rl_exp.getRankName(r);
            float corr = rl_exp.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh_exp.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh_exp.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps_exp[g] - 1;
                if (gap > 0) {
                    //double missPoints_exp = gcoh_exp.getMissPoints(g, rowName);
                    double missPoints_exp = 0;
                    runningScores_exp[g] -= gap * missPoints_exp;
                    if (Math.abs(ess_maxdev_exp[g]) < Math.abs(runningScores_exp[g])) { // @note abs here
                        ess_maxdev_exp[g] = (float) runningScores_exp[g]; // @note no abs here!
                    }
                }

                if (!isLastRun || gcoh_exp.isMember(i, rowName)) {
                    genesetJumps_exp[g] = r;
                    //double sr = gcoh.getHitPoints(g, rowName);
                    double sr_exp = Math.log(Math.abs(rl_exp.getScore(r)));
                    if (Double.isNaN(sr_exp) || Double.isInfinite(sr_exp)) { // does this if the total weight is also 0
                        sr_exp = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores_exp[g] += sr_exp;
                    if(posList){
                        runningScores_pos_exp[g]+=sr_exp;
                    }else{
                        runningScores_neg_exp[g]+=sr_exp;
                    }

                    if(hitCnt_exp[g]==hitIndices_exp[g].hitsIndices.length-1){
                        runningScores_pos_exp[g] = (-2)*runningScores_pos_exp[g];
                        runningScores_neg_exp[g] = (-2)*runningScores_neg_exp[g];
                        runningScores_exp[g] = (-2)*runningScores_exp[g];
                        if(runningScores_pos_exp[g]<runningScores_neg_exp[g]){
                            runningScores_exp[g]=(-1)*runningScores_exp[g];
                        }
                    }
                    
                    hitIndices_exp[g].hitsIndices[hitCnt_exp[g]++] = r;
                }                
                // END JG CHANGES //
                if (Math.abs(ess_maxdev_exp[g]) < Math.abs(runningScores_exp[g])) { // @note abs here
                    ess_maxdev_exp[g] = (float) runningScores_exp[g]; // @note no abs here!
                }
            }
        }

        //integrated dataset
        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    //double missPoints = gcoh.getMissPoints(g, rowName);
                    double missPoints = 0;
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (!isLastRun || gcoh.isMember(i, rowName)) {
                    genesetJumps[g] = r;
                    //double sr = gcoh.getHitPoints(g, rowName);
                    double sr = Math.log(Math.abs(rl.getScore(r)));
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;

                    if(hitCnt[g]==hitIndices[g].hitsIndices.length-1){
                        runningScores[g] = (-2)*runningScores[g];
                        if (ess_maxdev_exp[g] < 0) {
                            runningScores[g] = (float) ((-1) * runningScores[g]);
                        }
                    }

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop


                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }
                }
            }
        }

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    public EnrichmentScoreCohort[] calculateKSScore_all_modes_samgs(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final double[] runningScores_pos = new double[gcoh.getNumGeneSets()];
        final double[] runningScores_neg = new double[gcoh.getNumGeneSets()];

        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            runningScores_pos[g] = 0.0;
            runningScores_neg[g] = 0.0;
            hitCnt[g] = 0;
            genesetJumps[g] = -1;
        }

        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    //double missPoints = gcoh.getMissPoints(g, rowName);
                    double missPoints = 0;
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (!isLastRun || gcoh.isMember(i, rowName)) {
                    genesetJumps[g] = r;
                    //double sr = gcoh.getHitPoints(g, rowName);
                    double sr = Math.abs(rl.getScore(r));
                    sr=sr*sr;
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;
                    if(posList){
                        runningScores_pos[g]+=sr;
                    }else{
                        runningScores_neg[g]+=sr;
                    }

                    if(hitCnt[g]==hitIndices[g].hitsIndices.length-1){
                        if(runningScores_pos[g]<runningScores_neg[g]){
                            runningScores[g]=(-1)*runningScores[g];
                        }
                    }

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop

                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }

                }
            }
        }

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    public EnrichmentScoreCohort[] calculateKSScore_all_modes_samgs_parallel(final GeneSetCohort gcoh_exp, final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final HitIndices[] hitIndices_exp = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices_exp[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final float[] ess_maxdev_exp = new float[gcoh.getNumGeneSets()];
        final double[] runningScores_exp = new double[gcoh.getNumGeneSets()];
        final double[] runningScores_pos_exp = new double[gcoh.getNumGeneSets()];
        final double[] runningScores_neg_exp = new double[gcoh.getNumGeneSets()];

        final int[] hitCnt = new int[gcoh.getNumGeneSets()];
        final int[] hitCnt_exp = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];
        final int[] genesetJumps_exp = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_maxdev_exp[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            runningScores_exp[g] = 0.0;
            runningScores_pos_exp[g] = 0.0;
            runningScores_neg_exp[g] = 0.0;
            hitCnt[g] = 0;
            hitCnt_exp[g] = 0;
            genesetJumps[g] = -1;
        }

        // pure exp
        final RankedList rl_exp = gcoh_exp.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl_exp.getSize(); r++) {
            isLastRun = r == (rl_exp.getSize() - 1);
            String rowName = rl_exp.getRankName(r);
            float corr = rl_exp.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh_exp.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh_exp.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps_exp[g] - 1;
                if (gap > 0) {
                    //double missPoints_exp = gcoh_exp.getMissPoints(g, rowName);
                    double missPoints_exp = 0;
                    runningScores_exp[g] -= gap * missPoints_exp;
                    if (Math.abs(ess_maxdev_exp[g]) < Math.abs(runningScores_exp[g])) { // @note abs here
                        ess_maxdev_exp[g] = (float) runningScores_exp[g]; // @note no abs here!
                    }
                }

                if (!isLastRun || gcoh_exp.isMember(i, rowName)) {
                    genesetJumps_exp[g] = r;
                    //double sr = gcoh.getHitPoints(g, rowName);
                    double sr_exp = Math.abs(rl_exp.getScore(r));
                    sr_exp=sr_exp*sr_exp;
                    if (Double.isNaN(sr_exp) || Double.isInfinite(sr_exp)) { // does this if the total weight is also 0
                        sr_exp = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores_exp[g] += sr_exp;
                    if(posList){
                        runningScores_pos_exp[g]+=sr_exp;
                    }else{
                        runningScores_neg_exp[g]+=sr_exp;
                    }

                    if(hitCnt_exp[g]==hitIndices_exp[g].hitsIndices.length-1){
                        if(runningScores_pos_exp[g]<runningScores_neg_exp[g]){
                            runningScores_exp[g]=(-1)*runningScores_exp[g];
                        }
                    }

                    hitIndices_exp[g].hitsIndices[hitCnt_exp[g]++] = r;
                }
                // END JG CHANGES //
                if (Math.abs(ess_maxdev_exp[g]) < Math.abs(runningScores_exp[g])) { // @note abs here
                    ess_maxdev_exp[g] = (float) runningScores_exp[g]; // @note no abs here!
                }
            }
        }

        //integrated dataset
        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    //double missPoints = gcoh.getMissPoints(g, rowName);
                    double missPoints = 0;
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (!isLastRun || gcoh.isMember(i, rowName)) {
                    genesetJumps[g] = r;
                    //double sr = gcoh.getHitPoints(g, rowName);
                    double sr = Math.abs(rl.getScore(r));
                    sr=sr*sr;
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;
                    if (hitCnt[g] == hitIndices[g].hitsIndices.length-1) {
                        if (ess_maxdev_exp[g] < 0) {
                            runningScores[g] = (float) ((-1) * runningScores[g]);
                        }
                    }
                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop

                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }
                
                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }

                }
            }
        }

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    //no directionality
    /*public EnrichmentScoreCohort[] calculateKSScore_all_modes_fm(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + "
numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            hitCnt[g] = 0;
            genesetJumps[g] = -1;
        }

        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    //double missPoints = gcoh.getMissPoints(g, rowName);
                    double missPoints = 0;
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (!isLastRun || gcoh.isMember(i, rowName)) {
                    genesetJumps[g] = r;
                    //double sr = gcoh.getHitPoints(g, rowName);
                    double sr = Math.log(rl.getScore(r));
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;

                    if(hitCnt[g]==hitIndices[g].hitsIndices.length-1){
                        runningScores[g] = (-2)*runningScores[g];
                    }

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop


                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }
                }
            }
        }

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs
[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g],
scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g],
rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g],
scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g],
rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    //no directionality
    public EnrichmentScoreCohort[] calculateKSScore_all_modes_samgs(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndex = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndex = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndex[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + "
numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        //final double[] runningScores_pos_list = new double[gcoh.getNumGeneSets()];
        //final double[] runningScores_neg_list = new double[gcoh.getNumGeneSets()];

        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        final int[] genesetJumps = new int[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            //runningScores_pos_list[g] = 0.0;
            //runningScores_neg_list[g] = 0.0;
            hitCnt[g] = 0;
            genesetJumps[g] = -1;
        }

        final RankedList rl = gcoh.getRankedList();

        // START JG CHANGES //

        boolean isLastRun = false;
        for (int r = 0; r < rl.getSize(); r++) {

            isLastRun = r == (rl.getSize() - 1);

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            int[] genesetIndices = null;
            if (isLastRun) {
                // for the last run, we want to iterate over all genesets
                genesetIndices = new int[gcoh.getNumGeneSets()];
                for (int i = 0; i < genesetIndices.length; ++i) {
                    genesetIndices[i] = i;
                }
            } else {
                // otherwise, we only care about those genesets associated with this gene
                genesetIndices = gcoh.genesetIndicesForGene(rowName);
            }

            // gene not in any geneset
            if (genesetIndices == null) {
                continue;
            }

            for (int i = 0; i < genesetIndices.length; i++) {
                int g = genesetIndices[i];
                int gap = r - genesetJumps[g] - 1;
                if (gap > 0) {
                    //double missPoints = gcoh.getMissPoints(g, rowName);
                    double missPoints = 0;
                    if (storeRESPointByPoint) {
                        // backfill - this typically only happens on 'real', not permutations
                        double trun = runningScores[g];
                        for (int j = genesetJumps[g] + 1; j < r; j++) {
                            trun -= missPoints;
                            scoresAtEachPoint[g].setElement(j, trun);
                        }
                    }

                    runningScores[g] -= gap * missPoints;

                    if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                        ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                        rankAtMaxEs[g] = r - 1; // @note the -1
                        scoresAtMaxEs[g] = rl.getScore(r - 1); // @note the -1 effective when the score is NEG
                    }
                }

                if (!isLastRun || gcoh.isMember(i, rowName)) {
                    genesetJumps[g] = r;
                    //double sr = gcoh.getHitPoints(g, rowName);
                    double sr = Math.abs(rl.getScore(r));
                    sr=sr*sr;
                    if (Double.isNaN(sr) || Double.isInfinite(sr)) { // does this if the total weight is also 0
                        sr = 0.000001d;
                        //throw new IllegalStateException("Nan hit score for feature: " + rowName + " >" + sr);
                    }

                    runningScores[g] += sr;

                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) {// Only store for hits
                        scoresAtEachHitIndex[g].add((float) runningScores[g]);
                    }

                } // End last run loop

                // END JG CHANGES //

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }

                }
            }
        }

        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs
[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g],
scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g],
rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g],
scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g],
rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndex[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }*/

    public EnrichmentScoreCohort[] calculateKSScore_all_modes_AS_ORIG(final GeneSetCohort gcoh,
            final boolean storeDeep,
            final boolean storeRESPointByPoint) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param gcoh cannot be null");
        }

        TFloatArrayList[] scoresAtEachHitIndext = null;
        final int[] rankAtMaxEs = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs = new float[gcoh.getNumGeneSets()];
        if (storeDeep) {
            scoresAtEachHitIndext = new TFloatArrayList[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachHitIndext[g] = new TFloatArrayList();
            }
        }

        // Also store every point for nicer es plots when run from memory
        Vector[] scoresAtEachPoint = null;

        if (storeRESPointByPoint) {
            scoresAtEachPoint = new Vector[gcoh.getNumGeneSets()];
            // init
            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                scoresAtEachPoint[g] = new Vector(gcoh.getNumLabels());
            }
        }

        // always filled as needed by mann whitney (even if it isnt saved after here)
        final HitIndices[] hitIndices = new HitIndices[gcoh.getNumGeneSets()];
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            //log.debug("gene set: " + gcoh.getGeneSet(g).getName() + " numTrue: " + gcoh.getNumTrue(g) + " numMembers: " + gcoh.getGeneSet(g).getNumMembers());
            hitIndices[g] = new HitIndices(gcoh.getNumTrue(g));
        }

        final float[] ess_maxdev = new float[gcoh.getNumGeneSets()];
        final double[] runningScores = new double[gcoh.getNumGeneSets()];
        final int[] hitCnt = new int[gcoh.getNumGeneSets()];

        final float[] ess_pos_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_pos_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_pos_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_pos_list_maxdev = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list = new float[gcoh.getNumGeneSets()];

        final float[] ess_neg_list_maxdev = new float[gcoh.getNumGeneSets()];
        final int[] rankAtMaxEs_neg_list_maxdev = new int[gcoh.getNumGeneSets()];
        final float[] scoresAtMaxEs_neg_list_maxdev = new float[gcoh.getNumGeneSets()];

        // init arrays
        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            ess_maxdev[g] = 0.0f;
            ess_pos_list[g] = 0.0f;
            ess_pos_list_maxdev[g] = 0.0f;
            ess_neg_list[g] = 0.0f;
            ess_neg_list_maxdev[g] = 0.0f;
            runningScores[g] = 0.0;
            hitCnt[g] = 0;
        }

        final RankedList rl = gcoh.getRankedList();

        for (int r = 0; r < rl.getSize(); r++) {

            String rowName = rl.getRankName(r);
            float corr = rl.getScore(r);
            boolean posList = XMath.isPositive(corr);

            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                // this will barf if there are duplicate probes in the dataset
                if (gcoh.isMember(g, rowName)) {
                    double sr = gcoh.getHitPoints(g, rowName);

                    if (Double.isNaN(sr) || Double.isInfinite(sr)) {
                        throw new IllegalStateException("Nan hit score for feature: " + rowName);
                    }

                    runningScores[g] += sr;
                    //log.debug("geneset: " + gcoh.getGeneSet(g).getName() + " " + hitIndices[g].hitsIndices[hitCnt[g]]);
                    hitIndices[g].hitsIndices[hitCnt[g]++] = r;

                    if (storeDeep) { // Only store for hits
                        //System.out.println(">>>>>>>>>>> " + runningScores[g]);
                        scoresAtEachHitIndext[g].add((float) runningScores[g]);
                    }

                } else {
                    runningScores[g] -= gcoh.getMissPoints(g, rowName);
                }

                // @note OUTside the loop
                if (storeRESPointByPoint) {
                    scoresAtEachPoint[g].setElement(r, (float) runningScores[g]);
                }

                if (Math.abs(ess_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs here
                    ess_maxdev[g] = (float) runningScores[g]; // @note no abs here!
                    rankAtMaxEs[g] = r;
                    scoresAtMaxEs[g] = rl.getScore(r);
                }

                // Then the by pos/neg list stuff
                // @note NO abs in some cases - we want max POS (or NEG) deviations from zero
                if (posList) {
                    if (ess_pos_list[g] < runningScores[g]) { // @note NO abs
                        ess_pos_list[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list[g] = r;
                        scoresAtMaxEs_pos_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_pos_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note YES abs
                        ess_pos_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_pos_list_maxdev[g] = r;
                        scoresAtMaxEs_pos_list_maxdev[g] = rl.getScore(r);
                    }

                } else {
                    if (ess_neg_list[g] > runningScores[g]) { // Note NO abs
                        ess_neg_list[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list[g] = r;
                        scoresAtMaxEs_neg_list[g] = rl.getScore(r);
                    }

                    if (Math.abs(ess_neg_list_maxdev[g]) < Math.abs(runningScores[g])) { // @note abs
                        ess_neg_list_maxdev[g] = (float) runningScores[g];
                        rankAtMaxEs_neg_list_maxdev[g] = r;
                        scoresAtMaxEs_neg_list_maxdev[g] = rl.getScore(r);
                    }

                }
            }
        }


        final EnrichmentScoreCohort[] cohorts = new EnrichmentScoreCohort[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            final float mw = (float) XMath.mannWhitney(hitIndices[g].hitsIndices, rl.getSize());

            cohorts[g] = new EnrichmentScoreCohort();
            cohorts[g].es_maxdev_style = new EnrichmentScoreImplSlim(ess_maxdev[g], rankAtMaxEs[g], scoresAtMaxEs[g], cohorts[g]);
            cohorts[g].es_on_pos_list = new EnrichmentScoreImplSlim(ess_pos_list[g], rankAtMaxEs_pos_list[g], scoresAtMaxEs_pos_list[g], cohorts[g]);
            cohorts[g].es_on_pos_list_maxdev = new EnrichmentScoreImplSlim(ess_pos_list_maxdev[g], rankAtMaxEs_pos_list_maxdev[g], scoresAtMaxEs_pos_list_maxdev[g], cohorts[g]);
            cohorts[g].es_on_neg_list = new EnrichmentScoreImplSlim(ess_neg_list[g], rankAtMaxEs_neg_list[g], scoresAtMaxEs_neg_list[g], cohorts[g]);
            cohorts[g].es_on_neg_list_maxdev = new EnrichmentScoreImplSlim(ess_neg_list_maxdev[g], rankAtMaxEs_neg_list_maxdev[g], scoresAtMaxEs_neg_list_maxdev[g], cohorts[g]);

            cohorts[g].mw = mw;
            cohorts[g].numHits = gcoh.getNumTrue(g);

            if (storeDeep) {
                cohorts[g].fHitIndices_opt = hitIndices[g].hitsIndices;
                cohorts[g].scoresAtEachHitIndex_opt = new Vector(scoresAtEachHitIndext[g]);
            }

            if (storeRESPointByPoint) {
                cohorts[g].scoresAtEachPoint_opt = new Vector(scoresAtEachPoint[g]);
            }

        }

        return cohorts;
    }

    // highly optimized version
    // @note IMP the rl must contain the same set of rows as the ds that made the gcoh
    public float[] calculateKSScores(GeneSetCohort gcoh) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param sets cannot be null");
        }

        float[] esRaw = new float[gcoh.getNumGeneSets()];
        float[] runningScoreRaw = new float[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            esRaw[g] = 0.0f;
            runningScoreRaw[g] = 0.0f;
        }

        RankedList rl = gcoh.getRankedList();

        for (int r = 0; r < rl.getSize(); r++) {
            String name = rl.getRankName(r);

            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                if (gcoh.isMember(g, name)) {
                    runningScoreRaw[g] += gcoh.getHitPoints(g, name);
                } else {
                    runningScoreRaw[g] -= gcoh.getMissPoints(g, name);
                }

                if (Math.abs(esRaw[g]) < Math.abs(runningScoreRaw[g])) {
                    esRaw[g] = runningScoreRaw[g]; // @note no abs here!
                }
            }
        }

        return esRaw;
    }

    public float[] calculateKSScoresP2(GeneSetCohort gcoh) {

        if (gcoh == null) {
            throw new IllegalArgumentException("Param sets cannot be null");
        }

        float[] esRaw = new float[gcoh.getNumGeneSets()];
        float[] runningScoreRaw = new float[gcoh.getNumGeneSets()];

        for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
            esRaw[g] = 0.0f;
            runningScoreRaw[g] = 0.0f;
        }

        RankedList rl = gcoh.getRankedList();

        for (int r = 0; r < rl.getSize(); r++) {
            String name = rl.getRankName(r);

            for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                if (gcoh.isMember(g, name)) {
                    runningScoreRaw[g] += gcoh.getHitPoints_ssea(g, name);
                } else {
                    runningScoreRaw[g] -= gcoh.getMissPoints(g, name);
                }

                if (Math.abs(esRaw[g]) < Math.abs(runningScoreRaw[g])) {
                    esRaw[g] = runningScoreRaw[g]; // @note no abs here!
                }
            }
        }

        return esRaw;
    }

    /**
     * positions with hit or not
     * If hit -> true, else false
     */
    public BooleanVector calculateHitProfile(GeneSet gset, RankedList rl) {

        BooleanVector hits = new BooleanVector(rl.getSize());

        for (int i = 0; i < rl.getSize(); i++) {
            if (gset.isMember(rl.getRankName(i))) {
                hits.setElement(i, true);
                //log.info("Hit to: " + ds.getRowName(i) + " at: " + i);
            } else {
                hits.setElement(i, false);
            }
        }

        return hits;
    }

    protected static class HitIndices {

        int[] hitsIndices;

        HitIndices(int len) {
            this.hitsIndices = new int[len];
        }
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    private static class EnrichmentScoreCohort {

        private EnrichmentScore es_maxdev_style;
        private EnrichmentScore es_on_pos_list_maxdev;
        private EnrichmentScore es_on_pos_list;
        private EnrichmentScore es_on_neg_list;
        private EnrichmentScore es_on_neg_list_maxdev;
        private float mw;
        private int numHits;
        // @note deep only
        private int[] fHitIndices_opt;
        private Vector scoresAtEachHitIndex_opt;
        // @note this is redundant (but has more info) with the scoresAtEachHitIndex_opt var
        // usefil for richer plots
        private Vector scoresAtEachPoint_opt;

        private boolean hasDeepInfo() {
            return scoresAtEachHitIndex_opt != null;
        }

        private Vector getESProfile() {
            ensureDeep();
            scoresAtEachHitIndex_opt.setImmutable();
            return scoresAtEachHitIndex_opt;
        }

        private Vector getESProfile_full() {
            ensureDeep();
            scoresAtEachPoint_opt.setImmutable();
            return scoresAtEachPoint_opt;
        }

        private int getNumHits() {
            return numHits;
        }

        private int[] getHitIndices() {
            ensureDeep();
            return fHitIndices_opt;
        }

        private void ensureDeep() {
            if (!hasDeepInfo()) {
                throw new IllegalStateException("EnrichmentScore was NOT consructed in deep mode -- info not available");
            }
        }
    }

    /**
     * Inner class to represent one result
     * NO perms done so NO etc are not applicable
     */
    static class EnrichmentScoreImplSlim implements EnrichmentScore {

        private float fES;
        private int fRankAtES;
        private float fRankScoreAtES;
        private EnrichmentScoreCohort fScoreCoh;

        /**
         * Class constructor
         *
         * @param es
         * @param rankAtEs
         * @param corrAtEs
         * @param nes
         * @param np
         * @param fdr
         * @param fwer
         * @param hitIndices_opt
         * @param runningRaw_opt
         */
        EnrichmentScoreImplSlim(final float es,
                final int rankAtEs,
                final float corrAtEs,
                final EnrichmentScoreCohort coh) {
            this.fES = es;
            this.fRankAtES = rankAtEs;
            this.fRankScoreAtES = corrAtEs;

            // @note shared
            this.fScoreCoh = coh;
        }

        public boolean hasDeepInfo() {
            return fScoreCoh.hasDeepInfo();
        }

        public float getES() {
            return fES;
        }

        public int getRankAtES() {
            return fRankAtES;
        }

        public float getRankScoreAtES() {
            return fRankScoreAtES;
        }

        public float getNES() {
            return Float.NaN;
        }

        public float getNP() {
            return Float.NaN;
        }

        public float getFWER() {
            return Float.NaN;
        }

        public float getFDR() {
            return Float.NaN;
        }

        // @note change done in march 2006 to get nicer plots
        public Vector getESProfile() {
            return fScoreCoh.getESProfile();
        }

        public Vector getESProfile_point_by_point_opt() {
            return fScoreCoh.getESProfile_full();
        }

        public int getNumHits() {
            return fScoreCoh.getNumHits();
        }

        public int[] getHitIndices() {
            return fScoreCoh.getHitIndices();
        }

        public float getScore(Type type) {
            if (type == ES) {
                return getES();
            } else if (type == NES) {
                return getNES();
            } else if (type == FDR) {
                return getFDR();
            } else if (type == NP) {
                return getNP();
            } else {
                throw new IllegalArgumentException("Unknown Type: " + type);
            }
        }

        @Override
        public BigDecimal getESB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getNESB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getNPB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getFWERB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getFDRB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getScoreB(Type type) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }
    } // End inner class EnrichmentScoreImplSlim
} // End KSCore

