/**
 * The Broad Institute SOFTWARE COPYRIGHT NOTICE AGREEMENT This software and its
 * documentation are copyright 2004-2005 by the Broad Institute/Massachusetts
 * Institute of Technology. All rights are reserved. This software is supplied
 * without any warranty or guaranteed support whatsoever. Neither the Broad
 * Institute nor MIT can be responsible for its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.*;
import edu.mit.broad.genome.alg.markers.PermutationTestBuilder;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.esmatrix.db.*;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentScoreImplP;
import org.apache.log4j.Logger;
import edu.duke.igsp.app.algorithms.*;
import edu.mit.broad.genome.alg.markers.IntegratedPermutationTestBuilder;
import edu.mit.broad.genome.alg.markers.GsaasnpPermutationTestBuilder;
import edu.duke.igsp.app.tools.GsaaTool;
import edu.duke.igsp.app.tools.GsaaSeqTool;
import edu.duke.igsp.app.tools.BigDecimalTool;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.*;
import java.math.BigDecimal;
import xtools.api.param.OrderParam;
import xtools.api.param.SortParam;

/**
 * Kolmogorov-Smirnov Enrichment Test related methods
 * <p/>
 * Many things here are specific to gsea;
 *
 * @author Aravind Subramanian
 * @revised by Qing Xiong <qxiong@email.unc.edu>
 * @version %I%, %G%
 * @see KSCore
 */
public class KSTests {

    private final Logger log = XLogger.getLogger(this.getClass());
    private final KSCore core;
    private static final int LOG_FREQ = 5;
    private PrintStream sout;

    /**
     * Class Constructor. Almost Stateless os -> for quick stdout NOT for
     * logging
     */
    public KSTests(final PrintStream os) {
        this.sout = os;
        this.core = new KSCore();
    }

    /**
     * Class constructor Default output is System.out
     */
    public KSTests() {
        this(System.out);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    /**
     * @throws java.lang.Exception
     */
    public EnrichmentDb executeGsea(final Dataset ds,
            final Template t,
            final GeneSet[] gsets_orig,
            final int nperm,
            final GeneSetScoringTable table,
            final boolean permuteTemplate,
            final int numMarkers) {

        return executeGsea(ds, t, getGeneSets(ds, gsets_orig, 15, 500),
                nperm,
                new Metrics.Signal2Noise(), SortMode.REAL, Order.DESCENDING,
                new LabelledVectorProcessors.None(),
                new RandomSeedGenerators.Timestamp(),
                TemplateRandomizerType.NO_BALANCE, DatasetMetrics.getDefaultMetricParams(),
                new DefaultGeneSetCohort.Generator(table, true),
                permuteTemplate, numMarkers, null);
    }

    public static GeneSet[] getGeneSets(final Dataset ds,
            GeneSet[] gsets,
            final int geneSetMinSize,
            final int geneSetMaxSize) {

        gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, geneSetMinSize, ds, true);
        gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, geneSetMaxSize, ds, false);

        if (gsets.length == 0) {
            throw new IllegalArgumentException("No gene sets passed seize threshold");
        }

        return gsets;
    }

    public EnrichmentDb executeGsea(final DatasetTemplate dt,
            final GeneSet[] gsets,
            final int nperm,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final RandomSeedGenerator rst,
            final TemplateRandomizerType rt,
            final Map mps,
            final GeneSetCohortGenerator gcohgen,
            final boolean permuteTemplate,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt) throws Exception {

        return executeGsea(dt.getDataset(false), dt.getTemplate(), gsets,
                nperm, metric, sort, order, lvp, rst, rt, mps, gcohgen,
                permuteTemplate, numMarkers, store_rnd_ranked_lists_here_opt);
    }

    public EnrichmentDb executeGsaaSeqsp(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final DatasetTemplate dt,
            final GeneSet[] gsets,
            final int nperm,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final RandomSeedGenerator rst,
            final TemplateRandomizerType rt,
            final GeneSetCohortGenerator gcohgen,
            final boolean permuteTemplate,
            final List store_rnd_ranked_lists_here_opt) throws Exception {

        return executeGsaaSeqsp(demetric, gsametric, pthreshold, dt.getDataset(false), dt.getTemplate(), gsets,
                nperm, sort, order, lvp, rst, rt, gcohgen,
                permuteTemplate, store_rnd_ranked_lists_here_opt);
    }

    public EnrichmentDb executeGsaaM(final DatasetTemplate dt,
            final GeneSet[] gsets,
            final int nperm,
            final Metric metric,
            final String imetric,
            final String smetric,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final RandomSeedGenerator rst,
            final TemplateRandomizerType rt,
            final Map mps,
            final GeneSetCohortGenerator gcohgen,
            final boolean permuteTemplate,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params) throws Exception {

        return executeGsaaM(dt.getDataset(false), dt.getTemplate(), gsets,
                nperm, metric, imetric, smetric, sort, order, lvp, rst, rt, mps, gcohgen,
                permuteTemplate, numMarkers, store_rnd_ranked_lists_here_opt, datasets, templates, params);
    }

    public SnpEnrichmentDb executeGsaaSnpKS(final DatasetTemplate dt,
            final GeneSet[] gsets,
            final int nperm,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final RandomSeedGenerator rst,
            final TemplateRandomizerType rt,
            final GeneSetCohortGenerator gcohgen,
            final boolean permuteTemplate,
            final List store_rnd_ranked_lists_here_opt,
            final CMetric cmetric,
            final Template[] rndTemplates) throws Exception {

        return executeGsaaSnpKS(dt.getDataset(false), dt.getTemplate(), gsets,
                nperm, sort, order, lvp, rst, rt, gcohgen,
                permuteTemplate, store_rnd_ranked_lists_here_opt, cmetric, rndTemplates);
    }

    public EnrichmentDb executeGsea(final Dataset ds,
            final Template t,
            final GeneSet[] gsets,
            final int nperm,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final RandomSeedGenerator rst,
            final TemplateRandomizerType rt,
            final Map mps,
            final GeneSetCohortGenerator gcohgen,
            final boolean permuteTemplate,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt) {


        log.debug("!!!! Executing for: " + ds.getName() + " # samples: " + ds.getNumCol());

        EnrichmentDb edb;
        if (permuteTemplate) {
            edb = shuffleTemplate(nperm, metric, sort, order, mps, lvp,
                    ds, t, gsets, gcohgen, rt, rst, numMarkers, store_rnd_ranked_lists_here_opt);
        } else {
            edb = shuffleGeneSet(nperm, metric, sort, order, mps, lvp, ds, t, gsets, gcohgen, rst, true);
        }

        sout.println("Finished permutations ... creating reports");

        // Done algorithmics, now the reports
        return edb;
    }

    public EnrichmentDb executeGsaaSeqsp(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final Dataset ds,
            final Template t,
            final GeneSet[] gsets,
            final int nperm,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final RandomSeedGenerator rst,
            final TemplateRandomizerType rt,
            final GeneSetCohortGenerator gcohgen,
            final boolean permuteTemplate,
            final List store_rnd_ranked_lists_here_opt) {

        log.debug("!!!! Executing for: " + ds.getName() + " # samples: " + ds.getNumCol());

        EnrichmentDb edb;
        if (permuteTemplate) {
            if (gsametric.equalsIgnoreCase("Weighted_KS")) {
                edb = shuffleTemplateSeqsp_ks(demetric, gsametric, pthreshold, nperm, sort, order, lvp,
                        ds, t, gsets, gcohgen, rt, rst, store_rnd_ranked_lists_here_opt);
            } else {
                edb = shuffleTemplateSeqsp_p(demetric, gsametric, pthreshold, nperm, sort, order, lvp,
                        ds, t, gsets, gcohgen, rt, rst, store_rnd_ranked_lists_here_opt);
            }
        } else {
            if (gsametric.equalsIgnoreCase("Weighted_KS")) {
                edb = shuffleGeneSetSeq_ks(demetric, gsametric, pthreshold, nperm, sort, order, lvp, ds, t, gsets, gcohgen, rst, true);
            } else {
                edb = shuffleGeneSetSeq_p(demetric, gsametric, pthreshold, nperm, sort, order, lvp, ds, t, gsets, gcohgen, rst, true);
            }
        }

        sout.println("Finished permutations ... creating reports");

        // Done algorithmics, now the reports
        return edb;
    }

    public EnrichmentDb executeGsaaM(final Dataset ds,
            final Template t,
            final GeneSet[] gsets,
            final int nperm,
            final Metric metric,
            final String imetric,
            final String smetric,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final RandomSeedGenerator rst,
            final TemplateRandomizerType rt,
            final Map mps,
            final GeneSetCohortGenerator gcohgen,
            final boolean permuteTemplate,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params) {

        log.debug("!!!! Executing for: " + ds.getName() + " # samples: " + ds.getNumCol());

        EnrichmentDb edb;
        if (permuteTemplate) {
            edb = shuffleTemplateM(nperm, metric, imetric, smetric, sort, order, mps, lvp, ds, t, gsets, gcohgen, rt, rst, numMarkers, store_rnd_ranked_lists_here_opt, datasets, templates, params);
        } else {
            edb = shuffleGeneSetM(nperm, metric, sort, order, mps, lvp, ds, t, gsets, gcohgen, rst, true, datasets, templates, params);
        }

        sout.println("Finished permutations ... creating reports");

        // Done algorithmics, now the reports
        return edb;
    }

    public SnpEnrichmentDb executeGsaaSnpKS(final Dataset ds,
            final Template t,
            final GeneSet[] gsets,
            final int nperm,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final RandomSeedGenerator rst,
            final TemplateRandomizerType rt,
            final GeneSetCohortGenerator gcohgen,
            final boolean permuteTemplate,
            final List store_rnd_ranked_lists_here_opt,
            final CMetric cmetric,
            final Template[] rndTemplates) {

        log.debug("!!!! Executing for: " + ds.getName() + " # samples: " + ds.getNumCol());

        SnpEnrichmentDb edb;
        if (permuteTemplate) {
            edb = shuffleTemplateGsaaSnpKS(sort, order, lvp,
                    ds, t, gsets, gcohgen, store_rnd_ranked_lists_here_opt, cmetric, rndTemplates);
        } else {
            edb = shuffleGeneSetGsaaSnpKS(nperm, cmetric, sort, order, lvp, ds, t, gsets, gcohgen, rst, true);
        }

        sout.println("Finished permutations ... creating reports");

        // Done algorithmics, now the reports
        return edb;
    }

    public EnrichmentDb executeGsea(final RankedList rl_real,
            final GeneSet[] gsets,
            final int nperm,
            final RandomSeedGenerator rst,
            final Chip chip,
            final GeneSetCohortGenerator gcohgen) throws Exception {

        log.debug("!!!! Executing for: " + rl_real.getName() + " # features: " + rl_real.getSize());

        EnrichmentResult[] results = shuffleGeneSet_precannedRankedList(nperm, rl_real, null, gsets, chip, gcohgen, rst, false, true, true);
        return new EnrichmentDbImpl_one_shared_rl(rl_real.getName(),
                rl_real, null, null, results, new LabelledVectorProcessors.None(),
                new Metrics.None(),
                new HashMap(),
                SortMode.REAL,
                Order.DESCENDING, nperm, null);
    }

    public EnrichmentDb executeGsaaseqgp(final RankedList rl_real,
            final Dataset ds_shared_opt,
            final Template template_shared_opt,
            final GeneSet[] gsets,
            final int nperm,
            final RandomSeedGenerator rst,
            final Chip chip,
            final GeneSetCohortGenerator gcohgen) throws Exception {

        log.debug("!!!! Executing for: " + rl_real.getName() + " # features: " + rl_real.getSize());

        EnrichmentResult[] results = shuffleGeneSet_precannedRankedList_gsaaseqgp(nperm, rl_real, null, gsets, chip, gcohgen, rst, false, true, true);
        return new EnrichmentDbImpl_one_shared_rl(rl_real.getName(),
                rl_real, ds_shared_opt, template_shared_opt, results, new LabelledVectorProcessors.None(),
                new Metrics.None(),
                new HashMap(),
                SortMode.REAL,
                Order.DESCENDING, nperm, null);
    }

    public EnrichmentDb shuffleTemplate(final int nperm,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final TemplateRandomizerType rt,
            final RandomSeedGenerator rst,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt) {

        final Template[] rndTemplates = TemplateFactoryRandomizer.createRandomTemplates(nperm, template, rt, rst);
        log.debug("Done generating rnd templates: " + rndTemplates.length);
        return shuffleTemplate_canned_templates_gsea(metric, sort, order, metricParams, lvp, ds,
                template, gsets, gcohgen, rndTemplates, null, null, numMarkers, store_rnd_ranked_lists_here_opt);
    }

    public EnrichmentDb shuffleTemplateSeqsp_ks(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final int nperm,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final TemplateRandomizerType rt,
            final RandomSeedGenerator rst,
            final List store_rnd_ranked_lists_here_opt) {

        final Template[] rndTemplates = TemplateFactoryRandomizer.createRandomTemplates(nperm, template, rt, rst);
        log.debug("Done generating rnd templates: " + rndTemplates.length);
        return shuffleTemplate_canned_templates_gsaaseqsp_ks(demetric, gsametric, pthreshold, sort, order, lvp, ds,
                template, gsets, gcohgen, rndTemplates, null, null, store_rnd_ranked_lists_here_opt);

    }

    public EnrichmentDb shuffleTemplateSeqsp_p(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final int nperm,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final TemplateRandomizerType rt,
            final RandomSeedGenerator rst,
            final List store_rnd_ranked_lists_here_opt) {

        final Template[] rndTemplates = TemplateFactoryRandomizer.createRandomTemplates(nperm, template, rt, rst);
        log.debug("Done generating rnd templates: " + rndTemplates.length);
        if (gsametric.equalsIgnoreCase("L2Norm") || gsametric.equalsIgnoreCase("Mean")) {
            return shuffleTemplate_canned_templates_gsaaseqsp_p1(demetric, gsametric, pthreshold, sort, order, lvp, ds,
                    template, gsets, gcohgen, rndTemplates, null, null, store_rnd_ranked_lists_here_opt);
        } else {
            return shuffleTemplate_canned_templates_gsaaseqsp_p2(demetric, gsametric, pthreshold, sort, order, lvp, ds,
                    template, gsets, gcohgen, rndTemplates, null, null, store_rnd_ranked_lists_here_opt);
        }
    }

    public EnrichmentDb shuffleTemplateM(final int nperm,
            final Metric metric,
            final String imetric,
            final String smetric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final TemplateRandomizerType rt,
            final RandomSeedGenerator rst,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params) {

        HashMap rndTemplateSet = new HashMap();
        Template[] rndTemplates = new Template[nperm];
        if (!isTemplateIdentical(template, templates)) {
            rndTemplates = TemplateFactoryRandomizer.createRandomTemplates(nperm, template, rt, rst);
            if (datasets.containsKey("SNP")) {
                final Template[] rndTemplateSnp = (Template[]) ((HashMap) (params.get("SNP"))).get("RND_TEMPLATES");
                rndTemplateSet.put("SNP", rndTemplateSnp);
            }
        } else {
            rndTemplates = (Template[]) ((HashMap) (params.get("SNP"))).get("RND_TEMPLATES");
            if (datasets.containsKey("SNP")) {
                final Template[] rndTemplateSnp = (Template[]) ((HashMap) (params.get("SNP"))).get("RND_TEMPLATES");
                rndTemplateSet.put("SNP", rndTemplateSnp);
            }
        }

        HashMap[] hms = new HashMap[rndTemplates.length];
        Object[] keySet = rndTemplateSet.keySet().toArray();
        for (int i = 0; i < rndTemplates.length; i++) {
            HashMap hm = new HashMap();
            for (int j = 0; j < keySet.length; j++) {
                hm.put((String) keySet[j], ((Template[]) rndTemplateSet.get((String) keySet[j]))[i]);
            }
            hms[i] = hm;
        }

        if (smetric.equalsIgnoreCase("Weighted_KS")) {
            return shuffleTemplate_canned_templates_integrated_ks(metric, imetric, sort, order, metricParams, lvp, ds,
                    template, gsets, gcohgen, rndTemplates, null, null, numMarkers, store_rnd_ranked_lists_here_opt, datasets, templates, params, hms);
        } else if (smetric.equalsIgnoreCase("Fisher_Method")) {
            return shuffleTemplate_canned_templates_integrated_fm(metric, imetric, sort, order, metricParams, lvp, ds,
                    template, gsets, gcohgen, rndTemplates, null, null, numMarkers, store_rnd_ranked_lists_here_opt, datasets, templates, params, hms);
        } else {
            return shuffleTemplate_canned_templates_integrated_samgs(metric, imetric, sort, order, metricParams, lvp, ds,
                    template, gsets, gcohgen, rndTemplates, null, null, numMarkers, store_rnd_ranked_lists_here_opt, datasets, templates, params, hms);
        }
    }

    public SnpEnrichmentDb shuffleTemplateGsaaSnpKS(
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final List store_rnd_ranked_lists_here_opt,
            final CMetric cmetric,
            final Template[] rndTemplates) {

        log.debug("Done generating rnd templates: " + rndTemplates.length);
        return shuffleTemplate_canned_templates_gsaasnp_ks(sort, order, lvp, ds,
                template, gsets, gcohgen, rndTemplates, null, null, store_rnd_ranked_lists_here_opt, cmetric);
    }

    public EnrichmentDb shuffleTemplate(final int nperm,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final TemplateRandomizerType rt,
            final RandomSeedGenerator rst,
            final float fudgeLength,
            final float fudgeWeight,
            final int maxAtttemptsAllowed,
            final TemplateStats.RandomRankedListMode rmode,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt)
            throws Exception {

        final TemplateStats.TemplateHitStruc hs = TemplateStats.createRandomTemplates(fudgeLength,
                fudgeWeight,
                nperm,
                maxAtttemptsAllowed,
                sort,
                order,
                metric,
                metricParams,
                lvp, ds,
                template,
                rt,
                rst,
                rmode);

        Template[] rndTemplates = hs.getRandomTemplates(rmode);
        final RankedList[] rndRankedLists = hs.getRandomRankedLists(rmode);
        final RankedList realRankedList = hs.getRealRankedList();

        log.info("Done generating rnd templates: " + rndTemplates.length);
        return shuffleTemplate_canned_templates(metric, sort, order, metricParams, lvp, ds, template,
                gsets, gcohgen, rndTemplates, realRankedList, rndRankedLists, numMarkers, store_rnd_ranked_lists_here_opt);
    }

    public EnrichmentDb shuffleTemplate_canned_templates(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);
        final Chip chip = ds.getAnnot().getChip();

        final DatasetMetrics dm = new DatasetMetrics();
        final RankedList rlReal;
        PermutationTestBuilder ptest = new PermutationTestBuilder(dstName, numMarkers, rndTemplates.length, lvp,
                metric, sort, order, metricParams, ds, template, null, template.isCategorical());

        if (realRankedList_pre_calculated_opt != null) {
            rlReal = realRankedList_pre_calculated_opt;
        } else {
            rlReal = dm.scoreDataset(metric, sort, order, metricParams, lvp, ds, template);
        }

        // calc real scores
        if (rlReal.getSize() != ds.getNumRow()) {// sanity check
            throw new MismatchedSizeException();
        }

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true); // @note ASSUME already qualified
        final EnrichmentScore[] realScores = core.calculateKSScore(gcoh, true); // need to store details as we need the hit indices
        final Vector[] rndEss = new Vector[gsets.length];
        for (int g = 0; g < gsets.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

        // Each row is a "geneset", and each column a randomization
        for (int c = 0; c < rndTemplates.length; c++) {
            final RankedList rndRl;

            if (rndRankedLists_pre_calculated_opt != null) {
                rndRl = rndRankedLists_pre_calculated_opt[c];
            } else {
                rndRl = dm.scoreDataset(metric, sort, order, metricParams, lvp, ds, rndTemplates[c]);
            }

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);
            }

            if (c % LOG_FREQ == 0) {
                StringBuffer ib = new StringBuffer("Iteration: ").append(c + 1).append('/').append(rndTemplates.length);
                ib.append(" for ").append(dstName);
                //sout.println(ib.toString());    // dont use log!
                System.out.println(ib.toString());
            }

            // DO THE RND CALC
            // @note better to just clone the existing real gcoh rather than generate a whole new one
            // as only the ranked list has changed and not the feature or gene set content
            final GeneSetCohort gcohRnd = gcohgen.createGeneSetCohort(rndRl, gsets, false, false);
            //System.out.println("starting calc: " + gcoh.getNumGeneSets());
            final EnrichmentScore[] rndScores = core.calculateKSScore(gcohRnd, false);
            //System.out.println("done calc");

            for (int g = 0; g < gsets.length; g++) {
                rndEss[g].setElement(c, rndScores[g].getES());
            }

            ptest.addRnd(rndTemplates[c], rndRl);

        } // End computation loop

        // 1 result for every gene set
        final EnrichmentResult[] results = new EnrichmentResult[gsets.length];
        for (int g = 0; g < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], chip, realScores[g], rndEss[g]);
        }

        ptest.doCalc();

        return new EnrichmentDbImplWithPermTest(dstName, rlReal, ds, template,
                results, lvp, metric, metricParams, sort, order, rndTemplates.length, null, ptest);
    }

    public EnrichmentDb shuffleTemplate_canned_templates_gsea(final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);
        final Chip chip = ds.getAnnot().getChip();

        final DatasetMetrics dm = new DatasetMetrics();
        final RankedList rlReal;
        PermutationTestBuilder ptest = new PermutationTestBuilder(dstName, numMarkers, rndTemplates.length, lvp,
                metric, sort, order, metricParams, ds, template, null, template.isCategorical());

        if (realRankedList_pre_calculated_opt != null) {
            rlReal = realRankedList_pre_calculated_opt;
        } else {
            rlReal = dm.scoreDataset(metric, sort, order, metricParams, lvp, ds, template);
        }

        // calc real scores
        if (rlReal.getSize() != ds.getNumRow()) {// sanity check
            throw new MismatchedSizeException();
        }

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true); // @note ASSUME already qualified
        final EnrichmentScore[] realScores = core.calculateKSScore(gcoh, true); // need to store details as we need the hit indices
        final Vector[] rndEss = new Vector[gsets.length];
        for (int g = 0; g < gsets.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

        // Each row is a "geneset", and each column a randomization
        for (int c = 0; c < rndTemplates.length; c++) {
            final RankedList rndRl;

            if (rndRankedLists_pre_calculated_opt != null) {
                rndRl = rndRankedLists_pre_calculated_opt[c];
            } else {
                rndRl = dm.scoreDataset(metric, sort, order, metricParams, lvp, ds, rndTemplates[c]);
            }

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);
            }

            if (c % LOG_FREQ == 0) {
                StringBuffer ib = new StringBuffer("Iteration: ").append(c + 1).append('/').append(rndTemplates.length);
                ib.append(" for ").append(dstName);
                //sout.println(ib.toString());    // dont use log!
                System.out.println(ib.toString());
            }

            // DO THE RND CALC
            // @note better to just clone the existing real gcoh rather than generate a whole new one
            // as only the ranked list has changed and not the feature or gene set content
            final GeneSetCohort gcohRnd = gcohgen.createGeneSetCohort(rndRl, gsets, false, false);
            //System.out.println("starting calc: " + gcoh.getNumGeneSets());
            final EnrichmentScore[] rndScores = core.calculateKSScore(gcohRnd, false);
            //System.out.println("done calc");

            for (int g = 0; g < gsets.length; g++) {
                rndEss[g].setElement(c, rndScores[g].getES());
            }

            ptest.addRnd(rndTemplates[c], rndRl);

        } // End computation loop

        // 1 result for every gene set
        final EnrichmentResult[] results = new EnrichmentResult[gsets.length];
        for (int g = 0; g < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], chip, realScores[g], rndEss[g]);
        }

        ptest.doCalc();

        return new EnrichmentDbImplWithPermTest(dstName, rlReal, ds, template,
                results, lvp, metric, metricParams, sort, order, rndTemplates.length, null, ptest);
    }

    public EnrichmentDb shuffleTemplate_canned_templates_gsaaseqsp_ks(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final List store_rnd_ranked_lists_here_opt) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);
        //final Chip chip = ds.getAnnot().getChip();

        ArrayList descores = new ArrayList();
        try {
            if (demetric.equalsIgnoreCase("Signal2Noise_log2Ratio")) {
                ArrayList diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(diffs);
                ArrayList ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(ratios);
                GsaaSeqTool.normalizeByMeanSdRow(diffs, ratios);
                descores = GsaaSeqTool.combineByMeanOfScores(diffs, ratios);
                ArrayList descores_direct = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template, rndTemplates);
                for (int i = 0; i < descores.size(); i++) {
                    for (int j = 0; j < ((ArrayList) descores.get(i)).size(); j++) {
                        double d1 = ((Double) ((ArrayList)descores_direct.get(i)).get(j)).doubleValue();
                        double d2 = ((Double) ((ArrayList)descores.get(i)).get(j)).doubleValue();
                        if (d1 < 0) {
                            ((ArrayList)descores.get(i)).set(j, new Double(d2 * (-1)));
                        }
                    }
                }
            } else if (demetric.equalsIgnoreCase("Signal2Noise")) {
                descores = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template, rndTemplates);
                //GsaaSeqTool.normalizeByTotalCol(descores);
            } else if (demetric.equalsIgnoreCase("log2Ratio")) {
                descores = GsaaSeqTool.calcScoresLog2RatioSeq(ds, template, rndTemplates);
                //GsaaSeqTool.normalizeByTotalCol(descores);
            } else {
                throw (new Exception("Metric for defferential expression analysis is not applicable"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        final DatasetMetrics dm = new DatasetMetrics();
        final RankedList rlReal;

        if (realRankedList_pre_calculated_opt != null) {
            rlReal = realRankedList_pre_calculated_opt;
        } else {
            rlReal = dm.scoreDataset_gsaaseqsp_ks(descores, sort, order, lvp, ds, 0);
        }

        if (rlReal.getSize() != ds.getNumRow()) {
            throw new MismatchedSizeException();
        }

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true);
        final EnrichmentScore[] realScores = core.calculateKSScore(gcoh, true);
        final Vector[] rndEss = new Vector[gsets.length];
        for (int g = 0; g < gsets.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

        // Each row is a "geneset", and each column a randomization
        for (int c = 0; c < rndTemplates.length; c++) {
            final RankedList rndRl;

            if (rndRankedLists_pre_calculated_opt != null) {
                rndRl = rndRankedLists_pre_calculated_opt[c];
            } else {
                rndRl = dm.scoreDataset_gsaaseqsp_ks(descores, sort, order, lvp, ds, c + 1);
            }

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);
            }

            if (c % LOG_FREQ == 0) {
                StringBuffer ib = new StringBuffer("Iteration: ").append(c + 1).append('/').append(rndTemplates.length);
                ib.append(" for ").append(dstName);
                System.out.println(ib.toString());
            }

            // DO THE RND CALC
            // @note better to just clone the existing real gcoh rather than generate a whole new one
            // as only the ranked list has changed and not the feature or gene set content
            final GeneSetCohort gcohRnd = gcohgen.createGeneSetCohort(rndRl, gsets, false, false);
            //System.out.println("starting calc: " + gcoh.getNumGeneSets());
            final EnrichmentScore[] rndScores = core.calculateKSScore(gcohRnd, false);
            //System.out.println("done calc");

            for (int g = 0; g < gsets.length; g++) {
                rndEss[g].setElement(c, rndScores[g].getES());
            }

        } // End computation loop

        // 1 result for every gene set
        final EnrichmentResult[] results = new EnrichmentResult[gsets.length];
        for (int g = 0; g < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], null, realScores[g], rndEss[g]);
        }

        return new EnrichmentDbImplSeq_one_shared_rl(dstName, rlReal, ds, template,
                results, lvp, sort, order, rndTemplates.length, null);
    }

    public EnrichmentDb shuffleTemplate_canned_templates_gsaaseqsp_p(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final List store_rnd_ranked_lists_here_opt) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);

        ArrayList descores_direc = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template, rndTemplates);

        ArrayList descores = new ArrayList();
        ArrayList descoresP = new ArrayList();
        try {
            if (demetric.equalsIgnoreCase("Signal2Noise_log2Ratio")) {
                ArrayList diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(diffs);
                ArrayList ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(ratios);
                GsaaSeqTool.normalizeByMeanSdRow(diffs, ratios);
                descores = GsaaSeqTool.combineByMeanOfScores(diffs, ratios);
                descoresP = GsaaSeqTool.calcP(ratios, diffs);
            } else if (demetric.equalsIgnoreCase("Signal2Noise")) {
                descores = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(descores);
                descoresP = GsaaSeqTool.calcpMoreThanD(descores);
            } else if (demetric.equalsIgnoreCase("log2Ratio")) {
                descores = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(descores);
                descoresP = GsaaSeqTool.calcpMoreThanD(descores);
            } else {
                throw (new Exception("Metric for defferential expression analysis is not applicable"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }           
               
        for (int i = 0; i < descores.size(); i++) {
            for (int j = 0; j < ((ArrayList) descores.get(i)).size(); j++) {
                double d1 = ((Double) ((ArrayList) descores_direc.get(i)).get(j)).doubleValue();
                double d2 = ((Double) ((ArrayList) descores.get(i)).get(j)).doubleValue();
                double d3 = ((Double) ((ArrayList) descoresP.get(i)).get(j)).doubleValue();
                if (d1 < 0) {
                    ((ArrayList) descores.get(i)).set(j, new Double(d2 * (-1)));
                    ((ArrayList) descoresP.get(i)).set(j, new Double(d3 * (-1)));
                }
            }
        }       

        final DatasetMetrics dm = new DatasetMetrics();
        ArrayList rlAll = new ArrayList();
        ArrayList rlAllP = new ArrayList();
        ArrayList rlAllRank = new ArrayList();

        final RankedList rlReal = dm.scoreDataset_gsaaseqsp_p(descores, sort, order, lvp, ds, 0);
        final RankedList rlRealP = dm.scoreDataset_gsaaseqsp_p(descoresP, sort, order, lvp, ds, 0);    
        final RankedList rlRealRank = dm.scoreDataset_gsaaseqsp_p(descores, SortMode.ABSOLUTE, order, lvp, ds, 0);

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true);
        rlAll.add(rlReal);
        rlAllP.add(rlRealP);
        rlAllRank.add(rlRealRank);

        if (rlReal.getSize() != ds.getNumRow()) {
            throw new MismatchedSizeException();
        }

        for (int c = 0; c < rndTemplates.length; c++) {
            final RankedList rndRl = dm.scoreDataset_gsaaseqsp_p(descores, sort, order, lvp, ds, c + 1);
            final RankedList rndRlP = dm.scoreDataset_gsaaseqsp_p(descoresP, sort, order, lvp, ds, c + 1);
            final RankedList rndRlRank = dm.scoreDataset_gsaaseqsp_p(descores, SortMode.ABSOLUTE, order, lvp, ds, c + 1);

            rlAll.add(rndRl);
            rlAllP.add(rndRlP);
            rlAllRank.add(rndRlRank);

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);                
            }
        }

        BigDecimal[][] ess = new BigDecimal[gcoh.getNumGeneSets()][rlAll.size()];
        GeneSet[] genesets = gcoh.getGeneSets();

        for (int i = 0; i < genesets.length; i++) {
            ArrayList al = new ArrayList();
            ArrayList alP = new ArrayList();
            ArrayList alRank = new ArrayList();
            String[] members = genesets[i].getMembersArray();
            for (int j = 0; j < rlAll.size(); j++) {
                float[] scores = new float[members.length];
                float[] scoresP = new float[members.length];
                int[] ranks = new int[members.length];
                for (int a = 0; a < members.length; a++) {
                    scores[a] = ((RankedList) rlAll.get(j)).getScore(members[a]);
                    scoresP[a] = ((RankedList) rlAllP.get(j)).getScore(members[a]);
                    ranks[a] = ((RankedList) rlAllRank.get(j)).getRank(members[a]);
                }

                if (j == 0) {
                    for (int k = 0; k < scores.length; k++) {
                        ArrayList temp = new ArrayList();
                        temp.add(new Float(scores[k]));
                        al.add(temp);
                        ArrayList tempP = new ArrayList();
                        tempP.add(new Float(scoresP[k]));
                        alP.add(tempP);
                        ArrayList tempRank = new ArrayList();
                        tempRank.add(new Integer(ranks[k]));
                        alRank.add(tempRank);
                    }
                } else {
                    for (int k = 0; k < scores.length; k++) {
                        ((ArrayList) al.get(k)).add(new Float(scores[k]));
                        ((ArrayList) alP.get(k)).add(new Float(scoresP[k]));
                        ((ArrayList) alRank.get(k)).add(new Integer(ranks[k]));
                    }
                }
            }

            ArrayList values = new ArrayList();           
            try {
                if (gsametric.equalsIgnoreCase("L2Norm")) {
                    values = BigDecimalTool.calcL2NormTDirect(al);
                } else if (gsametric.equalsIgnoreCase("GeometricMean")) {
                    values = BigDecimalTool.calcGMTDirect(alP);
                } else if (gsametric.equalsIgnoreCase("Mean")) {
                    values = BigDecimalTool.calcMeanTDirect(al);
                } else if (gsametric.equalsIgnoreCase("FisherMethod")) {
                    values = BigDecimalTool.calcFisherScoreTDirect(alP);
                //} else if (gsametric.equalsIgnoreCase("RankSum")) {
                    //values = BigDecimalTool.calcRankSumTDirect(al, alRank);
                } else if (gsametric.equalsIgnoreCase("WeightedSigRatio")) {
                    values = BigDecimalTool.calcSigLnNormRatioTDirect(al, alP, pthreshold);
                } else if (gsametric.equalsIgnoreCase("SigRatio")) {
                    values = BigDecimalTool.calcSigNumRatioTDirect(alP, pthreshold);
                } else {
                    throw new Exception("Metric for gene set analysis is not recognizable");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            for (int l = 0; l < values.size(); l++) {
                ess[i][l] = (BigDecimal) values.get(l);
            }
        }
        
        /*
        BigDecimal[][] ess_abs = new BigDecimal[ess.length][ess[0].length];
        for(int m=0; m<ess.length; m++){
            for(int n=0; n<ess[m].length; n++){
                ess_abs[m][n]=ess[m][n].abs();
            }
        }

        BigDecimal[][] normess = BigDecimalTool.normalizeMean(ess_abs);
        
        for(int m=0; m<ess.length; m++){
            for(int n=0; n<ess[m].length; n++){
                int sign = ess[m][n].compareTo(BigDecimal.ZERO);
                if (sign == -1) {
                    normess[m][n] = normess[m][n].multiply(new BigDecimal(-1));
                }
            }
        }*/
        
        BigDecimal[][] normess = BigDecimalTool.normalizeMeanDirect(ess);

        BigDecimal[] nps;
        BigDecimal[] fdrs;
        BigDecimal[] fwers;
        if (gsametric.equalsIgnoreCase("GeometricMean") || gsametric.equalsIgnoreCase("RankSum") ) {
            nps = BigDecimalTool.calcPBDirec(normess);
            fdrs = BigDecimalTool.calcFdrBDirec(normess);
            fwers = BigDecimalTool.calcFwerBDirec(normess);
        } else {
            nps = BigDecimalTool.calcPBMoreThanDirec(normess);
            fdrs = BigDecimalTool.calcFdrBMoreThanDirec(normess);
            fwers = BigDecimalTool.calcFwerBMoreThanDirec(normess);
        }

        /*try {
         FileWriter output = new FileWriter("/Users/qx/simulation/test.txt");
         BufferedWriter bw = new BufferedWriter(output);
         bw.write("ess");
         bw.newLine();
         for (int i = 0; i < ess.length; i++) {
         for (int j = 0; j < ess[0].length; j++) {
         bw.write(ess[i][j] + "\t");
         }
         bw.newLine();
         }
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("normess");
         bw.newLine();
         for (int i = 0; i < normess.length; i++) {
         for (int j = 0; j < normess[0].length; j++) {
         bw.write(normess[i][j] + "\t");
         }
         bw.newLine();
         }
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("np");
         bw.newLine();
         for (int i = 0; i < nps.length; i++) {
         bw.write(nps[i] + "\t");
         bw.newLine();
         }
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("fdrs");
         bw.newLine();
         for (int i = 0; i < fdrs.length; i++) {
         bw.write(fdrs[i] + "\t");
         bw.newLine();
         }
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("fwers");
         bw.newLine();
         for (int i = 0; i < fwers.length; i++) {
         bw.write(fwers[i] + "\t");
         bw.newLine();
         }
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.write("-----------------------");
         bw.newLine();
         bw.close();
         } catch (Exception e) {
         e.printStackTrace();
         }*/

        final EnrichmentScore[] realScores = new EnrichmentScore[gcoh.getNumGeneSets()];

        final Vector[] rndEss = new Vector[gcoh.getNumGeneSets()];
        for (int g = 0; g < rndEss.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

        for (int c = 0; c < rlAll.size(); c++) {
            if (c == 0) {
                for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                    realScores[g] = new EnrichmentScoreImplP(ess[g][c], normess[g][c], nps[g], fdrs[g], fwers[g]);
                }
            } else {
                final EnrichmentScore[] rndScores = new EnrichmentScore[gcoh.getNumGeneSets()];
                for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                    rndEss[g].setElementB(c - 1, ess[g][c]);
                }
            }
        }
        final EnrichmentResult[] results = new EnrichmentResult[gcoh.getNumGeneSets()];
        for (int g = 0; g < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], null, realScores[g], rndEss[g]);
        }

        return new EnrichmentDbImplSeq_one_shared_rl(dstName, rlReal, ds, template,
                results, lvp, sort, order, rndTemplates.length, null);
    }
    
    public EnrichmentDb shuffleTemplate_canned_templates_gsaaseqsp_p1(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final List store_rnd_ranked_lists_here_opt) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);

        ArrayList descores_direc = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template, rndTemplates);

        ArrayList descores = new ArrayList();
        try {
            if (demetric.equalsIgnoreCase("Signal2Noise_log2Ratio")) {
                ArrayList diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(diffs);
                ArrayList ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(ratios);
                GsaaSeqTool.normalizeByMeanSdRow(diffs, ratios);
                descores = GsaaSeqTool.combineByMeanOfScores(diffs, ratios);
            } else if (demetric.equalsIgnoreCase("Signal2Noise")) {
                descores = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(descores);
            } else if (demetric.equalsIgnoreCase("log2Ratio")) {
                descores = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(descores);
            } else {
                throw (new Exception("Metric for defferential expression analysis is not applicable"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }    
               
        for (int i = 0; i < descores.size(); i++) {
            for (int j = 0; j < ((ArrayList) descores.get(i)).size(); j++) {
                double d1 = ((Double) ((ArrayList) descores_direc.get(i)).get(j)).doubleValue();
                double d2 = ((Double) ((ArrayList) descores.get(i)).get(j)).doubleValue();
                if (d1 < 0) {
                    ((ArrayList) descores.get(i)).set(j, new Double(d2 * (-1)));
                }
            }
        }       

        final DatasetMetrics dm = new DatasetMetrics();
        ArrayList rlAll = new ArrayList();

        final RankedList rlReal = dm.scoreDataset_gsaaseqsp_p(descores, sort, order, lvp, ds, 0);   

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true);
        rlAll.add(rlReal);

        if (rlReal.getSize() != ds.getNumRow()) {
            throw new MismatchedSizeException();
        }

        for (int c = 0; c < rndTemplates.length; c++) {
            final RankedList rndRl = dm.scoreDataset_gsaaseqsp_p(descores, sort, order, lvp, ds, c + 1);

            rlAll.add(rndRl);

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);                
            }
        }

        BigDecimal[][] ess = new BigDecimal[gcoh.getNumGeneSets()][rlAll.size()];
        GeneSet[] genesets = gcoh.getGeneSets();

        for (int i = 0; i < genesets.length; i++) {
            ArrayList al = new ArrayList();
            String[] members = genesets[i].getMembersArray();
            for (int j = 0; j < rlAll.size(); j++) {
                float[] scores = new float[members.length];
                for (int a = 0; a < members.length; a++) {
                    scores[a] = ((RankedList) rlAll.get(j)).getScore(members[a]);
                }

                if (j == 0) {
                    for (int k = 0; k < scores.length; k++) {
                        ArrayList temp = new ArrayList();
                        temp.add(new Float(scores[k]));
                        al.add(temp);
                    }
                } else {
                    for (int k = 0; k < scores.length; k++) {
                        ((ArrayList) al.get(k)).add(new Float(scores[k]));
                    }
                }
            }

            ArrayList values = new ArrayList();  
            try {
                if (gsametric.equalsIgnoreCase("L2Norm")) {
                    values = BigDecimalTool.calcL2NormTDirect(al);
                } else if (gsametric.equalsIgnoreCase("Mean")) {
                    values = BigDecimalTool.calcMeanTDirect(al);
                } else {
                    throw new Exception("Metric for gene set analysis is not recognizable");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            for (int l = 0; l < values.size(); l++) {
                ess[i][l] = (BigDecimal) values.get(l);
            }
        }
        
        /*
        BigDecimal[][] ess_abs = new BigDecimal[ess.length][ess[0].length];
        for(int m=0; m<ess.length; m++){
            for(int n=0; n<ess[m].length; n++){
                ess_abs[m][n]=ess[m][n].abs();
            }
        }

        BigDecimal[][] normess = BigDecimalTool.normalizeMean(ess_abs);
        
        for(int m=0; m<ess.length; m++){
            for(int n=0; n<ess[m].length; n++){
                int sign = ess[m][n].compareTo(BigDecimal.ZERO);
                if (sign == -1) {
                    normess[m][n] = normess[m][n].multiply(new BigDecimal(-1));
                }
            }
        }*/
        
        BigDecimal[][] normess = BigDecimalTool.normalizeMeanDirect(ess);

        BigDecimal[] nps;
        BigDecimal[] fdrs;
        BigDecimal[] fwers;
        nps = BigDecimalTool.calcPBMoreThanDirec(normess);
        fdrs = BigDecimalTool.calcFdrBMoreThanDirec(normess);
        fwers = BigDecimalTool.calcFwerBMoreThanDirec(normess);

        final EnrichmentScore[] realScores = new EnrichmentScore[gcoh.getNumGeneSets()];

        final Vector[] rndEss = new Vector[gcoh.getNumGeneSets()];
        for (int g = 0; g < rndEss.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

        for (int c = 0; c < rlAll.size(); c++) {
            if (c == 0) {
                for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                    realScores[g] = new EnrichmentScoreImplP(ess[g][c], normess[g][c], nps[g], fdrs[g], fwers[g]);
                }
            } else {
                final EnrichmentScore[] rndScores = new EnrichmentScore[gcoh.getNumGeneSets()];
                for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                    rndEss[g].setElementB(c - 1, ess[g][c]);
                }
            }
        }
        final EnrichmentResult[] results = new EnrichmentResult[gcoh.getNumGeneSets()];
        for (int g = 0; g < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], null, realScores[g], rndEss[g]);
        }

        return new EnrichmentDbImplSeq_one_shared_rl(dstName, rlReal, ds, template,
                results, lvp, sort, order, rndTemplates.length, null);
    }
    
    public EnrichmentDb shuffleTemplate_canned_templates_gsaaseqsp_p2(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final List store_rnd_ranked_lists_here_opt) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);

        ArrayList descores_direc = GsaaSeqTool.calcScoresSignal2NoiseSeq(ds, template, rndTemplates);

        ArrayList descores = new ArrayList();
        ArrayList descoresP = new ArrayList();
        try {
            if (demetric.equalsIgnoreCase("Signal2Noise_log2Ratio")) {
                ArrayList diffs = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(diffs);
                ArrayList ratios = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(ratios);
                GsaaSeqTool.normalizeByMeanSdRow(diffs, ratios);
                descores = GsaaSeqTool.combineByMeanOfScores(diffs, ratios);
                descoresP = GsaaSeqTool.calcP(ratios, diffs);
            } else if (demetric.equalsIgnoreCase("Signal2Noise")) {
                descores = GsaaSeqTool.calcScoresSignal2NoiseSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(descores);
                descoresP = GsaaSeqTool.calcpMoreThanD(descores);
            } else if (demetric.equalsIgnoreCase("log2Ratio")) {
                descores = GsaaSeqTool.calcScoresLog2RatioSeqAbs(ds, template, rndTemplates);
                GsaaSeqTool.normalizeByTotalCol(descores);
                descoresP = GsaaSeqTool.calcpMoreThanD(descores);
            } else {
                throw (new Exception("Metric for defferential expression analysis is not applicable"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }           
               
        for (int i = 0; i < descores.size(); i++) {
            for (int j = 0; j < ((ArrayList) descores.get(i)).size(); j++) {
                double d1 = ((Double) ((ArrayList) descores_direc.get(i)).get(j)).doubleValue();
                double d2 = ((Double) ((ArrayList) descores.get(i)).get(j)).doubleValue();
                double d3 = ((Double) ((ArrayList) descoresP.get(i)).get(j)).doubleValue();
                if (d1 < 0) {
                    ((ArrayList) descores.get(i)).set(j, new Double(d2 * (-1)));
                    ((ArrayList) descoresP.get(i)).set(j, new Double(d3 * (-1)));
                }
            }
        }       

        final DatasetMetrics dm = new DatasetMetrics();
        ArrayList rlAll = new ArrayList();
        ArrayList rlAllP = new ArrayList();

        final RankedList rlReal = dm.scoreDataset_gsaaseqsp_p(descores, sort, order, lvp, ds, 0);
        final RankedList rlRealP = dm.scoreDataset_gsaaseqsp_p(descoresP, sort, order, lvp, ds, 0);    

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true);
        rlAll.add(rlReal);
        rlAllP.add(rlRealP);

        if (rlReal.getSize() != ds.getNumRow()) {
            throw new MismatchedSizeException();
        }

        for (int c = 0; c < rndTemplates.length; c++) {
            final RankedList rndRl = dm.scoreDataset_gsaaseqsp_p(descores, sort, order, lvp, ds, c + 1);
            final RankedList rndRlP = dm.scoreDataset_gsaaseqsp_p(descoresP, sort, order, lvp, ds, c + 1);

            rlAll.add(rndRl);
            rlAllP.add(rndRlP);

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);                
            }
        }

        BigDecimal[][] ess = new BigDecimal[gcoh.getNumGeneSets()][rlAll.size()];
        GeneSet[] genesets = gcoh.getGeneSets();

        for (int i = 0; i < genesets.length; i++) {
            ArrayList al = new ArrayList();
            ArrayList alP = new ArrayList();
            String[] members = genesets[i].getMembersArray();
            for (int j = 0; j < rlAll.size(); j++) {
                float[] scores = new float[members.length];
                float[] scoresP = new float[members.length];
                for (int a = 0; a < members.length; a++) {
                    scores[a] = ((RankedList) rlAll.get(j)).getScore(members[a]);
                    scoresP[a] = ((RankedList) rlAllP.get(j)).getScore(members[a]);
                }

                if (j == 0) {
                    for (int k = 0; k < scores.length; k++) {
                        ArrayList temp = new ArrayList();
                        temp.add(new Float(scores[k]));
                        al.add(temp);
                        ArrayList tempP = new ArrayList();
                        tempP.add(new Float(scoresP[k]));
                        alP.add(tempP);
                    }
                } else {
                    for (int k = 0; k < scores.length; k++) {
                        ((ArrayList) al.get(k)).add(new Float(scores[k]));
                        ((ArrayList) alP.get(k)).add(new Float(scoresP[k]));
                    }
                }
            }

            ArrayList values = new ArrayList();           
            try {
                if (gsametric.equalsIgnoreCase("GeometricMean")) {
                    values = BigDecimalTool.calcGMTDirect(alP);
                } else if (gsametric.equalsIgnoreCase("FisherMethod")) {
                    values = BigDecimalTool.calcFisherScoreTDirect(alP);
                } else if (gsametric.equalsIgnoreCase("WeightedSigRatio")) {
                    values = BigDecimalTool.calcSigLnNormRatioTDirect(al, alP, pthreshold);
                } else if (gsametric.equalsIgnoreCase("SigRatio")) {
                    values = BigDecimalTool.calcSigNumRatioTDirect(alP, pthreshold);
                } else {
                    throw new Exception("Metric for gene set analysis is not recognizable");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            for (int l = 0; l < values.size(); l++) {
                ess[i][l] = (BigDecimal) values.get(l);
            }
        }
        
        /*
        BigDecimal[][] ess_abs = new BigDecimal[ess.length][ess[0].length];
        for(int m=0; m<ess.length; m++){
            for(int n=0; n<ess[m].length; n++){
                ess_abs[m][n]=ess[m][n].abs();
            }
        }

        BigDecimal[][] normess = BigDecimalTool.normalizeMean(ess_abs);
        
        for(int m=0; m<ess.length; m++){
            for(int n=0; n<ess[m].length; n++){
                int sign = ess[m][n].compareTo(BigDecimal.ZERO);
                if (sign == -1) {
                    normess[m][n] = normess[m][n].multiply(new BigDecimal(-1));
                }
            }
        }*/
        
        BigDecimal[][] normess = BigDecimalTool.normalizeMeanDirect(ess);

        BigDecimal[] nps;
        BigDecimal[] fdrs;
        BigDecimal[] fwers;
        if (gsametric.equalsIgnoreCase("GeometricMean")) {
            nps = BigDecimalTool.calcPBDirec(normess);
            fdrs = BigDecimalTool.calcFdrBDirec(normess);
            fwers = BigDecimalTool.calcFwerBDirec(normess);
        } else {
            nps = BigDecimalTool.calcPBMoreThanDirec(normess);
            fdrs = BigDecimalTool.calcFdrBMoreThanDirec(normess);
            fwers = BigDecimalTool.calcFwerBMoreThanDirec(normess);
        }

        final EnrichmentScore[] realScores = new EnrichmentScore[gcoh.getNumGeneSets()];

        final Vector[] rndEss = new Vector[gcoh.getNumGeneSets()];
        for (int g = 0; g < rndEss.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

        for (int c = 0; c < rlAll.size(); c++) {
            if (c == 0) {
                for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                    realScores[g] = new EnrichmentScoreImplP(ess[g][c], normess[g][c], nps[g], fdrs[g], fwers[g]);
                }
            } else {
                final EnrichmentScore[] rndScores = new EnrichmentScore[gcoh.getNumGeneSets()];
                for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
                    rndEss[g].setElementB(c - 1, ess[g][c]);
                }
            }
        }
        final EnrichmentResult[] results = new EnrichmentResult[gcoh.getNumGeneSets()];
        for (int g = 0; g < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], null, realScores[g], rndEss[g]);
        }

        return new EnrichmentDbImplSeq_one_shared_rl(dstName, rlReal, ds, template,
                results, lvp, sort, order, rndTemplates.length, null);
    }    

    public EnrichmentDb shuffleTemplate_canned_templates_integrated_ks(
            final Metric metric,
            final String imetric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final HashMap[] rndTemplateSet) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);
        final Chip chip = ds.getAnnot().getChip();
        String[] rowNames = ds.getRowNamesArray();

        List dsIntegrated = new ArrayList();
        ArrayList dsExpOriAbs = calcScoresAbs(ds, metric, template, rndTemplates, metricParams);
        List dsExpOri = calcScores(ds, metric, template, rndTemplates, metricParams);
        ArrayList dsExp = new ArrayList();
        HashMap dsSnp = new HashMap();
        if (imetric.equalsIgnoreCase("Fisher_Method")) {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    //GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, snpGeneStruc);
                    dsSnp = GsaaTool.calcPValue(snpGeneStruc);
                }
            }
            dsExp = GsaaTool.calcPValue(dsExpOriAbs);
            dsIntegrated = GsaaTool.combineByFisherMethod(rowNames, dsExp, dsSnp);
        } else if (imetric.equalsIgnoreCase("ZScore_Sum")) {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    dsSnp = snpGeneStruc;
                }
            }
            //GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, dsSnp);
            GsaaTool.normalizeByMeanSdRow(dsExpOriAbs, dsSnp);
            dsIntegrated = GsaaTool.combineBySumOfScores(rowNames, dsExpOriAbs, dsSnp);
        } else {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    //GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, snpGeneStruc);
                    dsSnp = GsaaTool.toRanks(snpGeneStruc);
                }
            }
            dsExp = GsaaTool.toRanks(dsExpOriAbs);
            dsIntegrated = GsaaTool.combineBySumOfScores(rowNames, dsExp, dsSnp);
        }
        GsaaTool.normalizeByMeanSdRow((ArrayList) dsIntegrated);

        final DatasetMetrics dm = new DatasetMetrics();
        final RankedList rlReal;
        final RankedList rlReal_exp;

        IntegratedPermutationTestBuilder ptest = new IntegratedPermutationTestBuilder(dstName, numMarkers, rndTemplates.length, lvp,
                metric, sort, order, metricParams, ds, dsIntegrated, rowNames, template, null, template.isCategorical(), datasets, templates, params);

        if (realRankedList_pre_calculated_opt != null) {
            rlReal = realRankedList_pre_calculated_opt;
            rlReal_exp = realRankedList_pre_calculated_opt;
        } else {
            rlReal = dm.scoreDataset_integrated(sort, order, lvp, ds, dsIntegrated, rowNames, datasets, templates, params, 0);
            rlReal_exp = dm.scoreDataset_integrated(sort, order, lvp, ds, dsExpOri, rowNames, datasets, templates, params, 0);
        }

        // calc real scores
        if (rlReal.getSize() != ds.getNumRow()) {// sanity check
            throw new MismatchedSizeException();
        }

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true); // @note ASSUME already qualified
        final GeneSetCohort gcoh_exp = gcohgen.createGeneSetCohort(rlReal_exp, gsets, false, true);
        final EnrichmentScore[] realScores = core.calculateKSScore_parallel(gcoh_exp, gcoh, true);
        final Vector[] rndEss = new Vector[gsets.length];
        for (int g = 0; g
                < gsets.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

        // Each row is a "geneset", and each column a randomization
        for (int c = 0; c
                < rndTemplates.length; c++) {
            final RankedList rndRl;
            final RankedList rndRl_exp;

            if (rndRankedLists_pre_calculated_opt != null) {
                rndRl = rndRankedLists_pre_calculated_opt[c];
                rndRl_exp = rndRankedLists_pre_calculated_opt[c];
            } else {
                rndRl = dm.scoreDataset_integrated(sort, order, lvp, ds, dsIntegrated, rowNames, datasets, rndTemplateSet[c], params, c + 1);
                rndRl_exp = dm.scoreDataset_integrated(sort, order, lvp, ds, dsExpOri, rowNames, datasets, rndTemplateSet[c], params, c + 1);
            }

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);
            }

            if (c % LOG_FREQ == 0) {
                StringBuffer ib = new StringBuffer("Iteration: ").append(c + 1).append('/').append(rndTemplates.length);
                ib.append(" for ").append(dstName);
                //sout.println(ib.toString());    // dont use log!
                System.out.println(ib.toString());
            }

            // DO THE RND CALC
            // @note better to just clone the existing real gcoh rather than generate a whole new one
            // as only the ranked list has changed and not the feature or gene set content
            final GeneSetCohort gcohRnd = gcohgen.createGeneSetCohort(rndRl, gsets, false, false);
            final GeneSetCohort gcohRnd_exp = gcohgen.createGeneSetCohort(rndRl_exp, gsets, false, false);
            //System.out.println("starting calc: " + gcoh.getNumGeneSets());
            //final EnrichmentScore[] rndScores = core.calculateKSScore(gcohRnd, false);
            final EnrichmentScore[] rndScores = core.calculateKSScore_parallel(gcohRnd_exp, gcohRnd, false);
            //System.out.println("done calc");

            for (int g = 0; g
                    < gsets.length; g++) {
                rndEss[g].setElement(c, rndScores[g].getES());
            }

            ptest.addRnd(rndTemplates[c], rndTemplateSet[c], rndRl);

        } // End computation loop

        // one result for every gene set
        final EnrichmentResult[] results = new EnrichmentResult[gsets.length];
        for (int g = 0; g
                < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], chip, realScores[g], rndEss[g]);
        }

        ptest.doCalc();

        return new EnrichmentDbImplWithPermTest(dstName, rlReal, ds, template,
                results, lvp, metric, metricParams, sort, order, rndTemplates.length, null, ptest);
    }

    public EnrichmentDb shuffleTemplate_canned_templates_integrated_fm(
            final Metric metric,
            final String imetric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final HashMap[] rndTemplateSet) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);
        final Chip chip = ds.getAnnot().getChip();
        String[] rowNames = ds.getRowNamesArray();

        List dsIntegrated = new ArrayList();
        ArrayList dsExpOriAbs = calcScoresAbs(ds, metric, template, rndTemplates, metricParams);
        List dsExpOri = calcScores(ds, metric, template, rndTemplates, metricParams);
        ArrayList dsExp = new ArrayList();
        HashMap dsSnp = new HashMap();
        if (imetric.equalsIgnoreCase("Fisher_Method")) {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, snpGeneStruc);
                    //values in dsSnp are larger than or equal to 0
                    dsSnp = GsaaTool.calcPValue(snpGeneStruc);
                }
            }
            dsExp = GsaaTool.calcPValue(dsExpOriAbs);
            dsIntegrated = GsaaTool.combineByFisherMethod(rowNames, dsExp, dsSnp);
        } else if (imetric.equalsIgnoreCase("ZScore_Sum")) {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    dsSnp = snpGeneStruc;
                }
            }
            GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, dsSnp);
            GsaaTool.normalizeByMeanSdRow(dsExpOriAbs, dsSnp);
            dsIntegrated = GsaaTool.combineBySumOfScores(rowNames, dsExpOriAbs, dsSnp);
        } else {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, snpGeneStruc);
                    dsSnp = GsaaTool.toRanks(snpGeneStruc);
                }
            }
            dsExp = GsaaTool.toRanks(dsExpOriAbs);
            dsIntegrated = GsaaTool.combineBySumOfScores(rowNames, dsExp, dsSnp);
        }

        List dsIntegratedP = GsaaTool.calcPValue((ArrayList) dsIntegrated);
        List dsExpOriP = GsaaTool.calcPTwoTailed_fm((ArrayList) dsExpOri);

        final DatasetMetrics dm = new DatasetMetrics();
        final RankedList rlReal;
        final RankedList rlReal_exp;

        IntegratedPermutationTestBuilder ptest = new IntegratedPermutationTestBuilder(dstName, numMarkers, rndTemplates.length, lvp,
                metric, sort, order, metricParams, ds, dsIntegratedP, rowNames, template, null, template.isCategorical(), datasets, templates, params);

        if (realRankedList_pre_calculated_opt != null) {
            rlReal = realRankedList_pre_calculated_opt;
            rlReal_exp = realRankedList_pre_calculated_opt;
        } else {
            //rlReal = dm.scoreDataset_m(metric, sort, order, metricParams, lvp, ds, template, datasets, templates, params, 0);
            rlReal = dm.scoreDataset_integrated(sort, order, lvp, ds, dsIntegratedP, rowNames, datasets, templates, params, 0);
            rlReal_exp = dm.scoreDataset_integrated(sort, order, lvp, ds, dsExpOriP, rowNames, datasets, templates, params, 0);
        }

// calc real scores
        if (rlReal.getSize() != ds.getNumRow()) {// sanity check
            throw new MismatchedSizeException();
        }

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true); // @note ASSUME already qualified
        final GeneSetCohort gcoh_exp = gcohgen.createGeneSetCohort(rlReal_exp, gsets, false, true);
        //final EnrichmentScore[] realScores = core.calculateKSScore_fm(gcoh, true); // need to store details as we need the hit indices
        final EnrichmentScore[] realScores = core.calculateKSScore_fm_parallel(gcoh_exp, gcoh, true);
        final Vector[] rndEss = new Vector[gsets.length];
        for (int g = 0; g
                < gsets.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

// Each row is a "geneset", and each column a randomization
        for (int c = 0; c
                < rndTemplates.length; c++) {
            final RankedList rndRl;
            final RankedList rndRl_exp;

            if (rndRankedLists_pre_calculated_opt != null) {
                rndRl = rndRankedLists_pre_calculated_opt[c];
                rndRl_exp = rndRankedLists_pre_calculated_opt[c];
            } else {
                //rndRl = dm.scoreDataset_m(metric, sort, order, metricParams, lvp, ds, rndTemplates[c], datasets, rndTemplateSet[c], params, c + 1);
                rndRl = dm.scoreDataset_integrated(sort, order, lvp, ds, dsIntegratedP, rowNames, datasets, rndTemplateSet[c], params, c + 1);
                rndRl_exp = dm.scoreDataset_integrated(sort, order, lvp, ds, dsExpOriP, rowNames, datasets, rndTemplateSet[c], params, c + 1);
            }

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);
            }

            if (c % LOG_FREQ == 0) {
                StringBuffer ib = new StringBuffer("Iteration: ").append(c + 1).append('/').append(rndTemplates.length);
                ib.append(" for ").append(dstName);
                //sout.println(ib.toString());    // dont use log!
                System.out.println(ib.toString());
            }

// DO THE RND CALC
// @note better to just clone the existing real gcoh rather than generate a whole new one
// as only the ranked list has changed and not the feature or gene set content
            final GeneSetCohort gcohRnd = gcohgen.createGeneSetCohort(rndRl, gsets, false, false);
            final GeneSetCohort gcohRnd_exp = gcohgen.createGeneSetCohort(rndRl_exp, gsets, false, false);
            //System.out.println("starting calc: " + gcoh.getNumGeneSets());
            //final EnrichmentScore[] rndScores = core.calculateKSScore_fm(gcohRnd, false);
            final EnrichmentScore[] rndScores = core.calculateKSScore_fm_parallel(gcohRnd_exp, gcohRnd, false);
            //System.out.println("done calc");

            for (int g = 0; g
                    < gsets.length; g++) {
                rndEss[g].setElement(c, rndScores[g].getES());
            }

            ptest.addRnd(rndTemplates[c], rndTemplateSet[c], rndRl);

        } // End computation loop

// 1 result for every gene set
        final EnrichmentResult[] results = new EnrichmentResult[gsets.length];
        for (int g = 0; g
                < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], chip, realScores[g], rndEss[g]);
        }

        ptest.doCalc();

        return new EnrichmentDbImplWithPermTest(dstName, rlReal, ds, template,
                results, lvp, metric, metricParams, sort, order, rndTemplates.length, null, ptest);
    }

    public EnrichmentDb shuffleTemplate_canned_templates_integrated_samgs(
            final Metric metric,
            final String imetric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final int numMarkers,
            final List store_rnd_ranked_lists_here_opt,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params,
            final HashMap[] rndTemplateSet) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);
        final Chip chip = ds.getAnnot().getChip();
        String[] rowNames = ds.getRowNamesArray();

        List dsIntegrated = new ArrayList();
        ArrayList dsExpOriAbs = calcScoresAbs(ds, metric, template, rndTemplates, metricParams);
        List dsExpOri = calcScores(ds, metric, template, rndTemplates, metricParams);
        ArrayList dsExp = new ArrayList();
        HashMap dsSnp = new HashMap();
        if (imetric.equalsIgnoreCase("Fisher_Method")) {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, snpGeneStruc);
                    //values in dsSnp are larger than or equal to 0
                    dsSnp = GsaaTool.calcPValue(snpGeneStruc);
                }
            }
            dsExp = GsaaTool.calcPValue(dsExpOriAbs);
            dsIntegrated = GsaaTool.combineByFisherMethod(rowNames, dsExp, dsSnp);
        } else if (imetric.equalsIgnoreCase("ZScore_Sum")) {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    dsSnp = snpGeneStruc;
                }
            }
            GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, dsSnp);
            GsaaTool.normalizeByMeanSdRow(dsExpOriAbs, dsSnp);
            dsIntegrated = GsaaTool.combineBySumOfScores(rowNames, dsExpOriAbs, dsSnp);
        } else {
            if (datasets.containsKey("SNP")) {
                HashMap snpGeneStruc = (HashMap) (datasets.get("SNP"));
                if (snpGeneStruc != null && snpGeneStruc.size() != 0) {
                    GsaaTool.normalizeByMeanSdCol(dsExpOriAbs, snpGeneStruc);
                    dsSnp = GsaaTool.toRanks(snpGeneStruc);
                }
            }
            dsExp = GsaaTool.toRanks(dsExpOriAbs);
            dsIntegrated = GsaaTool.combineBySumOfScores(rowNames, dsExp, dsSnp);
        }
        GsaaTool.normalizeByMeanSdRow((ArrayList) dsIntegrated);

        final DatasetMetrics dm = new DatasetMetrics();
        final RankedList rlReal;
        final RankedList rlReal_exp;

        IntegratedPermutationTestBuilder ptest = new IntegratedPermutationTestBuilder(dstName, numMarkers, rndTemplates.length, lvp,
                metric, sort, order, metricParams, ds, dsIntegrated, rowNames, template, null, template.isCategorical(), datasets, templates, params);

        if (realRankedList_pre_calculated_opt != null) {
            rlReal = realRankedList_pre_calculated_opt;
            rlReal_exp = realRankedList_pre_calculated_opt;
        } else {
            //rlReal = dm.scoreDataset_m(metric, sort, order, metricParams, lvp, ds, template, datasets, templates, params, 0);
            rlReal = dm.scoreDataset_integrated(sort, order, lvp, ds, dsIntegrated, rowNames, datasets, templates, params, 0);
            rlReal_exp = dm.scoreDataset_integrated(sort, order, lvp, ds, dsExpOri, rowNames, datasets, templates, params, 0);
        }

// calc real scores
        if (rlReal.getSize() != ds.getNumRow()) {// sanity check
            throw new MismatchedSizeException();
        }

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true); // @note ASSUME already qualified
        final GeneSetCohort gcoh_exp = gcohgen.createGeneSetCohort(rlReal_exp, gsets, false, true);
        //final EnrichmentScore[] realScores = core.calculateKSScore_samgs(gcoh, true); // need to store details as we need the hit indices
        final EnrichmentScore[] realScores = core.calculateKSScore_samgs_parallel(gcoh_exp, gcoh, true);
        final Vector[] rndEss = new Vector[gsets.length];
        for (int g = 0; g
                < gsets.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

// Each row is a "geneset", and each column a randomization
        for (int c = 0; c
                < rndTemplates.length; c++) {
            final RankedList rndRl;
            final RankedList rndRl_exp;

            if (rndRankedLists_pre_calculated_opt != null) {
                rndRl = rndRankedLists_pre_calculated_opt[c];
                rndRl_exp = rndRankedLists_pre_calculated_opt[c];
            } else {
                //rndRl = dm.scoreDataset_m(metric, sort, order, metricParams, lvp, ds, rndTemplates[c], datasets, rndTemplateSet[c], params, c + 1);
                rndRl = dm.scoreDataset_integrated(sort, order, lvp, ds, dsIntegrated, rowNames, datasets, rndTemplateSet[c], params, c + 1);
                rndRl_exp = dm.scoreDataset_integrated(sort, order, lvp, ds, dsExpOri, rowNames, datasets, rndTemplateSet[c], params, c + 1);
            }

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);
            }

            if (c % LOG_FREQ == 0) {
                StringBuffer ib = new StringBuffer("Iteration: ").append(c + 1).append('/').append(rndTemplates.length);
                ib.append(" for ").append(dstName);
                //sout.println(ib.toString());    // dont use log!
                System.out.println(ib.toString());
            }

// DO THE RND CALC
// @note better to just clone the existing real gcoh rather than generate a whole new one
// as only the ranked list has changed and not the feature or gene set content
            final GeneSetCohort gcohRnd = gcohgen.createGeneSetCohort(rndRl, gsets, false, false);
            final GeneSetCohort gcohRnd_exp = gcohgen.createGeneSetCohort(rndRl_exp, gsets, false, false);
            //System.out.println("starting calc: " + gcoh.getNumGeneSets());
            //final EnrichmentScore[] rndScores = core.calculateKSScore_samgs(gcohRnd, false);
            final EnrichmentScore[] rndScores = core.calculateKSScore_samgs_parallel(gcohRnd_exp, gcohRnd, false);
            //System.out.println("done calc");

            for (int g = 0; g
                    < gsets.length; g++) {
                rndEss[g].setElement(c, rndScores[g].getES());
            }

            ptest.addRnd(rndTemplates[c], rndTemplateSet[c], rndRl);

        } // End computation loop

// 1 result for every gene set
        final EnrichmentResult[] results = new EnrichmentResult[gsets.length];
        for (int g = 0; g
                < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], chip, realScores[g], rndEss[g]);
        }

        ptest.doCalc();

        return new EnrichmentDbImplWithPermTest(dstName, rlReal, ds, template,
                results, lvp, metric, metricParams, sort, order, rndTemplates.length, null, ptest);
    }

    public SnpEnrichmentDb shuffleTemplate_canned_templates_gsaasnp_ks(//final Metric metric,

            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gcohgen,
            final Template[] rndTemplates,
            final RankedList realRankedList_pre_calculated_opt,
            final RankedList[] rndRankedLists_pre_calculated_opt,
            final List store_rnd_ranked_lists_here_opt,
            final CMetric cmetric) {

        log.debug("shuffleTemplate with -- nperm: " + rndTemplates.length + " Order: " + order + " Sort: " + sort + " gsets: " + gsets.length);

        final String dstName = NamingConventions.generateName(ds, template, true);
        final Chip chip = ds.getAnnot().getChip();

        final DatasetMetrics dm = new DatasetMetrics();
        final RankedList rlReal;

        GsaasnpPermutationTestBuilder ptest = new GsaasnpPermutationTestBuilder(dstName, rndTemplates.length, lvp,
                sort, order, ds, template, null, template.isCategorical(), cmetric);

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }

        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        if (realRankedList_pre_calculated_opt != null) {
            rlReal = realRankedList_pre_calculated_opt;
        } else {
            rlReal = dm.scoreDataset_shuffleTemplate_gsaasnp_ks(sort, order, lvp, ds, template, 0);
        }

// calc real scores
        if (rlReal.getSize() != ds.getNumRow()) {// sanity check
            throw new MismatchedSizeException();
        }

        final GeneSetCohort gcoh = gcohgen.createGeneSetCohort(rlReal, gsets, false, true); // @note ASSUME already qualified
        final EnrichmentScore[] realScores = core.calculateKSScore_snp(gcoh, true);
        final Vector[] rndEss = new Vector[gsets.length];
        for (int g = 0; g
                < gsets.length; g++) {
            rndEss[g] = new Vector(rndTemplates.length);
        }

// Each row is a "geneset", and each column a randomization
        for (int c = 0; c
                < rndTemplates.length; c++) {
            final RankedList rndRl;

            if (rndRankedLists_pre_calculated_opt != null) {
                rndRl = rndRankedLists_pre_calculated_opt[c];
            } else {
                rndRl = dm.scoreDataset_shuffleTemplate_gsaasnp_ks(sort, order, lvp, ds, rndTemplates[c], c + 1);
            }

            if (store_rnd_ranked_lists_here_opt != null) {
                store_rnd_ranked_lists_here_opt.add(rndRl);
            }

            if (c % LOG_FREQ == 0) {
                StringBuffer ib = new StringBuffer("Iteration: ").append(c + 1).append('/').append(rndTemplates.length);
                ib.append(" for ").append(dstName);
                //sout.println(ib.toString());    // dont use log!
                System.out.println(ib.toString());
            }

// DO THE RND CALC
// @note better to just clone the existing real gcoh rather than generate a whole new one
// as only the ranked list has changed and not the feature or gene set content
            final GeneSetCohort gcohRnd = gcohgen.createGeneSetCohort(rndRl, gsets, false, false);
            //System.out.println("starting calc: " + gcoh.getNumGeneSets());
            //final EnrichmentScore[] rndScores = core.calculateKSScore_gsaasnp(gcohRnd, false);
            final EnrichmentScore[] rndScores = core.calculateKSScore_snp(gcohRnd, false);
            //System.out.println("done calc");

            for (int g = 0; g
                    < gsets.length; g++) {
                rndEss[g].setElement(c, rndScores[g].getES());
            }

            ptest.addRnd(rndTemplates[c], rndRl);

        } // End computation loop

// 1 result for every gene set
        final EnrichmentResult[] results = new EnrichmentResult[gsets.length];
        for (int g = 0; g
                < gsets.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, template, gsets[g], chip, realScores[g], rndEss[g]);
        }

        ptest.doCalc();

        return new SnpEnrichmentDbImplWithPermTest(dstName, rlReal, ds, template,
                results, lvp, cmetric, sort, order, rndTemplates.length, null, ptest);
    }

// ------------------------------------------------------------------------ //
// -------------------------------- GENE TAG CALCULATIONS ------------------//
// ------------------------------------------------------------------------ //
    /**
     * @param nperm
     * @param rlReal
     * @param gsetsReal
     * @param gcohgen
     * @param rst
     * @return
     * @throws Exception
     */
// this is the CORE method
    public EnrichmentResult[] shuffleGeneSet_precannedRankedList(final int nperm,
            final RankedList rlReal,
            final Template t_opt,
            final GeneSet[] gsetsReal,
            final Chip chip_opt,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean qualifyGeneSets,
            final boolean storeDeepForRL,
            final boolean storeRESPointByPoint_for_real) {

        final EnrichmentResult[] results = new EnrichmentResult[gsetsReal.length];
        final GeneSetCohort gcohReal = gcohgen.createGeneSetCohort(rlReal, gsetsReal, qualifyGeneSets, true);

        final EnrichmentScore[] real_scores = core.calculateKSScore(gcohReal, storeDeepForRL, storeRESPointByPoint_for_real); // @note ususally always store deep for the real one

        // The make rnd gene sets for every real one
        for (int g = 0; g
                < gsetsReal.length; g++) {
            if (g % LOG_FREQ == 0) {
                sout.println("shuffleGeneSet for GeneSet " + (g + 1) + "/" + gsetsReal.length + " nperm: " + nperm);
            }

// now create random GeneSets and calc the ksscore for every rnd GeneSet
//log.debug("started gsets");
            Vector rndEss;
            if (nperm > 0) {
                final GeneSet[] rndgsets = GeneSetGenerators.createRandomGeneSetsFixedSize(nperm, rlReal, gsetsReal[g], rst);
                final GeneSetCohort gcohRnd = gcohReal.clone(rndgsets, false);
                rndEss =
                        new Vector(rndgsets.length);
                final EnrichmentScore[] rnds = core.calculateKSScore(gcohRnd, false); // never store deep for rnds
                for (int r = 0; r
                        < rndgsets.length; r++) {
                    rndEss.setElement(r, rnds[r].getES());
                }

            } else {
                rndEss = new Vector(0);
            }

            results[g] = new EnrichmentResultImpl(rlReal, t_opt,
                    gsetsReal[g], chip_opt, real_scores[g], rndEss);
        }

        return results;
    }

    public EnrichmentResult[] shuffleGeneSet_precannedRankedList_gsaasnp(final int nperm,
            final RankedList rlReal,
            final Template t_opt,
            final GeneSet[] gsetsReal,
            final Chip chip_opt,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean qualifyGeneSets,
            final boolean storeDeepForRL,
            final boolean storeRESPointByPoint_for_real) {

        final EnrichmentResult[] results = new EnrichmentResult[gsetsReal.length];
        final GeneSetCohort gcohReal = gcohgen.createGeneSetCohort(rlReal, gsetsReal, qualifyGeneSets, true);

        final EnrichmentScore[] real_scores = core.calculateKSScore_gsaasnp(gcohReal, storeDeepForRL, storeRESPointByPoint_for_real); // @note ususally always store deep for the real one

        // The make rnd gene sets for every real one
        for (int g = 0; g
                < gsetsReal.length; g++) {
            if (g % LOG_FREQ == 0) {
                sout.println("shuffleGeneSet for GeneSet " + (g + 1) + "/" + gsetsReal.length + " nperm: " + nperm);
            }

// now create random GeneSets and calc the ksscore for every rnd GeneSet
//log.debug("started gsets");
            Vector rndEss;
            if (nperm > 0) {
                final GeneSet[] rndgsets = GeneSetGenerators.createRandomGeneSetsFixedSize(nperm, rlReal, gsetsReal[g], rst);
                final GeneSetCohort gcohRnd = gcohReal.clone(rndgsets, false);
                rndEss =
                        new Vector(rndgsets.length);
                final EnrichmentScore[] rnds = core.calculateKSScore_gsaasnp(gcohRnd, false); // never store deep for rnds
                for (int r = 0; r
                        < rndgsets.length; r++) {
                    rndEss.setElement(r, rnds[r].getES());
                }

            } else {
                rndEss = new Vector(0);
            }

            results[g] = new EnrichmentResultImpl(rlReal, t_opt,
                    gsetsReal[g], chip_opt, real_scores[g], rndEss);
        }

        return results;
    }

    public EnrichmentResult[] shuffleGeneSet_precannedRankedList_gsaaseqgp(final int nperm,
            final RankedList rlReal,
            final Template t_opt,
            final GeneSet[] gsetsReal,
            final Chip chip_opt,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean qualifyGeneSets,
            final boolean storeDeepForRL,
            final boolean storeRESPointByPoint_for_real) {

        final EnrichmentResult[] results = new EnrichmentResult[gsetsReal.length];
        final GeneSetCohort gcohReal = gcohgen.createGeneSetCohort(rlReal, gsetsReal, qualifyGeneSets, true);

        final EnrichmentScore[] real_scores = core.calculateKSScore_seq_ks(gcohReal, storeDeepForRL, storeRESPointByPoint_for_real); // @note ususally always store deep for the real one

        // The make rnd gene sets for every real one
        for (int g = 0; g
                < gsetsReal.length; g++) {
            if (g % LOG_FREQ == 0) {
                sout.println("shuffleGeneSet for GeneSet " + (g + 1) + "/" + gsetsReal.length + " nperm: " + nperm);
            }

// now create random GeneSets and calc the ksscore for every rnd GeneSet
//log.debug("started gsets");
            Vector rndEss;
            if (nperm > 0) {
                final GeneSet[] rndgsets = GeneSetGenerators.createRandomGeneSetsFixedSize(nperm, rlReal, gsetsReal[g], rst);
                final GeneSetCohort gcohRnd = gcohReal.clone(rndgsets, false);
                rndEss =
                        new Vector(rndgsets.length);
                final EnrichmentScore[] rnds = core.calculateKSScore_seq_ks(gcohRnd, false); // never store deep for rnds
                for (int r = 0; r
                        < rndgsets.length; r++) {
                    rndEss.setElement(r, rnds[r].getES());
                }

            } else {
                rndEss = new Vector(0);
            }

            results[g] = new EnrichmentResultImpl(rlReal, t_opt,
                    gsetsReal[g], chip_opt, real_scores[g], rndEss);
        }

        return results;
    }

    public EnrichmentResult[] shuffleGeneSet_precannedRankedList_seq_ks(final int nperm,
            final RankedList rlReal,
            final Template t_opt,
            final GeneSet[] gsetsReal,
            final Chip chip_opt,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean qualifyGeneSets,
            final boolean storeDeepForRL,
            final boolean storeRESPointByPoint_for_real) {

        final EnrichmentResult[] results = new EnrichmentResult[gsetsReal.length];
        final GeneSetCohort gcohReal = gcohgen.createGeneSetCohort(rlReal, gsetsReal, qualifyGeneSets, true);

        final EnrichmentScore[] real_scores = core.calculateKSScore_seq_ks(gcohReal, storeDeepForRL, storeRESPointByPoint_for_real); // @note ususally always store deep for the real one

        // The make rnd gene sets for every real one
        for (int g = 0; g
                < gsetsReal.length; g++) {
            if (g % LOG_FREQ == 0) {
                sout.println("shuffleGeneSet for GeneSet " + (g + 1) + "/" + gsetsReal.length + " nperm: " + nperm);
            }

            // now create random GeneSets and calc the ksscore for every rnd GeneSet
            //log.debug("started gsets");
            Vector rndEss;
            if (nperm > 0) {
                final GeneSet[] rndgsets = GeneSetGenerators.createRandomGeneSetsFixedSize(nperm, rlReal, gsetsReal[g], rst);
                final GeneSetCohort gcohRnd = gcohReal.clone(rndgsets, false);
                rndEss = new Vector(rndgsets.length);
                final EnrichmentScore[] rnds = core.calculateKSScore_seq_ks(gcohRnd, false); // never store deep for rnds
                for (int r = 0; r
                        < rndgsets.length; r++) {
                    rndEss.setElement(r, rnds[r].getES());
                }

            } else {
                rndEss = new Vector(0);
            }

            results[g] = new EnrichmentResultImpl(rlReal, t_opt,
                    gsetsReal[g], chip_opt, real_scores[g], rndEss);
        }

        return results;
    }

    public EnrichmentResult[] shuffleGeneSet_precannedRankedList_seq_p(
            final String gsametric,
            final int nperm,
            final RankedList rlReal,
            final RankedList rlRealP,
            final RankedList rlRealDirec,
            final Template t_opt,
            final GeneSet[] gsetsReal,
            final Chip chip_opt,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean qualifyGeneSets,
            final boolean storeDeepForRL,
            final boolean storeRESPointByPoint_for_real,
            final String pthreshold) {

        BigDecimal[][] ess = new BigDecimal[gsetsReal.length][nperm + 1];

        // The make rnd gene sets for every real one
        for (int g = 0; g < gsetsReal.length; g++) {
            if (g % LOG_FREQ == 0) {
                sout.println("shuffleGeneSet for GeneSet " + (g + 1) + "/" + gsetsReal.length + " nperm: " + nperm);
            }

            // now create random GeneSets and calc the ksscore for every rnd GeneSet
            //log.debug("started gsets");

            ArrayList al = new ArrayList();
            ArrayList alP = new ArrayList();
            ArrayList alRank = new ArrayList();
            String[] members = gsetsReal[g].getMembersArray();
            float[] scores = new float[members.length];
            float[] scoresP = new float[members.length];
            int[] ranks = new int[members.length];
            float[] scoresDirect = new float[members.length];
            for (int i = 0; i < members.length; i++) {
                scores[i] = rlReal.getScore(members[i]);
                scoresP[i] = rlRealP.getScore(members[i]);
                ranks[i] = rlReal.getRank(members[i]);
                scoresDirect[i] = rlRealDirec.getScore(members[i]);
                if(scoresDirect[i]<0){
                    scores[i]=scores[i]*(-1);
                    scoresP[i]=scoresP[i]*(-1);
                }
            }
            for (int j = 0; j < scores.length; j++) {
                ArrayList temp = new ArrayList();
                temp.add(new Float(scores[j]));
                al.add(temp);

                ArrayList tempP = new ArrayList();
                tempP.add(new Float(scoresP[j]));
                alP.add(tempP);

                ArrayList tempRank = new ArrayList();
                tempRank.add(new Integer(ranks[j]));
                alRank.add(tempRank);
            }
            if (nperm > 0) {
                final GeneSet[] rndgsets = GeneSetGenerators.createRandomGeneSetsFixedSize(nperm, rlReal, gsetsReal[g], rst);
                for (int k = 0; k < rndgsets.length; k++) {
                    String[] rndmembers = rndgsets[k].getMembersArray();
                    float[] rndscores = new float[rndmembers.length];
                    float[] rndscoresP = new float[rndmembers.length];
                    int[] rndRanks = new int[rndmembers.length];
                    float[] rndscoresDirec = new float[rndmembers.length];
                    for (int l = 0; l < rndmembers.length; l++) {
                        rndscores[l] = rlReal.getScore(rndmembers[l]);
                        rndscoresP[l] = rlRealP.getScore(rndmembers[l]);
                        rndRanks[l] = rlReal.getRank(rndmembers[l]);
                        rndscoresDirec[l] = rlRealDirec.getScore(rndmembers[l]);
                        if(rndscoresDirec[l]<0){
                            rndscores[l]=rndscores[l]*(-1);
                            rndscoresP[l]=rndscoresP[l]*(-1);
                        }
                    }
                    for (int m = 0; m < rndscores.length; m++) {
                        ((ArrayList) al.get(m)).add(new Float(rndscores[m]));
                        ((ArrayList) alP.get(m)).add(new Float(rndscoresP[m]));
                        ((ArrayList) alRank.get(m)).add(new Integer(rndRanks[m]));
                    }
                }
            }

            ArrayList values = new ArrayList();
            try {
                if (gsametric.equalsIgnoreCase("L2Norm")) {
                    values = BigDecimalTool.calcL2NormTDirect(al);
                } else if (gsametric.equalsIgnoreCase("GeometricMean")) {
                    values = BigDecimalTool.calcGMTDirect(alP);
                } else if (gsametric.equalsIgnoreCase("Mean")) {
                    values = BigDecimalTool.calcMeanTDirect(al);
                } else if (gsametric.equalsIgnoreCase("FisherMethod")) {
                    values = BigDecimalTool.calcFisherScoreTDirect(alP);
                } else if (gsametric.equalsIgnoreCase("RankSum")) {
                    values = BigDecimalTool.calcRankSumTDirect(al, alRank);
                } else if (gsametric.equalsIgnoreCase("WeightedSigRatio")) {
                    values = BigDecimalTool.calcSigLnNormRatioTDirect(al, alP, pthreshold);
                } else if (gsametric.equalsIgnoreCase("SigRatio")) {
                    values = BigDecimalTool.calcSigNumRatioTDirect(alP, pthreshold);
                } else {
                    throw new Exception("Metric for gene set analysis is not recognizable");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            for (int l = 0; l < values.size(); l++) {
                ess[g][l] = (BigDecimal) values.get(l);
            }
        }

        BigDecimal[][] normess = BigDecimalTool.normalizeMeanDirect(ess);

        BigDecimal[] nps;
        BigDecimal[] fdrs;
        BigDecimal[] fwers;
        if (gsametric.equalsIgnoreCase("GeometricMean") || gsametric.equalsIgnoreCase("RankSum")) {
            nps = BigDecimalTool.calcPBDirec(normess);
            fdrs = BigDecimalTool.calcFdrBDirec(normess);
            fwers = BigDecimalTool.calcFwerBDirec(normess);
        } else {
            nps = BigDecimalTool.calcPBMoreThanDirec(normess);
            fdrs = BigDecimalTool.calcFdrBMoreThanDirec(normess);
            fwers = BigDecimalTool.calcFwerBMoreThanDirec(normess);
        }

        final EnrichmentScore[] realScores = new EnrichmentScore[gsetsReal.length];

        final Vector[] rndEss = new Vector[gsetsReal.length];
        for (int g = 0; g < rndEss.length; g++) {
            rndEss[g] = new Vector(nperm);
        }

        for (int c = 0; c < ess[0].length; c++) {
            if (c == 0) {
                for (int g = 0; g < gsetsReal.length; g++) {
                    realScores[g] = new EnrichmentScoreImplP(ess[g][c], normess[g][c], nps[g], fdrs[g], fwers[g]);
                }
            } else {
                final EnrichmentScore[] rndScores = new EnrichmentScore[gsetsReal.length];
                for (int g = 0; g < gsetsReal.length; g++) {
                    rndEss[g].setElementB(c - 1, ess[g][c]);
                }
            }
        }

        final EnrichmentResult[] results = new EnrichmentResult[gsetsReal.length];
        for (int g = 0; g < gsetsReal.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, t_opt, gsetsReal[g], null, realScores[g], rndEss[g]);
        }

        return results;
    }
    
    public EnrichmentResult[] shuffleGeneSet_precannedRankedList_seq_p1(
            final String gsametric,
            final int nperm,
            final RankedList rlReal,
            final RankedList rlRealDirec,
            final Template t_opt,
            final GeneSet[] gsetsReal,
            final Chip chip_opt,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean qualifyGeneSets,
            final boolean storeDeepForRL,
            final boolean storeRESPointByPoint_for_real,
            final String pthreshold) {

        BigDecimal[][] ess = new BigDecimal[gsetsReal.length][nperm + 1];

        // The make rnd gene sets for every real one
        for (int g = 0; g < gsetsReal.length; g++) {
            if (g % LOG_FREQ == 0) {
                sout.println("shuffleGeneSet for GeneSet " + (g + 1) + "/" + gsetsReal.length + " nperm: " + nperm);
            }

            // now create random GeneSets and calc the ksscore for every rnd GeneSet
            //log.debug("started gsets");

            ArrayList al = new ArrayList();
            String[] members = gsetsReal[g].getMembersArray();
            float[] scores = new float[members.length];
            float[] scoresDirect = new float[members.length];
            for (int i = 0; i < members.length; i++) {
                scores[i] = rlReal.getScore(members[i]);
                scoresDirect[i] = rlRealDirec.getScore(members[i]);
                if(scoresDirect[i]<0){
                    scores[i]=scores[i]*(-1);
                }
            }
            for (int j = 0; j < scores.length; j++) {
                ArrayList temp = new ArrayList();
                temp.add(new Float(scores[j]));
                al.add(temp);
            }
            if (nperm > 0) {
                final GeneSet[] rndgsets = GeneSetGenerators.createRandomGeneSetsFixedSize(nperm, rlReal, gsetsReal[g], rst);
                for (int k = 0; k < rndgsets.length; k++) {
                    String[] rndmembers = rndgsets[k].getMembersArray();
                    float[] rndscores = new float[rndmembers.length];
                    float[] rndscoresDirec = new float[rndmembers.length];
                    for (int l = 0; l < rndmembers.length; l++) {
                        rndscores[l] = rlReal.getScore(rndmembers[l]);
                        rndscoresDirec[l] = rlRealDirec.getScore(rndmembers[l]);
                        if(rndscoresDirec[l]<0){
                            rndscores[l]=rndscores[l]*(-1);
                        }
                    }
                    for (int m = 0; m < rndscores.length; m++) {
                        ((ArrayList) al.get(m)).add(new Float(rndscores[m]));
                    }
                }
            }

            ArrayList values = new ArrayList();
            try {
                if (gsametric.equalsIgnoreCase("L2Norm")) {
                    values = BigDecimalTool.calcL2NormTDirect(al);
                } else if (gsametric.equalsIgnoreCase("Mean")) {
                    values = BigDecimalTool.calcMeanTDirect(al);
                } else {
                    throw new Exception("Metric for gene set analysis is not recognizable");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            for (int l = 0; l < values.size(); l++) {
                ess[g][l] = (BigDecimal) values.get(l);
            }
        }

        BigDecimal[][] normess = BigDecimalTool.normalizeMeanDirect(ess);

        BigDecimal[] nps;
        BigDecimal[] fdrs;
        BigDecimal[] fwers;
        nps = BigDecimalTool.calcPBMoreThanDirec(normess);
        fdrs = BigDecimalTool.calcFdrBMoreThanDirec(normess);
        fwers = BigDecimalTool.calcFwerBMoreThanDirec(normess);

        final EnrichmentScore[] realScores = new EnrichmentScore[gsetsReal.length];

        final Vector[] rndEss = new Vector[gsetsReal.length];
        for (int g = 0; g < rndEss.length; g++) {
            rndEss[g] = new Vector(nperm);
        }

        for (int c = 0; c < ess[0].length; c++) {
            if (c == 0) {
                for (int g = 0; g < gsetsReal.length; g++) {
                    realScores[g] = new EnrichmentScoreImplP(ess[g][c], normess[g][c], nps[g], fdrs[g], fwers[g]);
                }
            } else {
                final EnrichmentScore[] rndScores = new EnrichmentScore[gsetsReal.length];
                for (int g = 0; g < gsetsReal.length; g++) {
                    rndEss[g].setElementB(c - 1, ess[g][c]);
                }
            }
        }

        final EnrichmentResult[] results = new EnrichmentResult[gsetsReal.length];
        for (int g = 0; g < gsetsReal.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, t_opt, gsetsReal[g], null, realScores[g], rndEss[g]);
        }

        return results;
    }
    
    public EnrichmentResult[] shuffleGeneSet_precannedRankedList_seq_p2(
            final String gsametric,
            final int nperm,
            final RankedList rlReal,
            final RankedList rlRealP,
            final RankedList rlRealDirec,
            final Template t_opt,
            final GeneSet[] gsetsReal,
            final Chip chip_opt,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean qualifyGeneSets,
            final boolean storeDeepForRL,
            final boolean storeRESPointByPoint_for_real,
            final String pthreshold) {

        BigDecimal[][] ess = new BigDecimal[gsetsReal.length][nperm + 1];

        // The make rnd gene sets for every real one
        for (int g = 0; g < gsetsReal.length; g++) {
            if (g % LOG_FREQ == 0) {
                sout.println("shuffleGeneSet for GeneSet " + (g + 1) + "/" + gsetsReal.length + " nperm: " + nperm);
            }

            // now create random GeneSets and calc the ksscore for every rnd GeneSet
            //log.debug("started gsets");

            ArrayList al = new ArrayList();
            ArrayList alP = new ArrayList();
            String[] members = gsetsReal[g].getMembersArray();
            float[] scores = new float[members.length];
            float[] scoresP = new float[members.length];
            float[] scoresDirect = new float[members.length];
            for (int i = 0; i < members.length; i++) {
                scores[i] = rlReal.getScore(members[i]);
                scoresP[i] = rlRealP.getScore(members[i]);
                scoresDirect[i] = rlRealDirec.getScore(members[i]);
                if(scoresDirect[i]<0){
                    scores[i]=scores[i]*(-1);
                    scoresP[i]=scoresP[i]*(-1);
                }
            }
            for (int j = 0; j < scores.length; j++) {
                ArrayList temp = new ArrayList();
                temp.add(new Float(scores[j]));
                al.add(temp);

                ArrayList tempP = new ArrayList();
                tempP.add(new Float(scoresP[j]));
                alP.add(tempP);
            }
            if (nperm > 0) {
                final GeneSet[] rndgsets = GeneSetGenerators.createRandomGeneSetsFixedSize(nperm, rlReal, gsetsReal[g], rst);
                for (int k = 0; k < rndgsets.length; k++) {
                    String[] rndmembers = rndgsets[k].getMembersArray();
                    float[] rndscores = new float[rndmembers.length];
                    float[] rndscoresP = new float[rndmembers.length];
                    float[] rndscoresDirec = new float[rndmembers.length];
                    for (int l = 0; l < rndmembers.length; l++) {
                        rndscores[l] = rlReal.getScore(rndmembers[l]);
                        rndscoresP[l] = rlRealP.getScore(rndmembers[l]);
                        rndscoresDirec[l] = rlRealDirec.getScore(rndmembers[l]);
                        if(rndscoresDirec[l]<0){
                            rndscores[l]=rndscores[l]*(-1);
                            rndscoresP[l]=rndscoresP[l]*(-1);
                        }
                    }
                    for (int m = 0; m < rndscores.length; m++) {
                        ((ArrayList) al.get(m)).add(new Float(rndscores[m]));
                        ((ArrayList) alP.get(m)).add(new Float(rndscoresP[m]));
                    }
                }
            }

            ArrayList values = new ArrayList();
            try {
                if (gsametric.equalsIgnoreCase("GeometricMean")) {
                    values = BigDecimalTool.calcGMTDirect(alP);
                } else if (gsametric.equalsIgnoreCase("FisherMethod")) {
                    values = BigDecimalTool.calcFisherScoreTDirect(alP);
                } else if (gsametric.equalsIgnoreCase("WeightedSigRatio")) {
                    values = BigDecimalTool.calcSigLnNormRatioTDirect(al, alP, pthreshold);
                } else if (gsametric.equalsIgnoreCase("SigRatio")) {
                    values = BigDecimalTool.calcSigNumRatioTDirect(alP, pthreshold);
                } else {
                    throw new Exception("Metric for gene set analysis is not recognizable");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            for (int l = 0; l < values.size(); l++) {
                ess[g][l] = (BigDecimal) values.get(l);
            }
        }

        BigDecimal[][] normess = BigDecimalTool.normalizeMeanDirect(ess);

        BigDecimal[] nps;
        BigDecimal[] fdrs;
        BigDecimal[] fwers;
        if (gsametric.equalsIgnoreCase("GeometricMean")) {
            nps = BigDecimalTool.calcPBDirec(normess);
            fdrs = BigDecimalTool.calcFdrBDirec(normess);
            fwers = BigDecimalTool.calcFwerBDirec(normess);
        } else {
            nps = BigDecimalTool.calcPBMoreThanDirec(normess);
            fdrs = BigDecimalTool.calcFdrBMoreThanDirec(normess);
            fwers = BigDecimalTool.calcFwerBMoreThanDirec(normess);
        }

        final EnrichmentScore[] realScores = new EnrichmentScore[gsetsReal.length];

        final Vector[] rndEss = new Vector[gsetsReal.length];
        for (int g = 0; g < rndEss.length; g++) {
            rndEss[g] = new Vector(nperm);
        }

        for (int c = 0; c < ess[0].length; c++) {
            if (c == 0) {
                for (int g = 0; g < gsetsReal.length; g++) {
                    realScores[g] = new EnrichmentScoreImplP(ess[g][c], normess[g][c], nps[g], fdrs[g], fwers[g]);
                }
            } else {
                final EnrichmentScore[] rndScores = new EnrichmentScore[gsetsReal.length];
                for (int g = 0; g < gsetsReal.length; g++) {
                    rndEss[g].setElementB(c - 1, ess[g][c]);
                }
            }
        }

        final EnrichmentResult[] results = new EnrichmentResult[gsetsReal.length];
        for (int g = 0; g < gsetsReal.length; g++) {
            results[g] = new EnrichmentResultImpl(rlReal, t_opt, gsetsReal[g], null, realScores[g], rndEss[g]);
        }

        return results;
    }    

    public EnrichmentDbMulti createEdb_precannedRankedLists(
            final String name,
            final int nperm,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final RankedList[] rls,
            final Template[] tss_opt,
            final GeneSet[] gsets_alrdy_qualified,
            final Chip chip,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean storeDeepES_forRLs,
            final boolean storeRESPointByPoint_for_real)
            throws Exception {

        EnrichmentDb[] edbs = new EnrichmentDb[rls.length];
        for (int r = 0; r
                < rls.length; r++) {
            Template t_opt = null;
            if (tss_opt != null) {
                t_opt = tss_opt[r];
            }

            EnrichmentResult[] results = shuffleGeneSet_precannedRankedList(nperm, rls[r], t_opt, gsets_alrdy_qualified, chip, gcohgen, rst, false, storeDeepES_forRLs, storeRESPointByPoint_for_real);
            edbs[r] = new EnrichmentDbImpl_one_shared_rl(rls[r].getName(), rls[r], null, t_opt, results, lvp, metric, metricParams, sort, order, nperm, null);
            System.out.println("Done edb for rls: " + rls[r].getName() + " " + (r + 1) + " / " + rls.length);
        }

        return new EnrichmentDbMultiImpl(name, edbs);
    }

    public EnrichmentDbMulti createEdb_precannedRankedLists(
            final String name,
            final int nperm,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final RankedListDb rdb,
            final GeneSet[] gsets_alrdy_qualified,
            final Chip chip,
            final GeneSetCohortGenerator gcohgen,
            final RandomSeedGenerator rst,
            final boolean storeDeepES_forRLs,
            final boolean storeRESPointByPoint_for_real)
            throws Exception {

        final EnrichmentDb[] edbs = new EnrichmentDb[rdb.getNumRankedLists()];

        for (int r = 0; r
                < rdb.getNumRankedLists(); r++) {
            final RankedList rl = rdb.getRankedList(r);
            final EnrichmentResult[] results = shuffleGeneSet_precannedRankedList(nperm, rl,
                    null, gsets_alrdy_qualified, chip, gcohgen, rst, false, storeDeepES_forRLs, storeRESPointByPoint_for_real); // @todo should the template be got??
            edbs[r] = new EnrichmentDbImpl_one_shared_rl(rl.getName(), rl, null, null, results,
                    lvp, metric, metricParams, sort, order, nperm, null);

            System.out.println("Done edb for rls: " + rl.getName() + " " + (r + 1) + " / " + rdb.getNumRankedLists());
        }

        return new EnrichmentDbMultiImpl(name, edbs);
    }

    public EnrichmentDbMulti createEdb_precannedRankedLists(
            final String name,
            final int nperm,
            final RankedList[] rls,
            final Template[] tss_opt,
            final GeneSet[] gsets_alrdy_qualified,
            final Chip chip,
            final GeneSetScoringTable table,
            final boolean storeDeepES_forRLs,
            final boolean storeRESPointByPoint_for_real)
            throws Exception {

        return createEdb_precannedRankedLists(name,
                nperm,
                new Metrics.Signal2Noise(),
                SortMode.REAL,
                Order.DESCENDING,
                DatasetMetrics.getDefaultMetricParams(),
                new LabelledVectorProcessors.None(),
                rls,
                tss_opt,
                gsets_alrdy_qualified,
                chip,
                new DefaultGeneSetCohort.Generator(table, true),
                new RandomSeedGenerators.Timestamp(),
                storeDeepES_forRLs,
                storeRESPointByPoint_for_real);
    }

    public Dataset createEdb_precannedRankedLists(
            final String name,
            final int nperm,
            final Dataset ds_for_rl,
            final Template[] tss_opt,
            final GeneSet[] gsets_alrdy_qualified,
            final Chip chip,
            final GeneSetScoringTable table,
            final boolean storeDeepES_forRLs,
            final boolean storeRESPointByPoint_for_real)
            throws Exception {

        Metric metric = new Metrics.Signal2Noise();
        Map metricParams = DatasetMetrics.getDefaultMetricParams();


        GeneSetCohortGenerator gcohgen = new DefaultGeneSetCohort.Generator(table, true);

        RandomSeedGenerator rst = new RandomSeedGenerators.Timestamp();
        LabelledVectorProcessor lvp = new LabelledVectorProcessors.None();

        Matrix m_ess = new Matrix(gsets_alrdy_qualified.length, nperm);
        String[] rowNames = null;
        String[] colNames = new String[ds_for_rl.getNumCol()];
        for (int c = 0; c
                < ds_for_rl.getNumCol(); c++) {
            Template t_opt = null;
            if (tss_opt != null) {
                t_opt = tss_opt[c];
            }

            RankedList rl = ds_for_rl.getColumnRL(c, SortMode.REAL, Order.DESCENDING);
            colNames[c] = rl.getName();

            EnrichmentResult[] results = shuffleGeneSet_precannedRankedList(nperm, rl, t_opt, gsets_alrdy_qualified, chip, gcohgen, rst, false, storeDeepES_forRLs, storeRESPointByPoint_for_real);

            EnrichmentDb edbs = new EnrichmentDbImpl_one_shared_rl(rl.getName(), rl, null, t_opt, results, lvp, metric, metricParams, SortMode.REAL, Order.DESCENDING, nperm, null);

            if (rowNames == null) {
                rowNames = edbs.getGeneSetNamesArray();
            }

            System.out.println("Done edb for rls: " + ds_for_rl.getColumnName(c) + " " + (c + 1) + " / " + ds_for_rl.getNumCol());
        }

        return new DefaultDataset(name + "_ess", m_ess, rowNames, colNames, true);
    }

    /**
     * @param nperm
     * @param metric
     * @param sort
     * @param order
     * @param metricParams
     * @param ds
     * @param template
     * @param gsets
     * @param gen
     * @param rst
     * @return
     * @throws Exception
     */
    public EnrichmentDb shuffleGeneSet(
            final int nperm,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gen,
            final RandomSeedGenerator rst,
            final boolean storeRESPointByPoint_for_real) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

// The same (real template) scored dataset for all gsets
        final DatasetMetrics dm = new DatasetMetrics();
        final ScoredDataset rlReal = dm.scoreDataset(metric, sort, order, metricParams, lvp, ds, template);
        final Chip chip = ds.getAnnot().getChip();

        final EnrichmentResult[] results = shuffleGeneSet_precannedRankedList(nperm,
                rlReal, template, gsets, chip, gen, rst, false, true, storeRESPointByPoint_for_real); // @note store deep

        final String name = NamingConventions.generateName(ds, template, true);
        return new EnrichmentDbImpl_one_shared_rl(name, rlReal, ds, template,
                results, lvp, metric, metricParams, sort, order, nperm, null);
    }

    public EnrichmentDb shuffleGeneSetSeq_ks(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final int nperm,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gen,
            final RandomSeedGenerator rst,
            final boolean storeRESPointByPoint_for_real) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // The same (real template) scored dataset for all gsets
        final DatasetMetrics dm = new DatasetMetrics();
        final ScoredDataset rlReal = dm.scoreDataset_gsaaseq_ks(demetric, sort, order, lvp, ds, template);
        //final Chip chip = ds.getAnnot().getChip();

        final EnrichmentResult[] results = shuffleGeneSet_precannedRankedList(nperm,
                rlReal, template, gsets, null, gen, rst, false, true, storeRESPointByPoint_for_real);

        final String name = NamingConventions.generateName(ds, template, true);
        return new EnrichmentDbImplSeq_one_shared_rl(name, rlReal, ds, template,
                results, lvp, sort, order, nperm, null);
    }

    public EnrichmentDb shuffleGeneSetSeq_p(
            final String demetric,
            final String gsametric,
            final String pthreshold,
            final int nperm,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gen,
            final RandomSeedGenerator rst,
            final boolean storeRESPointByPoint_for_real) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        // The same (real template) scored dataset for all gsets
        final DatasetMetrics dm = new DatasetMetrics();
        final ScoredDataset rlRealDirect = dm.scoreDataset_gsaaseq_p_direc(demetric, sort, order, lvp, ds, template);
        final ScoredDataset rlReal = dm.scoreDataset_gsaaseq_p(demetric, sort, order, lvp, ds, template);
        
        EnrichmentResult[] results;
        if (gsametric.equalsIgnoreCase("L2Norm") || gsametric.equalsIgnoreCase("Mean")) {
            results = shuffleGeneSet_precannedRankedList_seq_p1(gsametric, nperm,
                    rlReal, rlRealDirect, template, gsets, null, gen, rst, false, true, storeRESPointByPoint_for_real, pthreshold);
        } else {
            final ScoredDataset rlRealP = dm.scoreDataset_gsaaseq_p_p(demetric, sort, order, lvp, ds, template);
            results = shuffleGeneSet_precannedRankedList_seq_p2(gsametric, nperm,
                    rlReal, rlRealP, rlRealDirect, template, gsets, null, gen, rst, false, true, storeRESPointByPoint_for_real, pthreshold);
        }

        final String name = NamingConventions.generateName(ds, template, true);
        return new EnrichmentDbImplSeq_one_shared_rl(name, rlReal, ds, template,
                results, lvp, sort, order, nperm, null);
    }

    public EnrichmentDb shuffleGeneSetM(
            final int nperm,
            final Metric metric,
            final SortMode sort,
            final Order order,
            final Map metricParams,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gen,
            final RandomSeedGenerator rst,
            final boolean storeRESPointByPoint_for_real,
            final HashMap datasets,
            final HashMap templates,
            final HashMap params) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

// The same (real template) scored dataset for all gsets
        final DatasetMetrics dm = new DatasetMetrics();
        final ScoredDataset rlReal = dm.scoreDataset_m(metric, sort, order, metricParams, lvp, ds, template, datasets, templates, params, 0);
        final Chip chip = ds.getAnnot().getChip();

        final EnrichmentResult[] results = shuffleGeneSet_precannedRankedList(nperm,
                rlReal, template, gsets, chip, gen, rst, false, true, storeRESPointByPoint_for_real); // @note store deep

        final String name = NamingConventions.generateName(ds, template, true);
        return new EnrichmentDbImpl_one_shared_rl(name, rlReal, ds, template,
                results, lvp, metric, metricParams, sort, order, nperm, null);
    }

    public SnpEnrichmentDb shuffleGeneSetGsaaSnpKS(
            final int nperm,
            final CMetric cmetric,
            final SortMode sort,
            final Order order,
            final LabelledVectorProcessor lvp,
            final Dataset ds,
            final Template template,
            final GeneSet[] gsets,
            final GeneSetCohortGenerator gen,
            final RandomSeedGenerator rst,
            final boolean storeRESPointByPoint_for_real) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

// The same (real template) scored dataset for all gsets
        final DatasetMetrics dm = new DatasetMetrics();
        int colScore = 0;
        final ScoredDataset rlReal = dm.scoreDataset_gsaasnp_ks(sort, order, lvp, ds, template, colScore);
        final Chip chip = ds.getAnnot().getChip();

        final EnrichmentResult[] results = shuffleGeneSet_precannedRankedList_gsaasnp(nperm,
                rlReal, template, gsets, chip, gen, rst, false, true, storeRESPointByPoint_for_real); // @note store deep

        final String name = NamingConventions.generateName(ds, template, true);
        return new SnpEnrichmentDbImpl_one_shared_rl(name, rlReal, ds, template,
                results, lvp, cmetric, sort, order, nperm, null);
    }

    public static boolean isTemplateIdentical(Template template_exp, HashMap templates) {
        int numOfClassExp = template_exp.getNumClasses();
        if (numOfClassExp != 2) {
            throw new IllegalArgumentException("The number of phenotype labels must remain 2. Got: " + numOfClassExp);
        }

        if (templates.isEmpty()) {
            return false;
        }

        Iterator itr = templates.keySet().iterator();
        while (itr.hasNext()) {
            String t_name = (String) itr.next();
            Template template = (Template) templates.get(t_name);
            int numOfClass = template.getNumClasses();
            if (numOfClass != 2) {
                throw new IllegalArgumentException("The number of phenotype labels must remain 2. Got: " + numOfClass);
            }

            for (int i = 0; i
                    < numOfClassExp; i++) {
                String nameOfClassExp = template_exp.getClass(i).getName();
                int classSizeExp = template_exp.getClass(nameOfClassExp).getSize();
                String nameOfClass = template.getClass(i).getName();
                int classSize = template.getClass(nameOfClass).getSize();
                if (!nameOfClassExp.equalsIgnoreCase(nameOfClass)) {
                    throw new IllegalArgumentException("Phenotype labels must remain the same. Got: " + nameOfClassExp + " and " + nameOfClass);
                }

                if (classSizeExp != classSize) {
                    return false;
                }

            }
        }

        return true;
    }

    public ArrayList calcScores(final Dataset ds, final Metric metric, final Template template, final Template[] rndTemplates, final Map metricParams) {
        ArrayList scores = new ArrayList();
        for (int i = 0; i < ds.getNumRow(); i++) {
            ArrayList al = new ArrayList();
            scores.add(al);
        }
        for (int i = 0; i < rndTemplates.length + 1; i++) {
            for (int j = 0; j < ds.getNumRow(); j++) {
                double dist_exp = 0;
                if (i == 0) {
                    dist_exp = metric.getScore(ds.getRow(j), template, metricParams);
                } else {
                    dist_exp = metric.getScore(ds.getRow(j), rndTemplates[i - 1], metricParams);
                }
                ((ArrayList) scores.get(j)).add(new Double(dist_exp));
            }
        }
        return scores;
    }

    public ArrayList calcScoresAbs(final Dataset ds, final Metric metric, final Template template, final Template[] rndTemplates, final Map metricParams) {
        ArrayList scores = new ArrayList();
        for (int i = 0; i < ds.getNumRow(); i++) {
            ArrayList al = new ArrayList();
            scores.add(al);
        }
        for (int i = 0; i < rndTemplates.length + 1; i++) {
            for (int j = 0; j < ds.getNumRow(); j++) {
                double dist_exp = 0;
                if (i == 0) {
                    dist_exp = metric.getScore(ds.getRow(j), template, metricParams);
                } else {
                    dist_exp = metric.getScore(ds.getRow(j), rndTemplates[i - 1], metricParams);
                }
                dist_exp = Math.abs(dist_exp);
                ((ArrayList) scores.get(j)).add(new Double(dist_exp));
            }
        }
        return scores;
    }

// ------------------------------------------------------------------------ //
// --------------------------- SIGNAL CALCULATIONS -------------------------//
// ---------------------------------------------------------------- //

    /*
     public SignalMatrix calculateSignals(final String name,
     final Metric metric,
     final SortMode sort,
     final Order order,
     final Map metricParams,
     final LabelledVectorProcessor lvp,
     final Dataset ds,
     final Template template,
     final GeneSet[] gsets,
     final GeneSetCohortGenerator gen,
     final boolean omitNegMetrics)
     throws IOException {


     DatasetMetrics dm = new DatasetMetrics();
     ScoredDataset sds = dm.scoreDataset(metric, sort, order, metricParams, lvp, ds, template);
     GeneSetCohort gcoh = gen.createGeneSetCohort(sds, gsets, true, true);
     return calculateSignal(name, gcoh, omitNegMetrics);
     }

     public SignalMatrix calculateSignals(final String name,
     final Metric metric,
     final SortMode sort,
     final Order order,
     final Map metricParams,
     final LabelledVectorProcessor lvp,
     final Dataset ds,
     final Template template,
     final GeneSet gset,
     final GeneSetCohortGenerator gen,
     final boolean omitNegMetrics)
     throws IOException {

     return calculateSignals(name, metric, sort, order, metricParams, lvp, ds, template, new GeneSet[]{gset}, gen, omitNegMetrics);
     }

     public GeneSetSignal calculateSignal(RankedList rl, GeneSet gset, GeneSetCohortGenerator gen, boolean omitNegMetrics) {
     GeneSetCohort gcoh = gen.createGeneSetCohort(rl, gset, true, true);
     EnrichmentScore score = core.calculateKSScoreStruc(gcoh, true)[0]; // imp must store deep
     return score.getSignal(gset, rl, omitNegMetrics);
     }


     public GeneSetSignal calculateSignal(final Metric metric,
     final SortMode sort,
     final Order order,
     final Map metricParams,
     final LabelledVectorProcessor lvp,
     final Dataset ds,
     final Template template,
     final GeneSet gset,
     final GeneSetCohortGenerator gen,
     final boolean omitNegMetrics) {

     DatasetMetrics dm = new DatasetMetrics();
     ScoredDataset sds = dm.scoreDataset(metric, sort, order, metricParams, lvp, ds, template);
     EnrichmentScore score = core.calculateKSScoreStruc(gen.createGeneSetCohort(sds, gset, true, true), true)[0];
     return score.getSignal(gset, sds, omitNegMetrics); // IMP sds and not ds!
     }

     public SignalMatrix calculateSignal(String name, GeneSetCohort gcoh, boolean omitNegMetrics) {
     EnrichmentScore[] scores = core.calculateKSScoreStruc(gcoh, true); // imp must store deep info

     GeneSetSignal[] signs = new GeneSetSignal[gcoh.getNumGeneSets()];
     for (int g = 0; g < gcoh.getNumGeneSets(); g++) {
     signs[g] = scores[g].getSignal(gcoh.getGeneSet(g), gcoh.getRankedList(), omitNegMetrics);
     if (g % 50 == 0) {
     sout.println("Done extracting signal for: " + (g + 1) + " / " + gcoh.getNumGeneSets());
     }
     }

     return new DefaultGseaSignalMatrix(name, signs, gcoh.getGeneSetMatrix());
     }

     */
    private static class EnrichmentScoreCohort {

        private EnrichmentScore es_maxdev_style;
        private EnrichmentScore es_on_pos_list_maxdev;
        private EnrichmentScore es_on_pos_list;
        private EnrichmentScore es_on_neg_list;
        private EnrichmentScore es_on_neg_list_maxdev;
        private float mw;
        private int numHits;
        // @note deep only
        private int[] fHitIndices_opt;
        private Vector scoresAtEachHitIndex_opt;
        // @note this is redundant (but has more info) with the scoresAtEachHitIndex_opt var
        // usefil for richer plots
        private Vector scoresAtEachPoint_opt;

        private boolean hasDeepInfo() {
            return scoresAtEachHitIndex_opt != null;
        }

        private Vector getESProfile() {
            ensureDeep();
            scoresAtEachHitIndex_opt.setImmutable();
            return scoresAtEachHitIndex_opt;
        }

        private Vector getESProfile_full() {
            ensureDeep();
            scoresAtEachPoint_opt.setImmutable();
            return scoresAtEachPoint_opt;
        }

        private int getNumHits() {
            return numHits;
        }

        private int[] getHitIndices() {
            ensureDeep();
            return fHitIndices_opt;
        }

        private void ensureDeep() {
            if (!hasDeepInfo()) {
                throw new IllegalStateException("EnrichmentScore was NOT consructed in deep mode -- info not available");
            }
        }
    }

    /**
     * Inner class to represent one result NO perms done so NO etc are not
     * applicable
     */
    static class EnrichmentScoreImplSlim implements EnrichmentScore {

        private float fES;
        private int fRankAtES;
        private float fRankScoreAtES;
        private EnrichmentScoreCohort fScoreCoh;

        /**
         * Class constructor
         *
         * @param es
         * @param rankAtEs
         * @param corrAtEs
         * @param nes
         * @param np
         * @param fdr
         * @param fwer
         * @param hitIndices_opt
         * @param runningRaw_opt
         */
        EnrichmentScoreImplSlim(final float es,
                final int rankAtEs,
                final float corrAtEs,
                final EnrichmentScoreCohort coh) {
            this.fES = es;
            this.fRankAtES = rankAtEs;
            this.fRankScoreAtES = corrAtEs;

            // @note shared
            this.fScoreCoh = coh;
        }

        public boolean hasDeepInfo() {
            return fScoreCoh.hasDeepInfo();
        }

        public float getES() {
            return fES;
        }

        public int getRankAtES() {
            return fRankAtES;
        }

        public float getRankScoreAtES() {
            return fRankScoreAtES;
        }

        public float getNES() {
            return Float.NaN;
        }

        public float getNP() {
            return Float.NaN;
        }

        public float getFWER() {
            return Float.NaN;
        }

        public float getFDR() {
            return Float.NaN;
        }

        // @note change done in march 2006 to get nicer plots
        public Vector getESProfile() {
            return fScoreCoh.getESProfile();
        }

        public Vector getESProfile_point_by_point_opt() {
            return fScoreCoh.getESProfile_full();
        }

        public int getNumHits() {
            return fScoreCoh.getNumHits();
        }

        public int[] getHitIndices() {
            return fScoreCoh.getHitIndices();
        }

        public float getScore(Type type) {
            if (type == ES) {
                return getES();
            } else if (type == NES) {
                return getNES();
            } else if (type == FDR) {
                return getFDR();
            } else if (type == NP) {
                return getNP();
            } else {
                throw new IllegalArgumentException("Unknown Type: " + type);
            }
        }

        @Override
        public BigDecimal getESB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getNESB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getNPB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getFWERB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getFDRB() {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public BigDecimal getScoreB(Type type) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }
    } // End inner class EnrichmentScoreImplSlim
}    // End KSTests
