/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Aug 7, 2003
 *         Time: 3:41:04 PM
 */
public abstract class MhtModes {

    private static Logger klog = XLogger.getLogger(MhtModes.class);
    public static final String RAW_NORM = "no_norm";
    public static final String AREA_NORM = "area_norm";
    public static final String VAR_NORM = "var_norm";
    public static final String VARMEAN_NORM = "varmean_norm";
    public static final String VARMEAN_NORM_01 = "varmean_norm_01";
    public static final String LINEAR_NORM = "linear_norm";

    public static final String[] ALL = new String[]{RAW_NORM, AREA_NORM, VAR_NORM, VARMEAN_NORM, LINEAR_NORM, VARMEAN_NORM_01};

    public static boolean isRaw(String choice) {
        return choice.equalsIgnoreCase(RAW_NORM);
    }

    public static boolean isAreaNorm(String choice) {
        return choice.equalsIgnoreCase(AREA_NORM);
    }

    public static boolean isVarNorm(String choice) {
        return choice.equalsIgnoreCase(VAR_NORM);
    }

    public static boolean isVarMeanNorm(String choice) {
        return choice.equalsIgnoreCase(VARMEAN_NORM);
    }

    public static boolean isVarMeanNorm01(String choice) {
        return choice.equalsIgnoreCase(VARMEAN_NORM_01);
    }

    public static boolean isLinearNorm(String choice) {
        return choice.equalsIgnoreCase(LINEAR_NORM);
    }

    // @maint needs to be manually maintained
    public static MhtMode[] createAll() {
        return new MhtMode[]{new Raw(), new Area(), new Var(), new VarMean(), new Linear()};
    }

    // @maint needs to be manually maintained
    public static MhtMode[] createModes(String[] choices) {
        MhtMode[] modes = new MhtMode[choices.length];
        for (int i = 0; i < choices.length; i++) {
            if (MhtModes.isRaw(choices[i])) {
                modes[i] = new MhtModes.Raw();
            } else if (MhtModes.isAreaNorm(choices[i])) {
                modes[i] = new MhtModes.Area();
            } else if (MhtModes.isVarNorm(choices[i])) {
                modes[i] = new MhtModes.Var();
            } else if (MhtModes.isVarMeanNorm(choices[i])) {
                modes[i] = new MhtModes.VarMean();
            } else if (MhtModes.isLinearNorm(choices[i])) {
                modes[i] = new MhtModes.Linear();
            } else {
                throw new IllegalArgumentException("Unknown MhtMode: " + choices[i]);
            }
        }

        return modes;
    }

    public static MhtMode lookup(Object obj) {
        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof MhtMode) {
            return (MhtMode) obj;
        }

        MhtMode[] all = createAll();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookup MhtMode by: " + obj);
    }

    /**
     *
     */
    abstract static class Base implements MhtMode {

        private String fName;
        protected EnrichmentResult[] fDtgs;

        /**
         * If used, there can be no duplicated TG names allowed
         */
        private List fTGNames; // lazilly inited

        /**
         * If used, there can be no duplicated GeneSet names allowed
         */
        private List fGeneSetNames; // lazilly inited

        /**
         * rows are genesets and cols are perms
         */
        protected Matrix fRndMatrix;
        protected Matrix fRndMatrix_mw;

        /**
         * Class constructor
         *
         * @param name
         */
        protected Base(String name) {
            this.fName = name;
        }

        public void setData(EnrichmentResult[] dtgs) {
            this.fDtgs = dtgs;

            // @note IMP must blank out
            this.fGeneSetNames = null;
            this.fTGNames = null;
        }

        public String getName() {
            return fName;
        }

        public String toString() {
            return fName;
        }

        public int hashCode() {
            return getName().hashCode();
        }

        public boolean equals(Object obj) {
            if (obj instanceof MhtMode) {
                return getName().equalsIgnoreCase(((MhtMode) obj).getName());
            }

            return false;
        }

        public float getDiscoveryPValue(int r) {
            return (float) XMath.getPValueTwoTailed(getRealNES(r), getRndNESMatrix());
        }

        // for the entire matrix, the top X %
        public float getDiscoveryPValue(int r, float topXPercent) {
            float topN = (float) getRndNESMatrix().getAbsoluteSize() * topXPercent;
            float realNES = getRealNES(r);
            return (float) XMath.getPValueTwoTailed(realNES, getDiscoveryModeMaxScores(realNES, getRndNESMatrix(), (int) topN));
        }

        // for every permutation, the top x%
        public float getValidationPValue(int r, float topXPercent) {
            float topN = (float) fDtgs.length * topXPercent;
            float realNES = getRealNES(r);
            return (float) XMath.getPValueTwoTailed(realNES, getValidationModeMaxScores(realNES, getRndNESMatrix(), (int) topN));
        }

        public float getValidationPValue(int r) {
            float realNES = getRealNES(r);
            return (float) XMath.getPValueTwoTailed(realNES, getValidationModeMaxScores(getRndNESMatrix()));
            // return (float) XMath.getPValueForValMode(realNES, getValidationModeMaxScores(realNES, getRndESSMatrix()));
        }

        public float getValidationType2PValue(int r) {
            float realNES = getRealNES(r);
            return (float) XMath.getPValueTwoTailed(realNES, getPabloModeScores(realNES, getRndNESMatrix()));
            //return (float) XMath.getPValueTwoTailedTypeA(realNES, getPabloModeScores(realNES, getRndESSMatrix()));
        }

        public Matrix getRndNESMatrix() {
            fRndMatrix.setImmutable();
            return fRndMatrix;
        }

        public int getDiscoveryMatrixSize() {
            return getRndNESMatrix().getAbsoluteSize();
        }

        public int getValidationMatrixSize() {
            return getRndNESMatrix().getNumCol();
        }

        // --- mw not implemented yet for most cases --//
        // see below
        public float getRealNMW(int r) {
            return Float.NaN;
        }

        public Vector getRndNMW(int r) {
            klog.warn("Not implemented");
            return new Vector(fDtgs[r].getRndESS().getSize());
        }

        public float getDiscoveryPValue_mw(int r) {
            return (float) XMath.getPValueTwoTailed(getRealNMW(r), getRndNMWMatrix());
        }

        public float getValidationPValue_mw(int r) {
            float realNMW = getRealNMW(r);
            return (float) XMath.getPValueTwoTailed(realNMW, getValidationModeMaxScores(getRndNMWMatrix()));
        }

        public Matrix getRndNMWMatrix() {
            if (fRndMatrix_mw == null) {
                return new Matrix(fRndMatrix.getNumRow(), fRndMatrix.getNumCol());
            } else {
                fRndMatrix_mw.setImmutable();
                return fRndMatrix_mw;
            }
        }

        // -----------------------------------------------------------//
        /**
         * ACCESS BY TEMPLATE-GENESET NAME
         */
        // -----------------------------------------------------------//
        public Vector getRndNES(String tgName) {
            return getRndNES(_indexOf(tgName));
        }

        public float getRealNES(String tgName) {
            return getRealNES(_indexOf(tgName));
        }

        public float getDiscoveryPValue(String tgName) {
            return getDiscoveryPValue(_indexOf(tgName));
        }

        public float getDiscoveryPValue(String tgName, float topXPercent) {
            return getDiscoveryPValue(_indexOf(tgName), topXPercent);
        }

        public float getValidationPValue(String tgName) {
            return getValidationPValue(_indexOf(tgName));
        }

        public float getValidationPValue(String tgName, float topXPercent) {
            return getValidationPValue(_indexOf(tgName), topXPercent);
        }

        public float getValidationType2PValue(String tgName) {
            return getValidationType2PValue(_indexOf(tgName));
        }

        // -----------------------------------------------------------//
        /**
         * ACCESS BY GENE SET NAME *
         */
        // -----------------------------------------------------------//
        public Vector getRndNES_bygsetname(String gsetName) {
            return getRndNES(_indexOf_gsetname(gsetName));
        }

        public float getRealNES_bygsetname(String gsetName) {
            return getRealNES(_indexOf_gsetname(gsetName));
        }

        public float getDiscoveryPValue_bygsetname(String gsetName) {
            return getDiscoveryPValue(_indexOf_gsetname(gsetName));
        }

        public float getDiscoveryPValue_bygsetname(String gsetName, float topXPercent) {
            return getDiscoveryPValue(_indexOf_gsetname(gsetName), topXPercent);
        }

        public float getValidationPValue_bygsetname(String gsetName) {
            return getValidationPValue(_indexOf_gsetname(gsetName));
        }

        public float getValidationPValue_bygsetname(String gsetName, float topXPercent) {
            return getValidationPValue(_indexOf_gsetname(gsetName), topXPercent);
        }

        public float getValidationType2PValue_bygsetname(String gsetName) {
            return getValidationPValue(_indexOf_gsetname(gsetName));
        }

        // -----------------------------------------------------------//
        // --------------------HELPER METHODS-------------------------//
        // -----------------------------------------------------------//

        private int _indexOf(String tgName) {
            if (fTGNames == null) {
                this.fTGNames = new ArrayList(fDtgs.length);
                for (int i = 0; i < fDtgs.length; i++) {
                    if (fDtgs[i].getGeneSetName() == null || fTGNames.contains(fDtgs[i].getGeneSetName())) {
                        throw new IllegalStateException("Cannot use access by TGName as there are duplicate/null TG's: " + fDtgs[i].getGeneSetName());
                    } else {
                        fTGNames.add(fDtgs[i].getGeneSetName());
                    }
                }
            }

            int index = fTGNames.indexOf(tgName);
            if (index == -1) {
                throw new IllegalArgumentException("In this MhtMode no TG for: " + tgName + "\nThe TGs are: " + fTGNames);
            } else {
                return index;
            }
        }

        private int _indexOf_gsetname(String gsetName) {
            if (fGeneSetNames == null) {
                this.fGeneSetNames = new ArrayList(fDtgs.length);
                for (int i = 0; i < fDtgs.length; i++) {
                    if (fDtgs[i].getGeneSet() == null || fGeneSetNames.contains(fDtgs[i].getGeneSet().getName())) {
                        throw new IllegalStateException("Cannot use access by GeneSetName as there are duplicate/null GeneSet(s): " + fDtgs[i].getGeneSet());
                    } else {
                        fGeneSetNames.add(fDtgs[i].getGeneSet().getName());
                    }
                }
            }

            int index = fGeneSetNames.indexOf(gsetName);
            if (index == -1) {
                throw new IllegalArgumentException("In this MhtMode no GeneSetName for: " + gsetName + "\nThe GeneSetNames are: " + fGeneSetNames);
            } else {
                return index;
            }
        }

        /*
        public float calcNes(float score, Vector rndEss) {
            throw new NotImplementedException();
        }

        public Vector calcNes(Vector rndEss) {
            throw new NotImplementedException();
        }
        */

        public float calcNes(float es, Vector rndEss, ScoreMode smode) {
            throw new NotImplementedException();
        }

        public Vector calcNes(Vector rndEss, ScoreMode smode) {
            throw new NotImplementedException();
        }

        public float calcVar(Vector v, ScoreMode smode) {
            throw new NotImplementedException();
        }

        public float calcMean(Vector v, ScoreMode smode) {
            throw new NotImplementedException();
        }

    } // End inner class Base

    /**
     * Remember that the mht stuff is nonsensical
     */
    public static class Raw extends Base {

        public Raw() {
            super(RAW_NORM);
        }

        public Raw(EnrichmentResult[] dtgs) {
            this();
            this.setData(dtgs);
        }

        public MhtMode cloneShallow() {
            return new Raw();
        }

        public void setData(EnrichmentResult[] dtgs) {
            super.setData(dtgs);
            int nperms = checkAndGetNumPerms(dtgs);

            // each row is a dtg, each col a perm
            fRndMatrix = new Matrix(dtgs.length, nperms);

            for (int r = 0; r < dtgs.length; r++) {
                if (dtgs[r] == null) {
                    throw new IllegalArgumentException("dtgs cannot be null at: " + r);
                }
                fRndMatrix.setRow(r, this.getRndNES(r));
            }

        }

        public Vector getRndNES(int r) {
            return fDtgs[r].getRndESS();
        }

        public float getRealNES(int r) {
            return fDtgs[r].getScore().getES(); // no norm, score == norm_score
        }

    } // End inner class Raw

    /**
     * Class
     */
    public static class Area extends Base {

        public Area() {
            super(AREA_NORM);
        }

        public Area(EnrichmentResult[] dtgs) {
            this();
            this.setData(dtgs);
        }

        public MhtMode cloneShallow() {
            return new Area();
        }

        public void setData(EnrichmentResult[] dtgs) {

            super.setData(dtgs);

            int nperms = checkAndGetNumPerms(dtgs);

            klog.debug("# dtgs: " + dtgs.length + " # of perms: " + nperms);

            // each row is a dtg, each col a perm
            fRndMatrix = new Matrix(dtgs.length, nperms);

            for (int r = 0; r < dtgs.length; r++) {
                if (dtgs[r] == null) {
                    throw new IllegalArgumentException("dtgs cannot be null at: " + r);
                }
                fRndMatrix.setRow(r, this.getRndNES(r));
            }

        }

        public Vector getRndNES(int r) {
            return fDtgs[r].getRndESS(); // ?? working??
        }

        public float getRealNES(int r) {
            return fDtgs[r].getScore().getES(); // ?? working??
        }

    } // End inner class Area

    /**
     * Class
     */
    public static class Var extends Base {

        private float[] fRVars;

        public Var() {
            super(VAR_NORM);
        }

        public Var(EnrichmentResult[] dtgs) {
            this();
            this.setData(dtgs);
        }

        public MhtMode cloneShallow() {
            return new Var();
        }

        public void setData(EnrichmentResult[] dtgs) {
            super.setData(dtgs);

            int nperms = checkAndGetNumPerms(dtgs);

            klog.debug("# dtgs: " + dtgs.length + " # of perms: " + nperms);

            // each row is a dtg
            // here we have to divide each row by the root of the var
            fRndMatrix = new Matrix(dtgs.length, nperms);
            fRVars = new float[dtgs.length];

            // first of all must init rvars
            for (int r = 0; r < dtgs.length; r++) {
                if (dtgs[r] == null) {
                    throw new IllegalArgumentException("dtgs cannot be null at: " + r);
                }
                //Vector v = this.getRndESS(r); // dont call !! it needs rvar instead directly tap source
                Vector v = dtgs[r].getRndESS();
                fRVars[r] = (float) Math.sqrt(v.var(false, false));
            }

            for (int r = 0; r < dtgs.length; r++) {
                if (dtgs[r] == null) {
                    throw new IllegalArgumentException("dtgs cannot be null at: " + r);
                }
                fRndMatrix.setRow(r, this.getRndNES(r));
            }

        }

        public Vector getRndNES(int r) {
            Vector v = fDtgs[r].getRndESS();
            v.divide(fRVars[r]);
            return v;
        }

        public float getRealNES(int r) {
            return fDtgs[r].getScore().getES() / fRVars[r];
        }

    } // End inner class Var

    /**
     * Class
     */
    public static class VarMean extends Base {
        private float[] fRVars;
        private float[] fMeans;

        private float[] fRVars_mw;

        private float[] fMeans_mw;

        public VarMean() {
            super(VARMEAN_NORM);
        }

        public VarMean(EnrichmentResult[] dtgs) {
            this();
            this.setData(dtgs);
        }

        public MhtMode cloneShallow() {
            return new VarMean();
        }

        //int cnt = 0;
        public void setData(EnrichmentResult[] dtgs) {
            super.setData(dtgs);
            //TraceUtils.showTrace("CNT = " + cnt++);
            int nperms = checkAndGetNumPerms(dtgs);

            // each row is a dtg
            // here we have to divide each row by the root of the var
            this.fRndMatrix = new Matrix(dtgs.length, nperms);
            this.fRndMatrix_mw = new Matrix(dtgs.length, nperms);
            this.fRVars = new float[dtgs.length];
            this.fMeans = new float[dtgs.length];
            this.fRVars_mw = new float[dtgs.length];
            this.fMeans_mw = new float[dtgs.length];

            // first of all must init rvars
            for (int r = 0; r < dtgs.length; r++) {
                if (dtgs[r] == null) {
                    throw new IllegalArgumentException("dtgs cannot be null at: " + r);
                }
                //Vector v = this.getRndESS(r); // dont call !! it needs rvar instead directly tap source
                Vector v = dtgs[r].getRndESS();
                //System.out.println("%%###^^ " + dtgs[r].getRealES() +  " " + v.toString('\t'));
                fRVars[r] = (float) Math.sqrt(v.var(false, false));
                fMeans[r] = (float) v.mean();

                /*
                Vector v_mw = dtgs[r].getScore().getRndMW();
                fRVars_mw[r] = (float) Math.sqrt(v_mw.var(false, false));
                fMeans_mw[r] = (float) v_mw.mean();
                */

                /*
                if (fRVars[r] == 1) {
                    throw new IllegalStateException("Dtg is already normalized: " + fRVars[r] + " " + dtgs[r]);
                }
                */

                //System.out.println("%% " + dtgs[r].getRealES(false) +  " " + getRealES(r) + " " + fRVars[r] + " " + fMeans[r]);

            }

            for (int r = 0; r < dtgs.length; r++) {
                if (dtgs[r] == null) {
                    throw new IllegalArgumentException("dtgs cannot be null at: " + r);
                }
                this.fRndMatrix.setRow(r, this.getRndNES(r));
                this.fRndMatrix_mw.setRow(r, this.getRndNMW(r));
            }

        }

        public Vector getRndNES(int r) {
            Vector v = fDtgs[r].getRndESS();
            v.subtract(fMeans[r]);
            v.divide(fRVars[r]);
            return v;
        }

        public float getRealNES(int r) {
            return (fDtgs[r].getScore().getES() - fMeans[r]) / (fRVars[r]);
        }

        // --- MANN WHITNEY --//
        // see below
        public float getRealNMW(int r) {
            return Float.NaN;
            //return (fDtgs[r].getRealMW() - fMeans_mw[r]) / (fRVars_mw[r]);
        }

        public Vector getRndNMW(int r) {
            //Vector v = fDtgs[r].getRndMW();
            //v.subtract(fMeans_mw[r]);
            //v.divide(fRVars_mw[r]);
            //            return v;
            throw new NotImplementedException();
        }

        /*
        public float calcNes(float score, Vector rndEss) {
            float mean = (float) rndEss.mean();
            float var = (float) Math.sqrt(rndEss.var(false, false));
            return (score - mean) / var;
        }

        public Vector calcNes(Vector rndEss) {
            Vector v = new Vector(rndEss, false);
            float mean = (float) rndEss.mean();
            float var = (float) Math.sqrt(rndEss.var(false, false));
            v.subtract(mean);
            v.divide(var);
            return v;
        }
        */

        public float calcNes(final float es, final Vector rndEss, final ScoreMode smode) {
            Vector v = rndEss.extract(es, smode);
            float mean = (float) v.mean();
            float var = (float) Math.sqrt(v.var(false, false));
            return (es - mean) / var;
        }

        public Vector calcNes(final Vector rndEss, final ScoreMode smode) {
            Vector v = new Vector(rndEss, smode);
            float mean = (float) v.mean();
            float var = (float) Math.sqrt(v.var(false, false));
            v.subtract(mean);
            v.divide(var);
            v.setImmutable();
            return v;
        }

        public float calcVar(Vector v, ScoreMode smode) {
            return (float) Math.sqrt(v.extract(smode).var(false, false));
        }

        public float calcMean(Vector v, ScoreMode smode) {
            return (float) v.extract(smode).mean();
        }

    } // End inner class VarMean

    public static class Linear extends Base {

        public Linear() {
            super(LINEAR_NORM);
        }

        public Linear(EnrichmentResult[] dtgs) {
            this();
            this.setData(dtgs);
        }

        public MhtMode cloneShallow() {
            return new Linear();
        }

        //int cnt = 0;
        public void setData(EnrichmentResult[] dtgs) {
            super.setData(dtgs);
            //TraceUtils.showTrace("CNT = " + cnt++);
            int nperms = checkAndGetNumPerms(dtgs);

            //System.out.println("#################### " + dtgs[0].getClass());

            // each row is a dtg
            // here we have to divide each row by the root of the var
            this.fRndMatrix = new Matrix(dtgs.length, nperms);
            this.fRndMatrix_mw = new Matrix(dtgs.length, nperms);

            for (int r = 0; r < dtgs.length; r++) {
                if (dtgs[r] == null) {
                    throw new IllegalArgumentException("dtgs cannot be null at: " + r);
                }
                this.fRndMatrix.setRow(r, this.getRndNES(r));
                this.fRndMatrix_mw.setRow(r, this.getRndNMW(r));
            }

        }

        public Vector getRndNES(int r) {
            Vector v = fDtgs[r].getRndESS();
            v.divide((float) fDtgs[r].getScore().getNumHits());
            return v;
        }

        public float getRealNES(int r) {
            int numHits = fDtgs[r].getScore().getNumHits();
            return fDtgs[r].getScore().getES() / (float) numHits;
        }


    } // End inner class VarMean

    public static int checkAndGetNumPerms(EnrichmentResult[] dtgs) {

        if (dtgs.length == 0) {
            klog.warn("Zero length dtgs");
            return 0;
        }

        int num = dtgs[0].getNumPerms();

        for (int i = 0; i < dtgs.length; i++) {
            if (num != dtgs[i].getNumPerms()) {
                throw new IllegalArgumentException("Mismatched numperms: " + num + " but DTG at: " + i + " had: " + dtgs[i].getNumPerms());
            }
        }

        return num;
    }

    // for each iter take the max aa_indev from 0 whether pos or negative
    // and see if it is more than the score
    private static Vector getValidationModeMaxScores(Matrix matrix) {
        Vector vm = new Vector(matrix.getNumCol());
        for (int c = 0; c < matrix.getNumCol(); c++) {
            Vector scores = matrix.getColumnV(c);
            vm.setElement(c, scores.maxDevFrom0());
        }

        return vm;
    }

    // from the entire matrix, the topN
    private static Vector getDiscoveryModeMaxScores(float realEs, Matrix matrix, int topN) {

        Vector v = new Vector(matrix.getAbsoluteSize());
        v.sort(SortMode.REAL, getOrder(realEs));
        Vector ret = new Vector(topN);

        for (int i = 0; i < topN; i++) {
            ret.setElement(i, v.getElement(i));
        }

        return ret;
    }

    // form each permutation, get the top N
    private static Vector getValidationModeMaxScores(float realEs, Matrix matrix, int topN) {
        Vector max = new Vector(matrix.getNumCol() * topN);
        int pos = 0;
        for (int c = 0; c < matrix.getNumCol(); c++) {
            Vector v = matrix.getColumnV(c);
            v.sort(SortMode.REAL, getOrder(realEs));
            //klog.debug("vector len: " + v.getSize() + " " + matrix.getNumRow() + " " + matrix.getNumCol());
            for (int p = 0; p < topN; p++) {
                max.setElement(pos++, v.getElement(p));
            }
        }

        return max;
    }

    // simply the max rnd value of every dtgs
    private static Vector getPabloModeScores(float realEs, Matrix matrix) {
        Vector v = new Vector(matrix.getNumRow()); // note rows and not cols
        for (int r = 0; r < matrix.getNumRow(); r++) {
            if (realEs >= 0) {
                v.setElement(r, matrix.getRowV(r).max());
            } else {
                v.setElement(r, matrix.getRowV(r).min());
            }
        }

        return v;
    }

    private static Order getOrder(float es) {
        if (es >= 0) {
            return Order.DESCENDING;
        } else {
            return Order.ASCENDING;
        }
    }


} // End class MhtModes

/* OLD OLD OLD DONT DELETE YET
private static Vector getValidationModeMaxScoresOld(float realEs, Matrix matrix) {
    Vector vm = new Vector(matrix.getNumCol());
    for (int c = 0; c < matrix.getNumCol(); c++) {
        if (realEs >= 0) {
            vm.setElement(c, matrix.getColumnv(c).max());
        } else {
            vm.setElement(c, matrix.getColumnv(c).min());
        }
    }

    return vm;
}
*/
