/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.ScoreMode;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.DefaultDataset;
import edu.mit.broad.genome.objects.LabelledVector;
import edu.duke.igsp.app.tools.GsaaTool;
import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Nov 21, 2004
 *         Time: 6:10:13 PM
 */
public class Norms {

    public static final String NONE = None.NAME;
    public static final String VARMEAN_POS_NEG_SEPERATE = VarMeanPosNegSeperate.NAME;
    public static final String VARMEAN_POS_NEG_COMBINED = VarMeanPosNegCombined.NAME;
    public static final String MEANDIV_POS_NEG_SEPERATE = MeanDivPosNegSeperate.NAME;
    public static final String MEAN_POS_NEG_SEPERATE = MeanPosNegSeperate.NAME;
    public static final String MEAN_POS_NEG_COMBINED = MeanPosNegCombined.NAME;
    public static final String AREA_NORM = AreaNorm.NAME;
    public static final String P_VALUE = Pvalue.NAME;
    public static final String Z_SCORE = Zscore.NAME;

    // @maint need to keep in synch with above and below
    public static String[] createNormModeNames() {
        return new String[]{
                    MEANDIV_POS_NEG_SEPERATE,
                    NONE
                };
    }
    
    public static String[] createNormModeNamesSeq() {
        return new String[]{
                    MEANDIV_POS_NEG_SEPERATE,
                };
    }    

    public static String[] createNormModeNames_all() {
        return new String[]{
                    MEANDIV_POS_NEG_SEPERATE,
                    P_VALUE,
                    Z_SCORE,
                    NONE,
                    VARMEAN_POS_NEG_SEPERATE,
                    VARMEAN_POS_NEG_COMBINED,
                    MEAN_POS_NEG_SEPERATE,
                    MEAN_POS_NEG_COMBINED,
                    AREA_NORM
                };
    }

    public static Norm createNorm(final String normModeName, final float realScore, final Vector rndScores) {

        if (normModeName == null) {
            throw new IllegalArgumentException("Param normModeName cannot be null");
        }

        if (normModeName.equals(NONE)) {
            return new None(realScore, rndScores);
        } else if (normModeName.equals(VARMEAN_POS_NEG_SEPERATE)) {
            return new Norms.VarMeanPosNegSeperate(realScore, rndScores);
        } else if (normModeName.equals(VARMEAN_POS_NEG_COMBINED)) {
            return new Norms.VarMeanPosNegCombined(realScore, rndScores);
        } else if (normModeName.equals(MEAN_POS_NEG_SEPERATE)) {
            return new Norms.MeanPosNegSeperate(realScore, rndScores);
        } else if (normModeName.equals(MEAN_POS_NEG_COMBINED)) {
            return new Norms.MeanPosNegCombined(realScore, rndScores);
        } else if (normModeName.equals(MEANDIV_POS_NEG_SEPERATE)) {
            return new Norms.MeanDivPosNegSeperate(realScore, rndScores);
        } else if (normModeName.equals(AREA_NORM)) {
            return new Norms.MeanDivPosNegSeperate(realScore, rndScores);
        } else if (normModeName.equals(P_VALUE)) {
            return new Norms.Pvalue(realScore, rndScores);
        } else if (normModeName.equals(Z_SCORE)) {
            return new Norms.Zscore(realScore, rndScores);  
        } else {
            throw new IllegalArgumentException("Unknown norm mode: " + normModeName);
        }
    }

    public static class Struc {

        public LabelledVector normReal;
        public Dataset normRnd;
    }

    // This is the key API
    public static Struc normalize(final String normName, final LabelledVector realScores, final Dataset rndScores_full) {

        final Vector normRealScores = new Vector(realScores.getSize());
        final String[] labels = new String[realScores.getSize()];
        final List labels_list = new ArrayList();

        // @note end ds may have more rows that in the real scores
        // we pick only those that we want

        final Matrix normRndScoresMatrix = new Matrix(realScores.getSize(), rndScores_full.getNumCol());

        for (int r = 0; r < realScores.getSize(); r++) {

            final String rowName = realScores.getLabel(r);
            final float real = realScores.getScore(rowName);
            final Norm norm = Norms.createNorm(normName, real, rndScores_full.getRow(rowName));

            normRndScoresMatrix.setRow(r, norm.getRandomNorm());
            normRealScores.setElement(r, norm.getRealNorm());
            labels[r] = rowName;
            labels_list.add(rowName);
        }

        if (normName.equalsIgnoreCase("Z_score")) {
            float min = 0;
            float minReal = normRealScores.min();
            float minRnd = normRndScoresMatrix.getColumnMins().min();
            if (minReal <= minRnd) {
                min = minReal;
            } else {
                min = minRnd;
            }
            for (int r = 0; r < realScores.getSize(); r++) {
                final String rowName = realScores.getLabel(r);
                float oriRealScore = realScores.getScore(rowName);
                float normRealScore = normRealScores.getElement(r);
                normRealScore = normRealScore + Math.abs(min);
                normRealScores.setElement(r, normRealScore);
                //final Norm norm = Norms.createNorm(normName, real, rndScores_full.getRow(rowName));

                Vector oriRndScores = rndScores_full.getRow(rowName);
                float[] normRndScores = normRndScoresMatrix.getRow(r);
                for (int i = 0; i < normRndScores.length; i++) {
                    float oriRndScore = oriRndScores.getElement(i);
                    float normRndScore = normRndScores[i];
                    normRndScore = normRndScore + Math.abs(min);
                    normRndScores[i] = normRndScore;
                }
                normRndScoresMatrix.setRow(r, normRndScores);
            }
        }

        Struc struc = new Struc();
        struc.normReal = new LabelledVector(realScores.getName() + "_norm", labels, normRealScores);
        struc.normRnd = new DefaultDataset("norm", normRndScoresMatrix, labels_list, rndScores_full.getColumnNames(), true, rndScores_full.getAnnot());

        return struc;
    }

    public static Dataset normalize(final Dataset rndScores, final String normName) {

        final Matrix normMatrix = new Matrix(rndScores.getNumRow(), rndScores.getNumCol());
        for (int r = 0; r < rndScores.getNumRow(); r++) {
            float real = 0; // @ note
            Norm norm = Norms.createNorm(normName, real, rndScores.getRow(r));
            normMatrix.setRow(r, norm.getRandomNorm());
        }

        return new DefaultDataset("norm", normMatrix, rndScores.getRowNames(), rndScores.getColumnNames(), true, null);
    }

    private abstract static class AbstractNormOne implements Norm {

        float real_orig;
        float realNorm;
        Vector rnd_orig;
        Vector rndNorm;
        String normModeName;

        AbstractNormOne(final String normModeName) {
            this.normModeName = normModeName;
        }

        public String getName() {
            return normModeName;
        }

        public Vector getRandomNorm() {
            return rndNorm;
        }

        public float getRealNorm() {
            return realNorm;
        }
    } // End class One

    public static class None extends AbstractNormOne {

        private static String NAME = "None";

        public None(final float real, final Vector rnd) {
            super(NAME);
            this.real_orig = real;
            this.rnd_orig = rnd;
            this.rndNorm = rnd_orig;
            this.realNorm = real_orig;
        }
    } // End class VarMean2Sided

    public static class VarMeanPosNegSeperate extends AbstractNormOne {

        private static String NAME = "VarMeanPosNegSeperate";
        int numRndPos;
        int numRndNeg;
        float meanPos_orig;
        float varPos_orig;
        float meanNeg_orig;
        float varNeg_orig;
        float meanNeg_after_norm;
        float varNeg_after_norm;
        float meanPos_after_norm;
        float varPos_after_norm;

        public VarMeanPosNegSeperate(final float real, final Vector rnd) {
            super(NAME);
            this.real_orig = real;
            this.rnd_orig = rnd;

            final Vector onlyPos = rnd_orig.extract(ScoreMode.POS_ONLY);
            final Vector onlyNeg = rnd_orig.extract(ScoreMode.NEG_ONLY);

            this.meanPos_orig = (float) onlyPos.mean();
            this.varPos_orig = (float) Math.sqrt(onlyPos.var(false, false));

            this.meanNeg_orig = (float) onlyNeg.mean();
            this.varNeg_orig = (float) Math.sqrt(onlyNeg.var(false, false));

            // first norm the rnds
            this.rndNorm = new Vector(rnd_orig.getSize());
            for (int i = 0; i < rnd_orig.getSize(); i++) {
                float score = rnd_orig.getElement(i);
                if (XMath.isPositive(score)) {
                    score = (score - meanPos_orig) / varPos_orig;


                    if (XMath.isNegative(score)) {
                        score = 0.0f;
                    }

                    numRndPos++;
                } else {
                    score = (score - meanNeg_orig) / varNeg_orig;


                    if (XMath.isPositive(score)) {
                        score = 0.0f;
                    }


                    numRndNeg++;
                }

                rndNorm.setElement(i, score);
            }

            // then norm the real
            if (XMath.isPositive(real_orig)) {
                this.realNorm = (real_orig - meanPos_orig) / varPos_orig;

                if (XMath.isNegative(realNorm)) {
                    realNorm = 0.0f;
                }
            } else {
                this.realNorm = (real_orig - meanNeg_orig) / varNeg_orig;

                if (XMath.isPositive(realNorm)) {
                    realNorm = 0.0f;
                }
            }

            this.meanPos_after_norm = (float) rndNorm.extract(ScoreMode.POS_ONLY).mean();
            this.varPos_after_norm = (float) rndNorm.extract(ScoreMode.POS_ONLY).stddev(false, false);

            this.meanNeg_after_norm = (float) rndNorm.extract(ScoreMode.NEG_ONLY).mean();
            this.varNeg_after_norm = (float) rndNorm.extract(ScoreMode.NEG_ONLY).stddev(false, false);
        }
    } // End class VarMean2Sided

    public static class MeanPosNegSeperate extends AbstractNormOne {

        private static String NAME = "MeanPosNegSeperate";
        int numRndPos;
        int numRndNeg;
        float meanPos_orig;
        float varPos_orig;
        float meanNeg_orig;
        float varNeg_orig;
        float meanNeg_after_norm;
        float varNeg_after_norm;
        float meanPos_after_norm;
        float varPos_after_norm;

        public MeanPosNegSeperate(final float real, final Vector rnd) {
            super(NAME);
            this.real_orig = real;
            this.rnd_orig = rnd;

            final Vector onlyPos = rnd_orig.extract(ScoreMode.POS_ONLY);
            final Vector onlyNeg = rnd_orig.extract(ScoreMode.NEG_ONLY);

            this.meanPos_orig = (float) onlyPos.mean();
            this.varPos_orig = (float) Math.sqrt(onlyPos.var(false, false));

            this.meanNeg_orig = (float) onlyNeg.mean();
            this.varNeg_orig = (float) Math.sqrt(onlyNeg.var(false, false));

            // first norm the rnds
            this.rndNorm = new Vector(rnd_orig.getSize());
            for (int i = 0; i < rnd_orig.getSize(); i++) {
                float score = rnd_orig.getElement(i);
                if (XMath.isPositive(score)) {
                    score = score - meanPos_orig;
                    numRndPos++;
                } else {
                    score = score - meanNeg_orig;
                    numRndNeg++;
                }

                rndNorm.setElement(i, score);
            }

            // then norm the real
            if (XMath.isPositive(real_orig)) {
                this.realNorm = real_orig - meanPos_orig;
            } else {
                this.realNorm = real_orig - meanNeg_orig;
            }

            this.meanPos_after_norm = (float) rndNorm.extract(ScoreMode.POS_ONLY).mean();
            this.varPos_after_norm = (float) rndNorm.extract(ScoreMode.POS_ONLY).stddev(false, false);

            this.meanNeg_after_norm = (float) rndNorm.extract(ScoreMode.NEG_ONLY).mean();
            this.varNeg_after_norm = (float) rndNorm.extract(ScoreMode.NEG_ONLY).stddev(false, false);
        }
    } // End class MeanPosNegSeperate

    // This is the GSEA pnas way
    public static class MeanDivPosNegSeperate extends AbstractNormOne {

        private static String NAME = "MeanDiv";
        int numRndPos;
        int numRndNeg;
        float meanPos_orig;
        float varPos_orig;
        float meanNeg_orig;
        float varNeg_orig;
        float meanNeg_after_norm;
        float varNeg_after_norm;
        float meanPos_after_norm;
        float varPos_after_norm;

        public MeanDivPosNegSeperate(final float real, final Vector rnd) {
            super(NAME);
            this.real_orig = real;
            this.rnd_orig = rnd;

            final Vector onlyPos = rnd_orig.extract(ScoreMode.POS_ONLY);
            final Vector onlyNeg = rnd_orig.extract(ScoreMode.NEG_ONLY);
            if (onlyPos.getSize() == 0) {
                this.meanPos_orig = 0.1f;
                this.varPos_orig = 0;
            } else {
                this.meanPos_orig = (float) onlyPos.mean();
                this.varPos_orig = (float) Math.sqrt(onlyPos.var(false, false));
            }

            if (onlyNeg.getSize() == 0) {
                this.meanNeg_orig = -0.1f;
                this.varNeg_orig = 0;
            } else {
                this.meanNeg_orig = (float) onlyNeg.mean();
                this.varNeg_orig = (float) Math.sqrt(onlyNeg.var(false, false));
            }

            // first norm the rnds
            this.rndNorm = new Vector(rnd_orig.getSize());
            for (int i = 0; i < rnd_orig.getSize(); i++) {
                float score = rnd_orig.getElement(i);
                if (XMath.isPositive(score)) {
                    score = score / meanPos_orig;
                    numRndPos++;
                } else {
                    score = score / Math.abs(meanNeg_orig); // @note abs
                    numRndNeg++;
                }

                rndNorm.setElement(i, score);
            }

            // then norm the real

            if (XMath.isPositive(real_orig)) {
                this.realNorm = real_orig / meanPos_orig;
            } else {
                this.realNorm = real_orig / Math.abs(meanNeg_orig); // @note abs
            }

            //this.meanPos_after_norm = (float) rndNorm.extract(ScoreMode.POS_ONLY).mean();
            //this.varPos_after_norm = (float) rndNorm.extract(ScoreMode.POS_ONLY).stddev(false, false);

            //this.meanNeg_after_norm = (float) rndNorm.extract(ScoreMode.NEG_ONLY).mean();
            //this.varNeg_after_norm = (float) rndNorm.extract(ScoreMode.NEG_ONLY).stddev(false, false);
        }
    } // End class MeanDivPosNegSeperate

    //only applicable for positive AS scores
    public static class Pvalue extends AbstractNormOne {

        private static String NAME = "P_value";

        public Pvalue(final float real, final Vector rnd) {
            super(NAME);
            this.real_orig = real;
            this.rnd_orig = rnd;
            Vector all = new Vector(rnd.getSize() + 1);
            all.setElement(0, real);
            for (int i = 0; i < rnd.getSize(); i++) {
                all.setElement(i + 1, rnd.getElement(i));
            }

            // first norm the rnds
            this.rndNorm = new Vector(all.getSize() - 1);
            for (int i = 1; i < all.getSize(); i++) {
                float score = all.getElement(i);
                score = (float) (1 - GsaaTool.getPValue(score, all));
                rndNorm.setElement(i - 1, score);
            }

            // then norm the real
            this.realNorm = (float) (1 - GsaaTool.getPValue(real, all));
        }
    } // End class Pvalue

    //only applicable for positive AS scores
    public static class Zscore extends AbstractNormOne {

        private static String NAME = "Z_score";
        float mean_orig;
        float var_orig;
        float mean_after_norm;
        float var_after_norm;

        public Zscore(final float real, final Vector rnd) {
            super(NAME);
            this.real_orig = real;
            this.rnd_orig = rnd;
            Vector rndAbs = new Vector(rnd.getSize());
            for (int i = 0; i < rnd.getSize(); i++) {
                rndAbs.setElement(i, Math.abs(rnd.getElement(i)));
            }
            this.mean_orig = (float) rndAbs.mean();
            this.var_orig = (float) Math.sqrt(rndAbs.var(false, false));

            // first norm the rnds
            this.rndNorm = new Vector(rnd_orig.getSize());
            for (int i = 0; i < rnd_orig.getSize(); i++) {
                float score = rnd_orig.getElement(i);
                float score_norm = (Math.abs(score) - mean_orig) / var_orig;
                rndNorm.setElement(i, score_norm);
            }

            // then norm the real
            this.realNorm = (Math.abs(real_orig) - mean_orig) / var_orig;
        }
    } // End class Zscore

    public static class VarMeanPosNegCombined extends AbstractNormOne {

        private static String NAME = "VarMeanPosNegCombined";
        float mean_orig;
        float var_orig;
        float mean_after_norm;
        float var_after_norm;

        public VarMeanPosNegCombined(final float real, final Vector rnd) {
            super(NAME);
            this.real_orig = real;
            this.rnd_orig = rnd;
            this.mean_orig = (float) rnd.mean();
            this.var_orig = (float) Math.sqrt(rnd.var(false, false));

            // first norm the rnds
            this.rndNorm = new Vector(rnd_orig.getSize());
            for (int i = 0; i < rnd_orig.getSize(); i++) {
                float score = rnd_orig.getElement(i);
                score = (score - mean_orig) / var_orig;
                rndNorm.setElement(i, score);
            }

            // then norm the real
            this.realNorm = (real_orig - mean_orig) / var_orig;

            this.mean_after_norm = (float) rndNorm.mean();
            this.var_after_norm = (float) rndNorm.stddev(false, false);
        }
    } // End class VarMeanSingle

    public static class MeanPosNegCombined extends AbstractNormOne {

        private static String NAME = "MeanPosNegCombined";
        float mean_orig;
        float var_orig;
        float mean_after_norm;
        float var_after_norm;

        public MeanPosNegCombined(final float real, final Vector rnd) {
            super(NAME);
            this.real_orig = real;
            this.rnd_orig = rnd;
            this.mean_orig = (float) rnd.mean();
            this.var_orig = (float) Math.sqrt(rnd.var(false, false));

            // first norm the rnds
            this.rndNorm = new Vector(rnd_orig.getSize());
            for (int i = 0; i < rnd_orig.getSize(); i++) {
                float score = rnd_orig.getElement(i);
                score = score - mean_orig;
                rndNorm.setElement(i, score);
            }

            // then norm the real
            this.realNorm = real_orig - mean_orig;

            this.mean_after_norm = (float) rndNorm.mean();
            this.var_after_norm = (float) rndNorm.stddev(false, false);
        }
    } // End class MeanSingle

    public static class AreaNorm extends AbstractNormOne {

        private static String NAME = "Area";
        int numRndPos;
        int numRndNeg;
        float meanPos_orig;
        float varPos_orig;
        float meanNeg_orig;
        float varNeg_orig;
        float meanNeg_after_norm;
        float varNeg_after_norm;
        float meanPos_after_norm;
        float varPos_after_norm;

        public AreaNorm(final float real, final Vector rnd) {
            super(NAME);

            this.real_orig = real;
            this.rnd_orig = rnd;

            final Vector onlyPos = rnd_orig.extract(ScoreMode.POS_ONLY);
            final Vector onlyNeg = rnd_orig.extract(ScoreMode.NEG_ONLY);

            this.meanPos_orig = (float) onlyPos.mean();
            this.varPos_orig = (float) Math.sqrt(onlyPos.var(false, false));

            this.meanNeg_orig = (float) onlyNeg.mean();
            this.varNeg_orig = (float) Math.sqrt(onlyNeg.var(false, false));

            // first norm the rnds
            this.rndNorm = new Vector(rnd_orig.getSize());
            for (int i = 0; i < rnd_orig.getSize(); i++) {
                float score = rnd_orig.getElement(i);
                if (XMath.isPositive(score)) {
                    score = score / meanPos_orig;
                    numRndPos++;
                } else {
                    score = score / Math.abs(meanNeg_orig); // @note abs
                    numRndNeg++;
                }

                rndNorm.setElement(i, score);
            }

            // then norm the real
            if (XMath.isPositive(real_orig)) {
                this.realNorm = real_orig / meanPos_orig;
            } else {
                this.realNorm = real_orig / Math.abs(meanNeg_orig); // @note abs
            }

            this.meanPos_after_norm = (float) rndNorm.extract(ScoreMode.POS_ONLY).mean();
            this.varPos_after_norm = (float) rndNorm.extract(ScoreMode.POS_ONLY).stddev(false, false);

            this.meanNeg_after_norm = (float) rndNorm.extract(ScoreMode.NEG_ONLY).mean();
            this.varNeg_after_norm = (float) rndNorm.extract(ScoreMode.NEG_ONLY).stddev(false, false);
        }
    } // End class MeanDivPosNegSeperate
}
