/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.markers;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.alg.DatasetMetrics;
import edu.mit.broad.genome.alg.DatasetStatsCore;
import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.alg.RankedListGenerators;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import gnu.trove.TIntArrayList;

import java.util.Map;
import java.util.HashMap;
import java.util.List;

/**
 * NamingConventions permutation tests
 *
 * @author Aravind Subramanian
 * @revised by Qing Xiong - qing.xiong@duke.edu
 *
 */
public class IntegratedPermutationTestBuilder extends AbstractIntegratedPermutationTest {

    private Template[] fRndTemplates;
    private HashMap[] fRndTemplateSet;

    private ScoredDataset fRealScoredDataset;

    /**
     * contains key -> rownames, values -> corresp MarkerScore objects -- only for categorical 2 way distinctions
     */
    private Map fMarkerScores;

    // holds points at which the metric changes sign
    private TIntArrayList fMetricCrossOverRanks;

    private MetricWeightStruc[] fRndMetricWeightStrucs;

    private Matrix nMarkersUpMatrix;
    private Matrix nMarkersDnMatrix;


    /**
     * Class constructor
     *
     * @param numMarkers
     * @param numPerms
     * @param lvp
     * @param metric
     * @param sort
     * @param order
     * @param metricParams
     * @param ds
     * @param template
     * @param siglevels
     * @param rst
     * @param rndType
     * @param saveFullRndMetricsMatrix
     * @throws Exception
     */
    public IntegratedPermutationTestBuilder(final String name,
                                  final int numMarkers,
                                  final int nperm,
                                  final LabelledVectorProcessor lvp,
                                  final Metric metric,
                                  final SortMode sort,
                                  final Order order,
                                  final Map metricParams,
                                  final Dataset ds,
                                  final List dsIntegrated,
                                  final String[] rowNames,
                                  final Template template,
                                  final float[] siglevels,
                                  final boolean categorical,
                                  final HashMap datasets,
                                  final HashMap templates,
                                  final HashMap params) {


        super.init(name, numMarkers, nperm, lvp, metric, sort, order, metricParams, ds, dsIntegrated, rowNames, template, siglevels, categorical, datasets, templates, params);

        final DatasetMetrics dm = new DatasetMetrics();
        // calc real metric levels
        DatasetMetrics.ScoredStruc realss = dm.scoreDatasetStruc_integrated(fSort, fOrder, fLabelledVectorProc, getDataset(), getDsIntegrated(), getRowNames(), getDatasets(), getTemplates(), getParams(), 0);
        this.fRealScoredDataset = realss.sds;

        // calc mean/median and stdev for each class for each marker
        /*if (categorical) {
            this.fMarkerScores = new DatasetStatsCore().calc2ClassCategoricalMetricMarkerScores(getDataset(), getTemplate(), fMetric, fMetricParams);
        }*/

        // Init arrays and matrices
        // each row is a rank, each col is a score for a perm
        this.nMarkersUpMatrix = new Matrix(fNumMarkers, fNumPerms);
        this.nMarkersDnMatrix = new Matrix(fNumMarkers, fNumPerms);
        // Init arrays
        this.fRndTemplates = new Template[fNumPerms];
        this.fRndTemplateSet = new HashMap[fNumPerms];
        this.fMetricCrossOverRanks = new TIntArrayList(fNumPerms);
        this.fRndMetricWeightStrucs = new MetricWeightStruc[fNumPerms];
    }

    private int fNumSetCalls;

    public void doCalc() {
        checkIfDone();
        calcAndSetSignificanceLevels(nMarkersUpMatrix, nMarkersDnMatrix);
    }

    public void addRnd(final Template rndTemplate, final HashMap rndTemplateSet, final RankedList rndRl) {
        if (fNumSetCalls == getNumPerms()) {
            throw new IllegalStateException("Already done fNumSetCalls: " + fNumSetCalls + " getNumPerms: " + getNumPerms());
        }

        this.fRndTemplates[fNumSetCalls] = rndTemplate;

        this.fRndTemplateSet[fNumSetCalls] = rndTemplateSet;

        for (int r = 0; r < fNumMarkers; r++) {
            nMarkersUpMatrix.setElement(r, fNumSetCalls, rndRl.getScore(r)); // Pth ranked random score
        }

        int p = getDataset().getNumRow() - 1;
        for (int cnt = 0; cnt < fNumMarkers; cnt++, p--) {
            nMarkersDnMatrix.setElement(cnt, fNumSetCalls, rndRl.getScore(p));
        }

        for (int r = 0; r < rndRl.getSize(); r++) {
            if (XMath.isNegative(rndRl.getScore(r))) {
                fMetricCrossOverRanks.add(r + 1);
                break;
            }
        }

        this.fRndMetricWeightStrucs[fNumSetCalls] = rndRl.getMetricWeightStruc();
        this.fNumSetCalls++;
    }


    private void checkIfDone() {
        if (fNumSetCalls != getNumPerms()) {
            throw new IllegalStateException("Not yet done: fNumSetCalls: " + fNumSetCalls + " nperm: " + getNumPerms());
        }
    }

    public MetricWeightStruc[] getRandomMetricWeights() {
        checkIfDone();
        return fRndMetricWeightStrucs;
    }

    public DatasetStatsCore.TwoClassMarkerStats getScore(final String featureName) {
        if (fMarkerScores == null) {
            throw new IllegalStateException("MarkerScores is null .. are you sure this is a categorical template: " + getTemplate().getName());
        }

        Object obj = fMarkerScores.get(featureName);
        if (obj == null) {
            throw new IllegalArgumentException("No such feature: " + featureName);
        } else {
            return (DatasetStatsCore.TwoClassMarkerStats) obj;
        }
    }

    public Template[] getTemplatesRandom() {
        checkIfDone();
        return fRndTemplates;
    }

    public TIntArrayList getMetricCrossOverRanks() {
        checkIfDone();
        return fMetricCrossOverRanks;
    }

    public int getRealMetricCrossOverRank() {

        int realX = 0;
        for (int r = 0; r < fRealScoredDataset.getSize(); r++) {
            if (XMath.isNegative(fRealScoredDataset.getScore(r))) {
                realX = r + 1;
                break;
            }
        }

        return realX;
    }

    public MetricWeightStruc getRealMetricWeight() {
        return fRealScoredDataset.getMetricWeightStruc();
    }

    // @note made lazilly
    private RankedList fRealRankedList;

    /**
     * Eventhough the sds is a ranked list, I dont fully understand if it ranks properly yet
     *
     * @return
     */
    public RankedList getRankedList() {
        if (fRealRankedList == null) {
            this.fRealRankedList = RankedListGenerators.createBySorting(fRealScoredDataset, fSort, fOrder);
        }

        return fRealRankedList;
    }

    public ScoredDataset getScoredDataset() {
        return fRealScoredDataset;
    }

    public int getNumSigMarkers(final float sigLevel) {
        throw new NotImplementedException();
    }

    public GeneSet getSigMarkers(final float sigLevel) {
        throw new NotImplementedException();
    }

} // End PvaluePermutationTestBuilder

