/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.markers;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.alg.*;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.TemplateRandomizerType;
import gnu.trove.TIntArrayList;

import java.util.Map;

/**
 * @author Aravind Subramanian
 *         Date: Feb 1, 2003
 *         Time: 1:48:32 PM
 */
public class PermutationTestImpl extends AbstractPermutationTest {

    private Template[] fRndTemplates;

    private ScoredDataset fRealScoredDataset;

    /**
     * Rows are gene names columns are the permutations
     * Not always filled
     */
    private Matrix fRndFullMatrix;

    /**
     * contains key -> rownames, values -> corresp MarkerScore objects -- only for categorical 2 way distinctions
     */
    private Map fMarkerScores;

    // holds points at which the metric changes sign
    private TIntArrayList fMetricCrossOverRanks;

    private MetricWeightStruc[] fRndMetricWeightStrucs;

    /**
     * Class constructor
     *
     * @param numMarkers
     * @param numPerms
     * @param rp
     * @param metric
     * @param sort
     * @param order
     * @param metricParams
     * @param ds
     * @param template
     * @param siglevels
     * @param rst
     * @param rndType
     * @param saveFullRndMetricsMatrix
     * @throws Exception
     */
    public PermutationTestImpl(final int numMarkers,
                               final int nperms,
                               final LabelledVectorProcessor rp,
                               final Metric metric,
                               final SortMode sort,
                               final Order order,
                               final Map metricParams,
                               final Dataset ds,
                               final Template template,
                               final float[] siglevels,
                               final RandomSeedGenerator rst,
                               final TemplateRandomizerType rndType,
                               final boolean saveFullRndMetricsMatrix,
                               final boolean categorical) throws Exception {

        this(new StringBuffer(NamingConventions.generateName(ds, template, true) + "_" + numMarkers + "_markers").toString(),
                numMarkers, nperms, rp, metric, sort, order,
                metricParams, ds, template, siglevels, rst, rndType, saveFullRndMetricsMatrix, categorical);
    }

    public PermutationTestImpl(final int numMarkers,
                               final int numPerms,
                               final Dataset ds,
                               final Template template,
                               final boolean categorical) throws Exception {

        this(numMarkers, numPerms,
                new LabelledVectorProcessors.None(),
                new Metrics.Signal2Noise(),
                SortMode.REAL, Order.DESCENDING,
                DatasetMetrics.getDefaultMetricParams(),
                ds, template, DEFAULT_SIG_LEVELS,
                new RandomSeedGenerators.Timestamp(),
                TemplateRandomizerType.NO_BALANCE, false, categorical
        );
    }

    /**
     * Class constructor
     *
     * @param name
     * @param nmarkers
     * @param nperms
     * @param rp
     * @param metric
     * @param sort
     * @param order
     * @param metricParams
     * @param ds
     * @param template
     * @param siglevels
     * @param rst
     * @param rndType
     * @param saveFullRndMetricsMatrix
     */
    public PermutationTestImpl(final String name,
                               final int nmarkers,
                               final int nperms,
                               final LabelledVectorProcessor rp,
                               final Metric metric,
                               final SortMode sort,
                               final Order order,
                               final Map metricParams,
                               final Dataset ds,
                               final Template template,
                               final float[] siglevels,
                               final RandomSeedGenerator rst,
                               final TemplateRandomizerType rndType,
                               final boolean saveFullRndMetricsMatrix,
                               final boolean categorical) {

        // shuffle the template
        final Template[] rndTemplates = TemplateFactoryRandomizer.createRandomTemplates(nperms, template, rndType, rst);

        initHere(name, nmarkers, nperms, rp, metric, sort, order,
                metricParams, ds, template, siglevels,
                saveFullRndMetricsMatrix, rndTemplates, categorical);

    }

    /**
     * does the real work
     */
    private void initHere(final String name,
                          final int numMarkers,
                          final int numPerms,
                          final LabelledVectorProcessor rp,
                          final Metric metric,
                          final SortMode sort,
                          final Order order,
                          final Map metricParams,
                          final Dataset ds,
                          final Template template,
                          final float[] siglevels,
                          final boolean saveFullRndMetricsMatrix,
                          final Template[] rndTemplates,
                          boolean categorical) {


        super.init(name, numMarkers, numPerms, rp, metric, sort, order, metricParams, ds, template, siglevels, categorical);

        this.fRndTemplates = rndTemplates;

        final DatasetMetrics dm = new DatasetMetrics();

        // calc real metric levels
        DatasetMetrics.ScoredStruc realss = dm.scoreDatasetStruc(fMetric, fSort, fOrder, fMetricParams, fLabelledVectorProc,
                getDataset(), getTemplate());
        this.fRealScoredDataset = realss.sds;
        this.fMetricCrossOverRanks = new TIntArrayList(fNumPerms);
        this.fRndMetricWeightStrucs = new MetricWeightStruc[fNumPerms];

        // calc mean/median and stdev for each class for each marker
        this.fMarkerScores = new DatasetStatsCore().calc2ClassCategoricalMetricMarkerScores(getDataset(), getTemplate(), fMetric, fMetricParams);

        // each row is a rank, each col is a score for a perm
        Matrix nMarkersTopMatrix = new Matrix(fNumMarkers, fNumPerms);
        Matrix nMarkersBotMatrix = new Matrix(fNumMarkers, fNumPerms);

        if (saveFullRndMetricsMatrix) {
            // rows are genes and columns are the rnds
            this.fRndFullMatrix = new Matrix(ds.getNumRow(), fNumPerms);
        }

        for (int c = 0; c < fNumPerms; c++) {

            final DatasetMetrics.ScoredStruc rndSS = dm.scoreDatasetStruc(fMetric, fSort, fOrder, fMetricParams, fLabelledVectorProc, getDataset(), fRndTemplates[c]);

            final ScoredDataset rndSds = rndSS.sds;

            for (int r = 0; r < fNumMarkers; r++) {
                nMarkersTopMatrix.setElement(r, c, rndSds.getScore(r)); // Pth ranked random score
            }

            int p = ds.getNumRow() - 1;
            for (int cnt = 0; cnt < fNumMarkers; cnt++, p--) {
                nMarkersBotMatrix.setElement(cnt, c, rndSds.getScore(p));
            }

            if (saveFullRndMetricsMatrix) {
                fRndFullMatrix.setColumn(c, rndSS.getDatasetSynchedScores());
            }

            for (int r = 0; r < rndSds.getSize(); r++) {
                if (XMath.isNegative(rndSds.getScore(r))) {
                    fMetricCrossOverRanks.add(r + 1);
                    break;
                }
            }

            this.fRndMetricWeightStrucs[c] = rndSds.getMetricWeightStruc();

            /*
            if (Float.isNaN(fRndMetricWeightStrucs[c].getTotalPosWeight())) {
                Printf.out(new LabelledVector(rndSds, true));
                throw new IllegalStateException("nan: " + fRndMetricWeightStrucs[c].getTotalPosWeight());
            }
            */

            if (c % 10 == 0) {
                System.out.println("Done permutation #: " + (c + 1) + " / " + fNumPerms + " for: " + getName());
            }
        }

        calcAndSetSignificanceLevels(nMarkersTopMatrix, nMarkersBotMatrix);
    }

    public Matrix getFullRandomMatrix() {
        return fRndFullMatrix;
    }

    public MetricWeightStruc[] getRandomMetricWeights() {
        return fRndMetricWeightStrucs;
    }

    public DatasetStatsCore.TwoClassMarkerStats getScore(final String featureName) {
        Object obj = fMarkerScores.get(featureName);
        if (obj == null) {
            throw new IllegalArgumentException("No such feature: " + featureName);
        } else {
            return (DatasetStatsCore.TwoClassMarkerStats) obj;
        }
    }

    public Template[] getTemplatesRandom() {
        return fRndTemplates;
    }

    public TIntArrayList getMetricCrossOverRanks() {
        return fMetricCrossOverRanks;
    }

    public int getRealMetricCrossOverRank() {

        int realX = 0;
        for (int r = 0; r < fRealScoredDataset.getSize(); r++) {
            if (XMath.isNegative(fRealScoredDataset.getScore(r))) {
                realX = r + 1;
                break;
            }
        }

        return realX;
    }

    public MetricWeightStruc getRealMetricWeight() {
        return fRealScoredDataset.getMetricWeightStruc();
    }

    // @note made lazilly
    private RankedList fRealRankedList;

    /**
     * Eventhough the sds is a ranked list, I dont fully understand if it ranks properly yet
     *
     * @return
     */
    public RankedList getRankedList() {
        if (fRealRankedList == null) {
            this.fRealRankedList = RankedListGenerators.createBySorting(fRealScoredDataset, fSort, fOrder);
        }

        return fRealRankedList;
    }

    public ScoredDataset getScoredDataset() {
        return fRealScoredDataset;
    }

    public int getNumSigMarkers(final float sigLevel) {
        throw new NotImplementedException();
    }

    public GeneSet getSigMarkers(final float sigLevel) {
        throw new NotImplementedException();
    }

} // End PermutationTests
