/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.charts;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.charts.jfreechart.editor.GenomeChartPanel;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Range;
import edu.mit.broad.genome.math.SimpleRange;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.models.XChartModelFactory;
import edu.mit.broad.genome.models.XYDatasetMultiTmp;
import edu.mit.broad.genome.models.XYDatasetProxy2;
import edu.mit.broad.genome.objects.Dataset;
import org.apache.log4j.Logger;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.title.Title;
import org.jfree.data.statistics.HistogramDataset;
import org.jfree.data.statistics.HistogramType;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

/**
 * Methods to easily create commonly used plot types.
 * Not meant to be very customizable - meant as:
 * <p/>
 * 1) quickie charts to use when
 * developing / investigating new visualizations.
 * 2) Underlies the cannedcharts functionality - works of simple datatstructures and not
 * all the rich objects that algs etc produce.
 * <p/>
 * IMP: This produces JFreeChart and similar chart objects - it should NOT produce XCharts
 * as XCharts are usually customizable for name, title, caption etc.
 * <p/>
 * <p/>
 * <p/>
 * Notes
 * <p/>
 * 1) 1 plot, 2 vertical y axis: http://serveurwind.cci-brest.fr/index.jsp
 * http://www.object-refinery.com/phorum-3.3.2a/read.php?f=2&i=7430&t=7430
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class XChartUtils {

    private static final transient Logger klog = XLogger.getLogger(XChartUtils.class);

    /**
     * Privatized Class Constructor.
     */
    private XChartUtils() {
    }

    public static JFreeChart createHistogramChart(final Dataset ds, final int nbins) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(ds, nbins);
        return _createHistogramChart(ds.getName(), ids);
    }

    public static JFreeChart createHistogramChart(final Dataset ds, final int nbins, final float min, final float max) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(ds, nbins, min, max);
        return _createHistogramChart(ds.getName(), ids);
    }

    public static JFreeChart createHistogramChart(final Dataset ds, final int nbins, final HistogramType htype) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(ds, nbins);
        ids.setType(htype);
        return _createHistogramChart(ds.getName(), ids);
    }

    public static JFreeChart createHistogramChart(final Dataset ds, final int nbins, final float min, final float max, final HistogramType htype) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(ds, nbins, min, max);
        ids.setType(htype);
        return _createHistogramChart(ds.getName(), ids);
    }

    public static JFreeChart createHistogramChart(final String title, final Vector v, final int nbins) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(v, nbins);
        return _createHistogramChart(title, ids);
    }

    public static JFreeChart createHistogramChart(final String title, final Vector v, final int nbins, final float min, final float max) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(v, nbins, min, max);
        return _createHistogramChart(title, ids);
    }

    public static JFreeChart createHistogramChart(final String title, final Vector v, final int nbins, final HistogramType htype) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(v, nbins);
        ids.setType(htype);
        return _createHistogramChart(title, ids);
    }

    public static JFreeChart createHistogramChart(final String title, final Vector v, final int nbins, final float min, final float max, final HistogramType htype) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(v, nbins, min, max);
        ids.setType(htype);
        return _createHistogramChart(title, ids);
    }

    public static JFreeChart createHistogramChart(final String title, final Matrix m, final int nbins) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(m, nbins);
        return _createHistogramChart(title, ids);
    }

    public static JFreeChart createHistogramChart(final String title, final Matrix m, final int nbins, final float min, final float max) {
        final HistogramDataset ids = XChartModelFactory.createHistogramDataset(m, nbins, min, max);
        return _createHistogramChart(title, ids);
    }

    // does the real charting work
    private static JFreeChart _createHistogramChart(final String title, final IntervalXYDataset dataset) {

        if (dataset == null) {
            throw new IllegalArgumentException("Param dataset cannot be null");
        }

        if (title == null) {
            throw new IllegalArgumentException("Param title cannot be null");
        }

        final JFreeChart chart = ChartFactory.createHistogram(title,
                "foox",
                "fooy",
                dataset,
                PlotOrientation.VERTICAL,
                true,
                false,
                false);
        chart.getXYPlot().setForegroundAlpha(0.75f);
        return chart;
    }

    /**
     * Simple line chart
     *
     * @param title
     * @param y
     * @return
     */
    public static JFreeChart lineY(final String title,
                                   final String xaxisTitle,
                                   final String yaxisTitle,
                                   final Vector y) {

        //XYDataset data = new XYDatasetProxy(title, y);
        XYDataset data = new XYDatasetProxy2(y, yaxisTitle); // @todo fix me
        return ChartFactory.createXYLineChart(title, xaxisTitle,
                yaxisTitle, data,
                PlotOrientation.VERTICAL, true, false, false);
    }

    public static JFreeChart lineOneXManyY(final String comboTitle,
                                           final String[] ytitles,
                                           final String xaxisTitle,
                                           final String yaxisTitle,
                                           final Vector[] yss) {


        XYDataset data = new XYDatasetMultiTmp(ytitles, yss);
        return ChartFactory.createXYLineChart(comboTitle, xaxisTitle,
                yaxisTitle, data,
                PlotOrientation.VERTICAL, true, false, false);
    }

    public static JFreeChart scatterOneXManyY(final String comboTitle,
                                              final String[] ytitles,
                                              final String xaxisTitle,
                                              final String yaxisTitle,
                                              final Vector[] yss) {


        XYDataset data = new XYDatasetMultiTmp(ytitles, yss);
        return ChartFactory.createScatterPlot(comboTitle, xaxisTitle,
                yaxisTitle, data,
                PlotOrientation.VERTICAL,
                true, false, false);
    }

    public static JFreeChart scatterOneXManyY(final String comboTitle,
                                              final String[] ytitles,
                                              final String xaxisTitle,
                                              final String yaxisTitle,
                                              final Vector xcommon,
                                              final Vector[] yss) {

        XYDataset data = new XYDatasetMultiTmp(ytitles, xcommon, yss);
        return ChartFactory.createScatterPlot(comboTitle, xaxisTitle,
                yaxisTitle, data,
                PlotOrientation.VERTICAL,
                false, false, false);
    }

    /**
     * @param title
     * @param y
     * @return
     */
    public static JFreeChart lineY(final String title, Vector y) {
        return lineY(title, "X", "Y", y);
    }

    /**
     * @param title
     * @param xaxistitle
     * @param yaxistitle
     * @param y
     * @return
     */
    public static XYPlot lineYHits(final String xaxistitle,
                                   final String yaxistitle,
                                   final String seriesName,
                                   final Vector y) {

        XYDataset data = new XYDatasetProxy2(y, seriesName);

        NumberAxis xAxis = new NumberAxis(xaxistitle);
        xAxis.setAutoRangeIncludesZero(false); // huh
        NumberAxis yAxis = new NumberAxis(yaxistitle);

        yAxis.setTickMarksVisible(false);
        yAxis.setTickLabelsVisible(true);
        //yAxis.setVisible(false);
        StandardXYItemRenderer rend = new StandardXYItemRenderer(StandardXYItemRenderer.DISCONTINUOUS_LINES);

        //StandardXYItemRenderer rend = new MyRend();
        return new XYPlot(data, xAxis, yAxis, rend);
    }

    public static void display(final XChart xchart) {
        display(xchart.getFreeChart());
    }

    public static void display(final JFreeChart chart) {

        ChartPanel cp = new ChartPanel(chart);
        JFrame frame = new JFrame();
        frame.setSize(500, 500);
        frame.setContentPane(cp);
        frame.setVisible(true);

        /*
        XChartFrame xcframe = new XChartFrame(chart);

        //GenomeChartPanel panel = xcframe.getChartPanel();
        // init common properties
        //panel.setGenerateToolTips(true);
        //panel.setMouseZoomable(true); // dragging for mouse based zooming

        xcframe.setBackground(Color.black);
        xcframe.pack();
        xcframe.setVisible(true);
        */

    }

    public static String getTitle(final JFreeChart chart) {
        Title title = chart.getTitle();

        if (title instanceof TextTitle) {
            return ((TextTitle) title).getText();
        } else {
            return "Dummy title";
        }
    }

    public static GenomeChartPanel createPanel(final XChart chart) {
        return createPanel(chart.getFreeChart());
    }

    public static GenomeChartPanel createPanel(final JFreeChart chart) {
        GenomeChartPanel panel = new GenomeChartPanel(chart);

        // init common properties
        panel.setGenerateToolTips(true);
        panel.setMouseZoomable(true); // dragging for mouse based zooming

        return panel;
    }

    public static JFreeChart lineManyXOneY(final String chartTitle,
                                           final String xaxisLabel,
                                           final Vector[] xValues,
                                           final String[] xNames,
                                           final String yaxisLabel,
                                           final Vector yValues) {

        ensureEqualSizes(yValues, xValues);

        if (xNames.length != xValues.length) {
            throw new IllegalArgumentException("Unequal lengths for xvectors: " + xValues.length + " and xlabels: " + xNames);
        }

        // dont see any way but to duplicate the data
        XYSeries[] serieses = new XYSeries[xValues.length];
        XYSeriesCollection dataset = new XYSeriesCollection();

        for (int i = 0; i < xValues.length; i++) {
            serieses[i] = new XYSeries(xNames[i]);
            for (int j = 0; j < xValues[i].getSize(); j++) {
                serieses[i].add(xValues[i].getElement(j), yValues.getElement(j));
            }
            dataset.addSeries(serieses[i]);
        }

        // create the chart...
        JFreeChart chart = ChartFactory.createXYLineChart(chartTitle,
                xaxisLabel,
                yaxisLabel,
                dataset,
                PlotOrientation.VERTICAL,
                true, // include legend
                true, // tooltips
                false // urls
        );

        // NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...
        chart.setBackgroundPaint(Color.WHITE);

        // get a reference to the plot for further customisation...
        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setBackgroundPaint(Color.LIGHT_GRAY);
        plot.setDomainGridlinePaint(Color.WHITE);
        plot.setRangeGridlinePaint(Color.WHITE);

        // make the real have dots and not line
        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
        renderer.setSeriesLinesVisible(0, false);
        renderer.setSeriesShapesVisible(1, false);
        plot.setRenderer(0, renderer);

        // change the auto tick unit selection to integer units only...
        NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
        rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());

        return chart;
    }

    public static void makeYAxisLogarithmic(final XChart chart) {

        XYPlot plot = chart.getFreeChart().getXYPlot();
        NumberAxis vla = new NumberAxis("range");
        plot.setRangeAxis(vla);

    }

    public static void makeXAxisLogarithmic(final XChart chart) {

        XYPlot plot = chart.getFreeChart().getXYPlot();
        NumberAxis hla = new NumberAxis("domain");
        plot.setDomainAxis(hla);

    }

    public static JFreeChart createHistogram(final String title,
                                             final boolean showlegend,
                                             final String categoryAxisLabel,
                                             final String valueAxisLabel,
                                             final Vector v,
                                             final boolean onlyLines,
                                             final int numBins,
                                             final HistogramType htype) {

        return createHistogram(title, showlegend, categoryAxisLabel, valueAxisLabel, new String[]{valueAxisLabel}, new Vector[]{v}, onlyLines, numBins, htype);
    }

    public static JFreeChart createHistogram(final String title,
                                             final boolean showlegend,
                                             final String categoryAxisLabel,
                                             final String valueAxisLabel,
                                             final String[] vnames,
                                             final Vector[] vss,
                                             final boolean onlyLines,
                                             final int numBins,
                                             final HistogramType htype) {

        float min = Float.NaN, max = Float.NaN;
        final HistogramDataset hds = new HistogramDataset();
        boolean tooltips = true;
        boolean urls = true;

        for (int r = 0; r < vnames.length; r++) {
            if (Float.isNaN(min)) {
                hds.addSeries(vnames[r], vss[r].toArrayDouble(), numBins);
            } else {
                hds.addSeries(vnames[r], vss[r].toArrayDouble(), numBins, min, max);
            }
        }


        hds.setType(htype);
        JFreeChart chart;

        if (onlyLines) {
            chart = ChartFactory.createXYLineChart(title, categoryAxisLabel,
                    valueAxisLabel, hds,
                    PlotOrientation.VERTICAL, showlegend, tooltips, urls);

        } else {
            chart = ChartFactory.createHistogram(title,
                    "fooX",
                    "fooY",
                    hds,
                    PlotOrientation.VERTICAL,
                    true,
                    false,
                    false);
        }

        chart.getXYPlot().setForegroundAlpha(0.75f);
        return chart;
    }

    private static void ensureEqualSizes(final Vector x, final Vector[] ys) {
        int size = x.getSize();

        for (int i = 0; i < ys.length; i++) {

            if (ys[i].getSize() != size) {
                throw new IllegalArgumentException("Unequal vector sizes x: " + x.getSize() + " yvector at: " + i + " size: " + ys[i].getSize());
            }

        }

    }

    public static Range[] createRanges(final HistogramDataset hds) {

        int nseries = hds.getSeriesCount();

        if (nseries > 1) {
            throw new NotImplementedException("Not implemented for more than 1 series: " + nseries);
        }

        java.util.List ranges = new ArrayList();
        for (int i = 0; i < nseries; i++) {
            int nitem = hds.getItemCount(i);
            for (int j = 0; j < nitem; j++) {
                double xstart = hds.getStartXValue(i, j);
                double xend = hds.getEndXValue(i, j);
                ranges.add(new SimpleRange(xstart, xend));
                //System.out.println("seriesn: " + i + " nitem: " + j + " xstart: " + xstart.floatValue() + " xend: " + xend.floatValue());
            }
        }

        return (Range[]) ranges.toArray(new Range[ranges.size()]);
    }
} // End class XChartFactory
