/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.charts.jfreechart.editor;


import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.TraceUtils;
import edu.mit.broad.xbench.core.api.Application;
import org.jfree.chart.*;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.event.ChartChangeEvent;
import org.jfree.chart.event.ChartChangeListener;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;


/**
 * A Swing GUI component for displaying a JFreeChart.
 * <p/>
 * Code initially from com.jrefinery.chart.GenomeChartPanel
 *
 * @see edu.mit.broad.genome.charts.XChartFrame for notes on why this is needed.
 *      <p/>
 *      The basic trick is to subsititute the Properties ... panle for
 *      our own (i.e the ChartPropertyEditPanel)
 * @see "As marked attemptEditChartProperties() below"
 *      <p/>
 *      <p/>
 *      <p/>
 *      The panel is registered to receive notification of changes to the chart, so that the chart can
 *      be redrawn automatically as required.
 *      </P>
 */
public class GenomeChartPanel extends ChartPanel implements
        ActionListener,
        MouseListener,
        MouseMotionListener,
        Printable,
        ChartChangeListener {

    /**
     * The chart that is displayed in the panel.
     */
    protected JFreeChart chart;

    /**
     * Storage for registered (chart) mouse listeners.
     */
    protected final List chartMouseListeners;

    /**
     * A flag that controls whether or not the off-screen buffer is used.
     */
    protected final boolean useBuffer;

    /**
     * A flag that indicates that the buffer should be refreshed.
     */
    protected boolean refreshBuffer;

    /**
     * A buffer for the rendered chart.
     */
    protected Image chartBuffer;

    /**
     * The height of the chart buffer.
     */
    protected int chartBufferHeight;

    /**
     * The width of the chart buffer.
     */
    protected int chartBufferWidth;

    /**
     * The minimum width for drawing a chart (uses scaling for smaller widths).
     */
    protected int minimumDrawWidth;

    /**
     * The minimum height for drawing a chart (uses scaling for smaller heights).
     */
    protected int minimumDrawHeight;

    /**
     * The maximum width for drawing a chart (uses scaling for bigger widths).
     */
    protected int maximumDrawWidth;

    /**
     * The maximum height for drawing a chart (uses scaling for bigger heights).
     */
    protected int maximumDrawHeight;

    /**
     * The popup menu for the frame.
     */
    protected JPopupMenu popup;

    /**
     * The drawing info collected the last time the chart was drawn.
     */
    protected final ChartRenderingInfo info;

    /**
     * The scale factor used to draw the chart.
     */
    protected double scaleX;

    /**
     * The scale factor used to draw the chart.
     */
    protected double scaleY;

    /**
     * The zoom rectangle (selected by the user with the mouse).
     */
    protected Rectangle2D zoomRectangle = null;

    /**
     * The zoom rectangle starting point (selected by the user with a mouse click)
     */
    protected Point2D zoomPoint = null;

    /**
     * Controls if the zoom rectangle is drawn as an outline or filled.
     */
    protected boolean fillZoomRectangle = false;

    protected boolean horizontalZoom = false;

    protected boolean verticalZoom = false;

    protected boolean horizontalAxisTrace = false;

    protected boolean verticalAxisTrace = false;

    /**
     * Menu item for zooming in on a chart (both axes).
     */
    protected JMenuItem zoomInBothAxesMenuItem;

    /**
     * Menu item for zooming in on a chart (horizontal axis).
     */
    protected JMenuItem zoomInHorizontalMenuItem;

    /**
     * Menu item for zooming in on a chart (vertical axis).
     */
    protected JMenuItem zoomInVerticalMenuItem;

    /**
     * Menu item for zooming out on a chart.
     */
    protected JMenuItem zoomOutBothMenuItem;

    /**
     * Menu item for zooming out on a chart (horizontal axis).
     */
    protected JMenuItem zoomOutHorizontalMenuItem;

    /**
     * Menu item for zooming out on a chart (vertical axis).
     */
    protected JMenuItem zoomOutVerticalMenuItem;

    /**
     * Menu item for resetting the zoom (both axes).
     */
    protected JMenuItem autoRangeBothMenuItem;

    /**
     * Menu item for resetting the zoom (horizontal axis only).
     */
    protected JMenuItem autoRangeHorizontalMenuItem;

    /**
     * Menu item for resetting the zoom (vertical axis only).
     */
    protected JMenuItem autoRangeVerticalMenuItem;

    /**
     * Constructs a JFreeChart panel.
     *
     * @param chart The chart.
     */
    public GenomeChartPanel(JFreeChart chart) {

        this(chart,
                DEFAULT_WIDTH,
                DEFAULT_HEIGHT,
                DEFAULT_MINIMUM_DRAW_WIDTH,
                DEFAULT_MINIMUM_DRAW_HEIGHT,
                DEFAULT_MAXIMUM_DRAW_WIDTH,
                DEFAULT_MAXIMUM_DRAW_HEIGHT,
                DEFAULT_BUFFER_USED,
                true, // properties
                true, // save
                true, // print
                true, // zoom
                true   // tooltips
        );

    }

    /**
     * Constructs a JFreeChart panel.
     *
     * @param chart The chart.
     */
    public GenomeChartPanel(JFreeChart chart,
                            boolean properties, boolean save, boolean print, boolean zoom,
                            boolean tooltips) {

        this(chart,
                DEFAULT_WIDTH,
                DEFAULT_HEIGHT,
                DEFAULT_MINIMUM_DRAW_WIDTH,
                DEFAULT_MINIMUM_DRAW_HEIGHT,
                DEFAULT_MAXIMUM_DRAW_WIDTH,
                DEFAULT_MAXIMUM_DRAW_HEIGHT,
                DEFAULT_BUFFER_USED,
                properties,
                save,
                print,
                zoom,
                tooltips
        );

    }

    /**
     * Constructs a JFreeChart panel.
     *
     * @param chart      The chart.
     * @param width      The preferred width of the panel.
     * @param height     The preferred height of the panel.
     * @param useBuffer  A flag that indicates whether to use the off-screen buffer to improve
     *                   performance (at the expense of memory).
     * @param properties A flag indicating whether or not the chart property editor should be
     *                   available via the popup menu.
     * @param save       A flag indicating whether or not save options should be available via the popup
     *                   menu.
     * @param print      A flag indicating whether or not the print option should be available via the
     *                   popup menu.
     * @param zoom       A flag indicating whether or not zoom options should be added to the popup menu.
     * @param tooltips   A flag indicating whether or not tooltips should be enabled for the chart.
     */
    public GenomeChartPanel(JFreeChart chart,
                            int width,
                            int height,
                            int minimumDrawWidth,
                            int minimumDrawHeight,
                            int maximumDrawWidth,
                            int maximumDrawHeight,
                            boolean useBuffer,
                            boolean properties,
                            boolean save,
                            boolean print,
                            boolean zoom,
                            boolean tooltips) {

        super(chart);
        this.chart = chart;
        this.chartMouseListeners = new java.util.ArrayList();
        this.info = new ChartRenderingInfo();
        setPreferredSize(new Dimension(width, height));
        this.useBuffer = useBuffer;
        this.refreshBuffer = false;
        this.chart.addChangeListener(this);
        this.minimumDrawWidth = minimumDrawWidth;
        this.minimumDrawHeight = minimumDrawHeight;
        this.maximumDrawWidth = maximumDrawWidth;
        this.maximumDrawHeight = maximumDrawHeight;

        // set up popup menu...
        this.popup = null;
        if (properties || save || print || zoom) {
            popup = this.createPopupMenu(properties, save, print, zoom);
        }

        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
        enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
        setGenerateToolTips(tooltips);
        this.addMouseListener(this);
        this.addMouseMotionListener(this);

        this.setOpaque(true);

    }

    /**
     * Returns the chart contained in the panel.
     *
     * @return The chart.
     */
    public JFreeChart getChart() {
        return chart;
    }

    /**
     * Sets the chart that is displayed in the panel.
     *
     * @param chart The chart.
     */
    public void setChart(JFreeChart chart) {

        // stop listening for changes to the existing chart...
        if (this.chart != null) {
            this.chart.removeChangeListener(this);
        }

        // add the new chart...
        this.chart = chart;
        this.chart.addChangeListener(this);
        if (this.useBuffer) {
            this.refreshBuffer = true;
        }

        // comm out after upgrade to 0.911
        //Plot plot = chart.getPlot();
        //this.horizontalZoom = this.horizontalZoom && (plot instanceof HorizontalValuePlot);
        //this.verticalZoom = this.verticalZoom && (plot instanceof VerticalValuePlot);

        repaint();

    }

    /**
     * Returns the minimum drawing width for charts.
     * <p/>
     * If the width available on the panel is less than this, then the chart is drawn at the
     * minimum width then scaled down to fit.
     *
     * @return The minimum drawing width.
     */
    public int getMinimumDrawWidth() {
        return this.minimumDrawWidth;
    }

    /**
     * Sets the minimum drawing width for the chart on this panel.
     * <p/>
     * At the time the chart is drawn on the panel, if the available width is less than this
     * amount, the chart will be drawn using the minimum width then scaled down to fit the
     * available space.
     *
     * @param width The width.
     */
    public void setMinimumDrawWidth(int width) {
        this.minimumDrawWidth = width;
    }

    /**
     * Returns the maximum drawing width for charts.
     * <p/>
     * If the width available on the panel is greater than this, then the chart is drawn at the
     * maximum width then scaled up to fit.
     *
     * @return The maximum drawing width.
     */
    public int getMaximumDrawWidth() {
        return this.maximumDrawWidth;
    }

    /**
     * Sets the maximum drawing width for the chart on this panel.
     * <p/>
     * At the time the chart is drawn on the panel, if the available width is greater than this
     * amount, the chart will be drawn using the maximum width then scaled up to fit the
     * available space.
     *
     * @param width The width.
     */
    public void setMaximumDrawWidth(int width) {
        this.maximumDrawWidth = width;
    }

    /**
     * Sets the minimum drawing height for the chart on this panel.
     * <p/>
     * At the time the chart is drawn on the panel, if the available height is less than this
     * amount, the chart will be drawn using the minimum height then scaled down to fit the
     * available space.
     *
     * @param height The height.
     */
    public void setMinimumDrawHeight(int height) {
        this.minimumDrawHeight = height;
    }

    /**
     * Returns the minimum drawing height for charts.
     * <p/>
     * If the height available on the panel is less than this, then the chart is drawn at the
     * minimum height then scaled down to fit.
     *
     * @return The minimum drawing height.
     */
    public int getMinimumDrawHeight() {
        return this.minimumDrawHeight;
    }

    /**
     * Returns the maximum drawing height for charts.
     * <p/>
     * If the height available on the panel is greater than this, then the chart is drawn at the
     * maximum height then scaled up to fit.
     *
     * @return The maximum drawing height.
     */
    public int getMaximumDrawHeight() {
        return this.maximumDrawHeight;
    }

    /**
     * Sets the maximum drawing height for the chart on this panel.
     * <p/>
     * At the time the chart is drawn on the panel, if the available height is greater than this
     * amount, the chart will be drawn using the maximum height then scaled up to fit the
     * available space.
     *
     * @param height The height.
     */
    public void setMaximumDrawHeight(int height) {
        this.maximumDrawHeight = height;
    }

    /**
     * A flag that controls mouse-based zooming.
     */
    public void setMouseZoomable(boolean flag) {
        this.setMouseZoomable(flag, true);
    }

    /**
     * Controls mouse zooming and how the zoom rectangle is displayed
     *
     * @param flag          true if zooming enabled
     * @param fillRectangle true if zoom rectangle is filled, false if rectangle is shown as outline only.
     */
    public void setMouseZoomable(boolean flag, boolean fillRectangle) {
        this.setHorizontalZoom(flag);
        this.setVerticalZoom(flag);
        this.setFillZoomRectangle(fillRectangle);
    }

    /**
     * A flag that controls mouse-based zooming on the horizontal axis.
     */
    public void setHorizontalZoom(boolean flag) {
        // comm out after 0.911 upgrade
        //this.horizontalZoom = flag && (chart.getPlot() instanceof HorizontalValuePlot);
        this.horizontalZoom = flag;
    }

    /**
     * A flag that controls how the zoom rectangle is drawn.
     */
    public void setFillZoomRectangle(boolean flag) {
        this.fillZoomRectangle = flag;
    }

    /**
     * A flag that controls mouse-based zooming on the vertical axis.
     */
    public void setVerticalZoom(boolean flag) {

        // comm out after 0.911 upgrade
        //this.verticalZoom = flag && (chart.getPlot() instanceof VerticalValuePlot);
        this.verticalZoom = flag;
    }

    /**
     * A flag that controls trace lines on the horizontal axis.
     */
    public void setHorizontalAxisTrace(boolean flag) {
        this.horizontalAxisTrace = flag;
    }

    /**
     * A flag that controls trace lines on the vertical axis.
     */
    public void setVerticalAxisTrace(boolean flag) {
        this.verticalAxisTrace = flag;
    }

    /**
     * Switches chart tooltip generation on or off.
     *
     * @param flag The flag.
     */
    public void setGenerateToolTips(boolean flag) {

        //this.info.setGenerateToolTips(flag);
        if (flag == true) {
            ToolTipManager.sharedInstance().registerComponent(this);
        } else {
            ToolTipManager.sharedInstance().unregisterComponent(this);
        }

    }

    /**
     * Returns a string for the tooltip.
     */
    public String getToolTipText(MouseEvent e) {
        return "na";

        /*
        if (this.info.isGenerateToolTips()) {
            EntityCollection entities = this.info.getEntityCollection();
            if (entities!=null) {
                Insets insets = this.getInsets();
                ChartEntity entity = entities.getEntity((int)((e.getX()-insets.left)/scaleX),
                                                        (int)((e.getY()-insets.top)/scaleY));
                if (entity!=null) {
                    result = entity.getToolTipText();
                }
            }
        }
        */
    }

    /**
     * Sets the refresh buffer flag.
     */
    public void setRefreshBuffer(boolean flag) {
        this.refreshBuffer = flag;
    }

    /**
     * Working storage for available panel area after deducting insets.
     */
    private final Rectangle2D available = new Rectangle2D.Double();

    /**
     * Working storage for the chart area.
     */
    private final Rectangle2D chartArea = new Rectangle2D.Double();

    /**
     * Paints the component by drawing the chart to fill the entire component, but
     * allowing for the insets (which will be non-zero if a border has been set for this
     * component).  To increase performance (at the expense of memory), an off-screen buffer image
     * can be used.
     *
     * @param g The graphics device for drawing on.
     */
    public void paintComponent(Graphics g) {

        if (g == null) {
            TraceUtils.showTrace();
            return;
        }

        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;

        // first determine the size of the chart rendering area...
        Dimension size = getSize();
        Insets insets = getInsets();
        available.setRect(insets.left,
                insets.top,
                size.getWidth() - insets.left - insets.right,
                size.getHeight() - insets.top - insets.bottom);

        // work out if scaling is required...
        boolean scale = false;
        double drawWidth = available.getWidth();
        double drawHeight = available.getHeight();
        scaleX = 1.0;
        scaleY = 1.0;

        if (drawWidth < this.minimumDrawWidth) {
            scaleX = drawWidth / minimumDrawWidth;
            drawWidth = minimumDrawWidth;
            scale = true;
        } else if (drawWidth > this.maximumDrawWidth) {
            scaleX = drawWidth / maximumDrawWidth;
            drawWidth = maximumDrawWidth;
            scale = true;
        }

        if (drawHeight < this.minimumDrawHeight) {
            scaleY = drawHeight / minimumDrawHeight;
            drawHeight = minimumDrawHeight;
            scale = true;
        } else if (drawHeight > this.maximumDrawHeight) {
            scaleY = drawHeight / maximumDrawHeight;
            drawHeight = maximumDrawHeight;
            scale = true;
        }

        chartArea.setRect(0.0, 0.0, drawWidth, drawHeight);
        AffineTransform scalingTransform = AffineTransform.getScaleInstance(scaleX, scaleY);

        // are we using the chart buffer?
        if (useBuffer) {

            // do we need to resize the buffer?
            if ((chartBuffer == null) || (chartBufferWidth != available.getWidth())
                    || (chartBufferHeight != available.getHeight())) {

                chartBufferWidth = (int) available.getWidth();
                chartBufferHeight = (int) available.getHeight();
                chartBuffer = createImage(chartBufferWidth, chartBufferHeight);
                refreshBuffer = true;

            }

            // do we need to redraw the buffer?
            if (refreshBuffer) {

                Rectangle2D bufferArea = new Rectangle2D.Double(0, 0,
                        chartBufferWidth,
                        chartBufferHeight);

                Graphics2D bufferG2 = (Graphics2D) chartBuffer.getGraphics();
                if (scale) {
                    AffineTransform saved = bufferG2.getTransform();
                    bufferG2.transform(scalingTransform);
                    chart.draw(bufferG2, chartArea, this.info);
                    bufferG2.setTransform(saved);
                } else {
                    chart.draw(bufferG2, bufferArea, this.info);
                }

                refreshBuffer = false;

            }

            // zap the buffer onto the panel...
            g.drawImage(chartBuffer, insets.left, insets.right, this);

        }

        // or redrawing the chart every time...
        else {

            AffineTransform saved = g2.getTransform();
            g2.translate(insets.left, insets.right);
            if (scale) {
                g2.transform(scalingTransform);
            }
            chart.draw(g2, chartArea, this.info);
            g2.setTransform(saved);

        }

        this.verticalTraceLine = null;
        this.horizontalTraceLine = null;

    }

    /**
     * Receives notification of changes to the chart, and redraws the chart.
     *
     * @param event Details of the chart change event.
     */
    public void chartChanged(ChartChangeEvent event) {

        this.refreshBuffer = true;
        this.repaint();

    }

    /**
     * Handles action events generated by the popup menu.
     *
     * @param event The event.
     */
    public void actionPerformed(ActionEvent event) {

        String command = event.getActionCommand();

        if (command.equals(PROPERTIES_COMMAND)) {
            this.attemptEditChartProperties();
        } else if (command.equals(SAVE_COMMAND)) {
            try {
                this.doSaveAs();
            } catch (IOException e) {
                System.err.println("GenomeChartPanel.doSaveAs: i/o exception = " + e.getMessage());
            }
        } else if (command.equals(PRINT_COMMAND)) {
            this.createChartPrintJob();
        } else if (command.equals(ZOOM_IN_BOTH_COMMAND)) {
            zoomInBoth(this.zoomPoint.getX(), this.zoomPoint.getY());
        } else if (command.equals(ZOOM_IN_DOMAIN_COMMAND)) {
            zoomInHorizontal(this.zoomPoint.getX());
        } else if (command.equals(ZOOM_IN_RANGE_COMMAND)) {
            zoomInVertical(this.zoomPoint.getY());
        } else if (command.equals(ZOOM_OUT_BOTH_COMMAND)) {
            zoomOutBoth(this.zoomPoint.getX(), this.zoomPoint.getY());
        } else if (command.equals(ZOOM_OUT_DOMAIN_COMMAND)) {
            zoomOutHorizontal(this.zoomPoint.getX());
        } else if (command.equals(ZOOM_OUT_RANGE_COMMAND)) {
            zoomOutVertical(this.zoomPoint.getY());
            //} else if (command.equals(AUTO_RANGE_BOTH_COMMAND)) {
            //  autoRangeBoth();
            //} else if (command.equals(AUTO_RANGE_HORIZONTAL_ACTION_COMMAND)) {
            //  autoRangeHorizontal();
            //} else if (command.equals(AUTO_RANGE_VERTICAL_ACTION_COMMAND)) {
            //   autoRangeVertical();
        }

    }

    /**
     * Handles a 'mouse entered' event.  This method does nothing, but is required for
     * implementation of the MouseListener interface.
     *
     * @param e The mouse event.
     */
    public void mouseEntered(MouseEvent e) {
        // do nothing
    }

    /**
     * Handles a 'mouse exited' event.  This method does nothing, but is required for
     * implementation of the MouseListener interface.
     *
     * @param e The mouse event.
     */
    public void mouseExited(MouseEvent e) {
        // do nothing
    }

    /**
     * Handles a 'mouse pressed' event.
     * <p/>
     * This event is the popup trigger on Unix/Linux.  For Windows, the popup trigger is the
     * 'mouse released' event.
     *
     * @param e The mouse event.
     */
    public void mousePressed(MouseEvent e) {


        if (zoomRectangle == null) {
            /*
            this.zoomPoint = RefineryUtilities.getPointInRectangle(e.getX(),
                                                                   e.getY(),
                                                                   this.getScaledDataArea());
            */
            // check for popup trigger...
            if (e.isPopupTrigger()) {
                if (popup != null) {
                    displayPopupMenu(e.getX(), e.getY());
                }
            }
        }

    }

    /**
     * Handles a 'mouse released' event.
     * <p/>
     * On Windows, we need to check if this is a popup trigger, but only if we haven't already
     * been tracking a zoom rectangle.
     *
     * @param e Information about the event.
     */
    public void mouseReleased(MouseEvent e) {

        /*
        if (zoomRectangle != null) {

            if (Math.abs(e.getX() - zoomPoint.getX()) >= MINIMUM_DRAG_ZOOM_SIZE) {
                if (e.getX() < zoomPoint.getX() && e.getY() < zoomPoint.getY()) {
                    autoRangeBoth();
                } else {
                    double w = Math.min(zoomRectangle.getWidth(),
                                        this.getScaledDataArea().getMaxX() - zoomPoint.getX());
                    double h = Math.min(zoomRectangle.getHeight(),
                                        this.getScaledDataArea().getMaxY() - zoomPoint.getY());
                    Rectangle2D zoomArea = new Rectangle2D.Double(zoomPoint.getX(), zoomPoint.getY(), w, h);
                    zoom(zoomArea);
                }
                this.zoomPoint = null;
                this.zoomRectangle = null;
            } else {
                Graphics2D g2 = (Graphics2D) getGraphics();
                g2.setXORMode(java.awt.Color.gray);
                if (fillZoomRectangle == true) {
                    g2.fill(zoomRectangle);
                } else {
                    g2.draw(zoomRectangle);
                }
                g2.dispose();
                this.zoomRectangle = null;
            }

        } else if (e.isPopupTrigger()) {
            if (popup != null) {
                displayPopupMenu(e.getX(), e.getY());
            }
        }
        */

    }

    /**
     * Receives notification of mouse clicks on the panel...these are translated and passed on
     * to any registered chart mouse click listeners.
     *
     * @param event Information about the mouse event.
     */
    public void mouseClicked(MouseEvent event) {

        Insets insets = getInsets();
        int x = (int) ((event.getX() - insets.left) / scaleX);
        int y = (int) ((event.getY() - insets.top) / scaleY);

        // old 'handle click' code...
        // as com out
        //chart.handleClick(x, y, this.info);

        // new entity code...
        if (this.chartMouseListeners.isEmpty()) {
            return;
        }

        ChartEntity entity = this.info.getEntityCollection().getEntity(x, y);
        ChartMouseEvent chartEvent = new ChartMouseEvent(chart, event, entity);

        Iterator iterator = chartMouseListeners.iterator();
        while (iterator.hasNext()) {
            ChartMouseListener listener = (ChartMouseListener) iterator.next();
            listener.chartMouseClicked(chartEvent);
        }

    }

    /**
     * Implementation of the MouseMotionListener's method
     *
     * @param e
     */
    public void mouseMoved(MouseEvent e) {

        if (this.horizontalAxisTrace) {
            drawHorizontalAxisTrace(e.getX());
        }

        if (this.verticalAxisTrace) {
            drawVerticalAxisTrace(e.getY());
        }

        if (this.chartMouseListeners.isEmpty()) {
            return;
        }

        Insets insets = getInsets();
        int x = (int) ((e.getX() - insets.left) / scaleX);
        int y = (int) ((e.getY() - insets.top) / scaleY);

        ChartEntity entity = this.info.getEntityCollection().getEntity(x, y);
        ChartMouseEvent event = new ChartMouseEvent(chart, e, entity);

        Iterator iterator = chartMouseListeners.iterator();
        while (iterator.hasNext()) {
            ChartMouseListener listener = (ChartMouseListener) iterator.next();
            listener.chartMouseMoved(event);
        }

    }

    /**
     * Handles a 'mouse dragged' event.
     *
     * @param e
     */
    public void mouseDragged(MouseEvent e) {

        // if the popup menu has already been triggered, then ignore dragging...
        if (popup != null && popup.isShowing()) {
            return;
        }

        Graphics2D g2 = (Graphics2D) getGraphics();

        // use XOR to erase the previous zoom rectangle (if any)...
        g2.setXORMode(java.awt.Color.gray);
        if (zoomRectangle != null) {
            if (fillZoomRectangle == true) {
                g2.fill(zoomRectangle);
            } else {
                g2.draw(zoomRectangle);
            }
        }

        Rectangle2D scaledDataArea = this.getScaledDataArea();
        if (this.horizontalZoom && this.verticalZoom) {
            // selected rectangle shouldn't extend outside the data area...
            double xmax = Math.min(e.getX(), scaledDataArea.getMaxX());
            double ymax = Math.min(e.getY(), scaledDataArea.getMaxY());
            zoomRectangle = new Rectangle2D.Double(zoomPoint.getX(), zoomPoint.getY(),
                    xmax - zoomPoint.getX(),
                    ymax - zoomPoint.getY());
        }

        if (zoomRectangle != null) {
            // use XOR to draw the new zoom rectangle...
            if (fillZoomRectangle == true) {
                g2.fill(zoomRectangle);
            } else {
                g2.draw(zoomRectangle);
            }
        }
        g2.dispose();

    }

    /**
     * Zooms in on an anchor point (measured in Java2D coordinates).
     *
     * @param x The x value.
     * @param y The y value.
     */
    public void zoomInBoth(double x, double y) {

        zoomInHorizontal(x);
        zoomInVertical(y);

    }

    /**
     * Decreases the range on the horizontal axis, centered about a Java2D x coordinate.
     * <p/>
     * The range on the x axis is halved.
     *
     * @param x coordinate in Java2D space.
     */
    public void zoomInHorizontal(double x) {

        /*// comm out after 0.911 upgrade
        if (chart.getPlot() instanceof HorizontalValuePlot) {
            HorizontalValuePlot hvp = (HorizontalValuePlot) chart.getPlot();
            ValueAxis axis = hvp.getHorizontalValueAxis();
            double value = axis.translateJava2DtoValue((float) x, this.info.getDataArea());
            axis.resizeRange(0.5, value);
        }
        */

    }

    /**
     * Decreases the range on the vertical axis, centered about a Java2D y coordinate.
     * <p/>
     * The range on the y axis is halved.
     *
     * @param y coordinate in Java2D space.
     */
    public void zoomInVertical(double y) {

        /* // comm out after 0.911 upgrade
        if (chart.getPlot() instanceof VerticalValuePlot) {
            VerticalValuePlot vvp = (VerticalValuePlot) chart.getPlot();
            ValueAxis axis = vvp.getVerticalValueAxis();
            double value = axis.translateJava2DtoValue((float) y, this.info.getDataArea());
            axis.resizeRange(0.5, value);
        }
        */

    }

    /**
     * Zooms out on an anchor point (measured in Java2D coordinates).
     *
     * @param x The x value.
     * @param y The y value.
     */
    public void zoomOutBoth(double x, double y) {

        zoomOutHorizontal(x);
        zoomOutVertical(y);

    }

    /**
     * Increases the range on the horizontal axis, centered about a Java2D x coordinate.
     * <p/>
     * The range on the x axis is doubled.
     *
     * @param x coordinate in Java2D space.
     */
    public void zoomOutHorizontal(double x) {

        /* // comm out after 0.911 upgrade
        if (chart.getPlot() instanceof HorizontalValuePlot) {
            HorizontalValuePlot hvp = (HorizontalValuePlot) chart.getPlot();
            ValueAxis axis = hvp.getHorizontalValueAxis();
            double value = axis.translateJava2DtoValue((float) x, this.info.getDataArea());
            axis.resizeRange(2.0, value);
        }
        */

    }

    /**
     * Increases the range on the vertical axis, centered about a Java2D y coordinate.
     * <p/>
     * The range on the y axis is doubled.
     *
     * @param y coordinate in Java2D space.
     */
    public void zoomOutVertical(double y) {

        /* // comm out after 0.911 upgrade
        if (chart.getPlot() instanceof VerticalValuePlot) {
            VerticalValuePlot vvp = (VerticalValuePlot) chart.getPlot();
            ValueAxis axis = vvp.getVerticalValueAxis();
            double value = axis.translateJava2DtoValue((float) y, this.info.getDataArea());
            axis.resizeRange(2.0, value);
        }
        */

    }

    /**
     * Zooms in on a selected region.
     *
     * @param selection The selected region.
     */
    public void zoom(Rectangle2D selection) {

        /* // comm out after 0.911 upgrade
        if ((selection.getHeight() > 0) && (selection.getWidth() > 0)) {

            Rectangle2D scaledDataArea = this.getScaledDataArea();
            if (chart.getPlot() instanceof HorizontalValuePlot) {
                HorizontalValuePlot hvp = (HorizontalValuePlot) chart.getPlot();
                ValueAxis axis = hvp.getHorizontalValueAxis();
                double lower = axis.translateJava2DtoValue((float) selection.getX(), scaledDataArea);
                double upper = axis.translateJava2DtoValue((float) selection.getMaxX(), scaledDataArea);
                axis.setRange(lower, upper);
            }

            if (chart.getPlot() instanceof VerticalValuePlot) {
                VerticalValuePlot vvp = (VerticalValuePlot) chart.getPlot();
                ValueAxis axis = vvp.getVerticalValueAxis();
                double lower = axis.translateJava2DtoValue((float) selection.getMaxY(), scaledDataArea);
                double upper = axis.translateJava2DtoValue((float) selection.getY(), scaledDataArea);
                axis.setRange(lower, upper);
            }
        }
        */

    }

    /**
     * Restores the auto-range calculation on both axes.
     */
    public void autoRangeBoth() {
        autoRangeHorizontal();
        autoRangeVertical();
    }

    /**
     * Restores the auto-range calculation on the horizontal axis.
     */
    public void autoRangeHorizontal() {
        /*// comm out after 0.911 upgrade
        if (chart.getPlot() instanceof HorizontalValuePlot) {
            HorizontalValuePlot hvp = (HorizontalValuePlot) chart.getPlot();
            ValueAxis axis = hvp.getHorizontalValueAxis();
            axis.setAutoRange(true);
        }
        */
    }

    /**
     * Restores the auto-range calculation on the vertical axis.
     */
    public void autoRangeVertical() {

        /* // comm out after 0.911 upgrade
        Plot p = chart.getPlot();
        if (p instanceof VerticalValuePlot) {
            VerticalValuePlot plot = (VerticalValuePlot) p;
            ValueAxis axis = plot.getVerticalValueAxis();
            axis.setAutoRange(true);
        }
        */

    }

    /**
     * Returns the data area for the chart (the area inside the axes) with the current scaling
     * applied.
     *
     * @return The scaled data area.
     */
    public Rectangle2D getScaledDataArea() {
        //Rectangle2D dataArea = this.info.getDataArea(); // before upgrade of jfree chart
        Rectangle2D dataArea = this.info.getChartArea();
        Insets insets = this.getInsets();
        double x = dataArea.getX() * scaleX + insets.left;
        double y = dataArea.getY() * scaleY + insets.top;
        double w = dataArea.getWidth() * scaleX;
        double h = dataArea.getHeight() * scaleY;
        return new Rectangle2D.Double(x, y, w, h);
    }

    private Line2D verticalTraceLine;
    private Line2D horizontalTraceLine;

    /**
     * Draws a vertical line used to trace the mouse position to the horizontal axis.
     *
     * @param x The x-coordinate of the trace line.
     */
    private void drawHorizontalAxisTrace(int x) {

        Graphics2D g2 = (Graphics2D) getGraphics();
        Rectangle2D dataArea = this.getScaledDataArea();

        g2.setXORMode(java.awt.Color.orange);
        if (((int) dataArea.getMinX() < x) && (x < (int) dataArea.getMaxX())) {

            if (verticalTraceLine != null) {
                g2.draw(verticalTraceLine);
                verticalTraceLine.setLine(x, (int) dataArea.getMinY(), x, (int) dataArea.getMaxY());
            } else {
                verticalTraceLine = new Line2D.Float(x, (int) dataArea.getMinY(),
                        x, (int) dataArea.getMaxY());
            }
            g2.draw(verticalTraceLine);
        }

    }

    /**
     * Draws a horizontal line used to trace the mouse position to the vertical axis.
     *
     * @param y The y-coordinate of the trace line.
     */
    private void drawVerticalAxisTrace(int y) {

        Graphics2D g2 = (Graphics2D) getGraphics();
        Rectangle2D dataArea = this.getScaledDataArea();

        g2.setXORMode(java.awt.Color.orange);
        if (((int) dataArea.getMinY() < y) && (y < (int) dataArea.getMaxY())) {

            if (horizontalTraceLine != null) {
                g2.draw(horizontalTraceLine);
                horizontalTraceLine.setLine((int) dataArea.getMinX(), y, (int) dataArea.getMaxX(), y);
            } else {
                horizontalTraceLine = new Line2D.Float((int) dataArea.getMinX(), y,
                        (int) dataArea.getMaxX(), y);
            }
            g2.draw(horizontalTraceLine);
        }

    }

    /**
     * Displays a dialog that allows the user to edit the properties for the current chart.
     *
     * @note as modified for xomics specific chart properties panel
     */
    // cache as intensive to make
    private GenomeChartPropertyEditPanel fChartEdit;

    private void attemptEditChartProperties() {

        // original
        //ChartPropertyEditPanel panel = new ChartPropertyEditPanel(chart);

        if (fChartEdit == null) {
            fChartEdit = new GenomeChartPropertyEditPanel(chart);
        }

        boolean doIt = Application.getWindowManager().showConfirm("Update Chart Properties?");
        if (doIt) {
            fChartEdit.updateChartProperties();
        }


    }

    /**
     * Opens a file chooser and gives the user an opportunity to save the chart in PNG format.
     */
    public void doSaveAs() throws IOException {
        throw new NotImplementedException();

        /*
        FileUtils.ExtensionFileFilter filter = new FileUtils.ExtensionFileFilter("PNG Image Files", ".png");
        XFileChooser fileChooser = Application.getFileManager().getFileChooser(filter);

        boolean proceed = fileChooser.showSaveDialog();
        if (proceed) {
            ChartUtilities.saveChartAsPNG(fileChooser.getSelectedFile(),
                    this.chart, this.getWidth(), this.getHeight());
        }
        */

    }

    /**
     * Creates a print job for the chart.
     */
    public void createChartPrintJob() {

        PrinterJob job = PrinterJob.getPrinterJob();
        job.setPrintable(this);
        if (job.printDialog()) {
            try {
                job.print();
            } catch (PrinterException e) {
                Application.getWindowManager().showError(e);
            }
        }

    }

    /**
     * Prints the chart on a single page.
     */
    public int print(Graphics g, PageFormat pf, int pageIndex) {

        if (pageIndex != 0) {
            return NO_SUCH_PAGE;
        }
        Graphics2D g2 = (Graphics2D) g;
        double x = pf.getImageableX();
        double y = pf.getImageableY();
        double w = pf.getImageableWidth();
        double h = pf.getImageableHeight();
        chart.draw(g2, new Rectangle2D.Double(x, y, w, h), null);
        return PAGE_EXISTS;

    }

    /**
     * Adds a listener to the list of objects listening for chart mouse events.
     *
     * @param listener The listener.
     */
    public void addChartMouseListener(ChartMouseListener listener) {
        this.chartMouseListeners.add(listener);
    }

    /**
     * Removes a listener from the list of objects listening for chart mouse events.
     *
     * @param listener The listener.
     */
    public void removeChartMouseListener(ChartMouseListener listener) {
        this.chartMouseListeners.remove(listener);
    }

    /**
     * Creates a popup menu for the panel.
     *
     * @param properties Include a menu item for the chart property editor.
     * @param save       Include a menu item for saving the chart.
     * @param print      Include a menu item for printing the chart.
     * @param zoom       Include menu items for zooming.
     * @return The popup menu.
     */
    protected JPopupMenu createPopupMenu(boolean properties,
                                         boolean save,
                                         boolean print,
                                         boolean zoom) {

        JPopupMenu result = new JPopupMenu("Chart:");
        boolean separator = false;

        if (properties) {
            JMenuItem propertiesItem = new JMenuItem("Properties...");
            propertiesItem.setActionCommand(PROPERTIES_COMMAND);
            propertiesItem.addActionListener(this);
            result.add(propertiesItem);
            separator = true;
        }

        if (save) {
            if (separator) {
                result.addSeparator();
                separator = false;
            }
            JMenuItem saveItem = new JMenuItem("Save as...");
            saveItem.setActionCommand(SAVE_COMMAND);
            saveItem.addActionListener(this);
            result.add(saveItem);
            separator = true;
        }

        if (print) {
            if (separator) {
                result.addSeparator();
                separator = false;
            }
            JMenuItem printItem = new JMenuItem("Print...");
            printItem.setActionCommand(PRINT_COMMAND);
            printItem.addActionListener(this);
            result.add(printItem);
            separator = true;
        }

        if (zoom) {
            if (separator) {
                result.addSeparator();
                separator = false;
            }

            JMenu zoomInMenu = new JMenu("Zoom In");

            JMenuItem zoomInBothAxesMenuItem = new JMenuItem("Both Axes");
            zoomInBothAxesMenuItem.setActionCommand(ZOOM_IN_BOTH_COMMAND);
            zoomInBothAxesMenuItem.addActionListener(this);
            zoomInMenu.add(zoomInBothAxesMenuItem);

            zoomInMenu.addSeparator();

            zoomInHorizontalMenuItem = new JMenuItem("Horizontal Axis");
            zoomInHorizontalMenuItem.setActionCommand(ZOOM_IN_RANGE_COMMAND);
            zoomInHorizontalMenuItem.addActionListener(this);
            zoomInMenu.add(zoomInHorizontalMenuItem);

            zoomInVerticalMenuItem = new JMenuItem("Vertical Axis");
            zoomInVerticalMenuItem.setActionCommand(ZOOM_IN_DOMAIN_COMMAND);
            zoomInVerticalMenuItem.addActionListener(this);
            zoomInMenu.add(zoomInVerticalMenuItem);

            result.add(zoomInMenu);

            JMenu zoomOutMenu = new JMenu("Zoom Out");

            JMenuItem zoomOutBothMenuItem = new JMenuItem("Both Axes");
            zoomOutBothMenuItem.setActionCommand(ZOOM_OUT_BOTH_COMMAND);
            zoomOutBothMenuItem.addActionListener(this);
            zoomOutMenu.add(zoomOutBothMenuItem);

            zoomOutMenu.addSeparator();

            zoomOutHorizontalMenuItem = new JMenuItem("Horizontal Axis");
            zoomOutHorizontalMenuItem.setActionCommand(ZOOM_OUT_RANGE_COMMAND);
            zoomOutHorizontalMenuItem.addActionListener(this);
            zoomOutMenu.add(zoomOutHorizontalMenuItem);

            zoomOutVerticalMenuItem = new JMenuItem("Vertical Axis");
            zoomOutVerticalMenuItem.setActionCommand(ZOOM_OUT_DOMAIN_COMMAND);
            zoomOutVerticalMenuItem.addActionListener(this);
            zoomOutMenu.add(zoomOutVerticalMenuItem);

            result.add(zoomOutMenu);

            JMenu autoRangeMenu = new JMenu("Auto Range");

            autoRangeBothMenuItem = new JMenuItem("Both Axes");
            //autoRangeBothMenuItem.setActionCommand(AUTO_RANGE_BOTH_ACTION_COMMAND);
            autoRangeBothMenuItem.addActionListener(this);
            autoRangeMenu.add(autoRangeBothMenuItem);

            autoRangeMenu.addSeparator();
            autoRangeHorizontalMenuItem = new JMenuItem("Horizontal Axis");
            //autoRangeHorizontalMenuItem.setActionCommand(AUTO_RANGE_HORIZONTAL_ACTION_COMMAND);
            autoRangeHorizontalMenuItem.addActionListener(this);
            autoRangeMenu.add(autoRangeHorizontalMenuItem);

            autoRangeVerticalMenuItem = new JMenuItem("Vertical Axis");
            //autoRangeVerticalMenuItem.setActionCommand(AUTO_RANGE_VERTICAL_ACTION_COMMAND);
            autoRangeVerticalMenuItem.addActionListener(this);
            autoRangeMenu.add(autoRangeVerticalMenuItem);

            result.addSeparator();
            result.add(autoRangeMenu);

        }

        return result;

    }

    /**
     * Incomplete method - the idea is to modify the zooming options depending on the type of
     * chart being displayed by the panel.
     */
    protected void displayPopupMenu(int x, int y) {

        /* // comm out after 0.911 upgrade
        if (popup != null) {

            // go through each zoom menu item and decide whether or not to enable it...
            Plot plot = this.chart.getPlot();
            if (plot instanceof HorizontalValuePlot) {
                //HorizontalValuePlot hvp = (HorizontalValuePlot)plot;
                //ValueAxis hAxis = hvp.getHorizontalValueAxis();
                // to be completed...
            }
            popup.show(this, x, y);
        }
        */

    }

}
