/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.ext;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.utils.ChainedException;
import org.apache.log4j.Logger;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;


/**
 * Make a system call from java.
 * <p/>
 * Originally from package edu.mit.broad.squid.util;
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class SystemCall extends Thread {

    private static final Logger klog = XLogger.getLogger(SystemCall.class);

    /**
     * Specified cmd is executed unchanged.
     * If waitfor, then java will wait for the process to complete before returning.
     * (but remember that if the whole thing is run in a thread, so the app as a whole may
     * not hang)
     */
    public static SystemCall.Result exec(final String cmd,
                                         final boolean waitFor) throws Exception {

        klog.info("cmd: >" + cmd + "<");

        Process proc = Runtime.getRuntime().exec(cmd);

        if (waitFor) {
            proc.waitFor();
        }

        SystemCall.Result res = new SystemCall.Result(cmd, proc, waitFor);

        //System.out.println(proc);
        //System.out.println("Exit value = " + proc.exitValue());
        return res;
    }

    /**
     * for example foo/bar.Excel.exe
     *
     * @param file
     * @param path2excel
     * @throws Exception
     */
    public static SystemCall.Result launchExternalApplication(final File file, final String path2App) throws Exception {
        return exec(path2App + " \"" + file.getAbsolutePath() + "\"", false);
    }

    public static SystemCall.Result launchExternalApplication(String path2LaunchOn, String path2App) throws Exception {
        return exec(path2App + " \"" + path2LaunchOn + "\"", false);
    }

    public static void launchExternalApplication(final String path2App) throws Exception {

        SystemCall.Result res = null;

        try {

            res = exec(path2App, false);

            klog.debug(res.getSummary());
        } catch (Exception e) {
            if (res != null) {
                throw new ChainedException(res.getSummary(), e);
            } else {
                throw e;
            }
        }
    }

    /**
     * Objectto encapsulate the result of a SystemCall
     */
    public static class Result {

        private String output = "na";
        private final String cmd;
        private int exit = -999;

        /**
         * Class Constructor.
         */
        private Result(String cmd, Process proc, boolean waitfor) throws Exception {

            this.cmd = cmd;

            // If this is not done, then we get an exception such as:

            /*
                    java.lang.IllegalThreadStateException: process has not exited
                    at java.lang.Win32Process.exitValue(Native Method)
                    at edu.mit.broad.genome.utils.SystemCall$Result.<init>(SystemCall.java:94)
                    at edu.mit.broad.genome.utils.SystemCall$Result.<init>(SystemCall.java:77)
                    at edu.mit.broad.genome.utils.SystemCall.exec(SystemCall.java:45)
                    at edu.mit.broad.genome.utils.SystemCall.launchExcel(SystemCall.java:65)
                    at sun.reflect.NativeMethodAccessorImpl.invoke0(Native Method)
                    at sun.reflect.NativeMethodAccessorImpl.invoke(NativeMethodAccessorImpl.java:42)
                    at sun.reflect.DelegatingMethodAccessorImpl.invoke(DelegatingMethodAccessorImpl.java:28)
                    etc.
                    */
            if (waitfor) {
                InputStream is = proc.getInputStream();
                String from = System.getProperty("file.encoding");
                exit = proc.exitValue();
                BufferedReader r = new BufferedReader(new InputStreamReader(is, from));
                String line = null;
                output = "";

                while ((line = r.readLine()) != null) {
                    output += line;
                }

                r.close();
                is.close();
            }
        }

        public int getExitValue() {
            return exit;
        }

        public String getOutput() {
            return output;
        }

        public String getSummary() {

            StringBuffer buf = new StringBuffer("Command: >").append(cmd).append("<\n");

            buf.append(getOutput()).append('\n');
            buf.append("Exit value: ").append(exit).append('\n');

            return buf.toString();
        }
    }    // End SystemCall.Result

}        // End SystemCall
