/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;


import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.utils.ArrayUtils;
import gnu.trove.TFloatArrayList;
import gnu.trove.TIntArrayList;
import gnu.trove.TIntHashSet;

import java.util.List;

/**
 * Vector of scores
 * that can also hold an address for every element
 * <p/>
 * So, every element: score (as usual) + address (eg a row index in a ds)
 * <p/>
 * So really a collection of DoubleElements
 *
 * @author
 * @version %I%, %G%
 */

//Dont extend, rather compose. There are methods in Vector that change
//the elements/element order and we dont want that hapenning without
//us knowing (i.e dont want the order changing without us knowing)
//public class AddressedVector extends Vector {

public class AddressedVector {

    /**
     * @maint IMP IMP IMP
     * If class vars are added make sure to check the constructors below
     * to see if they need to be updated
     * <p/>
     * The indices corresponding to the values/metrics/scores in fVector
     * <p/>
     * <p/>
     * The indices corresponding to the values/metrics/scores in fVector
     */

    // IMP IMMUTABILITY OF THIS OBJECT IS IMPORTANT

    /**
     * The indices corresponding to the values/metrics/scores in fVector
     */
    private int[] fAddresses;

    /**
     * Typically holds some set of values/metrics/scores
     */
    private Vector fVector;

    /**
     * Class Constructor.
     * Data is NOT shared - it is copied.
     *
     * @param v
     * @param addresses
     */
    public AddressedVector(final Vector v, final int[] addresses, final boolean share) {

        if (v == null) {
            throw new IllegalArgumentException("Param v cannot be null");
        }

        if (addresses == null) {
            throw new IllegalArgumentException("Parameter addresses cannot be null");
        }

        if (share) {
            init(v, addresses);
        } else {
            Vector dupv = new Vector(v);
            int[] dadd = new int[addresses.length];
            System.arraycopy(addresses, 0, dadd, 0, addresses.length);
            init(dupv, dadd);
        }
    }

    public AddressedVector(final TFloatArrayList scores, final TIntArrayList addresses) {

        if (scores == null) {
            throw new IllegalArgumentException("Param scores cannot be null");
        }

        if (addresses == null) {
            throw new IllegalArgumentException("Parameter addresses cannot be null");
        }

        init(new Vector(scores), addresses.toNativeArray());
    }

    /**
     * Data is NOT shared - it is copied.
     *
     * @param av
     */
    public AddressedVector(final AddressedVector av, final boolean share) {
        this(av.fVector, av.fAddresses, share);
    }

    /**
     * Class Constructor.
     * Data is NOT shared - it is copied.
     * Indices are inferred (in order) from the specified vector
     *
     * @param v
     */
    public AddressedVector(final Vector v, final boolean share) {
        this(v, ArrayUtils.rangeAsElements(0, _max(v)), share);
    }

    private static int _max(Vector v) {
        int max = v.getSize() - 1;
        if (v.getSize() == 0) {
            max = 0;
        }
        return max;
    }

    /**
     * Class constructor
     *
     * @param rl
     * @param ds
     */
    public AddressedVector(final RankedList rl, final Dataset ds) {
        Vector v = new Vector(rl.getSize());
        int[] addr = new int[rl.getSize()];
        for (int r = 0; r < rl.getSize(); r++) {
            v.setElement(r, rl.getScore(r));
            addr[r] = ds.getRowIndex(rl.getRankName(r));
        }
        this.init(v, addr);
    }

    /**
     * Data is NOT shared - it is copied.
     * Copied from 0 to numElements2Copy
     *
     * @param numElements2Copy
     * @param iv
     */
    public AddressedVector(final int numElements2Copy, final AddressedVector iv) {
        Vector v = new Vector(numElements2Copy, iv.fVector);
        int[] addr = new int[numElements2Copy];
        System.arraycopy(iv.fAddresses, 0, addr, 0, numElements2Copy); // @note duplication
        init(v, addr);
    }

    /**
     * Class Constructor.
     * Data is NOT chared - it is copied.
     * list of DoubleElements
     */
    public AddressedVector(final List listWithDoubleElements) {

        if (listWithDoubleElements == null) {
            throw new IllegalArgumentException("Param listWithDoubleElements cannot be null");
        }

        Vector v = new Vector((DoubleElement.valueArray(listWithDoubleElements)));
        int[] addr = DoubleElement.indexArray(listWithDoubleElements);
        init(v, addr);
    }

    /**
     * Class Constructor.
     *
     * @param dels
     */
    public AddressedVector(final DoubleElement[] dels) {

        if (dels == null) {
            throw new IllegalArgumentException("Param dels cannot be null");
        }

        Vector v = new Vector(DoubleElement.valueArray(dels));
        int[] addr = DoubleElement.indexArray(dels);
        init(v, addr);
    }

    // things must already be duplicated (if neccessary) before calling here
    private void init(final Vector v, final int[] addresses) {

        if (v == null) {
            throw new IllegalArgumentException("Parameter v cannot be null");
        }

        if (addresses == null) {
            throw new IllegalArgumentException("Parameter addresses cannot be null");
        }

        if (v.getSize() != addresses.length) {
            throw new MismatchedSizeException("Unequal lengths - Vector", v.getSize(),
                    "Index addresses", addresses.length);
        }

        this.fVector = v;
        this.fAddresses = addresses;
        this.fVector.setImmutable(); // @note IMP, so that if shared doesnt change on us
    }

    /**
     * A safe copy is returned - data IS copied
     *
     * @return
     */
    public int[] getAddresses() {
        int[] ret = new int[fAddresses.length];
        System.arraycopy(fAddresses, 0, ret, 0, fAddresses.length);
        return ret;
    }

    public int getAddress(final int vector_element_index) {
        return fAddresses[vector_element_index];
    }

    public float getScore(final int vector_element_index) {
        return fVector.getElement(vector_element_index);
    }

    public float[] getScoresArray() {
        return fVector.toArray();
    }

    public Vector getScoresV(final boolean clonedCopy) {
        return new Vector(fVector, clonedCopy);
    }

    public int getSize() {
        return fVector.getSize();
    }

    public TIntHashSet toAddressesSet() {
        TIntHashSet addrSet = new TIntHashSet(fAddresses.length);
        addrSet.addAll(fAddresses);

        if (addrSet.size() != fAddresses.length) {
            throw new IllegalStateException("Addresses NOT unique! # of addresses: " + fAddresses.length + " not preserved after hashing set size: " + addrSet.size());
        }

        return addrSet;
    }

}    // End AddressedVector


