/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.utils.ImmutedException;
import gnu.trove.TFloatArrayList;
import gnu.trove.TIntArrayList;


/**
 * Holds true and falses only
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class BooleanVector {

    // note: only elementCount data in elementData are valid.
    // elementData.length is the allocated size.
    // invariant: elementData.length >= elementCount.
    // @note (as) making these protected for package friendly use (by Matrix etc)

    /**
     * @maint IMP IMP IMP IMP
     * When adding methods that alter datain this BooleanVector, make sure
     * to do a immutability check
     * The number of elements in this vector
     * The number of elements in this vector
     */

    /** The number of elements in this vector */
    protected int elementCount;

    /**
     * The raw floats
     */
    protected boolean elementData[];

    /**
     * Flag for whether this BooleanVector can have its values altered or not
     */
    private boolean fImmutable;

    /**
     * Class Constructor.
     * Constructs a new generalized mathematic BooleanVector with zero value (0.0)
     * elements; length represents the number of elements in the
     * vector.
     *
     * @param length number of elements in this vector.
     */
    public BooleanVector(int length) {
        elementCount = length;
        elementData = new boolean[length];    // will be initialized to false
    }

    /**
     * Class Constructor.
     * Data IS copied - not shared.
     * Constructs a new generalized mathematic BooleanVector with zero
     * elements; length represents the number of elements in the
     * vector. !! this comment is a bug in Sun's API !!
     *
     * @param values the values for the new vector.
     */
    public BooleanVector(boolean values[]) {

        this(values.length);

        System.arraycopy(values, 0, elementData, 0, elementCount);
    }

    /**
     * Class Constructor.
     * Data IS copied - no sharing
     * Constructs a new BooleanVector and copies the initial values from
     * the parameter vector.
     * Resulting BooleanVector  is NOT immutable even if the specified BooleanVector was immutable
     * (permits manipulation)
     *
     * @param vector the source for the new BooleanVector's initial values
     */
    public BooleanVector(BooleanVector vector) {
        this(vector.elementCount, vector);    // all elements
    }

    /**
     * Class Constructor.
     * Data IS copied - no sharing
     * Constructs a new BooleanVector and copies the initial values from
     * the parameter vector.
     * Resulting vector  is NOT immutable even if the specified BooleanVector was immutable
     * (permits manipulation)
     *
     * @param vector the source for the new BooleanVector's initial values
     */
    public BooleanVector(int numelementstouse, BooleanVector vector) {

        this(numelementstouse);

        System.arraycopy(vector.elementData, 0, elementData, 0, numelementstouse);

        //this.fImmutable = vector.fImmutable; // NO DONT -- see note above
    }

    /**
     * Class Constructor.
     * Data IS copied - no sharing.
     * Constructs a new BooleanVector by copying length elements from the
     * array parameter.  The parameter length must be less than or
     * equal to vector.length.
     *
     * @param vector The array from which the values will be copied.
     * @param length The number of values copied from the array.
     */
    public BooleanVector(boolean vector[], int length) {

        // ArrayIndexOutOfBounds occur if length > vector.legnth
        this(length);

        System.arraycopy(vector, 0, elementData, 0, elementCount);
    }

    /**
     * Class Constructor.
     * BooleanVector is constructed by appending the elements of A and B
     * in order.
     * So, resulting vector is of lenth lenA + lenB.
     * Resulting vector  is NOT immutable even if the specified BooleanVector was immutable
     * (permits manipulation)
     *
     * @param vectorA
     * @param vectorB
     */
    public BooleanVector(BooleanVector vectorA, BooleanVector vectorB) {
        this(new BooleanVector[]{vectorA, vectorB});
    }

    /**
     * Class Constructor.
     * BooleanVector is constructed by appending elements of vectors in order.
     * So, length of this BooleanVector is the sum of lengths of individual vectors.
     * Resulting vector  is NOT immutable even if the specified BooleanVector was immutable
     * (permits manipulation)
     *
     * @param vectors
     */
    public BooleanVector(BooleanVector[] vectors) {

        if (vectors == null) {
            throw new IllegalArgumentException("Param vectors cannot be null");
        }

        int length = 0;

        for (int i = 0; i < vectors.length; i++) {
            length += vectors[i].getSize();
        }

        this.elementCount = length;
        this.elementData = new boolean[length];    // will be initialized to 0.0

        int pos = 0;

        for (int i = 0; i < vectors.length; i++) {
            System.arraycopy(vectors[i].elementData, 0, elementData, pos, vectors[i].elementCount);

            pos += vectors[i].elementCount;
        }
    }

    /**
     * shortens the vector
     */
    public void truncate(int newsize) {

        checkImmutable();

        if (newsize == elementCount) {
            return;    // no change needed
        }

        if (newsize > this.getSize()) {
            throw new IllegalArgumentException("Truncated size: " + newsize
                    + " cannot be more than current size: " + getSize());
        }

        boolean[] oldData = elementData;

        elementData = new boolean[newsize];

        System.arraycopy(oldData, 0, elementData, 0, newsize);

        elementCount = newsize;
    }

    /**
     * Sets the value of this vector to the values found in the
     * array parameter. The array should be at least equal in
     * length to the number of elements in the vector.
     *
     * @param vector the source array
     */
    public void set(boolean vector[]) {

        checkImmutable();

        // note: only this.elementCount data is copied.(no auto-grow)
        System.arraycopy(vector, 0, elementData, 0, elementCount);
    }

    /**
     * same as set but doesnt do a systemcopy. so memory space shared
     *
     * @see set(boolean vector[])
     */
    public void setDirectly(boolean[] data) {

        checkImmutable();

        this.elementData = data;
        this.elementCount = data.length;
    }

    /**
     * Sets the value of this vector to the values found in
     * vector vector.
     *
     * @param vector the source vector
     */
    public void set(BooleanVector vector) {

        checkImmutable();

        // note: only this.elementCount data is copied.(no auto-grow)
        System.arraycopy(vector.elementData, 0, elementData, 0, elementCount);
    }

    /**
     * Returns the number of elements in this vector.
     *
     * @return number of elements in this vector
     */
    public int getSize() {
        return elementCount;
    }

    public int getNumTrue() {

        int cnt = 0;

        for (int i = 0; i < elementCount; i++) {
            if (elementData[i]) {
                cnt++;
            }
        }

        return cnt;
    }

    public int getNumFalse() {

        int cnt = 0;

        for (int i = 0; i < elementCount; i++) {
            if (!elementData[i]) {
                cnt++;
            }
        }

        return cnt;
    }

    /**
     * Makes true -> 1
     * and false 0->
     * ALL elements included
     *
     * @return
     */
    public Vector toVector() {
        Vector v = new Vector(this.getSize());
        for (int i = 0; i < this.getSize(); i++) {
            if (elementData[i]) {
                v.setElement(i, 1);
            } else {
                v.setElement(i, 0);
            }
        }

        return v;

    }

    public int[] getTrueIndices() {

        TIntArrayList indices = new TIntArrayList();

        for (int i = 0; i < elementCount; i++) {
            if (elementData[i]) {
                indices.add(i);
            }
        }

        return indices.toNativeArray();
    }

    public Vector getTrueIndicesV() {

        TFloatArrayList indices = new TFloatArrayList();

        for (int i = 0; i < elementCount; i++) {
            if (elementData[i]) {
                indices.add(i);
            }
        }

        return new Vector(indices.toNativeArray());
    }

    public int[] getFalseIndices() {

        TIntArrayList indices = new TIntArrayList();

        for (int i = 0; i < elementCount; i++) {
            if (!elementData[i]) {
                indices.add(i);
            }
        }

        return indices.toNativeArray();
    }

    public Vector getFalseIndicesV() {

        TFloatArrayList indices = new TFloatArrayList();

        for (int i = 0; i < elementCount; i++) {
            if (!elementData[i]) {
                indices.add(i);
            }
        }

        return new Vector(indices.toNativeArray());
    }

    /**
     * Retrieves the value at the specified index value of this
     * vector.
     *
     * @param index the index of the element to retrieve (zero indexed)
     * @return the value at the indexed element
     */
    public boolean getElement(int index) {

        try {
            return elementData[index];
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new ArrayIndexOutOfBoundsException("index:" + index + "must be in [0, "
                    + (elementCount - 1) + "]");
        }
    }

    public boolean isTrue(int index) {
        return getElement(index);
    }

    public boolean isFalse(int index) {
        return !getElement(index);
    }

    /**
     * true -> 1.0
     * false -> 0.0
     *
     * @param index
     * @return
     */
    public float getElementAsFloat(int index) {

        boolean val = getElement(index);

        if (val) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Modifies the value at the specified index of this vector.
     *
     * @param index the index if the element to modify (zero indexed)
     * @param value the new vector element value
     */
    public void setElement(int index, boolean value) {

        checkImmutable();

        try {
            elementData[index] = value;
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new ArrayIndexOutOfBoundsException("index:" + index + " must be in [0, "
                    + (elementCount - 1) + "]");
        }
    }

    /**
     * Returns a string that contains the values of this BooleanVector.
     *
     * @return the String representation
     */
    public String toString() {

        StringBuffer buf = new StringBuffer();

        buf.append("(");

        for (int i = 0; i < elementCount - 1; i++) {
            buf.append(elementData[i]);
            buf.append(",");
        }

        buf.append(elementData[elementCount - 1]);
        buf.append(")");

        return buf.toString();
    }

    public String toString(char delim) {

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < elementCount - 1; i++) {
            buf.append(elementData[i]);
            buf.append(delim);
        }

        buf.append(elementData[elementCount - 1]);

        return buf.toString();
    }

    /**
     * returns a safe copy
     */
    public boolean[] toArray() {

        boolean[] fl = new boolean[elementData.length];

        System.arraycopy(elementData, 0, fl, 0, elementData.length);

        return fl;
    }

    /**
     * @param vectors
     * @return Max size of all specified vectors
     */
    public static int maxSize(BooleanVector[] vectors) {

        int max = 0;

        for (int i = 0; i < vectors.length; i++) {
            if (max < vectors[i].getSize()) {
                max = vectors[i].getSize();
            }
        }

        return max;
    }

    /**
     * Make this BooleanVector immutable.
     *
     * @param immutable
     */
    public void setImmutable() {
        this.fImmutable = true;
    }

    /**
     * @return wther ir not this vector is immutable
     */
    public boolean isImmutable() {
        return fImmutable;
    }

    private void checkImmutable() {

        if (fImmutable) {
            throw new ImmutedException();
        }
    }
}    // End BooleanVector

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
