/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.RankedListGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.RankedList;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

/**
 * "quick" version of stat calculators -> cached stats
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class CachedMetricStats {

    /**
     * For logging support
     */
    private static final transient Logger klog = XLogger.getLogger(CachedMetricStats.class);

    /**
     * Class constructor
     */
    public static class Pearson implements MetricStats {

        private double squaresum;
        private double sum;
        private Vector x;

        /**
         * Class constructor
         *
         * @param x
         */
        public Pearson(Vector x) {
            this.x = x;
            this.squaresum = x.squaresum();
            this.sum = x.sum();
        }

        public double getScore(MetricStats mc) {

            Pearson ps = (Pearson) mc;

            double N = (double) x.getSize();

            if (N == 0 || N == 1) {
                return Float.NaN;
            }

            //double numr = (sumprod(x, y) - x.sum()*y.sum()) / N;
            double numr = x.sumprod(ps.x) - ((sum * ps.sum) / N);
            double denr = ((squaresum - ((sum * sum) / N))) * ((ps.squaresum - ((ps.sum * ps.sum) / N)));

            denr = Math.sqrt(denr);

            //double denr = ((x.squaresum() - (x.sum()*x.sum())) / N) * ((y.squaresum() - (y.sum()*y.sum())) / N);
            //denr = Math.sqrt(denr);

            /*
            klog.debug("numr=" + numr + " denr=" + denr);
            klog.debug("x>>" + x.squaresum());
            klog.debug("x>>" + (x.sum()*x.sum()));
            klog.debug("x>>" + ((x.squaresum() - ((x.sum()*x.sum()) / N) )));
            klog.debug("y>>" + ps.x.squaresum());
            klog.debug("y>>" + (ps.x.sum()*ps.x.sum()));
            klog.debug("y>>" + ((ps.x.squaresum() - ((ps.x.sum()*ps.x.sum()) / N) )));
            */


            double score = numr / denr;

            if (Double.isInfinite(score)) {
                klog.warn("pstats: infinite:\n" + x.toString('\t') + "\n" + ps.x.toString('\t'));
            }

            return numr / denr;
        }

    }


    /**
     * Class constructor
     */

    // cacjing of NO help
    public static class Euclidean implements MetricStats {

        private Vector x;

        /**
         * Class constructor
         *
         * @param x
         */
        public Euclidean(Vector x) {
            this.x = x;
        }

        public double getScore(MetricStats mc) {

            // caching of NO help at all
            Euclidean ps = (Euclidean) mc;
            return XMath.euclidean(x, ps.x);
        }

    }

    /**
     * Class constructor
     */
    public static class Spearman implements MetricStats {

        private int[] ranks;

        /**
         * Class constructor
         *
         * @param x
         */
        public Spearman(Vector x, Dataset ds) {

            /*
            //log.info("Running: " + metric.getName() + " on: " + ds.getName());
            DoubleElement[] dels = new DoubleElement[x.getSize()];

            int rank = 1;
            for (int i = 0; i < x.getSize(); i++) {
                dels[i] = (new DoubleElement(rank++, x.getElement(i)));
            }

            dels = DoubleElement.sort(SortMode.REAL, Order.DESCENDING, dels);

            this.ranks = DoubleElement.indexArray(dels);
            */

            //log.info("Running: " + metric.getName() + " on: " + ds.getName());
            List dels = new ArrayList(x.getSize());

            for (int i = 0; i < x.getSize(); i++) {
                double dist = x.getElement(i);
                dels.add(new DoubleElement(i, dist));
            }

            AddressedVector av = new AddressedVector(dels);

            // @note colnames -> as sorting a profile
            RankedList rl = RankedListGenerators.sortByVectorAndGetRankedList(av, SortMode.REAL, Order.DESCENDING, ds.getColumnNames());

            int[] ranks = new int[ds.getNumCol()];
            for (int c = 0; c < ds.getNumCol(); c++) {
                ranks[c] = rl.getRank(ds.getColumnName(c)) + 1; // +1 as ranks for spearman from 1 to n
            }

            //Printf.out(ranks);

            this.ranks = ranks;

            //System.out.println(x.toString());
            //Printf.out(ranks);
        }

        public double getScore(MetricStats mc) {
            Spearman sp = (Spearman) mc;
            return XMath.spearman(ranks, sp.ranks);
        }

    } // End class Spearman


} // End CachedMetricStats


