/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;


import java.awt.*;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ColorSchemes {

    public static ColorScheme[] createAll() {
        return new ColorScheme[]{new BroadCancer(), new Stanford(),
                new HighContrast(), new ThreeColor(),
                new CorrColors(),
                new CorrColors2(),
                new MatlabColors()};
    }

    public static ColorScheme lookup(final Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof ColorScheme) {
            return (ColorScheme) obj;
        }

        ColorScheme[] all = createAll();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookupColorScheme for: " + obj);
    }


    /**
     * Base class
     */
    static abstract class BasicColorScheme implements ColorScheme {

        private final String fName;
        private final ColorValue[] fColorValues;

        /**
         * Privatized Class Constructor.
         */
        public BasicColorScheme(final String type, final int[] colors, final String[] colorlabels) {

            this.fName = type;
            this.fColorValues = new ColorValue[colors.length];

            for (int i = 0; i < colors.length; i++) {
                fColorValues[i] = new ColorValue(new Color(colors[i]), colorlabels[i]);
            }
        }

        public BasicColorScheme(final String type, final Color[] colors) {

            this.fName = type;
            this.fColorValues = new ColorValue[colors.length];

            for (int i = 0; i < colors.length; i++) {
                fColorValues[i] = new ColorValue(colors[i], "na");
            }
        }

        /**
         * Class constructor
         *
         * @param name
         * @param colors
         */
        public BasicColorScheme(final String name, final ColorValue[] colorvalues) {

            this.fName = name;
            this.fColorValues = new ColorValue[colorvalues.length];

            for (int i = 0; i < colorvalues.length; i++) {
                fColorValues[i] = new ColorValue(new Color(colorvalues[i].color.getRGB()), colorvalues[i].value);
            }
        }

        public ColorValue[] getColors() {
            return fColorValues;
        }

        public Color getMinColor() {
            return fColorValues[0].color;
        }

        public Color getMaxColor() {
            return fColorValues[fColorValues.length - 1].color;
        }

        public int getNumColors() {
            return fColorValues.length;
        }

        public Color getColor(int pos) {
            return fColorValues[pos].color;
        }

        public String getValue(int pos) {
            return fColorValues[pos].value;
        }

        public String toString() {
            return fName;
        }

        public boolean equals(final Object obj) {

            if (obj instanceof ColorScheme) {
                if (((BasicColorScheme) obj).fName.equals(this.fName)) {
                    return true;
                }
            }

            return false;
        }

        public int hashCode() {
            return fName.hashCode();
        }

        public String getName() {
            return fName;
        }

    } // BaseColorScheme


    public static class BroadCancer extends BasicColorScheme {
        /**
         * Default mit heat map
         * Note that there are 12 diff colors
         */
        private static final int[] MIT_COLORS_SCHEME = {
                0x4500ad, 0x2700d1, 0x6b58ef, 0x8888ff, 0xc7c1ff, 0xd5d5ff,
                0xffc0e5, 0xff8989, 0xff7080,
                0xff5a5a, 0xef4040, 0xd60c00
        };

        private static final String[] MIT_COLORS_LABELS = {
                "-3.0", "-2.5", "-2.0", "-1.5", "-1.0", "-0.5", "+0.5", "+1.0", "+1.5", "+2.0", "+2.5", "+3.0"
        };


        public BroadCancer() {
            super("MIT", MIT_COLORS_SCHEME, MIT_COLORS_LABELS);
        }
    }

    public static class BroadCancerRed extends BasicColorScheme {
        /**
         * Default mit heat map
         * Note that there are 12 diff colors
         */
        private static final int[] MIT_RED_COLORS_SCHEME = {
                0xffc0e5, 0xff8989, 0xff7080,
                0xff5a5a, 0xef4040, 0xd60c00
        };

        private static final String[] MIT_RED_COLORS_LABELS = {
                "+0.5", "+1.0", "+1.5", "+2.0", "+2.5", "+3.0"
        };


        public BroadCancerRed() {
            super("MIT_RED", MIT_RED_COLORS_SCHEME, MIT_RED_COLORS_LABELS);
        }
    }

    public static class BroadCancerBlue extends BasicColorScheme {
        /**
         * Default mit heat map
         * Note that there are 12 diff colors
         */
        private static final int[] MIT_BLUE_COLORS_SCHEME = {
                0x4500ad, 0x2700d1, 0x6b58ef, 0x8888ff, 0xc7c1ff, 0xd5d5ff
        };

        private static final String[] MIT_BLUE_COLORS_LABELS = {
                "-3.0", "-2.5", "-2.0", "-1.5", "-1.0", "-0.5"
        };


        public BroadCancerBlue() {
            super("MIT_BLUE", MIT_BLUE_COLORS_SCHEME, MIT_BLUE_COLORS_LABELS);
        }
    }


    public static class CorrColors extends BasicColorScheme {
        /**
         * Default MIT heat map
         * Note that there are 12 diff colors
         */
        private static final int[] CORR_COLORS_SCHEME = {
                0x4500ad, 0x6b58ef, 0x8888ff, 0xc7c1ff, 0xd5d5ff,
                0xffc0e5, 0xff8989, 0xff7080,
                0xff5a5a, 0xd60c00
        };

        private static final String[] CORR_COLORS_LABELS = {
                "-3.0", "-2.0", "-1.5", "-1.0", "-0.5", "+0.5", "+1.0", "+1.5", "+2.0", "+3.0"
        };


        public CorrColors() {
            super("Corr", CORR_COLORS_SCHEME, CORR_COLORS_LABELS);
        }
    }

    public static class MatlabColors extends BasicColorScheme {
        /**
         * Default MIT heat map
         * Note that there are 12 diff colors
         */
        private static final Color[] MATLAB_COLORS_SCHEME = new Color[]{

                new Color(0f, 0f, 0.5625f),
                new Color(0f, 0f, 0.625f),
                new Color(0f, 0f, 0.6875f),
                new Color(0f, 0f, 0.75f),
                new Color(0f, 0f, 0.8125f),
                new Color(0f, 0f, 0.875f),
                new Color(0f, 0f, 0.9375f),
                new Color(0f, 0f, 1f),
                new Color(0f, 0.0625f, 1f),
                new Color(0f, 0.125f, 1f),
                new Color(0f, 0.1875f, 1f),
                new Color(0f, 0.25f, 1f),
                new Color(0f, 0.3125f, 1f),
                new Color(0f, 0.375f, 1f),
                new Color(0f, 0.4375f, 1f),
                new Color(0f, 0.5f, 1f),
                new Color(0f, 0.5625f, 1f),
                new Color(0f, 0.625f, 1f),
                new Color(0f, 0.6875f, 1f),
                new Color(0f, 0.75f, 1f),
                new Color(0f, 0.8125f, 1f),
                new Color(0f, 0.875f, 1f),
                new Color(0f, 0.9375f, 1f),
                new Color(0f, 1f, 1f),
                new Color(0.0625f, 1f, 0.9375f),
                new Color(0.125f, 1f, 0.875f),
                new Color(0.1875f, 1f, 0.8125f),
                new Color(0.25f, 1f, 0.75f),
                new Color(0.3125f, 1f, 0.6875f),
                new Color(0.375f, 1f, 0.625f),
                new Color(0.4375f, 1f, 0.5625f),
                new Color(0.5f, 1f, 0.5f),
                new Color(0.5625f, 1f, 0.4375f),
                new Color(0.625f, 1f, 0.375f),
                new Color(0.6875f, 1f, 0.3125f),
                new Color(0.75f, 1f, 0.25f),
                new Color(0.8125f, 1f, 0.1875f),
                new Color(0.875f, 1f, 0.125f),
                new Color(0.9375f, 1f, 0.0625f),
                new Color(1f, 1f, 0f),
                new Color(1f, 0.9375f, 0f),
                new Color(1f, 0.875f, 0f),
                new Color(1f, 0.8125f, 0f),
                new Color(1f, 0.75f, 0f),
                new Color(1f, 0.6875f, 0f),
                new Color(1f, 0.625f, 0f),
                new Color(1f, 0.5625f, 0f),
                new Color(1f, 0.5f, 0f),
                new Color(1f, 0.4375f, 0f),
                new Color(1f, 0.375f, 0f),
                new Color(1f, 0.3125f, 0f),
                new Color(1f, 0.25f, 0f),
                new Color(1f, 0.1875f, 0f),
                new Color(1f, 0.125f, 0f),
                new Color(1f, 0.0625f, 0f),
                new Color(1f, 0f, 0f),
                new Color(0.9375f, 0f, 0f),
                new Color(0.875f, 0f, 0f),
                new Color(0.8125f, 0f, 0f),
                new Color(0.75f, 0f, 0f),
                new Color(0.6875f, 0f, 0f),
                new Color(0.625f, 0f, 0f),
                new Color(0.5625f, 0f, 0f),
                new Color(0.5f, 0f, 0f)
        };


        public MatlabColors() {
            super("MatlabColors", MATLAB_COLORS_SCHEME);
        }
    }

    public static class CorrColors2 extends BasicColorScheme {
        /**
         * Default MIT heat map
         * Note that there are 12 diff colors
         */
        private static final int[] CORR_COLORS_SCHEME = {
                0x4500ad, 0x8888ff, 0xc7c1ff, 0xd5d5ff,
                0xffc0e5, 0xff8989, 0xff7080,
                0xd60c00
        };

        private static final String[] CORR_COLORS_LABELS = {
                "-3.0", "-2.0", "-1.5", "-1.0", "+1.0", "+1.5", "+2.0", "+3.0"
        };


        public CorrColors2() {
            super("Corr2", CORR_COLORS_SCHEME, CORR_COLORS_LABELS);
        }
    }

    public static class ThreeColor extends BasicColorScheme {
        /**
         * Default MIT heat map
         * Note that there are 12 diff colors
         */
        private static final ColorValue[] THREE_COLORS_SCHEME = {
                /*
                new ColorValue(Color.GREEN, "-1"),
                new ColorValue(Color.WHITE, "0"),
                new ColorValue(Color.MAGENTA, "+1")
                */


                new ColorValue(Color.BLUE, "-1"),
                new ColorValue(Color.WHITE, "0"),
                new ColorValue(Color.RED, "+1")
        };

        public ThreeColor() {
            super("ThreeColor", THREE_COLORS_SCHEME);
        }
    }

    public static class ThreeColor_green extends BasicColorScheme {

        private static final ColorValue[] THREE_COLORS_SCHEME = {
                new ColorValue(Color.WHITE, "-1"),
                new ColorValue(Color.WHITE, "0"),
                new ColorValue(Color.GREEN, "+1")
        };

        public ThreeColor_green() {
            super("ThreeColor_green", THREE_COLORS_SCHEME);
        }
    }

    public static class MembershipColor extends BasicColorScheme {
        /**
         * Default MIT heat map
         * Note that there are 12 diff colors
         */
        private static final ColorValue[] MEMBERSHIP_COLORS_SCHEME = {

                new ColorValue(Color.WHITE, "-1"),
                new ColorValue(Color.WHITE, "0"),
                new ColorValue(Color.GREEN, "+1")
        };

        public MembershipColor() {
            super("MembershipColor", MEMBERSHIP_COLORS_SCHEME);
        }
    }

    public static class Stanford extends BasicColorScheme {
        /**
         * Default MIT heat map
         * Note that there are 12 diff colors
         */
        // seems to be 11 in all, with the middle being black80
        // green -> go from 0 to 255 (on the g)  in 5 steps
        // red -> go from 0 to 255 (on the r) in 5 steps
        private static final ColorValue[] STANFORD_COLORS_SCHEME = {
                new ColorValue(new Color(0, 255, 0), "-3.0"),
                new ColorValue(new Color(0, 200, 0), "-2.5"),
                new ColorValue(new Color(0, 150, 0), "-2.0"),
                new ColorValue(new Color(0, 100, 0), "-1.5"),
                new ColorValue(new Color(0, 50, 0), "-1.0"),
                new ColorValue(Color.black, "0"),
                new ColorValue(new Color(50, 0, 0), "+1.0"),
                new ColorValue(new Color(100, 0, 0), "+1.5"),
                new ColorValue(new Color(150, 0, 0), "+2.0"),
                new ColorValue(new Color(200, 0, 0), "+2.5"),
                new ColorValue(new Color(255, 0, 0), "+3.0"),
        };

        public Stanford() {
            super("Stanford", STANFORD_COLORS_SCHEME);
        }
    }


    public static class HighContrast extends BasicColorScheme {

        // Not really sure what i'm doing here
        // just the top 3either dir of bpog
        // Ok - so reducing the number of colors makes the contrast a lot more -- easier
        // for things to look different (and thus the moniker 'high contrast')
        // shows that the bpog can be misleading at times!
        private static final int[] HIGH_CONTRAST = {
                0x4500ad, 0x2700d1, 0x6b58ef,
                0xff5a5a, 0xef4040, 0xd60c00
        };

        private static final String[] HIGH_CONTRAST_LABELS = {
                "-3", "-2", "-1",
                "+1", "+2", "+3"
        };

        HighContrast() {
            super("High-Contrast", HIGH_CONTRAST, HIGH_CONTRAST_LABELS);
        }
    }


}    // End DefaultColorScheme

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

/**
 * for the four color scheme
 */
/*
public static final Color EQUAL_COLOR = Color.green;
public static final Color LOWER_COLOR = Color.blue;
public static final Color HIGHER_COLOR = Color.red;
public static final Color NAN_COLOR = Color.black;
public static final Color ERROR_COLOR = Color.white;
*/

/*

private static final ColorValue[] STANFORD2_COLORS_= {
new ColorValue(new Color(0, 255, 0), "-3.0"),
new ColorValue(new Color(0, 170, 0), "-2.5"),
new ColorValue(new Color(0, 85, 0), "-1.0"),

new ColorValue(Color.black, "0"),

new ColorValue(new Color(85, 0, 0), "+1.0"),
new ColorValue(new Color(170, 0, 0), "+2.0"),
new ColorValue(new Color(255, 0, 0), "+3.0"),
};

// fromdchip 0.6, 1.8, 3
private static final ColorValue[] STANFORD3_COLORS_ = {
new ColorValue(new Color(0, 255, 0), "-3.0"),
new ColorValue(new Color(0, 153, 0), "-1.8"),
new ColorValue(new Color(0, 51, 0), "-0.6"),

new ColorValue(Color.black, "0"),

new ColorValue(new Color(51, 0, 0), "+0.6"),
new ColorValue(new Color(153, 0, 0), "+1.8"),
new ColorValue(new Color(255, 0, 0), "+3.0"),
};

*/

/*
private static final ColorValue[] STANFORD4_COLORS = {
    new ColorValue(new Color(0, 255, 0), "-3.0"),
    new ColorValue(new Color(0, 153, 0), "-1.8"),
    new ColorValue(new Color(0, 51, 0), "-0.6"),
    new ColorValue(new Color(0, 20, 0), "-0.25"),
    new ColorValue(new Color(0, 10, 0), "-0.1"),

    new ColorValue(Color.black, "0"),

    new ColorValue(new Color(10, 0, 0), "+0.1"),
    new ColorValue(new Color(20, 0, 0), "+0.25"),
    new ColorValue(new Color(51, 0, 0), "+0.6"),
    new ColorValue(new Color(153, 0, 0), "+1.8"),
    new ColorValue(new Color(255, 0, 0), "+3.0"),
};
*/

/* // these are the values listed in the eisen paper
// but they are log scale - here we ae not log scale
private static final ColorValue[] STANFORD_COLORS = {
    new ColorValue(new Color(0, 255, 0), ">8"),
    new ColorValue(new Color(0, 200, 0), ">6"),
    new ColorValue(new Color(0, 150, 0), ">4"),
    new ColorValue(new Color(0, 100, 0), ">2"),
    new ColorValue(new Color(0, 50, 0), ">0.5"),

    new ColorValue(Color.black, "1:1"),

    new ColorValue(new Color(50, 0, 0), ">0.5"),
    new ColorValue(new Color(100, 0, 0), ">2"),
    new ColorValue(new Color(150, 0, 0), ">4"),
    new ColorValue(new Color(200, 0, 0), ">6"),
    new ColorValue(new Color(255, 0, 0), ">8"),
};
*/
