/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;


import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import java.util.Random;


/**
 * Methods related to data fdr.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DistributionGenerator {

    private final Logger log = XLogger.getLogger(DistributionGenerator.class);

    /**
     * Class Constructor.
     */
    public DistributionGenerator() {
    }

    /**
     * Generates npoints normally distributed with mean mean and variance var
     * <p/>
     * Some random seed based on current time is created om each call.
     * Hence results (same or diff) can be unpredicatbale if rapidly called.
     * For that cache a seed once and reuse (see method below)
     */
    public Vector norm(int npoints, double mean, double variance) {

        Random rnd = new Random(System.currentTimeMillis());

        return norm(npoints, mean, variance, rnd);
    }

    /**
     * Generates a vector with npoints elements normally distributed with mean mean and
     * variance var using the specified Random generator object.
     * <p/>
     * If the seed is the same, then the same set of numbers will be generated each call
     */
    public Vector norm(int npoints, double mean, double variance, Random rnd) {

        Vector v = new Vector(npoints);

        for (int i = 0; i < npoints; i++) {
            v.setElement(i, (float) (mean + Math.sqrt(variance) * rnd.nextGaussian()));
        }

        return v;
    }

    /**
     * Creates a 2 class matrix
     * Datapoints for ngenes (rows)
     * Each class's points are normally distributed around specified mean with specified variance.
     * <p/>
     * Generate a dataset with nsamples using a gaussian distribution.
     * So:
     * samples for class A centered around meanA with varA
     * samples for class B centered around meanB with varB
     */
    public Matrix norm(int ngenes, int nsamplesA, double meanA, double varA, int nsamplesB,
                       double meanB, double varB) {

        Random rnd = new Random(System.currentTimeMillis());    // cached
        Matrix matrix = new Matrix(ngenes, nsamplesA + nsamplesB);

        for (int i = 0; i < ngenes; i++) {
            Vector a = norm(nsamplesA, meanA, varA, rnd);
            Vector b = norm(nsamplesB, meanB, varB, rnd);
            Vector gene = new Vector(a, b);

            matrix.setRow(i, gene);
        }

        return matrix;
    }

    /**
     * Creates a Matrix with random data.
     * ngenes rows and nsamples columns.
     * Data points distributed randomly between min and max.
     */
    public Matrix random(int ngenes, int nsamples, float min, float max) {

        Random rnd = new Random(System.currentTimeMillis());    // cached
        Matrix matrix = new Matrix(ngenes, nsamples);
        float range = max - min;

        for (int i = 0; i < ngenes; i++) {
            Vector gene = new Vector(nsamples);

            for (int j = 0; j < nsamples; j++) {
                gene.setElement(j, min + range * rnd.nextFloat());
            }

            matrix.setRow(i, gene);
        }

        return matrix;
    }
}    // End Distributions

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
