/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import edu.mit.broad.genome.objects.LabelledVector;
import edu.mit.broad.genome.utils.ImmutedException;
import gnu.trove.TObjectIntHashMap;
import gnu.trove.TObjectIntIterator;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Jan 21, 2004
 *         Time: 3:19:22 PM
 */
public class FrequencyMap {

    private TObjectIntHashMap fElementInstanceCounter;

    private String fName;

    private boolean fImmutable;

    /**
     * Class constructor
     */
    public FrequencyMap(String name) {
        this.fElementInstanceCounter = new TObjectIntHashMap();
        this.fName = name;
        this.fImmutable = false;
    }

    public String getName() {
        return fName;
    }

    public void setImmutable() {
        this.fImmutable = true;
    }

    private void ensureMutable() {
        if (fImmutable) {
            throw new ImmutedException();
        }
    }

    public void add(Object obj) {
        ensureMutable();
        int curr = 0;
        if (fElementInstanceCounter.containsKey(obj)) {
            curr = fElementInstanceCounter.get(obj);
        }

        curr++;

        fElementInstanceCounter.put(obj, curr);
    }

    public void addAll(Collection coll) {
        ensureMutable();
        for (Iterator it = coll.iterator(); it.hasNext();) {
            add(it.next());
        }
    }

    public void addAll(Object[] objs) {
        ensureMutable();
        for (int i = 0; i < objs.length; i++) {
            add(objs[i]);
        }
    }

    public Set getElements(int minInstanceCountInclusive) {
        Set elements = new HashSet();
        // faster iteration by avoiding hasNext():
        TObjectIntIterator it = fElementInstanceCounter.iterator();
        for (int i = fElementInstanceCounter.size(); i-- > 0;) {
            it.advance();
            if (it.value() >= minInstanceCountInclusive) {
                elements.add(it.key());
            }
        }

        return elements;
    }

    public Object[] getElementsArray(int minInstanceCountInclusive) {
        Set elements = getElements(minInstanceCountInclusive);
        return elements.toArray(new Object[elements.size()]);
    }

    private LabelledVector lv; // lazilly made

    public LabelledVector toLabelledVector() {
        if (lv == null) {
            Object[] objs = getElementsArray();
            List labels = new ArrayList(objs.length);
            Vector v = new Vector(objs.length);
            for (int i = 0; i < objs.length; i++) {
                v.setElement(i, getFrequency(objs[i]));
                labels.add(objs[i].toString());
            }

            lv = new LabelledVector(labels, v, true);
        }

        return lv;
    }

    public int getSize() {
        return fElementInstanceCounter.size();
    }

    public Object[] getElementsArray() {
        return fElementInstanceCounter.keys();
    }

    public Set getElements() {
        Set all = new HashSet();
        Object[] objs = getElementsArray();
        for (int i = 0; i < objs.length; i++) {
            all.add(objs[i]);
        }

        if (all.size() != objs.length) {
            throw new IllegalStateException("Wierd duplicate keys");
        }

        return all;
    }

    public int getFrequency(Object obj) {
        if (fElementInstanceCounter.containsKey(obj) == false) {
            throw new IllegalArgumentException("No such key: " + obj);
        } else {
            return fElementInstanceCounter.get(obj);
        }
    }

} // End class FrequencyMap
