/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetMetrics;
import org.apache.log4j.Logger;

/**
 * @author Aravind Subramanian
 *         Date: Jul 30, 2003
 *         Time: 10:15:25 PM
 */
public class MatrixMetrics {

    private final Logger log = XLogger.getLogger(DatasetMetrics.class);

    public double getSayanMatrixProduct(Matrix a, Matrix b) {
        double numr = sum_AijBijProduct(a, b);
        double denr = Math.sqrt(sum_ijSquare(a)) * Math.sqrt(sum_ijSquare(b));
        return (numr / denr);
    }

    /**
     * NOT standardized (z-score)
     * The sum of the products of the space and time distances between all possible pairs of cases
     * sumi=1 to N sum j=1 to N sij tij
     * <p/>
     * Where N is the number of cases, tij is the distance between cases i and j in
     * time, and sij is the distance between cases i and j in space.
     * When space-time interaction is present cases near in space will also be near in time,
     * and the test statistic will be large if the reciprocal transformation is
     * applied to the distances.
     * <p/>
     * Standardized (-1 <= r <= +1)
     * <p/>
     * Alternatively, a standardized statistic may be used:
     * <p/>
     * (1/N**2 - N - 1) * sum i=1->N sum j=1->N [(sij - smean) * (tij - tmean) ] / stdevs * stdevt
     * <p/>
     * Where are average space and time distances and ss and st are standard deviations of
     * the space and time distances. r is a measure of matrix correlation with range -1 to 1
     * Both r and Z become large when the time distances are linearly dependent on the space
     * distances. A non-linear dependence of time on space therefore will not necessarily
     * result in a significant Mantel test.
     *
     * @param a
     * @param b
     * @return
     * @see http://zappa.nku.edu/~longa/geomed/stats/Mantel/more.html#Image82.gif
     */
    public double getMantelProduct(Matrix a, Matrix b, boolean useMedian, boolean standardized) {

        double score;

        if (!standardized) {
            score = sum_AijBijProduct(a, b);
        } else {
            int N = a.getNumRow();
            double nterm = 1.0d / (double) (N * N - N + 1);
            double numr = sum_Aij_minus_Amean_prd_Bij_minus_Bmean(a, b, useMedian);
            double denr = a.stddev(false, false) * b.stddev(false, false);
            score = nterm * numr / denr;
        }

        return score;
    }

    public double getMantelProduct(int customN, Matrix a, Matrix b, boolean useMedian) {

        double score;

        int N = customN;
        double nterm = 1.0d / (double) (N * N - N + 1);
        double numr = sum_Aij_minus_Amean_prd_Bij_minus_Bmean(a, b, useMedian);
        double denr = a.stddev(false, false) * b.stddev(false, false);
        score = nterm * numr / denr;


        return score;
    }

    /**
     * sum i=1->N sum j=1->N [(sij - smean) * (tij - tmean)
     *
     * @param a
     * @param b
     * @return
     */
    public double sum_Aij_minus_Amean_prd_Bij_minus_Bmean(Matrix a, Matrix b, boolean useMedian) {
        enforceEqualDimensions(a, b);

        double sum = 0;
        double amean;
        double bmean;

        if (useMedian) {
            amean = a.median();
        } else {
            amean = a.mean();
        }

        if (useMedian) {
            bmean = b.median();
        } else {
            bmean = b.mean();
        }

        for (int i = 0; i < a.elementData.length; i++) {
            sum += (a.elementData[i] - amean) * (b.elementData[i] - bmean);
        }

        return sum;
    }

    public double sum_AijBijProduct(Matrix a, Matrix b) {
        enforceEqualDimensions(a, b);

        double sum = 0;
        for (int i = 0; i < a.elementData.length; i++) {
            sum += a.elementData[i] * b.elementData[i];
        }

        return sum;
    }

    /**
     * sum(ij^^2)
     *
     * @return
     */
    public double sum_ijSquare(Matrix a) {
        double sum = 0;
        for (int i = 0; i < a.elementData.length; i++) {
            sum += a.elementData[i] * a.elementData[i];
        }

        return sum;
    }

    private void enforceEqualDimensions(Matrix a, Matrix b) {

        if (a.getNumRow() != b.getNumRow()) {
            throw new IllegalArgumentException("Mismatched matrices: must be of equal row lengths a: " + a.getNumRow() + " b: " + b.getNumRow());
        }

        if (a.getNumCol() != b.getNumCol()) {
            throw new IllegalArgumentException("Mismatched matrices: must be of equal col lengths a: " + a.getNumCol() + " b: " + b.getNumCol());
        }

    }

} // End MatrixMetrics
