/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.utils.ImmutedException;
import org.apache.log4j.Logger;


/**
 * Stripped down matrix to store objects (i.e a 2d array with bounds checking)
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ObjectMatrix implements IMatrix {

    /* (javax.vecmath note)
     * Implementation note:
     * The size of the matrix does NOT automaticly grow.
     * It does only when setSize is called. I believe this is the spec
     * and makes less confusion, less user bugs.
     */

    /**
     * @maint IMP IMP: if any fields are added to this class,
     * make sure to update the constructors and the
     * set(ObjectMatrix method
     * <p/>
     * The data of the ObjectMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * <p/>
     * The data of the ObjectMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     */

    /**
     * The data of the ObjectMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     */
    private Object elementData[];

    /**
     * The number of rows in this matrix.
     */
    private int fRowCnt;

    /**
     * The number of columns in this matrix.
     */
    private int fColCnt;
    private boolean fImmuted;
    private static final transient Logger klog = XLogger.getLogger(ObjectMatrix.class);

    /**
     * Class Constructor.
     * Must call set() after this form of teh constructor.
     */
    public ObjectMatrix() {
    }

    /**
     * Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     */
    public ObjectMatrix(int nrows, int ncols) {

        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = new Object[fRowCnt * fColCnt];
    }

    /**
     * Constructs a new ObjectMatrix and copies the initial values
     * from the parameter matrix.
     *
     * @param matrix the source of the initial values of the new ObjectMatrix
     *               <p/>
     *               as added: if shared values, then this matrix shares the elements with the specified amtric
     *               else the array is systemarrycopied
     */
    public ObjectMatrix(ObjectMatrix matrix, boolean shareValues) {

        this.fRowCnt = matrix.fRowCnt;
        this.fColCnt = matrix.fColCnt;
        this.fImmuted = matrix.fImmuted;

        if (shareValues) {
            this.fImmuted = matrix.fImmuted;
        } else {
            this.fImmuted = false;
        }

        if (shareValues) {
            this.elementData = matrix.elementData;
        } else {
            int newSize = fRowCnt * fColCnt;
            this.elementData = new String[newSize];
            System.arraycopy(matrix.elementData, 0, elementData, 0, newSize);
        }
    }


    /**
     * the cloned matrix is NOT immutable even if the matrix it was cloned from was.
     */
    public ObjectMatrix cloneDeep() {
        klog.debug("Making cloneDeep of ObjectMatrix");
        return new ObjectMatrix(this, false);
    }

    /**
     * Returns the number of rows in this matrix.
     *
     * @return number of rows in this matrix
     */
    public int getNumRow() {
        return fRowCnt;
    }

    public int getDim() {
        return getNumRow() * getNumCol();
    }

    /**
     * Returns the number of colmuns in this matrix.
     *
     * @return number of columns in this matrix
     */
    public int getNumCol() {
        return fColCnt;
    }

    /**
     * Retrieves the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be retrieved (zero indexed)
     * @param column the column number to be retrieved (zero indexed)
     * @return the value at the indexed element
     */
    public Object getElement(int row, int column) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        return elementData[row * fColCnt + column];
    }

    /**
     * Modifies the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be modified (zero indexed)
     * @param column the column number to be modified (zero indexed)
     * @param value  the new matrix element value
     */
    public void setElement(int row, int column, Object value) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " <= matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        elementData[row * fColCnt + column] = value;
    }

    /**
     * Copy the values from the array into the specified row of this
     * matrix.
     *
     * @param row   the row of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setRow(int row, Object array[]) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row: " + row + " > matrix's fRowCnt: "
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row: " + row + " < 0");
        }

        if (array.length < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("specified array length: " + array.length
                    + " < matrix's fColCnt: " + fColCnt);
        }

        System.arraycopy(array, 0, elementData, row * fColCnt, fColCnt);
    }

    /**
     * Copy the values from the array into the specified column of this
     * matrix.
     *
     * @param col   the column of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setColumn(int col, Object[] objs) {

        checkImmutable();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (objs.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + objs.length
                    + " < matrix's fRowCnt:" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            elementData[i * fColCnt + col] = objs[i];
        }
    }

    /**
     * REAL objects returned a safe copy is NOT returned.
     */
    public Object[] getColumn(int col) {

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        Object[] ret = new Object[fRowCnt];

        for (int i = 0; i < fRowCnt; i++) {
            ret[i] = elementData[i * fColCnt + col];
        }

        return ret;
    }

    /**
     * IMP REAL OBJECTS - NOT A SAFE COPY returned
     * NOT TESTED
     */
    public Object[] getRow(int row) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        Object[] ret = new Object[fColCnt];

        System.arraycopy(elementData, row * fColCnt, ret, 0, fColCnt);

        return ret;
    }


    public int hashCode() {

        int hash = 0;

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            long bits = (elementData[i].hashCode());

            hash ^= (int) (bits ^ (bits >> 32));
        }

        return hash;
    }

    public void setImmutable() {
        this.fImmuted = true;
    }

    public boolean isImmutable() {
        return fImmuted;
    }

    private void checkImmutable() {

        if (fImmuted) {
            throw new ImmutedException();
        }
    }
}    // End ObjectMatrix

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
