/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import java.util.StringTokenizer;

/**
 * Represent a real score and its significance
 *
 * @todo consider if the sig level can be abstracted better
 * Perhaops integrate with Sig and SigSet interfaces
 */
public class SigDetailed {

    private static SortMode DEFAULT_SORT = SortMode.REAL;
    private static Order DEFAULT_ORDER = Order.DESCENDING;

    private float fRealValue;
    private Vector fRndScores; // optional
    private float fPValue;
    private float fPtOnePercent;
    private float fOnePercent;
    private float fFivePercent;
    private int fRndCntMore;

    private SigDetailed() {
    }

    /**
     * rndScorescan be very large, so stored only of asked for
     * Class constructor
     *
     * @param realValue
     * @param rndScores
     * @param storeRndScores
     */
    public SigDetailed(float realValue, Vector rndScores, boolean storeRndScores) {
        rndScores.sort(DEFAULT_SORT, DEFAULT_ORDER);

        Vector storedRndValues = null;
        if (storeRndScores) {
            storedRndValues = rndScores;
        }

        this.init(realValue, XMath.getSignificanceLevelASIS(rndScores, 0.001f),
                XMath.getSignificanceLevelASIS(rndScores, 0.01f),
                XMath.getSignificanceLevelASIS(rndScores, 0.05f),
                XMath.getPValueCnt(realValue, rndScores),
                (float) XMath.getPValue(realValue, rndScores),
                storedRndValues
        );
    }

    /**
     * Class constructor
     *
     * @param realValue
     * @param ptOnePercent
     * @param onePercent
     * @param fivePercent
     * @param rndCntMore
     * @param pValue
     * @param rndScores    can be null is dont want to store large amt of data
     */
    public SigDetailed(float realValue, float ptOnePercent, float onePercent, float fivePercent,
                       int rndCntMore, float pValue, Vector rndScores) {

        this.init(realValue, ptOnePercent, onePercent, fivePercent,
                rndCntMore, pValue, rndScores);
    }

    // common init routine
    private void init(float realValue, float ptOnePercent, float onePercent, float fivePercent,
                      int rndCntMore, float pValue, Vector rndScores) {

        this.fRealValue = realValue;
        this.fPtOnePercent = ptOnePercent;
        this.fOnePercent = onePercent;
        this.fFivePercent = fivePercent;
        this.fRndCntMore = rndCntMore;
        this.fPValue = pValue;
        this.fRndScores = rndScores;
    }

    public float getRealValue() {
        return fRealValue;
    }

    public float getPValue() {
        return fPValue;
    }

    public float getPtOnePercent() {
        return fPtOnePercent;
    }

    public float getOnePercent() {
        return fOnePercent;
    }

    public float getFivePercent() {
        return fFivePercent;
    }

    public int getRndCntMore() {
        return fRndCntMore;
    }

    /**
     * NOT always available
     *
     * @return
     */
    public Vector getRndScores() {
        return fRndScores;
    }

    /**
     * @return
     * @maint order matter -- see parseSig below
     */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append(getRealValue()).append(':');
        buf.append(getPtOnePercent()).append(':');
        buf.append(getOnePercent()).append(':');
        buf.append(getFivePercent()).append(':');
        buf.append(getPValue()).append(':');
        buf.append(getRndCntMore());
        return buf.toString();
    }

    public static SigDetailed parseSig(String str) {
        StringTokenizer tok = new StringTokenizer(str, ":");

        if (tok.countTokens() != 6) {
            throw new RuntimeException("Bad format expected 6 tokens but got: " + tok.countTokens() + " >" + str + "<");
        }

        SigDetailed sig = new SigDetailed();
        sig.fRealValue = Float.parseFloat(tok.nextToken());
        sig.fPtOnePercent = Float.parseFloat(tok.nextToken());
        sig.fOnePercent = Float.parseFloat(tok.nextToken());
        sig.fFivePercent = Float.parseFloat(tok.nextToken());
        sig.fPValue = Float.parseFloat(tok.nextToken());
        sig.fRndCntMore = Integer.parseInt(tok.nextToken());
        return sig;
    }

} // End class Sig
