/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import edu.mit.broad.genome.alg.AlgMap;

import java.util.Map;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class VectorFunctors {

    /**
     * Privatized Class Constructor.
     */
    private VectorFunctors() {
    }

    // @maint add a functor and this needs attention
    public static VectorFunctor.Statistic[] createAllStats() {
        return new VectorFunctor.Statistic[]{new Mean(), new Median(), new StdDev(), new Sum()};
    }

    public static VectorFunctor.Statistic createMeanMedianStat(boolean useMedian) {
        if (useMedian) {
            return new Median();
        } else {
            return new Mean();
        }
    }

    public static VectorFunctor.Statistic lookupStat(Object obj) {
        if (obj == null) {
            throw new IllegalArgumentException("Parameter obj cannot be null");
        }

        if (obj instanceof VectorFunctor.Statistic) {
            return (VectorFunctor.Statistic) obj;
        }

        String name = obj.toString();

        VectorFunctor.Statistic[] all = createAllStats();
        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(name)) {
                return all[i];
            }
        }

        throw new IllegalArgumentException("Unknown VectorFunctor.Statistic: " + name);
    }

    static abstract class AbstractStat implements VectorFunctor.Statistic {

        public int hashCode() {
            return getName().hashCode();
        }

        public boolean equals(Object obj) {

            if (obj instanceof VectorFunctor.Statistic) {
                return getName().equalsIgnoreCase(((VectorFunctor.Statistic) obj).getName());
            }

            return false;
        }

        public String toString() {
            return getName();
        }
    }

    /**
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class Mean extends AbstractStat {

        public static final String NAME = "Mean";

        /**
         * not needed -- no params accepted
         *
         * @param map
         */
        public void setParam(Map map) {
        }

        /**
         * @param v
         * @return
         */
        public double getValue(Vector v) {
            return v.mean();
        }

        public String getName() {
            return NAME;
        }

    } // End inner class Mean

    /**
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class Median extends AbstractStat {

        public static final String NAME = "Median";

        /**
         * not needed -- no params accepted
         *
         * @param map
         */
        public void setParam(Map map) {
        }

        /**
         * No params accepted
         *
         * @param v
         * @return
         */
        public double getValue(Vector v) {
            return v.median();
        }

        public String getName() {
            return NAME;
        }

    } // End inner class Median


    /**
     * Params
     * <p/>
     * Metric.FIX_LOW
     * Metric.USE_BIASED,
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class StdDev extends AbstractStat {

        private boolean fUseBiased = false;
        private boolean fFixLow = true;
        public static final String NAME = "StdDev";

        public StdDev() {
            super();
        }

        public StdDev(boolean useBiased, boolean fixLow) {
            this.fUseBiased = useBiased;
            this.fFixLow = fixLow;
        }

        /**
         * Metric.FIX_LOW
         * Metric.USE_BIASED
         *
         * @param map
         */
        public void setParam(Map map) {
            this.fFixLow = AlgMap.isFixLowVar(map);
            this.fUseBiased = AlgMap.isBiased(map);
        }

        public double getValue(Vector v) {
            return v.stddev(fUseBiased, fFixLow);
        }

        public String getName() {
            return "StdDev";
        }

    } // End StdDev

    /**
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class Sum extends AbstractStat {

        public static final String NAME = "Sum";

        /**
         * not needed -- no params accepted
         *
         * @param map
         */
        public void setParam(Map map) {
        }

        /**
         * No params accepted
         *
         * @param v
         * @return
         */
        public double getValue(Vector v) {
            return v.sum();
        }

        public String getName() {
            return NAME;
        }

    } // End inner class Sum

} // End VectorFunctors