/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.StringDataframe;
import gnu.trove.TFloatArrayList;
import gnu.trove.TIntArrayList;
import org.apache.commons.math.stat.regression.SimpleRegression;
import org.apache.log4j.Logger;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

/**
 * Functionally extends java.lang.Math with additional math related methods.
 *
 * @author Michael Angelo (CMath in GeneCluster)
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class XMath1 {

    private static final Logger klog = XLogger.getLogger(XMath.class);
    public static Const SCORE = new Const("score");
    public static Const RANK = new Const("rank");

    public static class Const {

        private String fName;

        private Const(String name) {
            this.fName = name;
        }

        public String toString() {
            return fName;
        }

        public String getName() {
            return fName;
        }

        public int hashCode() {
            return fName.hashCode();
        }

        public boolean equals(Object obj) {
            if (obj instanceof Const) {
                return fName.equals(((Const) obj).fName);
            }

            return false;
        }

        public boolean isRank() {
            return fName.equals(RANK.fName);
        }

        public boolean isScore() {
            return fName.equals(SCORE.fName);
        }
    }

    /**
     * Privatized class constructor.
     */
    private XMath1() {
    }

    public static float distance2similarity(float dist) {
        if (XMath.isPositive(dist)) {
            return 1 - dist;
        } else {
            float sim = 1 - Math.abs(dist);
            return -1 * sim;
        }
    }

    public static boolean isAscending(int[] ints) {

        for (int i = 0; i < ints.length - 1; i++) {
            int thisValue = ints[i];
            int nextValue = ints[i + 1];
            if (nextValue < thisValue) {
                return false;
            }
        }

        return true;
    }

    public static boolean isContiguous(int[] ints) {

        for (int i = 0; i < ints.length - 1; i++) {
            int thisValue = ints[i];
            int nextValue = ints[i + 1];
            if (nextValue != (thisValue + 1)) {
                return false;
            }
        }

        return true;
    }

    public static float getFWERTwoTailed(final float realScore, final Matrix rndScores) {
        if (isPositive(realScore)) {
            return getFWER(realScore, rndScores, true);
        } else {
            return getFWERLessThan(realScore, rndScores);
        }
    }

    public static float getFWER(final float realScore, final Matrix rndScores, final boolean pos) {

        Vector best_of_each_perm;
        if (pos) {
            best_of_each_perm = rndScores.getColumnMaxes();
        } else {
            best_of_each_perm = rndScores.getColumnMins();
        }

        double fwer;
        if (pos) {
            fwer = XMath.getPValue(realScore, best_of_each_perm);
        } else {
            fwer = XMath.getPValueLessThan(realScore, best_of_each_perm);
        }

        return (float) fwer;
    }

    public static float getFWERLessThan(final float realScore, final Matrix rndScores) {

        Vector lowest_of_each_perm = new Vector(rndScores.getNumCol());
        for (int c = 0; c < rndScores.getNumCol(); c++) {
            lowest_of_each_perm.setElement(c, rndScores.getColumnV(c).min());
        }

        return (float) XMath.getPValueLessThan(realScore, lowest_of_each_perm);
    }

    public static int sum(final int[] elements) {
        int tot = 0;
        for (int i = 0; i < elements.length; i++) {
            tot += elements[i];
        }

        return tot;
    }

    public static float sum(final float[] elements) {
        float tot = 0;
        for (int i = 0; i < elements.length; i++) {
            tot += elements[i];
        }

        return tot;
    }

    public static float sum_nan_aware(final float[] elements, float replaceNanWith) {
        float tot = 0;
        for (int i = 0; i < elements.length; i++) {
            if (Float.isNaN(elements[i])) {
                tot += replaceNanWith;
            } else {
                tot += elements[i];
            }
        }

        return tot;
    }

    public static double sum(final double[] elements) {
        double tot = 0;
        for (int i = 0; i < elements.length; i++) {
            tot += elements[i];
        }

        return tot;
    }

    public static double varMeanNorm(final float score, final Vector v) {
        return (score - v.mean()) / v.var(false, false);
    }

    /**
     * @param d
     * @return
     * @see http://www.cs.utsa.edu/~wagner/laws/ALogs.html
     *      Java supplies a function to calculate natural logs,
     *      base e = 2.718281828459045. To calculate logs to other bases,
     *      you need to multiply by a fixed constant: for a log base b multiply by 1/logeb
     */
    public static double log2(final double d) {
        return Math.log(d) / Math.log(2.0);
    }

    // log10: Logarithm base 10
    public static double log10(final double d) {
        return Math.log(d) / Math.log(10.0);
    }

    // ooh for auto-boxing
    public static float[] toFloats(final double[] dss) {
        float[] fss = new float[dss.length];
        for (int i = 0; i < dss.length; i++) {
            fss[i] = (float) dss[i];
        }

        return fss;
    }

    public static float[] toFloats(final int[] iss) {
        float[] fss = new float[iss.length];
        for (int i = 0; i < iss.length; i++) {
            fss[i] = (float) iss[i];
        }

        return fss;
    }

    public static boolean isSameSign(final TFloatArrayList fls) {

        if (fls.size() == 0) {
            return true;
        }

        float start = fls.get(0);

        for (int i = 0; i < fls.size(); i++) {
            if (isSameSign(start, fls.get(i)) == false) {
                return false;
            }
        }

        return true;
    }

    public static boolean isSameSign(final float[] fls) {

        if (fls.length == 0) {
            return true;
        }

        float start = fls[0];

        for (int i = 0; i < fls.length; i++) {
            if (isSameSign(start, fls[i]) == false) {
                return false;
            }
        }

        return true;
    }

    public static boolean isSameSign(final float a, final float b) {
        if (a < 0 && b < 0) {
            return true;
        } else if (a == 0 && b == 0) {
            return true;
        } else if (a > 0 && b > 0) {
            return true;
        }

        return false;
    }

    // make the top larger if odd
    public static int[] splitTwoSided(final int num) {
        int bot = num / 2;
        int top = num - bot;

        return new int[]{top, bot};
    }

    public static boolean isEven(final int x) {

        if ((float) (x / 2) == x / (float) 2.0) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean isPositive(final int x) {
        return x >= 0;
    }

    public static boolean isPositive(final String x) {
        return isPositive(Float.parseFloat(x));
    }

    public static boolean isPositive(final float x) {
        return x >= 0;
    }

    public static boolean isPositiveBoth(final float x, final float y) {
        return isPositive(x) && isPositive(y);
    }

    public static boolean isNegativeBoth(final float x, final float y) {
        return isNegative(x) && isNegative(y);
    }

    public static boolean isPositive(final double x) {
        return x >= 0;
    }

    public static boolean isNegative(final int x) {
        return x <= 0;
    }

    public static boolean isNegative(final String x) {
        return isNegative(Float.parseFloat(x));
    }

    public static boolean isNegative(final float x) {
        return x <= 0;
    }

    public static boolean isDivisibleBy(final int x, final int y) {

        return (float) (x / y) == x / (float) y;
    }

    public static boolean isNumeric(final Object[] obj) {

        for (int i = 0; i < obj.length; i++) {
            try {
                Float.parseFloat(obj[i].toString());
            } catch (NumberFormatException e) {
                return false;
            }
        }

        return true;
    }

    public static boolean isNumeric(final Object obj) {

        if (obj == null) {
            return false;
        }

        try {
            Float.parseFloat(obj.toString());
        } catch (NumberFormatException e) {
            return false;
        }

        return true;
    }

    // from 0 to maxIndex-1
    public static int[] randomlyPickIndicesWithoutReplacement(final int numRndNeeded,
            final int maxIndexExclusive,
            final RandomSeedGenerator rsgen) {

        int[] inds = XMath.randomlySampleWithoutReplacement(numRndNeeded, maxIndexExclusive, rsgen);

        // sanity check
        if (inds.length != numRndNeeded) {
            throw new IllegalStateException("Prog error: indices length " + inds.length + " but wanted to pick: " + numRndNeeded + " highIndexExclusive: " + maxIndexExclusive);
        }

        return inds;
    }

    public static int[] toIndices(final int maxIndex, final boolean inclusive) {
        if (maxIndex <= 0) {
            throw new IllegalArgumentException("Specified max for indices must be more than 0, got: " + maxIndex);
        }

        int till;
        if (inclusive) {
            till = maxIndex + 1;
        } else {
            till = maxIndex;
        }

        int[] inds = new int[till];
        for (int i = 0; i < till; i++) {
            inds[i] = i;
        }

        return inds;
    }

    // other is to EXcluse
    public static int[] toIndices(final int[] all,
            final int[] sub,
            final boolean includeSpecifiedIndices) {

        TIntArrayList allT = new TIntArrayList(all);
        TIntArrayList subT = new TIntArrayList(sub);

        TIntArrayList values2removeFromAll = new TIntArrayList();

        if (includeSpecifiedIndices) {
            for (int i = 0; i < all.length; i++) {
                if (subT.contains(all[i]) == false) { // remove if it doesnt have it
                    values2removeFromAll.add(all[i]); // value NOT index
                }
            }
        } else { // EXclude
            for (int i = 0; i < all.length; i++) {
                if (subT.contains(all[i]) == true) { // remove if it has it
                    values2removeFromAll.add(all[i]);
                }
            }
        }

        // remove. Need to work with values as index will change as we remove stuff
        for (int i = 0; i < values2removeFromAll.size(); i++) {
            int index2remove = allT.indexOf(values2removeFromAll.get(i));
            if (index2remove == -1) {
                throw new IllegalStateException("Strangely this values doesnt exist in all: " + values2removeFromAll.get(i));
            }
            allT.remove(index2remove); // remove

            if (allT.indexOf(values2removeFromAll.get(i)) != -1) {
                throw new IllegalArgumentException("There were duplicates in the all array: " + Printf.format(all));
            }
        }

        return allT.toNativeArray();
    }

    /**
     * are n1 and n2 within fuzz of each other
     */
    public static boolean equals(final int fuzz, final int n1, final int n2) {

        if ((Math.abs(n1 - n2)) <= fuzz) {
            return true;
        }

        return false;
    }

    /**
     * Converts radians to degrees.
     *
     * @param rad the input value (in radians)
     * @return <code>rad</code> converted to degrees
     * @see #deg2rad(double)
     * @since org.dyndns.aoi.Utilities 1.0
     */
    public static double rad2deg(final double rad) {
        return (rad * 360) / (2 * Math.PI);
    }

    /**
     * Converts degrees to radians.
     *
     * @param deg the input value (in degrees)
     * @return <code>deg</code> converted to radians
     * @see #rad2deg(double)
     * @since org.dyndns.aoi.Utilities 1.0
     */
    public static double deg2rad(final double deg) {
        return (deg * (2 * Math.PI)) / 360;
    }

    /**
     * uses the common xmath random seed
     *
     * @param num
     * @return
     */
    public static int[] randomizeWithoutReplacement(final int num, final RandomSeedGenerator rsgen) {
        return randomizeWithoutReplacement(num, rsgen.getRandom());
    }

    /**
     * IMP num elements are returned AND the random numbers fall b/w 0 and num-1
     *
     * @see sampleWithoutReplacement for a diff way
     *      Creates an array whose elements are randomly arranged between
     *      0 and num - 1
     *      Example: randomize(5) could yeild: 0, 3, 1, 2, 4
     *      <p/>
     *      The same random number generator is employed (in this instance of the jvm)
     *      so safe to call multiple times.
     *      But as it uses the same seed each time, the rnd stays the same from
     *      jvm inoc to jvm invoc. See link below for more.
     * @see http://mindprod.com/gotchas.html#RANDOM
     */
    public static int[] randomizeWithoutReplacement(final int num, final Random rnd) {

        List seen = new ArrayList(num);
        int[] inds = new int[num];
        int cnt = 0;

        for (int i = 0; i < num;) {
            int r = rnd.nextInt(num);

            if (seen.contains(new Integer(r))) {
                continue;
            }

            seen.add(new Integer(r));

            inds[cnt++] = r;

            if (cnt == num) {
                break;
            }
        }

        return inds;
    }

    public static int[] randomlySampleWithoutReplacement(final int numRndNeeded, final int highestrandomnumExclusive, final RandomSeedGenerator rsgen) {
        return randomlySampleWithoutReplacement(numRndNeeded, highestrandomnumExclusive, rsgen.getRandom());
    }

    /**
     * @param numRndNeeded       number of random picks nmeeded
     * @param maxRndNumExclusive range -> picked from 0 to highestrandomnum-1
     * @param rnd
     * @return
     */
    public static int[] randomlySampleWithoutReplacement(final int numRndNeeded, final int maxRndNumExclusive, final Random rnd) {

        if (maxRndNumExclusive == numRndNeeded) { // no random picking needed, we have exactly as many as asked for
            return XMath.toIndices(maxRndNumExclusive, false);
        }

        if (numRndNeeded > maxRndNumExclusive) {
            throw new IllegalArgumentException("Cannot pick more numbers (no replacement) numRndNeeded: " + numRndNeeded + " than max possible number maxRndNumExclusive: " + maxRndNumExclusive);
        }

        List seen = new ArrayList(numRndNeeded);
        int[] inds = new int[numRndNeeded];
        int cnt = 0;

        for (int i = 0; i < numRndNeeded;) {
            int r = rnd.nextInt(maxRndNumExclusive);

            if (seen.contains(new Integer(r))) {
                continue;
            }

            seen.add(new Integer(r));

            inds[cnt++] = r;

            if (cnt == numRndNeeded) {
                break;
            }
        }

        return inds;
    }

    public static int[] randomlySampleWithoutReplacement(final int numRndNeeded, final int[] availableIndices, final Random rnd) {
        int[] inds = randomlySampleWithoutReplacement(numRndNeeded, availableIndices.length, rnd);
        int[] ret = new int[inds.length];
        for (int i = 0; i < inds.length; i++) {
            ret[i] = availableIndices[inds[i]];
        }

        return ret;
    }

    public static int[] randomlySampleWithoutReplacement(final int numRndNeeded, final int[] availableIndices, final RandomSeedGenerator rsgen) {
        return randomlySampleWithoutReplacement(numRndNeeded, availableIndices, rsgen.getRandom());
    }

    public static int[] randomlySampleWithReplacement(final int numrnd, final int highestrandomnum, final RandomSeedGenerator rsgen) {
        return randomlySampleWithReplacement(numrnd, highestrandomnum, rsgen.getRandom());
    }

    public static int[] randomlySampleWithReplacement(final int numrnd, final int highestrandomnum, final Random rnd) {

        int[] inds = new int[numrnd];

        for (int i = 0; i < numrnd; i++) {
            inds[i] = rnd.nextInt(highestrandomnum);
        }

        return inds;
    }

    public static double getPValue(final int tot_num_perms, final int num_greater) {
        return ((double) num_greater) / (double) tot_num_perms;
    }

    public static double getPValue(final double score, final double[] values) {

        int cntmore = 0;

        for (int i = 0; i < values.length; i++) {
            if (values[i] > score) {
                cntmore++;
            }
        }

        //log.debug("Number more than score=" + score + " is=" + cntmore);

        return ((double) cntmore) / (double) values.length;
    }

    public static double getPValue(final float score, final TFloatArrayList values) {

        int cntmore = 0;

        for (int i = 0; i < values.size(); i++) {
            if (values.getQuick(i) > score) {
                cntmore++;
            }
        }

        //log.debug("Number more than score=" + score + " is=" + cntmore);

        return ((double) cntmore) / (double) values.size();
    }

    public static double getPValue(final float score, final float[] values) {

        int cntmore = 0;

        for (int i = 0; i < values.length; i++) {
            if (values[i] > score) {
                cntmore++;
            }
        }

        //log.debug("Number more than score=" + score + " is=" + cntmore);

        return ((double) cntmore) / (double) values.length;
    }

    public static double getPValue(final float score, final ArrayList values) {

        int cntmore = 0;

        for (int i = 0; i < values.size(); i++) {
            if (((Float) values.get(i)).floatValue() > score) {
                cntmore++;
            }
        }

        //log.debug("Number more than score=" + score + " is=" + cntmore);

        return ((double) cntmore) / (double) values.size();
    }

    public static double getPValue(final double score, final ArrayList values) {

        int cntmore = 0;

        for (int i = 0; i < values.size(); i++) {
            if (((Double) values.get(i)).floatValue() > score) {
                cntmore++;
            }
        }

        //log.debug("Number more than score=" + score + " is=" + cntmore);

        return ((double) cntmore) / (double) values.size();
    }

    public static double getPValueLessThan(final float score, final float[] values) {

        int cntless = 0;

        for (int i = 0; i < values.length; i++) {
            if (values[i] < score) {
                cntless++;
            }
        }

        return ((double) cntless) / (double) values.length;
    }

    public static double getPValue(final int score, final int[] values) {

        int cntmore = 0;

        for (int i = 0; i < values.length; i++) {
            if (values[i] > score) {
                cntmore++;
            }
        }

        //log.debug("Number more than score=" + score + " is=" + cntmore);

        return ((double) cntmore) / (double) values.length;
    }

    public static double getPValue(final float score, final Vector values, final boolean pos) {
        if (pos) {
            return getPValue(score, values);
        } else {
            return XMath.getPValueLessThan(score, values);
        }
    }

    public static double getPValue(final float score, final Vector values) {
        return getPValue(score, values.elementData);
    }

    public static double getPValueLessThan(final float score, final Vector values) {
        return getPValueLessThan(score, values.elementData);
    }

    public static double getPValueLessThan(final float score, final Matrix values) {
        return getPValueLessThan(score, values.elementData);
    }

    public static double getPValue(final float score, final Matrix matrix) {
        return getPValue(score, matrix.elementData);
    }

    public static double getPValue(final double score, final Vector values) {
        return getPValue((float) score, values);
    }

    /**
     * If the score is negative look to the left (i.e count how many less)
     * If the score is positive look to the right (i.e count how many are less)
     *
     * @param score
     * @param values
     * @return
     */
    public static double getPValueTwoTailed(final float score, final float[] values) {

        int cnt = 0;

        if (score >= 0) {
            for (int i = 0; i < values.length; i++) {
                if (values[i] > score) {
                    cnt++;
                }
            }
        } else {
            for (int i = 0; i < values.length; i++) {
                if (values[i] < score) {
                    cnt++;
                }
            }
        }

        //log.debug("Number less/more than score=" + score + " is=" + cnt);

        return ((double) cnt) / (double) values.length;
    }

    public static double getPValueTwoTailed(final int score, final int[] values) {

        int cnt = 0;

        if (score >= 0) {
            for (int i = 0; i < values.length; i++) {
                if (values[i] > score) {
                    cnt++;
                }
            }
        } else {
            for (int i = 0; i < values.length; i++) {
                if (values[i] < score) {
                    cnt++;
                }
            }
        }

        //log.debug("Number less/more than score=" + score + " is=" + cnt);

        return ((double) cnt) / (double) values.length;
    }

    /*
    // IMP IMP FIX ME
    public static double getPValueTwoTailedTypeA(float score, Vector values) {
    return getPValueTwoTailedTypeA(score, values.elementData);
    }

    public static double getPValueTwoTailedTypeA(float score, Matrix values) {
    return getPValueTwoTailedTypeA(score, values.elementData);
    }

    // cnt only neg or pos ones
    public static double getPValueTwoTailedTypeA(float score, float[] values) {

    int cnt = 0;

    if (score > 0) {
    for (int i = 0; i < values.length; i++) {
    if (values[i] > 0 && values[i] > score) {
    cnt++;
    }
    }
    } else if (score < 0) {
    for (int i = 0; i < values.length; i++) {
    if (values[i] < 0 && values[i] < score) {
    cnt++;
    }
    }
    } else if (score == 0) {
    return Double.NaN;
    }

    //log.debug("Number less/more than score=" + score + " is=" + cnt);

    return ((double) cnt) / (double) values.length;
    }
     */
    public float maxDevFrom0(final float[] values) {
        Vector v = new Vector(values, true);
        return v.maxDevFrom0();
    }

    public static double getPValueTwoTailed(final float score, final Vector values) {
        return getPValueTwoTailed(score, values.elementData);
    }

    public static double getPValueTwoTailed(final float score, final Matrix values) {
        return getPValueTwoTailed(score, values.elementData);
    }

    public static float getPValueTwoTailed_pos_neg_seperate(float realEs, Vector rndEs) {
        Vector ex = rndEs.extract(realEs, ScoreMode.POS_AND_NEG_SEPERATELY); // @note isnt this redundant
        if (ex.getSize() == 0) {
            return 0;
        } else {
            return (float) XMath.getPValueTwoTailed(realEs, ex);
        }
    }

    public static int getPValueCnt(final float score, final float[] values) {
        int cntmore = 0;

        for (int i = 0; i < values.length; i++) {
            if (values[i] > score) {
                cntmore++;
            }
        }

        //log.debug("Number more than score=" + score + " is=" + cntmore);

        return cntmore;
    }

    public static int getPValueCnt(final float score, final Vector values) {
        return getPValueCnt(score, values.elementData);
    }

    public static int getPValueCnt(final double score, final Vector values) {
        return getPValueCnt((float) score, values);
    }

    public static double roundOff(final double number, final int decimal_places) {

        BigDecimal b = new BigDecimal(number);

        return b.setScale(decimal_places, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    public static int min(final Object[] a, final Object[] b) {

        if (a.length < b.length) {
            return a.length;
        }

        return b.length;
    }

    public static int max(final Object[] a, final Object[] b) {

        if (a.length > b.length) {
            return a.length;
        }

        return b.length;
    }

    public static int max(final int[] values) {
        if (values.length == 0) {
            klog.warn("FIX ME Zero length array");
            //return -999;
            throw new IllegalArgumentException("Zero length array not allowed");
        }

        int max = values[0];
        for (int i = 0; i < values.length; i++) {
            if (max < values[i]) {
                max = values[i];
            }
        }

        return max;
    }

    public static int max_allow_zero_length(final int[] values) {
        if (values.length == 0) {
            return 0;
        }

        int max = values[0];
        for (int i = 0; i < values.length; i++) {
            if (max < values[i]) {
                max = values[i];
            }
        }

        return max;
    }

    public static float max(final float[] values) {
        if (values.length == 0) {
            klog.warn("FIX ME Zero length array");

            //return -999f;

            throw new IllegalArgumentException("Zero length array not allowed");
        }

        float max = values[0];
        for (int i = 0; i < values.length; i++) {
            if (max < values[i]) {
                max = values[i];
            }
        }

        return max;
    }

    public static float max(final TFloatArrayList values) {
        if (values.size() == 0) {
            throw new IllegalArgumentException("Zero length array not allowed");
        }

        float max = values.get(0);
        for (int i = 0; i < values.size(); i++) {
            float s = values.get(i);
            if (max < s) {
                max = s;
            }
        }

        return max;
    }

    public static float max(final TIntArrayList values) {
        return max(values.toNativeArray());
    }

    public static float min(final float[] values) {
        if (values.length == 0) {
            throw new IllegalArgumentException("Zero length array not allowed");
        }

        float min = values[0];
        for (int i = 0; i < values.length; i++) {
            if (min > values[i]) {
                min = values[i];
            }
        }

        return min;
    }

    public static float min(final TFloatArrayList values) {
        if (values.size() == 0) {
            throw new IllegalArgumentException("Zero length array not allowed");
        }

        float min = values.get(0);
        for (int i = 0; i < values.size(); i++) {
            float s = values.get(i);
            if (min > s) {
                min = s;
            }
        }

        return min;
    }

    public static float maxAbs(final float[] values) {
        return values[maxAbsIndex(values)];
    }

    public static int maxAbsIndex(final float[] values) {
        if (values.length == 0) {
            throw new IllegalArgumentException("Zero length array not allowed");
        }

        float max = values[0];
        int maxIndex = 0;
        for (int i = 0; i < values.length; i++) {
            if (Math.abs(max) < Math.abs(values[i])) {
                max = values[i];
                maxIndex = i;
            }
        }

        return maxIndex;
    }

    public static float max(final float a, final float b) {
        return max(new float[]{a, b});
    }

    public static int max(final int a, final int b) {
        return max(new int[]{a, b});
    }

    public static float maxAbs(final float a, final float b) {
        return maxAbs(new float[]{a, b});
    }

    public static float min(final float a, final float b) {
        return min(new float[]{a, b});
    }

    public static int min(final int a, final int b) {
        return min(new int[]{a, b});
    }

    public static int max(final Vector[] values) {
        if (values.length == 0) {
            throw new IllegalArgumentException("Zero length array not allowed");
        }

        int max = values[0].getSize();
        for (int i = 0; i < values.length; i++) {
            if (max < values[i].getSize()) {
                max = values[i].getSize();
            }
        }

        return max;
    }

    public static int min(final int[] values) {

        if (values.length == 0) {
            throw new IllegalArgumentException("Zero length array not allowed");
        }

        int min = values[0];
        for (int i = 0; i < values.length; i++) {
            if (min > values[i]) {
                min = values[i];
            }
        }

        return min;
    }

    public static int[] minmax(final int[] values) {

        if (values.length == 0) {
            throw new IllegalArgumentException("Zero length array not allowed");
        }

        int min = values[0];
        int max = values[0];

        for (int i = 0; i < values.length; i++) {
            if (min > values[i]) {
                min = values[i];
            }

            if (max < values[i]) {
                max = values[i];
            }
        }

        return new int[]{min, max};
    }

    private static void enforceEqualSize(final Vector x, final Vector y) {

        if (x.getSize() != y.getSize()) {
            throw new IllegalArgumentException("Vector lengths not equal x=" + x.getSize() + " and y=" + y.getSize());
        }
    }

    private static void enforceEqualSize(final Vector[] vss) {
        if (vss.length == 0) {
            return;
        }

        int size = vss[0].getSize();

        for (int i = 0; i < vss.length; i++) {
            if (vss[i].getSize() != size) {
                throw new IllegalArgumentException("Vector lengths not equal first=" + size + " and y=" + vss[i].getSize() + " at index " + i);
            }
        }
    }

    private static void enforceEqualSize(final float[] x, final float[] y) {

        if (x.length != y.length) {
            throw new IllegalArgumentException("Vector lengths not equal x=" + x.length + " and y=" + y.length);
        }
    }

    private static void enforceEqualSize(final int[] x, final int[] y) {

        if (x.length != y.length) {
            throw new IllegalArgumentException("int[] Arrays lengths not equal x=" + x.length + " and y=" + y.length);
        }
    }

    /**
     * dist <- sqrt(sum((x-y)*(x-y)))
     * No parameters.
     */
    public static double euclidean(final Vector x, final Vector y) {

        enforceEqualSize(x, y);

        float sum = 0;

        for (int i = 0; i < x.getSize(); i++) {
            float diff = x.getElement(i) - y.getElement(i);

            sum += diff * diff;
        }

        return Math.sqrt(sum);
    }

    public static double euclidean_exclude_nan(final Vector x, final Vector y) {

        enforceEqualSize(x, y);

        float sum = 0;

        for (int i = 0; i < x.getSize(); i++) {
            if (Float.isNaN(x.getElement(i)) || Float.isNaN(y.getElement(i))) {
            } else {

                float diff = x.getElement(i) - y.getElement(i);

                sum += diff * diff;
            }
        }

        return Math.sqrt(sum);
    }

    /**
     * dist <- sum(abs(x-y))
     * No parameters.
     */
    public static double manhatten(final Vector x, final Vector y) {

        enforceEqualSize(x, y);

        double sum = 0.0;

        for (int i = 0; i < x.getSize(); i++) {
            sum += Math.abs(x.getElement(i) - y.getElement(i));
        }

        return sum;
    }

    public static double meansdiff(final Vector x, final Vector y) {
        return x.mean() - y.mean();
    }

    public static double meansratio(final Vector x, final Vector y) {
        return x.mean() / y.mean();
    }

    public static double mediansratio(final Vector x, final Vector y) {
        return x.median() / y.median();
    }

    public static double meanOrMedianRatio(final Vector x, final Vector y, final boolean useMean) {
        return x.meanOrMedian(useMean) / y.meanOrMedian(useMean);
    }

    public static double meanOrMedianDiff(final Vector x, final Vector y, final boolean useMean) {
        return x.meanOrMedian(useMean) - y.meanOrMedian(useMean);
    }

    public static double mediansdiff(final Vector x, final Vector y) {
        return x.median() - y.median();
    }

    public static Vector meanVector(final Vector[] vss) {
        enforceEqualSize(vss);
        int size = vss[0].getSize();
        float[] means = new float[size];

        for (int i = 0; i < size; i++) {
            Vector v = new Vector(vss.length);
            for (int c = 0; c < vss.length; c++) {
                v.setElement(c, vss[c].getElement(i));
            }

            means[i] = (float) v.mean();
        }

        return new Vector(means, true);
    }

    public static Vector medianVector(final Vector[] vss) {
        enforceEqualSize(vss);
        int size = vss[0].getSize();
        float[] medians = new float[size];

        for (int i = 0; i < size; i++) {
            Vector v = new Vector(vss.length);
            for (int c = 0; c < vss.length; c++) {
                v.setElement(c, vss[c].getElement(i));
            }

            medians[i] = (float) v.median();
        }

        return new Vector(medians);
    }

    public static Vector maxVector(final Vector[] vss) {
        enforceEqualSize(vss);
        int size = vss[0].getSize();
        float[] maxs = new float[size];

        for (int i = 0; i < size; i++) {
            Vector v = new Vector(vss.length);
            for (int c = 0; c < vss.length; c++) {
                v.setElement(c, vss[c].getElement(i));
            }

            maxs[i] = v.max();
        }

        return new Vector(maxs);
    }

    public static Vector maxVector_TTTT(final Vector[] vss, String name) {
        enforceEqualSize(vss);
        int size = vss[0].getSize();
        float[] maxs = new float[size];

        for (int i = 0; i < size; i++) {
            Vector v = new Vector(vss.length);
            for (int c = 0; c < vss.length; c++) {
                v.setElement(c, vss[c].getElement(i));
            }

            maxs[i] = v.max_TTTT(name);
        }

        return new Vector(maxs);
    }

    public static float mean_of_means(final Vector[] vss) {
        double[] means = new double[vss.length];

        for (int i = 0; i < vss.length; i++) {
            means[i] = vss[i].mean();
        }

        return mean(means);
    }

    public static float median_of_medians(final Vector[] vss) {
        float[] medians = new float[vss.length];

        for (int i = 0; i < vss.length; i++) {
            medians[i] = (float) vss[i].median();
        }

        return median(medians);
    }

    public static float mean(final float[] x) {

        if (x == null) {
            throw new IllegalArgumentException("Parameter x[] cannot be null");
        }

        if (x.length == 0) {
            throw new IllegalArgumentException("Parameter x[] cannot be of 0 length");
        }

        float sum = 0;

        for (int i = 0; i < x.length; i++) {
            sum += x[i];
        }

        return sum / (float) x.length;
    }

    public static float meanmed(final TFloatArrayList values, final boolean useMean) {
        if (useMean) {
            return mean(values.toNativeArray());
        } else {
            return median(values.toNativeArray());
        }
    }

    public static float mean_nan_aware(final float[] x, final float replaceNanWith) {

        if (x == null) {
            throw new IllegalArgumentException("Parameter x[] cannot be null");
        }

        if (x.length == 0) {
            throw new IllegalArgumentException("Parameter x[] cannot be of 0 length");
        }

        float sum = 0;

        for (int i = 0; i < x.length; i++) {
            if (Float.isNaN(x[i])) {
                sum += replaceNanWith;
            } else {
                sum += x[i];
            }
        }

        return sum / (float) x.length;
    }

    public static float mean(final double[] x) {

        if (x == null) {
            throw new IllegalArgumentException("Parameter x[] cannot be null");
        }

        if (x.length == 0) {
            throw new IllegalArgumentException("Parameter x[] cannot be of 0 length");
        }

        float sum = 0;

        for (int i = 0; i < x.length; i++) {
            sum += x[i];
        }

        return sum / (float) x.length;
    }

    public static float median(final float[] x) {

        if (x.length == 0) {
            return Float.NaN;
        }

        // mangelos
        int aLen = x.length;
        float[] v1 = new float[aLen];

        System.arraycopy(x, 0, v1, 0, aLen);
        Arrays.sort(v1);

        int ind = (aLen - 1) / 2;

        if (XMath.isEven(aLen)) {
            return (v1[ind] + v1[aLen / 2]) / 2;
        } else {
            return v1[ind];
        }
    }

    public static float median(final int[] x) {

        if (x.length == 0) {
            return Float.NaN;
        }

        // mangelos
        int aLen = x.length;
        int[] v1 = new int[aLen];

        System.arraycopy(x, 0, v1, 0, aLen);
        Arrays.sort(v1);

        int ind = (aLen - 1) / 2;

        if (XMath.isEven(aLen)) {
            return (v1[ind] + v1[aLen / 2]) / 2;
        } else {
            return v1[ind];
        }
    }

    public static double pearson(final float[] x, final float[] y) {
        return pearson(new Vector(x, true), new Vector(y, true));
    }

    public static double pearson(final int[] x, final int[] y) {
        return pearson(new Vector(x), new Vector(y));
    }

    /**
     * No parameters
     *
     * @see http://www.ruf.rice.edu/~lane/hyperstat/A51911.html
     *      <p/>
     *      will ret Nan if only 1 element in each vector
     */
    public static double pearson(final Vector x, final Vector y) {

        enforceEqualSize(x, y);

        double N = (double) x.getSize();

        if (N == 0 || N == 1) {
            return Float.NaN;
        }

        //double numr = (sumprod(x, y) - x.sum()*y.sum()) / N;
        double numr = x.sumprod(y) - ((x.sum() * y.sum()) / N);
        double denr = ((x.squaresum() - ((x.sum() * x.sum()) / N))) * ((y.squaresum() - ((y.sum() * y.sum()) / N)));

        denr = Math.sqrt(denr);

        //double denr = ((x.squaresum() - (x.sum()*x.sum())) / N) * ((y.squaresum() - (y.sum()*y.sum())) / N);
        //denr = Math.sqrt(denr);

        /*
        log.debug("numr=" + numr + " denr=" + denr);
        log.debug("x>>" + x.squaresum());
        log.debug("x>>" + (x.sum()*x.sum()));
        log.debug("x>>" + ((x.squaresum() - ((x.sum()*x.sum()) / N) )));
        log.debug("y>>" + y.squaresum());
        log.debug("y>>" + (y.sum()*y.sum()));
        log.debug("y>>" + ((y.squaresum() - ((y.sum()*y.sum()) / N) )));
         */

        return numr / denr;
    }

    public static double pearson_exclude_nan(final Vector origX, final Vector origY) {
        Vector[] vss = removeNaNsSynched(origX, origY);
        return pearson(vss[0], vss[1]);
    }

    public static double hamming_exclude_0_only_if_both(final Vector origX, final Vector origY) {


        float total_score = 0;

        float numCnt = 0;
        for (int i = 0; i < origX.getSize(); i++) {
            float val_x = origX.getElement(i);
            for (int j = 0; j < origY.getSize(); j++) {
                float val_y = origY.getElement(j);
                if (val_x == 1 && val_y == 0) {
                    total_score += 0.5;
                    numCnt++;
                } else if (val_x == -1 && val_y == 0) {
                    total_score += -0.5;
                    numCnt++;
                } else if (val_x == 1 && val_y == 1) {
                    total_score += 1;
                    numCnt++;
                } else if (val_x == 1 && val_y == -1) {
                    total_score += -1;
                    numCnt++;
                }
            }
        }

        return total_score / numCnt;

        /*
        Vector[] vss = remove0sSynched_only_if_both_are_0(origX, origY);
        return pearson(vss[0], vss[1]);
         */
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    public static SimpleRegression regression(final Vector x, final Vector y) {
        enforceEqualSize(x, y);
        SimpleRegression r = new SimpleRegression();
        for (int i = 0; i < x.getSize(); i++) {
            r.addData(x.getElement(i), y.getElement(i));
        }

        return r;
    }

    public static double regressionSlope(final Vector x, final Vector y) {
        enforceEqualSize(x, y);
        SimpleRegression r = new SimpleRegression();
        for (int i = 0; i < x.getSize(); i++) {
            r.addData(x.getElement(i), y.getElement(i));
        }

        return r.getSlope();
    }

    /**
     * see kens doc for details
     *
     * @param x
     * @param yTemplate
     * @param z
     * @param usebiased
     * @param usemedian
     * @param fixlow
     * @return
     */
    public static double regressionSlope(final Vector x,
            final Vector yTemplate,
            final Vector[] splits,
            final boolean biased,
            final boolean fixlow) {

        enforceEqualSize(x, yTemplate);

        double xsum = x.sum(); // cache to avoid recalc

        double N = (double) x.getSize();
        double numrA = N * x.sumprod(yTemplate);
        double numrB = xsum * yTemplate.sum();

        double denrA = N * x.squaresum();
        double denrB = xsum * xsum;

        double C = (numrA - numrB) / (denrA - denrB);

        double var = 0;
        for (int i = 0; i < splits.length; i++) {
            var += splits[i].stddev(biased, fixlow);
        }

        //System.out.println("C\t" + C + "\tvarsum\t" + var);

        if (var == 0) {
            return Float.NaN;
        } else {
            return C / var;
        }

    }

    // -------------------------------------------------------------------------------------------- //
    public static Vector[] removeNaNsSynched(final Vector origX, final Vector origY) {

        enforceEqualSize(origX, origY);
        TFloatArrayList xvalues = new TFloatArrayList();
        TFloatArrayList yvalues = new TFloatArrayList();

        for (int i = 0; i < origX.getSize(); i++) {
            float xv = origX.getElement(i);
            float yv = origY.getElement(i);
            if (Float.isNaN(xv) || Float.isNaN(yv)) {
            } else {
                xvalues.add(xv);
                yvalues.add(yv);
            }
        }

        return new Vector[]{new Vector(xvalues), new Vector(yvalues)};
    }

    public static Vector[] remove0sSynched_only_if_both_are_0(final Vector origX, final Vector origY) {

        enforceEqualSize(origX, origY);
        TFloatArrayList xvalues = new TFloatArrayList();
        TFloatArrayList yvalues = new TFloatArrayList();

        for (int i = 0; i < origX.getSize(); i++) {
            float xv = origX.getElement(i);
            float yv = origY.getElement(i);
            if (xv == 0 && yv == 0) {
            } else {
                xvalues.add(xv);
                yvalues.add(yv);
            }
        }

        return new Vector[]{new Vector(xvalues), new Vector(yvalues)};
    }

    /**
     * Count the number of features between the 2 (synched vectors) that have greater than fold or less than 1/fold ratio
     *
     * @param x
     * @param y
     * @param fold
     * @return
     */
    public static double kenvar(final Vector x, final Vector y, float fold) {

        enforceEqualSize(x, y);

        int cnt = 0;

        for (int i = 0; i < x.getSize(); i++) {

            float f = x.getElement(i) / y.getElement(i);
            if (f < 1.0f) {
                f = 1.0f / f;
            }

            if (f > fold) {
                cnt++;
            }
        }

        return cnt;
    }

    public static double kenvar(final float[] x, final float[] y, float fold) {

        enforceEqualSize(x, y);

        int cnt = 0;

        for (int i = 0; i < x.length; i++) {

            float f = x[i] / y[i];
            if (f < 1.0f) {
                f = 1.0f / f;
            }

            if (f > fold) {
                cnt++;
            }
        }

        return cnt;
    }

    /**
     * p 885
     * david sheskin textbook
     *
     * @param correspYRanks must be sorted (NOT 1 to n but according to YX -- see the book)
     * @return
     */
    public static double kendallstau(final int[] correspYRanks) {

        int totnc = 0;
        int totnd = 0;

        for (int i = 0; i < correspYRanks.length; i++) {

            int nc = 0;
            int nd = 0;

            if ((correspYRanks[i] <= 0) || (correspYRanks[i] > correspYRanks.length + 1)) { // sanity check
                throw new IllegalArgumentException("Ranks must be from 1 to n found: " + correspYRanks[i]);
            }

            for (int p = i + 1; p < correspYRanks.length; p++) {

                if (correspYRanks[i] == correspYRanks[p]) { // sanity check
                    throw new IllegalArgumentException("Ditto ranks not allowed rank at: " + i + " is: " + correspYRanks[i] + " same as rank at: " + p + " rank: " + correspYRanks[p]);
                }

                if (correspYRanks[i] > correspYRanks[p]) {
                    nc++;
                } else {
                    nd++;
                }

                totnc += nc;
                totnd += nd;
            }
        }

        int n = correspYRanks.length;
        int denom = n * (n - 1) / 2;
        klog.debug("denom: " + denom);
        return (totnc - totnd) / denom;
    }

    /**
     * http://mathworld.wolfram.com/SpearmanRankCorrelationCoefficient.html
     * r = 1 - 6*sum[d**2]/(n(n2-1)
     * <p/>
     * <p/>
     * IMP: ranks must be from 1 to n and NOT 0 to n-1
     *
     * @param xranks
     * @param yranks
     * @return
     */
    public static double spearman(final int[] xranks, final int[] yranks) {

        enforceEqualSize(xranks, yranks); // vectors have to be of same length (ie contain ranks for the same set of objects)

        double sum = 0;

        // checks
        int n = xranks.length;
        int expected_sum = n * (n + 1) / 2;

        int real_sum = 0;
        for (int i = 0; i < n; i++) {
            real_sum += xranks[i];
        }

        if (expected_sum == real_sum) {
            //log.info("correct sums: " + expected_sum);
        } else {
            throw new IllegalArgumentException("Bad sums expected: " + expected_sum + " but got: " + real_sum + " (expect ranks from 1 to n and not 0 to n-1)");
        }

        real_sum = 0;
        for (int i = 0; i < n; i++) {
            real_sum += yranks[i];
        }

        if (expected_sum == real_sum) {
            //log.info("correct sums: " + expected_sum);
        } else {
            throw new IllegalArgumentException("Bad sums expected: " + expected_sum + " but got: " + real_sum);
        }

        // ok, the real stuff
        for (int i = 0; i < xranks.length; i++) {
            double diff = xranks[i] - yranks[i];
            sum += diff * diff;
        }

        //log.debug("sum = " + sum);
        //log.debug("pairs = " + denr);
        //log.debug("length = " + x.length);

        double N = (double) xranks.length; // IMP to cast as double, else gets truncated
        double denr = N * ((N * N) - 1.0);

        sum = (6.0 * sum) / denr;

        return 1.0 - sum;
    }

    /**
     * dist <- sum(x*y) / (sqrt(sum(x^2) * sum(y^2)) );
     *
     * @todo check if this is a dist or simil?
     * i.e is a low number "good" or "bad"
     * Looks like its a "bad", and hence need to do 1-
     * <p/>
     * No parameters
     */
    public static double cosine(final Vector x, final Vector y) {
        return cosine(x.elementData, y.elementData);
    }

    public static double cosine(final float[] x, final float[] y) {

        enforceEqualSize(x, y);

        double mag_x = 0.0;
        double mag_y = 0.0;
        double sump = 0.0;

        for (int i = 0; i < x.length; i++) {
            mag_x += x[i] * x[i];
            mag_y += y[i] * y[i];
            sump += x[i] * y[i];
        }

        return 1.0d - (sump / Math.sqrt(mag_x * mag_y));
        //return (sump / Math.sqrt(mag_x * mag_y));
    }

    /**
     * s2n = mean1-mean2/root(var1+var2)
     * no root!!!
     */
    public static double s2n(final Vector x,
            final Vector y,
            final boolean usebiased,
            final boolean usemedian,
            final boolean fixlow) {

        double s2n;

        /*
        if (usemedian) {
        //s2n = (x.median() - y.median()) / Math.sqrt(x.var(usebiased) + y.var(usebiased));
        s2n = (x.median() - y.median()) / (x.stddev(usebiased) + y.stddev(usebiased));

        }
        else {
        s2n = (x.mean() - y.mean()) / (x.stddev(usebiased) + y.stddev(usebiased));
        }
        return s2n;

         */

        //s2n = (x.median() - y.median()) / (x.stddev(false) + y.stddev(false));
        if (usemedian) {
            s2n = (x.median() - y.median()) / (x.stddev(usebiased, fixlow) + y.stddev(usebiased, fixlow));
        } else {
            s2n = (x.mean() - y.mean()) / (x.stddev(usebiased, fixlow) + y.stddev(usebiased, fixlow));
        }

        return s2n;
    }

    /**
     * @see http://trochim.human.cornell.edu/kb/stat_t.htm
     */
    public static double tTest(final Vector x,
            final Vector y,
            final boolean usebiased,
            final boolean usemedian,
            final boolean fixlow) {

        double numr;

        if (usemedian) {
            numr = x.median() - y.median();
        } else {
            numr = x.mean() - y.mean();
        }

        double denr;

        if (usebiased) {
            denr = Math.sqrt((x.var(usebiased, fixlow) / (x.getSize() - 1)) + (y.var(usebiased, fixlow) / (y.getSize() - 1)));
        } else {
            denr = Math.sqrt((x.var(usebiased, fixlow) / (x.getSize())) + (y.var(usebiased, fixlow) / (y.getSize())));
        }

        return (numr / denr);
    }

    /**
     * @todo confirm formula with pt
     * @see http://trochim.human.cornell.edu/kb/stat_t.htm
     */
    public static double bhat(final Vector v1, final Vector v2, final boolean usebiased,
            final boolean fixlow) {

        double firstterm = (v2.mean() - v1.mean()) * (0.5 / (v1.var(usebiased, fixlow) + v2.var(usebiased, fixlow))) * (v2.mean() - v1.mean());
        double secterm = 0.5 * Math.log(((v1.var(usebiased, fixlow) + v2.var(usebiased, fixlow)) / 2) / Math.sqrt(v1.var(usebiased, fixlow) * v2.var(usebiased, fixlow)));

        return (firstterm + secterm);
    }

    /**
     * Pearson will only work if the the 2 classes are of equal size. Because of the Exy term in the formula.
     */
    public static double pearsonD(final Vector minusplusvec, final Vector y) {

        // lengths still have to be equal
        enforceEqualSize(minusplusvec, y);

        // now x is the minusplus vector -- paranoid check
        for (int i = 0; i < minusplusvec.getSize(); i++) {
            if ((minusplusvec.getElement(i) != -1) || (minusplusvec.getElement(i) != +1)) {
                throw new IllegalArgumentException("MinusPlus vector has invalid element: " + minusplusvec.getElement(i) + " only -1 and +1 allowed");
            }
        }

        // normalize the vector
        Vector v = new Vector(y);

        v.pnormalize();

        // @note the 1-x: this is to abide by the Metric contract
        return 1 - pearson(minusplusvec, v);
    }

    public static double mannWhitney(final int[] hitIndices, final int totSize) {

        // term 1
        double sr = 0; // sum of ranks of genes in the gene set
        for (int i = 0; i < hitIndices.length; i++) {
            sr += (double) hitIndices[i];
        }

        double N = (double) hitIndices.length; // size of gene set
        double term1 = (sr / N);

        // term 2
        double st = (totSize * (totSize - 1)) / 2; // sum of ranks of ALL genes

        double d = (double) totSize; // number of elements in the dataset

        double term2 = (st - sr) / (d - N);

        double mw = term1 - term2;

        //System.out.println("sr: " + sr + " st: " + st + " term1: " + term1 + " term2: " + term2 + " mw: " + mw);

        // now this is when a low rank sum is GOOD, and hence the mw score a negative number
        // but to put it on the same terms as our ES score, lets negate it

        return -1.0d * mw;
    }

    /**
     * max diff between the 2 cdf's
     *
     * @param a
     * @param datab
     * @return
     */
    public static double maxAbsDiff(final double[] a, final double[] b) {

        if (a.length != b.length) {
            throw new IllegalArgumentException("cdf's must be of equal length -- a: " + a.length + " b: " + b.length);
        }

        double max = Double.NEGATIVE_INFINITY;

        for (int i = 0; i < a.length; i++) {
            double diff = Math.abs(a[i] - b[i]);
            if (max < diff) {
                max = diff;
            }
        }

        return max;

    }

    public static float getSignificanceLevel(final SortMode sort, final Order order, final Vector v, final float level) {

        if (v.getSize() == 0) {
            return Float.NaN;
        }

        // first put the scores in whatever order the markers were in
        v.sort(sort, order); // can do directly as the v is a clone
        int index = (int) (level * v.getSize());
        //log.debug("significance level: " + level + " index: " + index);

        return v.getElement(index);
    }

    public static float getSignificanceLevelASIS(final Vector v, final float level) {

        if (v.getSize() == 0) {
            return Float.NaN;
        }

        // dont -0> use vector as is put the scores in whatever order the markers were in
        int index = (int) (level * v.getSize());
        //log.debug("significance level: " + level + " index: " + index);

        return v.getElement(index);
    }

    /**
     * From:
     * http://www-nlp.stanford.edu/nlp/javadoc/javanlp/edu/stanford/nlp/math/SloppyMath.html
     * <p/>
     * Find a hypergeometric distribution.  This uses exact math, trying as
     * hard as one can to avoid numeric overflow by interleaving
     * multiplications and divisions.
     * Warning: the way this is written is only efficient if r,m < n/2.
     * For other situations, some more code should be added.
     *
     * @param k The number of black balls drawn
     * @param N The total number of balls
     * @param K The number of black balls
     * @param n The number of balls drawn
     * @return The hypergeometric value
     */
    public static double hypergeometric(int k, int N, int K, int n) {

        if (k < 0) {
            throw new IllegalArgumentException("Invalid hypergeometric k < 0: " + k);
        }

        if (N <= 0) {
            throw new IllegalArgumentException("Invalid hypergeometric N <= 0: " + N);
        }

        if (n < 0) {
            throw new IllegalArgumentException("Invalid hypergeometric n < 0: " + n);
        }

        if (K < 0) {
            throw new IllegalArgumentException("Invalid hypergeometric K < 0: " + K);
        }

        if (K > N) {
            throw new IllegalArgumentException("Invalid hypergeometric r > n " + K + " " + N);
        }

        if (n > N) {
            throw new IllegalArgumentException("Invalid hypergeometric m > n: " + n + " " + N);
        }

        // exploit symmetry of problem
        if (n > N / 2) {
            n = N - n;
            k = K - k;
        }
        if (K > N / 2) {
            K = N - K;
            k = n - k;
        }
        if (n > K) {
            int temp = n;
            n = K;
            K = temp;
        }
        // now we have that k <= m <= r <= n/2

        if (k < (n + K) - N || k > n) {
            return 0.0;
        }

        // Do limit cases explicitly
        // It's unclear whether this is a good idea.  I put it in fearing
        // numerical errors when the numbers seemed off, but actually there
        // was a bug in the Fisher's exact routine.
        if (K == N) {
            if (k == n) {
                return 1.0;
            } else {
                return 0.0;
            }
        } else if (K == N - 1) {
            if (k == n) {
                return (N - n) / (double) N;
            } else if (k == n - 1) {
                return n / (double) N;
            } else {
                return 0.0;
            }
        } else if (n == 1) {
            if (k == 0) {
                return (N - K) / (double) N;
            } else if (k == 1) {
                return K / (double) N;
            } else {
                return 0.0;
            }
        } else if (n == 0) {
            if (k == 0) {
                return 1.0;
            } else {
                return 0.0;
            }
        } else if (k == 0) {
            double ans = 1.0;
            for (int m0 = 0; m0 < n; m0++) {
                ans *= ((N - K) - m0);
                ans /= (N - m0);
            }
            return ans;
        }

        double ans = 1.0;
        // do (n-r)x...x((n-r)-((m-k)-1))/n x...x (n-((m-k-1)))
        // leaving rest of denominator to get to multimply by (n-(m-1))
        // that's k things which goes into next loop
        for (int nr = N - K, n0 = N; nr > (N - K) - (n - k); nr--, n0--) {
            // System.out.println("Multiplying by " + nr);
            ans *= nr;
            // System.out.println("Dividing by " + n0);
            ans /= n0;
        }
        // System.out.println("Done phase 1");
        for (int k0 = 0; k0 < k; k0++) {
            ans *= (n - k0);
            // System.out.println("Multiplying by " + (m-k0));
            ans /= ((N - (n - k0)) + 1);
            // System.out.println("Dividing by " + ((n-(m+k0)+1)));
            ans *= (K - k0);
            // System.out.println("Multiplying by " + (r-k0));
            ans /= (k0 + 1);
            // System.out.println("Dividing by " + (k0+1));
        }
        return ans;
    }

    /**
     * From:
     * http://www-nlp.stanford.edu/nlp/javadoc/javanlp/edu/stanford/nlp/math/SloppyMath.html
     * Find a one-tailed Fisher's exact probability.  Chance of having seen
     * this or a more extreme departure from what you would have expected
     * given independence.  I.e., k >= the value passed in.
     * Warning: this was done just for collocations, where you are
     * concerned with the case of k being larger than predicted.  It doesn't
     * correctly handle other cases, such as k being smaller than expected.
     *
     * @param k The number of black balls drawn
     * @param n The total number of balls
     * @param r The number of black balls
     * @param m The number of balls drawn
     * @return The Fisher's exact p-value
     */
    public static double oneTailedFishersExact(int k, int n, int r, int m) {
        if (k < 0 || k < (m + r) - n || k > r || k > m || r > n || m > n) {
            throw new IllegalArgumentException("Invalid Fisher's exact: " +
                    "k=" + k + " n=" + n + " r=" + r + " m=" + m + " k<0=" + (k < 0) + " k<(m+r)-n=" +
                    (k < (m + r) - n) + " k>r=" + (k > r) + " k>m=" + (k > m) + " r>n=" + (r > n) +
                    "m>n=" + (m > n));
        }
        // exploit symmetry of problem
        if (m > n / 2) {
            m = n - m;
            k = r - k;
        }
        if (r > n / 2) {
            r = n - r;
            k = m - k;
        }
        if (m > r) {
            int temp = m;
            m = r;
            r = temp;
        }
        // now we have that k <= m <= r <= n/2

        double total = 0.0;
        if (k > m / 2) {
            // sum from k to m
            for (int k0 = k; k0 <= m; k0++) {
                // System.out.println("Calling hypg(" + k0 + "; " + n +
                // 		   ", " + r + ", " + m + ")");
                total += hypergeometric(k0, n, r, m);
            }
        } else {
            // sum from max(0, (m+r)-n) to k-1, and then subtract from 1
            int min = Math.max(0, (m + r) - n);
            for (int k0 = min; k0 < k; k0++) {
                // System.out.println("Calling hypg(" + k0 + "; " + n +
                // 		   ", " + r + ", " + m + ")");
                total += hypergeometric(k0, n, r, m);
            }
            total = 1.0 - total;
        }
        return total;
    }

    /**
     * From: http://www-nlp.stanford.edu/nlp/javadoc/javanlp/edu/stanford/nlp/math/SloppyMath.html
     * Find a 2x2 chi-square value.
     * Note: could do this more neatly using simplified formula for 2x2 case.
     *
     * @param k The number of black balls drawn
     * @param n The total number of balls
     * @param r The number of black balls
     * @param m The number of balls drawn
     * @return The Fisher's exact p-value
     */
    public static double chiSquare2by2(final int k, final int n, final int r, final int m) {
        int[][] cg = {{k, r - k}, {m - k, n - (k + (r - k) + (m - k))}};
        int[] cgr = {r, n - r};
        int[] cgc = {m, n - m};
        double total = 0.0;
        for (int i = 0; i < 2; i++) {
            for (int j = 0; j < 2; j++) {
                double exp = (double) cgr[i] * cgc[j] / n;
                total += (cg[i][j] - exp) * (cg[i][j] - exp) / exp;
            }
        }

        return total;
    }

    //****************************
    // Get minimum of three values
    //****************************
    public static int Minimum(final int a, final int b, final int c) {
        int mi;

        mi = a;
        if (b < mi) {
            mi = b;
        }
        if (c < mi) {
            mi = c;
        }
        return mi;
    }

    // code from http://www.merriampark.com/ld.htm#JAVA
    public class Distance {
    }

    //*****************************
    // Compute Levenshtein distance
    //*****************************
    public static int levenshtein(final String s, final String t) {

        int d[][]; // matrix
        int n; // length of s
        int m; // length of t
        int i; // iterates through s
        int j; // iterates through t
        char s_i; // ith character of s
        char t_j; // jth character of t
        int cost; // cost

        // Step 1

        n = s.length();
        m = t.length();
        if (n == 0) {
            return m;
        }
        if (m == 0) {
            return n;
        }
        d = new int[n + 1][m + 1];

        // Step 2

        for (i = 0; i <= n; i++) {
            d[i][0] = i;
        }

        for (j = 0; j <= m; j++) {
            d[0][j] = j;
        }

        // Step 3

        for (i = 1; i <= n; i++) {

            s_i = s.charAt(i - 1);

            // Step 4

            for (j = 1; j <= m; j++) {

                t_j = t.charAt(j - 1);

                // Step 5

                if (s_i == t_j) {
                    cost = 0;
                } else {
                    cost = 1;
                }

                // Step 6

                d[i][j] = Minimum(d[i - 1][j] + 1, d[i][j - 1] + 1, d[i - 1][j - 1] + cost);

            }

        }

        // Step 7

        return d[n][m];

    }

    public static int getMoreThanCount(final float value, final Vector sorted_pos_to_neg) {

        int cnt = 0;
        for (int i = 0; i < sorted_pos_to_neg.getSize(); i++) {
            if (value > sorted_pos_to_neg.getElement(i)) {
                break;
            } else {
                cnt++;
            }
        }

        return cnt;
    }

    public static int getLessThanCount(final float value, final Vector sorted_low_to_high) {

        int cnt = 0;
        for (int i = 0; i < sorted_low_to_high.getSize(); i++) {
            if (value < sorted_low_to_high.getElement(i)) {
                break;
            } else {
                cnt++;
            }
        }

        return cnt;
    }

    public static StringMatrix median(final StringDataframe[] sdfs) {
        StringMatrix[] sms = new StringMatrix[sdfs.length];
        for (int i = 0; i < sdfs.length; i++) {
            sms[i] = sdfs[i].getMatrix();
        }

        return median(sms);
    }

    /**
     * @param dss
     * @return
     */
    public static Matrix median(final Dataset[] dss) {
        Matrix[] mss = new Matrix[dss.length];
        for (int i = 0; i < dss.length; i++) {
            mss[i] = dss[i].getMatrix();
        }

        return median(mss);
    }

    /**
     * @param sdfs
     * @return
     */
    public static StringMatrix stdev(final StringDataframe[] sdfs) {
        StringMatrix[] sms = new StringMatrix[sdfs.length];
        for (int i = 0; i < sdfs.length; i++) {
            sms[i] = sdfs[i].getMatrix();
        }

        return stdev(sms);
    }

    public static StringMatrix median(final StringMatrix[] sms) {

        if (sms.length == 0) {
            throw new IllegalArgumentException("Cannot median 0 sms");
        }

        StringMatrix median = new StringMatrix(sms[0].getNumRow(), sms[0].getNumCol());

        for (int c = 0; c < median.getNumCol(); c++) {
            for (int r = 0; r < median.getNumRow(); r++) {
                TFloatArrayList floats = new TFloatArrayList();
                for (int i = 0; i < sms.length; i++) {
                    float curr = Float.parseFloat(sms[i].getElement(r, c));
                    floats.add(curr);
                }
                Vector v = new Vector(floats);
                median.setElement(r, c, (float) v.median());
            }
        }

        return median;
    }

    public static Matrix median(final Matrix[] mss) {

        if (mss.length == 0) {
            throw new IllegalArgumentException("Cannot median 0 mss");
        }

        Matrix median = new Matrix(mss[0].getNumRow(), mss[0].getNumCol());

        for (int c = 0; c < median.getNumCol(); c++) {
            for (int r = 0; r < median.getNumRow(); r++) {
                TFloatArrayList floats = new TFloatArrayList();
                for (int i = 0; i < mss.length; i++) {
                    float curr = mss[i].getElement(r, c);
                    floats.add(curr);
                }
                Vector v = new Vector(floats);
                median.setElement(r, c, (float) v.median());
            }
        }

        return median;
    }

    public static StringMatrix stdev(final StringMatrix[] sms) {

        if (sms.length == 0) {
            throw new IllegalArgumentException("Cannot median 0 sms");
        }

        StringMatrix median = new StringMatrix(sms[0].getNumRow(), sms[0].getNumCol());

        for (int c = 0; c < median.getNumCol(); c++) {
            for (int r = 0; r < median.getNumRow(); r++) {
                TFloatArrayList floats = new TFloatArrayList();
                for (int i = 0; i < sms.length; i++) {
                    float curr = Float.parseFloat(sms[i].getElement(r, c));
                    floats.add(curr);
                }
                Vector v = new Vector(floats);
                median.setElement(r, c, (float) v.stddev(false, false));
            }
        }

        return median;
    }

    public static double chiSquare(final Vector x, final Vector y) {
        double value = Math.pow(y.getElement(0) - x.getElement(0), 2) / x.getElement(0) + Math.pow(y.getElement(1) - x.getElement(1), 2) / x.getElement(1);
        return value;
    }
}    // End XMath

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

