package edu.mit.broad.genome.models;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.Dataframe;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import org.apache.log4j.Logger;
import org.jfree.chart.labels.XYToolTipGenerator;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Nov 22, 2005
 *         Time: 11:32:23 AM
 */
public class MiscChartModels {

    private static final Logger klog = XLogger.getLogger(MiscChartModels.class);

    /**
     * Plot data
     */
    public static class GaddyPlotModel extends AbstractXYDatasetStruc {

        private Dataset fDataset;
        private Chip chip;

        // cache these for performance
        // Even then the plot is slow because it has to display soo many points
        private float[] fRowIndexMeanHashMap;
        private float[] fRowIndexStdDevHashMap;

        public GaddyPlotModel(final Dataset ds) {

            this.fDataset = ds;
            this.chip = ds.getAnnot().getChip();

            this.fRowIndexMeanHashMap = new float[ds.getNumRow()];
            this.fRowIndexStdDevHashMap = new float[ds.getNumRow()];

            double xmin = Double.MAX_VALUE;
            double xmax = Double.MIN_VALUE;
            double ymin = Double.MAX_VALUE;
            double ymax = Double.MIN_VALUE;

            for (int r = 0; r < ds.getNumRow(); r++) {
                Vector v = ds.getRow(r);
                float mean = (float) v.mean();
                float stddev = (float) v.stddev(true, false);
                if (xmin > mean) {
                    xmin = mean;
                }
                if (xmax < mean) {
                    xmax = mean;
                }
                if (ymin > stddev) {
                    ymin = stddev;
                }
                if (ymax < stddev) {
                    ymax = stddev;
                }

                fRowIndexMeanHashMap[r] = mean;
                fRowIndexStdDevHashMap[r] = stddev;
            }

        }

        public int getItemCount(int i) {
            return fDataset.getNumRow();
        }

        public String getSeriesName(int series) {
            return fDataset.getName();
        }

        public int getSeriesCount() {
            return 1; // just the dataset
        }

        public java.lang.Number getX(int series, int item) {
            return new Float(fRowIndexMeanHashMap[item]);
        }

        public double getXValue(int series, int item) {
            return fRowIndexMeanHashMap[item];
        }

        public java.lang.Number getY(int series, int item) {
            return new Float(fRowIndexStdDevHashMap[item]);
        }

        public double getYValue(int series, int item) {
            return fRowIndexStdDevHashMap[item];
        }

        public Dataframe getSeriesDF() {
            final List labels = new ArrayList(getItemCount(0));
            final Matrix m = new Matrix(getItemCount(0), 2);

            for (int i = 0; i < getItemCount(0); i++) {
                labels.add(fDataset.getRowName(i));
                m.setElement(i, 0, getX(0, i).floatValue());
                m.setElement(i, 1, getY(0, i).floatValue());
            }

            return new Dataframe(getSeriesName(0), m, labels, new String[]{"X", "Y"}, true, true);
        }

        public XYToolTipGenerator createXYToolTipGenerator() {
            return new XYToolTipGenerator() {
                public String generateToolTip(org.jfree.data.xy.XYDataset xyDataset, int i, int i1) {
                    String rowName = fDataset.getRowName(i1); // column has no meaning
                    return formatToolTip(rowName, chip);

                }
            };
        } // End class DatasetToolTipGenerator

    } // End class GaddyPlotModel2


    public static class GaddyPlotModelMulti2 extends AbstractXYDatasetStruc {

        private Struc[] fStrucs;

        private Chip chip;

        // cache these for performance
        // Even then the plot is slow because it has to display soo many points


        public GaddyPlotModelMulti2(final Dataset ds, final GeneSet[] gsets_ascending_size_order) {

            // The gene sets NEED to be in ascending order of size - else the biggest will overwhelm

            this.fStrucs = new Struc[gsets_ascending_size_order.length];
            for (int i = 0; i < gsets_ascending_size_order.length; i++) {
                //klog.debug(">> GaddyPlotModelMulti2: " + gsets[i].getName(true) + " " + gsets[i].getNumMembers());
                fStrucs[i] = new Struc(ds, gsets_ascending_size_order[i]);
            }

            this.chip = ds.getAnnot().getChip();
        }

        public int getItemCount(int series) {
            return fStrucs[series].fRowIndexStdDevHashMap.length;
        }

        public String getSeriesName(int series) {
            return fStrucs[series].getName();
        }

        public int getSeriesCount() {
            return fStrucs.length;
        }

        public java.lang.Number getX(int series, int item) {
            return new Float(fStrucs[series].fRowIndexMeanHashMap[item]);
        }

        public double getXValue(int series, int item) {
            return fStrucs[series].fRowIndexMeanHashMap[item];
        }

        public java.lang.Number getY(int series, int item) {
            return new Float(fStrucs[series].fRowIndexStdDevHashMap[item]);
        }

        public double getYValue(int series, int item) {
            return fStrucs[series].fRowIndexStdDevHashMap[item];
        }

        public Dataframe getSeriesDF(int series) {
            final List labels = new ArrayList(getItemCount(series));
            final Matrix m = new Matrix(getItemCount(series), 2);

            for (int i = 0; i < getItemCount(series); i++) {
                labels.add(fStrucs[series].gset.getMember(i));
                m.setElement(i, 0, getX(series, i).floatValue());
                m.setElement(i, 1, getY(series, i).floatValue());
            }

            return new Dataframe(getSeriesName(series), m, labels, new String[]{"X", "Y"}, true, true);
        }

        public XYToolTipGenerator createXYToolTipGenerator() {
            return new XYToolTipGenerator() {
                public String generateToolTip(org.jfree.data.xy.XYDataset xyDataset, int series, int item) {
                    String rowName = fStrucs[series].gset.getMember(item);
                    return formatToolTip(rowName, chip);

                }
            };
        } // End class DatasetToolTipGenerator

    } // End class GaddyPlotModel2


    public static class StateFoldChangeByGexPlotModelMulti2 extends AbstractXYDatasetStruc {

        private StateFoldChangeStruc[] fStrucs;

        private Chip chip;

        // cache these for performance
        // Even then the plot is slow because it has to display soo many points


        public StateFoldChangeByGexPlotModelMulti2(final Dataset log2Dataset_samcol_into_states, final GeneSet[] gsets_ascending_size_order, final boolean useMean) {

            // The gene sets NEED to be in ascending order of size - else the biggest will overwhelm

            this.fStrucs = new StateFoldChangeStruc[gsets_ascending_size_order.length];
            for (int i = 0; i < gsets_ascending_size_order.length; i++) {
                fStrucs[i] = new StateFoldChangeStruc(log2Dataset_samcol_into_states, gsets_ascending_size_order[i], useMean);
            }

            this.chip = log2Dataset_samcol_into_states.getAnnot().getChip();
        }

        public int getItemCount(int series) {
            return fStrucs[series].fRowIndexStateLogMaxByMinHashMap.length;
        }

        public String getSeriesName(int series) {
            return fStrucs[series].getName();
        }

        public int getSeriesCount() {
            return fStrucs.length;
        }

        public java.lang.Number getX(int series, int item) {
            return new Float(fStrucs[series].fRowIndexMeanHashMap[item]);
        }

        public double getXValue(int series, int item) {
            return fStrucs[series].fRowIndexMeanHashMap[item];
        }

        public java.lang.Number getY(int series, int item) {
            return new Float(fStrucs[series].fRowIndexStateLogMaxByMinHashMap[item]);
        }

        public double getYValue(int series, int item) {
            return fStrucs[series].fRowIndexStateLogMaxByMinHashMap[item];
        }

        public Dataframe getSeriesDF(int series) {
            final List labels = new ArrayList(getItemCount(series));
            final Matrix m = new Matrix(getItemCount(series), 2);

            for (int i = 0; i < getItemCount(series); i++) {
                labels.add(fStrucs[series].gset.getMember(i));
                m.setElement(i, 0, getX(series, i).floatValue());
                m.setElement(i, 1, getY(series, i).floatValue());
            }

            return new Dataframe(getSeriesName(series), m, labels, new String[]{"X", "Y"}, true, true);
        }

        public XYToolTipGenerator createXYToolTipGenerator() {
            return new XYToolTipGenerator() {
                public String generateToolTip(org.jfree.data.xy.XYDataset xyDataset, int series, int item) {
                    String rowName = fStrucs[series].gset.getMember(item);
                    return formatToolTip(rowName, chip);

                }
            };
        } // End class DatasetToolTipGenerator

    } // End class GaddyPlotModel2

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    private static String formatToolTip(final String rowName, final Chip chip_opt) {
        if (chip_opt == null) {
            return rowName;
        }

        if (VdbRuntimeResources.isChipGeneSymbol(chip_opt)) {
            String title = chip_opt.getTitle(rowName, Chip.OMIT_NULLS);
            if (title != null) {
                return rowName + ": " + title;
            } else {
                return rowName;
            }
        } else {
            String symbol = chip_opt.getSymbol(rowName, Chip.OMIT_NULLS);
            String title = chip_opt.getTitle(rowName, Chip.OMIT_NULLS);
            if (symbol != null) {
                if (symbol.equalsIgnoreCase(rowName)) { // dont know why the chip mathch isnt enuf
                    return rowName + ": " + title;
                } else {
                    return rowName + ": " + symbol + " - " + title;
                }
            } else {
                return rowName + ": na";
            }
        }
    }

    static class Struc {

        private String name;
        private float[] fRowIndexMeanHashMap;
        private float[] fRowIndexStdDevHashMap;

        private GeneSet gset;

        Struc(final Dataset ds, final GeneSet gset) {

            int numRow = gset.getNumMembers();
            this.fRowIndexMeanHashMap = new float[numRow];
            this.fRowIndexStdDevHashMap = new float[numRow];

            double xmin = Double.MAX_VALUE;
            double xmax = Double.MIN_VALUE;
            double ymin = Double.MAX_VALUE;
            double ymax = Double.MIN_VALUE;

            for (int r = 0; r < gset.getNumMembers(); r++) {
                String name = gset.getMember(r);
                Vector v = ds.getRow(name);
                float mean = (float) v.mean();
                float stddev = (float) v.stddev(true, false);
                if (xmin > mean) {
                    xmin = mean;
                }
                if (xmax < mean) {
                    xmax = mean;
                }
                if (ymin > stddev) {
                    ymin = stddev;
                }
                if (ymax < stddev) {
                    ymax = stddev;
                }

                this.fRowIndexMeanHashMap[r] = mean;
                this.fRowIndexStdDevHashMap[r] = stddev;
            }

            this.name = gset.getName(true);
            this.gset = gset;
        }

        public String getName() {
            return gset.getName(true) + "(n=" + gset.getNumMembers() + ")";
        }

    } // End class Struc

    static class StateFoldChangeStruc {

        private String name;
        private float[] fRowIndexMeanHashMap;
        private float[] fRowIndexStateLogMaxByMinHashMap;

        private GeneSet gset;

        StateFoldChangeStruc(final Dataset log2Dataset_samcol_into_states, final GeneSet gset, final boolean useMean) {

            int numRow = gset.getNumMembers();
            this.fRowIndexMeanHashMap = new float[numRow];
            this.fRowIndexStateLogMaxByMinHashMap = new float[numRow];

            for (int r = 0; r < gset.getNumMembers(); r++) {
                String name = gset.getMember(r);
                Vector v = log2Dataset_samcol_into_states.getRow(name);
                //this.fRowIndexMeanHashMap[r] = (float) v.meanOrMedian(useMean);
                this.fRowIndexMeanHashMap[r] = (float) v.max();
                this.fRowIndexStateLogMaxByMinHashMap[r] = (float) v.max() / v.min();
            }

            this.name = gset.getName(true);
            this.gset = gset;
        }

        public String getName() {
            return gset.getName(true) + "(n=" + gset.getNumMembers() + ")";
        }

    } // End class Struc

} // End class MiscChartModels
