/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;


import edu.mit.broad.genome.math.Matrix;


/**
 * Could not use booleanmatrix as need a, p and m
 */
public class APMMatrix extends Matrix {

    public static final String PRESENT_STR = "P";
    public static final String ABSENT_STR = "A";
    public static final String MISSING_STR = "M";
    public static final float PRESENT = (float) 1.0;
    public static final float ABSENT = (float) 0.0;

    // using NaN doesnt work as for some reason Float.NaN == Float.NaN doesnt work
    // @see IsMissing below
    // -> solved -- should use Float.isNaN() not '=='
    public static final float MISSING = Float.NaN;

    /**
     * Class Constructor.
     */
    public APMMatrix(final int nrows, final int ncols) {
        super(nrows, ncols);
    }

    // override Matrix impl -- just for the correct class
    // there are no class vars in apmatrix, so no *extra* copying
    public APMMatrix(final APMMatrix matrix, final boolean sharedvalues) {
        super(matrix, sharedvalues);
    }

    public boolean isPresent(int row, int col) {
        return getElement(row, col) == PRESENT;
    }

    public boolean isAbsent(int row, int col) {
        return getElement(row, col) == ABSENT;
    }

    public void setElement(int r, int c, String call) {
        super.setElement(r, c, valueOf(call));
    }

    public boolean isMissing(int row, int col) {
        //System.out.println("checking: " + getElement(row, col) + " versus: " + MISSING);
        return Float.isNaN(getElement(row, col));
    }

    public char getElement_char(int row, int col) {
        float f = getElement(row, col);
        if (f == PRESENT) {
            return PRESENT_STR.toCharArray()[0];
        } else if (f == ABSENT) {
            return ABSENT_STR.toCharArray()[0];
        } else if (Float.isNaN(f)) {
            return MISSING_STR.toCharArray()[0];
        } else {
            throw new IllegalStateException("Unknown value: " + f + " at row: " + row + " col: " + col);
        }
    }

    public static float valueOf(String s) {

        if (s == null) {
            throw new NullPointerException("Null value of param not allowed");
        }

        if (s.equalsIgnoreCase(PRESENT_STR)) {
            return PRESENT;
        } else if (s.equalsIgnoreCase(ABSENT_STR)) {
            return ABSENT;
        } else if (s.equalsIgnoreCase(MISSING_STR)) {
            return MISSING;
        } else if (s.equals("0")) {
            return MISSING;    // @todo confirm see breast.res in gcm dataset
        } else {
            throw new IllegalArgumentException("Unknown AP call value >" + s + "<");
        }
    }

    public float[] getRowCallFrequencies(int row) {

        float[] freq = new float[3];

        for (int i = 0; i < getNumCol(); i++) {
            float val = getElement(row, i);

            if (val == PRESENT) {
                freq[0]++;
            } else if (val == ABSENT) {
                freq[1]++;
            } else if (Float.isNaN(val)) {
                freq[2]++;
            }
        }

        for (int i = 0; i < 3; i++) {
            freq[i] = freq[i] / getNumCol();
        }

        return freq;
    }

    public float[] getColCallFrequencies(int col) {

        float[] freq = new float[3];

        for (int i = 0; i < getNumRow(); i++) {
            float val = getElement(i, col);

            if (val == PRESENT) {
                freq[0]++;
            } else if (val == ABSENT) {
                freq[1]++;
            } else if (Float.isNaN(val)) {
                freq[2]++;
            }
        }

        for (int i = 0; i < 3; i++) {
            freq[i] = freq[i] / getNumRow();
        }

        return freq;
    }

    /**
     * over the entire matrix
     */
    public float[] getCallFrequencies() {

        float[] freq = new float[3];

        for (int r = 0; r < getNumRow(); r++) {
            for (int c = 0; c < getNumCol(); c++) {
                float val = getElement(r, c);

                if (val == PRESENT) {
                    freq[0]++;
                } else if (val == ABSENT) {
                    freq[1]++;
                } else if (Float.isNaN(val)) {
                    freq[2]++;
                }
            }
        }

        for (int i = 0; i < 3; i++) {
            freq[i] = freq[i] / (getNumCol() * getNumRow());
        }

        return freq;
    }
}    // End APMatrix

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
