/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.Errors;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.parsers.AuxUtils;
import gnu.trove.THashSet;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Nov 19, 2003
 *         Time: 12:30:56 AM
 */
public abstract class AbstractGeneSetMatrix extends AbstractObject implements GeneSetMatrix {

    /**
     * Each member is a GeneSet
     */
    private List fGeneSets;

    private Set fGeneSetNames_nonaux = null;

    /**
     * Subclasses must call initMatrix
     */
    protected AbstractGeneSetMatrix() {
    }

    public GeneSetMatrix cloneShallow(final String newName) {
        this.setName(newName);
        return this;
    }

    // common init routine
    // subclasses MUST call
    // gset names MUST all be UNIQUE - this IS checked here
    // @BNOTE ADDED JAN 27 2006 geset names are case INsensitive
    protected void initMatrix(final String name, final GeneSet[] gsets) {
        super.initialize(name);

        if (gsets == null) {
            throw new IllegalArgumentException("Param gsets cannot be null");
        }

        Set names = new HashSet();
        Errors errors = new Errors();

        this.fGeneSets = new ArrayList(gsets.length);
        for (int i = 0; i < gsets.length; i++) {
            if (gsets[i] == null) {
                throw new IllegalArgumentException("Null GeneSet not allowed at index: " + i + " total len: " + gsets.length);
            }

            if (names.contains(gsets[i].getName())) {
                errors.add("GeneSets should have unique names. The lookup is case INsensitive. Found duplicate name: " + gsets[i].getName());
            } else {
                names.add(gsets[i].getName());
            }

            fGeneSets.add(gsets[i]);

        }

        errors.barfIfNotEmptyRuntime();

    }

    public boolean containsSet(final String gsetName) {
        if (fGeneSetNames_nonaux == null) {
            // init it
            this.fGeneSetNames_nonaux = new THashSet();
            for (int i = 0; i < getNumGeneSets(); i++) {
                //log.debug("Adding: " + getGeneSet(i).getName());
                fGeneSetNames_nonaux.add(AuxUtils.getAuxNameOnlyNoHash(getGeneSet(i).getName()));
            }
        }

        if (fGeneSetNames_nonaux.contains(gsetName)) {
            return true;
        }

        return fGeneSetNames_nonaux.contains(AuxUtils.getAuxNameOnlyNoHash(gsetName));

    }

    public String getQuickInfo() {
        StringBuffer buf = new StringBuffer().append(getNumGeneSets()).append(" gene sets");
        return buf.toString();
    }

    public int getNumGeneSets() {
        return fGeneSets.size();
    }

    /**
     * return a ref to the real geneset
     *
     * @param i
     * @return
     */
    public GeneSet getGeneSet(int i) {
        return (GeneSet) fGeneSets.get(i);
    }

    public Dataset createTagMatrix(boolean genesAsRows, boolean bufferzeros) {

        float zerovalue = 0;
        if (bufferzeros) {
            zerovalue = 0.01f;
        }

        if (genesAsRows) {

            // rows are genes, cols are gene sets
            String[] gsetNames = this.getGeneSetNames(true);
            String[] geneNames = this.getAllMemberNamesOnlyOnce();
            log.info("Making TagMatrix ...rown: " + geneNames.length + " coln: " + gsetNames.length);
            Matrix tagMatrix = new Matrix(geneNames.length, gsetNames.length);
            for (int c = 0; c < gsetNames.length; c++) {
                GeneSet gset = this.getGeneSet(c);
                for (int r = 0; r < geneNames.length; r++) {
                    if (gset.isMember(geneNames[r])) {
                        tagMatrix.setElement(r, c, 1);
                    } else {
                        tagMatrix.setElement(r, c, zerovalue);
                    }
                }
            }

            Dataset tagDataset = new DefaultDataset(getName() + "_tag_matrix", tagMatrix, geneNames, gsetNames, true);
            log.debug("Done making TagMatrix:" + tagDataset.getQuickInfo());

            return tagDataset;
        } else {

            // cols are genes, rows are gene sets
            String[] gsetNames = this.getGeneSetNames(true);
            String[] geneNames = this.getAllMemberNamesOnlyOnce();
            log.info("Making TagMatrix ...rown: " + geneNames.length + " coln: " + gsetNames.length);
            Matrix tagMatrix = new Matrix(gsetNames.length, geneNames.length);
            for (int r = 0; r < gsetNames.length; r++) {
                GeneSet gset = this.getGeneSet(r);
                for (int c = 0; c < geneNames.length; c++) {
                    if (gset.isMember(geneNames[c])) {
                        tagMatrix.setElement(r, c, 1);
                    } else {
                        tagMatrix.setElement(r, c, zerovalue);
                    }
                }
            }

            Dataset tagDataset = new DefaultDataset(getName() + "_tag_matrix", tagMatrix, gsetNames, geneNames, true);
            log.debug("Done making TagMatrix:" + tagDataset.getQuickInfo());

            return tagDataset;
        }
    }

    private Map fNameGeneSetMap; // lazily filled

    // case IN sensitive query
    public GeneSet getGeneSet(final String gsetnameF) {

        // @todo fix me hack to overcome the errors cause by using a File sep char in gset names
        // @see edbparser
        String gsetname = gsetnameF.replace('|', '/');
        gsetname = AuxUtils.getAuxNameOnlyNoHash(gsetnameF); // @note
        gsetname = gsetname.toUpperCase();

        if (fNameGeneSetMap == null) {
            fNameGeneSetMap = new HashMap(getNumGeneSets());
            for (int i = 0; i < getNumGeneSets(); i++) {
                GeneSet gset = getGeneSet(i);
                fNameGeneSetMap.put(gset.getName().toUpperCase(), gset); // already guranteed unique names
                fNameGeneSetMap.put(AuxUtils.getAuxNameOnlyNoHash(gset.getName()).toUpperCase(), gset);
            }
        }

        Object obj = fNameGeneSetMap.get(gsetname);

        String tryharder = null;
        if (obj == null) {
            // help out
            tryharder = gsetname;
            if (gsetname.indexOf("#") == -1) {
                tryharder = getName() + "#" + gsetname;
            }
            obj = fNameGeneSetMap.get(tryharder);
        }

        // sometimes / gets replaced into \
        if (obj == null) {
            // help out
            tryharder = gsetname;
            if (gsetname.indexOf("#") == -1) {
                tryharder = getName() + "#" + gsetname;
            }

            tryharder = tryharder.replace('\\', '/');
            obj = fNameGeneSetMap.get(tryharder);

            if (obj == null) {
                tryharder = gsetname;
                if (gsetname.indexOf("#") == -1) {
                    tryharder = getName() + "#" + gsetname;
                }

                tryharder = tryharder.replace('/', '\\');
                obj = fNameGeneSetMap.get(tryharder);
            }

            if (obj == null) {
                tryharder = gsetname.replace('|', '/');
                if (gsetname.indexOf("#") == -1) {
                    tryharder = getName() + "#" + gsetname;
                }

                tryharder = tryharder.replace('/', '\\');
                obj = fNameGeneSetMap.get(tryharder);
            }

        }


        if (obj == null) {
            StringBuffer buf = new StringBuffer("In GeneSetMatrix: " + getName() + " no GeneSet found with name: " + gsetname);
            buf.append("\nAvailable GeneSets are: \n");
            for (Iterator it = fNameGeneSetMap.keySet().iterator(); it.hasNext();) {
                Object key = it.next();
                buf.append(key.toString()).append('\n');
            }

            buf.append("Also tried looking for gset: ").append(tryharder);

            throw new IllegalArgumentException(buf.toString());
            //log.warn("temp disabled - could get geneset: " + gsetname);
            //return new FSet(gsetname, new String[]{});
        } else {
            return (GeneSet) obj;
        }
    }

    /**
     * All gsets
     * directly -- no cloning
     *
     * @return
     */
    public GeneSet[] getGeneSets() {
        return (GeneSet[]) fGeneSets.toArray(new GeneSet[fGeneSets.size()]);
    }

    public List getGeneSetsL() {
        return Collections.unmodifiableList(fGeneSets);
    }

    public GeneSet[] getGeneSets(int minSizeInclusive, int maxSizeInclusive) {
        return GeneSetMatrixHelper.getGeneSets(this, minSizeInclusive, maxSizeInclusive);
    }

    public List getGeneSetsL(int minSizeInclusive, int maxSizeInclusive) {
        return GeneSetMatrixHelper.getGeneSetsL(this, minSizeInclusive, maxSizeInclusive);
    }

    public String[] getGeneSetNames(boolean stripAux) {
        List names = getGeneSetNamesL(stripAux);
        return (String[]) names.toArray(new String[names.size()]);
    }

    public List getGeneSetNamesL(boolean stripAux) {
        List names = new ArrayList();
        for (int i = 0; i < getNumGeneSets(); i++) {
            String name = getGeneSet(i).getName();
            if (stripAux) {
                names.add(AuxUtils.getAuxNameOnlyNoHash(name));
            } else {
                names.add(name);
            }
        }

        return names;
    }

    public Set getGeneSetNamesS(boolean stripAux) {

        Set names = new HashSet();
        for (int i = 0; i < getNumGeneSets(); i++) {
            String name = getGeneSet(i).getName();
            if (stripAux) {
                names.add(AuxUtils.getAuxNameOnlyNoHash(name));
            } else {
                names.add(name);
            }
        }

        return names;
    }

    /**
     * The number of members in the biggest GeneSet.
     *
     * @return
     */
    public int getMaxGeneSetSize() {
        return GeneSetMatrixHelper.getMaxMemberCount(fGeneSets);
    }

    /**
     * non-redundant list of names of features across
     * gsets in this v
     *
     * @return
     */
    public String[] getAllMemberNamesOnlyOnce() {
        return GeneSetMatrixHelper.getAllMemberNames(fGeneSets);
    }

    public GeneSet getAllMemberNames_gset() {
        return new FSet(getName(), getAllMemberNamesOnlyOnceS());
    }

    // does the real stuff
    public Set getAllMemberNamesOnlyOnceS() {
        return GeneSetMatrixHelper.getAllMemberNamesS(fGeneSets);
    }

    public List getAllMemberNamesL() {
        return GeneSetMatrixHelper.getAllMemberNameOccurrencesL(fGeneSets);
    }

    public String[] getAllMemberNames() {
        return GeneSetMatrixHelper.getAllMemberNameOccurrences(fGeneSets);
    }

    public String getGeneSetName(int g) {
        return getGeneSet(g).getName();
    }

    /*
    public void setChip(Chip chip) {
        for (int i = 0; i < fGeneSets.size(); i++) {
            GeneSet gset = (GeneSet) fGeneSets.get(i);
            gset = gset.cloneShallow(chip, true);
            fGeneSets.set(i, gset);
        }
    }
    */

    /**
     * Names of all gsets -- guaranteed unique
     *
     * @return
     */
    public String[] getAllGeneSetNames() {
        return GeneSetMatrixHelper.getAllGeneSetNames(this);
    }

    public String[] getGeneSetMembershipNames(final String probeName) {
        return GeneSetMatrixHelper.getGeneSetMembershipNames(probeName, this);
    }

    public BitSet getGeneSetMembershipBitSet(final String probeName) {
        return GeneSetMatrixHelper.getGeneSetMembershipBitSet(probeName, this);
    }

    public BitSetDataset getGeneSetMembershipBitSetDataset(final String name, final String[] memberNames) {
        return GeneSetMatrixHelper.getGeneSetMembershipBitSetDataset(name, memberNames, this);
    }

    public BitSetDataset getGeneSetMembershipBitSetDataset() {
        return GeneSetMatrixHelper.getGeneSetMembershipBitSetDataset(this);
    }

    public GeneSet[] getGeneSets(final Set gsetNames) {
        return GeneSetMatrixHelper.getGeneSets(gsetNames, this);
    }

    public GeneSetMatrix getGeneSetMatrix(final GeneSet gsetNames) {
        return GeneSetMatrixHelper.getGeneSetMatrix(gsetNames, this);
    }

    public GeneSet[] getGeneSets(final String[] gsetNames) {
        return GeneSetMatrixHelper.getGeneSets(gsetNames, this);
    }

    public GeneSet[] getGeneSetMembershipSets(final String probeName) {
        return GeneSetMatrixHelper.getGeneSetMemberships(probeName, this);
    }

} // End class AbstractGeneSetMatrix
