package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.Vector;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Dec 16, 2005
 *         Time: 8:27:50 PM
 */
public abstract class AbstractJITDataset extends AbstractObject implements Dataset {

    private List fColNames;

    private List fRowNames;

    protected void warn(String msg) {
        System.out.println(">>> warn: " + msg); // @note dont use log for speed
    }


    public Dataset cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    /**
     * Class constructor
     *
     * @param name
     * @param rowNames
     * @param colNames
     */
    public AbstractJITDataset(final String name, final String[] rowNames, final String[] colNames) {
        super.initialize(name);
        this.fRowNames = Arrays.asList(rowNames);
        this.fColNames = Arrays.asList(colNames);
    }

    public String[] getColumnNamesArray() {
        List colNames = getColumnNames();
        return (String[]) colNames.toArray(new String[colNames.size()]);
    }

    public int[] getColumnIndices(final String[] colNames) {
        int[] indices = new int[colNames.length];
        for (int i = 0; i < colNames.length; i++) {
            indices[i] = getColumnIndex(colNames[i]);
        }

        return indices;
    }


    public GeneSet getRowNamesGeneSet() {
        return new FSet(getName(), getRowNames(), false);
    }

    public String getQuickInfo() {
        return getNumRow() + "x" + getNumCol();
    }

    public GeneSet getColumnNamesGeneSet() {
        return new FSet(getName(), getColumnNames(), false);
    }

    public int getRowIndex(String rowName) {
        return getRowNames().indexOf(rowName);
    }

    // arbitarily make the first ranked lists row names this datasets row names
    public String getRowName(int rown) {
        return fRowNames.get(rown).toString();
    }

    public List getRowNames() {
        return Collections.unmodifiableList(fRowNames);
    }

    public String[] getRowNamesArray() {
        return (String[]) fRowNames.toArray(new String[fRowNames.size()]);
    }

    public List getColumnNames() {
        return fColNames;
    }

    public String getColumnName(int coln) {
        return fColNames.get(coln).toString();
    }

    public List getColumnNames(Template prefixTemplateClasses) {
        throw new NotImplementedException();
    }

    public int getColumnIndex(String colName) {
        return fColNames.indexOf(colName);
    }

    public int getNumRow() {
        return fRowNames.size();
    }

    public int getNumCol() {
        return fColNames.size();
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public float getElement(final int row, final String colName) {
        return getElement(row, getColumnIndex(colName));
    }

    public float getElement(final String rowName, final int col) {
        return getElement(getRowIndex(rowName), col);
    }

    public float getElement(final String rowName, final String colName) {
        return getElement(getRowIndex(rowName), getColumnIndex(colName));
    }

    public Vector getRow(String rowName) {
        return getRow(getRowIndex(rowName));
    }

    public LabelledVector getRowLV(final String rowName) {
        return getRowLV(getRowIndex(rowName));
    }

    public LabelledVector getRowLV(final int rown) {
        return new LabelledVector(getRowName(rown), getColumnNamesArray(), getMatrix().getRowV(rown));
    }

    public RankedList getRowRL(final String rowName, final SortMode sort, final Order order) {
        return getRowLV(rowName).sort(sort, order);
    }

    public RankedList getRowRL(final int rown, final SortMode sort, final Order order) {
        return getRowLV(rown).sort(sort, order);
    }

    public Vector getColumn(String colName) {
        return getColumn(getColumnIndex(colName));
    }

    public Vector[] getColumns(final String[] colNames) {
        Vector[] vss = new Vector[colNames.length];
        for (int i = 0; i < colNames.length; i++) {
            vss[i] = getColumn(colNames[i]);
        }
        return vss;
    }

    public LabelledVector getColumnLV(String colName) {
        return getColumnLV(getColumnIndex(colName));
    }

    public RankedList getColumnRL(String colName, SortMode sort, Order order) {
        return getColumnRL(getColumnIndex(colName), sort, order);
    }

    // -------------------------------------------------------------------------------------------- //
    // ------------------------------------NOT IMPL METHODS -------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Vector[] getRows(final GeneSet gset) {
        throw new NotImplementedException();
    }

    public Vector getColumn(final int coln, final GeneSet forTheseGenes) {
        throw new NotImplementedException();
    }

    public Vector getColumn(final String colName, final GeneSet forTheseGenes) {
        throw new NotImplementedException();
    }

    public int getDim() {
        throw new NotImplementedException();
    }

} // End class AbstractJITDataset
