package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.*;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Jun 7, 2005
 *         Time: 12:15:07 PM
 */
public abstract class AbstractNeighDataset extends AbstractObject implements Dataset, QuickNeighDataset {

    private Dataset fDataset;

    private MetricStats[] fMetricStats;

    /**
     * Class constructor
     * This auto inits the calculations  for the QuickPearsonNeighborhood
     * (might take awhile)
     *
     * @param ds
     * @param hood
     */
    protected AbstractNeighDataset() {
        super();
    }

    public Dataset cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    // SUB-CLASSES MUST CALL
    protected void initCache(final Dataset ds, MetricStats[] mcs) {

        super.initialize(ds.getName());

        if (mcs == null) {
            throw new IllegalArgumentException("Parameter mcs cannot be null");
        }

        if (ds.getNumRow() != mcs.length) {
            throw new IllegalArgumentException("Mismatched mcs: " + mcs.length + " and dataset: " + ds.getNumRow());
        }

        this.fDataset = ds;

        this.fMetricStats = mcs;
    }

    public GeneSet getNeighborsUpOrDn(final String featureName, final int num, final boolean up) {
        return getRankedList(featureName, SortMode.REAL, Order.DESCENDING).getNamesOfUpOrDnXRanks_gs(num, up);
    }

    public GeneSetPair getNeighborsUpAndDn(final String featureName, final int num) {
        return getRankedList(featureName, SortMode.REAL, Order.DESCENDING).getNamesOfUpAndDnXRanks(num);
    }

    public MetricStats getMetricStats(int rown) {
        return fMetricStats[rown];
    }

    public MetricStats getMetricStats(String rowName) {
        return fMetricStats[getRowIndex(rowName)];
    }

    public Annot getAnnot() {
        return fDataset.getAnnot();
    }

    // SUB-CLASSES MUST IMPLEMENT
    public abstract MetricStats createMetricStats(Vector v);


    public RankedList getRankedList(final String probeName, final SortMode sort, final Order order) {
        return _getRankedList(TemplateFactory.createContinuousTemplate(probeName, this), sort, order);
    }

    // does the real work
    private RankedList _getRankedList(Template contTemplate, SortMode sort, Order order) {

        MetricStats mstats = this.createMetricStats(contTemplate.toVector());

        List dels = new ArrayList(this.getNumRow());

        for (int r = 0; r < this.getNumRow(); r++) {
            float corr = (float) mstats.getScore(this.getMetricStats(r));
            dels.add(new DoubleElement(r, corr));
        }

        //@todo check me: was disabled after aug rl changes. Changed back Oct1, 2005
        DoubleElement.sort(sort, order, dels);
        AddressedVector av = new AddressedVector(dels);
        return new DefaultRankedList(av, this);

        /*
         // sanity check - has to be!! only for real templates
             if (real && rl.getRankLabel(0).equals(probeName) == false) {
                 //System.out.println("Bad prog expected at rank 0: " + probeName + " got: " + rl.getRankLabel(0) + " rank was: " + rl.getRank(probeName));
                 //throw new IllegalStateException();
             }
        */

    }

    //------------------------------------------------------------------------------------
    //-------------------- REST ARE MUNDANE DATASET IMPLEMENTATIONS ---------------------
    //------------------------------------------------------------------------------------

    public Vector getRow(int rown) {
        return fDataset.getRow(rown);
    }

    public Vector getColumn(int coln) {
        return fDataset.getColumn(coln);
    }

    public Vector getColumn(final int coln, final GeneSet forTheseGenes) {
        return fDataset.getColumn(coln, forTheseGenes);
    }

    public Vector getColumn(final String colName, final GeneSet forTheseGenes) {
        return fDataset.getColumn(colName, forTheseGenes);
    }

    public Vector getColumn(String colName) {
        return fDataset.getColumn(colName);
    }

    public Vector[] getColumns(final String[] colNames) {
        throw new NotImplementedException();
    }

    public LabelledVector getRowLV(final String rowName) {
        return getRowLV(getRowIndex(rowName));
    }

    public LabelledVector getRowLV(final int rown) {
        return new LabelledVector(getRowName(rown), getColumnNamesArray(), getMatrix().getRowV(rown));
    }

    public RankedList getRowRL(final String rowName, final SortMode sort, final Order order) {
        return getRowLV(rowName).sort(sort, order);
    }

    public RankedList getRowRL(final int rown, final SortMode sort, final Order order) {
        return getRowLV(rown).sort(sort, order);
    }

    public LabelledVector getColumnLV(String colName) {
        return fDataset.getColumnLV(colName);
    }

    public LabelledVector getColumnLV(int coln) {
        return fDataset.getColumnLV(coln);
    }

    public RankedList getColumnRL(final int coln, SortMode sort, Order order) {
        return fDataset.getColumnRL(coln, sort, order);
    }

    public RankedList getColumnRL(final String colName, SortMode sort, Order order) {
        return fDataset.getColumnRL(colName, sort, order);
    }

    public float getElement(int rown, int coln) {
        return fDataset.getElement(rown, coln);
    }

    public float getElement(final String rowName, final String colName) {
        return fDataset.getElement(rowName, colName);
    }


    public float getElement(final String rowName, final int col) {
        return fDataset.getElement(rowName, col);
    }

    public float getElement(final int row, final String colName) {
        return fDataset.getElement(row, colName);
    }

    public String getRowName(int rown) {
        return fDataset.getRowName(rown);
    }

    public Vector getRow(String rowName) {
        return fDataset.getRow(rowName);
    }

    public Vector[] getRows(GeneSet gset) {
        return fDataset.getRows(gset);
    }

    public List getRowNames() {
        return fDataset.getRowNames();
    }

    public int getRowIndex(String rowname) {
        return fDataset.getRowIndex(rowname);
    }

    public String[] getRowNamesArray() {
        return fDataset.getRowNamesArray();
    }

    public List getColumnNames() {
        return fDataset.getColumnNames();
    }

    public List getColumnNames(Template prefixTemplateClasses) {
        return fDataset.getColumnNames(prefixTemplateClasses);
    }

    public int getColumnIndex(String colname) {
        return fDataset.getColumnIndex(colname);
    }

    public int[] getColumnIndices(final String[] colNames) {
        throw new NotImplementedException();
    }

    public String[] getColumnNamesArray() {
        return fDataset.getColumnNamesArray();
    }

    public String getColumnName(int coln) {
        return fDataset.getColumnName(coln);
    }

    public int getNumRow() {
        return fDataset.getNumRow();
    }

    public int getNumCol() {
        return fDataset.getNumCol();
    }

    public int getDim() {
        return fDataset.getDim();
    }

    public Matrix getMatrix() {
        return fDataset.getMatrix();
    }

    public String getQuickInfo() {
        return fDataset.getQuickInfo();
    }

    public GeneSet getColumnNamesGeneSet() {
        return fDataset.getColumnNamesGeneSet();
    }

    public GeneSet getRowNamesGeneSet() {
        return fDataset.getRowNamesGeneSet();
    }

}
