package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.Errors;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.sampledb.SampleAnnot;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Jan 16, 2006
 *         Time: 1:55:30 PM
 */
public abstract class AbstractRankedListDb extends AbstractRankedListDb_base implements RankedListDb {

    private Dataset fFullDataset;

    private TemplateCohort fTch;

    private String fQuickInfo;

    /**
     * Class constructor
     * <p/>
     * Things should already be duplicated as needed
     *
     * @param rowNames
     * @param colNames
     */

    protected AbstractRankedListDb() {
    }

    // subclasses must override
    protected void init(final String name,
                        final List rowNames,
                        final List colNames,
                        final Chip chip,
                        final Annot annot_for_rl,
                        final Dataset fullDs,
                        final TemplateCohort tch) {

        // sanity checks
        // 1) The colnames must all be in the tch
        Errors errors = new Errors("Mismatched rdb and tch");
        for (int c = 0; c < colNames.size(); c++) {
            String compName = colNames.get(c).toString();
            if (tch != null && tch.hasComp(compName) == false) {
                errors.add("tch doesnt have: " + compName);
            }
        }

        errors.barfIfNotEmptyRuntime();

        // @todo check performance of this check
        /*
        errors = new Errors("Mismatched rdb and dataset features");
        for (int r = 0; r < rowNames.size(); r++) {
            String rowName = rowNames.get(r).toString();
            if (fullDs.getRowIndex(rowName) == -1) {
                errors.add(rowName);
            }
        }
        errors.barfIfNotEmptyRuntime();
        */

        if (chip == null) {
            throw new IllegalArgumentException("Param chip cannot be null");
        }
        if (annot_for_rl == null) {
            throw new IllegalArgumentException("Param annot_for_rl cannot be null");
        }

        if (annot_for_rl.getSampleAnnot_global() == null) {
            throw new IllegalArgumentException("Param annot_for_rl cannot have null sample annot");
        }

        // Ok, alls well
        super.init(name, rowNames, colNames, chip, annot_for_rl);
        this.fFullDataset = fullDs;
        // this.fChip = chip; // @note dont want to get this from the dataset as the ds might be jit
        this.fTch = tch;
    }

    public String getQuickInfo() {
        if (fQuickInfo == null) {
            _setQuickInfo();
        }

        return fQuickInfo;
    }


    public OrderedSampleNames getSampleNames(final int[] rlIndices, final boolean classA) {

        //SampleAnnot sa = getAnnot().getSampleAnnot_synched(getDataset().getColumnNamesArray()); // DONT - this is the rl's annots not the datasets
        final SampleAnnot sa = getDataset().getAnnot().getSampleAnnot_synched(getDataset().getColumnNamesArray());
        final List list = new ArrayList();
        final GeneSet[] gsets = new GeneSet[rlIndices.length];

        for (int i = 0; i < rlIndices.length; i++) {
            final String rlName = getRankedListName(rlIndices[i]);
            final TemplateCohort.Comp comp = getComp(rlName);
            final List sampleNames = comp.getSampleNames(sa, classA);
            list.addAll(sampleNames);
            gsets[i] = new FSet(rlName, sampleNames, true);
        }

        final OrderedSampleNames osn = new OrderedSampleNames();
        osn.names = (String[]) list.toArray(new String[list.size()]);
        osn.namesByRl = gsets;
        return osn;
    }

    // -------------------------------------------------------------------------------------------- //

    public Dataset getDataset() {
        return fFullDataset;
    }

    public TemplateCohort getTemplateCohort() {
        return fTch;
    }

    public TemplateCohort.Comp getComp(final String rlName) {
        return fTch.getComp(rlName);
    }

    public TemplateCohort.Comp getComp(final int rlColIndex) {
        return fTch.getComp(getRankedListName(rlColIndex));
    }

    // Feed in a dataset, a comp file and get out a rdb
    // the rdb has no meaning outside of the ds and comp file
    public Extracted extract(final String rlName) {
        return _exr(getRankedList(rlName));
    }

    public Extracted extract(final int coln) {
        return _exr(getRankedList(coln));
    }

    public Extracted extract(final String rlName, final GeneSet gset) {
        return _exr(getRankedList(rlName), gset);
    }

    public Extracted extract(final int colIndex, final GeneSet gset) {
        return _exr(getRankedList(colIndex), gset);
    }

    public Extracted extract(final String rlName, final int upAndDnX) {
        RankedList rl = getRankedList(rlName);
        return _exr(rl, rl.getNamesOfUpAndDnXRanks(upAndDnX).getBoth());
    }

    public Extracted extract(final int colIndex, final int upAndDnX) {
        RankedList rl = getRankedList(colIndex);
        return _exr(rl, rl.getNamesOfUpAndDnXRanks(upAndDnX).getBoth());
    }

    // does the real work
    private Extracted _exr(final RankedList rl, final GeneSet gset) {
        final TemplateCohort.Comp comp = fTch.getComp(rl.getName());
        //DatasetTemplate dt = fTch.getComp(rl.getName()).createDatasetTemplate(fFullDataset);
        // qualify to gene set
        final RankedList sub_rl = rl.extractRanked(gset);
        return new ExtractedImpl_GeneSet(rl.getName(), comp, sub_rl, gset);
    }

    // does the real work
    private Extracted _exr(final RankedList rl) {
        TemplateCohort.Comp comp = fTch.getComp(rl.getName());
        return new ExtractedImpl_GeneSet(rl.getName(), comp, rl, null);
    }

    // -------------------------------------------------------------------------------------------- //

    private void _setQuickInfo() {
        // @todo
        fQuickInfo = "todo";
        /*
        StringBuffer buf = new StringBuffer().append(getNumRow()).append('x').append(getNumRankedLists());
        if (this.getAnnot() != null) {
            FeatureAnnot fa = getAnnot().getFeatureAnnot();
            SampleAnnot sa = getAnnot().getSampleAnnot();
            buf.append(" (ann: ");
            if (fa != null) {
                buf.append(fa.getNumFeatures());
            } else {
                buf.append("na");
            }
            buf.append(",");

            if (sa != null) {
                buf.append(sa.getNumSamples());
            } else {
                buf.append("na");
            }

            buf.append(",");

            if (getAnnot().getChip() != null) {
                buf.append(getAnnot().getChip().getName());
            } else {
                buf.append("chip na");
            }

            buf.append(")");

        }
        fQuickInfo = buf.toString();

        */
    }

    /*
    public TemplateCohort.Comp getComp(int coln) {
        if (fTch == null) {
            throw new IllegalStateException("tch not set: " + fTch);
        }

        return fTch.getComp(coln);
    }


    public TemplateCohort.Comp getComp(final String rlName) {
        if (fTch == null) {
            throw new IllegalStateException("tch not set: " + fTch);
        }

        return fTch.getComp(rlName);
    }
    */

    // -------------------------------------------------------------------------------------------- //

    abstract class AbstractExtractedImpl implements RankedListDb.Extracted {

        String fName;
        TemplateCohort.Comp comp;
        RankedList rl;

        AbstractExtractedImpl(String name,
                              TemplateCohort.Comp comp,
                              RankedList rl) {
            this.fName = name;
            this.comp = comp;
            this.rl = rl;
        }

        protected void checkTemplate(final Template t) {
            if (t.getNumClasses() != 2) {
                throw new IllegalArgumentException("No biphasic: " + t.getNumClasses() + " t: " + t.getName());
            }

            if (t.getClass(0).getName().equals(comp.getClassAShortName()) == false) {
                throw new IllegalArgumentException("Mismatched classes: t: <" + t.getClass(0).getName() + "< >" + t.getClass(1).getName() +
                        "> comp: >" + comp.getClassAShortName() + "< >" + comp.getClassBShortName() + "<");
            }

            if (t.getClass(1).getName().equals(comp.getClassBShortName()) == false) {
                throw new IllegalArgumentException("Mismatched classes: t: <" + t.getClass(0).getName() + "< >" + t.getClass(1).getName() +
                        "> comp: >" + comp.getClassAShortName() + "< >" + comp.getClassBShortName() + "<");
            }
        }

        public TemplateCohort.Comp getComp() {
            return comp;
        }

        public RankedList getRankedList() {
            return rl;
        }

        public String getName() {
            return fName;
        }


    } // End class ExtractedImpl


    class ExtractedImpl_GeneSet extends AbstractExtractedImpl {

        private DatasetTemplate dt;

        private Dataset sortedDs;

        private GeneSet qualify_to_gset;

        ExtractedImpl_GeneSet(final String name,
                              final TemplateCohort.Comp comp,
                              final RankedList rl,
                              final GeneSet qualify_to_gset_opt) {
            super(name, comp, rl);
            this.qualify_to_gset = qualify_to_gset_opt;
        }

        private void initDT() {
            if (dt == null) {
                //log.debug("createDatasetTemplate");
                this.dt = getComp().createDatasetTemplate(fFullDataset);
                checkTemplate(dt.getTemplate());

                // always make contiguous -- @todo check the perf impact of this later
                //log.debug("extracting");
                this.dt = new DatasetGenerators().extract(dt.getDataset(), dt.getTemplate());
                checkTemplate(dt.getTemplate());
                //log.debug("done extraction");
                if (qualify_to_gset != null && qualify_to_gset.getNumMembers() > 0) {
                    // qualify to gene set
                    Dataset ds = new DatasetGenerators().extractRows(dt.getDataset(), qualify_to_gset);
                    this.dt = new DatasetTemplate(ds, dt.getTemplate());
                    //log.debug("done qual");
                }

            }
        }

        public DatasetTemplate getDatasetTemplate() {
            initDT();
            return dt;
        }

        public Dataset getDatasetSorted() {
            initDT();
            if (sortedDs == null) {
                sortedDs = new DatasetGenerators().orderDatasetRows(getName() + "_ordered", getRankedList().getRankedNamesArray(), dt.getDataset(false));
            }

            return sortedDs;
        }

    } // End class ExtractedImpl1


} // End class AbstractRankedListDb
