package edu.mit.broad.genome.objects;

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

import edu.mit.broad.genome.math.*;

import java.awt.*;

/**
 * A Dataset with logic to assign a Color to each element corresponding
 * to its float value.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ColorDatasetImpl extends DatasetProxy implements ColorDataset {

    protected ScaleMode fScaleMode;

    protected Orientation fOrientType;

    protected ColorScheme fColorScheme;

    protected GraphMode fGraphMode;

    private Color[][] fColors;

    /**
     * Class Constructor.
     *
     * @param ds
     * @param smode
     * @param colorScheme
     * @param orient
     */
    public ColorDatasetImpl(final Dataset ds,
                            final ScaleMode smode,
                            final ColorScheme colorScheme,
                            final Orientation orient,
                            final GraphMode gmode) {

        super("colored." + ds.getName(), ds);

        this.fScaleMode = smode;
        this.fColorScheme = colorScheme;
        this.fOrientType = orient;
        this.fGraphMode = gmode;

        cacheColors();
    }

    /**
     * Class Constructor.
     * Uses defaultss for color scheme, orientation etc.
     */
    public ColorDatasetImpl(final Dataset ds) {
        this(ds, DEFAULT_SCALE_MODE, DEFAULT_COLOR_SCHEME, DEFAULT_ORIENTATION, DEFAULT_GRAPH_MODE);
    }

    public ColorDatasetImpl(final ColorDataset cds, final ScaleMode smode) {
        this(cds, smode, cds.getColorScheme(), cds.getOrientation(), cds.getGraphMode());
    }

    public ColorDatasetImpl(final ColorDataset cds, final Orientation orient) {
        this(cds, cds.getScaleMode(), cds.getColorScheme(), orient, cds.getGraphMode());
    }

    public ColorDatasetImpl(final ColorDataset cds, final GraphMode gmode) {
        this(cds, cds.getScaleMode(), cds.getColorScheme(), cds.getOrientation(), gmode);
    }

    public ColorDatasetImpl(final ColorDataset cds, final ColorScheme colorScheme) {
        this(cds, cds.getScaleMode(), colorScheme, cds.getOrientation(), cds.getGraphMode());
    }

    /**
     * Class Constructor.
     */
    public ColorDatasetImpl(final Dataset ds, final ScaleMode type) {
        this(ds, type, DEFAULT_COLOR_SCHEME, DEFAULT_ORIENTATION, DEFAULT_GRAPH_MODE);
    }

    /**
     * Class Constructor.
     *
     * @param ds
     * @param colorScheme
     */
    public ColorDatasetImpl(final Dataset ds, final ColorScheme colorScheme) {
        this(ds, DEFAULT_SCALE_MODE, colorScheme, DEFAULT_ORIENTATION, DEFAULT_GRAPH_MODE);
    }

    public ColorDatasetImpl(final Dataset ds, final ScaleMode scaleMode, final ColorScheme colorScheme) {
        this(ds, scaleMode, colorScheme, DEFAULT_ORIENTATION, DEFAULT_GRAPH_MODE);
    }

    /**
     * Class Constructor.
     *
     * @param ds
     * @param orient
     */
    public ColorDatasetImpl(final Dataset ds, final Orientation orient) {
        this(ds, DEFAULT_SCALE_MODE, DEFAULT_COLOR_SCHEME, orient, DEFAULT_GRAPH_MODE);
    }

    /**
     * Class Constructor.
     *
     * @param ds
     * @param gmode
     */
    public ColorDatasetImpl(final Dataset ds, final GraphMode gmode) {
        this(ds, DEFAULT_SCALE_MODE, DEFAULT_COLOR_SCHEME, DEFAULT_ORIENTATION, gmode);
    }

    // @maint in properties are added, this method could need updating
    public ColorDataset cloneState(final Dataset ds) {
        ColorDatasetImpl cds = new ColorDatasetImpl(ds);
        cds.setColorScheme(fColorScheme);
        cds.setGraphMode(fGraphMode);
        cds.setOrientType(fOrientType);
        cds.setScaleMode(fScaleMode);
        return cds;
    }

    public Orientation getOrientation() {
        return fOrientType;
    }

    public GraphMode getGraphMode() {
        return fGraphMode;
    }

    public ScaleMode getScaleMode() {
        return fScaleMode;
    }

    protected void cacheColors() {

        if (fOrientType.isByRow()) {
            cacheColorsByRow();
        } else if (fOrientType.isByCol()) {
            cacheColorsByColumn();
        } else {
            throw new IllegalStateException("Unknown orientType option: " + fOrientType);
        }
    }

    protected void cacheColorsByRow() {

        // wierd for loop structure as trying to optimize calcs
        //log.debug("Beginning to cache fColors by ROW from: " + this.getClass());

        fColors = new Color[getNumRow()][getNumCol()];

        for (int r = 0; r < getNumRow(); r++) {
            Vector vr = getRow(r);
            Vector v = vr.toVectorNaNless(); // @note

            if (fScaleMode.isRelative()) {
                double center = v.meanOrMedian(fScaleMode.isMean());
                double max = v.max();
                double min = v.min();

                for (int c = 0; c < getNumCol(); c++) {
                    if (fScaleMode.isMean()) {

                        boolean coloraswhite = false;
                        if (fScaleMode == ScaleMode.REL_MEAN_ZERO_OMITTED) { // @todo improve
                            coloraswhite = true;
                        }

                        //fColors[r][c] = Helper.computeRelativeMeanColor(getElement(r, c), min, max, center, fGraphMode, fColorScheme);
                        fColors[r][c] = Helper.computeRelativeMeanColor(getElement(r, c), min, max, center, fGraphMode, fColorScheme, coloraswhite);
                    } else {
                        fColors[r][c] = Helper.computeRelativeMedianColor(getElement(r, c), min, max,
                                center, fGraphMode, fColorScheme);
                    }
                }
            } else {
                for (int c = 0; c < getNumCol(); c++) {
                    fColors[r][c] = Helper.computeAbsoluteColor(getElement(r, c), fGraphMode, fColorScheme);

                    //if (fScaleMode == SCALE_OPTIONS[2]) fColors[r][c] = computeAbsoluteColor(getElement(r, c), v);
                    //else if (fScaleMode == FOUR_COLOR) fColors[r][c] = computeFourColor(getElement(r, c), v);
                    //else throw new IllegalArgumentException("Unknwon ScaleMode" + fScaleMode);
                }
            }
        }

        //log.debug("Finished caching Colors: " + this.getClass());
    }

    protected void cacheColorsByColumn() {

        // wierd for loop structure as trying to optimize calcs
        log.info("Beginning to cache fColors by COL from: " + this.getClass());

        fColors = new Color[getNumRow()][getNumCol()];

        for (int c = 0; c < getNumCol(); c++) {
            Vector v = getColumn(c);

            if (fScaleMode.isRelative()) {
                double mean = v.mean();
                double median = v.median();
                double max = v.max();
                double min = v.min();

                for (int r = 0; r < getNumRow(); r++) {
                    if (fScaleMode.isMean()) {
                        fColors[r][c] = Helper.computeRelativeMeanColor(getElement(r, c), min, max, mean, fGraphMode, fColorScheme, false);
                    } else {
                        fColors[r][c] = Helper.computeRelativeMedianColor(getElement(r, c), min, max,
                                median, fGraphMode, fColorScheme);
                    }
                }
            } else {
                for (int r = 0; r < getNumRow(); r++) {
                    fColors[r][c] = Helper.computeAbsoluteColor(getElement(r, c), fGraphMode, fColorScheme);

                    //if (fScaleMode == SCALE_OPTIONS[2]) fColors[r][c] = computeAbsoluteColor(getElement(r, c), v);
                    //else if (fScaleMode == FOUR_COLOR) fColors[r][c] = computeFourColor(getElement(r, c), v);
                    //else throw new IllegalArgumentException("Unknwon ScaleMode" + fScaleMode);
                }
            }
        }

        log.info("Finished caching Colors: " + this.getClass());
    }

    public Color getColor(int row, int col) {
        return fColors[row][col];
    }

    /**
     * will cause colors to be (re)cached
     */
    public void setColorScheme(final ColorScheme colorScheme) {

        this.fColorScheme = colorScheme;

        cacheColors();
    }

    public ColorScheme getColorScheme() {
        return fColorScheme;
    }

    /**
     * will cause colors to be (re)cached
     * default -> traditional blue-pink-o-grams MIT color scheme
     */
    public void setDefaultColorScheme() {
        this.fColorScheme = new ColorSchemes.BroadCancer();

        cacheColors();
    }

    /**
     * will cause colors to be (re)cached
     */
    public void setScaleMode(final ScaleMode ctype) {

        this.fScaleMode = ctype;

        cacheColors();
    }

    /**
     * will cause colors to be (re)cached
     */
    public void setOrientType(Orientation otype) {

        this.fOrientType = otype;

        cacheColors();
    }

    /**
     * will cause colors to be (re)cached
     */
    public void setGraphMode(GraphMode gmode) {

        this.fGraphMode = gmode;

        cacheColors();
    }

    // @todo fix me for MV dataset in needed
    public static Color[] createColors(final Vector v, final ColorScheme colorScheme) {

        double min = v.min();
        double max = v.max();
        double mean = v.mean();

        int ind;

        Color[] colors = new Color[v.getSize()];

        for (int i = 0; i < v.getSize(); i++) {
            float x = v.getElement(i);
            double cVal = (x - min) / (max - min);
            if (x <= mean) {
                cVal = (float) .5 * (x - min) / (mean - min);
            } else {
                cVal = (float) .5 + (float) .5 * (x - mean) / (max - mean);
            }


            ind = (int) (cVal * colorScheme.getNumColors());
            //System.out.println("cVal="+ cVal + " ind: " + ind);

            // Sometimes cVal == 1 so correct for that here
            if (ind == colorScheme.getNumColors()) {
                ind = colorScheme.getNumColors() - 1;
            }

            colors[i] = colorScheme.getColor(ind);
        }

        return colors;

    }


}    // End ColorDataset

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
