/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.objects;


import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.utils.ArrayUtils;
import edu.mit.broad.genome.utils.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * Imp - this is seperate and distinct form a Dataset.
 * Eventhough it is very very similar in structure (except that no annotations/desc ever)
 * But thats not the main diff. The main diff is the intended use -
 * dfs are meant to be the xomics representation of the R dataframe / matrix structures.
 * dfs -> basically any labelled matrix (not necc a gene expression dataset)
 * <p/>
 * They can be seamlessless assocaited with actions that launch R scripts.
 * They are generally not as big as Datasets (though can be) -- for instance the
 * pairwise matrix formed after a Mantel test.
 * We dont not want Dataframes to automatically be available to the application
 * as a Dataset -- as they are not always interchangable. Options are available
 * to explicitly convert (and easily) a given df into a ds and the use the resulting
 * ds in whatever analysis that ds are available for.
 * But it would be wierd if a pairwise comparison matrix were available as a bpog option.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

// intentionally differntiate this from a dataset
//public class Dataframe extends AbstractObject implements Dataset {

public class Dataframe extends AbstractObject implements IDataframe {

    private Matrix fMatrix;
    private List fRowNames;
    private List fColNames;

    /**
     * subclasses MUST call init after using this form of the constructor.
     */
    protected Dataframe() {
    }

    public Dataframe cloneShallow(String newName) {
        super.setName(newName);
        return this;
    }

    /**
     * Class Constructor.
     * Specified data is copied over - data is NOT shared
     *
     * @param name
     * @param matrix
     * @param rowNames
     * @param colNames
     */
    public Dataframe(final String name, final Matrix matrix, final List rowNames, final List colNames, final boolean shareAll) {
        this(name, matrix, rowNames, colNames, shareAll, shareAll, shareAll);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     * @param colNames
     * @param shareAll
     */
    public Dataframe(final String name, final Matrix matrix, final List colNames, final boolean shareAll) {
        this(name, matrix, CollectionUtils.createSeriesStrings(0, matrix.getNumRow()), colNames, shareAll, true, shareAll);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     * @param shareAll
     * @param rowNames
     */
    public Dataframe(final String name, final Matrix matrix, final boolean shareAll, final List rowNames) {
        this(name, matrix, rowNames, CollectionUtils.createSeriesStrings(0, matrix.getNumCol()), shareAll, shareAll, true);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     * @param colNames
     * @param shareMatrix
     */
    public Dataframe(final String name, final Matrix matrix, final String[] colNames, final boolean shareMatrix) {
        this(name, matrix, CollectionUtils.createSeriesStrings(0, matrix.getNumRow()), ArrayUtils.toList(colNames), shareMatrix, true, true);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     * @param shareMatrix
     */
    public Dataframe(final String name, final Matrix matrix, final boolean shareMatrix) {
        this(name, matrix, CollectionUtils.createSeriesStrings(0, matrix.getNumRow()), CollectionUtils.createSeriesStrings(0, matrix.getNumCol(), "col_"), true, true, shareMatrix);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     * @param rowNames
     * @param colNames
     */
    public Dataframe(final String name, final Matrix matrix, final String[] rowNames, final List colNames, boolean shareMatrix, boolean shareColNames) {

        if (matrix == null) {
            throw new NullPointerException("Dataset matrix cant be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Param rowNames cannot be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("Param colNames cannot be null");
        }

        Matrix dmatrix;
        List drowNames;
        List dcolNames;

        if (shareMatrix) {
            dmatrix = matrix;
        } else {
            dmatrix = matrix.cloneDeep();
        }

        drowNames = new ArrayList(rowNames.length);
        for (int r = 0; r < rowNames.length; r++) {
            drowNames.add(rowNames[r]);
        }

        if (shareColNames) {
            dcolNames = colNames;
        } else {
            dcolNames = new ArrayList(colNames);
        }

        init(name, dmatrix, drowNames, dcolNames);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param dfs
     * @todo add row name checking if needed
     */
    public Dataframe(final String name, final Dataframe[] dfs) {

        if (dfs == null) {
            throw new NullPointerException("Paramters dfs cant be null");
        }

        Matrix[] mss = new Matrix[dfs.length];
        for (int i = 0; i < dfs.length; i++) {
            if (dfs[i] == null) {
                throw new IllegalArgumentException("Dataframe cannot be null at: " + i);
            }
            mss[i] = dfs[i].getMatrix();
        }

        Matrix dmatrix = new Matrix(mss, false);
        List rowNames = new ArrayList(dfs[0].getRowNames());
        List colNames = new ArrayList();

        for (int i = 0; i < dfs.length; i++) {
            String[] cn = dfs[i].getColumnNamesArray();
            for (int c = 0; c < cn.length; c++) {
                colNames.add(cn[i] + "_" + dfs[i].getName());
            }
        }

        init(name, dmatrix, rowNames, colNames);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     * @param rowNames
     * @param colNames
     * @param shareMatrix
     * @param shareRowNames
     */
    public Dataframe(final String name, final Matrix matrix, final List rowNames, final String[] colNames, boolean shareMatrix, boolean shareRowNames) {

        if (matrix == null) {
            throw new NullPointerException("Dataset matrix cant be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Param rowNames cannot be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("Param colNames cannot be null");
        }

        Matrix dmatrix;
        List drowNames;
        List dcolNames;

        if (shareMatrix) {
            dmatrix = matrix;
        } else {
            dmatrix = matrix.cloneDeep();
        }

        dcolNames = new ArrayList(colNames.length);
        for (int r = 0; r < colNames.length; r++) {
            dcolNames.add(colNames[r]);
        }

        if (shareRowNames) {
            drowNames = rowNames;
        } else {
            drowNames = new ArrayList(rowNames);
        }

        init(name, dmatrix, drowNames, dcolNames);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     * @param rowNames
     * @param colNames
     * @param shareMatrix
     */
    public Dataframe(final String name, final Matrix matrix, final String[] rowNames, final String[] colNames, boolean shareMatrix) {

        if (matrix == null) {
            throw new NullPointerException("Dataset matrix cant be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Param rowNames cannot be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("Param colNames cannot be null");
        }

        Matrix dmatrix;
        List drowNames;
        List dcolNames;

        if (shareMatrix) {
            dmatrix = matrix;
        } else {
            dmatrix = matrix.cloneDeep();
        }

        dcolNames = new ArrayList(colNames.length);
        for (int c = 0; c < colNames.length; c++) {
            dcolNames.add(colNames[c]);
        }

        drowNames = new ArrayList(rowNames.length);
        for (int r = 0; r < rowNames.length; r++) {
            drowNames.add(rowNames[r]);
        }

        init(name, dmatrix, drowNames, dcolNames);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     * @param rowNames
     * @param colNames
     * @param shareMatrix
     * @param shareRowNames
     * @param shareColNames
     */
    public Dataframe(final String name, final Matrix matrix, final List rowNames, final List colNames,
                     final boolean shareMatrix, final boolean shareRowNames, final boolean shareColNames) {

        if (matrix == null) {
            throw new IllegalArgumentException("Param matrix cant be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Param rownames cant be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("Param rownames cant be null");
        }

        Matrix dmatrix;
        List drowNames;
        List dcolNames;

        if (shareMatrix) {
            dmatrix = matrix;
        } else {
            dmatrix = matrix.cloneDeep();
        }

        if (shareRowNames) {
            drowNames = rowNames;
        } else {
            drowNames = new ArrayList(rowNames);
        }

        if (shareColNames) {
            dcolNames = colNames;
        } else {
            dcolNames = new ArrayList(colNames);
        }

        init(name, dmatrix, drowNames, dcolNames);

    }

    /**
     * assumes that when i am called the data has already been duplicated
     * i.e callers within this method are responsible for passing me
     * already duplicated data.
     */
    private void init(String dsname, Matrix matrix, List rowNames, List colNames) {

        super.initialize(dsname);

        if (matrix == null) {
            throw new IllegalArgumentException("Matrix cannot be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("rowNames cannot be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("colNames cannot be null");
        }

        if (matrix.getNumRow() != rowNames.size()) {
            throw new IllegalArgumentException("Matrix numrow: " + matrix.getNumRow() + " and rowNames: "
                    + rowNames.size() + " do not match in size");
        }

        if (matrix.getNumCol() != colNames.size()) {
            throw new IllegalArgumentException("Matrix numcol: " + matrix.getNumCol() + " and colNames: "
                    + colNames.size() + " do not match in size");
        }

        this.fMatrix = matrix;
        fMatrix.setImmutable();    // IMP notice.
        this.fRowNames = Collections.unmodifiableList(rowNames);
        ;
        this.fColNames = Collections.unmodifiableList(colNames);
        ;
    }

    public Vector getRow(int rown) {
        return fMatrix.getRowV(rown);
    }

    public Vector getRow(String rowName) {
        int index = getRowIndex(rowName);
        if (index == -1) {
            throw new IllegalArgumentException("No such rowName: " + rowName + " \n" + getRowNames());
        }

        return fMatrix.getRowV(index);
    }

    public Vector getColumn(int coln) {
        return fMatrix.getColumnV(coln);
    }

    public String[] getColumnAsStrings(int coln) {
        return Helper.getColumnAsStrings(this, coln);
    }

    public String[] getColumnAsStrings(final String colName) {
        return Helper.getColumnAsStrings(this, colName);
    }

    public float getElement(int rown, int coln) {
        return fMatrix.getElement(rown, coln);
    }

    public Object getElementObj(int rown, int coln) {
        return new Float(fMatrix.getElement(rown, coln));
    }

    public String getRowName(int rown) {
        return (String) fRowNames.get(rown);
    }

    public List getRowNames() {
        return Collections.unmodifiableList(fRowNames);
    }

    public int getRowIndex(String rowName) {
        return fRowNames.indexOf(rowName);
    }

    public String[] getRowNamesArray() {
        return (String[]) fRowNames.toArray(new String[fRowNames.size()]);
    }

    public List getColumnNames() {
        return Collections.unmodifiableList(fColNames);
    }

    public int getColumnIndex(String colName) {
        return fColNames.indexOf(colName);
    }

    public String[] getColumnNamesArray() {
        return (String[]) fColNames.toArray(new String[fColNames.size()]);
    }

    public String getColumnName(int coln) {
        return (String) fColNames.get(coln);
    }

    public int getNumRow() {
        return fMatrix.getNumRow();
    }

    public int getNumCol() {
        return fMatrix.getNumCol();
    }

    /**
     * @return An immutable view of the Matrix
     */
    public Matrix getMatrix() {

        fMatrix.setImmutable();

        return fMatrix;
    }

    /**
     * Data is NOT shared
     *
     * @param name
     * @return
     */
    public Dataset toDataset(final String name) {
        // imp to not share the data
        return new DefaultDataset(name, this.fMatrix, this.fRowNames, this.fColNames, false, null);

    }

    public Dataset toDataset(boolean shareAll) {
        // imp to not share the data
        return new DefaultDataset(getName(), this.fMatrix, this.fRowNames, this.fColNames, shareAll, null);
    }

    /**
     * this df is NOT touched
     * data IS duplicated
     *
     * @param newdfname
     * @param newColName
     * @param newColValues
     * @return
     */
    public Dataframe addColumn(String newdfname, String newColName, final Vector newColValues) {

        Matrix matrix = new Matrix(this.getNumRow(), this.getNumCol() + 1);
        for (int r = 0; r < this.getNumRow(); r++) {
            int c = 0;
            for (; c < this.getNumCol(); c++) {
                matrix.setElement(r, c, this.getElement(r, c));
            }

            matrix.setElement(r, c, newColValues.getElement(r));

        }

        List colNames = new ArrayList(this.getNumCol() + 1);
        for (int c = 0; c < this.getNumCol(); c++) {
            colNames.add(this.getColumnName(c));
        }
        colNames.add(newColName);

        return new Dataframe(newdfname, matrix, this.getRowNames(), colNames, true, false, true);

    }

    public String getQuickInfo() {
        StringBuffer buf = new StringBuffer().append(getNumRow()).append('x').append(getNumCol());
        return buf.toString();
    }

}    // End Dataframe

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
