/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.objects;


import edu.mit.broad.genome.Errors;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.parsers.GctParser;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.utils.ArrayUtils;
import edu.mit.broad.genome.utils.CollectionUtils;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import gnu.trove.TIntArrayList;
import gnu.trove.TObjectIntHashMap;

import java.io.File;
import java.util.*;


/**
 * Basic implementation of a Dataset.
 * Several forms of constructors to create Datasets.
 * <p/>
 * Data is always duplicated and never shared. Dataset is immutable after creation.
 * This is the chief class that provides access to Dataset objects. Its used everywhere!
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DefaultDataset extends AbstractDataset {

    private Matrix fMatrix;

    // these are Lists and not Sets as we want to be able to maintain the order
    // we however check for duplicates in the constructors

    private List fRowNames;
    private List fColNames;
    private GeneSet fColNamesGeneSet; // lazilly filled
    private GeneSet rowNamesGeneSet; // lazilly filled

    private Annot fAnn;

    private File dsFile;

    private String fQuickInfo;

    // Not always available
    private APMMatrix fAPMMatrix;

    public APMMatrix getAPMMatrix() {
        return fAPMMatrix;
    }

    public boolean hasAPMCalls() {
        return fAPMMatrix != null;
    }

    // @todo think
    public Dataset cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    public Dataset cloneShallow(final APMMatrix apm) {
        if (fAPMMatrix != null) {
            log.warn("Overwriting extant APM with: " + apm);
        }
        this.fAPMMatrix = apm;
        return this;
    }

    public Dataset cloneShallow(final String[] newColNames) {
        return cloneShallow(getName(), newColNames);
    }

    public Dataset cloneShallow(final Map oldColNameNewColNameMap) {
        final Errors errors = new Errors("Missing col names");
        final String[] newColNames = new String[getNumCol()];
        for (int c = 0; c < getNumCol(); c++) {
            Object obj = oldColNameNewColNameMap.get(getColumnName(c));
            if (obj == null) {
                errors.add(">" + getColumnName(c) + "<");
            } else {
                newColNames[c] = obj.toString();
            }
        }

        errors.barfIfNotEmptyRuntime();

        return cloneShallow(getName(), newColNames);
    }

    public Dataset cloneShallow(final String newName, final String[] newColNames) {
        if (newColNames == null) {
            throw new IllegalArgumentException("Param newColNames cannot be null");
        }
        this.fColNames = toList(newColNames);
        super.setName(newName);
        return this;
    }

    public Dataset cloneShallow(final Annot annot) {
        this.fAnn = annot;
        _setQuickInfo();
        return this;
    }

    /**
     * subclasses MUST call init after using this form of the constructor.
     */
    protected DefaultDataset() {
    }

    /**
     * Class Constructor.
     * Data is NOT shared. It is copied over into new object.
     */
    public DefaultDataset(final String name, final Dataset ds) {

        if (ds == null) {
            throw new IllegalArgumentException("Dataset param cant be null");
        }

        //Print.out(ds);
        //log.debug("numrows: " + ds.getRowNames().size() + " " + ds.getMatrix().getNumRow());
        //log.debug("numcols: " + ds.getColNames().size() + " " + ds.getMatrix().getNumCol());

        Matrix matrix = ds.getMatrix().cloneDeep();

        APMMatrix apm = null;
        if (ds instanceof DefaultDataset) {
            APMMatrix curr = ((DefaultDataset) ds).getAPMMatrix();
            if (curr != null) {
                apm = new APMMatrix(curr, false);
            }
        }

        // Hmmm row/col names actually not copied??
        init(name, matrix, ds.getRowNames(), ds.getColumnNames(), ds.getAnnot(), apm);
    }


    // @rls must all be od f the same size
    public DefaultDataset(final String name, final RankedList[] rls) {

        final List rowNames = new ArrayList(rls[0].getRankedNames());
        final List colNames = new ArrayList();
        int numRows = rls[0].getSize();
        Matrix m = new Matrix(numRows, rls.length);
        for (int c = 0; c < rls.length; c++) {
            colNames.add(rls[c].getName());
            for (int r = 0; r < numRows; r++) {
                m.setElement(r, c, rls[c].getScore(rowNames.get(r).toString()));
            }
        }

        init(name, m, rowNames, colNames, null);
    }


    public DefaultDataset(final String name, final LabelledVector[] lvs) {

        final List rowNames = new ArrayList(lvs[0].getLabels());
        final List colNames = new ArrayList();
        int numRows = lvs[0].getSize();
        Matrix m = new Matrix(numRows, lvs.length);
        for (int c = 0; c < lvs.length; c++) {
            colNames.add(lvs[c].getName());
            for (int r = 0; r < numRows; r++) {
                m.setElement(r, c, lvs[c].getScore(rowNames.get(r).toString()));
            }
        }

        init(name, m, rowNames, colNames, null);
    }

    // Note same row names are NOT explicitly enforced
    public DefaultDataset(final String name,
                          final Dataset[] dss,
                          final boolean addAsRows) {

        if (dss == null) {
            throw new IllegalArgumentException("Param dss cant be null");
        }

        if (addAsRows) {

            /*
           int numRows = 0;
            for (int i = 0; i < dss.length; i++) {
                numRows += dss[i].getNumRow();
            }

            final String[] colNames = dss[0].getColumnNamesArray();
            final Matrix m = new Matrix(rowNames.size(), colNames.length);

            int rowNum = 0;
            for (int i = 0; i < dss.length; i++) {
                for (int r = 0; r < dss[i].getNumRow(); r++) {
                    for (int c = 0; c < colNames.length; c++) {
                        m.setElement(rowNum, c, dss[i].getElement(r, c));
                    }
                    rowNum++;
                }
            }

            if (rowNum != rowNames.size()) {
                throw new IllegalStateException("rowNum: " + rowNum + " rowNames: " + rowNames.size());
            }

            init(name, m, rowNames, Arrays.asList(colNames), null);
            */

            throw new NotImplementedException();

        } else {

            Matrix[] mss = new Matrix[dss.length];
            List colNames = new ArrayList();
            for (int i = 0; i < dss.length; i++) {
                mss[i] = dss[i].getMatrix();
                for (int c = 0; c < dss[i].getNumCol(); c++) {
                    colNames.add(dss[i].getColumnName(c) + "_ds" + i); // so that its unique
                }
            }

            Matrix combo = new Matrix(mss, addAsRows); // add as columns

            // Hmmm row/col names actually not copied??
            init(name, combo, dss[0].getRowNames(), colNames, null);
        }
    }

    /**
     * Data is NOT shared.
     */
    public DefaultDataset(final String name, final ScoredDataset sds) {

        if (sds == null) {
            throw new IllegalArgumentException("Dataset param cant be null");
        }

        final Matrix matrix = sds.getMatrix().cloneDeep();
        final List drownames = new ArrayList(sds.getRankedNames());
        final List dcolnames = new ArrayList(sds.getColumnNames());
        init(name, matrix, drownames, dcolnames, sds.getAnnot());
    }


    /**
     * @param name
     * @param matrix
     * @param rowNames
     * @param colNames
     * @param shareAll
     */
    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final List rowNames,
                          final List colNames,
                          final boolean shareAll,
                          final Annot annOpt) {
        this(name, matrix, rowNames, colNames, shareAll, shareAll, shareAll, annOpt);
    }

    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final List rowNames,
                          final List colNames,
                          final boolean shareAll,
                          final Annot annOpt,
                          final APMMatrix apm) {
        this(name, matrix, rowNames, colNames, shareAll, shareAll, shareAll, annOpt, apm);
        //log.debug("SETTING APM TO: " + apm);
    }

    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final List rowNames,
                          final boolean shareMatrix,
                          final boolean shareRowNames,
                          final Annot annOpt) {
        this(name, matrix, rowNames,
                CollectionUtils.createSeriesStrings(0, matrix.getNumCol()),
                shareMatrix,
                shareRowNames,
                true,
                annOpt);
    }

    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final String[] rowNames,
                          final String[] colNames,
                          final boolean shareMatrix) {
        this(name, matrix, rowNames, colNames, shareMatrix, null);
    }

    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final String[] rowNames,
                          final String[] colNames,
                          final boolean shareMatrix,
                          final Annot annOpt) {
        this(name, matrix, toList(rowNames), toList(colNames), shareMatrix, true, true, annOpt);
    }

    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final String[] rowNames,
                          final String[] colNames,
                          final boolean shareMatrix,
                          final Annot annOpt,
                          final APMMatrix apm
    ) {
        this(name, matrix, toList(rowNames), toList(colNames), shareMatrix, true, true, annOpt, apm);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param matrix
     */
    public DefaultDataset(final String name, final Matrix matrix, final Annot annOpt) {

        List rowNames = new ArrayList(matrix.getNumRow());
        List colNames = new ArrayList(matrix.getNumCol());

        for (int r = 0; r < matrix.getNumRow(); r++) {
            rowNames.add("row_" + r);
        }

        for (int c = 0; c < matrix.getNumCol(); c++) {
            colNames.add("col_" + c);
        }

        init(name, matrix, rowNames, colNames, annOpt);
    }

    /* Dont as want to make the setting of ann explicit
    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final List rowNames,
                          final List colNames,
                          final boolean shareMatrix,
                          final boolean shareRowNames,
                          final boolean shareColNames) {
        this(name, matrix, rowNames, colNames, shareMatrix, shareRowNames, shareColNames, null);
    }
    */


    /**
     * LAZY initialization method
     * The ros and col names are read right away but the matrix (bulk of the data) is parsed only when needed
     *
     * @param dsFile
     * @param annOpt
     */
    public DefaultDataset(final File dsFile,
                          final Annot annOpt,
                          final APMMatrix apm_opt) throws Exception {

        final List[] lists = new GctParser().parseRowAndColNamesOnly(dsFile);
        this.init_rows_and_cols(NamingConventions.removeExtension(dsFile.getName()), lists[0], lists[1], annOpt, apm_opt);
        this.dsFile = dsFile;
    }


    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final List rowNames,
                          final List colNames,
                          final boolean shareMatrix,
                          final boolean shareRowNames,
                          final boolean shareColNames,
                          final Annot annOpt) {
        this(name, matrix, rowNames, colNames, shareMatrix, shareRowNames, shareColNames, annOpt, null);
    }

    /**
     * Class Constructor.
     * Specified data is copied over - data is shared only on demand
     * if share is true then Data is NOT cloned, its shared
     *
     * @todo does NOT yet work for a subset of rows/cols <-- ??? not sure what this means
     * see DatasetGen extract method for that impl.
     */
    public DefaultDataset(final String name,
                          final Matrix matrix,
                          final List rowNames,
                          final List colNames,
                          final boolean shareMatrix,
                          final boolean shareRowNames,
                          final boolean shareColNames,
                          final Annot annOpt,
                          final APMMatrix apm) {

        if (matrix == null) {
            throw new IllegalArgumentException("Param matrix cant be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Param rowNames cant be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("Param colNames cant be null");
        }

        Matrix dmatrix;
        List drowNames;
        List dcolNames;

        if (shareMatrix) {
            dmatrix = matrix;
        } else {
            dmatrix = matrix.cloneDeep();
        }

        if (shareRowNames) {
            drowNames = rowNames;
        } else {
            drowNames = new ArrayList(rowNames);
        }

        if (shareColNames) {
            dcolNames = colNames;
        } else {
            dcolNames = new ArrayList(colNames);
        }

        init(name, dmatrix, drowNames, dcolNames, annOpt, apm);

    }

    /**
     * Class Constructor.
     * Data is NOT shared. It is copied over into new objects.
     * Note floats and names will be duplicated
     */
    public DefaultDataset(final String name,
                          final int[] rowIndices,
                          final int[] colIndices,
                          final Dataset ds) {
        _row_col_array_based_constructor(name, rowIndices, colIndices, ds);
    }

    private void _row_col_array_based_constructor(final String name,
                                                  final int[] rowIndices,
                                                  final int[] colIndices,
                                                  final Dataset ds) {

        if (ds == null) {
            throw new IllegalArgumentException("Dataset param cant be null");
        }

        if (rowIndices == null) {
            throw new IllegalArgumentException("rowIndices param cant be null");
        }

        if (colIndices == null) {
            throw new IllegalArgumentException("colIndices param cant be null");
        }

        List rownames = new ArrayList(rowIndices.length);
        List colnames = new ArrayList(colIndices.length);
        Matrix matrix = new Matrix(rowIndices.length, colIndices.length);

        for (int r = 0; r < rowIndices.length; r++) {
            for (int c = 0; c < colIndices.length; c++) {
                matrix.setElement(r, c, ds.getElement(rowIndices[r], colIndices[c]));
            }

            rownames.add(ds.getRowName(rowIndices[r]));
        }

        for (int i = 0; i < colIndices.length; i++) {
            colnames.add(ds.getColumnName(colIndices[i]));
        }

        init(name, matrix, rownames, colnames, ds.getAnnot());
    }

    /**
     * Data is NOT shared. It is copied over into new objects.
     * uses all columns
     */
    public DefaultDataset(final String name, final int[] rowIndices, final Dataset ds) {
        _row_array_based_constructor(name, rowIndices, ds);
    }

    private void _row_array_based_constructor(final String name, final int[] rowIndices, final Dataset ds) {
        _row_col_array_based_constructor(name, rowIndices, ArrayUtils.fill(0, ds.getNumCol() - 1, 1), ds);
    }

    /**
     * Class Constructor.
     * Data is NOT shared. It is copied over into new objects.
     * uses all columns and all rows between startrow and stoprow.
     * Inclusive of startrow and stoprow
     */
    public DefaultDataset(final String name, final int startRow, final int stopRow, final Dataset ds) {
        this(name, ArrayUtils.fill(startRow, stopRow, 1), ArrayUtils.fill(0, ds.getNumCol() - 1, 1), ds);
    }

    /**
     * Class Constructor.
     *
     * @param step
     * @param ds
     */
    public DefaultDataset(final String name, final int step, final Dataset ds) {

        TIntArrayList indices = new TIntArrayList();

        for (int i = 0; i < ds.getNumRow(); i += step) {

            indices.add(i);
        }

        _row_array_based_constructor(name, indices.toNativeArray(), ds);

    }

    protected void init(final String dsname,
                        final Matrix matrix,
                        final List rowNames,
                        final List colNames,
                        final Annot annOpt) {
        init(dsname, matrix, rowNames, colNames, annOpt, null);
    }

    /**
     * assumes that when i am called the data has already been duplicated
     * i.e callers within this method are responsible for passing me
     * already duplicated (or manipulated/subsetted) data.
     * <p/>
     */
    protected void init(final String dsName,
                        final Matrix matrix,
                        final List rowNames,
                        final List colNames,
                        final Annot annOpt,
                        final APMMatrix apm) {

        init_rows_and_cols(dsName, rowNames, colNames, annOpt, apm);
        initMatrix(matrix, rowNames, colNames);
    }

    // This is one half of the CORE init (see below)
    private void initMatrix(final Matrix matrix, final List rowNames, final List colNames) {

        if (matrix == null) {
            throw new IllegalArgumentException("Matrix cannot be null");
        }


        if (matrix.getNumRow() != rowNames.size()) {
            throw new IllegalArgumentException("Matrix nrows: " + matrix.getNumRow() + " and rownames: "
                    + rowNames.size() + " do not match in size");
        }

        if (matrix.getNumCol() != colNames.size()) {
            throw new IllegalArgumentException("Matrix ncols: " + matrix.getNumCol() + " and colnames: "
                    + colNames.size() + " do not match in size");
        }

        this.fMatrix = matrix;
        this.fMatrix.setImmutable();    // IMP notice.
    }

    // This is one half of the CORE init (see above)
    protected void init_rows_and_cols(final String dsName,
                                      final List rowNames,
                                      final List colNames,
                                      final Annot annOpt,
                                      final APMMatrix apm) {

        super.initialize(dsName);

        if (rowNames == null) {
            throw new IllegalArgumentException("rowNames cannot be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("colNames cannot be null");
        }

        this.fRowNames = rowNames;
        this.fRowNames = Collections.unmodifiableList(fRowNames);
        this.fColNames = colNames;
        this.fColNames = Collections.unmodifiableList(fColNames);

        // annot sanity checks
        if (annOpt != null) {
            FeatureAnnot fa = annOpt.getFeatureAnnot();

            //  fannot can be oversize
            if (fa.getNumFeatures() < this.fRowNames.size()) {
                throw new IllegalArgumentException("Annot features is less than dataset rowNames: " + fa.getNumFeatures() + " " + fRowNames.size());
            }

            if (fa.hasNativeDescriptions() && fRowNames.size() > 1) {
                fa.getNativeDesc(fRowNames.get(0).toString()); // just a check
            }
        }
        this.fAnn = annOpt;

        // data integrity checks: no column names must be duplicated
        // ditto for rows but we avoid doing that as its sloooow
        ensureAllUniqueValues(fColNames);

        this.fAPMMatrix = apm;
    }


    protected static void ensureAllUniqueValues(List list) {
        Set set = new HashSet();
        for (int i = 0; i < list.size(); i++) {
            Object obj = list.get(i);
            if (set.contains(obj)) {
                throw new IllegalArgumentException("Duplicate COL names are NOT allowed in Datasets. The offending entry was: " + obj + " at pos: " + i);
            }
            set.add(obj);
        }

        set.clear();
    }

    //Made lazilly
    public Annot getAnnot() {
        if (fAnn == null) {
            //log.debug("Annotation is null -- will use a default ann " + getName());
            ColorMap.Rows cm = null;
            final FeatureAnnot fann = new FeatureAnnotImpl(getName(), fRowNames, null, cm);
            final SampleAnnot sann = new SampleAnnotImpl(getName(), getColumnNames(), null);
            this.fAnn = new AnnotImpl(fann, sann);
        }

        return fAnn;
    }


    public String getRowName(int rown) {
        return (String) fRowNames.get(rown);
    }

    public List getRowNames() {
        return Collections.unmodifiableList(fRowNames);
    }

    // @todo check impact: trove added Jan 2006
    private TObjectIntHashMap fRowIndexNameHashMap;

    public int getRowIndex(String rowName) {

        if (rowName == null) {
            throw new IllegalArgumentException("rowName cannot be null: " + rowName);
        }

        if (fRowIndexNameHashMap == null) {
            fRowIndexNameHashMap = new TObjectIntHashMap();
            for (int r = 0; r < fRowNames.size(); r++) {
                fRowIndexNameHashMap.put(fRowNames.get(r), r);
            }
        }

        int index = fRowIndexNameHashMap.get(rowName);

        // Theres som confusion over whether missing returns 0 or -1 from Trove
        if (index == 0) {
            if (fRowIndexNameHashMap.containsKey(rowName) == false) {
                return index = -1;
            }
        }

        return index;

        //return fRowNames.indexOf(rowName);
    }

    public String[] getRowNamesArray() {
        return (String[]) fRowNames.toArray(new String[fRowNames.size()]);
    }

    public List getColumnNames() {
        return Collections.unmodifiableList(fColNames);
    }

    public List getColumnNames(final Template prefixTemplateClasses) {
        List colNames = new ArrayList(fColNames);
        for (int c = 0; c < colNames.size(); c++) {
            colNames.set(c, prefixTemplateClasses.getClassByProfilePos(c).getName() + "_" + colNames.get(c));
        }
        return colNames;
    }

    public GeneSet getColumnNamesGeneSet() {
        if (fColNamesGeneSet == null) {
            fColNamesGeneSet = new FSet(getName(), new HashSet(fColNames));
        }
        return fColNamesGeneSet;
    }

    public GeneSet getRowNamesGeneSet() {
        if (rowNamesGeneSet == null) {
            rowNamesGeneSet = new FSet(getName(), new HashSet(fRowNames));
        }
        return rowNamesGeneSet;
    }

    public int getColumnIndex(String colName) {
        return fColNames.indexOf(colName);
    }

    public String[] getColumnNamesArray() {
        return (String[]) fColNames.toArray(new String[fColNames.size()]);
    }

    public String getColumnName(int coln) {
        return (String) fColNames.get(coln);
    }

    public int getNumRow() {
        return fRowNames.size();
    }

    public int getNumCol() {
        return fColNames.size();
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Vector getRow(String rowName) {
        return getRow(getRowIndex(rowName));
    }

    public Vector[] getRows(GeneSet gset) {
        Vector[] vss = new Vector[gset.getNumMembers()];
        for (int i = 0; i < gset.getNumMembers(); i++) {
            //System.out.println(">> " + gset.getMember(i));
            vss[i] = getRow(gset.getMember(i));
        }

        return vss;
    }

    public Vector getColumn(String colName) {
        return getColumn(getColumnIndex(colName));
    }

    public Vector[] getColumns(final String[] colNames) {
        Vector[] vss = new Vector[colNames.length];
        for (int i = 0; i < colNames.length; i++) {
            vss[i] = getColumn(colNames[i]);
        }
        return vss;
    }

    public Vector getColumn(final int coln, final GeneSet forTheseGenes) {
        Vector v = new Vector(forTheseGenes.getNumMembers());
        for (int i = 0; i < forTheseGenes.getNumMembers(); i++) {
            v.setElement(i, getElement(forTheseGenes.getMember(i), coln));
        }

        return v;
    }

    public Vector getColumn(final String colName, final GeneSet forTheseGenes) {
        return getColumn(getColumnIndex(colName), forTheseGenes);
    }

    public LabelledVector getColumnLV(String colName) {
        return getColumnLV(getColumnIndex(colName));
    }

    public LabelledVector getColumnLV(int coln) {
        return new LabelledVector(getColumnName(coln), getRowNamesArray(), _matrix().getColumnV(coln));
    }

    public float getElement(int rown, int coln) {
        return _matrix().getElement(rown, coln);
    }

    public Vector getRow(int rown) {
        return _matrix().getRowV(rown);
    }

    public Vector getColumn(int coln) {
        return _matrix().getColumnV(coln);
    }

    public RankedList getColumnRL(String colName, SortMode sort, Order order) {
        return getColumnLV(colName).sort(sort, order);
    }

    public RankedList getColumnRL(int coln, SortMode sort, Order order) {
        return getColumnLV(coln).sort(sort, order);
    }

    public float getElement(final int row, final String colName) {
        try {
            return getElement(row, getColumnIndex(colName));
        } catch (ArrayIndexOutOfBoundsException aei) {
            System.out.println("Could not find colName: " + colName);
            throw aei;
        }
    }

    public float getElement(final String rowName, final int col) {

        int rowIndex = getRowIndex(rowName);
        if (rowIndex == -1) {
            throw new IllegalArgumentException("Could not find rowName: " + rowName);
        }
        return getElement(rowIndex, col);
    }

    public float getElement(final String rowName, final String colName) {
        int row_index = getRowIndex(rowName);
        int col_index = getColumnIndex(colName);
        if (row_index == -1) {
            throw new IllegalArgumentException("No such row: >" + rowName + "<");
        }
        if (col_index == -1) {
            throw new IllegalArgumentException("No such col: >" + colName + "<");
        }

        return getElement(row_index, col_index);
    }

    public int getDim() {
        return _matrix().getDim();
    }

    /**
     * @return An immutable view of the Matrix
     */
    public Matrix getMatrix() {
        _matrix().setImmutable();
        return _matrix();
    }

    private Matrix _matrix() {
        if (fMatrix == null) {

            if (dsFile == null || !dsFile.exists()) {
                throw new IllegalStateException("ds file for lazy matrix loading is null or missing: " + dsFile);
            }

            try {
                log.debug("LAZY loading dataset from: " + dsFile.getPath());
                // @note the annotation, row names , col names etc are NOT used
                final Dataset ds = ParserFactory.readDataset(dsFile, true, true, false);
                this.initMatrix(((DefaultDataset) ds).fMatrix, fRowNames, fColNames);

            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }

        return fMatrix;
    }

    public String getQuickInfo() {
        if (fQuickInfo == null) {
            _setQuickInfo();
        }

        return fQuickInfo;
    }

    private void _setQuickInfo() {

        StringBuffer buf = new StringBuffer().append(getNumRow()).append('x').append(getNumCol());
        if (this.getAnnot() != null && getAnnot().getFeatureAnnot() != null && getAnnot().getSampleAnnot_global() != null) {
            FeatureAnnot fa = getAnnot().getFeatureAnnot();
            SampleAnnot sa = getAnnot().getSampleAnnot_global();

            buf.append(" (ann: ");
            if (fa != null) {
                buf.append(fa.getNumFeatures());
            } else {
                buf.append("na");
            }
            buf.append(",");

            if (sa != null) {
                buf.append(sa.getNumSamples());
            } else {
                buf.append("na");
            }

            buf.append(",");

            if (getAnnot().getChip() != null) {
                buf.append(getAnnot().getChip().getName());
            } else {
                buf.append("chip na");
            }

            buf.append(")");

        }
        fQuickInfo = buf.toString();

    }

    private static List toList(final String[] ss) {
        final List list = new ArrayList(ss.length);
        for (int i = 0; i < ss.length; i++) {
            list.add(ss[i]);
        }

        return list;
    }

}    // End DefaultDataset

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/