/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;


import edu.mit.broad.genome.parsers.AuxUtils;

import java.util.*;

/**
 * Essentially a collection of (probably related) GeneSet
 * In additin has colors and icons and names
 * <p/>
 * Lightweigth container for a bucng of gsets -- gset data is not duplicated
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DefaultGeneSetMatrix extends AbstractGeneSetMatrix {

    /**
     * Class Constructor.
     * gsets specified are used directly without cloning any data
     * <p/>
     * gset names must be unique
     */
    public DefaultGeneSetMatrix(final String name, final GeneSet[] gsets) {
        initMatrix(name, gsets);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gsets
     * @param removeAuxStuff
     */
    public DefaultGeneSetMatrix(final String name, final GeneSet[] gsets, final boolean removeAuxStuff) {
        if (removeAuxStuff) {
            GeneSet[] cgsets = new GeneSet[gsets.length];
            for (int i = 0; i < gsets.length; i++) {
                cgsets[i] = gsets[i].cloneShallow(AuxUtils.getAuxNameOnlyNoHash(gsets[i].getName()));
            }
            initMatrix(name, cgsets);
        } else {
            initMatrix(name, gsets);
        }
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gsets
     * @param minSize
     * @param maxSize
     */
    public DefaultGeneSetMatrix(final String name,
                                final GeneSet[] gsets,
                                final int minSize,
                                final int maxSize) {
        List list = new ArrayList();

        for (int i = 0; i < gsets.length; i++) {
            GeneSet gset = gsets[i];
            if (gset.getNumMembers() >= minSize && gset.getNumMembers() <= maxSize) {
                list.add(gset);
            }
        }

        initMatrix(name, (GeneSet[]) list.toArray(new GeneSet[list.size()]));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gms
     * @param removeNativeGmNames
     */
    public DefaultGeneSetMatrix(final String name, final GeneSetMatrix[] gms, final boolean removeNativeGmNames) {
        List gsetsList = new ArrayList();

        // combine
        for (int g = 0; g < gms.length; g++) {
            if (removeNativeGmNames) {
                GeneSet[] gsets = gms[g].getGeneSets();
                // get rid of the # in the gset names
                for (int i = 0; i < gsets.length; i++) {
                    gsetsList.add(gsets[i].cloneShallow(AuxUtils.getAuxNameOnlyNoHash(gsets[i].getName())));
                }
            } else {
                gsetsList.addAll(gms[g].getGeneSetsL());
            }
        }

        initMatrix(name, (GeneSet[]) gsetsList.toArray(new GeneSet[gsetsList.size()]));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gms
     * @param minSize
     * @param maxSize
     * @param removeNativeGmNames
     */
    public DefaultGeneSetMatrix(final String name, final GeneSetMatrix[] gms, final int minSize,
                                final int maxSize, final boolean removeNativeGmNames) {

        List gsets = new ArrayList();

        // combine
        for (int i = 0; i < gms.length; i++) {
            gsets.addAll(gms[i].getGeneSetsL(minSize, maxSize));
        }

        // strip off the source name if requested
        if (removeNativeGmNames) {
            for (int i = 0; i < gsets.size(); i++) {
                FSet fset = (FSet) gsets.get(i);
                String justAuxName = AuxUtils.getAuxNameOnlyNoHash(fset.getName());
                gsets.set(i, fset.cloneShallow(justAuxName));
            }
        }

        initMatrix(name, (GeneSet[]) gsets.toArray(new GeneSet[gsets.size()]));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gsets
     */
    public DefaultGeneSetMatrix(final String name, final List gsets) {
        initMatrix(name, (GeneSet[]) gsets.toArray(new GeneSet[gsets.size()]));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gsets
     * @param rll
     */
    public DefaultGeneSetMatrix(final String name, final GeneSet[] gsets, final RankedList[] rll) {
        initHere_R(name, gsets, rll);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gsets
     * @param rl
     */
    public DefaultGeneSetMatrix(final String name, final GeneSet[] gsets, final RankedList rl) {
        initHere_R(name, gsets, new RankedList[]{rl});
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gm
     * @param rl
     */
    public DefaultGeneSetMatrix(final String name, final GeneSetMatrix gm, final RankedList rl) {
        initHere_R(name, gm.getGeneSets(), new RankedList[]{rl});
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gm
     * @param rll
     */
    public DefaultGeneSetMatrix(final String name, final GeneSetMatrix gm, final RankedList[] rll) {
        this(name, gm.getGeneSets(), rll);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gsets
     * @param qualifyDss
     */
    public DefaultGeneSetMatrix(final String name, final GeneSet[] gsets, final Dataset[] qualifyDss) {
        initHere_D(name, gsets, qualifyDss);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gsets
     * @param qualifyDs
     */
    public DefaultGeneSetMatrix(final String name, final GeneSet[] gsets, final Dataset qualifyDs) {
        initHere_D(name, gsets, new Dataset[]{qualifyDs});
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gm
     * @param qualifyDss
     */
    public DefaultGeneSetMatrix(final String name, final GeneSetMatrix gm, final Dataset[] qualifyDss) {
        this(name, gm.getGeneSets(), qualifyDss);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param gms
     * @param qualifyDss
     */
    public DefaultGeneSetMatrix(final String name, final GeneSetMatrix[] gms, final Dataset[] qualifyDss) {
        List gsets = new ArrayList();

        // combine
        for (int i = 0; i < gms.length; i++) {
            gsets.addAll(gms[i].getGeneSetsL());
        }

        initHere_D(name, ((GeneSet[]) gsets.toArray(new GeneSet[gsets.size()])), qualifyDss);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param name_tag_pair
     * @return
     */
    public static DefaultGeneSetMatrix createGeneSetMatrix(final String name, final List name_tag_pair) {

        Map map = new HashMap();

        for (int i = 0; i < name_tag_pair.size(); i++) {
            StringTokenizer tok = new StringTokenizer(name_tag_pair.get(i).toString(), "\t");
            if (tok.countTokens() != 2) {
                throw new IllegalArgumentException("Bad format at line: " + (i + 1) + " expecting 2 tokens, got: " + tok.countTokens() + " " + name_tag_pair.get(i).toString());
            }
            String v = tok.nextToken(); // @note order
            String k = tok.nextToken();

            if (map.containsKey(k) == false) {
                map.put(k, new HashSet());
            }

            Set tags = (HashSet) map.get(k);
            tags.add(v);
        }

        GeneSet[] gsets = new GeneSet[map.size()];
        Iterator it = map.keySet().iterator();
        int cnt = 0;
        while (it.hasNext()) {
            String gsetname = it.next().toString();
            Set members = (HashSet) map.get(gsetname);
            gsets[cnt++] = new FSet(gsetname, members);
        }

        return new DefaultGeneSetMatrix(name, gsets);
    }

    // convenience init routine
    private void initHere_D(final String name, final GeneSet[] gsets, final Dataset[] qualifyDss) {
        GeneSet[] qualgsets = new GeneSet[gsets.length];

        for (int d = 0; d < qualifyDss.length; d++) {
            for (int g = 0; g < gsets.length; g++) {
                if (d == 0) {
                    qualgsets[g] = gsets[g].cloneDeep();
                }
                qualgsets[g] = qualgsets[g].cloneDeep(qualifyDss[d]);
            }
        }

        initMatrix(name, qualgsets);
    }

    public GeneSetMatrix cloneShallow(String newName) {
        this.setName(newName);
        return this;
    }

    // convenience init routine
    private void initHere_R(final String name, final GeneSet[] gsets, final RankedList[] qualifyRls) {
        GeneSet[] qualgsets = new GeneSet[gsets.length];

        for (int d = 0; d < qualifyRls.length; d++) {
            for (int g = 0; g < gsets.length; g++) {
                if (d == 0) {
                    qualgsets[g] = gsets[g].cloneDeep();
                }
                qualgsets[g] = qualgsets[g].cloneDeep(qualifyRls[d]);
            }
        }

        initMatrix(name, qualgsets);
    }

}    // End DefaultGeneSetMatrix

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
