package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.math.AddressedVector;
import edu.mit.broad.genome.math.ScoreMode;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.strucs.DefaultMetricWeightStruc;
import gnu.trove.TFloatArrayList;
import gnu.trove.TObjectIntHashMap;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Aug 12, 2005
 *         Time: 11:31:04 AM
 */
public class DefaultRankedList extends AbstractObject implements RankedList {

    private List fRankedNames;

    private Vector fRankedScores;


    /**
     * Class constructor
     *
     * @param name
     * @param rankedNames
     * @param rankedScores
     */
    public DefaultRankedList(final String name, final String[] rankedNames, final float[] rankedScores) {
        init(name, rankedNames, new Vector(rankedScores));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param rankedNames
     * @param rankedScores
     */
    public DefaultRankedList(final String name, final String[] rankedNames, final TFloatArrayList rankedScores) {
        init(name, rankedNames, new Vector(rankedScores));
    }

    public DefaultRankedList(final String name, final List rankedNames, final TFloatArrayList rankedScores) {
        init(name, rankedNames, new Vector(rankedScores));
    }

    public DefaultRankedList(final List rankedNames,
                             final Vector rankedScores,
                             final boolean shareNames,
                             final boolean shareScores) {

        this(null, rankedNames, rankedScores, shareNames, shareScores);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param rankedNames
     * @param rankedScores
     * @param shareNames
     * @param shareScores
     */
    public DefaultRankedList(final String name,
                             final List rankedNames,
                             final Vector rankedScores,
                             final boolean shareNames,
                             final boolean shareScores) {

        final Vector scores = new Vector(rankedScores, shareScores);

        List names;
        if (shareNames) {
            names = rankedNames;
        } else {
            names = new ArrayList(rankedNames);
        }

        this.init(name, names, scores);
    }

    public DefaultRankedList(final String name,
                             final String[] rankedNames,
                             final Vector rankedScores,
                             final boolean shareScores) {

        final Vector scores = new Vector(rankedScores, shareScores);

        final List names = new ArrayList();
        for (int i = 0; i < rankedNames.length; i++) {
            names.add(rankedNames[i]);
        }

        this.init(name, names, scores);
    }

    public DefaultRankedList(final AddressedVector rankedIv, final Dataset ds) {
        this(rankedIv, ds, true);
    }

    public DefaultRankedList(final AddressedVector rankedIv, final Dataset ds, boolean duplicateVector) {

        if (rankedIv == null) {
            throw new NullPointerException("Param iv cannot be null");
        }

        if (ds == null) {
            throw new NullPointerException("Param Dataset cannot be null");
        }

        if (rankedIv.getSize() != ds.getNumRow()) {
            throw new MismatchedSizeException("AddressedVector", rankedIv.getSize(),
                    "Dataset NumRows", ds.getNumRow());
        }

        Vector scores = new Vector(rankedIv.getScoresArray(), duplicateVector);
        List labels = new ArrayList(scores.getSize());
        for (int r = 0; r < scores.getSize(); r++) {
            int posinds = rankedIv.getAddress(r);
            labels.add(ds.getRowName(posinds));
        }

        init(null, labels, scores);
    }

    public RankedList cloneShallowRL(final String newName) {
        super.setName(newName);
        return this;
    }

    // common (pre)initialization routine
    // see below too
    // the vector must already be sorted as desired!!
    private void init(final String name, final String[] rankedNames, final Vector rankedScores) {

        if (rankedScores == null) {
            throw new IllegalArgumentException("Parameter v cannot be null");
        }

        if (rankedNames == null) {
            throw new IllegalArgumentException("Param alabels cannot be null");
        }

        if (rankedScores.getSize() != rankedNames.length) {
            throw new MismatchedSizeException("Vector", rankedScores.getSize(), "Labels", +rankedNames.length);
        }

        final List labels = new ArrayList(rankedNames.length);
        for (int i = 0; i < rankedNames.length; i++) {
            labels.add(rankedNames[i]);
        }

        init(name, labels, rankedScores);
    }

    // all duplication (if needed) and SORTING must be done BEFORE calling this method
    // the vector must already be sorted as desired!!
    private void init(final String name, final List rankedNames, final Vector rankedScores) {
        if (rankedScores == null) {
            throw new IllegalArgumentException("Param v cannot be null");
        }
        if (rankedNames == null) {
            throw new IllegalArgumentException("Param labels cannot be null");
        }

        if (rankedScores.getSize() != rankedNames.size()) {
            throw new MismatchedSizeException("Vector", rankedScores.getSize(), "Labels", +rankedNames.size());
        }

        // make sure no duplicates
        // @note this check is expensive so disabled for now

        if (name == null) {
            super.initialize(" " + getClass().hashCode() + System.currentTimeMillis());
        } else {
            super.initialize(name);
        }

        this.fRankedScores = rankedScores;
        this.fRankedNames = Collections.unmodifiableList(rankedNames);
        this.fRankedScores.setImmutable(); // @note imp
    }

    public String getQuickInfo() {
        return getSize() + " names";
    }

    public boolean isMember(final String name) {
        return fRankedNames.contains(name);
    }

    public String getRankName(final int rank) {
        return fRankedNames.get(rank).toString();
    }

    public List getRankedNames() {
        return Collections.unmodifiableList(fRankedNames);
    }

    public String[] getRankedNamesArray() {
        return (String[]) fRankedNames.toArray(new String[fRankedNames.size()]);
    }

    public String toString(final char delim) {
        StringBuffer buf = new StringBuffer();
        for (int r = 0; r < getSize(); r++) {
            buf.append(getRankName(r)).append('=').append(getScore(r)).append(delim);
        }

        return buf.toString();
    }

    public Vector getScoresV(final boolean clonedCopy) {
        return new Vector(fRankedScores, clonedCopy);
    }

    public float getValue(final String featureName,
                          final XMath.Const scoreorRank) {
        if (scoreorRank.isScore()) {
            return getScore(featureName);
        } else {
            return getRank(featureName);
        }
    }

    public float getScore(final int rank) {
        return fRankedScores.getElement(rank);
    }

    public int getRank(final String name) {
        int rank = _index(name);

        /* @note dont -- this behaviour should be like list's
        if (rank == -1) {
            throw new IllegalArgumentException("No such name: " + name);
        }
        */

        return rank;
    }

    public float getScore(final String name) {
        int index = _index(name);

        if (index == -1) {
            throw new IllegalArgumentException("No such name: " + name + " \navailable: " + fRankedNames + "\n but no: " + name);
        }

        return fRankedScores.getElement(index);
    }

    // dont need this as index is same as rank ??
    private int _index(final String rowName) {

        if (fRowNameSdsRowIndexMap == null) {
            cacheRowNameIndex();
        }

        int index = fRowNameSdsRowIndexMap.get(rowName);

        // IMP needed as returns 0 and not -1 on no hits!!
        if (index == 0) {
            if (!fRowNameSdsRowIndexMap.containsKey(rowName)) {
                return -1;
            } else {
                return 0;
            }
        }

        return index;
    }

    private TObjectIntHashMap fRowNameSdsRowIndexMap;

    private void cacheRowNameIndex() {
        if (fRowNameSdsRowIndexMap == null) {
            fRowNameSdsRowIndexMap = new TObjectIntHashMap();
            for (int r = 0; r < getSize(); r++) {
                fRowNameSdsRowIndexMap.put(fRankedNames.get(r), r);
            }
        }
    }

    // -------------------------------------------------------------------------------------------- //


    public float[] getScores(final GeneSet gset) {
        float[] scores = new float[gset.getNumMembers()];
        for (int i = 0; i < gset.getNumMembers(); i++) {
            scores[i] = getScore(gset.getMember(i));
        }

        return scores;
    }

    public int[] getRanks(final GeneSet gset) {
        return Helper.getRanks(gset, this);
    }

    public Vector getRanksV(final GeneSet gset) {
        return Helper.getRanksV(gset, this);
    }

    public RankedList extractRanked(final GeneSet gset) {
        return RankedList.Helper.extract(gset, this);
    }

    public RankedList extractRanked(final String[] names) {
        return RankedList.Helper.extract(names, this);
    }

    public RankedList extractRanked(final List names) {
        return RankedList.Helper.extract(names, this);
    }

    public GeneSet toGeneSet() {
        return new FSet(getName(), fRankedNames, false);
    }

    public String toString(final char delim, boolean newLine) {
        return RankedList.Helper.toString(delim, newLine, this);
    }

    public int getSize() {
        return fRankedScores.getSize();
    }

    public int getSize(final ScoreMode smode) {
        return fRankedScores.getSize(smode);
    }

    public List getNamesOfUpOrDnXRanks(int topOrBotX, boolean top) {
        return Helper.getLabelsOfUpOrDnXRanks(topOrBotX, top, this);
    }

    public String[] getNamesOfUpOrDnXRanks_array(int topOrBotX, boolean top) {
        return Helper.getLabelsOfUpOrDnXRanks_array(topOrBotX, top, this);
    }

    public GeneSet getNamesOfUpOrDnXRanks_gs(int topOrBotX, boolean top) {
        return Helper.getLabelsOfUpOrDnXRanks_gs(topOrBotX, top, this);
    }

    public GeneSetPair getNamesOfUpAndDnXRanks(int topAndBotX) {
        return Helper.getLabelsOfUpAndDnXRanks(topAndBotX, this);
    }

    public RankedList extractRanked(final int topAndBotX) {
        return Helper.extractRanked(topAndBotX, this);
    }

    public RankedList extractRanked(final int topOrBotX, final boolean top) {
        return Helper.extractRanked(topOrBotX, top, this);
    }

    public RankedList extractRanked(final ScoreMode smode) {
        return Helper.extractRanked(smode, this);
    }

    public RankedList extractRanked(float minRankScore, float maxRankScore) {
        return Helper.extract(minRankScore, maxRankScore, this);
    }

    public int getNumScores(float minRankScore, float maxRankScore) {
        return Helper.getNumScores(minRankScore, maxRankScore, this);
    }

    public GeneSet getScores_gs(float minRankScore, float maxRankScore) {
        return Helper.getScores_gs(minRankScore, maxRankScore, this);
    }

    private MetricWeightStruc mws;

    public MetricWeightStruc getMetricWeightStruc() {
        if (mws == null) {
            mws = new DefaultMetricWeightStruc(null, this);
        }

        return mws;
    }


} // End DefaultRankedList




