/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Jun 24, 2003
 *         Time: 10:06:58 PM
 */
class GeneSetMatrixHelper {

    /**
     * Privatized class constructor
     */
    private GeneSetMatrixHelper() {
    }

    static GeneSet[] getGeneSets(final GeneSetMatrix gm, final int minSizeInclusive, final int maxSizeInclusive) {
        List gsets = getGeneSetsL(gm, minSizeInclusive, maxSizeInclusive);
        return (GeneSet[]) gsets.toArray(new GeneSet[gsets.size()]);
    }

    static List getGeneSetsL(final GeneSetMatrix gm, final int minSizeInclusive, final int maxSizeInclusive) {
        List gsets = new ArrayList();
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            GeneSet gset = gm.getGeneSet(i);
            if (gset.getNumMembers() >= minSizeInclusive && gset.getNumMembers() <= maxSizeInclusive) {
                gsets.add(gset);
            }
        }
        return Collections.unmodifiableList(gsets);
    }

    /**
     * The number of members in the biggest GeneSet.
     *
     * @return
     */
    static int getMaxMemberCount(final List genesets) {
        int max = 0;
        for (int i = 0; i < genesets.size(); i++) {
            GeneSet gset = (GeneSet) genesets.get(i);
            if (max < gset.getNumMembers()) {
                max = gset.getNumMembers();
            }
        }

        return max;
    }

    /**
     * non-redundant list of names of features across
     * gsets in this GeneSetMatrix
     *
     * @return
     */
    static String[] getAllMemberNames(final List genesets) {
        Set names = getAllMemberNamesS(genesets);
        return (String[]) names.toArray(new String[names.size()]);
    }

    // does the real stuff
    static Set getAllMemberNamesS(final List genesets) {
        Set names = new HashSet();
        for (int i = 0; i < genesets.size(); i++) {
            GeneSet gset = (GeneSet) genesets.get(i);
            names.addAll(gset.getMembers());
        }

        return names;
    }

    static List getAllMemberNameOccurrencesL(final List genesets) {
        List names = new ArrayList();
        for (int i = 0; i < genesets.size(); i++) {
            GeneSet gset = (GeneSet) genesets.get(i);
            names.addAll(gset.getMembers());
        }

        return names;
    }

    static String[] getAllMemberNameOccurrences(final List genesets) {
        List names = new ArrayList();
        for (int i = 0; i < genesets.size(); i++) {
            GeneSet gset = (GeneSet) genesets.get(i);
            names.addAll(gset.getMembers());
        }

        return (String[]) names.toArray(new String[names.size()]);
    }

    /**
     * Names of all gsets -- guaranteed unique
     *
     * @return
     */
    static String[] getAllGeneSetNames(final GeneSetMatrix gm) {
        String[] names = new String[gm.getNumGeneSets()];
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            names[i] = gm.getGeneSet(i).getName();
        }

        return names;
    }

    static String[] getGeneSetMembershipNames(final String probeName, final GeneSetMatrix gm) {
        GeneSet[] gsets = getGeneSetMemberships(probeName, gm);
        String[] names = new String[gsets.length];
        for (int i = 0; i < gsets.length; i++) {
            names[i] = gsets[i].getName();
        }

        return names;
    }

    static BitSet getGeneSetMembershipBitSet(final String probeName, final GeneSetMatrix gm) {

        BitSet bs = new BitSet(gm.getNumGeneSets());
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            if (gm.getGeneSet(i).isMember(probeName)) {
                bs.set(i, true);
            }
        }

        return bs;
    }

    // rows aka bitsets are now GENES (not gsets) and columns are GENESETS and ON bits are when a gene is in the geneset
    static BitSetDataset getGeneSetMembershipBitSetDataset(final String name, final String[] probeNames, final GeneSetMatrix gm) {

        BitSet[] bss = new BitSet[probeNames.length];
        for (int i = 0; i < bss.length; i++) {
            bss[i] = getGeneSetMembershipBitSet(probeNames[i], gm);
            // sizes dont match - not sure how the bitset thing works??? @see BitSetDataset
            //System.out.println(">> " + bss[i].size() + " " + gm.getNumGeneSets());
        }

        return new BitSetDataset(name, bss, probeNames, gm.getGeneSetNames(true), true);
    }

    // for all members
    static BitSetDataset getGeneSetMembershipBitSetDataset(final GeneSetMatrix gm) {
        return getGeneSetMembershipBitSetDataset(gm.getName(), gm.getAllMemberNamesOnlyOnce(), gm);
    }

    static GeneSet[] getGeneSetMemberships(final String probeName, final GeneSetMatrix gm) {
        List gsets = new ArrayList();
        for (int i = 0; i < gm.getNumGeneSets(); i++) {
            if (gm.getGeneSet(i).isMember(probeName)) {
                gsets.add(gm.getGeneSet(i));
            }
        }

        return (GeneSet[]) gsets.toArray(new GeneSet[gsets.size()]);
    }

    static GeneSet[] getGeneSets(final Set gsetNames, final GeneSetMatrix gm) {
        GeneSet[] gsets = new GeneSet[gsetNames.size()];
        int i = 0;
        for (Iterator it = gsetNames.iterator(); it.hasNext();) {
            Object key = it.next();
            gsets[i++] = gm.getGeneSet(key.toString());
        }

        return gsets;
    }

    static GeneSetMatrix getGeneSetMatrix(final GeneSet gsetNames, final GeneSetMatrix gm) {

        if (gsetNames == null || gsetNames.getNumMembers() == 0) {
            throw new IllegalArgumentException("Param gsetNames cannot be null or empty " + gsetNames);
        }

        final GeneSet[] gsets = new GeneSet[gsetNames.getNumMembers()];
        for (int i = 0; i < gsetNames.getNumMembers(); i++) {
            gsets[i] = gm.getGeneSet(gsetNames.getMember(i));
        }

        return new DefaultGeneSetMatrix(gsetNames.getName(true), gsets);
    }

    static GeneSet[] getGeneSets(final String[] gsetNames, final GeneSetMatrix gm) {
        GeneSet[] gsets = new GeneSet[gsetNames.length];
        for (int i = 0; i < gsetNames.length; i++) {
            gsets[i] = gm.getGeneSet(gsetNames[i]);
        }

        return gsets;
    }

} // End GeneSetGeneSetMatrixHelper
