/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.Chip2ChipMapperSymbol;
import gnu.trove.TObjectIntHashMap;
import org.apache.log4j.Logger;

import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Apr 12, 2004
 *         Time: 8:31:59 PM
 */
public class GenesOfInterestImpl extends AbstractObject implements GenesOfInterest {

    private String fName;

    private GeneList[] fGeneLists;

    private Map fGeneListNameGeneListMap;

    private GeneSetMatrix fGeneSetMatrix; // made lazilly

    private ColorMap.Rows fColorMap; // made lazilly

    // key -> vate, value -> gl index
    private TObjectIntHashMap glNameIndexMap; // made lazilly

    private int fMaxNumGenes = -1;

    private static final Logger klog = XLogger.getLogger(GenesOfInterestImpl.class);

    /**
     * Class constructor
     *
     * @param name
     * @param recds
     */
    public GenesOfInterestImpl(final String name, final GeneList[] geneLists) {
        init(name, geneLists);
    }

    public String[] getMembershipNames(final String symbol) {
        List memberships = new ArrayList();
        for (int i = 0; i < getNumGeneLists(); i++) {
            if (getGeneList(i).isMember(symbol)) {
                memberships.add(getGeneList(i).getName());
            }
        }

        return (String[]) memberships.toArray(new String[memberships.size()]);
    }

    public ColorMap.Rows getAsColorMap() {

        if (fColorMap == null) {
            this.fColorMap = new MyColorMap();
        }

        return fColorMap;
    }

    public GeneSetMatrix getAsGeneSetMatrix() {
        if (fGeneSetMatrix == null) {
            GeneSet[] gsets = new GeneSet[getNumGeneLists()];
            for (int i = 0; i < gsets.length; i++) {
                gsets[i] = getGeneList(i).toGeneSet();
            }
            this.fGeneSetMatrix = new DefaultGeneSetMatrix(getName(), gsets);
        }

        return fGeneSetMatrix;
    }

    public GeneSet getAsOneGeneSet() {
        GeneSet gset = getAsGeneSetMatrix().getAllMemberNames_gset();
        log.debug("Total # of genes in GIN: " + gset.getNumMembers());
        return gset;
    }

    // @note assumes that GIN is always defined in terms of gene symbols
    public GeneSet getAsOneGeneSet(final Chip chip) throws Exception {
        final Chip2ChipMapper mapper = new Chip2ChipMapperSymbol(VdbRuntimeResources.getChip_Gene_Symbol(), chip);
        return mapper.map(getAsOneGeneSet(), false).getMappedGeneSet(true);
    }

    public GeneSet getGeneSet(final int i, final Chip chip) throws Exception {
        final Chip2ChipMapper mapper = new Chip2ChipMapperSymbol(VdbRuntimeResources.getChip_Gene_Symbol(), chip);
        return mapper.map(getGeneSet(i), false).getMappedGeneSet(true);
    }

    public GeneSet getGeneSet(final String glName, final Chip chip) throws Exception {
        final Chip2ChipMapper mapper = new Chip2ChipMapperSymbol(VdbRuntimeResources.getChip_Gene_Symbol(), chip);
        return mapper.map(getGeneSet(glName), false).getMappedGeneSet(true);
    }

    public int getMaxNumOfGenes() {
        if (fMaxNumGenes == -1) {
            for (int c = 0; c < getNumGeneLists(); c++) {
                if (getGeneList(c).getNumGenes() > fMaxNumGenes) {
                    fMaxNumGenes = getGeneList(c).getNumGenes();
                }
            }
        }

        return fMaxNumGenes;
    }

    public String[] getGeneListNames() {
        return (String[]) fGeneListNameGeneListMap.keySet().toArray(new String[fGeneListNameGeneListMap.size()]);
    }

    public GeneList[] getGeneLists() {
        return fGeneLists; // @todo consider cloning
    }

    public GeneSet[] getGeneSets() {
        GeneSet[] gsets = new GeneSet[getNumGeneLists()];
        for (int i = 0; i < gsets.length; i++) {
            gsets[i] = getGeneList(i).toGeneSet();
        }

        return gsets;
    }

    public GeneSet[] getGeneSets_as_symbols_restricted(final Chip chip) throws Exception {
        GeneSet[] gsets = new GeneSet[getNumGeneLists()];
        Set symbols = chip.getSymbols(Chip.OMIT_NULLS);
        for (int i = 0; i < gsets.length; i++) {
            gsets[i] = getGeneList(i).toGeneSet().cloneDeep(symbols);
        }

        return gsets;
    }

    public GeneSet[] getGeneSets(final Chip chip) throws Exception {
        GeneSet[] gsets = new GeneSet[getNumGeneLists()];
        for (int i = 0; i < gsets.length; i++) {
            gsets[i] = getGeneList(i).toGeneSet(chip);
        }

        return gsets;
    }

    public GeneList getGeneList(final int i) {
        return fGeneLists[i];
    }

    public GeneSet getGeneSet(final int i) {
        return fGeneLists[i].toGeneSet();
    }

    public GeneSet getGeneSet(final String glName) {
        return getGeneList(glName).toGeneSet();
    }

    public int getGeneListIndex(final String glName) {
        if (glNameIndexMap == null) {
            this.glNameIndexMap = new TObjectIntHashMap();
            for (int c = 0; c < getNumGeneLists(); c++) {
                this.glNameIndexMap.put(getGeneList(c), c);
            }
        }
        return glNameIndexMap.get(glName);
    }

    public GeneList getGeneList(final String glName) {
        Object obj = fGeneListNameGeneListMap.get(glName);
        if (obj == null) {
            throw new IllegalArgumentException("No such geneList: " + glName);
        } else {
            return (GeneList) obj;
        }
    }

    public boolean isGeneList(final String glName) {
        return fGeneListNameGeneListMap.containsKey(glName);
    }

    public int getNumGeneLists() {
        return fGeneLists.length;
    }

    private void init(final String name, final GeneList[] geneLists) {

        if (name == null) {
            throw new IllegalArgumentException("Parameter name cannot be null");
        }

        if (geneLists == null) {
            throw new IllegalArgumentException("Parameter geneLists cannot be null");
        }

        this.fGeneListNameGeneListMap = new HashMap();
        for (int i = 0; i < geneLists.length; i++) {
            if (fGeneListNameGeneListMap.containsKey(geneLists[i].getName())) {
                throw new IllegalArgumentException("Duplicate geneList: " + geneLists[i].getName());
            } else {
                fGeneListNameGeneListMap.put(geneLists[i].getName(), geneLists[i]);
            }
        }

        this.fName = name;
        this.fGeneLists = geneLists;

    }

    public String getQuickInfo() {
        return fGeneListNameGeneListMap.size() + "";
    }

    public String getName() {
        return FileUtils.getCrossPlatformName(fName);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    // each gene list (i.e gene list) is a column - all genes on this colu,n have the same color

    class MyColorMap implements ColorMap.Rows {

        public Color getColor(final String rowName, final String colName) {

            if (isGeneList(colName)) {
                GeneList gl = getGeneList(colName);

                if (gl.isMember(rowName)) {
                    return gl.getColor();
                }
            }

            return null;
        }

        public Color getColor(final String rowName, final int c) {
            return getColor(rowName, getColName(c));
        }

        public String getColName(final int col) {
            return getGeneList(col).getName();
        }

        public String getValue(final String rowName, final String colName) {

            if (isGeneList(colName)) {
                GeneList gl = getGeneList(colName);

                if (gl.isMember(rowName)) {
                    return gl.getGene(rowName).getNotes();
                } else {
                    return null;
                }
            } else {
                return null;
            }
        }

        public String getValue(final String rowName, final int col) {
            return getValue(rowName, getColName(col));
        }

        public int getNumCol() {
            return getNumGeneLists();
        }

        public boolean isInSymbols(int col) {
            return true;
        }

    } // End class MyColorMap


    /**
     * Internal class to hold a GeneList
     */
    public static class GeneListImpl implements GeneList {

        private String fName;

        private Color fColor;

        private GeneSet fGeneSet;

        private Gene[] fGenes;

        public GeneListImpl(final String parentGinFileName,
                            final String name,
                            final Color color,
                            final Gene[] genes) {

            this.fName = name;
            this.fColor = color;

            List names = new ArrayList();
            for (int i = 0; i < genes.length; i++) {
                if (names.contains(genes[i].getName())) {
                    throw new IllegalArgumentException("Duplicate gene name: " + genes[i] + " name: " + name + " parentGinFileName: " + parentGinFileName);
                } else {
                    names.add(genes[i].getName());
                }
            }

            this.fGenes = genes;
            this.fGeneSet = new FSet(parentGinFileName + "#" + getName(), names, false);
        }

        public GeneSet toGeneSet() {
            return fGeneSet;
        }

        public GeneSet toGeneSet(final Chip chip) throws Exception {
            final Chip2ChipMapper mapper = new Chip2ChipMapperSymbol(VdbRuntimeResources.getChip_Gene_Symbol(), chip);
            return mapper.map(toGeneSet(), false).getMappedGeneSet(true);
        }

        public String getName() {
            return fName;
        }

        public Color getColor() {
            return fColor;
        }

        public int getNumGenes() {
            return fGenes.length;
        }

        public Gene getGene(int i) {
            return fGenes[i];
        }

        public boolean isMember(String geneName) {
            return toGeneSet().isMember(geneName);
        }

        public Gene getGene(String geneName) {
            int index = toGeneSet().getPosition(geneName);
            if (index == -1) {
                throw new IllegalArgumentException("No such gene: " + geneName + "\n" + fGenes);
            }

            return fGenes[index];
        }

    } // End class GeneListImpl

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    /**
     * Inner class implementing a Record
     */
    public static class GeneImpl implements GenesOfInterest.Gene {

        private String fName;
        private String fLabel;
        private String fNotes;

        /**
         * Class constructor
         *
         * @param geneName
         * @param notes_opt
         */
        public GeneImpl(final String geneName, final String geneListName, String notes_opt) {
            this.fName = geneName;
            this.fLabel = geneListName;
            this.fNotes = notes_opt;
        }

        public String getName() {
            return fName;
        }

        public String getGeneListName() {
            return fLabel;
        }

        public String getNotes() {
            return fNotes;
        }

        public String toString() {
            return getName();
        }

        public int hashCode() {
            return getName().hashCode();
        }

        public boolean equals(Gene gene) {
            return gene.getName().equals(getName());
        }

        public boolean equals(Object obj) {

            //System.out.println("checking >" + obj + "< >" + getName());
            if (obj == null) {
                return false;
            }

            if (obj instanceof Gene) {
                return ((Gene) obj).getName().equals(getName());
            } else {
                return obj.toString().equals(getName());
            }
        }


    }

} // End class GenesOfInterestImpl