package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import org.genepattern.clustering.hierarchical.ArrayTreePanel;
import org.genepattern.clustering.hierarchical.AtrGtrReader;
import org.genepattern.clustering.hierarchical.GeneTreePanel;
import org.genepattern.clustering.hierarchical.image.HCLImage;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.heatmap.image.DisplaySettings;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

/**
 * @author Aravind Subramanian
 *         Date: Jan 13, 2006
 *         Time: 1:00:47 PM
 */
public class HClustDataset {

    private Logger log = XLogger.getLogger(HClustDataset.class);

    private Dataset fDataset_ordered;

    private File fGtrFile_opt;

    private File fAtrFile_opt;

    private ArrayTreePanel fArrayTreePanel;

    private GeneTreePanel fGeneTreePanel;

    /**
     * Class constructor
     */
    public HClustDataset(final Dataset ds_ordered,
                         final File gtrFile_opt,
                         final File atrFile_opt) throws IOException {


        if (ds_ordered == null) {
            throw new IllegalArgumentException("Param ds_ordered cannot be null");
        }


        if (gtrFile_opt != null) {
            this.fGtrFile_opt = gtrFile_opt;
            AtrGtrReader reader = new AtrGtrReader(ds_ordered.getRowNamesArray(), fGtrFile_opt.getName());
            this.fGeneTreePanel = new GeneTreePanel(reader);
        }

        if (atrFile_opt != null) {
            this.fAtrFile_opt = atrFile_opt;
            AtrGtrReader reader = new AtrGtrReader(ds_ordered.getRowNamesArray(), fGtrFile_opt.getName());
            this.fArrayTreePanel = new ArrayTreePanel(reader);
        }


        this.init(ds_ordered, fArrayTreePanel, fGeneTreePanel);
    }

    /**
     * Class constructor
     *
     * @param ds_ordered
     * @param arrayTreePanel
     * @param geneTreePanel
     */
    public HClustDataset(final Dataset ds_ordered,
                         final ArrayTreePanel arrayTreePanel,
                         final GeneTreePanel geneTreePanel) {
        this.init(ds_ordered, arrayTreePanel, geneTreePanel);
    }

    private void init(final Dataset ds_ordered,
                      final ArrayTreePanel arrayTreePanel,
                      final GeneTreePanel geneTreePanel) {

        this.fArrayTreePanel = arrayTreePanel;
        this.fGeneTreePanel = geneTreePanel;

        this.fDataset_ordered = ds_ordered;

        log.debug("Inited hclust to geneNames: " + fDataset_ordered.getNumRow() + " arrayNames: " + fDataset_ordered.getNumCol() +
                " fGtrFile: " + fGtrFile_opt + " fAtrFile: " + fAtrFile_opt + " fArrayTreePanel: " + fArrayTreePanel + " fGeneTreePanel: " + fGeneTreePanel);
    }

    public String[] getGeneNames() {
        return fDataset_ordered.getRowNamesArray();
    }

    public String[] getArrayNames() {
        return fDataset_ordered.getColumnNamesArray();
    }

    public Dataset getDatasetOrdered() {
        return fDataset_ordered;
    }

    public GeneTreePanel getGeneTreePanel() {
        return fGeneTreePanel;
    }

    public ArrayTreePanel getArrayTreePanel() {
        return fArrayTreePanel;
    }

    public boolean hasGtr() {
        return fGeneTreePanel != null;
    }

    public boolean hasAtr() {
        return fArrayTreePanel != null;
    }

    public Images createImages(final int bpogGridSize) {

        Dataset dsOrd = getDatasetOrdered();

        /*
        dsOrd = new DefaultDataset("foo", dsOrd.getMatrix(), dsOrd.getRowNamesArray(), dsOrd.getColumnNamesArray(),
                true, new AnnotImpl(dsOrd.getAnnot().getFeatureAnnot(), annot.cloneDeep(dsOrd.getColumnNamesArray())));
                */

        ArrayTreePanel arrayTree = getArrayTreePanel();

        if (arrayTree != null) {
            arrayTree.setBottomGutter(0);
            arrayTree.setTopGutter(0);
            arrayTree.setElementWidth(bpogGridSize);

            Dimension size = new Dimension(arrayTree.getPreferredSize().width, 150);
            arrayTree.setPreferredSize(size);
            arrayTree.setSize(size);
        }

        IExpressionData ied = GPWrappers.createIExpressionData(dsOrd);

        // -------------------------------------------------------------------------------------------- //
        final DisplaySettings set = new DisplaySettings();
        set.drawGrid = false;
        set.rowSize = bpogGridSize;
        set.columnSize = bpogGridSize;

        set.drawRowDescriptions = true;
        set.drawRowNames = true;
        set.drawGrid = false;

        set.gridLinesColor = Color.LIGHT_GRAY;
        set.sampleAnnotationSpacing = 10;
        set.sampleAnnonationsHeight = 20;
        set.showFeatureGridLines = true;
        set.showSampleGridLines = true;

        Images images = new Images();

        GeneTreePanel geneTree = getGeneTreePanel();
        if (geneTree != null) {
            geneTree.setElementHeight(bpogGridSize);
            Dimension size = new Dimension(geneTree.getPreferredSize().width, 2); // only the width matters
            geneTree.setPreferredSize(size);
            geneTree.setSize(size);
            images.geneTree = getGeneTreePanel().snapshot();
        }

        if (arrayTree != null) {
            images.arrayTree = arrayTree.snapshot();
        }

        images.fullImage = HCLImage.createImage(ied, images.geneTree,
                images.arrayTree, set,
                GPWrappers.createSampleAnnotator(dsOrd, null),
                GPWrappers.createFeatureAnnotator(dsOrd));

        return images;
    }

    public static class Images {

        public BufferedImage fullImage;
        public BufferedImage geneTree;
        public BufferedImage arrayTree;

    }

} // End class HClustDatasetJosh
