/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * HClustDataset
 * By itself, has no information on the hierarchical clustering
 */
public class HClustDatasetStanford extends AbstractObject implements PersistentObject {

    private Dataset fDataset;

    /**
     * wraps the dataset so that we can pass it out as a reference w/o a care
     */
    private DatasetProxy fDatasetProxy;

    private List fGeneXNames;

    private List fArrayXNames;

    /**
     * @param name
     * @param ds
     * @param genexnames  -> Names (Strings) NOT Nodes
     * @param arrayxnames -> Names (Strings) NOT Nodes
     * @param share
     */
    public HClustDatasetStanford(String name, Dataset ds, List genexnames, List arrayxnames, boolean share) {

        if (ds == null) {
            throw new IllegalArgumentException("Parameter ds cannot be null");
        }

        if (genexnames == null) {
            throw new IllegalArgumentException("Parameter genexnames cannot be null");
        }

        if (arrayxnames == null) {
            throw new IllegalArgumentException("Parameter arrayxnames cannot be null");
        }

        if (share) {
            init(name, ds, genexnames, arrayxnames);
        } else {

            Dataset dds = new DefaultDataset(ds.getName(), ds);
            List dgenexnames = new ArrayList(genexnames.size());
            for (int i = 0; i < genexnames.size(); i++) {
                dgenexnames.add(genexnames.get(i).toString());
            }

            List darrayxnames = new ArrayList(arrayxnames.size());
            for (int i = 0; i < arrayxnames.size(); i++) {
                darrayxnames.add(arrayxnames.get(i).toString());
            }

            init(name, dds, dgenexnames, darrayxnames);
        }

    }

    // data passed to this method must already be duplicated if appropriate
    private void init(String name, Dataset ds, List genexnames, List arrayxnames) {
        super.initialize(name);

        // sanity checks
        if (ds.getNumRow() != genexnames.size()) {
            throw new IllegalArgumentException("Dataset nrows: " + ds.getNumRow() + " must equal genexnames size: " + genexnames.size());
        }

        if (ds.getNumCol() != arrayxnames.size()) {
            throw new IllegalArgumentException("Dataset ncols: " + ds.getNumCol() + " must equal arrayxnames size: " + arrayxnames.size());
        }

        this.fDataset = ds;
        this.fDatasetProxy = new DatasetProxy(fDataset);
        this.fGeneXNames = Collections.unmodifiableList(genexnames);
        this.fArrayXNames = Collections.unmodifiableList(arrayxnames);
    }

    public Dataset getDataset() {
        return fDatasetProxy;
    }

    public String getGeneXName(int row) {
        return (String) fGeneXNames.get(row);
    }

    public int getGeneXIndex(String genexname) {
        return fGeneXNames.indexOf(genexname);
    }

    public String getArrayXName(int col) {
        return (String) fArrayXNames.get(col);
    }

    public int getArrayXIndex(String arrayxname) {
        return fArrayXNames.indexOf(arrayxname);
    }

    public String getQuickInfo() {
        return fDataset.getQuickInfo();
    }


} // End HClustDataset
