/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Object that represents a collection of hierarchically ordered Nodes
 */

public class HClustTree extends AbstractObject implements PersistentObject {

    private final Node[] fNodes;

    /**
     * Elements are Roots
     */
    private List fRoots;

    /**
     * @param name
     * @param nodes
     * @param shareNodeObjects
     */
    public HClustTree(String name, List nodes, boolean shareNodeObjects) {
        super(name);

        this.fNodes = new Node[nodes.size()];
        for (int n = 0; n < nodes.size(); n++) {
            Object node = nodes.get(n);
            if (node instanceof Node) {
                if (shareNodeObjects) {
                    fNodes[n] = (Node) node;
                } else {
                    // @todo impl non-sharing
                    //fNodes[n] = new Node((Node) node); // safe copy
                    fNodes[n] = (Node) node;
                }
            } else {
                throw new IllegalArgumentException("Only Node objects allowed. Found: " + node + " at: " + n);
            }
        }

        // init roots

        fRoots = new ArrayList();
        for (int n = 0; n < getNumNodes(); n++) {
            Node node = getNode(n);
            Object a = node.getParentA();
            if (a instanceof Root) {
                fRoots.add(a);
            }

            Object b = node.getParentB();
            if (b instanceof Root) {
                fRoots.add(b);
            }

        }

        //log.debug("# of nodes: " + fNodes.length);
        //log.debug("# of roots: " + fRoots.size());
        if (fNodes.length + 1 != fRoots.size()) {
            throw new IllegalArgumentException("Bad # of nodes: " + fNodes.length + " must be 1 less that # of roots: " + fRoots.size());
        }

        doNameNodeCheck();
    }

    // No two Nodes must have the same name
    private void doNameNodeCheck() {

        Set names = new HashSet();
        for (int n = 0; n < fNodes.length; n++) {
            if (names.contains(fNodes[n].getName())) {
                throw new IllegalArgumentException("Node names must be unique. Found duplicate Node: " + fNodes[n].getName() + " at: " + n);
            }
            names.add(fNodes[n].getName());
        }

        names = new HashSet();
        for (int i = 0; i < fRoots.size(); i++) {
            String name = ((Root) fRoots.get(i)).getName();
            if (names.contains(name)) {
                throw new IllegalArgumentException("Root names must be unique. Found duplicate Root: " + name + " at: " + i);
            }
            names.add(name);
        }

    }

    public int getNumNodes() {
        return fNodes.length;
    }

    public int getNumRoots() {
        return fRoots.size();
    }

    public Node getNode(int i) {
        return fNodes[i];
    }

    public Root[] getRoots() {
        List onodes = getRootsL();
        return (Root[]) onodes.toArray(new Root[onodes.size()]);
    }

    public List getRootsL() {
        return fRoots;
    }

    public String getQuickInfo() {
        StringBuffer buf = new StringBuffer().append(getNumNodes()).append(" nodes");
        return buf.toString();
    }

    // no parents ever
    public static class Root {
        private String fName;

        public Root(String name) {
            if (name == null) {
                throw new IllegalArgumentException("Parameter name cannot be null");
            }
            this.fName = name;
        }

        public String getName() {
            return fName;
        }

    }

    /**
     * A Node can be of the form:
     * 1) NODE1X	GENE51X	GENE24X	0.99798584
     * 2) NODE10X	NODE4X	NODE7X	0.993835449
     * 3) NODE13X	GENE1X	NODE5X	0.990478516
     * <p/>
     * i.e any combination of genes and other Node objects
     * <p/>
     * The a-b order does not matter within a Node
     */
    public static class Node {

        private Object fParentA;
        private Object fParentB;
        private String fName;
        private float fScore;

        /**
         * @param name
         * @param parentA
         * @param parentB
         * @param score
         */
        public Node(String name, Node parentA, Node parentB, float score) {
            init(name, parentA, parentB, score);
        }

        /**
         * @param name
         * @param parentA
         * @param b
         * @param score
         */
        public Node(String name, Root parentA, Node b, float score) {
            init(name, parentA, b, score);
        }

        /**
         * @param name
         * @param parentA
         * @param b
         * @param score
         */
        public Node(String name, Object parentA, Object b, float score) {

            init(name, parentA, b, score);
        }

        private void init(String name, Object a, Object b, float score) {
            if (name == null) {
                throw new IllegalArgumentException("Parameter name cannot be null");
            }

            if (a == null) {
                throw new IllegalArgumentException("Parameter a cannot be null");
            }

            if ((a instanceof Root) || (a instanceof Node)) {
                ;
            } else {
                throw new IllegalArgumentException("parentA can only be a Roor or a Node. Got: " + a + " " + a.getClass());
            }

            if (b == null) {
                throw new IllegalArgumentException("Parameter b cannot be null");
            }

            if ((b instanceof Root) || (b instanceof Node)) {
                ;
            } else {
                throw new IllegalArgumentException("parentB can only be a Roor or a Node. Got: " + b + " " + b.getClass());
            }

            this.fName = name;
            this.fScore = score;

            this.fParentA = a;
            this.fParentB = b;

        }

        public Object getParentA() {
            return fParentA;
        }

        public Object getParentB() {
            return fParentB;
        }

        public String getParentAName() {
            if (fParentA instanceof Root) {
                return ((Root) fParentA).getName();
            } else {
                return ((Node) fParentA).getName();
            }
        }

        public String getParentBName() {
            if (fParentB instanceof Root) {
                return ((Root) fParentB).getName();
            } else {
                return ((Node) fParentB).getName();
            }
        }

        public float getScore() {
            return fScore;
        }

        public String getName() {
            return fName;
        }

    } // End inner class Node

} // End class HClustTree


