/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.MismatchedSizeException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class LabelledObject extends AbstractObject implements LabelledList {

    private List fLabels;

    private List fObjects;

    public LabelledObject(final String name, List labels, List objects) {
        this.init(name, labels, objects);
    }

    public LabelledObject(final String name, String[] labels, String[] objects) {
        this.init(name, labels, objects);
    }

    // common (pre)initialization routine
    // see below too
    private void init(final String name, final String[] alabels, final Object[] objs) {

        if (alabels == null) {
            throw new IllegalArgumentException("Param alabels cannot be null");
        }

        if (objs == null) {
            throw new IllegalArgumentException("Parameter objs cannot be null");
        }

        if (objs.length != alabels.length) {
            throw new MismatchedSizeException("Objects", objs.length, "Labels", +alabels.length);
        }

        List labels = new ArrayList(alabels.length);
        List objects = new ArrayList(objs.length);
        for (int i = 0; i < alabels.length; i++) {
            labels.add(alabels[i]);
            objects.add(objs[i]);
        }

        init(name, labels, labels);
    }

    private void init(final String name, final List labels, final List objects) {
        super.initialize(name);

        if (labels == null) {
            throw new IllegalArgumentException("Param labels cannot be null");
        }

        if (objects == null) {
            throw new IllegalArgumentException("Parameter objects cannot be null");
        }

        if (labels.size() != objects.size()) {
            throw new MismatchedSizeException("Objects", objects.size(), "Labels", labels.size());
        }

        this.fLabels = Collections.unmodifiableList(labels);
        this.fObjects = Collections.unmodifiableList(objects);
    }

    public String getQuickInfo() {
        return Integer.toString(fLabels.size()) + " labels";
    }

    public List getRowNames() {
        return getLabels();
    }

    public String[] getRowNamesArray() {
        return getLabelsArray();
    }

    public String getLabel(final int index) {
        return fLabels.get(index).toString();
    }

    public Object getElement(final String label) {
        int elemIndex = getLabelIndex(label);
        if (elemIndex == -1) {
            throw new IllegalArgumentException("No match to label: " + label);
        }
        return fObjects.get(elemIndex);
    }

    public List getLabels() {
        return Collections.unmodifiableList(fLabels);
    }

    public String[] getLabelsArray() {
        return (String[]) fLabels.toArray(new String[fLabels.size()]);
    }

    public boolean isMember(final String label) {
        return fLabels.contains(label);
    }

    public int getLabelIndex(final String label) {
        return fLabels.indexOf(label);
    }

    public Object getElement(final int index) {
        return fObjects.get(index);
    }

    public String toString(final String delim) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < getSize(); i++) {
            buf.append(getLabel(i)).append(delim).append(getElement(i)).append('\n');
        }

        return buf.toString();
    }


    public int getSize() {
        return fLabels.size();
    }

} // End LabelledObject

