/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.strucs.DefaultMetricWeightStruc;
import edu.mit.broad.genome.parsers.AuxUtils;
import gnu.trove.*;

import java.util.*;

/**
 * A kind of Vector in which labels can be associated with elements
 * i.e Vector <-> LabelledVector relationship is similar to the
 * Matrix <-> Dataset relationship
 * <p/>
 * (ofcourse, much simpler -- simply row labels)
 * <p/>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class LabelledVector extends AbstractObject implements LabelledList {

    private List fLabels;

    private Vector fVector;

    public LabelledVector cloneShallow(String newName) {
        super.setName(newName);
        return this;
    }

    /**
     * Class constructor
     *
     * @param name
     * @param labels
     * @param floats
     */
    public LabelledVector(final String name, final String[] labels, final TFloatArrayList floats) {
        init(name, labels, new Vector(floats));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param labels
     * @param floats
     */
    public LabelledVector(final String name, final List labels, final TFloatArrayList floats) {
        init(name, labels, new Vector(floats));
    }

    /**
     * Class constructor
     *
     * @param map
     */
    public LabelledVector(TObjectFloatHashMap map) {

        String[] names = new String[map.size()];
        Vector values = new Vector(map.size());

        TObjectFloatIterator iterator = map.iterator();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            names[i] = iterator.key().toString();
            values.setElement(i, iterator.value());
        }

        init(names, values);
    }

    public LabelledVector(String name, TObjectFloatHashMap map) {

        String[] names = new String[map.size()];
        Vector values = new Vector(map.size());

        TObjectFloatIterator iterator = map.iterator();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            names[i] = iterator.key().toString();
            values.setElement(i, iterator.value());
        }

        init(name, names, values);
    }

    public LabelledVector(TObjectIntHashMap map) {
        this("", map);
    }

    public LabelledVector(String name, TObjectIntHashMap map) {

        String[] names = new String[map.size()];
        Vector values = new Vector(map.size());

        TObjectIntIterator iterator = map.iterator();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            names[i] = iterator.key().toString();
            values.setElement(i, iterator.value());
        }

        init(name, names, values);
    }

    /**
     * Class constructor
     *
     * @param rl
     * @param shareRowNames
     */
    public LabelledVector(final RankedList rl, final boolean shareRowNames) {

        List labels;

        if (shareRowNames) {
            labels = rl.getRankedNames();
        } else {
            labels = new ArrayList(rl.getSize());
        }

        Vector values = new Vector(rl.getSize());

        for (int r = 0; r < rl.getSize(); r++) {
            if (!shareRowNames) {
                labels.add(rl.getRankName(r));
            }
            values.setElement(r, rl.getScore(r));
        }

        init(rl.getName(), labels, values);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param labels
     * @param floats
     */
    public LabelledVector(final String name, final String[] labels, final float[] floats) {
        init(name, labels, new Vector(floats));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param labels
     * @param floats
     */
    public LabelledVector(final String name, final String[] labels, final Vector floats) {
        init(name, labels, floats);
    }

    /**
     * Class constructor
     *
     * @param gset
     * @param v
     * @param shareLabels
     */
    public LabelledVector(final GeneSet gset, final Vector v, final boolean shareLabels) {
        if (shareLabels) {
            init(gset.getMembers(), v);
        } else {
            init(gset.getMembersArray(), v); // array so that auto duplicated
        }
    }

    /**
     * Class constructor
     *
     * @param v
     */
    public LabelledVector(final Vector v) {
        List labels = new ArrayList(v.getSize());
        for (int i = 0; i < v.getSize(); i++) {
            labels.add("row_" + i);
        }

        init(labels, v);
    }

    /**
     * Class constructor
     *
     * @param labels
     * @param v
     * @param shareLabels
     */
    public LabelledVector(final List labels, final Vector v, final boolean shareLabels) {
        if (shareLabels) {
            this.init(labels, v);
        } else {
            this.init(new ArrayList(labels), v);
        }
    }

    /**
     * Class constructor
     *
     * @param name
     * @param labels
     * @param v
     * @param shareLabels
     */
    public LabelledVector(final String name, final List labels, final Vector v, final boolean shareLabels) {
        if (shareLabels) {
            this.init(name, labels, v);
        } else {
            this.init(name, new ArrayList(labels), v);
        }
    }

    /**
     * Class constructor
     * Scores AND Addresses(indices) are taken from the DEL
     * The names are taken from the LabelledList
     *
     * @param dels
     * @param ll
     */
    public LabelledVector(final DoubleElement[] dels, final LabelledList ll) {

        if (ll == null) {
            throw new NullPointerException("Param ll cannot be null");
        }

        if (dels == null) {
            throw new NullPointerException("Param dels cannot be null");
        }

        if (ll.getSize() != dels.length) {
            throw new MismatchedSizeException("LabelledList", ll.getSize(), "DoubleElement Array", dels.length);
        }

        Vector scores = new Vector(dels.length);
        List labels = new ArrayList(dels.length);
        for (int r = 0; r < scores.getSize(); r++) {
            int rowIndex = dels[r].fIndex;
            scores.setElement(r, (float) dels[r].fValue);
            labels.add(ll.getLabel(rowIndex));
        }

        init(labels, scores);
    }

    /**
     * Class constructor
     *
     * @param dsRowIndices
     * @param dsCol2Use
     * @param ds
     */
    public LabelledVector(final TIntArrayList dsRowIndices, final int dsCol2Use, final Dataset ds) {

        if (dsRowIndices == null) {
            throw new NullPointerException("Param indices cannot be null");
        }

        if (ds == null) {
            throw new NullPointerException("Param Dataset cannot be null");
        }

        if (dsRowIndices.size() > ds.getNumRow()) {
            throw new MismatchedSizeException("Indices", dsRowIndices.size(), "Dataset", ds.getNumRow());
        }

        Vector scores = new Vector(dsRowIndices.size());
        List labels = new ArrayList(scores.getSize());
        for (int r = 0; r < scores.getSize(); r++) {
            int rowIndex = dsRowIndices.get(r);
            scores.setElement(r, ds.getElement(rowIndex, dsCol2Use));
            labels.add(ds.getRowName(rowIndex));
        }

        init(labels, scores);
    }

    /**
     * Class constructor
     * labels must be of same length as the vector
     *
     * @param labels
     * @param v
     * @note vector used directly -- no copy made
     */
    public LabelledVector(final String[] labels, final Vector v) {
        init(labels, v);
    }

    /**
     * Class constructor
     *
     * @param v
     */
    public LabelledVector(String labelPrefix, final Vector v) {
        if (v == null) {
            throw new IllegalArgumentException("Parameter v cannot be null");
        }

        if (labelPrefix == null) {
            labelPrefix = "";
        }

        String[] labels = new String[v.getSize()];
        for (int i = 0; i < v.getSize(); i++) {
            labels[i] = labelPrefix + i;
        }

        init(labels, v);
    }

    /**
     * Class constructor
     *
     * @param iv
     * @param ds
     */
    public LabelledVector(final AddressedVector iv, final Dataset ds) {
        this(iv, ds, true);
    }

    /**
     * Class constructor
     *
     * @param iv
     * @param ds
     * @param duplicateVector
     */
    public LabelledVector(final AddressedVector iv, final Dataset ds, boolean duplicateVector) {

        if (iv == null) {
            throw new NullPointerException("Param iv cannot be null");
        }

        if (ds == null) {
            throw new NullPointerException("Param Dataset cannot be null");
        }

        if (iv.getSize() != ds.getNumRow()) {
            throw new MismatchedSizeException("AddressedVector", iv.getSize(),
                    "Dataset NumRows", ds.getNumRow());
        }

        Vector scores = new Vector(iv.getScoresArray(), duplicateVector);
        List labels = new ArrayList(scores.getSize());
        for (int r = 0; r < scores.getSize(); r++) {
            int posinds = iv.getAddress(r);
            labels.add(ds.getRowName(posinds));
        }

        init(labels, scores);
    }

    public LabelledVector(final LabelledVector[] lvs) {
        if (lvs == null) {
            throw new IllegalArgumentException("Param lvs cannot be null");
        }

        TFloatArrayList values = new TFloatArrayList();
        List labels = new ArrayList();
        for (int i = 0; i < lvs.length; i++) {
            values.add(lvs[i].getScoresV(false).toArray());
            labels.addAll(lvs[i].getLabels());
        }

        init(labels, new Vector(values));
    }

    public LabelledVector(final PersistentObject[] pobForlabels, final Vector v) {
        if (v == null) {
            throw new IllegalArgumentException("Param v cannot be null");
        }
        if (pobForlabels == null) {
            throw new IllegalArgumentException("Param labels cannot be null");
        }

        if (v.getSize() != pobForlabels.length) {
            throw new MismatchedSizeException("Vector", v.getSize(), "Labels from Pobs", pobForlabels.length);
        }

        this.fVector = v;
        this.fLabels = new ArrayList(v.getSize());
        for (int i = 0; i < pobForlabels.length; i++) {
            fLabels.add(pobForlabels[i].getName());
        }

    }

    private void init(final String[] alabels, final Vector v) {
        init(null, alabels, v);
    }

    // common (pre)initialization routine
    // see below too
    private void init(final String name, final String[] alabels, final Vector v) {

        if (v == null) {
            throw new IllegalArgumentException("Parameter v cannot be null");
        }

        if (alabels == null) {
            throw new IllegalArgumentException("Param alabels cannot be null");
        }

        if (v.getSize() != alabels.length) {
            throw new MismatchedSizeException("Vector", v.getSize(), "Labels", +alabels.length);
        }

        List labels = new ArrayList(alabels.length);
        for (int i = 0; i < alabels.length; i++) {
            labels.add(alabels[i]);
        }

        init(name, labels, v);

    }

    private void init(final List labels, final Vector v) {
        init(null, labels, v);
    }

    // all duplication (if needed) must be done BEFORE calling this method
    private void init(String name, final List labels, final Vector v) {
        if (v == null) {
            throw new IllegalArgumentException("Param v cannot be null");
        }
        if (labels == null) {
            throw new IllegalArgumentException("Param labels cannot be null");
        }

        if (v.getSize() != labels.size()) {
            throw new MismatchedSizeException("Vector", v.getSize(), "Labels", +labels.size());
        }

        // make sure no duplicates
        // @note this check is expensive so disabled for now

        if (name == null) {
            super.initialize(" " + getClass().hashCode() + System.currentTimeMillis());
        } else {
            super.initialize(name);
        }

        this.fVector = v;
        this.fLabels = labels;
        this.fVector.setImmutable(); // @note imp
    }

    public String getQuickInfo() {
        return Integer.toString(fLabels.size()) + " labels";
    }

    public boolean isMember(final String label) {
        return fLabels.contains(label);
    }

    public float getScore(final String label) {
        int elemIndex = _labelIndex(label);
        if (elemIndex == -1) {
            System.err.println("Labels are: \n" + getLabels() + "\n");
            throw new IllegalArgumentException("No match to label: " + label);
        }
        return fVector.getElement(elemIndex);
    }

    public float getScore(final int i) {
        return fVector.getElement(i);
    }

    public String getLabel(final int i) {
        return fLabels.get(i).toString();
    }

    public List getLabels() {
        return Collections.unmodifiableList(fLabels);
    }

    public String[] getLabelsForValuesArray(float minValueInclusive, float maxValueInclusive) {
        Set labels = getLabelsForValues(minValueInclusive, maxValueInclusive);
        return (String[]) labels.toArray(new String[labels.size()]);
    }

    public String[] getLabelsArray() {
        return (String[]) fLabels.toArray(new String[fLabels.size()]);
    }

    public List getLabelsSubset(int fromIndex, int toIndex) {
        return fLabels.subList(fromIndex, toIndex);
    }

    public Set getLabelsForValues(float minValueInclusive, float maxValueInclusive) {
        Set labels = new HashSet();
        for (int i = 0; i < getSize(); i++) {
            if (getScore(i) >= minValueInclusive && getScore(i) <= maxValueInclusive) {
                labels.add(getLabel(i));
            }
        }

        return labels;
    }

    public String[] getLabels_asis(int topX) {
        String[] labels = new String[topX];
        for (int r = 0; r < topX; r++) {
            labels[r] = getLabel(r);
        }

        return labels;
    }

    // synch the labels but NOT the scores
    public LabelledVector synchLabels(final LabelledVector lvSynchTo, boolean enforceSameStartingSize) {

        if (lvSynchTo == null) {
            throw new IllegalArgumentException("Param lv cannot be null");
        }

        if (enforceSameStartingSize && lvSynchTo.getSize() != this.getSize()) {
            throw new MismatchedSizeException(lvSynchTo.getName(), lvSynchTo.getSize(), this.getName(), this.getSize());
        }

        Vector v = new Vector(lvSynchTo.getSize());
        List labels = new ArrayList();
        for (int r = 0; r < lvSynchTo.getSize(); r++) {
            String name = lvSynchTo.getLabel(r);
            labels.add(name);
            v.setElement(r, getScore(name)); // score of this lv
        }

        return new LabelledVector(getName(), labels, v, false);
    }

    /**
     * Does NOT affect current object
     *
     * @param sort
     * @param order
     * @return
     * @maint ALG method placed in object. Because we often need to use this from within other objects
     * and that causes linkage errors if calling an alg class.
     */
    public RankedList sort(final SortMode sort, final Order order) {
        DoubleElement[] dels = this.toDoubleElements();
        dels = DoubleElement.sort(sort, order, dels);

        List labels = new ArrayList(dels.length);
        Vector v = new Vector(dels.length);
        for (int i = 0; i < dels.length; i++) {
            labels.add(this.getLabel(dels[i].fIndex));
            v.setElement(i, (float) dels[i].fValue);
        }

        return new DefaultRankedList(getName(), labels, v, true, true);
    }

    private int _labelIndex(final String label) {

        if (fLabelNameLabelIndexMap == null) {
            cacheRowNameIndex();
        }

        // IMP needed as returns 0 and not -1 on not hits!!
        if (fLabelNameLabelIndexMap.containsKey(label)) {
            return fLabelNameLabelIndexMap.get(label);
        }

        // try harder
        String al = AuxUtils.getAuxNameOnlyNoHash(label);
        if (fLabelNameLabelIndexMap.containsKey(al)) {
            return fLabelNameLabelIndexMap.get(al);
        }

        return -1;
    }

    private TObjectIntHashMap fLabelNameLabelIndexMap;

    private void cacheRowNameIndex() {
        if (fLabelNameLabelIndexMap == null) {
            fLabelNameLabelIndexMap = new TObjectIntHashMap();
            for (int rown = 0; rown < getSize(); rown++) {
                fLabelNameLabelIndexMap.put(getLabel(rown), rown);
            }
        }
    }

    public float[] getScoresArray() {
        return fVector.toArray();
    }

    /**
     * IMP IMP we dont make a safe copy when returning unless asked for
     * However, the Vector is IMMutable, so same effect, except quicker.
     *
     * @return
     */
    public Vector getScoresV(boolean clonedCopy) {

        if (clonedCopy) {
            return new Vector(fVector, false);
        } else {
            fVector.setImmutable();
            return fVector;
        }

        //return new Vector(fVector, clonedCopy);
    }

    public String toString(final char delim) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < getSize(); i++) {
            buf.append(getLabel(i)).append('=').append(getScore(i)).append(delim);
        }

        return buf.toString();
    }

    public String toString_newline(final char delim) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < getSize(); i++) {
            buf.append(getLabel(i)).append(delim).append(getScore(i)).append('\n');
        }

        return buf.toString();
    }

    // here because matrix uses it
    public static Vector[] toVectors(final LabelledVector[] lvs, boolean clonedCopy) {
        Vector[] vs = new Vector[lvs.length];
        for (int i = 0; i < lvs.length; i++) {
            vs[i] = lvs[i].getScoresV(clonedCopy);
        }

        return vs;
    }

    public int getSize() {
        return fVector.getSize();
    }

    public int getSize(ScoreMode smode) {
        return fVector.getSize(smode);
    }

    public DoubleElement[] toDoubleElements() {
        DoubleElement[] dels = new DoubleElement[getSize()];
        for (int i = 0; i < getSize(); i++) {
            dels[i] = new DoubleElement(i, getScore(i));
        }

        return dels;
    }

    public static int getMaxSize(final LabelledVector[] lvs) {
        int max = 0;
        for (int i = 0; i < lvs.length; i++) {
            if (lvs[i].getSize() > max) {
                max = lvs[i].getSize();
            }
        }

        return max;
    }

    public static LabelledVector getMaxSize_lv(final LabelledVector[] lvs) {
        if (lvs == null) {
            throw new IllegalArgumentException("Param lvs cannot be null");
        }

        if (lvs.length == 0) {
            throw new IllegalArgumentException("Param lvs cannot be 0 length");
        }

        LabelledVector lvMax = null;
        int max = -1;
        for (int i = 0; i < lvs.length; i++) {
            if (lvs[i].getSize() > max) {
                max = lvs[i].getSize();
                lvMax = lvs[i];
            }
        }

        return lvMax;
    }

    public static int getMinSize(final LabelledVector[] lvs) {
        int min = Integer.MAX_VALUE;
        for (int i = 0; i < lvs.length; i++) {
            if (lvs[i].getSize() < min) {
                min = lvs[i].getSize();
            }
        }

        return min;
    }

    private MetricWeightStruc ws;

    public MetricWeightStruc getMetricWeightStruc() {
        if (ws == null) {
            ws = new DefaultMetricWeightStruc(null, this);
        }

        return ws;
    }

    public LabelledVector multiply(final LabelledVector b) {
        if (getSize() != b.getSize()) {
            throw new MismatchedSizeException(this, b);
        }

        Vector mult = new Vector(getSize());
        String[] labels = new String[mult.getSize()];
        for (int i = 0; i < mult.getSize(); i++) {
            labels[i] = getLabel(i);
            mult.setElement(i, getScore(i) * b.getScore(labels[i]));
        }

        return new LabelledVector(labels, mult);
    }

    public LabelledVector multiply(final LabelledVector b, final LabelledVector c) {
        if (getSize() != b.getSize()) {
            throw new MismatchedSizeException(this, b);
        }

        if (this.getSize() != c.getSize()) {
            throw new MismatchedSizeException(this, c);
        }

        Vector mult = new Vector(getSize());
        String[] labels = new String[mult.getSize()];
        for (int i = 0; i < mult.getSize(); i++) {
            labels[i] = getLabel(i);
            mult.setElement(i, getScore(i) * b.getScore(labels[i]) * c.getScore(labels[i]));
        }

        return new LabelledVector(labels, mult);
    }

    // -------------------------------------------------------------------------------------------- //

    public LabelledVector extractLV(final GeneSet gset) {
        return extractLV(gset.getName(true), gset.getMembersArray());
    }

    public LabelledVector extractLV(final String name, final String[] labels) {
        if (labels == null) {
            throw new IllegalArgumentException("Param labels cannot be null");
        }

        float[] scores = new float[labels.length];
        for (int i = 0; i < labels.length; i++) {
            scores[i] = getScore(labels[i]);
        }

        return new LabelledVector(name, labels, scores);
    }

} // End LabelledVector
