/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.math.Vector;
import gnu.trove.TFloatArrayList;
import gnu.trove.THashMap;

import java.util.*;

/**
 * Nearest neighbors of a seed plus their correlations to the seed
 */

// not sure if this even needs to be a iset impl - trouble is that if it is then
// a tracking array of names has to be mainatiend n addd to the neighs
//public class NeighSet extends AbstractObject implements ISet {
public class NeighSet extends AbstractObject {

    // choosing to not encapsulate FSet, as i want to expressly NOT allow duplicates
    // safer as then have tp deal with dupl corr values etc
    private List fNeighs; // contains Neighbor objects


    private THashMap fNameNeighborMap; // lazilly inited

    /**
     * Class constructor
     *
     * @param seedName
     * @param neighs
     */
    public NeighSet(String seedName, Neighbor[] neighs) {
        init(seedName, neighs);
    }

    public NeighSet(String seedName, String[] names, Vector corrs) {
        Neighbor[] neighs = new Neighbor[names.length];
        for (int i = 0; i < names.length; i++) {
            neighs[i] = new Neighbor(names[i], corrs.getElement(i));
        }

        init(seedName, neighs);
    }

    public NeighSet(String seedName, List names, TFloatArrayList corrs) {
        Neighbor[] neighs = new Neighbor[names.size()];
        for (int i = 0; i < names.size(); i++) {
            neighs[i] = new Neighbor(names.get(i).toString(), corrs.get(i));
        }

        init(seedName, neighs);
    }

    /**
     * Class constructor
     *
     * @param seedName
     * @param neighsList
     */
    public NeighSet(String seedName, List neighsList) {
        this(seedName, (Neighbor[]) neighsList.toArray(new Neighbor[neighsList.size()]));
    }

    // expects data to already be duplicated if needed
    // cant use a set as the members and corrs need to be synched
    protected void init(String seedName, Neighbor[] neighs) {
        super.initialize(seedName);

        if (neighs == null) {
            throw new NullPointerException("neighsList param cant be null");
        }

        if (seedName == null) {
            throw new IllegalArgumentException("Parameter seedName cannot be null");
        }


        Map namesMap = new HashMap();
        this.fNeighs = new ArrayList(); // always make a safe copy

        for (int i = 0; i < neighs.length; i++) {
            Object obj = neighs[i];
            if (obj == null) {
                throw new IllegalArgumentException("Element cannot be null in neighsList at: " + i);
            }

            if (!(obj instanceof Neighbor)) {
                throw new IllegalArgumentException("Unexpected member class -- expecting Neighbor but got: " + obj.getClass());
            }

            Neighbor neigh = (Neighbor) obj;

            if (namesMap.containsKey(neigh.getName())) {
                throw new IllegalArgumentException("Duplicate Neighbors not allowed: " + neigh.getName());
            }

            namesMap.put(neigh.getName(), neigh);
            this.fNeighs.add(neigh);

        }

        // sanity check
        if (neighs.length != fNeighs.size()) {
            throw new IllegalStateException("Unexpected mismatch: " + neighs.length + " and " + fNeighs.size());
        }

    }

    /**
     * ditto to getName()
     *
     * @return
     */
    public String getSeedName() {
        return getName();
    }

    public String getQuickInfo() {
        return "" + getNumMembers();
    }

    public Neighbor getNeighbor(int i) {
        return (Neighbor) fNeighs.get(i);
    }

    /**
     * @return Number of members of this NeighSet
     */
    public int getNumMembers() {
        return fNeighs.size();
    }

    public Vector getCorrsV() {
        Vector v = new Vector(getNumMembers());
        for (int i = 0; i < v.getSize(); i++) {
            v.setElement(i, getNeighbor(i).getCorr());
        }

        return v;
    }

    public Neighbor[] getNeighbors(String[] neighNames) {
        Neighbor[] neighs = new Neighbor[neighNames.length];
        for (int i = 0; i < neighNames.length; i++) {
            neighs[i] = getNeighbor(neighNames[i]);
        }

        return neighs;
    }

    public Neighbor[] getNeighbors(float minCorrInclusive) {
        Set set = new HashSet();
        for (int i = 0; i < getNumMembers(); i++) {
            Neighbor neigh = getNeighbor(i);
            if (neigh.getCorr() >= minCorrInclusive) {
                set.add(neigh);
            }
        }

        return (Neighbor[]) set.toArray(new Neighbor[set.size()]);
    }

    public Set getNeighbors(Collection neighNames, boolean barfOnMissing) {
        initMap();
        Set neighs = new HashSet();
        for (Iterator it = neighNames.iterator(); it.hasNext();) {
            String neighName = it.next().toString();
            Object obj = fNameNeighborMap.get(neighName);
            if (obj == null && barfOnMissing) {
                throw new IllegalArgumentException("No such neighbor: " + neighName);
            }

            if (obj != null) {
                neighs.add(obj);
            }

        }

        return neighs;
    }

    public Neighbor[] getNeighborsArr(Collection neighNames, boolean barfOnMissing) {
        Set set = getNeighbors(neighNames, barfOnMissing);
        return (Neighbor[]) set.toArray(new Neighbor[set.size()]);
    }

    public boolean isMember(String name) {
        initMap();
        return fNameNeighborMap.containsKey(name);
    }

    public Neighbor getNeighbor(String name) {
        initMap();
        Object obj = fNameNeighborMap.get(name);
        if (obj != null) {
            return (Neighbor) obj;
        } else {
            throw new IllegalArgumentException("No such Neighbor: " + name);
        }
    }

    private void initMap() {
        if (fNameNeighborMap == null) {
            fNameNeighborMap = new THashMap();
            for (int i = 0; i < getNumMembers(); i++) {
                Neighbor neigh = getNeighbor(i);
                fNameNeighborMap.put(neigh.getName(), neigh);
            }
        }
    }

    /**
     * A safe and deep coopy is returned
     *
     * @return
     */
    public GeneSet toGeneSet() {
        return new FSet(getName(), getNameEnglish(), fNeighs, false);
    }

    // imp that it is toXX so that no memory is shared
    public GeneSet toGeneSet(float minCorrInclusive) {
        Neighbor[] neighs = getNeighbors(minCorrInclusive);
        return new FSet(getName() + "_r_" + minCorrInclusive, getNameEnglish(), Neighbor.toNames(neighs), false);
    }

} // End class NeighSet
