/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Feb 12, 2005
 *         Time: 9:30:03 PM
 */
public class PreTemplateImpl extends AbstractPreTemplate {

    private Pair[] fPairs;
    private Map fSampleNamePairMap;

    private boolean fContinuous;

    private int fNumClasses;

    /**
     * Class constructor
     *
     * @param justPhenoName
     * @param pairs
     */
    public PreTemplateImpl(final String sourcefileName,
                           final String justPhenoName,
                           final Pair[] pairs, boolean continuous) {
        super(sourcefileName + "#" + justPhenoName);

        // checks
        for (int i = 0; i < pairs.length; i++) {
            if (pairs[i] instanceof NumPair && !continuous) {
                throw new IllegalArgumentException("NumPair but not continuous");
            }

            if (pairs[i] instanceof StringPair && continuous) {
                throw new IllegalArgumentException("StringPair but continuous");
            }

        }

        this.fPairs = pairs;
        this.fContinuous = continuous;

        this.fSampleNamePairMap = new HashMap();
        Set uniqValues = new HashSet();
        for (int p = 0; p < pairs.length; p++) {
            if (pairs[p] == null) {
                throw new IllegalArgumentException("Param pairs cannot be null at index: " + p);
            }
            if (fSampleNamePairMap.containsKey(pairs[p].getSampleName())) {
                throw new IllegalArgumentException("Duplicate sample names: " + pairs[p].getSampleName());
            }

            this.fSampleNamePairMap.put(pairs[p].getSampleName(), pairs[p]);
            if (!fContinuous) {
                uniqValues.add(pairs[p].getValue());
            }
        }

        if (fContinuous) {
            this.fNumClasses = fSampleNamePairMap.size();
        } else {
            this.fNumClasses = uniqValues.size();
        }
    }

    public int getNumClasses() {
        return fNumClasses;
    }

    public boolean isContinuous() {
        return fContinuous;
    }

    public boolean isCategorical() {
        return !fContinuous;
    }

    public boolean isAux() {
        if (fContinuous) {
            return true;
        } else {
            return false; // @note is always aux
        }
    }

    public String getQuickInfo() {
        StringBuffer buf = new StringBuffer();
        if (isContinuous()) {
            buf.append(fPairs.length).append("=>C");
        } else {
            buf.append(fPairs.length).append("=>").append(getNumClasses());
        }

        return buf.toString();
    }

    public Pair[] getPairs() {
        return fPairs;
    }

    public boolean isSampleNamesFromDataset() {
        return true;
    }

    // only those samples that are in the dataset are used
    public Template createTemplate(final Dataset ds) {

        TemplateImpl template = new TemplateImpl(getName());
        boolean isNumeric = false;
        Map classNameClassMap = new HashMap();
        for (int c = 0; c < ds.getNumCol(); c++) {
            final String sampleName = ds.getColumnName(c);
            final Object obj = fSampleNamePairMap.get(sampleName);
            if (obj != null) {
                final int profilePos = ds.getColumnIndex(sampleName);
                Pair pair = (Pair) obj;
                Template.Item item;
                String id;
                if (pair instanceof StringPair) {
                    StringPair sp = (StringPair) pair;
                    id = sp.className;
                } else {
                    isNumeric = true;
                    NumPair np = (NumPair) pair;
                    id = np.classValue + "";
                }

                TemplateImpl.ClassImpl cl;
                final Object obcn = classNameClassMap.get(id);
                if (obcn != null) {
                    cl = (TemplateImpl.ClassImpl) obcn;
                } else {
                    cl = new TemplateImpl.ClassImpl(id);
                    classNameClassMap.put(id, cl);
                    template.add(cl);
                }

                // add item to template
                item = TemplateImpl.ItemImpl.createItem(id, profilePos);
                template.add(item);
                cl.add(item);
            }
        } // End for loop

        //template.assignItems(); // DO NOT call - custom addition already done above
        template.setContinuous(isNumeric);
        template.makeImmutable();
        return template;
    }

    public static interface Pair {

        public String getSampleName();

        public Object getValue();

    } // End interface Pair

    public static class StringPair implements Pair {
        String sampleName;
        String className;

        public StringPair(final String sampleName, final String className) {
            this.sampleName = sampleName;
            this.className = className;
        }

        public String getSampleName() {
            return sampleName;
        }

        public Object getValue() {
            return className;
        }

    } // End class StringPair

    public static class NumPair implements Pair {
        String sampleName;
        float classValue;

        public NumPair(final String sampleName, final String classValue) {
            this.sampleName = sampleName;
            this.classValue = Float.parseFloat(classValue);
        }

        public String getSampleName() {
            return sampleName;
        }

        public Object getValue() {
            return new Float(classValue);
        }

    } // End class NumPair


}
