/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.math.*;
import gnu.trove.TFloatArrayList;

import java.util.ArrayList;
import java.util.List;

/**
 * Different from LabelledList, here there IS  a sense of order (rankings) as well as a
 * name associated with every rank.
 * <p/>
 * IMP IMP: Ranks are always always from 0 to n-1
 * (scores can be something else, but ranks are always contiguous and 0 to n-1)
 */
public interface RankedList extends PersistentObject {

    public RankedList cloneShallowRL(String newName);

    public boolean isMember(String name);

    public int getSize();

    public int getSize(ScoreMode smode);

    public String toString(char delim);

    public float getValue(final String featureName,
                          final XMath.Const scoreorRank);

    // ----- SCORE METHODS ---- //

    public float getScore(final int rank);

    public float getScore(final String label);

    public float[] getScores(final GeneSet gset);

    public Vector getScoresV(final boolean clonedCopy);

    // -- RANK METHODS ---- //
    public int getRank(final String label);

    public int[] getRanks(final GeneSet gset);

    public Vector getRanksV(final GeneSet gset);

    // ---- ROW NAME METHODS ---- //

    public String getRankName(final int rank);

    //public String getRowName(int rank);

    public List getRankedNames();

    public String[] getRankedNamesArray();

    public GeneSetPair getNamesOfUpAndDnXRanks(final int upAndDnX);

    public List getNamesOfUpOrDnXRanks(final int upOrDnX, final boolean up);

    public String[] getNamesOfUpOrDnXRanks_array(final int upOrDnX, final boolean up);

    public GeneSet getNamesOfUpOrDnXRanks_gs(final int upOrDnX, final boolean up);

    public int getNumScores(float minRankScore, float maxRankScore);

    public GeneSet getScores_gs(float minRankScore, float maxRankScore);

    /**
     * EXTRACTER METHODS ***
     */

    public RankedList extractRanked(float minRankScore, float maxRankScore);

    public RankedList extractRanked(final int upOrDnX, final boolean up);

    public RankedList extractRanked(final int upAndDnX);

    public RankedList extractRanked(final GeneSet gset);

    public RankedList extractRanked(final String[] names);

    public RankedList extractRanked(final List names);

    public RankedList extractRanked(final ScoreMode smode);

    public GeneSet toGeneSet();

    // ---------------------------------------------------------- //
    // ---------------------------------------------------------- //
    // ---------------------------------------------------------- //

    public String toString(final char delim, boolean newLine);

    public MetricWeightStruc getMetricWeightStruc();

    // ---------------------------------------------------------- //
    // ---- HELPER CLASS ---- //
    // ---------------------------------------------------------- //

    static class Helper {

        public static int[] getRanks(final GeneSet gset, final RankedList rl) {
            final int[] ranks = new int[gset.getNumMembers()];
            for (int r = 0; r < gset.getNumMembers(); r++) {
                ranks[r] = rl.getRank(gset.getMember(r));
            }
            return ranks;
        }

        public static Vector getRanksV(final GeneSet gset, final RankedList rl) {
            return new Vector(getRanks(gset, rl));
        }

        public static String toString(final char delim, final boolean newLine, final RankedList rl) {

            if (newLine) {
                StringBuffer buf = new StringBuffer();
                for (int r = 0; r < rl.getSize(); r++) {
                    buf.append(rl.getRankName(r)).append(delim).append(rl.getScore(r)).append('\n');
                }

                return buf.toString();
            } else {
                StringBuffer buf = new StringBuffer();
                for (int r = 0; r < rl.getSize(); r++) {
                    buf.append(rl.getRankName(r)).append('=').append(rl.getScore(r)).append(delim);
                }

                return buf.toString();
            }
        }

        // collect and sort by rank
        // Remember that ranks are always 0(good) to n-1(bad)
        public static RankedList extract(final GeneSet gset, final RankedList rl) {

            DoubleElement[] dels = new DoubleElement[gset.getNumMembers()];
            for (int i = 0; i < gset.getNumMembers(); i++) {
                int rank = rl.getRank(gset.getMember(i));
                dels[i] = new DoubleElement(i, rank); // index is position in the gset
            }

            dels = DoubleElement.sort(SortMode.REAL, Order.ASCENDING, dels);

            final List rankedLabels = new ArrayList();
            final Vector rankedScores = new Vector(dels.length);

            for (int i = 0; i < dels.length; i++) {
                String name = gset.getMember(dels[i].fIndex);
                rankedLabels.add(name);
                rankedScores.setElement(i, rl.getScore(name));
            }

            return new DefaultRankedList(gset.getName(true), rankedLabels, rankedScores, true, true);
        }

        public static RankedList extract(final String[] names, final RankedList rl) {

            DoubleElement[] dels = new DoubleElement[names.length];
            for (int i = 0; i < names.length; i++) {
                int rank = rl.getRank(names[i]);
                dels[i] = new DoubleElement(i, rank); // index is position in the array
            }

            dels = DoubleElement.sort(SortMode.REAL, Order.ASCENDING, dels);

            final List rankedLabels = new ArrayList();
            final Vector rankedScores = new Vector(dels.length);

            for (int i = 0; i < dels.length; i++) {
                String name = names[dels[i].fIndex];
                rankedLabels.add(name);
                rankedScores.setElement(i, rl.getScore(name));
            }

            return new DefaultRankedList(rl.getName() + "_ex", rankedLabels, rankedScores, true, true);
        }


        public static RankedList extract(float minRankScore, float maxRankScore, final RankedList rl) {

            final List rankedLabels = new ArrayList();
            final TFloatArrayList rankedScores = new TFloatArrayList();

            for (int r = 0; r < rl.getSize(); r++) {
                float score = rl.getScore(r);
                if (score <= maxRankScore && score >= minRankScore) {
                    rankedLabels.add(rl.getRankName(r));
                    rankedScores.add(score);
                }
            }

            return new DefaultRankedList(rl.getName() + "_" + minRankScore + "_" + maxRankScore, rankedLabels, rankedScores);
        }

        public static int getNumScores(float minRankScore, float maxRankScore, final RankedList rl) {
            return getScores_gs(minRankScore, maxRankScore, rl).getNumMembers();
        }

        public static GeneSet getScores_gs(float minRankScore, float maxRankScore, final RankedList rl) {
            final List rankedLabels = new ArrayList();

            for (int r = 0; r < rl.getSize(); r++) {
                float score = rl.getScore(r);
                if (score <= maxRankScore && score >= minRankScore) {
                    rankedLabels.add(rl.getRankName(r));
                }
            }

            return new FSet(rl.getName() + "_" + minRankScore + "_" + maxRankScore, rankedLabels, true);
        }

        public static RankedList extract(final List names, final RankedList rl) {
            return extract((String[]) names.toArray(new String[names.size()]), rl);
        }

        public static RankedList extract(boolean onlyPos, final RankedList rl) {
            List rankedNames = new ArrayList();
            TFloatArrayList rankedScores = new TFloatArrayList();
            String suffix;

            if (onlyPos) {
                suffix = "pos";
            } else {
                suffix = "neg";
            }

            for (int i = 0; i < rl.getSize(); i++) {
                float score = rl.getScore(i);
                if (onlyPos && XMath.isPositive(score)) {
                    rankedNames.add(rl.getRankName(i));
                    rankedScores.add(score);
                } else if (!onlyPos && XMath.isNegative(score)) {
                    rankedNames.add(rl.getRankName(i));
                    rankedScores.add(score);
                } else {
                    // do nothing
                }
            }

            return new DefaultRankedList(rl.getName() + "_" + suffix, rankedNames, rankedScores);
        }

        public static RankedList extractRanked(final int upOrDnX, final boolean up, final RankedList rl) {
            return rl.extractRanked(rl.getNamesOfUpOrDnXRanks(upOrDnX, up));
        }

        public static RankedList extractRanked(final int upAndDnX, final RankedList rl) {
            List up = rl.getNamesOfUpOrDnXRanks(upAndDnX, true);
            up.addAll(rl.getNamesOfUpOrDnXRanks(upAndDnX, false));
            return rl.extractRanked(up);
        }

        public static String[] getLabelsOfUpOrDnXRanks_array(final int upOrDnX, final boolean up, final RankedList rl) {
            List list = getLabelsOfUpOrDnXRanks(upOrDnX, up, rl);
            return (String[]) list.toArray(new String[list.size()]);
        }

        public static GeneSet getLabelsOfUpOrDnXRanks_gs(final int upOrDnX, final boolean up, final RankedList rl) {
            String name = rl.getName();
            if (up) {
                name += "_up_" + upOrDnX;
            } else {
                name += "_dn_" + upOrDnX;
            }

            return new FSet(name, getLabelsOfUpOrDnXRanks(upOrDnX, up, rl), false);
        }

        public static GeneSetPair getLabelsOfUpAndDnXRanks(final int upAndDnX, final RankedList rl) {
            return new GeneSetPairImpl(rl.getName(), getLabelsOfUpOrDnXRanks_gs(upAndDnX, true, rl), getLabelsOfUpOrDnXRanks_gs(upAndDnX, false, rl));
        }

        public static List getLabelsOfUpOrDnXRanks(int upOrDnX, boolean up, final RankedList rl) {
            List list = new ArrayList();
            if (upOrDnX > rl.getSize()) {
                upOrDnX = rl.getSize();
            }

            if (up) {
                for (int r = 0; r < upOrDnX; r++) {
                    list.add(rl.getRankName(r));
                }
            } else {
                int start = rl.getSize() - 1;
                for (int r = 0; r < upOrDnX; r++) {
                    list.add(rl.getRankName(start));
                    start--;
                }
            }

            return list;
        }

        public static RankedList extractRanked(final ScoreMode smode, final RankedList rl) {
            List rankedNames = new ArrayList();
            TFloatArrayList rankedScores = new TFloatArrayList();
            String suffix = smode.getName();

            for (int i = 0; i < rl.getSize(); i++) {
                float score = rl.getScore(i);
                if (smode.isPostiveOnly() && XMath.isPositive(score)) {
                    rankedNames.add(rl.getRankName(i));
                    rankedScores.add(score);
                } else if (smode.isNegativeOnly() && XMath.isNegative(score)) {
                    rankedNames.add(rl.getRankName(i));
                    rankedScores.add(score);
                } else {
                    // do nothing
                }
            }

            return new DefaultRankedList(rl.getName() + "_" + suffix, rankedNames, rankedScores);
        }
    }

} // End RankedList

