package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.objects.strucs.DefaultMetricWeightStruc;
import gnu.trove.TObjectIntHashMap;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;


/**
 * A Dataset that is scored and sorted/ordered in some way. The data IS SHARED between original
 * Dataset and (this) (re)ordered Dataset. So efficient for use.
 * (Note: the getMatrix method does NOT return shared data - it duplicates as needed)
 * In addition, the scores that caused the ranking are available through getScore()
 * <p/>
 * IMP IMP IMP: Dataset is NOT sorted -> thats whay the indexed vector is for
 * see the sdsrown2posinds method
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ScoredDatasetImpl extends AbstractObject implements ScoredDataset {

    private Dataset fDataset;

    private AddressedVector fIndVector;

    private List fRowNamesInSdsOrder;

    private GeneSet fRowNamesGeneSet; // lazilly filled

    public Dataset cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    /**
     * Class Constructor.
     * Dataset data is NOT duplicated.
     * But see note for the getMatrix() method below.
     * The original dataset is not touched in any way.
     * Indexedvector data IS copied
     *
     * @todo hmm the row names are replicated. How to avoid?
     * @todo Hmm to avoid duplicating data we need a sorted matrix.
     * If this is possible to impl at a later stage, look into doing the same with FeatureList too.
     */
    public ScoredDatasetImpl(final String name, final int num, final AddressedVector iv, final Dataset ds) {
        init(name, num, iv, ds);
    }

    public ScoredDatasetImpl(final int num, final AddressedVector iv, final Dataset ds) {
        this(ds.getName(), num, iv, ds);
    }

    // dont expose as this can be very slow
    public ScoredDatasetImpl(final RankedList rl, final Dataset ds) {
        this(ds.getName(), new AddressedVector(rl, ds), ds);
    }

    /**
     * Class constructor
     *
     * @param iv
     * @param ds
     */
    public ScoredDatasetImpl(final AddressedVector iv, final Dataset ds) {
        // this(ds.getNumRow(), iv, ds); IMP NOT ds length -> it might be bigger than the iv
        this(iv.getSize(), iv, ds);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param iv
     * @param ds
     */
    public ScoredDatasetImpl(final String name, final AddressedVector iv, final Dataset ds) {
        // this(ds.getNumRow(), iv, ds); IMP NOT ds length -> it might be bigger than the iv
        this(name, iv.getSize(), iv, ds);
    }

    public RankedList cloneShallowRL(final String newName) {
        super.setName(newName);
        return this;
    }

    /**
     * Factory method to create a new ScoredDataset from specified
     * ScoredDataset with only elemnts above and including the specified
     * cutoff
     * The underlying dataset is NOT duplicated in anyway, so efficient.
     * Hmm the rownames are duplicated actually.
     *
     * @param cutoff
     * @param sds
     * @return
     */
    public ScoredDataset cloneShallowCutoff(final double cutoff) {

        List dels = new ArrayList();

        for (int i = 0; i < this.getNumRow(); i++) {
            float score = this.getScore(i);

            if (score >= cutoff) {
                dels.add(new DoubleElement(i, score));
            }
        }

        AddressedVector iv = new AddressedVector(dels);

        return new ScoredDatasetImpl(iv, this.fDataset);
    }

    // same addresses, reversed scores
    public ScoredDataset cloneShallowReversedScore() {
        String revname = getName() + "reversedscores";

        Vector revScores = new Vector(fIndVector.getSize());
        int[] revAddresses = new int[fIndVector.getSize()];
        int max = getNumRow() - 1;
        for (int i = 0; i < revScores.getSize(); i++) {
            revScores.setElement(i, fIndVector.getScore(i) * -1);
            revAddresses[i] = max - fIndVector.getAddress(i);
        }

        AddressedVector civ = new AddressedVector(revScores, revAddresses, true);
        return new ScoredDatasetImpl(revname, civ, fDataset);
    }

    /**
     * common initialization routine.
     */
    private void init(final String name, final int num, final AddressedVector iv, final Dataset ds) {

        if (iv == null) {
            throw new NullPointerException("Param AddressedVector cannot be null");
        }

        if (ds == null) {
            throw new NullPointerException("Param Dataset cannot be null");
        }

        super.initialize(name);

        if (num < 0) {
            throw new IllegalArgumentException("# features: " + num + " cannot be less than sero");
        }

        if (num > iv.getSize()) {
            throw new IllegalArgumentException("# features: " + num
                    + " cannot be > than number of sorted elements: "
                    + iv.getSize());
        }

        if (iv.getSize() > ds.getNumRow()) {
            throw new IllegalArgumentException("# sorted elements: " + iv.getSize()
                    + " cannot be > Dataset length: " + ds.getNumRow());
        }

        this.fDataset = ds;
        this.fIndVector = new AddressedVector(num, iv);    // data copied
        this.fRowNamesInSdsOrder = new ArrayList(num);

        for (int sdsrown = 0; sdsrown < num; sdsrown++) {
            int posinds = sdsrown2posinds(sdsrown);
            fRowNamesInSdsOrder.add(ds.getRowName(posinds)); // IMP -> note adding converted index order
        }

        this.fRowNamesInSdsOrder = Collections.unmodifiableList(fRowNamesInSdsOrder);
    }

    public Annot getAnnot() {
        return fDataset.getAnnot();
    }

    public Vector getRow(final int sdsrown) {
        return fDataset.getRow(sdsrown2posinds(sdsrown));
    }

    public Vector getRow(final String rowName) {
        return fDataset.getRow(rowName);
    }

    public Vector[] getRows(final GeneSet gset) {
        return fDataset.getRows(gset);
    }

    // -------------------------------------------------------------------------------------------- //

    public Vector getColumn(final String colName) {
        throw new NotImplementedException();
    }

    public int[] getColumnIndices(final String[] colNames) {
        int[] indices = new int[colNames.length];
        for (int i = 0; i < colNames.length; i++) {
            indices[i] = getColumnIndex(colNames[i]);
        }

        return indices;
    }

    public Vector[] getColumns(final String[] colNames) {
        throw new NotImplementedException();
        /*
        Vector[] vss = new Vector[colNames.length];
        for (int i = 0; i < colNames.length; i++) {
            vss[i] = getColumn(colNames[i]);
        }
        return vss;
        */
    }

    // vv expensove to implement

    public Vector getColumn(final int coln) {
        throw new NotImplementedException();
        // this is wrong because the order of entries in the column in the dataset is obviously not going to be the same as the order in the rls
        //return fDataset.getColumn(coln);
    }

    public LabelledVector getColumnLV(final String colName) {
        throw new NotImplementedException();
        //return getColumnLV(getColumnIndex(colName));
    }

    public LabelledVector getColumnLV(final int coln) {
        throw new NotImplementedException();
        //return new LabelledVector(getColumnName(coln), getRankedNamesArray(), fDataset.getColumn(coln));
    }

    public RankedList getColumnRL(final String colName, final SortMode sort, final Order order) {
        throw new NotImplementedException();
        //return getColumnLV(colName).sort(sort, order);
    }

    public RankedList getColumnRL(final int coln, final SortMode sort, final Order order) {
        throw new NotImplementedException();
        //return getColumnLV(coln).sort(sort, order);
    }

    // -------------------------------------------------------------------------------------------- //

    public Vector getColumn(final int coln, final GeneSet forTheseGenes) {
        return fDataset.getColumn(coln, forTheseGenes);
    }

    public Vector getColumn(final String colName, final GeneSet forTheseGenes) {
        return fDataset.getColumn(colName, forTheseGenes);
    }

    public float getElement(final int sdsrown, final int coln) {
        return fDataset.getElement(sdsrown2posinds(sdsrown), coln);
    }

    public float getElement(final int row, final String colName) {
        return getElement(row, getColumnIndex(colName));
    }

    public float getElement(final String rowName, final int col) {
        return getElement(getRowIndex(rowName), col);
    }

    public float getElement(final String rowName, final String colName) {
        return getElement(getRowIndex(rowName), getColumnIndex(colName));
    }

    public String getRowName(final int sdsrown) {
        // dont have to sdsrown2posinds as already done at init for efficiency
        return (String) fRowNamesInSdsOrder.get(sdsrown);
    }

    public String getRowSortName(final int sdsrown) {
        return "Sorted Rank: " + sdsrown + " PosInDs: " + sdsrown2posinds(sdsrown);
    }

    public List getRankedNames() {
        return Collections.unmodifiableList(fRowNamesInSdsOrder);
    }

    public List getRowNames() {
        return getRankedNames();
    }

    public GeneSet getRowNamesGeneSet() {
        if (fRowNamesGeneSet == null) {
            fRowNamesGeneSet = new FSet(getName(), new HashSet(getRankedNames()));
        }
        return fRowNamesGeneSet;
    }

    public GeneSet createOrderedGeneSet() {
        return new FSet(getName() + "_orderedgeneset", getNameEnglish(), Collections.unmodifiableList(fRowNamesInSdsOrder), true);
    }

    /**
     * changed to do this automatically on request
     * IMP very very slow for large
     * To avoid that first call cacheRowNameIndex
     * usual perf/memory tradeoffs
     *
     * @param rowName
     * @return
     * @see getRowNameIndexMap()
     * @see cacheRowNameIndex()
     */
    public int getRowIndex(final String rowName) {

        if (fRowNameSdsRowIndexMap == null) {
            cacheRowNameIndex();
        }

        int index = fRowNameSdsRowIndexMap.get(rowName);

        // IMP needed as returns 0 and not -1 on no hits!!
        if (index == 0) {
            if (!fRowNameSdsRowIndexMap.containsKey(rowName)) {
                return -1;
            } else {
                return 0;
            }
        }

        return index;
    }

    private TObjectIntHashMap fRowNameSdsRowIndexMap;

    private void cacheRowNameIndex() {
        if (fRowNameSdsRowIndexMap == null) {
            fRowNameSdsRowIndexMap = new TObjectIntHashMap();
            for (int sdsrown = 0; sdsrown < getNumRow(); sdsrown++) {
                fRowNameSdsRowIndexMap.put(getRowName(sdsrown), sdsrown);
            }
        }
    }

    public String[] getRankedNamesArray() {
        return (String[]) fRowNamesInSdsOrder.toArray(new String[fRowNamesInSdsOrder.size()]);
    }

    public String[] getRowNamesArray() {
        return getRankedNamesArray();
    }

    public List getColumnNames() {
        return fDataset.getColumnNames();
    }

    public List getColumnNames(Template prefixTemplateClasses) {
        return fDataset.getColumnNames(prefixTemplateClasses);
    }

    public int getColumnIndex(String colName) {
        return fDataset.getColumnIndex(colName);
    }

    public String[] getColumnNamesArray() {
        return fDataset.getColumnNamesArray();
    }

    public GeneSet getColumnNamesGeneSet() {
        return fDataset.getColumnNamesGeneSet();
    }

    public String getColumnName(int coln) {
        return fDataset.getColumnName(coln);
    }

    /**
     * @note IMP this is not necc the same size as ds
     */
    public int getNumRow() {
        return fIndVector.getSize();
    }

    public int getNumCol() {
        return fDataset.getNumCol();
    }

    public int getDim() {
        return fDataset.getDim();
    }

    public String getQuickInfo() {
        StringBuffer buf = new StringBuffer().append(getNumRow()).append('x').append(getNumCol());
        return buf.toString();
    }

    public String toString(final char delim) {
        StringBuffer buf = new StringBuffer();
        for (int r = 0; r < getSize(); r++) {
            buf.append(getRankName(r)).append('=').append(getScore(r)).append(delim);
        }

        return buf.toString();
    }

    /**
     * A brand new matrix is generated on each call.
     * Memory space is NOT shared.
     * So use with care
     */
    // Hmm to avoid duplicating data we need a sorted matrix - but dont want that.
    public Matrix getMatrix() {

        Matrix matrix = new Matrix(fIndVector.getSize(), fDataset.getNumCol());

        for (int i = 0; i < fIndVector.getSize(); i++) {
            matrix.setRow(i, fDataset.getRow(sdsrown2posinds(i)));
        }

        return matrix;
    }

    // very efficient as direct lookup in an array
    private int sdsrown2posinds(int index) {
        return fIndVector.getAddress(index);
    }

    /**
     * --------------------------------------------------------------
     * <p/>
     * ADDITIONAL SCORING / ORDERING / SORTING RELATED METHODS
     * IN ADDITION TO REGULAR DATASET METHODS
     * <p/>
     * --------------------------------------------------------------
     * return a safe copy of the sort order
     * <p/>
     * <p/>
     * return a safe copy of the sort order
     */

    /**
     * return a safe copy of the sort order
     */
    public int[] getSortOrder() {
        return fIndVector.getAddresses();
    }

    public int getUnscoredDatasetPosition(int sdsrown) {
        return sdsrown2posinds(sdsrown);
    }

    /**
     * synonym for getRowIndex()
     *
     * @param rowname
     * @return
     */
    public int getRank(String rowname) {
        return getRowIndex(rowname);
    }

    public boolean isMember(final String label) {
        return fRowNamesInSdsOrder.contains(label);
    }

    public float getValue(final String featureName,
                          final XMath.Const scoreorRank) {
        if (scoreorRank.isScore()) {
            return getScore(featureName);
        } else {
            return getRank(featureName);
        }
    }

    /**
     * score of gene at position i in the ScoredDataset
     *
     * @param posinds
     * @return
     */
    public float getScore(int sdsrown) {
        return fIndVector.getScore(sdsrown);
    }

    public float getScore(String rowName) {
        int sdsrown = getRowIndex(rowName);

        /*
        if (sdsrown == -1) {
            rowName = AuxUtils.getAuxNameOnlyNoHash(rowName);
            sdsrown = getRowIndex(rowName);
            if (sdsrown == -1) {
                throw new IllegalArgumentException("Could not find feat index for: " + sdsrown + " " + rowName);
            }
        }
        */

        if (sdsrown == -1) {
            throw new IllegalArgumentException("Could not find feat index for: " + sdsrown + " " + rowName);
        }

        return fIndVector.getScore(sdsrown);
    }

    /**
     * @return float[] of scores
     */
    public float[] getScores() {
        return fIndVector.getScoresArray();
    }

    public LabelledVector getRowLV(final String rowName) {
        return getRowLV(getRowIndex(rowName));
    }

    public LabelledVector getRowLV(final int rown) {
        return new LabelledVector(getRowName(rown), getColumnNamesArray(), getMatrix().getRowV(rown));
    }

    public RankedList getRowRL(final String rowName, final SortMode sort, final Order order) {
        return getRowLV(rowName).sort(sort, order);
    }

    public RankedList getRowRL(final int rown, final SortMode sort, final Order order) {
        return getRowLV(rown).sort(sort, order);
    }

    /**
     * @return Immutable Vector of scores
     */
    public Vector getScoresV(boolean clonedCopy) {
        return fIndVector.getScoresV(clonedCopy);
    }

    public Vector getScoreVector(int topX) {
        Vector v = fIndVector.getScoresV(true);
        v.truncate(topX);
        return v;
    }

    public LabelledVector getScoreVectorL() {
        return new LabelledVector(getRankedNamesArray(), fIndVector.getScoresV(true));
    }

    public float getMaxScore() {
        return fIndVector.getScoresV(false).max();
    }

    public float getMinScore() {
        return fIndVector.getScoresV(false).min();
    }

    public GeneSet getTopMarkers(int topX, boolean twoSided) {
        String name = getName() + "top" + topX + "_twosided" + twoSided;

        if (twoSided) {
            int[] tb = XMath.splitTwoSided(topX);
            int top = tb[0];
            int bot = tb[1];
            return new FSet(name, new GeneSet[]{_getTopMarkers(top), _getBottomMarkers(bot)});
        } else {
            return _getTopMarkers(topX);
        }
    }

    // @todo fixme
    public GeneSet getBottomMarkers(int botX, boolean twoSided) {
        String name = getName() + "bot" + botX + "_twosided" + twoSided;

        if (twoSided) {
            int[] tb = XMath.splitTwoSided(botX);
            int top = tb[0];
            int bot = tb[1];
            return new FSet(name, new GeneSet[]{_getTopMarkers(top), _getBottomMarkers(bot)});
        } else {
            return _getBottomMarkers(botX);
        }
    }

    private GeneSet _getTopMarkers(int topX) {
        List names = new ArrayList(topX);
        for (int i = 0; i < topX; i++) {
            names.add(getRowName(i));
        }

        return new FSet(getName() + "_top" + topX, getNameEnglish(), names, false);
    }

    private GeneSet _getBottomMarkers(int botX) {
        List names = new ArrayList(botX);
        int pos = getNumRow() - 1;
        for (int i = 0; i < botX; i++) {
            names.add(getRowName(pos));
            pos--;
        }

        return new FSet(getName() + "_bot" + botX, getNameEnglish(), names, false);
    }

    // RankedList impl
    public int getSize() {
        return getNumRow();
    }

    public int getSize(final ScoreMode smode) {
        return getScoresV(false).getSize(smode);
    }

    // RankedList impl
    public String getRankName(int rank) {
        return getRowName(rank);
    }

    public float[] getScores(final GeneSet gset) {
        float[] scores = new float[gset.getNumMembers()];
        for (int i = 0; i < gset.getNumMembers(); i++) {
            scores[i] = getScore(gset.getMember(i));
        }

        return scores;
    }

    public int[] getRanks(final GeneSet gset) {
        return Helper.getRanks(gset, this);
    }

    public Vector getRanksV(final GeneSet gset) {
        return Helper.getRanksV(gset, this);
    }

    public List getNamesOfUpOrDnXRanks(int topOrBotX, boolean top) {
        return Helper.getLabelsOfUpOrDnXRanks(topOrBotX, top, this);
    }

    public GeneSet getNamesOfUpOrDnXRanks_gs(int topOrBotX, boolean top) {
        return Helper.getLabelsOfUpOrDnXRanks_gs(topOrBotX, top, this);
    }

    public GeneSetPair getNamesOfUpAndDnXRanks(int topAndBotX) {
        return Helper.getLabelsOfUpAndDnXRanks(topAndBotX, this);
    }

    public String[] getNamesOfUpOrDnXRanks_array(int topOrBotX, boolean top) {
        return Helper.getLabelsOfUpOrDnXRanks_array(topOrBotX, top, this);
    }

    public RankedList extractRanked(final ScoreMode smode) {
        return Helper.extractRanked(smode, this);
    }

    public RankedList extractRanked(final int topAndBotX) {
        return Helper.extractRanked(topAndBotX, this);
    }

    public RankedList extractRanked(final GeneSet gset) {
        return RankedList.Helper.extract(gset, this);
    }

    public ScoredDataset extractSds(final GeneSet gset) {
        return extractSds(gset.getName(true), gset);
    }

    public ScoredDataset extractSds(final String newName, final GeneSet gset) {
        ScoredDatasetImpl sds = new ScoredDatasetImpl(extractRanked(gset), fDataset);
        sds.setName(newName);
        return sds;
    }

    public RankedList extractRanked(final int topOrBotX, final boolean top) {
        return Helper.extractRanked(topOrBotX, top, this);
    }

    public RankedList extractRanked(final float minRankScore, final float maxRankScore) {
        return Helper.extract(minRankScore, maxRankScore, this);
    }

    public int getNumScores(float minRankScore, float maxRankScore) {
        return Helper.getNumScores(minRankScore, maxRankScore, this);
    }

    public GeneSet getScores_gs(float minRankScore, float maxRankScore) {
        return Helper.getScores_gs(minRankScore, maxRankScore, this);
    }

    public RankedList extractRanked(final String[] names) {
        return RankedList.Helper.extract(names, this);
    }

    public RankedList extractRanked(final List names) {
        return RankedList.Helper.extract(names, this);
    }

    public GeneSet toGeneSet() {
        return fRowNamesGeneSet;
    }

    public String toString(final char delim, final boolean newLine) {
        return Helper.toString(delim, newLine, this);
    }

    private MetricWeightStruc ws;

    public MetricWeightStruc getMetricWeightStruc() {
        if (ws == null) {
            ws = new DefaultMetricWeightStruc(null, this);
        }

        return ws;
    }


}    // End ScoredDataset

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
