/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.objects;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.alg.AlgUtils;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.vdb.chip.Chip;
import gnu.trove.TFloatArrayList;

import java.util.*;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class StringDataframe extends AbstractObject implements IDataframe {

    protected StringMatrix fMatrix;

    protected List fRowNames;

    protected List fColNames;

    // name of the first row
    private String fRowLabelName;

    /**
     * Class Constructor.
     * Specified data is copied over - data is NOT shared
     *
     * @param name
     * @param smatrix
     * @param rowNames
     * @param colNames
     */
    public StringDataframe(final String name,
                           final StringMatrix smatrix,
                           final List rowNames,
                           final List colNames,
                           boolean shareAll) {
        this(name, smatrix, rowNames, colNames, shareAll, shareAll, shareAll);
    }

    public StringDataframe cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    public StringDataframe cloneDeep_probes_to_genes(final Chip chip) {

        StringMatrix sm = new StringMatrix(getNumRow(), getNumCol() + 2);

        for (int r = 0; r < getNumRow(); r++) {
            int coln = 0;
            sm.setElement(r, coln++, chip.getSymbol(getRowName(r), Chip.REPLACE_WITH_PROBEID));
            sm.setElement(r, coln++, chip.getTitle(getRowName(r), Chip.REPLACE_WITH_PROBEID));

            for (int cn = 0; cn < getNumCol(); cn++) {
                sm.setElement(r, coln++, getElement(r, cn));
            }
        }

        List colNames = new ArrayList();
        colNames.add("GENE");
        colNames.add("TITLE ");
        colNames.addAll(getColumnNames());
        return new StringDataframe(getName() + "_" + chip.getName(), sm, getRowNames(), colNames, true);
    }

    public StringDataframe cloneDeep_uppercase() {
        StringMatrix clonedM = fMatrix.toUpperCase();

        List clonedRowNames = new ArrayList(fRowNames);
        for (int i = 0; i < fRowNames.size(); i++) {
            Object obj = fRowNames.get(i);
            if (obj == null) {
                throw new IllegalStateException("Row name is null: " + obj + " at: " + i + "\n" + fRowNames);
            }
            clonedRowNames.set(i, obj.toString().toUpperCase());
        }

        List clonedColNames = new ArrayList(fColNames);
        for (int i = 0; i < fColNames.size(); i++) {
            clonedColNames.set(i, fColNames.get(i).toString().toUpperCase());
        }


        StringDataframe sdf = new StringDataframe(getName(), clonedM, clonedRowNames, clonedColNames, true);
        if (this.fRowLabelName != null) {
            sdf.fRowLabelName = this.fRowLabelName.toUpperCase();
        }
        return sdf;
    }

    public StringDataframe cloneDeep_by_row_names(final GeneSet gset, final boolean barfOnMissingNames) {
        return cloneDeep_by_row_names(gset.getMembersArray(), barfOnMissingNames);
    }

    public StringDataframe cloneDeep_by_row_names(final List rowNames, final boolean barfOnMissingNames) {
        return cloneDeep_by_row_names((String[]) rowNames.toArray(new String[rowNames.size()]), barfOnMissingNames);
    }

    public StringDataframe cloneDeep_by_row_names(final String[] rowNames, final boolean barfOnMissingNames) {
        StringMatrix smatrix = new StringMatrix(rowNames.length, getNumCol());
        for (int i = 0; i < rowNames.length; i++) {
            String rname = rowNames[i];
            int index = getRowIndex(rname);
            if (index == -1 && barfOnMissingNames) {
                throw new RuntimeException("No row in sdf for name: " + rname);
            } else if (index != -1) {
                smatrix.setRow(i, getRow(index));
            }
        }

        return new StringDataframe(getName() + "_" + rowNames.length, smatrix, rowNames, getColumnNamesArray(), false);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param smatrix
     * @param rowNames
     * @param colNames
     * @param shareAll
     */
    public StringDataframe(final String name,
                           final StringMatrix smatrix,
                           final String[] rowNames,
                           final String[] colNames,
                           boolean shareAll) {
        this(name, smatrix, toList(rowNames), toList(colNames), shareAll, shareAll, shareAll);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param sdfs
     */
    public StringDataframe(final String name,
                           final StringDataframe[] sdfs) {
        this(name, new StringMatrix(extractMatrices(sdfs)), extractRowNames(sdfs), sdfs[0].getColumnNames(), true);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param sdfs
     */
    public StringDataframe(final String name,
                           final List sdfs) {
        this(name, new StringMatrix(extractMatrices(sdfs)), extractRowNames(sdfs),
                ((StringDataframe) sdfs.get(0)).getColumnNames(), true);
    }


    public StringDataframe(final Dataset ds) {
        init(ds.getName(), new StringMatrix(ds.getMatrix()), ds.getRowNames(), ds.getColumnNames());
    }

    /**
     * @param name
     * @param lvs
     */
    public StringDataframe(final String name,
                           final LabelledVector[] lvs,
                           final boolean enforceEqualLength,
                           final int precision) {

        if (lvs == null) {
            throw new IllegalArgumentException("Param lvs cannot be null");
        }

        if (lvs.length == 0) {
            throw new IllegalArgumentException("Param lvs cannot be 0 length array: " + lvs.length);
        }

        if (enforceEqualLength) {

            if (lvs.length == 0) {
                throw new IllegalArgumentException("Zero length lvs not allowed: " + lvs.length);
            }

            List rowNamesOrder = lvs[0].getLabels();
            int expectedSize = lvs[0].getSize();

            StringMatrix sm = new StringMatrix(rowNamesOrder.size(), lvs.length);
            List colNames = new ArrayList();
            for (int c = 0; c < lvs.length; c++) {
                if (lvs[c].getSize() != expectedSize) {
                    throw new IllegalArgumentException("UNequal length lvs: " + lvs[c].getSize() + " expecting: " + expectedSize + " at index pos: " + c);
                }
                for (int r = 0; r < rowNamesOrder.size(); r++) {
                    sm.setElement(r, c, Printf.format(lvs[c].getScore(rowNamesOrder.get(r).toString()), precision));
                }
                colNames.add(lvs[c].getName());
            }

            init(name, sm, rowNamesOrder, colNames);
        } else {

            if (lvs.length == 0) {
                throw new IllegalArgumentException("Zero length lvs not allowed: " + lvs.length);
            }


            LabelledVector maxSize_lv = LabelledVector.getMaxSize_lv(lvs);
            int numRows = maxSize_lv.getSize();
            List rowNames = maxSize_lv.getLabels();

            StringMatrix sm = new StringMatrix(numRows, lvs.length);
            List colNames = new ArrayList();
            for (int c = 0; c < lvs.length; c++) {
                for (int r = 0; r < numRows; r++) {
                    String rowName = (String) rowNames.get(r);
                    if (lvs[c].isMember(rowName)) {
                        sm.setElement(r, c, Printf.format(lvs[c].getScore(rowName), precision));
                    } else {
                        sm.setElement(r, c, Constants.NA);
                    }
                }

                colNames.add(lvs[c].getName());
            }

            init(name, sm, rowNames, colNames);
        }
    }

    // simply places in consecutive columns each labelled vector
    public StringDataframe(final String name,
                           final LabelledVector[] lvs) {

        if (lvs == null) {
            throw new IllegalArgumentException("Param lvs cannot be null");
        }

        if (lvs.length == 0) {
            throw new IllegalArgumentException("Param lvs cannot be 0 length array: " + lvs.length);
        }

        if (lvs.length == 0) {
            throw new IllegalArgumentException("Zero length lvs not allowed: " + lvs.length);
        }

        LabelledVector maxSize_lv = LabelledVector.getMaxSize_lv(lvs);
        int numRows = maxSize_lv.getSize();

        StringMatrix sm = new StringMatrix(numRows, lvs.length * 2); // labels AND scores
        List colNames = new ArrayList();
        List rowNames = maxSize_lv.getLabels();
        int coln = 0;
        for (int c = 0; c < lvs.length; c++) {
            sm.setColumn(coln++, lvs[c].getLabels(), true);
            sm.setColumn(coln++, lvs[c].getScoresV(false), true);
            colNames.add(lvs[c].getName());
            colNames.add(lvs[c].getName() + "_score");
        }

        init(name, sm, rowNames, colNames);
    }

    public StringDataframe(final String name,
                           final StringMatrix smatrix,
                           final String[] colNames,
                           final boolean shareAll) {
        this(name, smatrix, createSeriesStrings(0, smatrix.getNumRow(), "row_"),
                toList(colNames), shareAll, shareAll, shareAll);
    }

    public StringDataframe(final String name,
                           final StringMatrix smatrix,
                           final List colNames,
                           final boolean shareAll) {
        this(name, smatrix, createSeriesStrings(0, smatrix.getNumRow(), "row_"), colNames, shareAll, true, shareAll);
    }

    public StringDataframe(final String name,
                           final StringMatrix smatrix,
                           boolean shareMatrix) {
        this(name, smatrix, createSeriesStrings(0, smatrix.getNumRow()),
                createSeriesStrings(0, smatrix.getNumCol(), "col_"), shareMatrix, true, true);
    }

    public StringDataframe(final String name,
                           final StringMatrix smatrix,
                           final List rowNames,
                           final List colNames,
                           final boolean shareMatrix,
                           final boolean shareRowNames,
                           final boolean shareColNames) {

        if (smatrix == null) {
            throw new IllegalArgumentException("Param matrix cant be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Param rowNames cant be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("Param rowNames cant be null");
        }

        //log.debug(rowNames);

        StringMatrix dmatrix;
        List drowNames;
        List dcolNames;

        if (shareMatrix) {
            dmatrix = smatrix;
        } else {
            dmatrix = smatrix.cloneDeep();
        }

        if (shareRowNames) {
            drowNames = rowNames;
        } else {
            drowNames = new ArrayList(rowNames);
        }

        if (shareColNames) {
            dcolNames = colNames;
        } else {
            dcolNames = new ArrayList(colNames);
        }

        init(name, dmatrix, drowNames, dcolNames);
    }

    public StringDataframe(final String name,
                           final List[] columnContents,
                           final String[] colNames) {

        if (columnContents == null) {
            throw new IllegalArgumentException("Param matrix cant be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("Param rowNames cant be null");
        }

        int numRows = 0;
        for (int i = 0; i < columnContents.length; i++) {
            if (numRows < columnContents[i].size()) {
                numRows = columnContents[i].size();
            }
        }

        final StringMatrix sm = new StringMatrix(numRows, columnContents.length);
        for (int c = 0; c < columnContents.length; c++) {
            for (int r = 0; r < columnContents[c].size(); r++) {
                sm.setElement(r, c, columnContents[c].get(r).toString());
            }
        }

        final List rowNames = new ArrayList();
        for (int i = 0; i < numRows; i++) {
            rowNames.add("row_" + (i + 1));
        }

        init(name, sm, rowNames, Arrays.asList(colNames));
    }

    /**
     * The ranked list need NOT be of equal lengths!
     *
     * @param name
     * @param rls
     */
    public StringDataframe(final String name,
                           final RankedList[] rls) {

        int nrows = AlgUtils.getMaxSize(rls);
        List rowNames = createSeriesStrings(0, nrows, "row_");
        List colNames = new ArrayList();

        StringMatrix sm = new StringMatrix(nrows, rls.length * 2);
        int coln = 0;
        for (int r = 0; r < rls.length; r++) {
            colNames.add(rls[r].getName());
            colNames.add(rls[r].getName() + "_SCORE");
            sm.setColumn(coln++, buffer(rls[r].getRankedNames(), nrows));
            sm.setColumn(coln++, buffer(rls[r].getScoresV(false), nrows));
        }

        init(name, sm, rowNames, colNames);
    }

    private static String[] buffer(List list, int lengthReqd) {
        String[] ret = new String[lengthReqd];
        for (int i = 0; i < lengthReqd; i++) {
            if (i < list.size()) {
                ret[i] = list.get(i).toString();
            } else {
                ret[i] = "";
            }
        }

        return ret;
    }

    private static Vector buffer(Vector v, int lengthReqd) {
        Vector ret = new Vector(lengthReqd);
        for (int i = 0; i < lengthReqd; i++) {
            if (i < v.getSize()) {
                ret.setElement(i, v.getElement(i));
            } else {
                ret.setElement(i, Float.NaN);
            }
        }

        return ret;
    }

    public StringDataframe(final String name,
                           final StringMatrix smatrix,
                           final List rowNames,
                           final String[] colNames,
                           final boolean shareMatrix,
                           final boolean shareRowNames) {
        this(name, smatrix, rowNames, toList(colNames), shareMatrix, shareRowNames, true);
    }

    // each List element must contain a String[] CAN be of different lenghts)
    public StringDataframe(final String name,
                           final List strArrLists,
                           final List rowOrColNames,
                           final boolean isRowLists) {

        if (rowOrColNames.size() != strArrLists.size()) {
            throw new IllegalArgumentException("Mismatched sizes: " + rowOrColNames.size() + " and " + strArrLists.size());
        }

        int maxlen = 0;
        for (int i = 0; i < strArrLists.size(); i++) {
            if (strArrLists.get(i) == null) {
                continue; // allowed
            }

            //noinspection PointlessBooleanExpression
            if (strArrLists.get(i) instanceof String[] == false) {
                throw new IllegalArgumentException("List elements must all be String[] objects");
            }

            String[] ss = (String[]) strArrLists.get(i);
            if (maxlen < ss.length) {
                maxlen = ss.length;
            }
        }

        int nrows;
        int ncols;
        StringMatrix smatrix;
        List rowNames;
        List colNames;


        if (isRowLists) {
            nrows = strArrLists.size(); // each String[] goes in a row
            ncols = maxlen;
            smatrix = new StringMatrix(nrows, ncols);
            for (int r = 0; r < nrows; r++) {
                smatrix.setRow(r, (String[]) strArrLists.get(r));
            }

            colNames = new ArrayList();
            for (int c = 0; c < ncols; c++) {
                colNames.add(c + "");
            }

            rowNames = rowOrColNames;

        } else {
            ncols = strArrLists.size(); // each String[] goes in a col
            nrows = maxlen;
            smatrix = new StringMatrix(nrows, ncols);
            for (int c = 0; c < ncols; c++) {
                String[] acol = (String[]) strArrLists.get(c); // one col of data
                for (int r = 0; r < acol.length; r++) {
                    smatrix.setElement(r, c, acol[r]);
                }
            }

            colNames = rowOrColNames;

            rowNames = new ArrayList();
            for (int r = 0; r < nrows; r++) {
                rowNames.add(r + "");
            }
        }

        init(name, smatrix, rowNames, colNames);
    }

    /**
     * assumes that when i am called the data has already been duplicated
     * i.e callers within this method are responsible for passing me
     * already duplicated data.
     */
    protected void init(final String name,
                        final StringMatrix smatrix,
                        final List rowNames,
                        final List colNames) {

        super.initialize(name);

        if (smatrix.getNumRow() != rowNames.size()) {
            throw new IllegalArgumentException("Matrix: " + smatrix.getNumRow() + " and rowNames: "
                    + rowNames.size() + " do not match in size");
        }

        if (smatrix.getNumCol() != colNames.size()) {
            throw new IllegalArgumentException("Matrix: " + smatrix.getNumCol() + " and colNames: "
                    + colNames.size() + " do not match in size");
        }

        if (smatrix.getNumCol() == 0) {
            log.warn("zero cols in StringMatrix");
        }

        if (smatrix.getNumRow() == 0) {
            //TraceUtils.showTrace();
            log.debug("zero rows in StringMatrix");
        }

        this.fMatrix = smatrix;
        this.fMatrix.setImmutable();    // IMP notice.

        this.fRowNames = rowNames;
        this.fColNames = colNames;
    }

    public String[] getRow(final int rown) {
        return fMatrix.getRow(rown);
    }

    public String[] getRow(final String rowName) {
        if (rowName == null) {
            throw new IllegalArgumentException("Parameter rowName cannot be null");
        }

        int rowIndex = getRowIndex(rowName);

        // try harder
        if (rowIndex == -1) {
            rowIndex = getRowIndex(AuxUtils.getAuxNameOnlyNoHash(rowName));
        }

        if (rowIndex == -1) {
            throw new IllegalArgumentException("No such row name: " + rowName + "\nRow Names are: " + Printf.outs(getRowNames()));
        }

        return getRow(rowIndex);
    }

    public String[] getColumn(final int coln) {
        return fMatrix.getColumn(coln);
    }

    public boolean hasColumn(final String colName) {
        int index = getColumnIndex(colName);
        if (index == -1) {
            return false;
        } else {
            return true;
        }
    }

    public String[] getColumn(final String colName) {
        int colIndex = getColumnIndex(colName);
        if (colIndex == -1) {
            throw new IllegalArgumentException("No such column: >" + colName + "<");
        }
        return fMatrix.getColumn(colIndex);
    }


    public String[] getColumnAsStrings(int coln) {
        return Helper.getColumnAsStrings(this, coln);
    }

    public String[] getColumnAsStrings(final String colName) {
        return Helper.getColumnAsStrings(this, colName);
    }

    public Vector getColumnAsVector(String colName, boolean omitNulls) {
        return getColumnAsVector(getColumnIndex(colName), omitNulls);
    }

    public Vector getColumnAsVector(int coln, boolean omitNulls) {
        TFloatArrayList floats = new TFloatArrayList();
        for (int r = 0; r < getNumRow(); r++) {
            String s = getElement(r, coln);
            if (s == null && omitNulls) {
                // do nothing
            } else if (s == null && !omitNulls) {
                floats.add(Float.NaN); // @note
            } else {
                floats.add(Float.parseFloat(s));
            }
        }

        return new Vector(floats);
    }

    public LabelledVector getColumnAsLV(final int coln) {
        return new LabelledVector("col_" + coln, getRowNamesArray(), getColumnAsVector(coln, false));
    }

    public LabelledVector getColumnAsLV(final String colName) {
        return new LabelledVector(colName, getRowNamesArray(), getColumnAsVector(colName, false));
    }

    public RankedList getColumnAsRL(final String colName) {
        return new LabelledVector(colName, getRowNamesArray(), getColumnAsVector(colName, false)).sort(SortMode.REAL, Order.DESCENDING);
    }

    public List getColumnAsList(final int coln) {
        return fMatrix.getColumnL(coln);
    }

    // duplicates will be overwriten
    public Set getColumnAsSet(final int coln) {
        Set set = new HashSet();
        for (int r = 0; r < getNumRow(); r++) {
            set.add(fMatrix.getElement(r, coln));
        }

        return set;
    }

    public String getElement(final int rown, final int coln) {
        return fMatrix.getElement(rown, coln);
    }

    public float getElementAsFloat(final int rown, final int coln) {
        return Float.parseFloat(fMatrix.getElement(rown, coln));
    }

    public float getElementAsFloat(final int rown, final String colName) {
        return Float.parseFloat(fMatrix.getElement(rown, getColumnIndex(colName)));
    }

    public String getElement(final String rowName, final int coln) {
        return fMatrix.getElement(getRowIndex(rowName), coln);
    }

    public String getElement(final int row, final String colName) {
        return fMatrix.getElement(row, getColumnIndex(colName));
    }

    public Object getElementObj(final int rown, final int coln) {
        return getElement(rown, coln);
    }

    public String getRowName(int rown) {
        return (String) fRowNames.get(rown);
    }

    public List getRowNames() {
        return Collections.unmodifiableList(fRowNames);
    }

    public int getRowIndex(String rowname) {
        return fRowNames.indexOf(rowname);
    }

    public String[] getRowNamesArray() {
        return (String[]) fRowNames.toArray(new String[fRowNames.size()]);
    }

    public List getColumnNames() {
        return Collections.unmodifiableList(fColNames);
    }

    public int getColumnIndex(String colName) {
        int index = fColNames.indexOf(colName);
        /*
        if (index == -1) {
            throw new IllegalArgumentException("No such colName >" + colName + "<\n" + getColumnNames());
        }
           */
        return index;
    }

    public String[] getColumnNamesArray() {
        return (String[]) fColNames.toArray(new String[fColNames.size()]);
    }

    public String getColumnName(int coln) {
        return fColNames.get(coln).toString();
    }

    public int getNumRow() {
        return fMatrix.getNumRow();
    }

    public int getNumCol() {
        return fMatrix.getNumCol();
    }

    public void setRowLabelName(String name) {
        this.fRowLabelName = name;
    }

    public String getRowLabelName() {
        return fRowLabelName;
    }

    /**
     * @return An immutable view of the Matrix
     */
    public StringMatrix getMatrix() {
        fMatrix.setImmutable();
        return fMatrix;
    }

    /**
     * unique elements across rows and columns
     *
     * @return
     */
    public String[] getUniqueElements() {

        Set uniqs = new HashSet();

        for (int r = 0; r < getNumRow(); r++) {
            for (int c = 0; c < this.getNumCol(); c++) {
                uniqs.add(getElement(r, c));
            }
        }

        return (String[]) uniqs.toArray(new String[uniqs.size()]);
    }

    public String getQuickInfo() {
        StringBuffer buf = new StringBuffer().append(getNumRow()).append('x').append(getNumCol());
        return buf.toString();
    }

    //-------------------- private static utility methods --------------//

    public static List createSeriesStrings(int start, int stop, String commonPrefix) {
        List list = new ArrayList(stop - start);
        for (int i = start; i < stop; i++) {
            list.add(commonPrefix + i);
        }

        return list;
    }

    private static List toList(String[] ss) {
        List list = new ArrayList(ss.length);
        for (int i = 0; i < ss.length; i++) {
            list.add(ss[i]);
        }

        return list;
    }

    private static StringMatrix[] extractMatrices(StringDataframe[] ss) {
        List list = new ArrayList(ss.length);
        for (int i = 0; i < ss.length; i++) {
            list.add(ss[i].getMatrix());
        }

        return (StringMatrix[]) list.toArray(new StringMatrix[list.size()]);
    }

    private static StringMatrix[] extractMatrices(List sdfs) {
        List list = new ArrayList(sdfs.size());
        for (int i = 0; i < sdfs.size(); i++) {
            list.add(((StringDataframe) sdfs.get(i)).getMatrix());
        }

        return (StringMatrix[]) list.toArray(new StringMatrix[list.size()]);
    }

    private static List createSeriesStrings(int start, int stop) {
        List list = new ArrayList(stop - start);
        for (int i = start; i < stop; i++) {
            list.add(i + "");
        }

        return list;
    }

    private static List extractRowNames(final StringDataframe[] sdfs) {
        List all = new ArrayList();
        for (int i = 0; i < sdfs.length; i++) {
            all.addAll(sdfs[i].getRowNames());
        }

        return all;
    }

    private static List extractRowNames(final List sdfs) {
        List all = new ArrayList();
        for (int i = 0; i < sdfs.size(); i++) {
            all.addAll(((StringDataframe) sdfs.get(i)).getRowNames());
        }

        return all;
    }

    public void replace(String thisStr, String withThisStr) {
        fMatrix.replace(thisStr, withThisStr);
    }

}    // End StringDataframe

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
