package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.Errors;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.objects.strucs.NamedColor;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.utils.CollectionUtils;
import edu.mit.broad.vdb.sampledb.Sample;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import org.apache.log4j.Logger;

import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Dec 3, 2005
 *         Time: 10:11:40 PM
 */
public class TemplateCohort extends AbstractObject {

    private Comp[] fComps;

    private static final Logger klog = XLogger.getLogger(TemplateCohort.class);

    private SampleAnnot fSampleAnnot;

    private List fCompNames; // want this to be ordered the same as comps

    /**
     * Class constructor
     *
     * @param name
     * @param comps
     */
    public TemplateCohort(final String name, final Comp[] comps) {
        super.initialize(name);

        // make sure the comps dont have a shared name

        Set names = new HashSet();
        for (int i = 0; i < comps.length; i++) {
            if (names.contains(comps[i].getName())) {
                throw new IllegalArgumentException("Duplicate comp name: " + comps[i].getName() + " at " + i);
            }
            names.add(comps[i].getName());
        }

        this.fComps = comps;
    }

    public String getQuickInfo() {
        return null;
    }

    public SampleAnnot getSampleAnnot() {

        if (fSampleAnnot == null) {
            ColumnsImpl ci = new ColumnsImpl(getNamedColors());
            this.fSampleAnnot = new SampleAnnotImpl(getName(), getCompNames(), ci);
        }

        return fSampleAnnot;
    }

    public int getNumComps() {
        return fComps.length;
    }

    public Comp getComp(final int i) {
        return fComps[i];
    }

    public String[] getCompNames() {
        if (this.fCompNames == null) {
            this.fCompNames = new ArrayList();
            for (int c = 0; c < getNumComps(); c++) {
                fCompNames.add(getComp(c).getName());
            }
        }

        return (String[]) fCompNames.toArray(new String[fCompNames.size()]);
    }

    public NamedColor[] getNamedColors() {
        final NamedColor[] ncs = new NamedColor[getNumComps()];
        for (int r = 0; r < getNumComps(); r++) {
            ncs[r] = new NamedColor(getComp(r).getClassAShortName(), getComp(r).getColor());
        }

        return ncs;
    }

    public Comp getComp(final String compName) {

        initCompMap();
        Object obj = compNameCompMap.get(compName);
        if (obj == null) {
            throw new IllegalArgumentException("No such comp: " + compName);
        } else {
            return (Comp) obj;
        }

    }

    private Map compNameCompMap;

    private void initCompMap() {
        if (compNameCompMap == null) {
            this.compNameCompMap = new HashMap();
            for (int c = 0; c < getNumComps(); c++) {
                compNameCompMap.put(getComp(c).getName(), getComp(c));
            }
        }
    }

    public boolean hasComp(final String compName) {
        initCompMap();
        return compNameCompMap.containsKey(compName);
    }

    // @note  IMP the templates will NOT be synched with the annot!!
    public Template[] createAllTemplates(final SampleAnnot annot) {
        Template[] tss = new Template[getNumComps()];
        for (int i = 0; i < getNumComps(); i++) {
            tss[i] = getComp(i).createTemplate(annot);
        }

        return tss;
    }

    public String[] getAllSampleNamesArray(final SampleAnnot annot) {

        Set set = getAllSampleNames(annot);
        return (String[]) set.toArray(new String[set.size()]);
    }

    public Set getAllSampleNames(final SampleAnnot annot) {
        Set set = new HashSet();
        for (int i = 0; i < getNumComps(); i++) {
            set.addAll(getComp(i).getSampleNames(annot, true));
            set.addAll(getComp(i).getSampleNames(annot, false));
        }

        return set;
    }

    /**
     * Inner class reprsenting one comparison
     */
    public static class Comp {

        private String fName;
        private Color fColor;

        private String fClassAShortName;
        private String fClassBShortName;

        private Color fClassAColor_opt;
        private Color fClassBColor_opt;

        private String fClassASpec;
        private String fClassBSpec;

        private String fCategoryName;

        /**
         * Class constructor
         * <p/>
         * <p/>
         * IMP:
         * class A and class Bnames might have +'s in them:
         *
         * @param name
         * @param classASpec
         * @param classBSpec
         */
        public Comp(final String name,
                    final Color color,
                    final String categoryName,
                    final String classAShortName,
                    final Color classAColor_opt,
                    final String classASpec,
                    final String classBShortName,
                    final Color classBColor_opt,
                    final String classBSpec) {

            this.fName = name;

            sanityCheck(classASpec, classBSpec);
            sanityCheck(classAShortName, classBShortName);

            if (categoryName == null || categoryName.length() == 0) {
                throw new IllegalArgumentException("Param categoryName cannot be null");
            }

            this.fCategoryName = categoryName;

            this.fColor = color;
            this.fClassAShortName = classAShortName;
            this.fClassBShortName = classBShortName;

            this.fClassASpec = classASpec;
            this.fClassBSpec = classBSpec;

            this.fClassAColor_opt = classAColor_opt;
            this.fClassBColor_opt = classBColor_opt;
        }

        private static void sanityCheck(String a, String b) {

            if (a == null || b == null) {
                throw new IllegalArgumentException("Class names cannot be null: >" + a + "< >" + b + "<");
            }

            if (a.length() == 0 || b.length() == 0) {
                throw new IllegalArgumentException("Class names cannot be zero: >" + a + "< >" + b + "<");
            }

            if (a.equalsIgnoreCase(b)) {
                throw new IllegalArgumentException("Class names cannot be identical: >" + a + "< >" + b + "<");
            }

        }

        public Color getColor() {
            if (fColor == null) {
                return Color.WHITE;
            } else {
                return fColor;
            }
        }

        public String getCategoryName() {
            return fCategoryName;
        }

        public String getClassASpec() {
            return fClassASpec;
        }

        public String getClassBSpec() {
            return fClassBSpec;
        }

        public String getClassAShortName() {
            return fClassAShortName;
        }

        public String getClassBShortName() {
            return fClassBShortName;
        }

        public Color getClassAColor() {
            return fClassAColor_opt;
        }

        public Color getClassBColor() {
            return fClassBColor_opt;
        }

        public String getClassShortName(boolean classA) {
            if (classA) {
                return getClassAShortName();
            } else {
                return getClassBShortName();
            }
        }

        public String getClassSpec(boolean classA) {
            if (classA) {
                return getClassASpec();
            } else {
                return getClassBSpec();
            }
        }

        public String getName() {
            return fName;
        }

        public String toString() {
            return fName;
        }

        public List getSampleNames(final SampleAnnot annot,
                                   final boolean classA) {

            if (annot == null) {
                throw new IllegalArgumentException("Param annot cannot be null");
            }

            String className = getClassSpec(classA);

            Set classNames = ParseUtils.string2stringsSet(className, "+", false);
            // IMP need to UC this
            classNames = CollectionUtils.toUpperCase(classNames);

            List useColNames = new ArrayList();
            for (int c = 0; c < annot.getNumSamples(); c++) {
                Sample sample = annot.getSample(c);
                String catValue = sample.getCategoryValue(getCategoryName()).toUpperCase();
                boolean isClass = classNames.contains(catValue);
                if (isClass) {
                    useColNames.add(sample.getName());
                }
            }

            return useColNames;
        }

        public SampleAnnot createSampleAnnot(final SampleAnnot annot,
                                             final boolean classA) {

            List sampleNames = getSampleNames(annot, classA);
            return annot.cloneDeep((String[]) sampleNames.toArray(new String[sampleNames.size()]));
        }


        // @note IMP IMP the tch has no idea what order the samples occur in the dataset
        // (it simply makes the template based on the order of samples it finds in the sin file)
        // so what it thinks is class A need not be the same class A as retuned in the subt
        // (thats why it returns a dt and not just a dataset)
        public DatasetTemplate createDatasetTemplate(final Dataset ds) {

            // @note IMP the createTemplate method simply makes things based on the order in the ann
            // but the order in the ann is not necc same as the order in the ds
            // so m,ake sure to clone

            //Template t = createTemplate(ds.getAnnot().getSampleAnnot());
            Template tx = createTemplate(ds.getAnnot().getSampleAnnot_synched(ds.getColumnNamesArray()));
            DatasetTemplate dt = new DatasetGenerators().extract(ds, tx);

            // we want to maintain 2 things:
            // 1) the contiguity that the template in dt has
            // (the t has no necc contiguity because it simply takes the sin positions)
            // 2) the class a class b order specified in t (i.e the comp file)
            // 3) ->>> the MAIn issue is that the comp file is made w/o any knowledge of the sample order but it anyways contains a class order

            //String classAName_comp = t.getClass(0).getName(); // @note pointless
            String classAName_comp = getClassAShortName(); // @note

            String classAName_dt = dt.getTemplate().getClass(0).getName();

            if (classAName_comp.equals(classAName_dt)) {
                klog.debug("Already ordered: " + classAName_comp + " " + dt.getTemplate().getName());
                //sanityCheck(t, dt.getTemplate());
                return dt; // Already in synch
            } else {
                // not this only synchs the CLASS ORDER (does NOT change the profile positions
                // so no need to extract a new dataset
                Template revt = TemplateFactory.createReversedBiphasicTemplate2(dt.getTemplate()); // @note dt not t
                klog.debug("Reversed template from: >" + dt.getTemplate().getName() + " " +
                        dt.getTemplate().getClass(0).getName() + " " + dt.getTemplate().getClass(1).getName() +
                        "< to rev >" + revt.getName() +
                        " " + revt.getClass(0).getName() + " " + revt.getClass(1).getName() + "<" +
                        " comp: >" + classAName_comp + " " + getClassBShortName() + "<");

                //DatasetTemplate rdt = new DatasetGenerators().extract(ds, revt);
                //klog.debug("After revex RDT: >" + rdt.getTemplate().getClass(0).getName() + " " + rdt.getTemplate().getClass(1).getName() + "<");
                //Printf.out(revt);
                //Printf.out(rdt.getTemplate());

                return new DatasetTemplate(dt.getDataset(false), revt);
            }
        }

        private static void sanityCheck(Template a, Template b) {

            Errors errors = new Errors();
            for (int i = 0; i < a.getNumClasses(); i++) {
                if (a.getClass(i).getName().equals(b.getClass(i).getName()) == false) {
                    errors.add(a.getClass(i).getName() + " and " + b.getClass(i).getName() + " at loc: " + i);
                }
            }


            errors.barfIfNotEmptyRuntime();
        }

        // @note IMP IMP the class order in the created template need not be the same as the order
        // in the comparison. This is because the template is always sorted by the order of samples
        // in the annot.
        public Template createTemplate(final SampleAnnot annot) {

            if (annot == null) {
                throw new IllegalArgumentException("Param annot cannot be null");
            }

            List samplesInClassA = getSampleNames(annot, true);
            List samplesInClassB = getSampleNames(annot, false);

            if (samplesInClassA.isEmpty() || samplesInClassB.isEmpty()) {
                throw new IllegalArgumentException("No matches to some class specs\nfClassASpec: " +
                        fClassASpec + " matches: " + samplesInClassA.size() + "\nfClassBSpec: " +
                        fClassBSpec + " matches: " + samplesInClassB.size());
            }

            /* wrong --- see above.
            final Template.Class[] classes = new Template.Class[]{ // @note order doesnt matter i think
                    new TemplateImpl.ClassImpl(fClassAName),
                    new TemplateImpl.ClassImpl(fClassBName)};
                    */

            final List items = new ArrayList();
            for (int c = 0; c < annot.getNumSamples(); c++) {
                String sampleName = annot.getSample(c).getName();
                if (samplesInClassA.contains(sampleName)) {
                    items.add(TemplateImpl.ItemImpl.createItem(fClassAShortName, c));
                } else if (samplesInClassB.contains(sampleName)) {
                    items.add(TemplateImpl.ItemImpl.createItem(fClassBShortName, c));
                } else {
                    // omit from template
                }
            }

            return TemplateFactory.createTemplate(getName(),
                    (Template.Item[]) items.toArray(new Template.Item[items.size()]), false);

        }

    } // End inner class Comp


} // End class TemplateCohort
