/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.ParserFactory;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Dec 2, 2004
 *         Time: 2:54:51 PM
 */
public class TemplateDerivatives {


    public static class ContTemplateDerivative implements TemplateDerivative {

        private String fAuxName;
        private String fJustName;
        private File fMainTemplateFile;

        // auxName is of the form foo#bar (and not just bar)
        public ContTemplateDerivative(final String auxName, final File mainTemplateFile) {
            if (auxName == null) {
                throw new IllegalArgumentException("Parameter auxName cannot be null");
            }
            if (mainTemplateFile == null) {
                throw new IllegalArgumentException("Parameter mainTemplateFile cannot be null");
            }

            this.fAuxName = auxName;
            this.fMainTemplateFile = mainTemplateFile;

            this.fJustName = AuxUtils.getAuxNameOnlyNoHash(fAuxName);
        }

        public Template getMainTemplate() {
            throw new IllegalStateException();
        }

        public String getName(final boolean parentNamePlusMyName, final boolean fullPath) {

            String ret;

            if (fullPath) {
                ret = fMainTemplateFile.getPath() + "#";
            } else if (parentNamePlusMyName) {
                ret = fMainTemplateFile.getName() + "#";
            } else {
                ret = "";
            }

            return ret + fJustName; // imp and not fAuxName as that has the template name on it too
        }

    } // End class AuxTemplateDerivative

    public static class PseudoTemplateDerivative implements TemplateDerivative {

        private Template fMainTemplate;

        // auxName is of the form foo#bar (and not just bar)
        public PseudoTemplateDerivative(final Template mainTemplate) {
            if (mainTemplate == null) {
                throw new IllegalArgumentException("Parameter mainTemplate cannot be null");
            }

            this.fMainTemplate = mainTemplate;
        }

        public Template getMainTemplate() {
            return fMainTemplate;
        }

        public String getName(final boolean parentNamePlusMyName, final boolean fullPath) {

            if (fullPath) {
                return ParserFactory.getCache().getSourcePath(fMainTemplate);
            } else {
                return fMainTemplate.getName();
            }

        }

    } // End class PseudoTemplateDerivative

    public static class AuxTemplateDerivative implements TemplateDerivative {

        private String fAuxName;
        private String fJustName;
        private Template fMainTemplate;

        // auxName is of the form foo#bar (and not just bar)
        public AuxTemplateDerivative(final String auxName, final Template mainTemplate) {
            if (auxName == null) {
                throw new IllegalArgumentException("Parameter auxName cannot be null");
            }
            if (mainTemplate == null) {
                throw new IllegalArgumentException("Parameter mainTemplate cannot be null");
            }

            this.fAuxName = auxName;
            this.fMainTemplate = mainTemplate;


            this.fJustName = AuxUtils.getAuxNameOnlyNoHash(fAuxName);
        }

        public Template getMainTemplate() {
            return fMainTemplate;
        }

        public String getName(final boolean parentNamePlusMyName, final boolean fullPath) {

            String ret;

            if (fullPath) {
                ret = ParserFactory.getCache().getSourcePath(fMainTemplate) + "#";
            } else if (parentNamePlusMyName) {
                ret = fMainTemplate.getName() + "#";
            } else {
                ret = "";
            }

            return ret + fJustName; // imp and not fAuxName as that has the template name on it too
        }

    } // End class AuxTemplateDerivative


    interface AutoTemplateSplitter extends TemplateDerivative {

    }

    /**
     * struc representing a template and all ts possible ovas
     */
    public static class OvaTemplate implements AutoTemplateSplitter {

        private Template fMainTemplate;

        private Template[] fOvas;

        public OvaTemplate(final Template mainTemplate) {
            this.fMainTemplate = mainTemplate;
        }

        public String getName(final boolean parentNamePlusMyName, final boolean fullPath) {

            String ret;

            if (fullPath) {
                ret = ParserFactory.getCache().getSourcePath(fMainTemplate) + "#";
            } else if (parentNamePlusMyName) {
                ret = fMainTemplate.getName() + "#";
            } else {
                ret = "";
            }

            return ret + Constants.ONE_VERSUS_ALL;
        }

        public Template[] createOvaTemplates() {
            if (fOvas == null) {
                this.fOvas = TemplateFactory.extractAllOvaTemplates(fMainTemplate, false);
            }
            return fOvas;
        }

        public Template getMainTemplate() {
            return fMainTemplate;
        }

    } // End class OvaTemplate

    public static class OvaOnlyForwardTemplate implements AutoTemplateSplitter {

        private Template fMainTemplate;

        private Template[] fOvas;

        public OvaOnlyForwardTemplate(final Template mainTemplate) {
            this.fMainTemplate = mainTemplate;

        }

        public Template[] createOvaTemplates() {
            if (fOvas == null) {
                this.fOvas = TemplateFactory.extractAllOvaTemplates(fMainTemplate, true);
            }
            return fOvas;
        }

        public Template getMainTemplate() {
            return fMainTemplate;
        }

        public String getName(final boolean parentNamePlusMyName, final boolean fullPath) {

            String ret;

            if (fullPath) {
                ret = ParserFactory.getCache().getSourcePath(fMainTemplate) + "#";
            } else if (parentNamePlusMyName) {
                ret = fMainTemplate.getName() + "#";
            } else {
                ret = "";
            }

            return ret + Constants.ONE_VERSUS_ALL_ONLY_FORWARD;
        }

    } // End class OvaTemplate

    /**
     * struc representing a templat and all ts possible ovas
     */
    public static class AllPairsTemplate implements AutoTemplateSplitter {

        private Template fMainTemplate;

        private Template[] fAllPairs;

        public AllPairsTemplate(final Template mainTemplate) {
            this.fMainTemplate = mainTemplate;

        }

        public Template[] createAllPairsTemplates() {
            if (fAllPairs == null) {
                this.fAllPairs = TemplateFactory.extractAllPairsTemplates(fMainTemplate);
            }

            return fAllPairs;
        }

        public Template getMainTemplate() {
            return fMainTemplate;
        }

        public String getName(final boolean parentNamePlusMyName, final boolean fullPath) {

            String ret;

            if (fullPath) {
                ret = ParserFactory.getCache().getSourcePath(fMainTemplate) + "#";
            } else if (parentNamePlusMyName) {
                ret = fMainTemplate.getName() + "#";
            } else {
                ret = "";
            }

            return ret + Constants.ALL_PAIRS;
        }

    } // End class OvaTemplate

}
