/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.objects;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.RandomSeedGenerator;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.vdb.sampledb.Sample;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.vdb.sampledb.State;
import gnu.trove.TIntArrayList;
import gnu.trove.TIntHashSet;
import gnu.trove.TIntObjectHashMap;
import org.apache.log4j.Logger;
import edu.duke.igsp.app.objects.SnpDataset;
import edu.duke.igsp.app.objects.StringMatrix;
import edu.duke.igsp.app.objects.NumDataset;

import java.util.*;


/**
 * A Template object that is mutable i.e has add() methods.
 * Generally used only by the ClsParser and methods(in here)that make new Templates.
 * <p/>
 * Also, factor methods for temnplates kept here so that we can control generation
 * (and access to non public fields needed for generation)
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class TemplateFactory extends TemplateImpl {

    private static final Logger klog = XLogger.getLogger(TemplateFactory.class);

    private static final String VS = "_versus_";

    /**
     * Class Constructor.
     * Creates a new TemplateFactory.
     */
    private TemplateFactory() {
    }

    public static class Struc2 {
        public Template template;
        public Sample[] samples_in_order;
    }


    // Each value becomes a different template class
    // This is the CORE template making method
    // AUTO makes the classes
    // @note need to order the samples by their order in the annot!!
    public static Struc2 createTemplate_struc(final String name,
                                              final State[] onlyUseTheseValues,
                                              final SampleAnnot annot) {

        List itemList = new ArrayList();
        for (int i = 0; i < onlyUseTheseValues.length; i++) {
            final Sample[] samples = onlyUseTheseValues[i].getSamples();
            for (int s = 0; s < samples.length; s++) {
                // @note not all samples in annot may be used
                // but the profile pos is maintained
                Template.Item item = TemplateImpl.ItemImpl.createItem(onlyUseTheseValues[i].getName(),
                        annot.getSampleIndex(samples[s]));
                itemList.add(item);
                //klog.debug(item.getId() + "\t" + item.getProfilePosition());
            }
        }

        final Template.Item[] items = (Template.Item[]) itemList.toArray(new Template.Item[itemList.size()]);
        TIntObjectHashMap profilePosItemMap = hashProfilePosItemMap(items);
        final int[] profilePos_ordered = getProfilePositionsSorted(profilePosItemMap);
        final List classNames_ordered = new ArrayList();
        for (int i = 0; i < profilePos_ordered.length; i++) {
            Item item = (Item) profilePosItemMap.get(profilePos_ordered[i]);
            if (classNames_ordered.contains(item.getId()) == false) {
                classNames_ordered.add(item.getId());
                //klog.debug(">>" + item.getId() + "\t" + classNames_ordered.size());
            }
        }

        final Template.Class[] classes = new Template.Class[classNames_ordered.size()];
        for (int i = 0; i < classNames_ordered.size(); i++) {
            classes[i] = new TemplateImpl.ClassImpl(classNames_ordered.get(i).toString());
        }

        Struc2 struc = new Struc2();
        struc.template = TemplateFactory.createTemplate_ordered_assign(name, items, classes, false);
        struc.samples_in_order = annot.getSamples(profilePos_ordered);
        return struc;
    }

    public static TemplateFromAnnot createTemplate_from_annot(final String name, final State[] useOnlyTheseStates, final SampleAnnot annot) {
        Struc2 struc = createTemplate_struc(name, useOnlyTheseStates, annot);
        return new TemplateFromAnnot(struc.template, struc.samples_in_order, null, null);
    }


    public static Template createTemplate(final String name, final State[] onlyUseTheseStates, final SampleAnnot annot) {
        return createTemplate_struc(name, onlyUseTheseStates, annot).template;
    }

    /**
     * The dataset returned has its columns in the same order as the item's
     * primary positions in the template. So, the dataset returned is now synched to the
     * template. (BUT remember that the input template's profile positions are NO longer synched
     * with the return dataset (if it was extracted)).
     *
     * @param fullDs
     * @param origT
     * @return
     * @note ALSO contiguize the classes (and ofcourse dataset)
     * @note DatasetTemplate needed as the extracted ds and incoming t may not be synched anymore
     * They WILL have the same number of columns/items but the profile positions of the items
     * will refer to profile positions in the old (unextracted) dataset. If all you care about is
     * the extracted dataset, ignore the modified template.
     */
    public static synchronized DatasetTemplate extract(final Dataset fullDs,
                                                       final Template origT,
                                                       final boolean verbose) {

        if (fullDs == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        if (origT == null) {
            throw new IllegalArgumentException("Parameter template cannot be null");
        }

        for (int i = 0; i < origT.getNumClasses(); i++) {
            if (origT.getClass(i).getSize() == 0) {
                throw new IllegalArgumentException("In template: " + origT.getName() + " 0 members in class: " + origT.getClass(i).getName());
            }
        }

        // no synching needed if:
        // 1) the dataset and template are of identical length
        // 2) the primary positions of the template are in ascending order
        // as an opt check for this and return unchanged ds if true
        if (fullDs.getNumCol() == origT.getNumItems() && origT.isAscendingProfilePositions()) {
            if (verbose) {
                klog.info("Already a synched dataset-template -- NO extracting done");
            }
            return new DatasetTemplate(fullDs, origT); // no extraction needed
        }

        if (verbose) {
            klog.info("Extracting ds: " + fullDs.getName() + " by template: " + origT.getName());
        }

        final Matrix matrix = new Matrix(fullDs.getNumRow(), origT.getNumItems());
        final List colNames = new ArrayList(origT.getNumItems());
        final Template.Item[] newItems = new Template.Item[origT.getNumItems()];

        //final Template.Item[] orig_items = origT.getItemsOrderedByProfilePos();
        final Template.Item[] orig_items = origT.getItemsOrderedByClassFirstAndThenProfilePos();
        for (int c = 0; c < orig_items.length; c++) {
            // first the dataset
            final int col2use = orig_items[c].getProfilePosition(); // @note extracting using the profile position
            //log.debug("col2use: " + col2use);
            matrix.setColumn(c, fullDs.getColumn(col2use));
            colNames.add(fullDs.getColumnName(col2use));
            // then the new template
            newItems[c] = TemplateImpl.ItemImpl.createItem(orig_items[c].getId(), c); // @note profile pos is changed
        }

        final String name = NamingConventions.generateName(fullDs, origT, true);
        //log.debug("$$$ gen name: " + name + " " + maybeAuxTemplate.getName());
        DefaultDataset newds = new DefaultDataset(name, matrix, fullDs.getRowNames(), colNames, true, false, true, fullDs.getAnnot());
        newds.setProperty("extracted", "true"); // huh

        //klog.debug(">>>>>>>>>>>>>> origT: " + origT.getName() + " newds: " + newds.getName());

        final Template newt = createTemplate(origT.getName() + "_repos", newItems, origT.isContinuous());

        //klog.debug("orig class a: " + origT.getClassName(0) + " made: " + newt.getClassName(0));

        return new DatasetTemplate(newds, newt);
    }

    //added by Qing Xiong - qing.xiong@duke.edu
    public static synchronized DatasetTemplate extractSnp(final SnpDataset snpDs,
                                                       final Template origT,
                                                       final boolean verbose) {

        if (snpDs == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        if (origT == null) {
            throw new IllegalArgumentException("Parameter template cannot be null");
        }

        for (int i = 0; i < origT.getNumClasses(); i++) {
            if (origT.getClass(i).getSize() == 0) {
                throw new IllegalArgumentException("In template: " + origT.getName() + " 0 members in class: " + origT.getClass(i).getName());
            }
        }

        // no synching needed if:
        // 1) the dataset and template are of identical length
        // 2) the primary positions of the template are in ascending order
        // as an opt check for this and return unchanged ds if true
        if (snpDs.getNumCol() == origT.getNumItems() && origT.isAscendingProfilePositions()) {
            if (verbose) {
                klog.info("Already a synched dataset-template -- NO extracting done");
            }
            return new DatasetTemplate(snpDs, origT); // no extraction needed
        }

        if (verbose) {
            klog.info("Extracting ds: " + snpDs.getName() + " by template: " + origT.getName());
        }

        final StringMatrix matrix = new StringMatrix(snpDs.getNumRow(), origT.getNumItems());
        final List colNames = new ArrayList(origT.getNumItems());
        final Template.Item[] newItems = new Template.Item[origT.getNumItems()];

        //final Template.Item[] orig_items = origT.getItemsOrderedByProfilePos();
        final Template.Item[] orig_items = origT.getItemsOrderedByClassFirstAndThenProfilePos();
        for (int c = 0; c < orig_items.length; c++) {
            // first the dataset
            final int col2use = orig_items[c].getProfilePosition(); // @note extracting using the profile position
            //log.debug("col2use: " + col2use);
            matrix.setColumn(c, snpDs.getColumnStr(col2use));
            colNames.add(snpDs.getColumnName(col2use));
            // then the new template
            newItems[c] = TemplateImpl.ItemImpl.createItem(orig_items[c].getId(), c); // @note profile pos is changed
        }

        final String name = NamingConventions.generateName(snpDs, origT, true);
        //log.debug("$$$ gen name: " + name + " " + maybeAuxTemplate.getName());
        SnpDataset newds = new SnpDataset(name, matrix, snpDs.getRowNames(), colNames, snpDs.getAnn());
        newds.setGenes(snpDs.getGenes());
        newds.setProperty("extracted", "true"); // huh

        //klog.debug(">>>>>>>>>>>>>> origT: " + origT.getName() + " newds: " + newds.getName());

        final Template newt = createTemplate(origT.getName() + "_repos", newItems, origT.isContinuous());

        //klog.debug("orig class a: " + origT.getClassName(0) + " made: " + newt.getClassName(0));

        return new DatasetTemplate(newds, newt);
    }

    public static synchronized DatasetTemplate extractNumDataset(final NumDataset numDs,
                                                       final Template origT,
                                                       final boolean verbose) {

        if (numDs == null) {
            throw new IllegalArgumentException("Parameter fullDs cannot be null");
        }

        if (origT == null) {
            throw new IllegalArgumentException("Parameter template cannot be null");
        }

        for (int i = 0; i < origT.getNumClasses(); i++) {
            if (origT.getClass(i).getSize() == 0) {
                throw new IllegalArgumentException("In template: " + origT.getName() + " 0 members in class: " + origT.getClass(i).getName());
            }
        }

        // no synching needed if:
        // 1) the dataset and template are of identical length
        // 2) the primary positions of the template are in ascending order
        // as an opt check for this and return unchanged ds if true
        if (numDs.getNumCol() == origT.getNumItems() && origT.isAscendingProfilePositions()) {
            if (verbose) {
                klog.info("Already a synched dataset-template -- NO extracting done");
            }
            return new DatasetTemplate(numDs, origT); // no extraction needed
        }

        if (verbose) {
            klog.info("Extracting ds: " + numDs.getName() + " by template: " + origT.getName());
        }

        final Matrix matrix = new Matrix(numDs.getNumRow(), origT.getNumItems());
        final List colNames = new ArrayList(origT.getNumItems());
        final Template.Item[] newItems = new Template.Item[origT.getNumItems()];

        //final Template.Item[] orig_items = origT.getItemsOrderedByProfilePos();
        final Template.Item[] orig_items = origT.getItemsOrderedByClassFirstAndThenProfilePos();
        for (int c = 0; c < orig_items.length; c++) {
            // first the dataset
            final int col2use = orig_items[c].getProfilePosition(); // @note extracting using the profile position
            //log.debug("col2use: " + col2use);
            matrix.setColumn(c, numDs.getColumn(col2use));
            colNames.add(numDs.getColumnName(col2use));
            // then the new template
            newItems[c] = TemplateImpl.ItemImpl.createItem(orig_items[c].getId(), c); // @note profile pos is changed
        }

        final String name = NamingConventions.generateName(numDs, origT, true);
        //log.debug("$$$ gen name: " + name + " " + maybeAuxTemplate.getName());
        NumDataset newds = new NumDataset(name, matrix, numDs.getRowNames(), colNames, numDs.getAnn());
        newds.setGenes(numDs.getGenes());
        newds.setProperty("extracted", "true"); // huh

        //klog.debug(">>>>>>>>>>>>>> origT: " + origT.getName() + " newds: " + newds.getName());

        final Template newt = createTemplate(origT.getName() + "_repos", newItems, origT.isContinuous());

        //klog.debug("orig class a: " + origT.getClassName(0) + " made: " + newt.getClassName(0));

        return new DatasetTemplate(newds, newt);
    }

    // ---------------------------------------------------------------------------------------------- //
    // ---------------------------------------- START CREATORS ------------------------------------ //
    // ---------------------------------------------------------------------------------------------- //

    /**
     * @param name
     * @param items
     * @param classes
     * @param continuous
     * @return
     */
    public static synchronized Template createTemplate(final String name,
                                                       final List items,
                                                       final List classes,
                                                       final boolean continuous) {
        return createTemplate_ordered_assign(name, (Item[]) items.toArray(new Item[items.size()]),
                (Template.Class[]) classes.toArray(new Template.Class[classes.size()]),
                continuous);
    }

    public static synchronized Template createTemplate(final String name,
                                                       final Item[] items,
                                                       final Template.Class[] classes,
                                                       final boolean continuous) {

        return createTemplate_ordered_assign(name, items, classes, continuous);
    }

    private static int _getColumnIndex(final List colNames, final String colName) {
        for (int c = 0; c < colNames.size(); c++) {
            final String cn = colNames.get(c).toString();
            if (colName.equalsIgnoreCase(cn)) {
                return c;
            }
        }

        return -1;
    }

    public static synchronized Template createCategoricalTemplate(final String name,
                                                                  final Dataset ds,
                                                                  final String classAName,
                                                                  final String[] classASampleNames,
                                                                  final String classBName,
                                                                  final String[] classBSampleNames) {
        return createCategoricalTemplate(name, ds.getColumnNames(), classAName, classASampleNames, classBName, classBSampleNames);
    }

    public static synchronized Template createCategoricalTemplate(final String name,
                                                                  final List all_col_names_ordered,
                                                                  final String classAName,
                                                                  final String[] classASampleNames,
                                                                  final String classBName,
                                                                  final String[] classBSampleNames) {

        //Printf.outl(classASampleNames);
        //Printf.outl(classBSampleNames);

        if (classASampleNames.length == 0) {
            throw new IllegalArgumentException("Empty classASampleNames");
        }

        if (classBSampleNames.length == 0) {
            throw new IllegalArgumentException("Empty classBSampleNames");
        }


        final GeneSet a = new FSet(classAName, classASampleNames);
        final GeneSet b = new FSet(classBName, classBSampleNames);

        return createCategoricalTemplate(name, all_col_names_ordered, new GeneSet[]{a, b});
    }

    public static synchronized Template createCategoricalTemplate(final String name,
                                                                  final Dataset ds,
                                                                  final GeneSet[] gsets_sample_names) {
        return createCategoricalTemplate(name, ds.getColumnNames(), gsets_sample_names);
    }


    public static synchronized Template createCategoricalTemplate(final String name,
                                                                  final List all_col_names_ordered,
                                                                  final GeneSet[] gsets_sample_names) {
        TemplateImpl template = new TemplateImpl(name);

        //List classes = new ArrayList();
        //List items = new ArrayList();
        for (int i = 0; i < gsets_sample_names.length; i++) {
            ClassImpl cl = new ClassImpl(gsets_sample_names[i].getName(true));
            template.add(cl);
            for (int r = 0; r < gsets_sample_names[i].getNumMembers(); r++) {
                String sampleName = gsets_sample_names[i].getMember(r).toUpperCase(); // @ugggh hack for things like dmap
                int pos = _getColumnIndex(all_col_names_ordered, sampleName);
                if (pos == -1) {
                    throw new IllegalArgumentException("No such sample in ds: " + sampleName + " \n" + Printf.outs(all_col_names_ordered));
                }
                Template.Item item = TemplateImpl.ItemImpl.createItem(gsets_sample_names[i].getName(true), pos);
                template.add(item);
                cl.add(item);
                //items.add(item);
            }

            //classes.add(cl);
        }

        /*
return createTemplate_ordered_assign(name,
        (Item[]) items.toArray(new Item[items.size()]),
        (Class[]) classes.toArray(new Class[classes.size()]),
        false);
        */

        //template.assignItems(); // DO NOT call - custom addition already done above
        template.setContinuous(false);
        template.setAux(false);
        template.makeImmutable();
        template.setSampleNamesFromDataset(true); // @note
        return template;

    }

    /**
     * @param name
     * @param items
     * @param classes
     * @param continuous
     * @return
     */
    // THIS IS THE CORE ASSIGNMENT METHOD
    private static synchronized Template createTemplate_ordered_assign(final String name,
                                                                       final Item[] items,
                                                                       final Template.Class[] classes,
                                                                       final boolean continuous) {

        //klog.debug(">>>>> name: " + name);

        if (items == null) {
            throw new IllegalArgumentException("Null items not allowed parameter");
        }

        if (classes == null) {
            throw new IllegalArgumentException("Null classes not allowed parameter");
        }

        TemplateImpl template = new TemplateImpl(name);

        for (int i = 0; i < items.length; i++) {
            template.add(items[i]);
        }

        // @note class order is determined by the spec here
        for (int i = 0; i < classes.length; i++) {
            template.add(classes[i]);
        }

        template.assignItems2ClassInOrder();
        template.setContinuous(continuous);
        template.makeImmutable(); // @note

        return template;
    }

    public static synchronized Template createTemplate(final String name,
                                                       final Item[] items,
                                                       final boolean continuous) {

        if (items == null) {
            throw new IllegalArgumentException("Null items not allowed parameter");
        }

        final List classNames = new ArrayList();

        // IMP need to sort the items by profile pos
        final Item[] items_sorted = getProfilePositionsSorted(items);
        for (int i = 0; i < items_sorted.length; i++) {
            String className = items_sorted[i].getId(); // @note that the original class names are lost here -> we use the class names according to their items name
            if (!classNames.contains(className)) {
                classNames.add(className);
            }
        }

        final Class[] classes = new Class[classNames.size()];
        for (int i = 0; i < classNames.size(); i++) {
            classes[i] = new TemplateImpl.ClassImpl(classNames.get(i).toString());
        }

        return createTemplate_ordered_assign(name, items, classes, continuous); // @note which items are sent doesnt matter (i think)
    }

    /**
     * Make a Template from specified vector (rep a genes expression profile for instance)
     * Template is continuous.
     */
    public static synchronized Template createContinuousTemplate(final String probeName,
                                                                 final Vector profile) {

        // what if 2 elements have the same value??
        // so num of classes may not be eq to num of items
        // is that ok in the context of numeric templates??
        // Yes - it is correct to make 1 item per class
        TemplateImpl template = new TemplateImpl(probeName);

        for (int i = 0; i < profile.getSize(); i++) {
            TemplateImpl.ClassImpl sept = new TemplateImpl.ClassImpl("class." + i);

            template.add(sept);
            TemplateImpl.ItemImpl item = TemplateImpl.ItemImpl.createItem(Float.toString(profile.getElement(i)), i);

            template.add(item);
            sept.add(item);
        }

        template.setContinuous(true);

        //template.assignItems(); // do not call
        template.makeImmutable();
        return template;
    }

    /**
     * Create a template based on the expression pattern of probe_name in the
     * specified Dataset
     *
     * @param probe_name
     * @param ds
     * @return
     */
    public static synchronized Template createContinuousTemplate(final String probe_name,
                                                                 final Dataset ds) {
        int rindex = ds.getRowIndex(probe_name);
        if (rindex == -1) {
            throw new IllegalArgumentException("No data in Dataset: " + ds.getName() + " for probe: " + probe_name);
        }

        Vector vector = ds.getRow(rindex);
        return createContinuousTemplate(probe_name, vector);
    }

    /**
     * @param probes
     * @param ds
     * @return
     */
    public static synchronized Template[] createContinuousTemplates(final GeneSet probes,
                                                                    final Dataset ds) {

        final Template[] tss = new Template[probes.getNumMembers()];

        for (int t = 0; t < tss.length; t++) {
            tss[t] = createContinuousTemplate(probes.getMember(t), ds);
        }

        return tss;
    }

    public static synchronized Template[] createContinuousTemplates(final String[] probeNames,
                                                                    final Dataset ds) {

        final Template[] tss = new Template[probeNames.length];

        for (int t = 0; t < tss.length; t++) {
            tss[t] = createContinuousTemplate(probeNames[t], ds);
        }

        return tss;
    }

    /**
     * Factory method.
     * Make a new Template from specified String[].
     * <p/>
     * Class labels are automatically generated one for each unique String[]
     * By default they are: class_strvalue
     * <p/>
     * if there are only numbers in the passed string[][ then the template is made numeric else its not.
     * <p/>
     * Template made is NOT continuous.
     */

    // make a new Template.Class for every unique String[] element
    public static synchronized Template createCategoricalTemplate(final String name,
                                                                  final String[] x) {

        //log.debug("array is x=" + x.length);
        ArrayList uniq = new ArrayList();

        List items = new ArrayList();
        List classes = new ArrayList();

        for (int i = 0; i < x.length; i++) {
            if (!uniq.contains(x[i])) {
                uniq.add(x[i]);
                TemplateImpl.ClassImpl sept = new TemplateImpl.ClassImpl("class." + x[i]);
                classes.add(sept);
            }

            Item item = TemplateImpl.ItemImpl.createItem(x[i], i);
            items.add(item);
        }

        return createTemplate(name, items, classes, false);
    }

    /**
     * Class labels are automatically generated.
     * By default they are: class_#
     * template made is always numeric
     *
     * @param aName        name of the Template
     * @param countbyclass -> number of of items in each class. Assigned in order.
     */
    public static synchronized Template createCategoricalTemplate(final String aName,
                                                                  final int[] countbyclass) {

        String[] names = new String[countbyclass.length];

        for (int i = 0; i < names.length; i++) {
            names[i] = "class_" + i;
        }

        return createCategoricalTemplate(aName, countbyclass, names);
    }

    /**
     * @param name
     * @param countbyclass
     * @param classnames
     * @return
     */
    public static synchronized Template createCategoricalTemplate(final String name,
                                                                  final int[] countbyclass,
                                                                  final String[] classnames) {

        if (countbyclass.length != classnames.length) {
            throw new IllegalArgumentException("Number of classes: " + countbyclass.length
                    + " not equal to number of classnames: "
                    + classnames.length);
        }

        List items = new ArrayList();
        List classes = new ArrayList();

        int pos = 0;

        for (int i = 0; i < countbyclass.length; i++) {
            int cnt = countbyclass[i];
            TemplateImpl.ClassImpl cl = new TemplateImpl.ClassImpl(classnames[i]);
            classes.add(cl);

            for (int j = 0; j < cnt; j++) {
                Item item = TemplateImpl.ItemImpl.createItem(Integer.toString(i), pos++);
                items.add(item);
            }
        }

        return createTemplate(name, items, classes, false);
    }

    /**
     * for biphasic templates
     */
    public static synchronized Template createBiphasicTemplate(final String aName,
                                                               final int zerocnt,
                                                               final int onecnt) {

        TemplateImpl template = new TemplateImpl(aName);
        TemplateImpl.ClassImpl zerosept = new TemplateImpl.ClassImpl("class_0");

        template.add(zerosept);

        int pos = 0;

        for (int i = 0; i < zerocnt; i++) {
            TemplateImpl.ItemImpl item = TemplateImpl.ItemImpl.createItem("0", pos++);
            template.add(item);
            zerosept.add(item);
        }

        TemplateImpl.ClassImpl onesept = new TemplateImpl.ClassImpl("class_1");

        template.add(onesept);

        for (int i = 0; i < onecnt; i++) {
            TemplateImpl.ItemImpl item = TemplateImpl.ItemImpl.createItem("1", pos++);
            template.add(item);
            onesept.add(item);
        }

        //template.assignItems(); // DO NOT call - custom addition already done above
        template.setContinuous(false);
        template.makeImmutable();
        return template;
    }

    public static Template createTemplate(final String name, final Matrix[] matrices) {

        int[] lens = new int[matrices.length];

        for (int i = 0; i < matrices.length; i++) {
            lens[i] = matrices[i].getNumCol();
        }

        return createCategoricalTemplate(name, lens);
    }

    /**
     * Exach line is of format:
     * name count
     * name count
     * ...
     *
     * @param s
     * @return
     * @throws Exception
     */
    public static Template createCategoricalTemplateFromClassCountByParsing(final String name,
                                                                            final String s) throws Exception {

        //int[] cntsbyclass = ParseUtils.string2ints(s, " \t\n");
        //return createTemplate(name, cntsbyclass);

        StringTokenizer tok = new StringTokenizer(s, "\n");
        List lines = new ArrayList();
        while (tok.hasMoreElements()) {
            String line = tok.nextToken();
            line = line.trim();
            if (line.length() > 0) {
                lines.add(line);
            }

        }

        String[] classnames = new String[lines.size()];
        int[] cntbyclass = new int[lines.size()];


        for (int i = 0; i < lines.size(); i++) {
            tok = new StringTokenizer(lines.get(i).toString(), " \t");
            classnames[i] = tok.nextToken();
            cntbyclass[i] = Integer.parseInt(tok.nextToken());
        }

        return createCategoricalTemplate(name, cntbyclass, classnames);
    }

    public static Template creatCategoricalTemplate_by_splitting_pos_neg(final RankedList rl) {
        Template.Item[] items = new Template.Item[rl.getSize()];
        for (int i = 0; i < rl.getSize(); i++) {
            float score = rl.getScore(i);
            String id;
            if (XMath.isPositive(score)) {
                id = "pos";
            } else {
                id = "neg";
            }
            items[i] = TemplateImpl.ItemImpl.createItem(id, i);
        }

        return createTemplate(rl.getName(), items, false);
    }

    public static Template createCategoricalTemplate(final String name,
                                                     final int numClassA_ordered,
                                                     final int numClassB_ordered) {
        Template.Item[] items = new Template.Item[numClassA_ordered + numClassB_ordered];
        int c = 0;
        for (int i = 0; i < numClassA_ordered; i++) {
            items[c] = TemplateImpl.ItemImpl.createItem("A", c);
            c++;
        }

        for (int i = 0; i < numClassB_ordered; i++) {
            items[c] = TemplateImpl.ItemImpl.createItem("B", c);
            c++;
        }

        return createTemplate(name, items, false);
    }

    // na for continuous templates
    // IMP check before using in algs - may not work
    // pseudo template class names are given so that multiple class 0's can be used
    public static Template createCombinedTemplateUsingClass0(final String name, final Template[] tss) {
        return createCombinedTemplateUsingClassFoo(name, tss, true);
    }

    public static Template createCombinedTemplateUsingClass1(final String name, final Template[] tss) {
        return createCombinedTemplateUsingClassFoo(name, tss, false);
    }

    // does the real work
    public static Template createCombinedTemplateUsingClassFoo(final String name, final Template[] tss, final boolean classA) {
        List items = new ArrayList();
        List classes = new ArrayList();

        int classNum;
        if (classA) {
            classNum = 0;
        } else {
            classNum = 1;
        }

        int pos = 0;
        for (int t = 0; t < tss.length; t++) {
            Template.Class cl = tss[t].getClass(classNum);
            String uniqname = cl.getName() + "_" + t; // so that its unique
            for (int c = 0; c < cl.getSize(); c++) {
                Item item = TemplateImpl.ItemImpl.createItem(uniqname, pos++);
                items.add(item);
            }

            Template.Class newcl = new TemplateImpl.ClassImpl(uniqname);
            classes.add(newcl);
        }

        return createTemplate(name, items, classes, false);
    }

    // ---------------------------------------------------------------------------------------------- //
    // ---------------------------------------- START EXTRACTORS ------------------------------------ //
    // ---------------------------------------------------------------------------------------------- //

    /**
     * Note col indices in new will NOT be the same as in orig (thats kinda the point)
     *
     * @param name
     * @param origT
     * @param colIndices_in_orig
     * @return
     */
    public static synchronized Template extractTemplate(final String name,
                                                        final Template origT,
                                                        final int[] colIndices_in_orig) {

        if (origT == null) {
            throw new IllegalArgumentException("Param origT cannot be null");
        }

        if (colIndices_in_orig == null) {
            throw new IllegalArgumentException("Param colIndices cannot be null");
        }

        final Item[] items = new Item[colIndices_in_orig.length];
        final Set classesSet = new HashSet();

        for (int c = 0; c < colIndices_in_orig.length; c++) {
            Item orig = origT.getItemByProfilePos(colIndices_in_orig[c]);
            //items[c] = orig.cloneDeep();
            //items[c].fProfilePos = c; // @note THIS MAY NOT BE WHAT IS ALWAYS WANTED
            items[c] = ItemImpl.createItem(orig.getId(), c); // @note 'c' THIS MAY NOT BE WHAT IS ALWAYS WANTED
            classesSet.add(origT.getClass(orig));
        }

        final Template.Class[] classes = (Template.Class[]) classesSet.toArray(new Template.Class[classesSet.size()]);

        return createTemplate_ordered_assign(name, items, classes, origT.isContinuous());
    }

    /**
     * @param name
     * @param origT
     * @param colIndices_in_orig
     * @param includeSpecifiedColIndices
     * @return
     */
    public static synchronized Template extractTemplate(final String name,
                                                        final Template origT,
                                                        final int[] colIndices_in_orig,
                                                        final boolean includeSpecifiedColIndices) {

        if (origT == null) {
            throw new IllegalArgumentException("Param origT cannot be null");
        }

        if (colIndices_in_orig == null) {
            throw new IllegalArgumentException("Param colIndices_in_orig cannot be null");
        }

        if (includeSpecifiedColIndices) {
            return extractTemplate(name, origT, colIndices_in_orig);
        } else { // exclude specified

            TIntHashSet excludeColIndices = new TIntHashSet(colIndices_in_orig);

            TIntArrayList includeColIndices = new TIntArrayList();
            int[] profile_pos = origT.getProfilePositionsOrdered();
            for (int i = 0; i < profile_pos.length; i++) {
                if (!excludeColIndices.contains(profile_pos[i])) {
                    includeColIndices.add(profile_pos[i]);
                }
            }

            return extractTemplate(name, origT, includeColIndices.toNativeArray());
        }

    }

    /**
     * @param origT
     * @param includeOrigT
     * @return
     * @throws Exception
     */
    public static Template[] extractAllPossibleTemplates(final Template origT,
                                                         final boolean includeOrigT) throws Exception {

        if (origT == null) {
            throw new IllegalArgumentException("Parameter origT cannot be null");
        }

        //klog.debug("###$ extracting all possible for: " + origT.getName() + " " + origT.isAux() + " " + origT.isContinuous());
        //TraceUtils.showTrace();

        /*
        if (origT.getName().indexOf('#') == -1) {
            options.add(extractForwardBiphasicTemplate(origT));
            options.add(extractReversedBiphasicTemplate(origT));
            TraceUtils.showTrace();
            return (Template[]) options.toArray(new Template[options.size()]);
        }
        */
        List options = new ArrayList();
        if (includeOrigT) {
            options.add(origT);
        }

        if (origT instanceof PreTemplate) {
            // do nothing
        } else if (origT instanceof TemplateImplFromSampleNames) {
            // because of the UI, always 2 class templates only
            Template straight = extractForwardBiphasicTemplate(origT);
            options.add(straight);
            Template rev = TemplateFactory.createReversedBiphasicTemplate(origT);
            options.add(rev);
            //System.out.println(">> " + options);
        } else if (!origT.isContinuous()) {

            // bludgeon our way through -- Template making is inexpensive
            if (origT.getNumClasses() > 2) {
                Template[] all = TemplateFactory.extractAll2ClassTemplates(origT, false); // all so both forward and rev
                for (int i = 0; i < all.length; i++) {
                    options.add(all[i]);
                }
            }

            if (origT.getNumClasses() == 2) {
                Template straight = extractForwardBiphasicTemplate(origT);
                options.add(straight);
                Template rev = TemplateFactory.createReversedBiphasicTemplate(origT);
                options.add(rev);
            }

            // always add 1 for each class, if not an aux
            if (!origT.isAux() && origT.getNumClasses() > 1) {
                Template[] unis = extractAllUniphasicTemplates(origT);
                for (int i = 0; i < unis.length; i++) {
                    options.add(unis[i]);
                }
            }


        } else { // continuous templates (they are not all in cache as cache clobbers multi cls from same file)
        }

        return (Template[]) options.toArray(new Template[options.size()]);
    }

    /**
     * INcluding the specified Template and its reverse if the specified template is itself 2-class
     *
     * @param parentMultiClassTemplate
     * @param doOva
     * @param doAllPairs
     * @return
     */
    public static Template[] extractAll2ClassTemplates(final Template parentMultiClassTemplate,
                                                       final boolean onlyForwardOvas) {
        List templates = new ArrayList();

        if (parentMultiClassTemplate.getNumClasses() == 2) {
            templates.add(parentMultiClassTemplate);
            templates.add(TemplateFactory.createReversedBiphasicTemplate(parentMultiClassTemplate));
            return (Template[]) templates.toArray(new Template[templates.size()]);
        }

        Template[] ovas = TemplateFactory.extractAllOvaTemplates(parentMultiClassTemplate, onlyForwardOvas);
        for (int i = 0; i < ovas.length; i++) {
            templates.add(ovas[i]);
        }


        Template[] allpairs = TemplateFactory.extractAllPairsTemplates(parentMultiClassTemplate);
        for (int i = 0; i < allpairs.length; i++) {
            templates.add(allpairs[i]);
        }

        //klog.debug("Generated # aux: " + templates.size() + " from orig: " + parentMultiClassTemplate.getName() + " # classes: " + parentMultiClassTemplate.getNumClasses());
        return (Template[]) templates.toArray(new Template[templates.size()]);
    }

    /**
     * @param parentMultiClassTemplate
     * @return
     */
    public static synchronized Template[] extractAllPairsTemplates(final Template parentMultiClassTemplate) {

        if (parentMultiClassTemplate.getNumClasses() < 2) {
            throw new IllegalArgumentException("At least 3 Template classes needed to do ALL_PAIRS");
        }

        if (parentMultiClassTemplate.isContinuous()) {
            throw new IllegalArgumentException("Cannot work on continuous Template: " + parentMultiClassTemplate.getName());
        }

        /*
        if (parentMultiClassTemplate.isAux()) {
            throw new IllegalArgumentException("Cannot work on aux templates: " + parentMultiClassTemplate.getName());
        }
        */

        // lets do all pairs including reverse ones
        List templatePairs = new ArrayList();
        //Map idTemplateMap = new HashMap();
        for (int i = 0; i < parentMultiClassTemplate.getNumClasses(); i++) {
            Template.Class a = parentMultiClassTemplate.getClass(i);
            for (int j = 0; j < parentMultiClassTemplate.getNumClasses(); j++) {
                if (i == j) {
                    continue;
                }
                Template.Class b = parentMultiClassTemplate.getClass(j);
                //klog.debug("Making aux pair: " + a)
                templatePairs.add(new Template.Class[]{a, b});
            }
        }

        List templates = new ArrayList(templatePairs.size());

        for (int t = 0; t < templatePairs.size(); t++) {
            final Template.Class[] pair = (Template.Class[]) templatePairs.get(t);
            // @maint naming convention
            final String newtName = parentMultiClassTemplate.getName() + "#" + pair[0].getName() + VS + pair[1].getName();
            final TemplateImpl newT = new TemplateImpl(newtName);
            final TemplateImpl.ClassImpl newClassA = new TemplateImpl.ClassImpl(pair[0].getName());
            final TemplateImpl.ClassImpl newClassB = new TemplateImpl.ClassImpl(pair[1].getName());
            newT.add(newClassA);
            newT.add(newClassB);

            for (int i = 0; i < pair[0].getSize(); i++) {
                Item orig = pair[0].getItem(i);
                Item newItem = orig.cloneDeep();// no change to profile pos
                newClassA.add(newItem);
                newT.add(newItem);
            }

            for (int i = 0; i < pair[1].getSize(); i++) {
                Item orig = pair[1].getItem(i);
                Item newItem = orig.cloneDeep();// no change to profile pos
                newClassB.add(newItem);
                newT.add(newItem);
            }

            // This will not work as we dont want the auto assignment to occur
            //return createTemplate_ordered_assign(name, items, new Template.Class[]{newClassA, newClassB}, false);
            //  templates.add(createTemplate(newtName, newItems, newClasses, false));

            newT.setContinuous(false);

            //template.assignItems(); // do not call
            newT.makeImmutable();
            templates.add(newT);
        }

        return (Template[]) templates.toArray(new Template[templates.size()]);
    }

    /**
     * @param multiClassTemplate
     * @param onlyForward
     * @return
     */
    public static synchronized Template[] extractAllOvaTemplates(final Template multiClassTemplate,
                                                                 final boolean onlyForward) {
        if (multiClassTemplate.getNumClasses() <= 2) {
            throw new IllegalArgumentException("At least 3 Template classes needed to make a set of ova templates but found: " + multiClassTemplate.getNumClasses() + "\n" + multiClassTemplate.getAsString(false));
        }

        List ovas = new ArrayList();

        for (int c = 0; c < multiClassTemplate.getNumClasses(); c++) {
            Template.Class one = multiClassTemplate.getClass(c);
            Template ova = extractOvaTemplate(multiClassTemplate, one);
            ovas.add(ova);
            if (!onlyForward) {
                String rname = multiClassTemplate.getName() + "#REST" + VS + one.getName();
                Template rova = _createReversedBiphasicTemplate(rname, ova);
                ovas.add(rova);
            }
        }

        // if (!sortByFreq)
        //klog.debug("From master template with # classes: " + multiClassTemplate.getNumClasses() + " made ovas #: " + ovas.size());
        return (Template[]) ovas.toArray(new Template[ovas.size()]);
    }

    /**
     * @param parentMultiClassTemplate
     * @param theOneClass
     * @return
     */
    public static synchronized Template extractOvaTemplate(final Template parentMultiClassTemplate,
                                                           final Template.Class theOneClass) {

        if (!parentMultiClassTemplate.isMemberClass(theOneClass)) {
            throw new IllegalArgumentException("Specified theOneClass: " + theOneClass.getName() + " is not a member of parentMultiClassTemplate: " + parentMultiClassTemplate.getName());
        }

        if (parentMultiClassTemplate.getNumClasses() <= 2) {
            throw new IllegalArgumentException("Need 3 or more classes to do a ova. Found: " + parentMultiClassTemplate.getNumClasses());
        }

        if (parentMultiClassTemplate.isContinuous()) {
            throw new IllegalArgumentException("Cannot work on continuous Template: " + parentMultiClassTemplate.getName());
        }

        // create Template
        //klog.debug("The one name: " + theOneClass.getName() + " restname: " + Constants.REST);
        // @maint naming convention
        String newtName = parentMultiClassTemplate.getName() + "#" + theOneClass.getName() + VS + Constants.REST;
        TemplateImpl newT = new TemplateImpl(newtName);
        TemplateImpl.ClassImpl clonedTheOneClass = new TemplateImpl.ClassImpl(theOneClass.getName());
        newT.add(clonedTheOneClass);

        final int index = parentMultiClassTemplate.getClassIndex(theOneClass);
        final TemplateImpl.ClassImpl restClass = new TemplateImpl.ClassImpl(Constants.REST);
        newT.add(restClass);

        // always put theOneClass first in the new template made
        for (int c = 0; c < theOneClass.getSize(); c++) {
            Item item = TemplateImpl.ItemImpl.createItem(theOneClass.getName(),
                    theOneClass.getItem(c).getProfilePosition()); // note position IS maintained
            newT.add(item);
            clonedTheOneClass.add(item);
        }

        for (int i = 0; i < parentMultiClassTemplate.getNumClasses(); i++) {
            Template.Class cl = parentMultiClassTemplate.getClass(i);
            if (i == index) {
                // already done
            } else {
                for (int c = 0; c < cl.getSize(); c++) {
                    Item item = TemplateImpl.ItemImpl.createItem(Constants.REST, cl.getItem(c).getProfilePosition()); // note position
                    //allItems.add(item);
                    newT.add(item);
                    restClass.add(item);
                }
            }
        }

        // add the 2 classes
        /*
        return createTemplate_ordered_assign(newtName,
                (Item[]) allItems.toArray(new Item[allItems.size()]),
                new Template.Class[]{clonedTheOneClass, restClass},
                false
        );
        */

        //template.assignItems(); // DO NOT call - custom addition already done above
        newT.setContinuous(false);
        newT.makeImmutable();
        return newT;
    }

    /**
     * @param origTemplate
     * @return
     */
    public static synchronized Template extractForwardBiphasicTemplate(final Template origTemplate) {
        if (origTemplate.getNumClasses() != 2) {
            throw new IllegalArgumentException("Cannot straight template as its not biphasic. # classes: " + origTemplate.getNumClasses() + " " + origTemplate.getName());
        }

        final String newName = origTemplate.getName() + "#" + origTemplate.getClassName(0) + VS + origTemplate.getClassName(1);
        final Template newTemplate = origTemplate.cloneDeep(newName);
        newTemplate.makeImmutable();
        return newTemplate;
    }

    public static synchronized Template createReversedBiphasicTemplate(final Template origTemplate) {
        return _createReversedBiphasicTemplate(origTemplate.getName() + "#" + origTemplate.getClassName(1)
                + VS + origTemplate.getClassName(0), origTemplate);
    }

    // simpler name (no aux stuff
    public static synchronized Template createReversedBiphasicTemplate2(final Template origTemplate) {
        return _createReversedBiphasicTemplate(origTemplate.getClassName(1) + VS +
                origTemplate.getClassName(0), origTemplate);
    }

    // @note IMP the profile positions are NOT changed
    // a a a b b
    // becomes:
    // b b a a a but b & b still have pp of  3 & 4
    // so ONLY changing the class order and items are NOT touched
    private static synchronized Template _createReversedBiphasicTemplate(final String name,
                                                                         final Template origTemplate) {

        if (origTemplate.getNumClasses() != 2) {
            throw new IllegalArgumentException("Cannot reverse template as its not biphasic. # classes: " + origTemplate.getNumClasses() + " " + origTemplate.getName());
        }

        if (origTemplate instanceof TemplateImplFromSampleNames) { // @note
            //TraceUtils.showTrace();
            return ((TemplateImplFromSampleNames) origTemplate).cloneDeepReversed(name);
        }

        final TemplateImpl newT = new TemplateImpl(name);

        // This is the magix (not the items)
        final TemplateImpl.ClassImpl newClassA = new TemplateImpl.ClassImpl(origTemplate.getClass(1).getName());
        for (int i = 0; i < origTemplate.getClass(1).getSize(); i++) {
            Item newItem = origTemplate.getClass(1).getItem(i);
            newClassA.add(newItem.cloneDeep()); // no change to profile pos
            newT.add(newItem);
        }

        final TemplateImpl.ClassImpl newClassB = new TemplateImpl.ClassImpl(origTemplate.getClassName(0));
        for (int i = 0; i < origTemplate.getClass(0).getSize(); i++) {
            Item newItem = origTemplate.getClass(0).getItem(i);
            newClassB.add(newItem.cloneDeep()); // no change to profile pos
            newT.add(newItem);
        }

        newT.add(newClassA);
        newT.add(newClassB);

        // This will not work as we dont want the auto assignment to occur
        //return createTemplate_ordered_assign(name, items, new Template.Class[]{newClassA, newClassB}, false);

        newT.setContinuous(false);

        //template.assignItems(); // do not call
        newT.makeImmutable();
        return newT;
    }

    /**
     * for biphasic templates
     */
    public static synchronized Template extractUniphasicTemplate(final Template fullTemplate,
                                                                 final Template.Class cl) {
        if (fullTemplate.isAux()) {
            throw new IllegalArgumentException("Cannot make uniphasic from aux templates: " + fullTemplate.getName());
        }

        TemplateImpl template = new TemplateImpl(fullTemplate.getName() + "#" + cl.getName());
        TemplateImpl.ClassImpl onlyClass = new TemplateImpl.ClassImpl(cl.getName());
        template.add(onlyClass);

        for (int i = 0; i < cl.getSize(); i++) {
            Item origItem = cl.getItem(i);
            Item item = origItem.cloneDeep();
            template.add(item);
            onlyClass.add(item);
        }

        //template.assignItems(); // DO NOT call - custom addition already done above
        template.setContinuous(false);
        template.setAux(true);
        template.makeImmutable();
        return template;
    }

    public static synchronized Template extractUniphasicTemplateUntested(final Template fullTemplate,
                                                                         final Template.Class cl) {

        TemplateImpl template = new TemplateImpl(fullTemplate.getName() + "#" + cl.getName());
        TemplateImpl.ClassImpl onlyClass = new TemplateImpl.ClassImpl(cl.getName());
        template.add(onlyClass);

        for (int i = 0; i < cl.getSize(); i++) {
            Item origItem = cl.getItem(i);
            Item item = TemplateImpl.ItemImpl.createItem(onlyClass.getName(), origItem.getProfilePosition());
            template.add(item);
            onlyClass.add(item);
        }

        //template.assignItems(); // DO NOT call - custom addition already done above
        template.setContinuous(false);
        template.setAux(true);
        template.makeImmutable();
        return template;
    }

    public static synchronized Template[] extractAllUniphasicTemplates(final Template fullTemplate) {

        Template[] unis = new Template[fullTemplate.getNumClasses()];
        for (int i = 0; i < fullTemplate.getNumClasses(); i++) {
            unis[i] = extractUniphasicTemplate(fullTemplate, fullTemplate.getClass(i));
        }

        return unis;
    }

    // REAL template
    public static Template createAuxEqualizedRealTemplate(final Template orig,
                                                          final RandomSeedGenerator rsgen) {

        if (orig.getNumClasses() != 2) {
            throw new IllegalArgumentException("Only 2 class templates possible");
        }

        // first equalize (randomly) Must eq within this loop else all rnds will share the same equalization
        int num2use = XMath.min(new int[]{orig.getClass(0).getSize(), orig.getClass(1).getSize()});
        return createAuxConstrainedRealTemplate(orig, num2use, num2use, rsgen);
    }

    public static Template createAuxConstrainedRealTemplate(final Template orig,
                                                            final int numFromC0inC0,
                                                            final int numFromC1inC1,
                                                            final RandomSeedGenerator rsgen) {

        /*
        if (orig.isAux()) {
            throw new IllegalArgumentException("Cannot make template in this mode for aux inputs: " + orig.getName());
        }
        */

        Template.Class class0 = orig.getClass(0);
        Template.Class class1 = orig.getClass(1);

        if (numFromC0inC0 > class0.getSize()) {
            throw new IllegalArgumentException("numFromC0inC0:  " + " cannot be larger than size of templateclass 0: " + class0.getSize());
        }

        if (numFromC1inC1 > class1.getSize()) {
            throw new IllegalArgumentException("numFromC1inC1: " + numFromC1inC1 + " cannot be larger than size of templateclass 1: " + class1.getSize());
        }

        TemplateImpl newt = new TemplateImpl(orig.getName() + "#" + numFromC0inC0 + "_" + numFromC1inC1);
        TemplateImpl.ClassImpl newc0 = new TemplateImpl.ClassImpl(class0.getName());
        TemplateImpl.ClassImpl newc1 = new TemplateImpl.ClassImpl(class1.getName());

        int[] inds0 = XMath.randomlyPickIndicesWithoutReplacement(numFromC0inC0, class0.getSize(), rsgen);
        for (int i = 0; i < numFromC0inC0; i++) {
            Item item = class0.getItem(inds0[i]);
            Item newitem = item.cloneDeep();
            newc0.add(newitem);
            newt.add(newitem);
        }

        int[] inds1 = XMath.randomlyPickIndicesWithoutReplacement(numFromC1inC1, class1.getSize(), rsgen);
        for (int i = 0; i < inds1.length; i++) {
            Item item = class1.getItem(inds1[i]);
            Item newitem = item.cloneDeep();
            newc1.add(newitem);
            newt.add(newitem);
        }

        // add the classes
        newt.add(newc0);
        newt.add(newc1);

        //newt.assignItems();  // DO NOT call - custom addition already done above

        newt.setClassOfInterestIndex(orig.getClassOfInterestIndex());

        // always
        newt.setAux(true);
        newt.setContinuous(false);
        newt.makeImmutable();

        return newt;
    }


}    // End TemplateFactory

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

