/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.*;
import gnu.trove.*;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * @author Aravind Subramanian
 *         Date: Dec 2, 2004
 *         Time: 12:34:30 PM
 */
class TemplateFactory_helper {

    private static final Logger klog = XLogger.getLogger(TemplateFactory_helper.class);

    protected static int[] uniformlyPick(int numBalls, int num2PickEachTime, int numTimes2Pick) {

        if (numBalls < num2PickEachTime) {
            throw new IllegalArgumentException("numBalls: " + numBalls + " cannot be less than num2PickEachTime: " + num2PickEachTime);
        }

        // init map
        List list = new ArrayList();

        for (int i = 0; i < numBalls; i++) {
            list.add(new DoubleElement(i, 0));
        }

        for (int i = 0; i < numTimes2Pick; i++) {
            DoubleElement[] bottomElements = pickBottomElements(num2PickEachTime, list);
            for (int j = 0; j < bottomElements.length; j++) {
                bottomElements[j].fValue = bottomElements[j].fValue + 1;
            }
        }

        int[] ret = new int[numBalls];
        for (int i = 0; i < numBalls; i++) {
            DoubleElement del = (DoubleElement) list.get(i);
            ret[i] = (int) del.fValue;
        }

        return ret;
    }

    protected static DoubleElement[] pickBottomElements(int num2Pick, List delList) {
        DoubleElement.sort(SortMode.REAL, Order.ASCENDING, delList);
        DoubleElement[] dels = new DoubleElement[num2Pick];
        for (int i = 0; i < num2Pick; i++) {
            dels[i] = (DoubleElement) delList.get(i);
        }

        return dels;
    }

    protected static int[] pickNoRepeatsAndRemove(int numNeeded, TIntArrayList all, RandomSeedGenerator rsgen) {

        TIntHashSet uniq = new TIntHashSet(all.toNativeArray());
        if (uniq.size() < numNeeded) {
            return null;
            //throw new IllegalArgumentException("uniq: " + uniq.size() + " numNeeded: " + numNeeded);
        }

        TIntHashSet got = new TIntHashSet();
        Random rnd = rsgen.getRandom();
        while (got.size() < numNeeded) {
            int index = rnd.nextInt(all.size());
            int value = all.get(index);
            if (got.contains(value) == false) {
                got.add(value);
                all.remove(index);
            }
        }

        int[] ret = got.toArray();
        return ret;
    }

    protected static int[] pickNoRepeatsAndRemoveUniformly(int numNeeded, TIntArrayList all) {

        TIntHashSet uniq = new TIntHashSet(all.toNativeArray());
        if (uniq.size() < numNeeded) {
            return null;
            //throw new IllegalArgumentException("uniq: " + uniq.size() + " numNeeded: " + numNeeded);
        }

        TIntHashSet got = new TIntHashSet();

        for (int i = 0; i < all.size(); i++) {
            int value = all.get(i);
            if (got.contains(value) == false) {
                all.remove(i);
                got.add(value);
            }

            if (got.size() == numNeeded) {
                break;
            }
        }

        int[] ret = got.toArray();
        return ret;
    }

    protected static TIntArrayList sortByFreq(TIntArrayList origList) {

        TIntIntHashMap map = new TIntIntHashMap();
        for (int i = 0; i < origList.size(); i++) {
            int key = origList.get(i);
            int curr = map.get(key);
            curr++;
            map.put(key, curr);
        }

        DoubleElement[] dels = new DoubleElement[map.size()];
        TIntIntIterator iterator = map.iterator();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            int key = iterator.key();
            int count = iterator.value();
            dels[i] = new DoubleElement(key, count);
        }

        dels = DoubleElement.sort(SortMode.REAL, Order.DESCENDING, dels);

        TIntArrayList newList = new TIntArrayList();
        for (int i = 0; i < dels.length; i++) {
            int index = dels[i].fIndex;
            int cnt = (int) dels[i].fValue;
            for (int j = 0; j < cnt; j++) {
                newList.add(index);
            }
            System.out.println(index + "=" + cnt);
        }

        if (newList.size() != origList.size()) {
            throw new IllegalStateException("newList.size: " + newList.size() + " origList.size: " + origList.size());
        }

        return newList;
    }

    protected static int correctedNumPerms(int numPerms, int largeClassSize) {
        int correctedNumPerms = numPerms / largeClassSize;
        correctedNumPerms = correctedNumPerms * largeClassSize;

        if (correctedNumPerms % largeClassSize != 0) {
            throw new IllegalStateException("correctedNumPerms: " + correctedNumPerms + " largeClassSize: " + largeClassSize);
        }


        klog.info("#of perms: " + numPerms + " corrected num: " + correctedNumPerms);

        return correctedNumPerms;
    }


    protected static Template createAuxRndTemplate(Template orig,
                                                   Template.Item[] realA_for_class_A,
                                                   Template.Item[] fauxB_for_class_A,
                                                   Template.Item[] realB_for_class_B,
                                                   Template.Item[] fauxA_for_class_B) {

        TIntHashSet profilePos = new TIntHashSet();

        TemplateImpl newt = new TemplateImpl(orig.getName());
        TemplateImpl.ClassImpl newcA = new TemplateImpl.ClassImpl(orig.getClass(0).getName());
        TemplateImpl.ClassImpl newcB = new TemplateImpl.ClassImpl(orig.getClass(1).getName());

        // first class A
        for (int i = 0; i < realA_for_class_A.length; i++) {
            Template.Item newItem = TemplateImpl.ItemImpl.createItem(realA_for_class_A[i].getId(), realA_for_class_A[i].getProfilePosition());
            newcA.add(newItem);
            newt.add(newItem);
            profilePos.add(realA_for_class_A[i].getProfilePosition());
        }

        for (int i = 0; i < fauxB_for_class_A.length; i++) {
            Template.Item newItem = TemplateImpl.ItemImpl.createItem(fauxB_for_class_A[i].getId(), fauxB_for_class_A[i].getProfilePosition());
            newcA.add(newItem);
            newt.add(newItem);
            profilePos.add(fauxB_for_class_A[i].getProfilePosition());
        }

        // Then class B
        for (int i = 0; i < realB_for_class_B.length; i++) {
            Template.Item newItem = TemplateImpl.ItemImpl.createItem(realB_for_class_B[i].getId(), realB_for_class_B[i].getProfilePosition());
            newcB.add(newItem);
            newt.add(newItem);
            profilePos.add(realB_for_class_B[i].getProfilePosition());
        }

        for (int i = 0; i < fauxA_for_class_B.length; i++) {
            Template.Item newItem = TemplateImpl.ItemImpl.createItem(fauxA_for_class_B[i].getId(), fauxA_for_class_B[i].getProfilePosition());
            newcB.add(newItem);
            newt.add(newItem);
            profilePos.add(fauxA_for_class_B[i].getProfilePosition());
        }

        int tot = realA_for_class_A.length + realB_for_class_B.length + fauxA_for_class_B.length + fauxB_for_class_A.length;
        if (profilePos.size() != tot) {
            throw new IllegalStateException("uniq profile pos: " + profilePos.size() + " tot: " + tot);
        }

        newt.add(newcA);
        newt.add(newcB);

        //newt.assignItems();  // DO NOT call - custom addition already done above

        newt.setClassOfInterestIndex(orig.getClassOfInterestIndex());

        if (orig.isAux()) {
            newt.setAux(true);
        }

        if (orig.isContinuous()) {
            newt.setContinuous(true);
        }

        newt.makeImmutable();

        return newt;
    }

    protected static class Binned {
        int[] bins;
        int[] extra_indixes;
    }

    protected static Binned createCounts(int cntPerBin, int numBins) {

        //int cntPerBin = totalCounts / numBins;
        int[] bins = new int[numBins];
        //int totalBinned = 0;
        for (int i = 0; i < numBins; i++) {
            bins[i] = cntPerBin;
            //totalBinned += cntPerBin;
        }

        /*
        int num2MakeUp = totalCounts - totalBinned;
        int[] all_indices = XMath.toIndices(numBins, false);
        int[] xx = XMath.toIndices(all_indices, exclude_for_extras, false);
        int[] indices = XMath.randomlySampleWithoutReplacement(num2MakeUp, xx, rsgen);
        for (int i=0; i < indices.length; i++) {
            bins[indices[i]]++;
        }

        // sanity check
        int sum = XMath.sum(bins);
        if (sum != totalCounts) {
            throw new IllegalStateException("bins: " + sum + " totalCounts: " + totalCounts);
        }
        */

        Binned binned = new Binned();
        binned.bins = bins;
        //binned.extra_indixes = indices;
        return binned;
    }

    protected static ClassBags createRealAndFauxIndexBags(final int cntPerItem,
                                                          final Template.Class cl) {

        TemplateFactory_helper.Binned real_binned = TemplateFactory_helper.createCounts(cntPerItem / 2, cl.getSize());
        TemplateFactory_helper.Binned faux_binned = TemplateFactory_helper.createCounts(cntPerItem / 2, cl.getSize());
        final ClassBags cs = new ClassBags();
        for (int i = 0; i < cl.getSize(); i++) {
            Template.Item item = cl.getItem(i);
            cs.initRealTo(item, real_binned.bins[i]);
            cs.initFauxTo(item, faux_binned.bins[i]);
        }

        return cs;

        /*
        int numPerItem = totalNumberOfSamplesNeededForAllPermutations / cl.getSize();

        final int num_times_one_item_is_seen_on_real_side = numPerItem / 2;
        final int num_times_one_item_is_seen_on_faux_side = numPerItem / 2;

        klog.debug("numPerItem: " + numPerItem + " num_times_one_item_is_seen_on_real_side: " + num_times_one_item_is_seen_on_real_side);

        final ClassBags cs = new ClassBags();

        int real_tot_actual = 0;
        int faux_tot_actual = 0;

        // First add the easy stuff - equal numbers of real and faux
        // every item gets its counter set
        for (int i = 0; i < cl.getSize(); i++) {
            Template.Item item = cl.getItem(i);
            cs.initRealTo(item, num_times_one_item_is_seen_on_real_side);
            real_tot_actual += num_times_one_item_is_seen_on_real_side;
            cs.initFauxTo(item, num_times_one_item_is_seen_on_faux_side);
            faux_tot_actual += num_times_one_item_is_seen_on_faux_side;
        }


        // now need to randomly add any that are left
        int make_up_real_cnt = totalNumberOfSamplesNeededForAllPermutations / 2 - real_tot_actual;
        klog.debug("totalNumberOfSamplesNeededForAllPermutations: " + totalNumberOfSamplesNeededForAllPermutations + " make_up_real_cnt: " + make_up_real_cnt);
        int[] inds = XMath.randomlyPickIndicesWithoutReplacement(make_up_real_cnt, cl.getSize(), rsgen);
        for (int i = 0; i < inds.length; i++) {
            cs.incrementReal(cl.getItem(inds[i]));
        }

        // similarly for the fauxs
        int make_up_faux_cnt = totalNumberOfSamplesNeededForAllPermutations / 2 - faux_tot_actual;
        klog.debug("totalNumberOfSamplesNeededForAllPermutations: " + totalNumberOfSamplesNeededForAllPermutations + " make_up_faux_cnt: " + make_up_faux_cnt);
        inds = XMath.randomlyPickIndicesWithoutReplacement(make_up_faux_cnt, cl.getSize(), rsgen);
        for (int i = 0; i < inds.length; i++) {
            cs.incrementFaux(cl.getItem(inds[i]));
        }

        // sanity check
        int total = cs.getRealTotal() + cs.getFauxTotal();
        if (total != totalNumberOfSamplesNeededForAllPermutations) {
            throw new IllegalStateException("totalNumberOfSamplesNeededForAllPermutations: " + totalNumberOfSamplesNeededForAllPermutations + " " + cs.getRealTotal() + " " + cs.getFauxTotal());
        }


        // sizes (i.e number of item keys) should be equal thiough the actual counts
        // might differ
        if (cs.realBucket.size() != cs.fauxBucket.size()) {
            throw new IllegalStateException("real bucket size: " + cs.realBucket.size() + " " + cs.fauxBucket.size());
        }

        return cs;
        */
    }

    protected static TemplateFactory_helper.ClassBags createRealAndFauxIndexBags(final int totalNumSamplesNeededAcrossAllPermutationsPerClass,
                                                                                 final Template.Class cl, RandomSeedGenerator rsgen) {


        int cntPerBin = totalNumSamplesNeededAcrossAllPermutationsPerClass / cl.getSize();
        int[] bins = new int[cl.getSize()];
        int totalBinned = 0;
        for (int i = 0; i < cl.getSize(); i++) {
            bins[i] = cntPerBin;
            totalBinned += cntPerBin;
        }

        int num2MakeUp = totalNumSamplesNeededAcrossAllPermutationsPerClass - totalBinned;
        int[] all_indices = XMath.toIndices(cl.getSize(), false);
        int[] indices = XMath.randomlySampleWithoutReplacement(num2MakeUp, all_indices, rsgen);
        for (int i = 0; i < indices.length; i++) {
            bins[indices[i]]++;
        }

        // sanity check
        int sum = XMath.sum(bins);
        if (sum != totalNumSamplesNeededAcrossAllPermutationsPerClass) {
            throw new IllegalStateException("bins: " + sum + " totalCounts: " + totalNumSamplesNeededAcrossAllPermutationsPerClass);
        }

        final TemplateFactory_helper.ClassBags cs = new TemplateFactory_helper.ClassBags();
        for (int i = 0; i < cl.getSize(); i++) {
            Template.Item item = cl.getItem(i);
            cs.initRealTo(item, bins[i]);
            cs.initFauxTo(item, bins[i]);
        }

        return cs;
    }

    protected static boolean endsWithMagicWord(String line) {
        return (line.endsWith(Constants.ONE_VERSUS_ALL) || line.endsWith(Constants.ALL_PAIRS) || line.endsWith(Constants.ONE_VERSUS_ALL_ONLY_FORWARD));
    }

    static class ClassBags {
        TObjectIntHashMap realBucket;
        TIntObjectHashMap realBucketTracker;
        TObjectIntHashMap fauxBucket;
        TIntObjectHashMap fauxBucketTracker;


        ClassBags() {
            this.realBucket = new TObjectIntHashMap();
            this.fauxBucket = new TObjectIntHashMap();
            this.realBucketTracker = new TIntObjectHashMap();
            this.fauxBucketTracker = new TIntObjectHashMap();
        }

        int getTotalBoth() {
            return _tot(realBucket) + _tot(fauxBucket);
        }

        void initRealTo(Template.Item item, int cnt) {
            if (cnt == 0) {
                throw new IllegalArgumentException("Cannot init to 0: " + cnt);
            }

            if (realBucketTracker.containsKey(item.getProfilePosition())) {
                throw new IllegalArgumentException("Duplicate item: " + item.getProfilePosition());
            }

            //klog.debug("Inited Real to: " + cnt);
            realBucket.put(item, cnt);
            realBucketTracker.put(item.getProfilePosition(), item);
        }

        void initFauxTo(Template.Item item, int cnt) {
            if (cnt == 0) {
                throw new IllegalArgumentException("Cannot init to 0: " + cnt);
            }

            if (fauxBucketTracker.containsKey(item.getProfilePosition())) {
                throw new IllegalArgumentException("Duplicate item: " + item.getProfilePosition());
            }

            //klog.debug("Inited Faux to: " + cnt);
            fauxBucket.put(item, cnt);
            fauxBucketTracker.put(item.getProfilePosition(), item);
        }

        void incrementReal(Template.Item item) {
            int curr = realBucket.get(item);
            if (curr == 0) {
                throw new IllegalArgumentException("Huh: " + curr);
            }

            curr++;
            realBucket.put(item, curr);
        }

        void incrementFaux(Template.Item item) {
            int curr = fauxBucket.get(item);
            if (curr == 0) {
                throw new IllegalArgumentException("Huh: " + curr);
            }
            curr++;
            fauxBucket.put(item, curr);
        }

        int getRealTotal() {
            return _tot(realBucket);
        }

        int getFauxTotal() {
            return _tot(fauxBucket);
        }

        static int _tot(TObjectIntHashMap map) {
            int tot = 0;
            TObjectIntIterator iterator = map.iterator();
            for (int i = map.size(); i-- > 0;) {
                iterator.advance();
                tot += iterator.value();
            }

            return tot;
        }

        void printfReal() {
            _printf(realBucket);
        }

        void printfFaux() {
            _printf(fauxBucket);
        }

        void _printf(TObjectIntHashMap map) {
            TObjectIntIterator iterator = map.iterator();
            for (int i = map.size(); i-- > 0;) {
                iterator.advance();
                System.out.println(((Template.Item) iterator.key()).getProfilePosition() + " " + iterator.value());
            }
        }

        Items2Use createRealAndFauxItems(int num2Pick, RandomSeedGenerator rsgen) {
            final int[] real_item_positions = realBucketTracker.keys(); // currently available items for the real side
            final int[] faux_item_positions = fauxBucketTracker.keys(); // currently available items for the faux side

            klog.debug("num2Pick: " + num2Pick + " remaining real_items: " + real_item_positions.length + " Remaining faux_items: " + faux_item_positions.length);
            klog.debug("real_item_positions: " + real_item_positions.length + " >" + Printf.format(real_item_positions));
            klog.debug("faux_item_positions: " + faux_item_positions.length + " >" + Printf.format(faux_item_positions));

            int[] first_pick_choices;
            int[] second_pick_choices;
            if (real_item_positions.length <= faux_item_positions.length) { // pick from the smaller bag first
                first_pick_choices = real_item_positions;
                second_pick_choices = faux_item_positions;
            } else {
                first_pick_choices = faux_item_positions;
                second_pick_choices = real_item_positions;
            }

            /* ok as some buckets could get emptied faster than others
            if (real_items.length != faux_items.length) { // must be equal in sizes though inside counts could differ
                throw new IllegalStateException("Remaining real_items: " + real_items.length + " but Remaining faux_items: " + faux_items.length);
            }
            */

            final TIntArrayList first_positions_to_use_list = new TIntArrayList();
            // In first positions, any that arent even a second option MUST be used
            final int[] first_position_musts = XMath.toIndices(first_pick_choices, second_pick_choices, false);
            klog.debug("first_position_musts: " + first_position_musts.length + " >" + Printf.format(first_position_musts));
            if (first_position_musts.length > num2Pick) { // need less than the musts, so discard some
                klog.debug("SUBSAMPLING MUSTS: " + first_position_musts.length + " num2Pick: " + num2Pick);
                int[] add = XMath.randomlySampleWithoutReplacement(num2Pick, first_position_musts, rsgen);
                first_positions_to_use_list.add(add);
            } else if (first_position_musts.length < num2Pick) { // happens most of the time
                int more_first_needed = num2Pick - first_position_musts.length;
                int[] xx = XMath.toIndices(first_pick_choices, first_position_musts, false);
                int[] add = XMath.randomlySampleWithoutReplacement(more_first_needed, xx, rsgen);
                first_positions_to_use_list.add(first_position_musts);
                first_positions_to_use_list.add(add);
            } else {
                first_positions_to_use_list.add(first_position_musts);
            }

            final int[] first_positions_to_use = first_positions_to_use_list.toNativeArray();
            final int[] available_second_pick_positions = XMath.toIndices(second_pick_choices, first_positions_to_use, false);

            klog.debug("first_positions_to_use: " + first_positions_to_use.length + " >" + Printf.format(first_positions_to_use));
            klog.debug("available_second_pick_positions : " + available_second_pick_positions.length + " >" + Printf.format(available_second_pick_positions));

            final int[] second_positions_to_use = XMath.randomlySampleWithoutReplacement(num2Pick, available_second_pick_positions, rsgen);
            final Items2Use i2u = new Items2Use();

            if (first_positions_to_use.length != second_positions_to_use.length) {
                throw new IllegalStateException("first_positions_to_use: " + first_positions_to_use.length + " second_positions_to_use: " + second_positions_to_use.length);
            }

            if (real_item_positions.length <= faux_item_positions.length) {
                i2u.real = extractItems(first_positions_to_use, realBucketTracker, realBucket);
                i2u.faux = extractItems(second_positions_to_use, fauxBucketTracker, fauxBucket);
            } else {
                i2u.real = extractItems(second_positions_to_use, realBucketTracker, realBucket);
                i2u.faux = extractItems(first_positions_to_use, fauxBucketTracker, fauxBucket);
            }

            i2u.checkUniqueness();

            if (i2u.real.length != i2u.faux.length) {
                throw new IllegalStateException("i2u.real.length: " + i2u.real.length + " i2u.faux.length: " + i2u.faux.length);
            }

            return i2u;
        }

        Template.Item[] extractItems(int[] positions, TIntObjectHashMap intMap, TObjectIntHashMap objMap) {

            Template.Item[] ret = new Template.Item[positions.length];
            for (int i = 0; i < positions.length; i++) {
                Template.Item item2use = (Template.Item) intMap.get(positions[i]);
                int curr = objMap.get(item2use);
                if (curr == 0) {
                    throw new IllegalStateException("Empty items must have been removed: " + curr);
                }

                curr--;
                if (curr == 0) {
                    objMap.remove(item2use);
                    intMap.remove(positions[i]);
                } else {
                    objMap.put(item2use, curr);
                }
                ret[i] = item2use;
            }

            return ret;
        }

    } // End inner class ClassBags

    static class Items2Use {
        Template.Item[] real;
        Template.Item[] faux;

        void checkUniqueness() {
            TIntHashSet set = new TIntHashSet();
            for (int i = 0; i < real.length; i++) {
                set.add(real[i].getProfilePosition());
            }

            for (int i = 0; i < faux.length; i++) {
                set.add(faux[i].getProfilePosition());
            }

            if (set.size() != real.length + faux.length) {
                throw new IllegalStateException("set: " + set.size() + " real: " + real.length + " faux: " + faux.length);
            }


        }

    }

}
