package edu.mit.broad.genome.objects.esmatrix.db;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.NamedColor;
import edu.mit.broad.genome.utils.Timer;
import edu.mit.broad.vdb.sampledb.SampleAnnot;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Jan 23, 2006
 *         Time: 8:14:31 PM
 */
public abstract class AbstractEnrichmentDbMulti extends AbstractObject implements EnrichmentDbMulti {

    private List fRowNames;

    private GeneSet fRowNamesGeneSet;

    private List fColNames; // i.e edb names

    private GeneSet fColNamesGeneSet;

    private int fNumRows;

    protected Dataset essDataset;

    protected Dataset nessDataset;

    protected Dataset npDataset;

    protected Dataset fdrDataset;

    /**
     * Class constructor
     */
    public AbstractEnrichmentDbMulti() {

    }

    // subclasses must call
    protected void init(final String name,
                        final String[] rowNames,
                        final String[] edbNames) {

        super.initialize(name);

        if (edbNames == null) {
            throw new IllegalArgumentException("Param edbNames cannot be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Param rowNames cannot be null");
        }

        this.fColNames = new ArrayList();
        for (int c = 0; c < edbNames.length; c++) {
            this.fColNames.add(edbNames[c]);
        }

        this.fRowNames = new ArrayList();
        for (int r = 0; r < rowNames.length; r++) {
            this.fRowNames.add(rowNames[r]);
        }

        this.fNumRows = rowNames.length;
    }

    public Dataset getESSDataset() {
        if (essDataset == null) {
            log.debug("Starting essDataset gen");
            Timer timer = new Timer();
            Matrix m = new Matrix(getNumRow(), getNumEdbs());
            for (int c = 0; c < getNumEdbs(); c++) {
                m.setColumn(c, getEdb(c).getESS());
                if (c % 5 == 0) {
                    System.out.println("Done ess: " + (c + 1) + " / " + getNumEdbs());
                }
            }

            timer.stop();
            timer.printTimeTakenS("Retrieved essDataset from edb");
            this.essDataset = new DefaultDataset(getName() + "_ess", m, getRowNamesArray(), getEdbNamesArray(), true);
            ((DefaultDataset) essDataset).cloneShallow(_ann());
        }

        return essDataset;
    }

    public Dataset getNESSDataset() {
        if (nessDataset == null) {
            log.debug("Starting nessDataset gen");
            Timer timer = new Timer();
            Matrix m = new Matrix(getNumRow(), getNumEdbs());
            for (int c = 0; c < getNumEdbs(); c++) {
                m.setColumn(c, getEdb(c).getNESS());
            }

            timer.stop();
            timer.printTimeTaken("Retrieved nessDataset from edb");
            this.nessDataset = new DefaultDataset(getName() + "_ness", m, getRowNamesArray(), getEdbNamesArray(), true);
            ((DefaultDataset) nessDataset).cloneShallow(_ann());
        }

        return nessDataset;
    }

    public Dataset getNPDataset() {
        if (npDataset == null) {
            log.debug("Starting npDataset gen");
            Timer timer = new Timer();
            Matrix m = new Matrix(getNumRow(), getNumEdbs());
            for (int c = 0; c < getNumEdbs(); c++) {
                m.setColumn(c, getEdb(c).getNPs());
            }

            timer.stop();
            timer.printTimeTaken("Retrieved npDataset from edb");
            this.npDataset = new DefaultDataset(getName() + "_np", m, getRowNamesArray(), getEdbNamesArray(), true);
            ((DefaultDataset) npDataset).cloneShallow(_ann());
        }

        return npDataset;
    }

    public Dataset getFDRDataset() {
        if (fdrDataset == null) {
            log.debug("Starting fdrDataset gen");
            Timer timer = new Timer();
            Matrix m = new Matrix(getNumRow(), getNumEdbs());
            for (int c = 0; c < getNumEdbs(); c++) {
                m.setColumn(c, getEdb(c).getFDRs());
            }

            timer.stop();
            timer.printTimeTaken("Retrieved fdrDataset from edb");
            this.fdrDataset = new DefaultDataset(getName() + "_fdr", m, getRowNamesArray(), getEdbNamesArray(), true);
            ((DefaultDataset) fdrDataset).cloneShallow(_ann());
        }

        return fdrDataset;
    }


    // @todo better msigdb annotations
    private Annot _ann() {
        FeatureAnnot da = new FeatureAnnotImpl("foo", getRowNames(), null);
        SampleAnnot sa = new SampleAnnotImpl("bar", getEdbNamesArray(), new ColumnsImpl(new NamedColor[]{}));
        return new AnnotImpl(da, sa);
    }

    public int getNumEdbs() {
        return fColNames.size();
    }

    public String getQuickInfo() {
        return null;
    }

    // -------------------------------------------------------------------------------------------- //

    // @note the order will be the order in here

    public EnrichmentResult[] getResults(final String gsetName) { // across all comps

        EnrichmentResult[] results = new EnrichmentResult[getNumEdbs()];
        for (int r = 0; r < results.length; r++) {
            results[r] = getEdb(r).getResultForGeneSet(gsetName);
        }

        return results;
    }
    // -------------------------------------------------------------------------------------------- //

    public List getRowNames() {
        return Collections.unmodifiableList(fRowNames);
    }

    public GeneSet getRowNamesGeneSet() {
        if (fRowNamesGeneSet == null) {
            fRowNamesGeneSet = new FSet(getName(), new HashSet(fRowNames));
        }
        return fRowNamesGeneSet;
    }

    public String[] getRowNamesArray() {
        return (String[]) fRowNames.toArray(new String[fRowNames.size()]);
    }

    public int getNumRow() {
        return fNumRows;
    }

    // -------------------------------------------------------------------------------------------- //

    public List getEdbNames() {
        return Collections.unmodifiableList(fColNames);
    }

    public GeneSet getEdbNamesGeneSet() {
        if (fColNamesGeneSet == null) {
            fColNamesGeneSet = new FSet(getName(), new HashSet(fColNames));
        }
        return fColNamesGeneSet;
    }

    public String[] getEdbNamesArray() {
        return (String[]) fColNames.toArray(new String[fColNames.size()]);
    }

    public String getEdbName(final int coln) {
        return (String) fColNames.get(coln);
    }

    public int getEdbIndex(final String colName) {
        return fColNames.indexOf(colName);
    }

    // -------------------------------------------------------------------------------------------- //


    public EnrichmentDb[] getEdbs(final String[] compNames) {
        EnrichmentDb[] edbs = new EnrichmentDb[compNames.length];
        for (int r = 0; r < compNames.length; r++) {
            edbs[r] = getEdb(compNames[r]);
        }

        return edbs;
    }

    public EnrichmentDb[] getEdbs(final int[] compIndices) {
        EnrichmentDb[] edbs = new EnrichmentDb[compIndices.length];
        for (int r = 0; r < compIndices.length; r++) {
            edbs[r] = getEdb(compIndices[r]);
        }

        return edbs;
    }

} // End class AbstractEnrichmentDbMulti
