package edu.mit.broad.genome.objects.esmatrix.db;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.alg.gsea.EdbAlgs;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.FdrStruc;

import java.io.File;
import java.math.BigDecimal;
import java.util.*;

/**
 *
 * @author Aravind Subramanian
 * @revised by Qing Xiong <qxiong@email.unc.edu>
 * Date: Jan 4, 2013
 * Time: 12:31 PM
 */
public abstract class AbstractEnrichmentDbSeq extends AbstractObject implements EnrichmentDb {

    private LabelledVectorProcessor fLvp;
    private SortMode fSortMode;
    private Order fOrder;
    private int fNumPerms;
    private File fEdbDir_opt;
    private Map fGeneSetNameResultMap;

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    // Subclasses must call init before doing much

    protected AbstractEnrichmentDbSeq() {

    }

    public EnrichmentDb cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    /**
     * Class constructor
     *
     * @param name
     */
    public void init(final String name,
                     final LabelledVectorProcessor lvp,
                     final SortMode sort,
                     final Order order,
                     final int nperm,
                     final File edb_dir_opt) {


        if (lvp == null) {
            throw new IllegalArgumentException("Param lvp cannot be null");
        }
        if (sort == null) {
            throw new IllegalArgumentException("Param sort cannot be null");
        }
        if (order == null) {
            throw new IllegalArgumentException("Param order cannot be null");
        }

        super.initialize(name);

        this.fLvp = lvp;
        this.fSortMode = sort;
        this.fOrder = order;
        this.fNumPerms = nperm;
        this.fEdbDir_opt = edb_dir_opt;
        initResultMaps();
    }

    public File getEdbDir() {
        return fEdbDir_opt;
    }

    protected void initResultMaps() {
        if (fGeneSetNameResultMap == null) {

            /*
            final Errors errors = new Errors();

            this.fNameEnResultMap = new HashMap();
            for (int i = 0; i < getNumResults(); i++) {
                EnrichmentResult result = getResult(i);
                System.out.println("%%% " + result.getName() + " " + result.getRankedList().getName());
                if (fNameEnResultMap.containsKey(result.getName())) {
                    errors.add("duplicate enrichment result name: " + result.getName() + " \n" + fNameEnResultMap.keySet() +" # results: " + getNumRecords());
                }
                fNameEnResultMap.put(result.getName(), result);
            }

            // @note IMP dont barf why not??
            errors.barfIfNotEmptyRuntime();
            */

            // Enforce that gene sets are unique (barf here)
            this.fGeneSetNameResultMap = EdbAlgs.hashByGeneSetName(getResults());
        }
    }

    // -------------------------------------------------------------------------------------------- //
    public int getNumScores(final boolean pos) {
        int cnt = 0;
        for (int i = 0; i < getNumResults(); i++) {

            if (pos) {

                if (XMath.isPositive(getResult(i).getScore().getES())) {
                    cnt++;
                }
            } else {
                if (XMath.isNegative(getResult(i).getScore().getES())) {
                    cnt++;
                }
            }
        }

        return cnt;
    }
    
    public int getNumScoresB(final boolean pos) {
        int cnt = 0;
        for (int i = 0; i < getNumResults(); i++) {
            int flag = (getResult(i).getScore().getESB()).compareTo(BigDecimal.ZERO);
            if (pos) {
                if (flag==1||flag==0) {
                    cnt++;
                }
            } else {
                if (flag==-1) {
                    cnt++;
                }
            }
        }

        return cnt;
    }    

    public EnrichmentResult[] getResults(float npMaxInclusive, float fdrMaxInclusive, final Comparator comp) {
        List list = new ArrayList();
        for (int i = 0; i < getNumResults(); i++) {
            EnrichmentResult res = getResult(i);
            if (res.getScore().getNP() <= npMaxInclusive && res.getScore().getFDR() <= fdrMaxInclusive) {
                list.add(res);
            }
        }

        Collections.sort(list, comp);
        return (EnrichmentResult[]) list.toArray(new EnrichmentResult[list.size()]);
    }

    public int getNumNominallySigB(final float npCutoffInclusive, final boolean pos) {
        int cnt = 0;
        for (int i = 0; i < getNumResults(); i++) {
            EnrichmentResult res = getResult(i);
            int flag1 = (res.getScore().getESB()).compareTo(BigDecimal.ZERO);
            int flag2 = (res.getScore().getNPB()).compareTo(new BigDecimal(npCutoffInclusive));
            if (flag1==1 && pos) {
                if (flag2==-1||flag2==0) {
                    cnt++;
                }
            }

            if (flag1==-1 && !pos) {
                if (flag2==-1||flag2==0) {
                    cnt++;
                }
            }

        }

        return cnt;
    }
    
    public int getNumNominallySig(final float npCutoffInclusive, final boolean pos) {
        int cnt = 0;
        for (int i = 0; i < getNumResults(); i++) {
            EnrichmentResult res = getResult(i);
            if (res.getScore().getES() > 0 && pos) {
                if (res.getScore().getNP() <= npCutoffInclusive) {
                    cnt++;
                }
            }

            if (res.getScore().getES() < 0 && !pos) {
                if (res.getScore().getNP() <= npCutoffInclusive) {
                    cnt++;
                }
            }

        }

        return cnt;
    }    

    public int getNumFDRSig(final float fdrCutoffInclusive, final boolean pos) {
        return getFDRSig(fdrCutoffInclusive, pos).length;
    }

    public FdrStruc[] getFDRSig(final float fdrCutoffInclusive, final boolean pos) {
        final FdrStruc[] fdrs = getFDR(pos);

        final List list = new ArrayList();
        for (int i = 0; i < fdrs.length; i++) {
            if (fdrs[i].getFdr() <= fdrCutoffInclusive) {
                list.add(fdrs[i]);
            }
        }

        return (FdrStruc[]) list.toArray(new FdrStruc[list.size()]);
    }
    
    public int getNumFDRSigSeqP(final float npCutoffInclusive, final boolean pos) {
        int cnt = 0;
        for (int i = 0; i < getNumResults(); i++) {
            EnrichmentResult res = getResult(i);
            if (res.getScore().getES() > 0 && pos) {
                if (res.getScore().getFDR() <= npCutoffInclusive) {
                    cnt++;
                }
            }

            if (res.getScore().getES() < 0 && !pos) {
                if (res.getScore().getFDR() <= npCutoffInclusive) {
                    cnt++;
                }
            }

        }

        return cnt;
    }    
    
    public int getNumFDRSigSeqPB(final float npCutoffInclusive, final boolean pos) {
        int cnt = 0;
        for (int i = 0; i < getNumResults(); i++) {
            EnrichmentResult res = getResult(i);
            int flag1 = (res.getScore().getESB()).compareTo(BigDecimal.ZERO);
            int flag2 = (res.getScore().getFDRB()).compareTo(new BigDecimal(npCutoffInclusive));            
            if (flag1==1 && pos) {
                if (flag2==-1||flag2==0) {
                    cnt++;
                }
            }

            if (flag1==-1 && !pos) {
                if (flag2==-1||flag2==0) {
                    cnt++;
                }
            }

        }

        return cnt;
    }     


    // lazilly init
    private FdrStruc[] fFdrPos;
    private FdrStruc[] fFdrNeg;

    private void initFDR() {

        if (fFdrPos != null) {
            return;
        }

        final List pos = new ArrayList();
        final List neg = new ArrayList();

        for (int i = 0; i < getNumResults(); i++) {
            final FdrStruc fdr = getResult(i).getFDR();
            if (fdr != null) {
                if (XMath.isPositive(fdr.getRealScore())) {
                    pos.add(fdr);
                } else {
                    neg.add(fdr);
                }
            }
        }

        this.fFdrPos = (FdrStruc[]) pos.toArray(new FdrStruc[pos.size()]);
        this.fFdrNeg = (FdrStruc[]) neg.toArray(new FdrStruc[neg.size()]);
    }

    public FdrStruc[] getFDR_all() {
        initFDR();
        List list = new ArrayList();
        for (int i = 0; i < fFdrPos.length; i++) {
            list.add(fFdrPos[i]);
        }

        for (int i = 0; i < fFdrNeg.length; i++) {
            list.add(fFdrNeg[i]);
        }

        return (FdrStruc[]) list.toArray(new FdrStruc[list.size()]);
    }

    public FdrStruc[] getFDR(boolean pos) {
        initFDR();
        if (pos && fFdrPos == null) {
            throw new IllegalStateException("Fdrs not yet calculated: " + pos);
        }

        if (!pos && fFdrNeg == null) {
            throw new IllegalStateException("Fdrs not yet calculated: " + pos);
        }


        if (pos) {
            return fFdrPos;
        } else {
            return fFdrNeg;
        }
    }

    public boolean containsGeneSet(String gsetName_no_aux) {
        return fGeneSetNameResultMap.containsKey(gsetName_no_aux);
    }

    public EnrichmentResult getResultForGeneSet(String gsetName_no_aux) {
        Object obj = fGeneSetNameResultMap.get(gsetName_no_aux);
        if (obj == null) {
            throw new IllegalArgumentException("No such gset result: " + gsetName_no_aux);
        } else {
            return (EnrichmentResult) obj;
        }
    }

    public String getQuickInfo() {
        return null;
    }

    public IDataframe getDataframe() {
        throw new NotImplementedException(); //@todo
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public GeneSet[] getGeneSets() {
        return EdbAlgs.getGeneSets(getResults());
    }

    public List getGeneSetNames() {
        return EdbAlgs.getGeneSetNames(getResults());
    }

    public String[] getGeneSetNamesArray() {
        List list = getGeneSetNames();
        return (String[]) list.toArray(new String[list.size()]);
    }

    public GeneSet[] getGeneSetSignals() {
        return EdbAlgs.getGeneSetSignals(getResults());
    }


    public GeneSet[] getGeneSets(final float npMaxInclusive,
                                 final float fdrMaxInclusive,
                                 final Comparator comp) {
        return EdbAlgs.getGeneSets(getResults(npMaxInclusive, fdrMaxInclusive, comp));
    }

    public GeneSet[] getGeneSetSignals(final float npMaxInclusive,
                                       final float fdrMaxInclusive,
                                       final Comparator comp) {
        return EdbAlgs.getGeneSetSignals(getResults(npMaxInclusive, fdrMaxInclusive, comp));
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /*
    public String[] getRecordNames() {
        return getResultNames();
    }

    public List getRecordNamesList() {
        return getResultNamesList();
    }

    public int getNumRecords() {
        return getNumResults();
    }

     public EnrichmentResult getResult(final String name) {

        final Object obj = fNameEnResultMap.get(name);

        if (obj == null) {
            throw new IllegalArgumentException("No such enrichment result: " + name);
        } else {
            return (EnrichmentResult) obj;
        }
    }

    public List getResultNamesList() {
        return Arrays.asList(getResultNames());
    }

    public String[] getResultNames() {
        String[] names = new String[getNumResults()];
        for (int i = 0; i < getNumRecords(); i++) {
            names[i] = getResult(i).getName();
        }

        return names;
    }
    */

    public EnrichmentResult[] getResults(final Comparator comp) {
        List list = getResultsList(comp);
        return (EnrichmentResult[]) list.toArray(new EnrichmentResult[list.size()]);
    }

    public EnrichmentResult getResult(final int i) {
        return getResults()[i];
    }

    public List getResultsList(final Comparator comp) {
        List list = getResultsList();
        Collections.sort(list, comp);
        return list;
    }


    public EnrichmentResult[] getResults(final Comparator comp, final boolean pos) {
        final EnrichmentResult[] all = getResults(comp);
        final List sub = new ArrayList();
        for (int i = 0; i < all.length; i++) {
            float es = all[i].getScore().getES();
            if (pos && XMath.isPositive(es)) {
                sub.add(all[i]);
            }

            if (!pos && XMath.isNegative(es)) {
                sub.add(all[i]);
            }
        }

        return (EnrichmentResult[]) sub.toArray(new EnrichmentResult[sub.size()]);
    }
    
    public EnrichmentResult[] getResultsB(final Comparator comp, final boolean pos) {
        final EnrichmentResult[] all = getResults(comp);
        final List sub = new ArrayList();
        for (int i = 0; i < all.length; i++) {
            BigDecimal es = all[i].getScore().getESB();
            if (pos && (es.compareTo(BigDecimal.ZERO)==1||es.compareTo(BigDecimal.ZERO)==0)) {
                sub.add(all[i]);
            }

            if (!pos && es.compareTo(BigDecimal.ZERO)==-1) {
                sub.add(all[i]);
            }
        }

        return (EnrichmentResult[]) sub.toArray(new EnrichmentResult[sub.size()]);
    }    

    public int getNumResults() {
        return getResults().length;
    }

    // -------------------------------------------------------------------------------------------- //
    // ---------------------------------------- metrics strucc--------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public LabelledVectorProcessor getRankedListProcessor() {
        return fLvp;
    }

    public SortMode getSortMode() {
        return fSortMode;
    }

    public Order getOrder() {
        return fOrder;
    }

    public int getNumPerm() {
        return fNumPerms;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
// @todo using gse names for the lss is not always ok

    public Vector getESS() {
        final Vector ess = new Vector(getNumResults());
        for (int i = 0; i < getNumResults(); i++) {
            ess.setElement(i, getResult(i).getScore().getES());
        }

        return ess;
    }

    public LabelledVector getESS_lv() {
        return new LabelledVector(getName() + "_ess", getGeneSetNamesArray(), getESS());
    }

    public Vector getNESS() {
        final Vector ness = new Vector(getNumResults());
        for (int i = 0; i < getNumResults(); i++) {
            ness.setElement(i, getResult(i).getScore().getNES());
        }

        return ness;
    }

    public LabelledVector getNESS_lv() {
        return new LabelledVector(getName() + "_ness", getGeneSetNamesArray(), getNESS());
    }

    public RankedList getNESS_rl() {
        return new LabelledVector(getName() + "_ness", getGeneSetNamesArray(), getNESS()).sort(SortMode.REAL, Order.DESCENDING);
    }

    public Vector getNPs() {
        final Vector ess = new Vector(getNumResults());
        for (int i = 0; i < getNumResults(); i++) {
            ess.setElement(i, getResult(i).getScore().getNP());
        }

        return ess;
    }

    public LabelledVector getNPs_lv() {
        return new LabelledVector(getName() + "_np", getGeneSetNamesArray(), getNPs());
    }

    public Vector getFDRs() {
        final Vector fdsr = new Vector(getNumResults());
        for (int i = 0; i < getNumResults(); i++) {
            fdsr.setElement(i, getResult(i).getScore().getFDR());
        }

        return fdsr;
    }

    public LabelledVector getFDRs_lv() {
        return new LabelledVector(getName() + "_fdr", getGeneSetNamesArray(), getFDRs());
    }

    public Vector getFWERs() {
        final Vector ess = new Vector(getNumResults());
        for (int i = 0; i < getNumResults(); i++) {
            ess.setElement(i, getResult(i).getScore().getFWER());
        }

        return ess;
    }

    public LabelledVector getFWERs_lv() {
        return new LabelledVector(getName() + "_fwer", getGeneSetNamesArray(), getFWERs());
    }

    // @todo check if needed
    public boolean isCompatible(final EnrichmentDb edb) {
        throw new NotImplementedException();
    }

    public void checkCompatibility(final EnrichmentDb edb) throws IllegalArgumentException {
        throw new NotImplementedException();
    }

}
