/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.esmatrix.db;

import edu.mit.broad.genome.alg.gsea.EdbAlgs;
import edu.mit.broad.genome.alg.gsea.MhtMode;
import edu.mit.broad.genome.alg.gsea.MhtModes;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.ObjectMatrix;
import edu.mit.broad.genome.objects.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Dataset like Datastructure where each element is a Dtg
 */
public class DtgDataset extends AbstractObject {

    private List fRowNames;

    private List fColNames;

    // A matrix of Dtg elements
    private ObjectMatrix fMatrix;

    private MhtMode[] fMhtModes;

    /**
     * The matrix IS SHARED
     * Class constructor
     *
     * @param name
     * @param dtgMatrix
     * @param rowNames
     * @param colNames
     */
    public DtgDataset(final String name,
                      final ObjectMatrix dtgMatrix,
                      final List rowNames,
                      final List colNames) {

        super(name);

        if (dtgMatrix == null) {
            throw new IllegalArgumentException("Parameter dtgMatrix cannot be null");
        }

        if (rowNames == null) {
            throw new IllegalArgumentException("Parameter rowNames cannot be null");
        }

        if (colNames == null) {
            throw new IllegalArgumentException("Parameter colNames cannot be null");
        }

        if (dtgMatrix.getNumRow() != rowNames.size()) {
            throw new IllegalArgumentException("Mismatched nrows");
        }

        if (dtgMatrix.getNumCol() != colNames.size()) {
            throw new IllegalArgumentException("Mismatched ncols");
        }

        this.fMatrix = dtgMatrix;
        this.fRowNames = rowNames;
        this.fColNames = colNames;

        this.fMhtModes = new MhtMode[fMatrix.getNumCol()];
        for (int c = 0; c < fMatrix.getNumCol(); c++) {
            Object[] objs = fMatrix.getColumn(c);
            EnrichmentResult[] dtgs = new EnrichmentResult[objs.length]; // needed as direct casting causes exception
            for (int i = 0; i < objs.length; i++) {
                dtgs[i] = (EnrichmentResult) objs[i];
            }

            fMhtModes[c] = new MhtModes.VarMean(dtgs); // @note hardcoded, also note hat setting the dtg data
        }

    }

    public Dataset createRealESDataset() {
        Matrix matrix = new Matrix(getNumRow(), getNumCol());

        for (int r = 0; r < getNumRow(); r++) {
            for (int c = 0; c < getNumCol(); c++) {
                matrix.setElement(r, c, getRealEs(r, c));
            }
        }

        return new DefaultDataset(getName() + ".ES", matrix, getRowNames(), getColNames(), true, null);
    }

    public Dataset createRealNESDataset() {
        Matrix matrix = new Matrix(getNumRow(), getNumCol());

        for (int r = 0; r < getNumRow(); r++) {
            for (int c = 0; c < getNumCol(); c++) {
                matrix.setElement(r, c, getNes(r, c));
            }
        }

        return new DefaultDataset(getName() + ".NES", matrix, getRowNames(), getColNames(), true, null);
    }

    public int getNumRow() {
        return fRowNames.size();
    }

    public int getNumCol() {
        return fColNames.size();
    }

    public String getQuickInfo() {
        return getNumRow() + "x" + getNumCol();
    }

    public String getRowName(int row) {
        return fRowNames.get(row).toString();
    }

    public List getRowNames() {
        return Collections.unmodifiableList(fRowNames);
    }

    public List getColNames() {
        return Collections.unmodifiableList(fColNames);
    }

    public String getColName(int col) {
        return fColNames.get(col).toString();
    }

    public float getRealEs(int row, int col) {
        return getDtg(row, col).getScore().getES();
    }

    public float getNes(int row, int col) {
        return fMhtModes[col].getRealNES(row);
    }

    public float getNominalPValue(int row, int col) {
        return EdbAlgs.getNominalPValue(getDtg(row, col));
    }

    public float getDiscoveryPValue(int row, int col) {
        return fMhtModes[col].getDiscoveryPValue(row);
    }

    public float getValidationPValue(int row, int col) {
        return fMhtModes[col].getValidationPValue(row);
    }

    public EnrichmentResult getDtg(int row, int col) {
        return (EnrichmentResult) fMatrix.getElement(row, col);
    }

    public EnrichmentResult[] getDtgRow(int row) {
        EnrichmentResult[] dtgs = new EnrichmentResult[getNumRow()];
        for (int c = 0; c < dtgs.length; c++) {
            dtgs[c] = getDtg(row, c);
        }
        return dtgs;
    }

    public EnrichmentResult[] getDtgCol(int col) {
        EnrichmentResult[] dtgs = new EnrichmentResult[getNumCol()];
        for (int r = 0; r < dtgs.length; r++) {
            dtgs[r] = getDtg(r, col);
        }
        return dtgs;
    }

    // all rows names must be identical (i.e same geneset)
    public void ensureSameGeneSetsForEveryRow() {
        for (int r = 0; r < getNumRow(); r++) {
            ensureSameGeneSetsInRow(r);
        }
    }

    public void ensureSameGeneSetsInRow(int row) {
        GeneSet ref = null;
        for (int c = 0; c < fMatrix.getNumCol(); c++) {
            EnrichmentResult dtg = getDtg(row, c);
            if (ref == null) {
                ref = dtg.getGeneSet();
            }

            if (!dtg.getGeneSet().equalsShallow(ref)) {
                throw new IllegalStateException("Mismatched gensets: " + ref + " and " + dtg.getGeneSet());
            }
        }
    }

    // all templates must be identical for the entire dtgds
    public void ensureSameTemplate() {
        Template ref = null;

        for (int r = 0; r < getNumRow(); r++) {
            for (int c = 0; c < getNumCol(); c++) {
                EnrichmentResult dtg = getDtg(r, c);
                if (ref == null) {
                    ref = dtg.getTemplate();
                }
                if (!dtg.getTemplate().equalsShallow(ref)) {
                    throw new IllegalStateException("Mismatched templates: " + ref + " and " + dtg.getTemplate());
                }
            }
        }

    }

    public List getGeneSetNamesForCol(int col) {
        List names = new ArrayList();
        for (int r = 0; r < getNumRow(); r++) {
            names.add(getDtg(r, col).getGeneSet().getName());
        }

        return names;
    }

    public List getGeneSetsForCol(int col) {
        List names = new ArrayList();
        for (int r = 0; r < getNumRow(); r++) {
            names.add(getDtg(r, col).getGeneSet());
        }

        return names;
    }

} // End DtgDataset
