/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.esmatrix.db;

import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.math.LabelledVectorProcessor;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.FdrStruc;
import edu.mit.broad.vdb.chip.Chip;
import edu.duke.igsp.app.algorithms.CMetric;

import java.io.File;
import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Aug 6, 2003
 *         Time: 10:11:55 PM
 */
public interface SnpEnrichmentDb extends PersistentObject {

    public SnpEnrichmentDb cloneDeep(final EnrichmentResult[] results);

    public SnpEnrichmentDb cloneShallow(final String newName);

    public SnpEnrichmentDb cloneShallow(final Dataset gexDataset);

    //public EnrichmentDb cloneShallow(final RankedList rl);

    public File getEdbDir();

    // @note more eff to make this api part of edb multi
    //public EnrichmentDb cloneDeep(final String[] gsetNames);

    // -------------------------------------------------------------------------------------------- //
    // @todo reconsider: So far seems like an edb from one list is likely the norm rather than the exception
    // YES!! -> an edb is off one ranked list. For multi ranked lists see edb multi
    // so perhaps if a more granualr edb is needed do that as a different interface

    public Template getTemplate();

    public Dataset getDataset();

    public Chip getChip();

    public RankedList getRankedList();

    // -------------------------------------------------------------------------------------------- //
    // Gene set methods
    public GeneSet[] getGeneSetSignals();

    public GeneSet[] getGeneSets();

    public List getGeneSetNames();

    public String[] getGeneSetNamesArray();

    public GeneSet[] getGeneSetSignals(final float npMaxInclusive,
                                       final float fdrMaxInclusive,
                                       final Comparator comp);

    public GeneSet[] getGeneSets(final float npMaxInclusive,
                                 final float fdrMaxInclusive,
                                 final Comparator comp);

    public boolean containsGeneSet(String gsetName);

    // -------------------------------------------------------------------------------------------- //
    // PValues and significance APIs

    public int getNumScores(boolean pos);

    public int getNumNominallySig(float npCutoffInclusive, boolean pos);

    public int getNumFDRSig(float fdrCutoffInclusive, boolean pos);

    public FdrStruc[] getFDRSig(float fdrCutoffInclusive, boolean pos);

    public FdrStruc[] getFDR_all();

    public FdrStruc[] getFDR(final boolean pos);

    // -------------------------------------------------------------------------------------------- //
    // EnrichmentResults based methods
    public EnrichmentResult getResult(final int i);

    public EnrichmentResult[] getResults(float npMaxInclusive, float fdrMaxInclusive, final Comparator comp);

    public EnrichmentResult[] getResults();

    public EnrichmentResult[] getResults(final Comparator comp);

    public EnrichmentResult[] getResults(final Comparator comp, final boolean pos);

    public List getResultsList(final Comparator comp);

    public List getResultsList();

    public int getNumResults();

    //public EnrichmentResult getResult(final String rlName, final String gsetName);

    public EnrichmentResult getResultForGeneSet(final String gsetName);

    // @note a enresult
    // public List getResultNamesList();

    //public String[] getResultNames();

    // -------------------------------------------------------------------------------------------- //
    // CONVENIENCE METHODS

    public Vector getESS();

    public LabelledVector getESS_lv();

    public Vector getNESS();

    public LabelledVector getNESS_lv();

    public RankedList getNESS_rl();

    public Vector getNPs();

    public LabelledVector getNPs_lv();

    public Vector getFDRs();

    public LabelledVector getFDRs_lv();

    public Vector getFWERs();

    public LabelledVector getFWERs_lv();

    // @todo check if needed
    public boolean isCompatible(final EnrichmentDb edb);

    public void checkCompatibility(final EnrichmentDb edb) throws IllegalArgumentException;

    // -------------------------------------------------------------------------------------------- //
    // METRICS RELATED API

    public LabelledVectorProcessor getRankedListProcessor();

    public CMetric getCMetric();

    public SortMode getSortMode();

    public Order getOrder();

    public int getNumPerm();

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * Helper inner class
     */
    public static class Helper {

        public static Set getAllTemplateNames(final EnrichmentResult[] results) {
            Set names = new HashSet();
            for (int i = 0; i < results.length; i++) {
                names.add(results[i].getTemplate().getName());
            }

            return Collections.unmodifiableSet(names);
        }

        public static String[] getAllGeneSetNames(final EnrichmentResult[] results) {
            String[] names = new String[results.length];
            for (int i = 0; i < results.length; i++) {
                names[i] = results[i].getGeneSetName();
            }

            return names;
        }

        public static Set getAllGeneSetNamesS(final EnrichmentResult[] results) {
            Set names = new HashSet();
            for (int i = 0; i < results.length; i++) {
                names.add(results[i].getGeneSet().getName());
            }

            return names;
        }
    }

} // End EnrichmentDb
