/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.utils.FileUtils;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 12, 2005
 *         Time: 4:51:28 PM
 */
public class DatasetTemplate implements DatasetWrapper, TemplateWrapper {

    private Dataset fDataset;
    private Template fTemplate;

    private File fDatasetFile;
    private File fTemplateFile;
    private boolean fAdd2Cache;

    private GeneSet fGeneSpace;
    private File fGeneSpaceFile;

    /**
     * Class constructor
     *
     * @param ds
     * @param t
     */
    public DatasetTemplate(final Dataset ds, final Template t) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        if (t == null) {
            throw new IllegalArgumentException("Param t cannot be null");
        }

        this.fDataset = ds;
        this.fTemplate = t;
    }

    /**
     * Class constructor
     *
     * @param ds
     * @param t
     * @param geneSpaceOpt
     */
    public DatasetTemplate(final Dataset ds, final Template t, final GeneSet geneSpaceOpt) {
        this(ds, t);
        this.fGeneSpace = geneSpaceOpt;
    }

    /**
     * Class constructor
     *
     * @param dsFile
     * @param tFile
     * @param add2cache
     */
    public DatasetTemplate(final File dsFile, final File tFile, boolean add2cache) {

        if (dsFile == null) {
            throw new IllegalArgumentException("Param dsFile cannot be null");
        }

        if (tFile == null) {
            throw new IllegalArgumentException("Param tFile cannot be null");
        }

        this.fDatasetFile = dsFile;
        this.fTemplateFile = tFile;
        this.fAdd2Cache = add2cache;
    }

    /**
     * Class constructor
     *
     * @param dsFile
     * @param tFile
     * @param add2cache
     * @param geneSpaceOptFile
     */
    public DatasetTemplate(final File dsFile, final File tFile, final boolean add2cache, final File geneSpaceOptFile) {
        this(dsFile, tFile, add2cache);
        this.fGeneSpaceFile = geneSpaceOptFile;
    }

    public String getName() {
        String dsname = FileUtils.removeExtension(getDatasetName());
        return NamingConventions.createSafeFileName(dsname + "_" + AuxUtils.getAuxNameOnlyNoHash(getTemplate())); // @note loads template
    }

    public String getDatasetName() {
        if (fDataset != null) {
            return fDataset.getName();
        } else {
            return NamingConventions.removeExtension(fDatasetFile.getName());
        }
    }

    public File getDatasetFile() {
        if (fDatasetFile == null) {
            this.fDatasetFile = ParserFactory.getCache().getSourceFile(fDataset);
        }

        return fDatasetFile;
    }

    public File getTemplateFile() {
        if (fTemplateFile == null) {
            this.fTemplateFile = ParserFactory.getCache().getSourceFile(fTemplate);
        }

        return fTemplateFile;
    }

    public String getTemplateName() {
        if (fTemplate != null) {
            return fTemplate.getName();
        } else {
            return NamingConventions.removeExtension(fTemplateFile.getName());
        }
    }

    public Dataset getDataset(boolean add2cache) {

        if (fDataset == null) {
            try {
                this.fDataset = ParserFactory.readDataset(fDatasetFile, add2cache, add2cache);
            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        }

        if (fGeneSpace != null) {
            return new DatasetGenerators().extractRows(fDataset, fGeneSpace);
        } else if (fGeneSpaceFile != null) {
            try {
                this.fGeneSpace = ParserFactory.readGeneSet(fGeneSpaceFile, false);
                return new DatasetGenerators().extractRows(fDataset, fGeneSpace);
            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        } else {
            return fDataset;
        }
    }

    public Dataset getDataset() {
        return getDataset(fAdd2Cache);
    }

    public Template getTemplate() {
        if (fTemplate == null) {
            try {
                this.fTemplate = ParserFactory.readTemplate(fTemplateFile, fAdd2Cache, fAdd2Cache);
            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        }

        return fTemplate;
    }

    public static Dataset[] toDatasets(final DatasetTemplate[] dts, boolean add2cache) {
        Dataset[] dss = new Dataset[dts.length];
        for (int i = 0; i < dts.length; i++) {
            dss[i] = dts[i].getDataset(add2cache);
        }

        return dss;
    }

    public String getPath() {
        throw new NotImplementedException();
    }

    public File getFile() {
        throw new NotImplementedException();
    }

} // End class DatasetTemplate


