/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.parsers.ParserFactory;
import org.apache.log4j.Logger;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 12, 2005
 *         Time: 4:51:28 PM
 */
public class DatasetWrapperImpl implements DatasetWrapper {

    private Dataset fDataset;

    private File fDatasetFile;

    private String[] fRestrictColNames_opt;

    private GeneSet fRestrict2Genes_opt;

    private boolean fAdd2Cache;

    private boolean fDoneExtraction;

    private Logger log = XLogger.getLogger(DatasetWrapperImpl.class);

    /**
     * Class constructor
     *
     * @param dsObjectOrDsFile
     */
    public DatasetWrapperImpl(final Object dsObjectOrDsFile, boolean add2cache) {

        if (dsObjectOrDsFile == null) {
            throw new IllegalArgumentException("Param dsObjectOrDsFile cannot be null");
        }

        if (dsObjectOrDsFile instanceof Dataset) {
            this.fDataset = (Dataset) dsObjectOrDsFile;
        } else if (dsObjectOrDsFile instanceof File) {
            this.fDatasetFile = (File) dsObjectOrDsFile;
            this.fAdd2Cache = add2cache;
        } else {
            throw new IllegalArgumentException("Unknown object type: " + dsObjectOrDsFile);
        }

        this.fDoneExtraction = false;
    }

    public DatasetWrapperImpl(final File dsFile, boolean add2cache) {
        this(dsFile, null, add2cache);
        if (dsFile == null) {
            throw new IllegalArgumentException("Param dsFile cannot be null");
        }

        this.fDatasetFile = dsFile;
        this.fAdd2Cache = add2cache;
        this.fDoneExtraction = false;
    }

    public DatasetWrapperImpl(final File dsFile,
                              final String[] restrictToTheseColumnNames_opt,
                              final boolean add2cache) {

        if (dsFile == null) {
            throw new IllegalArgumentException("Param dsFile cannot be null");
        }

        this.fDatasetFile = dsFile;
        this.fAdd2Cache = add2cache;
        this.fRestrictColNames_opt = restrictToTheseColumnNames_opt;
        this.fDoneExtraction = false;
    }

    public String getDatasetName() {
        if (fDataset != null) {
            return fDataset.getName();
        } else {
            String name = NamingConventions.removeExtension(fDatasetFile.getName());
            if (fRestrictColNames_opt != null) {
                name = name + "_" + fRestrictColNames_opt.length;
            }

            if (fRestrict2Genes_opt != null) {
                name = name + "_" + fRestrict2Genes_opt.getName(true);
            }
            return name;
        }
    }

    public File getFile() {
        if (fDatasetFile == null) {
            this.fDatasetFile = ParserFactory.getCache().getSourceFile(fDataset);
        }

        return fDatasetFile;
    }

    public String getPath() {
        return getFile().getPath();
    }

    // be careful, very careful!!
    public Dataset getDataset(final boolean add2cache) {

        // if (fDataset == null) { @note IMP IMP cant do this check as dataset might be set in a constructor
        boolean wasExtr = false;
        boolean wasParsed = false;
        if (!fDoneExtraction) {
            wasExtr = true;
            try {
                if (fDataset == null) { // @note dont re-read that might destroy the current state of dataset
                    wasParsed = true;
                    this.fDataset = ParserFactory.readDataset(fDatasetFile, add2cache, add2cache);
                }

                if (fRestrictColNames_opt != null) {
                    String name = fDataset.getName() + "_" + fRestrictColNames_opt.length;
                    this.fDataset = new DatasetGenerators().extractColumns(name, fDataset, fRestrictColNames_opt);
                    log.debug("Restricting columns: " + fRestrictColNames_opt.length + " " + fDataset.getName());
                }

                if (fRestrict2Genes_opt != null) {
                    String name = fDataset.getName() + "_" + fRestrict2Genes_opt.getName(true);
                    this.fDataset = new DatasetGenerators().extractRows(name, fDataset, fRestrict2Genes_opt);
                    log.debug("Restricting genes: " + fRestrict2Genes_opt.getNumMembers() + " " + fDataset.getName());
                }

                this.fDoneExtraction = true;

            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        }

        log.debug("#### DATASET: " + fDataset.getQuickInfo() + " was extr: " + wasExtr + " wasParsed: " + wasParsed);
        //TraceUtils.showTrace();
        return fDataset;
    }

    public Dataset getDataset() {
        return getDataset(fAdd2Cache);
    }

    public static Dataset[] toDatasets(final DatasetWrapper[] dts, final boolean add2cache) {
        Dataset[] dss = new Dataset[dts.length];
        for (int i = 0; i < dts.length; i++) {
            dss[i] = dts[i].getDataset(add2cache);
        }

        return dss;
    }


} // End class DatasetWrapperImpl


