/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.LabelledVector;
import edu.mit.broad.genome.objects.RankedList;

/**
 * @author Aravind Subramanian
 *         Date: Oct 26, 2004
 *         Time: 12:07:45 AM
 */
public class DefaultFdrStruc implements FdrStruc {

    private String name;

    private float nominalP;

    //public Distrib.Point[] moreThan;
    private float realScore;
    private float realScoreNorm;
    private float fdr;

    private int moreRealThan;
    private int moreRndThan;

    private float numr;
    private float denr;

    private int totRealCnt;
    private int totRndCnt;


    /**
     * @param createForName
     * @param real_scores
     * @param real_scores_norm_sorted
     * @param rnd_scores_by_row
     * @param all_rnd_scores_norm_sorted
     * @param order
     * @return
     */
    public static DefaultFdrStruc create(final String createForName,
                                         final LabelledVector real_scores,
                                         final RankedList real_scores_norm_sorted,
                                         final Dataset rnd_scores_by_row,
                                         final Vector all_rnd_scores_norm_sorted,
                                         final Order order) {


        final float real_score = real_scores.getScore(createForName);
        final Vector rnd_scores_for_name_only = rnd_scores_by_row.getRow(createForName);

        final float real_score_norm = real_scores_norm_sorted.getScore(createForName);
        final int rank = real_scores_norm_sorted.getRank(createForName) + 1;

        return new DefaultFdrStruc(createForName,
                real_score,
                real_score_norm,
                rnd_scores_for_name_only,
                all_rnd_scores_norm_sorted,
                rank,
                real_scores_norm_sorted.getSize(),
                order);

    }

    /**
     * USE WHEN SCORE = NORM_SCORE
     *
     * @param createForName
     * @param real_scores_sorted
     * @param all_rnd_scores_sorted
     * @param order
     * @return
     */
    public static DefaultFdrStruc create(final String createForName,
                                         final RankedList real_scores_sorted,
                                         final Vector rnd_scores_for_name_only,
                                         final Vector all_rnd_scores_sorted,
                                         final Order order) {


        final float real_score = real_scores_sorted.getScore(createForName);
        final int rank = real_scores_sorted.getRank(createForName) + 1;

        return new DefaultFdrStruc(createForName,
                real_score,
                real_score,
                rnd_scores_for_name_only,
                all_rnd_scores_sorted,
                rank,
                real_scores_sorted.getSize(),
                order);
    }

    /**
     * @param name
     * @param real_score_of_name
     * @param real_score_norm_of_name
     * @param rnd_scores
     * @param all_rnd_scores_norm_sorted
     * @param moreRealThan
     * @param totRealCnt
     * @param order
     */
    public DefaultFdrStruc(final String name,
                           final float real_score_of_name,
                           final float real_score_norm_of_name,
                           final Vector rnd_scores_of_name_only,
                           final Vector all_rnd_scores_norm_sorted,
                           final int moreRealThan,
                           final int totRealCnt,
                           final Order order) {

        if (rnd_scores_of_name_only == null) {
            throw new IllegalArgumentException("Param rnd_scores_of_name_only cannot be null");
        }

        if (all_rnd_scores_norm_sorted == null) {
            throw new IllegalArgumentException("Param all_rnd_scores_norm_sorted cannot be null");
        }

        if (rnd_scores_of_name_only.getSize() * totRealCnt != all_rnd_scores_norm_sorted.getSize()) {
            throw new MismatchedSizeException("calc_all", rnd_scores_of_name_only.getSize() * totRealCnt, "all_rnd", all_rnd_scores_norm_sorted.getSize());
        }

        float nominalP;
        int moreRndThan;
        if (order == Order.DESCENDING) {
            nominalP = (float) XMath.getPValue(real_score_of_name, rnd_scores_of_name_only);
            moreRndThan = XMath.getMoreThanCount(real_score_norm_of_name, all_rnd_scores_norm_sorted);
        } else if (order == Order.ASCENDING) {
            nominalP = (float) XMath.getPValueLessThan(real_score_of_name, rnd_scores_of_name_only);
            moreRndThan = XMath.getLessThanCount(real_score_norm_of_name, all_rnd_scores_norm_sorted);
        } else {
            throw new IllegalArgumentException("Unsupported Order: " + order);
        }

        init(name, real_score_of_name, real_score_norm_of_name, nominalP, moreRealThan, moreRndThan, totRealCnt, all_rnd_scores_norm_sorted.getSize());
    }

    /**
     * Class constructor
     *
     * @param name
     * @param real_score
     * @param moreRealThan
     * @param moreRndThan
     * @param totRealCnt
     * @param totRndCnt
     */
    public DefaultFdrStruc(final String name,
                           final float real_score,
                           final float real_score_norm,
                           final float nominalP,
                           final int moreRealThan,
                           final int moreRndThan,
                           final int totRealCnt,
                           final int totRndCnt) {

        init(name, real_score, real_score_norm, nominalP, moreRealThan, moreRndThan, totRealCnt, totRndCnt);
    }

    private void init(final String name,
                      final float real_score,
                      final float real_score_norm,
                      final float nominalP,
                      final int moreRealThan,
                      final int moreRndThan,
                      final int totRealCnt,
                      final int totRndCnt) {

        this.name = name;
        this.realScore = real_score;
        this.realScoreNorm = real_score_norm;
        this.nominalP = nominalP;

        this.moreRealThan = moreRealThan;
        this.moreRndThan = moreRndThan;

        this.totRealCnt = totRealCnt;
        this.totRndCnt = totRndCnt;

        this.denr = (float) this.moreRealThan / (float) this.totRealCnt;
        this.numr = (float) this.moreRndThan / (float) this.totRndCnt;
        this.fdr = numr / denr;

        if (this.fdr > 1.0f) {
            this.fdr = 1.0f;
        }
    }

    public String getName() {
        return name;
    }

    public float getRealScore() {
        return realScore;
    }

    public float getRealNormalizedScore() {
        return realScoreNorm;
    }

    public int getMoreRndThanCnt() {
        return moreRndThan;
    }

    public int getMoreRealThanCnt() {
        return moreRealThan;
    }

    public float getNumr() {
        return numr;
    }

    public float getDenr() {
        return denr;
    }

    public float getFdr() {
        return fdr;
    }

    public int getTotalRealCnt() {
        return totRealCnt;
    }

    public int getTotalRndCnt() {
        return totRndCnt;
    }

    public float getNominalPValue() {
        return nominalP;
    }

    // huh??
    public float getTotalRndSkew() {
        throw new NotImplementedException();
    }

} // End class DefaultFdr
