/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.gsea.MhtModes;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import org.apache.log4j.Logger;

/**
 * @author Aravind Subramanian
 *         Date: Oct 4, 2003
 *         Time: 10:27:51 AM
 */

// DONT STORE rnd scores to keep this lean and mean ditto for GCOH
// make the ranked list and hit indices etc optional

public class ProbeNeighborhoodEsStruc {

    private String fProbeName;

    private float[] fPreCalcRealNESS;
    private float[] fPreCalcNominalPValues;
    private float[] fPreCalcDiscPValues;
    private float[] fPreCalcValPValues;

    /**
     * only filled on NON-deep mode
     */
    private float[] fPreCalcRealESS;

    private int fSize;

    private static Logger klog = XLogger.getLogger(ProbeNeighborhoodEsStruc.class);

    // ONLY filled on deeep mode
    public ProbeGeneSetScoreStruc[] fGeneSetScoreStrucs;

    /**
     * Class constructor
     *
     * @param probeName
     * @param t
     * @param realEss
     * @param rndEss
     * @param gcoh
     */
    public ProbeNeighborhoodEsStruc(String probeName, float[] realEss,
                                    Vector[] rndEss, GeneSet[] gsets) {

        initCalc(probeName, realEss, rndEss, gsets);
    }

    /**
     * @param probeName
     * @param scores
     * @param gsets
     * @param storeDeep
     */
    public ProbeNeighborhoodEsStruc(String probeName, ProbeGeneSetScoreStruc[] scores, GeneSet[] gsets, boolean storeDeep) {
        Object obj;

        Vector[] rndEss = null;

        if (storeDeep) {
            obj = scores;
        } else {
            float[] realEss = new float[scores.length];
            rndEss = new Vector[scores.length];
            for (int f = 0; f < scores.length; f++) {
                realEss[f] = scores[f].getES();
                //rndEss[f] = scores[f].getRndES(); // @todo dec 2005
            }
            obj = realEss;
        }

        initCalc(probeName, obj, rndEss, gsets);
    }

    /**
     * @param probeName
     * @param realEss
     * @param realNess
     * @param nomPValues
     * @param discPValues
     */
    public ProbeNeighborhoodEsStruc(String probeName, float[] realEss,
                                    float[] realNess, float[] nomPValues,
                                    float[] discPValues, float[] valPValues) {


        init(probeName, realEss, realNess, nomPValues, discPValues, valPValues, null);
    }

    // does the calc and then the init
    private void initCalc(String probeName, Object gsetscorestrucsOrfloats, Vector[] rndEss, GeneSet[] gsets) {

        int num = gsets.length;

        EnrichmentResult[] dtgs = new EnrichmentResult[num];
        for (int i = 0; i < dtgs.length; i++) {
            float realEs;
            if (gsetscorestrucsOrfloats instanceof ProbeGeneSetScoreStruc[]) {
                //klog.debug("SETTING IN OBJECT MODE");
                realEs = ((ProbeGeneSetScoreStruc[]) gsetscorestrucsOrfloats)[i].getES();
                dtgs[i] = ((ProbeGeneSetScoreStruc[]) gsetscorestrucsOrfloats)[i];
            } else {
                //klog.debug("SETTING IN float MODE");
                realEs = ((float[]) gsetscorestrucsOrfloats)[i];
                // @todo disable after dec 2005
                throw new NotImplementedException();
                //dtgs[i] = new SlimEnrichmentResult(realEs, rndEss[i], gsets[i], null);
            }
        }

        // @note IMP IMP
        MhtModes.VarMean mht = new MhtModes.VarMean(dtgs); // @note hardcoded, also note setting the Dtgs data

        float[] realNess = new float[num];
        float[] nominalPValues = new float[num];
        float[] discPValues = new float[num];
        float[] valPValues = new float[num];

        for (int g = 0; g < num; g++) {
            realNess[g] = mht.getRealNES(g);
            // @todo dec 2005
            //nominalPValues[g] = (float) XMath.getPValueTwoTailed(dtgs[g].getES(), dtgs[g].getRndES());
            discPValues[g] = mht.getDiscoveryPValue(g);
            valPValues[g] = mht.getValidationPValue(g);
            //klog.debug("np: " + nominalPValues[g] + " real: " + realEss[g] + " " + rndEss[g].toString(','));
        }

        if (gsetscorestrucsOrfloats instanceof ProbeGeneSetScoreStruc[]) {
            init(probeName, null, realNess, nominalPValues, discPValues, valPValues, (ProbeGeneSetScoreStruc[]) gsetscorestrucsOrfloats);
        } else {
            init(probeName, (float[]) gsetscorestrucsOrfloats, realNess, nominalPValues, discPValues, valPValues, null);
        }

    }

    // either realEss OR scores must be specified
    private void init(String probeName, float[] realEss,
                      float[] realNess, float[] nomPValues,
                      float[] discPValues, float[] valPValues,
                      ProbeGeneSetScoreStruc[] scores) {

        if (probeName == null) {
            throw new IllegalArgumentException("Parameter probeName cannot be null");
        }
        int expLen = realNess.length;

        if (realEss != null && realEss.length != expLen) {
            throw new MismatchedSizeException();
        }

        if (expLen != realNess.length) {
            throw new MismatchedSizeException();
        }

        if (expLen != nomPValues.length) {
            throw new MismatchedSizeException();
        }

        if (expLen != discPValues.length) {
            throw new MismatchedSizeException();
        }

        if (expLen != valPValues.length) {
            throw new MismatchedSizeException();
        }

        if (scores != null && scores.length != expLen) {
            throw new MismatchedSizeException();
        }

        this.fSize = realNess.length;
        this.fProbeName = probeName;
        this.fPreCalcRealESS = realEss;
        this.fPreCalcRealNESS = realNess;
        this.fPreCalcNominalPValues = nomPValues;
        this.fPreCalcDiscPValues = discPValues;
        this.fPreCalcValPValues = valPValues;
        this.fGeneSetScoreStrucs = scores; // maybe null
    }

    public String getProbeName() {
        return fProbeName;
    }

    public float[] getRealEScores() {
        if (fPreCalcRealESS != null) {
            return fPreCalcRealESS;
        } else if (fGeneSetScoreStrucs != null) {
            this.fPreCalcRealESS = new float[fGeneSetScoreStrucs.length];
            for (int f = 0; f < fGeneSetScoreStrucs.length; f++) {
                this.fPreCalcRealESS[f] = fGeneSetScoreStrucs[f].getES();
            }
            return fPreCalcRealESS;
        } else {
            throw new IllegalStateException("Prog error - both realess and gsscorestruc are null");
        }
    }


    public float[] getRealNEScores() {
        return fPreCalcRealNESS;
    }

    public float[] getNominalPValues() {
        return fPreCalcNominalPValues;
    }

    public float[] getDiscoveryPValues() {
        return fPreCalcDiscPValues;
    }

    public float[] getValidationPValues() {
        return fPreCalcValPValues;
    }

    public PreCalc getPreCalc(int g) {

        float ess = Float.NaN;
        if (fPreCalcRealESS != null) {
            ess = fPreCalcRealESS[g];
        }

        return new PreCalc(ess, fPreCalcRealNESS[g],
                fPreCalcNominalPValues[g],
                fPreCalcDiscPValues[g],
                fPreCalcValPValues[g]);
    }

    public int getSize() {
        return fSize;
    }

    public ProbeGeneSetScoreStruc getDeepInfo(int i) {
        if (fGeneSetScoreStrucs == null) {
            throw new IllegalStateException("This object is not in deep info mode");
        }
        return fGeneSetScoreStrucs[i];
    }

    /**
     * Internal class
     */
    public static class PreCalc {

        private float fRealES;
        private float fRealNES;
        private float fNominalPValue;
        private float fDiscPValue;
        private float fValPValue;

        PreCalc(float realES, float realNES, float np, float dp, float vp) {
            this.fRealES = realES;
            this.fRealNES = realNES;
            this.fNominalPValue = np;
            this.fDiscPValue = dp;
            this.fValPValue = vp;
        }

        public float getRealES() {
            return fRealES;
        }

        public float getRealNES() {
            return fRealNES;
        }

        public float getNominalPValue() {
            return fNominalPValue;
        }

        public float getDiscoveryPValue() {
            return fDiscPValue;
        }

        public float getValidationPValue() {
            return fValPValue;
        }

    }

} // End ProbeNeighborhoodEsStruc
