/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.parsers;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.utils.Strings;
import org.apache.log4j.Logger;

import java.io.File;
import java.util.StringTokenizer;

/**
 * AuxFilePath
 * AuxFile
 * <p/>
 * AuxOnly
 * AuxOnlyNoHash
 * <p/>
 * FullNameWithAux
 */
public class AuxUtils {

    private static final Logger klog = XLogger.getLogger(AuxUtils.class);

    /**
     * Privatized class constructor
     * Static methods only
     */
    private AuxUtils() {
    }

    // test.cls#foo, text.cls#bar to foo, bar
    public static String[] toAuxNamesNoHash(String[] auxpaths) {
        String[] auxnames = new String[auxpaths.length];
        for (int i = 0; i < auxpaths.length; i++) {
            auxnames[i] = getAuxNameOnlyNoHash(auxpaths[i]);
        }

        return auxnames;

    }

    public static String getAuxNameOnlyIncludingHash(final String path) {
        StringTokenizer tok = new StringTokenizer(path, "#");
        tok.nextToken();
        return "#" + tok.nextToken();
    }

    public static String getAuxNameOnlyNoHash(final PersistentObject pob) {
        if (pob == null) {
            throw new IllegalArgumentException("Param pob cannot be null");
        }
        return getAuxNameOnlyNoHash(pob.getName());
    }

    public static String getAuxNameOnlyNoHash(final String fullPathOrNameMaybeWithAux) {
        StringTokenizer tok = new StringTokenizer(fullPathOrNameMaybeWithAux, "#");
        String nonaux = fullPathOrNameMaybeWithAux;

        while (tok.hasMoreTokens()) { // so that we always get the very last token
            nonaux = tok.nextToken();
        }

        return nonaux;
    }

    public static String getBaseNameOnly(String fullPathOrNameMaybeWithAux) {
        StringTokenizer tok = new StringTokenizer(fullPathOrNameMaybeWithAux, "#");
        return tok.nextToken();
    }

    public static String getAuxNameOnlyNoHash(File file) {
        return getAuxNameOnlyNoHash(file.getPath());
    }

    /**
     * either hash or @??
     *
     * @param auxfile
     * @return
     */
    public static File getBaseFileFromFullPath(String fullPathMaybeWithAux) {
        if (fullPathMaybeWithAux == null) {
            throw new IllegalArgumentException("Parameter fullPathMaybeWithAux cannot be null");
        }

        if (fullPathMaybeWithAux.length() == 0) {
            throw new IllegalArgumentException("Parameter fullPathMaybeWithAux cannot be empty");
        }

        StringTokenizer tok = new StringTokenizer(fullPathMaybeWithAux, "#@");

        File f;
        if (tok.countTokens() == 1) {
            f = new File(fullPathMaybeWithAux);
        } else {
            f = new File(tok.nextToken()); // just first token
        }

        //klog.debug("From aux string: " + fullPathMaybeWithAux + " GOT BASE FILE: " + f);
        return f;
    }

    public static File getBaseFileFromAuxFile(File auxfile) {
        return getBaseFileFromFullPath(auxfile.getPath());
    }

    public static String getBasePathFromAuxPath(String fullPathMaybeWithAux) {
        return getBaseFileFromFullPath(fullPathMaybeWithAux).getPath();
    }

    public static String getBaseStringFromAux(String auxname_or_auxpath) {
        StringTokenizer tok = new StringTokenizer(auxname_or_auxpath, "#");
        if (tok.countTokens() == 1) {
            return auxname_or_auxpath;
        } else {
            return tok.nextToken();
        }
    }

    public static boolean isAux(String aux_name_or_path) {
        StringTokenizer tok = new StringTokenizer(aux_name_or_path, "#");

        if (tok.countTokens() == 1) {
            return false;
        } else if (tok.countTokens() >= 2) {
            return true;
        } else {
            throw new RuntimeException("Cannot process files named # tokens found: " + tok.countTokens() + " aux_name_or_path>" + aux_name_or_path + "<");
        }

    }

    public static boolean isAuxFile(File file) {

        if (Strings.contains(file.getAbsolutePath(), "#")) {
            return true;
        }

        if (Strings.contains(file.getAbsolutePath(), "@")) {
            return true;
        }

        return false;
    }

    public static boolean isFromSameBase(String auxPathOne, String auxPathTwo) {

        String prefixOne = AuxUtils.getBaseStringFromAux(auxPathOne);
        String prefixTwo = AuxUtils.getBaseStringFromAux(auxPathTwo);

        return prefixOne.equals(prefixTwo);

    }

    public static boolean isAllFromSameBase(String auxPathOne, String[] auxPathRest) {
        for (int i = 0; i < auxPathRest.length; i++) {
            boolean is = isFromSameBase(auxPathOne, auxPathRest[i]);
            if (!is) {
                klog.debug("Not from same aux: " + auxPathOne + " and " + auxPathRest[i]);
                return false;
            }
        }

        return true;
    }

    public static boolean isAllFromSameBase(String[] auxPaths) {
        return isAllFromSameBase(auxPaths[0], auxPaths);
    }

    public static File getBaseDatasetSourceFile(Dataset maybeAux) {
        String fullName = maybeAux.getName();
        StringTokenizer tok = new StringTokenizer(fullName, "@");
        File f = ParserFactory.getCache().getSourceFile(maybeAux);
        if (f == null) {
            throw new RuntimeException("No source file for pwd's source dataset: " + maybeAux.getName());
        }
        if (tok.countTokens() == 1) {
            return f;
        } else {
            String baseName = tok.nextToken();
            File realFile = new File(f.getParentFile(), baseName);
            if (realFile.exists() == false) {
                throw new RuntimeException("No such file: " + realFile + " for full file: " + f + " dataset: " + maybeAux.getName());
            }

            return realFile;
        }

    }

    public static Dataset getBaseDataset(Dataset datasetMaybeAux) throws Exception {
        File f = getBaseDatasetSourceFile(datasetMaybeAux);
        return ParserFactory.readDataset(f, true);
    }

    public static Dataset getBaseDatasetSafe(Dataset datasetMaybeAux) {
        File f = getBaseDatasetSourceFile(datasetMaybeAux);
        try {
            return ParserFactory.readDataset(f, true);
        } catch (Throwable t) {
            System.out.println("Dataset na: " + datasetMaybeAux);
            return null;
        }
    }

} // End class AuxUtils
