/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.parsers;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.vdb.sampledb.SampleAnnot;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Simple dataset format with no res, gct cdt etc frills. Description field is optional.
 * TXT Dataset format -> no obvious extension
 * (txt is used for a lot of different things)
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class DChipDatasetParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public DChipDatasetParser() {
        super(Dataset.class);
    }

    /**
     * Only accepts Dataset objects.
     * see below for format produced
     */
    public void export(PersistentObject pob, File file) throws Exception {

        PrintWriter pw = startExport(pob, file);
        Dataset ds = (Dataset) pob;
        FeatureAnnot ann = ds.getAnnot().getFeatureAnnot();

        pw.print(Constants.NAME + "\t");
        pw.print(Constants.DESCRIPTION + "\t");

        for (int i = 0; i < ds.getNumCol(); i++) {
            pw.print(ds.getColumnName(i));
            pw.print('\t');
        }

        pw.println();

        for (int r = 0; r < ds.getNumRow(); r++) {
            String featureName = ds.getRowName(r);
            pw.print(featureName);
            pw.print('\t');
            pw.print(ann.getNativeDesc(featureName));
            pw.print('\t');
            pw.println(ds.getRow(r).toString('\t'));
        }

        pw.close();
        doneExport();
    }

    /**
     * Expects a TXT data format compatible store (no specific extension known)
     * Produces a Dataset object.
     * <p/>
     * BASIC TXT FORMAT (DESCRIPTION field is optional -> auto detected)
     * <p/>
     * NAME       DESCRIPTION	spo0	spo30	spo2	spo5	spo7	spo9	spo11
     * YAL003W	    some	    0.23	-1.79	-1.29	-1.56	-2.12	-1.09	-1.12
     * YAL004W	    some        0.41	-0.38	-0.89	-1.06	-1.6	-1.84	-1.6
     * YAL005C	    some	    0.61	-0.07	-1.29	-1.29	-2	    -1.84	-2.25
     */
    public List parse(final String hackINeedFullPath, final InputStream is) throws Exception {

        startImport(hackINeedFullPath);
        //int nlines = DataStoreUtils.countLines(is, true); // @note this form doesnt work  / not impl yet
        int nlines = FileUtils.countLines(hackINeedFullPath, true);
        int nfloatlines = nlines - 1;
        log.debug("Number of float lines = " + nfloatlines);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        String currLine = nextLine(bin);

        List colNames = ParseUtils.string2stringsList(currLine, " \t");
        int expectedNCols = colNames.size();
        colNames.remove(0); // get rid of name column
        colNames.remove(0); // get rid of desc column

        log.debug("Number of columns = " + colNames.size());

        // Initialize the Dataset
        List rowNames = new ArrayList(nfloatlines);
        List rowDescs = new ArrayList(nfloatlines);
        Matrix matrix = new Matrix(nfloatlines, colNames.size());

        int r = 0;
        currLine = nextLine(bin); // first line of float data
        while (currLine != null) {
            //System.out.println(">> " + currLine);

            List fields = ParseUtils.string2stringsList(currLine, "\t");
            if (fields.size() != expectedNCols) {
                throw new ParserException("Invalid format on line: " + currLine + " expected # fields = " + expectedNCols + " but found: " + fields.size());
            }

            String featureName = (String) fields.get(0);
            rowNames.add(featureName.trim());

            String featureDesc = (String) fields.get(1);
            rowDescs.add(featureDesc.trim());

            // -- then, onto the floats
            List floats = fields.subList(2, fields.size());
            if (floats.size() != colNames.size()) {
                throw new ParserException("# of floats: " + floats.size() + " on line: " + r + " not equal to # of columns in header: " + colNames.size());
            }

            for (int c = 0; c < floats.size(); c++) {
                matrix.setElement(r, c, Float.parseFloat((String) floats.get(c)));
            }

            r++;
            currLine = nextLine(bin);
        }

        bin.close();

        // Initialize the Dataset and Annotation
        String name = new File(hackINeedFullPath).getName();
        FeatureAnnot fann = new FeatureAnnotImpl(name, rowNames, rowDescs);
        SampleAnnot sann = new SampleAnnotImpl(name, colNames, null);
        fann.addComment(fComment.toString());

        Dataset ds = new DefaultDataset(name, matrix, rowNames, colNames, true, new AnnotImpl(fann, sann));
        ds.addComment(fComment.toString());

        doneImport();
        return unmodlist(new PersistentObject[]{ds});
    }

}    // End DChipDatasetParser
