/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.esmatrix.db.DtgDataset;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.reports.api.Report;
import edu.mit.broad.genome.utils.SystemUtils;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.msigdb.MSigDB;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.filechooser.FileView;
import java.io.File;
import java.io.FilenameFilter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Central repository for data formats.
 * A DataFormat is primarily identified by a file extension.
 * Also associates thungs like a desc, icon etc with DataFprmats.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DataFormat extends DataType implements Constants {


    /**
     * Class constructor
     * Does nothing. Dont use -- static methods only
     */
    public DataFormat() {
    }

    /**
     * For logging support
     */
    private static final Logger klog = XLogger.getLogger(DataFormat.class);

    public static final DataFormat XLS_FORMAT = new DataFormat(ExtFormat.class, "Excel",
            "Microsoft Excel", Constants.XLS,
            JarResources.getIcon("Xls.gif"));
    public static final DataFormat TXT_FORMAT = new DataFormat(ExtFormat.class, "Text",
            "Plain Text", "txt",
            JarResources.getIcon("Txt.gif"));
    public static final DataFormat XML_FORMAT = new DataFormat(ExtFormat.class, "XML",
            "Extensible Markup Language", Constants.XML,
            JarResources.getIcon("Xml.gif"));

    /**
     * BROAD CANCER res format for datasets
     */
    public static final DataFormat RES_FORMAT = new DataFormat(Dataset.class, "Dataset",
            "MIT Format for a Dataset with P-Calls", RES,
            JarResources.getIcon("Res16.gif"), ResParser.class);
    public static final DataFormat GCT_FORMAT = new DataFormat(Dataset.class, "Dataset",
            "MIT Format for a Dataset", GCT,
            JarResources.getIcon("Gct16.gif"), GctParser.class);

    public static final DataFormat TXT_DATASET_FORMAT = new DataFormat(Dataset.class, "Dataset",
            "generic Txt Format for Datasets", TXT,
            JarResources.getIcon("Txt.gif"), TxtDatasetParser.class);

    public static final DataFormat DCHIP_FORMAT = new DataFormat(Dataset.class, "Dataset",
            "DChip dataset format", DCHIP,
            JarResources.getIcon("Txt.gif"), DChipDatasetParser.class);

    public static final DataFormat PCL_FORMAT = new DataFormat(Dataset.class, "Dataset",
            "Stanford Format for a Dataset", "pcl",
            JarResources.getIcon("Pcl.gif"), PclParser.class);

    public static final DataFormat CDT_FORMAT = new DataFormat(Dataset.class, "HClustDataset",
            "Clustered Dataset Format", CDT,
            JarResources.getIcon("Cdt.png"), HClustDatasetParser.class);

    public static final DataFormat ATR_FORMAT = new DataFormat(Dataset.class, "ArrayTree",
            "Array Tree Format", ATR,
            JarResources.getIcon("Atr.png"), HClustTreeParser.class);
    // shared icons and parsers by atr and gtr
    public static final DataFormat GTR_FORMAT = new DataFormat(Dataset.class, "GeneTree",
            "GeneTree Format", GTR,
            JarResources.getIcon("Gtr.png"), HClustTreeParser.class);

    public static final DataFormat MAT_FORMAT = new DataFormat(Matrix.class, "Matrix",
            "Matrix Format", MAT,
            JarResources.getIcon("Mat.gif"), MatrixParser.class);

    /**
     * MIT cls format for class vectors
     */
    public static final DataFormat CLS_FORMAT = new DataFormat(Template.class, "Template",
            "MIT Format for Class Labels", CLS,
            JarResources.getIcon("Cls.gif"), ClsParser.class);


    public static final DataFormat SIN_FORMAT = new DataFormat(SampleAnnot.class,
            "Annotation", "Formatted sampl annnotation",
            Constants.SIN, JarResources.getIcon("Table16b.gif"), Object.class);

    public static final DataFormat GRP_FORMAT = new DataFormat(GeneSet.class, "GeneSet",
            "MIT GeneSet Format", GRP,
            JarResources.getIcon("Grp.gif"), FSetParser.class);

    public static final DataFormat RNK_FORMAT = new DataFormat(RankedList.class, "RankedList",
            "MIT RankedList Format", RNK,
            JarResources.getIcon("Rnk.png"), RankedListParser.class);

    public static final DataFormat GMX_FORMAT = new DataFormat(GeneSetMatrix.class, "GeneSetMatrix",
            "MIT format for a Matrix of Gene Sets",
            Constants.GMX, JarResources.getIcon("Gmx.png"), GmxParser.class);

    public static final DataFormat GMT_FORMAT = new DataFormat(GeneSetMatrix.class,
            "GeneSetMatrix_Transposed",
            "MIT format for a Matrix of Gene Sets",
            Constants.GMT, JarResources.getIcon("Gmt.png"), GmtParser.class);

    public static final DataFormat EDB_FORMAT = new DataFormat(EnrichmentDb.class,
            "Enrichment-Database",
            "MIT format for an enrichment database",
            Constants.EDB, JarResources.getIcon("Edb.png"), EdbFolderParser.class);

    public static final DataFormat DTGDS_FORMAT = new DataFormat(DtgDataset.class,
            "Dtg Dataset",
            "MIT format for an enrichment database",
            Constants.DTGDS, JarResources.getIcon("DtgDs.gif"), DtgDatasetParser.class);


    public static final DataFormat MSIGDB_FORMAT = new DataFormat(MSigDB.class,
            "MSigDB",
            "MSigDB format for a database of gene sets and their annotations",
            Constants.XML, JarResources.getIcon("Xml.gif"), MSigDBParser.class);

    public static final DataFormat DFR_FORMAT = new DataFormat(Dataframe.class, "Dataframe",
            "Format of a Dataframe - R compatible",
            Constants.DFR, JarResources.getIcon("Dfr16.gif"), DataframeParser.class);

    public static final DataFormat SDF_FORMAT = new DataFormat(StringDataframe.class, "StringDataframe",
            "Dataframe of Strings",
            Constants.SDF, JarResources.getIcon("Sdf16.gif"), StringDataframeParser.class);

    public static final DataFormat RPT_FORMAT = new DataFormat(Report.class, "Report",
            "Report for a program",
            Constants.RPT, JarResources.getIcon("Rpt.gif"), ReportParser.class);

    public static final DataFormat CHIP_FORMAT = new DataFormat(Chip.class, "Chip",
            "Chip",
            Constants.CHIP, JarResources.getIcon("Chip16.png"), ChipParser.class);

    public static final DataFormat GIN_FORMAT = new DataFormat(GenesOfInterest.class, "GenesOfInterest",
            "MIT Format for a Database of Genes of Interest", Constants.GIN,
            JarResources.getIcon("Gin16.png"), GinParser_xls.class);

    /**
     * @maint manually keep in synch with declared formats
     */

    // imp to not expose
    private static final DataFormat[] ALL = new DataFormat[]{
            RES_FORMAT, GCT_FORMAT, TXT_DATASET_FORMAT, DCHIP_FORMAT, PCL_FORMAT,
            CLS_FORMAT,
            CDT_FORMAT, ATR_FORMAT, GTR_FORMAT,

            SIN_FORMAT,

            GRP_FORMAT, GMT_FORMAT, GMX_FORMAT,

            MAT_FORMAT,

            RNK_FORMAT,

            MSIGDB_FORMAT,

            RPT_FORMAT,
            DFR_FORMAT, SDF_FORMAT,

            EDB_FORMAT, DTGDS_FORMAT,

            CHIP_FORMAT, GIN_FORMAT,

            XLS_FORMAT, TXT_FORMAT, XML_FORMAT
    };

    /**
     * @maint
     */
    public static final DataFormat[] ALL_DATASET_FORMATS = new DataFormat[]
            {GCT_FORMAT, RES_FORMAT, TXT_DATASET_FORMAT, DCHIP_FORMAT, PCL_FORMAT};

    public static final DataFormat[] ALL_TEMPLATE_FORMATS = new DataFormat[]
            {CLS_FORMAT};

    public static final DataFormat[] ALL_GENESET_FORMATS = new DataFormat[]
            {GRP_FORMAT, GMX_FORMAT, GMT_FORMAT};

    public static final DataFormat[] ALL_GENESETMATRIX_FORMATS = new DataFormat[]
            {GMX_FORMAT, GMT_FORMAT};


    public static DataFormat[] getPossibleDataFormats(String ext) {
        Class pobClass = DataFormat.getRepresentationClass(ext);
        if (pobClass == null) {
            return new DataFormat[]{};
        }

        return getPossibleDataFormats(pobClass);
    }

    public static DataFormat[] getPossibleDataFormats(Class pobClass) {
        _initParsableExts();
        Object dfs = kPobClassDataFormatMap.get(pobClass);
        if (dfs == null) {
            klog.debug("No data formats found for: " + pobClass);
            return new DataFormat[]{};
        } else {
            return (DataFormat[]) dfs;
        }
    }

    public static DataFormat[] getPossibleDataFormats(File file) {
        return getPossibleDataFormats(NamingConventions.getExtension(file));
    }

    public static DataFormat[] getPossibleDataFormats(PersistentObject pob) {
        Class cl = DataFormat.getRepresentationClass(pob);
        return getPossibleDataFormats(cl);
    }

    /**
     * @param file
     * @return
     */
    public static boolean isParsable(final File file) {

        if (file == null) {
            return false;
        }

        String ext = NamingConventions.getExtension(file);
        _initParsableExts();
        return kParsableExts.contains(ext);
    }

    /**
     * @return
     */
    public static String[] getParsableExtensions() {
        _initParsableExts();
        return (String[]) kParsableExts.toArray(new String[kParsableExts.size()]);
    }


    private static Set kParsableExts;

    /**
     * key -> a pob class, value -> DataFormat[]
     */
    private static Map kPobClassDataFormatMap;

    /**
     * Init it lazilly
     *
     * @maint Parsable extensions and association maintained here
     */
    private static void _initParsableExts() {

        if (kParsableExts != null) {
            return;
        }

        klog.debug("Initing parsable extensions");

        kParsableExts = new HashSet();

        kParsableExts.add(RES);
        kParsableExts.add(RES.toUpperCase());

        kParsableExts.add(GCT);
        kParsableExts.add(GCT.toUpperCase());

        kParsableExts.add(DCHIP);
        kParsableExts.add(DCHIP.toUpperCase());

        kParsableExts.add(PCL);
        kParsableExts.add(PCL.toUpperCase());

        kParsableExts.add(SDF);
        kParsableExts.add(SDF.toUpperCase());

        kParsableExts.add(DTGDS);
        kParsableExts.add(DTGDS.toUpperCase());

        kParsableExts.add(CDT);
        kParsableExts.add(CDT.toUpperCase());
        kParsableExts.add(ATR);
        kParsableExts.add(ATR.toUpperCase());
        kParsableExts.add(GTR);
        kParsableExts.add(GTR.toUpperCase());

        kParsableExts.add(CLS);
        kParsableExts.add(CLS.toUpperCase());

        kParsableExts.add(COMP);
        kParsableExts.add(COMP.toUpperCase());

        kParsableExts.add(CHIP);
        kParsableExts.add(CHIP.toUpperCase());

        kParsableExts.add(GRP);
        kParsableExts.add(GRP.toUpperCase());

        kParsableExts.add(RNK);
        kParsableExts.add(RNK.toUpperCase());

        kParsableExts.add(GMX);
        kParsableExts.add(GMX.toUpperCase());

        kParsableExts.add(GMT);
        kParsableExts.add(GMT.toUpperCase());

        kParsableExts.add(DFR);
        kParsableExts.add(DFR.toUpperCase());

        kParsableExts.add(MAP);
        kParsableExts.add(MAP.toUpperCase());

        kParsableExts.add(EDB);
        kParsableExts.add(EDB.toUpperCase());

        kParsableExts.add(RPT);
        kParsableExts.add(RPT.toUpperCase());

        kParsableExts.add(SIN);
        kParsableExts.add(SIN.toUpperCase());

        //@maint
        // here we make the associations b/w a pob and all the dataformats that it is saveable in
        // use the interface when available rather than a derived class
        // @todo add more
        kPobClassDataFormatMap = new HashMap();

        kPobClassDataFormatMap.put(Dataset.class, new DataFormat[]{
                DataFormat.RES_FORMAT,
                DataFormat.GCT_FORMAT,
                DataFormat.TXT_DATASET_FORMAT,
                DataFormat.DCHIP_FORMAT,
                DataFormat.PCL_FORMAT});

        kPobClassDataFormatMap.put(HClustDatasetStanford.class, new DataFormat[]{
                DataFormat.CDT_FORMAT,
                DataFormat.RES_FORMAT,
                DataFormat.GCT_FORMAT,
                DataFormat.TXT_DATASET_FORMAT,
                DataFormat.DCHIP_FORMAT});

        kPobClassDataFormatMap.put(Template.class, new DataFormat[]{DataFormat.CLS_FORMAT});

        kPobClassDataFormatMap.put(GeneSetMatrix.class, new DataFormat[]{DataFormat.GMX_FORMAT, DataFormat.GMT_FORMAT});
        kPobClassDataFormatMap.put(DefaultGeneSetMatrix.class, new DataFormat[]{DataFormat.GMX_FORMAT, DataFormat.GMT_FORMAT});

        kPobClassDataFormatMap.put(GeneSet.class, new DataFormat[]{DataFormat.GRP_FORMAT});
        kPobClassDataFormatMap.put(FSet.class, new DataFormat[]{DataFormat.GRP_FORMAT});

        kPobClassDataFormatMap.put(RankedList.class, new DataFormat[]{DataFormat.RNK_FORMAT});
        kPobClassDataFormatMap.put(LabelledVector.class, new DataFormat[]{DataFormat.RNK_FORMAT});

    }

    private static javax.swing.filechooser.FileFilter kParsableFileFilter = null;

    public static javax.swing.filechooser.FileFilter getParsableFileFilter() {

        if (kParsableFileFilter == null) {
            kParsableFileFilter = new ParsableFileFilter();
        }

        return kParsableFileFilter;
    }


    static class ParsableFileFilter extends javax.swing.filechooser.FileFilter implements FilenameFilter {

        /**
         * @todo make this case INsensitive lookup
         * The file extension used for the filtering
         */

        public ParsableFileFilter() {

        }

        public boolean accept(File dir, String name) {
            return accept(name);
        }

        public boolean accept(File pathName) {
            return accept(pathName.getName());
        }

        public boolean accept(String pathName) {
            _initParsableExts();
            String ext = NamingConventions.getExtension(pathName);
            return kParsableExts.contains(ext);
        }

        public String getDescription() {
            return "MIT Parsable files";
        }

    }    // End ParsableFileFilter

    static class ParsableFileView extends FileView {

        /**
         * Custom icons for file types that have associated actions.
         */
        public Icon getIcon(final File file) {
            return DataFormat.getIconOrNull(file);
        }

        /**
         * Default handling.
         * Let the L&F FileView figure this out.
         */
        public String getTypeDescription(final File file) {
            return null;
        }

        /**
         * Default handling.
         * Let the L&F FileView figure this out.
         */
        public String getDescription(final File file) {
            return DataFormat.getDesc(file);
        }

    }    // End class ParsableFileView


    public static FileView getParsableFileView() {
        return new ParsableFileView();
    }


    /**
     * @param obj Typically a DataFormat or ext.
     *            Ok, to specify the toString value alos (for example: Foo[bar])
     *            This is used typically form a combo box.
     * @return
     */
    public static DataFormat getExtension(Object obj) {
        if (obj == null) {
            throw new IllegalArgumentException("Parameter obj cannot be null");
        }

        String ext;
        if (obj instanceof DataFormat) {
            ext = ((DataFormat) obj).getExtension();
        } else {
            ext = obj.toString();
        }

        for (int i = 0; i < ALL.length; i++) {
            if (ALL[i].getExtension().equalsIgnoreCase(ext)) {
                return ALL[i];
            }

            if (ALL[i].toString().equalsIgnoreCase(ext)) { // ok, to specify the toString value alos (for example: Foo[bar])
                return ALL[i];
            }
        }

        throw new IllegalArgumentException("No DataFormat found for: " + ext);
    }


    /**
     * key -> Class (one of the pobs), value -> string ext
     */
    private static final Map kClassExtMap;

    /**
     * key -> ext, value -> DataFormat that represents that ext
     */
    private static final Map kExtDfMap;

    /**
     * key -> classname, value -> icon to represent the class
     */
    private static final Map kClassIconMap;

    /**
     * key -> class, value->corresp dataformats name
     */
    private static final Map kClassNameMap;

    static {

        /**
         * Hash the internal data formats as client code deals with them
         * in object form and nees to lookup things like, extension, icon etc.
         */
        kClassExtMap = new HashMap();
        kExtDfMap = new HashMap();
        kClassIconMap = new HashMap();
        kClassNameMap = new HashMap();

        for (int i = 0; i < ALL.length; i++) {
            kExtDfMap.put(ALL[i].getExtension(), ALL[i]);

            Class repClass = ALL[i].getRepresentationClass();

            if (ExtFormat.class.isAssignableFrom(repClass)) {
                // dont bother hashing
            } else {

                // use the first one as the default
                if (!kClassNameMap.containsKey(repClass)) {
                    kClassNameMap.put(ALL[i].getRepresentationClass(), ALL[i].getName());
                }
                if (!kClassExtMap.containsKey(repClass)) {
                    kClassExtMap.put(ALL[i].getRepresentationClass(), ALL[i].getExtension());
                }

                if (!kClassIconMap.containsKey(repClass)) {
                    kClassIconMap.put(ALL[i].getRepresentationClass(), ALL[i].getIcon());
                }
            }
        }
    }

    /**
     * Class variables
     */
    private String fName;
    private String fDesc;
    private Icon fIcon;
    private String fExt;
    private Class fParserClass;
    private FilenameFilter fFilt;

    /**
     * Class to represent a (and all) external data formats.
     * Such as ms excel, adobe pdf etc - things that have no pob's to
     * represent them.
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private class ExtFormat {
    }

    /**
     * Class Constructor.
     * <p/>
     * ext should not include the ".". For example cls files
     * have the ext as "cls" and not ".cls"
     *
     * @param ext The extension.
     * @throws java.lang.IllegalArgumentException
     *          on invalid ext and bad input params
     */

    public DataFormat(Class repClass, String name, String desc, String ext, Icon icon)
            throws IllegalArgumentException {
        this(repClass, name, desc, ext, icon, null);
    }

    public DataFormat(Class repClass, String name, String desc, String ext, Icon icon, Class parserClass)
            throws IllegalArgumentException {

        super(repClass, ext);

        if (name == null) {
            throw new IllegalArgumentException("Param name cannot be null");
        }

        if (desc == null) {
            throw new IllegalArgumentException("Param desc cannot be null");
        }

        if (!SystemUtils.isHeadless() && icon == null) {
            throw new IllegalArgumentException("Param icon cannot be null");
        }

        if (ext == null) {
            throw new IllegalArgumentException("Null extension is not allowed: " + ext);
        }

        if (ext.startsWith(".")) {
            throw new IllegalArgumentException("DataFormat extension cannot begin with a period '.' " + ext);
        }

        if (!(ext.length() > 0)) {
            throw new IllegalArgumentException("DataFormat extension cannot have zero length "
                    + ext);
        }

        // parser can be null

        this.fExt = ext;
        this.fIcon = icon;
        this.fName = name;
        this.fDesc = desc;
        this.fParserClass = parserClass;
    }

    /**
     * @return Name of this DataFormat
     *         In "English"
     */
    public String getName() {
        return fName;
    }

    /**
     * @return A short description of this DataFormat
     */
    public String getDesc() {
        return fDesc;
    }

    // lazily made
    public FilenameFilter getFilenameFilter() {
        if (fFilt == null) {
            fFilt = createFnf(fExt);
        }

        return fFilt;
    }

    public static FilenameFilter createFilenameFilter_for_dataset() {
        return createFilenameFilter(ALL_DATASET_FORMATS);
    }

    public static FilenameFilter createFilenameFilter_for_template() {
        return createFilenameFilter(ALL_TEMPLATE_FORMATS);
    }

    public static FilenameFilter createFilenameFilter_for_genesetmatrix() {
        return createFilenameFilter(ALL_GENESETMATRIX_FORMATS);
    }

    public static FilenameFilter createFilenameFilter_for_geneset() {
        return createFilenameFilter(ALL_GENESET_FORMATS);
    }

    public static FilenameFilter createFilenameFilter(DataFormat[] dfs) {
        Set exts = new HashSet();
        for (int i = 0; i < dfs.length; i++) {
            exts.add(dfs[i].getExtension());
        }

        return createFnf(exts);
        //return createFnf((String[]) exts.toArray(new String[exts.size()]));
    }

    public static FilenameFilter createFilenameFilter(String[] exts) {
        Set set = new HashSet();
        for (int i = 0; i < exts.length; i++) {
            set.add(exts[i]);
        }

        return createFnf(set);
        //return createFnf((String[]) exts.toArray(new String[exts.size()]));
    }

    private static FilenameFilter createFnf(final String ext) {
        return new FilenameFilter() {
            public boolean accept(File file, String s) {
                return s.endsWith(ext);
            }
        };
    }

    private static FilenameFilter createFnf(final Set exts) {
        return new FilenameFilter() {
            public boolean accept(File file, String s) {
                String ext = NamingConventions.getExtension(s);
                return exts.contains(ext);

                /*
                for (int i = 0; i < exts.length; i++) {
                    if (s.endsWith(exts[i])) {
                        return true;
                    }
                }
                return false;
                */
            }
        };
    }

    /**
     * @return Extension for this DataFormat
     * @see "IMP to NOT use. As this makes a
     *      String(foo) equal to an Extension("foo")"
     */

    //public int hashCode() {
    //return fExt.hashCode();
    //}

    /**
     * @return Extension for this DataFormat
     */
    public String getExtension() {
        return fExt;
    }

    /**
     * @return Icon standard for this DataFormat
     */
    public Icon getIcon() {
        return fIcon;
    }

    /**
     * Null if no associated parser
     *
     * @return
     */
    public Parser createParser() throws Exception {
        if (fParserClass == null) {
            return null;
        } else {
            return (Parser) fParserClass.newInstance();
        }
    }

    /**
     * @return
     * @note IMP IMP: overriding the base (java mime-type impl) to get a more user friendly output
     * -- used in combo boxes etc.
     * (dont leave spaces though as needs to be used in cmd line also)
     */
    public String toString() {
        return new StringBuffer(getName()).append("[").append(getExtension()).append("]").toString();
    }

    /**
     * @param obj
     * @return
     * @note IMP: Again, override base impl so that lookups based on the EXT value work
     */
    public boolean equals(Object obj) {

        if (obj == null) {
            return false;
        }

        if (obj instanceof DataFormat) {
            if (((DataFormat) obj).fExt.equals(this.fExt)) {
                return true;
            }
        } else {
            return fExt.equalsIgnoreCase(obj.toString());
        }

        return false;
    }

    //------------------------------------------------------------------------------//
    // --------------------------- STATIC LOOKUP METHODS ---------------------------//
    //------------------------------------------------------------------------------//

    /**
     * @param cl NamingConventions extension for specified class. If no
     *           standard is known, returns null.
     * @return
     */
    public static String getExtension(final Class clz) {

        Class cl = _deconvClass(clz);

        Object obj = kClassExtMap.get(cl);

        if (obj != null) {
            return obj.toString();
        } else {
            klog.warn("No extension found for class: " + cl);
            return "txt";
        }
    }

    /**
     * @param pob NamingConventions extension for specified PersistentObject.
     *            If no standard is known, returns null.
     * @return
     */
    public static String getExtension(PersistentObject pob) {

        if (pob == null) {
            return null;
        }

        return getExtension(pob.getClass());
    }

    /**
     * @param ext
     * @return True if specified ext is a known standard file extension
     *         Example: "res" (do not specify with the ".")
     */
    public static boolean isKnownExtension(String ext) {

        //log.info("checking ext for ext: " + ext);
        if (ext == null) {
            return false;
        }

        if (ext.startsWith(".")) {    // try to fix errors in which cient code specifies the dot
            ext = ext.substring(1, ext.length());
        }

        return kExtDfMap.containsKey(ext);
    }

    /**
     * @param ext
     * @return
     */
    public static DataFormat getDataFormat(String ext) {

        Object obj = kExtDfMap.get(ext);

        if (obj != null) {
            return (DataFormat) obj;
        } else {
            return null;
        }
    }

    public static Class getRepresentationClass(final String ext) {

        Object obj = kExtDfMap.get(ext);

        if (obj != null) {
            return ((DataFormat) obj).getRepresentationClass();
        } else {
            return null;
        }
    }

    /**
     * @param ext
     * @return standard Icon for specified ext, if known.
     *         Else JarResources.ICON_NOT_FOUND.
     */
    public static Icon getIcon(final String ext) {

        //log.info("Getting getIcon for ext: " + ext);
        DataFormat df = getDataFormat(ext);

        if (df != null) {
            return df.getIcon();
        } else {
            return JarResources.ICON_UNKNOWN_DATA_FORMAT;
        }
    }

    public static Icon getIconOrNull(final String ext) {

        //log.info("Getting getIcon for ext: " + ext);
        DataFormat df = getDataFormat(ext);

        if (df != null) {
            return df.getIcon();
        } else {
            return null;
        }
    }

    /**
     * @param ext
     * @return
     */
    public static String getDesc(String ext) {

        //log.info("Getting getIcon for ext: " + ext);
        DataFormat df = getDataFormat(ext);

        if (df != null) {
            return df.getDesc();
        } else {
            return null;
        }
    }

    public static String getDesc(File file) {

        //log.info("Getting getIcon for ext: " + ext);
        String ext = NamingConventions.getExtension(file).toLowerCase();
        DataFormat df = getDataFormat(ext);

        if (df != null) {
            return df.getDesc();
        } else {
            return null;
        }
    }

    /**
     * @param pobClass
     * @return NamingConventions english name for specified class, if known.
     *         Else Constants.UNKNOWN.
     */
    public static String getName(final Class pobClass) {

        Class cl = _deconvClass(pobClass);
        Object obj = kClassNameMap.get(cl);

        if (obj != null) {
            return (String) obj;
        } else {
            return Constants.UNKNOWN;
        }
    }

    private static Class _deconvClass(final Class pobClass) {

        if (kClassNameMap.containsKey(pobClass)) {
            return pobClass;
        }

        Class[] interfaces = pobClass.getInterfaces();
        for (int i = 0; i < interfaces.length; i++) {
            Object obj = kClassNameMap.get(interfaces[i]);
            if (obj != null) {
                return interfaces[i];
            }
        }

        // @todo fix me see nnm2genesetmatrix for why
        if (pobClass.equals(DefaultGeneSetMatrix.class)) {
            return GeneSetMatrix.class;
        }

        if (pobClass.equals(TemplateImpl.class)) {
            return Template.class;
        }

        return pobClass; // cant do anything
    }

    /**
     * @param obj
     * @return
     */
    public static Icon getIcon(final Object obj) {
        if (obj == null) {
            return JarResources.ICON_NOT_FOUND;
        } else if (obj instanceof File) {
            return getIcon(NamingConventions.getExtension((File) obj));
        } else if (obj instanceof String) {
            return getIcon(NamingConventions.getExtension(obj.toString()));
        } else {
            Object ic = kClassIconMap.get(obj.getClass());
            if (ic == null && obj instanceof PersistentObject) {
                Class repcl = getRepresentationClass((PersistentObject) obj);
                ic = kClassIconMap.get(repcl);
            }

            if (ic == null) {
                return JarResources.ICON_NOT_FOUND;
            } else {
                return (Icon) ic;
            }
        }
    }

    /**
     * @param cl
     * @return NamingConventions Icon for specified class, if known.
     *         Else JarResources.ICON_NOT_FOUND.
     */
    public static Icon getIcon(final Class clz) {

        Class cl = _deconvClass(clz);

        Object ic = kClassIconMap.get(cl);
        if (ic == null) {
            return JarResources.ICON_NOT_FOUND;
        } else {
            return (Icon) ic;
        }
    }


    /**
     * The class that represent specified object
     * Central location where objects are linked with registered classes.
     * <p/>
     * why? other generic ways are tough to implement when dealing with interfaces, subclasses etc
     *
     * @param pob
     * @return
     * @maint add new objects and this will need updating
     */
    public static Class getRepresentationClass(final PersistentObject pob) {

        if (pob instanceof GenesOfInterest) {
            return GenesOfInterest.class;
        } else if (pob instanceof HClustDatasetStanford) {
            return HClustDatasetStanford.class;
        } else if (pob instanceof HClustTree) {
            return HClustTree.class;
        } else if (pob instanceof Template) {
            return Template.class;
        } else if (pob instanceof FeatureAnnot) {
            return FeatureAnnot.class;
        } else if (pob instanceof GeneSet) {
            return GeneSet.class;
        } else if (pob instanceof RankedList) { // careful about dataset!
            return RankedList.class;
        } else if (pob instanceof Dataframe) {
            return Dataframe.class;
        } else if (pob instanceof Matrix) {
            return Matrix.class;
        } else if (pob instanceof GeneSetMatrix) {
            return GeneSetMatrix.class;
        } else if (pob instanceof StringDataframe) {
            return StringDataframe.class;
        } else if (pob instanceof Report) {
            return Report.class;
        } else if (pob instanceof EnrichmentDb) {
            return EnrichmentDb.class;
        } else if (pob instanceof Nedb) {
            return Nedb.class;
        } else if (pob instanceof Chip) {
            return Chip.class;
        } else if (pob instanceof DtgDataset) {
            return DtgDataset.class;
        } else if (pob instanceof Dataset) { // has to be at end so that other ds impls get precedence
            return Dataset.class;
        } else if (pob instanceof SampleAnnot) {
            return SampleAnnot.class;
        } else if (pob instanceof TemplateCohort) {
            return TemplateCohort.class;
        } else if (pob instanceof RankedListDb) {
            return RankedListDb.class;
        } else if (pob instanceof MSigDB) {
            return MSigDB.class;
        } else {
            throw new IllegalArgumentException("Unknown object: " + pob);
        }
    }

    /**
     * @param file
     * @return
     */
    public static Icon getIcon(final File file) {
        String ext = NamingConventions.getExtension(file);
        return getIcon(ext);
    }

    public static Icon getIconOrNull(final File file) {
        String ext = NamingConventions.getExtension(file).toLowerCase();
        return getIconOrNull(ext);
    }


    private static int findMax(Object[] fileArrays) {

        int max = Integer.MIN_VALUE;

        for (int i = 0; i < fileArrays.length; i++) {
            File[] files = (File[]) fileArrays[i];

            if (files.length > max) {
                max = files.length;
            }
        }

        return max;
    }

    public static boolean isCompatibleRepresentationClass(PersistentObject[] pobs, Class cl) {
        for (int i = 0; i < pobs.length; i++) {
            if (isCompatibleRepresentationClass(pobs[i], cl) == false) {
                return false;
            }
        }

        return true;
    }

    public static boolean isCompatibleRepresentationClass(final Object obj, final Class[] classes) {
        for (int c = 0; c < classes.length; c++) {
            if (isCompatibleRepresentationClass(obj, classes[c])) {
                return true;
            }
        }

        return false;
    }

    // @todo this is buggy (see saving reports for error)
    public static boolean isCompatibleRepresentationClass(Object obj, Class cl) {

        if (obj == null) {
            throw new IllegalArgumentException("Parameter obj cannot be null");
        }

        if (cl == null) {
            throw new IllegalArgumentException("Parameter cl cannot be null");
        }

        // first check directly
        if (obj.getClass().getName().equals(cl.getName())) {
            return true;
        }

        // then check interfaces in obj
        Class[] interfacesA = obj.getClass().getInterfaces();
        for (int i = 0; i < interfacesA.length; i++) {
            if (cl.getName().equals(interfacesA[i].getName())) {
                return true;
            }
        }

        // then check interfaces in cl
        Class[] interfacesB = cl.getInterfaces();
        for (int i = 0; i < interfacesB.length; i++) {
            if (obj.getClass().getName().equals(interfacesB[i].getName())) {
                return true;
            }
        }

        // then check interfaces vs interfaces
        for (int a = 0; a < interfacesA.length; a++) {
            for (int b = 0; b < interfacesB.length; b++) {
                if (interfacesA[a].getName().equals(interfacesB[b].getName())) {
                    return true;
                }
            }
        }

        // @todo fix me see GeneSetMatrixReqdParam uses for why
        if (obj instanceof GeneSetMatrix) {
            return true; // disable the check
        }

        if (obj instanceof Dataset) {
            return true; // disable check
        }

        /*
        if (obj instanceof GeneSetMatrix && cl.getName().equals(DefaultGeneSetMatrix.class.getName())) {
            return true;
        }
        */

        // ok, thats its
        return false;
    }

}    // End DataFormat
