package edu.mit.broad.genome.parsers;

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

import edu.mit.broad.genome.Errors;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.Version;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.vdb.Organism;
import edu.mit.broad.vdb.meg.EntrezGene;
import edu.mit.broad.vdb.meg.EntrezGeneDb;
import edu.mit.broad.vdb.meg.EntrezGeneImpl;
import edu.mit.broad.vdb.meg.GeneDb;

import java.io.*;
import java.util.*;

/**
 *
 */
public class EntrezGeneDbParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public EntrezGeneDbParser() {
        super(GeneDb.class);
    }

    /**
     * @see "Below for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {
        throw new NotImplementedException();

    }    // End export

    public List parse(String sourcepath, InputStream is) throws Exception {
        throw new NotImplementedException();

        //return unmodlist(parseEntrezDb(new File(sourcepath), new Organism[]{Organism.HUMAN})); // @note default only human
    }

    private static Set kValid_gene_types;

    // @see http://www.ncbi.nlm.nih.gov/IEB/ToolBox/CPP_DOC/lxr/source/src/objects/entrezgene/entrezgene.asn
    // @see ftp://ftp.ncbi.nih.gov/gene/README
    public static Set getValidGeneTypes() {
        if (kValid_gene_types == null) {
            kValid_gene_types = new HashSet();
            kValid_gene_types.add("protein-coding");
            kValid_gene_types.add("pseudo");
            kValid_gene_types.add("other");
            kValid_gene_types.add("miscRNA");
            kValid_gene_types.add("unknown");
            kValid_gene_types.add("snoRNA");
            kValid_gene_types.add("snRNA");

            kValid_gene_types = Collections.unmodifiableSet(kValid_gene_types);
        }
        return kValid_gene_types;
    }


    public EntrezGeneDb parseEntrezDb(final File entrezDir, final boolean onlyHuman) throws Exception {
        return parseEntrezDb(new EntrezFileSet(entrezDir, new Organism[]{Organism.HUMAN}), true);
    }

    public EntrezGeneDb parseEntrezDb(final EntrezFileSet efs, final boolean onlyHuman) throws Exception {
        return parseEntrezDb(efs, new Organism[]{Organism.HUMAN});
    }

    /**
     *
     */
    public EntrezGeneDb parseEntrezDb(final EntrezFileSet efs, final Organism[] orgs) throws Exception {

        final EntrezGeneDb ezdb = new EntrezGeneDbParser().parseEntrezDb_only_gene_info(efs.geneInfoFile, efs.getOrganisms());

        final EntrezGene.GeneIdAccessionSet[] sets_acc = parseEntrezGeneAccessions(efs.gene2accessionFile, efs.getOrganisms());
        final EntrezGene.GeneIdAccessionSet[] sets_refseq = parseEntrezGeneRefSeq(efs.gene2refseqFile, efs.getOrganisms());
        final EntrezGene.GeneIdAccessionSet[] sets_unigene = parseEntrezGeneUnigene(efs.gene2unigeneFile, efs.getOrganisms());

        List all = new ArrayList();
        all.addAll(Arrays.asList(sets_acc));
        all.addAll(Arrays.asList(sets_refseq));
        all.addAll(Arrays.asList(sets_unigene));

        final EntrezGene.GeneIdAccessionSet[] all_sets = (EntrezGene.GeneIdAccessionSet[]) all.toArray(new EntrezGene.GeneIdAccessionSet[all.size()]);


        log.info("sets_acc: " + sets_acc.length + " sets_refseq: " + sets_refseq.length + " sets_unigene: " + sets_unigene.length + " all: " + all_sets.length);

        Map geneIdAccSetMap = EntrezGene.Helper.toMap(all_sets);

        ezdb.cloneShallow(geneIdAccSetMap);

        return ezdb;
    }

    public EntrezGeneDb parseEntrezDb_only_gene_info(final File file, final boolean onlyHuman) throws Exception {
        return parseEntrezDb_only_gene_info(file, new Organism[]{Organism.HUMAN});
    }

    /**
     * The nomenclature status of the name, where
     * 0 = official from a nomenclature committee,
     * I = interim from a nomenclature committee,
     * - = NCBI-supplied.
     * IMP IMP:  You have to edit the geneinfo file so that the headers is UNcommented and the seperators are tabs
     */
    public EntrezGeneDb parseEntrezDb_only_gene_info(final File gene_info_file, final Organism[] orgs) throws Exception {

        final List entrezids = new ArrayList();

        final BufferedReader bin = new BufferedReader(new InputStreamReader(new FileInputStream(gene_info_file)));

        String currLine = nextLineTrimless(bin);
        currLine = nextLineTrimless(bin);

        final Set allowTheseTaxIds = Organism.getTaxIds(orgs);

        log.debug("Allowed tax ids: " + allowTheseTaxIds);

        final Set allowTheseGeneTypes = getValidGeneTypes();

        int lineCnt = 0;
        // tax_id	GeneID	Symbol	LocusTag	Synonyms	dbXrefs	chromosome	map_location 	description
        // 0         1        2      3            4             5    6             7             8
        // type_of_gene	Symbol_from_nomenclature_authority	Full_name_from_nomenclature_authority	Nomenclature_status
        //  9            10                                    11                                     12
        while (currLine != null) {

            final String[] fields = ParseUtils.string2strings(currLine, "\t", false);
            final String taxid = fields[0];
            boolean doIt = false;

            String type_of_gene = fields[9];

            if (allowTheseTaxIds.contains(taxid)) {
                doIt = true;
            }

            if (allowTheseGeneTypes.contains(type_of_gene) == false) {
                doIt = false;
            }

            if (doIt) {
                String geneid = fields[1];
                String symbol = fields[2];
                String title = fields[8];
                String cyto = fields[7];
                String a = fields[4];
                String status = fields[12];
                Set aliases = ParseUtils.string2stringsSet(a, "|", false);
                aliases.remove("-"); // @note

                /*
                if (status.equalsIgnoreCase("-")) {
                    System.out.println(symbol + " " + status);
                }
                */
                entrezids.add(new EntrezGeneImpl(Integer.parseInt(geneid), Integer.parseInt(taxid), symbol, title, aliases, cyto, null, status));
            }

            currLine = nextLineTrimless(bin); /// so that last col(s) can be a tab
            lineCnt++;

            if (lineCnt % 100000 == 0) {
                System.out.println("Done entrez line: " + lineCnt + " #=" + entrezids.size());
            }
        }

        bin.close();
        return new EntrezGeneDb(gene_info_file, new Version(1), (EntrezGene[]) entrezids.toArray(new EntrezGene[entrezids.size()]));
    }

    /*

#Format: tax_id GeneID status RNA_nucleotide_accession.version RNA_nucleotide_gi protein_accession.version protein_gi genomic_nucleotide_accession.version genomic_nucleotide_gi start_position_on_the_genomic_accession end_position_on_the_genomic_accession orientation assembly (tab is used as a separator, pound sign - start of a comment)


#Format:

tax_id GeneID status RNA_nucleotide_accession.version
0       1      2                3

dont care about the rest


9606	7157	-	X60017.1	506446	CAA42632.1	506447	-	-	-	-	?	-
9606	7157	-	X60018.1	506448	CAA42633.1	506449	-	-	-	-	?	-
9606	7157	-	X60019.1	506450	CAA42634.1	506451	-	-	-	-	?	-
9606	7157	-	X60020.1	506452	CAA42635.1	506453	-	-	-	-	?	-
9606	7157	REVIEWED	NM_000546.2	8400737	NP_000537.2	8400738	AC_000060.1	89161195	7598117	7617277	-	Alternate assembly (based on Celera assembly)
9606	7157	REVIEWED	NM_000546.2	8400737	NP_000537.2	8400738	NC_000017.9	51511734	7512463	7531641	-	Reference assembly
9606	7157	REVIEWED	NM_000546.2	8400737	NP_000537.2	8400738	NT_010718.15	51474229	7169087	7188265	-	Reference assembly
9606	7157	REVIEWED	NM_000546.2	8400737	NP_000537.2	8400738	NW_926584.1	89042384	7515613	7534773	-	Alternate assembly (based on Celera assembly)
9606	7158	-	-	-	-	-	AC018924.9	22262473	77540	180869	-	-
9606	7158	-	-	-	AAW69392.1	58258012	AY904026.1	58258011	-	-	?	-
    */
    public EntrezGene.GeneIdAccessionSet[] parseEntrezGeneAccessions(final File gene2Accession_file, final Organism[] orgs) throws Exception {

        // gene ids are unique across organisms
        Map entrezIdAccessionSetMap = new HashMap();

        final BufferedReader bin = new BufferedReader(new InputStreamReader(new FileInputStream(gene2Accession_file)));

        String currLine = nextLineTrimless(bin);
        //currLine = nextLineTrimless(bin);

        final Set allowTheseTaxIds = Organism.getTaxIds(orgs);

        log.debug("Allowed tax ids: " + allowTheseTaxIds);

        int lineCnt = 0;
        //  tax_id GeneID status RNA_nucleotide_accession.version
        // 0       1      2                3

        while (currLine != null) {

            final String[] fields = ParseUtils.string2strings(currLine, "\t", false);
            final long taxid = Long.parseLong(fields[0]);

            final long geneid_l = Long.parseLong(fields[1]);
            final String geneid_s = geneid_l + "";
            //String status = fields[2];
            String acc = fields[3];

            boolean doIt = false;

            if (allowTheseTaxIds.contains("" + taxid)) {
                doIt = true;
            }

            if (acc == null || acc.length() == 0 || acc.equals("-")) {
                doIt = false;
            }

            if (doIt) {
                acc = NamingConventions.removeExtension(acc); // get rid of the version
                Object curr = entrezIdAccessionSetMap.get(geneid_s);
                if (curr == null) {
                    curr = new HashSet();
                }
                ((Set) curr).add(acc);

                entrezIdAccessionSetMap.put(geneid_s, curr);
            }

            currLine = nextLineTrimless(bin); /// so that last col(s) can be a tab
            lineCnt++;

            if (lineCnt % 100000 == 0) {
                System.out.println("Done entrez gene accessions line: " + lineCnt + " #=" + entrezIdAccessionSetMap.size());
            }
        }

        bin.close();

        final EntrezGene.GeneIdAccessionSet[] g2as = new EntrezGene.GeneIdAccessionSet[entrezIdAccessionSetMap.size()];
        int cnt = 0;
        for (Iterator iterator = entrezIdAccessionSetMap.keySet().iterator(); iterator.hasNext();) {
            String key = iterator.next().toString();
            Object value = entrezIdAccessionSetMap.get(key);
            g2as[cnt++] = new EntrezGene.GeneIdAccessionSet(Long.parseLong(key), (Set) value);
        }

        log.info("Made SymbolAccessions: " + g2as.length);

        return g2as;
    }


    /*


#Format: tax_id GeneID status RNA_nucleotide_accession.version RNA_nucleotide_gi protein_accession.version protein_gi genomic_nucleotide_accession.version genomic_nucleotide_gi start_position_on_the_genomic_accession end_position_on_the_genomic_accession orientation assembly (tab is used as a separator, pound sign - start of a comment)


#Format:

tax_id GeneID status RNA_nucleotide_accession.version
0       1      2                3

dont care about the rest


9606	8912	REVIEWED	NM_001005407.1	53832010	NP_001005407.1	53832011	NW_926018.1	89040669	1134606	1203315	+	Alternate assembly (based on Celera assembly)
9606	8912	REVIEWED	NM_021098.2	53832008	NP_066921.2	53832009	AC_000059.1	89161194	1418956	1487665	+	Alternate assembly (based on Celera assembly)
9606	8912	REVIEWED	NM_021098.2	53832008	NP_066921.2	53832009	NC_000016.8	51511732	1143241	1211772	+	Reference assembly
9606	8912	REVIEWED	NM_021098.2	53832008	NP_066921.2	53832009	NT_037887.4	51473210	1143241	1211772	+	Reference assembly
9606	8912	REVIEWED	NM_021098.2	53832008	NP_066921.2	53832009	NW_926018.1	89040669	1134606	1203315	+	Alternate assembly (based on Celera assembly)
9606	8913	VALIDATED	NM_018896.3	38505266	NP_061496.2	20070163	AC_000060.1	89161195	45099810	45165916	+	Alternate assembly (based on Celera assembly)
9606	8913	VALIDATED	NM_018896.3	38505266	NP_061496.2	20070163	NC_000017.9	51511734	45993447	46059540	+	Reference assembly
9606	8913	VALIDATED	NM_018896.3	38505266	NP_061496.2	20070163	NT_010783.14	37544107	7291726	7357819	+	Reference assembly
9606	8913	VALIDATED	NM_018896.3	38505266	NP_061496.2	20070163	NW_926894.1	89042910	3030832	3096938	+	Alternate assembly (based on Celera assembly)
9606	8913	VALIDATED	NM_198376.1	38505267	NP_938190.1	38505268	AC_000060.1	89161195	45099810	45165916	+	Alternate assembly (based on Celera assembly)
9606	8913	VALIDATED	NM_198376.1	38505267	NP_938190.1	38505268	NC_000017.9	51511734	45993447	46059540	+	Reference assembly


    */
    public EntrezGene.GeneIdAccessionSet[] parseEntrezGeneRefSeq(final File gene2refseq_file, final Organism[] orgs) throws Exception {

        // gene ids are unique across organisms
        Map entrezIdAccessionSetMap = new HashMap();

        final BufferedReader bin = new BufferedReader(new InputStreamReader(new FileInputStream(gene2refseq_file)));

        String currLine = nextLineTrimless(bin);
        //currLine = nextLineTrimless(bin);

        final Set allowTheseTaxIds = Organism.getTaxIds(orgs);

        log.debug("Allowed tax ids: " + allowTheseTaxIds);

        int lineCnt = 0;
        //  tax_id GeneID status RNA_nucleotide_accession.version
        // 0       1      2                3

        while (currLine != null) {

            final String[] fields = ParseUtils.string2strings(currLine, "\t", false);
            final long taxid = Long.parseLong(fields[0]);

            long geneid_l = Long.parseLong(fields[1]);
            String geneid_s = geneid_l + "";
            //String status = fields[2];
            String acc = fields[3];

            boolean doIt = false;

            if (allowTheseTaxIds.contains(taxid + "")) {
                doIt = true;
            }

            if (acc == null || acc.length() == 0 || acc.equals("-")) {
                doIt = false;
            }

            if (doIt) {
                acc = NamingConventions.removeExtension(acc); // get rid of the version
                Object curr = entrezIdAccessionSetMap.get(geneid_s);
                if (curr == null) {
                    curr = new HashSet();
                }
                ((Set) curr).add(acc);

                entrezIdAccessionSetMap.put(geneid_s, curr);
            }

            currLine = nextLineTrimless(bin); /// so that last col(s) can be a tab
            lineCnt++;

            if (lineCnt % 100000 == 0) {
                System.out.println("Done entrez gene refseqs line: " + lineCnt + " #=" + entrezIdAccessionSetMap.size());
            }
        }

        bin.close();


        final EntrezGene.GeneIdAccessionSet[] g2as = new EntrezGene.GeneIdAccessionSet[entrezIdAccessionSetMap.size()];
        int cnt = 0;
        for (Iterator iterator = entrezIdAccessionSetMap.keySet().iterator(); iterator.hasNext();) {
            String key = iterator.next().toString();
            Object value = entrezIdAccessionSetMap.get(key);
            g2as[cnt++] = new EntrezGene.GeneIdAccessionSet(Long.parseLong(key), (Set) value);
        }

        log.info("Made GeneRefSeq: " + g2as.length);

        return g2as;
    }


    /*


#Format: GeneID UniGene_cluster (tab is used as a separator, pound sign - start of a comment)

GeneID UniGene_cluster
0       1

595	Hs.861
6652	Hs.878
650083	Hs.878
1178	Hs.889
2205	Hs.897
178	Hs.904
10865	Hs.920
1428	Hs.924
    */
    public EntrezGene.GeneIdAccessionSet[] parseEntrezGeneUnigene(final File gene2unigene_file, final Organism[] orgs) throws Exception {

        // gene ids are unique across organisms
        Map entrezIdAccessionSetMap = new HashMap();

        final BufferedReader bin = new BufferedReader(new InputStreamReader(new FileInputStream(gene2unigene_file)));

        String currLine = nextLineTrimless(bin);
        //currLine = nextLineTrimless(bin);

        final Set allowTheseUnigeneCodes = Organism.getUnigeneCodes(orgs);

        log.debug("Allowed tax ids: " + allowTheseUnigeneCodes);

        int lineCnt = 0;
        //  GeneID UniGene_cluster
        //      0       1

        while (currLine != null) {

            final String[] fields = ParseUtils.string2strings(currLine, "\t", false);

            if (fields.length != 2) {
                throw new ParserException("Expected 2 fields got: " + fields.length + " line >" + currLine + "<");
            }

            long geneid = Long.parseLong(fields[0]);
            String ug_acc = fields[1];

            if (ug_acc == null || ug_acc.length() == 0 || ug_acc.equals("-")) {
                continue;
            }

            String ug_code = NamingConventions.removeExtension(ug_acc);

            boolean doIt = false;

            if (allowTheseUnigeneCodes.contains(ug_code)) {
                doIt = true;
            }

            if (doIt) {
                Object curr = entrezIdAccessionSetMap.get("" + geneid);
                if (curr == null) {
                    curr = new HashSet();
                }
                ((Set) curr).add(ug_acc);

                entrezIdAccessionSetMap.put("" + geneid, curr);
            }

            currLine = nextLineTrimless(bin); /// so that last col(s) can be a tab
            lineCnt++;

            if (lineCnt % 25000 == 0) {
                System.out.println("Done entrez gene accessions line: " + lineCnt + " #=" + entrezIdAccessionSetMap.size());
            }
        }

        bin.close();


        final EntrezGene.GeneIdAccessionSet[] g2as = new EntrezGene.GeneIdAccessionSet[entrezIdAccessionSetMap.size()];
        int cnt = 0;
        for (Iterator iterator = entrezIdAccessionSetMap.keySet().iterator(); iterator.hasNext();) {
            String key = iterator.next().toString();
            Object value = entrezIdAccessionSetMap.get(key);
            g2as[cnt++] = new EntrezGene.GeneIdAccessionSet(Long.parseLong(key), (Set) value);
        }

        log.info("Made GeneRefSeq: " + g2as.length);

        return g2as;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public static class EntrezFileSet {

        /*
       FILES

       @see ftp://ftp.ncbi.nih.gov/gene/README

       // Hs.xxx => Unigene
       // gene_info => entrez genes
               tab-delimited
              one line per GeneID
              Column header line is the first line in the file.

       // gene2accession =>  NM and misc accessions
                This file is a comprehensive report of the accessions that are
                related to a GeneID.  It includes sequences from the international
                sequence collaboration, Swiss-Prot, and RefSeq.

       // accession2geneid => v (from refseq) NOT needed anymore
        */


        private File fBaseDir;

        private File geneInfoFile;

        private File gene2accessionFile;

        private File gene2refseqFile;

        private File gene2unigeneFile;

        private Organism[] fOrgs;

        /**
         * Class constructor
         *
         * @param baseDir
         */
        public EntrezFileSet(final File baseDir, final Organism org) throws Exception {
            this(baseDir, new Organism[]{org});
        }

        /**
         * Class constructor
         *
         * @param baseDir
         * @param orgs
         * @throws Exception
         */
        public EntrezFileSet(final File baseDir, final Organism[] orgs) throws Exception {

            if (baseDir == null || baseDir.exists() == false) {
                throw new IllegalArgumentException("Does not exist or null: " + baseDir);
            }

            this.fBaseDir = baseDir;

            this.fOrgs = orgs;

            Errors errors = new Errors();


            geneInfoFile = new File(getBaseDir(), "gene_info");
            gene2accessionFile = new File(getBaseDir(), "gene2accession");
            gene2refseqFile = new File(getBaseDir(), "gene2refseq");
            gene2unigeneFile = new File(getBaseDir(), "gene2unigene");

            if (geneInfoFile.exists() == false) {
                errors.add(geneInfoFile.getPath());
            }
            if (gene2accessionFile.exists() == false) {
                errors.add(gene2accessionFile.getPath());
            }
            if (gene2refseqFile.exists() == false) {
                errors.add(gene2refseqFile.getPath());
            }
            if (gene2unigeneFile.exists() == false) {
                errors.add(gene2unigeneFile.getPath());
            }

            errors.barfIfNotEmpty();
        }

        public File getBaseDir() {
            return fBaseDir;
        }

        public Organism[] getOrganisms() {
            return fOrgs;
        }

    } // End inner class EntrezFileSet

}    // End of class EntrezGeneDbParser
