/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.vdb.sampledb.SampleAnnot;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Parses a gct formatted dataset -- similar to dataframe except that formatted
 * so that legacy MIT gct data is accepted
 * <p/>
 * Format:
 * <p/>
 * #1.2
 * 12488	41
 * Name	Description	col_0 col_1 col_2 ...
 * row_o
 * row_1
 * ...
 * <p/>
 * NO AP calls
 */


public class GctParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public GctParser() {
        super(Dataset.class);
    }

    /**
     * Export a Dataset to file in gct format
     * Only works with Datasets
     *
     * @see "Above for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {
        _export(pob, startExport(pob, file));
    }

    public void export(final PersistentObject pob, final OutputStream os) throws Exception {
        _export(pob, startExport(pob, os, null));
    }

    private void _export(final PersistentObject pob, final PrintWriter pw) throws Exception {

        final Dataset ds = (Dataset) pob;
        FeatureAnnot ann = null;
        if (ds.getAnnot() != null) {
            ann = ds.getAnnot().getFeatureAnnot();
        }

        //log.debug("Annotation is: " + ann);

        pw.println("#1.2"); // not sure what the # means, but give the people what they want
        pw.println(ds.getNumRow() + "\t" + ds.getNumCol());
        pw.print(Constants.NAME + "\t" + Constants.DESCRIPTION + "\t");

        for (int i = 0; i < ds.getNumCol(); i++) {
            pw.print(ds.getColumnName(i));
            if (i != ds.getNumCol() - 1) {
                pw.print('\t');
            }
        }

        pw.println();

        // Give preference to Native desc if it exists
        // If not, use the symbol desc
        for (int r = 0; r < ds.getNumRow(); r++) {
            StringBuffer buf = new StringBuffer();
            String rowName = ds.getRowName(r);
            buf.append(rowName).append('\t');
            String desc = Constants.NA;
            if (ann != null) {
                if (ann.hasNativeDescriptions()) {
                    desc = ann.getNativeDesc(rowName);
                } else {
                    String symbol = ann.getGeneSymbol(rowName);
                    if (symbol != null) {
                        desc = symbol + ":" + ann.getGeneTitle(rowName);
                    }
                }
            }

            if (desc == null) {
                desc = Constants.NA;
            }

            buf.append(desc).append('\t');
            buf.append(ds.getRow(r).toString('\t'));
            pw.println(buf.toString());
        }

        pw.close();

        doneExport();
    }    // End export

    /**
     * @param name
     * @param s
     * @return
     * @throws Exception
     */
    public List parse(String objname, String s) throws Exception {

        StringReader sr = new StringReader(s);
        BufferedReader bin = new BufferedReader(sr);
        return _parse(objname, bin, true);

    }

    /**
     * Parse a Dataset from memory
     *
     * @param s
     * @return
     * @throws Exception
     */
    public List parse(String s) throws Exception {
        String name = NamingConventions.generateName(Constants.RES);
        return parse(name, s);
    }

    /**
     * @returns 1 Dataset object
     * Always DatasetAnnotation object produced , but if underlying df has none then na is used
     * @see above for format
     */
    public List parse(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        return _parse(sourcepath, bin, true);

    }

    public List parseFlippedNameDesc(final String sourcepath, final InputStream is) throws Exception {
        startImport(sourcepath);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        return _parse(sourcepath, bin, false); // @note

    }

    /// does the real parsing
    // expects the bin to be untouched
    private List _parse(String objName, BufferedReader bin, boolean nameBeforeDesc) throws Exception {
        objName = NamingConventions.removeExtension(objName);
        String currLine = nextLine(bin);

        // 1st  non-empty, non-comment line is numrows and numcols
        int[] nstuff = ParseUtils.string2ints(currLine, " \t");
        if (nstuff.length != 2) {
            throw new ParserException("Gct file with bad row/col info on line: " + currLine);
        }

        int nrows = nstuff[0];
        int ncols = nstuff[1];

        // First 2 fields name and desc are to be ignored
        currLine = nextLine(bin);
        //System.out.println(currLine);
        List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

        colnames.remove(0);                                 // first elem is always nonsense
        colnames.remove(0);

        if (colnames.size() != ncols) {
            throw new ParserException("Bad gct format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
        }

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List lines = new ArrayList();

        currLine = nextLineTrimless(bin);

        // save all rows so that we can determine how many rows exist
        while (currLine != null) {
            lines.add(currLine);
            //currLine = nextLine(bin);
            currLine = nextLineTrimless(bin); /// imp for mv datasets -> last col(s) can be a tab
        }

        if (lines.size() != nrows) {
            throw new ParserException("Bad gct format -- exepcted nrows from specification on header line: " + nrows + " but found in data: " + lines.size());
        }

        bin.close();

        return _parseHasDesc(objName, lines, colnames, nameBeforeDesc);
    }

    public List[] parseRowAndColNamesOnly(final File dsFile) throws Exception {

        final BufferedReader bin = new BufferedReader(new FileReader(dsFile));

        String currLine = nextLine(bin);

        // 1st  non-empty, non-comment line is numrows and numcols
        int[] nstuff = ParseUtils.string2ints(currLine, " \t");
        if (nstuff.length != 2) {
            throw new ParserException("Gct file with bad row/col info on line: " + currLine);
        }

        int nrows = nstuff[0];
        int ncols = nstuff[1];

        // First 2 fields name and desc are to be ignored
        currLine = nextLine(bin);
        //System.out.println(currLine);
        final List colNames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

        colNames.remove(0);                                 // first elem is always nonsense
        colNames.remove(0);

        if (colNames.size() != ncols) {
            throw new ParserException("Bad gct format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colNames.size());
        }

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List rowNames = new ArrayList();

        currLine = nextLineTrimless(bin);

        // save all rows so that we can determine how many rows exist
        // Makes this ULTRA fasts - thats the point
        while (currLine != null) {
            StringTokenizer tok = new StringTokenizer(currLine, "\t");
            rowNames.add(tok.nextToken());
            currLine = nextLineTrimless(bin); /// imp for mv datasets -> last col(s) can be a tab
        }

        if (rowNames.size() != nrows) {
            throw new ParserException("Bad gct format -- exepcted nrows from specification on header line: " + nrows + " but found in data: " + rowNames.size());
        }

        bin.close();

        return new List[]{rowNames, colNames};
    }

    private List _parseHasDesc(String objName, List lines, List colNames, boolean nameBeforeDesc) throws Exception {
        objName = NamingConventions.removeExtension(objName);
        Matrix matrix = new Matrix(lines.size(), colNames.size());
        List rowNames = new ArrayList();
        List rowDescs = new ArrayList();

        for (int i = 0; i < lines.size(); i++) {
            String currLine = (String) lines.get(i);
            List fields = string2stringsV2(currLine, colNames.size() + 2); // spaces allowed in name & desc field so DONT tokenize them

            if (fields.size() != colNames.size() + 1 + 1) {
                //System.out.println(">> " + fields);
                throw new ParserException("Bad format - expect ncols: " + (colNames.size() + 1 + 1)
                        + " but found: " + fields.size() + " on line >"
                        + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
            }

            String rowname = fields.get(0).toString().trim();
            if (rowname.length() == 0) {
                throw new ParserException("Bad rowname - cant be empty at: " + i + " >" + currLine);
            }

            String desc = fields.get(1).toString().trim();
            if (desc.length() == 0) {
                desc = Constants.NA; // dont exception out - genecluster like behavior
                //throw new ParserException("Bad rowdescname - cant be empty at: " + i + " >" + currLine);
            }

            if (nameBeforeDesc) {
                // the standard way, do nothing
            } else { // the flipped one
                String tmp = rowname;
                rowname = desc;
                desc = tmp;
            }

            rowDescs.add(desc);
            rowNames.add(rowname);

            for (int f = 2; f < fields.size(); f++) {
                String s = fields.get(f).toString().trim();
                float val;
                if (s.length() == 0) {
                    val = Float.NaN;
                } else {
                    try {
                        val = Float.parseFloat(s);
                    } catch (Exception e) {
                        System.out.println(">" + s + "<");
                        //val = Float.NaN;
                        throw e;
                    }
                }
                matrix.setElement(i, f - 2, val);
            }
        }

        final FeatureAnnot ann = new FeatureAnnotImpl(objName, rowNames, rowDescs);
        ann.addComment(fComment.toString());
        final SampleAnnot sann = new SampleAnnotImpl(objName, colNames, null);

        final Dataset ds = new DefaultDataset(objName, matrix, rowNames, colNames, true, new AnnotImpl(ann, sann));
        ds.addComment(fComment.toString());
        doneImport();
        return unmodlist(new PersistentObject[]{ds});
    }

}    // End of class GctParser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
