/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.parsers;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.vdb.sampledb.SampleAnnot;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * CDT -> Clustered Data Table (Clustered Dataset)
 * <p/>
 * A "native" xtools format but maintained similar to the Stanford cdt format
 * so that files are easily interoperable.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class HClustDatasetParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public HClustDatasetParser() {
        super(HClustDatasetStanford.class);
    }

    /**
     * Only accepts FeatureMap objects.
     */
    public void export(PersistentObject FeatureMap, File file) throws ParserException {
        // @todo
        throw new NotImplementedException();
    }

    /**
     * CDT -> Clustered Data Table
     * <p/>
     * - the GWEIGHTs are ignored
     * - the EWEIGHTs are ignored
     * - AID corresponds to sample names / sample nodes (corresponds to the atr file)
     * - GID corresponds to gene name / gene nodes (corresponds to the gtr file)
     * <p/>
     * GID	YORF	NAME	GWEIGHT	spo0	spo7	spo5	spo11	spo9	spo2	spo30     <---- use for sample names
     * AID				ARRY0X	ARRY4X	ARRY3X	ARRY6X	ARRY5X	ARRY2X	ARRY1X    <---- use for arrayx names (in the atr)
     * EWEIGHT				1	1	1	1	1	1	1         <---- ignore
     * GENE59X	YBR168W	YBR168W	1	0.07	0	0	2.49	2.11	0.11	0.32
     * GENE53X	YBR088C	POL30	1	-0.17	1.58	2.2	0.14	0.49	2.57	0.99
     * GENE23X	YBL015W	ACH1	1	0.52	1.49	1.75	0.19	0.58	1.49	1.01
     * ...
     * ...
     * cluster_name)  (name)  (desc)
     *
     * @note NO atr/gtr parsing or dependance here
     * @note not the greatest parser -> we read in the entire file into memory
     */
    public List parse(String sourcepath, InputStream is) throws Exception {

        startImport(sourcepath);

        //int nlines = DataStoreUtils.countLines(is, true); // @note doesnt disturnb the is???
        //int nfloatlines = nlines - 1 - 1 - 1; // @note diff from txt dataset
        //log.debug("Number of float lines = " + nfloatlines);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        String currLine = nextLine(bin);
        List colNames = ParserFormatsHelper.parseHClustColumnLine(currLine);
        colNames.remove(0);

        currLine = nextLine(bin);
        List arrayxnames = ParserFormatsHelper.parseAidLine(currLine);

        nextLine(bin); // ignore, EWEIGHT row
        currLine = nextLine(bin); // first line of float data
        List lines = new ArrayList();
        while (currLine != null) {
            lines.add(currLine);
            currLine = nextLine(bin);
        }
        bin.close();

        // Initialize the Dataset and Annotation
        Matrix matrix = new Matrix(lines.size(), colNames.size());
        StringMatrix annmatrix = new StringMatrix(lines.size(), 1);    // just the desc

        List genexnames = new ArrayList(lines.size());
        List rowNames = new ArrayList(lines.size());
        List rowDescs = new ArrayList(lines.size());
        for (int r = 0; r < lines.size(); r++) {
            //System.out.println(">> " + currLine);
            String line = lines.get(r).toString();
            List fields = ParseUtils.string2stringsList(line, "\t"); // so that we can accept empty desc

            int expectedsize = colNames.size() + 1 + 1 + 1 + 1; // @note diff from txt dataset parser
            if (fields.size() != expectedsize) {
                throw new ParserException("Invalid format on line at row: " + r + " content: " + line + " expected # fields = " + expectedsize + " but found: " + fields.size());
                // for each cdt file float entry, theres one genexname, one name and one desc
            }

            String genex = (String) fields.get(0);
            genexnames.add(genex);

            String name = (String) fields.get(1);
            rowNames.add(name);

            String desc = (String) fields.get(2);
            rowDescs.add(desc.trim());

            // -- then, onto the floats
            List floats = fields.subList(4, fields.size()); // ignoring the 4th column -- the GWEIGHT
            if (floats.size() != colNames.size()) {
                throw new ParserException("# of floats: " + floats.size() + " on line: " + r + " not equal to # of columns in header: " + colNames.size());
            }

            for (int c = 0; c < floats.size(); c++) {
                matrix.setElement(r, c, Float.parseFloat((String) floats.get(c)));
            }

            annmatrix.setElement(r, 0, desc);             // always column 0
        }

        lines.clear(); // clean up asap

        // Initialize the Dataset and Annotation
        final FeatureAnnot ann = new FeatureAnnotImpl(sourcepath, rowNames, rowDescs);
        ann.addComment(fComment.toString());

        final SampleAnnot sann = new SampleAnnotImpl(sourcepath, colNames, null);

        Dataset ds = new DefaultDataset(sourcepath, matrix, rowNames, colNames, true, new AnnotImpl(ann, sann));
        ds.addComment(fComment.toString());

        HClustDatasetStanford cds = new HClustDatasetStanford(sourcepath, ds, genexnames, arrayxnames, true);
        cds.addComment(fComment.toString());

        doneImport();
        return unmodlist(new PersistentObject[]{cds});

    }    // End parse()


}    // End HClustDatasetParser
