/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.parsers;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.HClustTree;
import edu.mit.broad.genome.objects.PersistentObject;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * xTR -> Clustered Tree (atr and gtr)
 * <p/>
 * A "native" xtools format but maintained similar to the Stanford atr/gtr formats
 * so that files are easily interoperable.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class HClustTreeParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public HClustTreeParser() {
        super(HClustTree.class);
    }

    /**
     * Only accepts FeatureMap objkects.
     */
    public void export(PersistentObject FeatureMap, File file) throws ParserException {
        throw new NotImplementedException();
    }

    /**
     * GTR -> GeneTree
     * ATR -> ArrayTree (sample in xomics lingo)
     * <p/>
     * Example format for GTR:
     * NODE1X	GENE51X	GENE24X	0.99798583984375
     * NODE2X	GENE41X	GENE40X	0.99786376953125
     * NODE3X	GENE90X	GENE89X	0.997802734375
     * ...
     * ...
     * <p/>
     * Example format for ATR:
     * NODE1X	ARRY4X	ARRY3X	0.948486328125
     * NODE2X	ARRY6X	ARRY5X	0.94122314453125
     * NODE3X	NODE1X	NODE2X	0.904541015625
     * ...
     * ...
     * <p/>
     * Sanity checks:
     * 1st column (the NODEx) must have unique elements
     */
    public List parse(String sourcepath, InputStream is) throws Exception {

        startImport(sourcepath);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        String currLine = null;

        List nodes = new ArrayList();

        /* key -> node name, value -> Node object */
        Map nodeNameNodeObjectMap = new HashMap();

        int r = 0;
        currLine = nextLine(bin); // first line of data
        while (currLine != null) {
            //System.out.println(">> " + currLine);

            List fields = ParseUtils.string2stringsList(currLine, "\t");

            if (fields.size() != 4) {
                throw new ParserException("Format error -- expected 4 columns but found: " + fields.size() + " line: " + currLine + " " + r);
            }

            String name = fields.get(0).toString();
            String aname = fields.get(1).toString();
            String bname = fields.get(2).toString();
            float score = Float.parseFloat(fields.get(3).toString());

            Object parentA;
            if (nodeNameNodeObjectMap.containsKey(aname)) {
                parentA = nodeNameNodeObjectMap.get(aname);
            } else {
                parentA = new HClustTree.Root(aname);
            }

            Object parentB;
            if (nodeNameNodeObjectMap.containsKey(bname)) {
                parentB = nodeNameNodeObjectMap.get(bname);
            } else {
                parentB = new HClustTree.Root(bname);
            }

            HClustTree.Node node = new HClustTree.Node(name, parentA, parentB, score);
            nodeNameNodeObjectMap.put(node.getName(), node);
            //log.debug("Adding node: " + node.getName());
            nodes.add(node);

            r++;
            currLine = nextLine(bin);
        }

        bin.close();

        HClustTree htree = new HClustTree(sourcepath, nodes, true);
        htree.addComment(fComment.toString());

        doneImport();
        return unmodlist(new PersistentObject[]{htree});


    }    // End parse()


}    // End HClustTreeParser
