/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.Medline;
import edu.mit.broad.genome.objects.PersistentObject;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *
 */


public class MedlineParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public MedlineParser() {
        super(Medline.class);
    }

    /**
     * Export a Dataset to file in gct format
     * Only works with Datasets
     *
     * @see "Above for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {
        throw new NotImplementedException();

    }    // End export

    /**
     * I RECORD (OUTPUT FROM ENDNOTE )
     * <p/>
     * Reference Type: Journal Article
     * Record Number: 303
     * Author: Alter, O.; Brown, P. O.; Botstein, D.
     * Year: 2003
     * Title: Generalized singular value decomposition for comparative analysis of genome-scale expression data sets of two different organisms
     * Journal: Proc Natl Acad Sci U S A
     * Volume: 100
     * Issue: 6
     * Pages: 3351-6
     * Date: Mar 18
     * Accession Number: 12631705
     * Keywords: Cell Cycle/genetics
     * Comparative Study
     * Data Interpretation, Statistical
     * Databases, Genetic
     * Gene Expression Profiling/*statistics & numerical data
     * Genes, Fungal/drug effects
     * Genomics/*statistics & numerical data
     * Humans
     * Oligonucleotide Array Sequence Analysis/statistics & numerical data
     * Pheromones/pharmacology
     * RNA, Fungal/genetics/metabolism
     * RNA, Messenger/genetics/metabolism
     * Research Support, Non-U.S. Gov't
     * Research Support, U.S. Gov't, Non-P.H.S.
     * Research Support, U.S. Gov't, P.H.S.
     * Saccharomyces cerevisiae/cytology/genetics/metabolism
     * Stress/genetics
     * Abstract: We describe a comparative mathematical framework for two genome-scale expression data sets. This framework formulates expression as superposition of the effects of regulatory programs, biological processes, and experimental artifacts common to both data sets, as well as those that are exclusive to one data set or the other, by using generalized singular value decomposition. This framework enables comparative reconstruction and classification of the genes and arrays of both data sets. We illustrate this framework with a comparison of yeast and human cell-cycle expression data sets.
     * Notes: 0027-8424
     * Journal Article
     * URL: http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve&db=PubMed&dopt=Citation&list_uids=12631705
     * Author Address: Department of Genetics, Stanford University, Stanford, CA 94305. orly@genome.stanford.edu
     */

    public Medline parseMedline(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);

        final BufferedReader bin = new BufferedReader(new InputStreamReader(is));

        // ignore the first 3 lines
        nextLine(bin);
        nextLine(bin);
        nextLine(bin);

        String currLine = nextLine(bin);
        log.debug("Staring with line: " + currLine);
        List lines = new ArrayList();
        List records = new ArrayList();
        while (currLine != null) {
            if (currLine.startsWith(Headers.EndNote.REFERENCE_TYPE) && lines.size() != 0) {
                Medline.Record recd = parseRecord_endnote(lines);
                if (recd != null) {
                    records.add(recd);
                }
                lines.clear();
            } else {
                lines.add(currLine);
            }
            currLine = nextLine(bin);
        }


        bin.close();

        log.debug("# of Medline records: " + records.size() + " " + records.get(0).toString());

        return new Medline(sourcepath, records);
    }


    private static String ab_prefix = "AB  - ";
    private static String ti_prefix = "TI  - "; // sometimes ab is missing

    private String _parseAbs(final List lines, final int startAtIndex) {

        StringBuffer buf = new StringBuffer();
        for (int i = startAtIndex; i < lines.size(); i++) {
            String line = lines.get(i).toString();

            if (line.startsWith(ab_prefix)) {
                line = line.substring(ab_prefix.length(), line.length());
            }
            if (line.startsWith("AD")) {
                break;
            }

            line = line.trim();
            buf.append(line).append('\n');
        }

        String s = buf.toString().replace('\n', ' ');
        return s.trim();
    }

    private String _parseTi(final List lines, final int startAtIndex) {

        StringBuffer buf = new StringBuffer();
        for (int i = startAtIndex; i < lines.size(); i++) {
            String line = lines.get(i).toString();

            if (line.startsWith(ti_prefix)) {
                line = line.substring(ti_prefix.length());
            }
            if (line.startsWith("PG") || line.startsWith("AU")) {
                break;
            }

            line = line.trim();
            buf.append(line).append('\n');
        }

        String s = buf.toString().replace('\n', ' ');
        return s.trim();
    }

    // from download_pubmed.pl
    public Medline.Record parseMedlineRecord(final String pmid, final File medlineRecordFile) throws Exception {
        String str = ParseUtils.slurp(medlineRecordFile, false);

        if (str == null || str.length() == 0) {
            throw new IllegalArgumentException("Param str cannot be null nor empty: >" + str + "<");
        }

        final List lines = ParseUtils.string2stringsList(str);


        Set keywords = new HashSet();
        String abs = null;
        String ti = null;

        for (int i = 0; i < lines.size(); i++) {
            String s = lines.get(i).toString();
            if (s.startsWith("AB")) {
                abs = _parseAbs(lines, i);
            } else if (s.startsWith("TI")) {
                ti = _parseTi(lines, i);
            } else if (s.startsWith("PMID")) {
                //String[] ss = ParseUtils.string2strings(s, false);
            } else if (s.startsWith("MH")) {
                // String[] ss = ParseUtils.string2strings(s, false);
            } else {
                // do nothing
            }

        }

        if (pmid == null) {
            throw new ParserException("Could not parse pmid from:\n" + str + "\n For file: " + medlineRecordFile.getName());
        }

        if (abs == null || abs.length() == 0) {
            // try harder
            abs = ti;
        }

        if (abs == null || abs.length() == 0) {
            throw new ParserException("Could not parse abs from:\n" + str + "\n For file: " + medlineRecordFile.getName());
        }

        return new Medline.Record(pmid, null, null, null, null, abs, null, keywords);
    }

    public List parse(String sourcepath, InputStream is) throws Exception {
        return unmodlist(parseMedline(sourcepath, is));
    }

    private Medline.Record parseRecord_endnote(final List lines) throws ParserException {
        Set keywords = null;
        String author = null;
        String year = null;
        String title = null;
        String journal = null;
        String accession_number = null;
        String abs = null;
        String url = null;

        //log.debug("# of lines in record: " + lines.size());
        /*
        if (lines.size() < 10) {
            log.debug("Ignoring for: " + lines);
            return null;
        }
        */

        for (int i = 0; i < lines.size(); i++) {
            String s = lines.get(i).toString();
            int lineNum = i + 1;

            if (s.equalsIgnoreCase("Journal Article")) {
                continue;
            }

            if (s.startsWith(Headers.EndNote.AUTHOR)) {
                author = _setField(s, lineNum);
            } else if (s.startsWith(Headers.EndNote.YEAR)) {
                year = _setField(s, lineNum);
            } else if (s.startsWith(Headers.EndNote.TITLE)) {
                title = _setField(s, lineNum);
            } else if (s.startsWith(Headers.EndNote.JOURNAL)) {
                journal = _setField(s, lineNum);
            } else if (s.startsWith(Headers.EndNote.ACCESSION_NUMBER)) {
                accession_number = _setField(s, lineNum);
            } else if (s.startsWith(Headers.EndNote.ABSTRACT)) {
                abs = _setField(s, lineNum);
            } else if (s.startsWith(Headers.EndNote.URL)) {
                url = _setField(s, lineNum);
            } else if (s.startsWith(Headers.EndNote.KEYWORDS)) {
                List kwords = new ArrayList();
                for (int k = i + 1; k < lines.size(); k++) {
                    String str = lines.get(k).toString();
                    if (str.indexOf(':') == -1) {
                        if (str.indexOf("/*") != -1) {
                            kwords.addAll(ParseUtils.string2stringsSet(str, "/*", false));
                        } else {
                            kwords.addAll(ParseUtils.string2stringsSet(str, ",", false));
                        }
                    } else {
                        break;
                    }
                }

                keywords = _setField((String[]) kwords.toArray(new String[kwords.size()]));
            } else {
                // do nothing
            }

        }

        //log.debug("Creating recd: " + accession_number);

        return new Medline.Record(accession_number, author, year, title, journal, abs, url, keywords);
    }

    private static String _setField(String s, int lineNum) throws ParserException {
        // Accession Number: 12631705
        int index = s.indexOf(':');
        if (index == -1) {
            throw new ParserException("Cannot parse: >" + s + "<" + " lineNum: " + lineNum);
        }

        String sub = s.substring(index + 1, s.length());
        return sub.trim();
    }

    static String[] ignore_keywords = new String[]{"U.S. Gov't", "Non-U.S. Gov't"};

    private static Set _setField(String[] ss) throws ParserException {
        Set set = new HashSet();
        for (int i = 0; i < ss.length; i++) {
            int index = ss[i].indexOf(':');
            if (index != -1) {
                throw new ParserException(": not allowed here: " + ss[i]);
            }

            boolean ignore = false;
            if (ss[i].startsWith("Research")) {
                ignore = true;
                // ignore these keywords
            }

            for (int t = 0; t < ignore_keywords.length; t++) {
                if (ss[i].indexOf(ignore_keywords[t]) != -1) {
                    ignore = true;
                    break;
                }
            }


            if (!ignore) {
                String s = ss[i].trim();
                if (s.startsWith("*") || s.startsWith("/")) {
                    s = s.substring(1, s.length());
                }
                s = s.replace('*', ' ');
                s = s.trim();
                set.add(s);
            }
        }

        return set;
    }

}    // End of class MedlineParser
