/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.vdb.sampledb.SampleAnnot;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Parses a gct formatted dataset -- similar to dataframe except that formatted
 * so that legacy gct data is accepted
 * <p/>
 * Format:
 * <p/>
 * #1.2
 * 12488	41
 * Name	Description	col_0 col_1 col_2 ...
 * row_o
 * row_1
 * ...
 * <p/>
 * NO AP calls
 */

// @todo make protected eta al as a regular parser

public class MissingValueDatasetParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public MissingValueDatasetParser() {
        super(Dataset.class);
    }

    /**
     * Export a Dataset to file in gct format
     * Only works with Datasets
     *
     * @see "Above for format"
     */
    public void export(PersistentObject pob, File file) throws Exception {

        PrintWriter pw = startExport(pob, file);

        Dataset ds = (Dataset) pob;

        pw.println("#1.2"); // not sure what the # means, but give the people what they want
        pw.println(ds.getNumRow() + "\t" + ds.getNumCol());
        pw.print(Constants.NAME + "\t" + Constants.DESCRIPTION + "\t");

        for (int i = 0; i < ds.getNumCol(); i++) {
            pw.print(ds.getColumnName(i));
            if (i != ds.getNumCol() - 1) {
                pw.print('\t');
            }
        }

        pw.println();

        for (int r = 0; r < ds.getNumRow(); r++) {
            pw.print(ds.getRowName(r));
            pw.print('\t');
            //out.print(ds.getR);
            pw.print(Constants.NA + "\t");
            pw.println(ds.getRow(r).toString('\t'));
        }

        pw.close();

        doneExport();
    }    // End export

    /**
     * @param name
     * @param s
     * @return
     * @throws Exception
     */
    public List parse(String objname, String s) throws Exception {

        StringReader sr = new StringReader(s);
        BufferedReader bin = new BufferedReader(sr);
        return _parse(objname, bin);

    }

    /**
     * Parse a Dataset from memory
     *
     * @param s
     * @return
     * @throws Exception
     */
    public List parse(String s) throws Exception {
        String name = NamingConventions.generateName(Constants.RES);
        return parse(name, s);
    }

    /**
     * @returns 1 Dataset object
     * Always DatasetAnnotation object produced , but if underlying df has none then na is used
     * @see above for format
     */
    public List parse(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        return _parse(sourcepath, bin);

    }

    public Dataset parseDataset(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        return (Dataset) _parse(sourcepath, bin).get(0);

    }

    /// does the real parsing
    // expects the bin to be untouched
    private List _parse(String objname, BufferedReader bin) throws Exception {

        String currLine = nextLine(bin);

        // 1st  non-empty, non-comment line is numrows and numcols
        int[] nstuff = ParseUtils.string2ints(currLine, " \t");
        if (nstuff.length != 2) {
            throw new ParserException("Gct file with bad row/col info on line: " + currLine);
        }

        int nrows = nstuff[0];
        int ncols = nstuff[1];

        // First 2 fields name and desc are to be ignored
        currLine = nextLine(bin);
        //System.out.println(currLine);
        List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

        colnames.remove(0);                                 // first elem is always nonesense
        colnames.remove(0);

        if (colnames.size() != ncols) {
            throw new ParserException("Bad gct format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
        }

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List lines = new ArrayList();

        currLine = nextLine(bin);

        // save all rows so that we can determine how many rows exist
        while (currLine != null) {
            lines.add(currLine);
            currLine = nextLine(bin);
        }

        if (lines.size() != nrows) {
            throw new ParserException("Bad gct format -- exepcted nrows from specification on header line: " + nrows + " but found in data: " + lines.size());
        }

        bin.close();

        return _parseHasDesc(objname, lines, colnames);
    }

    private List _parseHasDesc(String objName, List lines, List colnames) throws Exception {
        Matrix matrix = new Matrix(lines.size(), colnames.size());
        List rowNames = new ArrayList();
        List rowDescs = new ArrayList();

        for (int i = 0; i < lines.size(); i++) {
            String currLine = (String) lines.get(i);
            List fields = ParseUtils.string2stringsList(currLine, "\t"); // spaces allowed in name & desc field, so dont tokenize on them

            if (fields.size() != colnames.size() + 1 + 1) {
                throw new ParserException("Bad format - expect ncols: " + (colnames.size() + 1 + 1)
                        + " but found: " + fields.size() + " on line: "
                        + currLine + "\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
            }

            String rowName = (String) fields.get(0);
            String desc = (String) fields.get(1);
            rowNames.add(rowName);
            rowDescs.add(desc);

            for (int f = 2; f < fields.size(); f++) {
                String s = fields.get(f).toString();
                if (s.equalsIgnoreCase("na")) {
                    matrix.setElement(i, f - 2, Float.NaN);
                } else {
                    matrix.setElement(i, f - 2, Float.parseFloat(s));
                }
            }
        }


        final FeatureAnnot fann = new FeatureAnnotImpl(objName, rowNames, rowDescs);
        final SampleAnnot sann = new SampleAnnotImpl(objName, colnames, null);
        DefaultDataset ds = new DefaultDataset(objName, matrix, rowNames, colnames, true, new AnnotImpl(fann, sann));
        ds.addComment(fComment.toString());
        fann.addComment(fComment.toString());

        doneImport();
        return unmodlist(new PersistentObject[]{ds, fann});
    }

    static class Entry {
        String name;
        String desc;
        float[] floats;
    }

}    // End of class MissingValueDatasetParser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
