/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.*;
import edu.mit.broad.genome.alg.AlgUtils;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.utils.FileUtils;
import org.apache.log4j.Logger;

import java.awt.*;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.List;


/**
 * public static utilities to aid in parsing
 * <p/>
 * <bold> Implementation Notes </bold><br>
 * There are a range of possible exceptions that can occur duing parsing.
 * Rather than throw an error-specific exception on encountering an error,
 * a ParseException is thrown that captures the base exception(s).
 * <p/>
 * <confirm-this>
 * In addition,
 * during parsing, often runtime (unchecked) exceptions are show-stoppers. These
 * too are caught and parcelled into ParseException.
 * </confirm-this>
 *
 * @author Michael Angelo
 * @author Aravind Subramanian
 * @version 1.0
 */
public class ParseUtils {

    private static final char DEFAULT_FIELD_SEP_CSV = ',';

    /**
     * NamingConventions delimiters - <pre>" \t\n,;:"</pre>
     */
    public final static String DEFAULT_DELIMS = " \t\n,;:";
    public static final String COLOR_PKG = "java.awt.Color";

    /**
     * For logging support
     */
    private final static Logger klog = XLogger.getLogger(ParseUtils.class);
    public static final String NULL_CHAR = "*";
    /**
     * should occur in the first these many lines (file might be large)
     */
    public static final int MAX_SEARCH_LINES = 20;


    public static void checkColumnNames(String[] got, String[] expected) {

        if (got == null) {
            throw new IllegalArgumentException("Parameter got cannot be null");
        }

        if (expected == null) {
            throw new IllegalArgumentException("Parameter expected cannot be null");
        }

        if (got.length < expected.length) {
            Set expectedButNotFound = AlgUtils.subtract(expected, got);
            System.out.println("\nThe expected ones NOT got were\n" + expectedButNotFound);
            Printf.out(got);
            Printf.out(expected);
            throw new MismatchedSizeException("got", got.length, "expected", expected.length);
        } else if (got.length > expected.length) {
            Set foundButNotExpected = AlgUtils.subtract(got, expected);
            System.out.println("\nThe UNexpected ones got were\n" + foundButNotExpected);
            throw new MismatchedSizeException("got", got.length, "expected", expected.length);
        }

        for (int i = 0; i < got.length; i++) {
            if (!got[i].equalsIgnoreCase(expected[i])) {
                throw new IllegalArgumentException("Column names are mismatched at: " + i + " " + got[i] + " and " + expected[i]);
            }
        }

    }

    public static void checkColumnNames(StringDataframe sdf, String[] expected) {
        if (sdf == null) {
            throw new IllegalArgumentException("Parameter sdf cannot be null");
        }

        checkColumnNames(sdf.getColumnNamesArray(), expected);
    }


    /**
     * Reads contents of file into a List
     * <p/>
     * Comment lines are ignored or dnot depending on ignorecomments flag
     * All other lines except empty lines are read
     * Whitespace is trimmed
     * Empty lines are ignored
     */
    public List read(File file, boolean ignorecomments) throws Exception {

        List list = new ArrayList();
        BufferedReader buf = new BufferedReader(new FileReader(file));
        String line;

        line = ParseUtils.nextLine(buf);

        while (line != null) {
            list.add(line);

            line = ParseUtils.nextLine(buf);
        }

        buf.close();

        return list;
    }


    /**
     * the last token
     * Example: foo.baz -> baz
     *
     * @param s
     * @param delimiters
     * @return
     */
    public static String getLastToken(String s, String delimiters) {

        StringTokenizer tok = new StringTokenizer(s, delimiters);

        while (tok.hasMoreTokens()) {
            s = tok.nextToken();
        }

        return s;
    }

    /**
     * First token
     * Example: foo.baz -> foo
     *
     * @param s
     * @param delimiters
     * @return
     */
    public static String getFirstToken(String s, String delimiters) {

        StringTokenizer tok = new StringTokenizer(s, delimiters);

        return tok.nextToken();
    }

    /**
     * Splits specified String into an array of integers. The String is tokenized
     * on the ParseUtils.DELIMITERS
     *
     * @param s String to split.
     * @return int[] integers found in String
     */
    public static int[] string2ints(String s)
            throws NumberFormatException, IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        StringTokenizer tok = new StringTokenizer(s, DEFAULT_DELIMS);

        return _doIntParse(tok);
    }

    public static float[] string2floats(String s)
            throws NumberFormatException, IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        StringTokenizer tok = new StringTokenizer(s, DEFAULT_DELIMS);

        return _doFloatParse(tok);
    }

    public static Float[] string2Floats(String s)
            throws NumberFormatException, IllegalArgumentException {

        float[] floats = string2floats(s);
        Float[] f = new Float[floats.length];
        for (int i = 0; i < floats.length; i++) {
            f[i] = new Float(floats[i]);
        }

        return f;
    }

    public static Integer[] string2Ints(String s)
            throws NumberFormatException, IllegalArgumentException {

        int[] ints = string2ints(s);
        Integer[] bigi = new Integer[ints.length];
        for (int i = 0; i < ints.length; i++) {
            bigi[i] = new Integer(ints[i]);
        }

        return bigi;
    }

    public static int[] string2ints(String s, String delims)
            throws NumberFormatException, IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        StringTokenizer tok = new StringTokenizer(s, delims);

        return _doIntParse(tok);
    }

    public static int[] string2ints(String s, char delim)
            throws NumberFormatException, IllegalArgumentException {

        return string2ints(s, Character.toString(delim));
    }

    public static float[] string2floats(String s, char delim) {
        return string2floats(s, Character.toString(delim));
    }


    public static float[] string2floats(String s, String delims)
            throws NumberFormatException, IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        //System.out.println("##### delim>" + delims + "<");
        StringTokenizer tok = new StringTokenizer(s, delims);
        return _doFloatParse(tok);
    }

    public static Vector string2Vector(String s, char delim) {
        return new Vector(string2floats(s, Character.toString(delim)));
    }

    public static Vector string2Vector(String s, String delims) {
        return new Vector(string2floats(s, delims));
    }

    private static int[] _doIntParse(StringTokenizer tok) throws NumberFormatException {

        int[] ret = new int[tok.countTokens()];
        int i = 0;

        String curr;
        while (tok.hasMoreTokens()) {
            curr = tok.nextToken();
            // doing it the long way so that on error, the exact element that
            // caused it is specified in the exception message
            ret[i] = Integer.parseInt(curr);
            i++;
        }

        return ret;
    }

    private static float[] _doFloatParse(StringTokenizer tok) throws NumberFormatException {

        float[] ret = new float[tok.countTokens()];
        //klog.debug("# of tokens: " + tok.countTokens());

        int i = 0;

        String curr;
        while (tok.hasMoreTokens()) {
            curr = tok.nextToken();
            // doing it the long way so that on error, the exact element that
            // caused it is specified in the exception message
            ret[i] = Float.parseFloat(curr);
            i++;
        }

        return ret;
    }

    /**
     * Splits specified String into an array of Strings. The String is tokenized
     * on the ParseUtils.DELIMITERS
     *
     * @param s String to split.
     * @return String[] Strings found in String
     */
    public static String[] string2strings(String s, boolean useNullonMagicNullWord) throws IllegalArgumentException {
        return string2strings(s, DEFAULT_DELIMS, useNullonMagicNullWord);
    }

    public static String[] string2strings(String s, String delim, String[] magicNullWords) throws IllegalArgumentException {
        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        StringTokenizer tok = new StringTokenizer(s, delim);
        String[] ret = new String[tok.countTokens()];
        int i = 0;

        while (tok.hasMoreTokens()) {
            ret[i] = _magic(tok.nextToken().trim(), magicNullWords, true);
            i++;
        }

        return ret;
    }

    public static String[] string2strings(final String s,
                                          final String delim,
                                          final boolean useNullonMagicNullWord) throws IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot parse on null String");
        }

        /*
        s = s.trim();
        if (s.length() == 0) {
            return new String[]{};
        }
        */

        StringTokenizer tok = new StringTokenizer(s, delim);
        String[] ret = new String[tok.countTokens()];
        int i = 0;

        while (tok.hasMoreTokens()) {
            ret[i] = _magic(tok.nextToken().trim(), useNullonMagicNullWord);
            i++;
        }

        return ret;
    }

    public static String[] string2stringsV2(String s) throws IllegalArgumentException {
        List ret = string2stringsV2_list(s);
        return (String[]) ret.toArray(new String[ret.size()]);
    }

    // double tabs are tolerated and no need for the NULL thing
    public static List string2stringsV2_list(String s) throws IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        s = s.trim(); // no tabs before or after

        StringTokenizer tok = new StringTokenizer(s, "\t", true); // note including the delim in rets
        List ret = new ArrayList();

        //System.out.println("# " + tok.countTokens());
        String prev = null;
        while (tok.hasMoreTokens()) {
            String curr = tok.nextToken(); // dont trim!
            if (!curr.equals("\t")) {
                ret.add(curr);
            }

            if (curr.equals(prev)) { // 2 consecutive tabs
                ret.add(""); //empty field
            }

            prev = curr;
        }

        return ret;
    }

    public static Set string2stringsV2_set(String s) throws IllegalArgumentException {
        return new HashSet(string2stringsV2_list(s));
    }

    public static List string2stringsList(String s) throws IllegalArgumentException {
        return string2stringsList(s, DEFAULT_DELIMS);
    }

    public static List string2stringsList(String s, String delim) throws IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        StringTokenizer tok = new StringTokenizer(s, delim);
        List ret = new ArrayList(tok.countTokens());

        while (tok.hasMoreTokens()) {
            ret.add(tok.nextToken().trim());
        }

        return ret;
    }

    public static Set string2stringsSet(String s, String delim, boolean respectNullMagicWord) throws IllegalArgumentException {

        if (null == s) {
            return new HashSet();
        }

        StringTokenizer tok = new StringTokenizer(s, delim);
        Set ret = new HashSet();

        while (tok.hasMoreTokens()) {
            String elem = tok.nextToken().trim();
            elem = _magic(elem, respectNullMagicWord);
            if (elem != null) {
                ret.add(elem);
            }
        }

        return ret;
    }

    private static final String[] MAGIC_NULLS = new String[]{Constants.NULL, Constants.NA, Constants.HYPHEN, Headers.AFFX_NULL};

    private static String _magic(String s, boolean respectNullMagicWord) {
        return _magic(s, MAGIC_NULLS, respectNullMagicWord);
    }

    /**
     * @noinspection ReturnOfNull
     */
    private static String _magic(String s, String[] magicNulls, boolean respectNullMagicWord) {

        /*
        if (s == null) {
            throw new IllegalArgumentException("Parameter s cannot be null");
        }
        */

        if (respectNullMagicWord) {
            for (int i = 0; i < magicNulls.length; i++) {
                if (s.trim().equalsIgnoreCase(magicNulls[i])) {
                    return null;
                }
            }
        }

        return s;
    }

    /**
     * Searches specified string for presence of a
     * "oft-used" delimiter.
     *
     * @return '\0\ if no hits
     *         NamingConventions delimiters are: {'\t', ' ', ',', '\n'}
     *         From mangelo code base.
     */
    public static char getDelim(final String aStr) {

        char[] theDelims = {'\t', ' ', ',', '\n'};
        char theDelim = '\0';

        int curInd = -1;
        int ind = 0;

        // Check for possible delimiters of the class assignments
        while ((curInd == -1) && (ind < theDelims.length)) {
            theDelim = theDelims[ind];
            curInd = aStr.indexOf(theDelim, 0);

            ind++;
        }

        if (curInd == -1) {
            theDelim = '\0';
        }

        return theDelim;
    }

    /**
     * Splits a String of integers by specified delimiter and feeds the results
     * (the integers) into specified integer array.
     * <p/>
     * This is from ma's code base - hmm why not just return the int[]??
     *
     * @return The number of integers
     */
    public static int splitIntegers(String aStr, char aDelim, int[] aInts) throws Exception {

        //klog.debug("Splitting: " + aStr + " with delim: " + aDelim + " into ints: " + aInts.length);

        int curInd = 0, prevInd = 0;
        int i = 0, theLength = aStr.length();

        while (curInd < theLength) {
            curInd = aStr.indexOf(aDelim, prevInd);

            if (curInd < 0) {
                curInd = theLength;
            }

            String subStr = aStr.substring(prevInd, curInd).trim();

            //              System.out.println("Substr[" + i +"]: " + subStr);
            aInts[i] = Integer.valueOf(subStr).intValue();

            i++;

            prevInd = curInd + 1;

            if (i == aInts.length) {
                break;
            }
        }


        return i;
    }

    /**
     * Converts a char to a String
     */
    public static String charToString(char c) {

        char[] tmp = new char[1];

        tmp[0] = c;

        return new String(tmp);
    }

    /**
     * Converts a JDBC connection string of the form:
     * jdbc.url=jdbc:oracle:thin:@Holley.broad.mit.edu:1521:sandbox
     * to:
     * Holley.broad.mit.edu:1521:sandbox
     * (everything after the @)
     * The idea being to convert a JDBC sytring into a (smaller) meaningful name.
     * Returns param unchanged if it cant extract.
     */
    public static String jdbcUrl2Name(String url) {

        StringTokenizer tok = new StringTokenizer(url, "@");

        if (tok.countTokens() == 2) {
            tok.nextToken();
            return tok.nextToken().trim();
        } else {
            return url;
        }
    }

    /**
     * Color.toString() gives -> "java.awt.Color[r=255,g=102,b=102]"
     * This method strips off the java.awt.Color part and instead gives -> "[r=255,g=102,b=102]"
     * which makes the display a bit more user-friendly (and shorter).
     */
    public static String color2string(Color color) {

        String s = color.toString();

        return s.substring(COLOR_PKG.length());
    }

    public static List getUniqueTokens(StringTokenizer tok) {

        List uniqs = new ArrayList();
        while (tok.hasMoreElements()) {
            String t = tok.nextToken().trim();
            if (!uniqs.contains(t)) {
                uniqs.add(t);
            }
        }

        return uniqs;
    }

    public static void ensureEquals(String[] a, String[] b) {

        if (a.length != b.length) {
            throw new IllegalArgumentException("Mismatched lengths a: " + a.length + " b: " + b.length);
        }

        for (int i = 0; i < a.length; i++) {
            if (!a[i].equalsIgnoreCase(b[i])) {
                throw new IllegalArgumentException("Mismatched entries at: " + i + " a: " + a[i] + " and b: " + b[i]);
            }
        }
    }

    public static void ensureEquals(List a, String[] b) {

        if (a.size() != b.length) {
            throw new IllegalArgumentException("Mismatched lengths a: " + a.size() + " b: " + b.length);
        }

        for (int i = 0; i < a.size(); i++) {
            if (!a.get(i).toString().equalsIgnoreCase(b[i])) {
                throw new IllegalArgumentException("Mismatched entries at: " + i + " a: " + a.get(i) + " and b: " + b[i]);
            }
        }
    }

    public static void ensureHeader(File f, String[] expHeaders) throws IOException {
        ensureEquals(string2strings(getFirstNonWsLine(f), false), expHeaders);
    }

    public static BufferedReader buf(File file) throws IOException {
        if (file == null) {
            throw new IllegalArgumentException("Parameter file cannot be null");
        }

        return new BufferedReader(new FileReader(file));
    }

    /**
     * Convenience method to read in a ffn into an ArrayList Each vector
     * element corresponds to a single line (in order) from the ffn. (ffn->
     * file of file names) Blank lines in file are ignored Leading and trailing
     * whitespace is trimmed
     *
     * @param aFile Description of the Parameter
     * @return Description of the Return Value
     * @throws IOException Description of the Exception
     */
    public static List readFfn(File aFile) throws IOException {

        BufferedReader buf = new BufferedReader(new FileReader(aFile));
        String line;
        ArrayList lines = new ArrayList();

        line = nextLine(buf);

        while (line != null) {
            if (lines.contains(line) == false) {
                lines.add(line);
            }
            line = nextLine(buf);
        }

        buf.close();

        return lines;
    }

    /**
     * Convenience method that reads a properties file and returns just the
     * values as an object array.
     *
     * @param file Description of the Parameter
     * @return The values value
     * @throws IOException Description of the Exception
     */
    public static Object[] getValues(File file) throws IOException {

        Properties prop = new Properties();
        FileInputStream is = new FileInputStream(file);

        prop.load(is);

        return prop.values().toArray();
    }

    /**
     * Convenience method that reads a properties file and returns just the
     * keys as an object array.
     *
     * @param file Description of the Parameter
     * @return The keys value
     * @throws IOException Description of the Exception
     */
    public static Object[] getKeys(File file) throws IOException {

        Properties prop = new Properties();
        FileInputStream is = new FileInputStream(file);

        prop.load(is);

        return prop.keySet().toArray();
    }

    public static Map readKeyVal_consolidate_vals_into_list(final File file,
                                                            final boolean containsHeaderLine,
                                                            final boolean enforceValueMustExist) throws IOException {


        final BufferedReader buf = new BufferedReader(new FileReader(file));
        final Map keyValListMap = new HashMap();

        String line = nextLine(buf);

        if (containsHeaderLine) {
            line = nextLine(buf); // ignore the first line
        }

        int lineNum = 0;
        //List duplLines = new ArrayList();
        while (line != null) {
            //System.out.println(">>" + line + "<<");
            StringTokenizer tok = new StringTokenizer(line, "\t");
            String key;
            String val = null;
            if (tok.countTokens() == 2) {
                key = tok.nextToken().trim();
                val = tok.nextToken().trim();
            } else if (tok.countTokens() == 1) {
                key = tok.nextToken().trim();
            } else {
                throw new IOException("Bad line format: " + line + " # tokens: " + tok.countTokens() + " line: " + lineNum);
            }

            if (enforceValueMustExist) {
                if (val == null || val.length() == 0) {
                    throw new RuntimeException("Value must exist. Missing on line: " + lineNum + " val: " + val);
                }
            }

            if (key.length() == 0) {
                throw new RuntimeException("Empty key on line: " + lineNum);
            }

            Object curr = keyValListMap.get(key);
            if (curr == null) {
                curr = new ArrayList();
            }

            List curr_list = (List) curr;
            if (!curr_list.contains(val)) { // @note how duplicate key-value mappings are not used
                curr_list.add(val);
            }

            keyValListMap.put(key, curr_list);

            line = nextLine(buf);
            lineNum++;
        }

        buf.close();

        if (lineNum == 0) {
            throw new IllegalArgumentException("Empty input stream!! ");
        }

        return keyValListMap;
    }

    public static Properties readKeyVal(final File file,
                                        final boolean containsHeaderLine,
                                        final boolean enforceValueMustExist,
                                        final boolean enforceNonRepeatedKeys) throws IOException {
        return readKeyVal(new BufferedReader(new FileReader(file)), containsHeaderLine, enforceValueMustExist, enforceNonRepeatedKeys);
    }

    public static Properties readKeyVal_from_xls(final File xlsFile,
                                                 final boolean enforceValueMustExist,
                                                 final boolean enforceNonRepeatedKeys) throws Exception {

        StringDataframe sdf = MiscParsers.excel2sdf(xlsFile);

        final Properties prp = new Properties();

        List duplLines = new ArrayList();

        for (int r = 0; r < sdf.getNumRow(); r++) {
            String key = sdf.getRowName(r);
            String val = sdf.getElement(r, 0);

            if (enforceValueMustExist) {
                if (val == null || val.length() == 0) {
                    throw new RuntimeException("Value must exist. Missing on line: " + r + " val: " + val);
                }
            }

            if (key.length() == 0) {
                throw new RuntimeException("Empty key on line: " + r);
            }

            if (prp.containsKey(key) && enforceNonRepeatedKeys) {
                duplLines.add(key + "\t" + val);
            }

            //System.out.println(key + " " + val);
            prp.setProperty(key, val);
        }


        if (!duplLines.isEmpty() && enforceNonRepeatedKeys) {
            StringBuffer sbuf = new StringBuffer();
            for (int i = 0; i < duplLines.size(); i++) {
                sbuf.append(duplLines.get(i)).append('\n');
            }
            throw new RuntimeException("There are repeated keys:" + duplLines.size() + "\n" + duplLines);
        }

        return prp;
    }

    public static Properties readKeyVal(final URL url,
                                        final boolean containsHeaderLine,
                                        final boolean enforceValueMustExist,
                                        final boolean enforceNonRepeatedKeys) throws IOException {
        InputStreamReader isr = new InputStreamReader(url.openStream());
        return readKeyVal(new BufferedReader(isr), containsHeaderLine, enforceValueMustExist, enforceNonRepeatedKeys);
    }

    public static Properties readKeyVal(final String filepathORurl,
                                        final boolean containsHeaderLine,
                                        final boolean enforceValueMustExist,
                                        final boolean enforceNonRepeatedKeys) throws IOException {
        BufferedReader buf = FileUtils.toBufferedReader(filepathORurl);
        return readKeyVal(buf, containsHeaderLine, enforceValueMustExist, enforceNonRepeatedKeys);
    }

    /**
     * Convenience method to read in a file with key=value pairs
     * into a hashtable
     * Each element corresponds to a single line (NOT in order)
     * from the file.
     * Blank lines in file are ignored
     * Leading and trailing whitespace is trimmed
     * Optionally lines beginning with Constants.COMMENT_CHAR are ignored
     * key1\tval1
     * key2\tval2
     */
    public static Properties readKeyVal(final BufferedReader buf,
                                        final boolean containsHeaderLine,
                                        final boolean enforceValueMustExist,
                                        final boolean enforceNonRepeatedKeys) throws IOException {

        String line;
        final Properties prp = new Properties();

        //klog.debug("Reading from file: " + file);

        line = nextLine(buf);

        if (containsHeaderLine) {
            line = nextLine(buf);
        }

        int lineNum = 0;
        List duplLines = new ArrayList();
        while (line != null) {
            //System.out.println(">>" + line + "<<");
            StringTokenizer tok = new StringTokenizer(line, "\t");
            String key;
            String val = null;
            if (tok.countTokens() == 2) {
                key = tok.nextToken().trim();
                val = tok.nextToken().trim();
            } else if (tok.countTokens() == 1) {
                key = tok.nextToken().trim();

            } else {
                throw new IOException("Bad line format: " + line + " # tokens: " + tok.countTokens() + " line: " + lineNum);
            }

            if (enforceValueMustExist) {
                if (val == null || val.length() == 0) {
                    throw new RuntimeException("Value must exist. Missing on line: " + lineNum + " val: " + val);
                }
            }

            if (key.length() == 0) {
                throw new RuntimeException("Empty key on line: " + lineNum);
            }

            if (prp.containsKey(key) && enforceNonRepeatedKeys) {
                duplLines.add(line);
            }

            //System.out.println(key + " " + val);
            prp.setProperty(key, val);
            line = nextLine(buf);
            lineNum++;
        }

        buf.close();

        if (!duplLines.isEmpty() && enforceNonRepeatedKeys) {
            StringBuffer sbuf = new StringBuffer();
            for (int i = 0; i < duplLines.size(); i++) {
                sbuf.append(duplLines.get(i)).append('\n');
            }
            throw new RuntimeException("There are repeated keys:" + duplLines.size() + "\n" + duplLines);
        }

        if (lineNum == 0) {
            throw new IllegalArgumentException("Empty input stream!! ");
        }

        return prp;
    }

    public static List readKeyVal_listarr(File file, boolean containsHeaderLine, boolean enforceValueMustExist) throws IOException {
        return readKeyVal_listarr(new BufferedReader(new FileReader(file)), containsHeaderLine, enforceValueMustExist);
    }

    public static List readKeyVal_listarr(BufferedReader buf, boolean containsHeaderLine,
                                          boolean enforceValueMustExist) throws IOException {

        String line;
        List key_vals = new ArrayList();

        //klog.debug("Reading from file: " + file);

        line = nextLine(buf);

        if (containsHeaderLine) {
            line = nextLine(buf);
        }

        int lineNum = 0;
        //List duplLines = new ArrayList();
        while (line != null) {
            //System.out.println(">>" + line + "<<");
            StringTokenizer tok = new StringTokenizer(line, "\t");
            String key;
            String val = null;
            if (tok.countTokens() == 2) {
                key = tok.nextToken().trim();
                val = tok.nextToken().trim();
            } else if (tok.countTokens() == 1) {
                key = tok.nextToken().trim();

            } else {
                throw new IOException("Bad line format: " + line + " # tokens: " + tok.countTokens() + " line: " + lineNum);
            }

            if (enforceValueMustExist) {
                if (val == null || val.length() == 0) {
                    throw new RuntimeException("Value must exist. Missing on line: " + lineNum + " val: " + val);
                }
            }

            if (key.length() == 0) {
                throw new RuntimeException("Empty key on line: " + lineNum);
            }

            //System.out.println(key + " " + val);
            key_vals.add(new String[]{key, val});
            line = nextLine(buf);
            lineNum++;
        }

        buf.close();

        if (lineNum == 0) {
            throw new IllegalArgumentException("Empty input stream!! ");
        }

        return key_vals;
    }

    public static Properties readKeyValReqd(String filepathorurl) throws IOException {
        return readKeyValReqd(FileUtils.toBufferedReader(filepathorurl));
    }

    public static Properties readKeyValReqd(File file) throws IOException {
        return readKeyValReqd(new BufferedReader(new FileReader(file)));
    }

    public static Properties readKeyValReqd(URL url) throws IOException {
        InputStreamReader isr = new InputStreamReader(url.openStream());
        return readKeyValReqd(new BufferedReader(isr));
    }

    /**
     * same as readKeyVal()
     * except that all keys are required to have vals
     * So an exception is thrown on missing values. Instead "" is used
     *
     * @param file
     * @return
     * @throws IOException
     */
    public static Properties readKeyValReqd(BufferedReader buf) throws IOException {

        String line;
        Properties prp = new Properties();

        line = nextLine(buf);

        while (line != null) {
            StringTokenizer tok = new StringTokenizer(line, "=\t");

            if (tok.countTokens() != 2) {
                throw new IOException("Bad line format: " + line);
            }

            prp.setProperty(tok.nextToken(), tok.nextToken());

            line = nextLine(buf);
        }

        buf.close();

        return prp;
    }

    public static List readKeyValReqdIntoList(File file) throws IOException {

        return readKeyValReqdIntoList(new BufferedReader(new FileReader(file)));

    }

    public static List readKeyValReqdIntoList(URL url) throws IOException {

        InputStreamReader isr = new InputStreamReader(url.openStream());
        return readKeyValReqdIntoList(new BufferedReader(isr));

    }

    /**
     * same as readKeyVal()
     * except that all keys are required to have vals
     * So an exception is thrown on missing values
     * List with String[]{2} objects
     *
     * @param file
     * @return
     * @throws IOException
     */
    public static List readKeyValReqdIntoList(BufferedReader buf) throws IOException {

        String line;
        List list = new ArrayList();

        line = nextLine(buf);

        while (line != null) {
            StringTokenizer tok = new StringTokenizer(line, "=\t");

            if (tok.countTokens() != 2) {
                throw new IOException("Bad line format: " + line);
            }

            list.add(new String[]{tok.nextToken(), tok.nextToken()});
            line = nextLine(buf);
        }

        buf.close();

        return list;
    }

    /**
     * @return specified column from tab delimited file
     *         colns numbered from 0 to n-1
     */
    public static List readColumn(int coln, File file, boolean ignorecommentlines)
            throws IOException {

        BufferedReader buf = new BufferedReader(new FileReader(file));
        String line;
        List column = new ArrayList();

        while ((line = buf.readLine()) != null) {

            //System.out.println(line);
            line = line.trim();

            if (line.length() == 0) {
                continue;
            }

            if ((ignorecommentlines) && (line.startsWith(Constants.COMMENT_CHAR))) {
                continue;
            }

            StringTokenizer tok = new StringTokenizer(line, "=\t");

            if (tok.countTokens() < (coln + 1)) {
                throw new IOException("Bad line format, too few columns " + line + "found="
                        + tok.countTokens() + " expecting: " + (coln + 1));
            } else {
                for (int i = 0; i < coln; i++) {
                    tok.nextToken();
                }

                column.add(tok.nextToken());
            }
        }

        buf.close();

        return column;
    }

    public static List readTabularFile(File file, boolean allowNullColumns) throws IOException {
        return readTabular(file, allowNullColumns, "\t");
    }

    public static List readTabularFile(InputStream is, boolean allowNullColumns) throws IOException {
        return readTabular(is, allowNullColumns, "\t");
    }

    /**
     * @param file
     * @param allowNullColumns
     * @param delims
     * @return List with elements as String[]. Each string[] is of length as many columns as there
     *         are in each line. Usually identical length unless rows are unequal in size.
     * @throws IOException
     */
    public static List readTabular(File file, boolean allowNullColumns, String delims)
            throws IOException {

        return readTabular(new FileInputStream(file), allowNullColumns, delims);

    }

    public static List readTabular(InputStream is, boolean allowNullColumns, String delims)
            throws IOException {

        return readTabular(new BufferedReader(new InputStreamReader(is)), allowNullColumns, delims);
    }

    /**
     * @param buf
     * @param allowNullFields
     * @param delims
     * @return an array list with
     * @throws IOException
     */
    public static List readTabular(final BufferedReader buf, final boolean allowNullFields, final String delims)
            throws IOException {

        String line;
        final List rows = new ArrayList();
        int colLen = -999;
        int lineNum = 0;

        while ((line = buf.readLine()) != null) {

            //System.out.println(line);
            line = line.trim();

            if (line.length() == 0) {
                continue;
            }

            if (line.startsWith(Constants.COMMENT_CHAR)) {
                continue;
            }

            lineNum++;

            StringTokenizer tok = new StringTokenizer(line, delims);
            String[] cols = new String[tok.countTokens()];

            if (colLen == -999) {
                colLen = tok.countTokens();
            } else {
                if (cols.length != colLen) {
                    throw new RuntimeException("Bad line: " + lineNum + " found ncols: "
                            + tok.countTokens() + " expected: " + colLen
                            + tabbed(tok));
                }
            }

            int len = tok.countTokens();

            for (int i = 0; i < len; i++) {
                String s = tok.nextToken();

                if (s.equals(NULL_CHAR)) {
                    s = null;
                }

                if ((!allowNullFields) && (s == null)) {
                    throw new RuntimeException("Empty cols set to not allowed but found empty column in line: " + lineNum
                            + " >" + line + "<");
                }

                cols[i] = s;
            }

            rows.add(cols);
        }

        buf.close();

        return rows;
    }

    private static String tabbed(StringTokenizer tok) {

        StringBuffer buf = new StringBuffer('\n').append(tok.countTokens()).append('\n');

        while (tok.hasMoreTokens()) {
            buf.append(">").append(tok.nextToken()).append("<\n");
        }

        return buf.toString();
    }

    /**
     * Matrix file is a file of numbers (floats) only
     * Comments and empty lines are allowed
     * Empty columns are NOT allowed
     */
    public static Matrix readMatrixFile(File file) throws IOException {

        List rows = readTabularFile(file, false);
        int ncols = ((String[]) rows.get(0)).length;
        Matrix matrix = new Matrix(rows.size(), ncols);

        for (int r = 0; r < rows.size(); r++) {
            String[] row = (String[]) rows.get(r);

            if (row.length != ncols) {
                throw new RuntimeException("Unequal column lengths: " + ncols + " and: "
                        + row.length);
            }

//Print.out(row);
            for (int c = 0; c < row.length; c++) {
                matrix.setElement(r, c, Float.parseFloat(row[c]));
            }
        }

        return matrix;
    }

    /**
     * Convenience method to slurp an entire file into a string.
     * Dont use with large files.
     */
    public static String slurp(File file, boolean ignoreanycomments) throws IOException {

        return slurp(_buf(file), ignoreanycomments);

    }

    public static String slurp(URL url, boolean ignoreanycomments) throws IOException {
        return slurp(_buf(url), ignoreanycomments);
    }

    public static String slurp(BufferedReader buf, boolean ignoreanycomments) throws IOException {

        String line;
        String s = "";

        while ((line = buf.readLine()) != null) {
            if ((ignoreanycomments) && (line.startsWith(Constants.COMMENT_CHAR))) {
                ;
            } else {
                s = s.concat(line).concat("\n");
            }
        }

        buf.close();

        return s;
    }

    public static java.util.Vector slurpIntoVector(File file, boolean ignoreanycomments)
            throws IOException {

        return slurpIntoVector(_buf(file), ignoreanycomments);

    }

    public static java.util.Vector slurpIntoVector(URL url, boolean ignoreanycomments)
            throws IOException {

        return slurpIntoVector(_buf(url), ignoreanycomments);

    }

    /**
     * Convenience method to slurp an entire file into a Vector.
     * Each line in file is a single String element in the vector
     * Order is maintained
     * Lines have newline stripped off.
     * Blank lines in file are ignored
     * Leading and trailing whitespace is trimmed
     * Dont use with large files.
     */
    public static java.util.Vector slurpIntoVector(BufferedReader buf, boolean ignoreanycomments)
            throws IOException {

        java.util.Vector vector = new java.util.Vector();
        String line;

        while ((line = buf.readLine()) != null) {
            line = line.trim();

            if (line.length() == 0) {
                continue;
            }

            if ((ignoreanycomments) && (line.startsWith(Constants.COMMENT_CHAR))) {
                continue;
            }

            vector.addElement(line);
        }

        buf.close();

        return vector;
    }

    /**
     * Maybe a bit faster than slurpFileIntoVector
     *
     * @see slurpFileIntoVector
     */
    public static List slurpIntoList(File file, boolean ignoreanycomments) throws IOException {
        return slurpIntoList(_buf(file), ignoreanycomments);
    }

    public static String[] slurpIntoStrings(File file, boolean ignoreanycomments) throws IOException {
        List list = slurpIntoList(file, ignoreanycomments);
        return (String[]) list.toArray(new String[list.size()]);
    }

    public static String[] slurpIntoStringsUnique(File file, boolean ignoreanycomments) throws IOException {
        String[] ss = slurpIntoStrings(file, ignoreanycomments);
        List list = new ArrayList(); // use list so that the order is preserved
        for (int i = 0; i < ss.length; i++) {
            if (list.indexOf(ss[i]) == -1) {
                list.add(ss[i]);
            }
        }

        return (String[]) list.toArray(new String[list.size()]);
    }

    public static List slurpIntoList(URL url, boolean ignoreanycomments) throws IOException {
        return slurpIntoList(_buf(url), ignoreanycomments);
    }

    public static Set slurpIntoSet(File file, boolean ignoreanycomments) throws IOException {
        return slurpIntoSet(_buf(file), ignoreanycomments);
    }

    public static Set slurpIntoSet(URL url, boolean ignoreanycomments) throws IOException {
        return slurpIntoSet(_buf(url), ignoreanycomments);
    }

    public static String[] slurpIntoArray(URL url, boolean ignoreanycomments) throws IOException {
        Set set = slurpIntoSet(_buf(url), ignoreanycomments);
        return (String[]) set.toArray(new String[set.size()]);
    }

    public static List slurpIntoList(BufferedReader buf, boolean ignoreanycomments) throws IOException {
        List list = new ArrayList();
        String line;

        while ((line = buf.readLine()) != null) {
            line = line.trim();

            if (line.length() == 0) {
                continue;
            }

            if ((ignoreanycomments) && (line.startsWith(Constants.COMMENT_CHAR))) {
                continue;
            }

            list.add(line);
        }

        buf.close();

        return list;
    }

    public static Set slurpIntoSet(BufferedReader buf, boolean ignoreanycomments) throws IOException {
        Set set = new HashSet();
        String line;

        while ((line = buf.readLine()) != null) {
            line = line.trim();

            if (line.length() == 0) {
                continue;
            }

            if ((ignoreanycomments) && (line.startsWith(Constants.COMMENT_CHAR))) {
                continue;
            }

            set.add(line);
        }

        buf.close();

        return set;
    }

    public static int countLines(File file, boolean ignoreblanklines) throws IOException {
        return countLines(_buf(file), ignoreblanklines);
    }

    public static int countLines(URL url, boolean ignoreblanklines) throws IOException {
        return countLines(_buf(url), ignoreblanklines);
    }

    public static int countLines(BufferedReader buf, boolean ignoreblanklines) throws IOException {

        int numberOfLines = 0;

        if (!ignoreblanklines) {
            while (buf.readLine() != null) {
                numberOfLines++;
            }
        } else {
            String currLine = buf.readLine();

            /*
            * if (currLine = null) ;
            * else if (currLine.trim().length() == 0) numberOfLines = 0;
            * else numberOfLines = 1;
            */
            while (currLine != null) {
                if (currLine.trim().length() > 0) {
                    numberOfLines++;
                }

                currLine = buf.readLine();
            }
        }

        buf.close();

        return numberOfLines;
    }

    public static int countColumns(File file, int cols_on_line, boolean ignoreblanklines) throws IOException {
        return countColumns(_buf(file), cols_on_line, ignoreblanklines);
    }

    public static int countColumns(URL url, int cols_on_line, boolean ignoreblanklines) throws IOException {
        return countColumns(_buf(url), cols_on_line, ignoreblanklines);
    }

    public static int countColumns(BufferedReader buf, int cols_on_line, boolean ignoreblanklines)
            throws IOException {

        int numberOfLines = 0;
        String currLine;

        if (!ignoreblanklines) {
            while ((currLine = buf.readLine()) != null) {
                numberOfLines++;

                if (numberOfLines == cols_on_line) {
                    return _cntLine(currLine);
                }
            }
        } else {
            currLine = buf.readLine();

            /*
            * if (currLine = null) ;
            * else if (currLine.trim().length() == 0) numberOfLines = 0;
            * else numberOfLines = 1;
             */
            while (currLine != null) {
                if (currLine.trim().length() > 0) {
                    numberOfLines++;
                }

                if (numberOfLines == cols_on_line) {
                    return _cntLine(currLine);
                }

                currLine = buf.readLine();
            }
        }

        buf.close();

        return numberOfLines;
    }

    private static int _cntLine(String line) {

        StringTokenizer tok = new StringTokenizer(line, " \t");

        return tok.countTokens();
    }

    /**
     * adds any comment lines found to the fComnent class var
     *
     * @param buf
     * @return
     * @throws IOException
     */
    public static String nextLine(BufferedReader buf) throws IOException {

        String currLine = buf.readLine();

        if (currLine == null) {
            return null;
        }

        currLine = currLine.trim();

        while ((currLine != null)
                && ((currLine.length() == 0) || (currLine.startsWith(Constants.COMMENT_CHAR)))) {
            currLine = buf.readLine();

            if (currLine != null) {
                currLine = currLine.trim();
            }
        }

        return currLine;
    }

    public static Map slurpByColumnIntoHashWithValueStringed(File file, boolean byColA, String delim) throws Exception {
        return slurpByColumnIntoHashWithValueStringed(_buf(file), byColA, delim);
    }

    public static Map slurpByColumnIntoHashWithValueStringed(URL url, boolean byColA, String delim) throws Exception {
        return slurpByColumnIntoHashWithValueStringed(_buf(url), byColA, delim);
    }

    public static Map slurpByColumnIntoHashWithValueStringed(final BufferedReader buf, final boolean byColA, final String delim) throws Exception {

        List lines = slurpIntoList(buf, true);
        int cnt = 1;
        Map map = new HashMap();
        for (int i = 0; i < lines.size(); i++) {
            String line = lines.get(i).toString();
            StringTokenizer tok = new StringTokenizer(line, delim);
            int ntokens = tok.countTokens();
            String a = tok.nextToken();
            String b = "";
            if (ntokens == 1) {
                if (byColA) {
                    klog.warn("Only 1 token on line: " + cnt + " >" + a);
                } else {
                    throw new IllegalArgumentException("Bad format -- missing B column at: " + cnt);
                }
            } else if (ntokens == 2) {
                b = tok.nextToken();
            } else {
                throw new IllegalArgumentException("Bad format on line: " + line + "< expecting 1 or 2 tokens but got: " + ntokens);
            }

            if (!byColA) {
                String tmp = a;
                a = b;
                b = tmp;
            }

            Object curr = map.get(a);
            StringBuffer sb = new StringBuffer();
            if (curr != null) {
                sb = sb.append(curr.toString()).append(delim);
            }

            sb.append(b);
            map.put(a, sb.toString());
            cnt++;
        }

        return map;
    }

    public static Map slurpByColumnIntoHashWithValueAsSet(File file, boolean byColA, String delim) throws Exception {
        return slurpByColumnIntoHashWithValueAsSet(_buf(file), byColA, delim);
    }

    public static Map slurpByColumnIntoHashWithValueAsSet(URL url, boolean byColA, String delim) throws Exception {
        return slurpByColumnIntoHashWithValueAsSet(_buf(url), byColA, delim);
    }

    public static Map slurpByColumnIntoHashWithValueAsSet(BufferedReader buf, boolean byColA, String delim) throws Exception {

        List lines = slurpIntoList(buf, true);
        int cnt = 1;
        Map map = new HashMap();
        for (int i = 0; i < lines.size(); i++) {
            String line = lines.get(i).toString();
            StringTokenizer tok = new StringTokenizer(line, delim);
            int ntokens = tok.countTokens();
            String a = tok.nextToken();
            String b = "";
            if (ntokens == 1) {
                if (byColA) {
                    klog.warn("Only 1 token on line: " + cnt + " >" + a);
                } else {
                    throw new IllegalArgumentException("Bad format -- missing B column at: " + cnt);
                }
            } else if (ntokens == 2) {
                b = tok.nextToken();
            } else {
                throw new IllegalArgumentException("Bad format on line: " + line + "< expecting 1 or 2 tokens but got: " + ntokens);
            }

            if (!byColA) {
                String tmp = a;
                a = b;
                b = tmp;
            }

            Object curr = map.get(a);

            if (curr == null) {
                curr = new HashSet();
            }

            ((Set) curr).add(b);
            map.put(a, curr);
            cnt++;
        }

        return map;
    }

    private static BufferedReader _buf(URL url) throws IOException {
        if (url == null) {
            throw new IllegalArgumentException("Parameter url cannot be null");
        }
        InputStreamReader isr = new InputStreamReader(url.openStream());
        return new BufferedReader(isr);
    }

    private static BufferedReader _buf(File file) throws IOException {
        if (file == null) {
            throw new IllegalArgumentException("Parameter file cannot be null");
        }

        return new BufferedReader(new FileReader(file));
    }

    /**
     * @return The (trimmed)first line that has at least one non whitespace char
     *         Null if there are no such lines (i.e empty file or fily ws file).
     * @throws IOException on reading trouble
     */
    public static String getFirstNonWsLine(BufferedReader buf) throws IOException {

        String line = buf.readLine();

        if (line == null) {
            return null;
        }

        line = line.trim();

        if (line.length() > 0) {
            return line;
        }

        while (true) {
            line = buf.readLine();

            if (line == null) {
                return null;
            }

            line = line.trim();

            if (line.length() > 0) {
                return line;
            }
        }

    }

    /**
     * @return The (trimmed)first line that has at least one non whitespace char
     *         Null if there are no such lines (i.e empty file or fily ws file).
     * @throws IOException on reading trouble
     */
    public static String getFirstNonWsLine(File file) throws IOException {

        FileInputStream fis = new FileInputStream(file);

        return getFirstNonWsLine(new BufferedReader(new InputStreamReader(fis)));
    }

    /**
     * last -> last is actually the last in the first MAX_SEARCH_LINES and not in the entire file
     *
     * @param file
     * @return NULL if no comment line found
     * @throws IOException
     */
    public static String findLastCommentLine(File file) throws IOException {

        BufferedReader buf = new BufferedReader(new FileReader(file));
        String line;
        String comm_line = null;
        int linenum = 0;

        while ((line = buf.readLine()) != null) {

            //System.out.println(line);
            line = line.trim();

            if (line.length() == 0) {
                continue;
            }

            if (line.startsWith(Constants.COMMENT_CHAR)) {
                comm_line = line;
            }

            linenum++;

            if (linenum > MAX_SEARCH_LINES) {
                break;
            }
        }

        buf.close();

        return comm_line;
    }

    /**
     * parse: break the input String into fields
     *
     * @return java.util.Iterator containing each field
     *         from the original as a String, in order.
     */
    public static String[] string2strings_csv(final String csvLine) {
        final StringBuffer sb = new StringBuffer();

        final List list = new ArrayList();
        int i = 0;

        do {
            sb.setLength(0);
            if (i < csvLine.length() && csvLine.charAt(i) == '"') {
                i = advQuoted_for_csv(csvLine, sb, ++i);    // skip quote
            } else {
                i = advPlain_for_csv(csvLine, sb, i);
            }
            list.add(sb.toString());
            //log.debug("parse: " + sb.toString());
            i++;
        } while (i < csvLine.length());

        return (String[]) list.toArray(new String[list.size()]);
    }

    public static List string2stringsList_csv(final String csvLine) {
        final String[] ss = string2strings_csv(csvLine);
        final List list = new ArrayList();
        for (int i = 0; i < ss.length; i++) {
            list.add(ss[i]);
        }

        return list;
    }

    /**
     * advQuoted: quoted field; return index of next separator
     */
    private static int advQuoted_for_csv(final String s, final StringBuffer sb, final int i) {
        int j;
        int len = s.length();
        for (j = i; j < len; j++) {
            if (s.charAt(j) == '"' && j + 1 < len) {
                if (s.charAt(j + 1) == '"') {
                    j++; // skip escape char

                } else if (s.charAt(j + 1) == DEFAULT_FIELD_SEP_CSV) { //next delimeter

                    j++; // skip end quotes

                    break;
                }
            } else if (s.charAt(j) == '"' && j + 1 == len) { // end quotes at end of line

                break; //done

            }
            sb.append(s.charAt(j));    // regular character.

        }
        return j;
    }

    /**
     * advPlain: unquoted field; return index of next separator
     */
    private static int advPlain_for_csv(final String s, final StringBuffer sb, final int i) {
        int j;

        j = s.indexOf(DEFAULT_FIELD_SEP_CSV, i); // look for separator

        //klog.debug("csv: " + "i = " + i + " j = " + j);
        if (j == -1) {                   // none found

            sb.append(s.substring(i));
            return s.length();
        } else {
            sb.append(s.substring(i, j));
            return j;
        }
    }


}    // End ParseUtils

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
