/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.parsers;

import edu.mit.broad.genome.*;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.esmatrix.db.DtgDataset;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDbMulti;
import edu.mit.broad.genome.objects.strucs.TxtAffy;
import edu.mit.broad.genome.reports.RichDataframe;
import edu.mit.broad.genome.reports.api.Report;
import edu.mit.broad.genome.swing.ProgressMonitorInputStream2;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.msigdb.MSigDB;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.vdb.sampledb.SampleAnnotGcm;
import edu.mit.broad.vdb.sampledb.SampleAnnotGnf;
import edu.mit.broad.vdb.sampledb.SampleAnnotNCI60;
import edu.mit.broad.xbench.core.api.Application;
import org.apache.log4j.Logger;
import org.dom4j.Attribute;
import org.dom4j.Element;
import edu.duke.igsp.app.parsers.GenotypeParser;
import edu.duke.igsp.app.parsers.NumDatasetParser;
import edu.mit.broad.genome.io.FTPUtils;

import javax.swing.*;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.net.Authenticator;
import java.net.InetAddress;
import java.net.PasswordAuthentication;
import xapps.gsea.GseaWebResources;

/**
 * Facade pattern to easily use the parser classes and methods
 * for simple read in / write out stuff that is used 95% of the time.
 * <p/>
 * Also implements a static cache so that Files are parsed only once (by default)
 * for each jvm invocation.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ParserFactory implements Constants {

    private static final Logger klog = XLogger.getLogger(ParserFactory.class);
    private static ProgressMonitorHook kProgressMonitorHook = null;

    /**
     * Privatized Class constructor
     * static methods only.
     */
    private ParserFactory() {
    }
    private static final ObjectCache kDefaultObjectCache = new ObjectCache();

    // The default one is the generic NON-application related cache
    //additionally there are application specific classes
    static ObjectCache _getCache() {
        return kDefaultObjectCache;
    }

    /**
     * @return The Parsers object cache
     */
    public static ObjectCache getCache() {
        return _getCache();
    }

    //------------------------------------------------------------------------------------//
    //---------------------------- ACTION LIKE METHODS -----------------------------------//
    //------------------------------------------------------------------------------------//
    public static GeneSet extractGeneSet(Dataset ds) {
        GeneSet gset = ds.getRowNamesGeneSet();

        if (_getCache().isCached(ds)) {
            _getCache().add(_getCache().getSourceFile(ds), gset, GeneSet.class);
        }

        return gset;
    }

    public static void extractGeneSets(GeneSetMatrix gm) {
        _getCache().makeVisible(gm.getGeneSets(), GeneSet.class);
    }

    public static GeneSet combineIntoOne(final GeneSetMatrix gm) {
        final Set names = gm.getAllMemberNamesOnlyOnceS();
        final GeneSet gset = new FSet("combo_" + names.size() + "_" + gm.getName(), names);
        _getCache().makeVisible(gset, GeneSet.class);
        return gset;
    }

    //------------------------------------------------------------------------------------//
    //---------------------------- END ACTION LIKE METHODS --------------------------------//
    //------------------------------------------------------------------------------------//
    /**
     * IMP this is JVM wide -- will apply to all usages
     *
     * @param hook
     */
    public static void installProgressMonitorHook(final ProgressMonitorHook hook) {
        klog.debug("Installing ProgressMonitorHook: " + hook);
        kProgressMonitorHook = hook;
    }

    public static void uninstallProgressMonitorHook() {
        klog.debug("Uninstalling ProgressMonitorHook: " + kProgressMonitorHook);
        kProgressMonitorHook = null;
    }

    public static void registerWithCache(Template origTemplate, Template subTemplate) {
        registerWithCache(origTemplate, new Template[]{subTemplate});
    }

    public static void registerWithCache(Template origTemplate, Template[] subTemplates) {
        File origFile = _getCache().getSourceFile(origTemplate);

        for (int i = 0; i < subTemplates.length; i++) {
            File pseudo = new File(origFile.getParentFile(), subTemplates[i].getName()); // file doesnt really exist i.e pseudo
            _getCache().add(pseudo, subTemplates[i], Template.class, false);
        }

        _getCache().add(origFile, origTemplate, Template.class, true);
    }

    public static void registerWithCache(final File sourceFile, GeneSet[] gsets) {

        for (int i = 0; i < gsets.length; i++) {
            File pseudo = new File(sourceFile.getParentFile(), gsets[i].getName()); // file doesnt really exist i.e pseudo
            _getCache().add(pseudo, gsets[i], GeneSet.class, false);
        }


        _getCache().sortModel(GeneSet.class);
    }

    public static void registerWithCache(final File sourceFile, final GeneSetMatrix gm, final boolean fireAction) {
        _getCache().add(sourceFile, gm, GeneSetMatrix.class, fireAction);
    }

    public static void registerWithCache(final File sourceFile, final Dataset ds) {
        _getCache().add(sourceFile, ds, Dataset.class, false);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    public static Dataset readDatasetFromMatrix(final File file, final boolean useCache, final boolean add2cache) throws Exception {
        Matrix m = readMatrix(file.getPath(), createInputStream(file), useCache, true);
        String name = NamingConventions.removeExtension(file.getName());
        Dataset ds = new DefaultDataset(name, m, null);

        if (add2cache) {
            _getCache().add(file.getPath(), ds, Dataset.class);
        }

        return ds;
    }

    public static Matrix readMatrix(File file, boolean useCache) throws Exception {
        return readMatrix(file.getPath(), createInputStream(file), useCache, true);
    }

    public static Matrix readMatrix(File file, boolean useCache, boolean add2cache) throws Exception {
        return readMatrix(file.getPath(), createInputStream(file), useCache, add2cache);
    }

    public static Matrix readMatrix(String path, InputStream is, boolean useCache) throws Exception {
        return readMatrix(path, is, useCache, true);
    }

    public static Matrix readMatrix(String path, InputStream is, boolean useCache, boolean add2cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Matrix.class)) && (useCache)) {
            is.close();
            return (Matrix) _getCache().get(path, Matrix.class);
        }

        //log.debug("Parsing Report from: " + path);
        Parser parser = new MatrixParser();
        Matrix m = (Matrix) parser.parse(path, is).get(0);

        if (add2cache) {
            System.out.println(">>>> ### ADDED TO CACHE " + m.getQuickInfo());
            _getCache().add(path, m, Matrix.class);
        }

        is.close();

        return m;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    /**
     * @param file
     * @return
     * @throws Exception
     */
    public static Dataset readDataset(File file, boolean useCache) throws Exception {
        return readDataset(file.getPath(), createInputStream(file), useCache);
    }

    public static Dataset readDataset(File file, boolean useCache, boolean add2cache, boolean silent) throws Exception {
        return readDataset(file.getPath(), createInputStream(file), useCache, add2cache, silent);
    }

    public static Dataset readDataset(File file, boolean useCache, boolean add2cache) throws Exception {
        return readDataset(file.getPath(), createInputStream(file), useCache, add2cache, false);
    }

    public static Dataset readDataset(Element el, boolean useCache) throws Exception {
        return readDataset(toFile(el, Headers.DATASET), useCache);
    }

    public static Dataset readDataset(String path, InputStream is, boolean useCache) throws Exception {
        return readDataset(path, is, useCache, true);
    }

    public static Dataset readDataset(String path, InputStream is, boolean useCache,
            boolean add2cache) throws Exception {
        return readDataset(path, is, useCache, add2cache, false);
    }

    /**
     * @param path
     * @param is
     * @param useCache
     * @return
     * @throws Exception
     */
    public static Dataset readDataset(String path, InputStream is, boolean useCache,
            boolean add2cache, boolean silent) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if ((_getCache().isCached(path, Dataset.class)) && (useCache)) {
            is.close();
            return (Dataset) _getCache().get(path, Dataset.class);
        }

        // as a help to make datasets ext agnostic
        String ext = NamingConventions.getExtension(path);
        if (ext.equals(Constants.GCT)) {
            return readDatasetGct(path, is, useCache, add2cache, silent);
        }

        if (ext.equals(Constants.PCL)) {
            return readDatasetPcl(path, is, useCache, add2cache);
        }

        if (ext.equals(Constants.DCHIP)) {
            return readDatasetDChip(path, is, useCache, add2cache);
        }

        Dataset ds;
        Parser parser = new ResParser();
        parser.setSilentMode(silent);
        List list = parser.parse(toName(path), is);
        ds = (Dataset) list.get(0);

        // sometimes might not want to for memory reasons
        if (add2cache) {
            _getCache().add(path, ds, Dataset.class);
        }

        // also auto add a gset for the ds read in
        // disabled as its rarely used and sometimes leads to choice as the grp file in UI's
        //kObjectCache.add(path, ds.getRowNamesGeneSet(), GeneSet.class);

        is.close();
        return ds;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    /**
     * Dataset in genecluster transposed format -- similar to dataframe "gct"
     *
     * @param file
     * @return
     * @throws Exception
     */
    public static Dataset readDatasetGct(File file, boolean useCache) throws Exception {
        return readDatasetGct(file.getPath(), createInputStream(file), useCache);
    }

    public static Dataset readDatasetGct(File file, boolean useCache, boolean add2cache, boolean parseFlippedNameDesc) throws Exception {
        return readDatasetGct(file.getPath(), createInputStream(file), useCache, add2cache, parseFlippedNameDesc, false);
    }

    public static Dataset readDatasetGct(String path, InputStream is, boolean useCache) throws Exception {
        return readDatasetGct(path, is, useCache, true);
    }

    public static Dataset readDatasetGct(String path, InputStream is, boolean useCache, boolean add2cache) throws Exception {
        return readDatasetGct(path, is, useCache, add2cache, false, false);
    }

    public static Dataset readDatasetGct(String path, InputStream is, boolean useCache, boolean add2cache, boolean silent) throws Exception {
        return readDatasetGct(path, is, useCache, add2cache, false, silent);
    }

    public static Dataset readDatasetGct(String path, InputStream is,
            boolean useCache,
            boolean add2cache,
            boolean parseFlippedNameDesc,
            boolean silent) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Dataset.class)) && (useCache)) {
            return (Dataset) _getCache().get(path, Dataset.class);
        }

        // as a help to make datasets ext agnostic
        String ext = NamingConventions.getExtension(path);
        if (ext.equals(Constants.RES)) {
            return readDataset(path, is, useCache);
        }

        if (ext.equals(Constants.PCL)) {
            return readDatasetPcl(path, is, useCache);
        }

        if (ext.equals(Constants.DCHIP)) {
            return readDatasetDChip(path, is, useCache);
        }

        GctParser parser = new GctParser();
        parser.setSilentMode(silent);
        List list;
        if (parseFlippedNameDesc) {
            list = parser.parseFlippedNameDesc(toName(path), is);
        } else {
            list = parser.parse(toName(path), is);
        }

        Dataset ds = (Dataset) list.get(0);

        if (add2cache) {
            _getCache().add(path, ds, Dataset.class);
        }

        // also auto add a gset for the ds read in
        // disabled as its rarely used and sometimes leads to choice as the grp file in UI's
        //kObjectCache.add(path, ds.getRowNamesGeneSet(), GeneSet.class);

        is.close();
        return ds;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    /**
     * Dataset in genecluster transposed format -- similar to dataframe "gct"
     *
     * @param file
     * @return
     * @throws Exception
     */
    public static Dataset readDatasetTXT(File file, boolean useCache) throws Exception {
        return readDatasetTXT(file.getPath(), createInputStream(file), useCache);
    }

    public static Dataset readDatasetTXT(String path, InputStream is, boolean useCache) throws Exception {
        return readDatasetTXT(path, is, useCache, true);
    }

    public static Dataset readDatasetTXT(String path, InputStream is, boolean useCache, boolean add2cache) throws Exception {
        return readDatasetTXT(path, is, useCache, add2cache, false, false);
    }

    public static Dataset readDatasetTXT(String path, InputStream is, boolean useCache, boolean add2cache, boolean silent) throws Exception {
        return readDatasetTXT(path, is, useCache, add2cache, false, silent);
    }

    public static Dataset readDatasetTXT(String path, InputStream is,
            boolean useCache,
            boolean add2cache,
            boolean parseFlippedNameDesc,
            boolean silent) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Dataset.class)) && (useCache)) {
            return (Dataset) _getCache().get(path, Dataset.class);
        }

        // as a help to make datasets ext agnostic
        String ext = NamingConventions.getExtension(path);
        if (ext.equals(Constants.RES)) {
            return readDataset(path, is, useCache);
        }

        if (ext.equals(Constants.GCT)) {
            return readDatasetGct(path, is, useCache);
        }

        if (ext.equals(Constants.PCL)) {
            return readDatasetPcl(path, is, useCache);
        }

        if (ext.equals(DataFormat.DCHIP)) {
            return readDatasetDChip(path, is, useCache);
        }

        TxtDatasetParser parser = new TxtDatasetParser();
        parser.setSilentMode(silent);
        List list = parser.parse(toName(path), is);
        Dataset ds = (Dataset) list.get(0);

        if (add2cache) {
            _getCache().add(path, ds, Dataset.class);
        }

        // also auto add a gset for the ds read in
        // disabled as its rarely used and sometimes leads to choice as the grp file in UI's
        //kObjectCache.add(path, ds.getRowNamesGeneSet(), GeneSet.class);

        is.close();
        return ds;
    }

    // -------------------------------------------------------------------------------------------- //
    public static Dataset readDatasetPcl(File file, boolean useCache) throws Exception {
        return readDatasetPcl(file.getPath(), createInputStream(file), useCache);
    }

    public static Dataset readDatasetPcl(String path, InputStream is, boolean useCache) throws Exception {
        return readDatasetPcl(path, is, useCache, true);
    }

    public static Dataset readDatasetPcl(String path, InputStream is, boolean useCache, boolean add2cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Dataset.class)) && (useCache)) {
            return (Dataset) _getCache().get(path, Dataset.class);
        }

        // as a help to make datasets ext agnostic
        String ext = NamingConventions.getExtension(path);
        if (ext.equals(Constants.RES)) {
            return readDataset(path, is, useCache);
        }

        if (ext.equals(Constants.GCT)) {
            return readDatasetGct(path, is, useCache);
        }

        if (ext.equals(Constants.DCHIP)) {
            return readDatasetDChip(path, is, useCache);
        }


        Parser parser = new PclParser();
        List list = parser.parse(path, is); // @note IMP special hack for PCL parser
        Dataset ds = (Dataset) list.get(0);

        if (add2cache) {
            _getCache().add(path, ds, Dataset.class);
        }

        // also auto add a gset for the ds read in
        // disabled as its rarely used and sometimes leads to choice as the grp file in UI's
        //kObjectCache.add(path, ds.getRowNamesGeneSet(), GeneSet.class);

        is.close();
        return ds;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    public static Dataset readDatasetDChip(File file, boolean useCache) throws Exception {
        if (file == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }
        return readDatasetDChip(file.getPath(), createInputStream(file), useCache);
    }

    public static Dataset readDatasetDChip(String path, InputStream is, boolean useCache) throws Exception {
        return readDatasetDChip(path, is, useCache, true);
    }

    public static Dataset readDatasetDChip(String path, InputStream is, boolean useCache, boolean add2cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Dataset.class)) && (useCache)) {
            return (Dataset) _getCache().get(path, Dataset.class);
        }

        // as a help to make datasets ext agnostic
        String ext = NamingConventions.getExtension(path);
        if (ext.equals(Constants.RES)) {
            return readDataset(path, is, useCache);
        }

        if (ext.equals(Constants.GCT)) {
            return readDatasetGct(path, is, useCache);
        }

        if (ext.equals(Constants.PCL)) {
            return readDatasetPcl(path, is, useCache);
        }

        Parser parser = new DChipDatasetParser();
        List list = parser.parse(path, is);
        Dataset ds = (Dataset) list.get(0);

        if (add2cache) {
            _getCache().add(path, ds, Dataset.class);
        }

        is.close();
        return ds;
    }

    // ------------------------------------------------------------------------- //
    // ------------------------------------------------------------------------- //
    // ------------------------------------------------------------------------- //
    public static Dataset readDatasetRMA(File file, boolean useCache) throws Exception {
        if (file == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }
        return readDatasetRMA(file.getPath(), createInputStream(file), useCache);
    }

    public static Dataset readDatasetRMA(String path, InputStream is, boolean useCache) throws Exception {
        return readDatasetRMA(path, is, useCache, true);
    }

    public static Dataset readDatasetRMA(String path, InputStream is, boolean useCache, boolean add2cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Dataset.class)) && (useCache)) {
            return (Dataset) _getCache().get(path, Dataset.class);
        }

        // as a help to make datasets ext agnostic
        Parser parser = new RMADatasetParser();
        List list = parser.parse(path, is);
        Dataset ds = (Dataset) list.get(0);

        if (add2cache) {
            _getCache().add(path, ds, Dataset.class);
        }

        is.close();
        return ds;
    }

    /**
     * cdt formatted dataset
     * NO aux ability
     * IMP: also adds the contained Dataset
     *
     * @param file
     * @return
     * @throws Exception
     */
    public static HClustDatasetStanford readHClustDataset(final File file, final boolean useCache) throws Exception {
        return readHClustDataset(file.getPath(), createInputStream(file), useCache);
    }

    public static HClustDatasetStanford readHClustDataset(final String path, final InputStream is, final boolean useCache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, HClustDatasetStanford.class)) && (useCache)) {
            is.close();
            return (HClustDatasetStanford) _getCache().get(path, HClustDatasetStanford.class);
        }

        Parser parser = new HClustDatasetParser();
        List list = parser.parse(toName(path), is);
        HClustDatasetStanford cds = (HClustDatasetStanford) list.get(0);

        _getCache().add(path, cds, HClustDatasetStanford.class);
        // also add the Dataset
        _getCache().add(path, cds.getDataset(), Dataset.class);

        // also auto add a gset for the ds read in
        // disabled as its rarely used and sometimes leads to choice as the grp file in UI's
        //kObjectCache.add(path, cds.getDataset().getRowNamesGeneSet(), GeneSet.class);

        is.close();
        return cds;
    }

    public static HClustTree readHClustTree(final File file, final boolean useCache) throws Exception {
        return readHClustTree(file.getPath(), createInputStream(file), useCache);
    }

    public static HClustTree readHClustTree(final String path, final InputStream is, final boolean useCache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, HClustTree.class)) && (useCache)) {
            is.close();
            return (HClustTree) _getCache().get(path, HClustTree.class);
        }

        HClustTreeParser parser = new HClustTreeParser();
        List list = parser.parse(toName(path), is);
        HClustTree tree = (HClustTree) list.get(0);
        _getCache().add(path, tree, HClustTree.class);
        is.close();
        return tree;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    /**
     * @param file
     * @return
     * @throws Exception
     */
    public static Dataframe readDataframe(final File file, final boolean useCache, final boolean add2cache) throws Exception {
        return readDataframe(file.getPath(), createInputStream(file), useCache, add2cache);
    }

    public static Dataframe readDataframe(final File file, final boolean useCache) throws Exception {
        return readDataframe(file.getPath(), createInputStream(file), useCache, true);
    }

    public static Dataframe readDataframe(final String path, final InputStream is, final boolean useCache) throws Exception {
        return readDataframe(path, is, useCache, true);
    }

    public static Dataframe readDataframe(final String path, final InputStream is, final boolean useCache, final boolean add2cache) throws Exception {
        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Dataframe.class)) && (useCache)) {
            is.close();
            return (Dataframe) _getCache().get(path, Dataframe.class);
        }

        DataframeParser parser = new DataframeParser();
        List list = parser.parse(toName(path), is);
        Dataframe df = (Dataframe) list.get(0);

        if (add2cache) {
            _getCache().add(path, df, Dataframe.class);
        }
        is.close();
        return df;
    }

    /**
     * IMP NOT cached
     *
     * @param text
     * @return
     * @throws Exception
     */
    public static Dataframe readDataframe(final String text) throws Exception {
        return (Dataframe) new DataframeParser().parse(text).get(0);
    }

    public static StringDataframe readStringDataframe(File file, boolean useCache) throws Exception {
        return readStringDataframe(file.getPath(), createInputStream(file), useCache, true);
    }

    public static StringDataframe readStringDataframe(File file, boolean useCache, boolean add2cache) throws Exception {
        return readStringDataframe(file.getPath(), createInputStream(file), useCache, add2cache);
    }

    public static StringDataframe readStringDataframe(String path, InputStream is, boolean useCache) throws Exception {
        return readStringDataframe(path, is, useCache, true);
    }

    public static StringDataframe readStringDataframe(String path, InputStream is, boolean useCache, boolean add2cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, StringDataframe.class)) && (useCache)) {
            is.close();
            return (StringDataframe) _getCache().get(path, StringDataframe.class);
        }

        StringDataframeParser parser = new StringDataframeParser();
        List list = parser.parse(toName(path), is);
        StringDataframe sdf = (StringDataframe) list.get(0);

        if (add2cache) {
            _getCache().add(path, sdf, StringDataframe.class);
        }
        is.close();
        return sdf;
    }

    /**
     * IMP not cached
     *
     * @param text
     * @return
     * @throws Exception
     */
    public static StringDataframe readStringDataframe(String text) throws Exception {
        return new StringDataframeParser().parse(text);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    public static SampleAnnot readSampleAnnot(final String path, final boolean useCache) throws Exception {
        return readSampleAnnot(path, createInputStream(path), useCache);
    }

    public static SampleAnnot readSampleAnnot(String path, InputStream is, boolean useCache) throws Exception {
        return readSampleAnnot(path, is, useCache, true);
    }

    public static SampleAnnot readSampleAnnot(final File xlsFile, final boolean useCache, final boolean add2cache) throws Exception {
        return readSampleAnnot(xlsFile.getPath(), createInputStream(xlsFile), useCache, add2cache);
    }

    public static SampleAnnot readSampleAnnot(String path, InputStream is, boolean useCache, boolean add2Cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if ((_getCache().isCached(path, SampleAnnot.class) && (useCache))) {
            return (SampleAnnot) _getCache().get(path, SampleAnnot.class);
        }

        SampleAnnot sa;
        //final MiscParsers.ExcelSdfColorStruc exsr = MiscParsers.excel2sdf_and_colors(path, is);
        final MiscParsers.ExcelSdfColorStruc exsr = MiscParsers.excel2sdf_and_colors(new File(path)); // @note the underlying JXL api for is seems to be broken
        StringDataframe sdf = exsr.sdf;

        if (sdf.hasColumn("SCHEMA") == false) {
            throw new IllegalArgumentException("The sin file has no SCHEMA file >" + path + "<");
        }

        String value = sdf.getElement(0, "SCHEMA").toUpperCase();

        klog.debug("Schema: " + value);

        if (value.equalsIgnoreCase("GCM")) {
            sa = SampleAnnotGcm.parse(exsr);
        } else if (value.equalsIgnoreCase("GNF")) {
            sa = SampleAnnotGnf.parse(exsr);
        } else if (value.equalsIgnoreCase("NCI60")) {
            sa = SampleAnnotNCI60.parse(exsr);
        } else {
            throw new IllegalArgumentException("Unknown sample annot schema: >" + value + "<");
        }

        if (add2Cache) {
            _getCache().add(path, sa, SampleAnnot.class);
        }

        return sa;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    /**
     * Note the aux parsing business, so we might actually end of adding
     * more than 1 template
     *
     * @param file
     * @return
     * @throws Exception
     */
    public static Template readTemplate(File file) throws Exception {
        File f = AuxUtils.getBaseFileFromAuxFile(file);
        return readTemplate(file.getPath(), createInputStream(f), true, true, false); // use cache when possible
    }

    public static Template readTemplate(File file, boolean useCache) throws Exception {
        File f = AuxUtils.getBaseFileFromAuxFile(file);
        return readTemplate(file.getPath(), createInputStream(f), useCache, true, false);
    }

    public static Template readTemplate(File file, boolean useCache, boolean silentMode) throws Exception {
        File f = AuxUtils.getBaseFileFromAuxFile(file);
        return readTemplate(file.getPath(), createInputStream(f), useCache, true, silentMode);
    }

    public static Template readTemplate(File file, boolean useCache, boolean add2cache, boolean silentMode) throws Exception {
        File f = AuxUtils.getBaseFileFromAuxFile(file);
        return readTemplate(file.getPath(), createInputStream(f), useCache, add2cache, silentMode);
    }

    public static Template readTemplate(Element el, boolean useCache) throws Exception {
        return readTemplate(toFile(el, Headers.TEMPLATE), useCache, false);
    }

    /**
     * auxpath format is:
     * <p/>
     * filenameorpath#aux_template_name
     * <p/>
     * Example:
     * <p/>
     * all_aml.cls#all
     * <p/>
     * Default is to use the cache when available.
     *
     * @param path_and_aux
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @param file
     * @return
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception Uses cache when possible
     * @throws Exception
     */
    // public static Template readTemplate(String path_and_aux) throws Exception {
    //   return readTemplate(path_and_aux, true); // use cache when possible
    // }
    /**
     * Uses cache when possible
     *
     * @param file
     * @return
     * @throws Exception
     */
    public static Template[] readTemplates(File file, boolean silentMode) throws Exception {
        File f = AuxUtils.getBaseFileFromAuxFile(file);
        return _readTemplates(file.getPath(), createInputStream(f), true, true, silentMode); // use cache when possible

    }

    /**
     * @param file
     * @param useCache
     * @return
     * @throws Exception
     */
    public static Template[] readTemplates(File file, boolean useCache, boolean add2cache, boolean silentMode) throws Exception {
        File f = AuxUtils.getBaseFileFromAuxFile(file);
        return _readTemplates(file.getPath(), createInputStream(f), useCache, add2cache, silentMode);
    }

    public static Template[] readTemplates(String path, InputStream is, boolean useCache, boolean add2cache, boolean silentMode) throws Exception {
        return _readTemplates(path, is, useCache, add2cache, silentMode);
    }

    public static Template readTemplate(String path_and_aux, InputStream is, boolean useCache, boolean add2cache, boolean silentMode) throws Exception {
        if (path_and_aux == null) {
            throw new IllegalArgumentException("Param path_and_aux cannot be null");
        }

        final Template[] templates = readTemplates(path_and_aux, is, useCache, add2cache, silentMode);

        if (AuxUtils.isAux(path_and_aux)) {
            is.close();
            return findTemplate(toName(path_and_aux), templates);
        } else {
            is.close();
            return templates[0];// our main template is always the first one (auxs are hidden unless specifically asked for)
        }
    }

    // does the real stuff
    public static Template[] _readTemplates(final String path,
            final InputStream is,
            final boolean useCache,
            final boolean add2cache,
            final boolean silentMode) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        Set baseTemplates = new HashSet();
        File basefile;
        String base_path = AuxUtils.getBasePathFromAuxPath(path);
        basefile = new File(base_path);
        //klog.debug("useCache: " + useCache + " for: " + path + " isCached: " + kObjectCache.isCached(base_path, Template.class));

        if (useCache == true && _getCache().isCached(base_path, Template.class)) {
            //klog.debug("isCached: " + useCache + " for: " + path);
            Object obj = _getCache().get(base_path, Template.class);
            baseTemplates.add(obj);
            is.close();
        } else {

            Parser parser = new ClsParser();
            parser.setSilentMode(silentMode);

            if (AuxUtils.isAux(path)) {
                basefile = AuxUtils.getBaseFileFromAuxFile(new File(path));
                List list = parser.parse(basefile.getName(), basefile);
                // multiple only for numeric templates
                baseTemplates.addAll(list);
            } else {
                List list = parser.parse(toName(path), is);
                baseTemplates.addAll(list); // multiple only for numeric templates
                // just one
                basefile = new File(path);
            }
        }

        is.close();

        final List allTemplates = new ArrayList();
        Template[] allTss = (Template[]) baseTemplates.toArray(new Template[baseTemplates.size()]);

        for (int j = 0; j < allTss.length; j++) { // Typically for non-continuous templates, only done once

            final Template[] tss = TemplateFactory.extractAllPossibleTemplates(allTss[j], true); // @note here s where we turn a template into many subtemplates

            for (int i = 0; i < tss.length; i++) {
                allTemplates.add(tss[i]);
            }

            if (!silentMode) {
                klog.debug("From: " + path + " (and its supers & auxes) total # of templates made: " + allTemplates.size());
            }

            if (add2cache) {
                boolean fire = false;
                for (int t = 0; t < allTemplates.size(); t++) {
                    Template tm = (Template) allTemplates.get(t);
                    if (t == allTemplates.size() - 1) {
                        fire = true;
                    }

                    if (silentMode) {
                        fire = false; // @note
                    }

                    if (tm.isAux()) {
                        File pseudo = new File(basefile.getParentFile(), tm.getName()); // note how a pseudo file is made
                        _getCache().add(pseudo, tm, Template.class, fire);
                    } else {
                        _getCache().add(basefile, tm, Template.class, fire);
                    }
                }
            }
        }

        return (Template[]) allTemplates.toArray(new Template[allTemplates.size()]);
    }

    public static HClustTree getDefaultGeneTreeForHClustDataset(HClustDatasetStanford cds) throws Exception {
        return _getDefaultXTreeForHClustDataset(cds, Constants.GTR);
    }

    public static HClustTree getDefaultGeneTreeForHClustDatasetOrNull(HClustDatasetStanford cds) {
        try {
            return getDefaultGeneTreeForHClustDataset(cds);
        } catch (Throwable t) {
            return null;
        }
    }

    public static HClustTree getDefaultArrayTreeForHClustDataset(HClustDatasetStanford cds) throws Exception {
        return _getDefaultXTreeForHClustDataset(cds, Constants.ATR);
    }

    public static HClustTree getDefaultArrayTreeForHClustDatasetOrNull(HClustDatasetStanford cds) {
        try {
            return getDefaultArrayTreeForHClustDataset(cds);
        } catch (Throwable t) {
            return null;
        }
    }

    /**
     * Simply looks ion same folder as the cds for a file named sames as
     * cds except with an ext extension
     *
     * @param cds
     * @return
     */
    private static HClustTree _getDefaultXTreeForHClustDataset(HClustDatasetStanford cds, String ext) throws Exception {

        if (cds == null) {
            throw new IllegalArgumentException("Parameter cds cannot be null");
        }

        File cdtFile = _getCache().getSourceFile(cds);
        if (cdtFile == null) {
            throw new IllegalArgumentException("CDT file for HClustDataset does not exist: " + cds.getName());
        }

        String fn = FileUtils.removeExtension(cdtFile.getName());
        File trFile = new File(cdtFile.getParentFile(), fn + "." + ext);

        return readHClustTree(trFile, true);
    }

    /**
     * gsets can also be within a gmx/gmt file via the aux mechanism
     *
     * @param file
     * @return
     * @throws Exception
     */
    public static GeneSet readGeneSet(File file, boolean useCache) throws Exception {
        return readGeneSet(file, useCache, true);
    }

    public static GeneSet readGeneSet(File file, boolean useCache, boolean add2Cache) throws Exception {
        InputStream is = createInputStream(file);
        return readGeneSet(file.getPath(), is, useCache, add2Cache);
    }

    public static GeneSet readGeneSet(Element el, boolean useCache) throws Exception {
        return readGeneSet(toFile(el, Headers.GENESET), useCache);
    }

    // IMP: dont name this r..geneset as it needs to indicate that the parsing is specifically for a f gset formatted file
    // however the returned object (and cached object) are GeneSets (just like gct -> Dataset)
    public static GeneSet readGeneSet(String path, InputStream is, boolean useCache) throws Exception {
        return readGeneSet(path, is, useCache, true);
    }

    public static GeneSet readGeneSet(String path, InputStream is, boolean useCache, boolean add2Cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, GeneSet.class)) && (useCache)) {
            is.close();
            return (GeneSet) _getCache().get(path, GeneSet.class);
        }

        GeneSet gset;

        if (AuxUtils.isAux(path)) {
            String gsetName = AuxUtils.getAuxNameOnlyNoHash(path);
            int index = path.indexOf("gin.xls");
            if (index != -1) {
                File file = AuxUtils.getBaseFileFromFullPath(path);
                GenesOfInterest gin = readGenesOfInterest(file, useCache);
                return gin.getGeneSet(gsetName);
            } else {
                //System.out.println("was an aux");
                File f = AuxUtils.getBaseFileFromAuxFile(new File(path));
                GeneSetMatrix gm = readGeneSetMatrix(f, useCache, true, add2Cache);
                return gm.getGeneSet(gsetName);
            }

            /*
            // has to be a gmx/gmt file or a gin file
            Object obj = read(path, is, useCache);
            if (obj instanceof GeneSetMatrix) {
            klog.debug("Looking for gset: " + toName(path));
            gset = ((GeneSetMatrix) obj).getGeneSet(toName(path));
            } else {
            throw new IllegalArgumentException("Invalid data for GeneSet -- wrong file type: " + obj.getClass());
            }
             */


        } else if (path.endsWith("gin.xls")) {
            GenesOfInterest gin = readGenesOfInterest(path, useCache);
            GeneSet all = gin.getAsOneGeneSet();
            _getCache().addInvisibly(path, all);
            return all;
        } else {
            Parser parser = new FSetParser();
            gset = (GeneSet) parser.parse(toName(path), is).get(0);
            // for convenience make a genesetmatrix too (its not expensive)
            // but be careful where you are adding -- dont want call to be recursive
            //kObjectCache.add(path, new DefaultGeneSetMatrix(gset.getName(), new GeneSet[]{gset}), GeneSetMatrix.class);
            _getCache().addInvisibly(path, new DefaultGeneSetMatrix(gset.getName(), new GeneSet[]{gset}));
        }

        if (add2Cache) {
            _getCache().add(path, gset, GeneSet.class);
        }
        is.close();
        return gset;
    }

    /**
     * @param file
     * @return
     * @throws Exception
     */
    public static Report readReport(File file, boolean useCache) throws Exception {

        if (file.isDirectory()) {
            file = FileUtils.findFile(file, ".rpt", true); // @note hack to read the rpt
        }

        return readReport(file.getPath(), createInputStream(file), useCache);
    }

    public static Report readReport(String path, InputStream is, boolean useCache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }


        if ((_getCache().isCached(path, Report.class) && (useCache))) {
            is.close();
            return (Report) _getCache().get(path, Report.class);
        }

        //log.debug("Parsing Report from: " + path);
        ReportParser parser = new ReportParser();
        Report rpt = (Report) parser.parse(toName(path), is).get(0);

        if (useCache) { // note special -- dont add
            _getCache().add(path, rpt, Report.class);
        }

        is.close();
        return rpt;
    }

    public static Chip readChip(File file, boolean useCache) throws Exception {
        if (file == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }
        return readChip(file.getPath(), createInputStream(file), useCache);
    }

    public static Chip readChip(String sourcePath, boolean useCache) throws Exception {
        return readChip(sourcePath, createInputStream(sourcePath), useCache);
    }

    public static Chip readChip(String path, InputStream is, boolean useCache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Chip.class) && (useCache))) {
            is.close();
            return (Chip) _getCache().get(path, Chip.class);
        }

        //log.debug("Parsing Report from: " + path);
        Parser parser = new ChipParser();
        Chip chip = (Chip) parser.parse(path, is).get(0);

        if (useCache) {
            _getCache().add(path, chip, Chip.class);

            // @note also registering as a grp object
            // disabled for memory reasons also it leads to selection of chip as a grp by mistake easily
            //kObjectCache.add(path, chip.getProbeNamesAsGeneSet(), GeneSet.class);
        }

        is.close();

        return chip;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    public static RankedList readRankedList(File file, boolean useCache) throws Exception {
        return readRankedList(file.getPath(), createInputStream(file), useCache);
    }

    public static RankedList readRankedList(File file, boolean useCache, boolean silent, boolean fast_no_sort) throws Exception {
        return readRankedList(file.getPath(), createInputStream(file), useCache, useCache, silent, fast_no_sort);
    }

    public static RankedList readRankedList(String sourcePath, boolean useCache) throws Exception {
        return readRankedList(sourcePath, createInputStream(sourcePath), useCache);
    }

    public static RankedList readRankedList(String path, InputStream is, boolean useCache) throws Exception {
        return readRankedList(path, is, useCache, true, false, false);
    }

    public static RankedList readRankedList(String path,
            InputStream is,
            boolean useCache,
            boolean add2Cache,
            boolean silent,
            boolean parse_fast_no_sort) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, RankedList.class) && (useCache))) {
            is.close();
            return (RankedList) _getCache().get(path, RankedList.class);
        }

        //log.debug("Parsing Report from: " + path);
        RankedListParser parser = new RankedListParser();
        parser.setSilentMode(silent);
        RankedList rl;

        if (parse_fast_no_sort) {
            rl = (RankedList) parser.parse_fast_no_sort(path, is).get(0);
        } else {
            rl = (RankedList) parser.parse(path, is).get(0);
        }

        if (add2Cache) {
            _getCache().add(path, rl, RankedList.class);
            // as a convenience add thia
            //_cache().add(path, new FSet(rl.getName(), rl.getNameEnglish(), rl.getRankedNames(), false), GeneSet.class);
        }

        is.close();

        return rl;
    }

    // -------------------------------------------------------------------------------------------- //

    /*
    public static RankedListDb readRankedListDbJIT(File file, boolean useCache) throws Exception {
    return readRankedListDbJIT(file.getPath(), createInputStream(file), useCache);
    }

    public static RankedListDb readRankedListDbJIT(String sourcePath, boolean useCache) throws Exception {
    return readRankedListDbJIT(sourcePath, createInputStream(sourcePath), useCache);
    }

    public static RankedListDb readRankedListDbJIT(String path, InputStream is, boolean useCache) throws Exception {

    if (path == null) {
    throw new IllegalArgumentException("Param path cannot be null");
    }

    if (is == null) {
    throw new IllegalArgumentException("Param is cannot be null");
    }

    if ((_cache().isCached(path, RankedListDb.class) && (useCache))) {
    is.close();
    return (RankedListDb) _cache().get(path, RankedListDb.class);
    }

    //log.debug("Parsing Report from: " + path);
    Parser parser = new JITRankedListDbParser();
    RankedListDb rdb = (RankedListDb) parser.parse(path, is).get(0);

    if (useCache) {
    _cache().add(path, rdb, RankedListDb.class);
    }

    is.close();

    return rdb;
    }
     */
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    public static GenesOfInterest readGenesOfInterest(File file, boolean useCache) throws Exception {
        if (file == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }
        return readGenesOfInterest(file.getPath(), createInputStream(file), useCache);
    }

    public static GenesOfInterest readGenesOfInterest(String sourcePath, boolean useCache) throws Exception {
        return readGenesOfInterest(sourcePath, createInputStream(sourcePath), useCache);
    }

    public static GenesOfInterest readGenesOfInterest(String path, InputStream is, boolean useCache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, GenesOfInterest.class) && (useCache))) {
            is.close();
            return (GenesOfInterest) _getCache().get(path, GenesOfInterest.class);
        }

        //boolean add2Cache = true;
        Parser parser = new GinParser_xls();
        GenesOfInterest gin = (GenesOfInterest) parser.parse(path, is).get(0);

        // IMP also fetch and add all gsets in the gmx to cache
        File parentFile = new File(path).getParentFile();
        //boolean fire = false;
        for (int i = 0; i < gin.getNumGeneLists(); i++) {
            GeneSet gset = gin.getGeneList(i).toGeneSet();
            File pseudo = new File(parentFile, gset.getName());
            _getCache().add(pseudo, gset, GeneSet.class);
        }

        if (useCache) {
            _getCache().add(path, gin, GenesOfInterest.class);
        }

        is.close();

        return gin;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    public static TxtAffy readTxtAffy(File file, boolean useCache, boolean add2cache, boolean silent) throws Exception {
        return readTxtAffy(file.getPath(), createInputStream(file), useCache, add2cache, silent);
    }

    public static TxtAffy readTxtAffy(String path, InputStream is, boolean useCache, boolean add2cache, boolean silent) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, TxtAffy.class)) && (useCache)) {
            return (TxtAffy) _getCache().get(path, TxtAffy.class);
        }

        Parser parser = new TxtAffyParser();
        parser.setSilentMode(silent);
        List list = parser.parse(path, is); // @note IMP special hack for PCL parser
        TxtAffy taf = (TxtAffy) list.get(0);

        if (add2cache) {
            _getCache().add(path, taf, TxtAffy.class);
        }

        is.close();
        return taf;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    /**
     * @param gseaResultDir
     * @return
     * @throws Exception
     */
    public static EnrichmentDb readEdb(final File gseaResultDir, final boolean useCache) throws Exception {
        return readEdb(gseaResultDir, useCache, false);
    }

    public static EnrichmentDb readEdb(final String gseaResultDir_path, final boolean useCache) throws Exception {
        return readEdb(new File(gseaResultDir_path), useCache, false);
    }

    public static EnrichmentDb readEdb(final File gseaResultDir, final boolean useCache, final boolean silent) throws Exception {

        if (gseaResultDir == null) {
            throw new IllegalArgumentException("Param gseaResultDir cannot be null");
        }


        if ((_getCache().isCached(gseaResultDir, EnrichmentDb.class)) && (useCache)) {
            return (EnrichmentDb) _getCache().get(gseaResultDir, EnrichmentDb.class);
        }

        EdbFolderParser folderParser = new EdbFolderParser();
        folderParser.setSilentMode(silent);

        final EnrichmentDb edb = new EdbFolderParser().parseEdb(gseaResultDir);
        _getCache().add(gseaResultDir, edb, EnrichmentDb.class);
        return edb;
    }

    // -------------------------------------------------------------------------------------------- //
    public static DtgDataset readDtgDataset(File file, boolean useCache) throws Exception {
        return readDtgDataset(file.getPath(), createInputStream(file), useCache);
    }

    public static DtgDataset readDtgDataset(String path, InputStream is, boolean useCache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, DtgDataset.class)) && (useCache)) {
            is.close();
            return (DtgDataset) _getCache().get(path, DtgDataset.class);
        }

        DtgDatasetParser parser = new DtgDatasetParser();

        DtgDataset dtgds = (DtgDataset) parser.parse(toName(path), is).get(0);

        _getCache().add(path, dtgds, DtgDataset.class);

        is.close();
        return dtgds;
    }

    public static Dataset readSnpDataset(String path, InputStream is, boolean useCache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Dataset.class)) && (useCache)) {
            is.close();
            return (Dataset) _getCache().get(path, Dataset.class);
        }

        GenotypeParser parser = new GenotypeParser();
        List list = parser.parse(toName(path), is);
        Dataset ds = (Dataset) list.get(0);

        _getCache().add(path, ds, Dataset.class);

        // also auto add a gset for the ds read in
        // disabled as its rarely used and sometimes leads to choice as the grp file in UI's
        //kObjectCache.add(path, ds.getRowNamesGeneSet(), GeneSet.class);

        is.close();
        return ds;
    }

    public static Dataset readNumDataset(String path, InputStream is, boolean useCache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if ((_getCache().isCached(path, Dataset.class)) && (useCache)) {
            is.close();
            return (Dataset) _getCache().get(path, Dataset.class);
        }

        NumDatasetParser parser = new NumDatasetParser();
        List list = parser.parse(toName(path), is);
        Dataset ds = (Dataset) list.get(0);

        _getCache().add(path, ds, Dataset.class);

        // also auto add a gset for the ds read in
        // disabled as its rarely used and sometimes leads to choice as the grp file in UI's
        //kObjectCache.add(path, ds.getRowNamesGeneSet(), GeneSet.class);

        is.close();
        return ds;
    }

    /**
     * supports aux mechanism
     *
     * @param file
     * @return
     * @throws Exception
     */
    public static GeneSetMatrix readGeneSetMatrix(File file, boolean useCache) throws Exception {
        return readGeneSetMatrix(file.getPath(), createInputStream(file), useCache);
    }

    public static GeneSetMatrix readGeneSetMatrix(File file, boolean useCache, boolean checkforduplicates, boolean add2Cache) throws Exception {
        return readGeneSetMatrix(file.getPath(), createInputStream(file), useCache, checkforduplicates, add2Cache);
    }

    public static GeneSetMatrix readGeneSetMatrix(Element el, boolean useCache) throws Exception {
        return readGeneSetMatrix(toFile(el, Headers.GENESETMATRIX), useCache);
    }

    public static GeneSetMatrix readGeneSetMatrix(String path, InputStream is, boolean useCache) throws Exception {
        return readGeneSetMatrix(path, is, useCache, true, true);
    }

    private static GeneSetMatrix readGeneSetMatrix(String path,
            final InputStream is,
            final boolean useCache,
            final boolean checkforduplicates,
            final boolean add2Cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        // aux mechanism is a bit diff and much simpler from templates' we just strip it off
        // and register the gsetnames with the gmx -- no new objects are created
        if (AuxUtils.isAux(path)) {
            path = AuxUtils.getBasePathFromAuxPath(path);
        }

        String ext = NamingConventions.getExtension(path);
        if (ext.equals(Constants.GMT)) {
            return readGeneSetMatrixT(path, is, useCache, checkforduplicates, add2Cache);
        }

        if (ext.equals(Constants.GRP)) {
            GeneSet gset = readGeneSet(path, is, useCache, add2Cache);
            return new DefaultGeneSetMatrix(toName(path), new GeneSet[]{gset});
        }

        //log.debug("AuxStr: " + auxstr);

        if ((_getCache().isCached(path, GeneSetMatrix.class)) && (useCache)) {
            is.close();
            return (GeneSetMatrix) _getCache().get(path, GeneSetMatrix.class);
        }

        GmxParser parser = new GmxParser();
        parser.setCheckForDuplicates(checkforduplicates);
        GeneSetMatrix gmx = (GeneSetMatrix) parser.parse(toName(path), is).get(0);

        if (add2Cache) {
            _getCache().add(path, gmx, GeneSetMatrix.class);
        }

        // IMP also fetch and add all gsets in the gmx to cache
        File parentFile = new File(path).getParentFile();
        //boolean fire = false;
        for (int i = 0; i < gmx.getNumGeneSets(); i++) {
            File pseudo = new File(parentFile, gmx.getGeneSet(i).getName());

            //if (i == gmx.getNumGeneSets() - 1) {
            //   fire = true;
            //}

            if (add2Cache) {
                _getCache().addInvisibly(pseudo, gmx.getGeneSet(i));
            }

        }

        if (add2Cache) {
            _getCache().sortModel(GeneSet.class);
            // @todo fix me
            _getCache().hackAddAuxSets(gmx);
        }

        is.close();
        return gmx;
    }

    /**
     * @param file
     * @return
     * @throws Exception
     * @see readGeneSetMatrix for infor on the aux mechanims
     */
    public static GeneSetMatrix readGeneSetMatrixT(File file, boolean useCache) throws Exception {
        return readGeneSetMatrixT(file.getPath(), createInputStream(file), useCache);
    }

    public static GeneSetMatrix readGeneSetMatrixT(File file, boolean useCache, boolean checkForDuplicates, boolean add2Cache) throws Exception {
        return readGeneSetMatrixT(file.getPath(), createInputStream(file), useCache, checkForDuplicates, add2Cache);
    }

    public static GeneSetMatrix readGeneSetMatrixT(String path, InputStream is, boolean useCache) throws Exception {
        return readGeneSetMatrixT(path, is, useCache, true, true);
    }

    public static GeneSetMatrix readGeneSetMatrixT(String path,
            InputStream is,
            boolean useCache,
            boolean checkForDuplicates,
            boolean add2Cache) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        if (is == null) {
            throw new IllegalArgumentException("Param is cannot be null");
        }

        if (AuxUtils.isAux(path)) {
            path = AuxUtils.getBasePathFromAuxPath(path);
        }

        // @note as a help to make gm ext agnostic
        String ext = NamingConventions.getExtension(path);
        if (ext.equals(Constants.GMX)) {
            return readGeneSetMatrix(path, is, useCache, checkForDuplicates, add2Cache);
        }

        if (ext.equals(Constants.GRP)) {
            GeneSet gset = readGeneSet(path, is, useCache);
            return new DefaultGeneSetMatrix(toName(path), new GeneSet[]{gset});
        }

        if ((_getCache().isCached(path, GeneSetMatrix.class)) && (useCache)) {
            is.close();
            return (GeneSetMatrix) _getCache().get(path, GeneSetMatrix.class);
        }

        Parser parser = new GmtParser();
        //parser.setCheckForDuplicates(checkForDuplicates);
        GeneSetMatrix gmx = (GeneSetMatrix) parser.parse(toName(path), is).get(0);

        _getCache().add(path, gmx, GeneSetMatrix.class);

        //disabled after advance chooser
        // IMP also fetch and add all gsets in the gmx to cache
        File parentFile = new File(path).getParentFile();
        //boolean fire = false;
        for (int i = 0; i < gmx.getNumGeneSets(); i++) {
            File pseudo = new File(parentFile, gmx.getGeneSet(i).getName());

            //if (i == gmx.getNumGeneSets() - 1) {
            //  fire = true;
            //}

            //kObjectCache.add(pseudo, gmx.getGeneSet(i), GeneSet.class, fire);
            _getCache().addInvisibly(pseudo, gmx.getGeneSet(i));

        }

        //kObjectCache.sortModel(GeneSet.class);


        _getCache().hackAddAuxSets(gmx);

        is.close();
        return gmx;
    }

    // -------------------------------------------------------------------------------------------- //
    /**
     * @param file
     * @return
     * @throws Exception
     */
    public static MSigDB readMSigDB(final File file, final boolean useCache) throws Exception {
        return readMSigDB(file.getPath(), createInputStream(file), useCache, false);
    }

    public static MSigDB readMSigDB(final String sourcePath, final boolean useCache) throws Exception {
        return readMSigDB(sourcePath, createInputStream(sourcePath), useCache, false);
    }

    public static MSigDB readMSigDB(final String sourcePath, final boolean useCache, final boolean deepInitAtLoad) throws Exception {
        return readMSigDB(sourcePath, createInputStream(sourcePath), useCache, deepInitAtLoad);
    }

    public static MSigDB readMSigDB(final String path, final InputStream is, final boolean useCache, final boolean deepInitAtLoad) throws Exception {

        if (path == null) {
            throw new IllegalArgumentException("Param path cannot be null");
        }

        if ((_getCache().isCached(path, MSigDB.class)) && (useCache)) {
            return (MSigDB) _getCache().get(path, MSigDB.class);
        }

        MSigDBParser parser = new MSigDBParser();
        //parser.setDeepInitAtLoad(deepInitAtLoad);
        final MSigDB msigdb = (MSigDB) parser.parse(path, is).get(0);
        _getCache().add(path, msigdb, MSigDB.class);
        return msigdb;
    }

    // -------------------------------------------------------------------------------------------- //
    public static PersistentObject read(final String path, final InputStream is, boolean useCache) throws Exception {

        //log.debug("Reading from path: " + path);

        try {
            if (path == null) {
                throw new IllegalArgumentException("Param file cannot be null");
            }

            if (is == null) {
                throw new IllegalArgumentException("Param is cannot be null");
            }

            final String tmp = stripAt(path);
            final String ext = NamingConventions.getExtension(tmp).toLowerCase();

            if (ext.equalsIgnoreCase(RES)) {
                return readDataset(path, is, useCache);
            } else if (ext.equalsIgnoreCase(GCT)) {
                return readDatasetGct(path, is, useCache);
            } else if (ext.equalsIgnoreCase(TXT)) {
                return readDatasetTXT(path, is, useCache);
            } else if (ext.equalsIgnoreCase(MAT)) {
                return readMatrix(path, is, false);
            } else if (ext.equalsIgnoreCase(PCL)) {
                return readDatasetPcl(path, is, useCache);
            } else if (ext.equalsIgnoreCase(CDT)) {
                return readHClustDataset(path, is, useCache);
            } else if (ext.equalsIgnoreCase(GTR)) {
                return readHClustTree(path, is, useCache);
            } else if (ext.equalsIgnoreCase(ATR)) {
                return readHClustTree(path, is, useCache);
            } else if (ext.startsWith(CLS) || ext.startsWith(CLS_SNP) || ext.startsWith(CLS_CPN) || ext.startsWith(CLS_miRNA) || ext.startsWith(CLS_MET)) { // IMP note -- special for the aux hash
                return readTemplate(path, is, useCache, useCache, false);
            } else if (ext.equalsIgnoreCase(GRP)) {
                return readGeneSet(path, is, useCache);
            } else if (ext.equalsIgnoreCase(RNK)) {
                return readRankedList(path, is, useCache);
            } else if (ext.startsWith(GMX)) {// IMP note -- special for the aux hash
                return readGeneSetMatrix(path, is, useCache);
                /* dont do
                if (isAux(result.getName())) {
                String auxgsetname = getAuxOnly(result.getName());
                return gm.getGeneSet(auxgsetname);
                } else {
                return gm;
                }
                 */
            } else if (ext.startsWith(GMT)) {// IMP note -- special for the aux hash
                return readGeneSetMatrixT(path, is, useCache);
            } else if (ext.equalsIgnoreCase(DFR)) {
                return readDataframe(path, is, useCache);
            } else if (ext.equalsIgnoreCase(EDB)) {
                return readEdb(path, useCache);
            } else if (ext.equalsIgnoreCase(RPT)) {
                return readReport(path, is, useCache);
            } else if (ext.equalsIgnoreCase(CHIP) || ext.equalsIgnoreCase(CSV)) {
                return readChip(path, is, useCache);
            } else if (ext.equalsIgnoreCase(DTGDS)) {
                return readDtgDataset(path, is, useCache);
            } else if (ext.equalsIgnoreCase(GIN) || path.endsWith("gin.xls")) {
                return readGenesOfInterest(path, is, useCache);
            } else if (ext.equalsIgnoreCase(SIN) || path.endsWith("sin.xls")) {
                return readSampleAnnot(new File(path), useCache, useCache);
            } else if (ext.equalsIgnoreCase(DCHIP)) {
                return readDatasetDChip(path, is, useCache);
            } else if (ext.equalsIgnoreCase(SDF)) {
                return readStringDataframe(path, is, useCache);

            } else if (ext.equalsIgnoreCase(SNP)) {
                return readSnpDataset(path, is, useCache);
            } else if (ext.equalsIgnoreCase(NUM)) {
                return readNumDataset(path, is, useCache);
            } else {
                throw new IllegalArgumentException("Unkown file format: " + path + " no known Parser for ext: " + ext);
            }

        } catch (java.io.InterruptedIOException e) {
            klog.info("progress exception - possibly cancelled ... ignoring");
            return null;
        } finally {
            is.close();
        }
    }

    /**
     * default is to use the cache
     *
     * @param file
     * @return
     * @throws Exception
     */
    public static PersistentObject read(final File file) throws Exception {
        return read(file, true);
    }

    // IMP have to give back the  aux object if the file is an aux one
    // but have to read data from the base file
    public static PersistentObject read(final File file, final boolean useCache) throws Exception {
        File baseFile = AuxUtils.getBaseFileFromAuxFile(file);
        return read(file.getPath(), createInputStream(baseFile), useCache);
    }

    private static String stripAt(final String path) {
        StringTokenizer tok = new StringTokenizer(path, "@");
        return tok.nextToken();
    }

    public static PersistentObject[] read(File[] files) throws Exception {
        PersistentObject[] objs = new PersistentObject[files.length];
        for (int f = 0; f < files.length; f++) {
            objs[f] = read(files[f]);
        }
        return objs;
    }

    public static void save(File sourceFile, DataFormat format, File outDir) throws Exception {
        save(sourceFile, new DataFormat[]{format}, outDir);
    }

    public static void save(File sourceFile, DataFormat[] formats, File outDir) throws Exception {
        PersistentObject pob = read(sourceFile.getPath(), createInputStream(sourceFile), true);
        save(pob, formats, outDir);
    }

    public static void save(Object sourceFileOrPob, DataFormat[] formats, File outDir) throws Exception {
        if (sourceFileOrPob instanceof PersistentObject) {
            save((PersistentObject) sourceFileOrPob, formats, outDir);
        } else if (sourceFileOrPob instanceof File) {
            save((File) sourceFileOrPob, formats, outDir);
        } else {
            throw new IllegalArgumentException("Invalid object type only File or Pob allowed. Got: " + sourceFileOrPob);
        }
    }

    public static void save(Object sourceFileOrPob, DataFormat format, File outDir) throws Exception {
        save(sourceFileOrPob, new DataFormat[]{format}, outDir);
    }

    public static File[] save(PersistentObject pob, DataFormat[] formats, File outDir) throws Exception {
        File[] files = new File[formats.length];
        for (int f = 0; f < formats.length; f++) {
            Parser p = formats[f].createParser();
            if (p == null) {
                throw new IllegalArgumentException("No known parser for format: " + formats[f].getExtension());
            }

            String name = FileUtils.removeExtension(pob.getName()) + "." + formats[f].getExtension();
            files[f] = new File(outDir, name);
            p.export(pob, files[f]);
        }

        return files;
    }

    public static File save(PersistentObject pob, DataFormat format, File outDir) throws Exception {
        return save(pob, new DataFormat[]{format}, outDir)[0];
    }

    public static void saveTmp(PersistentObject pob) throws Exception {
        File file = File.createTempFile(pob.getName() + "_", "." + DataFormat.getExtension(pob));
        save(pob, file);
    }

    /**
     * auto adds appropriate extension if specified file doesnt already have it
     *
     * @param pob
     * @param file
     * @throws Exception
     * @maint add a new parser and this method needs attention
     * @todo change to use DataFormat directly to avoid else if statements
     */
    public static void save(PersistentObject pob, File file) throws Exception {

        if (pob == null) {
            throw new IllegalArgumentException("Parameter pob cannot be null");
        }

        if (file == null) {
            throw new IllegalArgumentException("Parameter file cannot be null");
        }

        if (pob instanceof Dataset) {
            file = ensureCorrectExt(file, RES);
            save((Dataset) pob, file);
        } else if (pob instanceof HClustDatasetStanford) {
            file = ensureCorrectExt(file, CDT);
            save((HClustDatasetStanford) pob, file);
        } else if (pob instanceof HClustTree) {
            file = ensureCorrectExt(file, GTR); // how to distinguish b/w atr and gtr
            save((HClustTree) pob, file);
        } else if (pob instanceof Template) {
            file = ensureCorrectExt(file, CLS);
            save((Template) pob, file);
        } else if (pob instanceof GeneSet) {
            file = ensureCorrectExt(file, GRP);
            save((GeneSet) pob, file);
        } else if (pob instanceof RankedList) {
            file = ensureCorrectExt(file, RNK);
            save((RankedList) pob, file);
        } else if (pob instanceof GeneSetMatrix) {
            file = ensureCorrectExt(file, GMX);
            save((GeneSetMatrix) pob, file);
        } else if (pob instanceof Dataframe) {
            file = ensureCorrectExt(file, DFR);
            save((Dataframe) pob, file);
        } else if (pob instanceof StringDataframe) {
            file = ensureCorrectExt(file, SDF);
            save((StringDataframe) pob, file);
        } else if (pob instanceof Report) {
            file = ensureCorrectExt(file, RPT);
            save((Report) pob, file);
        } else if (pob instanceof EnrichmentDb) {
            file = ensureCorrectExt(file, EDB);
            save((EnrichmentDb) pob, file);
        } else if (pob instanceof DtgDataset) {
            file = ensureCorrectExt(file, DTGDS);
            save((DtgDataset) pob, file);
        } else if (pob instanceof Matrix) {
            file = ensureCorrectExt(file, MAT);
            MiscParsers.save((Matrix) pob, file);
        } else if (pob instanceof RankedListDb) {
            throw new NotImplementedException();
            //file = ensureCorrectExt(file, RDB_JIT);
            //saveJIT(((RankedListDb) pob), file);
        } else if (pob instanceof EnrichmentDbMulti) {
            throw new NotImplementedException();
        } else {
            throw new IllegalArgumentException("No save method available for: " + pob.getName() + " class: " + pob.getClass());
        }
    }

    /**
     * @param file
     * @param exp_ext
     * @return
     */
    protected static File ensureCorrectExt(final File file, final String exp_ext) {
        final String ext = NamingConventions.getExtension(file);
        if (ext.equals(exp_ext)) {
            return file;
        } else {
            return new File(file.getAbsolutePath() + "." + exp_ext);
        }
    }

    /**
     * @param file
     * @param pob
     * @return
     */
    protected static File ensureCorrectExt(File file, PersistentObject pob) {
        String exp_ext = DataFormat.getExtension(pob);

        String ext = NamingConventions.getExtension(file);

        if (ext.equals(exp_ext)) {
            return file;
        } else {
            return new File(file.getAbsolutePath() + "." + exp_ext);
        }

    }

    /**
     * @param ds
     * @param toFile
     * @throws Exception
     */
    public static File save(Dataset ds, File toFile) throws Exception {

        //ResParser parser = new ResParser();
        GctParser parser = new GctParser();
        parser.export(ds, toFile);

        _getCache().add(toFile, ds, Dataset.class);

        return toFile;
    }

    /**
     * @param ds
     * @param toFile
     * @throws Exception
     */
    public static File saveGct(final Dataset ds, final File toFile) throws Exception {
        return saveGct(ds, toFile, true);
    }

    public static File saveGct(final Dataset ds, final File toFile, final boolean add2cache) throws Exception {

        Parser parser = new GctParser();

        parser.export(ds, toFile);

        if (add2cache) {
            _getCache().add(toFile, ds, Dataset.class);
        }

        return toFile;
    }

    public static File saveRes(final Dataset ds, final File toFile, final boolean add2cache) throws Exception {

        Parser parser = new ResParser();

        parser.export(ds, toFile);

        if (add2cache) {
            _getCache().add(toFile, ds, Dataset.class);
        }

        return toFile;
    }

    public static File saveCls(final String classAName, final String[] classASampleNames,
            final String classBName, final String[] classBSampleNames,
            final File toFile, final boolean add2cache) throws Exception {

        ClsBySampleNameParser parser = new ClsBySampleNameParser();

        parser.export(classAName, classASampleNames, classBName, classBSampleNames, toFile);

        if (add2cache) {
            Template t = (Template) parser.parse(toFile.getName(), createInputStream(toFile)).get(0);
            _getCache().add(toFile, t, Template.class);
        }

        return toFile;
    }

    public static File saveCls(final TemplateImplFromSampleNames tsn,
            final File toFile, final boolean add2cache) throws Exception {


        ClsBySampleNameParser parser = new ClsBySampleNameParser();

        parser.export(tsn.getClassName(0),
                tsn.getClassASampleNames(),
                tsn.getClassName(1),
                tsn.getClassBSampleNames(),
                toFile);

        if (add2cache) {
            _getCache().add(toFile, tsn, Template.class);
        }

        return toFile;
    }

    public static void savePcl(final Dataset ds, final File toFile) throws Exception {

        Parser parser = new PclParser();
        parser.export(ds, toFile);
        _getCache().add(toFile, ds, Dataset.class);
    }

    /*
    public static File saveJIT(final RankedListDb rdb, final File toFile) throws Exception {
    return saveJIT(rdb, toFile, true);
    }

    public static File saveJIT(final RankedListDb rdb, final File toFile, final boolean add2cache) throws Exception {

    Parser parser = new RankedListDbParser();

    parser.export(rdb, toFile);

    if (add2cache) {
    _cache().add(toFile, rdb, RankedListDb.class);
    }

    return toFile;
    }
     */
    public static void save(final Chip chip, final File toFile) throws Exception {

        Parser parser = new ChipParser();
        parser.export(chip, toFile);
        _getCache().add(toFile, chip, Chip.class);
    }

    public static void saveInvisibly2CacheGct(Dataset ds, File toFile) throws Exception {

        Parser parser = new GctParser();

        parser.export(ds, toFile);

        _getCache().addInvisibly(toFile, ds);

    }

    public static void saveTxt(Dataset ds, File toFile) throws Exception {

        Parser parser = new TxtDatasetParser();
        parser.export(ds, toFile);
        _getCache().add(toFile, ds, Dataset.class);

    }

    public static void saveInvisibly2CacheTxt(Dataset ds, File toFile) throws Exception {

        Parser parser = new TxtDatasetParser();
        parser.export(ds, toFile);
        _getCache().addInvisibly(toFile, ds);
    }

    public static void saveDChip(Dataset ds, File toFile) throws Exception {

        Parser parser = new DChipDatasetParser();
        parser.export(ds, toFile);
        _getCache().add(toFile, ds, Dataset.class);

    }

    public static void saveInvisibly2CacheDChip(Dataset ds, File toFile) throws Exception {

        Parser parser = new DChipDatasetParser();
        parser.export(ds, toFile);
        _getCache().addInvisibly(toFile, ds);

    }

    /**
     * @param dtgds
     * @param toFile
     * @throws Exception
     */
    public static void save(DtgDataset dtgds, File toFile) throws Exception {

        Parser parser = new DtgDatasetParser();
        parser.export(dtgds, toFile);
        _getCache().add(toFile, dtgds, DtgDataset.class);

    }

    /**
     * @param cdt
     * @param toFile
     * @throws Exception
     */
    public static void save(HClustDatasetStanford cdt, File toFile) throws Exception {

        Parser parser = new HClustDatasetParser();
        parser.export(cdt, toFile);
        _getCache().add(toFile, cdt, HClustDatasetStanford.class);

    }

    public static void save(HClustTree tree, File toFile) throws Exception {

        Parser parser = new HClustTreeParser();
        parser.export(tree, toFile);
        _getCache().add(toFile, tree, HClustTree.class);

    }

    public static void save(final IDataframe idf, final File toFile) throws Exception {

        if (idf == null) {
            throw new IllegalArgumentException("Param idf cannot be null");
        }

        if (idf instanceof Dataframe) {
            save((Dataframe) idf, toFile, false);
        } else if (idf instanceof StringDataframe) {
            save((StringDataframe) idf, toFile);
        } else if (idf instanceof RichDataframe) {
            StringDataframe sdf = (StringDataframe) ((RichDataframe) idf).getDataframe();
            save(sdf, toFile);
        } else { /// @note not sure if this works yet
            throw new NotImplementedException("class was for: " + idf.getClass());
        }

    }

    public static void saveInvisibly2Cache(IDataframe idf, File toFile) throws Exception {
        if (idf instanceof Dataframe) {
            saveInvisibly2Cache((Dataframe) idf, toFile);
        } else if (idf instanceof StringDataframe) {
            saveInvisibly2Cache((StringDataframe) idf, toFile);
        } else {
            throw new NotImplementedException();
        }
    }

    /**
     * @param df
     * @param toFile
     * @throws Exception
     */
    public static void save(final Dataframe df, final File toFile, final boolean add2Cache) throws Exception {

        Parser parser = new DataframeParser();

        parser.export(df, toFile);

        if (add2Cache) {
            _getCache().add(toFile, df, Dataframe.class);
        }
    }

    public static void saveInvisibly2Cache(final Dataframe df, final File toFile) throws Exception {
        Parser parser = new DataframeParser();
        parser.export(df, toFile);
        _getCache().addInvisibly(toFile, df);
    }

    public static void saveWithRowDescs(final Dataframe df, final List rowdescs, final File toFile) throws Exception {

        DataframeParser parser = new DataframeParser();

        parser.exportWithDescs(df, rowdescs, toFile);

        _getCache().add(toFile, df, Dataframe.class);
    }

    /**
     * @param dss
     * @param baseDir
     * @throws Exception
     */
    public static void save(final Dataset[] dss, final File baseDir) throws Exception {

        for (int i = 0; i < dss.length; i++) {
            save(dss[i], new File(baseDir, dss[i].getName() + "." + RES));
        }
    }

    /**
     * @param edb
     * @param toFile
     * @throws Exception
     */
    public static void save(final EnrichmentDb edb, final File toFile) throws Exception {

        Parser parser = new EdbFolderParser();

        parser.export(edb, toFile);

        _getCache().add(toFile, edb, EnrichmentDb.class);
    }

    public static void saveInvisibly2Cache(final EnrichmentDb edb, final File toFile) throws Exception {

        Parser parser = new EdbFolderParser();

        parser.export(edb, toFile);

        _getCache().addInvisibly(toFile, edb);
    }

    // -------------------------------------------------------------------------------------------- //
    public static void save(final RankedList rl, final File toFile) throws Exception {
        save(rl, toFile, false);
    }

    public static void save(final RankedList rl, final File toFile, final boolean add2cache) throws Exception {

        Parser parser = new RankedListParser();

        parser.export(rl, toFile);

        if (add2cache) {
            _getCache().add(toFile, rl, RankedList.class);
        }
    }

    /**
     * @param template
     * @param toFile
     * @throws Exception
     */
    public static void save(final Template template, final File toFile) throws Exception {
        save(template, false, toFile);
    }

    public static void save(final Template template, final boolean gcFormat, final File toFile) throws Exception {
        ClsParser parser = new ClsParser();
        parser.export(template, gcFormat, toFile);
        _getCache().add(toFile, template, Template.class);
    }

    public static void save(final Template template, final File toFile, final boolean add2cache) throws Exception {
        ClsParser parser = new ClsParser();
        parser.export(template, toFile);
        if (add2cache) {
            _getCache().add(toFile, template, Template.class);
        }
    }

    public static void saveInvisibly2Cache(final Template template, final File toFile) throws Exception {

        Parser parser = new ClsParser();

        parser.export(template, toFile);

        _getCache().addInvisibly(toFile, template);

    }

    /**
     * @param tss
     * @param baseDir
     * @throws Exception
     */
    public static void save(Template[] tss, File baseDir) throws Exception {

        for (int t = 0; t < tss.length; t++) {
            save(tss[t], new File(baseDir, tss[t].getName() + "." + CLS));
        }
    }

    /**
     * @param gset
     * @param toFile
     * @throws Exception
     */
    public static void save(GeneSet gset, File toFile) throws Exception {

        Parser parser = new FSetParser();

        parser.export(gset, toFile);

        _getCache().add(toFile, gset, GeneSet.class);

    }

    // ------------------------------------------------------------------------------------ //
    // ------------------------- GENE MATRIX SAVING METHODS ------------------------------- //
    // ------------------------------------------------------------------------------------ //
    /// cant use df due to unit loaded violations
    public static void saveGmt(GeneSetMatrix gmt, File toFile) throws Exception {
        Parser parser = new GmtParser();
        parser.export(gmt, toFile);
        _getCache().add(toFile, gmt, GeneSetMatrix.class);
    }

    public static void saveGmt(GeneSetMatrix gmt, File toFile, boolean add2cache) throws Exception {
        Parser parser = new GmtParser();
        parser.export(gmt, toFile);
        if (add2cache) {
            _getCache().add(toFile, gmt, GeneSetMatrix.class);
        }
    }

    /**
     * @param gmx
     * @param toFile
     * @throws Exception
     */
    public static void save(GeneSetMatrix gmx, File toFile) throws Exception {
        save(gmx, toFile, true);
    }

    public static void save(GeneSetMatrix gmx, File toFile, boolean add2cache) throws Exception {
        Parser parser = new GmxParser();
        parser.export(gmx, toFile);
        if (add2cache) {
            _getCache().add(toFile, gmx, GeneSetMatrix.class);
        }
    }

    // ------------------------------------------------------------------------------------ //
    // ------------------------- END GENE MATRIX SAVING METHODS --------------------------- //
    // ------------------------------------------------------------------------------------ //
    public static void save(Report rpt, File toFile) throws Exception {
        save(rpt, toFile, true);
    }

    public static void save(Report rpt, File toFile, boolean add2cache) throws Exception {
        Parser parser = new ReportParser();
        parser.export(rpt, toFile);
        if (add2cache) {
            _getCache().add(toFile, rpt, Report.class);
        }
    }

    public static void save(StringDataframe sdf, File toFile) throws Exception {
        save(sdf, toFile, true);
    }

    public static void save(StringDataframe sdf, File toFile, boolean add2cache) throws Exception {
        Parser parser = new StringDataframeParser();
        parser.export(sdf, toFile);
        if (add2cache) {
            _getCache().add(toFile, sdf, StringDataframe.class);
        }
    }

    public static void saveInvisibly2Cache(StringDataframe sdf, File toFile) throws Exception {
        Parser parser = new StringDataframeParser();
        parser.export(sdf, toFile);
        _getCache().addInvisibly(toFile, sdf);
    }

    public static Template findTemplate(String name, final Template[] tss) {

        // for cases where somehow it becomes: test.cls#test.cls
        if (AuxUtils.getAuxNameOnlyNoHash(name).equals(AuxUtils.getBaseStringFromAux(name))) {
            name = AuxUtils.getAuxNameOnlyNoHash(name);
        }

        if (AuxUtils.isAux(name) && name.endsWith(".cls")) { // hack for coo.cls@a_vs_b.cls
            name = name.substring(0, name.length() - 4);
        }

        for (int t = 0; t < tss.length; t++) {
            if (tss[t].getName().equals(name)) {
                return tss[t];
            }

            if (tss[t].isContinuous()) {
                String name_no_aux = AuxUtils.getAuxNameOnlyNoHash(name);
                if (tss[t].getName().equals(name_no_aux)) {
                    return tss[t];
                }
            }
        }

        StringBuffer buf = new StringBuffer("<html>\n" +
                "<body>\n" +
                "<p>No template for name: " + name + "</p>" +
                "<p>The Available templates are:</p>\n");
        for (int i = 0; i < tss.length; i++) {
            buf.append(tss[i].getName()).append("<br>\n");
        }
        buf.append("</body></html>");
        throw new IllegalArgumentException(buf.toString());
    }

    // @note convention
    private static String toName(final String path) {
        return new File(path).getName();
    }

    // if file doesnt exsits or is a dir then error out with an intuitive message
    // useful before using a new FileInputStream(f) as the error message from that is not
    // very explanatory
    private static InputStream createInputStream(File file) throws IOException {

        String path = file.getPath();

        // @note as a convebicne auto detect of the file is really a URL
        if (NamingConventions.isURL(file.getPath())) {
            return createInputStream(new URL(file.getPath()));
        }

        if (AuxUtils.isAuxFile(file)) {
            //klog.debug("Auto UNauxing file: " + file);
            file = AuxUtils.getBaseFileFromAuxFile(file);
        }

        if (!file.exists()) {
            throw new FileNotFoundException("File not found: " + file.getAbsolutePath());
        }

        if (file.isDirectory()) {
            throw new IOException("Specified path is a Directory (expecting a File): " + file.getAbsolutePath());
        }

        if (!file.canRead()) {
            throw new IOException("Check file permissions - cannot read data from file: " + file.getAbsolutePath());
        }

        if (kProgressMonitorHook != null) {
            return kProgressMonitorHook.getProgressMonitorInputStream(file, "Parsing: " + file.getPath());
        } else {
            return new FileInputStream(file);
        }
    }

    private static InputStream createInputStream(final URL url) throws IOException {
        klog.debug("Parsing URL: " + url.getPath() + " >> " + url.toString());

        if (kProgressMonitorHook != null) {
            return kProgressMonitorHook.getProgressMonitorInputStream(url, "Parsing URL: " + url.getPath());
        } else if (url.getProtocol().equalsIgnoreCase("ftp") && url.getHost().equalsIgnoreCase(GseaWebResources.getGseaFTPServer())) {
            try {
                return (FTPUtils.ftpDownloadAndCreateInputStream(url));
            }
            catch (Exception e) {
                throw new IOException(e);
            }
        } else {
            return url.openStream();
        }
    }

    public static InputStream createInputStream(final Object source) throws IOException {

        if (source == null) {
            throw new IllegalArgumentException("Parameter source cannot be null");
        }

        if (source instanceof File) {
            return createInputStream((File) source);
        } else if (source instanceof URL) {
            return createInputStream((URL) source);
        }

        String path = source.toString();


        if (NamingConventions.isURL(path)) {
            // common error is ftp.broad... while it should be ftp://ftp.broad...

            if (path.startsWith("ftp.")) {
                path = "ftp://" + path;
            } else if (path.startsWith("gseaftp.")) {
                path = "ftp://" + path;
            }

            return createInputStream(new URL(path));
        }

        // Ok, it might be a file

        File file = new File(path);

        if (file.exists()) {
            return createInputStream(file);
        }

        /*
          klog.debug("Reading url from: " + path);
          URL url = new URL(path);
          return createInputStream(url);
        */

        throw new IOException("Bad data source -- neither file nor url exists for: " + source);
    }
    
    public static InputStream createInputStreamforChip(final Object source) throws IOException {

        if (source == null) {
            throw new IllegalArgumentException("Parameter source cannot be null");
        }

        if (source instanceof File) {
            return createInputStream((File) source);
        } else if (source instanceof URL) {
            return createInputStream((URL) source);
        }

        String path = source.toString();
        System.out.println("path: "+path);


        if (NamingConventions.isURL(path)) {
            // common error is ftp.broad... while it should be ftp://ftp.broad...

            if (path.startsWith("ftp.")) {
                path = "ftp://" + path;
            } else if (path.startsWith("gseaftp.")) {
                path = "ftp://" + path;
            }
            return createInputStream(new URL(path));
        }

        // Ok, it might be a file

        File file = new File(path);

        if (file.exists()) {
            return createInputStream(file);
        }

        /*
        klog.debug("Reading url from: " + path);
        URL url = new URL(path);
        return createInputStream(url);
         */

        throw new IOException("Bad data source -- neither file nor url exists for: " + source);
    }    

    protected static File toFile(Element el, String attrName) {
        if (el == null) {
            throw new IllegalArgumentException("Parameter el cannot be null");
        }

        if (attrName == null) {
            throw new IllegalArgumentException("Parameter attrName cannot be null");
        }

        Attribute attr = el.attribute(attrName);

        if (attr == null) {
            throw new IllegalArgumentException("Element specified: " + el.getName() + " has no attribute called: " + attrName + "\nText: " + el.getText());
        }

        String path = attr.getValue();
        if (path == null) {
            throw new IllegalArgumentException("Element had no value for attribute_name: " + Headers.DATASET);
        } else {
            return new File(path);
        }
    }

    /**
     * Interface to define a callback for the pis
     */
    public static interface ProgressMonitorHook {

        public ProgressMonitorInputStream getProgressMonitorInputStream(File file, String msg) throws IOException;

        public ProgressMonitorInputStream getProgressMonitorInputStream(URL url, String msg) throws IOException;

        public OutputStream getProgressMonitorOutputStream(File file, String msg) throws IOException;

        public OutputStream getProgressMonitorOutputStream(URL url, String msg) throws IOException;
    }

    public static Runnable createImportTask(final File[] files) {
        return new Runnable() {

            public void run() {
                final StringBuffer buf_s = new StringBuffer("Loading ... " + files.length + " files\n\n"); // Uggghh
                final StringBuffer buf_h = new StringBuffer("Loading ... " + files.length + " files<br>");
                int sucess = 0;

                final Errors errors = new Errors();
                for (int f = 0; f < files.length; f++) {
                    if (files[f].isDirectory()) {
                        errors.add(new RuntimeException("Only files can be choosen - a directory was specified: " + files[f].getPath()));
                    } else {
                        try {
                            final FileInputStream in = new FileInputStream(files[f]);
                            final ProgressMonitorInputStream2 pis = new ProgressMonitorInputStream2(in);
                            ParserFactory.read(files[f].getPath(), pis, true);    // just gets placed in memory, no other thing
                            klog.info("Loaded file: " + files[f].getPath());
                            Application.getFileManager().registerRecentlyOpenedFile(files[f]);
                            buf_h.append(files[f].getName()).append("<br>");
                            buf_s.append(files[f].getName()).append("\n");
                            sucess++;
                        } catch (Throwable t) {
                            errors.add("Parsing trouble", t);
                        }
                    }
                }

                buf_h.append("<br>Files loaded successfully: ").append(sucess).append(" / ").append(files.length);
                buf_s.append("\nFiles loaded successfully: ").append(sucess).append(" / ").append(files.length).append('\n');
                buf_h.append("<br><hr><br>");
                if (errors.isEmpty()) {
                    buf_s.append("There were NO errors");
                    Application.getWindowManager().showMessage(buf_s.toString());
                } else {
                    Application.getWindowManager().showError(errors);
                }
            }
        };
    }
}    // End ParserFactory

