/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.StringDataframe;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Parses a StringStringDataframe
 * Format:
 * <p/>
 * row_names        col_0 col_1 col_2 ...
 * row_0                    34.5  45.6  56.8
 * row_1                    ...   ...   ...
 * row_2                    ...   ...   ...
 * ...
 * <p/>
 * Notes:
 * <p/>
 * NO desc fields
 * <p/>
 * First row is the column header
 * First column are the row names
 * Whitespace delimited - row names and col names cannot have whitespace
 * num of rows line is not required
 * scan line is not required
 * no A/P calls
 */


public class StringDataframeParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public StringDataframeParser() {
        super(StringDataframe.class);
    }

    /**
     * Export a StringDataframe to file in StringDataframe format
     * Only works with StringDataframes
     *
     * @see "Above for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {

        PrintWriter pw = startExport(pob, file);
        _export(pob, pw);

    }

    public void export(final PersistentObject pob, final OutputStream os) throws Exception {
        PrintWriter pw = startExport(pob, os, null);
        _export(pob, pw);
    }

    // does the real stuff
    private void _export(final PersistentObject pob, final PrintWriter pw) throws Exception {

        final StringDataframe sdf = (StringDataframe) pob;

        String s = sdf.getRowLabelName();
        if (s == null) {
            s = Headers.NAME;
        }

        pw.print(s + "\t");

        for (int i = 0; i < sdf.getNumCol(); i++) {
            pw.print(sdf.getColumnName(i));
            pw.print('\t');
        }

        pw.println();

        for (int r = 0; r < sdf.getNumRow(); r++) {
            pw.print(sdf.getRowName(r));
            pw.print('\t');
            String[] row = sdf.getRow(r);
            for (int c = 0; c < row.length; c++) {
                pw.print(row[c]);
                pw.print('\t');
            }
            pw.println();
        }

        pw.close();
        doneExport();

    }    // End export

    /**
     * @param name
     * @param s
     * @return
     * @throws Exception
     */
    public StringDataframe parse(String name, String s) throws Exception {
        startImport(name);
        StringReader sr = new StringReader(s);
        BufferedReader bin = new BufferedReader(sr);
        return parseSdf(name, bin);
    }

    /**
     * Parse a StringDataframe from memory
     *
     * @param s
     * @return
     * @throws Exception
     */
    public StringDataframe parse(String s) throws Exception {
        String name = NamingConventions.generateName(Constants.DFR);
        return parse(name, s);
    }

    /**
     * @returns 1 StringDataframe object
     * NO ann buiisness
     * @see above for format
     */
    public List parse(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);
        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        StringDataframe sdf = parseSdf(sourcepath, bin);
        return unmodlist(new PersistentObject[]{sdf});

    }

    /// does the real parsing
    // expects the bin to be untouched
    protected StringDataframe parseSdf(String objname, BufferedReader bin) throws Exception {
        return parseSdf(objname, bin, nextNonEmptyLine(bin));
    }

    public StringDataframe parseSdf(File file) throws Exception {
        BufferedReader bin = new BufferedReader(new FileReader(file));
        return parseSdf(file.getName(), bin, nextNonEmptyLine(bin));
    }

    protected StringDataframe parseSdf(String objname, BufferedReader bin, String firstLine) throws Exception {
        String currLine = firstLine;

        // 1st  non-empty, non-comment line is the col header:
        // First fields is to be ignored
        List colNames = ParseUtils.string2stringsList(currLine, "\t");

        //log.debug("# cols found: " + colnames.size());

        colNames.remove(0);                                 // first elem is always nonsense

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List lines = new ArrayList();

        //currLine = nextLine(bin);
        currLine = nextLineTrimless(bin); // @note Modified oct 25 2005

        // save all rows so that we can determine how many rows exist
        while (currLine != null) {
            lines.add(currLine);
            // @note Modified oct 25 2005
            //currLine = nextLine(bin);
            currLine = nextLineTrimless(bin); /// so that last col(s) can be a tab
        }

        StringDataframe sdf = _parse(objname, lines, colNames);

        bin.close();

        doneImport();
        return sdf;
    }

    // first line must have col names
    public StringDataframe parseSdf(final String name, final List lines) throws Exception {

        // 1st  non-empty, non-comment line is the col header:
        // First fields is to be ignored
        List colNames = ParseUtils.string2stringsList(lines.get(0).toString(), "\t");
        //log.debug("# cols found: " + colnames.size());
        colNames.remove(0);                                 // first elem is always nonsense
        lines.remove(0); // @note in list is being modified for perf

        return _parse(name, lines, colNames);
    }

    private StringDataframe _parse(String objname, List lines, List colNames) throws Exception {

        StringMatrix matrix = new StringMatrix(lines.size(), colNames.size());
        List rowNames = new ArrayList();
        String nstr = null;

        for (int i = 0; i < lines.size(); i++) {
            String currLine = (String) lines.get(i);

            //List fields = ParseUtils.string2stringsV2_list(currLine); // comm out fpor allowing empty cols, Oct 25, 2005
            List fields = string2stringsV2(currLine, colNames.size() + 1); // + 1 for the name col


            if (fields.size() != colNames.size() + 1) {
                throw new ParserException("Bad format - expect ncols: " + (colNames.size() + 1)
                        + " but found: " + fields.size() + " on line: "
                        + currLine);
            }

            String rowname = (String) fields.get(0);

            rowNames.add(rowname);
            for (int f = 1; f < fields.size(); f++) {
                String elem = fields.get(f).toString();
                if (elem.equalsIgnoreCase(Constants.NULL)) {
                    matrix.setElement(i, f - 1, nstr);
                } else {
                    matrix.setElement(i, f - 1, elem);
                }
            }

        }

        //log.info("Completed parsing StringDataframe");

        StringDataframe sdf = new StringDataframe(objname, matrix, rowNames, colNames, true);
        sdf.addComment(fComment.toString());

        return sdf;

    }

}    // End of class StringDataframeParser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
