package edu.mit.broad.genome.parsers;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.strucs.TxtAffy;

import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Dec 2, 2005
 *         Time: 4:03:20 PM
 */
public class TxtAffyParser extends AbstractParser {

    private final static String PROBE_SET_NAME = "Probe Set Name";

    private final static String SIGNAL = "Signal";

    private final static String DETECTION_PVALUE = "Detection p-value";

    private final static String DETECTION = "Detection";

    /**
     * Class Constructor.
     */
    public TxtAffyParser() {
        super(TxtAffy.class);
    }

    /**
     * Export a Dataset to file in pds format
     * Only works with Datasets
     *
     * @see "Above for format"
     */
    public void export(PersistentObject pob, File file) throws Exception {
        throw new NotImplementedException();
    }    // End export


    /**
     */
    public List parse(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        return _parse(sourcepath, bin);

    }

    /// does the real parsing
    // expects the bin to be untouched
    // @note added support for non affy txt files -> simply tab delimited txt no headers ect
    private List _parse(String objname, BufferedReader br) throws Exception {

        //System.out.println("Doing: " + objname);

        String firstLine = br.readLine(); // ignore 1st line // for txt affy: Expression Analysis: Metrics Tab

        boolean true_affy_txt = false;

        if (firstLine.toUpperCase().startsWith("DESCRIPTION")) {
            return _parse_joshs_mas5_txt(objname, firstLine, br);
        }

        if (firstLine.toUpperCase().startsWith("^SAMPLE")) {
            return _parse_geo_sample_soft(objname, firstLine, br);
        }

        List header = Arrays.asList(firstLine.split("\t"));
        if (header.indexOf("Probe Set") != -1 || header.indexOf(PROBE_SET_NAME) != -1) {
            return _parse_true_affy_txt(objname, firstLine, br);
        }

        if (firstLine.toUpperCase().startsWith("EXPRESSION")) {
            true_affy_txt = true;
        }

        final String[] fields = ParseUtils.string2strings(firstLine, "\t", false); // DONT USE SPACES
        if (fields.length < 2) {
            true_affy_txt = true;
        }

        if (!true_affy_txt) {
            return _parse_simple_only_txt(objname, firstLine, br);
        }


        return _parse_true_affy_cx(objname, firstLine, br); // complex affy
    }

    /**
     * Description	Accession	./DLBC41.CEL
     * 7129
     * AFFX-BioB-5_at	15.651	A
     * AFFX-BioB-M_at	8.6215	A
     * AFFX-BioB-3_at	20.9724	A
     * AFFX-BioC-5_at	199.384	A
     * AFFX-BioC-3_at	53.4667
     * <p/>
     * ...
     * ...
     *
     * @param objname
     * @param firstLine
     * @param buf
     * @return
     * @throws Exception
     */
    private List _parse_joshs_mas5_txt(String objname, String firstLine, BufferedReader buf) throws Exception {

        List strucs = new ArrayList();
        objname = NamingConventions.removeExtension(new File(objname).getName());

        firstLine = nextLine(buf); // 7129 (etc, ignore it)

        firstLine = nextLine(buf);// the data

        //float cnt = 0;
        while (firstLine != null) {

            final String[] fields = ParseUtils.string2strings(firstLine, "\t", false); // DONT USE SPACES
            if (fields.length < 2) {
                throw new ParserException("Bad file format exception - expected at least 2 fields but got: " + fields.length + " line>" + firstLine + "<");
            }

            final String probeSet = fields[0];
            final float signal = Float.parseFloat(fields[1]);

            char call;
            if (fields.length >= 3) {
                call = fields[2].charAt(0);
            } else {
                call = Character.UNASSIGNED;
            }

            strucs.add(new TxtAffy.Value(probeSet, call, Float.NaN, signal));
            firstLine = nextLine(buf);
        }

        buf.close();

        doneImport();

        TxtAffy ta = new TxtAffy(objname, (TxtAffy.Value[]) strucs.toArray(new TxtAffy.Value[strucs.size()]));
        return unmodlist(new PersistentObject[]{ta});
    }

    /**
     * ^SAMPLE
     * ...
     * !sample_table_begin
     * ID_REF	VALUE	ABS_CALL
     * 1007_s_at	8.745416
     * ...
     *
     * @param objname
     * @param firstLine
     * @param buf
     * @return
     * @throws Exception
     */
    private List _parse_geo_sample_soft(String objname, String firstLine, BufferedReader buf) throws Exception {

        List strucs = new ArrayList();
        objname = NamingConventions.removeExtension(new File(objname).getName());


        while (firstLine != null) {
            if (firstLine.toUpperCase().startsWith("!SAMPLE_TABLE_BEGIN")) {
                break;
            }
            firstLine = buf.readLine();
        }

        firstLine = nextLine(buf); // ID_REF	VALUE	ABS_CALL

        firstLine = nextLine(buf);// the data

        //float cnt = 0;
        while (firstLine != null) {

            final String[] fields = ParseUtils.string2strings(firstLine, "\t", false); // DONT USE SPACES
            if (fields.length < 2) {
                throw new ParserException("Bad file format exception - expected at least 2 fields but got: " + fields.length + " line>" + firstLine + "<");
            }

            final String probeSet = fields[0];
            final float signal = Float.parseFloat(fields[1]);

            char call;
            if (fields.length >= 3) {
                call = fields[2].charAt(0);
            } else {
                call = Character.UNASSIGNED;
            }

            strucs.add(new TxtAffy.Value(probeSet, call, Float.NaN, signal));

            //System.out.println(">> " + probeSet + " " + call + " " +  signal);
            firstLine = nextLine(buf);
        }

        buf.close();

        doneImport();

        TxtAffy ta = new TxtAffy(objname, (TxtAffy.Value[]) strucs.toArray(new TxtAffy.Value[strucs.size()]));
        return unmodlist(new PersistentObject[]{ta});
    }

    /**
     * No headers
     * Always: probe_name\t\value\pcall (optional)
     * <p/>
     * 100001_at	3.0
     * 100002_at	29.0
     * 100003_at	28.0
     * 100004_at	510.0
     * 100005_at	440.0
     *
     * @param objname
     * @param firstLine
     * @param br
     * @return
     */
    private List _parse_simple_only_txt(String objname, String firstLine, BufferedReader buf) throws Exception {

        List strucs = new ArrayList();
        objname = NamingConventions.removeExtension(new File(objname).getName());

        //float cnt = 0;
        while (firstLine != null) {

            final String[] fields = ParseUtils.string2strings(firstLine, "\t", false); // DONT USE SPACES
            if (fields.length < 2) {
                throw new ParserException("Bad file format exception - expected at least 2 fields but got: " + fields.length + " line>" + firstLine + "<");
            }

            final String probeSet = fields[0];
            final float signal = Float.parseFloat(fields[1]);

            char call;
            if (fields.length >= 3) {
                call = fields[2].charAt(0);
            } else {
                call = Character.UNASSIGNED;
            }

            strucs.add(new TxtAffy.Value(probeSet, call, Float.NaN, signal));
            firstLine = nextLine(buf);
        }

        buf.close();

        doneImport();

        TxtAffy ta = new TxtAffy(objname, (TxtAffy.Value[]) strucs.toArray(new TxtAffy.Value[strucs.size()]));
        return unmodlist(new PersistentObject[]{ta});

    }

    private List _parse_true_affy_cx(String objName, String firstLine, BufferedReader br) throws Exception {

        while (firstLine != null) {
            if (firstLine.toUpperCase().startsWith("EXPRESSION")) {
                break;
            }
            firstLine = br.readLine();
        }

        //String headerLine = nextLine(br);

        String headerLine = br.readLine();
        if (headerLine.trim().length() == 0) {
            headerLine = br.readLine();
        }

        return _parse_true_affy_txt(objName, headerLine, br);
    }

    private List _parse_true_affy_txt(String objName, String headerLine, BufferedReader br) throws Exception {

        if (headerLine == null) {
            throw new IOException("Header line is null.");
        }
        List header = Arrays.asList(headerLine.split("\t"));

        int probeSetColumn = header.indexOf(PROBE_SET_NAME);

        if (probeSetColumn == -1) {
            probeSetColumn = header.indexOf("Probe Set");
        }

        if (probeSetColumn == -1) {
            throw new IOException(PROBE_SET_NAME + " column not found. File: " + objName);
        }

        int signalColumn = header.indexOf(SIGNAL);
        if (signalColumn == -1) {
            throw new IOException(SIGNAL + " column not found. File: " + objName);
        }

        int dpValueColumn = header.indexOf(DETECTION_PVALUE);
        if (dpValueColumn == -1) {
            throw new IOException(DETECTION_PVALUE + " column not found. File: " + objName);
        }

        int callColumn = header.indexOf(DETECTION); // A or P or M
        if (callColumn == -1) {
            throw new IOException(DETECTION + " column not found. File: " + objName);

        }

        String s;
        List strucs = new ArrayList();
        while ((s = br.readLine()) != null) {
            final String[] tokens = s.split("\t");
            final String probeSet = tokens[probeSetColumn];
            final float signal = Float.parseFloat(tokens[signalColumn]);
            final char call = tokens[callColumn].charAt(0);
            final float dpvalue = Float.parseFloat(tokens[dpValueColumn]);
            strucs.add(new TxtAffy.Value(probeSet, call, dpvalue, signal));
        }

        doneImport();

        TxtAffy ta = new TxtAffy(objName, (TxtAffy.Value[]) strucs.toArray(new TxtAffy.Value[strucs.size()]));
        return unmodlist(new PersistentObject[]{ta});
    }

} // End class TxtAffyParser
