/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.Version;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.vdb.meg.Unigene;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.util.*;

/**
 *
 */


public class UnigeneParser extends AbstractParser {

    private static final String ID = "ID";
    private static final String SYMBOL = "GENE";
    private static final String TITLE = "TITLE";
    private static final String CYTOBAND = "CYTOBAND";
    private static final String LOCUSLINK = "LOCUSLINK";
    private static final String SEQUENCE = "SEQUENCE";

    /**
     * Class Constructor.
     */
    public UnigeneParser() {
        super(Unigene.class);
    }

    /**
     * Export a Dataset to file in gct format
     * Only works with Datasets
     *
     * @see "Below for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {
        throw new NotImplementedException();

    }    // End export

    public List parse(String sourcepath, InputStream is) throws Exception {
        return unmodlist(parseUnigene(new File(sourcepath), true)); // @note default
    }

    //------------------------------------------------------------------------//
    //-------------------- PARSING STUFF -------------------------------------//
    //------------------------------------------------------------------------//

    /**
     * //
     * ID          Hs.25022
     * DEFAULT_TITLE       growth differentiation factor 9
     * GENE        GDF9
     * CYTOBAND    5q31.1
     * LOCUSLINK   2661
     * HOMOL       1
     * EXPRESS     whole brain ; pooled ; testis_normal ; adrenal adenoma
     * GNM_TERMINUS      S
     * CHROMOSOME  5
     * PROTSIM     ORG=Caenorhabditis elegans; PROTGI=11275204; PROTID=pir:T43286; PCT=31; ALN=162
     * PROTSIM     ORG=Drosophila melanogaster; PROTGI=118409; PROTID=sp:P07713; PCT=32; ALN=102
     * PROTSIM     ORG=Homo sapiens; PROTGI=4885261; PROTID=ref:NP_005251.1; PCT=100; ALN=454
     * PROTSIM     ORG=Mus musculus; PROTGI=585195; PROTID=sp:Q07105; PCT=71; ALN=454
     * PROTSIM     ORG=Rattus norvegicus; PROTGI=11067393; PROTID=ref:NP_067704.1; PCT=72; ALN=454
     * SCOUNT      6
     * SEQUENCE    ACC=BX117145.1; NID=g27840413; CLONE=IMAGp998L13166_,_IMAGE:38339; LID=263; SEQTYPE=EST
     * SEQUENCE    ACC=NM_005260.2; NID=g6715598; PID=g4885261; SEQTYPE=mRNA
     * SEQUENCE    ACC=AW770109.1; NID=g7702148; CLONE=IMAGE:2978938; END=3'; LID=1042; SEQTYPE=EST
     * SEQUENCE    ACC=BF886517.1; NID=g12277613; LID=8536; SEQTYPE=EST
     * SEQUENCE    ACC=CA488820.1; NID=g24951605; CLONE=IMAGE:6721013; END=5'; LID=11244; SEQTYPE=EST
     * SEQUENCE    ACC=AA604716.1; NID=g2445580; CLONE=IMAGE:1112760; END=3'; LID=927; SEQTYPE=EST
     * //
     * ID          Hs.25025
     * DEFAULT_TITLE       Homo sapiens transcribed sequences
     * EXPRESS     whole brain ; colon
     *
     * @param file
     * @return
     * @throws Exception
     */
    // dont bother with memory stuff as this is run offline
    public Unigene parseUnigene(final File file, final boolean onlyAddMrnaAccessions) throws Exception {

        final BufferedReader buf = ParseUtils.buf(file);

        List lines = null;
        List records = new ArrayList();
        int linenum = 0;
        String currLine = ParseUtils.nextLine(buf);

        boolean startNewBlock = true;

        while (currLine != null) {
            //System.out.println(line);
            linenum++;

            // sanity checks
            if (startNewBlock && currLine.startsWith(ID) == false) {
                throw new IllegalArgumentException("Bad format pn line, expected : " + ID + " but got >" + currLine + "< > linenum: " + linenum);
            }

            if (currLine.startsWith(ID)) {
                lines = new ArrayList();
                startNewBlock = false;
            }
            if (currLine.equals("//")) {
                Unigene.Record recd = parseOneRecord(lines, onlyAddMrnaAccessions);
                records.add(recd);
                if (records.size() % 10000 == 0) {
                    System.out.println("Done unigene record #: " + records.size());
                }
                startNewBlock = true;
            }

            if (lines != null) {
                lines.add(currLine);
            }
            currLine = ParseUtils.nextLine(buf);
        }


        log.info("Total # of unigene records: " + records.size());

        return new Unigene(file, new Version("Oct2005"), (Unigene.Record[]) records.toArray(new Unigene.Record[records.size()]));
    }

    private static Unigene.Record parseOneRecord(List lines, boolean onlyAddMrnaAccessions) throws Exception {

        if (lines == null || lines.isEmpty()) {
            throw new IllegalArgumentException("Parameter lines cannot be null nor empty");
        }

        String id = null;
        String symbol = null;
        String cytoband = null;
        String title = null;
        String llid = null;
        Set accessions = new HashSet();

        for (int i = 0; i < lines.size(); i++) {
            String line = lines.get(i).toString();
            if (line.startsWith(ID)) {
                id = _parse(line, ID, id);
            }

            if (line.startsWith(SYMBOL)) {
                symbol = _parse(line, SYMBOL, symbol);
            }

            if (line.startsWith(CYTOBAND)) {
                cytoband = _parse(line, CYTOBAND, cytoband);
            }

            if (line.startsWith(TITLE)) {
                title = _parse(line, TITLE, title);
            }

            if (line.startsWith(LOCUSLINK)) {
                llid = _parse(line, LOCUSLINK, llid);
            }

            if (line.startsWith(SEQUENCE)) {
                _parseAccessions(line, accessions, onlyAddMrnaAccessions);
            }
        }

        accessions = removeAccessionExtensions(accessions);
        // save some memory by not parsing the title
        // but need the title for some things
        return new Unigene.Record(id, symbol, title, cytoband, llid, accessions);
    }

    private static String _parse(String line, String startsWith, String currValue) {

        if (line.startsWith(startsWith) == false) {
            throw new IllegalArgumentException("Does not start with startsWith: " + startsWith); // silly check
        }

        if (currValue != null) {
            throw new IllegalArgumentException("Current value is not null for field " + startsWith);
        }

        StringTokenizer tok = new StringTokenizer(line, " ");
        String one = tok.nextToken().trim();
        if (one.equals(startsWith) == false) {
            throw new IllegalArgumentException("Does not start with startsWith: " + startsWith);
        }

        StringBuffer rest = new StringBuffer();
        while (tok.hasMoreTokens()) {
            rest.append(tok.nextToken());
            if (tok.hasMoreTokens()) {
                rest.append(" ");
            }
        }

        return rest.toString().trim();
    }

    // BF773295.1 to ACC=BF773295
    private static Set removeAccessionExtensions(Set accs) throws ParserException {
        Set clean = new HashSet();
        for (Iterator it = accs.iterator(); it.hasNext();) {
            String acc = it.next().toString();
            StringTokenizer tok = new StringTokenizer(acc, ".");
            if (tok.countTokens() != 1 && tok.countTokens() != 2) {
                throw new ParserException("Bad accession format: " + acc + " invalid num tokens: " + tok.countTokens());
            }
            clean.add(tok.nextToken().trim()); // just the first one, always
        }

        return clean;
    }

    // SEQUENCE    ACC=BF773295.1; NID=g12121195; LID=7717; SEQTYPE=EST
    // SEQUENCE    ACC=BF821081.1; NID=g12160022; LID=7863; SEQTYPE=EST
    // SEQUENCE    ACC=BF916914.1; NID=g12308372; LID=8558; SEQTYPE=EST
    // SEQUENCE    ACC=D00137.1; NID=g219427; PID=g219428; SEQTYPE=mRNA

    private static final String SEQTYPEMRNA = "SEQTYPE=mRNA";

    private static void _parseAccessions(String line, Set curr, boolean onlyAddMrnaAccessions) throws ParserException {
        if (line.startsWith(SEQUENCE) == false) {
            throw new IllegalArgumentException("Does not start with startsWith: " + SEQUENCE); // silly check
        }

        if (onlyAddMrnaAccessions && line.indexOf(SEQTYPEMRNA) == -1) {
            return;
        }

        StringTokenizer tok = new StringTokenizer(line, " ;");
        String one = tok.nextToken().trim();
        if (one.equals(SEQUENCE) == false) {
            throw new IllegalArgumentException("Does not start with startsWith: " + SEQUENCE);
        }

        while (tok.hasMoreTokens()) {
            String str = tok.nextToken();
            if (str.startsWith("ACC")) {
                StringTokenizer acct = new StringTokenizer(str, "=");
                if (acct.countTokens() != 2) {
                    throw new ParserException("Bad format on acc line: " + str);
                }
                acct.nextToken();
                curr.add(acct.nextToken().trim());
            }
        }
    }


}    // End of class UnigeneParser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
