/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.charts.XChartImpl;
import edu.mit.broad.genome.charts.XChartUtils;
import edu.mit.broad.genome.math.ColorScheme;
import edu.mit.broad.genome.math.ColorSchemes;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.models.MiscChartModels;
import edu.mit.broad.genome.models.XChartModelFactory;
import edu.mit.broad.genome.models.XYDatasetProxy;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.reports.api.ToolReport;
import org.apache.log4j.Logger;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.data.statistics.HistogramType;
import org.jfree.data.xy.XYSeriesCollection;

import java.io.File;

/**
 * Needs to be here as it uises protected methods (the jfreechart_test getter) of XChart etc
 * <p/>
 * Different from XChartFactory in that the charts are more canned i.e more tuned to
 * what reports etc need (less generic than XChartFactory)
 */
public class DatasetCharts {

    private static final Logger klog = XLogger.getLogger(DatasetCharts.class);

    /**
     * Privatized Class Constructor
     */
    private DatasetCharts() {
    }

    // -------------------------------------------------------------------------------------------- //
    // usual affy 2 sample plot
    public XChart createGexScatterPlot(final Dataset ds, final int sampleColX, final int sampleColY) {

        final XYDatasetProxy xyds = new XYDatasetProxy("Scatter " + ds.getName(), ds.getColumn(sampleColX), ds.getColumn(sampleColY));

        final JFreeChart chart = ChartFactory.createScatterPlot(
                ds.getName() + " [ " + ds.getNumRow() + "R x " + ds.getNumCol() + "C ]",
                "expression for " + ds.getColumnName(sampleColX), "expression for " + ds.getColumnName(sampleColY),
                xyds,
                PlotOrientation.VERTICAL,
                true,
                true,
                false
        );

        final XYPlot plot = (XYPlot) chart.getPlot();

        final NumberAxis domainAxis = (NumberAxis) plot.getDomainAxis();
        domainAxis.setAutoRangeIncludesZero(false);

        return new XChartImpl(ds.getName() + "_scatter_plot", "XY plot for: " + ds.getName(), chart);
    }

    // -------------------------------------------------------------------------------------------- //
    public static XChart createGaddyPlot(final String name,
                                         final Dataset log2Dataset,
                                         final GeneSet gset,
                                         final ToolReport reportOpt,
                                         final File saveInDir_opt) {
        return createGaddyPlot(name, log2Dataset, new GeneSet[]{gset}, reportOpt, saveInDir_opt);
    }

    public static XChart createGaddyPlot(final String name,
                                         final Dataset log2Dataset,
                                         final GeneSet[] gsets_in_ascending_size_order,
                                         final ToolReport report_opt,
                                         final File saveInDir_opt) {

        final MiscChartModels.GaddyPlotModelMulti2 ps2 = new MiscChartModels.GaddyPlotModelMulti2(log2Dataset, gsets_in_ascending_size_order);


        if (report_opt != null) {
            // save the data, each gene sets gets 1 excel ss
            for (int i = 0; i < ps2.getSeriesCount(); i++) {
                report_opt.savePageDF(ps2.getSeriesName(i), ps2.getSeriesDF(i), saveInDir_opt);
            }
        }

        JFreeChart chart = ChartFactory.createScatterPlot(
                name + " [ " + log2Dataset.getNumRow() + "R x " + log2Dataset.getNumCol() + "C ]",
                "mean of log2 (expression)", "stdev of log2 expression",
                //ps,
                ps2,
                PlotOrientation.VERTICAL,
                true,
                true,
                false
        );

        XYPlot plot = (XYPlot) chart.getPlot();
        // tool tip
        plot.getRenderer(0).setToolTipGenerator(ps2.createXYToolTipGenerator());
        NumberAxis domainAxis = (NumberAxis) plot.getDomainAxis();
        domainAxis.setAutoRangeIncludesZero(false);
        return new XChartImpl(name + "_gaddy_plot", "Gaddy plot for: " + log2Dataset.getName(), chart);
    }

    public static XChart createGaddyPlot(final String name,
                                         final Dataset log2Dataset,
                                         final ToolReport report_opt,
                                         final File saveInDir_opt) {

        final MiscChartModels.GaddyPlotModel ps2 = new MiscChartModels.GaddyPlotModel(log2Dataset);


        if (report_opt != null) {
            // save the data
            report_opt.savePageDF(ps2.getSeriesName(0), ps2.getSeriesDF(), saveInDir_opt);
        }

        JFreeChart chart = ChartFactory.createScatterPlot(
                name + " [ " + log2Dataset.getNumRow() + "R x " + log2Dataset.getNumCol() + "C ]",
                "mean of log2 (expression)", "stdev of log2 expression",
                //ps,
                ps2,
                PlotOrientation.VERTICAL,
                true,
                true,
                false
        );

        XYPlot plot = (XYPlot) chart.getPlot();

        // tool tip
        plot.getRenderer(0).setToolTipGenerator(ps2.createXYToolTipGenerator());

        NumberAxis domainAxis = (NumberAxis) plot.getDomainAxis();
        domainAxis.setAutoRangeIncludesZero(false);

        return new XChartImpl(name + "_gaddy_plot", "Gaddy plot for: " + log2Dataset.getName(), chart);
    }


    public static XChart createStateFoldChange_by_gex_plot(final String name,
                                                           final Dataset log2Dataset_samcol_into_states,
                                                           final GeneSet[] gsets_in_ascending_size_order,
                                                           final boolean useMean,
                                                           final ToolReport report_opt,
                                                           final File saveInDir_opt) {

        final MiscChartModels.StateFoldChangeByGexPlotModelMulti2 ps2 = new MiscChartModels.StateFoldChangeByGexPlotModelMulti2(log2Dataset_samcol_into_states, gsets_in_ascending_size_order, useMean);


        if (report_opt != null) {
            // save the data, each gene sets gets 1 excel ss
            for (int i = 0; i < ps2.getSeriesCount(); i++) {
                report_opt.savePageDF(ps2.getSeriesName(i), ps2.getSeriesDF(i), saveInDir_opt);
            }
        }

        JFreeChart chart = ChartFactory.createScatterPlot(
                name + " [ " + log2Dataset_samcol_into_states.getNumRow() + "R x " + log2Dataset_samcol_into_states.getNumCol() + "C ]",
                "mean of log2 (expression)", "log2 (max/min) expression",
                //ps,
                ps2,
                PlotOrientation.VERTICAL,
                true,
                true,
                false
        );

        XYPlot plot = (XYPlot) chart.getPlot();
        // tool tip
        plot.getRenderer(0).setToolTipGenerator(ps2.createXYToolTipGenerator());
        NumberAxis domainAxis = (NumberAxis) plot.getDomainAxis();
        domainAxis.setAutoRangeIncludesZero(false);
        return new XChartImpl(name + "_statefc_plot", "State Fold Change plot for: " + log2Dataset_samcol_into_states.getName(), chart);
    }


    public static XChart createNullDistribPlot(final String gsetName, final float realScore, final Vector nullScores) {

        int numBins = 20;

        JFreeChart chart = XChartUtils.createHistogram("Null distribution for " + gsetName,
                false,
                "Real score",
                "# of random values",
                nullScores,
                true,
                numBins,
                HistogramType.FREQUENCY);

        return new XChartImpl(gsetName, "Histogram of random scores (real = " + realScore + " )", chart);
    }


    static ColorScheme RED = new ColorSchemes.BroadCancerRed();
    static ColorScheme BLUE = new ColorSchemes.BroadCancerBlue();

    public static XChart createAlignmentChart(final String title,
                                              final String xaxisTitle,
                                              final String[] labels,
                                              final Vector x1,
                                              final Vector x2) {

        NumberAxis xAxis = new NumberAxis(xaxisTitle);
        xAxis.setAutoRangeIncludesZero(false); // huh
        StandardXYItemRenderer rend = new StandardXYItemRenderer(StandardXYItemRenderer.LINES);

        NumberAxis yAxis = new NumberAxis("y");
        XYSeriesCollection data = new XChartModelFactory.XYSeriesCollImpl(labels, x1, x2);
        XYPlot plot = new XYPlot(data, xAxis, yAxis, rend);
        return new XChartImpl("alignment_chart", title, "some alignment", plot, false);
    }

} // End class XCannedCharts
