/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports;


import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.*;
import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.math.*;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DefaultMetricWeightStruc;
import edu.mit.broad.genome.reports.pages.HtmlPage;
import edu.mit.broad.genome.reports.web.LinkedFactory;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.chip.NullSymbolMode;
import edu.mit.broad.vdb.meg.Gene;
import edu.mit.broad.xbench.heatmap.GramImager;
import gnu.trove.TIntObjectHashMap;
import gnu.trove.TObjectIntHashMap;
import org.apache.log4j.Logger;
import xapps.api.vtools.viewers.VizFactory;

import java.awt.image.BufferedImage;
import java.util.*;

/**
 * More packaged than the base chart things
 * Canned Reports
 */
public class MiscReports {

    private static final transient Logger klog = XLogger.getLogger(MiscReports.class);

    /**
     * Privatized constructor
     */
    private MiscReports() {
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * Reporting class for PCalls
     */

    public static class PCallReports {

        public static StringDataframe createPCallReport_by_sample(final Dataset dataset_with_apm) {

            final APMMatrix apm = ((DefaultDataset) dataset_with_apm).getAPMMatrix();

            final String[] colNames = new String[]{"PCALL%", "ACALL%", "MCALL%",
                    "AVG_INTENSITY", "MEDIAN_INTENSITY",
                    "#GENES<100", "#GENES<500", "#GENES<1K", "#GENES<5K", "#GENES<10K", "#GENES<16K", "#GENES>16K"};

            final StringMatrix sm = new StringMatrix(dataset_with_apm.getNumCol(), colNames.length);
            final String[] rowNames = new String[dataset_with_apm.getNumCol()];

            for (int c = 0; c < dataset_with_apm.getNumCol(); c++) {
                rowNames[c] = dataset_with_apm.getColumnName(c);
                float[] freqs = apm.getColCallFrequencies(c);
                int coln = 0;
                sm.setElement(c, coln++, freqs[0] * 100);
                sm.setElement(c, coln++, freqs[1] * 100);
                sm.setElement(c, coln++, freqs[2] * 100);
                final Vector v = dataset_with_apm.getColumn(c);
                sm.setElement(c, coln++, v.mean());
                sm.setElement(c, coln++, v.median());

                final Range[] ranges = new Range[]{
                        new SimpleRange(0, 100),
                        new SimpleRange(100, 500),
                        new SimpleRange(500, 1000),
                        new SimpleRange(1000, 5000),
                        new SimpleRange(5000, 10000),
                        new SimpleRange(10000, 16000),
                        new SimpleRange(16000, Integer.MAX_VALUE)
                };

                v.bin(ranges, true);

                for (int j = 0; j < ranges.length; j++) {
                    sm.setElement(c, coln++, ranges[j].getCount());
                }

            }

            return new StringDataframe(dataset_with_apm.getName() + "_apm", sm, rowNames, colNames, true);
        }


        // @note ds and template must be synched (i.e no extra cols)
        // AP dataset can be anything (must have correct col names);
        public static StringDataframe createPCallReport_by_gene(final String name,
                                                                final DefaultDataset dataset_with_apm,
                                                                final Dataset dataset_with_gex,
                                                                final Template t) {

            /*see note above
            if (gex_dataset.getNumCol() != apm_dataset.getNumCol()) {
                throw new MismatchedSizeException("gex", gex_dataset.getNumCol(), "apm", apm_dataset.getNumCol());
            }
            */

            String ca = t.getClass(0).getName();
            String cb = t.getClass(1).getName();

            //List sampleNames = t.extractColumnNames(gex_dataset);
            final String[] colNames = new String[]{"GENE", "TITLE",
                    "PCALL_CNT", "PCALL_%", "TOTAL_CNT" + dataset_with_gex.getNumCol(),
                    "PCALL_CNT_" + ca, "PCALL%_" + ca, "TOTAL_CNT_" + ca,
                    "PCALL_CNT_" + cb, "PCALL%_" + cb, "TOTAL_CNT_" + cb,
                    "S2N", "LOG_MEAN", "STDDEV_EXPR",
                    "FOLD_CHANGE", "MEAN_" + ca, "MEAN_" + cb, "MEAN_ALL"

            };

            final StringMatrix sm = new StringMatrix(dataset_with_gex.getNumRow(), colNames.length);
            final TObjectIntHashMap probeNamePCallCounterMap = DatasetStatsCore.APM.hashPCallsByFeature(dataset_with_apm, dataset_with_gex.getColumnNames());
            final TObjectIntHashMap probeNamePCallCounterMap_ca = DatasetStatsCore.APM.hashPCallsByFeature(dataset_with_apm, t.getClass(0).extractColumnNames(dataset_with_gex));
            final TObjectIntHashMap probeNamePCallCounterMap_cb = DatasetStatsCore.APM.hashPCallsByFeature(dataset_with_apm, t.getClass(1).extractColumnNames(dataset_with_gex));

            DatasetStatsCore statsCore = new DatasetStatsCore();
            Map map_s2n = statsCore.calc2MetricMarkerScores(dataset_with_gex, t);
            Map map_fold = statsCore.calc2MetricMarkerScores_class_means(dataset_with_gex, t);

            for (int r = 0; r < dataset_with_gex.getNumRow(); r++) {
                int coln = 0;
                String rowName = dataset_with_gex.getRowName(r);
                sm.setElement(r, coln++, dataset_with_gex.getAnnot().getFeatureAnnot().getGeneSymbol(rowName)); // gene
                sm.setElement(r, coln++, dataset_with_gex.getAnnot().getFeatureAnnot().getGeneTitle(rowName)); // gene

                sm.setElement(r, coln++, probeNamePCallCounterMap.get(rowName));
                sm.setElement(r, coln++, ((100 * probeNamePCallCounterMap.get(rowName)) / dataset_with_gex.getNumCol()));
                sm.setElement(r, coln++, dataset_with_gex.getNumCol());

                sm.setElement(r, coln++, probeNamePCallCounterMap_ca.get(rowName));
                sm.setElement(r, coln++, ((100 * probeNamePCallCounterMap_ca.get(rowName)) / t.getClass(0).getSize()));
                sm.setElement(r, coln++, t.getClass(0).getSize());

                sm.setElement(r, coln++, probeNamePCallCounterMap_cb.get(rowName));
                sm.setElement(r, coln++, ((100 * probeNamePCallCounterMap_cb.get(rowName)) / t.getClass(1).getSize()));
                sm.setElement(r, coln++, t.getClass(1).getSize());

                DatasetStatsCore.TwoClassMarkerStats tcm = (DatasetStatsCore.TwoClassMarkerStats) map_s2n.get(rowName);
                sm.setElement(r, coln++, tcm.getScore());
                sm.setElement(r, coln++, tcm.getMe_all());
                sm.setElement(r, coln, tcm.getStdev_all());

                tcm = (DatasetStatsCore.TwoClassMarkerStats) map_fold.get(rowName);
                sm.setElement(r, coln++, tcm.getScore());
                sm.setElement(r, coln++, tcm.getMe0());
                sm.setElement(r, coln++, tcm.getMe1());
                sm.setElement(r, coln, tcm.getMe_all());
            }

            return new StringDataframe(name + "_pcall_report", sm, dataset_with_gex.getRowNames(), colNames, true, true);
        }

        public static StringDataframe createPCallReport_by_gene(final String name,
                                                                final Dataset dataset_with_apm,
                                                                final Dataset dataset_with_gex,
                                                                final Chip chip) {

            final String[] colNames = new String[]{"GENE", "TITLE", "PCALL_CNT", "PCALL_%", "LOG_MEAN", "STDDEV_EXPR"};
            final StringMatrix sm = new StringMatrix(dataset_with_gex.getNumRow(), colNames.length);
            final TObjectIntHashMap probeNamePCallCounterMap = DatasetStatsCore.APM.hashPCallsByFeature(dataset_with_apm);

            for (int r = 0; r < dataset_with_gex.getNumRow(); r++) {
                int coln = 0;
                String rowName = dataset_with_gex.getRowName(r);
                sm.setElement(r, coln++, chip.getSymbol(rowName, Chip.OMIT_NULLS));
                sm.setElement(r, coln++, chip.getTitle(rowName, Chip.OMIT_NULLS));
                sm.setElement(r, coln++, probeNamePCallCounterMap.get(rowName));
                sm.setElement(r, coln++, ((100 * probeNamePCallCounterMap.get(rowName)) / dataset_with_gex.getNumCol()));
                Vector v = dataset_with_gex.getRow(rowName);
                sm.setElement(r, coln++, v.mean());
                sm.setElement(r, coln, v.stddev(false, false));
            }

            return new StringDataframe(name + "_pcall_report", sm, dataset_with_gex.getRowNames(), colNames, true, true);
        }


    } // End class PCallReports

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    public static StringDataframe createRankedListsReport(final String name, final RankedList[] rls, final Chip chip) {

        List colNames = new ArrayList();
        colNames.add("GENE");
        for (int i = 0; i < rls.length; i++) {
            colNames.add(rls[i].getName() + "_score");
            colNames.add(rls[i].getName() + "_rank");
        }

        List rowNames = rls[0].getRankedNames();

        final StringMatrix sm = new StringMatrix(rowNames.size(), colNames.size());
        for (int r = 0; r < rowNames.size(); r++) {
            String probeName = rowNames.get(r).toString();
            sm.setElement(r, 0, chip.getSymbol(probeName, Chip.OMIT_NULLS));
        }

        for (int r = 0; r < rowNames.size(); r++) {
            final String probeName = rowNames.get(r).toString();
            int coln = 1;
            for (int c = 0; c < rls.length; c++) {
                sm.setElement(r, coln++, rls[c].getScore(probeName));
                sm.setElement(r, coln++, rls[c].getRank(probeName));
            }
        }

        return new StringDataframe(name, sm, rowNames, colNames, true);
    }

    public static StringDataframe createSizeReport(final GeneSetMatrixPair gmp,
                                                   final Chip chip,
                                                   final NullSymbolMode nmode) throws Exception {

        String[] colNames = new String[]{"#PROBES_UP", "#PROBES_DN", "#GENES_UP", "#GENES_DN"};
        //GeneSetStats stats = new GeneSetStats();
        GeneSetMatrixPair gmp_symbols = chip.getSymbols(gmp, nmode);

        StringMatrix sm = new StringMatrix(gmp.getSize(), colNames.length);
        List rowNames = new ArrayList();
        for (int r = 0; r < gmp.getSize(); r++) {
            int coln = 0;
            rowNames.add(gmp.get(r).getName());
            sm.setElement(r, coln++, gmp.get(r).getUp().getNumMembers());
            sm.setElement(r, coln++, gmp.get(r).getDown().getNumMembers());
            sm.setElement(r, coln++, gmp_symbols.get(r).getUp().getNumMembers());
            sm.setElement(r, coln, gmp_symbols.get(r).getDown().getNumMembers());
        }

        return new StringDataframe("gmp_report", sm, rowNames, colNames, true, true);
    }

    public static String toString(GenesOfInterest.Gene recd, char delim) {
        NullSafeStringBuffer buf = new NullSafeStringBuffer(false);
        return buf.append(recd.getNotes()).toString();
    }


    public static void appendBpogs(final Dataset[] dss,
                                   final Template[] tss,
                                   final GeneSet onlyDrawBpogForTheseGenes,
                                   final HtmlPage html) throws Exception {

        GramImager gi = VizFactory.createGramImager();


        for (int i = 0; i < dss.length; i++) {
            html.addImage(dss[i].getName(), "some image",
                    gi.createBpogImage(dss[i], tss[i], onlyDrawBpogForTheseGenes));
        }
    }

    //-----------------------------------------------------------------//
    public static String[] toSymbolStringArray(Set hugos) {
        return toSymbolStrings((Gene[]) hugos.toArray(new Gene[hugos.size()]));
    }

    public static String[] toTitleStringArray(final Set hugos) {
        return toTitleStrings((Gene[]) hugos.toArray(new Gene[hugos.size()]));
    }

    public static String[] toSymbolStrings(final Gene[] genes) {
        String[] ss = new String[genes.length];
        for (int i = 0; i < genes.length; i++) {
            ss[i] = genes[i].getSymbol();
        }

        return ss;
    }

    public static String[] toTitleStrings(final Gene[] genes) {
        String[] ss = new String[genes.length];
        for (int i = 0; i < genes.length; i++) {
            ss[i] = genes[i].getTitle();
        }

        return ss;
    }

    public static String toGeneSymbolTitle(Set hugos) {
        if (hugos.size() == 0) {
            return "";
        } else {
            StringBuffer buf = new StringBuffer();
            for (Iterator it = hugos.iterator(); it.hasNext();) {
                Gene gene = (Gene) it.next();
                buf.append(gene.getSymbol()).append('\t').append(gene.getTitle()).append('\n');
            }

            return buf.toString();
        }
    }


    public static class MarkerReportStruc {
        public Dataframe between_class;
        public Dataframe[] within_class;

        public Dataframe[] toDataframes() {
            Dataframe[] dfs = new Dataframe[1 + within_class.length];
            dfs[0] = between_class;
            for (int i = 0; i < within_class.length; i++) {
                dfs[i + 1] = within_class[i];
            }
            return dfs;
        }
    } // End class MarkerReport

    public static MarkerReportStruc createMarkerStatsReport(final Dataset ds, final Template t) {
        return createMarkerStatsReport(ds, t, new LabelledVectorProcessors.None());
    }

    /**
     * Class constructor
     *
     * @param ds
     * @param t
     * @param lp
     * @return
     */
    public static MarkerReportStruc createMarkerStatsReport(final Dataset ds, final Template t, LabelledVectorProcessor lp) {

        // First the metrics
        final Metric[] all2ClassCatMetrics = Metrics.createAllCategoricalMetrics();
        final Map mps = DatasetMetrics.getDefaultMetricParams();

        final DatasetMetrics dm = new DatasetMetrics();
        final DatasetGenerators gen = new DatasetGenerators();

        final Dataframe df = dm.calcUNSOMetrics(all2ClassCatMetrics, mps, lp, ds, t);
        MarkerReportStruc mr = new MarkerReportStruc();
        mr.between_class = df;
        mr.within_class = new Dataframe[t.getNumClasses()];

        // Then the within class stats
        final DatasetStatsCore dstats = new DatasetStatsCore();
        final VectorFunctor.Statistic[] vstats = VectorFunctors.createAllStats();
        for (int c = 0; c < t.getNumClasses(); c++) {
            Dataset subds = gen.extract(ds, t.getClass(c));
            mr.within_class[c] = dstats.calcRowStatsDf(vstats, subds);
        }

        return mr;
    }

    public static HtmlPage createDatasetHeatMapAndCorrelationPlots(final Dataset fullDs_opt,
                                                                   final Template template,
                                                                   final RankedList rl,
                                                                   final int topBotXGenes) {

        HtmlPage htmlPage = new HtmlPage("heat_map_corr_plot", "Heat map and correlation plot for " + rl.getName());

        htmlPage.addBreak();
        htmlPage.addHtml("&nbsp&nbsp");

        if (fullDs_opt != null) {
            try {
                BufferedImage bi = createHeatMap(fullDs_opt, template, rl, topBotXGenes);
                htmlPage.addImage("heat_map", "Heat Map of the top " + topBotXGenes + " features for each phenotype in " + fullDs_opt.getName(), bi);

            } catch (Throwable t) {
                htmlPage.addError("Trouble making heat map", t);
                t.printStackTrace();
            }
        }

        htmlPage.addBreak();
        htmlPage.addHtml("&nbsp&nbsp");

        String classAName;
        String classBName;

        if (template == null) {
            classAName = "classA";
            classBName = "classB";
        } else {
            classAName = template.getClassName(0);
            classBName = template.getClassName(1);
        }

        XChart xc = RankedListCharts.createRankedListChart(rl, classAName, classBName, true);
        htmlPage.addChart(xc, 500, 400);

        htmlPage.addBreak();
        return htmlPage;
    }

    public static BufferedImage createHeatMap(final Dataset fullDs,
                                              final Template template,
                                              final RankedList rl,
                                              final int topBotXGenes) {
        List useNames = rl.getNamesOfUpOrDnXRanks(topBotXGenes, true);
        useNames.addAll(rl.getNamesOfUpOrDnXRanks(topBotXGenes, false));
        Dataset ds = new DatasetGenerators().extractRows(fullDs, useNames);
        return VizFactory.createGramImager().createBpogImage(ds, template);
    }

    public static StringDataframe createRandomLengthAndWeightReport(final String metricName, final Dataset rndMetrics) throws Exception {

        final String[] colNames = new String[]{"POS_FRACTION_LENGTH", "POS_WEIGHT_FRACTION"};
        final StringMatrix sm = new StringMatrix(rndMetrics.getNumCol(), colNames.length);
        for (int r = 0; r < rndMetrics.getNumCol(); r++) {
            MetricWeightStruc mws = new DefaultMetricWeightStruc(metricName, rndMetrics.getColumn(r));
            sm.setElement(r, 0, mws.getTotalPosLength_frac());
            sm.setElement(r, 1, mws.getTotalPosWeight_frac());
        }

        return new StringDataframe("rnd_metrics_info", sm, colNames, true);
    }

    /**
     * exception safe
     *
     * @param dfWithRowNamesAsProbeNames
     * @param chip
     * @return
     */
    public static Dataframe annotateProbesNames(final Dataframe dfWithRowNamesAsProbeNames, final Chip chip) {
        try {
            java.util.List rowNames = new ArrayList(dfWithRowNamesAsProbeNames.getNumRow());
            for (int r = 0; r < dfWithRowNamesAsProbeNames.getNumRow(); r++) {
                String probeName = dfWithRowNamesAsProbeNames.getRowName(r);
                if (chip.isProbe(probeName)) {
                    Gene gene = chip.getHugo(probeName);
                    //System.out.println(">>> " + hugo);
                    probeName = gene.getSymbol() + "_" + probeName;
                }
                rowNames.add(probeName);
            }

            return new Dataframe(dfWithRowNamesAsProbeNames.getName(), dfWithRowNamesAsProbeNames.getMatrix(), rowNames,
                    dfWithRowNamesAsProbeNames.getColumnNames(), true);

        } catch (Throwable t) {
            t.printStackTrace();
            return dfWithRowNamesAsProbeNames;
        }

    }

    /**
     * Exception safe
     *
     * @param name
     * @param rl
     * @param fann_opt
     * @return
     */
    public static RichDataframe annotateProbesNames(final String name, final RankedList rl, final FeatureAnnot fann_opt) {
        final String[] colnames = new String[]{Headers.DESCRIPTION, Headers.GENE_SYMBOL, Headers.GENE_TITLE, "SCORE"};
        StringMatrix sm = new StringMatrix(rl.getSize(), colnames.length);
        TIntObjectHashMap linkMap = new TIntObjectHashMap();
        for (int r = 0; r < rl.getSize(); r++) {
            int coln = 0;
            String probeName = rl.getRankName(r);

            if (fann_opt != null) {
                sm.setElement(r, coln++, fann_opt.getNativeDesc(probeName));
                sm.setElement(r, coln++, fann_opt.getGeneSymbol(probeName));
                sm.setElement(r, coln++, fann_opt.getGeneTitle(probeName));
            } else {
                coln += 3;
            }

            sm.setElement(r, coln, Float.toString(rl.getScore(r)));

            linkMap.put(sm.getElementPos(r, 0), LinkedFactory.createLinkedProbeSet(probeName));

            if (fann_opt != null) {
                if (fann_opt.getGeneSymbol(probeName) != null) {
                    linkMap.put(sm.getElementPos(r, 1), LinkedFactory.createLinkedGeneSymbol(fann_opt.getGeneSymbol(probeName)));
                }
            }

        }

        StringDataframe sdf = new StringDataframe(name, sm, rl.getRankedNames(), colnames, true, true);
        return new RichDataframe(sdf, null, null, linkMap);
    }
    
    public static RichDataframe annotateProbesNames_gsaa(final String name, final RankedList rl, final FeatureAnnot fann_opt) {
        final String[] colnames = new String[]{Headers.P_VALUE_SNP, Headers.GENE_SYMBOL, Headers.GENE_TITLE, "SCORE"};
        StringMatrix sm = new StringMatrix(rl.getSize(), colnames.length);
        TIntObjectHashMap linkMap = new TIntObjectHashMap();
        for (int r = 0; r < rl.getSize(); r++) {
            int coln = 0;
            String probeName = rl.getRankName(r);

            if (fann_opt != null) {
                sm.setElement(r, coln++, fann_opt.getNativeDesc(probeName));
                sm.setElement(r, coln++, fann_opt.getGeneSymbol(probeName));
                sm.setElement(r, coln++, fann_opt.getGeneTitle(probeName));
            } else {
                coln += 3;
            }

            sm.setElement(r, coln, Float.toString(rl.getScore(r)));

            linkMap.put(sm.getElementPos(r, 0), LinkedFactory.createLinkedProbeSet(probeName));

            if (fann_opt != null) {
                if (fann_opt.getGeneSymbol(probeName) != null) {
                    linkMap.put(sm.getElementPos(r, 1), LinkedFactory.createLinkedGeneSymbol(fann_opt.getGeneSymbol(probeName)));
                }
            }

        }

        StringDataframe sdf = new StringDataframe(name, sm, rl.getRankedNames(), colnames, true, true);
        return new RichDataframe(sdf, null, null, linkMap);
    }

    public static RichDataframe annotateProbesNames_gsaasnp(final String name, final RankedList rl, final FeatureAnnot fann_opt) {
        final String[] colnames = new String[]{Headers.DESCRIPTION, "SCORE"};
        StringMatrix sm = new StringMatrix(rl.getSize(), colnames.length);
        TIntObjectHashMap linkMap = new TIntObjectHashMap();
        for (int r = 0; r < rl.getSize(); r++) {
            int coln = 0;
            String probeName = rl.getRankName(r);

            if (fann_opt != null) {
                sm.setElement(r, coln++, fann_opt.getNativeDesc(probeName));
            } else {
                coln += 1;
            }

            sm.setElement(r, coln, Float.toString(rl.getScore(r)));

            linkMap.put(sm.getElementPos(r, 0), LinkedFactory.createLinkedProbeSet(probeName));

            if (fann_opt != null) {
                if (fann_opt.getGeneSymbol(probeName) != null) {
                    linkMap.put(sm.getElementPos(r, 1), LinkedFactory.createLinkedGeneSymbol(fann_opt.getGeneSymbol(probeName)));
                }
            }

        }

        StringDataframe sdf = new StringDataframe(name, sm, rl.getRankedNames(), colnames, true, true);
        return new RichDataframe(sdf, null, null, linkMap);
    }    


} // End class CannedReports

