package edu.mit.broad.genome.reports;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.alg.AlgMap;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.DatasetStatsCore;
import edu.mit.broad.genome.alg.TemplateStats;
import edu.mit.broad.genome.alg.markers.PermutationTest;
import edu.mit.broad.genome.alg.markers.PermutationTestImpl;
import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.charts.XChartImpl;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.reports.api.ToolReport;
import edu.mit.broad.genome.reports.pages.HtmlPage;
import edu.mit.broad.genome.reports.pages.KeyValTable;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.xbench.heatmap.GramImager;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.IntervalMarker;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.TextAnchor;
import xapps.api.vtools.viewers.VizFactory;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Nov 24, 2005
 *         Time: 10:39:12 PM
 */
public class PermutationReports extends ChartHelper {

    private PermutationReports() {
    }

    // -------------------------------------------------------------------------------------------- //
    // ----------------------------REPORTS--------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public static void createDetailedReports(final PermutationTest ptest,
                                             final ToolReport report,
                                             final boolean createSubDir,
                                             final boolean createBpog,
                                             final boolean dump) throws Exception {

        File saveInDir = report.getReportDir();
        if (createSubDir) {
            saveInDir = report.createSubDir(ptest.getName());
        }

        createDetailedReports(ptest, report, saveInDir, createBpog, dump);
    }

    public static void createDetailedReports(final PermutationTest ptest,
                                             final ToolReport report,
                                             final File saveInDir,
                                             final boolean createBpog,
                                             final boolean dump) throws Exception {

        Template template = ptest.getTemplate();

        String classA_name_long;
        String classB_name_long;
        String classA_name;
        String classB_name;

        if (template.isContinuous()) {
            classA_name_long = "positive correlation with profile";
            classB_name_long = "negative correlation with profile";
            String nn = AuxUtils.getAuxNameOnlyNoHash(template);
            if (template.isContinuous()) {
                classA_name = nn;
                classB_name = nn;
            } else {
                classA_name = nn + "_pos";
                classB_name = nn + "_neg";
            }
        } else {
            classA_name_long = template.getClassName(0) + " (" + template.getClass(0).getSize() + " samples)";
            classB_name_long = template.getClassName(1) + " (" + template.getClass(1).getSize() + " samples)";
            classA_name = template.getClassName(0);
            classB_name = template.getClassName(1);
        }


        final HtmlPage htmlPage = report.getIndexPage();
        htmlPage.startSection("Marker permutation report for: " + ptest.getName() + "<br>classA: " + classA_name_long + "<br>classB: " + classB_name_long);

        // 1: save the heat map
        try {

            if (createBpog) {
                //klog.debug("DOING BPOG ... ");
                htmlPage.addImage("bpog", "Heat map of " + ptest.getNumMarkers(),
                        "Heat map of " + ptest.getNumMarkers(),
                        createUpDnMarkersBpog(ptest), 600, 600);
            }

        } catch (Throwable thr) {
            report.addError("trouble making bpog/charts", thr);
        }

        // 2. Butterfly plot
        htmlPage.addChart(createButterflyChart(ptest), 600, 600);

        // Save various result files
        // The ranked list annotated
        final RichDataframe rdfGeneList = MiscReports.annotateProbesNames(ptest.getName(),
                ptest.getRankedList(), ptest.getDataset().getAnnot().getFeatureAnnot());
        report.savePageXls(rdfGeneList, "ranked_gene_list", saveInDir);
        report.savePage(ptest.getUpandDnMarkers(), saveInDir); // gene sets
        report.savePageXls(createSignificanceReport(ptest, ptest.getDataset().getAnnot().getChip()),
                "permutation_" + classA_name + " _vs_" + classB_name,
                saveInDir);

        // Finally save cross over details
        final RichDataframe crossOver = createMetricCrossOverReport(ptest);
        final RichDataframe weights = createMetricWeightReport(ptest);
        htmlPage.addTable("cross_over_details", crossOver.getMetaData().getKvt());
        report.savePage("rnd_cross_over_distrib", "Distribution of marker cross-over fractions the random phenotypes", crossOver.getDataframe(), saveInDir, false);
        // add weight details
        htmlPage.addTable("weight_details", weights.getMetaData().getKvt());
        report.savePage("rnd_weight_distrib", "Distribution of marker weights for the random phenotypes", weights.getDataframe(), saveInDir, false);

        if (dump) {
            report.savePage(ptest.getRandomScores(true), saveInDir);
            report.savePage(ptest.getRandomScores(false), saveInDir);
        }

        htmlPage.endSection();
    }

    public static Vector calcHammingDistances(final PermutationTest ptest) {
        return TemplateStats.calcAverageHammingDists(ptest.getTemplate(), ptest.getTemplatesRandom());
    }

    public static RichDataframe createMetricCrossOverReport(final PermutationTest ptest) {

        final String[] colNames = new String[]{"RANK", "FRACTION"};
        final Vector crossOverRanks = new Vector(ptest.getMetricCrossOverRanks());
        final Vector crossOverFractions = new Vector(ptest.getMetricCrossOverRanks());
        crossOverFractions.divide(ptest.getDataset().getNumRow());

        StringMatrix sm = new StringMatrix(crossOverRanks.getSize(), colNames.length);
        sm.setColumn(0, crossOverRanks);
        sm.setColumn(1, crossOverFractions);

        StringDataframe sdf = new StringDataframe("crossover_points_" + ptest.getName(), sm, true);

        KeyValTable kvt = new KeyValTable(2);
        kvt.addRow("REAL_CROSS_OVER_RANK", ptest.getRealMetricCrossOverRank());
        kvt.addRow("REAL_CROSS_OVER_FRAC", ptest.getRealMetricCrossOverRank() / (float) ptest.getDataset().getNumRow());
        kvt.addRow("RANDOM_CROSSOVER_RANK_MEDIAN", crossOverRanks.median());
        kvt.addRow("RANDOM_CROSSOVER_RANK_STDEV", crossOverRanks.stddev(true, false));
        kvt.addRow("RANDOM_CROSSOVER_FRAC_MEDIAN", crossOverFractions.median());
        kvt.addRow("RANDOM_CROSSOVER_FRAC_STDEV", crossOverFractions.stddev(true, false));

        RichDataframe.MetaData metaData = new RichDataframe.MetaData(kvt);
        return new RichDataframe(sdf, metaData);
    }

    public static RichDataframe createMetricWeightReport(final PermutationTest ptest) {

        MetricWeightStruc[] mws = ptest.getRandomMetricWeights();

        String[] colNames = new String[]{"POS_WEIGHT", "NEG_WEIGHT", "DELTA", "TOTAL_WEIGHT", "POS_WEIGHT_FRACTION", "NEG_WEIGHT_FRACTION", "DELTA_WEIGHT_FRACTION"};
        StringMatrix m = new StringMatrix(mws.length, colNames.length);
        for (int r = 0; r < mws.length; r++) {
            MetricWeightStruc ws = mws[r];
            int coln = 0;
            m.setElement(r, coln++, ws.getTotalPosWeight());
            m.setElement(r, coln++, ws.getTotalNegWeight());
            m.setElement(r, coln++, ws.getDeltaWeight());
            m.setElement(r, coln++, ws.getTotalWeight());
            m.setElement(r, coln++, ws.getTotalPosWeight_frac());
            m.setElement(r, coln++, ws.getTotalNegWeight_frac());
            m.setElement(r, coln, ws.getDeltaWeight());

        }

        StringDataframe sdf = new StringDataframe("crossover_points_" + ptest.getName(), m, colNames, true);

        KeyValTable kvt = new KeyValTable(2);

        Vector deltav = m.getColumnAsVector(2, false);
        kvt.addRow("REAL_DELTA", ptest.getRealMetricWeight().getDeltaWeight());
        kvt.addRow("RANDOM_DELTA_MEAN", deltav.mean());
        kvt.addRow("RANDOM_DELTA_MEDIAN", deltav.median());
        kvt.addRow("RANDOM_DELTA_STDEV", deltav.stddev(true, false));

        Vector totalv = m.getColumnAsVector(3, false);
        kvt.addRow("REAL_TOTAL", ptest.getRealMetricWeight().getTotalWeight());
        kvt.addRow("RANDOM_TOTAL_MEAN", totalv.mean());
        kvt.addRow("RANDOM_TOTAL_MEDIAN", totalv.median());
        kvt.addRow("RANDOM_TOTAL_STDEV", totalv.stddev(true, false));

        RichDataframe.MetaData metaData = new RichDataframe.MetaData(kvt);
        return new RichDataframe(sdf, metaData);
    }

    /**
     * The Dataset with top n and bottom n markers
     *
     * @return
     */
    public static Dataset createUpDnMarkersDataset(final PermutationTest ptest) {
        final GeneSet gset = ptest.getUpandDnMarkers().getBoth();
        return new DatasetGenerators().extractRowsSorted(ptest.getScoredDataset(), gset);
    }

    public static BufferedImage createUpDnMarkersBpog(final PermutationTest ptest) {
        final ScoredDataset sds = ptest.getScoredDataset();
        final Dataset ex = new DatasetGenerators().extractRowsSorted(sds, ptest.getUpandDnMarkers().getBoth());
        return VizFactory.createGramImager().createBpogImage(ex, ptest.getTemplate());
    }


    /**
     * we want a combined dataset for the bpog
     * the combined ds should have the same order of columns as the original one
     * its rows should be ordered by the top (or bottom) x markers for each template considered
     * BUT not necc the same num of cols
     * # cls = sum of all class 0 cols
     *
     * @param fullDs
     * @param ptests
     * @return
     */
    public static BufferedImage createCombinedMarkersBpog(final Dataset fullDs,
                                                          final PermutationTest[] ptests,
                                                          final boolean up) {

        final List list = new ArrayList();
        final List orderedColNames = new ArrayList();
        final Template[] templates = new Template[ptests.length];
        for (int i = 0; i < ptests.length; i++) {
            final GeneSet gset = ptests[i].getUpandDnMarkers().get(up);
            list.addAll(gset.getMembers());
            //colOrder.add(ptests[i].getTemplate().getClass(0).getPositions()); // cant use positions - might be aux etc
            int[] cindices = ptests[i].getTemplate().getClass(0).getProfilePositions();
            for (int c = 0; c < cindices.length; c++) {
                orderedColNames.add(ptests[i].getDataset().getColumnName(cindices[c]));
            }
            templates[i] = ptests[i].getTemplate();
        }

        final DatasetGenerators gen = new DatasetGenerators();
        Dataset ds = gen.extractRows(fullDs, new FSet("combo", list, false));
        ds = gen.orderDatasetColumns(ds.getName() + "col_ordered", orderedColNames, ds, true);

        final Template combined = TemplateFactory.createCombinedTemplateUsingClass0("combined", templates);

        GramImager gi = VizFactory.createGramImager();
        return gi.createBpogImage(ds, combined);
    }

    // the standard perm analysis plot (see original allaml science paper)
    // but dont do the log thing for y-axis
    public static XChart createButterflyChart(final PermutationTest ptest) {

        List xValues = new ArrayList();

        Dataset pos_sig_levels = ptest.getSigLevels(true);
        Dataset neg_sig_levels = ptest.getSigLevels(false);
        final Vector yValues = new Vector(ptest.getNumMarkers());
        for (int i = 0; i < ptest.getNumMarkers(); i++) {
            yValues.setElement(i, i + 1);
        }
        final float[] sigLevels = ptest.getSigLevels();
        final List xLabels = new ArrayList();
        xLabels.add("Observed pos");
        xLabels.add("Observed neg");
        final RankedList rlReal = ptest.getRankedList();

        Vector x1 = new Vector(ptest.getNumMarkers());
        for (int r = 0; r < ptest.getNumMarkers(); r++) {
            x1.setElement(r, rlReal.getScore(r));
        }

        xValues.add(x1);

        int r = rlReal.getSize() - 1;
        x1 = new Vector(ptest.getNumMarkers());
        for (int cnt = 0; cnt < ptest.getNumMarkers(); cnt++, r--) {
            x1.setElement(cnt, rlReal.getScore(r));
        }
        xValues.add(x1);

        for (int c = 0; c < pos_sig_levels.getNumCol(); c++) {
            xValues.add(pos_sig_levels.getColumn(c));
            xValues.add(neg_sig_levels.getColumn(c));
            xLabels.add("Permuted pos " + Printf.format(sigLevels[c] * 100, 0) + "%");
            xLabels.add("Permuted neg " + Printf.format(sigLevels[c] * 100, 0) + "%");
        }

        String classAName = null;
        String classBName = null;
        if (ptest.getTemplate().isCategorical()) {
            classAName = ptest.getTemplate().getClassName(0);
            classBName = ptest.getTemplate().getClassName(1);
        }


        return new XChartImpl("butterfly_plot", "Butterfly plot of significance",
                _createButterflyChart("Butterfly plot for: " + ptest.getName(), "Score (" + ptest.getMetric().getName() + ")",
                        (Vector[]) xValues.toArray(new Vector[xValues.size()])
                        , (String[]) xLabels.toArray(new String[xLabels.size()]), "Gene rank",
                        yValues, classAName, classBName));
    }

    // assumes that vectors and labels are: pos, neg, pos, neg ...
    private static JFreeChart _createButterflyChart(final String chartTitle,
                                                    final String xaxisLabel,
                                                    final Vector[] xValues,
                                                    final String[] xNames,
                                                    final String yaxisLabel,
                                                    final Vector yValues,
                                                    final String classAName_opt,
                                                    final String classBName_opt) {

        //ensureEqualSizes(yValues, xValues);

        if (xNames.length != xValues.length) {
            throw new IllegalArgumentException("Unequal lengths for xvectors: " + xValues.length + " and xlabels: " + xNames);
        }

        // dont see any way but to duplicate the data
        XYSeries[] serieses = new XYSeries[xValues.length];
        XYSeriesCollection dataset = new XYSeriesCollection();

        for (int i = 0; i < xValues.length; i++) {
            serieses[i] = new XYSeries(xNames[i]);
            for (int j = 0; j < xValues[i].getSize(); j++) {
                serieses[i].add(xValues[i].getElement(j), yValues.getElement(j));
            }
            dataset.addSeries(serieses[i]);
        }

        // create the chart...
        JFreeChart chart = ChartFactory.createXYLineChart(chartTitle,
                xaxisLabel,
                yaxisLabel,
                dataset,
                PlotOrientation.VERTICAL,
                true, // include legend
                true, // tooltips
                false // urls
        );

        // NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...
        chart.setBackgroundPaint(Color.WHITE);

        // get a reference to the plot for further customisation...
        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setBackgroundPaint(Color.WHITE);
        plot.setDomainGridlinePaint(Color.GRAY);
        plot.setRangeGridlinePaint(Color.GRAY);

        // make the real have dots and not line
        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
        renderer.setSeriesPaint(0, Color.RED);
        renderer.setSeriesPaint(1, Color.BLUE);

        renderer.setSeriesShape(0, createCircleShape());
        renderer.setSeriesShape(1, createCircleShape());
        renderer.setSeriesLinesVisible(0, false); // realpos
        renderer.setSeriesLinesVisible(1, false); // real neg

        // Make colors same for pos and neg for teh same sig level
        List colors = new ArrayList(Arrays.asList(new Color[]{Color.BLACK, Color.PINK, Color.GREEN}));
        int ii = 0;
        for (int i = 2; i < serieses.length; i = i + 2) {
            renderer.setSeriesShapesVisible(i, false);
            renderer.setSeriesShapesVisible(i + 1, false);
            Paint color;
            if (ii < colors.size()) {
                color = (Color) colors.get(ii);
            } else {
                color = renderer.getSeriesPaint(i);
            }
            renderer.setSeriesPaint(i, color);
            renderer.setSeriesPaint(i + 1, color);

            ii++;
        }

        plot.setRenderer(0, renderer);

        // change the auto tick unit selection to integer units only...
        NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
        rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());

        float pos = yValues.max() / 2f;

        if (classAName_opt != null && classAName_opt.length() > 0) {
            // represents an interval to be highlighted in some manner
            IntervalMarker target = new IntervalMarker(pos, pos); // @note max max so width = 0
            target.setLabel("'" + classAName_opt + "' (Pos corr)");
            target.setLabelFont(new Font("SansSerif", Font.PLAIN, 10));
            target.setLabelAnchor(RectangleAnchor.RIGHT);
            target.setLabelTextAnchor(TextAnchor.CENTER_RIGHT);
            target.setLabelPaint(Color.RED);
            chart.getXYPlot().addRangeMarker(target);
        }

        if (classBName_opt != null && classBName_opt.length() > 0) {
            IntervalMarker target = new IntervalMarker(pos, pos);
            target.setLabel("'" + classBName_opt + "' (Neg corr)");
            target.setLabelFont(new Font("SansSerif", Font.PLAIN, 10));
            target.setLabelAnchor(RectangleAnchor.LEFT);
            target.setLabelTextAnchor(TextAnchor.CENTER_LEFT);
            target.setLabelPaint(Color.BLUE);
            chart.getXYPlot().addRangeMarker(target);
        }


        return chart;
    }

    // when MarkerScores objects are available for the rows
    // also add the stdev, median/mean etc values to the frame
    public static StringDataframe createSignificanceReport(final PermutationTest ptest, final Chip chipOpt) {

        final List rowNames = new ArrayList(ptest.getNumMarkers() * 2);
        final RankedList rlReal = ptest.getRankedList();
        final float[] levels = ptest.getSigLevels();

        final Matrix upSigMatrix = ptest.getSigLevelsMatrix(true);
        final Matrix dnSigMatrix = ptest.getSigLevelsMatrix(false);

        final boolean categ = ptest instanceof PermutationTestImpl;

        // @maint the order is imp -- see below
        List colNames = new ArrayList(levels.length + 1);
        colNames.add("Gene");
        colNames.add("Title");
        colNames.add("Score");

        if (categ) {
            colNames.add(AlgMap.getMedianOrMeanLabel(ptest.getMetricParams()) + "0");
            colNames.add("stdev0");
            colNames.add(AlgMap.getMedianOrMeanLabel(ptest.getMetricParams()) + "1");
            colNames.add("stdev1");
        }

        int colstart = colNames.size();
        for (int c = 0; c < levels.length; c++) {
            colNames.add("Sig at " + levels[c]);
        }

        StringMatrix matrix = new StringMatrix(ptest.getNumMarkers() * 2, colNames.size());

        int nrow = 0;
        //boolean exceptioned = false;
        for (int r = 0; r < ptest.getNumMarkers(); r++, nrow++) {
            int coln = 0;
            String rowName = rlReal.getRankName(r);
            String symbol = Constants.NULL;
            String title = Constants.NULL;

            if (chipOpt != null) {
                symbol = chipOpt.getSymbol(rowName, Chip.OMIT_NULLS);
                title = chipOpt.getSymbol(rowName, Chip.OMIT_NULLS);
            }

            matrix.setElement(r, coln++, symbol);
            matrix.setElement(r, coln++, title);
            matrix.setElement(r, coln++, rlReal.getScore(r));

            if (categ) {
                DatasetStatsCore.TwoClassMarkerStats scores = ptest.getScore(rowName);
                matrix.setElement(r, coln++, (float) scores.getMe0());
                matrix.setElement(r, coln++, (float) scores.getStdev0());
                matrix.setElement(r, coln++, (float) scores.getMe1());
                matrix.setElement(r, coln, (float) scores.getStdev1());
            }

            for (int c = 0; c < levels.length; c++) {
                matrix.setElement(r, c + colstart, upSigMatrix.getElement(r, c));
            }

            rowNames.add(rowName);
        }

        // Then for the bottom
        int r = rlReal.getSize() - 1;
        for (int cnt = 0; cnt < ptest.getNumMarkers(); cnt++, r--, nrow++) {
            int coln = 0;
            String rowName = rlReal.getRankName(r);
            String symbol = Constants.NULL;
            String title = Constants.NULL;
            if (chipOpt != null) {
                symbol = chipOpt.getSymbol(rowName, Chip.OMIT_NULLS);
                title = chipOpt.getSymbol(rowName, Chip.OMIT_NULLS);
            }

            matrix.setElement(nrow, coln++, symbol);
            matrix.setElement(nrow, coln++, title);
            matrix.setElement(nrow, coln++, rlReal.getScore(r));

            if (categ) {
                DatasetStatsCore.TwoClassMarkerStats scores = ptest.getScore(rowName);
                matrix.setElement(nrow, coln++, (float) scores.getMe0());
                matrix.setElement(nrow, coln++, (float) scores.getStdev0());
                matrix.setElement(nrow, coln++, (float) scores.getMe1());
                matrix.setElement(nrow, coln, (float) scores.getStdev1());
            }

            for (int c = 0; c < levels.length; c++) {
                matrix.setElement(nrow, c + colstart, dnSigMatrix.getElement(cnt, c));
            }

            rowNames.add(rowName);

        }

        return new StringDataframe("sig " + ptest.getName(), matrix, rowNames, colNames, true);
    }

} // End class PermReports

