/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports;

import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.genome.objects.IDataframe;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.objects.strucs.Linked;
import edu.mit.broad.genome.reports.pages.KeyValTable;
import gnu.trove.TIntIntHashMap;
import gnu.trove.TIntObjectHashMap;

import java.util.List;

/**
 * Dataframe that holds rich (i.e fancy formatting) info for all its elements
 * This is NOT a very efficient implementation; the goal is simplicity of use.
 * Shouldnt be a problem in practice - rich table outputs are not generally very verbose.
 * <p/>
 * RDF aspects: per element/row/col
 * <p/>
 * 1) color
 * 2) font-wight (normal, bold)
 * 3) font-type: (normal, italic, underlined)
 * 4) link (url)
 * <p/>
 * To optimize on memory usage, these are made as follows:
 * <p/>
 * none
 * per column/row
 * per element
 */
// kept it here as it uses objects (such as Linked) that arent linkage safe
public class RichDataframe extends AbstractObject implements IDataframe {

    // holds the real un-rich data
    private IDataframe fIdf;

    // Hoilds meta format data = formatting that is DATAFRaME-WIDE
    private MetaData fMetaData;

    // not always available
    // contains String objects as keys
    private Rich fRichColors;

    // not always available
    //private Rich fRichFonts;

    // not always available
    // contains Linked objects as values
    private Rich fRichLinks;

    static class Rich {
        private TIntObjectHashMap col;
        private TIntObjectHashMap row;
        private TIntObjectHashMap cell_id;
    }

    /**
     * Class constructor
     *
     * @param sdf
     */
    public RichDataframe(final StringDataframe sdf, final MetaData metaData) {
        if (sdf == null) {
            throw new IllegalArgumentException("Parameter sdf cannot be null");
        }

        this.init(sdf, metaData, null, null);
    }

    /**
     * Class constructor
     *
     * @param sdf
     * @param cell_id_colorMap
     * @param cell_id_linkMap
     */
    public RichDataframe(final StringDataframe sdf,
                         final MetaData metaData,
                         final TIntObjectHashMap cell_id_colorMap,
                         final TIntObjectHashMap cell_id_linkMap) {
        if (sdf == null) {
            throw new IllegalArgumentException("Parameter sdf cannot be null");
        }

        this.init(sdf, metaData, cell_id_colorMap, cell_id_linkMap);
    }

    /**
     */
    private void init(final StringDataframe sdf,
                      final MetaData metaData,
                      final TIntObjectHashMap cell_id_colorMap,
                      final TIntObjectHashMap cell_id_linkMap) {

        super.initialize(sdf.getName());
        this.fIdf = sdf;
        this.fMetaData = metaData;

        if (cell_id_colorMap != null && !cell_id_colorMap.isEmpty()) {
            fRichColors = new Rich();
            //log.debug("Setting colorMap to: " + cell_id_colorMap + " " + cell_id_colorMap.size());
            fRichColors.cell_id = cell_id_colorMap;
        }

        if (cell_id_linkMap != null && !cell_id_linkMap.isEmpty()) {
            fRichLinks = new Rich();
            fRichLinks.cell_id = cell_id_linkMap;
        }

    }

    public String getQuickInfo() {
        StringBuffer buf = new StringBuffer().append(fIdf.getNumRow()).append('x').append(fIdf.getNumCol());
        return buf.toString();
    }

    public String[] getColumnAsStrings(int coln) {
        return Helper.getColumnAsStrings(this, coln);
    }

    public String[] getColumnAsStrings(final String colName) {
        return Helper.getColumnAsStrings(this, colName);
    }

    // ----------------------------------------------------------------------------------- //
    // ----------------------------------THE RICH METHODS -------------------------------- //
    // ----------------------------------------------------------------------------------- //

    public IDataframe getDataframe() {
        return fIdf;
    }

    // May return null
    public MetaData getMetaData() {
        return fMetaData;
    }

    public void setColColor(int col, String color) {
        if (fRichColors == null) {
            fRichColors = new Rich();
            fRichColors.col = new TIntObjectHashMap();
        }

        fRichColors.col.put(col, color);
    }

    public void setRowColor(int row, String color) {
        if (fRichColors == null) {
            fRichColors = new Rich();
            fRichColors.row = new TIntObjectHashMap();
        }

        fRichColors.row.put(row, color);
    }

    public void setElementColor(int row, int col, String color) {
        if (fRichColors == null) {
            fRichColors = new Rich();
            fRichColors.cell_id = new TIntObjectHashMap();
        }

        fRichColors.cell_id.put(_cell_id(row, col), color);
    }

    public String getElementColor(int row, int col) {
        if (fRichColors != null && fRichColors.cell_id != null) {
            Object obj = fRichColors.cell_id.get(_cell_id(row, col));
            if (obj != null) {
                return obj.toString();
            }
        }

        return null;
    }

    public String getColColor(int col) {
        Object obj = _getCol(col, fRichColors);
        if (obj != null) {
            return (String) obj;
        } else {
            return null;
        }
    }

    public String getRowColor(int row) {
        Object obj = _getRow(row, fRichColors);
        if (obj != null) {
            return (String) obj;
        } else {
            return null;
        }
    }

    // ----------------------------------------------------------------------------------- //
    // -------------------------------------- LINKS ------------------------------------ //
    // ----------------------------------------------------------------------------------- //

    public Linked getElementLink(int row, int col) {
        if (fRichLinks != null && fRichLinks.cell_id != null) {
            Object obj = fRichLinks.cell_id.get(_cell_id(row, col));
            if (obj != null) {
                //System.out.println(">> " + obj.getClass());
                return (Linked) obj;
            }
        }

        return null;
    }

    // ----------------------------------------------------------------------------------- //
    // -------------------------------------- HELPERS ------------------------------------ //
    // ----------------------------------------------------------------------------------- //

    private Object _getCol(int col, Rich el) {
        if (el == null || el.col == null) {
            return null;
        } else {
            return el.col.get(col);
        }
    }

    private Object _getRow(int row, Rich el) {
        if (el == null || el.row == null) {
            return null;
        } else {
            return el.row.get(row);
        }
    }

    private int _cell_id(int row, int col) {
        return row * fIdf.getNumCol() + col;
    }

    private Object _getWithPriority(int row, int col, Rich el, boolean priority2Row) {
        if (el == null) {
            return null;
        }

        Object obj_row = null;
        Object obj_col = null;

        if (el.row != null) {
            obj_row = el.row.get(row);
        }

        if (el.col != null) {
            obj_col = el.col.get(col);
        }

        int cell_id = _cell_id(row, col);
        Object obj_cell_id = null;

        if (el.cell_id != null) {
            obj_cell_id = el.cell_id.get(cell_id);
        }

        if (obj_cell_id != null) {
            return obj_cell_id;
        }

        if (priority2Row && obj_row != null) {
            return obj_row;
        }

        if (!priority2Row && obj_col != null) {
            return obj_col;
        }

        if (obj_row != null) {
            return obj_row;
        }

        if (obj_col != null) {
            return obj_col;
        }

        return null;
    }

    // ----------------------------------------------------------------------------------- //
    // ----------------------------------THE GENERIC IDF IMPLEMENTATIONS ----------------- //
    // ----------------------------------------------------------------------------------- //

    public Object getElementObj(int rown, int coln) {
        return fIdf.getElementObj(rown, coln);
    }

    public String getRowName(int rown) {
        return fIdf.getRowName(rown);
    }

    public List getRowNames() {
        return fIdf.getRowNames();
    }

    public int getRowIndex(String rowName) {
        return fIdf.getRowIndex(rowName);
    }

    public String[] getRowNamesArray() {
        return fIdf.getRowNamesArray();
    }

    public List getColumnNames() {
        return fIdf.getColumnNames();
    }

    public int getColumnIndex(String colName) {
        return fIdf.getColumnIndex(colName);
    }

    public String[] getColumnNamesArray() {
        return fIdf.getColumnNamesArray();
    }

    public String getColumnName(int coln) {
        return fIdf.getColumnName(coln);
    }

    public int getNumRow() {
        return fIdf.getNumRow();
    }

    public int getNumCol() {
        return fIdf.getNumCol();
    }

    /**
     * Internal class representing dataframe wide formating data
     */
    public static class MetaData {

        private KeyValTable fKvt;

        private TIntObjectHashMap fColIndexAlignMap;
        private TIntIntHashMap fColIndexFloatPrecisionMap;

        private String fTitle;
        private String fCaption;

        /**
         * Class constructor
         *
         * @param title
         * @param caption
         * @param kvt
         * @param colIndexAlignMap
         */
        public MetaData(final String title,
                        final String caption,
                        final KeyValTable kvt,
                        final TIntObjectHashMap colIndexAlignMap,
                        final TIntIntHashMap colIndexFloatPrecisionMap) {
            this.fTitle = title;
            this.fCaption = caption;
            this.fKvt = kvt;
            this.fColIndexAlignMap = colIndexAlignMap;
            this.fColIndexFloatPrecisionMap = colIndexFloatPrecisionMap;
        }

        /**
         * Class constructor
         *
         * @param title
         * @param caption
         * @param kvt
         */
        public MetaData(final String title,
                        final String caption,
                        final KeyValTable kvt) {

            this(title, caption, kvt, null, null);
        }

        public MetaData(final String title,
                        final String caption) {

            this(title, caption, null, null, null);
        }

        public MetaData(final KeyValTable kvt) {
            this(null, null, kvt);
        }

        public String getTitle() {
            return fTitle;
        }

        public String getCaption() {
            return fCaption;
        }

        public KeyValTable getKvt() {
            return fKvt;
        }

        public String getColumnAlignment(int coln) {
            if (fColIndexAlignMap != null) {
                Object align = fColIndexAlignMap.get(coln);
                if (align != null) {
                    return align.toString();
                }
            }

            return null;
        }

        boolean gotNfe = false; // opt so that on one error we stop

        public Object adjustPrecision(final Object val, final int coln) {
            if (val != null && val.toString().length() > 0 && !gotNfe && val != null && fColIndexFloatPrecisionMap != null && fColIndexFloatPrecisionMap.containsKey(coln)) {

                try {
                    Float f = new Float(val.toString());
                    int precision = fColIndexFloatPrecisionMap.get(coln);
                    return Printf.format(f.floatValue(), precision);
                } catch (Throwable t) {
                    t.printStackTrace();
                    gotNfe = true;

                }

            }

            return val; // no adjustment
        }

    }

} // End class RichDataframe
