package edu.mit.broad.genome.reports.api;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.reports.pages.HtmlFormat;
import edu.mit.broad.genome.utils.ImageUtils;
import org.apache.ecs.html.BR;
import org.apache.ecs.html.Div;
import org.apache.ecs.html.IMG;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

/**
 * Inner class to capture a picture and its file
 * Name is not necc unique so we need to generate a name to save the file
 * and not use title by itself
 */
public class PicFile {

    private String srcName;
    private String name;
    private Object chartOrbi;
    private String title;
    private String caption;
    private int width;
    private int height;
    private int currPicCnt;

    private File fSavedFile;

    /**
     * Class constructor
     *
     * @param xchart
     * @param width
     * @param height
     * @param currPicCnt
     */
    public PicFile(final XChart xchart, final int width, final int height, final int currPicCnt) {
        String srcName = generateName(xchart) + ".png";
        init(srcName, xchart.getName(), xchart.getTitle(), xchart.getCaption(), xchart, width, height, currPicCnt);
    }

    public PicFile(final String name, final String title, final String caption, final BufferedImage bi, final int width, final int height, final int currPicCnt) {
        String srcName = generateNameForImage(name) + ".png";
        init(srcName, name, title, caption, bi, width, height, currPicCnt);
    }
    // Here because of loader linkage constraints

    private static int kImageCounter = 1;

    public static String generateName(final XChart xchart) {
        return createSafeFileName(xchart.getName()) + "_" + kImageCounter++;
    }

    public static String generateNameForImage(final String title) {
        return createSafeFileName(title) + "_" + kImageCounter++;
    }

    // replaces chars that make file paths barf such as @ and #
    public static File createSafeFile(final File inDir, final String name) {
        return new File(inDir, createSafeFileName(name));
    }

    public static String createSafeFileName(String name) {
        name = name.trim();
        String safename = name.replace('@', '_'); // this does all occurrences
        safename = safename.replace('#', '_');
        safename = safename.replace(' ', '_'); // get rid of whitespace
        safename = safename.replace('%', '_');
        safename = safename.replace('$', '_');
        safename = safename.replace(':', '_'); // IE often doesnt like colons in linked to files
        safename = safename.replace('*', '_');
        safename = safename.replace('\\', '_');
        safename = safename.replace('/', '_');
        safename = safename.replace(Constants.INTRA_FIELD_DELIM, '_');

        if (safename.length() >= 125) {
            String ext = getExtension(safename);
            safename = safename.substring(0, 125) + "." + ext; // too many chars make excel (for instance) not recognize the file
        }

        return safename;
    }

    /**
     * Gets the extension of a specified file name. The extension is
     * everything after the last dot.
     * <p/>
     * <pre>
     * foo.txt    --> "txt"
     * a\b\c.jpg  --> "jpg"
     * foo        --> ""
     * </pre>
     *
     * @param f String representing the file name
     * @return extension of the file (or <code>""</code> if it had none)
     */
    public static String getExtension(final String f) {

        String ext;
        int pos;

        pos = f.lastIndexOf(".");

        if (pos == -1) {
            //log.warn("No extension found - returning \"\" file=" + f);
            ext = "";
        } else {
            ext = f.substring(pos + 1);
        }

        //og.debug("got: " + f + " foind: " + ext);

        if (f.indexOf(".meta.") != -1) {
            return "meta." + ext;
        } else {
            return ext;
        }
    }


    // common init routine
    void init(final String srcName,
              final String name,
              final String title,
              final String caption,
              final Object chartOrbi,
              final int width,
              final int height,
              final int currPicCnt) {

        this.srcName = srcName;
        this.name = name;

        if (srcName == null) {
            throw new IllegalArgumentException("Parameter srcName cannot be null");
        }

        if (name == null) {
            throw new IllegalArgumentException("Parameter name cannot be null");
        }

        if (title == null) {
            throw new IllegalArgumentException("Parameter title cannot be null");
        }

        if (chartOrbi == null) {
            throw new IllegalArgumentException("Parameter chartOrbi cannot be null");
        }

        this.title = title;
        this.caption = caption;
        this.chartOrbi = chartOrbi;
        this.width = width;
        this.height = height;
        this.currPicCnt = currPicCnt;
    }

    // core image creation block
    // html properties of the image are set here
    public Div createIMG() {
        Div image = HtmlFormat.Divs.image();
        IMG img = new IMG(srcName, name);
        img.addElement(new BR()); // 2 breaks makes it look nicer
        img.addElement(new BR());

        String desc = "Fig " + currPicCnt + ": " + title;
        if (caption != null) {
            desc += " &nbsp&nbsp <br> " + caption;
        }

        img.addElement(HtmlFormat.caption(desc));
        image.addElement(img);
        return image;
    }

    public File save(File dir) throws IOException {
        if (chartOrbi instanceof XChart) {
            fSavedFile = new File(dir, srcName);
            //TextTitle currTitle = ((XChart)chartOrbi).getFreeChart().getTitle();
            ((XChart) chartOrbi).saveAsPNG(fSavedFile, width, height);
        } else {
            fSavedFile = new File(dir, srcName);
            ImageUtils.savePng((BufferedImage) chartOrbi, fSavedFile);
        }

        return fSavedFile;
    }

    public File getFile() {
        if (fSavedFile == null) {
            throw new IllegalStateException("Not yet saved");
        }

        return fSavedFile;
    }

} // End inner class PicFile
