/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports.api;

import edu.mit.broad.genome.*;
import edu.mit.broad.genome.alg.distrib.RangeUtils;
import edu.mit.broad.genome.alg.filters.FilteredDataset;
import edu.mit.broad.genome.alg.filters.FilteredDatasetImpl_only_genes;
import edu.mit.broad.genome.alg.gsea.DefaultGseaSignalMatrix;
import edu.mit.broad.genome.charts.XChart;
import edu.mit.broad.genome.charts.XComboDomainChart;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Range;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.parsers.MiscParsers;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.RichDataframe;
import edu.mit.broad.genome.reports.pages.*;
import edu.mit.broad.genome.utils.FileUtils;
import edu.mit.broad.genome.utils.ImageUtils;
import edu.mit.broad.genome.utils.SystemUtils;
import edu.mit.broad.genome.utils.Zipper;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.xbench.core.api.Application;
import gnu.trove.*;
import org.apache.ecs.Element;
import org.apache.log4j.Logger;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;
import xtools.api.Tool;
import xtools.api.param.DataFormatChooser;
import xtools.api.param.GeneSetMatrixFormatParam;
import xtools.api.param.ReportLabelParam;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URI;
import java.util.*;


/**
 * Both a Report and a ReportWriter (for a Tool)
 * <p/>
 * Automagically:
 * 1) makes reports dir if not already extant
 * 2) all reports data saved in tha dir
 * 3) makes files on request -- stamped with a common timestamp to avoid name clashes
 * 4) on complete, saves a Report of itself
 * <p/>
 * <p/>
 * A Report is a colection of zero, one or more files (pdfs, txts etc)
 * and charts.
 * <p/>
 * interface reprsenting a Report that is also a writing mechanism
 * Abstracts out the format - html, pdf etc
 * <p/>
 * Idea here is for the reports to maintain an "index file" with links etc to all the added stuff
 * This would be the kick off point for someone to look at the data
 * that the reports produced. If odf, would mostly all be in the file
 * if htnl, would be linked etc
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

// dont extend abstractobject -- easier to impl ourselves here as impl not pob but reports

public class ToolReport implements Report, ReportWriter {

    private transient Logger log = XLogger.getLogger(this.getClass());
    private transient static Logger klog = XLogger.getLogger(ToolReport.class);

    /**
     * @maint ALL OF THE OBJECTS STORED NEED TO BE SERIALIZABLE (as stored in http session)
     * @maint IMP IMP IMP
     * If you add more objects make sure to check if the sizeXXX and allXXX methods
     * need to be updated
     * @maint IMP IMP IMP
     * If you add more objects make sure to check if the sizeXXX and allXXX methods
     * need to be updated
     * @maint IMP IMP IMP
     * If you add more objects make sure to check if the sizeXXX and allXXX methods
     * need to be updated
     * @maint IMP IMP IMP
     * If you add more objects make sure to check if the sizeXXX and allXXX methods
     * need to be updated
     */

    /**
     * @maint IMP IMP IMP
     * If you add more objects make sure to check if the sizeXXX and allXXX methods
     * need to be updated
     */
    private static String COMMON_ERROR_PREFIX = "The Tool ran successfully but at least one part of the reports production failed\nwith the following details\nThe reports is INcomplete";

    /**
     * The tool for which this reports is being generated
     */

    // Try to NOT store the report as it could be heavy
    // // (see Web stuff for example where each page is a tool and we dont want to cache)
    private Tool fTool_opt; // not always cached

    private Class fProducerClass;

    private String fProducerName;

    /**
     * All reports files get saved within here - see gp note below
     */
    private File fReportDir;

    private ToolComments fToolComments;

    private final long fTimestamp = System.currentTimeMillis();
    /**
     * holds whether or not the reports has been closed
     */
    private boolean fClosed = false;

    private int fNumPagesAdded;

    /**
     * Name of this reports - NOT the analysis
     */
    private String fReportName;

    /**
     * File in which this reports gets saved
     */
    private File fReportParamsFile;

    private List fErrors;

    /**
     * Contains Page objects
     */
    private Pages fPages;

    /**
     * This is the central method that adds pages to
     *
     * @param page
     * @param file
     */
    private boolean fDoneAddingCss;

    // so that we know whether to remane or not
    private boolean fRptDirMadeExternally = false;

    private KeyValTable fKvt;

    // not always made --- tools can disable if needed
    private HtmlReportIndexPage fHtmlReportIndexPage;

    private File fHtmlIndexPageFile;
    private ReportIndexState fReportIndexState;

    /**
     * Class constructor
     *
     * @param reportForTool
     * @param makeReportSubDir
     * @throws IOException
     * @throws IllegalArgumentException
     */
    public ToolReport(final Tool reportForTool,
                      final boolean cacheToolObject,
                      final ReportIndexState indexState) throws IOException, IllegalArgumentException {

        if (reportForTool == null) {
            throw new IllegalArgumentException("Parameter reportForTool cannot be null");
        }

        // this is the PARENT dir (i.e -out) and NOT the specific rpt dir (the one with the timestamp)
        if (reportForTool.getParamSet().getAnalysisDirParam() == null) {
            throw new IllegalArgumentException("Specified tool does not have a declared fAnalysisDirParam(-out) -- cannot generate Report!");
        }

        // this is the PARENT dir (i.e -out) and NOT the specific rpt dir (i,e its not the one with the timestamp)
        File analysisEnvBaseDir = reportForTool.getParamSet().getAnalysisDirParam().getAnalysisDir();

        // This is the dir into which rpt files are written
        File rptDir = createIfNeededAndGetReportDir(analysisEnvBaseDir, reportForTool, this);

        init(reportForTool, cacheToolObject, rptDir, indexState);
    }


    /**
     * Class constructor
     *
     * @param reportForTool_opt
     * @param useThisRptDir
     * @param makeReportIndexPage
     * @throws IllegalArgumentException
     */
    public ToolReport(final Tool reportForTool_opt,
                      final boolean cacheToolObject,
                      final File useThisRptDir,
                      final ReportIndexState indexState) throws IllegalArgumentException {

        init(reportForTool_opt, cacheToolObject, useThisRptDir, indexState);
        this.fRptDirMadeExternally = true;
    }

    // central init routine
    private void init(final Tool reportForTool_opt,
                      final boolean cacheToolObject,
                      final File useThisRptDir,
                      final ReportIndexState indexState) throws IllegalArgumentException {

        /*
        if (reportForTool == null) {
            throw new IllegalArgumentException("Param reportForTool cannot be null");
        }
        */


        if (!useThisRptDir.exists()) {
            throw new IllegalArgumentException("Report dir does not exists!!: " + useThisRptDir.getAbsolutePath());
        }

        this.fReportDir = useThisRptDir;
        this.fToolComments = new ToolComments();

        if (reportForTool_opt != null) {
            this.fProducerClass = reportForTool_opt.getClass();
            this.fProducerName = reportForTool_opt.getName();

            if (cacheToolObject) {
                this.fTool_opt = reportForTool_opt;
            }

            // parameter checks
            // just a helpful tool check -- no real need in this class
            if (reportForTool_opt.getParamSet().getReportLabelParam() == null) {
                throw new IllegalArgumentException("Specified tool does not have a declared ReportLabelParam -- check Tool!");
            }
            this.fReportName = generateReportName(this, reportForTool_opt);
            // this is the PARENT dir (i.e -out) and NOT the specific rpt dir (the one with the timestamp)
            this.fReportParamsFile = new File(fReportDir, fReportName);
        }

        this.fPages = new Pages();

        this.fReportIndexState = indexState;

        if (fReportIndexState.makeReportIndexPage()) {
            this.fHtmlReportIndexPage = new HtmlReportIndexPage(this, fReportIndexState.getHeader());
        }
    }


    /**
     * Here because of linkage issues
     * Convention
     * label.toolname.timestamp.rpt
     *
     * @param report
     * @param tool
     * @return
     */
    private static String generateReportName(final Report report, final Tool tool) {

        final ReportLabelParam lp = tool.getParamSet().getReportLabelParam();
        String label = lp.getReportLabel();

        if (label == null) {
            label = "my_report";
        }

        label = label.trim();

        StringTokenizer tok = new StringTokenizer(tool.getName(), ".");
        String tn = null;

        while (tok.hasMoreTokens()) {
            tn = tok.nextToken();
        }

        StringBuffer rptName = new StringBuffer();
        rptName.append(label).append('.').append(tn).append('.').append(report.getTimestamp()).append('.');
        rptName.append(DataFormat.RPT_FORMAT.getExtension());

        return rptName.toString();
    }

    public File getZipReportFile() {
        return new File(getReportDir(), getName() + ".zip");
    }

    public File zipReport() {

        File zipped_file = getZipReportFile();

        try {

            File tmp_zipped_file = File.createTempFile(getName(), ".zip");
            Zipper zipper = new Zipper();
            zipper.zipIt(fReportDir, tmp_zipped_file);

            // ok, move it into the dir once it has been zipped up
            tmp_zipped_file.renameTo(zipped_file);

        } catch (Throwable t) {
            log.error(t);
            FileUtils.writeSafely(t.getMessage(), zipped_file);
        }

        return zipped_file;
    }

    public URI getReportIndex() {
        if (fHtmlIndexPageFile == null) {
            return null;
        } else {
            return fHtmlIndexPageFile.toURI();
        }
    }

    public File getParamsFile() {
        return new File(fReportParamsFile.getPath()); // safe clone
    }

    // handle all errored out stuff here
    // currently just a dir renaming for easy id
    // note that only used for complete errors
    // THIS MUST NEVER THROW AN EXCEPTION
    public void setErroredOut() {
        try {
            // dont rename if a report dir wasnt made here!!
            if (fReportDir != null && fReportDir.exists()) {
                File errorDir = new File(fReportDir.getParentFile(), "error_" + fReportDir.getName());
                if (!fRptDirMadeExternally) {
                    closeReport(false); // @note added june6 dont add to cache
                    log.info("Renaming rpt dir on error to: " + errorDir);
                    boolean renamed = fReportDir.renameTo(errorDir);
                    if (!renamed) {
                        log.warn("Could not rename for error to: " + errorDir);
                    }
                } else {
                    log.info("Pseudo Renaming rpt dir on error to: " + errorDir + " (ext made report dir so not renamed)");
                }
            } else {
                log.info("No report dir was made yet (but an error condition was detected)");
            }
        } catch (Throwable t) {
            System.out.println("Error while erroring out! (setErroredOut in ToolReport");
            t.printStackTrace();
        }
    }

    /**
     * @return The name of this reports
     * @see edu.mit.broad.genome.NamingConventions for format
     */
    public String getName() {
        return fReportName;
    }

    public long getTimestamp() {
        return fTimestamp;
    }

    /**
     * IMP -> if headless then never displayed irrespctive of the vdbgui param
     * Display the Report's contents in a simple GUI.
     * JFrame with a JList holding objects in this reports -> only Files and Charts
     * Double clicking / right popup on an elemnt brings up the File/Chart
     * in a viewer. For files, native viewers only -> Acrobat, etc
     * For Charts -> just put in a Chart displayer
     */

    // IMP -> make sure that xomics does not get initialized due to this call
    public void display() {

        if (SystemUtils.isHeadless()) {
            log.debug("Suppressing display reports as headless mode");
        } else if (SystemUtils.isPropertyTrue("GSEA")) {
            //log.debug("Suppressing display reports as gsea app");
        } else {
            log.debug("Displaying reports ...");
            ToolReportDisplay display = new ToolReportDisplay(this);
            display.show();
        }
    }

    public Id getId() {
        throw new NotImplementedException();
    }

    public String getNameEnglish() {
        return null;
    }

    public String getQuickInfo() {
        return "" + fNumPagesAdded;
    }

    public HtmlReportIndexPage getIndexPage() {
        if (fHtmlReportIndexPage == null) {
            throw new IllegalArgumentException("HtmlReportIndexpage was not made");
        }
        return fHtmlReportIndexPage;
    }

    public File getIndexPageFile() {
        return fHtmlIndexPageFile;
    }

    public File[] getFilesProduced() {
        List files = fPages.getFiles_list();
        if (fHtmlIndexPageFile != null && fHtmlIndexPageFile.exists() && !files.contains(fHtmlIndexPageFile)) {
            files.add(fHtmlIndexPageFile);
        }

        return (File[]) files.toArray(new File[files.size()]);
    }

    public Page[] getPagesProduced() {
        return fPages.getPages();
    }

    public int getNumPagesMade() {
        //System.out.println("$$$$$$$$$$$$$$$$$$ " + fPages.size() + " " + fNumPagesAdded);
        //return fPages.size();
        return fNumPagesAdded;
    }

    public void addComment(String comment) {
        fToolComments.add(comment);
    }

    // this must never fail
    public void addError(final String msg, final Throwable t) {
        if (fErrors == null) {
            fErrors = new ArrayList();
        }

        fErrors.add(t);
        log.error(msg, t);
    }

    public String getComment() {
        throw new RuntimeException("use getToolComment instead");
    }

    public ToolComments getToolComments() {
        return fToolComments;
    }

    public File getReportDir() {
        return fReportDir;
    }

    /**
     * Report impl
     *
     * @return
     */
    public Properties getParametersUsed() {
        if (fTool_opt != null) {
            return fTool_opt.getParamSet().toProperties();
        } else {
            throw new IllegalStateException("Tool not cached: " + fTool_opt);
        }
    }

    public Tool getTool() {
        if (fTool_opt != null) {
            return fTool_opt;
        } else {
            throw new IllegalStateException("Tool not cached: " + fTool_opt);
        }
    }

    /**
     * Report impl
     *
     * @return
     */
    public Class getProducer() {
        return fProducerClass;
    }

    public String getProducerName() {
        return fProducerName;
    }

    public void savePage(final String title, final String caption, final BufferedImage xc) {
        if (fHtmlReportIndexPage != null) {
            fHtmlReportIndexPage.addImage(title, caption, xc);
        } else {
            savePage(title, caption, xc, getReportDir());
        }
    }

    public void savePage(final String title, final String caption, final BufferedImage xc, int width, int height) {
        if (fHtmlReportIndexPage != null) {
            fHtmlReportIndexPage.addImage(title, caption, xc, width, height, null);
        } else {
            savePage(title, caption, xc, width, height, getReportDir());
        }
    }

    public File savePage(final String title, final String caption, final BufferedImage xc, final File inDir) {
        File file = _createFile(title, "png", inDir);

        try {
            if (fHtmlReportIndexPage != null) {
                fHtmlReportIndexPage.addImage(title, caption, xc);
            }

            ImageUtils.savePng(xc, file);
            return file;
        } catch (Throwable t) {
            addError("Trouble saving image", t);
        }

        return file;
    }

    public File savePage(final String title, final String caption, final BufferedImage xc, final int width, final int height, final File inDir) {
        File file = _createFile(title, "png", inDir);

        try {
            if (fHtmlReportIndexPage != null) {
                fHtmlReportIndexPage.addImage(title, caption, xc, width, height, null);
            }

            ImageUtils.savePng(xc, file);
            return file;
        } catch (Throwable t) {
            addError("Trouble saving image", t);
        }

        return file;
    }

    public void savePage(final DatasetTemplate dt, final File inDir) {
        savePage(dt.getDataset(), inDir);
        savePage(dt.getTemplate(), inDir);
        savePage_template(dt.getTemplate(), true, false, inDir);
    }

    public File savePage(final XChart xc, final int width, final int height) {
        return this.savePage(xc, width, height, fReportDir);
    }

    public File savePage(final XChart xc) {
        return this.savePage(xc, 500, 500, fReportDir);
    }

    public File savePage(final XComboDomainChart xc, final int width, final int height) {
        return this.savePage(xc.getCombinedChart(), width, height);
    }

    public File savePage(final XComboDomainChart xc) {
        return this.savePage(xc.getCombinedChart());
    }

    public File savePage(final XChart xc, final int width, final int height, final File inDir) {
        File file = _createFile(xc.getName(), "png", inDir);

        try {
            xc.saveAsPNG(file, width, height);
            return file;
        } catch (Throwable t) {
            addError("Trouble saving png image", t);
        }

        return file;
    }

    public File savePageXML(final String fileName, final org.dom4j.Document doc) {

        File file = null;

        try {

            file = _createFile(fileName, "xml");
            PrintWriter pw = new PrintWriter(new FileOutputStream(file));
            final OutputFormat format = OutputFormat.createPrettyPrint();
            final XMLWriter writer = new XMLWriter(pw, format);
            writer.write(doc);
            writer.close();

            /* // @note @todo central adding?/
            if (! addInvisibly2Cache) {
                _centralAddPage(page, file);
            }
            */

        } catch (Throwable t) {
            addError("Trouble saving Page", t);
        }

        return file;
    }

    public File savePageXML(final String fileName, final org.dom4j.Document doc, final File inDir) {

        File file = null;

        try {

            file = _createFile(fileName, "xml", inDir);
            PrintWriter pw = new PrintWriter(new FileOutputStream(file));
            final OutputFormat format = OutputFormat.createPrettyPrint();
            final XMLWriter writer = new XMLWriter(pw, format);
            writer.write(doc);
            writer.close();

            /* // @note @todo central adding?/
            if (! addInvisibly2Cache) {
                _centralAddPage(page, file);
            }
            */

        } catch (Throwable t) {
            addError("Trouble saving Page", t);
        }

        return file;
    }

    /**
     * The file created IS automatically added to the reports
     *
     * @return
     */
    public File savePage(final Page page) {
        return savePage(page, false);
    }

    public File savePage(final Page page, final boolean addInvisibly2Cache) {
        File file = null;

        try {

            file = _createFile(page);
            page.write(new FileOutputStream(file), getReportDir());
            if (!addInvisibly2Cache) {
                _centralAddPage(page, file);
            }

        } catch (Throwable t) {
            addError("Trouble saving Page", t);
        }

        return file;
    }

    public File savePage(final Page page, final File inDir) {
        return savePage(page, inDir, true);
    }

    public File savePage(final Page page, final File inDir, final boolean addInvisibly2Cache) {
        File file = null;

        try {

            file = _createFile(page.getName(), page.getExt(), inDir);
            page.write(new FileOutputStream(file), inDir);
            if (!addInvisibly2Cache) {
                _centralAddPage(page, file);
            }

        } catch (Throwable t) {
            addError("Trouble saving Page", t);
        }

        return file;
    }

    public void savePage(final Page[] pages, boolean addInvisibly2Cache) {
        for (int p = 0; p < pages.length; p++) {
            savePage(pages[p], addInvisibly2Cache);
        }
    }

    public void savePage(final String name, final String desc, final TIntIntHashMap map) {

        TIntIntIterator iterator = map.iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }

        savePage(name, desc, buf.toString());
    }

    public void savePage(final String name, final String desc, final TIntFloatHashMap map) {

        TIntFloatIterator iterator = map.iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }

        savePage(name, desc, buf.toString());
    }

    public void savePage(final String name, final String desc, final TFloatIntHashMap map) {

        TFloatIntIterator iterator = map.iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }

        savePage(name, desc, buf.toString());
    }

    public void savePage(final String name, final String desc, final TObjectIntHashMap map) {
        savePage(name, desc, map, false);
    }

    public void savePage(final String name, final String desc, final TObjectIntHashMap map, final boolean silent, final File dir) {
        TObjectIntIterator iterator = map.iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }


        savePage(name, desc, buf.toString(), "xls", dir, silent);
    }

    public void savePage(final String name, final String desc, final TObjectIntHashMap map, final boolean silent) {

        TObjectIntIterator iterator = map.iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }

        savePage(name, desc, buf.toString(), silent);
    }

    public void savePage(final String name, final String desc, final TObjectFloatHashMap map) {

        TObjectFloatIterator iterator = map.iterator();

        StringBuffer buf = new StringBuffer();
        for (int i = map.size(); i-- > 0;) {
            iterator.advance();
            buf.append(iterator.key()).append('\t').append(iterator.value()).append('\n');
        }

        savePage(name, desc, buf.toString());
    }

    public void savePageRanges_NameByMin(String name, String desc, Range[] ranges) {

        StringBuffer buf = new StringBuffer();
        float tot = (float) RangeUtils.getTotalCount(ranges);

        for (int r = 0; r < ranges.length; r++) {
            String rname = Double.toString(ranges[r].getMin());
            float frac = ((float) ranges[r].getCount()) / tot;
            buf.append(rname).append('\t').append(ranges[r].getCount()).append('\t').append(frac).append('\n');
        }

        String pdesc = "by minimum";
        if (desc != null) {
            pdesc += " " + desc;
        }

        savePage(name, pdesc, buf.toString());
    }

    public void savePageRanges_NameByMax(final String name, final String desc, final Range[] ranges) {

        float tot = (float) RangeUtils.getTotalCount(ranges);

        StringBuffer buf = new StringBuffer();
        for (int r = 0; r < ranges.length; r++) {
            String rname = Double.toString(ranges[r].getMin());
            float frac = ((float) ranges[r].getCount()) / tot;
            buf.append(rname).append('\t').append(ranges[r].getCount()).append('\t').append(frac).append('\n');
        }

        String pdesc = "by maximum";
        if (desc != null) {
            pdesc += " " + desc;
        }

        savePage(name, pdesc, buf.toString());
    }

    public void savePageRanges(final String name, final String desc, final Range[] ranges) {
        float tot = (float) RangeUtils.getTotalCount(ranges);

        StringBuffer buf = new StringBuffer();
        for (int r = 0; r < ranges.length; r++) {
            String rn = ranges[r].getName();
            float frac = ((float) ranges[r].getCount()) / tot;
            buf.append(rn).append('\t').append(ranges[r].getCount()).append('\t').append(frac).append('\n');
        }

        savePage(name, desc, buf.toString());
    }

    public void savePages_gmp(final GeneSetMatrixPair gmp) {
        savePage(gmp.getUp());
        savePage(gmp.getDown());
    }

    public void savePages_gmp(final GeneSetMatrixPair[] pairs) {
        for (int i = 0; i < pairs.length; i++) {
            savePages_gmp(pairs[i]);
        }

    }

    public File savePage(final PersistentObject pob) {
        if (pob == null) {
            addError("Null pob specified for saving + " + pob, new Throwable());
            return null;
        }

        try {
            return savePage(pob.getQuickInfo(), pob);
        } catch (Throwable t) {
            addError("Trouble saving pob to report", t);
        }

        return null;
    }

    public void savePage_templates(final String prefix,
                                   final TemplateFromAnnot[] dts,
                                   final boolean saveGcFormat,
                                   final boolean saveXtFormat,
                                   final boolean add2cache) throws Exception {

        if (saveXtFormat) {
            File inDir;
            if (prefix == null || prefix.length() == 0) {
                inDir = createSubDir("phenotypes");
            } else {
                inDir = createSubDir(prefix + "_phenotypes");
            }

            for (int t = 0; t < dts.length; t++) {
                savePage(dts[t], inDir, add2cache);
            }
        }

        if (saveGcFormat) {
            File inDir = createSubDir(prefix + "_phenotypes_gc");
            for (int t = 0; t < dts.length; t++) {
                File file = NamingConventions.createSafeFile(inDir, dts[t].getName() + ".cls");
                FileUtils.write(dts[t].getAsString(true), file);
            }
        }

    }

    public File savePage_template(final Template temp, final boolean gcFormat) {
        return savePage_template(temp, gcFormat, true);
    }

    public File savePage_template(final Template temp, final boolean gcFormat, final boolean centralAddPage) {
        String name = temp.getName();
        if (gcFormat) {
            name = name + "_gc";
        }
        return savePage_template(temp, gcFormat, name, centralAddPage);
    }

    public File savePage_template(final Template temp, final boolean gcFormat, final boolean centralAddPage, final File saveInDir) {
        String name = temp.getName();
        if (gcFormat) {
            name = name + "_gc";
        }
        return savePage_template(temp, gcFormat, name, centralAddPage, saveInDir);
    }

    public File savePage_template(final Template temp,
                                  final boolean gcFormat,
                                  final String name, final boolean centralAddPage) {

        return savePage_template(temp, gcFormat, name, centralAddPage, getReportDir());
    }

    public File savePage_template(final Template temp,
                                  final boolean gcFormat,
                                  final String name,
                                  final boolean centralAddPage,
                                  final File saveInDir) {

        File file = null;
        try {
            file = _createFile(name, DataFormat.getExtension(temp), saveInDir);
            ParserFactory.save(temp, gcFormat, file);
            _centralAddPage(new FileWrapperPage(file, temp.getQuickInfo()), centralAddPage);    // @note
        } catch (Throwable t) {
            addError("Trouble saving pob to report", t);
        }

        return file;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    public void savePage(final FilteredDataset fds) {
        savePage(fds, getReportDir());
    }

    public void savePage(final FilteredDataset fds, final File inDir) {

        savePage(fds.getGeneSet(true), inDir);
        savePage(fds.getGeneSet(false), inDir);

        if (!(fds instanceof FilteredDatasetImpl_only_genes)) {
            savePage(fds.getDataset(true), inDir);
            savePage(fds.getDataset(false), inDir);
        }

    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public File savePage(final PersistentObject pob, final File inDir) {
        return savePage(pob, inDir, true);
    }

    public File savePage(final PersistentObject pob, File inDir, boolean add2cache) {
        try {
            return savePage(pob.getQuickInfo(), pob, inDir, add2cache);
        } catch (Throwable t) {
            addError("Trouble saving pob to report", t);
        }

        return null;
    }


    public File savePage(final PersistentObject pob, boolean add2cache) {
        try {
            return savePage(pob.getQuickInfo(), pob, getReportDir(), add2cache);
        } catch (Throwable t) {
            addError("Trouble saving pob to report", t);
        }

        return null;
    }

    public File savePageXls(final StringDataframe sdf, boolean replaceNaNWithEmpty) {
        if (replaceNaNWithEmpty) {
            sdf.replace("NaN", "");
        }
        return savePageXls(sdf);
    }

    public File savePageXls(final StringDataframe sdf, boolean replaceNaNWithEmpty, final File inDir) {
        if (replaceNaNWithEmpty) {
            sdf.replace("NaN", "");
        }
        return savePageXls(sdf, inDir);
    }

    /*
    public File savePageXls_split(final StringDataframe sdf, int numColsPerRecord) {

        if (sdf.getNumCol() % numColsPerRecord != 0) {
            fErrors.add("Not divisible: " + numColsPerRecord);
            savePageXls(sdf);
        }

        int numRecdsPerPage = 256 / numColsPerRecord;



        List columns = new ArrayList();


    }
    */

    public File savePageXls(final IDataframe idf) {
        return this.savePageXls(idf, idf.getName());
    }

    public File savePageXls(final IDataframe idf, String fileName) {
        return this.savePageXls(idf, fileName, fReportDir);
    }

    public File savePageXls(final IDataframe idf, final File inDir) {
        return this.savePageXls(idf, idf.getName(), inDir);
    }

    public File savePageXls(final IDataframe idf, final String fileName, final File inDir) {

        if (idf instanceof RichDataframe) { // @note hack
            StringDataframe sdf = (StringDataframe) ((RichDataframe) idf).getDataframe();
            return savePageXls(sdf, fileName, inDir); // recall this
        }

        if (idf instanceof StringDataframe) {
            ((StringDataframe) idf).replace("NaN", ""); // @note default bhaviour
        }

        File file = _createFile(fileName, DataFormat.XLS_FORMAT.getExtension(), inDir);
        try {

            ParserFactory.saveInvisibly2Cache(idf, file);
            _centralAddPage(new FileWrapperPage(file, idf.getQuickInfo()));    // @note


        } catch (Throwable t) {
            addError("Trouble saving sdf to report", t);
        }

        return file;
    }

    /**
     * Use with care - remember that objects get auto added to the cache
     *
     * @param pob
     */
    public File savePage(final String desc, final PersistentObject pob) {
        return savePage(desc, pob, fReportDir);
    }

    public File savePage(final String name, final String desc, final PersistentObject pob) {
        return savePage(name, desc, pob, fReportDir, true);
    }

    public File savePage(final String desc, final PersistentObject pob, final File inDir) {
        return savePage(pob.getName(), desc, pob, inDir, true);
    }

    public File savePage(String name, String desc, final PersistentObject pob, boolean centralAddpage) {
        return savePage(name, desc, pob, fReportDir, centralAddpage);
    }

    public File savePage(String desc, final PersistentObject pob, File inDir, boolean centralAddpage) {
        return savePage(pob.getName(), desc, pob, inDir, centralAddpage);
    }

    public File savePage(String name, final PersistentObject pob, DataFormat df) {
        return savePage(name, pob.getQuickInfo(), pob, getReportDir(), true, df);
    }

    public File savePage(String name, String desc, final PersistentObject pob, File inDir, boolean centralAddPage, DataFormat df) {
        File file = null;
        try {
            file = ParserFactory.save(pob, df, inDir);
            _centralAddPage(new FileWrapperPage(file, desc), centralAddPage);    // @note
        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

        return file;
    }

    // save to SPECIFIC DIR SPECIFIED
    public File savePage(String name, String desc, final PersistentObject pob, File inDir, boolean centralAddPage) {
        File file = null;
        try {

            if (pob instanceof RankedListDb) {
                savePageRdb((RankedListDb) pob, inDir);
            } else if (pob instanceof RankedList) {
                file = _createFile(name, "xls", inDir);
                ParserFactory.save((RankedList) pob, file);
            } else if (pob instanceof Dataset) {
                file = _createFile(name, DataFormat.GCT_FORMAT.getExtension(), inDir);
                ParserFactory.saveGct((Dataset) pob, file);
            } else if (pob instanceof BitSetDataset) {
                Dataset ds = ((BitSetDataset) (pob)).toDataset(true, false);
                file = _createFile(name, "bsd", inDir);
                ParserFactory.saveGct(ds, file);
                File mf = _createFile(name, "mat", inDir);
                MiscParsers.save(ds.getMatrix(), mf);
                _centralAddPage(new FileWrapperPage(mf, desc));    // @note
            } else if (pob instanceof StringDataframe) {
                file = _createFile(name, DataFormat.getExtension(pob), inDir);
                ParserFactory.save((StringDataframe) pob, file, false);
            } else if (pob instanceof LabelledVector) {
                file = savePage(name, desc, (LabelledVector) pob);
            } else {
                file = _createFile(name, DataFormat.getExtension(pob), inDir);
                ParserFactory.save(pob, file);
            }

            //log.debug("******* " + file + " " + pob.getName());

            if (file != null) {
                _centralAddPage(new FileWrapperPage(file, desc), centralAddPage);    // @note
            }

        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

        return file;
    }

    public File savePage(final PersistentObject pob, final DataFormatChooser dfc) {
        File[] files = savePage(new PersistentObject[]{pob}, dfc);
        if (files != null && files.length == 1) {
            return files[0];
        } else {
            return null;
        }
    }

    public File savePage(final PersistentObject pob, final DataFormat df) {
        File[] files = savePage(new PersistentObject[]{pob}, df, fReportDir);
        if (files != null && files.length == 1) {
            return files[0];
        } else {
            return null;
        }
    }

    public File savePage(final PersistentObject pob, final DataFormatChooser dfc, File inDir) {
        File[] files = savePage(new PersistentObject[]{pob}, dfc, inDir);
        if (files != null && files.length == 1) {
            return files[0];
        } else {
            return null;
        }
    }

    public File[] savePage(final PersistentObject[] pobs, final DataFormatChooser dfc) {
        return savePage(pobs, dfc, getReportDir());
    }

    public File[] savePage(final PersistentObject[] pobs, final DataFormatChooser dfc, File inDir) {

        List files = new ArrayList();

        try {

            DataFormat df = dfc.getDataFormat();

            try {
                for (int i = 0; i < pobs.length; i++) {
                    File f = ParserFactory.save(pobs[i], df, inDir);
                    _centralAddPage(new FileWrapperPage(f, "data format " + df.getDesc())); // imp to add to reports else doesnt show up in reports!
                    files.add(f);
                }
            } catch (Throwable t) {
                addError("Trouble saving pob to reports", t);
            }

        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

        return (File[]) files.toArray(new File[files.size()]);
    }

    public File[] savePage(final PersistentObject[] pobs, final DataFormat df, File inDir) {

        List files = new ArrayList();

        try {

            try {
                for (int i = 0; i < pobs.length; i++) {
                    File f = ParserFactory.save(pobs[i], df, inDir);
                    _centralAddPage(new FileWrapperPage(f, "data format " + df.getDesc())); // imp to add to reports else doesnt show up in reports!
                    files.add(f);
                }
            } catch (Throwable t) {
                addError("Trouble saving pob to reports", t);
            }

        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

        return (File[]) files.toArray(new File[files.size()]);
    }

    // --------------------------------------------------------------------------------------------- //
    // ------------------------ START GENE MATRIX METHODS ------------------------------------------ //
    // --------------------------------------------------------------------------------------------- //

    public File savePage(final GeneSetMatrix gm, final GeneSetMatrixFormatParam gmf) {
        return savePage(gm.getQuickInfo(), gm, gmf);
    }

    public File savePage(final String desc, final GeneSetMatrix gm, final GeneSetMatrixFormatParam gmf) {

        try {
            if (gmf.getDataFormat() == DataFormat.GMT_FORMAT) {
                return savePageGmt(desc, gm);
            } else if (gmf.getDataFormat() == DataFormat.GMX_FORMAT) {
                return savePage(desc, gm);
            } else {
                log.warn("Unkown gm format: " + gmf.getDataFormat());
                return savePageGmt(desc, gm);
            }
        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

        return null;
    }

    public File[] savePage(final GeneSetPair gsp) {
        File up = savePage(gsp.getUp());
        File dn = savePage(gsp.getDown());
        return new File[]{up, dn};
    }

    public File[] savePage(final GeneSetPair gsp, final File saveInDir) {
        File up = savePage(gsp.getUp(), saveInDir);
        File dn = savePage(gsp.getDown(), saveInDir);
        return new File[]{up, dn};
    }

    public File savePage(String desc, final GeneSetMatrix gmx) {
        return savePage(desc, gmx, fReportDir);
    }

    public File savePage(String desc, final GeneSetMatrix gmx, final File saveInDir) {
        StringBuffer name = new StringBuffer(gmx.getName()).append('.').append(DataFormat.GMX_FORMAT.getExtension());
        File file = createSafeReportFile(name.toString(), saveInDir);
        log.debug("saving gmt in: " + file);

        try {

            ParserFactory.save(gmx, file);
            _centralAddPage(new FileWrapperPage(file, desc));    // @note

        } catch (Throwable t) {
            addError("Could not save object to reports object: " + gmx + " in file: " + file.getPath(), t);
        }

        return file;
    }

    public File savePageGmx(final GeneSetMatrix gmx) {
        return savePage(gmx.getQuickInfo(), gmx);
    }

    public File savePageGmx(final GeneSetMatrix gmx, final File saveInDir) {
        return savePage(gmx.getQuickInfo(), gmx, saveInDir);
    }

    public File savePageGmt(final GeneSetMatrix gmt) {
        return savePageGmt(gmt.getQuickInfo(), gmt);
    }

    public File savePageGmt(final String name, final GeneSet[] gsets) {
        return savePageGmt(new DefaultGeneSetMatrix(name, gsets));
    }

    public File savePageGmt(final String desc, final GeneSetMatrix gmt) {
        StringBuffer name = new StringBuffer(gmt.getName()).append('.').append(DataFormat.GMT_FORMAT.getExtension());
        File file = createSafeReportFile(name.toString());
        log.debug("saving gmt in: " + file);

        try {

            ParserFactory.saveGmt(gmt, file);
            _centralAddPage(new FileWrapperPage(file, desc));    // @note

        } catch (Throwable t) {
            addError("Could not save object to reports object: " + gmt + " in file: " + file.getPath(), t);
        }

        return file;
    }

    public void savePageInvisibly2Cache(GeneSetMatrix gm) {

        try {
            File file = _createFile(gm.getName(), DataFormat.getExtension(gm));
            ParserFactory.save(gm, file, false);
            _centralAddPage(new FileWrapperPage(file, gm.getQuickInfo()));    // @note

        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

    }

    public void savePageInvisibly2Cache(GeneSetMatrix gm, DataFormat df) {

        try {
            File file = _createFile(gm.getName(), df.getExtension());
            ParserFactory.save(gm, file, false);
            _centralAddPage(new FileWrapperPage(file, gm.getQuickInfo()));    // @note

        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

    }

    // -------------------------------------------------------------------------------------------- //
    public File savePageInvisibly2Cache(final Dataset ds) {
        return savePageInvisibly2Cache(ds, DataFormat.GCT_FORMAT);
    }

    public File savePageInvisibly2Cache(final Dataset ds, final File inDir) {
        return savePageInvisibly2Cache(ds, DataFormat.GCT_FORMAT, inDir);
    }

    public File savePageInvisibly2Cache(final Dataset ds, final DataFormat df) {
        return savePageInvisibly2Cache(ds.getName(), ds.getQuickInfo(), ds, df, fReportDir);
    }

    public File savePageInvisibly2Cache(String name, String desc, Dataset ds) {
        return savePageInvisibly2Cache(name, desc, ds, DataFormat.GCT_FORMAT, fReportDir);
    }

    public File savePageInvisibly2Cache(String name, String desc, Dataset ds, final File inDir) {
        return savePageInvisibly2Cache(name, desc, ds, DataFormat.GCT_FORMAT, inDir);
    }

    public File savePageInvisibly2Cache(final Dataset ds, final DataFormat df, final File inDir) {
        return savePageInvisibly2Cache(ds.getName(), ds.getQuickInfo(), ds, df, inDir);
    }


    public File savePageInvisibly2Cache(String name, String desc, Dataset ds, DataFormat df, final File inDir) {
        File file = null;
        try {
            file = _createFile(name, df.getExtension(), inDir);

            if (df == DataFormat.GCT_FORMAT) {
                ParserFactory.saveInvisibly2CacheGct(ds, file);
            } else if (df == DataFormat.DCHIP_FORMAT) {
                ParserFactory.saveInvisibly2CacheDChip(ds, file);
            } else if (df == DataFormat.TXT_DATASET_FORMAT) {
                ParserFactory.saveInvisibly2CacheTxt(ds, file);
            } else {
                log.warn("Unkown gm format: " + df.getExtension());
                ParserFactory.saveInvisibly2CacheGct(ds, file);
            }

            _centralAddPage(new FileWrapperPage(file, desc));    // @note
        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

        return file;
    }

    public File savePageInvisibly2Cache(GeneSetMatrix gm, GeneSetMatrixFormatParam gmf) {

        File file = _createFile(gm.getName(), gmf.getDataFormat().getExtension());

        try {

            if (gmf.getDataFormat() == DataFormat.GMT_FORMAT) {
                ParserFactory.saveGmt(gm, file, false);
            } else if (gmf.getDataFormat() == DataFormat.GMX_FORMAT) {
                ParserFactory.save(gm, file, false);
            } else {
                log.warn("Unkown gm format: " + gmf.getDataFormat());
                ParserFactory.save(gm, file, false);
            }

            _centralAddPage(new FileWrapperPage(file, gm.getQuickInfo()));    // @note

        } catch (Throwable t) {
            addError("Trouble saving pob to reports", t);
        }

        return file;

    }

    // --------------------------------------------------------------------------------------------- //
    // ------------------------ END GENE MATRIX METHODS -------------------------------------------- //
    // --------------------------------------------------------------------------------------------- //

    public void savePageGmx(final String name, final GeneSetSignal[] signs) {

        final GeneSet[] gsets = new GeneSet[signs.length];
        for (int i = 0; i < signs.length; i++) {
            gsets[i] = signs[i].getAsGeneSet();
        }

        savePage(new DefaultGeneSetMatrix(name, gsets));
    }

    public void savePageGmx(final String name, SignalMatrix sm) {

        GeneSet[] gsets = new GeneSet[sm.getNumSignals()];
        for (int i = 0; i < gsets.length; i++) {
            gsets[i] = sm.getSignal(i).getAsGeneSet();
        }

        savePage(new DefaultGeneSetMatrix(name, gsets));
    }

    public void savePageRL(String desc, final RankedList rl) {
        Chip chip = null;
        savePageRL(desc, rl, chip);
    }

    public void savePageRL(final String desc, final RankedList rl, final Chip chipOpt) {
        savePageRL(desc, rl, chipOpt, fReportDir, true);
    }

    public File savePageRL(final String desc, final RankedList rl, Chip chipOpt, final File saveInDir, final boolean add2Cache) {
        try {
            StringBuffer buf = new StringBuffer();
            for (int r = 0; r < rl.getSize(); r++) {
                String name = rl.getRankName(r);
                if (chipOpt != null) {
                    String symbol = chipOpt.getSymbol(name, Chip.REPLACE_WITH_PROBEID);
                    String title = chipOpt.getTitle(name, Chip.REPLACE_WITH_PROBEID);
                    buf.append(name).append('\t').append(symbol).append('\t').append(title).append('\t').append(rl.getScore(r)).append('\n');
                } else {
                    buf.append(name).append('\t').append(rl.getScore(r)).append('\n');
                }
            }
            return savePage(rl.getName(), desc, buf.toString(), true, add2Cache);
        } catch (Throwable t) {
            addError("Trouble saving rl to reports", t);
        }

        return null;
    }

    public void savePageRdb(final RankedListDb rdb, final File inDir) {
        for (int i = 0; i < rdb.getNumRankedLists(); i++) {
            savePage(rdb.getName() + "_" + i, rdb.getRankedList(i), inDir, false);
        }
    }

    public void savePage(final String[] descs, final PersistentObject[] pobs) {
        for (int i = 0; i < pobs.length; i++) {
            if (descs == null) {
                savePage(pobs[i]);
            } else {
                savePage(descs[i], pobs[i]);
            }
        }
    }

    public File savePageDF(final String desc, final Dataframe df) {
        return savePageDF(desc, df, "xls");
    }

    public File savePageDF(final String desc, final Dataframe df, final File saveInDir) {
        return savePageDF(desc, df, "xls", saveInDir);
    }

    public File savePageDF(final String desc, final Dataframe df, final String ext) {
        return savePageDF(desc, df, ext, getReportDir());
    }

    public File savePageDF(final String desc, final Dataframe df, final String ext, final File saveInDir) {
        StringBuffer name = new StringBuffer(df.getName()).append('.').append(ext);
        File file = createSafeReportFile(name.toString(), saveInDir);
        try {

            ParserFactory.save(df, file);
            _centralAddPage(new FileWrapperPage(file, desc));    // @note

        } catch (Throwable t) {
            addError("Could not save object to reports object: " + df + " in file: " + file.getPath(), t);
        }

        return file;
    }

    public void savePageDF(final Dataframe df) {
        this.savePageDF(df.getQuickInfo(), df);
    }

    public void savePagesDF(final Dataframe[] dfs) {
        for (int i = 0; i < dfs.length; i++) {
            savePageDF(dfs[i]);
        }
    }

    public void savePagesDF(final Dataframe[] dfs, File inDir, boolean saveInCache, Chip chipOpt) {
        for (int i = 0; i < dfs.length; i++) {
            savePage(dfs[i], inDir, saveInCache, chipOpt);
        }
    }

    public File savePage(final Dataframe df, File inDir, boolean add2cache, Chip chipOpt) {
        IDataframe savedf = df;
        if (chipOpt != null) {
            savedf = chipOpt.annotate(df);
        }

        return savePage(savedf, inDir, add2cache);
    }

    public void savePages(final Dataset[] dss) {
        for (int i = 0; i < dss.length; i++) {
            savePage(dss[i]);
        }
    }

    public void savePages(final StringDataframe[] sdf) {
        for (int i = 0; i < sdf.length; i++) {
            savePage(sdf[i]);
        }
    }

    public File savePage(final String desc, final StringDataframe sdf) {
        String ext = "xls";
        StringBuffer name = new StringBuffer(sdf.getName()).append('.').append(ext);
        File file = createSafeReportFile(name.toString());
        //log.debug("saving df as excel in: " + file);
        try {

            ParserFactory.saveInvisibly2Cache(sdf, file); // @note invisible to the parser
            _centralAddPage(new FileWrapperPage(file, desc));    // @note

        } catch (Throwable t) {
            addError("Could not save object to reports object: " + sdf + " in file: " + file.getPath(), t);
        }
        return file;
    }

    public File savePageInvisibly2Cache(final String desc, final IDataframe idf) {
        String ext = "xls";
        StringBuffer name = new StringBuffer(idf.getName()).append('.').append(ext);
        return savePageInvisibly2Cache(name.toString(), desc, idf);
    }

    public File savePageInvisibly2Cache(String name, final String desc, final String ext, final IDataframe idf) {
        name = name + "." + ext;
        return savePageInvisibly2Cache(name, desc, idf);
    }

    public File savePageInvisibly2Cache(final String name, final String desc, final IDataframe idf) {

        File file = createSafeReportFile(name);

        //log.debug("saving df as excel in: " + file);
        try {

            if (idf instanceof StringDataframe) {
                ParserFactory.saveInvisibly2Cache((StringDataframe) idf, file); // @note invisible to the parser
            } else if (idf instanceof Dataframe) {
                ParserFactory.saveInvisibly2Cache((Dataframe) idf, file); // @note invisible to the parser
            } else {
                throw new IllegalArgumentException("Unknown df for saving: " + idf.getClass());
            }

            // dont as we want it to be silent
            // _centralAddPage(new FileWrapperPage(file, desc));

        } catch (Throwable t) {
            addError("Could not save object to reports object: " + idf + " in file: " + file.getPath(), t);
        }

        return file;
    }

    public File savePage(final RichDataframe rdf) {
        if (rdf == null) {
            addError("param red cannot be null", new IllegalArgumentException("Param rdf cannot be null"));
            return null;
        }

        File file = createSafeReportFile(rdf.getName());

        try {

            ParserFactory.save(rdf.getDataframe(), file);
            KeyValTable kvt = rdf.getMetaData().getKvt();
            if (kvt != null) {
                HtmlPage htmlPage = new HtmlPage(rdf.getName() + "_details", null);
                htmlPage.addTable("details", kvt);
                savePage(htmlPage);
            }

        } catch (Throwable t) {
            addError("Trouble saving pob to report", t);
        }

        return file;
    }

    public File savePage(final String name, final String desc, final LabelledVector lv) {
        return savePage(name, desc, lv.toString_newline('\t'));
    }

    public File savePage(final String name, final String desc, final LabelledVector lv, File inDir) {
        return savePage(name, desc, lv.toString_newline('\t'), inDir);
    }

    public File savePage(final String name, final LabelledVector lv) {
        return savePage(name, lv.getQuickInfo(), lv);
    }

    public File savePage(final String name, final LabelledVector lv, final File inDir) {
        return savePage(name, lv.getQuickInfo(), lv, inDir);
    }

    public File savePage(final LabelledVector lv, final File inDir) {
        return savePage(lv.getName(), lv.getQuickInfo(), lv, inDir);
    }

    public File savePage(final String name, final String desc, final LabelledVector[] lvs) {
        StringBuffer buf = new StringBuffer();

        int[] sizes = new int[lvs.length];
        for (int i = 0; i < lvs.length; i++) {
            sizes[i] = lvs[i].getSize();
        }

        int nrows = XMath.max(sizes);

        for (int r = 0; r < nrows; r++) {
            for (int c = 0; c < lvs.length; c++) {
                if (lvs[c].getSize() > r) {
                    buf.append(lvs[c].getLabel(r)).append('\t');
                    buf.append(lvs[c].getScore(r)).append('\t');
                } else {
                    buf.append('\t');
                    buf.append('\t');
                }
            }
            buf.append('\n');
        }

        return savePage(name, desc, buf.toString());
    }

    public File savePage(final String name, final String desc, final RankedList[] rls) {
        StringBuffer buf = new StringBuffer();

        int[] sizes = new int[rls.length];
        for (int i = 0; i < rls.length; i++) {
            sizes[i] = rls[i].getSize();
        }

        int nrows = XMath.max(sizes);

        for (int r = 0; r < nrows; r++) {
            for (int c = 0; c < rls.length; c++) {
                if (rls[c].getSize() > r) {
                    buf.append(rls[c].getRankName(r)).append('\t');
                    buf.append(rls[c].getScore(r)).append('\t');
                } else {
                    buf.append('\t');
                    buf.append('\t');
                }
            }
            buf.append('\n');
        }

        return savePage(name, desc, buf.toString());
    }

    public File savePage(final String name, final String desc, final Vector v) {
        return savePage(name, desc, Printf.format(v, '\n'));
    }

    public File savePage(final String name, final String desc, final int[] ints) {
        return savePage(name, desc, Printf.format(ints, '\n'));
    }

    public File savePage(final String name, final String desc, final float[] floats) {
        return savePage(name, desc, Printf.format(floats, '\n'));
    }

    public File savePage(final String name, final String desc, final Vector[] vs) {
        StringBuffer buf = new StringBuffer();

        int nrows = vs[0].getSize();

        for (int r = 0; r < nrows; r++) {
            for (int c = 0; c < vs.length; c++) {
                buf.append(vs[c].getElement(r)).append('\t');
            }
        }

        return savePage(name, desc, buf.toString());
    }

    public File savePage(final String name, final GeneSetSignal[] signs, final GeneSetMatrix gmOrigQual) {
        return this.savePage(new DefaultGseaSignalMatrix(name, signs, gmOrigQual));
    }

    /*
    public void savePage(final String name, final SignalMatrix signMatrix, final Chip chip) throws Exception {

        List cols = new ArrayList();

        List colNames = new ArrayList();

        for (int s = 0; s < signMatrix.getNumSignals(); s++) {
            String[] labels = signMatrix.getSignal(s).getRankedLabels();
            cols.add(labels);

            if (chip != null) {
                Set labelsSet = new HashSet();
                for (int i = 0; i < labels.length; i++) {
                    labelsSet.add(labels[i]);
                }
                Set hugos = chip.getHugos(new FSet("some", labelsSet));
                cols.add(ReportWidgets.toSymbolStringArray(hugos));
                cols.add(ReportWidgets.toTitleStringArray(hugos));
            }

            float[] fl = signMatrix.getSignal(s).getMetricScores();
            String[] fls = new String[fl.length];
            for (int f = 0; f < fls.length; f++) {
                fls[f] = Float.toString(fl[f]);
            }
            cols.add(fls);

            int[] rl = signMatrix.getSignal(s).getRanks();
            String[] rls = new String[rl.length];
            for (int r = 0; r < rls.length; r++) {
                rls[r] = Integer.toString(rl[r]);
            }
            cols.add(rls);

            colNames.add(signMatrix.getSignal(s).getName() + "_NAME");
            if (chip != null) {
                colNames.add(signMatrix.getSignal(s).getName() + "_GENE");
                colNames.add(signMatrix.getSignal(s).getName() + "_TITLE");
            }
            colNames.add(signMatrix.getSignal(s).getName() + "_SCORE");
            colNames.add(signMatrix.getSignal(s).getName() + "_RANK");
        }

        StringDataframe sdf = new StringDataframe(name, cols, colNames, false);
        savePage(name, sdf);
    }
    */

    public void savePageSvmFu(final Dataset ds, final Template t) {

        try {
            String name = NamingConventions.removeExtension(ds) + ".svm";
            File f = createSafeReportFile(name);
            MiscParsers.exportSvmFu(ds, t, f);
            _centralAddPage(new FileWrapperPage(f, "svm_fu data"));    // @note

        } catch (Throwable te) {
            // save the error
            addError("Could not save object to reports object: " + ds + " to file", te);
        }

    }

    public void savePageWithRowDescriptions(final String filenameprefix, final String desc, final Dataframe df, final List rowdescs) {
        String ext = "xls";
        StringBuffer name = new StringBuffer(filenameprefix).append('.').append(df.getName()).append('.').append(ext);
        File file = createSafeReportFile(name.toString());

        try {

            ParserFactory.saveWithRowDescs(df, rowdescs, file);
            _centralAddPage(new FileWrapperPage(file, desc));    // @note

        } catch (Throwable t) {
            addError("Could not save object to reports object: " + df + " in file: " + file.getPath(), t);
        }

    }

    public void savePageWithRowDescriptions(final String fileprefix, final Dataframe df, final FeatureAnnot ann) {
        savePageWithRowDescriptions(fileprefix, df.getQuickInfo(), df, ann);
    }

    /**
     * any extra anns are ignored
     * any missing anns are replaced with Constants.HYPHEN
     *
     * @param fileprefix
     * @param df
     * @param ann
     */
    public void savePageWithRowDescriptions(final String fileprefix, final String desc, final Dataframe df, final FeatureAnnot ann) {
        List list = new ArrayList(df.getNumRow());
        for (int i = 0; i < df.getNumRow(); i++) {
            String name = df.getRowName(i);
            String desc_ann = ann.getNativeDesc(name);
            if (desc_ann == null) {
                list.add(Constants.HYPHEN);
            } else {
                list.add(desc_ann);
            }
        }

        savePageWithRowDescriptions(fileprefix, desc, df, list);
    }

    public void savePage(final String name, final String desc, final Set set) {
        StringBuffer buf = new StringBuffer();
        Iterator it = set.iterator();
        while (it.hasNext()) {
            Object key = it.next();
            buf.append(key.toString()).append('\n');
        }

        savePage(name, desc, buf.toString());
    }

/*
public void savePage(String name, String desc, TableData tableData, boolean numberRows) {
    int nrows = tableData.getNumRow();
    int ncols = tableData.getNumCol();
    if (numberRows) {
        ncols++;
    }

    String ext = "xls";
    StringBuffer fname = new StringBuffer(name).append('.').append(ext);
    File file = createSafeReportFile(fname.toString());
    log.debug("saving in: " + file);

    try {

        PrintWriter out = new PrintWriter(new FileOutputStream(file));
        StringBuffer buf = new StringBuffer();
        for (int c = 0; c < ncols; c++) {
            if ((numberRows) && (c == 0)) {
                buf.append("Row #").append('\t');
            } else {
                if (numberRows) {
                    buf.append(tableData.getColName(c - 1)).append('\t');
                } else {
                    buf.append(tableData.getColName(c)).append('\t');
                }
            }
        }

        buf.append('\n');
        out.write(buf.toString());
        buf = new StringBuffer();

        for (int r = 0; r < nrows; r++) {
            for (int c = 0; c < ncols; c++) {
                if ((numberRows) && (c == 0)) {
                    buf.append(r).append('\t');
                } else {
                    if (numberRows) {
                        buf.append(tableData.getTableModel().getValueAt(r, c - 1).toString()).append('\t');
                    } else {
                        buf.append(tableData.getTableModel().getValueAt(r, c).toString()).append('\t');
                    }
                }
            }
            buf.append('\n');
            out.write(buf.toString());
            buf = new StringBuffer();
        }

        out.flush();
        out.close();
        fPages.add(new FileWrapperPage(file, desc));    // @note

    } catch (Throwable t) {
        addError("Could not save object to reports tabledata in file: " + file.getPath(), t);
    }
}
*/

    public void savePage(String name, String desc, Map map) {
        StringBuffer buf = new StringBuffer();
        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object val = map.get(key);
            if (val == null) {
                val = "";
            }

            buf.append(key.toString()).append('\t');
            if (val instanceof Set) {
                buf.append(Printf.Full.toString((Set) val)).append('\t').append(((Set) val).size()).append('\n');
            } else {
                buf.append(val.toString()).append('\n');
            }
        }

        savePage(name, desc, buf.toString());
    }

    public File savePage(final String name, final String desc, final String[] ss) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < ss.length; i++) {
            buf.append(ss[i]).append('\n');
        }

        return savePage(name, desc, buf.toString());
    }

    public File savePage(final String name, final String desc, final String content) {
        return savePage(name, desc, content, "xls", false, true);
    }

    public File savePage(final String name, final String desc, final String content, final boolean silent, final boolean add2Cache) {
        return savePage(name, desc, content, "xls", silent, add2Cache);
    }

    public File savePage(final String name, final String desc, final String content, final boolean silent) {
        return savePage(name, desc, content, "xls", silent, true);
    }

    public File savePage(final String name, final String desc, final String content, final File inDir) {
        return savePage(name, desc, content, "xls", inDir, false);
    }

    public File savePageTxt(final String name, final String desc, final String[] lines) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < lines.length; i++) {
            buf.append(lines[i]).append('\n');
        }

        return savePage(name, desc, buf.toString());
    }

    public File savePageTxt(final String name, final String desc, final List lines) {
        return savePageTxt(name, desc, (String[]) lines.toArray(new String[lines.size()]));
    }

    public File savePageTxt(final String name, final String desc, final String[] lines, final File inDir) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < lines.length; i++) {
            buf.append(lines[i]).append('\n');
        }

        return savePage(name, desc, buf.toString(), inDir);
    }

    public File savePageTxt(final String name, final String desc, final String content) {
        return savePage(name, desc, content, "txt", false, true);
    }

    public File savePageTxt(final String name, final String desc, final String content, final String ext) {
        return savePage(name, desc, content, ext, false, true);
    }

    public File savePageTxt(final String name, final String desc, final String content, final File saveInDir) {
        return savePage(name, desc, content, "txt", saveInDir, false, true);
    }

    public File savePageTxt(final String name, final String desc, final String content, final File saveInDir, final boolean silent, final boolean add2cache) {
        return savePage(name, desc, content, "txt", saveInDir, silent, add2cache);
    }

    public File savePageTxt(final String name, final String desc, final String content, final boolean silent, final boolean add2cache) {
        return savePage(name, desc, content, "txt", silent, add2cache);
    }

    public File savePage(final String name, final String desc, final String content, final String ext, final boolean silent, final boolean add2Cache) {
        return savePage(name, desc, content, ext, getReportDir(), silent, add2Cache);
    }

    public File savePage(final String name, final String desc, final String content, final String ext, final File saveInDir, final boolean silent, final boolean add2Cache) {
        StringBuffer fname = new StringBuffer(name).append('.').append(ext);
        File file = createSafeReportFile(fname.toString(), saveInDir);

        try {

            if (!silent) {
                log.debug("saving in: " + file);
            }
            FileUtils.write(content, file);
            if (add2Cache) {
                _centralAddPage(new FileWrapperPage(file, desc));  // @note
            }

        } catch (Throwable t) {
            addError("Could not save object to reports data: " + name + " in file: " + file.getPath(), t);
        }

        return file;

    }

    public File savePage(final String name, final String desc, final String content, final String ext, final File inDir, final boolean silent) {
        StringBuffer fname = new StringBuffer(name).append('.').append(ext);
        File file = createSafeReportFile(fname.toString(), inDir);

        try {

            if (!silent) {
                log.debug("saving xls in: " + file);
            }
            FileUtils.write(content, file);
            _centralAddPage(new FileWrapperPage(file, desc));  // @note

        } catch (Throwable t) {
            addError("Could not save object to reports data: " + name + " in file: " + file.getPath(), t);
        }

        return file;

    }

    public void savePage(final String name, final String desc, final Matrix matrix) {
        String ext = "mat";
        StringBuffer fname = new StringBuffer(name).append('.').append(ext);
        File file = createSafeReportFile(fname.toString());

        try {

            log.debug("saving mat in: " + file);
            MiscParsers.save(matrix, file);
            _centralAddPage(new FileWrapperPage(file, desc));  // @note

        } catch (Throwable t) {
            addError("Could not save object to reports data: " + name + " in file: " + file.getPath(), t);
        }

    }

    public void savePageHtml(final String name, final String desc, final String content) {
        String ext = "html";
        StringBuffer fname = new StringBuffer(name).append('.').append(ext);
        File file = createSafeReportFile(fname.toString());

        try {

            log.debug("saving html in: " + file);
            FileUtils.write(content, file);
            _centralAddPage(new FileWrapperPage(file, desc));  // @note

        } catch (Throwable t) {
            addError("Could not save object to reports data: " + name + " in file: " + file.getPath(), t);
        }

    }

    private void _centralAddPage(final Page page, final File file) {
        if (fReportIndexState.keepTrackOfPages() && !(page instanceof HtmlReportIndexPage)) {
            fPages.add(page, file);
        }

        fNumPagesAdded++;

        // NOW FOR THE BUSINESS LOGIC associated with adding pages

        // 1) for html pages, auto add a CSS file. Only do this once
        if (page instanceof HtmlPage) {
            createCss();
        }

    }

    public void createCss() {

        if (!fDoneAddingCss) {
            createCss(fReportDir);
        }

        fDoneAddingCss = true;
    }

    public void createCss(final File inDir) {

        try {

            File cssFile = _createFile("xtools", "css", inDir);
            FileUtils.copy(JarResources.toURL("xtools.css"), cssFile);

        } catch (Throwable t) {
            log.error("Trouble copying over CSS", t);
        }

    }

    public File createSubDir(final String name) {
        File subDir = NamingConventions.createSafeFile(getReportDir(), name);
        if (!subDir.exists()) {
            boolean made = subDir.mkdir();
            if (!made) {
                throw new IllegalStateException("Unable to make an output folder for the report at: " + subDir.getPath());
            }
        }
        createCss(subDir); // @note
        return subDir;
    }

    public File createSubSubDir(final File subDir, final String name) {
        File subSubDir = NamingConventions.createSafeFile(subDir, name);
        if (!subSubDir.exists()) {
            boolean made = subSubDir.mkdir();
            if (!made) {
                throw new IllegalStateException("Unable to make an output folder for the report at: " + subSubDir.getPath());
            }
        }
        createCss(subSubDir); // @note
        return subSubDir;
    }

    private String fOptWebBase;

    public void setWebBaseRelativePath(String s) {
        this.fOptWebBase = s;
    }

    private Element fKvt_key_col_header;
    private Element fKvt_value_col_header;

    public void setKvtHeaders(Element keyColHeader, Element valueColHeader) {
        this.fKvt_key_col_header = keyColHeader;
        this.fKvt_value_col_header = valueColHeader;
    }

    public KeyValTable getKvt() { // placed it in for web module access
        return fKvt;
    }

    private void _centralAddPage(final FileWrapperPage page) {
        _centralAddPage(page, fReportIndexState.keepTrackOfPages());
    }

    private void _centralAddPage(final FileWrapperPage page, boolean keepTrack) {
        if (page == null) {
            throw new IllegalArgumentException("Param page cannot be null");
        }

        if (keepTrack) {
            fPages.add(page);
        }

        fNumPagesAdded++;

        if (fReportIndexState.keepTrackOfPagesInHtmlIndex()) {
            if (fKvt == null) {
                if (fKvt_key_col_header == null) {
                    this.fKvt = new KeyValTable();
                } else {
                    this.fKvt = new KeyValTable(fKvt_key_col_header, fKvt_value_col_header);
                }
            }
            fKvt.addRow(page.getDesc(), page.createLink(fReportDir, fOptWebBase));
        }
    }

    /**
     * Tools MUST call once done
     * IMP dont throw an exception -> dont want this to cause tool to hang
     * after its done plenty of work
     */
    /*
    public void closeReport() {
        closeReport(true);
    }
    */
    public void closeReport(boolean add2ParserFactory) {

        if (fClosed) {
            return;
        }

        // add the rpt params file at the very end
        if (fReportParamsFile != null && fReportParamsFile.exists()) {
            _centralAddPage(new FileWrapperPage(fReportParamsFile, "List of parameters used by the tool (rpt)"));
        }

        // @todo make the errors in the index page somehow
        StringBuffer ebuf = null;
        try {
            if ((fErrors != null) && (fErrors.size() != 0)) {
                ebuf = new StringBuffer(COMMON_ERROR_PREFIX);
                for (int i = 0; i < fErrors.size(); i++) {
                    ebuf.append("ERROR ").append(i + 1).append('\n');
                    Throwable t = (Throwable) fErrors.get(i);
                    ebuf.append(t.getMessage()).append('\n');
                    ebuf.append("---------------------------------------------------");
                    ebuf.append(TraceUtils.getAsString(t)).append('\n');
                }
                savePage("there_were_reporting_errors", "reporting errors", ebuf.toString());
            }
        } catch (Throwable t) {
            log.error("Errors adding errors to reports!", t);
        }

        // deal with the index page (if one was made)
        if (fHtmlReportIndexPage != null) {
            try {
                if (ebuf != null && ebuf.length() > 0) {
                    fHtmlReportIndexPage.addError(ebuf.toString());
                }

                if (fKvt != null && fReportIndexState.keepTrackOfPagesInHtmlIndex()) {
                    this.fHtmlReportIndexPage.addTable("Result files produced in this analysis", fKvt);
                }
                this.fHtmlIndexPageFile = savePage(fHtmlReportIndexPage, false); // we very much want it to be visible!

            } catch (Throwable t) {
                log.error("Error making HtmlIndexPage -- report content may otherwise be OK", t);
            }
        }

        //log.debug(">>>>>>>>>>>>>>>. " + fTool_opt + " " + add2ParserFactory + " fReportParamsFile: " + fReportParamsFile);

        if (fTool_opt != null && add2ParserFactory && fReportParamsFile != null && !fReportParamsFile.exists()) {
            try {
                // save the parameters always to file
                ParserFactory.save(this, fReportParamsFile);

            } catch (Throwable t) {
                log.error("Error closing report -- suppressing", t);
            }
        }

        if (add2ParserFactory) { // @note added July 2 2006

            try {
                // Hmmm add to cache if not asked to save to parserfactory?
                // if (fReportParamsFile != null && add2ParserFactory) {
                if (fReportParamsFile != null) { //
                    // also add to the reports cache
                    File cacheFile = new File(Application.getVdbManager().getReportsCacheDir(), fReportParamsFile.getName());
                    ParserFactory.save(this, cacheFile, false);
                }
            } catch (Throwable t) {
                log.error("Error saving report to cahche -- suppressing", t);
            }
        }


        fClosed = true;
    }

    private File _createFile(final Page page) {
        return _createFile(page.getName(), page.getExt());
    }

    public File createFile(final String name, final String desc) {
        File file = createSafeReportFile(name);
        _centralAddPage(new FileWrapperPage(file, desc));
        return file;
    }

    /**
     * @param filename
     * @param suffix
     * @return
     * @maint File naming conventionmanaged here:
     * name ->> name.timestamp.suffix
     * <p/>
     * Common timestamp to all
     */
    private File _createFile(final String fname, final String suffix) {
        return _createFile(fname, suffix, fReportDir);
    }

    private static File _createFile(final String fname, final String suffix, final File inDir) {

        File file = null;

        try {
            StringBuffer name;

            if (fname.endsWith(suffix)) {
                name = new StringBuffer(fname);
            } else {
                name = new StringBuffer(fname).append('.').append(suffix);
            }

            file = createSafeReportFile(name.toString(), inDir);

            if (FileUtils.isLocked(file)) {
                // dont do this as it can make the file name way too long
                // instead just give it a flavor (10 chars) of what the real name is
                //name = new StringBuffer(fname).append(".WARNING_renamed_on_detecting_lock").append(System.currentTimeMillis()).append('.').append(suffix);
                int max = fname.length();
                if (max > 10) {
                    max = 10;
                }
                name = new StringBuffer(fname.substring(0, max)).append(".WARNING_renamed_on_detecting_lock").append(System.currentTimeMillis()).append('.').append(suffix);
                String name_str = NamingConventions.createSafeFileName(name.toString()); // make doubly sure
                file = new File(inDir, name_str);
            }

            if (!file.exists()) {
                boolean made = file.createNewFile();

                if (!made) {
                    throw new IOException("Could not make file: " + file.getAbsolutePath());
                }
            }

        } catch (Throwable t) {
            if (file == null) {
                file = createSafeReportFile("tmp_report_error_file." + System.currentTimeMillis() + suffix, inDir);
            }
            StringBuffer err = new StringBuffer("Fatal error making file to save a component of the reports in");
            err.append("\nInstead using a result file: ").append(file.getPath());
            klog.fatal(err, t);
        }

        return file;
    }

    // replaces chars that make file paths barf such as @ and #
    private static File createSafeReportFile(final String name, final File inDir) {
        return NamingConventions.createSafeFile(inDir, name);
    }

    private File createSafeReportFile(final String name) {
        return createSafeReportFile(name, fReportDir);
    }

    /**
     * Makes a dir if not already extant
     * Add a pset info file with the timestamp
     *
     * @return sometimes we dont want tools to make explicit sub-dirs
     *         For example some programs, only likes to put output files in the parent dir
     *         Use a -D switch for this
     */
    private static File createIfNeededAndGetReportDir(final File rptWorkingBaseDir,
                                                      final Tool tool,
                                                      final Report rpt) throws IOException {

        if (!rptWorkingBaseDir.exists()) {
            boolean made = rptWorkingBaseDir.mkdir();

            if (!made) {
                throw new IOException("Could not make analysis(-out) results dir at: "
                        + rptWorkingBaseDir);
            }
        }

        // Heres a bit of magix for the GP stuff

        // Now make a report sub-folder ALWAYS
        // but name this WITHOUT a timestamp so that callers can figure out easily
        // Keep this here because of junit linking issues
        //final File rptDir = NamingConventions.generateReportDir(rpt, tool);
        final File rptDir = generateReportDir(rpt, tool);

        if (!rptDir.exists()) {
            boolean made = rptDir.mkdir();

            if (!made) {
                throw new IOException("Could not make a directory to store the Tool reports in. The location attempted was: "
                        + rptDir.getAbsolutePath());
            }
        }

        return rptDir;
    }

    /**
     * analysis_dir/
     * report_label.tool_name.timestamp/
     * <p/>
     * Now make a report sub-folder ALWAYS
     * but name this WITHOUT a timestamp so that callers can figure out easily
     *
     * @param report
     * @param tool
     * @return
     */
    private static File generateReportDir(final Report report, final Tool tool) {

        if (Conf.isMakeReportDirOffMode()) { // use the -out specified dir and dont make a subdir
            return tool.getParamSet().getAnalysisDirParam().getAnalysisDir();
        } else { // make a report sub dir

            final ReportLabelParam lp = tool.getParamSet().getReportLabelParam();
            String label = lp.getReportLabel();

            if (label == null || label.length() == 0) {
                label = "my_report";
            }

            label = label.trim();

            StringTokenizer tok = new StringTokenizer(tool.getName(), ".");
            String toolName = null;

            while (tok.hasMoreTokens()) {
                toolName = tok.nextToken();
            }

            final StringBuffer rptName = new StringBuffer(label).append('.').append(toolName);
            rptName.append('.').append(report.getTimestamp());

            return new File(tool.getParamSet().getAnalysisDirParam().getAnalysisDir(), rptName.toString());
        }
    }

    /**
     * Inner class for class objectiviy when adding pages
     * <p/>
     * Also manages some of the business rules when pages are added - event firing etc
     */
    static class Pages implements java.io.Serializable {

        private List plist;
        private List flist;

        /**
         * Utility fields for the event firing mechanism
         * <p/>
         * Maybe make a better event type later
         */
        //private PropertyChangeSupport pageAddedSupport;

        Pages() {
            this.plist = new ArrayList();
            this.flist = new ArrayList();
            //this.pageAddedSupport = new PropertyChangeSupport(this);
        }

        private void writeObject(java.io.ObjectOutputStream out) {
            klog.debug("Ignoring: " + out);
        }

        private void readObject(java.io.ObjectInputStream in) {
            klog.debug("Ignoring: " + in);
        }

        void add(Page page, File file) {
            plist.add(page);
            flist.add(file);

            // always fire an event to signifiy page added
            //pageAddedSupport.firePropertyChange(new PropertyChangeEvent(this, file.getName(), null, page));
        }

        void add(final FileWrapperPage page) {
            plist.add(page);
            flist.add(page.getFile());

            // always fire an event to signifiy page added
            //pageAddedSupport.firePropertyChange(new PropertyChangeEvent(this, page.getFile().getName(), null, page));

        }

        File[] getFiles() {
            return (File[]) flist.toArray(new File[flist.size()]);
        }

        List getFiles_list() {
            return flist;
        }

        Page[] getPages() {
            return (Page[]) plist.toArray(new Page[plist.size()]);
        }

        int size() {
            return plist.size();
        }

    } // End inner class Pages


    private Properties prp; // lazilly inited

    public void setProperty(String key, String value) {
        if (prp == null) {
            prp = new Properties();
        }
        prp.setProperty(key, value);
    }

    public boolean hasProperty(String key) {
        if (prp == null) {
            return false;
        }
        String val = prp.getProperty(key);
        return val != null && val.length() > 0;
    }

    public String getProperty(String key) {
        if (prp == null) {
            return null;
        }

        return prp.getProperty(key);
    }

}    // End ToolReport

/*--- Formatted in Sun Java Convention Style on Fri, Nov 15, '02 ---*/

