/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports.pages;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.strucs.Hyperlink;
import edu.mit.broad.genome.objects.strucs.Linked;
import org.apache.ecs.Doctype;
import org.apache.ecs.Document;
import org.apache.ecs.Element;
import org.apache.ecs.StringElement;
import org.apache.ecs.html.*;
import org.apache.log4j.Logger;
import xapps.gsea.GseaWebResources;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Sep 17, 2003
 *         Time: 3:34:43 PM
 */
public class HtmlFormat {

    private static final Logger klog = XLogger.getLogger(HtmlFormat.class);

    private static final String CENTER = "center";

    private static final String image = "image";
    private static final String footer = "footer";
    private static final String error = "error";
    private static final String richTable = "richTable";
    private static final String dataTable = "dataTable";
    private static final String keyValTable = "keyValTable";
    private static final String indexTable = "indexTable";
    private static final String table = "table";
    private static final String lessen = "lessen";
    private static final String highlight = "highlight";

    public static String createHoverStyle() {
        return "<style type=\"text/css\">\n" +
                "\n" +
                "div#hover1,div#test2 {\n" +
                "\tborder: 0px solid #000000;\n" +
                "\tmargin-left: 10%;\n" +
                "\twidth: 0px;\n" +
                "}\n" +
                "\n" +
                "div a:hover {\n" +
                "\ttext-decoration: none;\n" +
                "}\n" +
                "\n" +
                "div a span {\n" +
                "\tdisplay: none;\n" +
                "\tpadding: 5px;\n" +
                "\tcolor: #FF0000;\n" +
                "\tbackground-color: #E6EDF5;\n" +
                "\tfont-size: 12px;\n" +
                "\twidth: 250px;\n" +
                "}\n" +
                "\n" +
                "div a:hover span {\n" +
                "\tdisplay: block;\n" +
                "}\n" +
                "\n" +
                "</style>";
    }


    public static void addMeta(final String name, final String content, final Head head) {
        final Meta meta = new Meta();
        meta.setName(name);
        meta.setContent(content);
        head.addElement(meta);
    }

    public static final Link CSS_XTOOLS_CANNED_REPORTS = new Link();
    public static final Link ICON_CANNED_REPORTS = new Link();

    static {
        CSS_XTOOLS_CANNED_REPORTS.setRel("stylesheet");
        CSS_XTOOLS_CANNED_REPORTS.setHref("xtools.css");
        ICON_CANNED_REPORTS.setRel("shortcut icon");
        ICON_CANNED_REPORTS.setHref(GseaWebResources.getGseaBaseURL() + "/images/XBench.gif");
    }

    public static void setCommonDocThings(final String title, final Document doc) {

        // we want the HEAD to look like:
        // <head>
        // <title>Ensembl Genome Browser</title>
        // <link rel="stylesheet" href="/xtools.css">
        // can later add meta tags ala ensembl if needed
        // <meta name="keywords" content="Ensembl, genome, automated annotation, Human Genome Project, Human genetics, DNA sequencing, genome browser">
        //<meta name="description" content="Ensembl is a joint project between EMBL-EBI and the Sanger Institute to develop a software system which produces and maintains automatic annotation on eukaryotic genomes.">
        // <meta name="author" content="webmaster@ensembl.org">
        // </head>

        // set the global html properties needed
        // doctype. I prefer html to xhtml as not really going to be parsed by anything
        // also, the xhtml ecs api uses lower case java classes and thats annoying and not-very-java-like
        doc.setDoctype(new Doctype.Html401Transitional());

        // the title - shows up in the browser title bar and NOT in the content of the page
        doc.setTitle(new Title(title));

        // set the css
        // not this way - style tag isnt needed anymore
        // The STYLE tag is included only to ensure upward compatibility
        // Newer versions of HTML will include support for style sheets, and this tag can be used to
        // provide "in-line" style information. The tag should contain only valid style statements,
        // in the language indicated in the TYPE attribute.
        //this.fDoc.appendBody(new Style(Style.css).addElement("here!"));

        // instead set the css as a link in the head
        /*
        if (remoteCssURLBase != null && remoteCssURLBase.length() > 0) {
            doc.appendHead(remoteCssURLBase + "/" + CSS_XTOOLS_CANNED_REPORTS);
        } else {
            doc.appendHead(CSS_XTOOLS_CANNED_REPORTS);
        }
        */

        doc.appendHead(CSS_XTOOLS_CANNED_REPORTS);
        doc.appendHead(ICON_CANNED_REPORTS);
    }

    public static class GseaWebsite {

        public static final String HTML_HEADER = "\n<!-- START Header -->\n" +
                //"<div id=\"topbanner\">" + // dont use the brad one as it gives too much of gap b/w logo and nav bar
                "<div>" +
                "<a href=\"" + GseaWebResources.getGseaBaseURL() + "/index.html\"><img src=\"images/gsea.gif\" width=\"160\" height=\"80\" alt=\"GSEA logo\" border=\"0\" /></a> \n" +
                "</div>\n" +
                "\n<!-- END Header -->\n";

        /*
        public static final String HTML_BREADCRUMBS = "\n<!-- START Breadcrumbs -->\n" +
                // "<div id=\"breadcrumbs\" style=\"width: 904; height: 33\">\n" +
                "<div id=\"breadcrumbs\">\n" +
                "<a href=\"" + AppWebResources.getGseaBaseURL() + "/index.html\">Home</a><b>&nbsp;&nbsp;&circ;&nbsp;&nbsp;</b>\n" +
                "<a href=\"" + AppWebResources.getGseaBaseURL() + "/software/software_index.html\">Software</a><b>&nbsp;&nbsp;&circ;&nbsp;&nbsp;</b>\n" +
                "<a href=\"" + AppWebResources.getGseaBaseURL() + "/msigdb/msigdb_index.html\">MSigDb</a><b>&nbsp;&nbsp;&circ;&nbsp;&nbsp;</b>\n" +
                "<a href=\"" + AppWebResources.getGseaBaseURL() + "/msigdb/foor_index.html\">GeneSetPages</a><b>&nbsp;&nbsp;&circ;&nbsp;</b>\n" +
                "<a href=\"" + AppWebResources.getGseaBaseURL() + "/wiki\">Documentation</a><b>&nbsp;&nbsp;&circ;&nbsp;&nbsp;</b>\n" +
                "<a href=\"" + AppWebResources.getGseaBaseURL() + "/resources/resources_index.html\">Resources</a> \n" +
                "</div>\n" +
                "\n<!-- END Breadcrumbs -->\n";

        */

        public static final String HTML_BREADCRUMBS = "<!-- START Breadcrumbs -->\n" +
                "\n" +
                "    <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"740\">\n" +
                "        <tr>\n" +
                "            <td>\n" +
                "                <a href=\"index.html\"><img src=\"images/gsea.gif\" width=\"140\" height=\"80\" border=\"0\" alt=\"gsea logo\"/></a>\n" +
                "            </td>\n" +
                "            <td valign=\"top\">\n" +
                "                <div id=\"breadcrumbs_nobg\">\n" +
                "                    <a href=\"index.html\" target=\"_parent\"><img src=\"images/header_home.png\" border=\"0\" alt=\"Home\"/></a>\n" +
                "                    <a href=\"software/software_index.html\" target=\"_parent\"><img src=\"images/header_software.png\" border=\"0\" alt=\"Software\"/></a>\n" +
                "                    <a href=\"msigdb/msigdb_index.html\" target=\"_parent\"><img src=\"images/header_msigdb.png\" border=\"0\" alt=\"MSigDB\"/></a>\n" +
                "                    <a href=\"wiki\" target=\"_parent\"><img src=\"images/header_docs.png\" border=\"0\" alt=\"Documentation\"/></a>\n" +
                "                    <a href=\"resources/resources_index.html\" target=\"_parent\"><img src=\"images/header_resources.png\" border=\"0\" alt=\"Resources\"/></a>\n" +
                "<br>\n" +
                "                </div>\n" +
                "            </td>\n" +
                "\n" +
                "        </tr>\n" +
                "    </table>\n" +
                "\t<div id=\"header_bar\"></div>\n" +
                "    <br>\n" +
                "    <!-- END Breadcrumbs -->";

        public static final String HTML_FOOTER = "\n<!-- START FOOTER -->\n" +
                "<div id=\"footer\">\n" +
                "  <div id=\"toolLinks\">\n" +
                "  <a href=\"" + GseaWebResources.getGseaBaseURL() + "/index.html\">Home</a> | \n" +
                "  <a href=\"" + GseaWebResources.getCancerHomePageURL() + "\">Cancer Genomics</a> |\n" +
                "  <a href=\"" + GseaWebResources.getGseaBaseURL() + "/contact.html\">Contact Us</a> |\n" +
                "  <a href=\"" + GseaWebResources.getGseaBaseURL() + "/sitemap.html\">Site Map</a> |\n" +
                "  </div>\n" +
                "   <div id=\"copyright\">&copy; 2005 Broad Institute</div>\n" +
                "</div>\n" +
                "\n<!-- END FOOTER -->\n";

        public static final String STYLE_CSS_IMPORT = "\n<style type=\"text/css\" media=\"all\">\n" +
                "\t@import \"css/fonts.css\";\n" +
                "\t@import \"css/leftnav.css\";\n" +
                "\t@import \"css/structure.css\";\n" +
                "\t@import \"css/xtools.css\";\n" +
                "</style>\n";


        public static void addCssImports(final Head head) {
            String style = "<style type=\"text/css\" media=\"all\">\n" +
                    "\t@import \"" + GseaWebResources.getGseaBaseURL() + "/css/fonts.css\";\n" +
                    "\t@import \"" + GseaWebResources.getGseaBaseURL() + "/css/leftnav.css\";\n" +
                    "\t@import \"" + GseaWebResources.getGseaBaseURL() + "/css/structure.css\";\n" +
                    "\t@import \"" + GseaWebResources.getGseaBaseURL() + "/css/xtools.css\";\n" +

                    "#myvar_g {\n" +
                    "display:none;\n" +
                    "border:0px;\n" +
                    "background:#f2f2f2;\n" +
                    "padding:10px;\n" +
                    "}\n" +

                    "#myvar_a {\n" +
                    "display:none;\n" +
                    "border:0px;\n" +
                    "font-style:small;\n" +
                    "font-size:10pt;\n" +
                    "background:#c2c2c2;\n" +
                    "padding:10px;\n" +
                    "}\n" +

                    "#myvar_a_vis {\n" +
                    "border:0px;\n" +
                    "font-style:small;\n" +
                    "font-size:10pt;\n" +
                    "background:#c2c2c2;\n" +
                    "padding:10px;\n" +
                    "}\n" +

                    "</style>\n";
            head.addElement(style);
        }

        public static void addMeta(Head head) {
            HtmlFormat.addMeta("description", "Molecular signature database MSigDB", head);
            HtmlFormat.addMeta("keywords", "MSigDB, Molecule signature database, Gene Sets, GSEA, enrichment analysis, Broad Institute", head);
        }

    }

    // -------------------------------------------------------------------------------------------- //
    // ----------------------------- SPECIFIC CLASSES FOR STUFF ----------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    /**
     * Class Links
     */
    public static class Links {

        public static A createHoverLink(String href, String text, String hoverText) {
            A a = new A(href, text);
            Div div_desc = new Div();
            div_desc.setID("hover1");
            div_desc.addElement(new Span(hoverText));
            a.addElement(div_desc);
            return a;
        }

        public static StringElement createExcelHtml_sel(String txt, File excelFile, File htmlFile, File baseDir) {
            StringElement sel = new StringElement();
            sel.addElement(txt);
            sel.addElement(" (");
            A link = new A().addElement("excel");
            setHref(link, excelFile, baseDir);
            sel.addElement(link);
            sel.addElement(", ");

            link = new A().addElement("html");
            setHref(link, htmlFile, baseDir);
            sel.addElement(link);
            sel.addElement(")");

            return sel;
        }

        public static StringElement hyper(final String hyperLinkThisTerm,
                                          final File file,
                                          final String postTerm,
                                          final File baseDir) {
            A link = new A().addElement(hyperLinkThisTerm);
            setHref(link, file, baseDir);
            StringElement sel = new StringElement();
            sel.addElement(link);
            sel.addElement(" " + postTerm);
            return sel;
        }

        public static StringElement hyper(final String hyperLinkThisTerm,
                                          final String url,
                                          final String postTerm_opt) {
            A link = new A(url).addElement(hyperLinkThisTerm);
            StringElement sel = new StringElement();
            sel.addElement(link);
            if (postTerm_opt != null) {
                sel.addElement(" " + postTerm_opt);
            }
            return sel;
        }

        public static StringElement hyperDir(final String hyperLinkThisTerm,
                                             final File dir,
                                             final String postTerm) {

            A link = new A().addElement(hyperLinkThisTerm);
            setHrefToDir(link, dir);
            StringElement sel = new StringElement();
            sel.addElement(link);
            sel.addElement(" " + postTerm);
            return sel;
        }

        public static void setHref(final A link,
                                   final File linkThisFile,
                                   final File baseDir) {

            if (link == null || linkThisFile == null || baseDir == null) {
                return;
            }

            try {
                //link.setHref(file.toURL().toString()); // this makes links absolute and hence not transferable b/w file systems

                File fooBaseDir;
                if (linkThisFile.isDirectory()) {
                    fooBaseDir = linkThisFile;
                } else {
                    fooBaseDir = linkThisFile.getParentFile();
                }

                if (fooBaseDir.equals(baseDir)) {
                    link.setHref(linkThisFile.getName());
                } else { // makes an assumption that only 1 folder down
                    link.setHref(fooBaseDir.getName() + "/" + linkThisFile.getName());
                }

                //System.out.println("file: " + linkThisFile.getPath());
                //System.out.println("file: " + linkThisFile.toURL());
                //System.out.println("file: " + linkThisFile.toURI());

            } catch (Throwable t) {
                t.printStackTrace();
                link.setHref("there was an error: " + t.getMessage());
            }

        }


        public static void setHrefToDir(final A link,
                                        final File linkThisDir) {

            if (link == null || linkThisDir == null) {
                return;
            }

            if (!linkThisDir.isDirectory()) {
                klog.warn("Not a directory: " + linkThisDir);
            }

            try {

                //link.setHref(linkThisDir.getName());
                link.setHref(linkThisDir.toURI().toString());

                //System.out.println("file: " + linkThisDir.getPath());
                //System.out.println("file: " + linkThisDir.toURL());
                //System.out.println("file: " + linkThisDir.toURI());

            } catch (Throwable t) {
                t.printStackTrace();
                link.setHref("there was an error: " + t.getMessage());
            }

        }

        public static void setHref(final A link,
                                   final File linkThisFile,
                                   final String reltoThisBase) {

            if (linkThisFile != null) {
                link.setHref(reltoThisBase + "/" + linkThisFile.getName());
            }
        }

        public static StringElement hyper(final String preTerm,
                                          final String hyperLinkThisTern,
                                          final File file,
                                          final String postTerm,
                                          final File baseDir) {
            A link = new A().addElement(hyperLinkThisTern);
            setHref(link, file, baseDir);
            StringElement sel = new StringElement();
            sel.addElement(preTerm + " ");
            sel.addElement(link);
            sel.addElement(" " + postTerm);
            return sel;
        }


    } // End class Links

    /**
     * Class Divs
     */
    public static class Divs {

        public static Div footer() {
            return new MyDiv(footer);
        }

        public static Div image() {
            return new MyDiv(image);
        }

        public static Div error() {
            return new MyDiv(error);
        }

        public static Div richTable() {
            return new MyDiv(richTable);
        }

        public static Div dataTable() {
            return new MyDiv(dataTable);
        }

        public static Div indexTable() {
            return new MyDiv(indexTable);
        }

        public static Div keyValTable() {
            return new MyDiv(keyValTable);
        }

        public static Div footer(Element addElement) {
            return new MyDiv(footer, addElement);
        }

        public static Div image(Element addElement) {
            return new MyDiv(image, addElement);
        }

        public static Div error(Element addElement) {
            return new MyDiv(error, addElement);
        }

        public static Div richTable(Element addElement) {
            return new MyDiv(richTable, addElement);
        }

        public static Div dataTable(Element addElement) {
            return new MyDiv(dataTable, addElement);
        }

        public static Div keyValTable(Element addElement) {
            return new MyDiv(error, addElement);
        }

        public static Div indexTable(Element addElement) {
            return new MyDiv(indexTable, addElement);
        }

    } // End inner class Divs


    public static class Titles {

        public static Caption table(String text) {
            return new MyTitle(table, "Table: " + text);
        }

    }

    /**
     * Class Table Headers
     */
    public static class THs {
        public static TH indexTable() {
            return new MyTH(indexTable);
        }

        public static TH richTable() {
            return new MyTH(richTable);
        }

        public static TH dataTable() {
            return new MyTH(dataTable);
        }

        public static TH indexTable(String text) {
            return new MyTH(indexTable, text);
        }

        public static TH richTable(String text) {
            return new MyTH(richTable, text);
        }

        public static TH highlightTable(String text) {
            return new MyTH(highlight, text);
        }

        public static TH dataTable(String text) {
            return new MyTH(dataTable, text);
        }

        public static TH keyValTable(String text) {
            return new MyTH(keyValTable, text);
        }

        public static TH keyValTable() {
            return new MyTH(keyValTable);
        }
    }

    public static class TDs {
        public static TD lessen() {
            return new MyTD(lessen);
        }

        public static TD highlight() {
            return new MyTD(highlight);
        }

        public static TD lessen(String text) {
            return new MyTD(lessen, text);
        }

        public static TD highlight(String text) {
            return new MyTD(highlight, text);
        }
    }

    // to make the set class easier
    static class MyDiv extends Div {
        MyDiv(String className) {
            super();
            super.setClass(className);
        }

        MyDiv(String className, Element addElement) {
            this(className);
            super.addElement(addElement);
        }
    }

    static class MyTH extends TH {
        MyTH(String className) {
            super();
            super.setClass(className);
        }

        MyTH(String className, String text) {
            super(text);
            super.setClass(className);

        }
    }

    static class MyTD extends TD {
        MyTD(String className) {
            super();
            super.setClass(className);
        }

        MyTD(String className, String text) {
            super(text);
            super.setClass(className);

        }
    }

    static class MyTitle extends Caption { // Really a caption but in my lingo its a Title

        MyTitle(String className) {
            super();
            super.setClass(className);
        }

        MyTitle(String className, String text) {
            this(className);
            this.addElement(text);
        }
    }

    /**
     * Privatized class constructor
     */
    private HtmlFormat() {
    }

    //-----------------------------------------------------------------//
    public static Caption caption(String s) {
        Caption c = new Caption();
        c.addElement(s);
        return c;
    }

    // key here is to add the text to the font and then add the font to the td
    public static TD _td(String s) {
        if (s != null) {
            s = s.replace('', ' ');
        }

        TD td = new TD(s);
        if (s == null || s.equals(Constants.HYPHEN)) {
            td.setAlign(CENTER);
        }
        return td;
    }

    private static boolean isNullOrNA(String s) {
        if (s == null || s.length() == 0 || s.equalsIgnoreCase(Constants.NA) || s.equalsIgnoreCase(Constants.NULL)) {
            return true;
        }

        return false;
    }

    public static TD _td(final String term,
                         final String bgColor,
                         final Linked linked) {

        TD td = _td(term);

        if (linked != null) {
            Hyperlink[] links = linked.createAllLinks(); // @note we are creating them just in time - no caching etc till here

            if (links.length > 1) { // place the links seperate from the term

                td.addElement(new BR());
                for (int i = 0; i < links.length; i++) {
                    if (isNullOrNA(links[i].getURL())) {
                        td.addElement(new StringElement(links[i].getDisplayName()));
                    } else {
                        A a = new A(links[i].getURL(), links[i].getDisplayName());
                        td.addElement(a);
                    }
                    if (i != links.length - 1) {
                        td.addElement(", &nbsp");
                    }
                }
            } else if (links.length == 1) { // underline the term directly
                if (isNullOrNA(links[0].getURL())) {
                    td = new TD(new StringElement(term));
                } else {
                    A a = new A(links[0].getURL(), term);
                    td = new TD(a);
                }
            }
        }

        //klog.debug(">>>> bgcolor " + bgColor + " linked: " + linked + " term: " + term);
        if (bgColor != null) {
            td = td.setBgColor(bgColor);
        }

        return td;
    }

    protected static TD _td(float f) {
        return new TD(Float.toString(f));
    }

    protected static TD _td(Float f) {
        return _td(f.floatValue());
    }

    protected static TD _td(int i) {
        return _td(i + "");
    }

    protected static TD _td(Double d) {
        return _td(d.floatValue());
    }

    protected static TD _td(Element s) {
        return _td(s.toString());
    }

    protected static TD _td(final Linked l) {
        if (isNullOrNA(l.createDefaultLink().getURL())) {
            return _td(new StringElement(l.getText()));
        } else {
            A a = new A(l.createDefaultLink().getURL(), l.getText());
            return _td(a);
        }
    }

    protected static TD _td(Hyperlink l) {
        if (isNullOrNA(l.getURL())) {
            return _td(new StringElement(l.getDisplayName()));
        } else {
            A a = new A(l.getURL(), l.getDisplayName());
            return _td(a);
        }
    }

    protected static TD _td(Object obj) {
        if (obj instanceof Linked) {
            return _td((Linked) obj);
        } else if (obj instanceof Hyperlink) {
            return _td((Hyperlink) obj);
        } else if (obj instanceof Element) {
            return _td((Element) obj);
        } else if (obj == null) {
            TD td = _td(Constants.HYPHEN);
            td.setAlign(CENTER);
            return td;
        } else {
            return _td(obj.toString());
        }
    }

    protected static A _a(String href, String value) {
        A a = new A();
        a.setHref(href);
        a.setName(value);
        return a;
    }

    protected static String _toPage(String head, Element el) {
        return _toDocument(head, new Element[]{el}).toString();
    }

    protected static Document _toDocument(String head, Element[] els) {

        Document doc = new Document();

        doc.appendHead(head);

        for (int i = 0; i < els.length; i++) {
            doc.appendBody(els[i]);
            doc.appendBody(new BR());
        }

        return doc;
    }

    protected static String _toHtml(Document doc) {
        return doc.toString();
    }

} // End class HFormat
