/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports.pages;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.objects.strucs.Hyperlink;
import edu.mit.broad.genome.objects.strucs.Linked;
import org.apache.ecs.Element;
import org.apache.ecs.html.*;

import java.io.File;

/**
 * Simple table with:
 * <p/>
 * key value
 * <p/>
 * For example a header table with a several simple lines = values
 */
public class KeyValTable {

    private org.apache.ecs.html.Table fTable;

    private int fFloatFormatPrecision;

    /**
     * Class constructor
     */
    public KeyValTable() {
        this("", "");
    }

    public KeyValTable(int floatFormatPrecision) {
        this("", "", floatFormatPrecision);
    }

    public KeyValTable(final String keyTitle, final String valueTitle) {
        this(keyTitle, valueTitle, -1); // @note magic number
    }

    public KeyValTable(final String keyTitle, final String valueTitle, final int floatFormatPrecision) {
        this.fTable = new org.apache.ecs.html.Table(0);
        fTable.setCols(2); // always

        // add the col names
        fTable.addElement(HtmlFormat.THs.keyValTable(keyTitle));
        fTable.addElement(HtmlFormat.THs.keyValTable(valueTitle));
        this.fFloatFormatPrecision = floatFormatPrecision;
    }

    public KeyValTable(final Element keyTitle, final Element valueTitle) {
        this.fTable = new org.apache.ecs.html.Table(0);
        fTable.setCols(2); // always

        // add the col names
        fTable.addElement(new TH(keyTitle));
        fTable.addElement(new TH(valueTitle));
        this.fFloatFormatPrecision = -1;
    }

    public org.apache.ecs.html.Table getTable() {
        return fTable;
    }

    public void addRow(String key, Element el) {
        _addRow(key, el);
    }

    public void addRow(String key, String val) {
        _addRow(key, val);
    }

    public void addRow(String keyOnlyNoVal) {
        _addRow(keyOnlyNoVal, null);
    }

    public void addRow(String key, Linked linked) {
        _addRow(key, linked);
    }

    public void addRow(final String key, final Linked[] linkeds) {

        if (linkeds == null || linkeds.length == 0) {
            _addRow(key, "");
            return;
        }

        TR tr = new TR();
        tr.addElement(HtmlFormat._td(key));

        TD td = new TD();
        for (int i = 0; i < linkeds.length; i++) {
            A a = new A(linkeds[i].createDefaultLink().getURL(), linkeds[i].getText());
            td.addElement(a);
            if (i != linkeds.length - 1) {
                td.addElement("&nbsp&nbsp");
            }
            if (i != 0 && i % 3 == 0 & i != linkeds.length - 1) {
                td.addElement("<BR>");
            }
        }

        tr.addElement(td);

        fTable.addElement(tr);
    }

    public void addRow(String key, Hyperlink hlink) {
        _addRow(key, hlink);
    }

    public void addRow(String key, int val) {
        _addRow(key, new Integer(val));
    }

    public void addRow(final String key, final String[] values) {

        if (values == null || values.length == 0) {
            _addRow(key, "");
            return;
        }

        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < values.length; i++) {
            buf.append(values[i]);
            if (i != values.length - 1) {
                buf.append(Constants.INTRA_FIELD_DELIM);
            }
        }

        _addRow(key, buf.toString());
    }

    public void addRow(String key, float val) {
        Object obj;
        if (fFloatFormatPrecision != -1) {
            obj = Printf.format(val, fFloatFormatPrecision);
        } else {
            obj = new Float(val);
        }

        _addRow(key, obj);
    }

    public void addRow(String key, double val) {
        addRow(key, (float) val);
    }

    // @maint
    // assume in the same directory
    // assume naming conventions
    // @todo improve!!
    public void addRowLink(Page page) {
        if (page == null) {
            _addRow(Constants.NA, null);
        } else {
            String fileName = NamingConventions.createSafeFileName(page.getName());
            this.addRow("", new A(fileName + "." + page.getExt(), page.getDesc()));
        }
    }

    // @todo improve
    public void addRowLinkHtmlExcel(String desc, Page htmlPage, File excelFile) {

        String fileName = NamingConventions.createSafeFileName(htmlPage.getName());
        A html = new A(fileName + "." + htmlPage.getExt(), "HTML");

        A excel = new A(excelFile.getName(), "TEXT");

        Div div = new Div();
        div.addElement(html);
        div.addElement(new BR());
        div.addElement(excel);

        this.addRow(desc, div);
    }

    // does the real stuff
    public void _addRow(String key, Object val) {
        TR tr = new TR();
        tr.addElement(HtmlFormat._td(key));
        tr.addElement(HtmlFormat._td(val));
        fTable.addElement(tr);
    }

    public void _addRow_tmp(String key, Object val) {
        TR tr = new TR();

        TD td = new TD();
        td.setNoWrap(true);
        Font font = new Font(+1, "arial");
        font.addElement(key);
        td.addElement(font);
        tr.addElement(td);


        td = new TD();
        td.setNoWrap(true);
        font = new Font(+1, "arial");
        if (val != null) {
            font.addElement(val.toString());
        }
        td.addElement(font);
        tr.addElement(td);


        fTable.addElement(tr);
    }

} // End inner class Table
