/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.Id;
import edu.mit.broad.genome.utils.containers.ClassNamePair;
import edu.mit.broad.xbench.core.api.Application;
import gnu.trove.TIntArrayList;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.table.*;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.*;


/**
 * A collection of GUI related utilities and methods to aid UI related classes.
 * <p/>
 * <p/>
 * <p/>
 * Notes on the Error notification related methods (showError()): <br>
 * Goals:
 * <ul>
 * <li> Provide a simple api for client code to notify users of errors.
 * <li> Integrate with both the standard netbeans error notification system as
 * well as with a logging system.
 * <li> Alllow the netbeans error manager to take care of displaying exceptions and
 * any chained exceptions.
 * <li>
 * </ul>
 * <p/>
 * - central repositiory of UI related default constants and emepty arrays etc.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @see Ideoms#Helper
 */
public class GuiHelper implements Constants {

    /**
     * STANDARD SET OF ICONS USED
     */
    public static final Icon ICON_ERROR16 = JarResources.getIcon("Error.gif");
    public static final Icon ICON_REFRESH16 = JarResources.getIcon("Refresh16.png");
    public static final Icon ICON_START16 = JarResources.getIcon("Run16.png");
    public static final Icon ICON_PAUSE16 = JarResources.getIcon("Pause16.gif");
    public static final Icon ICON_INTEGER = JarResources.getIcon("Integer.gif");
    public static final Icon ICON_FLOAT = JarResources.getIcon("Float.gif");
    public static final Icon ICON_STRING = JarResources.getIcon("String.gif");
    public static final Icon ICON_DATE = JarResources.getIcon("Date.gif");
    public static final Icon ICON_BOOLEAN = JarResources.getIcon("Boolean.gif");
    public static final Icon ICON_COPY16 = JarResources.getIcon("Copy16.gif");
    public static final Icon ICON_COPY24 = JarResources.getIcon("Copy24.gif");
    public static final Icon ICON_HELP24 = JarResources.getIcon("Help24.gif");
    //public static final Icon ICON_HELP16 = JarResources.getIcon("Help16.png");
    public static final Icon ICON_HELP16 = JarResources.getIcon("Help16_v2.gif");
    public static final Icon ICON_BACK16 = JarResources.getIcon("back16.png");

    public static final Icon ICON_TREE_CLOSED = JarResources.getIcon("TreeClosed.png");
    public static final Icon ICON_TREE_OPENED = JarResources.getIcon("TreeOpen.png");
    public static final Icon ICON_ELLIPSIS = JarResources.getIcon("Ellipsis.png");


    /**
     * thanks to : David Wales (david.wales@westgeo.com)
     * http://groups.google.com/groups?hl=en&lr=lang_en&ie=UTF-8&threadm=8q4vgu%24dhp%241%40news.nextra.cz&rnum=2&prev=/groups%3Fq%3DJOptionpane%2Bicon%2Bgroup:comp.lang.java.vdbgui%26hl%3Den%26lr%3Dlang_en%26ie%3DUTF-8%26selm%3D8q4vgu%2524dhp%25241%2540news.nextra.cz%26rnum%3D2
     */
    public static final Icon ICON_OPTIONPANE_ERROR = UIManager.getIcon("OptionPane.errorIcon");

    public static final Icon ICON_OPTIONPANE_ERROR16 = JarResources.getIcon("Error16.gif");

    public static final Icon ICON_OPTIONPANE_INFO =
            UIManager.getIcon("OptionPane.informationIcon");

    public static final Icon ICON_OPTIONPANE_INFO16 =
            JarResources.getIcon("Inform16.gif");

    public static final Icon ICON_OPTIONPANE_WARNING = UIManager.getIcon("OptionPane.warningIcon");
    public static final Icon ICON_OPTIONPANE_QUESTION =
            UIManager.getIcon("OptionPane.questionIcon");
    public static final Icon ICON_SAVE16 = JarResources.getIcon("Save16.gif");
    public static final Icon ICON_SAVE24 = JarResources.getIcon("Save24.gif");
    public static final Icon ICON_RESTORE16 = JarResources.getIcon("RestoreDefaults16.gif");
    public static final Icon ICON_RESTORE24 = JarResources.getIcon("Undo24.gif");
    public static final Icon ICON_DELETE16 = JarResources.getIcon("Delete16.gif");
    public static final Icon ICON_DELETE24 = JarResources.getIcon("Delete24.gif");

    // commonly used wondow size
    public static final Dimension DIMENSION_STANDARD_WINDOW = new Dimension(500, 500);
    public static final Dimension DIMENSION_BIGGER_DIALOG = new Dimension(500, 400);
    public static final Dimension DIMENSION_BIGGER_BIGGER_DIALOG = new Dimension(500, 500);
    public static final Dimension DIMENSION_BIGGER_BIGGER_BIGGER_DIALOG = new Dimension(600, 600);
    public static final Dimension DIMENSION_STANDARD_FILE_CHOOSER = new Dimension(400, 350);
    public static final JComponent[] EMPTY_CONTROLS = new JComponent[0];
    public static final Font FONT_DEFAULT_BOLD = new Font("Helvetica",
            Font.BOLD, 12);
    public static final Font FONT_DEFAULT = new Font("Helvetica",
            Font.PLAIN, 12);
    public static final Font FONT_FIXED_TEXT = new Font("Courier",
            Font.PLAIN, 12);

    public static final Font FONT_FIXED_TEXT_BOLD = new Font("Courier",
            Font.BOLD, 12);

    public static final Font FONT_SMALL_FIXED_TEXT = new Font("Courier",
            Font.PLAIN, 8);
    public static final Color COLOR_LIGHT_GREEN = new Color(153, 247, 121);
    public static final Color COLOR_LIGHT_ORANGE = new Color(255, 172, 89);
    public static final Color COLOR_LIGHT_YELLOW = Color.decode("#FFFF99");
    public static final Color COLOR_DARK_GREEN = new Color(0, 81, 0);
    public static final Color COLOR_LIGHT_RED = new Color(255, 108, 108);

    public static Color COLOR_LIGHT_BLUE = new Color(150, 150, 255);

    public static final Color COLOR_DARK_BROWN = new Color(128, 64, 64);

    public static final Color COLOR_VERY_LIGHT_GRAY = new Color(239, 239, 239);

    public static final Color COLOR_DARK_BLUE = new Color(63, 64, 124);
    //public static final Color         lgray       = new Color(105, 120, 141);
    public static final GradientPaint COLOR_GRAY_BLUE_VERTICAL = new GradientPaint(0, 0, COLOR_DARK_BLUE,
            0, 50, new Color(105, 120, 141));

    public static final GradientPaint COLOR_GRAY_BLUE_HORIZONTAL = new GradientPaint(0, 0, COLOR_DARK_BLUE,
            300, 0, new Color(105, 120, 141));

    public static final GradientPaint COLOR_LIGHT_BLUE_TO_BLUE_HORIZONTAL = new GradientPaint(0, 0, new Color(150, 150, 255),
            300, 0, Color.BLUE);

    public static final GradientPaint COLOR_RED_TO_BLUE_HORIZONTAL = new GradientPaint(0, 0, Color.RED,
            300, 0, Color.BLUE);

    public static final GradientPaint COLOR_LIGHT_RED_TO_RED_HORIZONTAL = new GradientPaint(0, 0, COLOR_LIGHT_RED,
            300, 0, Color.RED);

    public static final GradientPaint COLOR_GRAY_BROWN_HORIZONTAL = new GradientPaint(0, 0, new Color(128, 64, 64),
            300, 0, new Color(105, 120, 141));

    // -- private statics --
    private static final Logger klog = XLogger.getLogger(GuiHelper.class);
    private static final Dimension kPlaceholderSize = new Dimension(200, 50);
    private static final String DND_TOOL_TIP_TEXT = "Drag and drop objects here to change view";

    //private static GBase kBase = new GBase();

    /**
     * Privatized class constructor.
     */
    private GuiHelper() {
    }


    /**
     * Defines the margin used in toolbar buttons.
     */
    private static final Insets TOOLBAR_BUTTON_MARGIN = new Insets(1, 1, 1, 1);

    /**
     * Creates and answers a <code>JScrollPane</code> that has an empty
     * border.
     */
    public static JScrollPane createStrippedScrollPane(Component component) {
        JScrollPane scrollPane = new JScrollPane(component);
        scrollPane.setBorder(BorderFactory.createEmptyBorder());
        return scrollPane;
    }

    /**
     * Creates and returns a <code>JSplitPane</code> that has empty borders.
     * Useful to avoid duplicate decorations, for example if the split pane
     * is contained by other components that already provide a border.
     *
     * @param orientation  the split pane's orientation: horizontal or vertical
     * @param comp1        the top/left component
     * @param comp2        the bottom/right component
     * @param resizeWeight indicates how to distribute extra space
     * @return a split panes that has an empty border
     */
    public static JSplitPane createStrippedSplitPane(int orientation,
                                                     Component comp1, Component comp2, double resizeWeight) {
        JSplitPane split = UIFSplitPane.createStrippedSplitPane(orientation, comp1, comp2);
        split.setResizeWeight(resizeWeight);
        return split;
    }

    /**
     * Creates and answers an <code>AbstractButton</code>
     * configured for use in a JToolBar.<p>
     * <p/>
     * Superceded by ToolBarButton from the JGoodies UI framework.
     */
    public static AbstractButton createToolBarButton(Action action) {
        JButton button = new JButton(action);
        button.setFocusPainted(false);
        button.setMargin(TOOLBAR_BUTTON_MARGIN);
        //button.setHorizontalTextPosition(SwingConstants.CENTER);
        //button.setVerticalTextPosition(SwingConstants.BOTTOM);
        button.setText("");
        return button;
    }

    public static TitledBorder createTitledBorderForComponent(String title) {

        Border b = BorderFactory.createLineBorder(Color.BLACK);
        return BorderFactory.createTitledBorder(b, title, TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.CENTER,
                FONT_DEFAULT, Color.GRAY);


    }

    /**
     * call after setting frames size
     *
     * @param comp
     */
    public static void centerComponent(Component comp) {

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension size = comp.getSize();

        comp.setLocation((screenSize.width - size.width) / 2,
                (screenSize.height - size.height) / 2);
    }

    /**
     * Fills specified panel with a JPlaceholder
     * To further use, JPanel must be removedAll'ed.
     */
    public static void buffer(final JPanel panel) {
        fill(panel, createNaPlaceholder());
    }

    public static void buffer(final JComponent bufferedcomp) {
        fill(bufferedcomp, createNaPlaceholder());
    }

    public static void buffer(final JScrollPane sp) {
        sp.getViewport().add(createNaPlaceholder(), null);
    }

    /**
     * Filss panel with an an icon rep disabled
     */
    public static void disable(final JPanel panel) {
        fill(panel, createDisabledPlaceholder());
    }

    public static void fill(final JComponent filledcomp, final JComponent filler) {

        filledcomp.removeAll();

        if (filledcomp instanceof JScrollPane) {
            ((JScrollPane) filledcomp).getViewport().add(filler, null);
        } else {
            filledcomp.setLayout(new GridLayout(1, 1));
            filledcomp.add(filler);
        }

        filledcomp.revalidate();
    }

    /**
     * Fills the specified panle with comp as its ONLY component.
     * Basically a simple GridLayout of 1,1
     * Any preexisting components in panel are removed.
     */

    /*
    public static void fill(JPanel panel, Component comp) {
        panel.removeAll();
        panel.setLayout(new GridLayout(1,1));
        panel.add(comp);
        panel.revalidate();
    }
    */
    public static JPanel createPlaceholderPanel(final Dimension prefsize, final String text) {
        return _createPanel(prefsize, text, null);
    }

    public static JPanel createPlaceholderPanel(final Dimension prefsize, final TextIconPair pair) {
        return _createPanel(prefsize, pair.text, pair.icon);
    }

    public static JPanel createPlaceholderPanel(final Dimension prefsize, final String text,
                                                final Icon icon) {
        return _createPanel(prefsize, text, icon);
    }

    public static JPanel createNaPlaceholder() {
        return createPlaceholderPanel(kPlaceholderSize, TextIconPair.NA_COMPONENT);
    }

    public static JPanel createDisabledPlaceholder() {
        return createPlaceholderPanel(kPlaceholderSize, TextIconPair.DISABLED_COMPONENT);
    }

    public static Component createEmptyFiller() {
        return Box.createRigidArea(kPlaceholderSize);
    }

    public static JPanel createWaitingPlaceholder() {
        return createPlaceholderPanel(kPlaceholderSize, TextIconPair.WAITING_FOR_TASK);
    }

    public static JPanel createErrorPanel() {
        return createPlaceholderPanel(kPlaceholderSize, TextIconPair.ERROR_COMPONENT);
    }

    private static JPanel _createPanel(final Dimension prefsize, final String text,
                                       final Icon icon) {

        JPanel panel = new JPanel();

        panel.setPreferredSize(prefsize);

        JLabel label = new JLabel(text);

        label.setSize(prefsize);
        label.setIcon(icon);

        //label.setIconTextGap(label.getIconTextGap() + 20);
        //label.setAlignmentX(JLabel.CENTER_ALIGNMENT);
        panel.setLayout(new EqualFlowLayout(EqualFlowLayout.CENTER));
        panel.add(label);

        //this.setLayout(new BorderLayout());
        //this.add(label, BorderLayout.CENTER);
        panel.setBackground(Color.white);

        return panel;
    }

    /**
     * a panel with an iocn indicating that its dndable
     */
    public static JComponent createDndComponent(final Icon prsnicon) {
        return createDndComponent(new Icon[]{prsnicon});
    }

    /**
     * a panel with indicating that its is dndable with many dnd falvors rep by theior icons.
     */
    public static JComponent createDndComponent(final Icon[] prsnicons) {

        JPanel dndPanel = new JPanel();
        JLabel label1 = new JLabel(JarResources.getIcon("Dnd.gif"));

        label1.setToolTipText(DND_TOOL_TIP_TEXT);
        dndPanel.add(label1);

        for (int i = 0; i < prsnicons.length; i++) {
            JLabel label2 = new JLabel(prsnicons[i]);

            label2.setToolTipText(DND_TOOL_TIP_TEXT);
            dndPanel.add(label2);
        }

        //dndPanel.setBackground(Color.white);
        return dndPanel;
    }

    // from a classna pair cb
    public static Class getSelectedClass(final JComboBox cnpBox) {

        if (cnpBox.getSelectedIndex() != -1) {
            return ((ClassNamePair) cnpBox.getSelectedItem()).getPairClass();
        } else if ((cnpBox.getSelectedIndex() == -1) && (cnpBox.getModel().getSize() > 0)) {
            return ((ClassNamePair) cnpBox.getModel().getElementAt(0)).getPairClass();
        } else {
            return null;
        }
    }

    // from a cnp jlist
    public static Class getSelectedClass(final JList cnpList) {

        if (cnpList.getSelectedIndex() != -1) {
            return ((ClassNamePair) cnpList.getSelectedValue()).getPairClass();
        } else if ((cnpList.getSelectedIndex() == -1) && (cnpList.getModel().getSize() > 0)) {
            return ((ClassNamePair) cnpList.getModel().getElementAt(0)).getPairClass();
        } else {
            return null;
        }
    }

    public static Class[] getSelectedClasses(final JList cnpList) {

        if (cnpList.getSelectedIndex() != -1) {
            Object[] pairs = cnpList.getSelectedValues();
            Class[] classes = new Class[pairs.length];

            for (int i = 0; i < pairs.length; i++) {
                classes[i] = ((ClassNamePair) pairs[i]).getPairClass();
            }

            return classes;
        } else if ((cnpList.getSelectedIndex() == -1) && (cnpList.getModel().getSize() > 0)) {
            Class[] classes = new Class[1];

            classes[0] = ((ClassNamePair) cnpList.getModel().getElementAt(0)).getPairClass();

            return classes;
        } else {
            return null;
        }
    }

    // from a classname pair cb
    public static String getSelectedClassName(final JComboBox cnpBox) {

        if (cnpBox.getSelectedIndex() != -1) {
            return ((ClassNamePair) cnpBox.getSelectedItem()).getPairName();
        } else if ((cnpBox.getSelectedIndex() == -1) && (cnpBox.getModel().getSize() > 0)) {
            return ((ClassNamePair) cnpBox.getModel().getElementAt(0)).getPairName();
        } else {
            return null;
        }
    }

    // from a classname pair cb
    public static String getSelectedClassName(final JList jl) {

        if (jl.getSelectedIndex() != -1) {
            return ((ClassNamePair) jl.getSelectedValue()).getPairName();
        } else if ((jl.getSelectedIndex() == -1) && (jl.getModel().getSize() > 0)) {
            return ((ClassNamePair) jl.getModel().getElementAt(0)).getPairName();
        } else {
            return null;
        }
    }

    /**
     * does nothing if:
     * 1) model is empty
     * 2) if something is already selected
     */
    public static void safeSelect(final JList jl) {

        if (jl.getSelectedIndex() != -1) {
            return;
        }

        if (jl.getModel().getSize() > 0) {
            jl.setSelectedIndex(0);
        }
    }

    public static void safeSelect(final JComboBox cb) {

        if (cb.getSelectedIndex() != -1) {
            return;
        }

        if (cb.getModel().getSize() > 0) {
            cb.setSelectedIndex(0);
        }
    }

    public static void bind(final JComboBox box, final ClassNamePair pairs[]) {
        box.setModel(new DefaultComboBoxModel(pairs));
        box.setRenderer(new RendererFactory.ClassNamePairListCellRenderer());
    }

    public static void bind(final JComboBox box, final Object objs[]) {
        box.setModel(new DefaultComboBoxModel(objs));
    }

    public static void bind(final JList jl, final ClassNamePair pairs[]) {

        DefaultComboBoxModel model = new DefaultComboBoxModel(pairs);

        jl.setModel(model);

        if (model.getSize() > 0) {
            jl.setSelectedIndex(0);
        }

        jl.setCellRenderer(new RendererFactory.ClassNamePairListCellRenderer());
    }

    public static void bind(final JList jl, final Object objs[]) {

        DefaultComboBoxModel model = new DefaultComboBoxModel(objs);

        jl.setModel(model);

        if (model.getSize() > 0) {
            jl.setSelectedIndex(0);
        }
    }

    public static DefaultComboBoxModel bind(final JComboBox cb, final String[] values) {

        DefaultComboBoxModel model = new DefaultComboBoxModel(values);

        cb.setModel(model);

        if (model.getSize() > 0) {
            cb.setSelectedIndex(0);
        }

        return model;
    }

    public static DefaultComboBoxModel bind(final JList jl, final String[] values) {

        DefaultComboBoxModel model = new DefaultComboBoxModel(values);

        jl.setModel(model);

        if (model.getSize() > 0) {
            jl.setSelectedIndex(0);
        }

        return model;
    }

    /**
     * List must have Class objects.
     */
    public static void bind(final JComboBox box, final java.util.List classes, final ListCellRenderer rend)
            throws Exception {

        DefaultComboBoxModel model = new DefaultComboBoxModel();

        for (int i = 0; i < classes.size(); i++) {
            Object obj = (((Class) classes.get(i)).newInstance());

            model.addElement(obj);
        }

        box.setModel(model);
        box.setRenderer(rend);
    }

    public static void bind(final JList jl, final java.util.List classes, final ListCellRenderer rend)
            throws Exception {

        DefaultComboBoxModel model = new DefaultComboBoxModel();

        for (int i = 0; i < classes.size(); i++) {
            Object obj = (((Class) classes.get(i)).newInstance());

            model.addElement(obj);
        }

        jl.setModel(model);
        jl.setCellRenderer(rend);
    }

    /**
     * Use br to wrap info when its long
     * IMP: truncate if too long - else rendering perf suffers
     */
    private static final int MAX_LENGTH = 200;

    public static String formatObjectToolTipText(String name, Id id, String info) {

        StringBuffer buf = new StringBuffer("<html>Name: ").append(name).append("<br>");

        buf.append("Id: ").append(id.id()).append("<br>");

        if (info == null) {
            info = "no summary available";
        }

        if (info.length() > MAX_LENGTH) {
            info = info.substring(0, MAX_LENGTH);
        }

        //info = info.replace("\n", "<br>");
        int step = 80;

        for (int i = 0; i < info.length(); i += step) {
            int last = i + step;

            if (last >= info.length()) {
                last = info.length() - 1;
            }

            buf.append(info.substring(i, last)).append("<br>");
        }

        //buf.append(info);
        buf.append("</html>");

        return buf.toString();
    }

    /**
     * JButton related helper methods
     */
    public static class Button {

        public static JPanel createButtonPanel(JButton[] buttons) {

            JPanel panel = new JPanel(new EqualFlowLayout(EqualFlowLayout.RIGHT));

            for (int i = 0; i < buttons.length; i++) {
                panel.add(buttons[i]);
            }

            return panel;
        }

        public static JPanel createCentralButtonPanel(JButton[] buttons) {

            JPanel panel = new JPanel();

            for (int i = 0; i < buttons.length; i++) {
                panel.add(buttons[i]);
            }

            return panel;
        }

        public static JPanel createVerticalButtonPanel(JButton[] buttons) {

            JPanel panel = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.MIDDLE));

            for (int i = 0; i < buttons.length; i++) {
                panel.add(buttons[i]);
            }

            return panel;
        }

        /**
         * centerpanel in center and buttons in another panel in south
         */
        public static JPanel createButtonedPanel(JComponent centercomp, JButton[] buttons) {

            JPanel panel = new JPanel(new BorderLayout());

            panel.add(centercomp, BorderLayout.CENTER);
            panel.add(createButtonPanel(buttons), BorderLayout.SOUTH);

            return panel;
        }

        public static JButton createRefreshButton() {
            JButton but = new JButton(ICON_REFRESH16);
            but.setToolTipText("Refresh the contents");
            return but;
        }

        /**
         * creates a standrad looking button with icon onlu (no text)
         */
        public static JButton createIconOnlyButton(final Icon icon) {

            JButton but = new JButton(icon);

            but.setContentAreaFilled(false);
            but.setBorderPainted(false);

            return but;
        }

        /**
         * Button with standard "start" icon and specified text label
         *
         * @param text
         * @return
         */
        public static JButton createStartButton(final String text) {

            JButton but = new JButton(text, ICON_START16);

            but.setContentAreaFilled(false);
            but.setBorderPainted(true);

            return but;
        }

        public static JButton createHelpButton(final String text) {

            JButton but = new JButton(text, ICON_HELP16);
            but.setToolTipText("Help for this action");

            but.setContentAreaFilled(false);
            but.setBorderPainted(true);

            return but;
        }

        public static JButton createBackButton() {

            JButton but = new JButton(ICON_BACK16);

            but.setContentAreaFilled(false);
            but.setBorderPainted(true);

            return but;
        }

        // IMP seems buggy with some LnF's (the border doesnt get UNpainted)
        // works ok if the button is used in a JToolBar instead of in a JPanel
        public static void addMouseOverRollOverAction(final AbstractButton but) {
            addMouseOverRollOverAction(but, false);
        }

        public static void addMouseOverRollOverAction(final AbstractButton but, final boolean onlyBorders) {
            //but.setBorder(BorderFactory.createRaisedBevelBorder());
            final Border bevel = new BevelBorder(BevelBorder.RAISED, GuiHelper.COLOR_DARK_BLUE, GuiHelper.COLOR_DARK_BLUE);

            //but.setBorder(BorderFactory.createLineBorder(Color.BLUE));
            but.setBorder(bevel);
            but.setBorderPainted(false);
            but.setRolloverEnabled(false); // needed for alloy lnf to work properly ??

            final Color bgColor = but.getBackground();

            but.addMouseListener(new MouseAdapter() {
                public void mouseEntered(MouseEvent e) {
                    but.setBorderPainted(true);

                    if (!onlyBorders) {
                        if (but.isSelected()) {
                            but.setBackground(GuiHelper.COLOR_DARK_BLUE);
                        } else {
                            but.setBackground(bgColor);
                        }
                    }
                }

                public void mouseExited(MouseEvent e) {
                    but.setBorderPainted(false);
                }
            });
        }

    }    // End GuiHelper.Button

    /**
     * Table related helper methods
     */
    public static class Table {

        public static DefaultTableModel createPlaceholderTableModel() {
            return new DefaultTableModel(2, 2);
        }

        public static void makeColumnSelectable(final JTable table) {

            JTableHeader header = table.getTableHeader(); // @note adding mouse listener to the header

            header.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    //System.out.println(">>> " + e);

                    JTableHeader header = table.getTableHeader();

                    TableColumnModel columns = header.getColumnModel();

                    if (!columns.getColumnSelectionAllowed())
                        return;

                    int column = header.columnAtPoint(e.getPoint());

                    if (column == -1)
                        return;

                    int count = table.getRowCount();

                    if (count != 0)
                        table.setRowSelectionInterval(0, count - 1);

                    ListSelectionModel selection = columns.getSelectionModel();

                    if (e.isShiftDown()) {
                        int anchor = selection.getAnchorSelectionIndex();
                        int lead = selection.getLeadSelectionIndex();

                        if (anchor != -1) {
                            boolean old = selection.getValueIsAdjusting();
                            selection.setValueIsAdjusting(true);

                            boolean anchorSelected = selection.isSelectedIndex(anchor);

                            if (lead != -1) {
                                if (anchorSelected)
                                    selection.removeSelectionInterval(anchor, lead);
                                else
                                    selection.addSelectionInterval(anchor, lead);
                                // The latter is quite unintuitive.
                            }

                            if (anchorSelected)
                                selection.addSelectionInterval(anchor, column);
                            else
                                selection.removeSelectionInterval(anchor, column);

                            selection.setValueIsAdjusting(old);
                        } else
                            selection.setSelectionInterval(column, column);
                    } else if (e.isControlDown()) {
                        if (selection.isSelectedIndex(column))
                            selection.removeSelectionInterval(column, column);
                        else
                            selection.addSelectionInterval(column, column);
                    } else {
                        selection.setSelectionInterval(column, column);
                    }

                }
            });

        }

        public static DefaultTableModel createPlaceholderTableModel(final int nrows,
                                                                    final int ncols) {
            return new DefaultTableModel(nrows, ncols);
        }

        public static String getRowAsString(int row, TableModel model) {

            StringBuffer buf = new StringBuffer();

            for (int i = 0; i < model.getColumnCount(); i++) {
                buf.append(model.getColumnName(i)).append(": ").append(model.getValueAt(row,
                        i)).append('\n');
            }

            return buf.toString();
        }

        /**
         * code replicated so that the row-col format is preserved
         *
         * @retrun null if there are no selections or if the block is not contigous and only
         * contigous selection is allowed (after showing a joptiona pane message)
         */
        public static String selectionAsString(JTable table, boolean allow_non_contigous) {

            StringBuffer sbf = new StringBuffer();
            int numcols = table.getSelectedColumnCount();
            int numrows = table.getSelectedRowCount();

            if ((numcols == 0) && (numrows == 0)) {
                klog.info("No cols or rows to copy");

                return null;
            }

            int[] rowsselected = table.getSelectedRows();
            int[] colsselected = table.getSelectedColumns();

            // Check to ensure we have selected only a contiguous block of cells
            if (allow_non_contigous == false) {
                if (!(((numrows - 1 == rowsselected[rowsselected.length - 1]
                        - rowsselected[0]) && (numrows == rowsselected.length)) && ((numcols
                        - 1 == colsselected[colsselected.length - 1]
                        - colsselected[0]) && (numcols == colsselected
                        .length)))) {
                    Application.getWindowManager().showError("Invalid Copy Selection\n Must be continuous!");

                    return null;
                } else {
                    for (int i = 0; i < numrows; i++) {
                        for (int j = 0; j < numcols; j++) {
                            sbf.append(table.getValueAt(rowsselected[i], colsselected[j]));

                            if (j < numcols - 1) {
                                sbf.append("\t");
                            }
                        }

                        sbf.append("\n");
                    }

                    return sbf.toString();
                }
            } else {
                for (int r = 0; r < rowsselected.length; r++) {
                    for (int c = 0; c < colsselected.length; c++) {
                        sbf.append(table.getValueAt(rowsselected[r], colsselected[c]));

                        if (c < numcols - 1) {
                            sbf.append("\t");
                        }
                    }

                    sbf.append("\n");
                }

                return sbf.toString();
            }
        }    // End selectionAsString

        public static void setColumnSize(int size, int col, JTable table, boolean alsoMax) {
            // the order of set calls is apparently important
            TableColumn column = table.getColumnModel().getColumn(col);

            column.setMinWidth(0);

            if (alsoMax) {
                column.setMaxWidth(size);
            }
            column.setPreferredWidth(size);
        }

        /**
         * From:
         * http://www.chka.de/swing/table/cell-sizes.html
         * If scanallcells, then all cells are scanned an the table set to max size.
         * Else only the header cells are scanned (if they exist)
         * <p/>
         * ncells2scan -> only the firrst nrowspercol2scan in each columns are scanned for size
         */
        public static void sizeCells2FitByScanning(JTable table, int nrowspercol2scan) {

            JTableHeader header = table.getTableHeader();
            TableCellRenderer defaultHeaderRenderer = null;

            if (header != null) {
                defaultHeaderRenderer = header.getDefaultRenderer();
            }

            TableColumnModel columns = table.getColumnModel();
            TableModel data = table.getModel();
            int margin = columns.getColumnMargin();    // only JDK1.3
            int rowCount = data.getRowCount();

            //int totalWidth = 0;
            for (int i = columns.getColumnCount() - 1; i >= 0; --i) {
                javax.swing.table.TableColumn column = columns.getColumn(i);
                int columnIndex = column.getModelIndex();
                int width = -1;
                TableCellRenderer h = column.getHeaderRenderer();

                if (h == null) {
                    h = defaultHeaderRenderer;
                }

                if (h != null) {                                  // Not explicitly impossible
                    Component c = h.getTableCellRendererComponent(table, column.getHeaderValue(),
                            false, false, -1, i);

                    width = c.getPreferredSize().width;
                }

                int cnt = 0;

                for (int row = rowCount - 1; row >= 0; --row) {
                    TableCellRenderer r = table.getCellRenderer(row, i);
                    Component c = r.getTableCellRendererComponent(table,
                            data.getValueAt(row, columnIndex), false, false, row,
                            i);

                    width = Math.max(width, c.getPreferredSize().width);

                    cnt++;

                    if (cnt >= nrowspercol2scan) {
                        break;
                    }
                }

                if (width >= 0) {
                    column.setPreferredWidth(width + margin);    // <1.3: without margin
                } else {
                    ;                                            // ???
                }

                //totalWidth += column.getPreferredWidth();
            }

            /* If you like; This does not make sense for two many columns!
            Dimension size = table.getPreferredScrollableViewportSize();
            size.width = totalWidth;
            table.setPreferredScrollableViewportSize(size);
            */

            // table.sizeColumnsToFit(-1); <1.3; possibly even table.revalidate()
        }

        /**
         * whole table scanned
         */
        public static void sizeCells2FitByScanning(JTable table) {

            sizeCells2FitByScanning(table,
                    table.getModel().getRowCount()
                            * table.getModel().getColumnCount());
        }

        /**
         * size table columns to fit just based on the column header
         * (unlike the sizeCells2Fit methd, here only the columns are scanned.
         */
        public static void sizeColumnHeaders2Fit(JTable table) {

            JTableHeader header = table.getTableHeader();
            TableCellRenderer defaultHeaderRenderer = null;

            if (header != null) {
                defaultHeaderRenderer = header.getDefaultRenderer();
            }

            TableColumnModel columns = table.getColumnModel();
            int margin = columns.getColumnMargin();    // only JDK1.3

            for (int i = columns.getColumnCount() - 1; i >= 0; --i) {
                javax.swing.table.TableColumn column = columns.getColumn(i);
                int width = -1;
                TableCellRenderer h = column.getHeaderRenderer();

                if (h == null) {
                    h = defaultHeaderRenderer;
                }

                if (h != null) {                                  // Not explicitly impossible
                    Component c = h.getTableCellRendererComponent(table, column.getHeaderValue(),
                            false, false, -1, i);

                    width = c.getPreferredSize().width;
                }

                if (width >= 0) {
                    column.setPreferredWidth(width + margin);    // <1.3: without margin
                } else {
                    ;                                            // ???
                }
            }
        }

        //----------------------------------------------------------------------
        // Auto size table column
        //----------------------------------------------------------------------
        // {{{ autoSizeTableColumn

        /**
         * Resize a table column so its fits exactly the broadest of its visible
         * cells. This will set the columns width as well as its preferred width.
         * Only works if the table's parent is a JViewport, i.e. the table is
         * nested in a JScrollPane.
         * <p/>
         * * The Original Code is qfs.de code.
         * <p/>
         * The Initial Developer of the Original Code is Gregor Schmid.
         * Portions created by Gregor Schmid are
         * Copyright (C) 1999 Quality First Software, Gregor Schmid.
         *
         * @param table  The table that contains the column.
         * @param column The colunm index in table coordinates.
         */
        public static void sizeTableToColumnByScanning(JTable table, int column) {

            if ((table.getRowCount() == 0) || !(table.getParent() instanceof JViewport)) {
                return;
            }

            JViewport vp = (JViewport) table.getParent();
            Rectangle rect = vp.getViewRect();
            int start = table.rowAtPoint(new Point(0, rect.y));
            int end = table.rowAtPoint(new Point(0, rect.y + rect.height));

            if (end < 0) {
                end = table.getRowCount() - 1;
            }

            int size = 0;

            for (int i = start; i <= end; i++) {
                Component com = table.prepareRenderer(table.getCellRenderer(i, column), i, column);

                if (com instanceof JComponent) {
                    size = Math.max(size, com.getPreferredSize().width);
                } else {
                    size = Math.max(size, com.getSize().width);
                }
            }

            int ccol = table.convertColumnIndexToModel(column);

            table.getColumnModel().getColumn(ccol).setPreferredWidth(size);
            table.getColumnModel().getColumn(ccol).setWidth(size);
            table.revalidate();
        }

        /**
         * Calculate the required width of specified table column
         *
         * @author Core Swing: Advanced Programming by Kim Topley AdvancedSwing.Chapter6;
         */
        public static int scanMaxColumnWidth(JTable table, int columnIndex) {

            int width = 0;    // The return value
            int rowCount = table.getRowCount();

            for (int i = 0; i < rowCount; i++) {
                TableCellRenderer renderer = table.getCellRenderer(i, columnIndex);
                Component comp = renderer.getTableCellRendererComponent(table,
                        table.getValueAt(i, columnIndex), false, false, i,
                        columnIndex);
                int thisWidth = comp.getPreferredSize().width;

                if (thisWidth > width) {
                    width = thisWidth;
                }
            }

            return width;
        }
    }    // End GuiHelper.Table


    public static class List2 {

        public static void setSelected(final Object[] selected_vals, final JList jlist, final DefaultListModel listModel) {

            //Print.outl(listModel);
            if (selected_vals == null) {
                klog.error("Null arg for selected selected_vals");
                return;
            }

            TIntArrayList indices = new TIntArrayList();

            for (int i = 0; i < selected_vals.length; i++) {

                int index = listModel.indexOf(selected_vals[i]);
                if (index != -1) {
                    indices.add(index);
                }
                //log.debug("Checking for: " + selected_vals[i] + " class: " + selected_vals[i].getClass() + " found index: " + index);
            }

            jlist.setSelectedIndices(indices.toNativeArray());
            //log.debug(" >>> " + jlist.getSelectionMode() + " " + ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            //log.debug(">>> # selected: " + jlist.getSelectedIndices().length);
        }


    } // End class List

    /**
     * Windowing related helper methods.
     */
    public static class Window {

        /**
         * Centers an AWT {@link Window} on the screen.
         *
         * @param w the window to center
         * @author Alexander Hvostov <root@aoi.dyndns.org>
         * @see Toolkit#getScreenSize()
         * @see Toolkit#getDefaultToolkit()
         * @see Window#getSize()
         * @see Window
         * @since org.dyndns.aoi.Utilities 1.0
         */
        public static void centerWindowOnScreen(final java.awt.Window w) {

            // Determine the screen size.
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

            // Determine the window size.
            Dimension windowSize = w.getSize();

            // Center the window.
            w.setLocation((screenSize.width / 2) - (windowSize.width / 2),
                    (screenSize.height / 2) - (windowSize.height / 2));
        }

        /**
         * Add a KeyboardListener to a Window that posts a WINDOW_CLOSING event
         * on it when Escape is pressed. Useful for modal dialogs.
         * <p/>
         * The Original Code is qfs.de code.
         * <p/>
         * The Initial Developer of the Original Code is Gregor Schmid.
         * Portions created by Gregor Schmid are
         * Copyright (C) 1999 Quality First Software, Gregor Schmid.
         *
         * @param win
         */
        public static void closeOnEscape(final java.awt.Window win) {

            win.addKeyListener(new KeyAdapter() {

                public void keyPressed(KeyEvent e) {

                    if ((e.getKeyCode() == KeyEvent.VK_ESCAPE) && (e.getModifiers() == 0)) {
                        Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(new WindowEvent(win, WindowEvent.WINDOW_CLOSING));
                        e.consume();
                    }
                }
            });
        }
    }    // End GuiHelper.Window

    /**
     * JTree related helper methods
     */
    public static class Tree {

        //----------------------------------------------------------------------
        // Tree expand/collapse all
        //----------------------------------------------------------------------
        // {{{ expandAll

        /**
         * Expand a tree node and all its child nodes recursively.
         * <p/>
         * The Original Code is qfs.de code.
         * <p/>
         * The Initial Developer of the Original Code is Gregor Schmid.
         * Portions created by Gregor Schmid are
         * Copyright (C) 1999 Quality First Software, Gregor Schmid.
         *
         * @param tree The tree whose nodes to expand.
         * @param path Path to the node to start at.
         */
        public static void expandAll(JTree tree, TreePath path) {

            Object node = path.getLastPathComponent();
            TreeModel model = tree.getModel();

            if (model.isLeaf(node)) {
                return;
            }

            tree.expandPath(path);

            int num = model.getChildCount(node);

            for (int i = 0; i < num; i++) {
                expandAll(tree, path.pathByAddingChild(model.getChild(node, i)));
            }
        }

        /**
         * Collapse a tree node and all its child nodes recursively.
         * <p/>
         * The Original Code is qfs.de code.
         * <p/>
         * The Initial Developer of the Original Code is Gregor Schmid.
         * Portions created by Gregor Schmid are
         * Copyright (C) 1999 Quality First Software, Gregor Schmid.
         *
         * @param tree The tree whose nodes to collapse.
         * @param path Path to the node to start at.
         */
        public static void collapseAll(JTree tree, TreePath path) {

            Object node = path.getLastPathComponent();
            TreeModel model = tree.getModel();

            if (model.isLeaf(node)) {
                return;
            }

            //  if (tree.isExpanded(path)
            //  || tree.hasBeenExpanded(path)) {
            int num = model.getChildCount(node);

            for (int i = 0; i < num; i++) {
                collapseAll(tree, path.pathByAddingChild(model.getChild(node, i)));
            }

            tree.collapsePath(path);
        }
    }    // End GuiHelper.Tree

    /**
     * event related stuff
     */
    public static class Event {

        /*
        * sends ACTION_PERFORMED event for double-click
        * and ENTER key
        */
        public static void enableDoubleClick(final JList jl, final ActionListener al) {

            jl.addMouseListener(new MouseAdapter() {

                public void mouseClicked(MouseEvent me) {

                    Object ob[] = jl.getSelectedValues();

                    if (ob.length > 1) {
                        return;
                    }

                    if (me.getClickCount() == 2) {
                        //log.debug("Sending dclick ACTION_PERFORMED to ActionListener");
                        al.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
                                ob[0].toString()));
                        me.consume();
                    }
                }
            });
            jl.addKeyListener(new KeyAdapter() {

                public void keyReleased(KeyEvent ke) {

                    Object ob[] = jl.getSelectedValues();

                    if (ob.length > 1) {
                        return;
                    }

                    if (ke.getKeyCode() == KeyEvent.VK_ENTER) {
                        klog.debug("Sending ACTION_PERFORMED to ActionListener");
                        al.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
                                ob[0].toString()));
                        ke.consume();
                    }
                }
            });
        }
    }    // End GuiHelper.Event
}        // End GuiHelper

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
