package edu.mit.broad.genome.swing;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.URL;

/**
 * An application used for viewing images in disk files.
 */
public class ImageViewerFrame extends JFrame {

    public static final String APPNAME = "ImageViewerFrame";
    private ImageComponent imageComp = null;

    /**
     * Class constructor
     *
     * @param image
     * @param scaled
     * @param title
     */
    public ImageViewerFrame(Image image, boolean scaled, String title) {

        JMenuBar mbar = new JMenuBar();
        JMenu menu = new JMenu("File");
        menu.add(new LoadImageAction());
        menu.addSeparator();
        menu.add(new ExitAction());
        mbar.add(menu);
        setJMenuBar(mbar);

        setTitle(title);

        imageComp = new ImageComponent(image, scaled);
        if (scaled) {
            getContentPane().add(imageComp);
        } else {
            getContentPane().add(new JScrollPane(imageComp));
        }
    }

    public ImageComponent getImageComponent() {
        return imageComp;
    }

    public static ImageComponent createImageComponent(final Image image, final boolean scaled) {
        return new ImageComponent(image, scaled);
    }

    // An action class for opening a new image
    protected class LoadImageAction extends AbstractAction {
        LoadImageAction() {
            putValue(NAME, "Load Image...");
            putValue(SHORT_DESCRIPTION, "Load an image file from the local discs.");
        }

        public void actionPerformed(ActionEvent e) {
            JFileChooser chooser = new JFileChooser();
            FileFilter filter = new ExtensionFileFilter(
                    new String[]{"gif", "GIF"}, "GIF image files");
            chooser.addChoosableFileFilter(filter);
            filter = new ExtensionFileFilter(
                    new String[]{"jpg", "JPG", "jpeg", "JPEG"}, "JPG image files");
            chooser.addChoosableFileFilter(filter);
            filter = new ExtensionFileFilter(
                    new String[]{"gif", "GIF", "jpg", "JPG", "jpeg", "JPEG"}, "GIF and JPG image files");
            chooser.addChoosableFileFilter(filter);
            chooser.removeChoosableFileFilter(
                    chooser.getAcceptAllFileFilter());
            int returnVal = chooser.showOpenDialog(ImageViewerFrame.this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                String path = chooser.getSelectedFile().getPath();
                System.out.println(path);
                Image image = Toolkit.getDefaultToolkit().getImage(path);
                if (image != null) {
                    imageComp.setImage(image);
                    setTitle(APPNAME + ": " + path);
                }
            }
        }
    }

    // file filter for opening files with a specified extension
    protected class ExtensionFileFilter extends FileFilter {
        ExtensionFileFilter(String[] extensions, String description) {
            this.extensions = extensions;
            this.description = description;
        }

        public boolean accept(File f) {
            if (f.isDirectory()) {
                return true;
            }

            String name = f.getName().toUpperCase();
            for (int i = 0; i < extensions.length; i++) {
                if (name.endsWith("." + extensions[i])) {
                    return true;
                }
            }
            return false;
        }

        public String getDescription() {
            return description;
        }

        private String[] extensions;
        private String description;
    }

    protected class ExitAction extends AbstractAction {
        ExitAction() {
            putValue(NAME, "Exit");
        }

        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }

    public static void main(String[] args) {
        String title = APPNAME;
        String ifilename = null;
        Image image = null;
        boolean scaled = false;
        try {
            for (int i = 0; i < args.length; i++) {
                if (args[i].equals("-scaled")) {
                    scaled = true;
                } else if (args[i].equals("-file")) {
                    ifilename = args[++i];
                    image = Toolkit.getDefaultToolkit().getImage(ifilename);
                } else if (args[i].equals("-url")) {
                    ifilename = args[++i];
                    image = Toolkit.getDefaultToolkit().getImage(new URL(ifilename));
                } else {
                    throw new Exception();
                }
            }
        }
        catch (Exception e) {
            e.printStackTrace();
            System.out.println(
                    "Parameters: [-scaled] [-file <file name>] [-url <URL]"
            );
            System.exit(0);
        }

        if (image != null) {
            title = title + ": " + ifilename;
        }

        JFrame f = new ImageViewerFrame(image, scaled, title);
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        f.pack();
        f.setVisible(true);
        //f.show();
    }

} // End class ImageViewerFrame

