/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing;

import javax.swing.*;
import java.io.*;


/**
 * Original code from: javax.swing.ProgressMonitorInputStream
 * Difference -> updates specified JProgressBar instead of a javax.swing.ProgressMonitor
 * <p/>
 * Monitors the progress of reading from some InputStream. This ProgressMonitor
 * is normally invoked in roughly this form:
 * <pre>
 * InputStream in = new BufferedInputStream(
 *                          new ProgressMonitorInputStream2(
 *                                  "Reading " + fileName,
 *                                  new FileInputStream(fileName)));
 * </pre><p>
 * This creates a progress monitor to monitor the progress of reading
 * the input stream.  If it's taking a while, a ProgressDialog will
 * be popped up to inform the user.  If the user hits the Cancel button
 * an InterruptedIOException will be thrown on the next read.
 * All the right cleanup is done when the stream is closed.
 * <p/>
 * <p/>
 * <p/>
 * <p/>
 * For further documentation and examples see
 * <a href="http://java.sun.com/docs/books/tutorial/uiswing/components/progress.html">How to Monitor Progress</a>,
 * a section in <em>The Java Tutorial.</em>
 *
 * @author James Gosling
 * @version 1.17 12/03/01
 * @see javax.swing.ProgressMonitor
 */
public class ProgressMonitorInputStream2 extends FilterInputStream {

    private int nread = 0;
    private int size = 0;

    private JProgressBar fBar;

    public ProgressMonitorInputStream2(File file) throws FileNotFoundException {
        this(new FileInputStream(file));
    }

    public ProgressMonitorInputStream2(File file, JProgressBar shared) throws FileNotFoundException {
        this(new FileInputStream(file), shared);
    }

    /**
     * Constructs an object to monitor the progress of an input stream.
     *
     * @param message Descriptive text to be placed in the dialog box
     *                if one is popped up.
     *                being monitored.
     * @param in      The input stream to be monitored.
     */
    public ProgressMonitorInputStream2(InputStream in) {
        super(in);
        try {
            size = in.available();
        } catch (IOException ioe) {
            size = 0;
        }

        try {
            size = in.available();
        } catch (IOException ioe) {
            size = 0;
        }

        this.fBar = new JProgressBar(0, size);

    }

    public ProgressMonitorInputStream2(InputStream in, JProgressBar shared) {
        super(in);
        try {
            size = in.available();
        } catch (IOException ioe) {
            size = 0;
        }

        try {
            size = in.available();
        } catch (IOException ioe) {
            size = 0;
        }

        this.fBar = shared;
        fBar.setMinimum(0);
        fBar.setMaximum(size);
        fBar.setValue(0);

    }

    public JProgressBar getProgressBar() {
        return fBar;
    }

    /**
     * Overrides <code>FilterInputStream.read</code>
     * to update the progress monitor after the read.
     */
    public int read() throws IOException {
        int c = in.read();
        if (c >= 0) fBar.setValue(++nread);
        return c;
    }


    /**
     * Overrides <code>FilterInputStream.read</code>
     * to update the progress monitor after the read.
     */
    public int read(byte b[]) throws IOException {
        int nr = in.read(b);
        if (nr > 0) fBar.setValue(nread += nr);
        return nr;
    }


    /**
     * Overrides <code>FilterInputStream.read</code>
     * to update the progress monitor after the read.
     */
    public int read(byte b[],
                    int off,
                    int len) throws IOException {
        int nr = in.read(b, off, len);
        if (nr > 0) fBar.setValue(nread += nr);
        return nr;
    }


    /**
     * Overrides <code>FilterInputStream.skip</code>
     * to update the progress monitor after the skip.
     */
    public long skip(long n) throws IOException {
        long nr = in.skip(n);
        if (nr > 0) fBar.setValue(nread += nr);
        return nr;
    }

    /**
     * Overrides <code>FilterInputStream.close</code>
     * to close the progress monitor as well as the stream.
     */
    public void close() throws IOException {
        in.close();
        fBar.setValue(fBar.getMaximum());
    }


    /**
     * Overrides <code>FilterInputStream.reset</code>
     * to reset the progress monitor as well as the stream.
     */
    public synchronized void reset() throws IOException {
        in.reset();
        nread = size - in.available();
        fBar.setValue(nread);
    }

}
