/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import edu.mit.broad.genome.XLogger;
import gnu.trove.TFloatArrayList;
import org.apache.log4j.Logger;

import java.util.*;


/**
 * Native array related utilities.
 * Functionally extends java.util.Arrays.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ArrayUtils {

    private static final Logger klog = XLogger.getLogger(ArrayUtils.class);

    private ArrayUtils() {
    }


    public static String[] subtract(final List all, final String[] removeThese) {
        List minus = new ArrayList(all);
        for (int i = 0; i < removeThese.length; i++) {
            minus.remove(removeThese[i]);
        }

        return (String[]) minus.toArray(new String[minus.size()]);
    }

    public static List toList(String[] ss) {
        List list = new ArrayList(ss.length);
        for (int i = 0; i < ss.length; i++) {
            list.add(ss[i]);
        }

        return list;
    }

    public static Set toSet(String[] ss) {
        Set set = new HashSet();
        for (int i = 0; i < ss.length; i++) {
            set.add(ss[i]);
        }

        return set;
    }

    /**
     * complement
     * feeds into ret array all indices from 0 to max -1
     * that are NOT in the specified exclude array.
     * <p/>
     * returned array will have length max - exclude.length
     * <p/>
     * all elements of exlude array must be >= 0 and <= max.
     *
     * @todo improve algorithm
     * Think binary search cant be used as the arrays need to be sorted (see javadoc)
     */
    public static int[] complement(int[] exclude, int max) {

        List list = new ArrayList(max);

        for (int i = 0; i < exclude.length; i++) {
            if (exclude[i] < 0) {
                throw new IllegalArgumentException("Exclude array has negative element value: "
                        + exclude[i] + " at: " + i);
            }

            if (exclude[i] >= max) {
                throw new IllegalArgumentException("Exclude array has element of value: "
                        + exclude[i] + " that is  >= max: " + max
                        + " at pos: " + i);
            }

            list.add(new Integer(exclude[i]));
        }

        int[] invrows = new int[max - exclude.length];
        int cnt = 0;

        for (int i = 0; i < max; i++) {
            if (list.contains(new Integer(i))) {
                ;
            } else {
                invrows[cnt++] = i;
            }
        }

        return invrows;
    }

    public static String[] removeElement(int index, String[] objs) {

        if (index >= objs.length) {
            throw new IllegalArgumentException("Index for removal: " + index
                    + " is greater than array len: " + objs.length);
        }

        String[] robjs = new String[objs.length - 1];
        int cnt = 0;

        for (int i = 0; i < objs.length; i++) {
            if (i == index) {
                ;
            } else {
                robjs[cnt++] = objs[i];
            }
        }

        return robjs;
    }

    /**
     * creates a new integer arry with elements as all consecutive integers between
     * min and max INclusive.
     * Length = max - min + 1
     */
    public static int[] rangeAsElements(final int min, final int max) {

        if (max < min) {
            throw new IllegalArgumentException("Max: " + max + " less than min: " + min);
        }

        int[] ret = new int[max - min + 1];
        int curr = min;

        for (int i = 0; i < (max - min + 1); i++) {
            ret[i] = curr++;
        }

        return ret;
    }

    /**
     * Inclusive of min and max.
     *
     * @param min
     * @param max
     * @return
     */
    public static java.util.Vector rangeAsUtilVector(final int min, final int max) {

        if (max < min) {
            throw new IllegalArgumentException("Max: " + max + " less than min: " + min);
        }

        java.util.Vector vector = new java.util.Vector(max - min + 1);
        int curr = min;

        for (int i = 0; i < (max - min + 1); i++) {
            vector.add(new Integer(curr++));
        }

        return vector;
    }

    /**
     * @param compmin
     * @param compmax
     * @param max
     * @return
     */
    public static int[] complementRangeAsElements(final int compmin, final int compmax,
                                                  final int max) {

        if (compmax < compmin) {
            throw new IllegalArgumentException("Max: " + compmax + " less than min: " + compmin);
        }

        int[] ret = new int[max - (compmax - compmin + 1)];
        int cnt = 0;

        for (int i = 0; i < max; i++) {
            if ((i >= compmin) && (i <= compmax)) {
                ;
            } else {
                ret[cnt++] = i;
            }
        }

        return ret;
    }

    /**
     * Specified array param is not modified in any way.
     * All duplicate (upto floats order of precision) elements are removed.
     *
     * @done improve algorithm
     * <br>-> Moved to using gnu.trove
     */

    // 377837 # of uniq elements: 376403
    // 377837 # of uniq elements: 30737
    // 377837 # of uniq elements: 30737
    public static float[] unique(final float[] values) {

        TFloatArrayList list = new TFloatArrayList();
        float[] copy = new float[values.length];

        System.arraycopy(values, 0, copy, 0, values.length);
        Arrays.sort(copy);

        float curr = copy[0];

        list.add(curr);

        for (int i = 1; i < values.length; i++) {
            if (copy[i] == curr) {
                ;
            } else {
                list.add(copy[i]);

                curr = copy[i];
            }
        }

        klog.debug("# of non-uniq elements: " + values.length + " # of uniq elements: "
                + list.size());

        return list.toNativeArray();
    }


    /**
     * element 0 to num-1 (forward)
     *
     * @param num
     * @param data
     * @return
     */
    public static int[] sub(int num, int[] data) {

        if (num > data.length) {
            num = data.length;
        }

        int[] ret = new int[num];

        System.arraycopy(data, 0, ret, 0, num);

        return ret;
    }


    /**
     * Two tailed subarray with the first nfromtop elements of the array
     * first followed by the last nfrombottom elements.
     * <p/>
     * Any overlap that might exist is NOT gotten rid off
     * Hence length of returned array is always top + bottom
     */
    public static int[] sub(int nfromtop, int nfrombottom, int[] data) {

        int[] indices = new int[nfromtop + nfrombottom];    // any overlap that might exist is NOT gotten rid off

        // first include the top genes
        int cnt = 0;

        for (int i = 0; i < nfromtop; i++) {
            indices[cnt++] = data[i];
        }

        // Then the from the bottom
        for (int i = data.length - nfrombottom; i < data.length; i++) {
            indices[cnt++] = data[i];
        }

        return indices;
    }

    /**
     * Expands each limit in the orig list by specified delimiter
     * For example:
     * aagff
     * aff:kkk
     * arrr:ppp
     * <p/>
     * expanded by ':' becomes the list
     * aagff
     * aff
     * kkk
     * arrr
     * ppp
     * <p/>
     * makes sense genrally only if orig list contains Strings
     */
    public static List expand(List orig, String delimiter) {

        List list = new ArrayList();

        for (int i = 0; i < orig.size(); i++) {
            StringTokenizer tok = new StringTokenizer((String) orig.get(i), delimiter);

            while (tok.hasMoreElements()) {
                list.add(tok.nextElement());
            }
        }

        return list;
    }

    /**
     * includes start and stop
     *
     * @todo better way please!
     */
    public static int[] fill(int start, int stop, int step) {

        int cnt = 0;

        for (int i = start; i <= stop; i += step) {
            cnt++;
        }

        int[] ret = new int[cnt];

        cnt = 0;

        for (int i = start; i <= stop; i += step) {
            ret[cnt++] = i;
        }

        return ret;
    }
}    // End ArrayUtils

/**
 * All pairwise comparisons
 * If is_symmetric then # of comps = n(n - 1) / 2
 * Else n**2
 *
 * comparisons doen by reference (==) and NOT by equaliy (.equals)
 */
/*
public static Map pairs(List elements, boolean is_symmetric) {

   Map map = new Hashtable();

   for (int i=0; i < elements.size(); i++) {
       for (int j=0; j < elements.size(); j++) {
           if (elements.get(i) != elements.get(j)  map.put(elements.get(i), elements.get(j));
       }
   }

   if (! is_symmetric) return map;
   else {
       Map tmap = new Hashtable();
       Iterator it = map.keySet().iterator();
       while (it.hasNext()) {
           Object key = it.next();
           Object val = map.get(key);
           System.out.println("checking: " + key + " " + val);
           if (key == val) ; // gets rid of foo to foo comparisons
           else if (tmap.get(key) == val) ;
           else if (tmap.get(val) == key) ;
           else {
               System.out.println("PUT it");
               tmap.put(key, val);
           }
       }

       int n = elements.size();
       int expectedn = (n*(n-1))/2;
       if (tmap.size() != expectedn) throw new RuntimeException("Invalid pairing expected: " + expectedn + " but got size: " + tmap.size());

       return tmap;
   }

}
*/

/**
 * arg arr is untouched
 */
/*
    public static float[] subarray(float[] a, int numb, Direction dir) {
        float[] res = new float[numb];
        if (dir == BOT2TOP) {
            System.arraycopy(a, 0, res, 0, numb);
        }
        else if (dir == TOP2BOT) {
            System.arraycopy(a, a.length-numb, res, 0, numb);
        }
        else if (dir == TWOTAILED) {
            if (!Direction.isEven(numb)) throw new IllegalArgumentException("Numb must be even for double sided");
            System.arraycopy(a, 0, res, 0, numb/2);
            System.arraycopy(a, a.length-numb, res, numb/2, numb/2);
        }
        return res;
    }
    */

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
