/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.strucs.NamedColor;
import org.apache.log4j.Logger;

import java.awt.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * This class provides methods for converting colors to and from string
 * representations. Known colors (those defined as constants in the
 * <code>Color</code> class) are converted directly to or from symbolic names
 * such as "green" or "magenta". Other colors are converted to or from RGB
 * specifications in the format #RRGGBB - a '#' followed by 6 hexadecimal
 * digits.
 *
 * @author Mark Lindner
 * @author PING Software Group
 * @version 1.0 (11/98)
 * @see java.awt.Color
 */
public class ColorUtils {


    public static Color COLOR_LIGHT_BLUE = new Color(150, 150, 255);

    public static final Color COLOR_DARK_BROWN = new Color(128, 64, 64);

    public static final Color COLOR_VERY_LIGHT_GRAY = new Color(239, 239, 239);

    public static final Color COLOR_DARK_BLUE = new Color(63, 64, 124);
    //public static final Color         lgray       = new Color(105, 120, 141);

    public static final Color COLOR_LIGHT_GREEN = new Color(153, 247, 121);
    public static final Color COLOR_LIGHT_ORANGE = new Color(255, 172, 89);

    /**
     * Colors often used, with well know english names
     */
    public static final NamedColor[] COMMON_COLORS = new NamedColor[]{
            // @maint alphabetical order please
            new NamedColor("Black", Color.black),
            new NamedColor("Blue", Color.blue),
            new NamedColor("Brown", new Color(Integer.decode("#A52A2A").intValue())),
            new NamedColor("CadetBlue", new Color(Integer.decode("#5F9EA0").intValue())),
            new NamedColor("Chocolate", new Color(Integer.decode("#8B4513").intValue())),
            new NamedColor("Cyan", Color.cyan),

            new NamedColor("DarkBlue", COLOR_DARK_BLUE),
            new NamedColor("DarkBrown", COLOR_DARK_BROWN),
            new NamedColor("DarkGray", Color.darkGray),
            new NamedColor("DarkGreen", new Color(Integer.decode("#006400").intValue())),


            new NamedColor("Gold", new Color(Integer.decode("#FFD700").intValue())),
            new NamedColor("Gray", Color.gray),
            new NamedColor("Green", Color.green),

            new NamedColor("LightGray", Color.lightGray),
            new NamedColor("LightGreen", COLOR_LIGHT_GREEN),
            new NamedColor("LightOrange", COLOR_LIGHT_ORANGE),
            new NamedColor("LightRed", new Color(Integer.decode("#FF3030").intValue())),

            new NamedColor("Magenta", Color.magenta),
            new NamedColor("Orange", Color.orange),
            new NamedColor("Pink", Color.pink),
            new NamedColor("Red", Color.red),
            new NamedColor("SeaGreen", new Color(Integer.decode("#54FF9F").intValue())),
            new NamedColor("SkyBlue", new Color(Integer.decode("#87CEEB").intValue())),
            new NamedColor("Turquoise", new Color(Integer.decode("#00F5FF").intValue())),
            new NamedColor("White", Color.white),
            new NamedColor("Yellow", Color.yellow)

    };

    private static final Logger klog = XLogger.getLogger(ColorUtils.class);

    private static final Map kUcNameColorMap = new HashMap();
    private static final Map kColorUcNameMap = new HashMap();

    // init hashes for later lookup
    static {

        //@note store all in upper case
        for (int i = 0; i < COMMON_COLORS.length; i++) {
            kUcNameColorMap.put(COMMON_COLORS[i].name.toUpperCase(), COMMON_COLORS[i].color);
            kColorUcNameMap.put(COMMON_COLORS[i].color, COMMON_COLORS[i].name.toUpperCase());
        }

    }


    /**
     * Get a name for a color. Returns a name for the color, if the color is one
     * of the colors predefined in the <code>Color</code> class: black, blue,
     * cyan, gray, green, magenta, orange, pink, red, white, or yellow.
     *
     * @param c The color.
     * @return The name of the color, or <b>java.awt.Color(rbg)</b> if
     *         <code>c</code> is no one of the colors listed above.
     * @see #nameForColor
     */
    public static String nameForColor(Color c) {
        //log.debug("nameForColor: " + c);
        if (c == null) {
            throw new IllegalArgumentException("Param c cannot be null");
        }

        Object name = kColorUcNameMap.get(c);

        if (name != null) {
            return (String) name;
        } else {
            return "new java.awt.Color(" + c.getRed() + "," + c.getGreen() + "," + c.getBlue() + ")";
        }
    }

    /**
     * Get a color for a name. Returns a color for a color name, if the name
     * identifies one of the colors predefined in the <code>Color</code> class.
     *
     * @param name The name of the color.
     * @return The <code>Color</code> object for the given name, or <b>null</b>
     *         if <code>name</code> does not identify one of the colors listed above.
     * @see #colorForName
     */
    public static Color colorForName(String name, boolean barfonbadname) {

        if (name == null) {
            throw new IllegalArgumentException("Param name cannot be null");
        }

        // all colors are stored in upper case
        String ucn = name.toUpperCase();
        Object color = kUcNameColorMap.get(ucn);

        if (color != null) {
            return (Color) color;
        } else {

            if (barfonbadname) {
                throw new IllegalArgumentException("Invalid color specfied: " + name);
            } else {
                klog.warn("Unknown color name: " + name + " using black");
                return Color.black;
            }
        }

    }

    /**
     * No! Always just return the hex dec thing.
     * Format a color as a string. Returns a string representation of the given
     * color as either a symbolic name (if the color is one of the colors
     * predefined in the <code>Color</code> class), or a hex representation of
     * the color in the format <tt>#RRGGBB</tt>.
     *
     * @param color The <code>Color</code> to parse.
     * @return A string representation of the color.
     * @see #parse
     */
    public static String toHex(final Color color) {

        //String s = nameForColor(color);
        //if(s != null) return(s);
        // otherwise format it as #RRGGBB
        StringBuffer sb = new StringBuffer();

        sb.append('#');

        String s = Integer.toHexString(color.getRGB() & 0x1FFFFFF);

        sb.append(s.substring(1));

        return (sb.toString());
    }

    /**
     * Parse a color representation, returning an appropriate <code>Color</code>
     * object.
     *
     * @param name The name of the color; one of the strings <i>black, blue,
     *             cyan, gray, green, magenta, orange, pink, red, white, yellow</i>, or an
     *             RGB color specification of the form <tt>#RRGGBB</tt>.
     * @return An appropriate <code>Color</code> object.
     * @throws kiwi.text.ParsingException If <code>name</code> is an
     *                                    invalid color representation.
     * @see #toHex
     */
    public static Color parse(String name, boolean barfonbadname) throws IllegalArgumentException {

        Color r;

        if (name.charAt(0) == '#') {

            /* process #RRGGBB specification */
            try {
                Integer rgb = Integer.valueOf(name.substring(1), 16);

                return (new Color(rgb.intValue()));
            } catch (NumberFormatException e) {
                throw (new IllegalArgumentException("Invalid RGB color syntax: " + name));
            }
        } else {
            r = colorForName(name, barfonbadname);

            if (r == null) {
                throw (new IllegalArgumentException("Unknown color name: " + name));
            } else {
                return (r);
            }
        }
    }

    /**
     * Randomnly generate ncolor objects
     */
    public static Color[] generate(int ncolors) {

        Color[] colors = new Color[ncolors];

        for (int i = 0; i < ncolors; i++) {
            colors[i] = new Color((float) Math.random(), (float) Math.random(),
                    (float) Math.random());
        }

        return colors;
    }

    public static Color[] generate(int ncolors, Color[] firstUseThese) {

        java.util.List colors = new ArrayList();

        for (int i = 0; i < ncolors; i++) {
            if (i < firstUseThese.length) {
                colors.add(firstUseThese[i]);
            } else {
                colors.add(generate());
            }
        }

        return (Color[]) colors.toArray(new Color[colors.size()]);

    }

    public static Color generate() {
        return new Color((float) Math.random(), (float) Math.random(),
                (float) Math.random());
    }

}    // End ColorUtils

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
