/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;


/**
 * Date related utilities.
 * Includes methods to get dates formatted in several ways.
 * Note - this class is not completely static methods - the data formatting (kiwi) related
 * methods are to be used as a regular non-static java object.
 * <p/>
 * Overview of the kiwi methods in this class: <br>
 * <p/>
 * This class represents a high-level date parser/formatter that is almost
 * fully format-string-compatible with the UNIX <b>strftime(3)</b> function.
 * Date formats are specified by formatting directives; these directives are
 * enumerated in the table below:
 * <p/>
 * <center><table border=1 cellspacing=0 cellpadding=2>
 * <tr><th>Directive</th><th align=left>Meaning</th></tr>
 * <tr><td>%a</td>      <td>Abbreviated weekday name (3 characters).</td></tr>
 * <tr><td>%A</td>      <td>Full weekday name.</td></tr>
 * <tr><td>%b</td>      <td>Abbreviated month name (3 characters).</td></tr>
 * <tr><td>%B</td>      <td>Full month name.</td></tr>
 * <tr><td>%d</td>      <td>Day of month (01-31).</td></tr>
 * <tr><td>%e</td>      <td>Day of month (no leading zero).</td></tr>
 * <tr><td>%H</td>      <td>24-hour clock hour (00-23).</td></tr>
 * <tr><td>%I</td>      <td>12-hour clock hour (01-12).</td></tr>
 * <tr><td>%j</td>      <td>Day in year (001-366).</td></tr>
 * <tr><td>%m</td>      <td>Month in year (01-12).</td></tr>
 * <tr><td>%M</td>      <td>Minutes (00-59).</td></tr>
 * <tr><td>%N</td>      <td>Era name.</td></tr>
 * <tr><td>%p</td>      <td>AM/PM character.</td></tr>
 * <tr><td>%r</td>      <td>Equivalent to %I:%M:%S %p.</td></tr>
 * <tr><td>%R</td>      <td>Equivalent to %H:%M.</td></tr>
 * <tr><td>%s</td>      <td>Milliseconds (000-999).</td></tr>
 * <tr><td>%S</td>      <td>Seconds (00-59).</td></tr>
 * <tr><td>%t</td>      <td>Tab character.</td></tr>
 * <tr><td>%T</td>      <td>Equivalent to %H:%M%:S.</td></tr>
 * <tr><td>%u</td>      <td>Equivalent to %a.</td></tr>
 * <tr><td>%U</td>      <td>Week of year.</td></tr>
 * <tr><td>%y</td>      <td>Year (2 digits).</td></tr>
 * <tr><td>%Y</td> <td>Year (4 digits).</td></tr>
 * <tr><td>%Z</td> <td>Time zone name (4 characters).</td></tr>
 * <tr><td>%%</td> <td>The % character.</td></tr>
 * <tr><td>. : / - , ;</td> <td>These characters are interpreted literally.
 * </td></tr>
 * </table></center>
 *
 * @author Mark Lindner (kiwi)
 * @author PING Software Group (kiwi)
 * @author David Gilbert (david.gilbert@object-refinery.com)
 * @author Aravind Subramanian (xomics)
 * @version %I%, %G%
 */
public class DateUtils implements MonthConstants {

    // -------------------------------------------------- //
    // --------------------- KIWI ----------------------- //
    // -------------------------------------------------- //
    private final SimpleDateFormat fmt;

    /**
     * A working calendar.
     */
    private static final Calendar CALENDAR = Calendar.getInstance();

    /**
     * Class Constructor.
     * Construct a new <code>DateFormatter</code>. See the table of formatting
     * directives above.
     *
     * @param pattern The pattern to be used by this formatter.
     * @throws IllegalArgumentException If the pattern contains
     *                                  unrecognized formatting directives.
     */
    public DateUtils(String pattern) throws IllegalArgumentException {

        fmt = new SimpleDateFormat(constructPattern(pattern));

        fmt.setTimeZone(TimeZone.getDefault());
    }

    public static String toDayMonYear(long timeinmillis) {

        Date d = new Date(timeinmillis);
        //SimpleDateFormat sdf = new SimpleDateFormat();

        java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("EEE, MMM d, ''yy");

        return sdf.format(d);

    }

    public static String toHourMin(long timeinmillis) {

        Date d = new Date(timeinmillis);
        //SimpleDateFormat sdf = new SimpleDateFormat();
        java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("K a m");
        return sdf.format(d);

    }

    /**
     * Parse a date, returning a <code>Date</code> object.
     *
     * @param text The text to parse.
     * @return A <code>Date</code> object corresponding to the parsed date.
     * @throws ParseException If a parsing error occurred.
     * @see #format
     */
    public Date parse(String text) throws java.text.ParseException {
        return (fmt.parse(text));
    }

    /**
     * Format a <code>Calendar</code> object. The date is formatted according
     * to the pattern specified in this object's constructor.
     *
     * @param date The <code>Date</code> to format.
     * @return The date formatted as a string.
     * @see #parse
     */
    public synchronized String format(Calendar date) {
        return (format(date.getTime()));
    }

    /**
     * Format the current date. The date is formatted according to the pattern
     * specified in this object's constructor.
     *
     * @return The date formatted as a string.
     * @see #parse
     */
    public synchronized String format() {
        return (format(new Date()));
    }

    /**
     * Format a date from a <code>Date</code> object. The date is formatted
     * according to the pattern specified in this object's constructor.
     *
     * @param date The <code>Date</code> to format.
     * @return The date formatted as a string.
     * @see #parse
     */
    public synchronized String format(Date date) {
        return (fmt.format(date));
    }

    /* pattern translator */
    private String constructPattern(String text) throws IllegalArgumentException {

        StringBuffer sb = new StringBuffer();
        boolean escaped = false;

        for (int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);

            if (!escaped) {
                switch (c) {

                    case':':
                    case';':
                    case'/':
                    case'.':
                    case',':
                    case'-':
                    case' ':
                    case'\t':
                    case'(':
                    case')':
                        sb.append(c);

                        escaped = false;
                        break;

                    case'%':
                        if (escaped) {
                            sb.append(c);

                            escaped = false;
                        } else {
                            escaped = true;
                        }
                        break;

                    default:
                        throw (new IllegalArgumentException("Unknown format character :" + c));
                }
            } else {
                switch (c) {

                    case'a':
                        sb.append("EE");
                        break;

                    case'A':
                        sb.append("EEEE");
                        break;

                    case'b':
                        sb.append("MMM");
                        break;

                    case'B':
                        sb.append("MMMM");
                        break;

                    case'd':
                        sb.append("dd");
                        break;

                    case'e':
                        sb.append('d');
                        break;

                    case'H':
                        sb.append("HH");
                        break;

                    case'I':
                        sb.append("hh");
                        break;

                    case'j':
                        sb.append("DDD");
                        break;

                    case'm':
                        sb.append("MM");
                        break;

                    case'M':
                        sb.append("mm");
                        break;

                    case'N':
                        sb.append('G');
                        break;

                    case'p':
                        sb.append('a');
                        break;

                    case'r':
                        sb.append(constructPattern("%I:%M:%S %p"));
                        break;

                    case'R':
                        sb.append(constructPattern("%H:%M"));
                        break;

                    case'S':
                        sb.append("ss");
                        break;

                    case't':
                        sb.append('\t');
                        break;

                    case'T':
                        sb.append(constructPattern("%H:%M:%S"));
                        break;

                    case'u':
                        sb.append("EE");
                        break;

                    case'U':
                        sb.append("ww");
                        break;

                    case's':
                        sb.append("SSS");
                        break;

                    case'y':
                        sb.append("yy");
                        break;

                    case'Y':
                        sb.append("yyyy");
                        break;

                    case'Z':
                        sb.append("zzzz");
                        break;

                    default:
                        throw (new IllegalArgumentException("Unknown escape sequence: %" + c));
                }

                escaped = false;
            }
        }

        return (sb.toString());
    }    // end constructPattern

    // -------------------------------------------------- //
    // ---------------------- End KIWI ------------------ //
    // -------------------------------------------------- //

    /**
     * Format example: "01-11-04_00-01-AM"
     */
    public final static int HYPHENDED_DIGITS = 1;

    /**
     * Type is one of the static int's defined in this class.
     */
    public static String getDateString(int type) {

        if (type == HYPHENDED_DIGITS) {
            return hyphened_digits();
        } else {
            throw new IllegalArgumentException("Invalid format type " + type);
        }
    }

    /**
     * @returns todays date localized to the env.
     * Example: Sep 17, 2001
     */
    public static String getDate() {

        Calendar calendar = new GregorianCalendar();
        Date date = calendar.getTime();
        DateFormat localFormat = DateFormat.getDateInstance();

        return localFormat.format(date);
    }

    /**
     * Creates a date.
     *
     * @param yyyy  the year.
     * @param month the month (1 - 12).
     * @param day   the day.
     * @return a date.
     */
    public static synchronized Date createDate(int yyyy, int month, int day) {
        CALENDAR.set(yyyy, month - 1, day);
        return CALENDAR.getTime();
    }

    /**
     * Creates a date.
     *
     * @param yyyy  the year.
     * @param month the month (1 - 12).
     * @param day   the day.
     * @param hour  the hour.
     * @param min   the minute.
     * @return a date.
     */
    public static synchronized Date createDate(int yyyy, int month, int day, int hour, int min) {

        CALENDAR.set(yyyy, month, day, hour, min);
        return CALENDAR.getTime();

    }

    /*
    public static synchronized String createDate(long timeinmillisecs) {
        Date d = new Date(timeinmillisecs);

        return d.

    }
    */

    /**
     * @return Data in format "01-11-04_00-01-AM"
     * @credits mike levin
     * http://www.mike-levin.com/learning-java/recursion-zipoutputstream.html
     */
    private synchronized static String hyphened_digits() {

        // this creates a Calender object full of current day information.
        Calendar calendar = Calendar.getInstance();

        // This sequence sets String objects.
        // The +"" forces a conversion to String objects.
        String YY = (calendar.get(Calendar.YEAR) + "").substring(2);
        String MM = ((calendar.get(Calendar.MONTH) + 1) + "");
        String DD = (calendar.get(Calendar.DAY_OF_MONTH) + "");
        String HH = (calendar.get(Calendar.HOUR) + "");
        String MI = (calendar.get(Calendar.MINUTE) + "");

        // The AM_PM fields contains 0 or 1, which I have to convert to AM or PM.
        int AMint = (calendar.get(Calendar.AM_PM));
        String AMStr;

        if (AMint == 0) {
            AMStr = "AM";
        } else {
            AMStr = "PM";
        }

        // And I do this for consistant date appearance. For example, 00-06-10_04-42-PM.
        if (MM.length() == 1) {
            MM = "0" + MM;
        }

        if (DD.length() == 1) {
            DD = "0" + DD;
        }

        if (HH.length() == 1) {
            HH = "0" + HH;
        }

        if (MI.length() == 1) {
            MI = "0" + MI;
        }

        return (YY + "-" + MM + "-" + DD + "_" + HH + "-" + MI + "-" + AMStr);
    }
}    // End DateUtils

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
