/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.utils;


import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.parsers.ParseUtils;
import org.apache.log4j.Logger;

import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.zip.GZIPInputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;


/**
 * Collection of utility methods related to Files.
 * <p/>
 * Also see org.apache.avalon.excalibur.io.FileUtil.java for additional methods not ported
 * here yet.
 *
 * @author org.apache.avalon.excalibur.io.FileUtil.java
 * @author Netbeans org.openide.filesystems.FileUtil.java
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class FileUtils {


    /**
     * For logging support
     */
    private final static Logger klog = XLogger.getLogger(FileUtils.class);

    /**
     * ref to the system's result dir, whereever that may be
     */
    public static final File TMP_DIR = new File(System.getProperty("java.io.tmpdir"));

    public static String path2name(final String path) {
        if (path.indexOf('/') == -1 && path.indexOf('\\') == -1) {
            return path;
        }

        StringTokenizer tok = new StringTokenizer(path, "/\\");
        String ret = tok.nextToken();
        while (tok.hasMoreTokens()) {
            ret = tok.nextToken();
        }

        return ret;
    }


    /**
     * The file is opened and closed after counting
     *
     * @param file
     * @param ignoreblanklines
     * @return
     * @throws IOException
     */
    public static int countLines(File file, boolean ignoreblanklines) throws IOException {
        BufferedReader bin = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
        return _countLines(bin, ignoreblanklines);
    }

    public static int countLines(String path, boolean ignoreblanklines) throws IOException {
        return countLines(new File(path), ignoreblanklines);
    }

    // reader should be untouched, reader is closed when done
    private static int _countLines(BufferedReader reader, boolean ignoreblanklines) throws IOException {
        int numberOfLines = 0;

        if (ignoreblanklines == false) {
            while (reader.readLine() != null) {
                numberOfLines++;
            }
        } else {
            String currLine = reader.readLine();

            while (currLine != null) {
                if (currLine.trim().length() > 0) {
                    numberOfLines++;
                }

                currLine = reader.readLine();
            }
        }

        reader.close();

        return numberOfLines;
    }

    public static void writeFofn(final List files, final boolean absPath, final File inFile) throws IOException {
        writeFofn((File[]) files.toArray(new File[files.size()]), absPath, inFile);
    }

    public static void writeFofn(final File[] files, final boolean absPath, final File inFile) throws IOException {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < files.length; i++) {
            if (absPath) {
                buf.append(files[i].getAbsolutePath()).append('\n');
            } else {
                buf.append(files[i].getName()).append('\n');
            }
        }

        write(buf.toString(), inFile);
    }

    // @todo doent work yet
    public static String getRelativePath(final File file, final File someParentDir) {
        String path = file.getPath();

        /*
        int index = path.lastIndexOf(someParentDir.getPath());
        if (index == -1) {
            throw new IllegalArgumentException("Not a parent: " + someParentDir + " of: " + file);
        }

        String got = path.substring(index + 1, path.length());
        */

        return path.substring(someParentDir.getPath().length() + 1, path.length());
        //klog.debug("file: " + file + " someParentDir: " + someParentDir + " reltPath: " + got);
    }

    public static boolean isSubFile(final File file, final File dir) {
        int index = file.getPath().indexOf(dir.getPath());
        return index != -1;
    }

    public static String unixPath(File file) {
        return file.getPath().replace('\\', '/');
    }

    public static String unixPath(String s) {
        return s.replace('\\', '/');
    }

    public static String windowsPath(File file) {
        return file.getPath().replace('/', '\\');
    }

    public static String windowsPath(String s) {
        return s.replace('/', '\\');
    }

    /**
     * Commonly
     */
    public static boolean isWindowsFileSystem() {
        return File.separatorChar == '\\';
    }

    public static boolean isUnixFileSystem() {
        return File.separatorChar == '/';
    }

    public static boolean isMacFileSystem() {
        return File.separatorChar == ':';
    }

    /**
     * To avoid the windows nt error
     * From BasicFileSystem in ext file chooser code
     *
     * @return
     */
    private static List windows_roots;

    public static File[] bugFixListRoots() {

        if (isWindowsFileSystem()) {

            if (windows_roots == null) {
                // hack alert
                // ->>aravind hack dont bother with floppy drive
                //String floppy = "A:\\";

                //Fake the hard drives. Test whether a drive is
                //mounted or not, add the mounted drives.
                windows_roots = new ArrayList();
                for (char c = 'C'; c <= 'Z'; c++) {
                    char drv[] = {c, ':', '\\'};
                    String drvName = new String(drv);
                    File drvFile = new File(drvName);
                    if (drvFile.exists()) {
                        windows_roots.add(new File(drvName));
                    }
                }
            }

            return (File[]) windows_roots.toArray(new File[windows_roots.size()]);

        } else {
            return File.listRoots();
        }

    }

    /**
     * usefule when file is in cwd -> parent tunrs out to be null
     *
     * @param parentFile
     * @param fileName
     * @return
     */
    public static File getSafeChildFile(File parentFile, String fileName) {

        if (parentFile == null) {
            return new File(fileName);
        } else {
            return new File(parentFile, fileName);
        }

    }

    /**
     * If the file exists, checks if is it a file and if it is rw
     */
    public static boolean isRWIfFileExists(File file) {

        if (!file.exists()) {
            return true;
        } else {
            if (!file.canRead()) {
                return false;
            }

            if (!file.canWrite()) {
                return false;
            }

            if (file.isDirectory()) {
                return false;
            }

            return true;
        }
    }

    public static boolean isRWFile(File file) {

        if (!file.exists()) {
            return false;
        }

        if (!file.isFile()) {
            return false;
        }

        if (!file.canRead()) {
            return false;
        }

        if (!file.canWrite()) {
            return false;
        }

        return true;    // hooray, we're good
    }

    public static boolean isValidDir(File file) {

        if (!file.exists()) {
            return false;
        }

        if (!file.isDirectory()) {
            return false;
        }

        return true;
    }

    /**
     * Creates a temporary file with specified file suffix.
     * Example: for a file extension of .gif, specify suffix of ".gif"
     * <p/>
     * Temporary file will be deleted when the jvm exits (gracefully).
     * Temporary file name is constructed using the current time in millis
     * as the file suffix.
     * So, not foolproof, but close enough for most needs.
     * The file is made wherever java makes its result files.
     *
     * @return the temp File made.
     * @throws java.io.IOException
     */
    public static File createTempFile(String ext) throws java.io.IOException {

        StringBuffer n = new StringBuffer();

        n.append(System.currentTimeMillis());

        return File.createTempFile(n.toString(), ext);
    }

    public static FilenameFilter createExtFileFilter(String ext) {
        return new ExtensionFileFilter(ext);
    }

    public static FilenameFilter createExtsFileNameFilter(String[] exts) {
        return new ExtensionsFileFilter(exts);
    }

    public static javax.swing.filechooser.FileFilter createExtsFileFilter(String[] exts) {
        return new ExtensionsFileFilter(exts);
    }

    public static FilenameFilter createExtsFileFilter(Set exts) {
        return new ExtensionsFileFilter(exts);
    }

    /**
     * Makes a temp file with suffix "result"
     *
     * @return the temp File made.
     * @throws java.io.IOException
     */
    public static File createTempFile() throws java.io.IOException {
        return createTempFile(".result");
    }

// shorten as easier to read

    public static String shortenedPathRepresentation(File file) {

        if ((file != null) && (file.getParentFile() != null)) {
            return (".." + File.separator + file.getParentFile().getName() + File.separator + file.getName());
        } else if (file != null) {
            return file.getName();
        } else {
            return null;
        }

    }

    public static String shortenedPathRepresentation(String path) {
        return shortenedPathRepresentation(new File(path));
    }

    /**
     * Copies fileA to fileB
     * Should work for all file types as it copies at the byte level
     * I'm as astonished as you that java didnt make this a method in the File class
     *
     * @throws IOException on trouble
     */
    public static File copy(File sourceFile, File toFile) throws IOException {
        FileInputStream source = new FileInputStream(sourceFile);
        FileOutputStream target = new FileOutputStream(toFile);
        copy(source, target);
        return toFile;
    }

    public static void copy(URL url, File fileB) throws IOException {
        InputStream source = url.openStream();
        FileOutputStream target = new FileOutputStream(fileB);
        copy(source, target);
    }

    public static void copy(InputStream source, OutputStream target) throws IOException {

        byte[] buffer = new byte[4096];
        int bytes_read;

        while ((bytes_read = source.read(buffer)) != -1) {
            target.write(buffer, 0, bytes_read);
        }

        source.close();
        target.close();
    }

/*
 *   public static void copy(ZipEntry z, File outFile) throws IOException {
 *       InputStream source =
 *     FileOutputStream target = new FileOutputStream(fileB);
 *       copy(source, target);
 *
 *   }
 */

    /**
     * Writes specified String to specified file.
     * File must exist. Any existing content is overwritten.
     *
     * @throws IOException on errors
     */
    public static void write(String s, File f) throws IOException {

        // open PrintWriter in overwrite mode
        PrintWriter out = new PrintWriter(new FileOutputStream(f));

        out.print(s);
        out.close();
    }

    public static void writeSafely(String s, File f) {

        try {
            // open PrinWriter in overwrite mode
            PrintWriter out = new PrintWriter(new FileOutputStream(f));

            out.print(s);
            out.close();
        } catch (Throwable t) {
            klog.error("Writing to file: " + f + " failed");
        }
    }

    /**
     * One element per line
     *
     * @param list
     * @param file
     * @throws IOException
     */
    public static void write(List list, File file) throws IOException {

        PrintWriter out = new PrintWriter(new FileOutputStream(file));

        for (int i = 0; i < list.size(); i++) {
            out.println(list.get(i));
        }

        out.close();
    }

    public static void write(String[] ss, File file, boolean singleLine) throws IOException {

        PrintWriter out = new PrintWriter(new FileOutputStream(file));

        for (int i = 0; i < ss.length; i++) {
            if (singleLine) {
                out.print(ss[i]);
                if (i != ss.length - 1) {
                    out.print('\t');
                }
            } else {
                out.println(ss[i]);
            }
        }

        out.close();
    }

    /**
     * @param list should contain String[]
     * @param file
     * @throws IOException
     */
    public static void writeTabularFile(List list, File file) throws IOException {

        PrintWriter out = new PrintWriter(new FileOutputStream(file));

        for (int i = 0; i < list.size(); i++) {
            String[] ss = (String[]) list.get(i);
            StringBuffer buf = new StringBuffer();

            for (int s = 0; s < ss.length; s++) {
                buf.append(ss[s]);

                if (s != ss.length - 1) {
                    buf.append('\t');
                }
            }

            buf.append('\n');
            out.write(buf.toString());
        }

        out.close();
    }

//-----------------------------------------------------------------------//

    public static void write(Map map, File file) throws IOException {

        PrintWriter out = new PrintWriter(new FileOutputStream(file));
        Iterator it = map.keySet().iterator();

        while (it.hasNext()) {
            Object key = it.next();

            out.println(key + "\t" + map.get(key));
        }

        out.close();
    }

//-----------------------------------------------------------------------//
//-----------------------------------------------------------------------//

    public static void writeByRow(Matrix matrix, File file) throws IOException {

        PrintWriter out = new PrintWriter(new FileOutputStream(file));

        for (int i = 0; i < matrix.getNumRow(); i++) {
            out.println(matrix.getRowV(i).toString('\t'));
        }

        out.close();
    }

    public static File writeByColumn(Matrix matrix, File file) throws IOException {

        PrintWriter out = new PrintWriter(new FileOutputStream(file));

        for (int i = 0; i < matrix.getNumCol(); i++) {
            out.println(matrix.getColumnV(i).toString('\t'));
        }

        out.close();

        return file;
    }

//-----------------------------------------------------------------------//
//-----------------------------------------------------------------------//

    /**
     * Vectors need NOT be of equal length
     */
    public static File write(edu.mit.broad.genome.math.Vector[] vectors, File file, boolean vertically)
            throws IOException {

        PrintWriter pw = new PrintWriter(new FileOutputStream(file));

        _out(vectors, pw, vertically);

        return file;
    }

    public static File append(edu.mit.broad.genome.math.Vector[] vectors, File file, boolean vertically)
            throws IOException {

        PrintWriter pw = new PrintWriter(new FileOutputStream(file.getAbsolutePath(), true));

        _out(vectors, pw, vertically);

        return file;
    }

    private static void _out(edu.mit.broad.genome.math.Vector[] vectors, PrintWriter out, boolean vertically) {

        int maxsize = edu.mit.broad.genome.math.Vector.maxSize(vectors);

        //System.out.println(">>>>>>." + maxsize);
        //Printf.out(vectors[0]);

        if (vertically) {
            for (int i = 0; i < maxsize; i++) {
                StringBuffer buf = new StringBuffer();

                for (int v = 0; v < vectors.length; v++) {
                    if (i >= vectors[v].getSize()) {
                        // do nothing
                    } else {
                        buf.append(vectors[v].getElement(i));
                    }

                    if (v != (vectors.length - 1)) {
                        buf.append('\t');
                    }
                }

                out.println(buf.toString());
            }
        } else {
            for (int v = 0; v < vectors.length; v++) {
                out.println(vectors[v].toString('\t'));
            }
        }

        out.close();
    }

    public static File write(edu.mit.broad.genome.math.Vector vector, File file, boolean vertically)
            throws IOException {
        return write(new edu.mit.broad.genome.math.Vector[]{vector}, file, vertically);
    }

    public static File append(edu.mit.broad.genome.math.Vector vector, File file, boolean vertically)
            throws IOException {
        return append(new edu.mit.broad.genome.math.Vector[]{vector}, file, vertically);
    }

    public static File write(edu.mit.broad.genome.math.Vector vector, File file, int horizChunkSize)
            throws IOException {

        PrintWriter pw = new PrintWriter(new FileOutputStream(file));

        int nrows = 1;
        if (vector.getSize() >= horizChunkSize) {
            if (vector.getSize() % horizChunkSize != 0) {
                klog.warn("Ignoring chunk size vector size: " + vector.getSize() + " not a multiple of horiz chunk size: " + horizChunkSize);
            }

            nrows = vector.getSize() / horizChunkSize;
        }

        int cnt = 0;
        for (int r = 0; r < nrows; r++) {
            StringBuffer buf = new StringBuffer();
            for (int c = 0; c < horizChunkSize; c++) {
                buf.append(vector.getElement(cnt++)).append('\t');
            }
            pw.println(buf.toString());
        }

        // just dump out the rest
        StringBuffer buf = new StringBuffer();
        for (int i = cnt; i < vector.getSize(); i++) {
            buf.append(vector.getElement(i)).append('\t');
        }

        if (buf.length() > 0) {
            pw.println(buf.toString());
        }

        pw.close();

        return file;

    }

//-----------------------------------------------------------------------//
//-----------------------------------------------------------------------//

    /**
     * Appends specified String to specified file.
     * File must exist.
     *
     * @throws IOException on errors
     */
    public static void append(String s, File f) throws IOException {
        if (f == null || !f.exists()) {
            throw new IOException("File does not exist: " + f);
        }

        // open PrintWriter in append mode
        PrintWriter out = new PrintWriter(new FileOutputStream(f.getPath(), true));

        out.print(s);
        out.close();
    }

    public static boolean hasExtension(String f, String ext) {

        int pos;

        pos = f.lastIndexOf("." + ext);

        return pos != -1;

    }

    public static String getExtension(File f) {
        return getExtension(f.getName());
    }

    public static String getExtension(String s) {

        int pos = s.lastIndexOf('.');

        if (pos == -1) {
            return "";
        } else {
            return s.substring(pos + 1, s.length());
        }

    }

    /**
     * returns +1 if f1 has a higher (i.e ext starts with a higher - alphabetically letter)
     * than f2.
     * 0 is equal
     * -1 if f2 > f1
     * <p/>
     * case insensitive.
     *
     * @param f1
     * @param f2
     */
    public static int compareExt(File f1, File f2) {
        return NamingConventions.getExtension(f1).compareToIgnoreCase(NamingConventions.getExtension(f2));
    }

    /**
     *
     */

    /*
     * public final static DataFormat toExtension(File file) {
     *   return toExtension(file.getName());
     * }
     */
    public static String changeExtension(String path, String newext) {

        int pos = path.lastIndexOf(".");

        if (pos == -1) {
            return path;    // no extension to change
        } else {
            return path.substring(0, pos).concat(".").concat(newext);
        }
    }

    public static String stripSuffixSlash(String p) {

        if (p == null) {
            return null;
        }

        if (p.length() == 0) {
            return p;
        }

        if (p.equals("/")) {
            return "";
        }

        if (p.endsWith("/")) {
            p = p.substring(0, p.lastIndexOf("/"));
        }

        return p;
    }

    public static void removeDuplicateLines(File inFile, File outFile) throws IOException {

        List list = new ArrayList();
        BufferedReader buf = new BufferedReader(new FileReader(inFile));
        String line;
        int totcnt = 0;
        int dupcnt = 0;

        line = ParseUtils.nextLine(buf);

        while (line != null) {
            totcnt++;

            if (list.contains(line)) {
                klog.debug("dup: " + line);

                dupcnt++;
            } else {
                list.add(line);
            }

            line = ParseUtils.nextLine(buf);
        }

        buf.close();
        klog.info("Original # of lines = " + totcnt + " # of duplicate lines = " + dupcnt);
        FileUtils.write(list, outFile);
    }

    /**
     * Convert from a <code>URL</code> to a <code>File</code>.
     *
     * @param url File URL.
     * @return The equivalent <code>File</code> object, or <code>null</code> if the URL's protocol
     *         is not <code>file</code>
     */
    public static File toFile(final URL url) {

        if (!url.getProtocol().equals("file")) {
            return null;
        } else {
            final String filename = url.getFile().replace('/', File.separatorChar);

            return new File(filename);
        }
    }

    /**
     * Convert the array of Files into a list of URLs.
     *
     * @param file the file
     * @return the array of URLs
     * @throws IOException if an error occurs
     */
    public static URL toURL(final File file) throws IOException {
        return file.toURL();
    }

    public static BufferedReader toBufferedReader(String filepathORurl) throws IOException {
        URL url = toURL(filepathORurl);
        return new BufferedReader(new InputStreamReader(url.openStream()));
    }

    private static URL toURL(String filepathORurl) throws IOException {
        if (filepathORurl.indexOf(':') == -1) {
            return new File((filepathORurl)).toURL();
        } else {
            return new URL(filepathORurl);
        }
    }

    /**
     * Convert the array of Files into a list of URLs.
     *
     * @param files the array of files
     * @return the array of URLs
     * @throws IOException if an error occurs
     */
    public static URL[] toURLs(final File[] files) throws IOException {

        final URL[] urls = new URL[files.length];

        for (int i = 0; i < urls.length; i++) {
            urls[i] = files[i].toURL();
        }

        return urls;
    }

    /**
     * Remove extension from filename.
     * ie
     * <pre>
     * foo.txt    --> foo
     * a\b\c.jpg --> a\b\c
     * a\b\c     --> a\b\c
     * </pre>
     *
     * @param filename the filename
     * @return the filename minus extension
     */
    public static String removeExtension(final String filename) {
        return NamingConventions.removeExtension(filename);
    }

    public static String removeExtension(final File file) {
        return NamingConventions.removeExtension(file.getName());
    }

    public static String removeExtensionIfPresent(final File f, final String ext) {
        return removeExtensionIfPresent(f.getName(), ext);
    }

    public static String removeExtensionIfPresent(final String s, final String ext) {
        int index = s.indexOf(ext);
        if (index != -1) {
            return s.substring(0, index);
        } else {
            return s;
        }
    }

    /**
     * Remove path from filename. Equivalent to the unix command <code>basename</code>
     * ie.
     * <pre>
     * a/b/c.txt --> c.txt
     * a.txt     --> a.txt
     * </pre>
     *
     * @param filepath the filepath
     * @return the filename minus path
     */
    public static String removePath(final String filepath) {
        return removePath(filepath, File.separatorChar);
    }

    /**
     * Remove path from filename.
     * ie.
     * <pre>
     * a/b/c.txt --> c.txt
     * a.txt     --> a.txt
     * </pre>
     *
     * @param filepath the filepath
     * @return the filename minus path
     */
    public static String removePath(final String filepath, final char fileSeparatorChar) {

        final int index = filepath.lastIndexOf(fileSeparatorChar);

        if (-1 == index) {
            return filepath;
        } else {
            return filepath.substring(index + 1);
        }
    }

    /**
     * Get path from filename. Roughly equivalent to the unix command <code>dirname</code>.
     * ie.
     * <pre>
     * a/b/c.txt --> a/b
     * a.txt     --> ""
     * </pre>
     *
     * @param filepath the filepath
     * @return the filename minus path
     */
    public static String getPath(final String filepath) {
        return getPath(filepath, File.separatorChar);
    }

    /**
     * Get path from filename.
     * ie.
     * <pre>
     * a/b/c.txt --> a/b
     * a.txt     --> ""
     * </pre>
     *
     * @param filepath the filepath
     * @return the filename minus path
     */
    public static String getPath(final String filepath, final char fileSeparatorChar) {

        final int index = filepath.lastIndexOf(fileSeparatorChar);

        if (-1 == index) {
            return "";
        } else {
            return filepath.substring(0, index);
        }
    }

    public static String getCrossPlatformName(final String filepath) {
        String tmp = getName(filepath, '/');
        tmp = getName(tmp, '\\');
        return tmp;
    }

    public static String getCrossPlatformName(final File file) {
        return getCrossPlatformName(file.getPath());
    }

    // @todo known issue with file paths that end with a sep char
    public static String getName(final String filepath, char fileSeparatorChar) {

        final int index = filepath.lastIndexOf(fileSeparatorChar);

        if (-1 == index) {
            return filepath;
        } else {
            return filepath.substring(index + 1, filepath.length());
        }
    }

    /**
     * checks if specified String "looks like" a file path (alt is file name)
     */
    public static boolean isPath(String s) {

        StringTokenizer tok = new StringTokenizer(s, File.pathSeparator);

        return tok.countTokens() != 1;

    }

    /**
     * Delete recursivly
     * <br>
     * This method from:
     * // IO.java in package org.w3c.util;
     * // $Id: IO.java,v 1.5 2000/08/16 21:37:58 ylafon Exp $
     * // (c) COPYRIGHT MIT and INRIA, 1998.
     * // Please first read the full copyright statement in file COPYRIGHT.html
     *
     * @param file the file (or directory) to delete.
     * @author Benoit Mahe (bmahe@w3.org)
     */
    public static boolean deleteRecursive(File file) {

        if (file.exists()) {
            if (file.isDirectory()) {
                if (deleteOnlyFilesRecursively(file)) {
                    return file.delete();
                } else {
                    return false;
                }
            } else {
                return file.delete();
            }
        }

        return true;
    }

    /**
     * Clean recursivly. Clean -> delete files but not dirs.
     * <br>
     * This method from:
     * // IO.java in package org.w3c.util;
     * // $Id: IO.java,v 1.5 2000/08/16 21:37:58 ylafon Exp $
     * // (c) COPYRIGHT MIT and INRIA, 1998.
     * // Please first read the full copyright statement in file COPYRIGHT.html
     *
     * @param file the directory to clean
     * @author Benoit Mahe (bmahe@w3.org)
     */
    public static boolean deleteOnlyFilesRecursively(File file) {

        if (file.isDirectory()) {
            String filen[] = file.list();

            for (int i = 0; i < filen.length; i++) {
                File subfile = new File(file, filen[i]);

                if ((subfile.isDirectory()) && (!deleteOnlyFilesRecursively(subfile))) {
                    return false;
                } else if (!subfile.delete()) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * is NOT recursive
     *
     * @param dirs
     */
    public static void deleteFilesInDirs(File[] dirs) {
        for (int i = 0; i < dirs.length; i++) {
            deleteFilesInDir(dirs[i]);
        }
    }

    /**
     * Is NOT recursive, just this dir
     * does its best, doesnt barf on errors
     *
     * @param dir
     * @return
     */
    public static boolean deleteFilesInDir(File dir) {
        if (dir == null) {
            throw new IllegalArgumentException("Parameter dir cannot be null");
        }

        boolean totalSuccess = true;

        if (!dir.isDirectory()) {
            klog.error("Not a directory!" + dir);
            totalSuccess = false;
        }

        klog.info("Deleting files in dir: " + dir.getAbsolutePath());
        File[] files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            if (files[i].isFile()) {
                boolean success = files[i].delete();
                if (!success) {
                    klog.error("could not delete file: " + files[i]);
                    totalSuccess = false;
                }
            }
        }

        return totalSuccess;
    }

    /**
     * only dirs that are empty will get deleted
     *
     * @param dirs
     */
    public static boolean deleteDirsNotRecursive(File[] dirs) {
        boolean allSuccess = true;
        for (int i = 0; i < dirs.length; i++) {
            boolean success = dirs[i].delete();
            if (!success) {
                allSuccess = success;
            }

        }

        return allSuccess;
    }

    public static File ungz(final File source) throws Exception {

        if (source == null) {
            throw new IllegalArgumentException("Param source  cannot be null");
        }

        if (!source.getName().endsWith(".gz")) {
            throw new IllegalArgumentException("Not a gz file: " + source.getName());
        }

        File targetFile = new File(source.getParentFile(), removeExtensionIfPresent(source.getName(), ".gz"));
        ungz(source, targetFile);
        return targetFile;
    }

    public static void ungz(final File source, final File dest) throws IOException {

        if (source.lastModified() > dest.lastModified()) {
            klog.info("Expanding " + source.getAbsolutePath() + " to "
                    + dest.getAbsolutePath());

            FileOutputStream out = null;
            GZIPInputStream zIn = null;
            FileInputStream fis = null;
            BufferedInputStream bis = null;

            try {
                out = new FileOutputStream(dest);
                fis = new FileInputStream(source);
                bis = new BufferedInputStream(fis);
                zIn = new GZIPInputStream(bis);
                byte[] buffer = new byte[8 * 1024];
                int count = 0;
                do {
                    out.write(buffer, 0, count);
                    count = zIn.read(buffer, 0, buffer.length);
                } while (count != -1);
            } catch (IOException ioe) {
                String msg = "Problem expanding gz " + ioe.getMessage();
                throw new IOException(msg + " " + ioe.getMessage());
            } finally {
                if (bis != null) {
                    bis.close();
                }
                if (fis != null) {
                    fis.close();
                }
                if (out != null) {
                    out.close();
                }

                if (zIn != null) {
                    zIn.close();
                }
            }
        }
    }

/*
// assumes only 1 file within the zipped file
public static File unzipSingleFile(File zippedFile) throws Exception {

    if (zippedFile == null) {
        throw new IllegalArgumentException("Param zippedFile cannot be null");
    }

    if (! zippedFile.exists()) {
        throw new IOException("Zipped file does not exist");
    }

    Enumeration entries;
    ZipFile zipFile = new ZipFile(zippedFile);

    entries = zipFile.entries();

    File outFile = null;
    int cnt = 0;
    while (entries.hasMoreElements()) {
        ZipEntry entry = (ZipEntry) entries.nextElement();

        System.err.println("Extracting file: " + entry.getName());
        outFile = new File(entry.getName());
        _copyInputStream(zipFile.getInputStream(entry),
                new BufferedOutputStream(new FileOutputStream(outFile)));

        if (cnt > 0) {
            throw new IllegalArgumentException("More than 1 file within the zipped file: " + zippedFile);
        }

        cnt++;
    }

    zipFile.close();

    return outFile;
}
*/

    /*
    public static File unzip(File zippedFile) throws Exception {

        if (zippedFile == null) {
            throw new IllegalArgumentException("Param zippedFile cannot be null");
        }

        if (zippedFile.exists() == false) {
            throw new IOException("File does not exist");
        }

        InputStream in = new BufferedInputStream(new FileInputStream(zippedFile));
        ZipInputStream zin = new ZipInputStream(in);
        ZipEntry e;

        while ((e = zin.getNextEntry()) != null) {
            _unzip(zin, e.getName());
        }
        zin.close();


    }

    private static void _unzip(ZipInputStream zin, String s) throws IOException {
        System.out.println("unzipping " + s);
        FileOutputStream out = new FileOutputStream(s);
        byte [] b = new byte[512];
        int len = 0;
        while ((len = zin.read(b)) != -1) {
            out.write(b, 0, len);
        }
        out.close();
    }
    */


    /**
     * Lists the contents of a zip archive.
     */
    public static void listArchive(File file) throws java.io.IOException {

        ZipFile z = new ZipFile(file);

        for (Enumeration e = z.entries(); e.hasMoreElements();) {
            ZipEntry ze = (ZipEntry) e.nextElement();

            System.out.println(ze.getName());
        }
    }


    public static List listFilesL(File[] dirs, FilenameFilter filt, boolean recursive) {

        List files = new ArrayList();

        for (int i = 0; i < dirs.length; i++) {
            files.addAll(listFilesL(dirs[i], filt, recursive));
        }

        return files;
    }


    public static File findFile(final File dir, final String endsWith, boolean barfOnMissing) {

        File[] all = listFiles_by_ends_with(dir, endsWith, false);

        if (all != null && all.length == 1) {
            return all[0];
        } else {
            if (barfOnMissing) {
                throw new IllegalArgumentException("No file with endsWith: " + endsWith + " found in dir: "
                        + dir + "\n" + Printf.outs(all));
            } else {
                return null;
            }
        }

    }

    public static File findFile(final File dir, final String[] endsWiths, boolean barfOnMissing) {

        final List files = new ArrayList();

        for (int i = 0; i < endsWiths.length; i++) {
            files.addAll(Arrays.asList(listFiles_by_ends_with(dir, endsWiths[i], false)));
        }

        final File[] all = (File[]) files.toArray(new File[files.size()]);

        if (all.length == 1) {
            return all[0];
        } else {
            if (barfOnMissing) {
                throw new IllegalArgumentException("No file found in dir: <"
                        + dir + "< with endsWiths\n" + Printf.outs(endsWiths));
            } else {
                return null;
            }
        }

    }

    /**
     * @param dir
     * @param ext
     */
    public static File[] listFiles(File dir, String ext, boolean recursive) {
        return listFiles(dir, new ExtensionFileFilter(ext), recursive);
    }

    public static Set listFiles_set(File dir, String ext, boolean recursive) {
        return new HashSet(Arrays.asList(listFiles(dir, ext, recursive)));
    }

    public static File[] listFiles(File dir, String[] exts, boolean recursive) {
        return listFiles(dir, new ExtensionsFileFilter(exts), recursive);
    }

    public static File[] listFiles_by_ends_with(final File dir, final String endsWith, final boolean recursive) {
        return listFiles(dir, new EndsWithFileFilter(endsWith), recursive);
    }

    public static File[] listFiles_by_starts_with(File dir, String startsWith, boolean recursive) {
        return listFiles(dir, new StartsWithFileFilter(startsWith), recursive);
    }

    public static File[] listFiles_by_starts_with_ends_with(File dir, String startsWith, String endsWith, boolean recursive) {
        return listFiles(dir, new StartsWithEndsWithFileFilter(startsWith, endsWith), recursive);
    }

    public static File[] listFiles_by_contains(File dir, String endsWith, boolean recursive) {
        return listFiles(dir, new ContainsFileFilter(endsWith), recursive);
    }

    public static File[] listFiles(final File dir, final FilenameFilter filt, final boolean recursive) {

        if (recursive) {
            return _listFilesRecursive(dir, filt);
        } else {
            if (filt == null) {
                return dir.listFiles();
            } else {
                return dir.listFiles(filt);
            }
        }
    }

    public static File[] listFiles(File dir, boolean recursive) {
        FilenameFilter filt = null;
        return listFiles(dir, filt, recursive);
    }

    public static List listFilesL(File dir, FilenameFilter filt, boolean recursive) {
        return _toList(listFiles(dir, filt, recursive));
    }

    public static File[] listFileByContains(String contains, File dir) {

        Set files = new HashSet();

        File[] fls = dir.listFiles();
        for (int i = 0; i < fls.length; i++) {
            if (Strings.contains(fls[i].getName(), contains, true)) {
                files.add(fls[i]);
            }
        }

        return (File[]) files.toArray(new File[files.size()]);

    }

    /**
     * NOT recursive, just in the specified file
     *
     * @param dir
     * @return
     */
    public static File[] listDirs(File dir, boolean recursive) {
        File[] files = listFiles(dir, recursive);
        List dirs = new ArrayList();

        for (int i = 0; i < files.length; i++) {
            if (files[i].isDirectory()) {
                dirs.add(files[i]);
            }
        }

        return (File[]) dirs.toArray(new File[dirs.size()]);
    }

    // returns whether this path directory or not
    private static boolean isDirectory(String path) {
        File f = new File(path);
        return f.isDirectory();
    }

    /**
     * this function returns Enumeration of files from this path
     */
    private static File[] _listFilesRecursive(final String path, final FilenameFilter filt) {

        if (!new File(path).isDirectory()) {
            klog.warn("You asked me to list a File!: " + path);
        }

        final List files = new ArrayList();

        scanTree(path, files, filt);

        return (File[]) files.toArray(new File[files.size()]);
    }

    private static File[] _listFilesRecursive(final File dir, final FilenameFilter filt) {
        return _listFilesRecursive(dir.getPath(), filt);
    }

    /**
     * This recursive method scans folders and files from path
     * till the end of file.
     * Filenamefilt can be null
     */
    private static void scanTree(String path, List files, FilenameFilter filt) {

        File f = new File(path);
        String[] fileList;

        fileList = _getList_for_tree_scan(f, filt);

        for (int i = 0; i < fileList.length; i++) {
            if (isDirectory(path + File.separator + fileList[i])) {
                scanTree(path + File.separator + fileList[i], files, filt);
            } else {
                File fa = new File(path + File.separator + fileList[i]);
                //log.debug("Adding: " + fa);
                files.add(fa);
            }
        }
    }

    /**
     * This function returns list of files and folders on path
     */
    private static String[] _getList_for_tree_scan(File path, FilenameFilter filt) {

        try {
            if (filt != null) {
                return path.list(filt);
            } else {
                return path.list();
            }
        } catch (Exception e) {
            klog.error(e);
        }

        return null;
    }

    /**
     * Need -> to detect of some other external process (like ms excel) has a file open and thus
     * locked. If java tries to write to such a file, a IOException is thrown.
     * This method will attempt to detect if the specified file is locked by some such external process
     * Java doesnt have this built -- this method is a hck copied from a javalang.help posting.
     * Not sure if it will work in all cases, but seems reasonable.
     * <p/>
     * If file doesnt exts, returns false always
     *
     * @param file
     * @return
     */
    public static boolean isLocked(File file) {

        if (file.exists() == false) {
            return false;    // cant be locked if it doesnt exist!
        }

        File origFile = new File(file.getAbsolutePath());

// cant use this as it actually makes a result file by itslef, and then cant rename into it
//File newFile = File.createTempFile(file.getName(), ".lock_check");
        StringBuffer btmp = new StringBuffer(file.getName()).append(System.currentTimeMillis()).append(".lock_check");
        File newFile = new File(SystemUtils.getTmpDir(), btmp.toString());

//log.debug("Created lock result file: " + newFile.getPath() + " file: " + file.getAbsolutePath());
        boolean able2rename = file.renameTo(newFile);

//log.debug("able2rename: " + able2rename);
        if (!able2rename) {
            return true;    // couldnt rename, so we take this to mean that it must be locked
        } else {

// move it back!
            boolean movedback = newFile.renameTo(origFile);

            if (!movedback) {
                throw new IllegalStateException("Bad error - couldnt move file back after lock check");
            }

            return false;
        }
    }


    private static List _toList(File[] files) {
        List list = new ArrayList(files.length);
        for (int f = 0; f < files.length; f++) {
            list.add(files[f]);
        }

        return list;
    }

    /**
     * <p>Implementation of FilenameFilter for extension </p>
     * <p> </p>
     * <p> </p>
     * <p> </p>
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static final class ExtensionFileFilter implements FilenameFilter {

        /**
         * The file extension used for the filtering
         */
        public final String fExt;

        public ExtensionFileFilter(final String ext) {
            if (ext == null) {
                throw new IllegalArgumentException("Parameter ext cannot be null");
            }
            if (ext.startsWith(".")) {
                this.fExt = ext.substring(1, ext.length());
            } else {
                this.fExt = ext;
            }
        }

        /**
         * @param dir
         * @param name
         * @return false if the extension is NOT specified extension
         */
        public boolean accept(File dir, String name) {
            String ext = NamingConventions.getExtension(name);
            return ext.equalsIgnoreCase(fExt);
        }
    }    // End ExtensionFilter

    public static final class ExtensionsFileFilter extends javax.swing.filechooser.FileFilter implements FilenameFilter, java.io.FileFilter {

        /**
         * @todo make this case INsensitive lookup
         * The file extension used for the filtering
         */
        public final Set fExts;

        public ExtensionsFileFilter(final String[] exts) {
            if (exts == null) {
                throw new IllegalArgumentException("Parameter ext cannot be null");
            }

            this.fExts = new HashSet();
            for (int i = 0; i < exts.length; i++) {
                this.fExts.add(exts[i]);
                this.fExts.add(exts[i].toUpperCase());
                this.fExts.add(exts[i].toLowerCase());
            }
        }

        public ExtensionsFileFilter(final Set exts) {
            if (exts == null) {
                throw new IllegalArgumentException("Parameter ext cannot be null");
            }

            this.fExts = exts;
        }

        /**
         * @param dir
         * @param name
         * @return false if the extension is NOT specified extension
         */
        public boolean accept(File dir, String name) {
            String ext = NamingConventions.getExtension(name);
            return fExts.contains(ext);
        }

        public boolean accept(File file) {
            String ext = NamingConventions.getExtension(file);
            return fExts.contains(ext);
        }

        public String getDescription() {
            return "some desc"; // @todo
        }

    }    // End ExtensionFilter


    public static final class FileFilterWrapper extends javax.swing.filechooser.FileFilter implements FilenameFilter, java.io.FileFilter {


        public java.io.FileFilter fFilt;

        public FileFilterWrapper(java.io.FileFilter ff) {
            if (ff == null) {
                throw new IllegalArgumentException("Parameter ff cannot be null");
            }

            this.fFilt = ff;
        }

        public boolean accept(File dir, String name) {
            return fFilt.accept(new File(dir, name));
        }

        public boolean accept(File pathname) {
            return fFilt.accept(pathname);
        }

        public String getDescription() {
            return "some desc";
        }

    }    // End MyFileFilter


    public static class EndsWithFileFilter implements FilenameFilter {

        /**
         * The file extension used for the filtering
         */
        public final String fEndsWith;

        public EndsWithFileFilter(String endsWith) {
            if (endsWith == null) {
                throw new IllegalArgumentException("Parameter endsWith cannot be null");
            }
            this.fEndsWith = endsWith;
        }

        /**
         * @param dir
         * @param name
         * @return false if the extension is NOT specified extension
         */
        public boolean accept(File dir, String name) {
            return name.endsWith(fEndsWith);
        }
    }    // End EndsWithFilter

    public static class StartsWithFileFilter implements FilenameFilter {

        /**
         * The file extension used for the filtering
         */
        public final String fStartsWith;

        public StartsWithFileFilter(String startsWith) {
            if (startsWith == null) {
                throw new IllegalArgumentException("Parameter startsWith cannot be null");
            }
            this.fStartsWith = startsWith;
        }

        /**
         * @param dir
         * @param name
         * @return false if the extension is NOT specified extension
         */
        public boolean accept(File dir, String name) {
            return name.startsWith(fStartsWith);
        }
    }    // End StartsWithFilter

    public static final class StartsWithEndsWithFileFilter implements FilenameFilter {

        /**
         * The file extension used for the filtering
         */
        public final String fStartsWith;
        public final String fEndsWith;

        public StartsWithEndsWithFileFilter(String startsWith, String endsWith) {
            if (startsWith == null) {
                throw new IllegalArgumentException("Parameter startsWith cannot be null");
            }
            this.fStartsWith = startsWith;
            this.fEndsWith = endsWith;
        }

        /**
         * @param dir
         * @param name
         * @return false if the extension is NOT specified extension
         */
        public boolean accept(File dir, String name) {
            if (name.startsWith(fStartsWith) && name.endsWith(fEndsWith)) {
                return true;
            } else {
                return false;
            }
        }
    }    // End StartsWithEndsWithFileFilter

    public static final class ContainsFileFilter implements FilenameFilter {

        /**
         * The file extension used for the filtering
         */
        public final String fContains;

        public ContainsFileFilter(String contains) {
            if (contains == null) {
                throw new IllegalArgumentException("Parameter endsWith cannot be null");
            }
            this.fContains = contains;
        }

        /**
         * @param dir
         * @param name
         * @return false if the extension is NOT specified extension
         */
        public boolean accept(File dir, String name) {
            int index = name.indexOf(fContains);
            return index != -1;
        }
    }    // End ContainsFileFilter

    /**
     * File name filter that accepts all files except those that have an extension of "clazz"
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */

// Files named .clazz rather than .class tro avoid any confusion with real java class files.

    public static final class ClazzExcluderFileFilter implements FilenameFilter {

        /**
         * The file extension used fo rthe stub (extensions always are w/o the leading .)
         */
        public static final String CLAZZ_EXT = "clazz";

        public boolean accept(File dir, String name) {

            String ext = NamingConventions.getExtension(name);

            return !ext.equals(CLAZZ_EXT);

        }
    }    // End ClazzExcluderFileFilter

}        // End FileUtils

/*--- Formatted in Sun Java Convention Style on Sun, Nov 3, '02 ---*/
