/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import java.io.File;
import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.TreeMap;

/**
 * Utility methods related to the System class
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class SystemUtils {

    /**
     * For logging support
     */
    private static final Logger klog = XLogger.getLogger(SystemUtils.class);

    /**
     * prints all system env properties to stdout
     */
    public static void dump() {
        klog.info(asString());
    }

    private static Boolean kIsLinuxOrUnix;

    public static boolean isLinuxOrUnix() {
        //TraceUtils.showTrace();
        if (kIsLinuxOrUnix == null) {
            String osName = System.getProperty("os.name");
            //klog.debug("os.name=" + osName);
            osName = osName.toLowerCase();
            if (osName.indexOf("unix") != -1 || osName.indexOf("linux") != -1 || osName.equalsIgnoreCase("OSF1")) {
                kIsLinuxOrUnix = Boolean.TRUE;
            } else {
                kIsLinuxOrUnix = Boolean.FALSE;
            }
        }

        return kIsLinuxOrUnix.booleanValue();
    }

    public static boolean isMac() {
        String lcOSName = System.getProperty("os.name").toLowerCase();
        return lcOSName.startsWith("mac os x");
    }

    public static boolean getProperty_b(String key, boolean caseSensitive) {
        String val = getProperty(key, caseSensitive);
        try {
            return Boolean.valueOf(val);
        } catch (Throwable t) {
            return false;
        }
    }

    public static String getProperty(String key, boolean caseSensitive) {
        if (caseSensitive) {
            return System.getProperty(key);
        }

        String s = System.getProperty(key);
        if (s == null || s.length() == 0) {
            s = System.getProperty(key.toLowerCase());
        }

        if (s == null || s.length() == 0) {
            s = System.getProperty(key.toUpperCase());
        }

        return s;
    }

    public static String asString() {

        StringBuffer buf = new StringBuffer();
        Properties prop = System.getProperties();

        //System.out.println(prop);
        Iterator it = prop.keySet().iterator();

        while (it.hasNext()) {
            String obj = it.next().toString();
            Object val = prop.getProperty(obj);

            buf.append(obj).append('=').append(val).append('\n');
        }

        return buf.toString();
    }

    public static TableModel asTableModel() {
        Properties prop = System.getProperties();

        // so that we can get aan alphabetic key sorting
        TreeMap tm = new TreeMap();
        Iterator it = prop.keySet().iterator();
        while (it.hasNext()) {
            String obj = it.next().toString();
            Object val = prop.getProperty(obj);
            tm.put(obj, val);
        }

        it = tm.keySet().iterator();
        DefaultTableModel model = new DefaultTableModel(0, 2);
        while (it.hasNext()) {
            Object obj = it.next();
            Object val = tm.get(obj);
            model.addRow(new Object[]{obj, val});
        }

        model.setColumnIdentifiers(new String[]{"Name", "Value"});

        return model;
    }


    public static String getUserName() {
        return System.getProperty("user.name");
    }

    public static File getUserHome() {
        return new File(System.getProperty("user.home"));
    }

    public static boolean endsWith(File file, String str) {

        if (file.getName().endsWith(str) || file.getName().endsWith(str + "/") || file.getName().endsWith(str + "\\")) {
            return true;
        }

        return false;
    }

    public static String getUserHomePath() {
        return System.getProperty("user.home");
    }

    public static String getJavaPath() {
        //return getJavaHome() + File.pathSeparator + "bin" + File.pathSeparator + "java";
        return getJavaHome() + File.separator + "bin" + File.separator + "java";
    }

    public static String getJavaHome() {
        return System.getProperty("java.home").trim();
    }

    public static String getClassPath() {
        return System.getProperty("java.class.path").trim();
    }

    public static File getPwd() {
        return new File(System.getProperty("user.dir"));
    }

    public static File getTmpDir() {
        return new File(System.getProperty("java.io.tmpdir"));
    }

    /**
     * IMP strange method
     * Only possible if bot a mon and a max were specified
     * Format: jnlpx.heapsize	-> "96m,512m"
     * Only available if launched under jnlp with the
     * -max-heap-size tags
     * <p/>
     * else returns Float.Nan
     *
     * @return
     */
    public static String getJnlpHeapMaxSize() {
        String hs = getJnlpHeapValue();
        if (hs == null) {
            return null;
        }

        try {

            StringTokenizer tok = new StringTokenizer(hs, ",");
            if (tok.countTokens() == 2) {
                tok.nextToken();
                return tok.nextToken();
            }

        } catch (Throwable t) {
            klog.debug(t);
        }

        return null;

    }

    /**
     * formatted
     * jnlpx.heapsize	-> "96m,512m"
     *
     * @return
     */
    public static String getJnlpHeapValue() {
        return System.getProperty("jnlpx.heapsize");
    }

    // @note that the -D commands are case sensitive
    // luckilly the boolean parsing isnt
    public static boolean isHeadless() {
        return isPropertyTrue("java.awt.headless");
    }

    public static boolean isPropertyTrue(String prpName) {
        String p = getProperty(prpName, false);
        return Boolean.valueOf(p).booleanValue();
    }

    public static boolean isPropertyDefined(String prpName) {
        String p = getProperty(prpName, false);
        return p != null && p.trim().length() != 0;
    }

    public static File getFile(String prpName) {
        String p = System.getProperty(prpName);
        return new File(p);
    }

}    // End SystemUtils

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
