/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.viewers;

import com.jidesoft.grid.SortableTable;
import com.jidesoft.popup.JidePopup;
import com.jidesoft.swing.JideButton;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.models.NumberedProxyModel;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.VerticalFlowLayout;
import edu.mit.broad.xbench.actions.ext.ExcelAction;
import edu.mit.broad.xbench.core.api.Application;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.table.TableModel;
import java.awt.*;
import java.io.File;


/**
 * Base class for several Viewer.
 * Contains commonly useful initialization and methods.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public abstract class AbstractViewer extends JPanel implements Viewer {

    protected Logger log;

    protected static final Logger klog = XLogger.getLogger(AbstractViewer.class);

    private Icon fIcon;

    private String fName;

    private String fTitle;

    private static final Icon SMALL_EXCEL_ICON = JarResources.getIcon("Excel13.gif");
    private static final Icon SMALL_EXCEL_NA_ICON = JarResources.getIcon("ExcelNa.gif");


    /**
     * Class constructor
     *
     * @param name
     * @param icon
     */
    /*
    public AbstractViewer(final String name, final Icon icon) {
        this(name, icon, (String) null);
    }
    */

    // Users of this method must call init
    protected AbstractViewer() {
        this.log = XLogger.getLogger(this.getClass());
    }

    /**
     * Class constructor
     *
     * @param name
     * @param icon
     * @param title
     */
    public AbstractViewer(final String name, final Icon icon, final String title) {
        init(name, icon, title);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param icon
     * @param pob_for_setting_title
     */
    public AbstractViewer(final String name, final Icon icon, final PersistentObject pob_for_setting_title) {
        this(name, icon, formatTitle(pob_for_setting_title, name));
    }

    protected void init(final String name, final Icon icon, final String title) {

        if (name == null) {
            throw new IllegalArgumentException("Param name cannot be null");
        }

        if (title == null) {
            throw new IllegalArgumentException("Param title cannot be null");
        }

        this.fIcon = icon;
        this.fName = name;
        this.fTitle = title;
        if (log == null) {
            this.log = XLogger.getLogger(this.getClass());
        }

    }

    /* dotn allow this -- all viewers must have a name
    public AbstractViewer(final Icon icon) {
        this(null, icon);
    }
    */

    public boolean allowClose() {
        return true;
    }

    public JComponent getWindowUI() {
        _checkInit();
        return this;
    }

    public Icon getIcon() {
        _checkInit();
        return fIcon;
    }

    public String getName() {
        _checkInit();
        return fName;
    }

    private void _checkInit() {
        if (fName == null) {
            throw new IllegalStateException("Viewer likely not init'ed name: " + fName);
        }

        if (fTitle == null) {
            throw new IllegalStateException("Viewer likely not init'ed title: " + fTitle);
        }
    }

    public String getTitle() {
        return fTitle;
    }

    /*
    protected void setTitle(final String title) {
        this.fTitle = title;
    }

    // The reason we have this is that sometimes viewers are
    protected void setTitle(final PersistentObject pob) {
        this.fTitle = formatTitle(pob, getName());
    }*/

    public JMenuBar getJMenuBar() {
        return EMPTY_MENU_BAR;
    }

    protected void showError(Throwable t) {
        Application.getWindowManager().showError(t);
    }

    protected void showError(String msg) {
        Application.getWindowManager().showError(msg);
    }

    protected static JButton createButton(final Action action) {
        JideButton but = new JideButton(action);
        but.setButtonStyle(JideButton.TOOLBOX_STYLE);
        //but.setPreferredSize(new Dimension(100, 20));
        GuiHelper.Button.addMouseOverRollOverAction(but);
        return but;
    }

    protected static JComponent createToolBar(Action[] actions) {
        JPanel panel = new JPanel(new VerticalFlowLayout());
        for (int i = 0; i < actions.length; i++) {
            panel.add(createButton(actions[i]));
        }

        return panel;
    }


    protected static JPanel wrap(final JComponent comp) {
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(comp, BorderLayout.CENTER);
        return panel;
    }

    // protected static final Dimension kPlaceholderSize = new Dimension(200, 200);

    protected static Dimension getPlaceHolderSize() {
        return new Dimension(200, 200);
    }

    protected static Component createEmptyFiller() {
        JPanel panel = new JPanel();
        panel.setPreferredSize(getPlaceHolderSize());
        panel.setBackground(Color.WHITE);
        return panel;
        //return Box.createRigidArea(kPlaceholderSize);// doesnt do it
    }

    protected static Component createEmptyFiller(int height) {
        JPanel panel = new JPanel();
        panel.setPreferredSize(new Dimension(getPlaceHolderSize().width, height));
        panel.setBackground(Color.WHITE);
        return panel;
        //return Box.createRigidArea(kPlaceholderSize);// doesnt do it
    }

    protected static void showInPopup(final Component comp, final Component parent) {
        final JidePopup popup = new JidePopup();
        popup.setMovable(false);
        popup.getContentPane().setLayout(new BorderLayout());

        popup.getContentPane().add(comp);

        popup.setDefaultFocusComponent(comp);

        popup.setOwner(parent);
        popup.setResizable(true);
        popup.setMovable(true);
        if (popup.isPopupVisible()) {
            popup.hidePopup();
        } else {
            popup.showPopup();
        }
    }

    /**
     * useful to give windows a title that includes the pob's
     * name.
     * Null pob is ok - just returns the prefix
     *
     * @param pob
     * @return
     */
    protected static String formatTitle(PersistentObject pob, String name) {

        if (pob == null) {
            return "na -- " + name;
        } else {
            return pob.getName();
        }
        /* @note The icon is good enough to denote the "type" of viewer
        else {
            StringBuffer buf = new StringBuffer(fTitle);
            buf.append(": ").append(pob.getName());
            return buf.toString();
        }
        */
    }

    protected static String formatTitle(final PersistentObject pob) {
        if (pob == null) {
            return "na";
        } else {
            return pob.getName();
        }
    }

    protected static void setColumnSize(int size, int col, JTable table, boolean alsoMax) {
        GuiHelper.Table.setColumnSize(size, col, table, alsoMax);
    }

    protected static JScrollPane createExcelScrollPane(JTable table, PersistentObject pob) {

        JScrollPane sp = createAlwaysScrollPane(table);
        JButton excelButton;
        if (ParserFactory.getCache().isCached(pob)) {
            File file = ParserFactory.getCache().getSourceFile(pob);
            excelButton = new JButton(new ExcelAction(file.getPath()));
            excelButton.setText("");
            excelButton.setIcon(SMALL_EXCEL_ICON);
            excelButton.setToolTipText("Click me to open the object in Excel");
        } else {
            excelButton = new JButton(SMALL_EXCEL_NA_ICON);
            excelButton.setText("");
            excelButton.setToolTipText("Object constructed in memory and hence na for excel view");
        }

        sp.setCorner(JScrollPane.LOWER_RIGHT_CORNER, excelButton);
        return sp;
    }

    protected static JScrollPane createAlwaysScrollPane(JTable table) {

        JScrollPane sp = new JScrollPane(table);
        sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        return sp;

    }

    protected static SortableTable createTable(final TableModel model,
                                               final boolean addRowNumCol,
                                               final boolean boldHeaders) {


        TableModel amodel = model;

        if (addRowNumCol) {
            amodel = new NumberedProxyModel(model);
        }

        //JTable table = new JTable(amodel);
        SortableTable table = new SortableTable(amodel); // @note changed for jide

        // @note comm out renderers Dec 2005 .. the move to jgoodies lnf makes the headers look not so good

        if (addRowNumCol) { // has to be done after setting model
            //TableCellRenderer cellrend = new RendererFactory2.ColoredBgCellRenderer((GuiHelper.COLOR_VERY_LIGHT_GRAY));
            //table.setDefaultRenderer(String.class, new RendererFactory2.RendererColumnAdapter(cellrend, 0));
            setColumnSize(35, 0, table, true);
        }

        /*
        if (boldHeaders) {
            DefaultTableCellRenderer hrend = new RendererFactory2.BoldHeaderRenderer();
            table.getTableHeader().setDefaultRenderer(hrend);
        }
        */

        //table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        table.setCellSelectionEnabled(true);
        return table;
    }

} // End AbstractViewer
