package edu.mit.broad.genome.viewers;

import com.jidesoft.grid.SortableTable;
import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.math.ColorScheme;
import edu.mit.broad.genome.math.ColorSchemes;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.xbench.actions.ObjectAction;
import edu.mit.broad.xbench.core.Widget;
import gnu.trove.TFloatArrayList;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import java.awt.*;
import java.util.ArrayList;


/**
 * FSet viewer.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DatasetFeatureAnnViewer extends AbstractViewer {

    private static final String NAME = "Dataset feature annotation";

    private static final Icon ICON = JarResources.getIcon("Gin16.png");

    private Dataset fDataset;

    private ModelWithMetrics fModelWithMetrics;

    //                                                         // 0     1       2       3     4
    private static final String[] STAT_COL_NAMES = new String[]{"LOW", "HIGH", "MEAN", "STD", "MAD"};

    /**
     * Builds a Viewer on specified FSet object
     *
     * @param gset
     */
    public DatasetFeatureAnnViewer(final Dataset ds) {
        super(NAME, ICON, ds);

        this.fDataset = ds;

        jbInit();
    }

    private void jbInit() {
        this.fModelWithMetrics = new ModelWithMetrics(fDataset);
        JTable table = createTable(fModelWithMetrics, true, true);

        TableCellRenderer cellrend = new RendererForBackgroundSortable(fDataset);
        table.setDefaultRenderer(Object.class, cellrend);

        setColumnSize(10, 1, table, false);
        int start = 2;

        if (fDataset instanceof RankedList) {
            setColumnSize(25, 2, table, true);
            start = 3;
        }

        /*sizing doesnt seem to have any effect!!
        for (int i = 0; i < STAT_COL_NAMES.length; i++) {
            setColumnSize(25, i + start, table, true);
        }
        */

        this.setLayout(new BorderLayout());
        this.add(createExcelScrollPane(table, fDataset), BorderLayout.CENTER);
        this.revalidate();

    }

    public JMenuBar getJMenuBar() {
        return EMPTY_MENU_BAR;
    }

    static ColorScheme RED = new ColorSchemes.BroadCancerRed();
    static ColorScheme BLUE = new ColorSchemes.BroadCancerBlue();

    public static class RendererForBackgroundSortable extends DefaultTableCellRenderer {

        private Dataset fDataset;

        public RendererForBackgroundSortable(final Dataset ds) {
            this.fDataset = ds;

        }

        public Component getTableCellRendererComponent(final JTable table,
                                                       final Object value,
                                                       boolean isSelected,
                                                       boolean hasFocus,
                                                       int row,
                                                       int col) {


            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);

            // @note the color map is a a VERTICAL map but we display it horiz and hence need to convert
            // each row is a sample
            // also not every value has a color

            Color color = Color.WHITE;

            if (!isSelected) { // else cell selections cant be seen

                ColorMap.Rows cm = fDataset.getAnnot().getFeatureAnnot().getColorMap();

                if (cm != null) {
                    int actualRowIndex = ((SortableTable) table).getActualRowAt(row);
                    String featureName = fDataset.getRowName(actualRowIndex);
                    String colName = table.getColumnName(col);
                    color = cm.getColor(featureName, colName);
                }

                //System.out.println("^^ " + color);
                this.setBackground(color);

            }


            return this;
        }
    }    // End ColoredBgCellRenderer


    public static class RendererSimple extends DefaultTableCellRenderer {

        private ColorDataset cds;

        private Dataset fDataset;

        private JTable fTable;

        //         // doesnt work yet because the cds is made for RANGES and not for row index (rank index)
        public RendererSimple(final Dataset ds, JTable table) {
            this.fDataset = ds;
            this.fTable = table;
            if (fDataset instanceof RankedList) {
                this.cds = new DatasetGenerators().createColorDataset(100, (RankedList) fDataset, RED);
            }

        }

        public RendererSimple(final Dataset ds) {
            this.fDataset = ds;
        }

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected,
                                                       boolean hasFocus,
                                                       int row,
                                                       int col) {

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);

            String ttTxt = "";

            /*
             // doesnt work yet because the cds is made for RANGES and not for row index (rank index)
            if (!isSelected) { // else cell selections cant be seen
                Color color = Color.WHITE;

                if (cds != null && fTable.getColumnName(col).equals(Headers.SCORE)) {
                    color = cds.getColor(0, row);
                } else {
                    ColorMap.Rows cm = fDataset.getAnnot().getFeatureAnnot().getColorMap();
                    if (value != null) {
                        ttTxt = value.toString();
                    }

                    if (cm != null) {
                        String featureName = fDataset.getRowName(row);
                        String colName = table.getColumnName(col);
                        color = cm.getColor(featureName, colName);
                    }
                }

                //System.out.println("^^ " + color);
                this.setBackground(color);
            }
            */

            if (!isSelected) { // else cell selections cant be seen
                Color color = Color.WHITE;

                ColorMap.Rows cm = fDataset.getAnnot().getFeatureAnnot().getColorMap();
                if (value != null) {
                    ttTxt = value.toString();
                }

                if (cm != null) {
                    String featureName = fDataset.getRowName(row);
                    String colName = table.getColumnName(col);
                    color = cm.getColor(featureName, colName);
                }

                //System.out.println("^^ " + color);
                this.setBackground(color);
            }

            this.setToolTipText(ttTxt);

            return this;
        }

    }    // End RendererSimple


    /**
     * Inner class representing an action for the viewer\
     */
    public static class Action extends ObjectAction {

        private Dataset fDataset;

        /**
         * Class constructor
         */
        public Action() {

            this.putValue(Action.NAME, getName());
            this.putValue(Action.SMALL_ICON, getIcon());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        /**
         * Class constructor
         *
         * @param ds
         */
        public Action(Dataset ds) {
            this();
            this.fDataset = ds;
        }

        public void setObject(Object obj) {
            this.fDataset = (Dataset) obj;
        }

        public Widget getWidget() {
            return new DatasetFeatureAnnViewer(fDataset);
        }

        public String getId() {
            return "DatasetFeatureAnnotationViewerAction";
        }

        public String getName() {
            return NAME;
        }

        public Icon getIcon() {
            return ICON;
        }

        public String getDescription() {
            return "View feature annotations";
        }

    }    // End Action


    /**
     * An implementation of AbstractTableModel for SampleAnnot.
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class ModelWithMetrics extends AbstractTableModel {

        /**
         * The underlying object being modell'ed
         */
        private Dataset fDataset;

        private FeatureAnnot fFeatureAnnot;

        private ColorMap.Rows fColorMap;

        private String[] fColNames;

        private TFloatArrayList[] fStatsPerRow; // @note this is synched to the dataset and NOT the annot

        /**
         * Class Constructor.
         * Initializes model to specified Dataset
         * <p/>
         * Get rid of the desc field if it is na
         * Fill in a color field if the dataset is really a scored dataset
         */
        public ModelWithMetrics(final Dataset ds) {
            this.fDataset = ds;
            this.fFeatureAnnot = ds.getAnnot().getFeatureAnnot();
            this.fColorMap = fFeatureAnnot.getColorMap();
            final java.util.List names = new ArrayList();
            names.add(Headers.NAME);

            if (ds instanceof RankedList) {
                names.add(Headers.SCORE);
            }


            if (fFeatureAnnot.hasNativeDescriptions()) {
                names.add(Headers.DESCRIPTION);
            }

            // no point adding symbol if the chip is already a symbol chip
            if (VdbRuntimeResources.isChipGeneSymbol(fFeatureAnnot.getChip())) {
                names.add(Headers.GENE_SYMBOL);
            }

            names.add(Headers.GENE_TITLE);

            if (fColorMap != null) {
                for (int c = 0; c < fColorMap.getNumCol(); c++) {
                    names.add(fColorMap.getColName(c));
                }
            }

            this.fStatsPerRow = new TFloatArrayList[STAT_COL_NAMES.length];

            for (int c = 0; c < STAT_COL_NAMES.length; c++) {
                fStatsPerRow[c] = new TFloatArrayList();
                names.add(STAT_COL_NAMES[c]);
            }

            edu.mit.broad.genome.utils.Timer timer = new edu.mit.broad.genome.utils.Timer();
            timer.start();
            for (int r = 0; r < ds.getNumRow(); r++) {
                Vector v = ds.getRow(r);
                fStatsPerRow[0].add(v.min());
                fStatsPerRow[1].add(v.max());
                fStatsPerRow[2].add((float) v.mean());
                fStatsPerRow[3].add((float) v.stddev(true, false));
                fStatsPerRow[4].add((float) v.mad(false));
            }

            timer.stop();
            timer.printTimeTaken("DS stats");

            this.fColNames = (String[]) names.toArray(new String[names.size()]);
        }

        public int getColumnCount() {
            return fColNames.length;
        }

        public int getRowCount() {
            return fDataset.getNumRow();
        }

        public String getColumnName(int col) {
            return fColNames[col];
        }

        public Object getValueAt(int row, int col) {

            final String featureName = fDataset.getRowName(row);

            if (col == 0) {
                return featureName;
            }

            final String colName = getColumnName(col);

            if (colName.equals(Headers.SCORE)) {
                return Printf.format(((RankedList) fDataset).getScore(featureName), 2);
            } else if (colName.equals(Headers.DESCRIPTION)) {
                return fFeatureAnnot.getNativeDesc(featureName);
            } else if (colName.equals(Headers.GENE_SYMBOL)) {
                return fFeatureAnnot.getGeneSymbol(featureName);
            } else if (colName.equals(Headers.GENE_TITLE)) {
                return fFeatureAnnot.getGeneTitle(featureName);
            }

            for (int c = 0; c < STAT_COL_NAMES.length; c++) {
                if (colName.equals(STAT_COL_NAMES[c])) {
                    return Printf.format(fStatsPerRow[c].get(row), 2); // @note this is synched with the dataset and not the fa
                }
            }

            // finally
            if (fColorMap != null) {
                return fColorMap.getValue(featureName, getColumnName(col));
            }

            return null;
        }

        public Class getColumnClass(int col) {
            return String.class;
        }

        public boolean isEditable() {
            return false;
        }

        public boolean isCellEditable(int row, int col) {
            return false;
        }

    }    // End Inner class ModelWithMetrics


    public static class Model extends AbstractTableModel {

        private Dataset fDataset;

        private FeatureAnnot fFeatureAnnot;

        private ColorMap.Rows fColorMap;

        private String[] fColNames;

        /**
         * Class Constructor.
         * Initializes model to specified Dataset
         * <p/>
         * Get rid of the desc field if it is na
         * Fill in a color field if the dataset is really a scored dataset
         */
        public Model(final Dataset ds) {
            this.fDataset = ds;
            this.fFeatureAnnot = ds.getAnnot().getFeatureAnnot();
            this.fColorMap = fFeatureAnnot.getColorMap();
            final java.util.List names = new ArrayList();
            names.add(Headers.NAME);

            if (fFeatureAnnot.hasNativeDescriptions()) {
                names.add(Headers.DESCRIPTION);
            }

            // no point adding symbol if the chip is already a symbol chip
            if (VdbRuntimeResources.isChipGeneSymbol(fFeatureAnnot.getChip())) {
                names.add(Headers.GENE_SYMBOL);
            }

            names.add(Headers.GENE_TITLE);

            if (fColorMap != null) {
                for (int c = 0; c < fColorMap.getNumCol(); c++) {
                    names.add(fColorMap.getColName(c));
                }
            }

            this.fColNames = (String[]) names.toArray(new String[names.size()]);
        }

        public int getColumnCount() {
            return fColNames.length;
        }

        public int getRowCount() {
            return fDataset.getNumRow();
        }

        public String getColumnName(int col) {
            return fColNames[col];
        }

        public Object getValueAt(int row, int col) {

            final String featureName = fDataset.getRowName(row);

            if (col == 0) {
                return featureName;
            }

            final String colName = getColumnName(col);

            if (colName.equals(Headers.DESCRIPTION)) {
                return fFeatureAnnot.getNativeDesc(featureName);
            } else if (colName.equals(Headers.GENE_SYMBOL)) {
                return fFeatureAnnot.getGeneSymbol(featureName);
            } else if (colName.equals(Headers.GENE_TITLE)) {
                return fFeatureAnnot.getGeneTitle(featureName);
            }

            // finally
            if (fColorMap != null) {
                return fColorMap.getValue(featureName, getColumnName(col));
            }

            return null;
        }

        public Class getColumnClass(int col) {
            return String.class;
        }

        public boolean isEditable() {
            return false;
        }

        public boolean isCellEditable(int row, int col) {
            return false;
        }

    }    // End Inner class Model


}        // End DatasetFeatureAnnotationViewer

/*--- Formatted in Sun Java Convention Style on Thu, Oct 17, '02 ---*/
